dojo.provide("itme.ChassisMap.ChassisMap");

dojo.require("itme.ChassisMap._Utils");
dojo.require("itme.ChassisMap._Layout");
dojo.require("itme.ChassisMap._Selection");
dojo.require("itme.ChassisMap._Overlays");
dojo.require("itme.ChassisMap.ChassisMapTooltip");
dojo.require("itme.ChassisMap.ImageResources");
dojo.require("ibm.tivoli.navmodel.Dataset");
dojo.require("itme.ChassisMap.KnownHardwareInfo");

dojo.require("dijit.Dialog");
dojo.require("dijit.form.Button");
dojo.require("dijit.Tooltip");

dojo.require("dojo.html");
dojo.require("dojo.i18n");

dojo.declare("itme.ChassisMap.ChassisMap", [itme.ChassisMap._Utils,
                                 itme.ChassisMap._Layout,
                                 itme.ChassisMap._Selection,
                                 itme.ChassisMap._Overlays], {
  // private members
  _dataSet: null,
  _demoMode: false,
  _args: null,
  _itemsList: null,
  _itemIndex: 0,
  _datasetTarget: "CHASSIS_MAP_COLLECTION",
  _datasetParms: null,
  _refreshTime: null,
  _isRendered: false,
  _isRefreshPath: false,
  _noDataEncouteredCount: 0,
  _defaultProperties: null,
  //_TEST_NODATA: false,
  _issues: {
    warnings: []
  },
  MRI: null,
  IMAGES : new itme.ChassisMap.ImageResources(),

  constructor: function(kwArgs, chassisMapParentDOMNodeID, groupTarget, chassisId){
    this._args = {    };

    dojo.requireLocalization("itme", "Resources");
    this.MRI = dojo.i18n.getLocalization("itme", "Resources");

    dojo.addOnUnload(dojo.hitch(this, this.cleanUp));

    itmeUtils.CMTrace("MAIN", "Constructor");

    this._defaultProperties = ["AccessState", "OTHERHWVIEWPROPS", "ChassisSlot", "OccupiedSlots", "Model", "STATUS",
                               "MachineType", "SerialNumber", "Bay", "ObjectType", "COMPLIANCEINFO", "IPv4Address",
                               "EnclosureFRU", "Status", "DisplayName", "ACTIVESTATUSINFO", "CommunicationState",
                               "Architecture", "Manufacturer", "PowerState", "SystemBoardUUID", "MACAddress",
                               "DisplayOperationalStatusTime", "Problems", "OperatingState", "HostName"];

    this._unSetBadPrototypes();

    dojo.mixin(this._args, kwArgs);

    if ( groupTarget )
      this._datasetTarget = groupTarget;
    if ( chassisId )
      this._datasetParms = chassisId;
    //this._chassisMapLayout = new itme.ChassisMapLayout({});

    //setTimeout(dojo.hitch(this, this._NoDataTestM01), 30000);

    this.cleanUp(); // ensures all previous data and dojo widgets are cleaned up
  },

  // TYPE: RENDER
  // public : set initial dimensions from the given node.
  setInitialSizingDimensionsFromNode: function(nodeToGetInitialSizingFrom)
  {
    this.setDimensionsBasedOnNode(nodeToGetInitialSizingFrom);
  },

  // public : given the parameters in the constructor, build and render the chassis map
  // TYPE: MAIN
  renderMap: function(dataset)
  {
    if ( dataset )
      this._acquireData(dataset);
    else
      this._acquireData();
  },

  // TYPE: MAIN
  // private
  _acquireData: function(dataset)
  {

    itmeUtils.CMTrace("MAIN", "ChassisMap.acquireData() method ENTERED");
    var onChassisRestQueryReturnFunctions = { // execution will resume here after makeRestQuery
      onComplete: dojo.hitch(this, this._onChassisDataRetrieved),
      onError: dojo.hitch(this, this._onChassisDataRetrievalFailed)
    };

    if ( ! this._isRefreshPath )
    {
      this.cleanUp();
      this._buildChassisMapHTML();
      this._toggleWaitingPanel(true, this.MRI.Loading );
    }

    if ( dataset )
      this._dataset = dataset;
    else {
      itmeUtils.CMTrace("MAIN", "ChassisMap.acquireData() - building dataset for target: " + this._datasetTarget + " , datasetParms: "+ this._datasetParms);
      try {
        this._dataset = new ibm.tivoli.navmodel.Dataset({
            baseUrl: "/ibm/tivoli/rest",
            provider: "USMI",
            datasource: "",
            dataset: this._datasetTarget,
            parameters: {
                  homeParms: this._datasetParms
            },
            defaultProperties: this._defaultProperties
        });
      } catch (err) {
        this._onChassisDataRetrievalFailed("dataset constructor failed");
        return;
      }
    }
    itmeUtils.CMTrace("MAIN", "fetching dataset items...");
    itmeUtils.initializeTimer("ChassiMapTimer");

    this._dataset.addEventListener(this, dojo.hitch(this, this._eventHandler));
    this._dataset.getItemsAndLinks(onChassisRestQueryReturnFunctions);
    itmeUtils.CMTrace("MAIN", "sent request for dataset items...");
  },

  _eventTimeout: null,
  _initialEventTime: null,
  // when an event is fired on a data item we're listening to, refresh the hardware view.
  // Note: many events may come through really quickly, so after 3 seconds of no more events, the refresh will fire.
  _eventHandler: function(listenerObject, event) {
    itmeUtils.CMTrace("MAIN", "EVENT RECIEVED");
    //itmeUtils.CMTrace("MAIN", "event datasetId=" + event.getDataset().getId() + " event=" + event.getType() +  " eventId=" + event.getId());

    clearTimeout(this._eventTimeout); // reset the timeout window.
    if ( this._initialEventTime == null ) {
      this._initialEventTime = new Date();
      this._eventTimeout = setTimeout(dojo.hitch(this, this._refreshView, "evented"), 5000);
    // FORCE a refresh if blocked for 30 seconds or more by constant incoming events.
    } else if ( ((new Date()).getTime() - this._initialEventTime.getTime()) > 30000 ){
      itmeUtils.CMTrace("MAIN", "Persistent events coming in.  30 seconds since first event, forcing a refresh.");
      clearTimeout(this._eventTimeout);
      this._refreshView("evented");
    }
    else {
      this._eventTimeout = setTimeout(dojo.hitch(this, this._refreshView, "evented"), 5000);
    }
  },

  // TYPE: MAIN
  // private: execution resumes here when dataset query completes
  _onChassisDataRetrieved: function(items, links, childDatasetItems)
  {
    itmeUtils.CMTrace("MAIN", "chassis items retrieval RETURNED count: " + items.length);
    itmeUtils.traceTimeDelta("ChassiMapTimer", "HWView - Dataset Retrieval", true);

    this._fetchItemsProperties(items);

    if (this._itemsList.length === 0)  { // DO NOT FAIL, wait x seconds and try again.
      this._noDataEncouteredCount++;
      //itmeUtils.CMTrace("MAIN", "DEBUG: No Data (or test no data).  Count = " + this._noDataEncouteredCount);
      if (this._noDataEncouteredCount > 3) {
        this._noDataEncouteredCount = 0;
        //itmeUtils.CMTrace("MAIN", "DEBUG: No data exceeded 3 attempts, swapping to table view");
        this._displayErrorAndChangeToTableView(this.MRI.ERROR_HWVIEW_NoData);
      } else {
        //itmeUtils.CMTrace("MAIN", "DEBUG: Refresh in 10 seconds...");
        clearTimeout(this._eventTimeout);
        this._eventTimeout = setTimeout(dojo.hitch(this, this._refreshView, "evented"), 10000);
        this._toggleWaitingPanel(false);
      }
      return;
    } else
      this._noDataEncouteredCount = 0;


    itmeUtils.traceTimeDelta("ChassiMapTimer", "HWView - Propterties Retrieval", true);

    //Process the list of links
    //if (links) {
    //  for (var index = 0;  index < links.length;  ++index) {
    //    var link = links[index];
    //
    //    itmeUtils.CMTrace("MAIN", "dataset link["+index+"] : " + link);
    //  }
    //}

    // buildChassisMap
    this._toggleWaitingPanel(false);
    this._renderMap();
  },

  /*
  _NoDataTestM01: function() {
    itmeUtils.CMTrace("MAIN", "DEBUG: _NoDataTestM01 ENTERED");
    this._TEST_NODATA = true;
    this._eventHandler("","");
    setTimeout(dojo.hitch(this, this._NoDataTestM02), 45000);
  },

  _NoDataTestM02: function() {
    itmeUtils.CMTrace("MAIN", "DEBUG: _NoDataTestM02 ENTERED");
    this._TEST_NODATA = false;
    //this._eventHandler("","");
    //setTimeout(dojo.hitch(this, this._NoDataTestM02), 45000);
  },
  */

  _fetchItemsProperties: function(items) {
    var index;
    this._itemsList = items; // store the item list of other member functions to access

    for (index = 0;  index < this._itemsList.length;  ++index) {
      this._itemIndex = index;
      var item = this._itemsList[index];
      itmeUtils.CMTrace("MAIN", (index+1)+". id: " + item.getId() + " , type: " + item.getType() + " , label: " + item.getLabel() );

      var itemPropArgs = {
          onComplete: dojo.hitch(this, this._onPropertiesDataRetrieved, index),
          onError:    dojo.hitch(this, this._onPropertiesDataRetrievedFailed)
      };

      //this._itemsList[index].getProperties(itemPropArgs);
    }
  },

  // TYPE: ERROR HANDLING
  _onChassisDataRetrievalFailed: function(error, request){
    itmeUtils.CMError("MAIN", "The items request to the datastore failed.  " + error); // go straight to the error log
    itmeUtils.CMError("MAIN", " - request: " + request);

    this._displayErrorAndChangeToTableView(this.MRI.ERROR_HWView_Unknown);
  },

  // TYPE: ERROR HANDLING
  _onPropertiesDataRetrievedFailed: function(error, request){
    itmeUtils.CMError("MAIN", "The properties request to the datastore failed.  " + error); // go straight to the error log
    itmeUtils.CMError("MAIN", " - request: " + request);

    this._displayErrorAndChangeToTableView(this.MRI.ERROR_HWView_PropertyDataRequestFailed);
    //if ( request )
    //  errorMessage += "<br><br>Request:<pre>"+request+"</pre>";
    //errorMessage += "<pre>"+error +"</pre>"
    //              + "<button dojoType='dijit.form.Button' type='submit' onClick='processRefresh(\"tableView\");'>"+ this.MRI.SwitchToTableView + "</button>";
    //var foo = new dijit.Dialog({ title: this.MRI.ERROR_HWView_Title, content: errorMessage });
    //foo.show();
  },

  // TYPE: ERROR HANDLING
  _displayErrorAndChangeToTableView: function(errorMessage) {

    var errorDialogString = this.MRI.ERROR_HWView_DataRequestFailed;
    //if ( request )
    //  errorDialogString += "<br><br>Request:<pre>"+request+"</pre>";
    errorDialogString += "<br><pre>"+errorMessage +"</pre>";

    if (errorMessage == this.MRI.ERROR_HWVIEW_NoData)
      processRefresh("nodata");
    else {
      //alert(FSMFlightRecorder.toString("\n"));
      processRefresh("tableView");
      setTimeout('processRefresh("tableView_cmerror'+ errorDialogString+'")', 3000);
    }
  },

  // TYPE: MAIN
  _onPropertiesDataRetrieved: function(itemIndex, properties, args){
    var i;

    if ( false  ) {
      itmeUtils.CMTrace("MAIN", (itemIndex+1) +". " + this._itemsList[itemIndex].getLabel());

      for (i = 0; i < properties.length; i++) {
        itmeUtils.CMTrace("MAIN", "  [ " + properties[i].getId() + " ] - " + properties[i].getDisplayValue());
      }
    }
  },

  // TYPE: RENDER
  _renderMap: function()  {
    var index, i;
    itmeUtils.CMTrace("MAIN", " _renderMap reached");

    if ( this._isRefreshPath ) {
      this.cleanUp({renderedOnly:true}); // true indicates only cleaning rendered objects
      this._buildChassisMapHTML();
      this._isRefreshPath = false;
    }
    this._isRendered = true;
    this._insertRefreshButton();

    itmeUtils.traceTimeDelta("ChassiMapTimer", "HWView - Chassis Map HTML Building", true);

    // iterate through items.
    for (index = 0; index < this._itemsList.length; index++) {
      var item = this._itemsList[index];

      var mapElement = this._associateAndModifyDomElementForItem(item);
      if (mapElement) { // if the element is not compliant, its skipped (mock objects)
        // add selection handler
        itmeUtils.makeEvent1Param(mapElement, dojo.hitch(this, this._onElementClicked), item, "click");
        this._buildTooltip(item, mapElement);
      }
      else
        itmeUtils.CMTrace("MAIN", "bad element skipped: " + item.getId() + " - " + item.getLabel());
    }

    itmeUtils.CMTrace("MAIN", "Warnings during render: " + this._issues.warnings.length);
    /*if ( this._issues.warnings.length > 0 ) {
      this._insertNotificationArea();
      var tooltipStr = "";
      for ( i = 0; i < this._issues.warnings.length; i++)
        tooltipStr = this._issues.warnings[i] + "<br>";

      new dijit.Tooltip({
        connectId: ["info_buttonContainer"],
        label: tooltipStr,
        position: "above",
        id: "warnings_tooltip"
      });
    }*/

    itmeUtils.traceTimeDelta("ChassiMapTimer", "HWView - ChassisMap Item Processing", true);

    this._insertOverlays(this._chassisOverlayElement, this._itemsList);
    this._setItemsForOverlays(this._itemsList);
    this._renderDefaultOverlay();
    this._prefetchAsynchOverlayData(this._itemsList, this._defaultProperties);
  },

  _insertNotificationArea: function() {

    this._appendFilterButtonIconStyleToDocument("notification_icon", "StatusWarningMinor");

    var buttonContainer = itmeUtils.createElement("info_buttonContainer",
                                                  this._chassisOverlayElement,
                                                  { elementType: "span",
                                                    styleClass: "hwViewWarnings",
                                                    style: { "position":"absolute", "right":"30px"} });

    buttonContainer.innerHTML = this.IMAGES.buildIconImageTag("StatusInformational", "", 22);
    /*var button = new dijit.form.Button({
            optionsTitle: "Refresh the hardware view",
            onClick: dojo.hitch(this, this._refreshView),
            iconClass: "refresh_icon",
            id: "refresh_dojoButton",
            showLabel: false,
            type: "button"
        });
    buttonContainer.appendChild(button.domNode);
     */

  },


  _insertRefreshButton: function() {

    this._appendFilterButtonIconStyleToDocument("refresh_icon", "RefreshIcon");

    var buttonContainer = itmeUtils.createElement("refresh_buttonContainer",
                                                  this._chassisOverlayElement,
                                                  { elementType: "span",
                                                    styleClass: "buttonContainer",
                                                    style: { "position":"absolute", "right":"0px"} });

    var button = new dijit.form.Button({
            optionsTitle: "Refresh the hardware view",
            onClick: dojo.hitch(this, this._refreshView),
            iconClass: "refresh_icon",
            id: "refresh_dojoButton",
            showLabel: false,
            type: "button"
        });
    buttonContainer.appendChild(button.domNode);


    new dijit.Tooltip({
      connectId: ["refresh_dojoButton"],
      label: "Refresh the hardware view",
      position: "above",
      id: "refresh_buttonTooltip"
    });
  },

  _refreshView: function(source) {
    //FSMFlightRecorder.test();
    itmeUtils.CMTrace("MAIN", "refreshing...");
    itmeUtils.CMTrace("MAIN", "DEBUG: refreshView - isRendered: " + this._isRendered);
    if ( !this._isRendered )
      return;
    if ( this._currentOverlay !== null )
      dojo.cookie("chasisMap.currentOverlay", this._currentOverlay.getId(), { expires: 1});

    this._toggleWaitingPanel(true, this.MRI.Refreshing);
    this._isRefreshPath = true;

    if ( source && (source == "evented") )
      ;
    else {
      processRefresh("refresh");
    }

    this.cleanUp({ dataOnly: true });
    this.renderMap();
  },

  // TYPE: RENDER
  // private.  Given a chassis item, connect it to its associated dom node and modify that node
  _associateAndModifyDomElementForItem: function(item) { // return mapElement
    var mapElement = null;
    var isChassis = false;
    var occupiedSlots, chassisSlot = null;

    var frmProps = dojo.fromJson( itmeUtils.getProperty(item, "OTHERHWVIEWPROPS") );

    // CHEAT.  For now, handle specific types of nodes in a specific way.
    switch (item.getType()) {
      case "PowerSupply":
        if ( frmProps.SLOT && frmProps.SLOT[0] ) {
          mapElement = dojo.byId(this.PWR_Name + frmProps.SLOT[0]);
          item.myImage = this.IMAGES.getImage("PowerUnit");
        }
        else {
          itmeUtils.CMError("MAIN", "DM SLOT positioning is unavailable for: " + item.getId() + ", " + item.getLabel());
          this._issues.warnings.push("The hardware view does not have slot positioning information for item: " + item.getId() + ", " + item.getLabel());
        }
        break;
      case "Fan":
        if ( frmProps.SLOT && frmProps.SLOT[0] ) {
          if ( frmProps.SLOT[0] == 5 || frmProps.SLOT[0] == 10 )
            item.myImage = this.IMAGES.getImage("FanUnit2");
          else
            item.myImage = this.IMAGES.getImage("FanUnit1");
          mapElement = dojo.byId(this.FAN_Name + frmProps.SLOT[0]);
        }
        else {
          itmeUtils.CMError("MAIN", "DM SLOT positioning is unavailable for: " + item.getId() + ", " + item.getLabel());
          this._issues.warnings.push("The hardware view does not have slot positioning information for item: " + item.getId() + ", " + item.getLabel());
        }
        break;
      case "PhysicalPackage":
        if ( frmProps.SLOT && frmProps.SLOT[0] ) {
          item.myImage = this.IMAGES.getImage("FanControlUnit");
          mapElement = dojo.byId(this.FANCTRL_Name + frmProps.SLOT[0]);
        }
        else {
          itmeUtils.CMError("MAIN", "DM SLOT positioning is unavailable for: " + item.getId() + ", " + item.getLabel());
          this._issues.warnings.push("The hardware view does not have slot positioning information for item: " + item.getId() + ", " + item.getLabel());
        }
        break;
      case "com.ibm.sysmgt.platform.PhysicalPlatformFactory":
        if ( frmProps.SLOT && frmProps.SLOT[0] ) {
          if ( frmProps.SLOT.length > 1 )
            this.mergeSlots(frmProps.SLOT[0], this.ITE_Name, frmProps.SLOT);

          mapElement = dojo.byId(this.ITE_Name + (frmProps.SLOT[0]));

          if (frmProps.EXPANSIONTYPE == "SEN") {
            item.myImage = this.IMAGES.getImage("ITE_C42x0M4_SEN"); // this._getBladeImageByItem(item, frmProps.SLOT.length);
          } else if (frmProps.EXPANSIONTYPE == "PEN") {
            item.myImage = this.IMAGES.getImage("ITE_C42x0M4_PEN"); ;
          } else {
            item.myImage = this._getBladeImageByItem(item, frmProps.SLOT.length);
          }

        }
        else {
          itmeUtils.CMWarn("MAIN", "DM SLOT positioning is unavailable for: " + item.getId() + ", " + item.getLabel());
          chassisSlot = itmeUtils.getProperty(item, "ChassisSlot");
          if (chassisSlot == null) {// no SLOT or ChassisSlot... not enough info to place in map
            itmeUtils.CMError("The hardware view does not have any slot positioning information for item: " + item.getId() + ", " + item.getLabel());
            this._issues.warnings.push("The hardware view does not have slot positioning information for item: " + item.getId() + ", " + item.getLabel());
            break;
          }

          occupiedSlots = itmeUtils.getProperty(item, "OccupiedSlots");
          if ( occupiedSlots && occupiedSlots.length > 1 ) {
            numSlots = occupiedSlots.length;
            this.mergeSlots(chassisSlot, this.ITE_Name, occupiedSlots);
            item.myImage = this._getBladeImageByItem(item, occupiedSlots.length);
          }
          else
            item.myImage = this._getBladeImageByItem(item, 1);

          mapElement = dojo.byId(this.ITE_Name + chassisSlot);
        }
        break;
      case "Switch":
      case "PassThroughModule":
        //occupiedSlots = itmeUtils.getProperty(item, "OccupiedSlots");
        if ( frmProps.SLOT && frmProps.SLOT[0] )
          mapElement = dojo.byId(this.IOM_Name + frmProps.SLOT[0]);
        else {
          itmeUtils.CMError("MAIN", "DM SLOT positioning is unavailable for: " + item.getId() + ", " + item.getLabel());
          this._issues.warnings.push("The hardware view does not have slot positioning information for item: " + item.getId() + ", " + item.getLabel());
        }

        item.myImage = this._getIOModuleImage(item.getType(), frmProps.PRODUCTID);
        break;
      case "BladeAdminServer":
        if ( frmProps.SLOT && frmProps.SLOT[0] )
          mapElement = dojo.byId(this.MGT_Name + frmProps.SLOT[0]);
        else {
          itmeUtils.CMError("MAIN", "DM SLOT positioning is unavailable for: " + item.getId() + ", " + item.getLabel());
          this._issues.warnings.push("The hardware view does not have slot positioning information for item: " + item.getId() + ", " + item.getLabel());
        }
        item.myImage = this.IMAGES.getImage("CMM");
        break;
      case "SystemChassis": // special case
        mapElement = this._chassisNameElement;
        isChassis = true;
        setTimeout(dojo.hitch(this, this._DisplayUndiscoveredElements, frmProps.UNDISCOVERED), 750);
        break;
      case "StoragePlatform":
        occupiedSlots = itmeUtils.getProperty(item, "OccupiedSlots");
        chassisSlot = itmeUtils.getProperty(item, "ChassisSlot");
          if ( occupiedSlots && occupiedSlots.length > 1 ) {
            numSlots = occupiedSlots.length;
            this.mergeSlots(frmProps.SLOT[0], this.ITE_Name, occupiedSlots);
            item.myImage = this._getBladeImageByItem(item, occupiedSlots.length);
          }
          else {
             item.myImage = this._getBladeImageByItem(item, 1);
          }
        mapElement = dojo.byId(this.ITE_Name + frmProps.SLOT[0]);
        break;
    }

    if (mapElement) {
      dojo.addClass(mapElement, "ComponentSlotOccupied");
      dojo.removeClass(mapElement, "EmptySlot");
      item.mapElement = mapElement;

      if (isChassis === false) { // chassis logic was already handled
        imageString = "<img id='" + mapElement.id + "_IMG' src='" + item.myImage + "' width='" + mapElement.style.width + "' height='" + mapElement.style.height + "' alt='" + item.getLabel() + "'>";
        mapElement.innerHTML = imageString; // + "<span style='position:absolute;top:0;left:0'><font size='-2' color='white'>" + item.getLabel() + "</font></span>";
      }

      if (this._currentSelectionsByItemId[item.getId()+"_item"]) {
        itmeUtils.CMTrace("SELECTION", "selectedItem: " + item.getId());
        dojo.addClass(mapElement, "ComponentSelected");
      }
    }

    return mapElement;
  },

  _DisplayUndiscoveredElements: function(undiscoveredElements) {
    var i, mapElement, image, unDiscoveredItemDivElement, typeID, typeString, statusString;
    itmeUtils.CMTrace("MAIN", "DEBUG:  undiscoveredElements.length = " + undiscoveredElements.length);
    for (i = 0; i < undiscoveredElements.length; i++)
    {
      if ( ! (undiscoveredElements[i].SLOT) )
        continue;

      switch(undiscoveredElements[i].TYPE)
      { case "ITE":
          typeID = this.ITE_Name;
          typeString = this.MRI.Node;
          break;
        case "Switch":
          typeID = this.IOM_Name;
          image = this._getIOModuleImage("Switch", undiscoveredElements[i].PRODUCTID);
          typeString = this.MRI.Switch;
          break;
        case "Fan":
          typeID = this.FAN_Name;
          if ( undiscoveredElements[i].SLOT[0] == 5 || undiscoveredElements[i].SLOT[0] == 10 )
            image = this.IMAGES.getImage("FanUnit2");
          else
            image = this.IMAGES.getImage("FanUnit1");
          typeString = this.MRI.Fan;
          break;
        case "PowerSupply":
          typeID = this.PWR_Name;
          image = this.IMAGES.getImage("PowerUnit");
          typeString = this.MRI.PowerSupply;
          break;
        case "CMM":
          typeID = this.MGT_Name;
          image = this.IMAGES.getImage("CMM");
          typeString = this.MRI.BladeMgmtModule;
          break;
        case "Passthrough":
          typeID = this.IOM_Name;
          image = this._getIOModuleImage("PassThroughModule", undiscoveredElements[i].PRODUCTID);
          typeString = this.MRI.PassthroughModule;
          break;
        case "FanLogic":
          typeID = this.FANCTRL_Name;
          image = this.IMAGES.getImage("FanControlUnit");
          typeString = this.MRI.FanLogicModule;
          break;
        case "SCU":
          typeID = this.ITE_Name;
          image = this._getStorageBladeImage(undiscoveredElements[i]);
          typeString = this._getStorageBladeDescription(undiscoveredElements[i]);
          break;
        default:
          // Without continue here, an unmatched undiscovered type will have the previous
          // typeID set and may apply the previous image to a slot of another type.
          continue;
       }


      // whoopsies
      mapElement = dojo.byId(typeID + undiscoveredElements[i].SLOT[0]);
      if ( dojo.hasClass(mapElement, "ComponentSlotOccupied") ) {
        itmeUtils.CMTrace("MAIN", "UnDiscovered "+undiscoveredElements[i].TYPE+" trying to fill occupied slot: " + undiscoveredElements[i].SLOT[0]);
        continue;
      }

      if ( undiscoveredElements[i].SLOT.length > 1 )
        mapElement = this.mergeSlots(undiscoveredElements[i].SLOT[0], this.ITE_Name, undiscoveredElements[i].SLOT);

      dojo.removeClass(mapElement, "EmptySlot");
      dojo.addClass(mapElement, "ComponentNotDiscovered");

      unDiscoveredItemDivElement = itmeUtils.createElement(
        mapElement.id + "_unDiscEl",
        mapElement,
        { styleClass: "ComponentNotDiscoveredOverlay",
          baseDimensionsOnNode: mapElement} );

      var tooltip = new itme.ChassisMap.ChassisMapTooltip({}, mapElement);
      tooltip.setTitle(dojo.replace("Detected {0}", [typeString]));

      if ( undiscoveredElements[i].TYPE == "SCU" ) {
           unDiscoveredItemDivElement.innerHTML = this.MRI.Pending;
      }

      if ( undiscoveredElements[i].TYPE == "ITE" )
      {
        if (undiscoveredElements[i].EXPANSIONTYPE) {
          image = this._getBladeImageWithExpansion(undiscoveredElements[i].EXPANSIONTYPE);
        } else if ( undiscoveredElements[i].MTYPE )
          image = this._getBladeImage("", undiscoveredElements[i].MTYPE, undiscoveredElements[i].SLOT.length);
        else
          image = this._getBladeImage("", "", undiscoveredElements[i].SLOT.length);

        switch (undiscoveredElements[i].STATE)
        { case 0:
            if ( undiscoveredElements[i].STATUS == 15 )
              statusString = this.MRI.Flashing;
            else
              statusString = this.MRI.Discovering;
            tooltip.addBaseProperty( this.MRI.Status , statusString);
            unDiscoveredItemDivElement.innerHTML = this.MRI.Pending;
            break;
          case 5:
            tooltip.addBaseProperty( this.MRI.Status , this.IMAGES.buildIconImageTag("StatusNormal", "", 16) + " " + this.MRI.OK);
            unDiscoveredItemDivElement.innerHTML = this.MRI.Pending;
            break;
          case 10:
            tooltip.addBaseProperty( this.MRI.Status , this.IMAGES.buildIconImageTag("StatusWarningMinor", "", 16) + " " + this.MRI.Warning);
            unDiscoveredItemDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusWarningMinor", "", 16) + " " + this.MRI.Pending;
            break;
          case 20:
            tooltip.addBaseProperty( this.MRI.Status , this.IMAGES.buildIconImageTag("StatusCritical", "", 16) + " " + this.MRI.Critical);
            unDiscoveredItemDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusCritical", "", 16) + " " + this.MRI.Pending;
            break;
          case 30:
            if ( undiscoveredElements[i].STATUS == 12 )
              statusString = this.MRI.CommError;
            else
              statusString = this.MRI.Error;
            tooltip.addBaseProperty( this.MRI.Status , this.IMAGES.buildIconImageTag("StatusCritical", "", 16) + " " + statusString);
            tooltip.addBaseProperty( "" , this.MRI.HWNodeWarning);
            unDiscoveredItemDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusCritical", "", 16) + " " + this.MRI.Error;
            break;
        }
      }

      if ( undiscoveredElements[i].TYPE == "SCU" || undiscoveredElements[i].TYPE == "ITE" ) {
        if ( undiscoveredElements[i].SERIAL )
          tooltip.addBaseProperty( this.MRI.SerialNumber, undiscoveredElements[i].SERIAL );
        if ( undiscoveredElements[i].ARCH )
          tooltip.addBaseProperty( this.MRI.Architecture, undiscoveredElements[i].ARCH );
        if ( undiscoveredElements[i].MTYPE )
          tooltip.addBaseProperty( this.MRI.MachineType, undiscoveredElements[i].MTYPE );
        if ( undiscoveredElements[i].MNUMBER )
          tooltip.addBaseProperty( this.MRI.Model, undiscoveredElements[i].MNUMBER );
      }

      tooltip.buildTooltip();
      imageString = "<img id='und_"+undiscoveredElements[i].TYPE+"_" + undiscoveredElements[i].SLOT[0] + "_IMG' src='" + image + "' width='" + mapElement.style.width + "' height='" + mapElement.style.height + "' >";
      mapElement.innerHTML = imageString + mapElement.innerHTML;
    }
  },

  _getBladeImageByItem: function (item, numSlots) {
    var model = this._getProperty(item, "Model");
    var machineType = this._getProperty(item, "MachineType");
    return this._getBladeImage(model, machineType, numSlots);
  },

  _getBladeImage: function(model, machineType, numSlots) {
    var hwInfo = new itme.ChassisMap.KnownHardwareInfo();
    var imgName = hwInfo.getImageResourceForModelAndType(model, machineType, numSlots);

    if ( imgName )
      return this.IMAGES.getImage(imgName);
    else {
      if ( numSlots > 1)
        return this.IMAGES.getImage("DefaultITE_2x");
      else
        return this.IMAGES.getImage("DefaultITE_1x");
    }
  },

  _getBladeImageWithExpansion: function (expansionType) {
    if (expansionType == "PEN") {
      return this.IMAGES.getImage("ITE_C42x0M4_PEN");
    } else {
      return  this.IMAGES.getImage("ITE_C42x0M4_SEN");
    }
  },

  _getStorageBladeImage: function (item) {
    if (item.PRODUCTID == "384") { return this.IMAGES.getImage("STRW_493949"); } // 0x0180 = SRC
    if (item.PRODUCTID == "385") { return this.IMAGES.getImage("STRW_493929"); } // 0x0181 = SDE
    if (item.model == "A49") { return this.IMAGES.getImage("STRW_493949"); } // 4939A49 = SRC
    if (item.model == "A29") { return this.IMAGES.getImage("STRW_493929"); } // 4939A29 = SDE
    return this.IMAGES.getImage("STRW_493949"); // SRC
  },

  _getStorageBladeDescription: function (item) {
    if (item.PRODUCTID == "384") { return this.MRI.StorageRaidController; } // 0x0180 = SRC
    if (item.PRODUCTID == "385") { return this.MRI.StorageDiskExpansion; }  // 0x0181 = SDE
    if (item.model == "A49") { return this.MRI.StorageRaidController; } // 4939A49 = SRC
    if (item.model == "A29") { return this.MRI.StorageDiskExpansion; }  // 4939A29 = SDE
    return this.MRI.StorageRaidController; // SRC
  },

  _getIOModuleImage: function(type, productId) {
    var hwInfo = new itme.ChassisMap.KnownHardwareInfo();
    var imgName = hwInfo.getImageResourceForProductId(productId);

    if ( imgName )
      return this.IMAGES.getImage(imgName);
    else {
      if (type == "PassThroughModule")
        return this.IMAGES.getImage("SwitchEN4091");
      else
        return this.IMAGES.getImage("SwitchEN2092");
    }
  },

  _endsWith: function(string, suffix) {
    return string.indexOf(suffix, string.length - suffix.length) !== -1;
  },

    // TYPE: RENDER
  _buildTooltip: function(item, mapElement){

    // get properties we may display
    var commState = item.getProperty("CommunicationState");
    var model = this._getProperty(item, "Model");
    //var status = this._getProperty(item, "STATUS");
    var machineType = this._getProperty(item, "MachineType");
    var serialNumber = this._getProperty(item, "SerialNumber");
    var bay = this._getProperty(item, "Bay");
    var accessState = this._getProperty(item, "AccessState");
    itmeUtils.CMTrace("OVERLAY", "building new tooltip for " + mapElement.id);
    var tooltip = new itme.ChassisMap.ChassisMapTooltip({}, mapElement);
    tooltip.setTitle( item.getLabel() ); //itemStyle.getLabel() );
    var objProp = item.getProperty("ObjectType");
    var problems = dojo.fromJson( this._getProperty(item, "Problems") );

    if (model && machineType && machineType==="4939") {
      // Replace Storage Platform with more specific type
      if (this._endsWith(model,"49")) {
        tooltip.addBaseProperty( this.MRI.Type, this.MRI.StorageRaidController ); //item.getLabel());
      } else {
        tooltip.addBaseProperty( this.MRI.Type, this.MRI.StorageDiskExpansion ); //item.getLabel());
      }
    } else {
      tooltip.addBaseProperty( this.MRI.Type, objProp.getDisplayValue() ); //item.getLabel());
    }


    // FRM - HA properties
    var frmProps = dojo.fromJson( itmeUtils.getProperty(item, "OTHERHWVIEWPROPS") );

    if ( frmProps.ISLOGICALFRM == true) {
      tooltip.addBaseProperty(this.MRI.ProductHardware, this.MRI.SWProductName );
      if ( frmProps.HA )
        tooltip.addBaseProperty(this.MRI.HARoleSubProp, frmProps.HA );
    }

    if (bay)
      tooltip.addBaseProperty(this.MRI.Bay , bay );
    //if (status)
    //  tooltip.addBaseProperty(this.MRI.Status , "<img src='" + status.getImage() + "'> "+status.getLabel() );
    if (problems)
    {
      var probString = "";
      if (problems.PBLM_E && (problems.PBLM_E > 0))
        probString = problems.PBLM_E + this.IMAGES.buildIconImageTag("StatusCritical", "", 16) + " ";
      if (problems.PBLM_W && (problems.PBLM_W > 0))
        probString = probString + problems.PBLM_W + this.IMAGES.buildIconImageTag("StatusWarningMinor", "", 16) + " ";
      if (problems.PBLM_I && (problems.PBLM_I > 0))
        probString = probString + problems.PBLM_I + this.IMAGES.buildIconImageTag("StatusInformational", "", 16) + " ";

      if (commState && (commState.getValue() == 3) )
        probString = probString + this.IMAGES.buildIconImageTag("AccessOfflineOK", "", 16) + " " + this.MRI.Offline + " ";

      if (probString == "") // no other 'problems', so show 'ok'
        probString = this.IMAGES.buildIconImageTag("StatusNormal", "", 16) + " " + this.MRI.OK;

      tooltip.addBaseProperty(this.MRI.Problems , probString );
    }
    if (model && machineType)
      tooltip.addBaseProperty(this.MRI.ModelMachineType , model + " ("+this.MRI.Type + " " + machineType + ")");
    else if (model)
      tooltip.addBaseProperty(this.MRI.Model , model);
    if (serialNumber)
      tooltip.addBaseProperty(this.MRI.SerialNumber , serialNumber);

    var clusterName = frmProps.CLUSTERNAME;
    var enclosureId = frmProps.ENCLOSUREID;
    if (clusterName) {
      tooltip.addBaseProperty(this.MRI.ClusterName, clusterName);
    }
    if (enclosureId) {
      tooltip.addBaseProperty(this.MRI.EnclosureId , enclosureId);
    }

    tooltip.buildTooltip();
  },

  // TYPE: ACTION, STATE
  _onElementClicked: function(item, event)
  {
    //If a refresh is in progress, don't allow selection, simply return
    if ( dojo.byId("hardwareViewWaitingPane")){
        if(dojo.byId("hardwareViewWaitingPane").style.display == 'block') {
            return;
        }
    }
    //itmeUtils.CMTrace("SELECTION", "PING PING ON CLICK PING");
    //itmeUtils.CMTrace("SELECTION", "Event??? " + event);
    var multiSelect = itmeUtils.isMultiSelectKeyUsed(event);
    if (item) {

      clearTimeout(this._refreshDelay);
      this._refreshDelay = false;

      if (multiSelect === true)
        this._handleMultiSelectEvent(item);
      else  // single selection
        this._handleSingleSelectEvent(item);

      //this._clearStorageElementSelection(multiSelect);

      dojo.cookie("chasisMap.currentOverlay", this._currentOverlay.getId(), { expires: 1});
      dojo.cookie("chasisMap.currentSelection", itmeUtils.myJoin(this._currentSelectionsByItemId), { expires: 1});
      itmeUtils.CMTrace("SELECTION", "COOKIE: adding cookie to set current overlay to: " + this._currentOverlay.getId());
      itmeUtils.CMTrace("SELECTION", "COOKIE: adding cookie to set selection to: " + itmeUtils.myJoin(this._currentSelectionsByItemId));

      // page reload now if single select case
      //TODO: reevaluate if we need any extra processing for multi select, treating it the same as single select (Alex)
    //  if (multiSelect === false)
    //  {
        //chassisMapRefresh(); //jsUtils
        //this.cleanUp();
        //Use cookie instead of item, to handle both single/multi selection with this call
        processRefresh( dojo.cookie("chasisMap.currentSelection") );
        this._toggleWaitingPanel(true, this.MRI.Refreshing);
        this._isRefreshPath = true;
        this._hideRefreshPanelWhenAjaxIsDone();
    }
  },

   _selectedStorageElement: null, // Used by jsUtils to build menu
  _selectedStorageElementGraphic: null,

  // DHM: Commented out for interim GA2 (or GA3 depending on who's counting)
  // _clearStorageElementSelection: function(multiSelect) {
  //   this._selectedStorageElement = null;
  //   if (multiSelect === true) {
  //     // DHM: need to ensure the SCU selection frame is cleared, single select clears the selection
  //     dojo.removeClass(this._selectedStorageElementGraphic, "ComponentSelected");
  //   }
  //   this._selectedStorageElementGraphic = null;
  // },

  // DHM: Commented out for interim GA2 (or GA3 depending on who's counting)
  // _onStorageElementClicked: function (item, event) {
  //     //If a refresh is in progress, don't allow selection, simply return
  //     if ( dojo.byId("hardwareViewWaitingPane")){
  //         if(dojo.byId("hardwareViewWaitingPane").style.display == 'block') {
  //             return;
  //         }
  //     }
  //     if (item) {
  //       clearTimeout(this._refreshDelay);
  //       this._refreshDelay = false;
  //       this._currentSelectionsByItemId = [];
  //         dojo.query(".ComponentSelected").forEach(function(node) {
  //           dojo.removeClass(node, "ComponentSelected");
  //         });
  //       dojo.addClass(item, "ComponentSelected");
  //       this._selectedStorageElement = item.SCU;
  //       this._selectedStorageElementGraphic = item;
  //     }

  //      processRefresh("9999"); // Seems to require a value
  //      // processRefresh(item.SCU.OID);

  //     this._toggleWaitingPanel(true, this.MRI.Refreshing);
  //     this._isRefreshPath = true;
  //     this._hideRefreshPanelWhenAjaxIsDone();
  //     // dojo.html.set(dojo.byId("prediscoveredActions"), "<p>loaded!</p>");
  // },

  /*_showRefreshPanel: function() {
    if ( dojo.byId("refreshPending"))
	    dojo.byId("refreshPending").style.display = "block";
    else {
        itmeUtils.createElement("refreshPending",
                                this._chassisMapFaceElements[0],
                                {styleClass: "refreshPendingDiv",
                                 style: { "position":"absolute", "top":"20px", "left":"20px"},
                                 innerHTML: this.IMAGES.buildIconImageTag("Processing", "", 22) + " " + this.MRI.Refreshing});
     }
  },*/

  _hideRefreshPanelWhenAjaxIsDone: function() {
		if(ajaxInProgress) {
			setTimeout(dojo.hitch(this, this._hideRefreshPanelWhenAjaxIsDone), 120);
		} else {
			dojo.byId("hardwareViewWaitingPane").style.display = "none";
		}


   },

  // public: unrender or otherwise delete resources on close or refresh
  //  parms: renderedOnly , do not clear out backend data hooks
  //  parms: dataOnly , clear out back end data hooks only, not the visuals
   // default: clear everything
  cleanUp: function(parms)
  {
    var button, counter, aTooltip, i;
    if (!parms) parms = {};
    renderedOnly = (typeof(renderedOnly) != 'undefined') ? renderedOnly : false;
    itmeUtils.CMTrace("MAIN", "HWView cleanUp entry");

    // if refreshTimer (caused by incoming event notifications) is active for any reason....
    if (!parms.renderedOnly) { // clear data unless rendered only
      itmeUtils.CMTrace("MAIN", "HWView cleanUp clearing data and event listener (if applicable)");
      clearTimeout(this._eventTimeout);
      this._initialEventTime = null;
      if ( this._dataset ) {
        this._dataset.destroy();
        this._dataset = null;
      }
      this._issues.warnings = [];
    }

    if ( !parms.dataOnly ) { // clear rendered unless data only
      this._isRendered = false;
      if ( dojo.byId(this._chassisMapParentDOMNodeID))
        dojo.empty(this._chassisMapParentDOMNodeID);
      this.refreshPrep();

      for ( i = 0; i < this._overlays.length; i++) {
        //itmeUtils.CMTrace("MAIN", "DESTROYING : " + this._overlays[i].getId()+"_dojoButton");
        button = dijit.byId(this._overlays[i].getId()+"_dojoButton");
        if ( button ) {
          button.destroyRecursive();
        }
      }
      button = dijit.byId("refresh_dojoButton");
      if ( button ) {
        button.destroyRecursive();
      }

      counter = 0;
      // cleanUp dojo tooltips.
      aTooltip = dijit.byId("myTooltip_"+counter);
      while (aTooltip) {
        aTooltip.close();
        aTooltip.destroyRecursive();
        counter++;
        aTooltip = dijit.byId("myTooltip_"+counter);
      }
      myTooltipCounter = 0;

      for (i = 0; i < this._overlays.length; i++)
      {
        buttonTooltip = dijit.byId(this._overlays[i].getId()+"_buttonTooltip");
        if ( buttonTooltip )
        {
          buttonTooltip.close();
          buttonTooltip.destroyRecursive();
        }
      }

      buttonTooltip = dijit.byId("refresh_buttonTooltip");
      if ( buttonTooltip )
      {
        buttonTooltip.close();
        buttonTooltip.destroyRecursive();
      }
      warningsTooltip = dijit.byId("warnings_tooltip");
      if ( warningsTooltip )
      {
        warningsTooltip.close();
        warningsTooltip.destroyRecursive();
      }
    }
  },

  refreshPrep: function() {
    this._currentOverlay = null; // reset this so we can re-render the overlays
    this._numFaceElements = 0;
  }


});
