dojo.provide("itme.ChassisMap.EnvironmentalsOverlay");

dojo.require("itme.ChassisMap.OverlayBase");
dojo.require("itme.ChassisMap.OverlayFilterItem");

/*
 * Inherit from base overlay class itme.ChassisMap.OverlayBase
 * Declare the following variables in the constructor:
 *   _overlayId
 *   _description
 *   _title
 *   _requiredProperties (unimplemented)
 */
dojo.declare("itme.ChassisMap.EnvironmentalsOverlay", [itme.ChassisMap.OverlayBase], {
  myIndex: 0,
  _MAX_MIDPOINT: 40,
  _midPointTemp: 40,
  constructor: function(kwArgs){
    // parent constructor is always auto-called first
    this._overlayId = "heat";
    this._description = this.MRI.EnvButtonTooltip;
    this._title = this.MRI.ThermalTitle;
    this._requiredProperties = [ "AEM_DATA" ];
    this._iconId = "ThermalIcon_22";
    
    this._availableFilters = [];
    //this._coolingSubsys = new itme.ChassisMap.OverlayFilterItem({}, "coolingGauge", "ThermalIcon_22", this.MRI.SubsystemDetails, true, true );
    //this._availableFilters[0] = this._coolingSubsys;
 
    
    this._midPointTemp = 40;
  },
  
  _buildTooltip: true,
  
  /* Before we hit the main rendering path, we need to do some calculations based on the midpoint temperature for all blades + chassis
   * 
  */
  _preProcessing: function(items) {
    var min = 1000, max = -1000;
    for (var i = 0; i < items.length; i++) {
      if (this._items[i].mapElement) {
        var aemData = dojo.fromJson( itmeUtils.getProperty(this._items[i], "AEM_DATA") );
        if ( aemData.ENV_AMBIENTTEMP && (aemData.ENV_AMBIENTTEMP > 0) ) {
          itmeUtils.CMTrace("OVERLAY", "new temp: " + aemData.ENV_AMBIENTTEMP + " , curmin: " + min + " , curmax: " + max);
          if (aemData.ENV_AMBIENTTEMP > max)
            max = aemData.ENV_AMBIENTTEMP;
          if (aemData.ENV_AMBIENTTEMP < min)
            min = aemData.ENV_AMBIENTTEMP;
        }
      }
    }
    this._midPointTemp = (min + max) / 2;
    if ( this._midPointTemp > this._MAX_MIDPOINT )
      this._midPointTemp = this._MAX_MIDPOINT;
    itmeUtils.CMTrace("OVERLAY", "min: " + min + " , max: " + max + " , MIDPOINT: " + this._midPointTemp);
  },
  
  _renderItem: function(item, buildTooltip){
    var colorClass;
    itmeUtils.CMTrace("OVERLAY", "renderItem ("+item.getLabel()+")");

    this._buildTooltip = buildTooltip;
    
    var overlayDivElement = dojo.byId(item.mapElement.id + "_heat");

    // if we've already created the element before just show it, otherwise create it
    if ( overlayDivElement)  {
      itmeUtils.CMTrace("OVERLAY", "renderItem ("+item.getLabel()+") - re-Enabling overlay element");
      overlayDivElement.style.display = "block";
      dojo.byId(item.mapElement.id + "_heat2").style.display = "block";
      if ( item.getType() == "SystemChassis" ) {
         if ( dojo.byId(item.mapElement.id + "_heatBackExhaust")) 
            dojo.byId(item.mapElement.id + "_heatBackExhaust").style.display = "block";
      }
    } else { 
      itmeUtils.CMTrace("OVERLAY", "renderItem ("+item.getLabel()+") - building overlay element");
      
      // create it
      overlayDivElement = itmeUtils.createElement(
                  item.mapElement.id + "_heat",
                  item.mapElement,
                  { styleClass: "HEATOverlayElement",
                    baseDimensionsOnNode: item.mapElement} );  
        
      var DataArray = dojo.fromJson( itmeUtils.getProperty(item, "AEM_DATA") );

      // verticalize text      
      this._setDimensionsAndTextDirection(item, overlayDivElement);  

      
      overlayDivElement.innerHTML = "";
      // if No Data....
      colorClass = "elementName"; // default
      itmeUtils.CMTrace("OVERLAY", "testing no data");
      if ( this._isNoRenderableData(DataArray) ) {
        dojo.addClass(overlayDivElement, "NoData");
        colorClass = "elementName";
        overlayDivElement.innerHTML = this.MRI.NoData;
        if ( buildTooltip)
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, "Environmetals", this.MRI.NoData );
      }
      else {
        colorClass = this._mapAmbientTempToColorScale(DataArray);

        
        itmeUtils.CMTrace("OVERLAY", "adding props");
        if ( item.getType() == "SystemChassis" ) {
          this._addAmbientTemp(DataArray, item, overlayDivElement, true);
          this._addExhaustTempAndAirFlow(DataArray, item, overlayDivElement, true);
          this._addInputPower(DataArray, item, overlayDivElement, true);
          this._addOutputPower(DataArray, item, overlayDivElement, true);
        }
        else {
          this._addAmbientTemp(DataArray, item, overlayDivElement);
          //this._addExhaustTemp(DataArray, item, overlayDivElement, false);
          this._addInputPower(DataArray, item, overlayDivElement, false);
          this._addOutputPower(DataArray, item, overlayDivElement, false);
        }

        this._addPowerCap(DataArray, item, overlayDivElement);       

        // tooltip only property
        if ( DataArray.TIME_LAST_POLLED && buildTooltip)      
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.TimeLastPolled, DataArray.TIME_LAST_POLLED );

      }
      
            // add element name
      if ( item.getType() == "com.ibm.sysmgt.platform.PhysicalPlatformFactory" ||
           item.getType() == "SystemChassis" ) 
        itmeUtils.insertEllipsedTextSpanIntoElement(overlayDivElement, item.getLabel(), {styleClass: "secondaryElementName"}); // el, class, text

      // add yet another div for critical status, the css class should give it a red or yellow hue            
      itmeUtils.createElement(
                item.mapElement.id + "_heat2", 
                item.mapElement,
                { styleClass: "HEATOverlayElementShadow "+colorClass,
                  baseDimensionsOnNode: item.mapElement} ) ; 
          
      this.myIndex++;
    }
    
  },

  _isNoRenderableData: function(dataArray) {
    if ( !dataArray )
      return true;
    if ( ( !dataArray.ENV_AMBIENTTEMP || (dataArray.ENV_AMBIENTTEMP < 0) ) &&  
         ( !dataArray.ENV_EXHAUSTTEMP || (dataArray.ENV_EXHAUSTTEMP < 0) ) &&
         ( !dataArray.ENV_AIRFLOW || (dataArray.ENV_AIRFLOW < 0) ) &&
         ( !dataArray.PWR_ACAVGINPUT || (dataArray.PWR_ACAVGINPUT < 0) ) &&
         ( !dataArray.PWR_DCAVGINPUT || (dataArray.PWR_DCAVGINPUT < 0) ) &&
         ( !dataArray.PWR_DCAVGOUTPUT || (dataArray.PWR_DCAVGOUTPUT < 0) ) &&
         ( !dataArray.PWR_CAP || (dataArray.PWR_CAP < 0) ) )
         return true;
     return false;
  }, 
  
  _mapAmbientTempToColorScale: function(dataArray) {
  	var colorClass = "GrayGradient";

    itmeUtils.CMTrace("OVERLAY", "tempRange: " + dataArray.ENV_AMBTEMP_RANGE);
      switch (dataArray.ENV_AMBTEMP_RANGE) {
        case "10":  colorClass = "BlueGradient";    break;
        case "20":  colorClass = "GreenGradient";   break;
        case "30":  colorClass = "Yellow1Gradient"; break;
        case "40":  colorClass = "Orange1Gradient"; break;
        case "50":  colorClass = "RedGradient";     break;
      }
      return colorClass;
  },

  _addAmbientTemp: function(dataArray, item, divElement, showLabel) {
    var label = "";
    if ( dataArray.ENV_AMBIENTTEMP && (dataArray.ENV_AMBIENTTEMP > 0) ) {
      if ( showLabel )
        label = this.MRI.AmbientTemp;
      divElement.innerHTML = divElement.innerHTML + label + " " + Math.round(dataArray.ENV_AMBIENTTEMP) + " " + dataArray.TEMP_UNIT+"<br>";
      if ( this._buildTooltip )
      item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.AmbientTemp, dataArray.ENV_AMBIENTTEMP + " " + dataArray.TEMP_UNIT);
    }
  },

  _addExhaustTempAndAirFlow: function(dataArray, item, divElement, isChassis) {
    var overlayDivElementChassisBack, chassisBackFaceNode = dojo.byId("chassisback");
    if ( dataArray.ENV_EXHAUSTTEMP && (dataArray.ENV_EXHAUSTTEMP > 0) ) {
      if (this._buildTooltip)
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ExhaustTemp, dataArray.ENV_EXHAUSTTEMP + " " + dataArray.TEMP_UNIT);
      
      if ( ! isChassis ) 
        divElement.innerHTML = divElement.innerHTML + "" + Math.round(dataArray.ENV_EXHAUSTTEMP) + " " + dataArray.TEMP_UNIT+"<br>";
      else { // Chassis is a special case, we build a special element for the back chassis frame and add this text to it
        overlayDivElementChassisBack = dojo.byId(item.mapElement.id + "_heatBackExhaust");
        if (!overlayDivElementChassisBack)
	        overlayDivElementChassisBack = itmeUtils.createElement(
	                 item.mapElement.id + "_heatBackExhaust", 
	                 dojo.byId("chassisback"),
	                 { styleClass: "HEATOverlayElement",
	                   baseDimensionsOnNode:chassisBackFaceNode } );

        overlayDivElementChassisBack.innerHTML = overlayDivElementChassisBack.innerHTML + this.MRI.ExhaustTemp + " " + Math.floor(dataArray.ENV_EXHAUSTTEMP) + " " + dataArray.TEMP_UNIT + "<br>";
      }
    }
    
    if ( dataArray.ENV_AIRFLOW && (dataArray.ENV_AIRFLOW > 0) ) {
        if (this._buildTooltip)
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.Airflow, dataArray.ENV_AIRFLOW + " " + this.MRI.CFM);
        
        if ( ! isChassis ) 
          divElement.innerHTML = divElement.innerHTML + "" + Math.round(dataArray.ENV_AIRFLOW) + " " + this.MRI.CFM+"<br>";
        else { // Chassis is a special case, we build a special element for the back chassis frame and add this text to it
            overlayDivElementChassisBack = dojo.byId(item.mapElement.id + "_heatBackExhaust");
            if (!overlayDivElementChassisBack)
              overlayDivElementChassisBack = itmeUtils.createElement(
                         item.mapElement.id + "_heatBackExhaust", 
                         dojo.byId("chassisback"),
                         { styleClass: "HEATOverlayElement",
                           baseDimensionsOnNode:chassisBackFaceNode } );

          overlayDivElementChassisBack.innerHTML = divElement.innerHTML + "" + Math.round(dataArray.ENV_AIRFLOW) + " " + this.MRI.CFM+"<br>";
        }
      }
    
  },

  _addInputPower: function(dataArray, item, divElement, isChassis) {
    
    if ( dataArray.PWR_DCAVGINPUT && (dataArray.PWR_DCAVGINPUT > 0) ) {
      if (this._buildTooltip)
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.PowerIn, dataArray.PWR_DCAVGINPUT + " " + dataArray.POWER_UNIT );
      divElement.innerHTML = divElement.innerHTML + "<span id='"+item.mapElement.id+"_inputPower'>" + "" + Math.round(dataArray.PWR_DCAVGINPUT) + " " + dataArray.POWER_UNIT+"</span>";
    }
    else if ( dataArray.PWR_ACAVGINPUT && (dataArray.PWR_ACAVGINPUT > 0) ) {
      if (this._buildTooltip)
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.PowerIn, dataArray.PWR_ACAVGINPUT + " " + dataArray.POWER_UNIT );
      if ( !isChassis )
        divElement.innerHTML = divElement.innerHTML + "" + Math.round(dataArray.PWR_ACAVGINPUT) + " " + dataArray.POWER_UNIT;
      else 
        divElement.innerHTML = divElement.innerHTML + "<span id='"+item.mapElement.id+"_inputPower' class='chassisBottomText'>"+ "" + Math.round(dataArray.PWR_ACAVGINPUT) + " " + dataArray.POWER_UNIT +"</span>";        
      
    }
    // TODO: Handle Chassis Specific rendering location.
    
    // LEFT OFF HERE
    //  - AC goes into the chassis, DC out.  All other components in a chassis is DC in
    //  - need to clean up remaining properties
    //  - merge RTC code to get Alex's fix.
    //  - Work on power capping, slot thermal readings... ... see AEMOverlayNotes.txt
    
        
        
  },

  _addOutputPower: function(dataArray, item, divElement, isChassis) {
    if ( (dataArray.PWR_DCAVGOUTPUT) && (dataArray.PWR_DCAVGOUTPUT != -1) ) {
      if (this._buildTooltip)
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.PowerOut, dataArray.PWR_DCAVGOUTPUT + " " + dataArray.POWER_UNIT );
      if ( !isChassis )
        divElement.innerHTML = divElement.innerHTML + "" + Math.round(dataArray.PWR_DCAVGOUTPUT) + " " + dataArray.POWER_UNIT + "<br>";
      //else {
        // divElement.innerHTML = divElement.innerHTML + "<span class='chassisBottomText'>"+ "" + Math.round(dataArray.PWR_DCAVGOUTPUT) + " " + dataArray.POWER_UNIT +"</span>";
      //}
    }
    
  },

  _addPowerCap: function(dataArray, item, divElement) {
    var elly, inputPower;
    if ( (dataArray.PWR_CAP) && (dataArray.PWR_CAP != -1) ) {
      
      elly = dojo.byId(item.mapElement.id+"_inputPower");
      if ( !elly )
        elly = divElement;
      elly.innerHTML = elly.innerHTML + " / <span class='powerCapText'>" + dataArray.PWR_CAP + " " + dataArray.POWER_UNIT + "</span>";
      if (this._buildTooltip)
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.PowerCap, dataArray.PWR_CAP + " " + dataArray.POWER_UNIT );
      
      inputPower = 0;
      // get input power - compare to cap - if within 10%, colorize text
      if ( dataArray.PWR_DCAVGINPUT && (dataArray.PWR_DCAVGINPUT > 0) ) 
        inputPower = Math.round(dataArray.PWR_DCAVGINPUT); 
      else if ( dataArray.PWR_ACAVGINPUT && (dataArray.PWR_ACAVGINPUT > 0) ) 
        inputPower = Math.round(dataArray.PWR_ACAVGINPUT);
      // TEST:
      //inputPower = 700;
      itmeUtils.CMTrace("OVERLAY", "inputPower / Cap -   " + inputPower + " / " + dataArray.PWR_CAP + " = "+ ( inputPower / dataArray.PWR_CAP));
      if ( (inputPower / dataArray.PWR_CAP) > 0.9) 
        dojo.addClass(elly, "powerCapWarning");
      //else if ((inputPower / dataArray.PWR_CAP) > 0.99) 
      //  dojo.addClass(elly, "powerCapReached");
    }
  },

  // Unrender the overlay.  Parent function should suffice unless there is general, non-item specific things to deal with
  unRenderOverlay: function() {
	  this.inherited(arguments);
	  itmeUtils.CMTrace("OVERLAY", "unRenderOverlay (HEAT) ");
	  
	  var pieChartDivElement = dojo.byId("subsysPieChart");
	  if ( pieChartDivElement) 
		  pieChartDivElement.style.display = "none";

  },
  
  _unsetOverlayForItem: function(item)
  // Hide what was rendered by renderOverlay.  Override this if:
    // 1. you did not use "_"+_overlayId to append element divs  or...
    // 2. you need to unrender more than 1 created div
    // 3. you do not want to unrender "_"+_overlayId default that this base class does
  {
    this.inherited(arguments);
    itmeUtils.CMTrace("OVERLAY", "_unsetOverlayForItem (HEAT): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "2");
    if ( itemsDivElement) 
        itemsDivElement.style.display = "none";
    if ( item.getType() == "SystemChassis" ) {
        itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "BackExhaust");
        if ( itemsDivElement) 
            itemsDivElement.style.display = "none";
    }
  },
  
  // Delete the overlay.  Parent function should suffice unless there is general, non-item specific things to deal with
  deleteOverlay: function() {
    this.inherited(arguments);
    itmeUtils.CMTrace("OVERLAY", "deleting overlay (HEAT) ");

    if ( dijit.byId('subsysPieChart') )
      dijit.byId('subsysPieChart').destroyRecursive();
  },
  
  _deleteOverlayForItem: function(item) {
    this.inherited(arguments);
    itmeUtils.CMTrace("OVERLAY", "_deleteOverlayForItem (HEAT): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "2");
    if ( itemsDivElement)
      itemsDivElement.parentNode.removeChild(itemsDivElement);
    if ( item.getType() == "SystemChassis" ) {
      itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "BackExhaust");
      if ( itemsDivElement)
        itemsDivElement.parentNode.removeChild(itemsDivElement);
    }
  },
  
  _convertFahrenheitToCelsius: function(f) {
    // F = (9/5 * C) + 32
    // C = F - 32 * 5/9;
    return f - 32 * 5 / 9;
  }

  
  // itemInformationModified: function(item)
  // Unused
  
  // getId: function()
  // Return the overlay id.  This is set in the constructor and should not need overriding.

  // getTitle: function() {
  // Return the title for the overlay.  This is set in the constructor and should not need overriding.

  // getDescription: function() {
  // Return the description for the overlay.  This is set in the constructor and should not need overriding.
  
  // requiresProperties: function() {
  // Unused
  // Gets the required properties for this overlay.  This is set in the constructor and should not need overriding.
  
  // setItems: function(items) {
  // Sets the items for the overlay. Parent function should suffice unless special processing is needed 
  
});