dojo.provide("itme.ChassisMap.OverlayBase");

dojo.require("dojo.i18n");
dojo.require("itme.ChassisMap.ImageResources");


/* Overlay interface rules 
 * 
 * Required Public Functions: 
 *   constructor(kwArgs , items (opt) ); 
 *   getId();     // return unique string for the overlay
 *   getTitle();  // return translatable string for the overlay
 *   getDescription(); // return translatable string describing the overlays purpose
 *   requiresProperties(); // return the dataset properties required for this overlay
 *   setItems(items); // chassis items list.  Note items.mapElement (if it exists) is the dom element associated with the item.
 *   renderOverlay(); // render or show the overlay
 *   unRenderOverlay(); // remove or hide the overlay rendering
 *   itemInformationModified(); NOT IMPLEMENTED YET // notification that info for an item has changed (re-render that element)
 *   
 *   // Guidelines for behavior
 *   // - for resizing to work, all created elements should become children of the items associated element (item.mapElement)
 *   // - unRender should hide created elements, so re-rendering only needs to show them instead of re-creating them.
 *   // - title and description should be translatable
 */

dojo.declare("itme.ChassisMap.OverlayBase", null, {
  _demoMode: false,
  _args: null,
  _items: null,
  _overlayId: null,
  _description: null,
  _title: null,
  _iconId: null,
  _requiredProperties: null,
  _availableFilters: null,
  _isEnabled: false,
  _flagNoTooltipRebuild: false,
  _hasBeenRendered: false,
  _isActive: null,
  MRI: null,
  IMAGES : new itme.ChassisMap.ImageResources(),
   
  constructor: function(kwArgs , items /* opt */){
    this._args = {    };
    dojo.requireLocalization("itme", "Resources");
    this.MRI = dojo.i18n.getLocalization("itme", "Resources");
    //this._elements = elements;
    this._items = items;
    dojo.mixin(this._args, kwArgs);
  },
  
  setEnabled: function(isEnabled) {
    this._isEnabled = isEnabled;
  },
  
  isEnabled: function() {
    return this._isEnabled;
  },
  
  /* dom usable id name */
  getId: function()
  {
    return this._overlayId;
  },
  
  getTitle: function() {
    return this._title;
  },
  
  getDescription: function() {
    return this._description;
  },
  
  getIconId: function() {
    return this._iconId;
  },
  
  getAvailableFilters: function() {
    return this._availableFilters;
  },
  
  // utils should be able to do a comparison between available and the return
  requiresProperties: function() {
    return this._requiredProperties;
  },
  
  setItems: function(items) {
    itmeUtils.CMTrace("OVERLAY", "setItems reached");
    this._items = items;
  },
  
  renderOverlay: function(includeTooltip){
    if ( typeof(includeTooltip) == 'undefined' ) {
      if ( !this._hasBeenRendered )
        includeTooltip = true;
      else
        includeTooltip = !this._flagNoTooltipRebuild; // default buildTooltip to true if not passed in
    }
    this._flagNoTooltipRebuild = false;
    this._isEnabled = true;
    
    // iterate through items... which have the associated dom elements set as item.mapElement
    itmeUtils.CMTrace("OVERLAY", "rendering "+this._overlayId+" overlay - demoMode? "+ this._demoMode + " - +tooltip? " + includeTooltip);
    this._preProcessing(this._items);
    for (var i = 0; i < this._items.length; i++) {
      if (this._items[i].mapElement) {
        this._renderItem(this._items[i], includeTooltip);
        this._items[i].mapElement.CMTooltip.toggleFocusEnabled(this._overlayId, true);
        this._items[i].mapElement.CMTooltip.buildTooltip();
      }
    }
    this._postProcessing(this._items);
    this._hasBeenRendered = true;
  },
  
  
  _renderItem: function(item) {
    // OVERRIDE REQUIRED
  },
  
  _preProcessing: function(items) {
    // OVERRIDE IF NEEDED
  },
  
  _postProcessing: function(items) {
    // OVERRIDE IF NEEDED
  },
  
  unRenderOverlay: function(){
    this._isEnabled = false;
    itmeUtils.CMTrace("OVERLAY", "unrendering (BASE) "+this._overlayId+" overlay");
    for (var i = 0; i < this._items.length; i++)   {
      if (this._items[i].mapElement) {
        this._unsetOverlayForItem(this._items[i]);
        this._items[i].mapElement.CMTooltip.toggleFocusEnabled(this._overlayId, false);
        this._items[i].mapElement.CMTooltip.buildTooltip();
      }
    }
  },
  
  deleteOverlay: function() {
    // iterates through each element created by this overlay and deletes them, to be re-rendered
    this.unRenderOverlay();
    this._flagNoTooltipRebuild = true; // this is because when deleting the overlay, we don't delete the tooltips
    
    for (var i = 0; i < this._items.length; i++)   {
      if (this._items[i].mapElement) {
        this._deleteOverlayForItem(this._items[i]);
        this._items[i].mapElement.CMTooltip.toggleFocusEnabled(this._overlayId, false);
        //this._items[i].mapElement.CMTooltip.buildTooltip();
      }
    }
    
  },

  //override this method if 
  // 1. you did not use "_"+_overlayId to append element divs  or...
  // 2. you need to unrender more than 1 created div
  // 3. you do not want to unrender "_"+_overlayId default that this one does
  _unsetOverlayForItem: function(item)
  {
    itmeUtils.CMTrace("OVERLAY", "_unsetOverlayForItem (BASE): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId);
    if ( itemsDivElement) 
      itemsDivElement.style.display = "none";
  },
  
  _deleteOverlayForItem: function(item) {
    //itmeUtils.CMTrace("OVERLAY", "_deleteOverlayForItem (BASE): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId);
    if ( itemsDivElement) 
      itemsDivElement.parentNode.removeChild(itemsDivElement);
  },
  
  itemInformationModified: function(item){
    /* override me */
  },
  
  // PROTECTED - for use as needed:
  _setDimensionsAndTextDirection: function(item, itemsDivElement){
      
      var HeightToWidthFactorForTextRotation = 1.5;
      
      var cBox = dojo.contentBox(item.mapElement);
      
      
      if ((cBox.h / cBox.w) < HeightToWidthFactorForTextRotation) 
        // no text rotation
        dojo.contentBox(itemsDivElement, {w: cBox.w, h: cBox.h});
      else {
        // rotate text 
        dojo.addClass(itemsDivElement, "verticalizeText");
        
        if (dojo.isFF || itmeUtils.isChrome()) { // MOZILLA & CHROME 
          // mozilla based browsers leave the box in a strange position... 
          // ... this is a hack to fix that when the css rotates it 
          //itmeUtils.CMTrace("OVERLAY", "FF & MOzilla based rendering chkpoint");
          dojo.contentBox(itemsDivElement, { w: cBox.h, h: cBox.w });
          
          var magicNumber = (cBox.h - cBox.w) / 2; // I don't quite understand why this works, but it does.
          itemsDivElement.style.left = "-" + magicNumber + "px";
          itemsDivElement.style.top = magicNumber + "px";
        }
        else { // IE
          //itmeUtils.CMTrace("OVERLAY", "IE based rendering chkpoint");
          dojo.contentBox(itemsDivElement, { w: cBox.w, h: cBox.h });
        }
      }
  },
  
  _addPropertyToTooltipIfAvailable: function(item, propId) {
    prop = item.getProperty(propId);
    if ( prop ) {
      itmeUtils.CMTrace("OVERLAY", item.mapElement.id + " : Prop [ "+prop.getLabel()+"("+propId+") ] = " + prop.getDisplayValue());
      item.mapElement.CMTooltip.addFocusProperty(this._overlayId, prop.getLabel(), prop.getDisplayValue());
    }
  },
  
  setOverlayActiveState: function(activeState) {
    this._isActive = activeState;
  },
  
  isOverlayActive: function() {
    return this._isActive
  }

  

});