dojo.provide("itme.ChassisMap._Overlays");

dojo.require("itme.ChassisMap.StatusOverlay");
dojo.require("itme.ChassisMap.PropertiesOverlay");
dojo.require("itme.ChassisMap.LEDOverlay");
dojo.require("itme.ChassisMap.EnvironmentalsOverlay");
dojo.require("itme.ChassisMap.AccessOverlay");
dojo.require("itme.ChassisMap.ComplianceOverlay");
dojo.require("itme.ChassisMap.ImageResources");

dojo.require("itme.DojoMenuOverride");
dojo.require("dojox.json.ref");
dojo.require("dijit.CheckedMenuItem");
dojo.require("dijit.form.ComboButton");
dojo.require("dijit.form.Button");
dojo.require("dijit.Tooltip");

var menuTimer = null;

dojo.declare("itme.ChassisMap._Overlays", null, {

  _currentOverlay: null,
  _overlays: [], 
  IMAGES : new itme.ChassisMap.ImageResources(),

  constructor: function(kwArgs){
    var StatOverlay = new itme.ChassisMap.StatusOverlay({}, null);
    this._overlays[0] = StatOverlay;
    var UpdatesOverlay = new itme.ChassisMap.ComplianceOverlay({}, null);
    this._overlays[1] = UpdatesOverlay;
    var AccessOverlay = new itme.ChassisMap.AccessOverlay({}, null);
    this._overlays[2] = AccessOverlay;
    var LEDOverlay = new itme.ChassisMap.LEDOverlay({}, null);
    this._overlays[3] = LEDOverlay;
    var NameOverlay = new itme.ChassisMap.PropertiesOverlay({}, null);
    this._overlays[4] = NameOverlay;
    var HeatOverlay = new itme.ChassisMap.EnvironmentalsOverlay({}, null);
    this._overlays[5] = HeatOverlay;
  },
  
  // protected
  _setItemsForOverlays: function(itemsList) {
    itmeUtils.CMTrace("OVERLAY", "overlay.setItems");
    for (var i = 0; i < this._overlays.length; i++)   
    {
      this._overlays[i].setItems(itemsList);
    }
  },
  
  // protected
  _renderDefaultOverlay: function() {
    var curOverlayId = dojo.cookie("chasisMap.currentOverlay");
    dojo.cookie("chasisMap.currentOverlay", null, { expires: -1}); // remove the cookie if it exists
    itmeUtils.CMTrace("OVERLAY", "COOKIE: removing cookie for overlay.  (was " + curOverlayId + ")");

    this._switchOverlays("status"); // always pre-render status for the colorization 
    if (curOverlayId) {
      this._switchOverlays(curOverlayId);
      this._currentOverlay = this._overlays[this._getOverlayIndex(curOverlayId)];
    }
  },
  
  // protected
  _switchOverlays: function(overlayId) {
    var overlayIndex = this._getOverlayIndex(overlayId);
    itmeUtils.CMTrace("OVERLAY", "switching overlay to: "+ overlayId);
    if (this._currentOverlay) {
      if (this._currentOverlay.getId() == overlayId) {
        itmeUtils.CMTrace("OVERLAY", "same overlay selected - not switching");
        return;
      }
      
      // unrender current overlay
      this._currentOverlay.unRenderOverlay();
      this._setButtonStyleUnpressed(this._currentOverlay.getId());
    }
    itmeUtils.CMTrace("OVERLAY", "switching overlays to: " + overlayId);
    
    // render selected overlay
    this._overlays[overlayIndex].renderOverlay();
    this._currentOverlay = this._overlays[overlayIndex];
    this._setButtonStylePressed(overlayId);
  },
  
  // protected
  _getOverlayIndex: function(overlayId)
  {
    for (var i = 0; i < this._overlays.length; i++)   
    {
      if ( this._overlays[i].getId() == overlayId )
        return i;
    }
  },
  

  // protected
  _insertOverlays: function(overlayFrameElement, itemsList){
    var i, buttonContainer, overlaysFilters, menu, labelString, button;
    // Add icons (as styles) for each overlay class
    for (i = 0; i < this._overlays.length; i++) 
      this._appendFilterButtonIconStyleToDocument(this._overlays[i].getId()+"_icon", this._overlays[i].getIconId());
    
    // build the menu button for each overlay class  
    for (i = 0; i < this._overlays.length; i++) {
      
      if ( this._isOverlayActive(this._overlays[i], itemsList) ) {

        buttonContainer = itmeUtils.createElement(
            this._overlays[i].getId() + "_buttonContainer", 
            overlayFrameElement, 
            { elementType: "span", 
              styleClass: "buttonContainer" } );
        
        overlaysFilters = this._overlays[i].getAvailableFilters();  
        
        button = dijit.byId(this._overlays[i].getId()+"_dojoButton");
        if ( button ) {
          button.destroyRecursive();
        }
        
        if ( overlaysFilters.length > 0 )
        { // build combobutton with menu flyout
          menu = new itme.DojoMenuOverride({
                style: "display: none;",
                onMouseLeave: function(e) { menuTimer = setTimeout(dojo.hitch(this, this.onCancel, true), 1000);},
                onMouseEnter: function(e) { clearTimeout(menuTimer); }
          });
          for ( var j = 0; j < overlaysFilters.length; j++ )
          {
            labelString = overlaysFilters[j].getDescription();
            if ( overlaysFilters[j].getIconId() )
              labelString = this.IMAGES.buildIconImageTag(overlaysFilters[j].getIconId(), "menuImage", 22) + overlaysFilters[j].getDescription();
            var menuItem1 = new dijit.CheckedMenuItem({
                label: labelString,
                disabled: (!overlaysFilters[j].allowModification()),
                checked: this._getFilterEnabledSettingFromCookieOrObject(this._overlays[i].getId(), overlaysFilters[j]),
                onClick: dojo.hitch(this, this._filterToggled, i, j)
            });
            menu.addChild(menuItem1);
            
            overlaysFilters[j].setEnabled( this._getFilterEnabledSettingFromCookieOrObject(this._overlays[i].getId(), overlaysFilters[j] ) );
          }
  
          itmeUtils.CMTrace("OVERLAY", "creating button: " + this._overlays[i].getId());
          button = new dijit.form.ComboButton({
              //label: this._overlays[i].getTitle(),
              dropDown: menu,
              optionsTitle: this._overlays[i].getDescription(),
              onClick: dojo.hitch(this, this._comboClicked, this._overlays[i].getId()),
              iconClass: this._overlays[i].getId()+"_icon",
              id: this._overlays[i].getId()+"_dojoButton",
              showLabel: false,
              type: "button",
              dropDownPosition: ["above"]
          });
          buttonContainer.appendChild(button.domNode);
          dojo.addClass(dojo.byId(this._overlays[i].getId()+"_dojoButton"), "overlayButton");
          this._setButtonStyleUnpressed(this._overlays[i].getId());
        }
        else
        { // build a standard dojo button
          itmeUtils.CMTrace("OVERLAY", "creating button: " + this._overlays[i].getId());
          button = new dijit.form.Button({
              optionsTitle: this._overlays[i].getDescription(),
              onClick: dojo.hitch(this, this._comboClicked, this._overlays[i].getId()),
              iconClass: this._overlays[i].getId()+"_icon",
              id: this._overlays[i].getId()+"_dojoButton",
              showLabel: false,
              type: "button"
          });
          buttonContainer.appendChild(button.domNode);
          dojo.addClass(dojo.byId(this._overlays[i].getId()+"_dojoButton"), "overlayButton");
          this._setButtonStyleUnpressed(this._overlays[i].getId());
        }     
          
        
        itmeUtils.createElement(
              "myButtonSeperator_"+i, 
              overlayFrameElement, 
              { styleClass: "buttonSeperator",
                innerHTML: this.IMAGES.buildIconImageTag("Seperator", "seperatorImg" + ((dojo.isIE) ? "" : " notIE"), 18) } );

        new dijit.Tooltip({
          connectId: [this._overlays[i].getId()+"_dojoButton"],
          label: this._overlays[i].getDescription(),
          position: "above",
          id: this._overlays[i].getId()+"_buttonTooltip"
        });
      }
    }
    itmeUtils.CMTrace("OVERLAY", "done inserting overlays");
  },
  
  
  _isOverlayActive: function(overlay, itemsList) {
    var requiredPropertiesArray, showOverlay, j, k, tempProp;
    
    if ( overlay.isOverlayActive() !== null ) 
      return overlay.isOverlayActive();
    else
    {
      itmeUtils.CMTrace("MAIN", "isOverlayActive: " + overlay.getId());
      showOverlay = true;
      requiredPropertiesArray = overlay.requiresProperties();
      if ( requiredPropertiesArray ) {
        for (j = 0; (j < requiredPropertiesArray.length) && (showOverlay==true) ; j++) {
          tempProp = itmeUtils.getProperty(itemsList[0], requiredPropertiesArray[j]);
          if ( (tempProp != null) && (tempProp == "DATA_COLLECTION_INACTIVE")) // no elements will have data
          {
            itmeUtils.CMTrace("MAIN", "overlay property is null or INACTIVE, NOT SHOWING");
            overlay.setOverlayActiveState(false);
            return false;
          }
          else if ((null == tempProp) || ("" == tempProp)) // maybe other elements in the list will have this property
          { 
            showOverlay = false;
            itmeUtils.CMTrace("MAIN", "prop for 1st element was empty, iterating over all items");
            for (k = 1; (k < itemsList.length) && (showOverlay == false); k++) { //  check other elements
              tempProp = itmeUtils.getProperty(itemsList[k], requiredPropertiesArray[j]);
              if ( (null != tempProp) && ("" != tempProp) )
                showOverlay = true;
            }
          }
        }
      }
      itmeUtils.CMTrace("MAIN", "isOverlayActive: " + overlay.getId() + " is returning - " + showOverlay);
      overlay.setOverlayActiveState(showOverlay);
      return showOverlay;
    }
  },
  
  _prefetchAsynchOverlayData: function(itemsList, alreadyFetchedProps) {
    var requiredPropertiesArray = [], i, j, overlayReqProps; 
    for (i = 0; i < this._overlays.length; i++) { 
      itmeUtils.CMTrace("OVERLAY", "getting props for overlay: " + this._overlays[i].getId());
      if ( this._isOverlayActive(this._overlays[i], itemsList) ) {
        overlayReqProps = this._overlays[i].requiresProperties();
        if ( overlayReqProps ) {
          // prefetch only those properties not in the default list 
          for (j = 0; j < overlayReqProps.length; j++) {
            if ( dojo.indexOf(alreadyFetchedProps, overlayReqProps[j]) < 0 )
              requiredPropertiesArray = requiredPropertiesArray.concat(overlayReqProps[j]);
          }
        }
      }
    }
      
    itmeUtils.CMTrace("MAIN", "prefetching overlay data for: " + requiredPropertiesArray);
    if ( requiredPropertiesArray ) {
      for (i = 0; (i < itemsList.length); i++) {
        itemsList[i].getProperties({properties: requiredPropertiesArray});
      }
    }
    itmeUtils.CMTrace("MAIN", "done prefetching overlay data for: " + requiredPropertiesArray);
  },
  
  
  _appendFilterButtonIconStyleToDocument: function(styleName, iconId) {
    
    var head = document.getElementsByTagName('head')[0];

    var style = document.createElement('style');
    var rules = document.createTextNode("."+styleName+" { background-image: url('"+this.IMAGES.iconPath(iconId, 22)+"'); background-repeat:no-repeat; width:23px; height:23px; margin:1px !important;}");
    style.type = 'text/css';
    if(style.styleSheet) {
      style.styleSheet.cssText = rules.nodeValue;
    } else { 
      style.appendChild(rules);
    }
    head.appendChild(style);
  },
  
  _comboClicked: function(overlayID) {
    try {
      this._switchOverlays(overlayID);
    } catch (err) { itmeUtils.CMTrace("OVERLAY", err); }
    return false;
  },
  
  _filterToggled: function(overlayIndex, filterIndex) {
    var wasEnabled = this._overlays[overlayIndex]._availableFilters[filterIndex].isEnabled();
    this._overlays[overlayIndex]._availableFilters[filterIndex].setEnabled(!wasEnabled);
    
    this._storeFilterStates();
    
    if ( this._overlays[overlayIndex].isEnabled() ) 
    {
      this._overlays[overlayIndex].deleteOverlay();
      this._overlays[overlayIndex].renderOverlay(false);
    }
    else
      this._overlays[overlayIndex].deleteOverlay();
    
  },
  
  _storeFilterStates: function() {
    var objOfOverlaysAndFilterStates = {};
    objOfOverlaysAndFilterStates.overlays = [];
    for (var i = 0; i < this._overlays.length; i++)   
    {
      var arrayOfFilters = [];
      var overlaysFilters = this._overlays[i].getAvailableFilters();  
      if ( overlaysFilters.length > 0 ) {
    	for ( var j = 0; j < overlaysFilters.length; j++ ) {
    	  arrayOfFilters[j] = {"id": overlaysFilters[j].getId(),
    						   "enabled": overlaysFilters[j].isEnabled()};
    	}
      }
      objOfOverlaysAndFilterStates.overlays[i] = {"id": this._overlays[i].getId(),
    		  									  "filters": arrayOfFilters};
    }
    myJsonString = dojox.json.ref.toJson(objOfOverlaysAndFilterStates);
    dojo.cookie("chasisMap.overlayFilterSettings", myJsonString, { expires: 365 }); 
    itmeUtils.CMTrace("OVERLAY", myJsonString);
  },
  
  _getFilterEnabledSettingFromCookieOrObject: function(overlayId, filterObject) {
    var currentFilterSettingsJSON = dojo.cookie("chasisMap.overlayFilterSettings");
    var currentFilterSettings = dojox.json.ref.fromJson(currentFilterSettingsJSON);
    if ( currentFilterSettings ) {
      for (i = 0; i < currentFilterSettings.overlays.length; i++) {
        if ( currentFilterSettings.overlays[i].id == overlayId) {
          for (j = 0; j < currentFilterSettings.overlays[i].filters.length; j++) {
            if (currentFilterSettings.overlays[i].filters[j].id == filterObject.getId())
              return currentFilterSettings.overlays[i].filters[j].enabled;
          }
        }
      }
    }
    return filterObject.isEnabled();
  },
  
  _setButtonStyleUnpressed: function(overlayID) {

    this._toggleButtonPressedStyleForElement(dojo.byId(overlayID+"_dojoButton_button"), false);
    this._toggleButtonPressedStyleForElement(dojo.byId(overlayID+"_dojoButton_arrow"), false);

    var simpleButtonElement = dojo.byId(overlayID+"_dojoButton");
    if ( simpleButtonElement )
      this._toggleButtonPressedStyleForElement(simpleButtonElement, false);
  },

  _setButtonStylePressed: function(overlayID) {
    this._toggleButtonPressedStyleForElement(dojo.byId(overlayID+"_dojoButton_button"), true);
    this._toggleButtonPressedStyleForElement(dojo.byId(overlayID+"_dojoButton_arrow"), true);

    var simpleButtonElement = dojo.byId(overlayID+"_dojoButton");
    if ( simpleButtonElement )
      this._toggleButtonPressedStyleForElement(simpleButtonElement, true);
  },
  
  _toggleButtonPressedStyleForElement: function(buttonElement, isPressed) 
  {
    if ( ! buttonElement )
      return;
      
    dojo.removeClass(buttonElement, (isPressed) ? "toggleButtonUnPressed" : "toggleButtonPressed");
    dojo.addClass(buttonElement, (isPressed) ? "toggleButtonPressed" : "toggleButtonUnPressed");
  }
  
});
