/*
	Copyright (c) 2004-2010, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["itme.ChassisMap.AccessOverlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.AccessOverlay"] = true;
dojo.provide("itme.ChassisMap.AccessOverlay");

dojo.require("itme.ChassisMap.OverlayBase");
dojo.require("itme.ChassisMap.OverlayFilterItem");

dojo.declare("itme.ChassisMap.AccessOverlay", [itme.ChassisMap.OverlayBase], {

  _diffForBorder: null,

  constructor: function(kwArgs){
    // parent constructor is always auto-called first
    this._overlayId = "access";
    this._description = this.MRI.AccessOverlayDetail;
    this._title = this.MRI.AccessTitle;
    this._requiredProperties = [ "AccessState" ];
    this._iconId = "AccessOnlineNone";
    
    this._availableFilters = [];
    this._okAccessOnlineFilter = new itme.ChassisMap.OverlayFilterItem({}, "okAccessOnline", "AccessOnlineOK", this.MRI.OKAccessOnline, true, true); 
    this._availableFilters[0] = this._okAccessOnlineFilter; 
    this._okAccessOfflinefilter = new itme.ChassisMap.OverlayFilterItem({}, "okAccessOffline", "AccessOfflineOK", this.MRI.OKAccessOffline, true, true); 
    this._availableFilters[1] = this._okAccessOfflinefilter; 
    this._partialAccessOnlineFilter = new itme.ChassisMap.OverlayFilterItem({}, "partialAccessOnline", "AccessOnlinePartial", this.MRI.PartialAccessOnline, true, true); 
    this._availableFilters[2] = this._partialAccessOnlineFilter; 
    this._partialAccessOfflinefilter = new itme.ChassisMap.OverlayFilterItem({}, "partialAccessOffline", "AccessOfflinePartial", this.MRI.PartialAccessOffline, true, true); 
    this._availableFilters[3] = this._partialAccessOfflinefilter; 
    this._noAccessOnlineFilter = new itme.ChassisMap.OverlayFilterItem({}, "noAccessOnline", "AccessOnlineNone", this.MRI.NoAccessOnline, true, true);
    this._availableFilters[4] = this._noAccessOnlineFilter;
    this._noAccessOfflineFilter = new itme.ChassisMap.OverlayFilterItem({}, "noAccessOffline", "AccessOfflineNone", this.MRI.NoAccessOffline, true, true );
    this._availableFilters[5] = this._noAccessOfflineFilter;
    this._unknownAccessFilter = new itme.ChassisMap.OverlayFilterItem({}, "unknownAccess", "AccessUnknown", this.MRI.UnknownAccess, true, true); 
    this._availableFilters[6] = this._unknownAccessFilter; 
    this._NAAccessFilter = new itme.ChassisMap.OverlayFilterItem({}, "NAAccess", "AccessNotApplicable", this.MRI.NAAccess, false, true); 
    this._availableFilters[7] = this._NAAccessFilter; 
    this._NotTrustedFilter = new itme.ChassisMap.OverlayFilterItem({}, "notTrusted", "AccessNotTrusted", this.MRI.NotTrustedAccess, true, true); 
    this._availableFilters[8] = this._NotTrustedFilter; 

  },
  
  _test: 100,
  _renderItem: function(item, buildTooltip){
    
    var itemsAccessDivElement = dojo.byId(item.mapElement.id + "_access");
    
    // if we've already created the element before just show it, otherwise create it
    if (itemsAccessDivElement) {
      itemsAccessDivElement.style.display = "block";
    }
    else {
      // create element
      itemsAccessDivElement = itmeUtils.createElement(item.mapElement.id + "_access",
                                                      item.mapElement,
                                                      { styleClass: "defaultStatus",
                                                        baseDimensionsOnNode: item.mapElement} ); 
      
      // add element name
      if ( item.getType() == "com.ibm.sysmgt.platform.PhysicalPlatformFactory" ||
           item.getType() == "SystemChassis" ) { 
        itmeUtils.insertEllipsedTextSpanIntoElement(itemsAccessDivElement, item.getLabel(), {styleClass: "secondaryElementName"}); // el, class, text
      }

      
      var accessState = itmeUtils.getProperty(item, "AccessState");
      if ( !accessState )
        itmeUtils.CMTrace("OVERLAY", "No Access property on item.  Nothing to render");
      else {
        
        itmeUtils.CMTrace("OVERLAY", "ITEM Access State ID: " + accessState.getId() );
        itmeUtils.CMTrace("OVERLAY", "ITEM Access State Value: " + accessState.getValue() );
        itmeUtils.CMTrace("OVERLAY", "ITEM Access State Img: " + accessState.getImage() );
        itmeUtils.CMTrace("OVERLAY", "ITEM Access State Scalar Img: " + accessState.getScalarImage() );
      
        //itmeUtils.CMTrace("OVERLAY", "Affecting CMTooltip : " + item.mapElement.CMTooltip);
  
        var imgIdString = "";
        var showMe = false;
        var switchValue = accessState.getValue();
        if ( this._demoMode )
          switchValue = this._test++;
        switch (switchValue) {//(accessState.getValue()) {
          case 100: 
            imgIdString = "AccessOnlineNone";
            showMe = this._noAccessOnlineFilter.isEnabled(); 
            break;
          case 101: 
            imgIdString = "AccessOfflineNone"; 
            showMe = this._noAccessOfflineFilter.isEnabled();
            break;
          case 102: 
            imgIdString = "AccessOnlinePartial"; 
            showMe = this._partialAccessOnlineFilter.isEnabled();
            break;
          case 103: 
            imgIdString = "AccessOfflinePartial"; 
            showMe = this._partialAccessOfflinefilter.isEnabled();
            break;
          case 104: 
            imgIdString = "AccessOnlineOK"; 
            showMe = this._okAccessOnlineFilter.isEnabled();
            break;
          case 105: 
            imgIdString = "AccessOfflineOK"; 
            showMe = this._okAccessOfflinefilter.isEnabled();
            break;
          case 106: 
            imgIdString = "AccessUnknown"; 
            showMe = this._unknownAccessFilter.isEnabled();
            break;
          case 107: 
            imgIdString = "AccessNotApplicable"; 
            showMe = this._NAAccessFilter.isEnabled();
            break;
          case 108: 
            //imgIdString = "AccessNotApplicable"; 
            //showMe = this._NAAccessFilter.isEnabled();
            break;
          case 109: 
            imgIdString = "AccessNotTrusted"; 
            showMe = this._NotTrustedFilter.isEnabled();
            break;
          default: break;
        }

        if ( showMe ) {  
          var iconSize = 26;
          if ( item.mapElement.constraint < 26 )
            iconSize = item.mapElement.constraint;
          itemsAccessDivElement.innerHTML = this.IMAGES.buildIconImageTag(imgIdString, 'accessImage', iconSize) + itemsAccessDivElement.innerHTML;
        }
        
        if ( buildTooltip )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.AccessState, this.IMAGES.buildIconImageTag(imgIdString, 'accessImage', "16") + " " + accessState.getLabel() );
      }
    }

  },

  _calculateBorderDiff: function(cBox, element) {
    this._diffForBorder = 4; // HACK:  IE failed to calculate border correctly.  CSS Margin = 2.  2x2 = 4.
    if (this._diffForBorder) {
      itmeUtils.CMTrace("OVERLAY", "calc border diff returning : " + this._diffForBorder);
      return this._diffForBorder;
    }
      
    var mBox = dojo.marginBox(element);
    this._diffForBorder = mBox.w - cBox.w;
    return this._diffForBorder;
  },
  
  // renderOverlay: function(){
  // Default preprocessing for rendering.  Calls into the REQUIRED OVERRIDE method _renderItem
  
  // unRenderOverlay: function()
  // Unrender the overlay.  Parent function should suffice for all cases
  
  _unsetOverlayForItem: function(item)
  // Hide what was rendered by renderOverlay.  Override this if:
    // 1. you did not use "_"+_overlayId to append element divs  or...
    // 2. you need to unrender more than 1 created div
    // 3. you do not want to unrender "_"+_overlayId default that this base class does
  {
    this.inherited(arguments);
    //itmeUtils.CMTrace("OVERLAY", "_unsetOverlayForItem (STATUS): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "2");
    if ( itemsDivElement) 
      itemsDivElement.style.display = "none";
    this._test = 100;
  },
  
  _deleteOverlayForItem: function(item) {
    this.inherited(arguments);
    //itmeUtils.CMTrace("OVERLAY", "_deleteOverlayForItem (BASE): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+"2");
    if ( itemsDivElement) 
      itemsDivElement.parentNode.removeChild(itemsDivElement);
  }
  
  // itemInformationModified: function(item)
  // Unused
  
  // getId: function()
  // Return the overlay id.  This is set in the constructor and should not need overriding.

  // getTitle: function() {
  // Return the title for the overlay.  This is set in the constructor and should not need overriding.

  // getDescription: function() {
  // Return the description for the overlay.  This is set in the constructor and should not need overriding.
  
  // requiresProperties: function() {
  // Unused
  // Gets the required properties for this overlay.  This is set in the constructor and should not need overriding.
  
  // setItems: function(items) {
  // Sets the items for the overlay. Parent function should suffice unless special processing is needed 
  
  
});

}
