/*
	Copyright (c) 2004-2010, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["itme.ChassisMap.ComplianceOverlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.ComplianceOverlay"] = true;
dojo.provide("itme.ChassisMap.ComplianceOverlay");

dojo.require("itme.ChassisMap.OverlayBase");
dojo.require("itme.ChassisMap.OverlayFilterItem");

dojo.declare("itme.ChassisMap.ComplianceOverlay", [itme.ChassisMap.OverlayBase], {
   
  constructor: function(kwArgs){
    // parent constructor is always auto-called first
    this._overlayId = "updates";
    this._description = this.MRI.ComplianceOverlayTitle;
    this._title = this.MRI.ComplianceOverlayTitle;
    this._iconId = "ComplianceIcon_22";
    //this._requiredProperties = [ "COMPLIANCEINFO" ];
    
    this._availableFilters = [];
    
    this._demoTestCounter = 0;
  },
  
  _renderItem: function(item){

    var frmProps = dojo.fromJson( itmeUtils.getProperty(item, "OTHERHWVIEWPROPS") );
    
    if ( item.getType() == "SystemChassis")
      return; // no-op on System Chassis node
      
    var overlayDivElement = dojo.byId(item.mapElement.id + "_updates");
    
    // if we've already created the element before just show it, otherwise create it
    if ( overlayDivElement)  
      overlayDivElement.style.display = "block";
    else { 
      // create it
      overlayDivElement = itmeUtils.createElement(
            item.mapElement.id + "_updates", 
            item.mapElement,
            { styleClass: "UpdatesOverlayElement" } );

      iconSize = 22;
      if ( item.mapElement.constraint < 22 )
        iconSize = item.mapElement.constraint;

      if ( frmProps.FIRMWARE ) {
        for ( var i = 0; i < frmProps.FIRMWARE.length; i++) {
          //item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.FirmwareLevel,  frmProps.FWLEVEL);
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, "Firmware: " + frmProps.FIRMWARE[i].FWTYPE, frmProps.FIRMWARE[i].FWLEVEL + " &nbsp&nbsp&nbsp&nbsp " + frmProps.FIRMWARE[i].FWDATE);
        }
      }
      
      var complianceData = dojo.fromJson( itmeUtils.getProperty(item, "COMPLIANCEINFO") ); 
      var frmProps = dojo.fromJson( itmeUtils.getProperty(item, "OTHERHWVIEWPROPS") );
      
      if ( complianceData.COMPLIANCELEVEL == "HEALTHY" )
      {
        overlayDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusNormal", "", iconSize) + "<span class='textBackgroundShadow'> " + this.MRI.Healthy + "</span>";
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ComplianceStatus, this.IMAGES.buildIconImageTag("StatusNormal", "", 32) + " " + this.MRI.Healthy );        
      }  
      else if ( complianceData.COMPLIANCELEVEL == "INFO" )
      {
        overlayDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusInformational", "", iconSize) + "<span class='textBackgroundShadow'> " + this.MRI.Informational + "</span>";
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ComplianceStatus, this.IMAGES.buildIconImageTag("StatusInformational", "", 32) + " " + this.MRI.Informational );        
      }
      else if ( complianceData.COMPLIANCELEVEL == "WARNING" )
      {
        overlayDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusWarningMinor", "", iconSize) + "<span class='textBackgroundShadow'> " + this.MRI.Warnings + "</span>";
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ComplianceStatus, this.IMAGES.buildIconImageTag("StatusWarningMinor", "", 32) + " " + this.MRI.Warnings );        
      }
      else if ( complianceData.COMPLIANCELEVEL == "ERROR" )
      {
        overlayDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusCritical", "", iconSize) + "<span class='textBackgroundShadow'> " + this.MRI.Errors + "</span>";
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ComplianceStatus, this.IMAGES.buildIconImageTag("StatusCritical", "", 32) + " " + this.MRI.Errors );        
      }
      else if ( complianceData.COMPLIANCELEVEL == "NOPOLICY" )
      {      
        // use the nodata style for NO POLICY      
        dojo.addClass(overlayDivElement, "NoData");
        dojo.addClass(overlayDivElement, "elementName");
        if ( item.getType() != "Fan" && 
             item.getType() != "PowerSupply" &&
             item.getType() != "PhysicalPackage" ) {
          overlayDivElement.innerHTML = this.MRI.NoPolicy;
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ComplianceStatus, this.MRI.NoCompliancePolicy );
        } else
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ComplianceStatus, this.MRI.NotApplicable );
      }
      
      if ( item.getType() == "com.ibm.sysmgt.platform.PhysicalPlatformFactory") {
        if ( complianceData.BLADEWITHOS == false ) {
          overlayDivElement.innerHTML = overlayDivElement.innerHTML + "<br>" + this.IMAGES.buildIconImageTag("StatusInformational", "", iconSize) + "<span class='textBackgroundShadow'> " + this.MRI.NoOSDiscovered + "</span>";
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("StatusInformational", "", 22),  this.MRI.NoOSDiscoveredUpdates);
        }
      }

      // verticalize text      
      this._setDimensionsAndTextDirection(item, overlayDivElement);  
    }

  }
  
  // renderOverlay: function(){
  // Default preprocessing for rendering.  Calls into the REQUIRED OVERRIDE method _renderItem
  
  // unRenderOverlay: function()
  // Unrender the overlay.  Parent function should suffice for all cases
  
  // _unsetOverlayForItem: function(item)
  // Hide what was rendered by renderOverlay.  Override this if:
    // 1. you did not use "_"+_overlayId to append element divs  or...
    // 2. you need to unrender more than 1 created div
    // 3. you do not want to unrender "_"+_overlayId default that this base class does
  
  // itemInformationModified: function(item)
  // Unused
  
  // getId: function()
  // Return the overlay id.  This is set in the constructor and should not need overriding.

  // getTitle: function() {
  // Return the title for the overlay.  This is set in the constructor and should not need overriding.

  // getDescription: function() {
  // Return the description for the overlay.  This is set in the constructor and should not need overriding.
  
  // requiresProperties: function() {
  // Unused
  // Gets the required properties for this overlay.  This is set in the constructor and should not need overriding.
  
  // setItems: function(items) {
  // Sets the items for the overlay. Parent function should suffice unless special processing is needed 
  
});

}
