/*
	Copyright (c) 2004-2010, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["itme.ChassisMap.LEDOverlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.LEDOverlay"] = true;
dojo.provide("itme.ChassisMap.LEDOverlay");

dojo.require("itme.ChassisMap.OverlayBase");
dojo.require("itme.ChassisMap.OverlayFilterItem");

/*
 * Inherit from base overlay class itme.ChassisMapOverlay.
 * Declare the following variables in the constructor:
 *   _overlayId
 *   _description
 *   _title
 *   _requiredProperties (unimplemented)
 */
dojo.declare("itme.ChassisMap.LEDOverlay", [itme.ChassisMap.OverlayBase], {
  myIndex: 0,

  constructor: function(kwArgs){
    // parent constructor is always auto-called first
    this._overlayId = "led";
    this._description = "Highlight front panel LEDs";
    this._title = this.MRI.LEDTitle;
    this._iconId = "LEDFilterIcon";
    this._requiredProperties = ["LEDSERVICEINFO"];

    this._availableFilters = [];
    this._powerFilter = new itme.ChassisMap.OverlayFilterItem({}, "powerFilter", "LEDPowerOn", this.MRI.PowerLED, true, true );
    this._availableFilters[0] = this._powerFilter;
    this._errorFilter = new itme.ChassisMap.OverlayFilterItem({}, "errorFilter", "LEDErrorLogOn", "Event Log LED", true, true);
    this._availableFilters[1] = this._errorFilter;
    this._locationFilter = new itme.ChassisMap.OverlayFilterItem({}, "locationFilter", "LEDLocationOn", "Location LED", false, true);
    this._availableFilters[2] = this._locationFilter;
    this._faultFilter = new itme.ChassisMap.OverlayFilterItem({}, "faultFilter", "LEDFaultOn", "Fault LED", true, true);
    this._availableFilters[3] = this._faultFilter;
    // DHM: Other LED filter
    this._otherFilter = new itme.ChassisMap.OverlayFilterItem({}, "trayOpenFilter", "LEDTrayOpenOn", "Tray Open LED", true, true);
    this._availableFilters[3] = this._faultFilter;
    // **

    this._onlyOnFilter = new itme.ChassisMap.OverlayFilterItem({}, "onlyOnFilter", null, "Only Show ON", true, true);
    this._availableFilters[4] = this._onlyOnFilter;
  },

  _renderItem: function(item, buildTooltip){

    var overlayDivElement = dojo.byId(item.mapElement.id + "_led");

    // if we've already created the element before just show it, otherwise create it
    if ( overlayDivElement)  {
      overlayDivElement.style.display = "block";
      if ( dojo.byId(item.mapElement.id + "_led2") )
        dojo.byId(item.mapElement.id + "_led2").style.display = "block";
    }
    else {
      // create it
      overlayDivElement = itmeUtils.createElement(
            item.mapElement.id + "_led",
            item.mapElement,
            { styleClass: "LEDOverlayElement" } );

      this._addNameElement(item);
      //Set all LED's to unknown first
      var powerLedStatus = 3;
      var powerLedDescription = '';
      var eventLogLedStatus = 3;
      var eventLogLedDescription = '';
      var locationLedStatus = 3;
      var locationLedDescription = '';
      var faultLedStatus = 3;
      var faultLedDescription = '';
      var LEDStatus = dojo.fromJson(itmeUtils.getProperty(item, "LEDSERVICEINFO"));
      if ( LEDStatus && LEDStatus.LEDS ) {
        itmeUtils.CMTrace("OVERLAY", "LEDSERVICEINFO on item ("+item.mapElement.id+") = " + LEDStatus.LEDS);
        itmeUtils.CMTrace("OVERLAY", "LEDSERVICEINFO on item ("+item.mapElement.id+") = .length" + LEDStatus.LEDS.length);
        for (i = 0; i <  LEDStatus.LEDS.length; i++) {
          itmeUtils.CMTrace("OVERLAY", "LED: " + i);
          var currentLedDesc = this._getDisplayableLed(LEDStatus.LEDS[i].DESCRIPTION, item.getType());
          if(currentLedDesc == 'Power Led') {
              powerLedStatus = this._getLedStatus(LEDStatus.LEDS[i].STATE);
              itmeUtils.CMTrace("OVERLAY", "powerLedStatus: " + powerLedStatus);
          } else if(currentLedDesc == 'Event Log Led') {
              eventLogLedStatus = this._getLedStatus(LEDStatus.LEDS[i].STATE);
              itmeUtils.CMTrace("OVERLAY", "eventLogLedStatus: " + eventLogLedStatus);
          } else if(currentLedDesc == 'Location Led') {
              locationLedStatus = this._getLedStatus(LEDStatus.LEDS[i].STATE);
              itmeUtils.CMTrace("OVERLAY", "locationLedStatus: " + locationLedStatus);
          } else if(currentLedDesc == 'Fault Led') {
              faultLedStatus = this._getLedStatus(LEDStatus.LEDS[i].STATE);
              itmeUtils.CMTrace("OVERLAY", "faultLedStatus: " + faultLedStatus);
          }
        }
      }

      // set contents



      overlayDivElement.innerHTML = "";

      var iconSize = 22;
      if ( item.mapElement.constraint < 22 )
          iconSize = item.mapElement.constraint;

      if ( this._powerFilter.isEnabled() )
      { if ( powerLedStatus == 1 )
          overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDPowerOn", "", iconSize);
        else if ( !this._onlyOnFilter.isEnabled() )
          if(powerLedStatus == 0) {
              overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDPowerOff", "", iconSize);
          } else if ( eventLogLedStatus == 2 ) { //TODO: handle binking status, for now pretend it's same as ON
              overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDPowerOn", "", iconSize);
          } else { //TODO: handle unknown status

          }
      }
      if ( this._errorFilter.isEnabled() )
      { if ( eventLogLedStatus == 1 )
          overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDErrorLogOn", "", iconSize);
        else if ( !this._onlyOnFilter.isEnabled() )
            if(eventLogLedStatus == 0) {
              overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDErrorLogOff", "", iconSize);
            } else if ( eventLogLedStatus == 2 ) { //TODO: handle binking status, for now pretend it's same as ON
                overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDErrorLogOn", "", iconSize);
            } else { //TODO: handle unknown status

            }
      }
      if ( this._locationFilter.isEnabled() ) {
        if ( locationLedStatus == 1 || locationLedStatus == 2) { // Treat blink same as on
          overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDLocationOn", "", iconSize);
        } else if ( !this._onlyOnFilter.isEnabled() ) { // if we should show all
          if ( locationLedStatus == 0 ) {
            overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDLocationOff", "", iconSize);
          }
          else {
            // ignore unknown status
          }
        }
      }

      if ( this._faultFilter.isEnabled() )
      { if ( faultLedStatus == 1 )
          overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDFaultOn", "", iconSize);
        else if ( !this._onlyOnFilter.isEnabled() )
            if(faultLedStatus == 0) {
                 overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDFaultOff", "", iconSize);
      } else if ( eventLogLedStatus == 2 ) { //TODO: handle binking status, for now pretend it's same as ON
          overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDFaultOn", "", iconSize);
      } else { //TODO: handle unknown status

      }
      }


      if ( buildTooltip )
      {
        if ( powerLedStatus == 1 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDPowerOn", "", 22) + " Power LED", "On" );
        else if ( powerLedStatus == 0 )
            item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDPowerOff", "", 22) + " Power LED", "Off" );
        else if ( powerLedStatus == 2 )
            item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDPowerOn", "", 22) + " Power LED", "Blinking" );

        if ( eventLogLedStatus == 1 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDErrorLogOn", "", 22) + " Event Log LED", "On" );
        else if ( eventLogLedStatus == 0 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDErrorLogOff", "", 22) + " Event Log LED", "Off" );
        else if ( powerLedStatus == 2 )
            item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDErrorLogOn", "", 22) + " Event Log LED", "Blinking" );

        if ( locationLedStatus == 1 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDLocationOn", "", 22) + " Location LED", "On" );
        else if ( locationLedStatus == 0 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDLocationOff", "", 22) + " Location LED", "Off" );
        else if ( locationLedStatus == 2 )
            item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDLocationOn", "", 22) + " Location LED", "Blinking" );

        if ( faultLedStatus == 1 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDFaultOn", "", 22) + " Fault LED", "On" );
        else if ( faultLedStatus == 0 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDFaultOff", "", 22) + " Fault LED", "Off" );
        else if ( powerLedStatus == 2 )
            item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDFaultOn", "", 22) + " Fault LED", "Blinking" );


      }


      //itmeUtils.CMTrace("OVERLAY", "Affecting CMTooltip : " + item.mapElement.CMTooltip);
      this.myIndex++;
    }
  },
  _getLedStatus: function(ledState) {
      if(ledState == 'Off') {
          return 0;
      } else if(ledState == 'On') {
          return 1;
      } else if(ledState == 'Blinking') {
          return 2;
      } else if(ledState == 'Unknown') {
          return 3;
      }
  },
  _getDisplayableLed: function(ledDescription, resourceType) {
      itmeUtils.CMTrace("OVERLAY", "description: " + ledDescription + ", type: " + resourceType);


      if (ledDescription == null || ledDescription == "") {
          return null;
      }
      if (resourceType === "Chassis" || resourceType === "SystemChassis") {
          return this._getDisplayableChassisLed(ledDescription);
      } else if (resourceType === "com.ibm.sysmgt.platform.PhysicalPlatformFactory") {
          return this._getDisplayableBladeLed(ledDescription);
      } else if (resourceType === "PowerSupply") {
          return this._getDisplayablePowerSupplyLed(ledDescription);
      } else if (resourceType === "Fan") {
          return this._getDisplayableFanLed(ledDescription);
      } else if (resourceType === "Iomodule" || resourceType === "Switch") {
          return this._getDisplayableIOModuleLed(ledDescription);
      }
      return null;
  },

  _getDisplayableChassisLed : function (ledDescription) {
      itmeUtils.CMTrace("OVERLAY", "chassis led description: " + ledDescription);
      //Accipiter chassis leds:
      if (ledDescription === "Cmm 1 Active" || ledDescription === "cmm 1 Active") {
          return "Power Led";
      } else if (ledDescription === "Chassis Information") {
          return "Event Log Led";
      } else if (ledDescription === "Chassis Location") {
          return "Location Led";
      } else if (ledDescription === "Chassis FAULT") {
          return "Fault Led";
      } //Legacy chassis leds:
      else if (ledDescription === "Information") {
          return "Event Log Led";
      } else if (ledDescription === "Location") {
          return "Location Led";
      } else if (ledDescription === "Fault") {
          return "Fault Led";
      }
      return null;
  },

  _getDisplayableBladeLed : function (ledDescription) {
      if (ledDescription === "Power") {
          return "Power Led";
      } else if (ledDescription === "Check Log") {
          return "Event Log Led";
      } else if (ledDescription === "Identification") {
          return "Location Led";
      } else if (ledDescription === "Fault") {
          return "Fault Led";
      }
      return null;
  },
  _getDisplayablePowerSupplyLed : function (ledDescription) {
      if (ledDescription.indexOf('FAULT') != -1) {
          return "Fault Led";
      }
      return null;
  },
  _getDisplayableFanLed : function (ledDescription) {
      if (ledDescription.indexOf('FAULT') != -1) {
          return "Fault Led";
      }
      return null;
  },
  _getDisplayableIOModuleLed : function (ledDescription) {
      if (ledDescription.indexOf('Enclosure Identify') != -1) {
          return "Location Led";
      } else if (ledDescription.indexOf('Power') != -1) {
          return "Power Led";
      } else if (ledDescription.indexOf('Fault') != -1) {
          return "Fault Led";
      }
      return null;
  },
  _addNameElement: function(item) {
    if ( item.getType() == "com.ibm.sysmgt.platform.PhysicalPlatformFactory") {
      var ledOverlayTextSpanElement = itmeUtils.createElement(
                  item.mapElement.id + "_led2",
                  item.mapElement,
                  { elementType: "span",
                    baseDimensionsOnNode: item.mapElement,
                    innerHTML: "<span class='secondaryElementName'>"+item.getLabel()+"</span>"});

      // verticalize text
      this._setDimensionsAndTextDirection(item, ledOverlayTextSpanElement);
    }
  },

  // unRenderOverlay: function()
  // Unrender the overlay.  Parent function should suffice for all cases

  _unsetOverlayForItem: function(item)
  // Hide what was rendered by renderOverlay.  Override this if:
    // 1. you did not use "_"+_overlayId to append element divs  or...
    // 2. you need to unrender more than 1 created div
    // 3. you do not want to unrender "_"+_overlayId default that this base class does
  {
    this.inherited(arguments);
    itmeUtils.CMTrace("OVERLAY", "_unsetOverlayForItem (LED): " + item.getLabel());
    var itemsLedDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "2");
    if ( itemsLedDivElement)
      itemsLedDivElement.style.display = "none";
    this.myIndex = 0;
  },

  _deleteOverlayForItem: function(item) {
    this.inherited(arguments);
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+"2");
    if ( itemsDivElement)
      itemsDivElement.parentNode.removeChild(itemsDivElement);
  }
  // itemInformationModified: function(item)
  // Unused

  // getId: function()
  // Return the overlay id.  This is set in the constructor and should not need overriding.

  // getTitle: function() {
  // Return the title for the overlay.  This is set in the constructor and should not need overriding.

  // getDescription: function() {
  // Return the description for the overlay.  This is set in the constructor and should not need overriding.

  // requiresProperties: function() {
  // Unused
  // Gets the required properties for this overlay.  This is set in the constructor and should not need overriding.

  // setItems: function(items) {
  // Sets the items for the overlay. Parent function should suffice unless special processing is needed

});

}
