/*
	Copyright (c) 2004-2010, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["itme.ChassisMap.PropertiesOverlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.PropertiesOverlay"] = true;
dojo.provide("itme.ChassisMap.PropertiesOverlay");

dojo.require("itme.ChassisMap.OverlayBase");
dojo.require("itme.ChassisMap.OverlayFilterItem");
dojo.require("itme.ChassisMap.KnownHardwareInfo");

dojo.declare("itme.ChassisMap.PropertiesOverlay", [itme.ChassisMap.OverlayBase], {
   
  constructor: function(kwArgs){
    // parent constructor is always auto-called first
    this._overlayId = "name";
    this._description = this.MRI.NameOverlayDetail;
    this._title = this.MRI.NameOverlayTitle;
    this._iconId = "LabelIcon_22";
    
    this._availableFilters = [];
    this._nameFilter = new itme.ChassisMap.OverlayFilterItem({}, "nameFilter", "", this.MRI.Name, true, true); 
    this._availableFilters[0] = this._nameFilter; 
    this._ipFilter = new itme.ChassisMap.OverlayFilterItem({}, "ipFilter", "", "IP Address(s)", false, true );
    this._availableFilters[1] = this._ipFilter;
    this._fruFilter = new itme.ChassisMap.OverlayFilterItem({}, "fruFilter", "", "FRU", false, true);
    this._availableFilters[2] = this._fruFilter;
    this._serialNumberFilter = new itme.ChassisMap.OverlayFilterItem({}, "serialNumber", "", this.MRI.SerialNumber, false, true); 
    this._availableFilters[3] = this._serialNumberFilter; 

  },
  
  _renderItem: function(item, buildTooltip){
    var text = "", prop, enabledCount = 0, fontClass;
    
    // if we've already created the element before just show it, otherwise create it
    var overlayDivElement = dojo.byId(item.mapElement.id + "_name");
    if ( overlayDivElement)  {
      overlayDivElement.style.display = "block";
    }
    else { 
      // create it
      overlayDivElement = itmeUtils.createElement(
          item.mapElement.id + "_name", 
          item.mapElement,
          { elementType: "span",
            styleClass: "elementName" } );

      overlayDivElement.innerHTML = "";
      // set contents
      
      for ( var i = 0; i < this._availableFilters.length; i++ )
        if ( this._availableFilters[i].isEnabled() )
          enabledCount++;
          
      fontClass = "propElement";
      if ( enabledCount > 1) // 2 or 3
        fontClass = "propElement_smallFont";
      if ( enabledCount > 3) // 4 (max supported)
        fontClass = "propElement_smallestFont";

      var propStrings = [];
      var propIndex = 0;
      
      if ( this._nameFilter.isEnabled() ) 
        propStrings[propIndex++] = item.getLabel();

      if ( this._ipFilter.isEnabled() ) {
        prop = item.getProperty("IPv4Address");
        if ( prop && (prop.getDisplayValue() != "")) {
          /*var ellipsedString = itmeUtils.insertEllipsedTextSpanIntoElement(item.mapElement, 
                                                                           "IP: " + prop.getDisplayValue(), 
                                                                           { returnOnly: true, 
                                                                             insertAfter: "IP: ",
                                                                             styleClass: fontClass + " elementName"});*/
          propStrings[propIndex++] = "IP: " + prop.getDisplayValue();//ellipsedString;
        }
      }
      
      if ( this._fruFilter.isEnabled() ) {
        prop = item.getProperty("EnclosureFRU");
        if ( prop && (prop.getDisplayValue() != "")) 
          propStrings[propIndex++] = "FRU: " + prop.getDisplayValue();
      }
      
      if ( this._serialNumberFilter.isEnabled() ) {
        prop = item.getProperty("SerialNumber");
        if ( prop && (prop.getDisplayValue() != "") ) 
          propStrings[propIndex++] = "SN: " + prop.getDisplayValue();
      }
      
      itmeUtils.CMTrace("OVERLAY", "numProps: " + propIndex);

      for (i = 0; i < propStrings.length; i++) {
        itmeUtils.CMTrace("OVERLAY", "prop["+i+"]: " + propStrings[i]);
        if ( ( i == 1) && ( item.getType() == "SystemChassis" ) ) 
        {
          itmeUtils.createElement(
              item.mapElement.id + "_nameChassisPt2", 
              overlayDivElement,
              { elementType: "span",
                styleClass: "chassisBottomText " + fontClass,
                style: {"zIndex": "3"},
                innerHTML: propStrings[i]} ) ; 
        } else if ( ( i > 1) && ( item.getType() == "SystemChassis" ) )
          ;
        else {
          overlayDivElement.innerHTML = overlayDivElement.innerHTML + "<span class='"+fontClass+"'>"+propStrings[i]+"</span>";
        }
      }
      
      
      // verticalize text      
      this._setDimensionsAndTextDirection(item, overlayDivElement);  

      //this._addPropertyToTooltipIfAvailable(item, "DisplayName");
      if ( buildTooltip ) {
        this._addPropertyToTooltipIfAvailable(item, "ChassisSlot");
        this._addPropertyToTooltipIfAvailable(item, "OccupiedSlots");
        this._addPropertyToTooltipIfAvailable(item, "DisplayOperationalStatusTime");
        this._addOSProperty(item);
        this._addPropertyToTooltipIfAvailable(item, "HostName");
        this._addPropertyToTooltipIfAvailable(item, "IPv4Address");
        this._addPropertyToTooltipIfAvailable(item, "MACAddress");
        
        this._addPropertyToTooltipIfAvailable(item, "SystemBoardUUID");
        
        this._addPropertyToTooltipIfAvailable(item, "Architecture");
        this._addPropertyToTooltipIfAvailable(item, "Manufacturer");
        //this._addPropertyToTooltipIfAvailable(item, "SerialNumber");
        this._addPropertyToTooltipIfAvailable(item, "EnclosureFRU");
      }
    }

    overlayDivElement.innerHtml = "<span id='itemsNameOverlay'>"+item.getLabel() + "</span>";
  
  },
  
  _addOSProperty: function(item) {
    var frmProps = dojo.fromJson( itmeUtils.getProperty(item, "OTHERHWVIEWPROPS") );
    if ( this.isFSM(item) ) {// REDUNDANT FSM check, Do NOT show OS for FSM hardware.
      itmeUtils.CMTrace("OVERLAY", "node is FSM, NOT showing OS info for: " + item.getLabel());
      return;
    }
    if ( frmProps.OS_DATA ) {
      itmeUtils.CMTrace("OVERLAY", "node is NOT FSM, showing OS info if available for: " + item.getLabel());
      for ( var i = 0; i < frmProps.OS_DATA.length ; i++ ) {
        itmeUtils.CMTrace("OVERLAY", item.mapElement.id + " : Prop [ OS ["+i+"] ]"+frmProps.OS_DATA[i].OSLABEL+" / " + frmProps.OS_DATA[i].OSTYPE);
        if ( frmProps.OS_DATA[i].OSTYPE )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.OperatingSystem, frmProps.OS_DATA[i].OSTYPE);
        //item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.OSLABEL, frmProps.OS_DATA[i].OSLABEL+" / " + frmProps.OS_DATA[i].OSTYPE);
      }
    }
  },
  
  isFSM: function(item) {
    var prop = item.getProperty("MachineType");
    if ( !prop )
      return false;
    var machineType = prop.getDisplayValue();
    var hwInfo = new itme.ChassisMap.KnownHardwareInfo();
    itmeUtils.CMTrace("OVERLAY", "is FSM for : " + machineType);
    return hwInfo.isFSM(machineType);
  }
  
  /*_testIfStringNeedsEllipse: function(parentElement, propString) {
    itmeUtils.CMTrace("OVERLAY", "chkpt1");
    var parentBox = dojo.contentBox(parentElement);
    itmeUtils.CMTrace("OVERLAY", "chkpt2");
    var childNode = dojo.create("span", {innerHTML: "IP: " + propString});
    itmeUtils.CMTrace("OVERLAY", "chkpt3");
    var childBox = dojo.contentBox(childNode);
    itmeUtils.CMTrace("OVERLAY", "chkpt4");
    itmeUtils.CMTrace("OVERLAY", "parentW = " + parentBox.w + " , childW = "+ childBox.w);
    if ( childBox.w > parentBox.w )
      return true;
    return false;
  }*/
  
  // renderOverlay: function(){
  // Default preprocessing for rendering.  Calls into the REQUIRED OVERRIDE method _renderItem
  
  // unRenderOverlay: function()
  // Unrender the overlay.  Parent function should suffice for all cases
  
  // _unsetOverlayForItem: function(item)
  // Hide what was rendered by renderOverlay.  Override this if:
    // 1. you did not use "_"+_overlayId to append element divs  or...
    // 2. you need to unrender more than 1 created div
    // 3. you do not want to unrender "_"+_overlayId default that this base class does
  
  // itemInformationModified: function(item)
  // Unused
  
  // getId: function()
  // Return the overlay id.  This is set in the constructor and should not need overriding.

  // getTitle: function() {
  // Return the title for the overlay.  This is set in the constructor and should not need overriding.

  // getDescription: function() {
  // Return the description for the overlay.  This is set in the constructor and should not need overriding.
  
  // requiresProperties: function() {
  // Unused
  // Gets the required properties for this overlay.  This is set in the constructor and should not need overriding.
  
  // setItems: function(items) {
  // Sets the items for the overlay. Parent function should suffice unless special processing is needed 
  
});

}
