/*
	Copyright (c) 2004-2010, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["itme.ChassisMap.StatusOverlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.StatusOverlay"] = true;
dojo.provide("itme.ChassisMap.StatusOverlay");

dojo.require("itme.ChassisMap.OverlayBase");
dojo.require("itme.ChassisMap.OverlayFilterItem");

dojo.declare("itme.ChassisMap.StatusOverlay", [itme.ChassisMap.OverlayBase], {

  _diffForBorder: null,
  _maxShownActiveStatusEntriesForTooltip: 6,
    
  constructor: function(kwArgs){
    // parent constructor is always auto-called first
    this._overlayId = "status";
    this._description = this.MRI.StatusOverlayDetail;
    this._title = this.MRI.StatusTitle;
    this._requiredProperties = [ "STATUS" ];
    this._iconId = "StatusFilterIcon";
    
    this._availableFilters = [];
    this._criticalFilter = new itme.ChassisMap.OverlayFilterItem({}, "criticalFilter", "StatusCritical", this.MRI.Critical, true, false );
    this._availableFilters[0] = this._criticalFilter;
    this._warningFilter = new itme.ChassisMap.OverlayFilterItem({}, "warningFilter", "StatusWarningMinor", this.MRI.Warnings, true, true);
    this._availableFilters[1] = this._warningFilter;
    this._informationalFilter = new itme.ChassisMap.OverlayFilterItem({}, "informationalFilter", "StatusInformational", this.MRI.Informational, true, true); 
    this._availableFilters[2] = this._informationalFilter; 
    this._normalFilter = new itme.ChassisMap.OverlayFilterItem({}, "normalFilter", "StatusNormal", this.MRI.Normal, false, true); 
    this._availableFilters[3] = this._normalFilter; 
  },
  
  _renderItem: function(item, buildTooltip){
    var iconSize = 22, problems, statusStringForCSS, CommStateProp, isOK = true;
    var mapElementCBox = dojo.contentBox(item.mapElement);
    var itemsStatusDivElement = dojo.byId(item.mapElement.id + "_status");
    
    // if we've already created the element before just show it, otherwise create it
    if (itemsStatusDivElement) {
      itemsStatusDivElement.style.display = "block";
      if ( dojo.byId(item.mapElement.id + "_status2") )
        dojo.byId(item.mapElement.id + "_status2").style.display = "block";
    }
    else {
      // create element
      itemsStatusDivElement = itmeUtils.createElement(
          item.mapElement.id + "_status", 
          item.mapElement,
          { styleClass: "defaultStatus",
            baseDimensionsOnNode: item.mapElement} );
     
      // add element name
      if ( item.getType() == "com.ibm.sysmgt.platform.PhysicalPlatformFactory" ||
           item.getType() == "SystemChassis" ) 
        itmeUtils.insertEllipsedTextSpanIntoElement(itemsStatusDivElement, item.getLabel(), {styleClass: "secondaryElementName"}); // el, class, text
      
      if ( item.mapElement.constraint < iconSize )
        iconSize = item.mapElement.constraint;//imgStyleString = " style='height:"+item.mapElement.constraint+"px' ";

      problems = dojo.fromJson( itmeUtils.getProperty(item, "Problems") );
      if ( problems ) {
        //var problemTooltipHTML = "";  
        statusStringForCSS = null;

        CommStateProp = item.getProperty("CommunicationState");
        if ( CommStateProp && ( CommStateProp.getValue() == 3) ) // lost communication (aka OFFLINE) 
        {
          statusStringForCSS = "Offline";
          itemsStatusDivElement.innerHTML = this.IMAGES.buildIconImageTag("AccessOfflineOK", "", iconSize) + itemsStatusDivElement.innerHTML;
          isOK = false;
        }
        
        if (problems.PBLM_I && (problems.PBLM_I > 0)) {
          isOK = false;
          //problemTooltipHTML = problems.PBLM_I + this.IMAGES.buildIconImageTag("StatusInformational", "", 16);
          if ( this._informationalFilter.isEnabled() ) 
            itemsStatusDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusInformational", "", iconSize) + itemsStatusDivElement.innerHTML;
        } 
        if (problems.PBLM_W && (problems.PBLM_W > 0)) {
          isOK = false;
          //problemTooltipHTML = problems.PBLM_W + this.IMAGES.buildIconImageTag("StatusWarningMinor", "", 16) + " " + problemTooltipHTML;
          if ( this._warningFilter.isEnabled() ) {
            statusStringForCSS = "Attention";
            itemsStatusDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusWarningMinor", "", iconSize) + itemsStatusDivElement.innerHTML;
          }
        }
        if (problems.PBLM_E && (problems.PBLM_E > 0)) {
          isOK = false;
          //problemTooltipHTML = problems.PBLM_E + this.IMAGES.buildIconImageTag("StatusCritical", "", 16) + " " + problemTooltipHTML;
          if ( this._criticalFilter.isEnabled() ) {
            statusStringForCSS = "Critical";
            itemsStatusDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusCritical", "", iconSize) + itemsStatusDivElement.innerHTML;
          }
        }
        
        if ( (isOK == true) && (this._normalFilter.isEnabled()) ) {
          itemsStatusDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusNormal", "", iconSize) + itemsStatusDivElement.innerHTML;
        }
        
      }
      
      if ( buildTooltip ) {
        /*var statusArray = item.getStatuses();
        for (var i = 0; i < statusArray.length; i++) {
          itmeUtils.CMTrace("OVERLAY", "ITEM Status["+i+"]: " + statusArray[i].getId() + " - " + statusArray[i].getLabel());
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.Status, "<img src='" + statusArray[i].getImage() + "'> "+statusArray[i].getLabel() );
        }
        */
        //this._addPropertyToTooltipIfAvailable(item, "OperatingState");

        if ( CommStateProp && ( CommStateProp.getValue() == 3) ) // lost communication (aka OFFLINE), add offline icon to 'offline' string. 
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, CommStateProp.getLabel(), this.IMAGES.buildIconImageTag("AccessOfflineOK", "", 16) + " " + CommStateProp.getDisplayValue());
        else
          this._addPropertyToTooltipIfAvailable(item, "CommunicationState");
        
        //this._addPropertyToTooltipIfAvailable(item, "PowerState");

        this._addActiveStatusEntries(item);
      }
    
      // if we have a non-normal status we handle:
      if ( statusStringForCSS ) 
      {
        // for the initial div, append the status class
        dojo.addClass(itemsStatusDivElement, "ComponentStatus"+statusStringForCSS);
        
        // due to the border for this overlay, we shrink the element based our css border size x2
        this._calculateBorderDiff(mapElementCBox, itemsStatusDivElement);
        dojo.contentBox(itemsStatusDivElement, {w: mapElementCBox.w - this._diffForBorder,h: mapElementCBox.h - this._diffForBorder});
        
        // add yet another div for critical status, the css class should give it a red or yellow hue            
        var itemsStatusHighlightDivElement = itmeUtils.createElement(
            item.mapElement.id + "_status2", 
            item.mapElement,
            { styleClass: "ComponentStatus"+statusStringForCSS+"Shadow" } );

        dojo.contentBox(itemsStatusHighlightDivElement, {w: mapElementCBox.w, h: mapElementCBox.h }); 
      }
    }
  },

  _addActiveStatusEntries: function(item) {
    var activeStatusEntries = dojo.fromJson( itmeUtils.getProperty(item, "ACTIVESTATUSINFO") );
    if ( activeStatusEntries.STATUS_ENTRIES && (activeStatusEntries.STATUS_ENTRIES.length > 0) ) {

      if ( item.getType() == "Fan" || item.getType() == "PowerSupply")
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, "", this.MRI.FindActiveStatusOnChassis);

    	item.mapElement.CMTooltip.addFocusProperty(this._overlayId, "<hr>", "<hr>");
    	  
    	itmeUtils.CMTrace("OVERLAY", "Status entries on item ("+item.mapElement.id+") = " + activeStatusEntries);

      var sortedStatusEntryList = this._sortStatusEntryList(activeStatusEntries.STATUS_ENTRIES);
        
      for (var i = 0; i < sortedStatusEntryList.length; i++) {

        if ( i == this._maxShownActiveStatusEntriesForTooltip )
      	  item.mapElement.CMTooltip.addFocusProperty(this._overlayId, "", dojo.replace(this.MRI.MoreStatusEntries, [ sortedStatusEntryList.length - this._maxShownActiveStatusEntriesForTooltip ] ) );
        else if ( i > this._maxShownActiveStatusEntriesForTooltip )
      	  break;
        else {
          var imageTagString = "";
          if ( this._isStatusCritical(sortedStatusEntryList[i].SEVERITY))
        	  imageTagString = this.IMAGES.buildIconImageTag("StatusCritical", "", 16);
          if ( this._isStatusWarning(sortedStatusEntryList[i].SEVERITY))
        	  imageTagString = this.IMAGES.buildIconImageTag("StatusWarningMinor", "", 16);
          if ( this._isStatusInfo(sortedStatusEntryList[i].SEVERITY))
        	  imageTagString = this.IMAGES.buildIconImageTag("StatusInformational", "", 16);
          
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, imageTagString, sortedStatusEntryList[i].PROBLEMNAME);
        }
      }
    }
  },
  
  _sortStatusEntryList: function(statusEntryList) {
	  var newStatusEntryList = [];
	  var newListCounter = 0;
	  for ( var i = 0; i < statusEntryList.length; i++ ) {
		  if ( this._isStatusCritical(statusEntryList[i].SEVERITY))
			  newStatusEntryList[newListCounter++] = statusEntryList[i];
	  }
	  for ( i = 0; i < statusEntryList.length; i++ ) {
		  if ( this._isStatusWarning(statusEntryList[i].SEVERITY))
			  newStatusEntryList[newListCounter++] = statusEntryList[i];
	  }
	  for ( i = 0; i < statusEntryList.length; i++ ) {
		  if ( this._isStatusInfo(statusEntryList[i].SEVERITY))
			  newStatusEntryList[newListCounter++] = statusEntryList[i];
	  }
	  
	  return newStatusEntryList;
  },
  
  _isStatusCritical: function(statusId) {
	  if ( (statusId == 0)  || (statusId == 1))
		  return true;
	  return false;
  },
  
  _isStatusWarning: function(statusId) {
	  if ( (statusId == 2) || (statusId == 3) ) // MINOR or WARNING 
		return true;
	  return false;
  },
  
  _isStatusInfo: function(statusId) {
	  if ( statusId == 4 )
		  return true;
	  return false;
  },
  
  _isStatusUnknown: function(statusId) {
	  if ( statusId == 5 )
		  return true;
	  return false;
  },
  
  _isStatusNormal: function(statusId) {
	  if ( statusId == -1 )
		  return true;
	  return false;
  },
  
  _calculateBorderDiff: function(cBox, element) {
    this._diffForBorder = 4; // HACK:  IE failed to calculate border correctly.  CSS Margin = 2.  2x2 = 4.
    if (this._diffForBorder) {
      //itmeUtils.CMTrace("OVERLAY", "calc border diff returning : " + this._diffForBorder);
      return this._diffForBorder;
    }
      
    var mBox = dojo.marginBox(element);
    this._diffForBorder = mBox.w - cBox.w;
    return this._diffForBorder;
  },
  
  // renderOverlay: function(){
  // Default preprocessing for rendering.  Calls into the REQUIRED OVERRIDE method _renderItem
  
  // unRenderOverlay: function()
  // Unrender the overlay.  Parent function should suffice for all cases
  
  _unsetOverlayForItem: function(item)
  // Hide what was rendered by renderOverlay.  Override this if:
    // 1. you did not use "_"+_overlayId to append element divs  or...
    // 2. you need to unrender more than 1 created div
    // 3. you do not want to unrender "_"+_overlayId default that this base class does
  {
    this.inherited(arguments);
    //itmeUtils.CMTrace("OVERLAY", "_unsetOverlayForItem (STATUS): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "2");
    if ( itemsDivElement) 
      itemsDivElement.style.display = "none";
  },
  
  _deleteOverlayForItem: function(item) {
    this.inherited(arguments);
    //itmeUtils.CMTrace("OVERLAY", "_deleteOverlayForItem (BASE): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+"2");
    if ( itemsDivElement) 
      itemsDivElement.parentNode.removeChild(itemsDivElement);
  }
  
  // itemInformationModified: function(item)
  // Unused
  
  // getId: function()
  // Return the overlay id.  This is set in the constructor and should not need overriding.

  // getTitle: function() {
  // Return the title for the overlay.  This is set in the constructor and should not need overriding.

  // getDescription: function() {
  // Return the description for the overlay.  This is set in the constructor and should not need overriding.
  
  // requiresProperties: function() {
  // Unused
  // Gets the required properties for this overlay.  This is set in the constructor and should not need overriding.
  
  // setItems: function(items) {
  // Sets the items for the overlay. Parent function should suffice unless special processing is needed 
  
  
});

}
