/*
	Copyright (c) 2004-2010, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["itme.ChassisMap._Selection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap._Selection"] = true;
dojo.provide("itme.ChassisMap._Selection");

dojo.declare("itme.ChassisMap._Selection", null, {

  _currentSelectionsByItemId: [],
  _refreshDelay: false,
  
  constructor: function(kwArgs){
    this._currentSelectionsByItemId = [];
  },

  // PUBLIC
  // A String or Comma delimited string containing 1 or more item ids of selected elements
  setSelection: function(selectedElementsByItemId) {
    if (selectedElementsByItemId) {
      itmeUtils.CMTrace("SELECTION", "Incoming Selection (ignored): " + selectedElementsByItemId);
    }

    var selectionList = dojo.cookie("chasisMap.currentSelection");
    itmeUtils.CMTrace("SELECTION", "selectionList = " + selectionList);

    if (selectionList) {
      var itsSplit = selectionList.split(",");
      itmeUtils.CMTrace("SELECTION", "TMP Split: " + itsSplit);
      for (var i = 0; i < itsSplit.length; i++)   
      {
        itmeUtils.CMTrace("SELECTION", "Selection["+i+"]: "+itsSplit[i]);
        this._currentSelectionsByItemId[itsSplit[i]+"_item"] = itsSplit[i];
      }
    }
    
    dojo.cookie("chasisMap.currentSelection", null, { expires: -1}); // remove the cookie if it exists
  },
  
  // protected - builds the selection list and kicks off a timer to fire the change
  _handleMultiSelectEvent: function(item) {
    // if this item is already selected, remove it
    if (this._currentSelectionsByItemId[item.getId() + "_item"]) {
      delete this._currentSelectionsByItemId[item.getId() + "_item"];
      dojo.removeClass(item.mapElement, "ComponentSelected");
    }
    else { // else add it 
      this._currentSelectionsByItemId[item.getId() + "_item"] = item.getId();
      dojo.addClass(item.mapElement, "ComponentSelected");
    }
    
    //TODO: I commented out all the ticker/refresh code since i don't know if it is needed (Alex)
    /*
    itmeUtils.CMTrace("SELECTION", "multi-select: current selections: " + itmeUtils.myJoin(this._currentSelectionsByItemId));
    itmeUtils.CMTrace("SELECTION", "Assoc array length: " + itmeUtils.associativeArrayLength(this._currentSelectionsByItemId));
    if (itmeUtils.associativeArrayLength(this._currentSelectionsByItemId) > 0) {
      itmeUtils.CMTrace("SELECTION", "refreshing ticker");
      this._refreshTicker();
    }
    else { // if the countdown for multi select is shown, hide it.
      itmeUtils.CMTrace("SELECTION", "hiding timer");
      var refreshDivElement = dojo.byId("refreshIncoming");
      if (refreshDivElement) {
        itmeUtils.CMTrace("SELECTION", "hiding timer");
        refreshDivElement.style.display = "none";
      }
    }*/
  },
  
  // protected - sets the selection list and kicks off a timer to fire the change
  _handleSingleSelectEvent: function(item) {
    // empty the selections list
    this._currentSelectionsByItemId = [];
    dojo.query(".ComponentSelected").forEach(function(node){
      dojo.removeClass(node, "ComponentSelected");
    });

    // if the countdown for multi select is shown, hide it.
    var refreshDivElement = dojo.byId("refreshIncoming");
    if (refreshDivElement)
      refreshDivElement.style.display = "none";
    
    // set selection
    this._currentSelectionsByItemId[item.getId()+"_item"] = item.getId();
    dojo.addClass(item.mapElement, "ComponentSelected");
  },
  
  // TYPE: ACTION
  _refreshTicker: function()
  {
    var refreshDivElement = dojo.byId("refreshIncoming");
    if(this._refreshDelay === false) {
      this._refreshTime = 5; // num seconds to pop

      itmeUtils.CMTrace("SELECTION", "refreshDivElement");
      if ( refreshDivElement ){
        refreshDivElement.style.display = "block";
      } else {
        refreshDivElement = itmeUtils.createElement(
            "refreshIncoming", 
            this._chassisMapFaceElements["front"],
            { styleClass: "refreshTimer", 
              style: {  "position": "absolute",
                        "right": "0px",
                        "top": "0px",
                        "overflow": "hidden",
                        "text-overflow": "ellipsis" },
              innerHTML: this.MRI.Refresh + this._refreshTime + this.MRI.Seconds } );
      }
      this._refreshDelay = setTimeout(dojo.hitch(this, this._refreshTicker), 1000);
      
    } 
    else {
      this._refreshTime -= 1; // num seconds to pop
      refreshDivElement.innerHTML = this.MRI.Refresh + this._refreshTime + this.MRI.Seconds;
      if (this._refreshTime > 0)
        this._refreshDelay = setTimeout(dojo.hitch(this, this._refreshTicker), 1000); // recurse in 1 second 
      else {
        // Process Page Refresh
        chassisMapRefresh(); //jsUtils
        this.cleanUp();
        processRefresh( dojo.cookie("chasisMap.currentSelection") );
        itmeUtils.CMTrace("SELECTION", "Refreshing page now: " + dojo.cookie("chasisMap.currentSelection") );

      }
    }
  }
});

}
