/*
	Copyright (c) 2004-2010, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/


if(!dojo._hasResource["itme.finder.Finder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.finder.Finder"] = true;
dojo.provide("itme.finder.Finder");

dojo.require("itme.finder.TaskDataStore");
dojo.require("itme.finder.ResourceDataStore");
dojo.require("itme.finder.GroupDataStore");
dojo.require("itme.finder.ContextTaskDataStore");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.TooltipDialog");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("ibm.tivoli.navmodel.Dataset");

dojo.require("dojo.i18n");

dojo.declare("itme.finder.Finder", null, {
  MRI: null,
	_searchTextHintString: null, 
    _finderIdPrefix: null,
    _contextFinderMenuText: null,
    _tabbedContainer: null,
	// jsp input fields references, for communication between finder and bean    
  	_selectedTaskInput: null,
    _selectedTaskButton: null,  
    _selectedChassisListInput: null,   
    _selectedChassisMapListInput: null, 
    _selectedChassisTextInput: null,  
    _homeParamsForDataSet: null,     
    // references to finder elements
    _finderDomObj: null, 				//finder main dom container
  	_searchDetailDlg: null,  			//finder results tooltip dialog
  	
  	// finder state variables
  	_searchDlgIsOpen: false,
  	_searchDlgInitialized: false,
    _latestSearchEntry: null,
    _currentView: '0', 
    _refreshImage: null,
    
  	//Data stores containing cached lists
  	_taskDataStore: null,
    _resourceDataStore: null,  
    _groupDataStore: null,  
    _contextTaskDataStore: null,   
    
    
    constructor: function(domId, finderIdPrefix){  	
      dojo.requireLocalization("itme", "Resources", null, "ROOT");
      this.MRI = dojo.i18n.getLocalization("itme", "Resources");
      
   		itmeUtils.initializeTimer("finderTimer");   		
    	this._finderIdPrefix = finderIdPrefix;
    	this.initializeFieldReferences(domId);
    	if(this._finderDomObj) {} else {return;}
    	if(finderIdPrefix == 'finder') {
    		this._searchTextHintString = this.MRI.FinderInitialText;  
    	} else {
    		var prefixString = this.MRI.ContextFinderInitialText;
    		var menuString = this.MRI.ContextFinderTaskMenuFor;
    		var endString = '';
    		var selected;
    		if(isChassisMapRendered()) {
    			selected = this._selectedChassisMapListInput.innerHTML.split(','); 
    		} else {   	
    			selected = this._selectedChassisListInput.innerHTML.split(',');	
    		}
    		if(selected.length > 0) {
    			if(selected.length == 1) {
    				endString = this._selectedChassisTextInput.innerHTML;
    				menuString = menuString + this._selectedChassisTextInput.innerHTML;
    			} else {
    				var resourceType = this.MRI.Chassis;
    				if(isChassisMapRendered()) {
    					resourceType = this.MRI.Resources;
    				}
    				endString = selected.length + " " + resourceType;
    				menuString = menuString + selected.length + " " + resourceType;
    			}
    			this._searchTextHintString = prefixString + endString;      
    			this._contextFinderMenuText = menuString;
    		}	
    		
    	}
    	this.initializeFinderHtml();
    	if(finderIdPrefix == 'finder') {
   			this.initializeDataStores(domId); 	
    	} 
    	setTimeout(dojo.hitch(this, 'incrementRefreshMinute'), 60000);
  	},
  	incrementRefreshMinute: function() {
  	    this._selectedChassisMapListInput.title = parseInt(this._selectedChassisMapListInput.title) + 1;
  		if(this._refreshImage != null) {
  			this.changeRefreshCount();
  		}  		
    	setTimeout(dojo.hitch(this, 'incrementRefreshMinute'), 60000);
  	},
  	changeRefreshCount: function() {
  		var count = parseInt(this._selectedChassisMapListInput.title);
  		if (count < 60) {
	  		console.info(count + "," + this.MRI.RefreshCache + "," + this.MRI.Minutes);
  			this._refreshImage.title = dojo.replace(this.MRI.RefreshCache, [count, this.MRI.Minutes]);  			
	  		console.info(count);
  		} else if (count < 1440) {
  			var hours = Math.floor(count/60);
  			this._refreshImage.title = dojo.replace(this.MRI.RefreshCache, [hours, this.MRI.Hours]);   			
  		} else {
  		    var days = Math.floor(count/1440);  			
  			this._refreshImage.title = dojo.replace(this.MRI.RefreshCache, [days, this.MRI.Days]);    		
  		}
  	},
  	initializeDataStores: function(domId) {  	
    	this._taskDataStore = new itme.finder.TaskDataStore('tasks', domId);   
    	this._resourceDataStore = new itme.finder.ResourceDataStore('resources', domId); 
    	this._groupDataStore = new itme.finder.GroupDataStore('groups', domId); 
  	},
  	initializeFieldReferences: function(domId) {  	
    	this._selectedTaskInput = dojo.byId(domId + ':finderSelected');
        this._selectedTaskButton = dojo.byId(domId + ':launchButton');  
        this._selectedChassisListInput = dojo.byId(domId + ':selectedChassisList');  
        this._selectedChassisTextInput = dojo.byId(domId + ':selectedChassisText');  
        this._selectedChassisMapListInput = dojo.byId(domId + ':selectedChassisMapOidsCSVReadOnly');  
  		this._finderDomObj = dojo.byId(this._finderIdPrefix); 
  		var homeParmsValue = dojo.byId(domId + ':chassisMapHomeParms2').innerHTML;		
  		this._homeParamsForDataSet =  Math.floor(Math.random()*50000) + '#' + homeParmsValue;
  		
  	},
  	initializeFinderHtml: function() {  
  		dojo.empty(this._finderDomObj);	 
  		if(dijit.byId(this._finderIdPrefix + "searchBox")) {
  			dijit.byId(this._finderIdPrefix + "searchBox").destroy();
  		}
  		if(dijit.byId(this._finderIdPrefix + "dialog")) {
  			dijit.byId(this._finderIdPrefix + "dialog").destroy();
  		}
  		var tooltipHelp = this.MRI.FinderTooltipHelp;
        if(this._finderIdPrefix == 'contextFinder') {
            tooltipHelp = this.MRI.ContextFinderTooltipHelp;
        }   
    	var innerDiv = dojo.create("div", {title: tooltipHelp}, this._finderDomObj);
    	var finderInputFieldDomObj = dojo.create("input", {}, innerDiv);  
       	this._searchDetailDlg = new dijit.TooltipDialog({id:this._finderIdPrefix + "dialog", style:"width:450px;height:150px;"});
    	this._searchDetailDlg.startup(); 
    	
  		     
  		var urlString = contextPath + '/images/ac16_search.png';
        phxTrace(urlString);
  		var textBoxxerroo = new dijit.form.TextBox({
    		value:this._searchTextHintString, 
    		style:"padding-top:3px; width:160px; padding-left:20px;", 
    		id:this._finderIdPrefix + "searchBox", 
    		title: tooltipHelp,
    		
    		onKeyUp:dojo.hitch(this, this.finderKeyPressed), 
    		onBlur:dojo.hitch(this, this.doSearchBoxBlur), 
    		onFocus:dojo.hitch(this, this.doSearchBoxFocus)}, finderInputFieldDomObj);  
    	  	
  		textBoxxerroo.attr("class", "finderSearchBox");
  	},
    fetchFailed: function(error, request) {
                phxTrace("lookup failed.");
                phxTrace(error);
    }, 
	doSearchBoxBlur: function() {
		if (dijit.byId(this._finderIdPrefix + "searchBox").attr("value") == "") {
			dijit.byId(this._finderIdPrefix + "searchBox").attr("value", this._searchTextHintString);
  		} 
	},
	doSearchBoxFocus: function() {
		if (dijit.byId(this._finderIdPrefix + "searchBox").attr("value") == this._searchTextHintString) {
			dijit.byId(this._finderIdPrefix + "searchBox").attr("value", "");
  		} 
  		dijit.byId(this._finderIdPrefix + "searchBox").focus();
	},	
	refreshCache: function() {
   		this._selectedTaskInput.value = "REFRESH_CACHE";
		this._selectedTaskButton.click();	
	
	},
	closeSearchDetailDlg: function() {	    
   		if (this._searchDetailDlg) { dijit.popup.close(this._searchDetailDlg); }
// 		dijit.byId(this._finderIdPrefix + "searchBox").attr("value", this._searchTextHintString)
    	this._searchDlgIsOpen = false;
    	this._latestSearchEntry = '';
//    	var obj = dijit.byId(this._finderIdPrefix + "searchBox").domNode;
//    	obj.blur();
	},	
	doTaskSearch: function(myitems, myrequest) {		
    	this.doSearch(myitems, myrequest, this._taskDataStore, 'special#_com.ibm.usmi.console.taskmap', this.MRI.MoreTasks, 'tasks', false);
 	},
 	doContextTaskSearch: function(myitems, myrequest) {		
    	this.doSearch(myitems, myrequest, this._contextTaskDataStore, 'special#_com.ibm.usmi.console.taskmap', this.MRI.MoreTasks, 'contextTasks', false);
 	},
	doResourceSearch: function(myitems, myrequest) {		
    	this.doSearch(myitems, myrequest, this._resourceDataStore, 'special#_com.ibm.usmi.console.tasks.QuickFindTask', this.MRI.MoreResources, 'resources', false);
	},	
	doGroupSearch: function(myitems, myrequest) {		
    	this.doSearch(myitems, myrequest, this._groupDataStore, 'special#_none', this.MRI.MoreGroups, 'groups', true);
	},		
	doSearch: function(myitems, myrequest, dataStore, moreItemsTask, moreItemsText, typeOfSearch, moreItemsNotClickable) {		
    	phxTrace('searchCompleted: ' + myitems.length + ' results');      	
  		var searchString = dijit.byId(this._finderIdPrefix + "searchBox").attr("value");
 		if (searchString == "" || searchString == this._searchTextHintString) return; 
 		if(myitems) {
 			if(myitems.length > 0) {
 				dataStore.clearSelection(); 				
 				itmeUtils.traceTimeDelta("finderTimer", "processing results", true); 
 				dataStore.populateCategoriesWithSelectedItems(myitems);
 				itmeUtils.traceTimeDelta("finderTimer", "results populated", true); 
  				this.initializeDialogConstruct(typeOfSearch, myitems.length);    	      	   	
  				itmeUtils.traceTimeDelta("finderTimer", "initialized dialog", true); 	
 				dataStore.createCategoryHTMLElements(myitems.length, this, moreItemsTask, moreItemsText, moreItemsNotClickable);
 				itmeUtils.traceTimeDelta("finderTimer", "results UI created", true);  
 			} else {
 				this.initializeDialogConstruct(typeOfSearch);  			
				dojo.create("div", {innerHTML:this.MRI.NoMatchesFound}, dojo.byId(this._finderIdPrefix + typeOfSearch + "_containerDiv")); 
 			}	 	
  			//dojo.byId(this._finderIdPrefix + typeOfSearch + "Menu_count").innerHTML = " (" +myitems.length + ")";		
 		}
 		if(typeOfSearch == 'tasks') { 	//Last Tab kicks off the render logic	
        	this._tabbedContainer.startup();	
 			var selectedTab = this._tabbedContainer.selectedChildWidget
 			dojo.removeClass(selectedTab.domNode, "dijitHidden");
			dojo.addClass(selectedTab.domNode, "dijitVisible");
 		}
 		
	},		
		
	doClick: function(event) {		
		var selectedTaskId = event.currentTarget.id;				
		this.closeSearchDetailDlg();
		phxTrace('doclick' + selectedTaskId);
		var type = selectedTaskId.split('#_')[0];
		var itemId = selectedTaskId.split('#_')[1];
		if(type.indexOf('categorySection') != -1) {
			var category = this._resourceDataStore.getCategories()[itemId];
			var ids = category.getIds();
			selectedTaskId = 'resourceList#_';
			var i;
			for (i = 0; i < ids.length; i++) {
				if(i != 0) {
					selectedTaskId = selectedTaskId + ",";
				}
				selectedTaskId = selectedTaskId + ids[i];
			}
		} else if(type == 'resources') {
			if(dojo.byId('ChassisMap')) {
				
			}
			dojo.cookie("chasisMap.currentSelection", itemId, { expires: 1});
		} 
		
		this._selectedTaskInput.value = selectedTaskId + "#_" + dijit.byId(this._finderIdPrefix + "searchBox").attr("value");
		this._selectedTaskButton.click();	
	},
  	finderKeyPressed: function(event){
  		phxTrace('keyPressed');
    	if (event.keyCode == 27) {
     		dijit.byId(this._finderIdPrefix + "searchBox").attr("value", "");
  		}  		
  		var searchStringEntered = dijit.byId(this._finderIdPrefix + "searchBox").attr("value");
  		if(this._latestSearchEntry == searchStringEntered) {
  			return;
  		} else {
  			this._latestSearchEntry = searchStringEntered;
  		}
  		if ((searchStringEntered == "") || (searchStringEntered == this._searchTextHintString)) {   //|| searchStringEntered.length == 1
       		this.closeSearchDetailDlg();
    		return;
  		}   	
  		if (!this._searchDlgIsOpen) {
       		dijit.popup.open({    popup: this._searchDetailDlg,
	                      around: dijit.byId(this._finderIdPrefix + "searchBox").domNode, 
                          onCancel: dojo.hitch(this, this.closeSearchDetailDlg),
                          parent: dijit.byId(this._finderIdPrefix + "searchBox")}); 
        }  
		this._searchDlgIsOpen = true;
        dijit.byId(this._finderIdPrefix + "searchBox").focus();
		var searchTerm = searchStringEntered;  
  		phxTrace('query: ' + searchTerm); 
  		this._searchDlgInitialized = false;
  		if(this._finderIdPrefix == 'contextFinder') {
  			
  		    dojo.destroy('searchingText');
  			this.initiateContextSearch(searchTerm, false, 0);
  		} else {
  			this._resourceDataStore.initiateTaskSearch(searchTerm, this); 	
  			this._groupDataStore.initiateTaskSearch(searchTerm, this);  	
  			this._taskDataStore.initiateTaskSearch(searchTerm, this);   		
  		}
  		
  		
 	},
 	initiateContextSearch: function(searchTerm, isTextCreated, state) {
 		var stateNumber = state % 3;
 		if( this._contextTaskDataStore) { 
 			if(isTextCreated) {
  				dojo.destroy('searchingText'); 
  			}	
  			this._contextTaskDataStore.initiateTaskSearch(searchTerm, this); 
  		} else {
  			if(!isTextCreated) {
  				dojo.create("div", {id:'searchingText', innerHTML:this.MRI['Searching' + stateNumber]}, this._searchDetailDlg.containerNode); 
  			} else {
  				dojo.byId('searchingText').innerHTML= this.MRI['Searching' + stateNumber];
  			}
  			setTimeout(dojo.hitch(this, 'initiateContextSearch', searchTerm, true, state + 1), 200);
  		} 
 	},
 	initializeDialogConstruct: function(typeOfSearch, items) {
 		
 		if(this._searchDlgInitialized == false) {
 			this._searchDlgInitialized = true;
 			dojo.empty(this._searchDetailDlg.containerNode);   
  			dojo.create("img",{onclick:dojo.hitch(this, this.closeSearchDetailDlg),
   		   					src:contextPath + "/images/title_close.gif", 
   		   					title:this.MRI.Close, 
   		   					className:"closeImg"}, this._searchDetailDlg.containerNode); 
   		   	this._refreshImage = dojo.create("img",{onclick:dojo.hitch(this, this.refreshCache),
   		   					src:contextPath + "/images/hardwareView/ac16_refresh.png",    		   					
   		   					className:"refreshImg"}, this._searchDetailDlg.containerNode); 
   		   	this.changeRefreshCount();
 			var header = dojo.create("span", {id: this._finderIdPrefix + "finderMenu", 
 									          style:"display:inline;"},  this._searchDetailDlg.containerNode);        	
        	if(typeOfSearch == 'contextTasks') {
        		var span1 = dojo.create("span", {id: this._finderIdPrefix + 'contextTasks' + "Menu", 
        										 className:'finderMenuItemSelected', 
        										 innerHTML:this._contextFinderMenuText},  header);
       	 		var div = dojo.create("div", {id: this._finderIdPrefix + 'contextTasks' + "_containerDiv"},  this._searchDetailDlg.containerNode);
        	} else {
        		var mainDivTabContainer = dojo.create("div", {id:'mainDivTabContainer'}, this._searchDetailDlg.containerNode);
 				if(dijit.byId(this._finderIdPrefix + "tab")) {
  					dijit.byId(this._finderIdPrefix + "tab").destroy();
  				}    		
  				this._tabbedContainer = new dijit.layout.TabContainer({id:this._finderIdPrefix + "tab", style:"width:100%;", doLayout:true, useMenu:false, useSlider:false}, mainDivTabContainer);
 				var lastSelectedTab = dojo.cookie("finder.currentView");     
 				if(lastSelectedTab != null && lastSelectedTab.length > 0) {
 					this._currentView = lastSelectedTab;
 				}   		
 				this._tabbedContainer.addChild(this.initializeMenu('resources', this.MRI.FinderResourceMenu, header, this._tabbedContainer, (this._currentView == 'resources')));
        		this._tabbedContainer.addChild(this.initializeMenu('groups', this.MRI.FinderGroupMenu, header, this._tabbedContainer, (this._currentView == 'groups')));
        		this._tabbedContainer.addChild(this.initializeMenu('tasks', this.MRI.FinderTaskMenu , header, this._tabbedContainer, (this._currentView == 'tasks')));
        		dojo.connect(this._tabbedContainer,"selectChild", dojo.hitch(this, this.tabSelected));        		
	    		
        	}
 		}     
 		//add the count in parenthesis to the tabs
 		var numItems;
 		if(items) numItems = items;
 		else numItems = 0;
 		if(this._tabbedContainer) {
        	var children = this._tabbedContainer.getChildren();
        	var idx = 0;
        	if(typeOfSearch == 'groups') {
        		idx = 1;
        	} else if(typeOfSearch == 'tasks') {
        		idx = 2;
        	}       		
        	children[idx].title = children[idx].title + "(" + numItems + ")"; 
        }	
 	},
 	tabSelected: function(child) { 	
 		if(child.content.id.indexOf('tasks') != -1) {
 			this._currentView = 'tasks';
 		} else if(child.content.id.indexOf('resources') != -1){
 			this._currentView = 'resources'; 			
 		} else if(child.content.id.indexOf('groups') != -1){
 			this._currentView = 'groups'; 			
 		} 		
	    dojo.cookie("finder.currentView", this._currentView, { expires: 30});
 		phxTrace(child);
 		phxTrace('Current selection is ' + this._currentView);
 	},
 	initializeMenu: function(type, headerText, header, tabbedContainer, isSelected) {
 			var div = dojo.create("div", {id: this._finderIdPrefix + type + "_containerDiv"});
       	 	var cp1 = new dijit.layout.ContentPane({title: headerText,content: div, refreshOnShow:true, selected: isSelected});
       	 	return cp1;        	       
 	},
 	initializeTaskCache: function() {
 	phxTrace('selectedChassisList: ' + this._selectedChassisListInput.innerHTML);
    
    var collectionId = "ALL_CHASSIS";
    var selection = this._selectedChassisListInput.innerHTML;
    if(isChassisMapRendered()) {
    	selection = this._selectedChassisMapListInput.innerHTML;
    	collectionId = "CHASSIS_MAP_COLLECTION";
    }
    if(selection == "") {
    	return;
    }
 	var dataset = new ibm.tivoli.navmodel.Dataset({
        baseUrl: "/ibm/tivoli/rest",
        provider: "USMI",                 
        datasource: "",
        dataset: collectionId,    
        parameters: {
              homeParms: this._homeParamsForDataSet
        },   
        defaultProperties: ["all"],
        useDatastore: false
    });  
    phxTrace("fetching dataset items... for finder");
    dataset._fetchTasks({onComplete: dojo.hitch(this, this.datasetTasksRetrieved),  
    				  onError: dojo.hitch(this, this.datasetRetrievedError),
    				  itemIds: selection});    
    phxTrace("done fetching dataset items for finder");
    phxTrace(dataset);
 	}, 
 	
 	datasetItemsRetrieved: function(items) {    
    	phxTrace(items);    	
    	for(var i = 0; i < items.length; i++) {
    		phxTrace('getting tasks for ' + items[i].getId());
    		items[i].getDataset()._fetchTasks({onComplete: dojo.hitch(this, this.datasetTasksRetrieved),  
    				  						  onError: dojo.hitch(this, this.datasetRetrievedError),
    				  						  itemIds: items[i].getId()});
    	}
    	
  	},
  	datasetTasksRetrieved: function(tasks, args) {
    	var taskList = this.createRecursiveContextTaskList(tasks, '', true, 'General');   
    	this._contextTaskDataStore = new itme.finder.ContextTaskDataStore('contextTasks', taskList);  
  	},
  	createRecursiveContextTaskList: function(tasks, taskList, first, category) {
  		for(var i = 0; i < tasks.length; i++) {
    		
    		var task = tasks[i];
    		if(task.getType() == 'separator') {
    			continue;
    		} else if(task.getType() == 'cascade') {
    			taskList = this.createRecursiveContextTaskList(task.getChildTasks(), taskList, first, task.getLabel());
    			continue;
    		} else if(first) { first = false;} else {
    			taskList = taskList + '#_';
    		}
    		var taskId = task.getId();
    		if(taskId == null || taskId.length == 0) {
    			if(i == 0) first = true;    
    			continue;			
    		} 
    		taskList = taskList + taskId + ';';
    		taskList = taskList + task.getLabel() + ';';
    		taskList = taskList + category;    		
    	}
    	return taskList;
  	},
 	datasetRetrievedError: function(error, request){
    	phxTrace("The items request to the datastore failed.  " + error);
    	phxTrace("- request: " + request);
  	}
 	
 	
 	
});

}
