/*
	Copyright (c) 2004-2010, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/

/*
	This is an optimized version of Dojo, built for deployment and not for
	development. To get sources and documentation, please visit:

		http://dojotoolkit.org
*/

;(function(){

	/*
	dojo, dijit, and dojox must always be the first three, and in that order.
	djConfig.scopeMap = [
		["dojo", "fojo"],
		["dijit", "fijit"],
		["dojox", "fojox"]
	
	]
	*/

	/**Build will replace this comment with a scoped djConfig **/

	//The null below can be relaced by a build-time value used instead of djConfig.scopeMap.
	var sMap = null;

	//See if new scopes need to be defined.
	if((sMap || (typeof djConfig != "undefined" && djConfig.scopeMap)) && (typeof window != "undefined")){
		var scopeDef = "", scopePrefix = "", scopeSuffix = "", scopeMap = {}, scopeMapRev = {};
		sMap = sMap || djConfig.scopeMap;
		for(var i = 0; i < sMap.length; i++){
			//Make local variables, then global variables that use the locals.
			var newScope = sMap[i];
			scopeDef += "var " + newScope[0] + " = {}; " + newScope[1] + " = " + newScope[0] + ";" + newScope[1] + "._scopeName = '" + newScope[1] + "';";
			scopePrefix += (i == 0 ? "" : ",") + newScope[0];
			scopeSuffix += (i == 0 ? "" : ",") + newScope[1];
			scopeMap[newScope[0]] = newScope[1];
			scopeMapRev[newScope[1]] = newScope[0];
		}

		eval(scopeDef + "dojo._scopeArgs = [" + scopeSuffix + "];");

		dojo._scopePrefixArgs = scopePrefix;
		dojo._scopePrefix = "(function(" + scopePrefix + "){";
		dojo._scopeSuffix = "})(" + scopeSuffix + ")";
		dojo._scopeMap = scopeMap;
		dojo._scopeMapRev = scopeMapRev;
	}

/*=====
// note:
//		'djConfig' does not exist under 'dojo.*' so that it can be set before the
//		'dojo' variable exists.
// note:
//		Setting any of these variables *after* the library has loaded does
//		nothing at all.

djConfig = {
	// summary:
	//		Application code can set the global 'djConfig' prior to loading
	//		the library to override certain global settings for how dojo works.
	//
	// isDebug: Boolean
	//		Defaults to `false`. If set to `true`, ensures that Dojo provides
	//		extended debugging feedback via Firebug. If Firebug is not available
	//		on your platform, setting `isDebug` to `true` will force Dojo to
	//		pull in (and display) the version of Firebug Lite which is
	//		integrated into the Dojo distribution, thereby always providing a
	//		debugging/logging console when `isDebug` is enabled. Note that
	//		Firebug's `console.*` methods are ALWAYS defined by Dojo. If
	//		`isDebug` is false and you are on a platform without Firebug, these
	//		methods will be defined as no-ops.
	isDebug: false,
	// debugAtAllCosts: Boolean
	//		Defaults to `false`. If set to `true`, this triggers an alternate
	//		mode of the package system in which dependencies are detected and
	//		only then are resources evaluated in dependency order via
	//		`<script>` tag inclusion. This may double-request resources and
	//		cause problems with scripts which expect `dojo.require()` to
	//		preform synchronously. `debugAtAllCosts` can be an invaluable
	//		debugging aid, but when using it, ensure that all code which
	//		depends on Dojo modules is wrapped in `dojo.addOnLoad()` handlers.
	//		Due to the somewhat unpredictable side-effects of using
	//		`debugAtAllCosts`, it is strongly recommended that you enable this
	//		flag as a last resort. `debugAtAllCosts` has no effect when loading
	//		resources across domains. For usage information, see the
	//		[Dojo Book](http://dojotoolkit.org/book/book-dojo/part-4-meta-dojo-making-your-dojo-code-run-faster-and-better/debugging-facilities/deb)
	debugAtAllCosts: false,
	// locale: String
	//		The locale to assume for loading localized resources in this page,
	//		specified according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt).
	//		Must be specified entirely in lowercase, e.g. `en-us` and `zh-cn`.
	//		See the documentation for `dojo.i18n` and `dojo.requireLocalization`
	//		for details on loading localized resources. If no locale is specified,
	//		Dojo assumes the locale of the user agent, according to `navigator.userLanguage`
	//		or `navigator.language` properties.
	locale: undefined,
	// extraLocale: Array
	//		No default value. Specifies additional locales whose
	//		resources should also be loaded alongside the default locale when
	//		calls to `dojo.requireLocalization()` are processed.
	extraLocale: undefined,
	// baseUrl: String
	//		The directory in which `dojo.js` is located. Under normal
	//		conditions, Dojo auto-detects the correct location from which it
	//		was loaded. You may need to manually configure `baseUrl` in cases
	//		where you have renamed `dojo.js` or in which `<base>` tags confuse
	//		some browsers (e.g. IE 6). The variable `dojo.baseUrl` is assigned
	//		either the value of `djConfig.baseUrl` if one is provided or the
	//		auto-detected root if not. Other modules are located relative to
	//		this path. The path should end in a slash.
	baseUrl: undefined,
	// modulePaths: Object
	//		A map of module names to paths relative to `dojo.baseUrl`. The
	//		key/value pairs correspond directly to the arguments which
	//		`dojo.registerModulePath` accepts. Specifiying
	//		`djConfig.modulePaths = { "foo": "../../bar" }` is the equivalent
	//		of calling `dojo.registerModulePath("foo", "../../bar");`. Multiple
	//		modules may be configured via `djConfig.modulePaths`.
	modulePaths: {},
	// afterOnLoad: Boolean 
	//		Indicates Dojo was added to the page after the page load. In this case
	//		Dojo will not wait for the page DOMContentLoad/load events and fire
	//		its dojo.addOnLoad callbacks after making sure all outstanding
	//		dojo.required modules have loaded. Only works with a built dojo.js,
	//		it does not work the dojo.js directly from source control.
	afterOnLoad: false,
	// addOnLoad: Function or Array
	//		Adds a callback via dojo.addOnLoad. Useful when Dojo is added after
	//		the page loads and djConfig.afterOnLoad is true. Supports the same
	//		arguments as dojo.addOnLoad. When using a function reference, use
	//		`djConfig.addOnLoad = function(){};`. For object with function name use
	//		`djConfig.addOnLoad = [myObject, "functionName"];` and for object with
	//		function reference use
	//		`djConfig.addOnLoad = [myObject, function(){}];`
	addOnLoad: null,
	// require: Array
	//		An array of module names to be loaded immediately after dojo.js has been included
	//		in a page.
	require: [],
	// defaultDuration: Array
	//		Default duration, in milliseconds, for wipe and fade animations within dijits.
	//		Assigned to dijit.defaultDuration.
	defaultDuration: 200,
	// dojoBlankHtmlUrl: String
	//		Used by some modules to configure an empty iframe. Used by dojo.io.iframe and
	//		dojo.back, and dijit popup support in IE where an iframe is needed to make sure native
	//		controls do not bleed through the popups. Normally this configuration variable 
	//		does not need to be set, except when using cross-domain/CDN Dojo builds.
	//		Save dojo/resources/blank.html to your domain and set `djConfig.dojoBlankHtmlUrl`
	//		to the path on your domain your copy of blank.html.
	dojoBlankHtmlUrl: undefined,
	//	ioPublish: Boolean?
	//		Set this to true to enable publishing of topics for the different phases of
	// 		IO operations. Publishing is done via dojo.publish. See dojo.__IoPublish for a list
	// 		of topics that are published.
	ioPublish: false,
	//  useCustomLogger: Anything?
	//		If set to a value that evaluates to true such as a string or array and
	//		isDebug is true and Firebug is not available or running, then it bypasses
	//		the creation of Firebug Lite allowing you to define your own console object.
	useCustomLogger: undefined,
	// transparentColor: Array
	//		Array containing the r, g, b components used as transparent color in dojo.Color;
	//		if undefined, [255,255,255] (white) will be used.
	transparentColor: undefined,
	// skipIeDomLoaded: Boolean
	//		For IE only, skip the DOMContentLoaded hack used. Sometimes it can cause an Operation
	//		Aborted error if the rest of the page triggers script defers before the DOM is ready.
	//		If this is config value is set to true, then dojo.addOnLoad callbacks will not be
	//		triggered until the page load event, which is after images and iframes load. If you
	//		want to trigger the callbacks sooner, you can put a script block in the bottom of
	//		your HTML that calls dojo._loadInit();. If you are using multiversion support, change
	//		"dojo." to the appropriate scope name for dojo.
	skipIeDomLoaded: false
}
=====*/

(function(){
	// firebug stubs

	if(typeof this["loadFirebugConsole"] == "function"){
		// for Firebug 1.2
		this["loadFirebugConsole"]();
	}else{
		this.console = this.console || {};

		//	Be careful to leave 'log' always at the end
		var cn = [
			"assert", "count", "debug", "dir", "dirxml", "error", "group",
			"groupEnd", "info", "profile", "profileEnd", "time", "timeEnd",
			"trace", "warn", "log"
		];
		var i=0, tn;
		while((tn=cn[i++])){
			if(!console[tn]){
				(function(){
					var tcn = tn+"";
					console[tcn] = ('log' in console) ? function(){
						var a = Array.apply({}, arguments);
						a.unshift(tcn+":");
						console["log"](a.join(" "));
					} : function(){}
					console[tcn]._fake = true;
				})();
			}
		}
	}

	//TODOC:  HOW TO DOC THIS?
	// dojo is the root variable of (almost all) our public symbols -- make sure it is defined.
	if(typeof dojo == "undefined"){
		dojo = {
			_scopeName: "dojo",
			_scopePrefix: "",
			_scopePrefixArgs: "",
			_scopeSuffix: "",
			_scopeMap: {},
			_scopeMapRev: {}
		};
	}

	var d = dojo;

	//Need placeholders for dijit and dojox for scoping code.
	if(typeof dijit == "undefined"){
		dijit = {_scopeName: "dijit"};
	}
	if(typeof dojox == "undefined"){
		dojox = {_scopeName: "dojox"};
	}

	if(!d._scopeArgs){
		d._scopeArgs = [dojo, dijit, dojox];
	}

/*=====
dojo.global = {
	//	summary:
	//		Alias for the global scope
	//		(e.g. the window object in a browser).
	//	description:
	//		Refer to 'dojo.global' rather than referring to window to ensure your
	//		code runs correctly in contexts other than web browsers (e.g. Rhino on a server).
}
=====*/
	d.global = this;

	d.config =/*===== djConfig = =====*/{
		isDebug: false,
		debugAtAllCosts: false
	};

	if(typeof djConfig != "undefined"){
		for(var opt in djConfig){
			d.config[opt] = djConfig[opt];
		}
	}

/*=====
	// Override locale setting, if specified
	dojo.locale = {
		// summary: the locale as defined by Dojo (read-only)
	};
=====*/
	dojo.locale = d.config.locale;

	var rev = "$Rev: 22487 $".match(/\d+/);

/*=====
	dojo.version = function(){
		// summary:
		//		Version number of the Dojo Toolkit
		// major: Integer
		//		Major version. If total version is "1.2.0beta1", will be 1
		// minor: Integer
		//		Minor version. If total version is "1.2.0beta1", will be 2
		// patch: Integer
		//		Patch version. If total version is "1.2.0beta1", will be 0
		// flag: String
		//		Descriptor flag. If total version is "1.2.0beta1", will be "beta1"
		// revision: Number
		//		The SVN rev from which dojo was pulled
		this.major = 0;
		this.minor = 0;
		this.patch = 0;
		this.flag = "";
		this.revision = 0;
	}
=====*/
	dojo.version = {
		major: 0, minor: 0, patch: 0, flag: "dev",
		revision: rev ? +rev[0] : NaN,
		toString: function(){
			with(d.version){
				return major + "." + minor + "." + patch + flag + " (" + revision + ")";	// String
			}
		}
	}

		// Register with the OpenAjax hub
	if(typeof OpenAjax != "undefined"){
		OpenAjax.hub.registerLibrary(dojo._scopeName, "http://dojotoolkit.org", d.version.toString());
	}
	
	var extraNames, extraLen, empty = {};
	for(var i in {toString: 1}){ extraNames = []; break; }
	dojo._extraNames = extraNames = extraNames || ["hasOwnProperty", "valueOf", "isPrototypeOf",
		"propertyIsEnumerable", "toLocaleString", "toString", "constructor"];
	extraLen = extraNames.length;

	dojo._mixin = function(/*Object*/ target, /*Object*/ source){
		// summary:
		//		Adds all properties and methods of source to target. This addition
		//		is "prototype extension safe", so that instances of objects
		//		will not pass along prototype defaults.
		var name, s, i;
		for(name in source){
			// the "tobj" condition avoid copying properties in "source"
			// inherited from Object.prototype.  For example, if target has a custom
			// toString() method, don't overwrite it with the toString() method
			// that source inherited from Object.prototype
			s = source[name];
			if(!(name in target) || (target[name] !== s && (!(name in empty) || empty[name] !== s))){
				target[name] = s;
			}
		}
				// IE doesn't recognize some custom functions in for..in
		if(extraLen && source){
			for(i = 0; i < extraLen; ++i){
				name = extraNames[i];
				s = source[name];
				if(!(name in target) || (target[name] !== s && (!(name in empty) || empty[name] !== s))){
					target[name] = s;
				}
			}
		}
				return target; // Object
	}

	dojo.mixin = function(/*Object*/obj, /*Object...*/props){
		// summary:
		//		Adds all properties and methods of props to obj and returns the
		//		(now modified) obj.
		//	description:
		//		`dojo.mixin` can mix multiple source objects into a
		//		destination object which is then returned. Unlike regular
		//		`for...in` iteration, `dojo.mixin` is also smart about avoiding
		//		extensions which other toolkits may unwisely add to the root
		//		object prototype
		//	obj:
		//		The object to mix properties into. Also the return value.
		//	props:
		//		One or more objects whose values are successively copied into
		//		obj. If more than one of these objects contain the same value,
		//		the one specified last in the function call will "win".
		//	example:
		//		make a shallow copy of an object
		//	|	var copy = dojo.mixin({}, source);
		//	example:
		//		many class constructors often take an object which specifies
		//		values to be configured on the object. In this case, it is
		//		often simplest to call `dojo.mixin` on the `this` object:
		//	|	dojo.declare("acme.Base", null, {
		//	|		constructor: function(properties){
		//	|			// property configuration:
		//	|			dojo.mixin(this, properties);
		//	|	
		//	|			console.log(this.quip);
		//	|			//  ...
		//	|		},
		//	|		quip: "I wasn't born yesterday, you know - I've seen movies.",
		//	|		// ...
		//	|	});
		//	|
		//	|	// create an instance of the class and configure it
		//	|	var b = new acme.Base({quip: "That's what it does!" });
		//	example:
		//		copy in properties from multiple objects
		//	|	var flattened = dojo.mixin(
		//	|		{
		//	|			name: "Frylock",
		//	|			braces: true
		//	|		},
		//	|		{
		//	|			name: "Carl Brutanananadilewski"
		//	|		}
		//	|	);
		//	|	
		//	|	// will print "Carl Brutanananadilewski"
		//	|	console.log(flattened.name);
		//	|	// will print "true"
		//	|	console.log(flattened.braces);
		if(!obj){ obj = {}; }
		for(var i=1, l=arguments.length; i<l; i++){
			d._mixin(obj, arguments[i]);
		}
		return obj; // Object
	}

	dojo._getProp = function(/*Array*/parts, /*Boolean*/create, /*Object*/context){
		var obj=context || d.global;
		for(var i=0, p; obj && (p=parts[i]); i++){
			if(i == 0 && d._scopeMap[p]){
				p = d._scopeMap[p];
			}
			obj = (p in obj ? obj[p] : (create ? obj[p]={} : undefined));
		}
		return obj; // mixed
	}

	dojo.setObject = function(/*String*/name, /*Object*/value, /*Object?*/context){
		// summary:
		//		Set a property from a dot-separated string, such as "A.B.C"
		//	description:
		//		Useful for longer api chains where you have to test each object in
		//		the chain, or when you have an object reference in string format.
		//		Objects are created as needed along `path`. Returns the passed
		//		value if setting is successful or `undefined` if not.
		//	name:
		//		Path to a property, in the form "A.B.C".
		//	context:
		//		Optional. Object to use as root of path. Defaults to
		//		`dojo.global`.
		//	example:
		//		set the value of `foo.bar.baz`, regardless of whether
		//		intermediate objects already exist:
		//	|	dojo.setObject("foo.bar.baz", value);
		//	example:
		//		without `dojo.setObject`, we often see code like this:
		//	|	// ensure that intermediate objects are available
		//	|	if(!obj["parent"]){ obj.parent = {}; }
		//	|	if(!obj.parent["child"]){ obj.parent.child= {}; }
		//	|	// now we can safely set the property
		//	|	obj.parent.child.prop = "some value";
		//		wheras with `dojo.setObject`, we can shorten that to:
		//	|	dojo.setObject("parent.child.prop", "some value", obj);
		var parts=name.split("."), p=parts.pop(), obj=d._getProp(parts, true, context);
		return obj && p ? (obj[p]=value) : undefined; // Object
	}

	dojo.getObject = function(/*String*/name, /*Boolean?*/create, /*Object?*/context){
		// summary:
		//		Get a property from a dot-separated string, such as "A.B.C"
		//	description:
		//		Useful for longer api chains where you have to test each object in
		//		the chain, or when you have an object reference in string format.
		//	name:
		//		Path to an property, in the form "A.B.C".
		//	create:
		//		Optional. Defaults to `false`. If `true`, Objects will be
		//		created at any point along the 'path' that is undefined.
		//	context:
		//		Optional. Object to use as root of path. Defaults to
		//		'dojo.global'. Null may be passed.
		return d._getProp(name.split("."), create, context); // Object
	}

	dojo.exists = function(/*String*/name, /*Object?*/obj){
		//	summary:
		//		determine if an object supports a given method
		//	description:
		//		useful for longer api chains where you have to test each object in
		//		the chain. Useful only for object and method detection.
		//		Not useful for testing generic properties on an object.
		//		In particular, dojo.exists("foo.bar") when foo.bar = ""
		//		will return false. Use ("bar" in foo) to test for those cases.
		//	name:
		//		Path to an object, in the form "A.B.C".
		//	obj:
		//		Object to use as root of path. Defaults to
		//		'dojo.global'. Null may be passed.
		//	example:
		//	|	// define an object
		//	|	var foo = {
		//	|		bar: { }
		//	|	};
		//	|
		//	|	// search the global scope
		//	|	dojo.exists("foo.bar"); // true
		//	|	dojo.exists("foo.bar.baz"); // false
		//	|
		//	|	// search from a particular scope
		//	|	dojo.exists("bar", foo); // true
		//	|	dojo.exists("bar.baz", foo); // false
		return !!d.getObject(name, false, obj); // Boolean
	}

	dojo["eval"] = function(/*String*/ scriptFragment){
		//	summary:
		//		A legacy method created for use exclusively by internal Dojo methods. Do not use
		//		this method directly, the behavior of this eval will differ from the normal
		//		browser eval.
		//	description:
		//		Placed in a separate function to minimize size of trapped
		//		exceptions. Calling eval() directly from some other scope may
		//		complicate tracebacks on some platforms.
		//	returns:
		//		The result of the evaluation. Often `undefined`
		return d.global.eval ? d.global.eval(scriptFragment) : eval(scriptFragment); 	// Object
	}

	/*=====
		dojo.deprecated = function(behaviour, extra, removal){
			//	summary:
			//		Log a debug message to indicate that a behavior has been
			//		deprecated.
			//	behaviour: String
			//		The API or behavior being deprecated. Usually in the form
			//		of "myApp.someFunction()".
			//	extra: String?
			//		Text to append to the message. Often provides advice on a
			//		new function or facility to achieve the same goal during
			//		the deprecation period.
			//	removal: String?
			//		Text to indicate when in the future the behavior will be
			//		removed. Usually a version number.
			//	example:
			//	|	dojo.deprecated("myApp.getTemp()", "use myApp.getLocaleTemp() instead", "1.0");
		}

		dojo.experimental = function(moduleName, extra){
			//	summary: Marks code as experimental.
			//	description:
			//	 	This can be used to mark a function, file, or module as
			//	 	experimental.  Experimental code is not ready to be used, and the
			//	 	APIs are subject to change without notice.  Experimental code may be
			//	 	completed deleted without going through the normal deprecation
			//	 	process.
			//	moduleName: String
			//	 	The name of a module, or the name of a module file or a specific
			//	 	function
			//	extra: String?
			//	 	some additional message for the user
			//	example:
			//	|	dojo.experimental("dojo.data.Result");
			//	example:
			//	|	dojo.experimental("dojo.weather.toKelvin()", "PENDING approval from NOAA");
		}
	=====*/

	//Real functions declared in dojo._firebug.firebug.
	d.deprecated = d.experimental = function(){};

})();
// vim:ai:ts=4:noet

/*
 * loader.js - A bootstrap module.  Runs before the hostenv_*.js file. Contains
 * all of the package loading methods.
 */

(function(){
	var d = dojo;

	d.mixin(d, {
		_loadedModules: {},
		_inFlightCount: 0,
		_hasResource: {},

		_modulePrefixes: {
			dojo: 	{	name: "dojo", value: "." },
			// dojox: 	{	name: "dojox", value: "../dojox" },
			// dijit: 	{	name: "dijit", value: "../dijit" },
			doh: 	{	name: "doh", value: "../util/doh" },
			tests: 	{	name: "tests", value: "tests" }
		},

		_moduleHasPrefix: function(/*String*/module){
			// summary: checks to see if module has been established
			var mp = d._modulePrefixes;
			return !!(mp[module] && mp[module].value); // Boolean
		},

		_getModulePrefix: function(/*String*/module){
			// summary: gets the prefix associated with module
			var mp = d._modulePrefixes;
			if(d._moduleHasPrefix(module)){
				return mp[module].value; // String
			}
			return module; // String
		},

		_loadedUrls: [],

		//WARNING: 
		//		This variable is referenced by packages outside of bootstrap:
		//		FloatingPane.js and undo/browser.js
		_postLoad: false,
		
		//Egad! Lots of test files push on this directly instead of using dojo.addOnLoad.
		_loaders: [],
		_unloaders: [],
		_loadNotifying: false
	});


		dojo._loadPath = function(/*String*/relpath, /*String?*/module, /*Function?*/cb){
		// 	summary:
		//		Load a Javascript module given a relative path
		//
		//	description:
		//		Loads and interprets the script located at relpath, which is
		//		relative to the script root directory.  If the script is found but
		//		its interpretation causes a runtime exception, that exception is
		//		not caught by us, so the caller will see it.  We return a true
		//		value if and only if the script is found.
		//
		// relpath: 
		//		A relative path to a script (no leading '/', and typically ending
		//		in '.js').
		// module: 
		//		A module whose existance to check for after loading a path.  Can be
		//		used to determine success or failure of the load.
		// cb: 
		//		a callback function to pass the result of evaluating the script

		var uri = ((relpath.charAt(0) == '/' || relpath.match(/^\w+:/)) ? "" : d.baseUrl) + relpath;
		try{
			return !module ? d._loadUri(uri, cb) : d._loadUriAndCheck(uri, module, cb); // Boolean
		}catch(e){
			console.error(e);
			return false; // Boolean
		}
	}

	dojo._loadUri = function(/*String*/uri, /*Function?*/cb){
		//	summary:
		//		Loads JavaScript from a URI
		//	description:
		//		Reads the contents of the URI, and evaluates the contents.  This is
		//		used to load modules as well as resource bundles. Returns true if
		//		it succeeded. Returns false if the URI reading failed.  Throws if
		//		the evaluation throws.
		//	uri: a uri which points at the script to be loaded
		//	cb: 
		//		a callback function to process the result of evaluating the script
		//		as an expression, typically used by the resource bundle loader to
		//		load JSON-style resources

		if(d._loadedUrls[uri]){
			return true; // Boolean
		}
		d._inFlightCount++; // block addOnLoad calls that arrive while we're busy downloading
		var contents = d._getText(uri, true);
		if(contents){ // not 404, et al
			d._loadedUrls[uri] = true;
			d._loadedUrls.push(uri);
			if(cb){
				contents = '('+contents+')';
			}else{
				//Only do the scoping if no callback. If a callback is specified,
				//it is most likely the i18n bundle stuff.
				contents = d._scopePrefix + contents + d._scopeSuffix;
			}
			if(!d.isIE){ contents += "\r\n//@ sourceURL=" + uri; } // debugging assist for Firebug
			var value = d["eval"](contents);
			if(cb){ cb(value); }
		}
		// Check to see if we need to call _callLoaded() due to an addOnLoad() that arrived while we were busy downloading
		if(--d._inFlightCount == 0 && d._postLoad && d._loaders.length){
			// We shouldn't be allowed to get here but Firefox allows an event 
			// (mouse, keybd, async xhrGet) to interrupt a synchronous xhrGet. 
			// If the current script block contains multiple require() statements, then after each
			// require() returns, inFlightCount == 0, but we want to hold the _callLoaded() until
			// all require()s are done since the out-of-sequence addOnLoad() presumably needs them all.
			// setTimeout allows the next require() to start (if needed), and then we check this again.
			setTimeout(function(){ 
				// If inFlightCount > 0, then multiple require()s are running sequentially and 
				// the next require() started after setTimeout() was executed but before we got here.
				if(d._inFlightCount == 0){ 
					d._callLoaded();
				}
			}, 0);
		}
		return !!contents; // Boolean: contents? true : false
	}
	
	// FIXME: probably need to add logging to this method
	dojo._loadUriAndCheck = function(/*String*/uri, /*String*/moduleName, /*Function?*/cb){
		// summary: calls loadUri then findModule and returns true if both succeed
		var ok = false;
		try{
			ok = d._loadUri(uri, cb);
		}catch(e){
			console.error("failed loading " + uri + " with error: " + e);
		}
		return !!(ok && d._loadedModules[moduleName]); // Boolean
	}

	dojo.loaded = function(){
		// summary:
		//		signal fired when initial environment and package loading is
		//		complete. You should use dojo.addOnLoad() instead of doing a 
		//		direct dojo.connect() to this method in order to handle
		//		initialization tasks that require the environment to be
		//		initialized. In a browser host,	declarative widgets will 
		//		be constructed when this function finishes runing.
		d._loadNotifying = true;
		d._postLoad = true;
		var mll = d._loaders;

		//Clear listeners so new ones can be added
		//For other xdomain package loads after the initial load.
		d._loaders = [];

		for(var x = 0; x < mll.length; x++){
			mll[x]();
		}

		d._loadNotifying = false;
		
		//Make sure nothing else got added to the onload queue
		//after this first run. If something did, and we are not waiting for any
		//more inflight resources, run again.
		if(d._postLoad && d._inFlightCount == 0 && mll.length){
			d._callLoaded();
		}
	}

	dojo.unloaded = function(){
		// summary:
		//		signal fired by impending environment destruction. You should use
		//		dojo.addOnUnload() instead of doing a direct dojo.connect() to this 
		//		method to perform page/application cleanup methods. See 
		//		dojo.addOnUnload for more info.
		var mll = d._unloaders;
		while(mll.length){
			(mll.pop())();
		}
	}

	d._onto = function(arr, obj, fn){
		if(!fn){
			arr.push(obj);
		}else if(fn){
			var func = (typeof fn == "string") ? obj[fn] : fn;
			arr.push(function(){ func.call(obj); });
		}
	}

	dojo.ready = dojo.addOnLoad = function(/*Object*/obj, /*String|Function?*/functionName){
		// summary:
		//		Registers a function to be triggered after the DOM and dojo.require() calls 
		//		have finished loading.
		//
		// description:
		//		Registers a function to be triggered after the DOM has finished
		//		loading and `dojo.require` modules have loaded. Widgets declared in markup 
		//		have been instantiated if `djConfig.parseOnLoad` is true when this fires. 
		//
		//		Images and CSS files may or may not have finished downloading when
		//		the specified function is called.  (Note that widgets' CSS and HTML
		//		code is guaranteed to be downloaded before said widgets are
		//		instantiated, though including css resouces BEFORE any script elements
		//		is highly recommended).
		//
		// example:
		//	Register an anonymous function to run when everything is ready
		//	|	dojo.addOnLoad(function(){ doStuff(); });
		//
		// example:
		//	Register a function to run when everything is ready by pointer:
		//	|	var init = function(){ doStuff(); }
		//	|	dojo.addOnLoad(init);
		//
		// example:
		//	Register a function to run scoped to `object`, either by name or anonymously:
		//	|	dojo.addOnLoad(object, "functionName");
		//	|	dojo.addOnLoad(object, function(){ doStuff(); });

		d._onto(d._loaders, obj, functionName);

		//Added for xdomain loading. dojo.addOnLoad is used to
		//indicate callbacks after doing some dojo.require() statements.
		//In the xdomain case, if all the requires are loaded (after initial
		//page load), then immediately call any listeners.
		if(d._postLoad && d._inFlightCount == 0 && !d._loadNotifying){
			d._callLoaded();
		}
	}

	//Support calling dojo.addOnLoad via djConfig.addOnLoad. Support all the
	//call permutations of dojo.addOnLoad. Mainly useful when dojo is added
	//to the page after the page has loaded.
	var dca = d.config.addOnLoad;
	if(dca){
		d.addOnLoad[(dca instanceof Array ? "apply" : "call")](d, dca);
	}

	dojo._modulesLoaded = function(){
		if(d._postLoad){ return; }
		if(d._inFlightCount > 0){ 
			console.warn("files still in flight!");
			return;
		}
		d._callLoaded();
	}

	dojo._callLoaded = function(){

		// The "object" check is for IE, and the other opera check fixes an
		// issue in Opera where it could not find the body element in some
		// widget test cases.  For 0.9, maybe route all browsers through the
		// setTimeout (need protection still for non-browser environments
		// though). This might also help the issue with FF 2.0 and freezing
		// issues where we try to do sync xhr while background css images are
		// being loaded (trac #2572)? Consider for 0.9.
		if(typeof setTimeout == "object" || (d.config.useXDomain && d.isOpera)){
			setTimeout(
				d.isAIR ? function(){ d.loaded(); } : d._scopeName + ".loaded();",
				0);
		}else{
			d.loaded();
		}
	}

	dojo._getModuleSymbols = function(/*String*/modulename){
		// summary:
		//		Converts a module name in dotted JS notation to an array
		//		representing the path in the source tree
		var syms = modulename.split(".");
		for(var i = syms.length; i>0; i--){
			var parentModule = syms.slice(0, i).join(".");
			if(i == 1 && !d._moduleHasPrefix(parentModule)){		
				// Support default module directory (sibling of dojo) for top-level modules 
				syms[0] = "../" + syms[0];
			}else{
				var parentModulePath = d._getModulePrefix(parentModule);
				if(parentModulePath != parentModule){
					syms.splice(0, i, parentModulePath);
					break;
				}
			}
		}
		return syms; // Array
	}

	dojo._global_omit_module_check = false;

	dojo.loadInit = function(/*Function*/init){
		//	summary:
		//		Executes a function that needs to be executed for the loader's dojo.requireIf
		//		resolutions to work. This is needed mostly for the xdomain loader case where
		//		a function needs to be executed to set up the possible values for a dojo.requireIf
		//		call.
		//	init:
		//		a function reference. Executed immediately.
		//	description: This function is mainly a marker for the xdomain loader to know parts of
		//		code that needs be executed outside the function wrappper that is placed around modules.
		//		The init function could be executed more than once, and it should make no assumptions
		//		on what is loaded, or what modules are available. Only the functionality in Dojo Base
		//		is allowed to be used. Avoid using this method. For a valid use case,
		//		see the source for dojox.gfx.
		init();
	}

	dojo._loadModule = dojo.require = function(/*String*/moduleName, /*Boolean?*/omitModuleCheck){
		//	summary:
		//		loads a Javascript module from the appropriate URI
		//	moduleName:
		//		module name to load, using periods for separators,
		//		 e.g. "dojo.date.locale".  Module paths are de-referenced by dojo's
		//		internal mapping of locations to names and are disambiguated by
		//		longest prefix. See `dojo.registerModulePath()` for details on
		//		registering new modules.
		//	omitModuleCheck:
		//		if `true`, omitModuleCheck skips the step of ensuring that the
		//		loaded file actually defines the symbol it is referenced by.
		//		For example if it called as `dojo.require("a.b.c")` and the
		//		file located at `a/b/c.js` does not define an object `a.b.c`,
		//		and exception will be throws whereas no exception is raised
		//		when called as `dojo.require("a.b.c", true)`
		//	description:
		// 		Modules are loaded via dojo.require by using one of two loaders: the normal loader
		// 		and the xdomain loader. The xdomain loader is used when dojo was built with a
		// 		custom build that specified loader=xdomain and the module lives on a modulePath
		// 		that is a whole URL, with protocol and a domain. The versions of Dojo that are on
		// 		the Google and AOL CDNs use the xdomain loader.
		// 
		// 		If the module is loaded via the xdomain loader, it is an asynchronous load, since
		// 		the module is added via a dynamically created script tag. This
		// 		means that dojo.require() can return before the module has loaded. However, this 
		// 		should only happen in the case where you do dojo.require calls in the top-level
		// 		HTML page, or if you purposely avoid the loader checking for dojo.require
		// 		dependencies in your module by using a syntax like dojo["require"] to load the module.
		// 
		// 		Sometimes it is useful to not have the loader detect the dojo.require calls in the
		// 		module so that you can dynamically load the modules as a result of an action on the
		// 		page, instead of right at module load time.
		// 
		// 		Also, for script blocks in an HTML page, the loader does not pre-process them, so
		// 		it does not know to download the modules before the dojo.require calls occur.
		// 
		// 		So, in those two cases, when you want on-the-fly module loading or for script blocks
		// 		in the HTML page, special care must be taken if the dojo.required code is loaded
		// 		asynchronously. To make sure you can execute code that depends on the dojo.required
		// 		modules, be sure to add the code that depends on the modules in a dojo.addOnLoad()
		// 		callback. dojo.addOnLoad waits for all outstanding modules to finish loading before
		// 		executing. Example:
		// 
		//	   	|	<script type="text/javascript">
		//		|	dojo.require("foo");
		//		|	dojo.require("bar");
		//	   	|	dojo.addOnLoad(function(){
		//	   	|		//you can now safely do something with foo and bar
		//	   	|	});
		//	   	|	</script>
		// 
		// 		This type of syntax works with both xdomain and normal loaders, so it is good
		// 		practice to always use this idiom for on-the-fly code loading and in HTML script
		// 		blocks. If at some point you change loaders and where the code is loaded from,
		// 		it will all still work.
		// 
		// 		More on how dojo.require
		//		`dojo.require("A.B")` first checks to see if symbol A.B is
		//		defined. If it is, it is simply returned (nothing to do).
		//	
		//		If it is not defined, it will look for `A/B.js` in the script root
		//		directory.
		//	
		//		`dojo.require` throws an excpetion if it cannot find a file
		//		to load, or if the symbol `A.B` is not defined after loading.
		//	
		//		It returns the object `A.B`, but note the caveats above about on-the-fly loading and
		// 		HTML script blocks when the xdomain loader is loading a module.
		//	
		//		`dojo.require()` does nothing about importing symbols into
		//		the current namespace.  It is presumed that the caller will
		//		take care of that. For example, to import all symbols into a
		//		local block, you might write:
		//	
		//		|	with (dojo.require("A.B")) {
		//		|		...
		//		|	}
		//	
		//		And to import just the leaf symbol to a local variable:
		//	
		//		|	var B = dojo.require("A.B");
		//	   	|	...
		//	returns: the required namespace object
		omitModuleCheck = d._global_omit_module_check || omitModuleCheck;

		//Check if it is already loaded.
		var module = d._loadedModules[moduleName];
		if(module){
			return module;
		}

		// convert periods to slashes
		var relpath = d._getModuleSymbols(moduleName).join("/") + '.js';

		var modArg = !omitModuleCheck ? moduleName : null;
		var ok = d._loadPath(relpath, modArg);

		if(!ok && !omitModuleCheck){
			throw new Error("Could not load '" + moduleName + "'; last tried '" + relpath + "'");
		}

		// check that the symbol was defined
		// Don't bother if we're doing xdomain (asynchronous) loading.
		if(!omitModuleCheck && !d._isXDomain){
			// pass in false so we can give better error
			module = d._loadedModules[moduleName];
			if(!module){
				throw new Error("symbol '" + moduleName + "' is not defined after loading '" + relpath + "'"); 
			}
		}

		return module;
	}

	dojo.provide = function(/*String*/ resourceName){
		//	summary:
		//		Register a resource with the package system. Works in conjunction with `dojo.require`
		//
		//	description:
		//		Each javascript source file is called a resource.  When a
		//		resource is loaded by the browser, `dojo.provide()` registers
		//		that it has been loaded.
		//
		//		Each javascript source file must have at least one
		//		`dojo.provide()` call at the top of the file, corresponding to
		//		the file name.  For example, `js/dojo/foo.js` must have
		//		`dojo.provide("dojo.foo");` before any calls to
		//		`dojo.require()` are made.
		//	
		//		For backwards compatibility reasons, in addition to registering
		//		the resource, `dojo.provide()` also ensures that the javascript
		//		object for the module exists.  For example,
		//		`dojo.provide("dojox.data.FlickrStore")`, in addition to
		//		registering that `FlickrStore.js` is a resource for the
		//		`dojox.data` module, will ensure that the `dojox.data`
		//		javascript object exists, so that calls like 
		//		`dojo.data.foo = function(){ ... }` don't fail.
		//
		//		In the case of a build where multiple javascript source files
		//		are combined into one bigger file (similar to a .lib or .jar
		//		file), that file may contain multiple dojo.provide() calls, to
		//		note that it includes multiple resources.
		//
		// resourceName: String
		//		A dot-sperated string identifying a resource. 
		//
		// example:
		//	Safely create a `my` object, and make dojo.require("my.CustomModule") work
		//	|	dojo.provide("my.CustomModule"); 

		//Make sure we have a string.
		resourceName = resourceName + "";
		return (d._loadedModules[resourceName] = d.getObject(resourceName, true)); // Object
	}

	//Start of old bootstrap2:

	dojo.platformRequire = function(/*Object*/modMap){
		//	summary:
		//		require one or more modules based on which host environment
		//		Dojo is currently operating in
		//	description:
		//		This method takes a "map" of arrays which one can use to
		//		optionally load dojo modules. The map is indexed by the
		//		possible dojo.name_ values, with two additional values:
		//		"default" and "common". The items in the "default" array will
		//		be loaded if none of the other items have been choosen based on
		//		dojo.name_, set by your host environment. The items in the
		//		"common" array will *always* be loaded, regardless of which
		//		list is chosen.
		//	example:
		//		|	dojo.platformRequire({
		//		|		browser: [
		//		|			"foo.sample", // simple module
		//		|			"foo.test",
		//		|			["foo.bar.baz", true] // skip object check in _loadModule (dojo.require)
		//		|		],
		//		|		default: [ "foo.sample._base" ],
		//		|		common: [ "important.module.common" ]
		//		|	});

		var common = modMap.common || [];
		var result = common.concat(modMap[d._name] || modMap["default"] || []);

		for(var x=0; x<result.length; x++){
			var curr = result[x];
			if(curr.constructor == Array){
				d._loadModule.apply(d, curr);
			}else{
				d._loadModule(curr);
			}
		}
	}

	dojo.requireIf = function(/*Boolean*/ condition, /*String*/ resourceName){
		// summary:
		//		If the condition is true then call `dojo.require()` for the specified
		//		resource
		//
		// example:
		//	|	dojo.requireIf(dojo.isBrowser, "my.special.Module");
		
		if(condition === true){
			// FIXME: why do we support chained require()'s here? does the build system?
			var args = [];
			for(var i = 1; i < arguments.length; i++){ 
				args.push(arguments[i]);
			}
			d.require.apply(d, args);
		}
	}

	dojo.requireAfterIf = d.requireIf;

	dojo.registerModulePath = function(/*String*/module, /*String*/prefix){
		//	summary: 
		//		Maps a module name to a path
		//	description: 
		//		An unregistered module is given the default path of ../[module],
		//		relative to Dojo root. For example, module acme is mapped to
		//		../acme.  If you want to use a different module name, use
		//		dojo.registerModulePath. 
		//	example:
		//		If your dojo.js is located at this location in the web root:
		//	|	/myapp/js/dojo/dojo/dojo.js
		//		and your modules are located at:
		//	|	/myapp/js/foo/bar.js
		//	|	/myapp/js/foo/baz.js
		//	|	/myapp/js/foo/thud/xyzzy.js
		//		Your application can tell Dojo to locate the "foo" namespace by calling:
		//	|	dojo.registerModulePath("foo", "../../foo");
		//		At which point you can then use dojo.require() to load the
		//		modules (assuming they provide() the same things which are
		//		required). The full code might be:
		//	|	<script type="text/javascript" 
		//	|		src="/myapp/js/dojo/dojo/dojo.js"></script>
		//	|	<script type="text/javascript">
		//	|		dojo.registerModulePath("foo", "../../foo");
		//	|		dojo.require("foo.bar");
		//	|		dojo.require("foo.baz");
		//	|		dojo.require("foo.thud.xyzzy");
		//	|	</script>
		d._modulePrefixes[module] = { name: module, value: prefix };
	}

	dojo.requireLocalization = function(/*String*/moduleName, /*String*/bundleName, /*String?*/locale, /*String?*/availableFlatLocales){
		// summary:
		//		Declares translated resources and loads them if necessary, in the
		//		same style as dojo.require.  Contents of the resource bundle are
		//		typically strings, but may be any name/value pair, represented in
		//		JSON format.  See also `dojo.i18n.getLocalization`.
		//
		// description:
		//		Load translated resource bundles provided underneath the "nls"
		//		directory within a package.  Translated resources may be located in
		//		different packages throughout the source tree.  
		//
		//		Each directory is named for a locale as specified by RFC 3066,
		//		(http://www.ietf.org/rfc/rfc3066.txt), normalized in lowercase.
		//		Note that the two bundles in the example do not define all the
		//		same variants.  For a given locale, bundles will be loaded for
		//		that locale and all more general locales above it, including a
		//		fallback at the root directory.  For example, a declaration for
		//		the "de-at" locale will first load `nls/de-at/bundleone.js`,
		//		then `nls/de/bundleone.js` and finally `nls/bundleone.js`.  The
		//		data will be flattened into a single Object so that lookups
		//		will follow this cascading pattern.  An optional build step can
		//		preload the bundles to avoid data redundancy and the multiple
		//		network hits normally required to load these resources.
		//
		// moduleName: 
		//		name of the package containing the "nls" directory in which the
		//		bundle is found
		//
		// bundleName: 
		//		bundle name, i.e. the filename without the '.js' suffix. Using "nls" as a
		//		a bundle name is not supported, since "nls" is the name of the folder
		//		that holds bundles. Using "nls" as the bundle name will cause problems
		//		with the custom build.
		//
		// locale: 
		//		the locale to load (optional)  By default, the browser's user
		//		locale as defined by dojo.locale
		//
		// availableFlatLocales: 
		//		A comma-separated list of the available, flattened locales for this
		//		bundle. This argument should only be set by the build process.
		//
		//	example:
		//		A particular widget may define one or more resource bundles,
		//		structured in a program as follows, where moduleName is
		//		mycode.mywidget and bundleNames available include bundleone and
		//		bundletwo:
		//	|		...
		//	|	mycode/
		//	|		mywidget/
		//	|			nls/
		//	|				bundleone.js (the fallback translation, English in this example)
		//	|				bundletwo.js (also a fallback translation)
		//	|				de/
		//	|					bundleone.js
		//	|					bundletwo.js
		//	|				de-at/
		//	|					bundleone.js
		//	|				en/
		//	|					(empty; use the fallback translation)
		//	|				en-us/
		//	|					bundleone.js
		//	|				en-gb/
		//	|					bundleone.js
		//	|				es/
		//	|					bundleone.js
		//	|					bundletwo.js
		//	|				  ...etc
		//	|				...
		//

		d.require("dojo.i18n");
		d.i18n._requireLocalization.apply(d.hostenv, arguments);
	};


	var ore = new RegExp("^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\\?([^#]*))?(#(.*))?$"),
		ire = new RegExp("^((([^\\[:]+):)?([^@]+)@)?(\\[([^\\]]+)\\]|([^\\[:]*))(:([0-9]+))?$");

	dojo._Url = function(/*dojo._Url|String...*/){
		// summary: 
		//		Constructor to create an object representing a URL.
		//		It is marked as private, since we might consider removing
		//		or simplifying it.
		// description: 
		//		Each argument is evaluated in order relative to the next until
		//		a canonical uri is produced. To get an absolute Uri relative to
		//		the current document use:
		//      	new dojo._Url(document.baseURI, url)

		var n = null,
			_a = arguments,
			uri = [_a[0]];
		// resolve uri components relative to each other
		for(var i = 1; i<_a.length; i++){
			if(!_a[i]){ continue; }

			// Safari doesn't support this.constructor so we have to be explicit
			// FIXME: Tracked (and fixed) in Webkit bug 3537.
			//		http://bugs.webkit.org/show_bug.cgi?id=3537
			var relobj = new d._Url(_a[i]+""),
				uriobj = new d._Url(uri[0]+"");

			if(
				relobj.path == "" &&
				!relobj.scheme &&
				!relobj.authority &&
				!relobj.query
			){
				if(relobj.fragment != n){
					uriobj.fragment = relobj.fragment;
				}
				relobj = uriobj;
			}else if(!relobj.scheme){
				relobj.scheme = uriobj.scheme;

				if(!relobj.authority){
					relobj.authority = uriobj.authority;

					if(relobj.path.charAt(0) != "/"){
						var path = uriobj.path.substring(0,
							uriobj.path.lastIndexOf("/") + 1) + relobj.path;

						var segs = path.split("/");
						for(var j = 0; j < segs.length; j++){
							if(segs[j] == "."){
								// flatten "./" references
								if(j == segs.length - 1){
									segs[j] = "";
								}else{
									segs.splice(j, 1);
									j--;
								}
							}else if(j > 0 && !(j == 1 && segs[0] == "") &&
								segs[j] == ".." && segs[j-1] != ".."){
								// flatten "../" references
								if(j == (segs.length - 1)){
									segs.splice(j, 1);
									segs[j - 1] = "";
								}else{
									segs.splice(j - 1, 2);
									j -= 2;
								}
							}
						}
						relobj.path = segs.join("/");
					}
				}
			}

			uri = [];
			if(relobj.scheme){ 
				uri.push(relobj.scheme, ":");
			}
			if(relobj.authority){
				uri.push("//", relobj.authority);
			}
			uri.push(relobj.path);
			if(relobj.query){
				uri.push("?", relobj.query);
			}
			if(relobj.fragment){
				uri.push("#", relobj.fragment);
			}
		}

		this.uri = uri.join("");

		// break the uri into its main components
		var r = this.uri.match(ore);

		this.scheme = r[2] || (r[1] ? "" : n);
		this.authority = r[4] || (r[3] ? "" : n);
		this.path = r[5]; // can never be undefined
		this.query = r[7] || (r[6] ? "" : n);
		this.fragment  = r[9] || (r[8] ? "" : n);

		if(this.authority != n){
			// server based naming authority
			r = this.authority.match(ire);

			this.user = r[3] || n;
			this.password = r[4] || n;
			this.host = r[6] || r[7]; // ipv6 || ipv4
			this.port = r[9] || n;
		}
	}

	dojo._Url.prototype.toString = function(){ return this.uri; };

	dojo.moduleUrl = function(/*String*/module, /*dojo._Url||String*/url){
		//	summary: 
		//		Returns a `dojo._Url` object relative to a module.
		//	example:
		//	|	var pngPath = dojo.moduleUrl("acme","images/small.png");
		//	|	console.dir(pngPath); // list the object properties
		//	|	// create an image and set it's source to pngPath's value:
		//	|	var img = document.createElement("img");
		// 	|	// NOTE: we assign the string representation of the url object
		//	|	img.src = pngPath.toString(); 
		//	|	// add our image to the document
		//	|	dojo.body().appendChild(img);
		//	example: 
		//		you may de-reference as far as you like down the package
		//		hierarchy.  This is sometimes handy to avoid lenghty relative
		//		urls or for building portable sub-packages. In this example,
		//		the `acme.widget` and `acme.util` directories may be located
		//		under different roots (see `dojo.registerModulePath`) but the
		//		the modules which reference them can be unaware of their
		//		relative locations on the filesystem:
		//	|	// somewhere in a configuration block
		//	|	dojo.registerModulePath("acme.widget", "../../acme/widget");
		//	|	dojo.registerModulePath("acme.util", "../../util");
		//	|	
		//	|	// ...
		//	|	
		//	|	// code in a module using acme resources
		//	|	var tmpltPath = dojo.moduleUrl("acme.widget","templates/template.html");
		//	|	var dataPath = dojo.moduleUrl("acme.util","resources/data.json");

		var loc = d._getModuleSymbols(module).join('/');
		if(!loc){ return null; }
		if(loc.lastIndexOf("/") != loc.length-1){
			loc += "/";
		}
		
		//If the path is an absolute path (starts with a / or is on another
		//domain/xdomain) then don't add the baseUrl.
		var colonIndex = loc.indexOf(":");
		if(loc.charAt(0) != "/" && (colonIndex == -1 || colonIndex > loc.indexOf("/"))){
			loc = d.baseUrl + loc;
		}

		return new d._Url(loc, url); // dojo._Url
	}
})();

/*=====
dojo.isBrowser = {
	//	example:
	//	|	if(dojo.isBrowser){ ... }
};

dojo.isFF = {
	//	example:
	//	|	if(dojo.isFF > 1){ ... }
};

dojo.isIE = {
	// example:
	//	|	if(dojo.isIE > 6){
	//	|		// we are IE7
	// 	|	}
};

dojo.isSafari = {
	//	example:
	//	|	if(dojo.isSafari){ ... }
	//	example: 
	//		Detect iPhone:
	//	|	if(dojo.isSafari && navigator.userAgent.indexOf("iPhone") != -1){ 
	//	|		// we are iPhone. Note, iPod touch reports "iPod" above and fails this test.
	//	|	}
};

dojo = {
	// isBrowser: Boolean
	//		True if the client is a web-browser
	isBrowser: true,
	//	isFF: Number | undefined
	//		Version as a Number if client is FireFox. undefined otherwise. Corresponds to
	//		major detected FireFox version (1.5, 2, 3, etc.)
	isFF: 2,
	//	isIE: Number | undefined
	//		Version as a Number if client is MSIE(PC). undefined otherwise. Corresponds to
	//		major detected IE version (6, 7, 8, etc.)
	isIE: 6,
	//	isKhtml: Number | undefined
	//		Version as a Number if client is a KHTML browser. undefined otherwise. Corresponds to major
	//		detected version.
	isKhtml: 0,
	//	isWebKit: Number | undefined
	//		Version as a Number if client is a WebKit-derived browser (Konqueror,
	//		Safari, Chrome, etc.). undefined otherwise.
	isWebKit: 0,
	//	isMozilla: Number | undefined
	//		Version as a Number if client is a Mozilla-based browser (Firefox,
	//		SeaMonkey). undefined otherwise. Corresponds to major detected version.
	isMozilla: 0,
	//	isOpera: Number | undefined
	//		Version as a Number if client is Opera. undefined otherwise. Corresponds to
	//		major detected version.
	isOpera: 0,
	//	isSafari: Number | undefined
	//		Version as a Number if client is Safari or iPhone. undefined otherwise.
	isSafari: 0,
	//	isChrome: Number | undefined
	//		Version as a Number if client is Chrome browser. undefined otherwise.
	isChrome: 0
	//	isMac: Boolean
	//		True if the client runs on Mac
}
=====*/

if(typeof window != 'undefined'){
	dojo.isBrowser = true;
	dojo._name = "browser";


	// attempt to figure out the path to dojo if it isn't set in the config
	(function(){
		var d = dojo;

		// this is a scope protection closure. We set browser versions and grab
		// the URL we were loaded from here.

		// grab the node we were loaded from
		if(document && document.getElementsByTagName){
			var scripts = document.getElementsByTagName("script");
			var rePkg = /dojo(\.xd)?\.js(\W|$)/i;
			for(var i = 0; i < scripts.length; i++){
				var src = scripts[i].getAttribute("src");
				if(!src){ continue; }
				var m = src.match(rePkg);
				if(m){
					// find out where we came from
					if(!d.config.baseUrl){
						d.config.baseUrl = src.substring(0, m.index);
					}
					// and find out if we need to modify our behavior
					var cfg = scripts[i].getAttribute("djConfig");
					if(cfg){
						var cfgo = eval("({ "+cfg+" })");
						for(var x in cfgo){
							dojo.config[x] = cfgo[x];
						}
					}
					break; // "first Dojo wins"
				}
			}
		}
		d.baseUrl = d.config.baseUrl;

		// fill in the rendering support information in dojo.render.*
		var n = navigator;
		var dua = n.userAgent,
			dav = n.appVersion,
			tv = parseFloat(dav);

		if(dua.indexOf("Opera") >= 0){ d.isOpera = tv; }
		if(dua.indexOf("AdobeAIR") >= 0){ d.isAIR = 1; }
		d.isKhtml = (dav.indexOf("Konqueror") >= 0) ? tv : 0;
		d.isWebKit = parseFloat(dua.split("WebKit/")[1]) || undefined;
		d.isChrome = parseFloat(dua.split("Chrome/")[1]) || undefined;
		d.isMac = dav.indexOf("Macintosh") >= 0;

		// safari detection derived from:
		//		http://developer.apple.com/internet/safari/faq.html#anchor2
		//		http://developer.apple.com/internet/safari/uamatrix.html
		var index = Math.max(dav.indexOf("WebKit"), dav.indexOf("Safari"), 0);
		if(index && !dojo.isChrome){
			// try to grab the explicit Safari version first. If we don't get
			// one, look for less than 419.3 as the indication that we're on something
			// "Safari 2-ish".
			d.isSafari = parseFloat(dav.split("Version/")[1]);
			if(!d.isSafari || parseFloat(dav.substr(index + 7)) <= 419.3){
				d.isSafari = 2;
			}
		}

				if(dua.indexOf("Gecko") >= 0 && !d.isKhtml && !d.isWebKit){ d.isMozilla = d.isMoz = tv; }
		if(d.isMoz){
			//We really need to get away from this. Consider a sane isGecko approach for the future.
			d.isFF = parseFloat(dua.split("Firefox/")[1] || dua.split("Minefield/")[1]) || undefined;
		}
		if(document.all && !d.isOpera){
			d.isIE = parseFloat(dav.split("MSIE ")[1]) || undefined;
			//In cases where the page has an HTTP header or META tag with
			//X-UA-Compatible, then it is in emulation mode.
			//Make sure isIE reflects the desired version.
			//document.documentMode of 5 means quirks mode.
			//Only switch the value if documentMode's major version
			//is different from isIE's major version.
			var mode = document.documentMode;
			if(mode && mode != 5 && Math.floor(d.isIE) != mode){
				d.isIE = mode;
			}
		}

		//Workaround to get local file loads of dojo to work on IE 7
		//by forcing to not use native xhr.
		if(dojo.isIE && window.location.protocol === "file:"){
			dojo.config.ieForceActiveXXhr=true;
		}
		
		d.isQuirks = document.compatMode == "BackCompat";

		// TODO: is the HTML LANG attribute relevant?
		d.locale = dojo.config.locale || (d.isIE ? n.userLanguage : n.language).toLowerCase();

		// These are in order of decreasing likelihood; this will change in time.
				d._XMLHTTP_PROGIDS = ['Msxml2.XMLHTTP', 'Microsoft.XMLHTTP', 'Msxml2.XMLHTTP.4.0'];
		
		d._xhrObj = function(){
			// summary: 
			//		does the work of portably generating a new XMLHTTPRequest object.
			var http, last_e;
						if(!dojo.isIE || !dojo.config.ieForceActiveXXhr){
							try{ http = new XMLHttpRequest(); }catch(e){}
						}
			if(!http){
				for(var i=0; i<3; ++i){
					var progid = d._XMLHTTP_PROGIDS[i];
					try{
						http = new ActiveXObject(progid);
					}catch(e){
						last_e = e;
					}

					if(http){
						d._XMLHTTP_PROGIDS = [progid];  // so faster next time
						break;
					}
				}
			}
			
			if(!http){
				throw new Error("XMLHTTP not available: "+last_e);
			}

			return http; // XMLHTTPRequest instance
		}

		d._isDocumentOk = function(http){
			var stat = http.status || 0,
				lp = location.protocol;
			return (stat >= 200 && stat < 300) || 	// Boolean
				stat == 304 || 						// allow any 2XX response code
				stat == 1223 || 						// get it out of the cache
				// Internet Explorer mangled the status code OR we're Titanium/browser chrome/chrome extension requesting a local file
				(!stat && (lp == "file:" || lp == "chrome:" || lp == "chrome-extension:" || lp == "app:") );
		}

		//See if base tag is in use.
		//This is to fix http://trac.dojotoolkit.org/ticket/3973,
		//but really, we need to find out how to get rid of the dojo._Url reference
		//below and still have DOH work with the dojo.i18n test following some other
		//test that uses the test frame to load a document (trac #2757).
		//Opera still has problems, but perhaps a larger issue of base tag support
		//with XHR requests (hasBase is true, but the request is still made to document
		//path, not base path).
		var owloc = window.location+"";
		var base = document.getElementsByTagName("base");
		var hasBase = (base && base.length > 0);

		d._getText = function(/*URI*/ uri, /*Boolean*/ fail_ok){
			// summary: Read the contents of the specified uri and return those contents.
			// uri:
			//		A relative or absolute uri. If absolute, it still must be in
			//		the same "domain" as we are.
			// fail_ok:
			//		Default false. If fail_ok and loading fails, return null
			//		instead of throwing.
			// returns: The response text. null is returned when there is a
			//		failure and failure is okay (an exception otherwise)

			// NOTE: must be declared before scope switches ie. this._xhrObj()
			var http = d._xhrObj();

			if(!hasBase && dojo._Url){
				uri = (new dojo._Url(owloc, uri)).toString();
			}

			if(d.config.cacheBust){
				//Make sure we have a string before string methods are used on uri
				uri += "";
				uri += (uri.indexOf("?") == -1 ? "?" : "&") + String(d.config.cacheBust).replace(/\W+/g,"");
			}

			http.open('GET', uri, false);
			try{
				http.send(null);
				if(!d._isDocumentOk(http)){
					var err = Error("Unable to load "+uri+" status:"+ http.status);
					err.status = http.status;
					err.responseText = http.responseText;
					throw err;
				}
			}catch(e){
				if(fail_ok){ return null; } // null
				// rethrow the exception
				throw e;
			}
			return http.responseText; // String
		}
		

		var _w = window;
		var _handleNodeEvent = function(/*String*/evtName, /*Function*/fp){
			// summary:
			//		non-destructively adds the specified function to the node's
			//		evtName handler.
			// evtName: should be in the form "onclick" for "onclick" handlers.
			// Make sure you pass in the "on" part.
			var _a = _w.attachEvent || _w.addEventListener;
			evtName = _w.attachEvent ? evtName : evtName.substring(2);
			_a(evtName, function(){
				fp.apply(_w, arguments);
			}, false);
		};


		d._windowUnloaders = [];
		
		d.windowUnloaded = function(){
			// summary:
			//		signal fired by impending window destruction. You may use
			//		dojo.addOnWindowUnload() to register a listener for this
			//		event. NOTE: if you wish to dojo.connect() to this method
			//		to perform page/application cleanup, be aware that this
			//		event WILL NOT fire if no handler has been registered with
			//		dojo.addOnWindowUnload. This behavior started in Dojo 1.3.
			//		Previous versions always triggered dojo.windowUnloaded. See
			//		dojo.addOnWindowUnload for more info.
			var mll = d._windowUnloaders;
			while(mll.length){
				(mll.pop())();
			}
			d = null;
		};

		var _onWindowUnloadAttached = 0;
		d.addOnWindowUnload = function(/*Object?|Function?*/obj, /*String|Function?*/functionName){
			// summary:
			//		registers a function to be triggered when window.onunload
			//		fires. 
			//	description:
			//		The first time that addOnWindowUnload is called Dojo
			//		will register a page listener to trigger your unload
			//		handler with. Note that registering these handlers may
			//		destory "fastback" page caching in browsers that support
			//		it. Be careful trying to modify the DOM or access
			//		JavaScript properties during this phase of page unloading:
			//		they may not always be available. Consider
			//		dojo.addOnUnload() if you need to modify the DOM or do
			//		heavy JavaScript work since it fires at the eqivalent of
			//		the page's "onbeforeunload" event.
			// example:
			//	|	dojo.addOnWindowUnload(functionPointer)
			//	|	dojo.addOnWindowUnload(object, "functionName");
			//	|	dojo.addOnWindowUnload(object, function(){ /* ... */});

			d._onto(d._windowUnloaders, obj, functionName);
			if(!_onWindowUnloadAttached){
				_onWindowUnloadAttached = 1;
				_handleNodeEvent("onunload", d.windowUnloaded);
			}
		};

		var _onUnloadAttached = 0;
		d.addOnUnload = function(/*Object?|Function?*/obj, /*String|Function?*/functionName){
			// summary:
			//		registers a function to be triggered when the page unloads.
			//	description:
			//		The first time that addOnUnload is called Dojo will
			//		register a page listener to trigger your unload handler
			//		with. 
			//
			//		In a browser enviroment, the functions will be triggered
			//		during the window.onbeforeunload event. Be careful of doing
			//		too much work in an unload handler. onbeforeunload can be
			//		triggered if a link to download a file is clicked, or if
			//		the link is a javascript: link. In these cases, the
			//		onbeforeunload event fires, but the document is not
			//		actually destroyed. So be careful about doing destructive
			//		operations in a dojo.addOnUnload callback.
			//
			//		Further note that calling dojo.addOnUnload will prevent
			//		browsers from using a "fast back" cache to make page
			//		loading via back button instantaneous. 
			// example:
			//	|	dojo.addOnUnload(functionPointer)
			//	|	dojo.addOnUnload(object, "functionName")
			//	|	dojo.addOnUnload(object, function(){ /* ... */});

			d._onto(d._unloaders, obj, functionName);
			if(!_onUnloadAttached){
				_onUnloadAttached = 1;
				_handleNodeEvent("onbeforeunload", dojo.unloaded);
			}
		};

	})();

	//START DOMContentLoaded
	dojo._initFired = false;
	dojo._loadInit = function(e){
		if(dojo._scrollIntervalId){
			clearInterval(dojo._scrollIntervalId);
			dojo._scrollIntervalId = 0;
		}

		if(!dojo._initFired){
			dojo._initFired = true;

			//Help out IE to avoid memory leak.
			if(!dojo.config.afterOnLoad && window.detachEvent){
				window.detachEvent("onload", dojo._loadInit);
			}

			if(dojo._inFlightCount == 0){
				dojo._modulesLoaded();
			}
		}
	}

	if(!dojo.config.afterOnLoad){		
		if(document.addEventListener){
			//Standards. Hooray! Assumption here that if standards based,
			//it knows about DOMContentLoaded. It is OK if it does not, the fall through
			//to window onload should be good enough.
			document.addEventListener("DOMContentLoaded", dojo._loadInit, false);
			window.addEventListener("load", dojo._loadInit, false);
		}else if(window.attachEvent){
			window.attachEvent("onload", dojo._loadInit);

			//DOMContentLoaded approximation. Diego Perini found this MSDN article
			//that indicates doScroll is available after DOM ready, so do a setTimeout
			//to check when it is available.
			//http://msdn.microsoft.com/en-us/library/ms531426.aspx
			if(!dojo.config.skipIeDomLoaded && self === self.top){
				dojo._scrollIntervalId = setInterval(function (){
					try{
						//When dojo is loaded into an iframe in an IE HTML Application 
						//(HTA), such as in a selenium test, javascript in the iframe
						//can't see anything outside of it, so self===self.top is true,
						//but the iframe is not the top window and doScroll will be 
						//available before document.body is set. Test document.body
						//before trying the doScroll trick
						if(document.body){
							document.documentElement.doScroll("left");
							dojo._loadInit();
						}
					}catch (e){}
				}, 30);
			}
		}
	}

		if(dojo.isIE){
		try{
			(function(){
				document.namespaces.add("v", "urn:schemas-microsoft-com:vml");
				var vmlElems = ["*", "group", "roundrect", "oval", "shape", "rect", "imagedata", "path", "textpath", "text"],
					i = 0, l = 1, s = document.createStyleSheet();
				if(dojo.isIE >= 8){
					i = 1;
					l = vmlElems.length;
				}
				for(; i < l; ++i){
					s.addRule("v\\:" + vmlElems[i], "behavior:url(#default#VML); display:inline-block");
				}
			})();
		}catch(e){}
	}
		//END DOMContentLoaded


	/*
	OpenAjax.subscribe("OpenAjax", "onload", function(){
		if(dojo._inFlightCount == 0){
			dojo._modulesLoaded();
		}
	});

	OpenAjax.subscribe("OpenAjax", "onunload", function(){
		dojo.unloaded();
	});
	*/
} //if (typeof window != 'undefined')

//Register any module paths set up in djConfig. Need to do this
//in the hostenvs since hostenv_browser can read djConfig from a
//script tag's attribute.
(function(){
	var mp = dojo.config["modulePaths"];
	if(mp){
		for(var param in mp){
			dojo.registerModulePath(param, mp[param]);
		}
	}
})();

//Load debug code if necessary.
if(dojo.config.isDebug){
	dojo.require("dojo._firebug.firebug");
}

if(dojo.config.debugAtAllCosts){
	dojo.config.useXDomain = true;
	dojo.require("dojo._base._loader.loader_xd");
	dojo.require("dojo._base._loader.loader_debug");
	
}

if(!dojo._hasResource["dojo._base.lang"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.lang"] = true;
dojo.provide("dojo._base.lang");

(function(){
	var d = dojo, opts = Object.prototype.toString;

	// Crockford (ish) functions

	dojo.isString = function(/*anything*/ it){
		//	summary:
		//		Return true if it is a String
		return (typeof it == "string" || it instanceof String); // Boolean
	}

	dojo.isArray = function(/*anything*/ it){
		//	summary:
		//		Return true if it is an Array.
		//		Does not work on Arrays created in other windows.
		return it && (it instanceof Array || typeof it == "array"); // Boolean
	}

	dojo.isFunction = function(/*anything*/ it){
		// summary:
		//		Return true if it is a Function
		return opts.call(it) === "[object Function]";
	};

	dojo.isObject = function(/*anything*/ it){
		// summary:
		//		Returns true if it is a JavaScript object (or an Array, a Function
		//		or null)
		return it !== undefined &&
			(it === null || typeof it == "object" || d.isArray(it) || d.isFunction(it)); // Boolean
	}

	dojo.isArrayLike = function(/*anything*/ it){
		//	summary:
		//		similar to dojo.isArray() but more permissive
		//	description:
		//		Doesn't strongly test for "arrayness".  Instead, settles for "isn't
		//		a string or number and has a length property". Arguments objects
		//		and DOM collections will return true when passed to
		//		dojo.isArrayLike(), but will return false when passed to
		//		dojo.isArray().
		//	returns:
		//		If it walks like a duck and quacks like a duck, return `true`
		return it && it !== undefined && // Boolean
			// keep out built-in constructors (Number, String, ...) which have length
			// properties
			!d.isString(it) && !d.isFunction(it) &&
			!(it.tagName && it.tagName.toLowerCase() == 'form') &&
			(d.isArray(it) || isFinite(it.length));
	}

	dojo.isAlien = function(/*anything*/ it){
		// summary:
		//		Returns true if it is a built-in function or some other kind of
		//		oddball that *should* report as a function but doesn't
		return it && !d.isFunction(it) && /\{\s*\[native code\]\s*\}/.test(String(it)); // Boolean
	}

	dojo.extend = function(/*Object*/ constructor, /*Object...*/ props){
		// summary:
		//		Adds all properties and methods of props to constructor's
		//		prototype, making them available to all instances created with
		//		constructor.
		for(var i=1, l=arguments.length; i<l; i++){
			d._mixin(constructor.prototype, arguments[i]);
		}
		return constructor; // Object
	}

	dojo._hitchArgs = function(scope, method /*,...*/){
		var pre = d._toArray(arguments, 2);
		var named = d.isString(method);
		return function(){
			// arrayify arguments
			var args = d._toArray(arguments);
			// locate our method
			var f = named ? (scope||d.global)[method] : method;
			// invoke with collected args
			return f && f.apply(scope || this, pre.concat(args)); // mixed
		} // Function
	}

	dojo.hitch = function(/*Object*/scope, /*Function|String*/method /*,...*/){
		//	summary:
		//		Returns a function that will only ever execute in the a given scope.
		//		This allows for easy use of object member functions
		//		in callbacks and other places in which the "this" keyword may
		//		otherwise not reference the expected scope.
		//		Any number of default positional arguments may be passed as parameters 
		//		beyond "method".
		//		Each of these values will be used to "placehold" (similar to curry)
		//		for the hitched function.
		//	scope:
		//		The scope to use when method executes. If method is a string,
		//		scope is also the object containing method.
		//	method:
		//		A function to be hitched to scope, or the name of the method in
		//		scope to be hitched.
		//	example:
		//	|	dojo.hitch(foo, "bar")();
		//		runs foo.bar() in the scope of foo
		//	example:
		//	|	dojo.hitch(foo, myFunction);
		//		returns a function that runs myFunction in the scope of foo
		//	example:
		//		Expansion on the default positional arguments passed along from
		//		hitch. Passed args are mixed first, additional args after.
		//	|	var foo = { bar: function(a, b, c){ console.log(a, b, c); } };
		//	|	var fn = dojo.hitch(foo, "bar", 1, 2);
		//	|	fn(3); // logs "1, 2, 3"
		//	example:
		//	|	var foo = { bar: 2 };
		//	|	dojo.hitch(foo, function(){ this.bar = 10; })();
		//		execute an anonymous function in scope of foo
		
		if(arguments.length > 2){
			return d._hitchArgs.apply(d, arguments); // Function
		}
		if(!method){
			method = scope;
			scope = null;
		}
		if(d.isString(method)){
			scope = scope || d.global;
			if(!scope[method]){ throw(['dojo.hitch: scope["', method, '"] is null (scope="', scope, '")'].join('')); }
			return function(){ return scope[method].apply(scope, arguments || []); }; // Function
		}
		return !scope ? method : function(){ return method.apply(scope, arguments || []); }; // Function
	}

	/*=====
	dojo.delegate = function(obj, props){
		//	summary:
		//		Returns a new object which "looks" to obj for properties which it
		//		does not have a value for. Optionally takes a bag of properties to
		//		seed the returned object with initially.
		//	description:
		//		This is a small implementaton of the Boodman/Crockford delegation
		//		pattern in JavaScript. An intermediate object constructor mediates
		//		the prototype chain for the returned object, using it to delegate
		//		down to obj for property lookup when object-local lookup fails.
		//		This can be thought of similarly to ES4's "wrap", save that it does
		//		not act on types but rather on pure objects.
		//	obj:
		//		The object to delegate to for properties not found directly on the
		//		return object or in props.
		//	props:
		//		an object containing properties to assign to the returned object
		//	returns:
		//		an Object of anonymous type
		//	example:
		//	|	var foo = { bar: "baz" };
		//	|	var thinger = dojo.delegate(foo, { thud: "xyzzy"});
		//	|	thinger.bar == "baz"; // delegated to foo
		//	|	foo.thud == undefined; // by definition
		//	|	thinger.thud == "xyzzy"; // mixed in from props
		//	|	foo.bar = "thonk";
		//	|	thinger.bar == "thonk"; // still delegated to foo's bar
	}
	=====*/

	dojo.delegate = dojo._delegate = (function(){
		// boodman/crockford delegation w/ cornford optimization
		function TMP(){}
		return function(obj, props){
			TMP.prototype = obj;
			var tmp = new TMP();
			TMP.prototype = null;
			if(props){
				d._mixin(tmp, props);
			}
			return tmp; // Object
		}
	})();

	/*=====
	dojo._toArray = function(obj, offset, startWith){
		//	summary:
		//		Converts an array-like object (i.e. arguments, DOMCollection) to an
		//		array. Returns a new Array with the elements of obj.
		//	obj: Object
		//		the object to "arrayify". We expect the object to have, at a
		//		minimum, a length property which corresponds to integer-indexed
		//		properties.
		//	offset: Number?
		//		the location in obj to start iterating from. Defaults to 0.
		//		Optional.
		//	startWith: Array?
		//		An array to pack with the properties of obj. If provided,
		//		properties in obj are appended at the end of startWith and
		//		startWith is the returned array.
	}
	=====*/

	var efficient = function(obj, offset, startWith){
		return (startWith||[]).concat(Array.prototype.slice.call(obj, offset||0));
	};

		var slow = function(obj, offset, startWith){
		var arr = startWith||[];
		for(var x = offset || 0; x < obj.length; x++){
			arr.push(obj[x]);
		}
		return arr;
	};
	
	dojo._toArray =
				d.isIE ?  function(obj){
			return ((obj.item) ? slow : efficient).apply(this, arguments);
		} :
				efficient;

	dojo.partial = function(/*Function|String*/method /*, ...*/){
		//	summary:
		//		similar to hitch() except that the scope object is left to be
		//		whatever the execution context eventually becomes.
		//	description:
		//		Calling dojo.partial is the functional equivalent of calling:
		//		|	dojo.hitch(null, funcName, ...);
		var arr = [ null ];
		return d.hitch.apply(d, arr.concat(d._toArray(arguments))); // Function
	}

	var extraNames = d._extraNames, extraLen = extraNames.length, empty = {};

	dojo.clone = function(/*anything*/ o){
		// summary:
		//		Clones objects (including DOM nodes) and all children.
		//		Warning: do not clone cyclic structures.
		if(!o || typeof o != "object" || d.isFunction(o)){
			// null, undefined, any non-object, or function
			return o;	// anything
		}
		if(o.nodeType && "cloneNode" in o){
			// DOM Node
			return o.cloneNode(true); // Node
		}
		if(o instanceof Date){
			// Date
			return new Date(o.getTime());	// Date
		}
		var r, i, l, s, name;
		if(d.isArray(o)){
			// array
			r = [];
			for(i = 0, l = o.length; i < l; ++i){
				if(i in o){
					r.push(d.clone(o[i]));
				}
			}
// we don't clone functions for performance reasons
//		}else if(d.isFunction(o)){
//			// function
//			r = function(){ return o.apply(this, arguments); };
		}else{
			// generic objects
			r = o.constructor ? new o.constructor() : {};
		}
		for(name in o){
			// the "tobj" condition avoid copying properties in "source"
			// inherited from Object.prototype.  For example, if target has a custom
			// toString() method, don't overwrite it with the toString() method
			// that source inherited from Object.prototype
			s = o[name];
			if(!(name in r) || (r[name] !== s && (!(name in empty) || empty[name] !== s))){
				r[name] = d.clone(s);
			}
		}
				// IE doesn't recognize some custom functions in for..in
		if(extraLen){
			for(i = 0; i < extraLen; ++i){
				name = extraNames[i];
				s = o[name];
				if(!(name in r) || (r[name] !== s && (!(name in empty) || empty[name] !== s))){
					r[name] = s; // functions only, we don't clone them
				}
			}
		}
				return r; // Object
	}

	/*=====
	dojo.trim = function(str){
		//	summary:
		//		Trims whitespace from both sides of the string
		//	str: String
		//		String to be trimmed
		//	returns: String
		//		Returns the trimmed string
		//	description:
		//		This version of trim() was selected for inclusion into the base due
		//		to its compact size and relatively good performance
		//		(see [Steven Levithan's blog](http://blog.stevenlevithan.com/archives/faster-trim-javascript)
		//		Uses String.prototype.trim instead, if available.
		//		The fastest but longest version of this function is located at
		//		dojo.string.trim()
		return "";	// String
	}
	=====*/

	dojo.trim = String.prototype.trim ?
		function(str){ return str.trim(); } :
		function(str){ return str.replace(/^\s\s*/, '').replace(/\s\s*$/, ''); };

	/*=====
	dojo.replace = function(tmpl, map, pattern){
		//	summary:
		//		Performs parameterized substitutions on a string. Throws an
		//		exception if any parameter is unmatched. 
		//	tmpl: String
		//		String to be used as a template.
		//	map: Object|Function
		//		If an object, it is used as a dictionary to look up substitutions.
		//		If a function, it is called for every substitution with following
		//		parameters: a whole match, a name, an offset, and the whole template
		//		string (see https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Global_Objects/String/replace
		//		for more details).
		//	pattern: RegEx?
		//		Optional regular expression objects that overrides the default pattern.
		//		Must be global and match one item. The default is: /\{([^\}]+)\}/g,
		//		which matches patterns like that: "{xxx}", where "xxx" is any sequence
		//		of characters, which doesn't include "}".
		//	returns: String
		//		Returns the substituted string.
		//	example:
		//	|	// uses a dictionary for substitutions:
		//	|	dojo.replace("Hello, {name.first} {name.last} AKA {nick}!",
		//	|	  {
		//	|	    nick: "Bob",
		//	|	    name: {
		//	|	      first:  "Robert",
		//	|	      middle: "X",
		//	|	      last:   "Cringely"
		//	|	    }
		//	|	  });
		//	|	// returns: Hello, Robert Cringely AKA Bob!
		//	example:
		//	|	// uses an array for substitutions:
		//	|	dojo.replace("Hello, {0} {2}!",
		//	|	  ["Robert", "X", "Cringely"]);
		//	|	// returns: Hello, Robert Cringely!
		//	example:
		//	|	// uses a function for substitutions:
		//	|	function sum(a){
		//	|	  var t = 0;
		//	|	  dojo.forEach(a, function(x){ t += x; });
		//	|	  return t;
		//	|	}
		//	|	dojo.replace(
		//	|	  "{count} payments averaging {avg} USD per payment.",
		//	|	  dojo.hitch(
		//	|	    { payments: [11, 16, 12] },
		//	|	    function(_, key){
		//	|	      switch(key){
		//	|	        case "count": return this.payments.length;
		//	|	        case "min":   return Math.min.apply(Math, this.payments);
		//	|	        case "max":   return Math.max.apply(Math, this.payments);
		//	|	        case "sum":   return sum(this.payments);
		//	|	        case "avg":   return sum(this.payments) / this.payments.length;
		//	|	      }
		//	|	    }
		//	|	  )
		//	|	);
		//	|	// prints: 3 payments averaging 13 USD per payment.
		//	example:
		//	|	// uses an alternative PHP-like pattern for substitutions:
		//	|	dojo.replace("Hello, ${0} ${2}!",
		//	|	  ["Robert", "X", "Cringely"], /\$\{([^\}]+)\}/g);
		//	|	// returns: Hello, Robert Cringely!
		return "";	// String
	}
	=====*/

	var _pattern = /\{([^\}]+)\}/g;
	dojo.replace = function(tmpl, map, pattern){
		return tmpl.replace(pattern || _pattern, d.isFunction(map) ?
			map : function(_, k){ return d.getObject(k, false, map); });
	};
})();

}

if(!dojo._hasResource["dojo._base.array"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.array"] = true;

dojo.provide("dojo._base.array");

(function(){
	var _getParts = function(arr, obj, cb){
		return [ 
			(typeof arr == "string") ? arr.split("") : arr, 
			obj || dojo.global,
			// FIXME: cache the anonymous functions we create here?
			(typeof cb == "string") ? new Function("item", "index", "array", cb) : cb
		];
	};

	var everyOrSome = function(/*Boolean*/every, /*Array|String*/arr, /*Function|String*/callback, /*Object?*/thisObject){
		var _p = _getParts(arr, thisObject, callback); arr = _p[0];
		for(var i=0,l=arr.length; i<l; ++i){
			var result = !!_p[2].call(_p[1], arr[i], i, arr);
			if(every ^ result){
				return result; // Boolean
			}
		}
		return every; // Boolean
	};

	dojo.mixin(dojo, {
		indexOf: function(	/*Array*/		array, 
							/*Object*/		value,
							/*Integer?*/	fromIndex,
							/*Boolean?*/	findLast){
			// summary:
			//		locates the first index of the provided value in the
			//		passed array. If the value is not found, -1 is returned.
			// description:
			//		This method corresponds to the JavaScript 1.6 Array.indexOf method, with one difference: when
			//		run over sparse arrays, the Dojo function invokes the callback for every index whereas JavaScript 
			//		1.6's indexOf skips the holes in the sparse array.
			//		For details on this method, see:
			//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/indexOf

			var step = 1, end = array.length || 0, i = 0;
			if(findLast){
				i = end - 1;
				step = end = -1;
			}
			if(fromIndex != undefined){ i = fromIndex; }
			if((findLast && i > end) || i < end){
				for(; i != end; i += step){
					if(array[i] == value){ return i; }
				}
			}
			return -1;	// Number
		},

		lastIndexOf: function(/*Array*/array, /*Object*/value, /*Integer?*/fromIndex){
			// summary:
			//		locates the last index of the provided value in the passed
			//		array. If the value is not found, -1 is returned.
			// description:
			//		This method corresponds to the JavaScript 1.6 Array.lastIndexOf method, with one difference: when
			//		run over sparse arrays, the Dojo function invokes the callback for every index whereas JavaScript 
			//		1.6's lastIndexOf skips the holes in the sparse array.
			//		For details on this method, see:
			// 			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/lastIndexOf
			return dojo.indexOf(array, value, fromIndex, true); // Number
		},

		forEach: function(/*Array|String*/arr, /*Function|String*/callback, /*Object?*/thisObject){
			//	summary:
			//		for every item in arr, callback is invoked. Return values are ignored.
			//		If you want to break out of the loop, consider using dojo.every() or dojo.some().
			//		forEach does not allow breaking out of the loop over the items in arr.
			//	arr:
			//		the array to iterate over. If a string, operates on individual characters.
			//	callback:
			//		a function is invoked with three arguments: item, index, and array
			//	thisObject:
			//		may be used to scope the call to callback
			//	description:
			//		This function corresponds to the JavaScript 1.6 Array.forEach() method, with one difference: when 
			//		run over sparse arrays, this implemenation passes the "holes" in the sparse array to
			//		the callback function with a value of undefined. JavaScript 1.6's forEach skips the holes in the sparse array.
			//		For more details, see:
			//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/forEach
			//	example:
			//	|	// log out all members of the array:
			//	|	dojo.forEach(
			//	|		[ "thinger", "blah", "howdy", 10 ],
			//	|		function(item){
			//	|			console.log(item);
			//	|		}
			//	|	);
			//	example:
			//	|	// log out the members and their indexes
			//	|	dojo.forEach(
			//	|		[ "thinger", "blah", "howdy", 10 ],
			//	|		function(item, idx, arr){
			//	|			console.log(item, "at index:", idx);
			//	|		}
			//	|	);
			//	example:
			//	|	// use a scoped object member as the callback
			//	|	
			//	|	var obj = {
			//	|		prefix: "logged via obj.callback:", 
			//	|		callback: function(item){
			//	|			console.log(this.prefix, item);
			//	|		}
			//	|	};
			//	|	
			//	|	// specifying the scope function executes the callback in that scope
			//	|	dojo.forEach(
			//	|		[ "thinger", "blah", "howdy", 10 ],
			//	|		obj.callback,
			//	|		obj
			//	|	);
			//	|	
			//	|	// alternately, we can accomplish the same thing with dojo.hitch()
			//	|	dojo.forEach(
			//	|		[ "thinger", "blah", "howdy", 10 ],
			//	|		dojo.hitch(obj, "callback")
			//	|	);

			// match the behavior of the built-in forEach WRT empty arrs
			if(!arr || !arr.length){ return; }

			// FIXME: there are several ways of handilng thisObject. Is
			// dojo.global always the default context?
			var _p = _getParts(arr, thisObject, callback); arr = _p[0];
			for(var i=0,l=arr.length; i<l; ++i){ 
				_p[2].call(_p[1], arr[i], i, arr);
			}
		},

		every: function(/*Array|String*/arr, /*Function|String*/callback, /*Object?*/thisObject){
			// summary:
			//		Determines whether or not every item in arr satisfies the
			//		condition implemented by callback.
			// arr:
			//		the array to iterate on. If a string, operates on individual characters.
			// callback:
			//		a function is invoked with three arguments: item, index,
			//		and array and returns true if the condition is met.
			// thisObject:
			//		may be used to scope the call to callback
			// description:
			//		This function corresponds to the JavaScript 1.6 Array.every() method, with one difference: when 
			//		run over sparse arrays, this implemenation passes the "holes" in the sparse array to
			//		the callback function with a value of undefined. JavaScript 1.6's every skips the holes in the sparse array.
			//		For more details, see:
			//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/every
			// example:
			//	|	// returns false
			//	|	dojo.every([1, 2, 3, 4], function(item){ return item>1; });
			// example:
			//	|	// returns true 
			//	|	dojo.every([1, 2, 3, 4], function(item){ return item>0; });
			return everyOrSome(true, arr, callback, thisObject); // Boolean
		},

		some: function(/*Array|String*/arr, /*Function|String*/callback, /*Object?*/thisObject){
			// summary:
			//		Determines whether or not any item in arr satisfies the
			//		condition implemented by callback.
			// arr:
			//		the array to iterate over. If a string, operates on individual characters.
			// callback:
			//		a function is invoked with three arguments: item, index,
			//		and array and returns true if the condition is met.
			// thisObject:
			//		may be used to scope the call to callback
			// description:
			//		This function corresponds to the JavaScript 1.6 Array.some() method, with one difference: when 
			//		run over sparse arrays, this implemenation passes the "holes" in the sparse array to
			//		the callback function with a value of undefined. JavaScript 1.6's some skips the holes in the sparse array.
			//		For more details, see:
			//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/some
			// example:
			//	|	// is true
			//	|	dojo.some([1, 2, 3, 4], function(item){ return item>1; });
			// example:
			//	|	// is false
			//	|	dojo.some([1, 2, 3, 4], function(item){ return item<1; });
			return everyOrSome(false, arr, callback, thisObject); // Boolean
		},

		map: function(/*Array|String*/arr, /*Function|String*/callback, /*Function?*/thisObject){
			// summary:
			//		applies callback to each element of arr and returns
			//		an Array with the results
			// arr:
			//		the array to iterate on. If a string, operates on
			//		individual characters.
			// callback:
			//		a function is invoked with three arguments, (item, index,
			//		array),  and returns a value
			// thisObject:
			//		may be used to scope the call to callback
			// description:
			//		This function corresponds to the JavaScript 1.6 Array.map() method, with one difference: when 
			//		run over sparse arrays, this implemenation passes the "holes" in the sparse array to
			//		the callback function with a value of undefined. JavaScript 1.6's map skips the holes in the sparse array.
			//		For more details, see:
			//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/map
			// example:
			//	|	// returns [2, 3, 4, 5]
			//	|	dojo.map([1, 2, 3, 4], function(item){ return item+1 });

			var _p = _getParts(arr, thisObject, callback); arr = _p[0];
			var outArr = (arguments[3] ? (new arguments[3]()) : []);
			for(var i=0,l=arr.length; i<l; ++i){
				outArr.push(_p[2].call(_p[1], arr[i], i, arr));
			}
			return outArr; // Array
		},

		filter: function(/*Array*/arr, /*Function|String*/callback, /*Object?*/thisObject){
			// summary:
			//		Returns a new Array with those items from arr that match the
			//		condition implemented by callback.
			// arr:
			//		the array to iterate over.
			// callback:
			//		a function that is invoked with three arguments (item,
			//		index, array). The return of this function is expected to
			//		be a boolean which determines whether the passed-in item
			//		will be included in the returned array.
			// thisObject:
			//		may be used to scope the call to callback
			// description:
			//		This function corresponds to the JavaScript 1.6 Array.filter() method, with one difference: when 
			//		run over sparse arrays, this implemenation passes the "holes" in the sparse array to
			//		the callback function with a value of undefined. JavaScript 1.6's filter skips the holes in the sparse array. 
			//		For more details, see:
			//			https://developer.mozilla.org/en/Core_JavaScript_1.5_Reference/Objects/Array/filter
			// example:
			//	|	// returns [2, 3, 4]
			//	|	dojo.filter([1, 2, 3, 4], function(item){ return item>1; });

			var _p = _getParts(arr, thisObject, callback); arr = _p[0];
			var outArr = [];
			for(var i=0,l=arr.length; i<l; ++i){
				if(_p[2].call(_p[1], arr[i], i, arr)){
					outArr.push(arr[i]);
				}
			}
			return outArr; // Array
		}
	});
})();
/*
*/

}

if(!dojo._hasResource["dojo._base.declare"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.declare"] = true;
dojo.provide("dojo._base.declare");




(function(){
	var d = dojo, mix = d._mixin, op = Object.prototype, opts = op.toString,
		xtor = new Function, counter = 0, cname = "constructor";

	function err(msg){ throw new Error("declare: " + msg); }

	// C3 Method Resolution Order (see http://www.python.org/download/releases/2.3/mro/)
	function c3mro(bases){
		var result = [], roots = [{cls: 0, refs: []}], nameMap = {}, clsCount = 1,
			l = bases.length, i = 0, j, lin, base, top, proto, rec, name, refs;

		// build a list of bases naming them if needed
		for(; i < l; ++i){
			base = bases[i];
			if(!base){
				err("mixin #" + i + " is unknown. Did you use dojo.require to pull it in?");
			}else if(opts.call(base) != "[object Function]"){
				err("mixin #" + i + " is not a callable constructor.");
			}
			lin = base._meta ? base._meta.bases : [base];
			top = 0;
			// add bases to the name map
			for(j = lin.length - 1; j >= 0; --j){
				proto = lin[j].prototype;
				if(!proto.hasOwnProperty("declaredClass")){
					proto.declaredClass = "uniqName_" + (counter++);
				}
				name = proto.declaredClass;
				if(!nameMap.hasOwnProperty(name)){
					nameMap[name] = {count: 0, refs: [], cls: lin[j]};
					++clsCount;
				}
				rec = nameMap[name];
				if(top && top !== rec){
					rec.refs.push(top);
					++top.count;
				}
				top = rec;
			}
			++top.count;
			roots[0].refs.push(top);
		}

		// remove classes without external references recursively
		while(roots.length){
			top = roots.pop();
			result.push(top.cls);
			--clsCount;
			// optimization: follow a single-linked chain
			while(refs = top.refs, refs.length == 1){
				top = refs[0];
				if(!top || --top.count){
					// branch or end of chain => do not end to roots
					top = 0;
					break;
				}
				result.push(top.cls);
				--clsCount;
			}
			if(top){
				// branch
				for(i = 0, l = refs.length; i < l; ++i){
					top = refs[i];
					if(!--top.count){
						roots.push(top);
					}
				}
			}
		}
		if(clsCount){
			err("can't build consistent linearization");
		}

		// calculate the superclass offset
		base = bases[0];
		result[0] = base ?
			base._meta && base === result[result.length - base._meta.bases.length] ?
				base._meta.bases.length : 1 : 0;

		return result;
	}

	function inherited(args, a, f){
		var name, chains, bases, caller, meta, base, proto, opf, pos,
			cache = this._inherited = this._inherited || {};

		// crack arguments
		if(typeof args == "string"){
			name = args;
			args = a;
			a = f;
		}
		f = 0;

		caller = args.callee;
		name = name || caller.nom;
		if(!name){
			err("can't deduce a name to call inherited()");
		}

		meta = this.constructor._meta;
		bases = meta.bases;

		pos = cache.p;
		if(name != cname){
			// method
			if(cache.c !== caller){
				// cache bust
				pos = 0;
				base = bases[0];
				meta = base._meta;
				if(meta.hidden[name] !== caller){
					// error detection
					chains = meta.chains;
					if(chains && typeof chains[name] == "string"){
						err("calling chained method with inherited: " + name);
					}
					// find caller
					do{
						meta = base._meta;
						proto = base.prototype;
						if(meta && (proto[name] === caller && proto.hasOwnProperty(name) || meta.hidden[name] === caller)){
							break;
						}
					}while(base = bases[++pos]); // intentional assignment
					pos = base ? pos : -1;
				}
			}
			// find next
			base = bases[++pos];
			if(base){
				proto = base.prototype;
				if(base._meta && proto.hasOwnProperty(name)){
					f = proto[name];
				}else{
					opf = op[name];
					do{
						proto = base.prototype;
						f = proto[name];
						if(f && (base._meta ? proto.hasOwnProperty(name) : f !== opf)){
							break;
						}
					}while(base = bases[++pos]); // intentional assignment
				}
			}
			f = base && f || op[name];
		}else{
			// constructor
			if(cache.c !== caller){
				// cache bust
				pos = 0;
				meta = bases[0]._meta;
				if(meta && meta.ctor !== caller){
					// error detection
					chains = meta.chains;
					if(!chains || chains.constructor !== "manual"){
						err("calling chained constructor with inherited");
					}
					// find caller
					while(base = bases[++pos]){ // intentional assignment
						meta = base._meta;
						if(meta && meta.ctor === caller){
							break;
						}
					}
					pos = base ? pos : -1;
				}
			}
			// find next
			while(base = bases[++pos]){	// intentional assignment
				meta = base._meta;
				f = meta ? meta.ctor : base;
				if(f){
					break;
				}
			}
			f = base && f;
		}

		// cache the found super method
		cache.c = f;
		cache.p = pos;

		// now we have the result
		if(f){
			return a === true ? f : f.apply(this, a || args);
		}
		// intentionally if a super method was not found
	}

	function getInherited(name, args){
		if(typeof name == "string"){
			return this.inherited(name, args, true);
		}
		return this.inherited(name, true);
	}

	// emulation of "instanceof"
	function isInstanceOf(cls){
		var bases = this.constructor._meta.bases;
		for(var i = 0, l = bases.length; i < l; ++i){
			if(bases[i] === cls){
				return true;
			}
		}
		return this instanceof cls;
	}

	function mixOwn(target, source){
		var name, i = 0, l = d._extraNames.length;
		// add props adding metadata for incoming functions skipping a constructor
		for(name in source){
			if(name != cname && source.hasOwnProperty(name)){
				target[name] = source[name];
			}
		}
		// process unenumerable methods on IE
		for(; i < l; ++i){
			name = d._extraNames[i];
			if(name != cname && source.hasOwnProperty(name)){
				target[name] = source[name];
			}
		}
	}

	// implementation of safe mixin function
	function safeMixin(target, source){
		var name, t, i = 0, l = d._extraNames.length;
		// add props adding metadata for incoming functions skipping a constructor
		for(name in source){
			t = source[name];
			if((t !== op[name] || !(name in op)) && name != cname){
				if(opts.call(t) == "[object Function]"){
					// non-trivial function method => attach its name
					t.nom = name;
				}
				target[name] = t;
			}
		}
		// process unenumerable methods on IE
		for(; i < l; ++i){
			name = d._extraNames[i];
			t = source[name];
			if((t !== op[name] || !(name in op)) && name != cname){
				if(opts.call(t) == "[object Function]"){
					// non-trivial function method => attach its name
					t.nom = name;
				}
				target[name] = t;
			}
		}
		return target;
	}

	function extend(source){
		safeMixin(this.prototype, source);
		return this;
	}

	// chained constructor compatible with the legacy dojo.declare()
	function chainedConstructor(bases, ctorSpecial){
		return function(){
			var a = arguments, args = a, a0 = a[0], f, i, m,
				l = bases.length, preArgs;

			if(!(this instanceof a.callee)){
				// not called via new, so force it
				return applyNew(a);
			}

			//this._inherited = {};
			// perform the shaman's rituals of the original dojo.declare()
			// 1) call two types of the preamble
			if(ctorSpecial && (a0 && a0.preamble || this.preamble)){
				// full blown ritual
				preArgs = new Array(bases.length);
				// prepare parameters
				preArgs[0] = a;
				for(i = 0;;){
					// process the preamble of the 1st argument
					a0 = a[0];
					if(a0){
						f = a0.preamble;
						if(f){
							a = f.apply(this, a) || a;
						}
					}
					// process the preamble of this class
					f = bases[i].prototype;
					f = f.hasOwnProperty("preamble") && f.preamble;
					if(f){
						a = f.apply(this, a) || a;
					}
					// one peculiarity of the preamble:
					// it is called if it is not needed,
					// e.g., there is no constructor to call
					// let's watch for the last constructor
					// (see ticket #9795)
					if(++i == l){
						break;
					}
					preArgs[i] = a;
				}
			}
			// 2) call all non-trivial constructors using prepared arguments
			for(i = l - 1; i >= 0; --i){
				f = bases[i];
				m = f._meta;
				f = m ? m.ctor : f;
				if(f){
					f.apply(this, preArgs ? preArgs[i] : a);
				}
			}
			// 3) continue the original ritual: call the postscript
			f = this.postscript;
			if(f){
				f.apply(this, args);
			}
		};
	}


	// chained constructor compatible with the legacy dojo.declare()
	function singleConstructor(ctor, ctorSpecial){
		return function(){
			var a = arguments, t = a, a0 = a[0], f;

			if(!(this instanceof a.callee)){
				// not called via new, so force it
				return applyNew(a);
			}

			//this._inherited = {};
			// perform the shaman's rituals of the original dojo.declare()
			// 1) call two types of the preamble
			if(ctorSpecial){
				// full blown ritual
				if(a0){
					// process the preamble of the 1st argument
					f = a0.preamble;
					if(f){
						t = f.apply(this, t) || t;
					}
				}
				f = this.preamble;
				if(f){
					// process the preamble of this class
					f.apply(this, t);
					// one peculiarity of the preamble:
					// it is called even if it is not needed,
					// e.g., there is no constructor to call
					// let's watch for the last constructor
					// (see ticket #9795)
				}
			}
			// 2) call a constructor
			if(ctor){
				ctor.apply(this, a);
			}
			// 3) continue the original ritual: call the postscript
			f = this.postscript;
			if(f){
				f.apply(this, a);
			}
		};
	}

	// plain vanilla constructor (can use inherited() to call its base constructor)
	function simpleConstructor(bases){
		return function(){
			var a = arguments, i = 0, f, m;

			if(!(this instanceof a.callee)){
				// not called via new, so force it
				return applyNew(a);
			}

			//this._inherited = {};
			// perform the shaman's rituals of the original dojo.declare()
			// 1) do not call the preamble
			// 2) call the top constructor (it can use this.inherited())
			for(; f = bases[i]; ++i){ // intentional assignment
				m = f._meta;
				f = m ? m.ctor : f;
				if(f){
					f.apply(this, a);
					break;
				}
			}
			// 3) call the postscript
			f = this.postscript;
			if(f){
				f.apply(this, a);
			}
		};
	}

	function chain(name, bases, reversed){
		return function(){
			var b, m, f, i = 0, step = 1;
			if(reversed){
				i = bases.length - 1;
				step = -1;
			}
			for(; b = bases[i]; i += step){ // intentional assignment
				m = b._meta;
				f = (m ? m.hidden : b.prototype)[name];
				if(f){
					f.apply(this, arguments);
				}
			}
		};
	}

	// forceNew(ctor)
	// return a new object that inherits from ctor.prototype but
	// without actually running ctor on the object.
	function forceNew(ctor){
		// create object with correct prototype using a do-nothing
		// constructor
		xtor.prototype = ctor.prototype;
		var t = new xtor;
		xtor.prototype = null;	// clean up
		return t;
	}

	// applyNew(args)
	// just like 'new ctor()' except that the constructor and its arguments come
	// from args, which must be an array or an arguments object
	function applyNew(args){
		// create an object with ctor's prototype but without
		// calling ctor on it.
		var ctor = args.callee, t = forceNew(ctor);
		// execute the real constructor on the new object
		ctor.apply(t, args);
		return t;
	}

	d.declare = function(className, superclass, props){
		// crack parameters
		if(typeof className != "string"){
			props = superclass;
			superclass = className;
			className = "";
		}
		props = props || {};

		var proto, i, t, ctor, name, bases, chains, mixins = 1, parents = superclass;

		// build a prototype
		if(opts.call(superclass) == "[object Array]"){
			// C3 MRO
			bases = c3mro(superclass);
			t = bases[0];
			mixins = bases.length - t;
			superclass = bases[mixins];
		}else{
			bases = [0];
			if(superclass){
				if(opts.call(superclass) == "[object Function]"){
					t = superclass._meta;
					bases = bases.concat(t ? t.bases : superclass);
				}else{
					err("base class is not a callable constructor.");
				}
			}else if(superclass !== null){
				err("unknown base class. Did you use dojo.require to pull it in?")
			}
		}
		if(superclass){
			for(i = mixins - 1;; --i){
				proto = forceNew(superclass);
				if(!i){
					// stop if nothing to add (the last base)
					break;
				}
				// mix in properties
				t = bases[i];
				(t._meta ? mixOwn : mix)(proto, t.prototype);
				// chain in new constructor
				ctor = new Function;
				ctor.superclass = superclass;
				ctor.prototype = proto;
				superclass = proto.constructor = ctor;
			}
		}else{
			proto = {};
		}
		// add all properties
		safeMixin(proto, props);
		// add constructor
		t = props.constructor;
		if(t !== op.constructor){
			t.nom = cname;
			proto.constructor = t;
		}

		// collect chains and flags
		for(i = mixins - 1; i; --i){ // intentional assignment
			t = bases[i]._meta;
			if(t && t.chains){
				chains = mix(chains || {}, t.chains);
			}
		}
		if(proto["-chains-"]){
			chains = mix(chains || {}, proto["-chains-"]);
		}

		// build ctor
		t = !chains || !chains.hasOwnProperty(cname);
		bases[0] = ctor = (chains && chains.constructor === "manual") ? simpleConstructor(bases) :
			(bases.length == 1 ? singleConstructor(props.constructor, t) : chainedConstructor(bases, t));

		// add meta information to the constructor
		ctor._meta  = {bases: bases, hidden: props, chains: chains,
			parents: parents, ctor: props.constructor};
		ctor.superclass = superclass && superclass.prototype;
		ctor.extend = extend;
		ctor.prototype = proto;
		proto.constructor = ctor;

		// add "standard" methods to the prototype
		proto.getInherited = getInherited;
		proto.inherited = inherited;
		proto.isInstanceOf = isInstanceOf;

		// add name if specified
		if(className){
			proto.declaredClass = className;
			d.setObject(className, ctor);
		}

		// build chains and add them to the prototype
		if(chains){
			for(name in chains){
				if(proto[name] && typeof chains[name] == "string" && name != cname){
					t = proto[name] = chain(name, bases, chains[name] === "after");
					t.nom = name;
				}
			}
		}
		// chained methods do not return values
		// no need to chain "invisible" functions

		return ctor;	// Function
	};

	d.safeMixin = safeMixin;

	/*=====
	dojo.declare = function(className, superclass, props){
		//	summary:
		//		Create a feature-rich constructor from compact notation.
		//	className: String?:
		//		The optional name of the constructor (loosely, a "class")
		//		stored in the "declaredClass" property in the created prototype.
		//		It will be used as a global name for a created constructor.
		//	superclass: Function|Function[]:
		//		May be null, a Function, or an Array of Functions. This argument
		//		specifies a list of bases (the left-most one is the most deepest
		//		base).
		//	props: Object:
		//		An object whose properties are copied to the created prototype.
		//		Add an instance-initialization function by making it a property
		//		named "constructor".
		//	returns:
		//		New constructor function.
		//	description:
		//		Create a constructor using a compact notation for inheritance and
		//		prototype extension.
		//
		//		Mixin ancestors provide a type of multiple inheritance.
		//		Prototypes of mixin ancestors are copied to the new class:
		//		changes to mixin prototypes will not affect classes to which
		//		they have been mixed in.
		//
		//		Ancestors can be compound classes created by this version of
		//		dojo.declare. In complex cases all base classes are going to be
		//		linearized according to C3 MRO algorithm
		//		(see http://www.python.org/download/releases/2.3/mro/ for more
		//		details).
		//
		//		"className" is cached in "declaredClass" property of the new class,
		//		if it was supplied. The immediate super class will be cached in
		//		"superclass" property of the new class.
		//
		//		Methods in "props" will be copied and modified: "nom" property
		//		(the declared name of the method) will be added to all copied
		//		functions to help identify them for the internal machinery. Be
		//		very careful, while reusing methods: if you use the same
		//		function under different names, it can produce errors in some
		//		cases.
		//
		//		It is possible to use constructors created "manually" (without
		//		dojo.declare) as bases. They will be called as usual during the
		//		creation of an instance, their methods will be chained, and even
		//		called by "this.inherited()".
		//
		//		Special property "-chains-" governs how to chain methods. It is
		//		a dictionary, which uses method names as keys, and hint strings
		//		as values. If a hint string is "after", this method will be
		//		called after methods of its base classes. If a hint string is
		//		"before", this method will be called before methods of its base
		//		classes.
		//
		//		If "constructor" is not mentioned in "-chains-" property, it will
		//		be chained using the legacy mode: using "after" chaining,
		//		calling preamble() method before each constructor, if available,
		//		and calling postscript() after all constructors were executed.
		//		If the hint is "after", it is chained as a regular method, but
		//		postscript() will be called after the chain of constructors.
		//		"constructor" cannot be chained "before", but it allows
		//		a special hint string: "manual", which means that constructors
		//		are not going to be chained in any way, and programmer will call
		//		them manually using this.inherited(). In the latter case
		//		postscript() will be called after the construction.
		//
		//		All chaining hints are "inherited" from base classes and
		//		potentially can be overridden. Be very careful when overriding
		//		hints! Make sure that all chained methods can work in a proposed
		//		manner of chaining.
		//
		//		Once a method was chained, it is impossible to unchain it. The
		//		only exception is "constructor". You don't need to define a
		//		method in order to supply a chaining hint.
		//
		//		If a method is chained, it cannot use this.inherited() because
		//		all other methods in the hierarchy will be called automatically.
		//
		//		Usually constructors and initializers of any kind are chained
		//		using "after" and destructors of any kind are chained as
		//		"before". Note that chaining assumes that chained methods do not
		//		return any value: any returned value will be discarded.
		//
		//	example:
		//	|	dojo.declare("my.classes.bar", my.classes.foo, {
		//	|		// properties to be added to the class prototype
		//	|		someValue: 2,
		//	|		// initialization function
		//	|		constructor: function(){
		//	|			this.myComplicatedObject = new ReallyComplicatedObject();
		//	|		},
		//	|		// other functions
		//	|		someMethod: function(){
		//	|			doStuff();
		//	|		}
		//	|	});
		//
		//	example:
		//	|	var MyBase = dojo.declare(null, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var MyClass1 = dojo.declare(MyBase, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var MyClass2 = dojo.declare(MyBase, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var MyDiamond = dojo.declare([MyClass1, MyClass2], {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//
		//	example:
		//	|	var F = function(){ console.log("raw constructor"); };
		//	|	F.prototype.method = function(){
		//	|		console.log("raw method");
		//	|	};
		//	|	var A = dojo.declare(F, {
		//	|		constructor: function(){
		//	|			console.log("A.constructor");
		//	|		},
		//	|		method: function(){
		//	|			console.log("before calling F.method...");
		//	|			this.inherited(arguments);
		//	|			console.log("...back in A");
		//	|		}
		//	|	});
		//	|	new A().method();
		//	|	// will print:
		//	|	// raw constructor
		//	|	// A.constructor
		//	|	// before calling F.method...
		//	|	// raw method
		//	|	// ...back in A
		//
		//	example:
		//	|	var A = dojo.declare(null, {
		//	|		"-chains-": {
		//	|			destroy: "before"
		//	|		}
		//	|	});
		//	|	var B = dojo.declare(A, {
		//	|		constructor: function(){
		//	|			console.log("B.constructor");
		//	|		},
		//	|		destroy: function(){
		//	|			console.log("B.destroy");
		//	|		}
		//	|	});
		//	|	var C = dojo.declare(B, {
		//	|		constructor: function(){
		//	|			console.log("C.constructor");
		//	|		},
		//	|		destroy: function(){
		//	|			console.log("C.destroy");
		//	|		}
		//	|	});
		//	|	new C().destroy();
		//	|	// prints:
		//	|	// B.constructor
		//	|	// C.constructor
		//	|	// C.destroy
		//	|	// B.destroy
		//
		//	example:
		//	|	var A = dojo.declare(null, {
		//	|		"-chains-": {
		//	|			constructor: "manual"
		//	|		}
		//	|	});
		//	|	var B = dojo.declare(A, {
		//	|		constructor: function(){
		//	|			// ...
		//	|			// call the base constructor with new parameters
		//	|			this.inherited(arguments, [1, 2, 3]);
		//	|			// ...
		//	|		}
		//	|	});
		//
		//	example:
		//	|	var A = dojo.declare(null, {
		//	|		"-chains-": {
		//	|			m1: "before"
		//	|		},
		//	|		m1: function(){
		//	|			console.log("A.m1");
		//	|		},
		//	|		m2: function(){
		//	|			console.log("A.m2");
		//	|		}
		//	|	});
		//	|	var B = dojo.declare(A, {
		//	|		"-chains-": {
		//	|			m2: "after"
		//	|		},
		//	|		m1: function(){
		//	|			console.log("B.m1");
		//	|		},
		//	|		m2: function(){
		//	|			console.log("B.m2");
		//	|		}
		//	|	});
		//	|	var x = new B();
		//	|	x.m1();
		//	|	// prints:
		//	|	// B.m1
		//	|	// A.m1
		//	|	x.m2();
		//	|	// prints:
		//	|	// A.m2
		//	|	// B.m2
		return new Function(); // Function
	};
	=====*/

	/*=====
	dojo.safeMixin = function(target, source){
		//	summary:
		//		Mix in properties skipping a constructor and decorating functions
		//		like it is done by dojo.declare.
		//	target: Object
		//		Target object to accept new properties.
		//	source: Object
		//		Source object for new properties.
		//	description:
		//		This function is used to mix in properties like dojo._mixin does,
		//		but it skips a constructor property and decorates functions like
		//		dojo.declare does.
		//
		//		It is meant to be used with classes and objects produced with
		//		dojo.declare. Functions mixed in with dojo.safeMixin can use
		//		this.inherited() like normal methods.
		//
		//		This function is used to implement extend() method of a constructor
		//		produced with dojo.declare().
		//
		//	example:
		//	|	var A = dojo.declare(null, {
		//	|		m1: function(){
		//	|			console.log("A.m1");
		//	|		},
		//	|		m2: function(){
		//	|			console.log("A.m2");
		//	|		}
		//	|	});
		//	|	var B = dojo.declare(A, {
		//	|		m1: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("B.m1");
		//	|		}
		//	|	});
		//	|	B.extend({
		//	|		m2: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("B.m2");
		//	|		}
		//	|	});
		//	|	var x = new B();
		//	|	dojo.safeMixin(x, {
		//	|		m1: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("X.m1");
		//	|		},
		//	|		m2: function(){
		//	|			this.inherited(arguments);
		//	|			console.log("X.m2");
		//	|		}
		//	|	});
		//	|	x.m2();
		//	|	// prints:
		//	|	// A.m1
		//	|	// B.m1
		//	|	// X.m1
	};
	=====*/

	/*=====
	Object.inherited = function(name, args, newArgs){
		//	summary:
		//		Calls a super method.
		//	name: String?
		//		The optional method name. Should be the same as the caller's
		//		name. Usually "name" is specified in complex dynamic cases, when
		//		the calling method was dynamically added, undecorated by
		//		dojo.declare, and it cannot be determined.
		//	args: Arguments
		//		The caller supply this argument, which should be the original
		//		"arguments".
		//	newArgs: Object?
		//		If "true", the found function will be returned without
		//		executing it.
		//		If Array, it will be used to call a super method. Otherwise
		//		"args" will be used.
		//	returns:
		//		Whatever is returned by a super method, or a super method itself,
		//		if "true" was specified as newArgs.
		//	description:
		//		This method is used inside method of classes produced with
		//		dojo.declare to call a super method (next in the chain). It is
		//		used for manually controlled chaining. Consider using the regular
		//		chaining, because it is faster. Use "this.inherited()" only in
		//		complex cases.
		//
		//		This method cannot me called from automatically chained
		//		constructors including the case of a special (legacy)
		//		constructor chaining. It cannot be called from chained methods.
		//
		//		If "this.inherited()" cannot find the next-in-chain method, it
		//		does nothing and returns "undefined". The last method in chain
		//		can be a default method implemented in Object, which will be
		//		called last.
		//
		//		If "name" is specified, it is assumed that the method that
		//		received "args" is the parent method for this call. It is looked
		//		up in the chain list and if it is found the next-in-chain method
		//		is called. If it is not found, the first-in-chain method is
		//		called.
		//
		//		If "name" is not specified, it will be derived from the calling
		//		method (using a methoid property "nom").
		//
		//	example:
		//	|	var B = dojo.declare(A, {
		//	|		method1: function(a, b, c){
		//	|			this.inherited(arguments);
		//	|		},
		//	|		method2: function(a, b){
		//	|			return this.inherited(arguments, [a + b]);
		//	|		}
		//	|	});
		//	|	// next method is not in the chain list because it is added
		//	|	// manually after the class was created.
		//	|	B.prototype.method3 = function(){
		//	|		console.log("This is a dynamically-added method.");
		//	|		this.inherited("method3", arguments);
		//	|	};
		//	example:
		//	|	var B = dojo.declare(A, {
		//	|		method: function(a, b){
		//	|			var super = this.inherited(arguments, true);
		//	|			// ...
		//	|			if(!super){
		//	|				console.log("there is no super method");
		//	|				return 0;
		//	|			}
		//	|			return super.apply(this, arguments);
		//	|		}
		//	|	});
		return	{};	// Object
	}
	=====*/

	/*=====
	Object.getInherited = function(name, args){
		//	summary:
		//		Returns a super method.
		//	name: String?
		//		The optional method name. Should be the same as the caller's
		//		name. Usually "name" is specified in complex dynamic cases, when
		//		the calling method was dynamically added, undecorated by
		//		dojo.declare, and it cannot be determined.
		//	args: Arguments
		//		The caller supply this argument, which should be the original
		//		"arguments".
		//	returns:
		//		Returns a super method (Function) or "undefined".
		//	description:
		//		This method is a convenience method for "this.inherited()".
		//		It uses the same algorithm but instead of executing a super
		//		method, it returns it, or "undefined" if not found.
		//
		//	example:
		//	|	var B = dojo.declare(A, {
		//	|		method: function(a, b){
		//	|			var super = this.getInherited(arguments);
		//	|			// ...
		//	|			if(!super){
		//	|				console.log("there is no super method");
		//	|				return 0;
		//	|			}
		//	|			return super.apply(this, arguments);
		//	|		}
		//	|	});
		return	{};	// Object
	}
	=====*/

	/*=====
	Object.isInstanceOf = function(cls){
		//	summary:
		//		Checks the inheritance chain to see if it is inherited from this
		//		class.
		//	cls: Function
		//		Class constructor.
		//	returns:
		//		"true", if this object is inherited from this class, "false"
		//		otherwise.
		//	description:
		//		This method is used with instances of classes produced with
		//		dojo.declare to determine of they support a certain interface or
		//		not. It models "instanceof" operator.
		//
		//	example:
		//	|	var A = dojo.declare(null, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var B = dojo.declare(null, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var C = dojo.declare([A, B], {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|	var D = dojo.declare(A, {
		//	|		// constructor, properties, and methods go here
		//	|		// ...
		//	|	});
		//	|
		//	|	var a = new A(), b = new B(), c = new C(), d = new D();
		//	|
		//	|	console.log(a.isInstanceOf(A)); // true
		//	|	console.log(b.isInstanceOf(A)); // false
		//	|	console.log(c.isInstanceOf(A)); // true
		//	|	console.log(d.isInstanceOf(A)); // true
		//	|
		//	|	console.log(a.isInstanceOf(B)); // false
		//	|	console.log(b.isInstanceOf(B)); // true
		//	|	console.log(c.isInstanceOf(B)); // true
		//	|	console.log(d.isInstanceOf(B)); // false
		//	|
		//	|	console.log(a.isInstanceOf(C)); // false
		//	|	console.log(b.isInstanceOf(C)); // false
		//	|	console.log(c.isInstanceOf(C)); // true
		//	|	console.log(d.isInstanceOf(C)); // false
		//	|
		//	|	console.log(a.isInstanceOf(D)); // false
		//	|	console.log(b.isInstanceOf(D)); // false
		//	|	console.log(c.isInstanceOf(D)); // false
		//	|	console.log(d.isInstanceOf(D)); // true
		return	{};	// Object
	}
	=====*/

	/*=====
	Object.extend = function(source){
		//	summary:
		//		Adds all properties and methods of source to constructor's
		//		prototype, making them available to all instances created with
		//		constructor. This method is specific to constructors created with
		//		dojo.declare.
		//	source: Object
		//		Source object which properties are going to be copied to the
		//		constructor's prototype.
		//	description:
		//		Adds source properties to the constructor's prototype. It can
		//		override existing properties.
		//
		//		This method is similar to dojo.extend function, but it is specific
		//		to constructors produced by dojo.declare. It is implemented
		//		using dojo.safeMixin, and it skips a constructor property,
		//		and properly decorates copied functions.
		//
		//	example:
		//	|	var A = dojo.declare(null, {
		//	|		m1: function(){},
		//	|		s1: "Popokatepetl"
		//	|	});
		//	|	A.extend({
		//	|		m1: function(){},
		//	|		m2: function(){},
		//	|		f1: true,
		//	|		d1: 42
		//	|	});
	};
	=====*/
})();

}

if(!dojo._hasResource["dojo._base.connect"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.connect"] = true;
dojo.provide("dojo._base.connect");


// this file courtesy of the TurboAjax Group, licensed under a Dojo CLA

// low-level delegation machinery
dojo._listener = {
	// create a dispatcher function
	getDispatcher: function(){
		// following comments pulled out-of-line to prevent cloning them 
		// in the returned function.
		// - indices (i) that are really in the array of listeners (ls) will 
		//   not be in Array.prototype. This is the 'sparse array' trick
		//   that keeps us safe from libs that take liberties with built-in 
		//   objects
		// - listener is invoked with current scope (this)
		return function(){
			var ap=Array.prototype, c=arguments.callee, ls=c._listeners, t=c.target;
			// return value comes from original target function
			var r = t && t.apply(this, arguments);
			// make local copy of listener array so it is immutable during processing
			var i, lls;
											lls = [].concat(ls);
							
			// invoke listeners after target function
			for(i in lls){
				if(!(i in ap)){
					lls[i].apply(this, arguments);
				}
			}
			// return value comes from original target function
			return r;
		};
	},
	// add a listener to an object
	add: function(/*Object*/ source, /*String*/ method, /*Function*/ listener){
		// Whenever 'method' is invoked, 'listener' will have the same scope.
		// Trying to supporting a context object for the listener led to 
		// complexity. 
		// Non trivial to provide 'once' functionality here
		// because listener could be the result of a dojo.hitch call,
		// in which case two references to the same hitch target would not
		// be equivalent. 
		source = source || dojo.global;
		// The source method is either null, a dispatcher, or some other function
		var f = source[method];
		// Ensure a dispatcher
		if(!f || !f._listeners){
			var d = dojo._listener.getDispatcher();
			// original target function is special
			d.target = f;
			// dispatcher holds a list of listeners
			d._listeners = []; 
			// redirect source to dispatcher
			f = source[method] = d;
		}
		// The contract is that a handle is returned that can 
		// identify this listener for disconnect. 
		//
		// The type of the handle is private. Here is it implemented as Integer. 
		// DOM event code has this same contract but handle is Function 
		// in non-IE browsers.
		//
		// We could have separate lists of before and after listeners.
		return f._listeners.push(listener); /*Handle*/
	},
	// remove a listener from an object
	remove: function(/*Object*/ source, /*String*/ method, /*Handle*/ handle){
		var f = (source || dojo.global)[method];
		// remember that handle is the index+1 (0 is not a valid handle)
		if(f && f._listeners && handle--){
			delete f._listeners[handle];
		}
	}
};

// Multiple delegation for arbitrary methods.

// This unit knows nothing about DOM, but we include DOM aware documentation
// and dontFix argument here to help the autodocs. Actual DOM aware code is in
// event.js.

dojo.connect = function(/*Object|null*/ obj, 
						/*String*/ event, 
						/*Object|null*/ context, 
						/*String|Function*/ method,
						/*Boolean?*/ dontFix){
	// summary:
	//		`dojo.connect` is the core event handling and delegation method in
	//		Dojo. It allows one function to "listen in" on the execution of
	//		any other, triggering the second whenever the first is called. Many
	//		listeners may be attached to a function, and source functions may
	//		be either regular function calls or DOM events.
	//
	// description:
	//		Connects listeners to actions, so that after event fires, a
	//		listener is called with the same arguments passed to the original
	//		function.
	//
	//		Since `dojo.connect` allows the source of events to be either a
	//		"regular" JavaScript function or a DOM event, it provides a uniform
	//		interface for listening to all the types of events that an
	//		application is likely to deal with though a single, unified
	//		interface. DOM programmers may want to think of it as
	//		"addEventListener for everything and anything".
	//
	//		When setting up a connection, the `event` parameter must be a
	//		string that is the name of the method/event to be listened for. If
	//		`obj` is null, `dojo.global` is assumed, meaning that connections
	//		to global methods are supported but also that you may inadvertently
	//		connect to a global by passing an incorrect object name or invalid
	//		reference.
	//
	//		`dojo.connect` generally is forgiving. If you pass the name of a
	//		function or method that does not yet exist on `obj`, connect will
	//		not fail, but will instead set up a stub method. Similarly, null
	//		arguments may simply be omitted such that fewer than 4 arguments
	//		may be required to set up a connection See the examples for details.
	//
	//		The return value is a handle that is needed to 
	//		remove this connection with `dojo.disconnect`.
	//
	// obj: 
	//		The source object for the event function. 
	//		Defaults to `dojo.global` if null.
	//		If obj is a DOM node, the connection is delegated 
	//		to the DOM event manager (unless dontFix is true).
	//
	// event:
	//		String name of the event function in obj. 
	//		I.e. identifies a property `obj[event]`.
	//
	// context: 
	//		The object that method will receive as "this".
	//
	//		If context is null and method is a function, then method
	//		inherits the context of event.
	//	
	//		If method is a string then context must be the source 
	//		object object for method (context[method]). If context is null,
	//		dojo.global is used.
	//
	// method:
	//		A function reference, or name of a function in context. 
	//		The function identified by method fires after event does. 
	//		method receives the same arguments as the event.
	//		See context argument comments for information on method's scope.
	//
	// dontFix:
	//		If obj is a DOM node, set dontFix to true to prevent delegation 
	//		of this connection to the DOM event manager.
	//
	// example:
	//		When obj.onchange(), do ui.update():
	//	|	dojo.connect(obj, "onchange", ui, "update");
	//	|	dojo.connect(obj, "onchange", ui, ui.update); // same
	//
	// example:
	//		Using return value for disconnect:
	//	|	var link = dojo.connect(obj, "onchange", ui, "update");
	//	|	...
	//	|	dojo.disconnect(link);
	//
	// example:
	//		When onglobalevent executes, watcher.handler is invoked:
	//	|	dojo.connect(null, "onglobalevent", watcher, "handler");
	//
	// example:
	//		When ob.onCustomEvent executes, customEventHandler is invoked:
	//	|	dojo.connect(ob, "onCustomEvent", null, "customEventHandler");
	//	|	dojo.connect(ob, "onCustomEvent", "customEventHandler"); // same
	//
	// example:
	//		When ob.onCustomEvent executes, customEventHandler is invoked
	//		with the same scope (this):
	//	|	dojo.connect(ob, "onCustomEvent", null, customEventHandler);
	//	|	dojo.connect(ob, "onCustomEvent", customEventHandler); // same
	//
	// example:
	//		When globalEvent executes, globalHandler is invoked
	//		with the same scope (this):
	//	|	dojo.connect(null, "globalEvent", null, globalHandler);
	//	|	dojo.connect("globalEvent", globalHandler); // same

	// normalize arguments
	var a=arguments, args=[], i=0;
	// if a[0] is a String, obj was omitted
	args.push(dojo.isString(a[0]) ? null : a[i++], a[i++]);
	// if the arg-after-next is a String or Function, context was NOT omitted
	var a1 = a[i+1];
	args.push(dojo.isString(a1)||dojo.isFunction(a1) ? a[i++] : null, a[i++]);
	// absorb any additional arguments
	for(var l=a.length; i<l; i++){	args.push(a[i]); }
	// do the actual work
	return dojo._connect.apply(this, args); /*Handle*/
}

// used by non-browser hostenvs. always overriden by event.js
dojo._connect = function(obj, event, context, method){
	var l=dojo._listener, h=l.add(obj, event, dojo.hitch(context, method)); 
	return [obj, event, h, l]; // Handle
}

dojo.disconnect = function(/*Handle*/ handle){
	// summary:
	//		Remove a link created by dojo.connect.
	// description:
	//		Removes the connection between event and the method referenced by handle.
	// handle:
	//		the return value of the dojo.connect call that created the connection.
	if(handle && handle[0] !== undefined){
		dojo._disconnect.apply(this, handle);
		// let's not keep this reference
		delete handle[0];
	}
}

dojo._disconnect = function(obj, event, handle, listener){
	listener.remove(obj, event, handle);
}

// topic publish/subscribe

dojo._topics = {};

dojo.subscribe = function(/*String*/ topic, /*Object|null*/ context, /*String|Function*/ method){
	//	summary:
	//		Attach a listener to a named topic. The listener function is invoked whenever the
	//		named topic is published (see: dojo.publish).
	//		Returns a handle which is needed to unsubscribe this listener.
	//	context:
	//		Scope in which method will be invoked, or null for default scope.
	//	method:
	//		The name of a function in context, or a function reference. This is the function that
	//		is invoked when topic is published.
	//	example:
	//	|	dojo.subscribe("alerts", null, function(caption, message){ alert(caption + "\n" + message); });
	//	|	dojo.publish("alerts", [ "read this", "hello world" ]);																	

	// support for 2 argument invocation (omitting context) depends on hitch
	return [topic, dojo._listener.add(dojo._topics, topic, dojo.hitch(context, method))]; /*Handle*/
}

dojo.unsubscribe = function(/*Handle*/ handle){
	//	summary:
	//	 	Remove a topic listener. 
	//	handle:
	//	 	The handle returned from a call to subscribe.
	//	example:
	//	|	var alerter = dojo.subscribe("alerts", null, function(caption, message){ alert(caption + "\n" + message); };
	//	|	...
	//	|	dojo.unsubscribe(alerter);
	if(handle){
		dojo._listener.remove(dojo._topics, handle[0], handle[1]);
	}
}

dojo.publish = function(/*String*/ topic, /*Array*/ args){
	//	summary:
	//	 	Invoke all listener method subscribed to topic.
	//	topic:
	//	 	The name of the topic to publish.
	//	args:
	//	 	An array of arguments. The arguments will be applied 
	//	 	to each topic subscriber (as first class parameters, via apply).
	//	example:
	//	|	dojo.subscribe("alerts", null, function(caption, message){ alert(caption + "\n" + message); };
	//	|	dojo.publish("alerts", [ "read this", "hello world" ]);	

	// Note that args is an array, which is more efficient vs variable length
	// argument list.  Ideally, var args would be implemented via Array
	// throughout the APIs.
	var f = dojo._topics[topic];
	if(f){
		f.apply(this, args||[]);
	}
}

dojo.connectPublisher = function(	/*String*/ topic, 
									/*Object|null*/ obj, 
									/*String*/ event){
	//	summary:
	//	 	Ensure that every time obj.event() is called, a message is published
	//	 	on the topic. Returns a handle which can be passed to
	//	 	dojo.disconnect() to disable subsequent automatic publication on
	//	 	the topic.
	//	topic:
	//	 	The name of the topic to publish.
	//	obj: 
	//	 	The source object for the event function. Defaults to dojo.global
	//	 	if null.
	//	event:
	//	 	The name of the event function in obj. 
	//	 	I.e. identifies a property obj[event].
	//	example:
	//	|	dojo.connectPublisher("/ajax/start", dojo, "xhrGet");
	var pf = function(){ dojo.publish(topic, arguments); }
	return event ? dojo.connect(obj, event, pf) : dojo.connect(obj, pf); //Handle
};

}

if(!dojo._hasResource["dojo._base.Deferred"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.Deferred"] = true;
dojo.provide("dojo._base.Deferred");


(function(){
	var mutator = function(){};		
	var freeze = Object.freeze || function(){};
	// A deferred provides an API for creating and resolving a promise.
	dojo.Deferred = function(/*Function?*/canceller){
	// summary:
	//		Deferreds provide a generic means for encapsulating an asynchronous
	// 		operation and notifying users of the completion and result of the operation. 
	// description:
	//		The dojo.Deferred API is based on the concept of promises that provide a
	//		generic interface into the eventual completion of an asynchronous action.
	//		The motivation for promises fundamentally is about creating a 
	//		separation of concerns that allows one to achieve the same type of 
	//		call patterns and logical data flow in asynchronous code as can be 
	//		achieved in synchronous code. Promises allows one 
	//		to be able to call a function purely with arguments needed for 
	//		execution, without conflating the call with concerns of whether it is 
	//		sync or async. One shouldn't need to alter a call's arguments if the 
	//		implementation switches from sync to async (or vice versa). By having 
	//		async functions return promises, the concerns of making the call are 
	//		separated from the concerns of asynchronous interaction (which are 
	//		handled by the promise).
	// 
	//  	The dojo.Deferred is a type of promise that provides methods for fulfilling the 
	// 		promise with a successful result or an error. The most important method for 
	// 		working with Dojo's promises is the then() method, which follows the 
	// 		CommonJS proposed promise API. An example of using a Dojo promise:
	//		
	//		| 	var resultingPromise = someAsyncOperation.then(function(result){
	//		|		... handle result ...
	//		|	},
	//		|	function(error){
	//		|		... handle error ...
	//		|	});
	//	
	//		The .then() call returns a new promise that represents the result of the 
	// 		execution of the callback. The callbacks will never affect the original promises value.
	//
	//		The dojo.Deferred instances also provide the following functions for backwards compatibility:
	//
	//			* addCallback(handler)
	//			* addErrback(handler)
	//			* callback(result)
	//			* errback(result)
	//
	//		Callbacks are allowed to return promisesthemselves, so
	//		you can build complicated sequences of events with ease.
	//
	//		The creator of the Deferred may specify a canceller.  The canceller
	//		is a function that will be called if Deferred.cancel is called
	//		before the Deferred fires. You can use this to implement clean
	//		aborting of an XMLHttpRequest, etc. Note that cancel will fire the
	//		deferred with a CancelledError (unless your canceller returns
	//		another kind of error), so the errbacks should be prepared to
	//		handle that error for cancellable Deferreds.
	// example:
	//	|	var deferred = new dojo.Deferred();
	//	|	setTimeout(function(){ deferred.callback({success: true}); }, 1000);
	//	|	return deferred;
	// example:
	//		Deferred objects are often used when making code asynchronous. It
	//		may be easiest to write functions in a synchronous manner and then
	//		split code using a deferred to trigger a response to a long-lived
	//		operation. For example, instead of register a callback function to
	//		denote when a rendering operation completes, the function can
	//		simply return a deferred:
	//
	//		|	// callback style:
	//		|	function renderLotsOfData(data, callback){
	//		|		var success = false
	//		|		try{
	//		|			for(var x in data){
	//		|				renderDataitem(data[x]);
	//		|			}
	//		|			success = true;
	//		|		}catch(e){ }
	//		|		if(callback){
	//		|			callback(success);
	//		|		}
	//		|	}
	//
	//		|	// using callback style
	//		|	renderLotsOfData(someDataObj, function(success){
	//		|		// handles success or failure
	//		|		if(!success){
	//		|			promptUserToRecover();
	//		|		}
	//		|	});
	//		|	// NOTE: no way to add another callback here!!
	// example:
	//		Using a Deferred doesn't simplify the sending code any, but it
	//		provides a standard interface for callers and senders alike,
	//		providing both with a simple way to service multiple callbacks for
	//		an operation and freeing both sides from worrying about details
	//		such as "did this get called already?". With Deferreds, new
	//		callbacks can be added at any time.
	//
	//		|	// Deferred style:
	//		|	function renderLotsOfData(data){
	//		|		var d = new dojo.Deferred();
	//		|		try{
	//		|			for(var x in data){
	//		|				renderDataitem(data[x]);
	//		|			}
	//		|			d.callback(true);
	//		|		}catch(e){ 
	//		|			d.errback(new Error("rendering failed"));
	//		|		}
	//		|		return d;
	//		|	}
	//
	//		|	// using Deferred style
	//		|	renderLotsOfData(someDataObj).then(null, function(){
	//		|		promptUserToRecover();
	//		|	});
	//		|	// NOTE: addErrback and addCallback both return the Deferred
	//		|	// again, so we could chain adding callbacks or save the
	//		|	// deferred for later should we need to be notified again.
	// example:
	//		In this example, renderLotsOfData is syncrhonous and so both
	//		versions are pretty artificial. Putting the data display on a
	//		timeout helps show why Deferreds rock:
	//
	//		|	// Deferred style and async func
	//		|	function renderLotsOfData(data){
	//		|		var d = new dojo.Deferred();
	//		|		setTimeout(function(){
	//		|			try{
	//		|				for(var x in data){
	//		|					renderDataitem(data[x]);
	//		|				}
	//		|				d.callback(true);
	//		|			}catch(e){ 
	//		|				d.errback(new Error("rendering failed"));
	//		|			}
	//		|		}, 100);
	//		|		return d;
	//		|	}
	//
	//		|	// using Deferred style
	//		|	renderLotsOfData(someDataObj).then(null, function(){
	//		|		promptUserToRecover();
	//		|	});
	//
	//		Note that the caller doesn't have to change his code at all to
	//		handle the asynchronous case.
		var result, finished, isError, head, nextListener;
		var promise = this.promise = {};
		
		function complete(value){
			if(finished){
				throw new Error("This deferred has already been resolved");				
			}
			result = value;
			finished = true;
			notify();
		}
		function notify(){
			var mutated;
			while(!mutated && nextListener){
				var listener = nextListener;
				nextListener = nextListener.next;
				if(mutated = (listener.progress == mutator)){ // assignment and check
					finished = false;
				}
				var func = (isError ? listener.error : listener.resolved);
				if (func) {
					try {
						var newResult = func(result);
						if (newResult && typeof newResult.then === "function") {
							newResult.then(dojo.hitch(listener.deferred, "resolve"), dojo.hitch(listener.deferred, "reject"));
							continue;
						}
						var unchanged = mutated && newResult === undefined;
						listener.deferred[unchanged && isError ? "reject" : "resolve"](unchanged ? result : newResult);
					}
					catch (e) {
						listener.deferred.reject(e);
					}
				}else {
					if(isError){
						listener.deferred.reject(result);
					}else{
						listener.deferred.resolve(result);
					}
				}
			}	
		}
		// calling resolve will resolve the promise
		this.resolve = this.callback = function(value){
			// summary:
			//		Fulfills the Deferred instance successfully with the provide value
			this.fired = 0;
			this.results = [value, null];
			complete(value);
		};
		
		
		// calling error will indicate that the promise failed
		this.reject = this.errback = function(error){
			// summary:
			//		Fulfills the Deferred instance as an error with the provided error 
			isError = true;
			this.fired = 1;
			complete(error);
			this.results = [null, error];
			if(!error || error.log !== false){
				(dojo.config.deferredOnError || function(x){ console.error(x); })(error);
			}
		};
		// call progress to provide updates on the progress on the completion of the promise
		this.progress = function(update){
			// summary
			//		Send progress events to all listeners
			var listener = nextListener;
			while(listener){
				var progress = listener.progress;
				progress && progress(update);
				listener = listener.next;	
			}
		};
		this.addCallbacks = function(/*Function?*/callback, /*Function?*/errback){
			this.then(callback, errback, mutator);
			return this;
		};
		// provide the implementation of the promise
		this.then = promise.then = function(/*Function?*/resolvedCallback, /*Function?*/errorCallback, /*Function?*/progressCallback){
			// summary
			// 		Adds a fulfilledHandler, errorHandler, and progressHandler to be called for 
			// 		completion of a promise. The fulfilledHandler is called when the promise 
			// 		is fulfilled. The errorHandler is called when a promise fails. The 
			// 		progressHandler is called for progress events. All arguments are optional 
			// 		and non-function values are ignored. The progressHandler is not only an 
			// 		optional argument, but progress events are purely optional. Promise 
			// 		providers are not required to ever create progress events.
			// 
			// 		This function will return a new promise that is fulfilled when the given 
			// 		fulfilledHandler or errorHandler callback is finished. This allows promise 
			// 		operations to be chained together. The value returned from the callback 
			// 		handler is the fulfillment value for the returned promise. If the callback 
			// 		throws an error, the returned promise will be moved to failed state.
			//	
			// example:
			// 		An example of using a CommonJS compliant promise:
  			//		|	asyncComputeTheAnswerToEverything().
			//		|		then(addTwo).
			//		|		then(printResult, onError);
  			//		|	>44 
			// 		
			var returnDeferred = progressCallback == mutator ? this : new dojo.Deferred(promise.cancel);
			var listener = {
				resolved: resolvedCallback, 
				error: errorCallback, 
				progress: progressCallback, 
				deferred: returnDeferred
			}; 
			if(nextListener){
				head = head.next = listener;
			}
			else{
				nextListener = head = listener;
			}
			if(finished){
				notify();
			}
			return returnDeferred.promise;
		};
		var deferred = this;
		this.cancel = promise.cancel = function () {
			// summary:
			//		Cancels the asynchronous operation
			if(!finished){
				var error = canceller && canceller(deferred);
				if(!finished){
					if (!(error instanceof Error)) {
						error = new Error(error);
					}
					error.log = false;
					deferred.reject(error);
				}
			}
		}
		freeze(promise);
	};
	dojo.extend(dojo.Deferred, {
		addCallback: function (/*Function*/callback) {
			return this.addCallbacks(dojo.hitch.apply(dojo, arguments));
		},
	
		addErrback: function (/*Function*/errback) {
			return this.addCallbacks(null, dojo.hitch.apply(dojo, arguments));
		},
	
		addBoth: function (/*Function*/callback) {
			var enclosed = dojo.hitch.apply(dojo, arguments);
			return this.addCallbacks(enclosed, enclosed);
		},
		fired: -1
	});
})();
dojo.when = function(promiseOrValue, /*Function?*/callback, /*Function?*/errback, /*Function?*/progressHandler){
	// summary:
	//		This provides normalization between normal synchronous values and 
	//		asynchronous promises, so you can interact with them in a common way
	//	example:
	//		|	function printFirstAndList(items){
	//		|		dojo.when(findFirst(items), console.log);
	//		|		dojo.when(findLast(items), console.log);
	//		|	}
	//		|	function findFirst(items){
	//		|		return dojo.when(items, function(items){
	//		|			return items[0];
	//		|		});
	//		|	}
	//		|	function findLast(items){
	//		|		return dojo.when(items, function(items){
	//		|			return items[items.length];
	//		|		});
	//		|	}
	//		And now all three of his functions can be used sync or async.
	//		|	printFirstAndLast([1,2,3,4]) will work just as well as
	//		|	printFirstAndLast(dojo.xhrGet(...));
	
	if(promiseOrValue && typeof promiseOrValue.then === "function"){
		return promiseOrValue.then(callback, errback, progressHandler);
	}
	return callback(promiseOrValue);
};

}

if(!dojo._hasResource["dojo._base.json"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.json"] = true;
dojo.provide("dojo._base.json");

dojo.fromJson = function(/*String*/ json){
	// summary:
	// 		Parses a [JSON](http://json.org) string to return a JavaScript object.
	// description:
	// 		Throws for invalid JSON strings, but it does not use a strict JSON parser. It
	// 		delegates to eval().  The content passed to this method must therefore come
	//		from a trusted source.
	// json: 
	//		a string literal of a JSON item, for instance:
	//			`'{ "foo": [ "bar", 1, { "baz": "thud" } ] }'`

	return eval("(" + json + ")"); // Object
}

dojo._escapeString = function(/*String*/str){
	//summary:
	//		Adds escape sequences for non-visual characters, double quote and
	//		backslash and surrounds with double quotes to form a valid string
	//		literal.
	return ('"' + str.replace(/(["\\])/g, '\\$1') + '"').
		replace(/[\f]/g, "\\f").replace(/[\b]/g, "\\b").replace(/[\n]/g, "\\n").
		replace(/[\t]/g, "\\t").replace(/[\r]/g, "\\r"); // string
}

dojo.toJsonIndentStr = "\t";
dojo.toJson = function(/*Object*/ it, /*Boolean?*/ prettyPrint, /*String?*/ _indentStr){
	//	summary:
	//		Returns a [JSON](http://json.org) serialization of an object.
	//	description:
	//		Returns a [JSON](http://json.org) serialization of an object.
	//		Note that this doesn't check for infinite recursion, so don't do that!
	//	it:
	//		an object to be serialized. Objects may define their own
	//		serialization via a special "__json__" or "json" function
	//		property. If a specialized serializer has been defined, it will
	//		be used as a fallback.
	//	prettyPrint:
	//		if true, we indent objects and arrays to make the output prettier.
	//		The variable `dojo.toJsonIndentStr` is used as the indent string --
	//		to use something other than the default (tab), change that variable
	//		before calling dojo.toJson().
	//	_indentStr:
	//		private variable for recursive calls when pretty printing, do not use.
	//	example:
	//		simple serialization of a trivial object
	//		|	var jsonStr = dojo.toJson({ howdy: "stranger!", isStrange: true });
	//		|	doh.is('{"howdy":"stranger!","isStrange":true}', jsonStr);
	//	example:
	//		a custom serializer for an objects of a particular class:
	//		|	dojo.declare("Furby", null, {
	//		|		furbies: "are strange",
	//		|		furbyCount: 10,
	//		|		__json__: function(){
	//		|		},
	//		|	});

	if(it === undefined){
		return "undefined";
	}
	var objtype = typeof it;
	if(objtype == "number" || objtype == "boolean"){
		return it + "";
	}
	if(it === null){
		return "null";
	}
	if(dojo.isString(it)){ 
		return dojo._escapeString(it); 
	}
	// recurse
	var recurse = arguments.callee;
	// short-circuit for objects that support "json" serialization
	// if they return "self" then just pass-through...
	var newObj;
	_indentStr = _indentStr || "";
	var nextIndent = prettyPrint ? _indentStr + dojo.toJsonIndentStr : "";
	var tf = it.__json__||it.json;
	if(dojo.isFunction(tf)){
		newObj = tf.call(it);
		if(it !== newObj){
			return recurse(newObj, prettyPrint, nextIndent);
		}
	}
	if(it.nodeType && it.cloneNode){ // isNode
		// we can't seriailize DOM nodes as regular objects because they have cycles
		// DOM nodes could be serialized with something like outerHTML, but
		// that can be provided by users in the form of .json or .__json__ function.
		throw new Error("Can't serialize DOM nodes");
	}

	var sep = prettyPrint ? " " : "";
	var newLine = prettyPrint ? "\n" : "";

	// array
	if(dojo.isArray(it)){
		var res = dojo.map(it, function(obj){
			var val = recurse(obj, prettyPrint, nextIndent);
			if(typeof val != "string"){
				val = "undefined";
			}
			return newLine + nextIndent + val;
		});
		return "[" + res.join("," + sep) + newLine + _indentStr + "]";
	}
	/*
	// look in the registry
	try {
		window.o = it;
		newObj = dojo.json.jsonRegistry.match(it);
		return recurse(newObj, prettyPrint, nextIndent);
	}catch(e){
		// console.log(e);
	}
	// it's a function with no adapter, skip it
	*/
	if(objtype == "function"){
		return null; // null
	}
	// generic object code path
	var output = [], key;
	for(key in it){
		var keyStr, val;
		if(typeof key == "number"){
			keyStr = '"' + key + '"';
		}else if(typeof key == "string"){
			keyStr = dojo._escapeString(key);
		}else{
			// skip non-string or number keys
			continue;
		}
		val = recurse(it[key], prettyPrint, nextIndent);
		if(typeof val != "string"){
			// skip non-serializable values
			continue;
		}
		// FIXME: use += on Moz!!
		//	 MOW NOTE: using += is a pain because you have to account for the dangling comma...
		output.push(newLine + nextIndent + keyStr + ":" + sep + val);
	}
	return "{" + output.join("," + sep) + newLine + _indentStr + "}"; // String
}

}

if(!dojo._hasResource["dojo._base.Color"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.Color"] = true;
dojo.provide("dojo._base.Color");



(function(){

	var d = dojo;

	dojo.Color = function(/*Array|String|Object*/ color){
		// summary:
		//	 	Takes a named string, hex string, array of rgb or rgba values,
		//	 	an object with r, g, b, and a properties, or another `dojo.Color` object
		//	 	and creates a new Color instance to work from.
		//
		// example:
		//		Work with a Color instance:
		//	 | var c = new dojo.Color();
		//	 | c.setColor([0,0,0]); // black
		//	 | var hex = c.toHex(); // #000000
		//
		// example:
		//		Work with a node's color:
		//	 | var color = dojo.style("someNode", "backgroundColor");
		//	 | var n = new dojo.Color(color);
		//	 | // adjust the color some
		//	 | n.r *= .5;
		//	 | console.log(n.toString()); // rgb(128, 255, 255);
		if(color){ this.setColor(color); }
	};

	// FIXME:
	// 	there's got to be a more space-efficient way to encode or discover
	// 	these!!  Use hex?
	dojo.Color.named = {
		black:      [0,0,0],
		silver:     [192,192,192],
		gray:       [128,128,128],
		white:      [255,255,255],
		maroon:		[128,0,0],
		red:        [255,0,0],
		purple:		[128,0,128],
		fuchsia:	[255,0,255],
		green:	    [0,128,0],
		lime:	    [0,255,0],
		olive:		[128,128,0],
		yellow:		[255,255,0],
		navy:       [0,0,128],
		blue:       [0,0,255],
		teal:		[0,128,128],
		aqua:		[0,255,255],
		transparent: d.config.transparentColor || [255,255,255]
	};

	dojo.extend(dojo.Color, {
		r: 255, g: 255, b: 255, a: 1,
		_set: function(r, g, b, a){
			var t = this; t.r = r; t.g = g; t.b = b; t.a = a;
		},
		setColor: function(/*Array|String|Object*/ color){
			// summary:
			//		Takes a named string, hex string, array of rgb or rgba values,
			//		an object with r, g, b, and a properties, or another `dojo.Color` object
			//		and sets this color instance to that value.
			//
			// example:
			//	|	var c = new dojo.Color(); // no color
			//	|	c.setColor("#ededed"); // greyish
			if(d.isString(color)){
				d.colorFromString(color, this);
			}else if(d.isArray(color)){
				d.colorFromArray(color, this);
			}else{
				this._set(color.r, color.g, color.b, color.a);
				if(!(color instanceof d.Color)){ this.sanitize(); }
			}
			return this;	// dojo.Color
		},
		sanitize: function(){
			// summary:
			//		Ensures the object has correct attributes
			// description:
			//		the default implementation does nothing, include dojo.colors to
			//		augment it with real checks
			return this;	// dojo.Color
		},
		toRgb: function(){
			// summary:
			//		Returns 3 component array of rgb values
			// example:
			//	|	var c = new dojo.Color("#000000");
			//	| 	console.log(c.toRgb()); // [0,0,0]
			var t = this;
			return [t.r, t.g, t.b];	// Array
		},
		toRgba: function(){
			// summary:
			//		Returns a 4 component array of rgba values from the color
			//		represented by this object.
			var t = this;
			return [t.r, t.g, t.b, t.a];	// Array
		},
		toHex: function(){
			// summary:
			//		Returns a CSS color string in hexadecimal representation
			// example:
			//	| 	console.log(new dojo.Color([0,0,0]).toHex()); // #000000
			var arr = d.map(["r", "g", "b"], function(x){
				var s = this[x].toString(16);
				return s.length < 2 ? "0" + s : s;
			}, this);
			return "#" + arr.join("");	// String
		},
		toCss: function(/*Boolean?*/ includeAlpha){
			// summary:
			//		Returns a css color string in rgb(a) representation
			// example:
			//	|	var c = new dojo.Color("#FFF").toCss();
			//	|	console.log(c); // rgb('255','255','255')
			var t = this, rgb = t.r + ", " + t.g + ", " + t.b;
			return (includeAlpha ? "rgba(" + rgb + ", " + t.a : "rgb(" + rgb) + ")";	// String
		},
		toString: function(){
			// summary:
			//		Returns a visual representation of the color
			return this.toCss(true); // String
		}
	});

	dojo.blendColors = function(
		/*dojo.Color*/ start,
		/*dojo.Color*/ end,
		/*Number*/ weight,
		/*dojo.Color?*/ obj
	){
		// summary:
		//		Blend colors end and start with weight from 0 to 1, 0.5 being a 50/50 blend,
		//		can reuse a previously allocated dojo.Color object for the result
		var t = obj || new d.Color();
		d.forEach(["r", "g", "b", "a"], function(x){
			t[x] = start[x] + (end[x] - start[x]) * weight;
			if(x != "a"){ t[x] = Math.round(t[x]); }
		});
		return t.sanitize();	// dojo.Color
	};

	dojo.colorFromRgb = function(/*String*/ color, /*dojo.Color?*/ obj){
		// summary:
		//		Returns a `dojo.Color` instance from a string of the form
		//		"rgb(...)" or "rgba(...)". Optionally accepts a `dojo.Color`
		//		object to update with the parsed value and return instead of
		//		creating a new object.
		// returns:
		//		A dojo.Color object. If obj is passed, it will be the return value.
		var m = color.toLowerCase().match(/^rgba?\(([\s\.,0-9]+)\)/);
		return m && dojo.colorFromArray(m[1].split(/\s*,\s*/), obj);	// dojo.Color
	};

	dojo.colorFromHex = function(/*String*/ color, /*dojo.Color?*/ obj){
		// summary:
		//		Converts a hex string with a '#' prefix to a color object.
		//		Supports 12-bit #rgb shorthand. Optionally accepts a
		//		`dojo.Color` object to update with the parsed value.
		//
		// returns:
		//		A dojo.Color object. If obj is passed, it will be the return value.
		//
		// example:
		//	 | var thing = dojo.colorFromHex("#ededed"); // grey, longhand
		//
		// example:
		//	| var thing = dojo.colorFromHex("#000"); // black, shorthand
		var t = obj || new d.Color(),
			bits = (color.length == 4) ? 4 : 8,
			mask = (1 << bits) - 1;
		color = Number("0x" + color.substr(1));
		if(isNaN(color)){
			return null; // dojo.Color
		}
		d.forEach(["b", "g", "r"], function(x){
			var c = color & mask;
			color >>= bits;
			t[x] = bits == 4 ? 17 * c : c;
		});
		t.a = 1;
		return t;	// dojo.Color
	};

	dojo.colorFromArray = function(/*Array*/ a, /*dojo.Color?*/ obj){
		// summary:
		//		Builds a `dojo.Color` from a 3 or 4 element array, mapping each
		//		element in sequence to the rgb(a) values of the color.
		// example:
		//		| var myColor = dojo.colorFromArray([237,237,237,0.5]); // grey, 50% alpha 
		// returns:
		//		A dojo.Color object. If obj is passed, it will be the return value.
		var t = obj || new d.Color();
		t._set(Number(a[0]), Number(a[1]), Number(a[2]), Number(a[3]));
		if(isNaN(t.a)){ t.a = 1; }
		return t.sanitize();	// dojo.Color
	};

	dojo.colorFromString = function(/*String*/ str, /*dojo.Color?*/ obj){
		// summary:
		//		Parses `str` for a color value. Accepts hex, rgb, and rgba
		//		style color values.
		// description:
		//		Acceptable input values for str may include arrays of any form
		//		accepted by dojo.colorFromArray, hex strings such as "#aaaaaa", or
		//		rgb or rgba strings such as "rgb(133, 200, 16)" or "rgba(10, 10,
		//		10, 50)"
		// returns:
		//		A dojo.Color object. If obj is passed, it will be the return value.
		var a = d.Color.named[str];
		return a && d.colorFromArray(a, obj) || d.colorFromRgb(str, obj) || d.colorFromHex(str, obj);
	};
})();

}

if(!dojo._hasResource["dojo._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base"] = true;
dojo.provide("dojo._base");









}

if(!dojo._hasResource["dojo._base.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.window"] = true;
dojo.provide("dojo._base.window");

/*=====
dojo.doc = {
	// summary:
	//		Alias for the current document. 'dojo.doc' can be modified
	//		for temporary context shifting. Also see dojo.withDoc().
	// description:
	//    Refer to dojo.doc rather
	//    than referring to 'window.document' to ensure your code runs
	//    correctly in managed contexts.
	// example:
	// 	|	n.appendChild(dojo.doc.createElement('div'));
}
=====*/
dojo.doc = window["document"] || null;

dojo.body = function(){
	// summary:
	//		Return the body element of the document
	//		return the body object associated with dojo.doc
	// example:
	// 	|	dojo.body().appendChild(dojo.doc.createElement('div'));

	// Note: document.body is not defined for a strict xhtml document
	// Would like to memoize this, but dojo.doc can change vi dojo.withDoc().
	return dojo.doc.body || dojo.doc.getElementsByTagName("body")[0]; // Node
}

dojo.setContext = function(/*Object*/globalObject, /*DocumentElement*/globalDocument){
	// summary:
	//		changes the behavior of many core Dojo functions that deal with
	//		namespace and DOM lookup, changing them to work in a new global
	//		context (e.g., an iframe). The varibles dojo.global and dojo.doc
	//		are modified as a result of calling this function and the result of
	//		`dojo.body()` likewise differs.
	dojo.global = globalObject;
	dojo.doc = globalDocument;
};

dojo.withGlobal = function(	/*Object*/globalObject, 
							/*Function*/callback, 
							/*Object?*/thisObject, 
							/*Array?*/cbArguments){
	// summary:
	//		Invoke callback with globalObject as dojo.global and
	//		globalObject.document as dojo.doc.
	// description:
	//		Invoke callback with globalObject as dojo.global and
	//		globalObject.document as dojo.doc. If provided, globalObject
	//		will be executed in the context of object thisObject
	//		When callback() returns or throws an error, the dojo.global
	//		and dojo.doc will be restored to its previous state.

	var oldGlob = dojo.global;
	try{
		dojo.global = globalObject;
		return dojo.withDoc.call(null, globalObject.document, callback, thisObject, cbArguments);
	}finally{
		dojo.global = oldGlob;
	}
}

dojo.withDoc = function(	/*DocumentElement*/documentObject, 
							/*Function*/callback, 
							/*Object?*/thisObject, 
							/*Array?*/cbArguments){
	// summary:
	//		Invoke callback with documentObject as dojo.doc.
	// description:
	//		Invoke callback with documentObject as dojo.doc. If provided,
	//		callback will be executed in the context of object thisObject
	//		When callback() returns or throws an error, the dojo.doc will
	//		be restored to its previous state.

	var oldDoc = dojo.doc,
		oldLtr = dojo._bodyLtr,
		oldQ = dojo.isQuirks;

	try{
		dojo.doc = documentObject;
		delete dojo._bodyLtr; // uncache
		dojo.isQuirks = dojo.doc.compatMode == "BackCompat"; // no need to check for QuirksMode which was Opera 7 only

		if(thisObject && typeof callback == "string"){
			callback = thisObject[callback];
		}

		return callback.apply(thisObject, cbArguments || []);
	}finally{
		dojo.doc = oldDoc;
		delete dojo._bodyLtr; // in case it was undefined originally, and set to true/false by the alternate document
		if(oldLtr !== undefined){ dojo._bodyLtr = oldLtr; }
		dojo.isQuirks = oldQ;
	}
};
	

}

if(!dojo._hasResource["dojo._base.event"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.event"] = true;
dojo.provide("dojo._base.event");


// this file courtesy of the TurboAjax Group, licensed under a Dojo CLA

(function(){
	// DOM event listener machinery
	var del = (dojo._event_listener = {
		add: function(/*DOMNode*/ node, /*String*/ name, /*Function*/ fp){
			if(!node){return;} 
			name = del._normalizeEventName(name);
			fp = del._fixCallback(name, fp);
			var oname = name;
			if(
								!dojo.isIE && 
								(name == "mouseenter" || name == "mouseleave")
			){
				var ofp = fp;
				//oname = name;
				name = (name == "mouseenter") ? "mouseover" : "mouseout";
				fp = function(e){
					if(!dojo.isDescendant(e.relatedTarget, node)){
						// e.type = oname; // FIXME: doesn't take? SJM: event.type is generally immutable.
						return ofp.call(this, e); 
					}
				}
			}
			node.addEventListener(name, fp, false);
			return fp; /*Handle*/
		},
		remove: function(/*DOMNode*/ node, /*String*/ event, /*Handle*/ handle){
			// summary:
			//		clobbers the listener from the node
			// node:
			//		DOM node to attach the event to
			// event:
			//		the name of the handler to remove the function from
			// handle:
			//		the handle returned from add
			if(node){
				event = del._normalizeEventName(event);
				if(!dojo.isIE && (event == "mouseenter" || event == "mouseleave")){
					event = (event == "mouseenter") ? "mouseover" : "mouseout";
				}

				node.removeEventListener(event, handle, false);
			}
		},
		_normalizeEventName: function(/*String*/ name){
			// Generally, name should be lower case, unless it is special
			// somehow (e.g. a Mozilla DOM event).
			// Remove 'on'.
			return name.slice(0,2) =="on" ? name.slice(2) : name;
		},
		_fixCallback: function(/*String*/ name, fp){
			// By default, we only invoke _fixEvent for 'keypress'
			// If code is added to _fixEvent for other events, we have
			// to revisit this optimization.
			// This also applies to _fixEvent overrides for Safari and Opera
			// below.
			return name != "keypress" ? fp : function(e){ return fp.call(this, del._fixEvent(e, this)); };
		},
		_fixEvent: function(evt, sender){
			// _fixCallback only attaches us to keypress.
			// Switch on evt.type anyway because we might 
			// be called directly from dojo.fixEvent.
			switch(evt.type){
				case "keypress":
					del._setKeyChar(evt);
					break;
			}
			return evt;
		},
		_setKeyChar: function(evt){
			evt.keyChar = evt.charCode ? String.fromCharCode(evt.charCode) : '';
			evt.charOrCode = evt.keyChar || evt.keyCode;
		},
		// For IE and Safari: some ctrl-key combinations (mostly w/punctuation) do not emit a char code in IE
		// we map those virtual key codes to ascii here
		// not valid for all (non-US) keyboards, so maybe we shouldn't bother
		_punctMap: { 
			106:42, 
			111:47, 
			186:59, 
			187:43, 
			188:44, 
			189:45, 
			190:46, 
			191:47, 
			192:96, 
			219:91, 
			220:92, 
			221:93, 
			222:39 
		}
	});

	// DOM events
	
	dojo.fixEvent = function(/*Event*/ evt, /*DOMNode*/ sender){
		// summary:
		//		normalizes properties on the event object including event
		//		bubbling methods, keystroke normalization, and x/y positions
		// evt: Event
		//		native event object
		// sender: DOMNode
		//		node to treat as "currentTarget"
		return del._fixEvent(evt, sender);
	}

	dojo.stopEvent = function(/*Event*/ evt){
		// summary:
		//		prevents propagation and clobbers the default action of the
		//		passed event
		// evt: Event
		//		The event object. If omitted, window.event is used on IE.
		evt.preventDefault();
		evt.stopPropagation();
		// NOTE: below, this method is overridden for IE
	}

	// the default listener to use on dontFix nodes, overriden for IE
	var node_listener = dojo._listener;
	
	// Unify connect and event listeners
	dojo._connect = function(obj, event, context, method, dontFix){
		// FIXME: need a more strict test
		var isNode = obj && (obj.nodeType||obj.attachEvent||obj.addEventListener);
		// choose one of three listener options: raw (connect.js), DOM event on a Node, custom event on a Node
		// we need the third option to provide leak prevention on broken browsers (IE)
		var lid = isNode ? (dontFix ? 2 : 1) : 0, l = [dojo._listener, del, node_listener][lid];
		// create a listener
		var h = l.add(obj, event, dojo.hitch(context, method));
		// formerly, the disconnect package contained "l" directly, but if client code
		// leaks the disconnect package (by connecting it to a node), referencing "l" 
		// compounds the problem.
		// instead we return a listener id, which requires custom _disconnect below.
		// return disconnect package
		return [ obj, event, h, lid ];
	}

	dojo._disconnect = function(obj, event, handle, listener){
		([dojo._listener, del, node_listener][listener]).remove(obj, event, handle);
	}

	// Constants

	// Public: client code should test
	// keyCode against these named constants, as the
	// actual codes can vary by browser.
	dojo.keys = {
		// summary:
		//		Definitions for common key values
		BACKSPACE: 8,
		TAB: 9,
		CLEAR: 12,
		ENTER: 13,
		SHIFT: 16,
		CTRL: 17,
		ALT: 18,
		META: dojo.isSafari ? 91 : 224,		// the apple key on macs
		PAUSE: 19,
		CAPS_LOCK: 20,
		ESCAPE: 27,
		SPACE: 32,
		PAGE_UP: 33,
		PAGE_DOWN: 34,
		END: 35,
		HOME: 36,
		LEFT_ARROW: 37,
		UP_ARROW: 38,
		RIGHT_ARROW: 39,
		DOWN_ARROW: 40,
		INSERT: 45,
		DELETE: 46,
		HELP: 47,
		LEFT_WINDOW: 91,
		RIGHT_WINDOW: 92,
		SELECT: 93,
		NUMPAD_0: 96,
		NUMPAD_1: 97,
		NUMPAD_2: 98,
		NUMPAD_3: 99,
		NUMPAD_4: 100,
		NUMPAD_5: 101,
		NUMPAD_6: 102,
		NUMPAD_7: 103,
		NUMPAD_8: 104,
		NUMPAD_9: 105,
		NUMPAD_MULTIPLY: 106,
		NUMPAD_PLUS: 107,
		NUMPAD_ENTER: 108,
		NUMPAD_MINUS: 109,
		NUMPAD_PERIOD: 110,
		NUMPAD_DIVIDE: 111,
		F1: 112,
		F2: 113,
		F3: 114,
		F4: 115,
		F5: 116,
		F6: 117,
		F7: 118,
		F8: 119,
		F9: 120,
		F10: 121,
		F11: 122,
		F12: 123,
		F13: 124,
		F14: 125,
		F15: 126,
		NUM_LOCK: 144,
		SCROLL_LOCK: 145,
		// virtual key mapping
		copyKey: dojo.isMac && !dojo.isAIR ? (dojo.isSafari ? 91 : 224 ) : 17
	};
	
	var evtCopyKey = dojo.isMac ? "metaKey" : "ctrlKey";
	
	dojo.isCopyKey = function(e){
		// summary:
		//		Checks an event for the copy key (meta on Mac, and ctrl anywhere else)
		// e: Event
		//		Event object to examine
		return e[evtCopyKey];	// Boolean
	};

	// Public: decoding mouse buttons from events

/*=====
	dojo.mouseButtons = {
		// LEFT: Number
		//		Numeric value of the left mouse button for the platform.
		LEFT:   0,
		// MIDDLE: Number
		//		Numeric value of the middle mouse button for the platform.
		MIDDLE: 1,
		// RIGHT: Number
		//		Numeric value of the right mouse button for the platform.
		RIGHT:  2,
	
		isButton: function(e, button){
			// summary:
			//		Checks an event object for a pressed button
			// e: Event
			//		Event object to examine
			// button: Number
			//		The button value (example: dojo.mouseButton.LEFT)
			return e.button == button; // Boolean
		},
		isLeft: function(e){
			// summary:
			//		Checks an event object for the pressed left button
			// e: Event
			//		Event object to examine
			return e.button == 0; // Boolean
		},
		isMiddle: function(e){
			// summary:
			//		Checks an event object for the pressed middle button
			// e: Event
			//		Event object to examine
			return e.button == 1; // Boolean
		},
		isRight: function(e){
			// summary:
			//		Checks an event object for the pressed right button
			// e: Event
			//		Event object to examine
			return e.button == 2; // Boolean
		}
	};
=====*/

		if(dojo.isIE){
		dojo.mouseButtons = {
			LEFT:   1,
			MIDDLE: 4,
			RIGHT:  2,
			// helper functions
			isButton: function(e, button){ return e.button & button; },
			isLeft:   function(e){ return e.button & 1; },
			isMiddle: function(e){ return e.button & 4; },
			isRight:  function(e){ return e.button & 2; }
		};
	}else{
			dojo.mouseButtons = {
			LEFT:   0,
			MIDDLE: 1,
			RIGHT:  2,
			// helper functions
			isButton: function(e, button){ return e.button == button; },
			isLeft:   function(e){ return e.button == 0; },
			isMiddle: function(e){ return e.button == 1; },
			isRight:  function(e){ return e.button == 2; }
		};
		}
	
		// IE event normalization
	if(dojo.isIE){ 
		var _trySetKeyCode = function(e, code){
			try{
				// squelch errors when keyCode is read-only
				// (e.g. if keyCode is ctrl or shift)
				return (e.keyCode = code);
			}catch(e){
				return 0;
			}
		}

		// by default, use the standard listener
		var iel = dojo._listener;
		var listenersName = (dojo._ieListenersName = "_" + dojo._scopeName + "_listeners");
		// dispatcher tracking property
		if(!dojo.config._allow_leaks){
			// custom listener that handles leak protection for DOM events
			node_listener = iel = dojo._ie_listener = {
				// support handler indirection: event handler functions are 
				// referenced here. Event dispatchers hold only indices.
				handlers: [],
				// add a listener to an object
				add: function(/*Object*/ source, /*String*/ method, /*Function*/ listener){
					source = source || dojo.global;
					var f = source[method];
					if(!f||!f[listenersName]){
						var d = dojo._getIeDispatcher();
						// original target function is special
						d.target = f && (ieh.push(f) - 1);
						// dispatcher holds a list of indices into handlers table
						d[listenersName] = [];
						// redirect source to dispatcher
						f = source[method] = d;
					}
					return f[listenersName].push(ieh.push(listener) - 1) ; /*Handle*/
				},
				// remove a listener from an object
				remove: function(/*Object*/ source, /*String*/ method, /*Handle*/ handle){
					var f = (source||dojo.global)[method], l = f && f[listenersName];
					if(f && l && handle--){
						delete ieh[l[handle]];
						delete l[handle];
					}
				}
			};
			// alias used above
			var ieh = iel.handlers;
		}

		dojo.mixin(del, {
			add: function(/*DOMNode*/ node, /*String*/ event, /*Function*/ fp){
				if(!node){return;} // undefined
				event = del._normalizeEventName(event);
				if(event=="onkeypress"){
					// we need to listen to onkeydown to synthesize
					// keypress events that otherwise won't fire
					// on IE
					var kd = node.onkeydown;
					if(!kd || !kd[listenersName] || !kd._stealthKeydownHandle){
						var h = del.add(node, "onkeydown", del._stealthKeyDown);
						kd = node.onkeydown;
						kd._stealthKeydownHandle = h;
						kd._stealthKeydownRefs = 1;
					}else{
						kd._stealthKeydownRefs++;
					}
				}
				return iel.add(node, event, del._fixCallback(fp));
			},
			remove: function(/*DOMNode*/ node, /*String*/ event, /*Handle*/ handle){
				event = del._normalizeEventName(event);
				iel.remove(node, event, handle); 
				if(event=="onkeypress"){
					var kd = node.onkeydown;
					if(--kd._stealthKeydownRefs <= 0){
						iel.remove(node, "onkeydown", kd._stealthKeydownHandle);
						delete kd._stealthKeydownHandle;
					}
				}
			},
			_normalizeEventName: function(/*String*/ eventName){
				// Generally, eventName should be lower case, unless it is
				// special somehow (e.g. a Mozilla event)
				// ensure 'on'
				return eventName.slice(0,2) != "on" ? "on" + eventName : eventName;
			},
			_nop: function(){},
			_fixEvent: function(/*Event*/ evt, /*DOMNode*/ sender){
				// summary:
				//		normalizes properties on the event object including event
				//		bubbling methods, keystroke normalization, and x/y positions
				// evt:
				//		native event object
				// sender:
				//		node to treat as "currentTarget"
				if(!evt){
					var w = sender && (sender.ownerDocument || sender.document || sender).parentWindow || window;
					evt = w.event; 
				}
				if(!evt){return(evt);}
				evt.target = evt.srcElement; 
				evt.currentTarget = (sender || evt.srcElement); 
				evt.layerX = evt.offsetX;
				evt.layerY = evt.offsetY;
				// FIXME: scroll position query is duped from dojo.html to
				// avoid dependency on that entire module. Now that HTML is in
				// Base, we should convert back to something similar there.
				var se = evt.srcElement, doc = (se && se.ownerDocument) || document;
				// DO NOT replace the following to use dojo.body(), in IE, document.documentElement should be used
				// here rather than document.body
				var docBody = ((dojo.isIE < 6) || (doc["compatMode"] == "BackCompat")) ? doc.body : doc.documentElement;
				var offset = dojo._getIeDocumentElementOffset();
				evt.pageX = evt.clientX + dojo._fixIeBiDiScrollLeft(docBody.scrollLeft || 0) - offset.x;
				evt.pageY = evt.clientY + (docBody.scrollTop || 0) - offset.y;
				if(evt.type == "mouseover"){ 
					evt.relatedTarget = evt.fromElement;
				}
				if(evt.type == "mouseout"){ 
					evt.relatedTarget = evt.toElement;
				}
				evt.stopPropagation = del._stopPropagation;
				evt.preventDefault = del._preventDefault;
				return del._fixKeys(evt);
			},
			_fixKeys: function(evt){
				switch(evt.type){
					case "keypress":
						var c = ("charCode" in evt ? evt.charCode : evt.keyCode);
						if (c==10){
							// CTRL-ENTER is CTRL-ASCII(10) on IE, but CTRL-ENTER on Mozilla
							c=0;
							evt.keyCode = 13;
						}else if(c==13||c==27){
							c=0; // Mozilla considers ENTER and ESC non-printable
						}else if(c==3){
							c=99; // Mozilla maps CTRL-BREAK to CTRL-c
						}
						// Mozilla sets keyCode to 0 when there is a charCode
						// but that stops the event on IE.
						evt.charCode = c;
						del._setKeyChar(evt);
						break;
				}
				return evt;
			},
			_stealthKeyDown: function(evt){
				// IE doesn't fire keypress for most non-printable characters.
				// other browsers do, we simulate it here.
				var kp = evt.currentTarget.onkeypress;
				// only works if kp exists and is a dispatcher
				if(!kp || !kp[listenersName]){ return; }
				// munge key/charCode
				var k=evt.keyCode;
				// These are Windows Virtual Key Codes
				// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/WinUI/WindowsUserInterface/UserInput/VirtualKeyCodes.asp
				var unprintable = k!=13 && k!=32 && k!=27 && (k<48||k>90) && (k<96||k>111) && (k<186||k>192) && (k<219||k>222);
				// synthesize keypress for most unprintables and CTRL-keys
				if(unprintable||evt.ctrlKey){
					var c = unprintable ? 0 : k;
					if(evt.ctrlKey){
						if(k==3 || k==13){
							return; // IE will post CTRL-BREAK, CTRL-ENTER as keypress natively 
						}else if(c>95 && c<106){ 
							c -= 48; // map CTRL-[numpad 0-9] to ASCII
						}else if((!evt.shiftKey)&&(c>=65&&c<=90)){ 
							c += 32; // map CTRL-[A-Z] to lowercase
						}else{ 
							c = del._punctMap[c] || c; // map other problematic CTRL combinations to ASCII
						}
					}
					// simulate a keypress event
					var faux = del._synthesizeEvent(evt, {type: 'keypress', faux: true, charCode: c});
					kp.call(evt.currentTarget, faux);
					evt.cancelBubble = faux.cancelBubble;
					evt.returnValue = faux.returnValue;
					_trySetKeyCode(evt, faux.keyCode);
				}
			},
			// Called in Event scope
			_stopPropagation: function(){
				this.cancelBubble = true; 
			},
			_preventDefault: function(){
				// Setting keyCode to 0 is the only way to prevent certain keypresses (namely
				// ctrl-combinations that correspond to menu accelerator keys).
				// Otoh, it prevents upstream listeners from getting this information
				// Try to split the difference here by clobbering keyCode only for ctrl 
				// combinations. If you still need to access the key upstream, bubbledKeyCode is
				// provided as a workaround.
				this.bubbledKeyCode = this.keyCode;
				if(this.ctrlKey){_trySetKeyCode(this, 0);}
				this.returnValue = false;
			}
		});
				
		// override stopEvent for IE
		dojo.stopEvent = function(evt){
			evt = evt || window.event;
			del._stopPropagation.call(evt);
			del._preventDefault.call(evt);
		}
	}
	
	del._synthesizeEvent = function(evt, props){
			var faux = dojo.mixin({}, evt, props);
			del._setKeyChar(faux);
			// FIXME: would prefer to use dojo.hitch: dojo.hitch(evt, evt.preventDefault); 
			// but it throws an error when preventDefault is invoked on Safari
			// does Event.preventDefault not support "apply" on Safari?
			faux.preventDefault = function(){ evt.preventDefault(); }; 
			faux.stopPropagation = function(){ evt.stopPropagation(); }; 
			return faux;
	}
	
		// Opera event normalization
	if(dojo.isOpera){
		dojo.mixin(del, {
			_fixEvent: function(evt, sender){
				switch(evt.type){
					case "keypress":
						var c = evt.which;
						if(c==3){
							c=99; // Mozilla maps CTRL-BREAK to CTRL-c
						}
						// can't trap some keys at all, like INSERT and DELETE
						// there is no differentiating info between DELETE and ".", or INSERT and "-"
						c = c<41 && !evt.shiftKey ? 0 : c;
						if(evt.ctrlKey && !evt.shiftKey && c>=65 && c<=90){
							// lowercase CTRL-[A-Z] keys
							c += 32;
						}
						return del._synthesizeEvent(evt, { charCode: c });
				}
				return evt;
			}
		});
	}
	
		// Webkit event normalization
	if(dojo.isWebKit){
				del._add = del.add;
		del._remove = del.remove;

		dojo.mixin(del, {
			add: function(/*DOMNode*/ node, /*String*/ event, /*Function*/ fp){
				if(!node){return;} // undefined
				var handle = del._add(node, event, fp);
				if(del._normalizeEventName(event) == "keypress"){
					// we need to listen to onkeydown to synthesize
					// keypress events that otherwise won't fire
					// in Safari 3.1+: https://lists.webkit.org/pipermail/webkit-dev/2007-December/002992.html
					handle._stealthKeyDownHandle = del._add(node, "keydown", function(evt){
						//A variation on the IE _stealthKeydown function
						//Synthesize an onkeypress event, but only for unprintable characters.
						var k=evt.keyCode;
						// These are Windows Virtual Key Codes
						// http://msdn.microsoft.com/library/default.asp?url=/library/en-us/winui/WinUI/WindowsUserInterface/UserInput/VirtualKeyCodes.asp
						var unprintable = k!=13 && k!=32 && (k<48 || k>90) && (k<96 || k>111) && (k<186 || k>192) && (k<219 || k>222);
						// synthesize keypress for most unprintables and CTRL-keys
						if(unprintable || evt.ctrlKey){
							var c = unprintable ? 0 : k;
							if(evt.ctrlKey){
								if(k==3 || k==13){
									return; // IE will post CTRL-BREAK, CTRL-ENTER as keypress natively 
								}else if(c>95 && c<106){ 
									c -= 48; // map CTRL-[numpad 0-9] to ASCII
								}else if(!evt.shiftKey && c>=65 && c<=90){ 
									c += 32; // map CTRL-[A-Z] to lowercase
								}else{ 
									c = del._punctMap[c] || c; // map other problematic CTRL combinations to ASCII
								}
							}
							// simulate a keypress event
							var faux = del._synthesizeEvent(evt, {type: 'keypress', faux: true, charCode: c});
							fp.call(evt.currentTarget, faux);
						}
					});
				}
				return handle; /*Handle*/
			},

			remove: function(/*DOMNode*/ node, /*String*/ event, /*Handle*/ handle){
				if(node){
					if(handle._stealthKeyDownHandle){
						del._remove(node, "keydown", handle._stealthKeyDownHandle);
					}
					del._remove(node, event, handle);
				}
			},
			_fixEvent: function(evt, sender){
				switch(evt.type){
					case "keypress":
						if(evt.faux){ return evt; }
						var c = evt.charCode;
						c = c>=32 ? c : 0;
						return del._synthesizeEvent(evt, {charCode: c, faux: true});
				}
				return evt;
			}
		});
		}
	})();

if(dojo.isIE){
	// keep this out of the closure
	// closing over 'iel' or 'ieh' b0rks leak prevention
	// ls[i] is an index into the master handler array
	dojo._ieDispatcher = function(args, sender){
		var ap = Array.prototype,
			h = dojo._ie_listener.handlers,
			c = args.callee,
			ls = c[dojo._ieListenersName],
			t = h[c.target];
		// return value comes from original target function
		var r = t && t.apply(sender, args);
		// make local copy of listener array so it's immutable during processing
		var lls = [].concat(ls);
		// invoke listeners after target function
		for(var i in lls){
			var f = h[lls[i]];
			if(!(i in ap) && f){
				f.apply(sender, args);
			}
		}
		return r;
	}
	dojo._getIeDispatcher = function(){
		// ensure the returned function closes over nothing ("new Function" apparently doesn't close)
		return new Function(dojo._scopeName + "._ieDispatcher(arguments, this)"); // function
	}
	// keep this out of the closure to reduce RAM allocation
	dojo._event_listener._fixCallback = function(fp){
		var f = dojo._event_listener._fixEvent;
		return function(e){ return fp.call(this, f(e, this)); };
	}
}

}

if(!dojo._hasResource["dojo._base.html"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.html"] = true;

dojo.provide("dojo._base.html");

// FIXME: need to add unit tests for all the semi-public methods

try{
	document.execCommand("BackgroundImageCache", false, true);
}catch(e){
	// sane browsers don't have cache "issues"
}

// =============================
// DOM Functions
// =============================

/*=====
dojo.byId = function(id, doc){
	//	summary:
	//		Returns DOM node with matching `id` attribute or `null`
	//		if not found. If `id` is a DomNode, this function is a no-op.
	//
	//	id: String|DOMNode
	//	 	A string to match an HTML id attribute or a reference to a DOM Node
	//
	//	doc: Document?
	//		Document to work in. Defaults to the current value of
	//		dojo.doc.  Can be used to retrieve
	//		node references from other documents.
	//
	//	example:
	//	Look up a node by ID:
	//	|	var n = dojo.byId("foo");
	//
	//	example:
	//	Check if a node exists, and use it.
	//	|	var n = dojo.byId("bar");
	//	|	if(n){ doStuff() ... }
	//
	//	example:
	//	Allow string or DomNode references to be passed to a custom function:
	//	|	var foo = function(nodeOrId){
	//	|		nodeOrId = dojo.byId(nodeOrId);
	//	|		// ... more stuff
	//	|	}
=====*/

if(dojo.isIE || dojo.isOpera){
	dojo.byId = function(id, doc){
		if(typeof id != "string"){
			return id;
		}
		var _d = doc || dojo.doc, te = _d.getElementById(id);
		// attributes.id.value is better than just id in case the 
		// user has a name=id inside a form
		if(te && (te.attributes.id.value == id || te.id == id)){
			return te;
		}else{
			var eles = _d.all[id];
			if(!eles || eles.nodeName){
				eles = [eles];
			}
			// if more than 1, choose first with the correct id
			var i=0;
			while((te=eles[i++])){
				if((te.attributes && te.attributes.id && te.attributes.id.value == id)
					|| te.id == id){
					return te;
				}
			}
		}
	};
}else{
	dojo.byId = function(id, doc){
		// inline'd type check
		return (typeof id == "string") ? (doc || dojo.doc).getElementById(id) : id; // DomNode
	};
}
/*=====
};
=====*/

(function(){
	var d = dojo;
	var byId = d.byId;

	var _destroyContainer = null,
		_destroyDoc;
		d.addOnWindowUnload(function(){
		_destroyContainer = null; //prevent IE leak
	});
	
/*=====
	dojo._destroyElement = function(node){
		// summary:
		// 		Existing alias for `dojo.destroy`. Deprecated, will be removed
		// 		in 2.0
	}
=====*/
	dojo._destroyElement = dojo.destroy = function(/*String|DomNode*/node){
		//	summary:
		//		Removes a node from its parent, clobbering it and all of its
		//		children.
		//
		//	description:
		//		Removes a node from its parent, clobbering it and all of its
		//		children. Function only works with DomNodes, and returns nothing.
		//
		//	node:
		//		A String ID or DomNode reference of the element to be destroyed
		//
		//	example:
		//	Destroy a node byId:
		//	|	dojo.destroy("someId");
		//
		//	example:
		//	Destroy all nodes in a list by reference:
		//	|	dojo.query(".someNode").forEach(dojo.destroy);

		node = byId(node);
		try{
			var doc = node.ownerDocument;
			// cannot use _destroyContainer.ownerDocument since this can throw an exception on IE
			if(!_destroyContainer || _destroyDoc != doc){
				_destroyContainer = doc.createElement("div");
				_destroyDoc = doc;
			}
			_destroyContainer.appendChild(node.parentNode ? node.parentNode.removeChild(node) : node);
			// NOTE: see http://trac.dojotoolkit.org/ticket/2931. This may be a bug and not a feature
			_destroyContainer.innerHTML = "";
		}catch(e){
			/* squelch */
		}
	};

	dojo.isDescendant = function(/*DomNode|String*/node, /*DomNode|String*/ancestor){
		//	summary:
		//		Returns true if node is a descendant of ancestor
		//	node: string id or node reference to test
		//	ancestor: string id or node reference of potential parent to test against
		//
		// example:
		//	Test is node id="bar" is a descendant of node id="foo"
		//	|	if(dojo.isDescendant("bar", "foo")){ ... }
		try{
			node = byId(node);
			ancestor = byId(ancestor);
			while(node){
				if(node == ancestor){
					return true; // Boolean
				}
				node = node.parentNode;
			}
		}catch(e){ /* squelch, return false */ }
		return false; // Boolean
	};

	dojo.setSelectable = function(/*DomNode|String*/node, /*Boolean*/selectable){
		//	summary: 
		//		Enable or disable selection on a node
		//	node:
		//		id or reference to node
		//	selectable:
		//		state to put the node in. false indicates unselectable, true 
		//		allows selection.
		//	example:
		//	Make the node id="bar" unselectable
		//	|	dojo.setSelectable("bar"); 
		//	example:
		//	Make the node id="bar" selectable
		//	|	dojo.setSelectable("bar", true);
		node = byId(node);
				if(d.isMozilla){
			node.style.MozUserSelect = selectable ? "" : "none";
		}else if(d.isKhtml || d.isWebKit){
					node.style.KhtmlUserSelect = selectable ? "auto" : "none";
				}else if(d.isIE){
			var v = (node.unselectable = selectable ? "" : "on");
			d.query("*", node).forEach("item.unselectable = '"+v+"'");
		}
				//FIXME: else?  Opera?
	};

	var _insertBefore = function(/*DomNode*/node, /*DomNode*/ref){
		var parent = ref.parentNode;
		if(parent){
			parent.insertBefore(node, ref);
		}
	};

	var _insertAfter = function(/*DomNode*/node, /*DomNode*/ref){
		//	summary:
		//		Try to insert node after ref
		var parent = ref.parentNode;
		if(parent){
			if(parent.lastChild == ref){
				parent.appendChild(node);
			}else{
				parent.insertBefore(node, ref.nextSibling);
			}
		}
	};

	dojo.place = function(node, refNode, position){
		//	summary:
		//		Attempt to insert node into the DOM, choosing from various positioning options.
		//		Returns the first argument resolved to a DOM node.
		//
		//	node: String|DomNode
		//		id or node reference, or HTML fragment starting with "<" to place relative to refNode
		//
		//	refNode: String|DomNode
		//		id or node reference to use as basis for placement
		//
		//	position: String|Number?
		//		string noting the position of node relative to refNode or a
		//		number indicating the location in the childNodes collection of refNode.
		//		Accepted string values are:
		//	|	* before
		//	|	* after
		//	|	* replace
		//	|	* only
		//	|	* first
		//	|	* last
		//		"first" and "last" indicate positions as children of refNode, "replace" replaces refNode,
		//		"only" replaces all children.  position defaults to "last" if not specified
		//
		//	returns: DomNode
		//		Returned values is the first argument resolved to a DOM node.
		//
		//		.place() is also a method of `dojo.NodeList`, allowing `dojo.query` node lookups.
		//
		// example:
		//		Place a node by string id as the last child of another node by string id:
		//	|	dojo.place("someNode", "anotherNode");
		//
		// example:
		//		Place a node by string id before another node by string id
		//	|	dojo.place("someNode", "anotherNode", "before");
		//
		// example:
		//		Create a Node, and place it in the body element (last child):
		//	|	dojo.place("<div></div>", dojo.body());
		//
		// example:
		//		Put a new LI as the first child of a list by id:
		//	|	dojo.place("<li></li>", "someUl", "first");

		refNode = byId(refNode);
		if(typeof node == "string"){ // inline'd type check
			node = node.charAt(0) == "<" ? d._toDom(node, refNode.ownerDocument) : byId(node);
		}
		if(typeof position == "number"){ // inline'd type check
			var cn = refNode.childNodes;
			if(!cn.length || cn.length <= position){
				refNode.appendChild(node);
			}else{
				_insertBefore(node, cn[position < 0 ? 0 : position]);
			}
		}else{
			switch(position){
				case "before":
					_insertBefore(node, refNode);
					break;
				case "after":
					_insertAfter(node, refNode);
					break;
				case "replace":
					refNode.parentNode.replaceChild(node, refNode);
					break;
				case "only":
					d.empty(refNode);
					refNode.appendChild(node);
					break;
				case "first":
					if(refNode.firstChild){
						_insertBefore(node, refNode.firstChild);
						break;
					}
					// else fallthrough...
				default: // aka: last
					refNode.appendChild(node);
			}
		}
		return node; // DomNode
	}

	// Box functions will assume this model.
	// On IE/Opera, BORDER_BOX will be set if the primary document is in quirks mode.
	// Can be set to change behavior of box setters.

	// can be either:
	//	"border-box"
	//	"content-box" (default)
	dojo.boxModel = "content-box";

	// We punt per-node box mode testing completely.
	// If anybody cares, we can provide an additional (optional) unit 
	// that overrides existing code to include per-node box sensitivity.

	// Opera documentation claims that Opera 9 uses border-box in BackCompat mode.
	// but experiments (Opera 9.10.8679 on Windows Vista) indicate that it actually continues to use content-box.
	// IIRC, earlier versions of Opera did in fact use border-box.
	// Opera guys, this is really confusing. Opera being broken in quirks mode is not our fault.

		if(d.isIE /*|| dojo.isOpera*/){
		// client code may have to adjust if compatMode varies across iframes
		d.boxModel = document.compatMode == "BackCompat" ? "border-box" : "content-box";
	}
	
	// =============================
	// Style Functions
	// =============================

	// getComputedStyle drives most of the style code.
	// Wherever possible, reuse the returned object.
	//
	// API functions below that need to access computed styles accept an 
	// optional computedStyle parameter.
	// If this parameter is omitted, the functions will call getComputedStyle themselves.
	// This way, calling code can access computedStyle once, and then pass the reference to 
	// multiple API functions.

/*=====
	dojo.getComputedStyle = function(node){
		//	summary:
		//		Returns a "computed style" object.
		//
		//	description:
		//		Gets a "computed style" object which can be used to gather
		//		information about the current state of the rendered node.
		//
		//		Note that this may behave differently on different browsers.
		//		Values may have different formats and value encodings across
		//		browsers.
		//
		//		Note also that this method is expensive.  Wherever possible,
		//		reuse the returned object.
		//
		//		Use the dojo.style() method for more consistent (pixelized)
		//		return values.
		//
		//	node: DOMNode
		//		A reference to a DOM node. Does NOT support taking an
		//		ID string for speed reasons.
		//	example:
		//	|	dojo.getComputedStyle(dojo.byId('foo')).borderWidth;
		//
		//	example:
		//	Reusing the returned object, avoiding multiple lookups:
		//	|	var cs = dojo.getComputedStyle(dojo.byId("someNode"));
		//	|	var w = cs.width, h = cs.height;
		return; // CSS2Properties
	}
=====*/

	// Although we normally eschew argument validation at this
	// level, here we test argument 'node' for (duck)type,
	// by testing nodeType, ecause 'document' is the 'parentNode' of 'body'
	// it is frequently sent to this function even 
	// though it is not Element.
	var gcs;
		if(d.isWebKit){
			gcs = function(/*DomNode*/node){
			var s;
			if(node.nodeType == 1){
				var dv = node.ownerDocument.defaultView;
				s = dv.getComputedStyle(node, null);
				if(!s && node.style){
					node.style.display = "";
					s = dv.getComputedStyle(node, null);
				}
			}
			return s || {};
		};
		}else if(d.isIE){
		gcs = function(node){
			// IE (as of 7) doesn't expose Element like sane browsers
			return node.nodeType == 1 /* ELEMENT_NODE*/ ? node.currentStyle : {};
		};
	}else{
		gcs = function(node){
			return node.nodeType == 1 ?
				node.ownerDocument.defaultView.getComputedStyle(node, null) : {};
		};
	}
		dojo.getComputedStyle = gcs;

		if(!d.isIE){
			d._toPixelValue = function(element, value){
			// style values can be floats, client code may want
			// to round for integer pixels.
			return parseFloat(value) || 0;
		};
		}else{
		d._toPixelValue = function(element, avalue){
			if(!avalue){ return 0; }
			// on IE7, medium is usually 4 pixels
			if(avalue == "medium"){ return 4; }
			// style values can be floats, client code may
			// want to round this value for integer pixels.
			if(avalue.slice && avalue.slice(-2) == 'px'){ return parseFloat(avalue); }
			with(element){
				var sLeft = style.left;
				var rsLeft = runtimeStyle.left;
				runtimeStyle.left = currentStyle.left;
				try{
					// 'avalue' may be incompatible with style.left, which can cause IE to throw
					// this has been observed for border widths using "thin", "medium", "thick" constants
					// those particular constants could be trapped by a lookup
					// but perhaps there are more
					style.left = avalue;
					avalue = style.pixelLeft;
				}catch(e){
					avalue = 0;
				}
				style.left = sLeft;
				runtimeStyle.left = rsLeft;
			}
			return avalue;
		}
	}
		var px = d._toPixelValue;

	// FIXME: there opacity quirks on FF that we haven't ported over. Hrm.
	/*=====
	dojo._getOpacity = function(node){
			//	summary:
			//		Returns the current opacity of the passed node as a
			//		floating-point value between 0 and 1.
			//	node: DomNode
			//		a reference to a DOM node. Does NOT support taking an
			//		ID string for speed reasons.
			//	returns: Number between 0 and 1
			return; // Number
	}
	=====*/

		var astr = "DXImageTransform.Microsoft.Alpha";
	var af = function(n, f){
		try{
			return n.filters.item(astr);
		}catch(e){
			return f ? {} : null;
		}
	};

		dojo._getOpacity =
			d.isIE ? function(node){
			try{
				return af(node).Opacity / 100; // Number
			}catch(e){
				return 1; // Number
			}
		} :
			function(node){
			return gcs(node).opacity;
		};

	/*=====
	dojo._setOpacity = function(node, opacity){
			//	summary:
			//		set the opacity of the passed node portably. Returns the
			//		new opacity of the node.
			//	node: DOMNode
			//		a reference to a DOM node. Does NOT support taking an
			//		ID string for performance reasons.
			//	opacity: Number
			//		A Number between 0 and 1. 0 specifies transparent.
			//	returns: Number between 0 and 1
			return; // Number
	}
	=====*/

	dojo._setOpacity =
				d.isIE ? function(/*DomNode*/node, /*Number*/opacity){
			var ov = opacity * 100, opaque = opacity == 1;
			node.style.zoom = opaque ? "" : 1;

			if(!af(node)){
				if(opaque){
					return opacity;
				}
				node.style.filter += " progid:" + astr + "(Opacity=" + ov + ")";
			}else{
				af(node, 1).Opacity = ov;
			}

			// on IE7 Alpha(Filter opacity=100) makes text look fuzzy so disable it altogether (bug #2661),
			//but still update the opacity value so we can get a correct reading if it is read later.
			af(node, 1).Enabled = !opaque;

			if(node.nodeName.toLowerCase() == "tr"){
				d.query("> td", node).forEach(function(i){
					d._setOpacity(i, opacity);
				});
			}
			return opacity;
		} :
				function(node, opacity){
			return node.style.opacity = opacity;
		};

	var _pixelNamesCache = {
		left: true, top: true
	};
	var _pixelRegExp = /margin|padding|width|height|max|min|offset/;  // |border
	var _toStyleValue = function(node, type, value){
		type = type.toLowerCase(); // FIXME: should we really be doing string case conversion here? Should we cache it? Need to profile!
				if(d.isIE){
			if(value == "auto"){
				if(type == "height"){ return node.offsetHeight; }
				if(type == "width"){ return node.offsetWidth; }
			}
			if(type == "fontweight"){
				switch(value){
					case 700: return "bold";
					case 400:
					default: return "normal";
				}
			}
		}
				if(!(type in _pixelNamesCache)){
			_pixelNamesCache[type] = _pixelRegExp.test(type);
		}
		return _pixelNamesCache[type] ? px(node, value) : value;
	};

	var _floatStyle = d.isIE ? "styleFloat" : "cssFloat",
		_floatAliases = { "cssFloat": _floatStyle, "styleFloat": _floatStyle, "float": _floatStyle }
	;

	// public API

	dojo.style = function(	/*DomNode|String*/ node,
							/*String?|Object?*/ style,
							/*String?*/ value){
		//	summary:
		//		Accesses styles on a node. If 2 arguments are
		//		passed, acts as a getter. If 3 arguments are passed, acts
		//		as a setter.
		//	description:
		//		Getting the style value uses the computed style for the node, so the value
		//		will be a calculated value, not just the immediate node.style value.
		//		Also when getting values, use specific style names,
		//		like "borderBottomWidth" instead of "border" since compound values like
		//		"border" are not necessarily reflected as expected.
		//		If you want to get node dimensions, use `dojo.marginBox()`, 
		//		`dojo.contentBox()` or `dojo.position()`.
		//	node:
		//		id or reference to node to get/set style for
		//	style:
		//		the style property to set in DOM-accessor format
		//		("borderWidth", not "border-width") or an object with key/value
		//		pairs suitable for setting each property.
		//	value:
		//		If passed, sets value on the node for style, handling
		//		cross-browser concerns.  When setting a pixel value,
		//		be sure to include "px" in the value. For instance, top: "200px".
		//		Otherwise, in some cases, some browsers will not apply the style.
		//	example:
		//		Passing only an ID or node returns the computed style object of
		//		the node:
		//	|	dojo.style("thinger");
		//	example:
		//		Passing a node and a style property returns the current
		//		normalized, computed value for that property:
		//	|	dojo.style("thinger", "opacity"); // 1 by default
		//
		//	example:
		//		Passing a node, a style property, and a value changes the
		//		current display of the node and returns the new computed value
		//	|	dojo.style("thinger", "opacity", 0.5); // == 0.5
		//
		//	example:
		//		Passing a node, an object-style style property sets each of the values in turn and returns the computed style object of the node:
		//	|	dojo.style("thinger", {
		//	|		"opacity": 0.5,
		//	|		"border": "3px solid black",
		//	|		"height": "300px"
		//	|	});
		//
		// 	example:
		//		When the CSS style property is hyphenated, the JavaScript property is camelCased.
		//		font-size becomes fontSize, and so on.
		//	|	dojo.style("thinger",{
		//	|		fontSize:"14pt",
		//	|		letterSpacing:"1.2em"
		//	|	});
		//
		//	example:
		//		dojo.NodeList implements .style() using the same syntax, omitting the "node" parameter, calling
		//		dojo.style() on every element of the list. See: `dojo.query()` and `dojo.NodeList()`
		//	|	dojo.query(".someClassName").style("visibility","hidden");
		//	|	// or
		//	|	dojo.query("#baz > div").style({
		//	|		opacity:0.75,
		//	|		fontSize:"13pt"
		//	|	});

		var n = byId(node), args = arguments.length, op = (style == "opacity");
		style = _floatAliases[style] || style;
		if(args == 3){
			return op ? d._setOpacity(n, value) : n.style[style] = value; /*Number*/
		}
		if(args == 2 && op){
			return d._getOpacity(n);
		}
		var s = gcs(n);
		if(args == 2 && typeof style != "string"){ // inline'd type check
			for(var x in style){
				d.style(node, x, style[x]);
			}
			return s;
		}
		return (args == 1) ? s : _toStyleValue(n, style, s[style] || n.style[style]); /* CSS2Properties||String||Number */
	}

	// =============================
	// Box Functions
	// =============================

	dojo._getPadExtents = function(/*DomNode*/n, /*Object*/computedStyle){
		//	summary:
		// 		Returns object with special values specifically useful for node
		// 		fitting.
		//	description:
		//		Returns an object with `w`, `h`, `l`, `t` properties:
		//	|		l/t = left/top padding (respectively)
		//	|		w = the total of the left and right padding 
		//	|		h = the total of the top and bottom padding
		//		If 'node' has position, l/t forms the origin for child nodes.
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		var 
			s = computedStyle||gcs(n),
			l = px(n, s.paddingLeft),
			t = px(n, s.paddingTop);
		return {
			l: l,
			t: t,
			w: l+px(n, s.paddingRight),
			h: t+px(n, s.paddingBottom)
		};
	}

	dojo._getBorderExtents = function(/*DomNode*/n, /*Object*/computedStyle){
		//	summary:
		//		returns an object with properties useful for noting the border
		//		dimensions.
		//	description:
		// 		* l/t = the sum of left/top border (respectively)
		//		* w = the sum of the left and right border
		//		* h = the sum of the top and bottom border
		//
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		var 
			ne = "none",
			s = computedStyle||gcs(n),
			bl = (s.borderLeftStyle != ne ? px(n, s.borderLeftWidth) : 0),
			bt = (s.borderTopStyle != ne ? px(n, s.borderTopWidth) : 0);
		return {
			l: bl,
			t: bt,
			w: bl + (s.borderRightStyle!=ne ? px(n, s.borderRightWidth) : 0),
			h: bt + (s.borderBottomStyle!=ne ? px(n, s.borderBottomWidth) : 0)
		};
	}

	dojo._getPadBorderExtents = function(/*DomNode*/n, /*Object*/computedStyle){
		//	summary:
		//		Returns object with properties useful for box fitting with
		//		regards to padding.
		// description:
		//		* l/t = the sum of left/top padding and left/top border (respectively)
		//		* w = the sum of the left and right padding and border
		//		* h = the sum of the top and bottom padding and border
		//
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		var 
			s = computedStyle||gcs(n),
			p = d._getPadExtents(n, s),
			b = d._getBorderExtents(n, s);
		return {
			l: p.l + b.l,
			t: p.t + b.t,
			w: p.w + b.w,
			h: p.h + b.h
		};
	}

	dojo._getMarginExtents = function(n, computedStyle){
		//	summary:
		//		returns object with properties useful for box fitting with
		//		regards to box margins (i.e., the outer-box).
		//
		//		* l/t = marginLeft, marginTop, respectively
		//		* w = total width, margin inclusive
		//		* h = total height, margin inclusive
		//
		//		The w/h are used for calculating boxes.
		//		Normally application code will not need to invoke this
		//		directly, and will use the ...box... functions instead.
		var 
			s = computedStyle||gcs(n),
			l = px(n, s.marginLeft),
			t = px(n, s.marginTop),
			r = px(n, s.marginRight),
			b = px(n, s.marginBottom);
		if(d.isWebKit && (s.position != "absolute")){
			// FIXME: Safari's version of the computed right margin
			// is the space between our right edge and the right edge 
			// of our offsetParent.
			// What we are looking for is the actual margin value as 
			// determined by CSS.
			// Hack solution is to assume left/right margins are the same.
			r = l;
		}
		return {
			l: l,
			t: t,
			w: l+r,
			h: t+b
		};
	}

	// Box getters work in any box context because offsetWidth/clientWidth
	// are invariant wrt box context
	//
	// They do *not* work for display: inline objects that have padding styles
	// because the user agent ignores padding (it's bogus styling in any case)
	//
	// Be careful with IMGs because they are inline or block depending on 
	// browser and browser mode.

	// Although it would be easier to read, there are not separate versions of 
	// _getMarginBox for each browser because:
	// 1. the branching is not expensive
	// 2. factoring the shared code wastes cycles (function call overhead)
	// 3. duplicating the shared code wastes bytes

	dojo._getMarginBox = function(/*DomNode*/node, /*Object*/computedStyle){
		// summary:
		//		returns an object that encodes the width, height, left and top
		//		positions of the node's margin box.
		var s = computedStyle || gcs(node), me = d._getMarginExtents(node, s);
		var l = node.offsetLeft - me.l, t = node.offsetTop - me.t, p = node.parentNode;
				if(d.isMoz){
			// Mozilla:
			// If offsetParent has a computed overflow != visible, the offsetLeft is decreased
			// by the parent's border.
			// We don't want to compute the parent's style, so instead we examine node's
			// computed left/top which is more stable.
			var sl = parseFloat(s.left), st = parseFloat(s.top);
			if(!isNaN(sl) && !isNaN(st)){
				l = sl, t = st;
			}else{
				// If child's computed left/top are not parseable as a number (e.g. "auto"), we
				// have no choice but to examine the parent's computed style.
				if(p && p.style){
					var pcs = gcs(p);
					if(pcs.overflow != "visible"){
						var be = d._getBorderExtents(p, pcs);
						l += be.l, t += be.t;
					}
				}
			}
		}else if(d.isOpera || (d.isIE > 7 && !d.isQuirks)){
			// On Opera and IE 8, offsetLeft/Top includes the parent's border
			if(p){
				be = d._getBorderExtents(p);
				l -= be.l;
				t -= be.t;
			}
		}
				return {
			l: l,
			t: t,
			w: node.offsetWidth + me.w,
			h: node.offsetHeight + me.h 
		};
	}

	dojo._getContentBox = function(node, computedStyle){
		// summary:
		//		Returns an object that encodes the width, height, left and top
		//		positions of the node's content box, irrespective of the
		//		current box model.

		// clientWidth/Height are important since the automatically account for scrollbars
		// fallback to offsetWidth/Height for special cases (see #3378)
		var s = computedStyle || gcs(node),
			pe = d._getPadExtents(node, s),
			be = d._getBorderExtents(node, s),
			w = node.clientWidth,
			h
		;
		if(!w){
			w = node.offsetWidth, h = node.offsetHeight;
		}else{
			h = node.clientHeight, be.w = be.h = 0;
		}
		// On Opera, offsetLeft includes the parent's border
				if(d.isOpera){ pe.l += be.l; pe.t += be.t; };
				return {
			l: pe.l,
			t: pe.t,
			w: w - pe.w - be.w,
			h: h - pe.h - be.h
		};
	}

	dojo._getBorderBox = function(node, computedStyle){
		var s = computedStyle || gcs(node),
			pe = d._getPadExtents(node, s),
			cb = d._getContentBox(node, s)
		;
		return {
			l: cb.l - pe.l,
			t: cb.t - pe.t,
			w: cb.w + pe.w,
			h: cb.h + pe.h
		};
	}

	// Box setters depend on box context because interpretation of width/height styles
	// vary wrt box context.
	//
	// The value of dojo.boxModel is used to determine box context.
	// dojo.boxModel can be set directly to change behavior.
	//
	// Beware of display: inline objects that have padding styles
	// because the user agent ignores padding (it's a bogus setup anyway)
	//
	// Be careful with IMGs because they are inline or block depending on 
	// browser and browser mode.
	//
	// Elements other than DIV may have special quirks, like built-in
	// margins or padding, or values not detectable via computedStyle.
	// In particular, margins on TABLE do not seems to appear 
	// at all in computedStyle on Mozilla.

	dojo._setBox = function(/*DomNode*/node, /*Number?*/l, /*Number?*/t, /*Number?*/w, /*Number?*/h, /*String?*/u){
		//	summary:
		//		sets width/height/left/top in the current (native) box-model
		//		dimentions. Uses the unit passed in u.
		//	node:
		//		DOM Node reference. Id string not supported for performance
		//		reasons.
		//	l:
		//		left offset from parent.
		//	t:
		//		top offset from parent.
		//	w:
		//		width in current box model.
		//	h:
		//		width in current box model.
		//	u:
		//		unit measure to use for other measures. Defaults to "px".
		u = u || "px";
		var s = node.style;
		if(!isNaN(l)){ s.left = l + u; }
		if(!isNaN(t)){ s.top = t + u; }
		if(w >= 0){ s.width = w + u; }
		if(h >= 0){ s.height = h + u; }
	}

	dojo._isButtonTag = function(/*DomNode*/node) {
		// summary:
		//		True if the node is BUTTON or INPUT.type="button".
		return node.tagName == "BUTTON"
			|| node.tagName=="INPUT" && (node.getAttribute("type")||'').toUpperCase() == "BUTTON"; // boolean
	}

	dojo._usesBorderBox = function(/*DomNode*/node){
		//	summary:
		//		True if the node uses border-box layout.

		// We could test the computed style of node to see if a particular box
		// has been specified, but there are details and we choose not to bother.

		// TABLE and BUTTON (and INPUT type=button) are always border-box by default.
		// If you have assigned a different box to either one via CSS then
		// box functions will break.

		var n = node.tagName;
		return d.boxModel=="border-box" || n=="TABLE" || d._isButtonTag(node); // boolean
	}

	dojo._setContentSize = function(/*DomNode*/node, /*Number*/widthPx, /*Number*/heightPx, /*Object*/computedStyle){
		//	summary:
		//		Sets the size of the node's contents, irrespective of margins,
		//		padding, or borders.
		if(d._usesBorderBox(node)){
			var pb = d._getPadBorderExtents(node, computedStyle);
			if(widthPx >= 0){ widthPx += pb.w; }
			if(heightPx >= 0){ heightPx += pb.h; }
		}
		d._setBox(node, NaN, NaN, widthPx, heightPx);
	}

	dojo._setMarginBox = function(/*DomNode*/node, 	/*Number?*/leftPx, /*Number?*/topPx,
													/*Number?*/widthPx, /*Number?*/heightPx,
													/*Object*/computedStyle){
		//	summary:
		//		sets the size of the node's margin box and placement
		//		(left/top), irrespective of box model. Think of it as a
		//		passthrough to dojo._setBox that handles box-model vagaries for
		//		you.

		var s = computedStyle || gcs(node),
		// Some elements have special padding, margin, and box-model settings.
		// To use box functions you may need to set padding, margin explicitly.
		// Controlling box-model is harder, in a pinch you might set dojo.boxModel.
			bb = d._usesBorderBox(node),
			pb = bb ? _nilExtents : d._getPadBorderExtents(node, s)
		;
		if(d.isWebKit){
			// on Safari (3.1.2), button nodes with no explicit size have a default margin
			// setting an explicit size eliminates the margin.
			// We have to swizzle the width to get correct margin reading.
			if(d._isButtonTag(node)){
				var ns = node.style;
				if(widthPx >= 0 && !ns.width) { ns.width = "4px"; }
				if(heightPx >= 0 && !ns.height) { ns.height = "4px"; }
			}
		}
		var mb = d._getMarginExtents(node, s);
		if(widthPx >= 0){ widthPx = Math.max(widthPx - pb.w - mb.w, 0); }
		if(heightPx >= 0){ heightPx = Math.max(heightPx - pb.h - mb.h, 0); }
		d._setBox(node, leftPx, topPx, widthPx, heightPx);
	}

	var _nilExtents = { l:0, t:0, w:0, h:0 };

	// public API

	dojo.marginBox = function(/*DomNode|String*/node, /*Object?*/box){
		//	summary:
		//		Getter/setter for the margin-box of node.
		//	description:
		//		Getter/setter for the margin-box of node.
		//		Returns an object in the expected format of box (regardless
		//		if box is passed). The object might look like:
		//			`{ l: 50, t: 200, w: 300: h: 150 }`
		//		for a node offset from its parent 50px to the left, 200px from
		//		the top with a margin width of 300px and a margin-height of
		//		150px.
		//	node:
		//		id or reference to DOM Node to get/set box for
		//	box:
		//		If passed, denotes that dojo.marginBox() should
		//		update/set the margin box for node. Box is an object in the
		//		above format. All properties are optional if passed.
		//	example:
		//	Retrieve the marginbox of a passed node
		//	|	var box = dojo.marginBox("someNodeId");
		//	|	console.dir(box);
		//
		//	example:
		//	Set a node's marginbox to the size of another node
		//	|	var box = dojo.marginBox("someNodeId");
		//	|	dojo.marginBox("someOtherNode", box);
		
		var n = byId(node), s = gcs(n), b = box;
		return !b ? d._getMarginBox(n, s) : d._setMarginBox(n, b.l, b.t, b.w, b.h, s); // Object
	}

	dojo.contentBox = function(/*DomNode|String*/node, /*Object?*/box){
		//	summary:
		//		Getter/setter for the content-box of node.
		//	description:
		//		Returns an object in the expected format of box (regardless if box is passed).
		//		The object might look like:
		//			`{ l: 50, t: 200, w: 300: h: 150 }`
		//		for a node offset from its parent 50px to the left, 200px from
		//		the top with a content width of 300px and a content-height of
		//		150px. Note that the content box may have a much larger border
		//		or margin box, depending on the box model currently in use and
		//		CSS values set/inherited for node.
		//		While the getter will return top and left values, the
		//		setter only accepts setting the width and height.
		//	node:
		//		id or reference to DOM Node to get/set box for
		//	box:
		//		If passed, denotes that dojo.contentBox() should
		//		update/set the content box for node. Box is an object in the
		//		above format, but only w (width) and h (height) are supported.
		//		All properties are optional if passed.
		var n = byId(node), s = gcs(n), b = box;
		return !b ? d._getContentBox(n, s) : d._setContentSize(n, b.w, b.h, s); // Object
	}

	// =============================
	// Positioning 
	// =============================

	var _sumAncestorProperties = function(node, prop){
		if(!(node = (node||0).parentNode)){return 0}
		var val, retVal = 0, _b = d.body();
		while(node && node.style){
			if(gcs(node).position == "fixed"){
				return 0;
			}
			val = node[prop];
			if(val){
				retVal += val - 0;
				// opera and khtml #body & #html has the same values, we only
				// need one value
				if(node == _b){ break; }
			}
			node = node.parentNode;
		}
		return retVal;	//	integer
	}

	dojo._docScroll = function(){
		var n = d.global;
		return "pageXOffset" in n? { x:n.pageXOffset, y:n.pageYOffset } :
			(n=d.doc.documentElement, n.clientHeight? { x:d._fixIeBiDiScrollLeft(n.scrollLeft), y:n.scrollTop } :
			(n=d.body(), { x:n.scrollLeft||0, y:n.scrollTop||0 }));
	};

	dojo._isBodyLtr = function(){
		return "_bodyLtr" in d? d._bodyLtr :
			d._bodyLtr = (d.body().dir || d.doc.documentElement.dir || "ltr").toLowerCase() == "ltr"; // Boolean 
	}

		dojo._getIeDocumentElementOffset = function(){
		//	summary:
		//		returns the offset in x and y from the document body to the
		//		visual edge of the page
		//	description:
		// The following values in IE contain an offset:
		//	|		event.clientX
		//	|		event.clientY
		//	|		node.getBoundingClientRect().left
		//	|		node.getBoundingClientRect().top
		//	 	But other position related values do not contain this offset,
		//	 	such as node.offsetLeft, node.offsetTop, node.style.left and
		//	 	node.style.top. The offset is always (2, 2) in LTR direction.
		//	 	When the body is in RTL direction, the offset counts the width
		//	 	of left scroll bar's width.  This function computes the actual
		//	 	offset.

		//NOTE: assumes we're being called in an IE browser

		var de = d.doc.documentElement; // only deal with HTML element here, _abs handles body/quirks 

		if(d.isIE < 8){
			var r = de.getBoundingClientRect(); // works well for IE6+
			//console.debug('rect left,top = ' + r.left+','+r.top + ', html client left/top = ' + de.clientLeft+','+de.clientTop + ', rtl = ' + (!d._isBodyLtr()) + ', quirks = ' + d.isQuirks);
			var l = r.left,
			    t = r.top;
			if(d.isIE < 7){
				l += de.clientLeft;	// scrollbar size in strict/RTL, or,
				t += de.clientTop;	// HTML border size in strict
			}
			return {
				x: l < 0? 0 : l, // FRAME element border size can lead to inaccurate negative values
				y: t < 0? 0 : t
			};
		}else{
			return {
				x: 0,
				y: 0
			};
		}

	};
	
	dojo._fixIeBiDiScrollLeft = function(/*Integer*/ scrollLeft){
		// In RTL direction, scrollLeft should be a negative value, but IE < 8
		// returns a positive one. All codes using documentElement.scrollLeft
		// must call this function to fix this error, otherwise the position
		// will offset to right when there is a horizontal scrollbar.

				var dd = d.doc;
		if(d.isIE < 8 && !d._isBodyLtr()){
			var de = d.isQuirks ? dd.body : dd.documentElement;
			return scrollLeft + de.clientWidth - de.scrollWidth; // Integer
		}
				return scrollLeft; // Integer
	}

	// FIXME: need a setter for coords or a moveTo!!
	dojo._abs = dojo.position = function(/*DomNode*/node, /*Boolean?*/includeScroll){
		//	summary:
		//		Gets the position and size of the passed element relative to
		//		the viewport (if includeScroll==false), or relative to the
		//		document root (if includeScroll==true).
		//
		//	description:
		//		Returns an object of the form:
		//			{ x: 100, y: 300, w: 20, h: 15 }
		//		If includeScroll==true, the x and y values will include any
		//		document offsets that may affect the position relative to the
		//		viewport.
		//		Uses the border-box model (inclusive of border and padding but
		//		not margin).  Does not act as a setter.

		var db = d.body(), dh = db.parentNode, ret;
		node = byId(node);
		if(node["getBoundingClientRect"]){
			// IE6+, FF3+, super-modern WebKit, and Opera 9.6+ all take this branch
			ret = node.getBoundingClientRect();
			ret = { x: ret.left, y: ret.top, w: ret.right - ret.left, h: ret.bottom - ret.top };
					if(d.isIE){
				// On IE there's a 2px offset that we need to adjust for, see _getIeDocumentElementOffset()
				var offset = d._getIeDocumentElementOffset();

				// fixes the position in IE, quirks mode
				ret.x -= offset.x + (d.isQuirks ? db.clientLeft+db.offsetLeft : 0);
				ret.y -= offset.y + (d.isQuirks ? db.clientTop+db.offsetTop : 0);
			}else if(d.isFF == 3){
				// In FF3 you have to subtract the document element margins.
				// Fixed in FF3.5 though.
				var cs = gcs(dh);
				ret.x -= px(dh, cs.marginLeft) + px(dh, cs.borderLeftWidth);
				ret.y -= px(dh, cs.marginTop) + px(dh, cs.borderTopWidth);
			}
				}else{
			// FF2 and older WebKit
			ret = {
				x: 0,
				y: 0,
				w: node.offsetWidth,
				h: node.offsetHeight
			};
			if(node["offsetParent"]){
				ret.x -= _sumAncestorProperties(node, "scrollLeft");
				ret.y -= _sumAncestorProperties(node, "scrollTop");

				var curnode = node;
				do{
					var n = curnode.offsetLeft,
						t = curnode.offsetTop;
					ret.x += isNaN(n) ? 0 : n;
					ret.y += isNaN(t) ? 0 : t;

					cs = gcs(curnode);
					if(curnode != node){
								if(d.isMoz){
							// tried left+right with differently sized left/right borders
							// it really is 2xleft border in FF, not left+right, even in RTL!
							ret.x += 2 * px(curnode,cs.borderLeftWidth);
							ret.y += 2 * px(curnode,cs.borderTopWidth);
						}else{
									ret.x += px(curnode, cs.borderLeftWidth);
							ret.y += px(curnode, cs.borderTopWidth);
								}
							}
					// static children in a static div in FF2 are affected by the div's border as well
					// but offsetParent will skip this div!
							if(d.isMoz && cs.position=="static"){
						var parent=curnode.parentNode;
						while(parent!=curnode.offsetParent){
							var pcs=gcs(parent);
							if(pcs.position=="static"){
								ret.x += px(curnode,pcs.borderLeftWidth);
								ret.y += px(curnode,pcs.borderTopWidth);
							}
							parent=parent.parentNode;
						}
					}
							curnode = curnode.offsetParent;
				}while((curnode != dh) && curnode);
			}else if(node.x && node.y){
				ret.x += isNaN(node.x) ? 0 : node.x;
				ret.y += isNaN(node.y) ? 0 : node.y;
			}
		}
		// account for document scrolling
		// if offsetParent is used, ret value already includes scroll position
		// so we may have to actually remove that value if !includeScroll
		if(includeScroll){
			var scroll = d._docScroll();
			ret.x += scroll.x;
			ret.y += scroll.y;
		}

		return ret; // Object
	}

	dojo.coords = function(/*DomNode|String*/node, /*Boolean?*/includeScroll){
		//	summary:
		//		Deprecated: Use position() for border-box x/y/w/h
		//		or marginBox() for margin-box w/h/l/t.
		//		Returns an object representing a node's size and position.
		//
		//	description:
		//		Returns an object that measures margin-box (w)idth/(h)eight
		//		and absolute position x/y of the border-box. Also returned
		//		is computed (l)eft and (t)op values in pixels from the
		//		node's offsetParent as returned from marginBox().
		//		Return value will be in the form:
		//|			{ l: 50, t: 200, w: 300: h: 150, x: 100, y: 300 }
		//		Does not act as a setter. If includeScroll is passed, the x and
		//		y params are affected as one would expect in dojo.position().
		var n = byId(node), s = gcs(n), mb = d._getMarginBox(n, s);
		var abs = d.position(n, includeScroll);
		mb.x = abs.x;
		mb.y = abs.y;
		return mb;
	}

	// =============================
	// Element attribute Functions
	// =============================

	// dojo.attr() should conform to http://www.w3.org/TR/DOM-Level-2-Core/

	var _propNames = {
			// properties renamed to avoid clashes with reserved words
			"class":   "className",
			"for":     "htmlFor",
			// properties written as camelCase
			tabindex:  "tabIndex",
			readonly:  "readOnly",
			colspan:   "colSpan",
			frameborder: "frameBorder",
			rowspan:   "rowSpan",
			valuetype: "valueType"
		},
		_attrNames = {
			// original attribute names
			classname: "class",
			htmlfor:   "for",
			// for IE
			tabindex:  "tabIndex",
			readonly:  "readOnly"
		},
		_forcePropNames = {
			innerHTML: 1,
			className: 1,
			htmlFor:   d.isIE,
			value:     1
		};

	var _fixAttrName = function(/*String*/ name){
		return _attrNames[name.toLowerCase()] || name;
	};

	var _hasAttr = function(node, name){
		var attr = node.getAttributeNode && node.getAttributeNode(name);
		return attr && attr.specified; // Boolean
	};

	// There is a difference in the presence of certain properties and their default values
	// between browsers. For example, on IE "disabled" is present on all elements,
	// but it is value is "false"; "tabIndex" of <div> returns 0 by default on IE, yet other browsers
	// can return -1.

	dojo.hasAttr = function(/*DomNode|String*/node, /*String*/name){
		//	summary:
		//		Returns true if the requested attribute is specified on the
		//		given element, and false otherwise.
		//	node:
		//		id or reference to the element to check
		//	name:
		//		the name of the attribute
		//	returns:
		//		true if the requested attribute is specified on the
		//		given element, and false otherwise
		var lc = name.toLowerCase();
		return _forcePropNames[_propNames[lc] || name] || _hasAttr(byId(node), _attrNames[lc] || name);	// Boolean
	}

	var _evtHdlrMap = {}, _ctr = 0,
		_attrId = dojo._scopeName + "attrid",
		// the next dictionary lists elements with read-only innerHTML on IE
		_roInnerHtml = {col: 1, colgroup: 1,
			// frameset: 1, head: 1, html: 1, style: 1,
			table: 1, tbody: 1, tfoot: 1, thead: 1, tr: 1, title: 1};

	dojo.attr = function(/*DomNode|String*/node, /*String|Object*/name, /*String?*/value){
		//	summary:
		//		Gets or sets an attribute on an HTML element.
		//	description:
		//		Handles normalized getting and setting of attributes on DOM
		//		Nodes. If 2 arguments are passed, and a the second argumnt is a
		//		string, acts as a getter.
		//
		//		If a third argument is passed, or if the second argument is a
		//		map of attributes, acts as a setter.
		//
		//		When passing functions as values, note that they will not be
		//		directly assigned to slots on the node, but rather the default
		//		behavior will be removed and the new behavior will be added
		//		using `dojo.connect()`, meaning that event handler properties
		//		will be normalized and that some caveats with regards to
		//		non-standard behaviors for onsubmit apply. Namely that you
		//		should cancel form submission using `dojo.stopEvent()` on the
		//		passed event object instead of returning a boolean value from
		//		the handler itself.
		//	node:
		//		id or reference to the element to get or set the attribute on
		//	name:
		//		the name of the attribute to get or set.
		//	value:
		//		The value to set for the attribute
		//	returns:
		//		when used as a getter, the value of the requested attribute
		//		or null if that attribute does not have a specified or
		//		default value;
		//
		//		when used as a setter, the DOM node
		//
		//	example:
		//	|	// get the current value of the "foo" attribute on a node
		//	|	dojo.attr(dojo.byId("nodeId"), "foo");
		//	|	// or we can just pass the id:
		//	|	dojo.attr("nodeId", "foo");
		//
		//	example:
		//	|	// use attr() to set the tab index
		//	|	dojo.attr("nodeId", "tabIndex", 3);
		//	|
		//
		//	example:
		//	Set multiple values at once, including event handlers:
		//	|	dojo.attr("formId", {
		//	|		"foo": "bar",
		//	|		"tabIndex": -1,
		//	|		"method": "POST",
		//	|		"onsubmit": function(e){
		//	|			// stop submitting the form. Note that the IE behavior
		//	|			// of returning true or false will have no effect here
		//	|			// since our handler is connect()ed to the built-in
		//	|			// onsubmit behavior and so we need to use
		//	|			// dojo.stopEvent() to ensure that the submission
		//	|			// doesn't proceed.
		//	|			dojo.stopEvent(e);
		//	|
		//	|			// submit the form with Ajax
		//	|			dojo.xhrPost({ form: "formId" });
		//	|		}
		//	|	});
		//
		//	example:
		//	Style is s special case: Only set with an object hash of styles
		//	|	dojo.attr("someNode",{
		//	|		id:"bar",
		//	|		style:{
		//	|			width:"200px", height:"100px", color:"#000"
		//	|		}
		//	|	});
		//
		//	example:
		//	Again, only set style as an object hash of styles:
		//	|	var obj = { color:"#fff", backgroundColor:"#000" };
		//	|	dojo.attr("someNode", "style", obj);
		//	|
		//	|	// though shorter to use `dojo.style()` in this case:
		//	|	dojo.style("someNode", obj);

		node = byId(node);
		var args = arguments.length, prop;
		if(args == 2 && typeof name != "string"){ // inline'd type check
			// the object form of setter: the 2nd argument is a dictionary
			for(var x in name){
				d.attr(node, x, name[x]);
			}
			return node; // DomNode
		}
		var lc = name.toLowerCase(),
			propName = _propNames[lc] || name,
			forceProp = _forcePropNames[propName],
			attrName = _attrNames[lc] || name;
		if(args == 3){
			// setter
			do{
				if(propName == "style" && typeof value != "string"){ // inline'd type check
					// special case: setting a style
					d.style(node, value);
					break;
				}
				if(propName == "innerHTML"){
					// special case: assigning HTML
										if(d.isIE && node.tagName.toLowerCase() in _roInnerHtml){
						d.empty(node);
						node.appendChild(d._toDom(value, node.ownerDocument));
					}else{
											node[propName] = value;
										}
										break;
				}
				if(d.isFunction(value)){
					// special case: assigning an event handler
					// clobber if we can
					var attrId = d.attr(node, _attrId);
					if(!attrId){
						attrId = _ctr++;
						d.attr(node, _attrId, attrId);
					}
					if(!_evtHdlrMap[attrId]){
						_evtHdlrMap[attrId] = {};
					}
					var h = _evtHdlrMap[attrId][propName];
					if(h){
						d.disconnect(h);
					}else{
						try{
							delete node[propName];
						}catch(e){}
					}
					// ensure that event objects are normalized, etc.
					_evtHdlrMap[attrId][propName] = d.connect(node, propName, value);
					break;
				}
				if(forceProp || typeof value == "boolean"){
					// special case: forcing assignment to the property
					// special case: setting boolean to a property instead of attribute
					node[propName] = value;
					break;
				}
				// node's attribute
				node.setAttribute(attrName, value);
			}while(false);
			return node; // DomNode
		}
		// getter
		// should we access this attribute via a property or
		// via getAttribute()?
		value = node[propName];
		if(forceProp && typeof value != "undefined"){
			// node's property
			return value;	// Anything
		}
		if(propName != "href" && (typeof value == "boolean" || d.isFunction(value))){
			// node's property
			return value;	// Anything
		}
		// node's attribute
		// we need _hasAttr() here to guard against IE returning a default value
		return _hasAttr(node, attrName) ? node.getAttribute(attrName) : null; // Anything
	}

	dojo.removeAttr = function(/*DomNode|String*/ node, /*String*/ name){
		//	summary:
		//		Removes an attribute from an HTML element.
		//	node:
		//		id or reference to the element to remove the attribute from
		//	name:
		//		the name of the attribute to remove
		byId(node).removeAttribute(_fixAttrName(name));
	}

	dojo.getNodeProp = function(/*DomNode|String*/ node, /*String*/ name){
		//	summary:
		//		Returns an effective value of a property or an attribute.
		//	node:
		//		id or reference to the element to remove the attribute from
		//	name:
		//		the name of the attribute
		node = byId(node);
		var lc = name.toLowerCase(),
			propName = _propNames[lc] || name;
		if((propName in node) && propName != "href"){
			// node's property
			return node[propName];	// Anything
		}
		// node's attribute
		var attrName = _attrNames[lc] || name;
		return _hasAttr(node, attrName) ? node.getAttribute(attrName) : null; // Anything
	}

	dojo.create = function(tag, attrs, refNode, pos){
		//	summary:
		//		Create an element, allowing for optional attribute decoration
		//		and placement.
		//
		// description:
		//		A DOM Element creation function. A shorthand method for creating a node or
		//		a fragment, and allowing for a convenient optional attribute setting step,
		//		as well as an optional DOM placement reference.
		//|
		//		Attributes are set by passing the optional object through `dojo.attr`.
		//		See `dojo.attr` for noted caveats and nuances, and API if applicable.
		//|
		//		Placement is done via `dojo.place`, assuming the new node to be the action 
		//		node, passing along the optional reference node and position.
		//
		// tag: String|DomNode
		//		A string of the element to create (eg: "div", "a", "p", "li", "script", "br"),
		//		or an existing DOM node to process.
		//
		// attrs: Object
		//		An object-hash of attributes to set on the newly created node.
		//		Can be null, if you don't want to set any attributes/styles.
		//		See: `dojo.attr` for a description of available attributes.
		//
		// refNode: String?|DomNode?
		//		Optional reference node. Used by `dojo.place` to place the newly created
		//		node somewhere in the dom relative to refNode. Can be a DomNode reference
		//		or String ID of a node.
		//
		// pos: String?
		//		Optional positional reference. Defaults to "last" by way of `dojo.place`,
		//		though can be set to "first","after","before","last", "replace" or "only"
		//		to further control the placement of the new node relative to the refNode.
		//		'refNode' is required if a 'pos' is specified.
		//
		// returns: DomNode
		//
		// example:
		//	Create a DIV:
		//	|	var n = dojo.create("div");
		//
		// example:
		//	Create a DIV with content:
		//	|	var n = dojo.create("div", { innerHTML:"<p>hi</p>" });
		//
		// example:
		//	Place a new DIV in the BODY, with no attributes set
		//	|	var n = dojo.create("div", null, dojo.body());
		//
		// example:
		//	Create an UL, and populate it with LI's. Place the list as the first-child of a 
		//	node with id="someId":
		//	|	var ul = dojo.create("ul", null, "someId", "first");
		//	|	var items = ["one", "two", "three", "four"];
		//	|	dojo.forEach(items, function(data){
		//	|		dojo.create("li", { innerHTML: data }, ul);
		//	|	});
		//
		// example:
		//	Create an anchor, with an href. Place in BODY:
		//	|	dojo.create("a", { href:"foo.html", title:"Goto FOO!" }, dojo.body());
		//
		// example:
		//	Create a `dojo.NodeList()` from a new element (for syntatic sugar):
		//	|	dojo.query(dojo.create('div'))
		//	|		.addClass("newDiv")
		//	|		.onclick(function(e){ console.log('clicked', e.target) })
		//	|		.place("#someNode"); // redundant, but cleaner.

		var doc = d.doc;
		if(refNode){
			refNode = byId(refNode);
			doc = refNode.ownerDocument;
		}
		if(typeof tag == "string"){ // inline'd type check
			tag = doc.createElement(tag);
		}
		if(attrs){ d.attr(tag, attrs); }
		if(refNode){ d.place(tag, refNode, pos); }
		return tag; // DomNode
	}

	/*=====
	dojo.empty = function(node){
			//	summary:
			//		safely removes all children of the node.
			//	node: DOMNode|String
			//		a reference to a DOM node or an id.
			//	example:
			//	Destroy node's children byId:
			//	|	dojo.empty("someId");
			//
			//	example:
			//	Destroy all nodes' children in a list by reference:
			//	|	dojo.query(".someNode").forEach(dojo.empty);
	}
	=====*/

	d.empty =
				d.isIE ?  function(node){
			node = byId(node);
			for(var c; c = node.lastChild;){ // intentional assignment
				d.destroy(c);
			}
		} :
				function(node){
			byId(node).innerHTML = "";
		};

	/*=====
	dojo._toDom = function(frag, doc){
			//	summary:
			//		instantiates an HTML fragment returning the corresponding DOM.
			//	frag: String
			//		the HTML fragment
			//	doc: DocumentNode?
			//		optional document to use when creating DOM nodes, defaults to
			//		dojo.doc if not specified.
			//	returns: DocumentFragment
			//
			//	example:
			//	Create a table row:
			//	|	var tr = dojo._toDom("<tr><td>First!</td></tr>");
	}
	=====*/

	// support stuff for dojo._toDom
	var tagWrap = {
			option: ["select"],
			tbody: ["table"],
			thead: ["table"],
			tfoot: ["table"],
			tr: ["table", "tbody"],
			td: ["table", "tbody", "tr"],
			th: ["table", "thead", "tr"],
			legend: ["fieldset"],
			caption: ["table"],
			colgroup: ["table"],
			col: ["table", "colgroup"],
			li: ["ul"]
		},
		reTag = /<\s*([\w\:]+)/,
		masterNode = {}, masterNum = 0,
		masterName = "__" + d._scopeName + "ToDomId";

	// generate start/end tag strings to use
	// for the injection for each special tag wrap case.
	for(var param in tagWrap){
		var tw = tagWrap[param];
		tw.pre  = param == "option" ? '<select multiple="multiple">' : "<" + tw.join("><") + ">";
		tw.post = "</" + tw.reverse().join("></") + ">";
		// the last line is destructive: it reverses the array,
		// but we don't care at this point
	}

	d._toDom = function(frag, doc){
		//	summary:
		// 		converts HTML string into DOM nodes.

		doc = doc || d.doc;
		var masterId = doc[masterName];
		if(!masterId){
			doc[masterName] = masterId = ++masterNum + "";
			masterNode[masterId] = doc.createElement("div");
		}

		// make sure the frag is a string.
		frag += "";

		// find the starting tag, and get node wrapper
		var match = frag.match(reTag),
			tag = match ? match[1].toLowerCase() : "",
			master = masterNode[masterId],
			wrap, i, fc, df;
		if(match && tagWrap[tag]){
			wrap = tagWrap[tag];
			master.innerHTML = wrap.pre + frag + wrap.post;
			for(i = wrap.length; i; --i){
				master = master.firstChild;
			}
		}else{
			master.innerHTML = frag;
		}

		// one node shortcut => return the node itself
		if(master.childNodes.length == 1){
			return master.removeChild(master.firstChild); // DOMNode
		}

		// return multiple nodes as a document fragment
		df = doc.createDocumentFragment();
		while(fc = master.firstChild){ // intentional assignment
			df.appendChild(fc);
		}
		return df; // DOMNode
	}

	// =============================
	// (CSS) Class Functions
	// =============================
	var _className = "className";

	dojo.hasClass = function(/*DomNode|String*/node, /*String*/classStr){
		//	summary:
		//		Returns whether or not the specified classes are a portion of the
		//		class list currently applied to the node.
		//
		//	node:
		//		String ID or DomNode reference to check the class for.
		//
		//	classStr:
		//		A string class name to look for.
		//
		//	example:
		//	Do something if a node with id="someNode" has class="aSillyClassName" present
		//	|	if(dojo.hasClass("someNode","aSillyClassName")){ ... }

		return ((" "+ byId(node)[_className] +" ").indexOf(" " + classStr + " ") >= 0);  // Boolean
	};

	var spaces = /\s+/, a1 = [""],
		str2array = function(s){
			if(typeof s == "string" || s instanceof String){
				if(s.indexOf(" ") < 0){
					a1[0] = s;
					return a1;
				}else{
					return s.split(spaces);
				}
			}
			// assumed to be an array
			return s || "";
		};

	dojo.addClass = function(/*DomNode|String*/node, /*String|Array*/classStr){
		//	summary:
		//		Adds the specified classes to the end of the class list on the
		//		passed node. Will not re-apply duplicate classes.
		//
		//	node:
		//		String ID or DomNode reference to add a class string too
		//
		//	classStr:
		//		A String class name to add, or several space-separated class names,
		//		or an array of class names.
		//
		// example:
		//	Add a class to some node:
		//	|	dojo.addClass("someNode", "anewClass");
		//
		// example:
		//	Add two classes at once:
		//	|	dojo.addClass("someNode", "firstClass secondClass");
		//
		// example:
		//	Add two classes at once (using array):
		//	|	dojo.addClass("someNode", ["firstClass", "secondClass"]);
		//
		// example:
		//	Available in `dojo.NodeList` for multiple additions
		//	|	dojo.query("ul > li").addClass("firstLevel");

		node = byId(node);
		classStr = str2array(classStr);
		var cls = node[_className], oldLen;
		cls = cls ? " " + cls + " " : " ";
		oldLen = cls.length;
		for(var i = 0, len = classStr.length, c; i < len; ++i){
			c = classStr[i];
			if(c && cls.indexOf(" " + c + " ") < 0){
				cls += c + " ";
			}
		}
		if(oldLen < cls.length){
			node[_className] = cls.substr(1, cls.length - 2);
		}
	};

	dojo.removeClass = function(/*DomNode|String*/node, /*String|Array?*/classStr){
		// summary:
		//		Removes the specified classes from node. No `dojo.hasClass`
		//		check is required.
		//
		// node:
		// 		String ID or DomNode reference to remove the class from.
		//
		// classStr:
		//		An optional String class name to remove, or several space-separated
		//		class names, or an array of class names. If omitted, all class names
		//		will be deleted.
		//
		// example:
		//	Remove a class from some node:
		//	|	dojo.removeClass("someNode", "firstClass");
		//
		// example:
		//	Remove two classes from some node:
		//	|	dojo.removeClass("someNode", "firstClass secondClass");
		//
		// example:
		//	Remove two classes from some node (using array):
		//	|	dojo.removeClass("someNode", ["firstClass", "secondClass"]);
		//
		// example:
		//	Remove all classes from some node:
		//	|	dojo.removeClass("someNode");
		//
		// example:
		//	Available in `dojo.NodeList()` for multiple removal
		//	|	dojo.query(".foo").removeClass("foo");

		node = byId(node);
		var cls;
		if(classStr !== undefined){
			classStr = str2array(classStr);
			cls = " " + node[_className] + " ";
			for(var i = 0, len = classStr.length; i < len; ++i){
				cls = cls.replace(" " + classStr[i] + " ", " ");
			}
			cls = d.trim(cls);
		}else{
			cls = "";
		}
		if(node[_className] != cls){ node[_className] = cls; }
	};

	dojo.toggleClass = function(/*DomNode|String*/node, /*String|Array*/classStr, /*Boolean?*/condition){
		//	summary:
		//		Adds a class to node if not present, or removes if present.
		//		Pass a boolean condition if you want to explicitly add or remove.
		//	condition:
		//		If passed, true means to add the class, false means to remove.
		//
		// example:
		//	|	dojo.toggleClass("someNode", "hovered");
		//
		// example:
		//	Forcefully add a class
		//	|	dojo.toggleClass("someNode", "hovered", true);
		//
		// example:
		//	Available in `dojo.NodeList()` for multiple toggles
		//	|	dojo.query(".toggleMe").toggleClass("toggleMe");

		if(condition === undefined){
			condition = !d.hasClass(node, classStr);
		}
		d[condition ? "addClass" : "removeClass"](node, classStr);
	};

})();

}

if(!dojo._hasResource["dojo._base.NodeList"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.NodeList"] = true;
dojo.provide("dojo._base.NodeList");



(function(){

	var d = dojo;

	var ap = Array.prototype, aps = ap.slice, apc = ap.concat;

	var tnl = function(/*Array*/ a, /*dojo.NodeList?*/ parent, /*Function?*/ NodeListCtor){
		// summary:
		// 		decorate an array to make it look like a `dojo.NodeList`.
		// a:
		// 		Array of nodes to decorate.
		// parent:
		// 		An optional parent NodeList that generated the current
		// 		list of nodes. Used to call _stash() so the parent NodeList
		// 		can be accessed via end() later.
		// NodeListCtor:
		// 		An optional constructor function to use for any
		// 		new NodeList calls. This allows a certain chain of
		// 		NodeList calls to use a different object than dojo.NodeList.
		if(!a.sort){
			// make sure it's a real array before we pass it on to be wrapped
			a = aps.call(a, 0);
		}
		var ctor = NodeListCtor || this._NodeListCtor || d._NodeListCtor;
		a.constructor = ctor;
		dojo._mixin(a, ctor.prototype);
		a._NodeListCtor = ctor;
		return parent ? a._stash(parent) : a;
	};

	var loopBody = function(f, a, o){
		a = [0].concat(aps.call(a, 0));
		o = o || d.global;
		return function(node){
			a[0] = node;
			return f.apply(o, a);
		};
	};

	// adapters

	var adaptAsForEach = function(f, o){
		//	summary:
		//		adapts a single node function to be used in the forEach-type
		//		actions. The initial object is returned from the specialized
		//		function.
		//	f: Function
		//		a function to adapt
		//	o: Object?
		//		an optional context for f
		return function(){
			this.forEach(loopBody(f, arguments, o));
			return this;	// Object
		};
	};

	var adaptAsMap = function(f, o){
		//	summary:
		//		adapts a single node function to be used in the map-type
		//		actions. The return is a new array of values, as via `dojo.map`
		//	f: Function
		//		a function to adapt
		//	o: Object?
		//		an optional context for f
		return function(){
			return this.map(loopBody(f, arguments, o));
		};
	};

	var adaptAsFilter = function(f, o){
		//	summary:
		//		adapts a single node function to be used in the filter-type actions
		//	f: Function
		//		a function to adapt
		//	o: Object?
		//		an optional context for f
		return function(){
			return this.filter(loopBody(f, arguments, o));
		};
	};

	var adaptWithCondition = function(f, g, o){
		//	summary:
		//		adapts a single node function to be used in the map-type
		//		actions, behaves like forEach() or map() depending on arguments
		//	f: Function
		//		a function to adapt
		//	g: Function
		//		a condition function, if true runs as map(), otherwise runs as forEach()
		//	o: Object?
		//		an optional context for f and g
		return function(){
			var a = arguments, body = loopBody(f, a, o);
			if(g.call(o || d.global, a)){
				return this.map(body);	// self
			}
			this.forEach(body);
			return this;	// self
		};
	};

	var magicGuard = function(a){
		//	summary:
		//		the guard function for dojo.attr() and dojo.style()
		return a.length == 1 && (typeof a[0] == "string"); // inline'd type check
	};

	var orphan = function(node){
		//	summary:
		//		function to orphan nodes
		var p = node.parentNode;
		if(p){
			p.removeChild(node);
		}
	};
	// FIXME: should we move orphan() to dojo.html?

	dojo.NodeList = function(){
		//	summary:
		//		dojo.NodeList is an of Array subclass which adds syntactic
		//		sugar for chaining, common iteration operations, animation, and
		//		node manipulation. NodeLists are most often returned as the
		//		result of dojo.query() calls.
		//	description:
		//		dojo.NodeList instances provide many utilities that reflect
		//		core Dojo APIs for Array iteration and manipulation, DOM
		//		manipulation, and event handling. Instead of needing to dig up
		//		functions in the dojo.* namespace, NodeLists generally make the
		//		full power of Dojo available for DOM manipulation tasks in a
		//		simple, chainable way.
		//	example:
		//		create a node list from a node
		//		|	new dojo.NodeList(dojo.byId("foo"));
		//	example:
		//		get a NodeList from a CSS query and iterate on it
		//		|	var l = dojo.query(".thinger");
		//		|	l.forEach(function(node, index, nodeList){
		//		|		console.log(index, node.innerHTML);
		//		|	});
		//	example:
		//		use native and Dojo-provided array methods to manipulate a
		//		NodeList without needing to use dojo.* functions explicitly:
		//		|	var l = dojo.query(".thinger");
		//		|	// since NodeLists are real arrays, they have a length
		//		|	// property that is both readable and writable and
		//		|	// push/pop/shift/unshift methods
		//		|	console.log(l.length);
		//		|	l.push(dojo.create("span"));
		//		|
		//		|	// dojo's normalized array methods work too:
		//		|	console.log( l.indexOf(dojo.byId("foo")) );
		//		|	// ...including the special "function as string" shorthand
		//		|	console.log( l.every("item.nodeType == 1") );
		//		|
		//		|	// NodeLists can be [..] indexed, or you can use the at()
		//		|	// function to get specific items wrapped in a new NodeList:
		//		|	var node = l[3]; // the 4th element
		//		|	var newList = l.at(1, 3); // the 2nd and 4th elements
		//	example:
		//		the style functions you expect are all there too:
		//		|	// style() as a getter...
		//		|	var borders = dojo.query(".thinger").style("border");
		//		|	// ...and as a setter:
		//		|	dojo.query(".thinger").style("border", "1px solid black");
		//		|	// class manipulation
		//		|	dojo.query("li:nth-child(even)").addClass("even");
		//		|	// even getting the coordinates of all the items
		//		|	var coords = dojo.query(".thinger").coords();
		//	example:
		//		DOM manipulation functions from the dojo.* namespace area also
		//		available:
		//		|	// remove all of the elements in the list from their
		//		|	// parents (akin to "deleting" them from the document)
		//		|	dojo.query(".thinger").orphan();
		//		|	// place all elements in the list at the front of #foo
		//		|	dojo.query(".thinger").place("foo", "first");
		//	example:
		//		Event handling couldn't be easier. `dojo.connect` is mapped in,
		//		and shortcut handlers are provided for most DOM events:
		//		|	// like dojo.connect(), but with implicit scope
		//		|	dojo.query("li").connect("onclick", console, "log");
		//		|
		//		|	// many common event handlers are already available directly:
		//		|	dojo.query("li").onclick(console, "log");
		//		|	var toggleHovered = dojo.hitch(dojo, "toggleClass", "hovered");
		//		|	dojo.query("p")
		//		|		.onmouseenter(toggleHovered)
		//		|		.onmouseleave(toggleHovered);
		//	example:
		//		chainability is a key advantage of NodeLists:
		//		|	dojo.query(".thinger")
		//		|		.onclick(function(e){ /* ... */ })
		//		|		.at(1, 3, 8) // get a subset
		//		|			.style("padding", "5px")
		//		|			.forEach(console.log);

		return tnl(Array.apply(null, arguments));
	};

	//Allow things that new up a NodeList to use a delegated or alternate NodeList implementation.
	d._NodeListCtor = d.NodeList;

	var nl = d.NodeList, nlp = nl.prototype;

	// expose adapters and the wrapper as private functions

	nl._wrap = nlp._wrap = tnl;
	nl._adaptAsMap = adaptAsMap;
	nl._adaptAsForEach = adaptAsForEach;
	nl._adaptAsFilter  = adaptAsFilter;
	nl._adaptWithCondition = adaptWithCondition;

	// mass assignment

	// add array redirectors
	d.forEach(["slice", "splice"], function(name){
		var f = ap[name];
		//Use a copy of the this array via this.slice() to allow .end() to work right in the splice case.
		// CANNOT apply ._stash()/end() to splice since it currently modifies
		// the existing this array -- it would break backward compatibility if we copy the array before
		// the splice so that we can use .end(). So only doing the stash option to this._wrap for slice.
		nlp[name] = function(){ return this._wrap(f.apply(this, arguments), name == "slice" ? this : null); };
	});
	// concat should be here but some browsers with native NodeList have problems with it

	// add array.js redirectors
	d.forEach(["indexOf", "lastIndexOf", "every", "some"], function(name){
		var f = d[name];
		nlp[name] = function(){ return f.apply(d, [this].concat(aps.call(arguments, 0))); };
	});

	// add conditional methods
	d.forEach(["attr", "style"], function(name){
		nlp[name] = adaptWithCondition(d[name], magicGuard);
	});

	// add forEach actions
	d.forEach(["connect", "addClass", "removeClass", "toggleClass", "empty", "removeAttr"], function(name){
		nlp[name] = adaptAsForEach(d[name]);
	});

	dojo.extend(dojo.NodeList, {
		_normalize: function(/*String||Element||Object||NodeList*/content, /*DOMNode?*/refNode){
			// summary:
			// 		normalizes data to an array of items to insert.
			// description:
			// 		If content is an object, it can have special properties "template" and
			// 		"parse". If "template" is defined, then the template value is run through
			// 		dojo.string.substitute (if dojo.string.substitute has been dojo.required elsewhere),
			// 		or if templateFunc is a function on the content, that function will be used to
			// 		transform the template into a final string to be used for for passing to dojo._toDom.
			// 		If content.parse is true, then it is remembered for later, for when the content
			// 		nodes are inserted into the DOM. At that point, the nodes will be parsed for widgets
			// 		(if dojo.parser has been dojo.required elsewhere).

			//Wanted to just use a DocumentFragment, but for the array/NodeList
			//case that meant  using cloneNode, but we may not want that.
			//Cloning should only happen if the node operations span
			//multiple refNodes. Also, need a real array, not a NodeList from the
			//DOM since the node movements could change those NodeLists.

			var parse = content.parse === true ? true : false;

			//Do we have an object that needs to be run through a template?
			if(typeof content.template == "string"){
				var templateFunc = content.templateFunc || (dojo.string && dojo.string.substitute);
				content = templateFunc ? templateFunc(content.template, content) : content;
			}

			var type = (typeof content);
			if(type == "string" || type == "number"){
				content = dojo._toDom(content, (refNode && refNode.ownerDocument));
				if(content.nodeType == 11){
					//DocumentFragment. It cannot handle cloneNode calls, so pull out the children.
					content = dojo._toArray(content.childNodes);
				}else{
					content = [content];
				}
			}else if(!dojo.isArrayLike(content)){
				content = [content];
			}else if(!dojo.isArray(content)){
				//To get to this point, content is array-like, but
				//not an array, which likely means a DOM NodeList. Convert it now.
				content = dojo._toArray(content);
			}

			//Pass around the parse info
			if(parse){
				content._runParse = true;
			}
			return content; //Array
		},

		_cloneNode: function(/*DOMNode*/ node){
			// summary:
			// 		private utiltity to clone a node. Not very interesting in the vanilla
			// 		dojo.NodeList case, but delegates could do interesting things like
			// 		clone event handlers if that is derivable from the node.
			return node.cloneNode(true);
		},

		_place: function(/*Array*/ary, /*DOMNode*/refNode, /*String*/position, /*Boolean*/useClone){
			// summary:
			// 		private utility to handle placing an array of nodes relative to another node.
			// description:
			// 		Allows for cloning the nodes in the array, and for
			// 		optionally parsing widgets, if ary._runParse is true.

			//Avoid a disallowed operation if trying to do an innerHTML on a non-element node.
			if(refNode.nodeType != 1 && position == "only"){
				return;
			}
			var rNode = refNode, tempNode;

			//Always cycle backwards in case the array is really a
			//DOM NodeList and the DOM operations take it out of the live collection.
			var length = ary.length;
			for(var i = length - 1; i >= 0; i--){
				var node = (useClone ? this._cloneNode(ary[i]) : ary[i]);

				//If need widget parsing, use a temp node, instead of waiting after inserting into
				//real DOM because we need to start widget parsing at one node up from current node,
				//which could cause some already parsed widgets to be parsed again.
				if(ary._runParse && dojo.parser && dojo.parser.parse){
					if(!tempNode){
						tempNode = rNode.ownerDocument.createElement("div");
					}
					tempNode.appendChild(node);
					dojo.parser.parse(tempNode);
					node = tempNode.firstChild;
					while(tempNode.firstChild){
						tempNode.removeChild(tempNode.firstChild);
					}
				}

				if(i == length - 1){
					dojo.place(node, rNode, position);
				}else{
					rNode.parentNode.insertBefore(node, rNode);
				}
				rNode = node;
			}
		},

		_stash: function(parent){
			// summary:
			// 		private function to hold to a parent NodeList. end() to return the parent NodeList.
			//
			// example:
			// How to make a `dojo.NodeList` method that only returns the third node in
			// the dojo.NodeList but allows access to the original NodeList by using this._stash:
			//	|	dojo.extend(dojo.NodeList, {
			//	|		third: function(){
			//  |			var newNodeList = dojo.NodeList(this[2]);
			//	|			return newNodeList._stash(this);
			//	|		}
			//	|	});
			//	|	// then see how _stash applies a sub-list, to be .end()'ed out of
			//	|	dojo.query(".foo")
			//	|		.third()
			//	|			.addClass("thirdFoo")
			//	|		.end()
			//	|		// access to the orig .foo list
			//	|		.removeClass("foo")
			//	|
			//
			this._parent = parent;
			return this; //dojo.NodeList
		},

		end: function(){
			// summary:
			// 		Ends use of the current `dojo.NodeList` by returning the previous dojo.NodeList
			// 		that generated the current dojo.NodeList.
			// description:
			// 		Returns the `dojo.NodeList` that generated the current `dojo.NodeList`. If there
			// 		is no parent dojo.NodeList, an empty dojo.NodeList is returned.
			// example:
			//	|	dojo.query("a")
			//	|		.filter(".disabled")
			//	|			// operate on the anchors that only have a disabled class
			//	|			.style("color", "grey")
			//	|		.end()
			//	|		// jump back to the list of anchors
			//	|		.style(...)
			//
			if(this._parent){
				return this._parent;
			}else{
				//Just return empy list.
				return new this._NodeListCtor();
			}
		},

		// http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array#Methods

		// FIXME: handle return values for #3244
		//		http://trac.dojotoolkit.org/ticket/3244

		// FIXME:
		//		need to wrap or implement:
		//			join (perhaps w/ innerHTML/outerHTML overload for toString() of items?)
		//			reduce
		//			reduceRight

		/*=====
		slice: function(begin, end){
			// summary:
			//		Returns a new NodeList, maintaining this one in place
			// description:
			//		This method behaves exactly like the Array.slice method
			//		with the caveat that it returns a dojo.NodeList and not a
			//		raw Array. For more details, see Mozilla's (slice
			//		documentation)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:slice]
			// begin: Integer
			//		Can be a positive or negative integer, with positive
			//		integers noting the offset to begin at, and negative
			//		integers denoting an offset from the end (i.e., to the left
			//		of the end)
			// end: Integer?
			//		Optional parameter to describe what position relative to
			//		the NodeList's zero index to end the slice at. Like begin,
			//		can be positive or negative.
			return this._wrap(a.slice.apply(this, arguments));
		},

		splice: function(index, howmany, item){
			// summary:
			//		Returns a new NodeList, manipulating this NodeList based on
			//		the arguments passed, potentially splicing in new elements
			//		at an offset, optionally deleting elements
			// description:
			//		This method behaves exactly like the Array.splice method
			//		with the caveat that it returns a dojo.NodeList and not a
			//		raw Array. For more details, see Mozilla's (splice
			//		documentation)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:splice]
			// 		For backwards compatibility, calling .end() on the spliced NodeList
			// 		does not return the original NodeList -- splice alters the NodeList in place.
			// index: Integer
			//		begin can be a positive or negative integer, with positive
			//		integers noting the offset to begin at, and negative
			//		integers denoting an offset from the end (i.e., to the left
			//		of the end)
			// howmany: Integer?
			//		Optional parameter to describe what position relative to
			//		the NodeList's zero index to end the slice at. Like begin,
			//		can be positive or negative.
			// item: Object...?
			//		Any number of optional parameters may be passed in to be
			//		spliced into the NodeList
			// returns:
			//		dojo.NodeList
			return this._wrap(a.splice.apply(this, arguments));
		},

		indexOf: function(value, fromIndex){
			//	summary:
			//		see dojo.indexOf(). The primary difference is that the acted-on 
			//		array is implicitly this NodeList
			// value: Object:
			//		The value to search for.
			// fromIndex: Integer?:
			//		The loction to start searching from. Optional. Defaults to 0.
			//	description:
			//		For more details on the behavior of indexOf, see Mozilla's
			//		(indexOf
			//		docs)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:indexOf]
			//	returns:
			//		Positive Integer or 0 for a match, -1 of not found.
			return d.indexOf(this, value, fromIndex); // Integer
		},

		lastIndexOf: function(value, fromIndex){
			// summary:
			//		see dojo.lastIndexOf(). The primary difference is that the
			//		acted-on array is implicitly this NodeList
			//	description:
			//		For more details on the behavior of lastIndexOf, see
			//		Mozilla's (lastIndexOf
			//		docs)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:lastIndexOf]
			// value: Object
			//		The value to search for.
			// fromIndex: Integer?
			//		The loction to start searching from. Optional. Defaults to 0.
			// returns:
			//		Positive Integer or 0 for a match, -1 of not found.
			return d.lastIndexOf(this, value, fromIndex); // Integer
		},

		every: function(callback, thisObject){
			//	summary:
			//		see `dojo.every()` and the (Array.every
			//		docs)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:every].
			//		Takes the same structure of arguments and returns as
			//		dojo.every() with the caveat that the passed array is
			//		implicitly this NodeList
			// callback: Function: the callback
			// thisObject: Object?: the context
			return d.every(this, callback, thisObject); // Boolean
		},

		some: function(callback, thisObject){
			//	summary:
			//		Takes the same structure of arguments and returns as
			//		`dojo.some()` with the caveat that the passed array is
			//		implicitly this NodeList.  See `dojo.some()` and Mozilla's
			//		(Array.some
			//		documentation)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:some].
			// callback: Function: the callback
			// thisObject: Object?: the context
			return d.some(this, callback, thisObject); // Boolean
		},
		=====*/

		concat: function(item){
			// summary:
			//		Returns a new NodeList comprised of items in this NodeList
			//		as well as items passed in as parameters
			// description:
			//		This method behaves exactly like the Array.concat method
			//		with the caveat that it returns a `dojo.NodeList` and not a
			//		raw Array. For more details, see the (Array.concat
			//		docs)[http://developer.mozilla.org/en/docs/Core_JavaScript_1.5_Reference:Global_Objects:Array:concat]
			// item: Object?
			//		Any number of optional parameters may be passed in to be
			//		spliced into the NodeList
			// returns:
			//		dojo.NodeList

			//return this._wrap(apc.apply(this, arguments));
			// the line above won't work for the native NodeList :-(

			// implementation notes:
			// 1) Native NodeList is not an array, and cannot be used directly
			// in concat() --- the latter doesn't recognize it as an array, and
			// does not inline it, but append as a single entity.
			// 2) On some browsers (e.g., Safari) the "constructor" property is
			// read-only and cannot be changed. So we have to test for both
			// native NodeList and dojo.NodeList in this property to recognize
			// the node list.

			var t = d.isArray(this) ? this : aps.call(this, 0),
				m = d.map(arguments, function(a){
					return a && !d.isArray(a) &&
						(typeof NodeList != "undefined" && a.constructor === NodeList || a.constructor === this._NodeListCtor) ?
							aps.call(a, 0) : a;
				});
			return this._wrap(apc.apply(t, m), this);	// dojo.NodeList
		},

		map: function(/*Function*/ func, /*Function?*/ obj){
			//	summary:
			//		see dojo.map(). The primary difference is that the acted-on
			//		array is implicitly this NodeList and the return is a
			//		dojo.NodeList (a subclass of Array)
			///return d.map(this, func, obj, d.NodeList); // dojo.NodeList
			return this._wrap(d.map(this, func, obj), this); // dojo.NodeList
		},

		forEach: function(callback, thisObj){
			//	summary:
			//		see `dojo.forEach()`. The primary difference is that the acted-on 
			//		array is implicitly this NodeList. If you want the option to break out
			//		of the forEach loop, use every() or some() instead.
			d.forEach(this, callback, thisObj);
			// non-standard return to allow easier chaining
			return this; // dojo.NodeList 
		},

		/*=====
		coords: function(){
			//	summary:
			//		Returns the box objects of all elements in a node list as
			//		an Array (*not* a NodeList). Acts like `dojo.coords`, though assumes
			//		the node passed is each node in this list.

			return d.map(this, d.coords); // Array
		},

		position: function(){
			//	summary:
			//		Returns border-box objects (x/y/w/h) of all elements in a node list
			//		as an Array (*not* a NodeList). Acts like `dojo.position`, though
			//		assumes the node passed is each node in this list. 

			return d.map(this, d.position); // Array
		},

		attr: function(property, value){
			//	summary:
			//		gets or sets the DOM attribute for every element in the
			//		NodeList. See also `dojo.attr`
			//	property: String
			//		the attribute to get/set
			//	value: String?
			//		optional. The value to set the property to
			//	returns:
			//		if no value is passed, the result is an array of attribute values
			//		If a value is passed, the return is this NodeList
			//	example:
			//		Make all nodes with a particular class focusable:
			//	|	dojo.query(".focusable").attr("tabIndex", -1);
			//	example:
			//		Disable a group of buttons:
			//	|	dojo.query("button.group").attr("disabled", true);
			//	example:
			//		innerHTML can be assigned or retreived as well:
			//	|	// get the innerHTML (as an array) for each list item
			//	|	var ih = dojo.query("li.replaceable").attr("innerHTML");
			return; // dojo.NodeList
			return; // Array
		},

		style: function(property, value){
			//	summary:
			//		gets or sets the CSS property for every element in the NodeList
			//	property: String
			//		the CSS property to get/set, in JavaScript notation
			//		("lineHieght" instead of "line-height")
			//	value: String?
			//		optional. The value to set the property to
			//	returns:
			//		if no value is passed, the result is an array of strings.
			//		If a value is passed, the return is this NodeList
			return; // dojo.NodeList
			return; // Array
		},

		addClass: function(className){
			//	summary:
			//		adds the specified class to every node in the list
			//	className: String|Array
			//		A String class name to add, or several space-separated class names,
			//		or an array of class names.
			return; // dojo.NodeList
		},

		removeClass: function(className){
			//	summary:
			//		removes the specified class from every node in the list
			//	className: String|Array?
			//		An optional String class name to remove, or several space-separated
			//		class names, or an array of class names. If omitted, all class names
			//		will be deleted.
			//	returns:
			//		dojo.NodeList, this list
			return; // dojo.NodeList
		},

		toggleClass: function(className, condition){
			//	summary:
			//		Adds a class to node if not present, or removes if present.
			//		Pass a boolean condition if you want to explicitly add or remove.
			//	condition: Boolean?
			//		If passed, true means to add the class, false means to remove.
			//	className: String
			//		the CSS class to add
			return; // dojo.NodeList
		},

		connect: function(methodName, objOrFunc, funcName){
			//	summary:
			//		attach event handlers to every item of the NodeList. Uses dojo.connect()
			//		so event properties are normalized
			//	methodName: String
			//		the name of the method to attach to. For DOM events, this should be
			//		the lower-case name of the event
			//	objOrFunc: Object|Function|String
			//		if 2 arguments are passed (methodName, objOrFunc), objOrFunc should
			//		reference a function or be the name of the function in the global
			//		namespace to attach. If 3 arguments are provided
			//		(methodName, objOrFunc, funcName), objOrFunc must be the scope to 
			//		locate the bound function in
			//	funcName: String?
			//		optional. A string naming the function in objOrFunc to bind to the
			//		event. May also be a function reference.
			//	example:
			//		add an onclick handler to every button on the page
			//		|	dojo.query("div:nth-child(odd)").connect("onclick", function(e){
			//		|		console.log("clicked!");
			//		|	});
			// example:
			//		attach foo.bar() to every odd div's onmouseover
			//		|	dojo.query("div:nth-child(odd)").connect("onmouseover", foo, "bar");
		},

		empty: function(){
			//	summary:
			//		clears all content from each node in the list. Effectively
			//		equivalent to removing all child nodes from every item in
			//		the list.
			return this.forEach("item.innerHTML='';"); // dojo.NodeList
			// FIXME: should we be checking for and/or disposing of widgets below these nodes?
		},
		=====*/

		// useful html methods
		coords:	adaptAsMap(d.coords),
		position: adaptAsMap(d.position),

		// FIXME: connectPublisher()? connectRunOnce()?

		/*
		destroy: function(){
			//	summary:
			//		destroys every item in 	the list.
			this.forEach(d.destroy);
			// FIXME: should we be checking for and/or disposing of widgets below these nodes?
		},
		*/

		place: function(/*String||Node*/ queryOrNode, /*String*/ position){
			//	summary:
			//		places elements of this node list relative to the first element matched
			//		by queryOrNode. Returns the original NodeList. See: `dojo.place`
			//	queryOrNode:
			//		may be a string representing any valid CSS3 selector or a DOM node.
			//		In the selector case, only the first matching element will be used 
			//		for relative positioning.
			//	position:
			//		can be one of:
			//		|	"last" (default)
			//		|	"first"
			//		|	"before"
			//		|	"after"
			//		|	"only"
			//		|	"replace"
			// 		or an offset in the childNodes property
			var item = d.query(queryOrNode)[0];
			return this.forEach(function(node){ d.place(node, item, position); }); // dojo.NodeList
		},

		orphan: function(/*String?*/ simpleFilter){
			//	summary:
			//		removes elements in this list that match the simple filter
			//		from their parents and returns them as a new NodeList.
			//	simpleFilter:
			//		single-expression CSS rule. For example, ".thinger" or
			//		"#someId[attrName='value']" but not "div > span". In short,
			//		anything which does not invoke a descent to evaluate but
			//		can instead be used to test a single node is acceptable.
			//	returns:
			//		`dojo.NodeList` containing the orpahned elements 
			return (simpleFilter ? d._filterQueryResult(this, simpleFilter) : this).forEach(orphan); // dojo.NodeList
		},

		adopt: function(/*String||Array||DomNode*/ queryOrListOrNode, /*String?*/ position){
			//	summary:
			//		places any/all elements in queryOrListOrNode at a
			//		position relative to the first element in this list.
			//		Returns a dojo.NodeList of the adopted elements.
			//	queryOrListOrNode:
			//		a DOM node or a query string or a query result.
			//		Represents the nodes to be adopted relative to the
			//		first element of this NodeList.
			//	position:
			//		can be one of:
			//		|	"last" (default)
			//		|	"first"
			//		|	"before"
			//		|	"after"
			//		|	"only"
			//		|	"replace"
			// 		or an offset in the childNodes property
			return d.query(queryOrListOrNode).place(this[0], position)._stash(this);	// dojo.NodeList
		},

		// FIXME: do we need this?
		query: function(/*String*/ queryStr){
			//	summary:
			//		Returns a new list whose memebers match the passed query,
			//		assuming elements of the current NodeList as the root for
			//		each search.
			//	example:
			//		assume a DOM created by this markup:
			//	|	<div id="foo">
			//	|		<p>
			//	|			bacon is tasty, <span>dontcha think?</span>
			//	|		</p>
			//	|	</div>
			//	|	<div id="bar">
			//	|		<p>great commedians may not be funny <span>in person</span></p>
			//	|	</div>
			//		If we are presented with the following defintion for a NodeList:
			//	|	var l = new dojo.NodeList(dojo.byId("foo"), dojo.byId("bar"));
			//		it's possible to find all span elements under paragraphs
			//		contained by these elements with this sub-query:
			//	| 	var spans = l.query("p span");

			// FIXME: probably slow
			if(!queryStr){ return this; }
			var ret = this.map(function(node){
				// FIXME: why would we ever get undefined here?
				return d.query(queryStr, node).filter(function(subNode){ return subNode !== undefined; });
			});
			return this._wrap(apc.apply([], ret), this);	// dojo.NodeList
		},

		filter: function(/*String|Function*/ simpleFilter){
			//	summary:
			// 		"masks" the built-in javascript filter() method (supported
			// 		in Dojo via `dojo.filter`) to support passing a simple
			// 		string filter in addition to supporting filtering function
			// 		objects.
			//	simpleFilter:
			//		If a string, a single-expression CSS rule. For example,
			//		".thinger" or "#someId[attrName='value']" but not "div >
			//		span". In short, anything which does not invoke a descent
			//		to evaluate but can instead be used to test a single node
			//		is acceptable.
			//	example:
			//		"regular" JS filter syntax as exposed in dojo.filter:
			//		|	dojo.query("*").filter(function(item){
			//		|		// highlight every paragraph
			//		|		return (item.nodeName == "p");
			//		|	}).style("backgroundColor", "yellow");
			// example:
			//		the same filtering using a CSS selector
			//		|	dojo.query("*").filter("p").styles("backgroundColor", "yellow");

			var a = arguments, items = this, start = 0;
			if(typeof simpleFilter == "string"){ // inline'd type check
				items = d._filterQueryResult(this, a[0]);
				if(a.length == 1){
					// if we only got a string query, pass back the filtered results
					return items._stash(this); // dojo.NodeList
				}
				// if we got a callback, run it over the filtered items
				start = 1;
			}
			return this._wrap(d.filter(items, a[start], a[start + 1]), this);	// dojo.NodeList
		},

		/*
		// FIXME: should this be "copyTo" and include parenting info?
		clone: function(){
			// summary:
			//		creates node clones of each element of this list
			//		and returns a new list containing the clones
		},
		*/

		addContent: function(/*String||DomNode||Object||dojo.NodeList*/ content, /*String||Integer?*/ position){
			//	summary:
			//		add a node, NodeList or some HTML as a string to every item in the
			//		list.  Returns the original list.
			//	description:
			//		a copy of the HTML content is added to each item in the
			//		list, with an optional position argument. If no position
			//		argument is provided, the content is appended to the end of
			//		each item.
			//	content:
			//		DOM node, HTML in string format, a NodeList or an Object. If a DOM node or
			// 		NodeList, the content will be cloned if the current NodeList has more than one
			// 		element. Only the DOM nodes are cloned, no event handlers. If it is an Object,
			// 		it should be an object with at "template" String property that has the HTML string
			// 		to insert. If dojo.string has already been dojo.required, then dojo.string.substitute
			// 		will be used on the "template" to generate the final HTML string. Other allowed
			// 		properties on the object are: "parse" if the HTML
			// 		string should be parsed for widgets ( to get that
			// 		option to work), and "templateFunc" if a template function besides dojo.string.substitute
			// 		should be used to transform the "template".
			//	position:
			//		can be one of:
			//		|	"last"||"end" (default)
			//		|	"first||"start"
			//		|	"before"
			//		|	"after"
			//		|	"replace" (replaces nodes in this NodeList with new content)
			//		|	"only" (removes other children of the nodes so new content is hte only child)
			// 		or an offset in the childNodes property
			//	example:
			//		appends content to the end if the position is ommitted
			//	|	dojo.query("h3 > p").addContent("hey there!");
			//	example:
			//		add something to the front of each element that has a
			//		"thinger" property:
			//	|	dojo.query("[thinger]").addContent("...", "first");
			//	example:
			//		adds a header before each element of the list
			//	|	dojo.query(".note").addContent("<h4>NOTE:</h4>", "before");
			//	example:
			//		add a clone of a DOM node to the end of every element in
			//		the list, removing it from its existing parent.
			//	|	dojo.query(".note").addContent(dojo.byId("foo"));
			//  example:
			//  	Append nodes from a templatized string.
			// 		
			// 		dojo.query(".note").addContent({
			//  		template: '<b>${id}: </b><span>${name}</span>',
			// 			id: "user332",
			//  		name: "Mr. Anderson"
			//  	});
			//  example:
			//  	Append nodes from a templatized string that also has widgets parsed.
			//  	
			//  	
			//  	var notes = dojo.query(".note").addContent({
			//  		template: '<button dojoType="dijit.form.Button">${text}</button>',
			//  		parse: true,
			//  		text: "Send"
			//  	});
			content = this._normalize(content, this[0]);
			for(var i = 0, node; node = this[i]; i++){
				this._place(content, node, position, i > 0);
			}
			return this; //dojo.NodeList
		},

		instantiate: function(/*String|Object*/ declaredClass, /*Object?*/ properties){
			//	summary:
			//		Create a new instance of a specified class, using the
			//		specified properties and each node in the nodeList as a
			//		srcNodeRef.
			//	example:
			//		Grabs all buttons in the page and converts them to diji.form.Buttons.
			//	|	var buttons = dojo.query("button").instantiate("dijit.form.Button", {showLabel: true});
			var c = d.isFunction(declaredClass) ? declaredClass : d.getObject(declaredClass);
			properties = properties || {};
			return this.forEach(function(node){
				new c(properties, node);
			});	// dojo.NodeList
		},

		at: function(/*===== index =====*/){
			//	summary:
			//		Returns a new NodeList comprised of items in this NodeList
			//		at the given index or indices.
			//
			//	index: Integer...
			//		One or more 0-based indices of items in the current
			//		NodeList. A negative index will start at the end of the
			//		list and go backwards. 
			//
			//	example:
			//	Shorten the list to the first, second, and third elements
			//	|	dojo.query("a").at(0, 1, 2).forEach(fn); 
			//
			//	example:
			//	Retrieve the first and last elements of a unordered list:
			//	|	dojo.query("ul > li").at(0, -1).forEach(cb);
			//
			//	example:
			//	Do something for the first element only, but end() out back to
			//	the original list and continue chaining:
			//	|	dojo.query("a").at(0).onclick(fn).end().forEach(function(n){
			//	|		console.log(n); // all anchors on the page. 
			//	|	})	
			//
			//	returns:
			//		dojo.NodeList
			var t = new this._NodeListCtor();
			d.forEach(arguments, function(i){ 
				if(i < 0){ i = this.length + i }
				if(this[i]){ t.push(this[i]); }
			}, this);
			return t._stash(this); // dojo.NodeList
		}

	});

	nl.events = [
		// summary: list of all DOM events used in NodeList
		"blur", "focus", "change", "click", "error", "keydown", "keypress",
		"keyup", "load", "mousedown", "mouseenter", "mouseleave", "mousemove",
		"mouseout", "mouseover", "mouseup", "submit"
	];
	
	// FIXME: pseudo-doc the above automatically generated on-event functions

	// syntactic sugar for DOM events
	d.forEach(nl.events, function(evt){
			var _oe = "on" + evt;
			nlp[_oe] = function(a, b){
				return this.connect(_oe, a, b);
			}
				// FIXME: should these events trigger publishes?
				/*
				return (a ? this.connect(_oe, a, b) :
							this.forEach(function(n){
								// FIXME:
								//		listeners get buried by
								//		addEventListener and can't be dug back
								//		out to be triggered externally.
								// see:
								//		http://developer.mozilla.org/en/docs/DOM:element

								console.log(n, evt, _oe);

								// FIXME: need synthetic event support!
								var _e = { target: n, faux: true, type: evt };
								// dojo._event_listener._synthesizeEvent({}, { target: n, faux: true, type: evt });
								try{ n[evt](_e); }catch(e){ console.log(e); }
								try{ n[_oe](_e); }catch(e){ console.log(e); }
							})
				);
				*/
		}
	);

})();

}

if(!dojo._hasResource["dojo._base.query"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.query"] = true;
if(typeof dojo != "undefined"){
	dojo.provide("dojo._base.query");
	
	

}

/*
	dojo.query() architectural overview:

		dojo.query is a relatively full-featured CSS3 query library. It is
		designed to take any valid CSS3 selector and return the nodes matching
		the selector. To do this quickly, it processes queries in several
		steps, applying caching where profitable.
		
		The steps (roughly in reverse order of the way they appear in the code):
			1.) check to see if we already have a "query dispatcher"
				- if so, use that with the given parameterization. Skip to step 4.
			2.) attempt to determine which branch to dispatch the query to:
				- JS (optimized DOM iteration)
				- native (FF3.1+, Safari 3.1+, IE 8+)
			3.) tokenize and convert to executable "query dispatcher"
				- this is where the lion's share of the complexity in the
				  system lies. In the DOM version, the query dispatcher is
				  assembled as a chain of "yes/no" test functions pertaining to
				  a section of a simple query statement (".blah:nth-child(odd)"
				  but not "div div", which is 2 simple statements). Individual
				  statement dispatchers are cached (to prevent re-definition)
				  as are entire dispatch chains (to make re-execution of the
				  same query fast)
			4.) the resulting query dispatcher is called in the passed scope
			    (by default the top-level document)
				- for DOM queries, this results in a recursive, top-down
				  evaluation of nodes based on each simple query section
				- for native implementations, this may mean working around spec
				  bugs. So be it.
			5.) matched nodes are pruned to ensure they are unique (if necessary)
*/

;(function(d){
	// define everything in a closure for compressability reasons. "d" is an
	// alias to "dojo" (or the toolkit alias object, e.g., "acme").

	////////////////////////////////////////////////////////////////////////
	// Toolkit aliases
	////////////////////////////////////////////////////////////////////////

	// if you are extracing dojo.query for use in your own system, you will
	// need to provide these methods and properties. No other porting should be
	// necessary, save for configuring the system to use a class other than
	// dojo.NodeList as the return instance instantiator
	var trim = 			d.trim;
	var each = 			d.forEach;
	// 					d.isIE; // float
	// 					d.isSafari; // float
	// 					d.isOpera; // float
	// 					d.isWebKit; // float
	// 					d.doc ; // document element
	var qlc = d._NodeListCtor = 		d.NodeList;

	var getDoc = function(){ return d.doc; };
	// NOTE(alex): the spec is idiotic. CSS queries should ALWAYS be case-sensitive, but nooooooo
	var cssCaseBug = ((d.isWebKit||d.isMozilla) && ((getDoc().compatMode) == "BackCompat"));

	////////////////////////////////////////////////////////////////////////
	// Global utilities
	////////////////////////////////////////////////////////////////////////


	// on browsers that support the "children" collection we can avoid a lot of
	// iteration on chaff (non-element) nodes.
	// why.
	var childNodesName = !!getDoc().firstChild["children"] ? "children" : "childNodes";

	var specials = ">~+";

	// global thunk to determine whether we should treat the current query as
	// case sensitive or not. This switch is flipped by the query evaluator
	// based on the document passed as the context to search.
	var caseSensitive = false;

	// how high?
	var yesman = function(){ return true; };

	////////////////////////////////////////////////////////////////////////
	// Tokenizer
	////////////////////////////////////////////////////////////////////////

	var getQueryParts = function(query){
		//	summary: 
		//		state machine for query tokenization
		//	description:
		//		instead of using a brittle and slow regex-based CSS parser,
		//		dojo.query implements an AST-style query representation. This
		//		representation is only generated once per query. For example,
		//		the same query run multiple times or under different root nodes
		//		does not re-parse the selector expression but instead uses the
		//		cached data structure. The state machine implemented here
		//		terminates on the last " " (space) charachter and returns an
		//		ordered array of query component structures (or "parts"). Each
		//		part represents an operator or a simple CSS filtering
		//		expression. The structure for parts is documented in the code
		//		below.


		// NOTE: 
		//		this code is designed to run fast and compress well. Sacrifices
		//		to readibility and maintainability have been made.  Your best
		//		bet when hacking the tokenizer is to put The Donnas on *really*
		//		loud (may we recommend their "Spend The Night" release?) and
		//		just assume you're gonna make mistakes. Keep the unit tests
		//		open and run them frequently. Knowing is half the battle ;-)
		if(specials.indexOf(query.slice(-1)) >= 0){
			// if we end with a ">", "+", or "~", that means we're implicitly
			// searching all children, so make it explicit
			query += " * "
		}else{
			// if you have not provided a terminator, one will be provided for
			// you...
			query += " ";
		}

		var ts = function(/*Integer*/ s, /*Integer*/ e){
			// trim and slice. 

			// take an index to start a string slice from and an end position
			// and return a trimmed copy of that sub-string
			return trim(query.slice(s, e));
		}

		// the overall data graph of the full query, as represented by queryPart objects
		var queryParts = []; 


		// state keeping vars
		var inBrackets = -1, inParens = -1, inMatchFor = -1, 
			inPseudo = -1, inClass = -1, inId = -1, inTag = -1, 
			lc = "", cc = "", pStart;

		// iteration vars
		var x = 0, // index in the query
			ql = query.length,
			currentPart = null, // data structure representing the entire clause
			_cp = null; // the current pseudo or attr matcher

		// several temporary variables are assigned to this structure durring a
		// potential sub-expression match:
		//		attr:
		//			a string representing the current full attribute match in a
		//			bracket expression
		//		type:
		//			if there's an operator in a bracket expression, this is
		//			used to keep track of it
		//		value:
		//			the internals of parenthetical expression for a pseudo. for
		//			:nth-child(2n+1), value might be "2n+1"

		var endTag = function(){
			// called when the tokenizer hits the end of a particular tag name.
			// Re-sets state variables for tag matching and sets up the matcher
			// to handle the next type of token (tag or operator).
			if(inTag >= 0){
				var tv = (inTag == x) ? null : ts(inTag, x); // .toLowerCase();
				currentPart[ (specials.indexOf(tv) < 0) ? "tag" : "oper" ] = tv;
				inTag = -1;
			}
		}

		var endId = function(){
			// called when the tokenizer might be at the end of an ID portion of a match
			if(inId >= 0){
				currentPart.id = ts(inId, x).replace(/\\/g, "");
				inId = -1;
			}
		}

		var endClass = function(){
			// called when the tokenizer might be at the end of a class name
			// match. CSS allows for multiple classes, so we augment the
			// current item with another class in its list
			if(inClass >= 0){
				currentPart.classes.push(ts(inClass+1, x).replace(/\\/g, ""));
				inClass = -1;
			}
		}

		var endAll = function(){
			// at the end of a simple fragment, so wall off the matches
			endId(); endTag(); endClass();
		}

		var endPart = function(){
			endAll();
			if(inPseudo >= 0){
				currentPart.pseudos.push({ name: ts(inPseudo+1, x) });
			}
			// hint to the selector engine to tell it whether or not it
			// needs to do any iteration. Many simple selectors don't, and
			// we can avoid significant construction-time work by advising
			// the system to skip them
			currentPart.loops = (	
					currentPart.pseudos.length || 
					currentPart.attrs.length || 
					currentPart.classes.length	);

			currentPart.oquery = currentPart.query = ts(pStart, x); // save the full expression as a string


			// otag/tag are hints to suggest to the system whether or not
			// it's an operator or a tag. We save a copy of otag since the
			// tag name is cast to upper-case in regular HTML matches. The
			// system has a global switch to figure out if the current
			// expression needs to be case sensitive or not and it will use
			// otag or tag accordingly
			currentPart.otag = currentPart.tag = (currentPart["oper"]) ? null : (currentPart.tag || "*");

			if(currentPart.tag){
				// if we're in a case-insensitive HTML doc, we likely want
				// the toUpperCase when matching on element.tagName. If we
				// do it here, we can skip the string op per node
				// comparison
				currentPart.tag = currentPart.tag.toUpperCase();
			}

			// add the part to the list
			if(queryParts.length && (queryParts[queryParts.length-1].oper)){
				// operators are always infix, so we remove them from the
				// list and attach them to the next match. The evaluator is
				// responsible for sorting out how to handle them.
				currentPart.infixOper = queryParts.pop();
				currentPart.query = currentPart.infixOper.query + " " + currentPart.query;
				/*
				console.debug(	"swapping out the infix", 
								currentPart.infixOper, 
								"and attaching it to", 
								currentPart);
				*/
			}
			queryParts.push(currentPart);

			currentPart = null;
		}

		// iterate over the query, charachter by charachter, building up a 
		// list of query part objects
		for(; lc=cc, cc=query.charAt(x), x < ql; x++){
			//		cc: the current character in the match
			//		lc: the last charachter (if any)

			// someone is trying to escape something, so don't try to match any
			// fragments. We assume we're inside a literal.
			if(lc == "\\"){ continue; } 
			if(!currentPart){ // a part was just ended or none has yet been created
				// NOTE: I hate all this alloc, but it's shorter than writing tons of if's
				pStart = x;
				//	rules describe full CSS sub-expressions, like:
				//		#someId
				//		.className:first-child
				//	but not:
				//		thinger > div.howdy[type=thinger]
				//	the indidual components of the previous query would be
				//	split into 3 parts that would be represented a structure
				//	like:
				//		[
				//			{
				//				query: "thinger",
				//				tag: "thinger",
				//			},
				//			{
				//				query: "div.howdy[type=thinger]",
				//				classes: ["howdy"],
				//				infixOper: {
				//					query: ">",
				//					oper: ">",
				//				}
				//			},
				//		]
				currentPart = {
					query: null, // the full text of the part's rule
					pseudos: [], // CSS supports multiple pseud-class matches in a single rule
					attrs: [], 	// CSS supports multi-attribute match, so we need an array
					classes: [], // class matches may be additive, e.g.: .thinger.blah.howdy
					tag: null, 	// only one tag...
					oper: null, // ...or operator per component. Note that these wind up being exclusive.
					id: null, 	// the id component of a rule
					getTag: function(){
						return (caseSensitive) ? this.otag : this.tag;
					}
				};

				// if we don't have a part, we assume we're going to start at
				// the beginning of a match, which should be a tag name. This
				// might fault a little later on, but we detect that and this
				// iteration will still be fine.
				inTag = x; 
			}

			if(inBrackets >= 0){
				// look for a the close first
				if(cc == "]"){ // if we're in a [...] clause and we end, do assignment
					if(!_cp.attr){
						// no attribute match was previously begun, so we
						// assume this is an attribute existance match in the
						// form of [someAttributeName]
						_cp.attr = ts(inBrackets+1, x);
					}else{
						// we had an attribute already, so we know that we're
						// matching some sort of value, as in [attrName=howdy]
						_cp.matchFor = ts((inMatchFor||inBrackets+1), x);
					}
					var cmf = _cp.matchFor;
					if(cmf){
						// try to strip quotes from the matchFor value. We want
						// [attrName=howdy] to match the same 
						//	as [attrName = 'howdy' ]
						if(	(cmf.charAt(0) == '"') || (cmf.charAt(0)  == "'") ){
							_cp.matchFor = cmf.slice(1, -1);
						}
					}
					// end the attribute by adding it to the list of attributes. 
					currentPart.attrs.push(_cp);
					_cp = null; // necessary?
					inBrackets = inMatchFor = -1;
				}else if(cc == "="){
					// if the last char was an operator prefix, make sure we
					// record it along with the "=" operator. 
					var addToCc = ("|~^$*".indexOf(lc) >=0 ) ? lc : "";
					_cp.type = addToCc+cc;
					_cp.attr = ts(inBrackets+1, x-addToCc.length);
					inMatchFor = x+1;
				}
				// now look for other clause parts
			}else if(inParens >= 0){
				// if we're in a parenthetical expression, we need to figure
				// out if it's attached to a pseduo-selector rule like
				// :nth-child(1)
				if(cc == ")"){
					if(inPseudo >= 0){
						_cp.value = ts(inParens+1, x);
					}
					inPseudo = inParens = -1;
				}
			}else if(cc == "#"){
				// start of an ID match
				endAll();
				inId = x+1;
			}else if(cc == "."){
				// start of a class match
				endAll();
				inClass = x;
			}else if(cc == ":"){
				// start of a pseudo-selector match
				endAll();
				inPseudo = x;
			}else if(cc == "["){
				// start of an attribute match. 
				endAll();
				inBrackets = x;
				// provide a new structure for the attribute match to fill-in
				_cp = {
					/*=====
					attr: null, type: null, matchFor: null
					=====*/
				};
			}else if(cc == "("){
				// we really only care if we've entered a parenthetical
				// expression if we're already inside a pseudo-selector match
				if(inPseudo >= 0){
					// provide a new structure for the pseudo match to fill-in
					_cp = { 
						name: ts(inPseudo+1, x), 
						value: null
					}
					currentPart.pseudos.push(_cp);
				}
				inParens = x;
			}else if(
				(cc == " ") && 
				// if it's a space char and the last char is too, consume the
				// current one without doing more work
				(lc != cc)
			){
				endPart();
			}
		}
		return queryParts;
	};
	

	////////////////////////////////////////////////////////////////////////
	// DOM query infrastructure
	////////////////////////////////////////////////////////////////////////

	var agree = function(first, second){
		// the basic building block of the yes/no chaining system. agree(f1,
		// f2) generates a new function which returns the boolean results of
		// both of the passed functions to a single logical-anded result. If
		// either are not possed, the other is used exclusively.
		if(!first){ return second; }
		if(!second){ return first; }

		return function(){
			return first.apply(window, arguments) && second.apply(window, arguments);
		}
	};

	var getArr = function(i, arr){
		// helps us avoid array alloc when we don't need it
		var r = arr||[]; // FIXME: should this be 'new d._NodeListCtor()' ?
		if(i){ r.push(i); }
		return r;
	};

	var _isElement = function(n){ return (1 == n.nodeType); };

	// FIXME: need to coalesce _getAttr with defaultGetter
	var blank = "";
	var _getAttr = function(elem, attr){
		if(!elem){ return blank; }
		if(attr == "class"){
			return elem.className || blank;
		}
		if(attr == "for"){
			return elem.htmlFor || blank;
		}
		if(attr == "style"){
			return elem.style.cssText || blank;
		}
		return (caseSensitive ? elem.getAttribute(attr) : elem.getAttribute(attr, 2)) || blank;
	};

	var attrs = {
		"*=": function(attr, value){
			return function(elem){
				// E[foo*="bar"]
				//		an E element whose "foo" attribute value contains
				//		the substring "bar"
				return (_getAttr(elem, attr).indexOf(value)>=0);
			}
		},
		"^=": function(attr, value){
			// E[foo^="bar"]
			//		an E element whose "foo" attribute value begins exactly
			//		with the string "bar"
			return function(elem){
				return (_getAttr(elem, attr).indexOf(value)==0);
			}
		},
		"$=": function(attr, value){
			// E[foo$="bar"]	
			//		an E element whose "foo" attribute value ends exactly
			//		with the string "bar"
			var tval = " "+value;
			return function(elem){
				var ea = " "+_getAttr(elem, attr);
				return (ea.lastIndexOf(value)==(ea.length-value.length));
			}
		},
		"~=": function(attr, value){
			// E[foo~="bar"]	
			//		an E element whose "foo" attribute value is a list of
			//		space-separated values, one of which is exactly equal
			//		to "bar"

			// return "[contains(concat(' ',@"+attr+",' '), ' "+ value +" ')]";
			var tval = " "+value+" ";
			return function(elem){
				var ea = " "+_getAttr(elem, attr)+" ";
				return (ea.indexOf(tval)>=0);
			}
		},
		"|=": function(attr, value){
			// E[hreflang|="en"]
			//		an E element whose "hreflang" attribute has a
			//		hyphen-separated list of values beginning (from the
			//		left) with "en"
			var valueDash = " "+value+"-";
			return function(elem){
				var ea = " "+_getAttr(elem, attr);
				return (
					(ea == value) ||
					(ea.indexOf(valueDash)==0)
				);
			}
		},
		"=": function(attr, value){
			return function(elem){
				return (_getAttr(elem, attr) == value);
			}
		}
	};

	// avoid testing for node type if we can. Defining this in the negative
	// here to avoid negation in the fast path.
	var _noNES = (typeof getDoc().firstChild.nextElementSibling == "undefined");
	var _ns = !_noNES ? "nextElementSibling" : "nextSibling";
	var _ps = !_noNES ? "previousElementSibling" : "previousSibling";
	var _simpleNodeTest = (_noNES ? _isElement : yesman);

	var _lookLeft = function(node){
		// look left
		while(node = node[_ps]){
			if(_simpleNodeTest(node)){ return false; }
		}
		return true;
	};

	var _lookRight = function(node){
		// look right
		while(node = node[_ns]){
			if(_simpleNodeTest(node)){ return false; }
		}
		return true;
	};

	var getNodeIndex = function(node){
		var root = node.parentNode;
		var i = 0,
			tret = root[childNodesName],
			ci = (node["_i"]||-1),
			cl = (root["_l"]||-1);

		if(!tret){ return -1; }
		var l = tret.length;

		// we calcuate the parent length as a cheap way to invalidate the
		// cache. It's not 100% accurate, but it's much more honest than what
		// other libraries do
		if( cl == l && ci >= 0 && cl >= 0 ){
			// if it's legit, tag and release
			return ci;
		}

		// else re-key things
		root["_l"] = l;
		ci = -1;
		for(var te = root["firstElementChild"]||root["firstChild"]; te; te = te[_ns]){
			if(_simpleNodeTest(te)){ 
				te["_i"] = ++i;
				if(node === te){ 
					// NOTE:
					// 	shortcuting the return at this step in indexing works
					// 	very well for benchmarking but we avoid it here since
					// 	it leads to potential O(n^2) behavior in sequential
					// 	getNodexIndex operations on a previously un-indexed
					// 	parent. We may revisit this at a later time, but for
					// 	now we just want to get the right answer more often
					// 	than not.
					ci = i;
				}
			}
		}
		return ci;
	};

	var isEven = function(elem){
		return !((getNodeIndex(elem)) % 2);
	};

	var isOdd = function(elem){
		return ((getNodeIndex(elem)) % 2);
	};

	var pseudos = {
		"checked": function(name, condition){
			return function(elem){
				return !!("checked" in elem ? elem.checked : elem.selected);
			}
		},
		"first-child": function(){ return _lookLeft; },
		"last-child": function(){ return _lookRight; },
		"only-child": function(name, condition){
			return function(node){ 
				if(!_lookLeft(node)){ return false; }
				if(!_lookRight(node)){ return false; }
				return true;
			};
		},
		"empty": function(name, condition){
			return function(elem){
				// DomQuery and jQuery get this wrong, oddly enough.
				// The CSS 3 selectors spec is pretty explicit about it, too.
				var cn = elem.childNodes;
				var cnl = elem.childNodes.length;
				// if(!cnl){ return true; }
				for(var x=cnl-1; x >= 0; x--){
					var nt = cn[x].nodeType;
					if((nt === 1)||(nt == 3)){ return false; }
				}
				return true;
			}
		},
		"contains": function(name, condition){
			var cz = condition.charAt(0);
			if( cz == '"' || cz == "'" ){ //remove quote
				condition = condition.slice(1, -1);
			}
			return function(elem){
				return (elem.innerHTML.indexOf(condition) >= 0);
			}
		},
		"not": function(name, condition){
			var p = getQueryParts(condition)[0];
			var ignores = { el: 1 }; 
			if(p.tag != "*"){
				ignores.tag = 1;
			}
			if(!p.classes.length){
				ignores.classes = 1;
			}
			var ntf = getSimpleFilterFunc(p, ignores);
			return function(elem){
				return (!ntf(elem));
			}
		},
		"nth-child": function(name, condition){
			var pi = parseInt;
			// avoid re-defining function objects if we can
			if(condition == "odd"){
				return isOdd;
			}else if(condition == "even"){
				return isEven;
			}
			// FIXME: can we shorten this?
			if(condition.indexOf("n") != -1){
				var tparts = condition.split("n", 2);
				var pred = tparts[0] ? ((tparts[0] == '-') ? -1 : pi(tparts[0])) : 1;
				var idx = tparts[1] ? pi(tparts[1]) : 0;
				var lb = 0, ub = -1;
				if(pred > 0){
					if(idx < 0){
						idx = (idx % pred) && (pred + (idx % pred));
					}else if(idx>0){
						if(idx >= pred){
							lb = idx - idx % pred;
						}
						idx = idx % pred;
					}
				}else if(pred<0){
					pred *= -1;
					// idx has to be greater than 0 when pred is negative;
					// shall we throw an error here?
					if(idx > 0){
						ub = idx;
						idx = idx % pred;
					}
				}
				if(pred > 0){
					return function(elem){
						var i = getNodeIndex(elem);
						return (i>=lb) && (ub<0 || i<=ub) && ((i % pred) == idx);
					}
				}else{
					condition = idx;
				}
			}
			var ncount = pi(condition);
			return function(elem){
				return (getNodeIndex(elem) == ncount);
			}
		}
	};

	var defaultGetter = (d.isIE) ? function(cond){
		var clc = cond.toLowerCase();
		if(clc == "class"){ cond = "className"; }
		return function(elem){
			return (caseSensitive ? elem.getAttribute(cond) : elem[cond]||elem[clc]);
		}
	} : function(cond){
		return function(elem){
			return (elem && elem.getAttribute && elem.hasAttribute(cond));
		}
	};

	var getSimpleFilterFunc = function(query, ignores){
		// generates a node tester function based on the passed query part. The
		// query part is one of the structures generatd by the query parser
		// when it creates the query AST. The "ignores" object specifies which
		// (if any) tests to skip, allowing the system to avoid duplicating
		// work where it may have already been taken into account by other
		// factors such as how the nodes to test were fetched in the first
		// place
		if(!query){ return yesman; }
		ignores = ignores||{};

		var ff = null;

		if(!("el" in ignores)){
			ff = agree(ff, _isElement);
		}

		if(!("tag" in ignores)){
			if(query.tag != "*"){
				ff = agree(ff, function(elem){
					return (elem && (elem.tagName == query.getTag()));
				});
			}
		}

		if(!("classes" in ignores)){
			each(query.classes, function(cname, idx, arr){
				// get the class name
				/*
				var isWildcard = cname.charAt(cname.length-1) == "*";
				if(isWildcard){
					cname = cname.substr(0, cname.length-1);
				}
				// I dislike the regex thing, even if memozied in a cache, but it's VERY short
				var re = new RegExp("(?:^|\\s)" + cname + (isWildcard ? ".*" : "") + "(?:\\s|$)");
				*/
				var re = new RegExp("(?:^|\\s)" + cname + "(?:\\s|$)");
				ff = agree(ff, function(elem){
					return re.test(elem.className);
				});
				ff.count = idx;
			});
		}

		if(!("pseudos" in ignores)){
			each(query.pseudos, function(pseudo){
				var pn = pseudo.name;
				if(pseudos[pn]){
					ff = agree(ff, pseudos[pn](pn, pseudo.value));
				}
			});
		}

		if(!("attrs" in ignores)){
			each(query.attrs, function(attr){
				var matcher;
				var a = attr.attr;
				// type, attr, matchFor
				if(attr.type && attrs[attr.type]){
					matcher = attrs[attr.type](a, attr.matchFor);
				}else if(a.length){
					matcher = defaultGetter(a);
				}
				if(matcher){
					ff = agree(ff, matcher);
				}
			});
		}

		if(!("id" in ignores)){
			if(query.id){
				ff = agree(ff, function(elem){ 
					return (!!elem && (elem.id == query.id));
				});
			}
		}

		if(!ff){
			if(!("default" in ignores)){
				ff = yesman; 
			}
		}
		return ff;
	};

	var _nextSibling = function(filterFunc){
		return function(node, ret, bag){
			while(node = node[_ns]){
				if(_noNES && (!_isElement(node))){ continue; }
				if(
					(!bag || _isUnique(node, bag)) &&
					filterFunc(node)
				){
					ret.push(node);
				}
				break;
			}
			return ret;
		}
	};

	var _nextSiblings = function(filterFunc){
		return function(root, ret, bag){
			var te = root[_ns];
			while(te){
				if(_simpleNodeTest(te)){
					if(bag && !_isUnique(te, bag)){
						break;
					}
					if(filterFunc(te)){
						ret.push(te);
					}
				}
				te = te[_ns];
			}
			return ret;
		}
	};

	// get an array of child *elements*, skipping text and comment nodes
	var _childElements = function(filterFunc){
		filterFunc = filterFunc||yesman;
		return function(root, ret, bag){
			// get an array of child elements, skipping text and comment nodes
			var te, x = 0, tret = root[childNodesName];
			while(te = tret[x++]){
				if(
					_simpleNodeTest(te) &&
					(!bag || _isUnique(te, bag)) &&
					(filterFunc(te, x))
				){ 
					ret.push(te);
				}
			}
			return ret;
		};
	};
	
	/*
	// thanks, Dean!
	var itemIsAfterRoot = d.isIE ? function(item, root){
		return (item.sourceIndex > root.sourceIndex);
	} : function(item, root){
		return (item.compareDocumentPosition(root) == 2);
	};
	*/

	// test to see if node is below root
	var _isDescendant = function(node, root){
		var pn = node.parentNode;
		while(pn){
			if(pn == root){
				break;
			}
			pn = pn.parentNode;
		}
		return !!pn;
	};

	var _getElementsFuncCache = {};

	var getElementsFunc = function(query){
		var retFunc = _getElementsFuncCache[query.query];
		// if we've got a cached dispatcher, just use that
		if(retFunc){ return retFunc; }
		// else, generate a new on

		// NOTE:
		//		this function returns a function that searches for nodes and
		//		filters them.  The search may be specialized by infix operators
		//		(">", "~", or "+") else it will default to searching all
		//		descendants (the " " selector). Once a group of children is
		//		founde, a test function is applied to weed out the ones we
		//		don't want. Many common cases can be fast-pathed. We spend a
		//		lot of cycles to create a dispatcher that doesn't do more work
		//		than necessary at any point since, unlike this function, the
		//		dispatchers will be called every time. The logic of generating
		//		efficient dispatchers looks like this in pseudo code:
		//
		//		# if it's a purely descendant query (no ">", "+", or "~" modifiers)
		//		if infixOperator == " ":
		//			if only(id):
		//				return def(root):
		//					return d.byId(id, root);
		//
		//			elif id:
		//				return def(root):
		//					return filter(d.byId(id, root));
		//
		//			elif cssClass && getElementsByClassName:
		//				return def(root):
		//					return filter(root.getElementsByClassName(cssClass));
		//
		//			elif only(tag):
		//				return def(root):
		//					return root.getElementsByTagName(tagName);
		//
		//			else:
		//				# search by tag name, then filter
		//				return def(root):
		//					return filter(root.getElementsByTagName(tagName||"*"));
		//
		//		elif infixOperator == ">":
		//			# search direct children
		//			return def(root):
		//				return filter(root.children);
		//
		//		elif infixOperator == "+":
		//			# search next sibling
		//			return def(root):
		//				return filter(root.nextElementSibling);
		//
		//		elif infixOperator == "~":
		//			# search rightward siblings
		//			return def(root):
		//				return filter(nextSiblings(root));

		var io = query.infixOper;
		var oper = (io ? io.oper : "");
		// the default filter func which tests for all conditions in the query
		// part. This is potentially inefficient, so some optimized paths may
		// re-define it to test fewer things.
		var filterFunc = getSimpleFilterFunc(query, { el: 1 });
		var qt = query.tag;
		var wildcardTag = ("*" == qt);
		var ecs = getDoc()["getElementsByClassName"]; 

		if(!oper){
			// if there's no infix operator, then it's a descendant query. ID
			// and "elements by class name" variants can be accelerated so we
			// call them out explicitly:
			if(query.id){
				// testing shows that the overhead of yesman() is acceptable
				// and can save us some bytes vs. re-defining the function
				// everywhere.
				filterFunc = (!query.loops && wildcardTag) ? 
					yesman : 
					getSimpleFilterFunc(query, { el: 1, id: 1 });

				retFunc = function(root, arr){
					var te = d.byId(query.id, (root.ownerDocument||root));
					if(!te || !filterFunc(te)){ return; }
					if(9 == root.nodeType){ // if root's a doc, we just return directly
						return getArr(te, arr);
					}else{ // otherwise check ancestry
						if(_isDescendant(te, root)){
							return getArr(te, arr);
						}
					}
				}
			}else if(
				ecs && 
				// isAlien check. Workaround for Prototype.js being totally evil/dumb.
				/\{\s*\[native code\]\s*\}/.test(String(ecs)) && 
				query.classes.length &&
				!cssCaseBug
			){
				// it's a class-based query and we've got a fast way to run it.

				// ignore class and ID filters since we will have handled both
				filterFunc = getSimpleFilterFunc(query, { el: 1, classes: 1, id: 1 });
				var classesString = query.classes.join(" ");
				retFunc = function(root, arr, bag){
					var ret = getArr(0, arr), te, x=0;
					var tret = root.getElementsByClassName(classesString);
					while((te = tret[x++])){
						if(filterFunc(te, root) && _isUnique(te, bag)){
							ret.push(te);
						}
					}
					return ret;
				};

			}else if(!wildcardTag && !query.loops){
				// it's tag only. Fast-path it.
				retFunc = function(root, arr, bag){
					var ret = getArr(0, arr), te, x=0;
					var tret = root.getElementsByTagName(query.getTag());
					while((te = tret[x++])){
						if(_isUnique(te, bag)){
							ret.push(te);
						}
					}
					return ret;
				};
			}else{
				// the common case:
				//		a descendant selector without a fast path. By now it's got
				//		to have a tag selector, even if it's just "*" so we query
				//		by that and filter
				filterFunc = getSimpleFilterFunc(query, { el: 1, tag: 1, id: 1 });
				retFunc = function(root, arr, bag){
					var ret = getArr(0, arr), te, x=0;
					// we use getTag() to avoid case sensitivity issues
					var tret = root.getElementsByTagName(query.getTag());
					while((te = tret[x++])){
						if(filterFunc(te, root) && _isUnique(te, bag)){
							ret.push(te);
						}
					}
					return ret;
				};
			}
		}else{
			// the query is scoped in some way. Instead of querying by tag we
			// use some other collection to find candidate nodes
			var skipFilters = { el: 1 };
			if(wildcardTag){
				skipFilters.tag = 1;
			}
			filterFunc = getSimpleFilterFunc(query, skipFilters);
			if("+" == oper){
				retFunc = _nextSibling(filterFunc);
			}else if("~" == oper){
				retFunc = _nextSiblings(filterFunc);
			}else if(">" == oper){
				retFunc = _childElements(filterFunc);
			}
		}
		// cache it and return
		return _getElementsFuncCache[query.query] = retFunc;
	};

	var filterDown = function(root, queryParts){
		// NOTE:
		//		this is the guts of the DOM query system. It takes a list of
		//		parsed query parts and a root and finds children which match
		//		the selector represented by the parts
		var candidates = getArr(root), qp, x, te, qpl = queryParts.length, bag, ret;

		for(var i = 0; i < qpl; i++){
			ret = [];
			qp = queryParts[i];
			x = candidates.length - 1;
			if(x > 0){
				// if we have more than one root at this level, provide a new
				// hash to use for checking group membership but tell the
				// system not to post-filter us since we will already have been
				// gauranteed to be unique
				bag = {};
				ret.nozip = true;
			}
			var gef = getElementsFunc(qp);
			for(var j = 0; (te = candidates[j]); j++){
				// for every root, get the elements that match the descendant
				// selector, adding them to the "ret" array and filtering them
				// via membership in this level's bag. If there are more query
				// parts, then this level's return will be used as the next
				// level's candidates
				gef(te, ret, bag);
			}
			if(!ret.length){ break; }
			candidates = ret;
		}
		return ret;
	};

	////////////////////////////////////////////////////////////////////////
	// the query runner
	////////////////////////////////////////////////////////////////////////

	// these are the primary caches for full-query results. The query
	// dispatcher functions are generated then stored here for hash lookup in
	// the future
	var _queryFuncCacheDOM = {},
		_queryFuncCacheQSA = {};

	// this is the second level of spliting, from full-length queries (e.g.,
	// "div.foo .bar") into simple query expressions (e.g., ["div.foo",
	// ".bar"])
	var getStepQueryFunc = function(query){
		var qparts = getQueryParts(trim(query));

		// if it's trivial, avoid iteration and zipping costs
		if(qparts.length == 1){
			// we optimize this case here to prevent dispatch further down the
			// chain, potentially slowing things down. We could more elegantly
			// handle this in filterDown(), but it's slower for simple things
			// that need to be fast (e.g., "#someId").
			var tef = getElementsFunc(qparts[0]);
			return function(root){
				var r = tef(root, new qlc());
				if(r){ r.nozip = true; }
				return r;
			}
		}

		// otherwise, break it up and return a runner that iterates over the parts recursively
		return function(root){
			return filterDown(root, qparts);
		}
	};

	// NOTES:
	//	* we can't trust QSA for anything but document-rooted queries, so
	//	  caching is split into DOM query evaluators and QSA query evaluators
	//	* caching query results is dirty and leak-prone (or, at a minimum,
	//	  prone to unbounded growth). Other toolkits may go this route, but
	//	  they totally destroy their own ability to manage their memory
	//	  footprint. If we implement it, it should only ever be with a fixed
	//	  total element reference # limit and an LRU-style algorithm since JS
	//	  has no weakref support. Caching compiled query evaluators is also
	//	  potentially problematic, but even on large documents the size of the
	//	  query evaluators is often < 100 function objects per evaluator (and
	//	  LRU can be applied if it's ever shown to be an issue).
	//	* since IE's QSA support is currently only for HTML documents and even
	//	  then only in IE 8's "standards mode", we have to detect our dispatch
	//	  route at query time and keep 2 separate caches. Ugg.

	// we need to determine if we think we can run a given query via
	// querySelectorAll or if we'll need to fall back on DOM queries to get
	// there. We need a lot of information about the environment and the query
	// to make the determiniation (e.g. does it support QSA, does the query in
	// question work in the native QSA impl, etc.).
	var nua = navigator.userAgent;
	// some versions of Safari provided QSA, but it was buggy and crash-prone.
	// We need te detect the right "internal" webkit version to make this work.
	var wk = "WebKit/";
	var is525 = (
		d.isWebKit && 
		(nua.indexOf(wk) > 0) && 
		(parseFloat(nua.split(wk)[1]) > 528)
	);

	// IE QSA queries may incorrectly include comment nodes, so we throw the
	// zipping function into "remove" comments mode instead of the normal "skip
	// it" which every other QSA-clued browser enjoys
	var noZip = d.isIE ? "commentStrip" : "nozip";

	var qsa = "querySelectorAll";
	var qsaAvail = (
		!!getDoc()[qsa] && 
		// see #5832
		(!d.isSafari || (d.isSafari > 3.1) || is525 )
	);

	//Don't bother with n+3 type of matches, IE complains if we modify those.
	var infixSpaceRe = /n\+\d|([^ ])?([>~+])([^ =])?/g;
	var infixSpaceFunc = function(match, pre, ch, post) {
		return ch ? (pre ? pre + " " : "") + ch + (post ? " " + post : "") : /*n+3*/ match;
	};

	var getQueryFunc = function(query, forceDOM){
		//Normalize query. The CSS3 selectors spec allows for omitting spaces around
		//infix operators, >, ~ and +
		//Do the work here since detection for spaces is used as a simple "not use QSA"
		//test below.
		query = query.replace(infixSpaceRe, infixSpaceFunc);

		if(qsaAvail){
			// if we've got a cached variant and we think we can do it, run it!
			var qsaCached = _queryFuncCacheQSA[query];
			if(qsaCached && !forceDOM){ return qsaCached; }
		}

		// else if we've got a DOM cached variant, assume that we already know
		// all we need to and use it
		var domCached = _queryFuncCacheDOM[query];
		if(domCached){ return domCached; }

		// TODO: 
		//		today we're caching DOM and QSA branches separately so we
		//		recalc useQSA every time. If we had a way to tag root+query
		//		efficiently, we'd be in good shape to do a global cache.

		var qcz = query.charAt(0);
		var nospace = (-1 == query.indexOf(" "));

		// byId searches are wicked fast compared to QSA, even when filtering
		// is required
		if( (query.indexOf("#") >= 0) && (nospace) ){
			forceDOM = true;
		}

		var useQSA = ( 
			qsaAvail && (!forceDOM) &&
			// as per CSS 3, we can't currently start w/ combinator:
			//		http://www.w3.org/TR/css3-selectors/#w3cselgrammar
			(specials.indexOf(qcz) == -1) && 
			// IE's QSA impl sucks on pseudos
			(!d.isIE || (query.indexOf(":") == -1)) &&

			(!(cssCaseBug && (query.indexOf(".") >= 0))) &&

			// FIXME:
			//		need to tighten up browser rules on ":contains" and "|=" to
			//		figure out which aren't good
			//		Latest webkit (around 531.21.8) does not seem to do well with :checked on option
			//		elements, even though according to spec, selected options should
			//		match :checked. So go nonQSA for it:
			//		http://bugs.dojotoolkit.org/ticket/5179
			(query.indexOf(":contains") == -1) && (query.indexOf(":checked") == -1) && 
			(query.indexOf("|=") == -1) // some browsers don't grok it
		);

		// TODO: 
		//		if we've got a descendant query (e.g., "> .thinger" instead of
		//		just ".thinger") in a QSA-able doc, but are passed a child as a
		//		root, it should be possible to give the item a synthetic ID and
		//		trivially rewrite the query to the form "#synid > .thinger" to
		//		use the QSA branch


		if(useQSA){
			var tq = (specials.indexOf(query.charAt(query.length-1)) >= 0) ? 
						(query + " *") : query;
			return _queryFuncCacheQSA[query] = function(root){
				try{
					// the QSA system contains an egregious spec bug which
					// limits us, effectively, to only running QSA queries over
					// entire documents.  See:
					//		http://ejohn.org/blog/thoughts-on-queryselectorall/
					//	despite this, we can also handle QSA runs on simple
					//	selectors, but we don't want detection to be expensive
					//	so we're just checking for the presence of a space char
					//	right now. Not elegant, but it's cheaper than running
					//	the query parser when we might not need to
					if(!((9 == root.nodeType) || nospace)){ throw ""; }
					var r = root[qsa](tq);
					// skip expensive duplication checks and just wrap in a NodeList
					r[noZip] = true;
					return r;
				}catch(e){
					// else run the DOM branch on this query, ensuring that we
					// default that way in the future
					return getQueryFunc(query, true)(root);
				}
			}
		}else{
			// DOM branch
			var parts = query.split(/\s*,\s*/);
			return _queryFuncCacheDOM[query] = ((parts.length < 2) ? 
				// if not a compound query (e.g., ".foo, .bar"), cache and return a dispatcher
				getStepQueryFunc(query) : 
				// if it *is* a complex query, break it up into its
				// constituent parts and return a dispatcher that will
				// merge the parts when run
				function(root){
					var pindex = 0, // avoid array alloc for every invocation
						ret = [],
						tp;
					while((tp = parts[pindex++])){
						ret = ret.concat(getStepQueryFunc(tp)(root));
					}
					return ret;
				}
			);
		}
	};

	var _zipIdx = 0;

	// NOTE:
	//		this function is Moo inspired, but our own impl to deal correctly
	//		with XML in IE
	var _nodeUID = d.isIE ? function(node){
		if(caseSensitive){
			// XML docs don't have uniqueID on their nodes
			return (node.getAttribute("_uid") || node.setAttribute("_uid", ++_zipIdx) || _zipIdx);

		}else{
			return node.uniqueID;
		}
	} : 
	function(node){
		return (node._uid || (node._uid = ++_zipIdx));
	};

	// determine if a node in is unique in a "bag". In this case we don't want
	// to flatten a list of unique items, but rather just tell if the item in
	// question is already in the bag. Normally we'd just use hash lookup to do
	// this for us but IE's DOM is busted so we can't really count on that. On
	// the upside, it gives us a built in unique ID function. 
	var _isUnique = function(node, bag){
		if(!bag){ return 1; }
		var id = _nodeUID(node);
		if(!bag[id]){ return bag[id] = 1; }
		return 0;
	};

	// attempt to efficiently determine if an item in a list is a dupe,
	// returning a list of "uniques", hopefully in doucment order
	var _zipIdxName = "_zipIdx";
	var _zip = function(arr){
		if(arr && arr.nozip){ 
			return (qlc._wrap) ? qlc._wrap(arr) : arr;
		}
		// var ret = new d._NodeListCtor();
		var ret = new qlc();
		if(!arr || !arr.length){ return ret; }
		if(arr[0]){
			ret.push(arr[0]);
		}
		if(arr.length < 2){ return ret; }

		_zipIdx++;
		
		// we have to fork here for IE and XML docs because we can't set
		// expandos on their nodes (apparently). *sigh*
		if(d.isIE && caseSensitive){
			var szidx = _zipIdx+"";
			arr[0].setAttribute(_zipIdxName, szidx);
			for(var x = 1, te; te = arr[x]; x++){
				if(arr[x].getAttribute(_zipIdxName) != szidx){ 
					ret.push(te);
				}
				te.setAttribute(_zipIdxName, szidx);
			}
		}else if(d.isIE && arr.commentStrip){
			try{
				for(var x = 1, te; te = arr[x]; x++){
					if(_isElement(te)){ 
						ret.push(te);
					}
				}
			}catch(e){ /* squelch */ }
		}else{
			if(arr[0]){ arr[0][_zipIdxName] = _zipIdx; }
			for(var x = 1, te; te = arr[x]; x++){
				if(arr[x][_zipIdxName] != _zipIdx){ 
					ret.push(te);
				}
				te[_zipIdxName] = _zipIdx;
			}
		}
		return ret;
	};

	// the main executor
	d.query = function(/*String*/ query, /*String|DOMNode?*/ root){
		//	summary:
		//		Returns nodes which match the given CSS3 selector, searching the
		//		entire document by default but optionally taking a node to scope
		//		the search by. Returns an instance of dojo.NodeList.
		//	description:
		//		dojo.query() is the swiss army knife of DOM node manipulation in
		//		Dojo. Much like Prototype's "$$" (bling-bling) function or JQuery's
		//		"$" function, dojo.query provides robust, high-performance
		//		CSS-based node selector support with the option of scoping searches
		//		to a particular sub-tree of a document.
		//
		//		Supported Selectors:
		//		--------------------
		//
		//		dojo.query() supports a rich set of CSS3 selectors, including:
		//
		//			* class selectors (e.g., `.foo`)
		//			* node type selectors like `span`
		//			* ` ` descendant selectors
		//			* `>` child element selectors 
		//			* `#foo` style ID selectors
		//			* `*` universal selector
		//			* `~`, the immediately preceeded-by sibling selector
		//			* `+`, the preceeded-by sibling selector
		//			* attribute queries:
		//			|	* `[foo]` attribute presence selector
		//			|	* `[foo='bar']` attribute value exact match
		//			|	* `[foo~='bar']` attribute value list item match
		//			|	* `[foo^='bar']` attribute start match
		//			|	* `[foo$='bar']` attribute end match
		//			|	* `[foo*='bar']` attribute substring match
		//			* `:first-child`, `:last-child`, and `:only-child` positional selectors
		//			* `:empty` content emtpy selector
		//			* `:checked` pseudo selector
		//			* `:nth-child(n)`, `:nth-child(2n+1)` style positional calculations
		//			* `:nth-child(even)`, `:nth-child(odd)` positional selectors
		//			* `:not(...)` negation pseudo selectors
		//
		//		Any legal combination of these selectors will work with
		//		`dojo.query()`, including compound selectors ("," delimited).
		//		Very complex and useful searches can be constructed with this
		//		palette of selectors and when combined with functions for
		//		manipulation presented by dojo.NodeList, many types of DOM
		//		manipulation operations become very straightforward.
		//		
		//		Unsupported Selectors:
		//		----------------------
		//
		//		While dojo.query handles many CSS3 selectors, some fall outside of
		//		what's resaonable for a programmatic node querying engine to
		//		handle. Currently unsupported selectors include:
		//		
		//			* namespace-differentiated selectors of any form
		//			* all `::` pseduo-element selectors
		//			* certain pseduo-selectors which don't get a lot of day-to-day use:
		//			|	* `:root`, `:lang()`, `:target`, `:focus`
		//			* all visual and state selectors:
		//			|	* `:root`, `:active`, `:hover`, `:visisted`, `:link`,
		//				  `:enabled`, `:disabled`
		//			* `:*-of-type` pseudo selectors
		//		
		//		dojo.query and XML Documents:
		//		-----------------------------
		//		
		//		`dojo.query` (as of dojo 1.2) supports searching XML documents
		//		in a case-sensitive manner. If an HTML document is served with
		//		a doctype that forces case-sensitivity (e.g., XHTML 1.1
		//		Strict), dojo.query() will detect this and "do the right
		//		thing". Case sensitivity is dependent upon the document being
		//		searched and not the query used. It is therefore possible to
		//		use case-sensitive queries on strict sub-documents (iframes,
		//		etc.) or XML documents while still assuming case-insensitivity
		//		for a host/root document.
		//
		//		Non-selector Queries:
		//		---------------------
		//
		//		If something other than a String is passed for the query,
		//		`dojo.query` will return a new `dojo.NodeList` instance
		//		constructed from that parameter alone and all further
		//		processing will stop. This means that if you have a reference
		//		to a node or NodeList, you can quickly construct a new NodeList
		//		from the original by calling `dojo.query(node)` or
		//		`dojo.query(list)`.
		//
		//	query:
		//		The CSS3 expression to match against. For details on the syntax of
		//		CSS3 selectors, see <http://www.w3.org/TR/css3-selectors/#selectors>
		//	root:
		//		A DOMNode (or node id) to scope the search from. Optional.
		//	returns: dojo.NodeList
		//		An instance of `dojo.NodeList`. Many methods are available on
		//		NodeLists for searching, iterating, manipulating, and handling
		//		events on the matched nodes in the returned list.
		//	example:
		//		search the entire document for elements with the class "foo":
		//	|	dojo.query(".foo");
		//		these elements will match:
		//	|	<span class="foo"></span>
		//	|	<span class="foo bar"></span>
		//	|	<p class="thud foo"></p>
		//	example:
		//		search the entire document for elements with the classes "foo" *and* "bar":
		//	|	dojo.query(".foo.bar");
		//		these elements will match:
		//	|	<span class="foo bar"></span>
		//		while these will not:
		//	|	<span class="foo"></span>
		//	|	<p class="thud foo"></p>
		//	example:
		//		find `<span>` elements which are descendants of paragraphs and
		//		which have a "highlighted" class:
		//	|	dojo.query("p span.highlighted");
		//		the innermost span in this fragment matches:
		//	|	<p class="foo">
		//	|		<span>...
		//	|			<span class="highlighted foo bar">...</span>
		//	|		</span>
		//	|	</p>
		//	example:
		//		set an "odd" class on all odd table rows inside of the table
		//		`#tabular_data`, using the `>` (direct child) selector to avoid
		//		affecting any nested tables:
		//	|	dojo.query("#tabular_data > tbody > tr:nth-child(odd)").addClass("odd");
		//	example:
		//		remove all elements with the class "error" from the document
		//		and store them in a list:
		//	|	var errors = dojo.query(".error").orphan();
		//	example:
		//		add an onclick handler to every submit button in the document
		//		which causes the form to be sent via Ajax instead:
		//	|	dojo.query("input[type='submit']").onclick(function(e){
		//	|		dojo.stopEvent(e); // prevent sending the form
		//	|		var btn = e.target;
		//	|		dojo.xhrPost({
		//	|			form: btn.form,
		//	|			load: function(data){
		//	|				// replace the form with the response
		//	|				var div = dojo.doc.createElement("div");
		//	|				dojo.place(div, btn.form, "after");
		//	|				div.innerHTML = data;
		//	|				dojo.style(btn.form, "display", "none");
		//	|			}
		//	|		});
		//	|	});

		//Set list constructor to desired value. This can change
		//between calls, so always re-assign here.
		qlc = d._NodeListCtor;

		if(!query){
			return new qlc();
		}

		if(query.constructor == qlc){
			return query;
		}
		if(typeof query != "string"){ // inline'd type check
			return new qlc(query); // dojo.NodeList
		}
		if(typeof root == "string"){ // inline'd type check
			root = d.byId(root);
			if(!root){ return new qlc(); }
		}

		root = root||getDoc();
		var od = root.ownerDocument||root.documentElement;

		// throw the big case sensitivity switch

		// NOTE:
		// 		Opera in XHTML mode doesn't detect case-sensitivity correctly
		// 		and it's not clear that there's any way to test for it
		caseSensitive = (root.contentType && root.contentType=="application/xml") || 
						(d.isOpera && (root.doctype || od.toString() == "[object XMLDocument]")) ||
						(!!od) && 
						(d.isIE ? od.xml : (root.xmlVersion||od.xmlVersion));

		// NOTE: 
		//		adding "true" as the 2nd argument to getQueryFunc is useful for
		//		testing the DOM branch without worrying about the
		//		behavior/performance of the QSA branch.
		var r = getQueryFunc(query)(root);

		// FIXME:
		//		need to investigate this branch WRT #8074 and #8075
		if(r && r.nozip && !qlc._wrap){
			return r;
		}
		return _zip(r); // dojo.NodeList
	}

	// FIXME: need to add infrastructure for post-filtering pseudos, ala :last
	d.query.pseudos = pseudos;

	// one-off function for filtering a NodeList based on a simple selector
	d._filterQueryResult = function(nodeList, simpleFilter){
		var tmpNodeList = new d._NodeListCtor();
		var filterFunc = getSimpleFilterFunc(getQueryParts(simpleFilter)[0]);
		for(var x = 0, te; te = nodeList[x]; x++){
			if(filterFunc(te)){ tmpNodeList.push(te); }
		}
		return tmpNodeList;
	}
})(this["queryPortability"]||this["acme"]||dojo);

/*
*/

}

if(!dojo._hasResource["dojo._base.xhr"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.xhr"] = true;
dojo.provide("dojo._base.xhr");





(function(){
	var _d = dojo, cfg = _d.config;

	function setValue(/*Object*/obj, /*String*/name, /*String*/value){
		//summary:
		//		For the named property in object, set the value. If a value
		//		already exists and it is a string, convert the value to be an
		//		array of values.

		//Skip it if there is no value
		if(value === null){
			return;
		}

		var val = obj[name];
		if(typeof val == "string"){ // inline'd type check
			obj[name] = [val, value];
		}else if(_d.isArray(val)){
			val.push(value);
		}else{
			obj[name] = value;
		}
	}
	
	dojo.fieldToObject = function(/*DOMNode||String*/ inputNode){
		// summary:
		//		Serialize a form field to a JavaScript object.
		//
		// description:
		//		Returns the value encoded in a form field as
		//		as a string or an array of strings. Disabled form elements
		//		and unchecked radio and checkboxes are skipped.	Multi-select
		//		elements are returned as an array of string values.
		var ret = null;
		var item = _d.byId(inputNode);
		if(item){
			var _in = item.name;
			var type = (item.type||"").toLowerCase();
			if(_in && type && !item.disabled){
				if(type == "radio" || type == "checkbox"){
					if(item.checked){ ret = item.value }
				}else if(item.multiple){
					ret = [];
					_d.query("option", item).forEach(function(opt){
						if(opt.selected){
							ret.push(opt.value);
						}
					});
				}else{
					ret = item.value;
				}
			}
		}
		return ret; // Object
	}

	dojo.formToObject = function(/*DOMNode||String*/ formNode){
		// summary:
		//		Serialize a form node to a JavaScript object.
		// description:
		//		Returns the values encoded in an HTML form as
		//		string properties in an object which it then returns. Disabled form
		//		elements, buttons, and other non-value form elements are skipped.
		//		Multi-select elements are returned as an array of string values.
		//
		// example:
		//		This form:
		//		|	<form id="test_form">
		//		|		<input type="text" name="blah" value="blah">
		//		|		<input type="text" name="no_value" value="blah" disabled>
		//		|		<input type="button" name="no_value2" value="blah">
		//		|		<select type="select" multiple name="multi" size="5">
		//		|			<option value="blah">blah</option>
		//		|			<option value="thud" selected>thud</option>
		//		|			<option value="thonk" selected>thonk</option>
		//		|		</select>
		//		|	</form>
		//
		//		yields this object structure as the result of a call to
		//		formToObject():
		//
		//		|	{ 
		//		|		blah: "blah",
		//		|		multi: [
		//		|			"thud",
		//		|			"thonk"
		//		|		]
		//		|	};

		var ret = {};
		var exclude = "file|submit|image|reset|button|";
		_d.forEach(dojo.byId(formNode).elements, function(item){
			var _in = item.name;
			var type = (item.type||"").toLowerCase();
			if(_in && type && exclude.indexOf(type) == -1 && !item.disabled){
				setValue(ret, _in, _d.fieldToObject(item));
				if(type == "image"){
					ret[_in+".x"] = ret[_in+".y"] = ret[_in].x = ret[_in].y = 0;
				}
			}
		});
		return ret; // Object
	}

	dojo.objectToQuery = function(/*Object*/ map){
		//	summary:
		//		takes a name/value mapping object and returns a string representing
		//		a URL-encoded version of that object.
		//	example:
		//		this object:
		//
		//		|	{ 
		//		|		blah: "blah",
		//		|		multi: [
		//		|			"thud",
		//		|			"thonk"
		//		|		]
		//		|	};
		//
		//	yields the following query string:
		//	
		//	|	"blah=blah&multi=thud&multi=thonk"

		// FIXME: need to implement encodeAscii!!
		var enc = encodeURIComponent;
		var pairs = [];
		var backstop = {};
		for(var name in map){
			var value = map[name];
			if(value != backstop[name]){
				var assign = enc(name) + "=";
				if(_d.isArray(value)){
					for(var i=0; i < value.length; i++){
						pairs.push(assign + enc(value[i]));
					}
				}else{
					pairs.push(assign + enc(value));
				}
			}
		}
		return pairs.join("&"); // String
	}

	dojo.formToQuery = function(/*DOMNode||String*/ formNode){
		// summary:
		//		Returns a URL-encoded string representing the form passed as either a
		//		node or string ID identifying the form to serialize
		return _d.objectToQuery(_d.formToObject(formNode)); // String
	}

	dojo.formToJson = function(/*DOMNode||String*/ formNode, /*Boolean?*/prettyPrint){
		// summary:
		//		Create a serialized JSON string from a form node or string
		//		ID identifying the form to serialize
		return _d.toJson(_d.formToObject(formNode), prettyPrint); // String
	}

	dojo.queryToObject = function(/*String*/ str){
		// summary:
		//		Create an object representing a de-serialized query section of a
		//		URL. Query keys with multiple values are returned in an array.
		//
		// example:
		//		This string:
		//
		//	|		"foo=bar&foo=baz&thinger=%20spaces%20=blah&zonk=blarg&"
		//		
		//		results in this object structure:
		//
		//	|		{
		//	|			foo: [ "bar", "baz" ],
		//	|			thinger: " spaces =blah",
		//	|			zonk: "blarg"
		//	|		}
		//	
		//		Note that spaces and other urlencoded entities are correctly
		//		handled.

		// FIXME: should we grab the URL string if we're not passed one?
		var ret = {};
		var qp = str.split("&");
		var dec = decodeURIComponent;
		_d.forEach(qp, function(item){
			if(item.length){
				var parts = item.split("=");
				var name = dec(parts.shift());
				var val = dec(parts.join("="));
				if(typeof ret[name] == "string"){ // inline'd type check
					ret[name] = [ret[name]];
				}

				if(_d.isArray(ret[name])){
					ret[name].push(val);
				}else{
					ret[name] = val;
				}
			}
		});
		return ret; // Object
	}

	// need to block async callbacks from snatching this thread as the result
	// of an async callback might call another sync XHR, this hangs khtml forever
	// must checked by watchInFlight()

	dojo._blockAsync = false;

	// MOW: remove dojo._contentHandlers alias in 2.0
	var handlers = _d._contentHandlers = dojo.contentHandlers = {
		// summary: 
		//		A map of availble XHR transport handle types. Name matches the
		//		`handleAs` attribute passed to XHR calls.
		//
		// description:
		//		A map of availble XHR transport handle types. Name matches the
		//		`handleAs` attribute passed to XHR calls. Each contentHandler is
		//		called, passing the xhr object for manipulation. The return value
		//		from the contentHandler will be passed to the `load` or `handle` 
		//		functions defined in the original xhr call. 
		//		
		// example:
		//		Creating a custom content-handler:
		//	|	dojo.contentHandlers.makeCaps = function(xhr){
		//	|		return xhr.responseText.toUpperCase();
		//	|	}
		//	|	// and later:
		//	|	dojo.xhrGet({ 
		//	|		url:"foo.txt",
		//	|		handleAs:"makeCaps",
		//	|		load: function(data){ /* data is a toUpper version of foo.txt */ }
		//	|	});

		text: function(xhr){ 
			// summary: A contentHandler which simply returns the plaintext response data
			return xhr.responseText; 
		},
		json: function(xhr){
			// summary: A contentHandler which returns a JavaScript object created from the response data
			return _d.fromJson(xhr.responseText || null);
		},
		"json-comment-filtered": function(xhr){ 
			// summary: A contentHandler which expects comment-filtered JSON. 
			// description: 
			//		A contentHandler which expects comment-filtered JSON. 
			//		the json-comment-filtered option was implemented to prevent
			//		"JavaScript Hijacking", but it is less secure than standard JSON. Use
			//		standard JSON instead. JSON prefixing can be used to subvert hijacking.
			//		
			//		Will throw a notice suggesting to use application/json mimetype, as
			//		json-commenting can introduce security issues. To decrease the chances of hijacking,
			//		use the standard `json` contentHandler, and prefix your "JSON" with: {}&& 
			//		
			//		use djConfig.useCommentedJson = true to turn off the notice
			if(!dojo.config.useCommentedJson){
				console.warn("Consider using the standard mimetype:application/json."
					+ " json-commenting can introduce security issues. To"
					+ " decrease the chances of hijacking, use the standard the 'json' handler and"
					+ " prefix your json with: {}&&\n"
					+ "Use djConfig.useCommentedJson=true to turn off this message.");
			}

			var value = xhr.responseText;
			var cStartIdx = value.indexOf("\/*");
			var cEndIdx = value.lastIndexOf("*\/");
			if(cStartIdx == -1 || cEndIdx == -1){
				throw new Error("JSON was not comment filtered");
			}
			return _d.fromJson(value.substring(cStartIdx+2, cEndIdx));
		},
		javascript: function(xhr){ 
			// summary: A contentHandler which evaluates the response data, expecting it to be valid JavaScript

			// FIXME: try Moz and IE specific eval variants?
			return _d.eval(xhr.responseText);
		},
		xml: function(xhr){
			// summary: A contentHandler returning an XML Document parsed from the response data
			var result = xhr.responseXML;
						if(_d.isIE && (!result || !result.documentElement)){
				//WARNING: this branch used by the xml handling in dojo.io.iframe,
				//so be sure to test dojo.io.iframe if making changes below.
				var ms = function(n){ return "MSXML" + n + ".DOMDocument"; }
				var dp = ["Microsoft.XMLDOM", ms(6), ms(4), ms(3), ms(2)];
				_d.some(dp, function(p){
					try{
						var dom = new ActiveXObject(p);
						dom.async = false;
						dom.loadXML(xhr.responseText);
						result = dom;
					}catch(e){ return false; }
					return true;
				});
			}
						return result; // DOMDocument
		},
		"json-comment-optional": function(xhr){
			// summary: A contentHandler which checks the presence of comment-filtered JSON and 
			//		alternates between the `json` and `json-comment-filtered` contentHandlers.
			if(xhr.responseText && /^[^{\[]*\/\*/.test(xhr.responseText)){
				return handlers["json-comment-filtered"](xhr);
			}else{
				return handlers["json"](xhr);
			}
		}
	};

	/*=====
	dojo.__IoArgs = function(){
		//	url: String
		//		URL to server endpoint.
		//	content: Object?
		//		Contains properties with string values. These
		//		properties will be serialized as name1=value2 and
		//		passed in the request.
		//	timeout: Integer?
		//		Milliseconds to wait for the response. If this time
		//		passes, the then error callbacks are called.
		//	form: DOMNode?
		//		DOM node for a form. Used to extract the form values
		//		and send to the server.
		//	preventCache: Boolean?
		//		Default is false. If true, then a
		//		"dojo.preventCache" parameter is sent in the request
		//		with a value that changes with each request
		//		(timestamp). Useful only with GET-type requests.
		//	handleAs: String?
		//		Acceptable values depend on the type of IO
		//		transport (see specific IO calls for more information).
		// 	rawBody: String?
		// 		Sets the raw body for an HTTP request. If this is used, then the content
		// 		property is ignored. This is mostly useful for HTTP methods that have
		// 		a body to their requests, like PUT or POST. This property can be used instead
		// 		of postData and putData for dojo.rawXhrPost and dojo.rawXhrPut respectively.
		//	ioPublish: Boolean?
		//		Set this explicitly to false to prevent publishing of topics related to
		// 		IO operations. Otherwise, if djConfig.ioPublish is set to true, topics
		// 		will be published via dojo.publish for different phases of an IO operation.
		// 		See dojo.__IoPublish for a list of topics that are published.
		//	load: Function?
		//		This function will be
		//		called on a successful HTTP response code.
		//	error: Function?
		//		This function will
		//		be called when the request fails due to a network or server error, the url
		//		is invalid, etc. It will also be called if the load or handle callback throws an
		//		exception, unless djConfig.debugAtAllCosts is true.  This allows deployed applications
		//		to continue to run even when a logic error happens in the callback, while making
		//		it easier to troubleshoot while in debug mode.
		//	handle: Function?
		//		This function will
		//		be called at the end of every request, whether or not an error occurs.
		this.url = url;
		this.content = content;
		this.timeout = timeout;
		this.form = form;
		this.preventCache = preventCache;
		this.handleAs = handleAs;
		this.ioPublish = ioPublish;
		this.load = function(response, ioArgs){
			// ioArgs: dojo.__IoCallbackArgs
			//		Provides additional information about the request.
			// response: Object
			//		The response in the format as defined with handleAs.
		}
		this.error = function(response, ioArgs){
			// ioArgs: dojo.__IoCallbackArgs
			//		Provides additional information about the request.
			// response: Object
			//		The response in the format as defined with handleAs.
		}
		this.handle = function(loadOrError, response, ioArgs){
			// loadOrError: String
			//		Provides a string that tells you whether this function
			//		was called because of success (load) or failure (error).
			// response: Object
			//		The response in the format as defined with handleAs.
			// ioArgs: dojo.__IoCallbackArgs
			//		Provides additional information about the request.
		}
	}
	=====*/

	/*=====
	dojo.__IoCallbackArgs = function(args, xhr, url, query, handleAs, id, canDelete, json){
		//	args: Object
		//		the original object argument to the IO call.
		//	xhr: XMLHttpRequest
		//		For XMLHttpRequest calls only, the
		//		XMLHttpRequest object that was used for the
		//		request.
		//	url: String
		//		The final URL used for the call. Many times it
		//		will be different than the original args.url
		//		value.
		//	query: String
		//		For non-GET requests, the
		//		name1=value1&name2=value2 parameters sent up in
		//		the request.
		//	handleAs: String
		//		The final indicator on how the response will be
		//		handled.
		//	id: String
		//		For dojo.io.script calls only, the internal
		//		script ID used for the request.
		//	canDelete: Boolean
		//		For dojo.io.script calls only, indicates
		//		whether the script tag that represents the
		//		request can be deleted after callbacks have
		//		been called. Used internally to know when
		//		cleanup can happen on JSONP-type requests.
		//	json: Object
		//		For dojo.io.script calls only: holds the JSON
		//		response for JSONP-type requests. Used
		//		internally to hold on to the JSON responses.
		//		You should not need to access it directly --
		//		the same object should be passed to the success
		//		callbacks directly.
		this.args = args;
		this.xhr = xhr;
		this.url = url;
		this.query = query;
		this.handleAs = handleAs;
		this.id = id;
		this.canDelete = canDelete;
		this.json = json;
	}
	=====*/


	/*=====
	dojo.__IoPublish = function(){
		// 	summary:
		// 		This is a list of IO topics that can be published
		// 		if djConfig.ioPublish is set to true. IO topics can be
		// 		published for any Input/Output, network operation. So,
		// 		dojo.xhr, dojo.io.script and dojo.io.iframe can all
		// 		trigger these topics to be published.
		//	start: String
		//		"/dojo/io/start" is sent when there are no outstanding IO
		// 		requests, and a new IO request is started. No arguments
		// 		are passed with this topic.
		//	send: String
		//		"/dojo/io/send" is sent whenever a new IO request is started.
		// 		It passes the dojo.Deferred for the request with the topic.
		//	load: String
		//		"/dojo/io/load" is sent whenever an IO request has loaded
		// 		successfully. It passes the response and the dojo.Deferred
		// 		for the request with the topic.
		//	error: String
		//		"/dojo/io/error" is sent whenever an IO request has errored.
		// 		It passes the error and the dojo.Deferred
		// 		for the request with the topic.
		//	done: String
		//		"/dojo/io/done" is sent whenever an IO request has completed,
		// 		either by loading or by erroring. It passes the error and
		// 		the dojo.Deferred for the request with the topic.
		//	stop: String
		//		"/dojo/io/stop" is sent when all outstanding IO requests have
		// 		finished. No arguments are passed with this topic.
		this.start = "/dojo/io/start";
		this.send = "/dojo/io/send";
		this.load = "/dojo/io/load";
		this.error = "/dojo/io/error";
		this.done = "/dojo/io/done";
		this.stop = "/dojo/io/stop";
	}
	=====*/


	dojo._ioSetArgs = function(/*dojo.__IoArgs*/args,
			/*Function*/canceller,
			/*Function*/okHandler,
			/*Function*/errHandler){
		//	summary: 
		//		sets up the Deferred and ioArgs property on the Deferred so it
		//		can be used in an io call.
		//	args:
		//		The args object passed into the public io call. Recognized properties on
		//		the args object are:
		//	canceller:
		//		The canceller function used for the Deferred object. The function
		//		will receive one argument, the Deferred object that is related to the
		//		canceller.
		//	okHandler:
		//		The first OK callback to be registered with Deferred. It has the opportunity
		//		to transform the OK response. It will receive one argument -- the Deferred
		//		object returned from this function.
		//	errHandler:
		//		The first error callback to be registered with Deferred. It has the opportunity
		//		to do cleanup on an error. It will receive two arguments: error (the 
		//		Error object) and dfd, the Deferred object returned from this function.

		var ioArgs = {args: args, url: args.url};

		//Get values from form if requestd.
		var formObject = null;
		if(args.form){ 
			var form = _d.byId(args.form);
			//IE requires going through getAttributeNode instead of just getAttribute in some form cases, 
			//so use it for all.  See #2844
			var actnNode = form.getAttributeNode("action");
			ioArgs.url = ioArgs.url || (actnNode ? actnNode.value : null); 
			formObject = _d.formToObject(form);
		}

		// set up the query params
		var miArgs = [{}];
	
		if(formObject){
			// potentially over-ride url-provided params w/ form values
			miArgs.push(formObject);
		}
		if(args.content){
			// stuff in content over-rides what's set by form
			miArgs.push(args.content);
		}
		if(args.preventCache){
			miArgs.push({"dojo.preventCache": new Date().valueOf()});
		}
		ioArgs.query = _d.objectToQuery(_d.mixin.apply(null, miArgs));
	
		// .. and the real work of getting the deferred in order, etc.
		ioArgs.handleAs = args.handleAs || "text";
		var d = new _d.Deferred(canceller);
		d.addCallbacks(okHandler, function(error){
			return errHandler(error, d);
		});

		//Support specifying load, error and handle callback functions from the args.
		//For those callbacks, the "this" object will be the args object.
		//The callbacks will get the deferred result value as the
		//first argument and the ioArgs object as the second argument.
		var ld = args.load;
		if(ld && _d.isFunction(ld)){
			d.addCallback(function(value){
				return ld.call(args, value, ioArgs);
			});
		}
		var err = args.error;
		if(err && _d.isFunction(err)){
			d.addErrback(function(value){
				return err.call(args, value, ioArgs);
			});
		}
		var handle = args.handle;
		if(handle && _d.isFunction(handle)){
			d.addBoth(function(value){
				return handle.call(args, value, ioArgs);
			});
		}

		//Plug in topic publishing, if dojo.publish is loaded.
		if(cfg.ioPublish && _d.publish && ioArgs.args.ioPublish !== false){
			d.addCallbacks(
				function(res){
					_d.publish("/dojo/io/load", [d, res]);
					return res;
				},
				function(res){
					_d.publish("/dojo/io/error", [d, res]);
					return res;
				}
			);
			d.addBoth(function(res){
				_d.publish("/dojo/io/done", [d, res]);
				return res;
			});
		}

		d.ioArgs = ioArgs;
	
		// FIXME: need to wire up the xhr object's abort method to something
		// analagous in the Deferred
		return d;
	}

	var _deferredCancel = function(/*Deferred*/dfd){
		// summary: canceller function for dojo._ioSetArgs call.
		
		dfd.canceled = true;
		var xhr = dfd.ioArgs.xhr;
		var _at = typeof xhr.abort;
		if(_at == "function" || _at == "object" || _at == "unknown"){
			xhr.abort();
		}
		var err = dfd.ioArgs.error;
		if(!err){
			err = new Error("xhr cancelled");
			err.dojoType="cancel";
		}
		return err;
	}
	var _deferredOk = function(/*Deferred*/dfd){
		// summary: okHandler function for dojo._ioSetArgs call.

		var ret = handlers[dfd.ioArgs.handleAs](dfd.ioArgs.xhr);
		return ret === undefined ? null : ret;
	}
	var _deferError = function(/*Error*/error, /*Deferred*/dfd){
		// summary: errHandler function for dojo._ioSetArgs call.

		if(!dfd.ioArgs.args.failOk){
			console.error(error);
		}
		return error;
	}

	// avoid setting a timer per request. It degrades performance on IE
	// something fierece if we don't use unified loops.
	var _inFlightIntvl = null;
	var _inFlight = [];
	
	
	//Use a separate count for knowing if we are starting/stopping io calls.
	//Cannot use _inFlight.length since it can change at a different time than
	//when we want to do this kind of test. We only want to decrement the count
	//after a callback/errback has finished, since the callback/errback should be
	//considered as part of finishing a request.
	var _pubCount = 0;
	var _checkPubCount = function(dfd){
		if(_pubCount <= 0){
			_pubCount = 0;
			if(cfg.ioPublish && _d.publish && (!dfd || dfd && dfd.ioArgs.args.ioPublish !== false)){
				_d.publish("/dojo/io/stop");
			}
		}
	};

	var _watchInFlight = function(){
		//summary: 
		//		internal method that checks each inflight XMLHttpRequest to see
		//		if it has completed or if the timeout situation applies.
		
		var now = (new Date()).getTime();
		// make sure sync calls stay thread safe, if this callback is called
		// during a sync call and this results in another sync call before the
		// first sync call ends the browser hangs
		if(!_d._blockAsync){
			// we need manual loop because we often modify _inFlight (and therefore 'i') while iterating
			// note: the second clause is an assigment on purpose, lint may complain
			for(var i = 0, tif; i < _inFlight.length && (tif = _inFlight[i]); i++){
				var dfd = tif.dfd;
				var func = function(){
					if(!dfd || dfd.canceled || !tif.validCheck(dfd)){
						_inFlight.splice(i--, 1); 
						_pubCount -= 1;
					}else if(tif.ioCheck(dfd)){
						_inFlight.splice(i--, 1);
						tif.resHandle(dfd);
						_pubCount -= 1;
					}else if(dfd.startTime){
						//did we timeout?
						if(dfd.startTime + (dfd.ioArgs.args.timeout || 0) < now){
							_inFlight.splice(i--, 1);
							var err = new Error("timeout exceeded");
							err.dojoType = "timeout";
							dfd.errback(err);
							//Cancel the request so the io module can do appropriate cleanup.
							dfd.cancel();
							_pubCount -= 1;
						}
					}
				};
				if(dojo.config.debugAtAllCosts){
					func.call(this);
				}else{
					try{
						func.call(this);
					}catch(e){
						dfd.errback(e);
					}
				}
			}
		}

		_checkPubCount(dfd);

		if(!_inFlight.length){
			clearInterval(_inFlightIntvl);
			_inFlightIntvl = null;
			return;
		}
	}

	dojo._ioCancelAll = function(){
		//summary: Cancels all pending IO requests, regardless of IO type
		//(xhr, script, iframe).
		try{
			_d.forEach(_inFlight, function(i){
				try{
					i.dfd.cancel();
				}catch(e){/*squelch*/}
			});
		}catch(e){/*squelch*/}
	}

	//Automatically call cancel all io calls on unload
	//in IE for trac issue #2357.
		if(_d.isIE){
		_d.addOnWindowUnload(_d._ioCancelAll);
	}
	
	_d._ioNotifyStart = function(/*Deferred*/dfd){
		// summary:
		// 		If dojo.publish is available, publish topics
		// 		about the start of a request queue and/or the
		// 		the beginning of request.
		// description:
		// 		Used by IO transports. An IO transport should
		// 		call this method before making the network connection.
		if(cfg.ioPublish && _d.publish && dfd.ioArgs.args.ioPublish !== false){
			if(!_pubCount){
				_d.publish("/dojo/io/start");
			}
			_pubCount += 1;
			_d.publish("/dojo/io/send", [dfd]);
		}
	}

	_d._ioWatch = function(dfd, validCheck, ioCheck, resHandle){
		// summary: 
		//		Watches the io request represented by dfd to see if it completes.
		// dfd: Deferred
		//		The Deferred object to watch.
		// validCheck: Function
		//		Function used to check if the IO request is still valid. Gets the dfd
		//		object as its only argument.
		// ioCheck: Function
		//		Function used to check if basic IO call worked. Gets the dfd
		//		object as its only argument.
		// resHandle: Function
		//		Function used to process response. Gets the dfd
		//		object as its only argument.
		var args = dfd.ioArgs.args;
		if(args.timeout){
			dfd.startTime = (new Date()).getTime();
		}
		
		_inFlight.push({dfd: dfd, validCheck: validCheck, ioCheck: ioCheck, resHandle: resHandle});
		if(!_inFlightIntvl){
			_inFlightIntvl = setInterval(_watchInFlight, 50);
		}
		// handle sync requests
		//A weakness: async calls in flight
		//could have their handlers called as part of the
		//_watchInFlight call, before the sync's callbacks
		// are called.
		if(args.sync){
			_watchInFlight();
		}
	}

	var _defaultContentType = "application/x-www-form-urlencoded";

	var _validCheck = function(/*Deferred*/dfd){
		return dfd.ioArgs.xhr.readyState; //boolean
	}
	var _ioCheck = function(/*Deferred*/dfd){
		return 4 == dfd.ioArgs.xhr.readyState; //boolean
	}
	var _resHandle = function(/*Deferred*/dfd){
		var xhr = dfd.ioArgs.xhr;
		if(_d._isDocumentOk(xhr)){
			dfd.callback(dfd);
		}else{
			var err = new Error("Unable to load " + dfd.ioArgs.url + " status:" + xhr.status);
			err.status = xhr.status;
			err.responseText = xhr.responseText;
			dfd.errback(err);
		}
	}

	dojo._ioAddQueryToUrl = function(/*dojo.__IoCallbackArgs*/ioArgs){
		//summary: Adds query params discovered by the io deferred construction to the URL.
		//Only use this for operations which are fundamentally GET-type operations.
		if(ioArgs.query.length){
			ioArgs.url += (ioArgs.url.indexOf("?") == -1 ? "?" : "&") + ioArgs.query;
			ioArgs.query = null;
		}		
	}

	/*=====
	dojo.declare("dojo.__XhrArgs", dojo.__IoArgs, {
		constructor: function(){
			//	summary:
			//		In addition to the properties listed for the dojo._IoArgs type,
			//		the following properties are allowed for dojo.xhr* methods.
			//	handleAs: String?
			//		Acceptable values are: text (default), json, json-comment-optional,
			//		json-comment-filtered, javascript, xml. See `dojo.contentHandlers`
			//	sync: Boolean?
			//		false is default. Indicates whether the request should
			//		be a synchronous (blocking) request.
			//	headers: Object?
			//		Additional HTTP headers to send in the request.
			//	failOk: Boolean?
			//		false is default. Indicates whether a request should be
			//		allowed to fail (and therefore no console error message in
			//		the event of a failure)
			this.handleAs = handleAs;
			this.sync = sync;
			this.headers = headers;
			this.failOk = failOk;
		}
	});
	=====*/

	dojo.xhr = function(/*String*/ method, /*dojo.__XhrArgs*/ args, /*Boolean?*/ hasBody){
		//	summary:
		//		Sends an HTTP request with the given method.
		//	description:
		//		Sends an HTTP request with the given method.
		//		See also dojo.xhrGet(), xhrPost(), xhrPut() and dojo.xhrDelete() for shortcuts
		//		for those HTTP methods. There are also methods for "raw" PUT and POST methods
		//		via dojo.rawXhrPut() and dojo.rawXhrPost() respectively.
		//	method:
		//		HTTP method to be used, such as GET, POST, PUT, DELETE.  Should be uppercase.
		//	hasBody:
		//		If the request has an HTTP body, then pass true for hasBody.

		//Make the Deferred object for this xhr request.
		var dfd = _d._ioSetArgs(args, _deferredCancel, _deferredOk, _deferError);
		var ioArgs = dfd.ioArgs;

		//Pass the args to _xhrObj, to allow alternate XHR calls based specific calls, like
		//the one used for iframe proxies.
		var xhr = ioArgs.xhr = _d._xhrObj(ioArgs.args);
		//If XHR factory fails, cancel the deferred.
		if(!xhr){
			dfd.cancel();
			return dfd;
		}

		//Allow for specifying the HTTP body completely.
		if("postData" in args){
			ioArgs.query = args.postData;
		}else if("putData" in args){
			ioArgs.query = args.putData;
		}else if("rawBody" in args){
			ioArgs.query = args.rawBody;
		}else if((arguments.length > 2 && !hasBody) || "POST|PUT".indexOf(method.toUpperCase()) == -1){
			//Check for hasBody being passed. If no hasBody,
			//then only append query string if not a POST or PUT request.
			_d._ioAddQueryToUrl(ioArgs);
		}

		// IE 6 is a steaming pile. It won't let you call apply() on the native function (xhr.open).
		// workaround for IE6's apply() "issues"
		xhr.open(method, ioArgs.url, args.sync !== true, args.user || undefined, args.password || undefined);
		if(args.headers){
			for(var hdr in args.headers){
				if(hdr.toLowerCase() === "content-type" && !args.contentType){
					args.contentType = args.headers[hdr];
				}else if(args.headers[hdr]){
					//Only add header if it has a value. This allows for instnace, skipping
					//insertion of X-Requested-With by specifying empty value.
					xhr.setRequestHeader(hdr, args.headers[hdr]);
				}
			}
		}
		// FIXME: is this appropriate for all content types?
		xhr.setRequestHeader("Content-Type", args.contentType || _defaultContentType);
		if(!args.headers || !("X-Requested-With" in args.headers)){
			xhr.setRequestHeader("X-Requested-With", "XMLHttpRequest");
		}
		// FIXME: set other headers here!
		_d._ioNotifyStart(dfd);
		if(dojo.config.debugAtAllCosts){
			xhr.send(ioArgs.query);
		}else{
			try{
				xhr.send(ioArgs.query);
			}catch(e){
				ioArgs.error = e;
				dfd.cancel();
			}
		}
		_d._ioWatch(dfd, _validCheck, _ioCheck, _resHandle);
		xhr = null;
		return dfd; // dojo.Deferred
	}

	dojo.xhrGet = function(/*dojo.__XhrArgs*/ args){
		//	summary: 
		//		Sends an HTTP GET request to the server.
		return _d.xhr("GET", args); // dojo.Deferred
	}

	dojo.rawXhrPost = dojo.xhrPost = function(/*dojo.__XhrArgs*/ args){
		//	summary:
		//		Sends an HTTP POST request to the server. In addtion to the properties
		//		listed for the dojo.__XhrArgs type, the following property is allowed:
		//	postData:
		//		String. Send raw data in the body of the POST request.
		return _d.xhr("POST", args, true); // dojo.Deferred
	}

	dojo.rawXhrPut = dojo.xhrPut = function(/*dojo.__XhrArgs*/ args){
		//	summary:
		//		Sends an HTTP PUT request to the server. In addtion to the properties
		//		listed for the dojo.__XhrArgs type, the following property is allowed:
		//	putData:
		//		String. Send raw data in the body of the PUT request.
		return _d.xhr("PUT", args, true); // dojo.Deferred
	}

	dojo.xhrDelete = function(/*dojo.__XhrArgs*/ args){
		//	summary:
		//		Sends an HTTP DELETE request to the server.
		return _d.xhr("DELETE", args); //dojo.Deferred
	}

	/*
	dojo.wrapForm = function(formNode){
		//summary:
		//		A replacement for FormBind, but not implemented yet.

		// FIXME: need to think harder about what extensions to this we might
		// want. What should we allow folks to do w/ this? What events to
		// set/send?
		throw new Error("dojo.wrapForm not yet implemented");
	}
	*/
})();

}

if(!dojo._hasResource["dojo._base.fx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.fx"] = true;
dojo.provide("dojo._base.fx");





/*
	Animation loosely package based on Dan Pupius' work, contributed under CLA:
		http://pupius.co.uk/js/Toolkit.Drawing.js
*/
(function(){
	var d = dojo;
	var _mixin = d._mixin;

	dojo._Line = function(/*int*/ start, /*int*/ end){
		//	summary:
		//		dojo._Line is the object used to generate values from a start value
		//		to an end value
		//	start: int
		//		Beginning value for range
		//	end: int
		//		Ending value for range
		this.start = start;
		this.end = end;
	};

	dojo._Line.prototype.getValue = function(/*float*/ n){
		//	summary: Returns the point on the line
		//	n: a floating point number greater than 0 and less than 1
		return ((this.end - this.start) * n) + this.start; // Decimal
	};

	dojo.Animation = function(args){
		//	summary:
		//		A generic animation class that fires callbacks into its handlers
		//		object at various states.
		//	description:
		//		A generic animation class that fires callbacks into its handlers
		//		object at various states. Nearly all dojo animation functions
		//		return an instance of this method, usually without calling the
		//		.play() method beforehand. Therefore, you will likely need to
		//		call .play() on instances of `dojo.Animation` when one is
		//		returned.
		// args: Object
		//		The 'magic argument', mixing all the properties into this
		//		animation instance.

		_mixin(this, args);
		if(d.isArray(this.curve)){
			this.curve = new d._Line(this.curve[0], this.curve[1]);
		}

	};

	// Alias to drop come 2.0:
	d._Animation = d.Animation;

	d.extend(dojo.Animation, {
		// duration: Integer
		//		The time in milliseonds the animation will take to run
		duration: 350,

	/*=====
		// curve: dojo._Line|Array
		//		A two element array of start and end values, or a `dojo._Line` instance to be
		//		used in the Animation.
		curve: null,

		// easing: Function?
		//		A Function to adjust the acceleration (or deceleration) of the progress
		//		across a dojo._Line
		easing: null,
	=====*/

		// repeat: Integer?
		//		The number of times to loop the animation
		repeat: 0,

		// rate: Integer?
		//		the time in milliseconds to wait before advancing to next frame
		//		(used as a fps timer: 1000/rate = fps)
		rate: 20 /* 50 fps */,

	/*=====
		// delay: Integer?
		//		The time in milliseconds to wait before starting animation after it
		//		has been .play()'ed
		delay: null,

		// beforeBegin: Event?
		//		Synthetic event fired before a dojo.Animation begins playing (synchronous)
		beforeBegin: null,

		// onBegin: Event?
		//		Synthetic event fired as a dojo.Animation begins playing (useful?)
		onBegin: null,

		// onAnimate: Event?
		//		Synthetic event fired at each interval of a `dojo.Animation`
		onAnimate: null,

		// onEnd: Event?
		//		Synthetic event fired after the final frame of a `dojo.Animation`
		onEnd: null,

		// onPlay: Event?
		//		Synthetic event fired any time a `dojo.Animation` is play()'ed
		onPlay: null,

		// onPause: Event?
		//		Synthetic event fired when a `dojo.Animation` is paused
		onPause: null,

		// onStop: Event
		//		Synthetic event fires when a `dojo.Animation` is stopped
		onStop: null,

	=====*/

		_percent: 0,
		_startRepeatCount: 0,

		_getStep: function(){
			var _p = this._percent,
				_e = this.easing
			;
			return _e ? _e(_p) : _p;
		},
		_fire: function(/*Event*/ evt, /*Array?*/ args){
			//	summary:
			//		Convenience function.  Fire event "evt" and pass it the
			//		arguments specified in "args".
			//	description:
			//		Convenience function.  Fire event "evt" and pass it the
			//		arguments specified in "args".
			//		Fires the callback in the scope of the `dojo.Animation`
			//		instance.
			//	evt:
			//		The event to fire.
			//	args:
			//		The arguments to pass to the event.
			var a = args||[];
			if(this[evt]){
				if(d.config.debugAtAllCosts){
					this[evt].apply(this, a);
				}else{
					try{
						this[evt].apply(this, a);
					}catch(e){
						// squelch and log because we shouldn't allow exceptions in
						// synthetic event handlers to cause the internal timer to run
						// amuck, potentially pegging the CPU. I'm not a fan of this
						// squelch, but hopefully logging will make it clear what's
						// going on
						console.error("exception in animation handler for:", evt);
						console.error(e);
					}
				}
			}
			return this; // dojo.Animation
		},

		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			// summary:
			//		Start the animation.
			// delay:
			//		How many milliseconds to delay before starting.
			// gotoStart:
			//		If true, starts the animation from the beginning; otherwise,
			//		starts it from its current position.
			// returns: dojo.Animation
			//		The instance to allow chaining.

			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			if(gotoStart){
				_t._stopTimer();
				_t._active = _t._paused = false;
				_t._percent = 0;
			}else if(_t._active && !_t._paused){
				return _t;
			}

			_t._fire("beforeBegin", [_t.node]);

			var de = delay || _t.delay,
				_p = dojo.hitch(_t, "_play", gotoStart);

			if(de > 0){
				_t._delayTimer = setTimeout(_p, de);
				return _t;
			}
			_p();
			return _t;
		},

		_play: function(gotoStart){
			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			_t._startTime = new Date().valueOf();
			if(_t._paused){
				_t._startTime -= _t.duration * _t._percent;
			}

			_t._active = true;
			_t._paused = false;
			var value = _t.curve.getValue(_t._getStep());
			if(!_t._percent){
				if(!_t._startRepeatCount){
					_t._startRepeatCount = _t.repeat;
				}
				_t._fire("onBegin", [value]);
			}

			_t._fire("onPlay", [value]);

			_t._cycle();
			return _t; // dojo.Animation
		},

		pause: function(){
			// summary: Pauses a running animation.
			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			_t._stopTimer();
			if(!_t._active){ return _t; /*dojo.Animation*/ }
			_t._paused = true;
			_t._fire("onPause", [_t.curve.getValue(_t._getStep())]);
			return _t; // dojo.Animation
		},

		gotoPercent: function(/*Decimal*/ percent, /*Boolean?*/ andPlay){
			//	summary:
			//		Sets the progress of the animation.
			//	percent:
			//		A percentage in decimal notation (between and including 0.0 and 1.0).
			//	andPlay:
			//		If true, play the animation after setting the progress.
			var _t = this;
			_t._stopTimer();
			_t._active = _t._paused = true;
			_t._percent = percent;
			if(andPlay){ _t.play(); }
			return _t; // dojo.Animation
		},

		stop: function(/*boolean?*/ gotoEnd){
			// summary: Stops a running animation.
			// gotoEnd: If true, the animation will end.
			var _t = this;
			if(_t._delayTimer){ _t._clearTimer(); }
			if(!_t._timer){ return _t; /* dojo.Animation */ }
			_t._stopTimer();
			if(gotoEnd){
				_t._percent = 1;
			}
			_t._fire("onStop", [_t.curve.getValue(_t._getStep())]);
			_t._active = _t._paused = false;
			return _t; // dojo.Animation
		},

		status: function(){
			// summary:
			//		Returns a string token representation of the status of
			//		the animation, one of: "paused", "playing", "stopped"
			if(this._active){
				return this._paused ? "paused" : "playing"; // String
			}
			return "stopped"; // String
		},

		_cycle: function(){
			var _t = this;
			if(_t._active){
				var curr = new Date().valueOf();
				var step = (curr - _t._startTime) / (_t.duration);

				if(step >= 1){
					step = 1;
				}
				_t._percent = step;

				// Perform easing
				if(_t.easing){
					step = _t.easing(step);
				}

				_t._fire("onAnimate", [_t.curve.getValue(step)]);

				if(_t._percent < 1){
					_t._startTimer();
				}else{
					_t._active = false;

					if(_t.repeat > 0){
						_t.repeat--;
						_t.play(null, true);
					}else if(_t.repeat == -1){
						_t.play(null, true);
					}else{
						if(_t._startRepeatCount){
							_t.repeat = _t._startRepeatCount;
							_t._startRepeatCount = 0;
						}
					}
					_t._percent = 0;
					_t._fire("onEnd", [_t.node]);
					!_t.repeat && _t._stopTimer();
				}
			}
			return _t; // dojo.Animation
		},

		_clearTimer: function(){
			// summary: Clear the play delay timer
			clearTimeout(this._delayTimer);
			delete this._delayTimer;
		}

	});

	// the local timer, stubbed into all Animation instances
	var ctr = 0,
		timer = null,
		runner = {
			run: function(){}
		};

	d.extend(d.Animation, {

		_startTimer: function(){
			if(!this._timer){
				this._timer = d.connect(runner, "run", this, "_cycle");
				ctr++;
			}
			if(!timer){
				timer = setInterval(d.hitch(runner, "run"), this.rate);
			}
		},

		_stopTimer: function(){
			if(this._timer){
				d.disconnect(this._timer);
				this._timer = null;
				ctr--;
			}
			if(ctr <= 0){
				clearInterval(timer);
				timer = null;
				ctr = 0;
			}
		}

	});

	var _makeFadeable =
				d.isIE ? function(node){
			// only set the zoom if the "tickle" value would be the same as the
			// default
			var ns = node.style;
			// don't set the width to auto if it didn't already cascade that way.
			// We don't want to f anyones designs
			if(!ns.width.length && d.style(node, "width") == "auto"){
				ns.width = "auto";
			}
		} :
				function(){};

	dojo._fade = function(/*Object*/ args){
		//	summary:
		//		Returns an animation that will fade the node defined by
		//		args.node from the start to end values passed (args.start
		//		args.end) (end is mandatory, start is optional)

		args.node = d.byId(args.node);
		var fArgs = _mixin({ properties: {} }, args),
			props = (fArgs.properties.opacity = {});

		props.start = !("start" in fArgs) ?
			function(){
				return +d.style(fArgs.node, "opacity")||0;
			} : fArgs.start;
		props.end = fArgs.end;

		var anim = d.animateProperty(fArgs);
		d.connect(anim, "beforeBegin", d.partial(_makeFadeable, fArgs.node));

		return anim; // dojo.Animation
	};

	/*=====
	dojo.__FadeArgs = function(node, duration, easing){
		//	node: DOMNode|String
		//		The node referenced in the animation
		//	duration: Integer?
		//		Duration of the animation in milliseconds.
		//	easing: Function?
		//		An easing function.
		this.node = node;
		this.duration = duration;
		this.easing = easing;
	}
	=====*/

	dojo.fadeIn = function(/*dojo.__FadeArgs*/ args){
		// summary:
		//		Returns an animation that will fade node defined in 'args' from
		//		its current opacity to fully opaque.
		return d._fade(_mixin({ end: 1 }, args)); // dojo.Animation
	};

	dojo.fadeOut = function(/*dojo.__FadeArgs*/  args){
		// summary:
		//		Returns an animation that will fade node defined in 'args'
		//		from its current opacity to fully transparent.
		return d._fade(_mixin({ end: 0 }, args)); // dojo.Animation
	};

	dojo._defaultEasing = function(/*Decimal?*/ n){
		// summary: The default easing function for dojo.Animation(s)
		return 0.5 + ((Math.sin((n + 1.5) * Math.PI)) / 2);
	};

	var PropLine = function(properties){
		// PropLine is an internal class which is used to model the values of
		// an a group of CSS properties across an animation lifecycle. In
		// particular, the "getValue" function handles getting interpolated
		// values between start and end for a particular CSS value.
		this._properties = properties;
		for(var p in properties){
			var prop = properties[p];
			if(prop.start instanceof d.Color){
				// create a reusable temp color object to keep intermediate results
				prop.tempColor = new d.Color();
			}
		}
	};

	PropLine.prototype.getValue = function(r){
		var ret = {};
		for(var p in this._properties){
			var prop = this._properties[p],
				start = prop.start;
			if(start instanceof d.Color){
				ret[p] = d.blendColors(start, prop.end, r, prop.tempColor).toCss();
			}else if(!d.isArray(start)){
				ret[p] = ((prop.end - start) * r) + start + (p != "opacity" ? prop.units || "px" : 0);
			}
		}
		return ret;
	};

	/*=====
	dojo.declare("dojo.__AnimArgs", [dojo.__FadeArgs], {
		// Properties: Object?
		//	A hash map of style properties to Objects describing the transition,
		//	such as the properties of dojo._Line with an additional 'units' property
		properties: {}

		//TODOC: add event callbacks
	});
	=====*/

	dojo.animateProperty = function(/*dojo.__AnimArgs*/ args){
		// summary:
		//		Returns an animation that will transition the properties of
		//		node defined in `args` depending how they are defined in
		//		`args.properties`
		//
		// description:
		//		`dojo.animateProperty` is the foundation of most `dojo.fx`
		//		animations. It takes an object of "properties" corresponding to
		//		style properties, and animates them in parallel over a set
		//		duration.
		//
		// example:
		//		A simple animation that changes the width of the specified node.
		//	|	dojo.animateProperty({
		//	|		node: "nodeId",
		//	|		properties: { width: 400 },
		//	|	}).play();
		//		Dojo figures out the start value for the width and converts the
		//		integer specified for the width to the more expressive but
		//		verbose form `{ width: { end: '400', units: 'px' } }` which you
		//		can also specify directly. Defaults to 'px' if ommitted.
		//
		// example:
		//		Animate width, height, and padding over 2 seconds... the
		//		pedantic way:
		//	|	dojo.animateProperty({ node: node, duration:2000,
		//	|		properties: {
		//	|			width: { start: '200', end: '400', units:"px" },
		//	|			height: { start:'200', end: '400', units:"px" },
		//	|			paddingTop: { start:'5', end:'50', units:"px" }
		//	|		}
		//	|	}).play();
		//		Note 'paddingTop' is used over 'padding-top'. Multi-name CSS properties
		//		are written using "mixed case", as the hyphen is illegal as an object key.
		//
		// example:
		//		Plug in a different easing function and register a callback for
		//		when the animation ends. Easing functions accept values between
		//		zero and one and return a value on that basis. In this case, an
		//		exponential-in curve.
		//	|	dojo.animateProperty({
		//	|		node: "nodeId",
		//	|		// dojo figures out the start value
		//	|		properties: { width: { end: 400 } },
		//	|		easing: function(n){
		//	|			return (n==0) ? 0 : Math.pow(2, 10 * (n - 1));
		//	|		},
		//	|		onEnd: function(node){
		//	|			// called when the animation finishes. The animation
		//	|			// target is passed to this function
		//	|		}
		//	|	}).play(500); // delay playing half a second
		//
		// example:
		//		Like all `dojo.Animation`s, animateProperty returns a handle to the
		//		Animation instance, which fires the events common to Dojo FX. Use `dojo.connect`
		//		to access these events outside of the Animation definiton:
		//	|	var anim = dojo.animateProperty({
		//	|		node:"someId",
		//	|		properties:{
		//	|			width:400, height:500
		//	|		}
		//	|	});
		//	|	dojo.connect(anim,"onEnd", function(){
		//	|		console.log("animation ended");
		//	|	});
		//	|	// play the animation now:
		//	|	anim.play();
		//
		// example:
		//		Each property can be a function whose return value is substituted along.
		//		Additionally, each measurement (eg: start, end) can be a function. The node
		//		reference is passed direcly to callbacks.
		//	|	dojo.animateProperty({
		//	|		node:"mine",
		//	|		properties:{
		//	|			height:function(node){
		//	|				// shrink this node by 50%
		//	|				return dojo.position(node).h / 2
		//	|			},
		//	|			width:{
		//	|				start:function(node){ return 100; },
		//	|				end:function(node){ return 200; }
		//	|			}
		//	|		}
		//	|	}).play();
		//

		var n = args.node = d.byId(args.node);
		if(!args.easing){ args.easing = d._defaultEasing; }

		var anim = new d.Animation(args);
		d.connect(anim, "beforeBegin", anim, function(){
			var pm = {};
			for(var p in this.properties){
				// Make shallow copy of properties into pm because we overwrite
				// some values below. In particular if start/end are functions
				// we don't want to overwrite them or the functions won't be
				// called if the animation is reused.
				if(p == "width" || p == "height"){
					this.node.display = "block";
				}
				var prop = this.properties[p];
				if(d.isFunction(prop)){
					prop = prop(n);
				}
				prop = pm[p] = _mixin({}, (d.isObject(prop) ? prop: { end: prop }));

				if(d.isFunction(prop.start)){
					prop.start = prop.start(n);
				}
				if(d.isFunction(prop.end)){
					prop.end = prop.end(n);
				}
				var isColor = (p.toLowerCase().indexOf("color") >= 0);
				function getStyle(node, p){
					// dojo.style(node, "height") can return "auto" or "" on IE; this is more reliable:
					var v = { height: node.offsetHeight, width: node.offsetWidth }[p];
					if(v !== undefined){ return v; }
					v = d.style(node, p);
					return (p == "opacity") ? +v : (isColor ? v : parseFloat(v));
				}
				if(!("end" in prop)){
					prop.end = getStyle(n, p);
				}else if(!("start" in prop)){
					prop.start = getStyle(n, p);
				}

				if(isColor){
					prop.start = new d.Color(prop.start);
					prop.end = new d.Color(prop.end);
				}else{
					prop.start = (p == "opacity") ? +prop.start : parseFloat(prop.start);
				}
			}
			this.curve = new PropLine(pm);
		});
		d.connect(anim, "onAnimate", d.hitch(d, "style", anim.node));
		return anim; // dojo.Animation
	};

	dojo.anim = function(	/*DOMNode|String*/	node,
							/*Object*/			properties,
							/*Integer?*/		duration,
							/*Function?*/		easing,
							/*Function?*/		onEnd,
							/*Integer?*/		delay){
		//	summary:
		//		A simpler interface to `dojo.animateProperty()`, also returns
		//		an instance of `dojo.Animation` but begins the animation
		//		immediately, unlike nearly every other Dojo animation API.
		//	description:
		//		`dojo.anim` is a simpler (but somewhat less powerful) version
		//		of `dojo.animateProperty`.  It uses defaults for many basic properties
		//		and allows for positional parameters to be used in place of the
		//		packed "property bag" which is used for other Dojo animation
		//		methods.
		//
		//		The `dojo.Animation` object returned from `dojo.anim` will be
		//		already playing when it is returned from this function, so
		//		calling play() on it again is (usually) a no-op.
		//	node:
		//		a DOM node or the id of a node to animate CSS properties on
		//	duration:
		//		The number of milliseconds over which the animation
		//		should run. Defaults to the global animation default duration
		//		(350ms).
		//	easing:
		//		An easing function over which to calculate acceleration
		//		and deceleration of the animation through its duration.
		//		A default easing algorithm is provided, but you may
		//		plug in any you wish. A large selection of easing algorithms
		//		are available in `dojo.fx.easing`.
		//	onEnd:
		//		A function to be called when the animation finishes
		//		running.
		//	delay:
		//		The number of milliseconds to delay beginning the
		//		animation by. The default is 0.
		//	example:
		//		Fade out a node
		//	|	dojo.anim("id", { opacity: 0 });
		//	example:
		//		Fade out a node over a full second
		//	|	dojo.anim("id", { opacity: 0 }, 1000);
		return d.animateProperty({ // dojo.Animation
			node: node,
			duration: duration || d.Animation.prototype.duration,
			properties: properties,
			easing: easing,
			onEnd: onEnd
		}).play(delay || 0);
	};
})();

}

if(!dojo._hasResource["dojo._base.browser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo._base.browser"] = true;
dojo.provide("dojo._base.browser");










//Need this to be the last code segment in base, so do not place any
//dojo.requireIf calls in this file. Otherwise, due to how the build system
//puts all requireIf dependencies after the current file, the require calls
//could be called before all of base is defined.
dojo.forEach(dojo.config.require, function(i){
	dojo["require"](i);
});

}

if(!dojo._hasResource["itme.ChassisMap._Utils"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap._Utils"] = true;
dojo.provide("itme.ChassisMap._Utils");


  // Scoped Utils =============================
dojo.declare("itme.ChassisMap._Utils", null, {

  constructor: function(kwArgs) {
  },

  // Something about director's consolecore.jar (json*.js) breaks dojo's tooltip.  This hack fixes it
  _unSetBadPrototypes: function() {
    
    // json*.js sets the below prototypes.  This breaks dojo's tooltip.  Deleting the prototype
    //   appears to fix the problem.  Note however that dynamic refresh won't work without these 
    //   defined prototypes.
    delete Object.prototype.toJSONString;
    //delete Array.prototype.toJSONString;
    //String.prototype.parseJSON = "";
  },
  
  // protected
  _findChassisNodeIndexFromItemsList: function(itemsList){
    for (var index = 0; index < itemsList.length; ++index) {
      if ( itemsList[index].getType() == "SystemChassis")
        return index;
    }
  },

  // protected
  _getProperty: function(item, propertyName){
    var propValue = null;
    try {
      var prop = item.getProperty(propertyName);
      if (prop) {
        propValue = prop.getValue();
      }
    } 
    catch (err) {
      itmeUtils.CMTrace("MAIN", "an error occurred getting properties for " + item.getLabel() + " : " + err);
    }
    return propValue;
  }

});
  

}

if(!dojo._hasResource["itme.ChassisMap.ImageResources"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.ImageResources"] = true;
dojo.provide("itme.ChassisMap.ImageResources");

dojo.declare("itme.ChassisMap.ImageResources", null, {

  _defaultImagePath : "/images/hardwareView/hardware/",
  _ImageTable : {     // HARDWARE

    // CHASSIS
    ChassisFront: "re_chassis.png",
    ChassisBack: "re_backchassis.png",

    // BLADE SLOTS
    DefaultITE_1x: "itme1_194x43a.jpg",  // OLD
    DefaultITE_2x: "obj10.jpg",          // OLD
    FSM: "re_FSM.png", // IBM Flex System Manager
    ITE_C42x0M4: "re_blacktipITE.png", // blacktip: C4240 M4 , C4220 M4
    ITE_C42x0M4_PEN: "re_PEN.png", // blacktip: C4240 M4 , C4220 M4 w PEN
    ITE_C42x0M4_SEN: "re_SEN.png", // blacktip: C4240 M4 , C4220 M4 w SEN
    ITE_C4440M4: "re_congoITEfullwide.png", // C4440 M4
    ITE_C5440P7: "re_firebird4SITEfullwide.png", // firebird: C5440 P7
    ITE_C5240P7: "re_firebird2SITE.png", // firebird: C5240 P7
    EmptyBladeSlot: "re_blank.png",
    PME: "re_pmeITE.png",
    STRW_493949: "re_storwizeSRC.png",  //V7000 storage node SRC Controller model 4939-49
    STRW_493929: "re_storwizeSDE.png",  //V7000 storage node SDE Expansion model 4939-29

    // SWITCHES
    SwitchUnit: "SilverSwitch_26x343px.png",  // OLD
    SwitchEN2092: "SwitchEN2092.png",
    SwitchEN4091: "SwitchEN4091.png",
    SwitchEN4093: "SwitchEN4093.png",
    SwitchFC3171: "SwitchFC3171.png",
    SwitchFC5022: "SwitchFC5022.png",
    SwitchIB6131: "SwitchIB6131.png",
    SwitchFiller: "SwitchFiller.png",

    // POWER, FANS, CMMs & SPACERS
    PowerUnit: "re_powermodule.png",
    PowerUnitEmpty: "re_powermodulefiller.png",
    ITMEUnit: "CMM_b.png",
    CMM: "re_cmm.png",
    CMMFiller: "re_cmmfiller.png",
    FanUnit1: "re_largefan.png",
    FanUnit2: "re_smallfan.png",
    FanControlUnit: "re_fancontrolcard.png",
    FanControlFiller: "FanControlCardBlank_75x16px.png",
    FanFillerLarge: "re_largefanfiller.png",
    FanFillerSmall: "smallFanblank_75x36.png",
    TallSpacer: "re_chassisspacer.png",
    ShortSpacer: "SmallSpacer_b.png" // OLD
  },

  _basePath : "/images/",
  _defaultIconPath : "/images/hardwareView/",

  _IconTable : {
  // STATUS
    StatusFilterIcon_22: "status/statusCluster_22.png",
    StatusFilterIcon_32: "status/statusFilter_32.png",

    StatusCritical_16: "status/st16_critical_24.png",
    StatusCritical_22: "status/st22_critical_24.png",
    StatusCritical_34: "status/st34critical_shad.png",

    StatusWarningMinor_16: "status/ac16_WarningMinor_24.png",
    StatusWarningMinor_22: "status/ac22_WarningMinor_24.png",
    StatusWarningMinor_34: "status/st34minorwarning_shad.png",

    StatusInformational_16: "status/st16_informational_24.png",
    StatusInformational_22: "status/st22_info_24.png",
    StatusInformational_32: "status/st32_informational.png",
    StatusInformational_34: "status/st34informational_shad.png",

    StatusNormal_16: "status/st16_normalSTG_24.png",
    StatusNormal_22: "status/st22_normalSTG_24.png",
    StatusNormal_32: "status/st32_normalSTG_24.png",

  // ACCESS STATES
    //    ACCESS_STATE_NONE_ONLINE = 100;
    AccessOnlineNone_16: "access/st16_normalNoAccess_24.png",
    AccessOnlineNone_22: "access/st22_normalNoAcess_24.png",//"status_online_noAccess_obj.gif",
    // MISSING: AccessOnlineNone_32: "st32_normalNoAcess_24.png",

    //    ACCESS_STATE_NONE_OFFLINE = 101;
    AccessOfflineNone_16: "access/st16_offlineNoAccess_24.png",//"status_offline_noAccess_obj.gif",
    AccessOfflineNone_22: "access/st22_offlineNoAccess_24.png",
    // MISSING: AccessOfflineNone_32: "st32_offlineNoAccess_24.png",

    //    ACCESS_STATE_PARTIAL_ONLINE = 102;
    AccessOnlinePartial_SVG: "access/test.svg",//statusOnlinePartialAccess.svg",
    // MISSING: AccessOnlinePartial_16: "st16_normalPartialAccess_24.png",
    AccessOnlinePartial_22: "access/st22_normalPartialAccess_24.png",//"statusOnlinePartialAccess_obj22.png",
    // MISSING: AccessOnlinePartial_32: "st32_normalPartialAccess_24.png",

    //    ACCESS_STATE_PARTIAL_OFFLINE = 103;
    AccessOfflinePartial_16: "access/st16_offlinePartialAccess_24.png",
    AccessOfflinePartial_22: "access/st22_offlinePartialAccess_24.png",
    // MISSING: AccessOfflinePartial_32: "st32_offlinePartialAccess_24.png",

    //    ACCESS_STATE_FULL_ONLINE = 104;
    AccessOnlineOK_16: "access/st16_fullonlineaccess_24.png", // note path points to this._statusIconsPath.  Icon Re-Use
    AccessOnlineOK_22: "access/st22_fullonlineaccess_24.png",
    //AccessOnlineOK_32: "status/st16_normalSTG_24.png",

    //    ACCESS_STATE_FULL_OFFLINE = 105;
    //AccessOfflineOK_SVG: "Stopped_v1.svg",
    AccessOfflineOK_16: "access/st16_offline_24.png",
    AccessOfflineOK_22: "access/st22_offline_24.png",
    AccessOfflineOK_32: "access/st32_offline_24.png",

    //    ACCESS_STATE_UNKNOWN = 106;
    AccessUnknown_16: "access/st16_unknown_24.png",
    AccessUnknown_22: "access/st22_unknown_24.png",
    AccessUnknown_32: "access/st32_unknown_24.png",

    //    ACCESS_STATE_NOT_APPLICABLE = 107;
    AccessNotApplicable_16: "access/st16_undefined_24.png",
    AccessNotApplicable_22: "access/st22_undefined_24.png",
    AccessNotApplicable_32: "access/st32_undefined_24.png",

    //    ACCESS NOT TRUESTED = 109
    AccessNotTrusted_16: "access/st16_nottrustedcert_24.png",
    //AccessNotTrusted_22: "access/ .png",
    AccessNotTrusted_32: "access/st32_nottrustedcert_24.png",


  // LED
    LEDFilterIcon_22: "leds/LEDCluster.png",
    LEDPowerOn_16: "leds/ac16_power_24.png",
    LEDPowerOn_22: "leds/ac22_power_24.png",
    LEDPowerOff_16: "leds/ac16_powerOFF_24.png",
    LEDPowerOff_22: "leds/ac22_powerOFF_24.png",
    LEDErrorLogOn_16: "leds/ac16_errorLog_24.png",
    LEDErrorLogOn_22: "leds/ac22_errorLog_24.png",
    LEDErrorLogOff_16: "leds/ac16_errorLogOFF_24.png",
    LEDErrorLogOff_22: "leds/ac22_errorLogOFF_24.png",
    LEDLocationOn_16: "leds/ac16_location_24.png",
    LEDLocationOn_22: "leds/ac22_location_24.png",
    LEDLocationOff_16: "leds/ac16_locationOFF_24.png",
    LEDLocationOff_22: "leds/ac22_locationOFF_24.png",
    LEDFaultOn_16: "leds/ac16_enclosureFault_24.png",
    LEDFaultOn_22: "leds/ac22_enclosureFault_24.png",
    LEDFaultOff_16: "leds/ac16_enclosureFaultOFF_24.png",
    LEDFaultOff_22: "leds/ac22_enclosureFaultOFF_24.png",
    // DHM: Define the tray open LEDs
    // LEDTrayOpen_


  // LABELS
    LabelIcon_22: "ac22_tag_24.png",

  // THERMALS
    //ThermalIcon_22: "ac22_thermal_24.png",
    ThermalIcon_22: "Environment_22px.png",

  // REFRESH
    Processing_16: "processing.gif",
    RefreshIcon_16: "ac16_refresh.png",
    RefreshIcon_22: "ac22_refresh_24.png",

  // COMPLIANCE & OS
    ComplianceIcon_22: "ac22_compliance_24.png",

  // OTHER
    Seperator_18: "Separator.png"
  },

  getImage : function(ImageId) {
    //itmeUtils.CMTrace("MAIN", "getImage("+ImageId+"): " + contextPath + this._ImageTable[ImageId]);
    return contextPath + this._defaultImagePath + this._ImageTable[ImageId];
  },

  buildIconImageTag : function(IconId, styleClass, size) {
    var hasSVG = false;
    var imgTag = "";
    var iconPath = this._defaultIconPath;

    // special cases, they don't live in the chassis map icon path
    if ( ( IconId.indexOf("Processing") === 0 ) ||
         ( IconId.indexOf("Seperator") === 0 ) )
      iconPath = this._basePath;

    // find an icon of == or larger size
    for (var i = size; i < 40; i++) {
      if ( this._IconTable[IconId+"_"+i]) {
        imgTag = "<img src='"+contextPath+iconPath+this._IconTable[IconId+"_"+i]+"' width='"+parseInt(size)+"px' height='"+parseInt(size)+"px' class='"+styleClass+"'>";
        break;
      }
    }
    if (imgTag == "") // no images >= , then search <
    {
      for (i = size; i > 12; i--) {
        if ( this._IconTable[IconId+"_"+i]) {
          imgTag = "<img src='"+contextPath+iconPath+this._IconTable[IconId+"_"+i]+"' width='"+parseInt(size)+"px' height='"+parseInt(size)+"px' class='"+styleClass+"'>";
          break;
        }
      }
    }

    // if we have an SVG wrapper the img tag with it.  Browsers will render the Object tag first if it can
    if (false) {//this._IconTable[IconId+"_SVG"]) {
      imgTag = "<object type='image/svg+xml' data='"+contextPath+iconPath+this._IconTable[IconId+"_SVG"]+"' width='"+parseInt(size)+"' height='"+parseInt(size)+"' >" + imgTag + "</object>";
    }

    //itmeUtils.CMTrace("MAIN", "buildIconImageTag("+IconId+","+styleClass+","+size+"): " +imgTag);

    return imgTag;
  },

  // return the path to the icon id given.  If no _xx size is given in the id or 'size' parm doesn't match, largest match is returned
  iconPath: function(IconId, size) {
    var iconPath = this._defaultIconPath;

    // special cases, they don't live in the chassis map icon path
    if ( ( IconId.indexOf("Processing") === 0 ) ||
         ( IconId.indexOf("Seperator") === 0 ) )
      iconPath = this._basePath;

    if ( this._IconTable[IconId] )
      return contextPath + iconPath + this._IconTable[IconId];
    if ( size )
      if ( this._IconTable[IconId+"_"+size] )
        return contextPath + iconPath + this._IconTable[IconId+"_"+size];

    // iconID was no a direct match & size was not given or didn't match

    // then... find largest match
    for (var i = 40; i > 14; i-=2) {
      if ( this._IconTable[IconId+"_"+i]) {
        return contextPath + iconPath + this._IconTable[IconId+"_"+i];
      }
    }
    return "";
  }
});

}

if(!dojo._hasResource["dojo.i18n"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.i18n"] = true;
dojo.provide("dojo.i18n");

/*=====
dojo.i18n = {
	// summary: Utility classes to enable loading of resources for internationalization (i18n)
};
=====*/

dojo.i18n.getLocalization = function(/*String*/packageName, /*String*/bundleName, /*String?*/locale){
	//	summary:
	//		Returns an Object containing the localization for a given resource
	//		bundle in a package, matching the specified locale.
	//	description:
	//		Returns a hash containing name/value pairs in its prototypesuch
	//		that values can be easily overridden.  Throws an exception if the
	//		bundle is not found.  Bundle must have already been loaded by
	//		`dojo.requireLocalization()` or by a build optimization step.  NOTE:
	//		try not to call this method as part of an object property
	//		definition (`var foo = { bar: dojo.i18n.getLocalization() }`).  In
	//		some loading situations, the bundle may not be available in time
	//		for the object definition.  Instead, call this method inside a
	//		function that is run after all modules load or the page loads (like
	//		in `dojo.addOnLoad()`), or in a widget lifecycle method.
	//	packageName:
	//		package which is associated with this resource
	//	bundleName:
	//		the base filename of the resource bundle (without the ".js" suffix)
	//	locale:
	//		the variant to load (optional).  By default, the locale defined by
	//		the host environment: dojo.locale

	locale = dojo.i18n.normalizeLocale(locale);

	// look for nearest locale match
	var elements = locale.split('-');
	var module = [packageName,"nls",bundleName].join('.');
	var bundle = dojo._loadedModules[module];
	if(bundle){
		var localization;
		for(var i = elements.length; i > 0; i--){
			var loc = elements.slice(0, i).join('_');
			if(bundle[loc]){
				localization = bundle[loc];
				break;
			}
		}
		if(!localization){
			localization = bundle.ROOT;
		}

		// make a singleton prototype so that the caller won't accidentally change the values globally
		if(localization){
			var clazz = function(){};
			clazz.prototype = localization;
			return new clazz(); // Object
		}
	}

	throw new Error("Bundle not found: " + bundleName + " in " + packageName+" , locale=" + locale);
};

dojo.i18n.normalizeLocale = function(/*String?*/locale){
	//	summary:
	//		Returns canonical form of locale, as used by Dojo.
	//
	//  description:
	//		All variants are case-insensitive and are separated by '-' as specified in [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt).
	//		If no locale is specified, the dojo.locale is returned.  dojo.locale is defined by
	//		the user agent's locale unless overridden by djConfig.

	var result = locale ? locale.toLowerCase() : dojo.locale;
	if(result == "root"){
		result = "ROOT";
	}
	return result; // String
};

dojo.i18n._requireLocalization = function(/*String*/moduleName, /*String*/bundleName, /*String?*/locale, /*String?*/availableFlatLocales){
	//	summary:
	//		See dojo.requireLocalization()
	//	description:
	// 		Called by the bootstrap, but factored out so that it is only
	// 		included in the build when needed.

	var targetLocale = dojo.i18n.normalizeLocale(locale);
 	var bundlePackage = [moduleName, "nls", bundleName].join(".");
	// NOTE: 
	//		When loading these resources, the packaging does not match what is
	//		on disk.  This is an implementation detail, as this is just a
	//		private data structure to hold the loaded resources.  e.g.
	//		`tests/hello/nls/en-us/salutations.js` is loaded as the object
	//		`tests.hello.nls.salutations.en_us={...}` The structure on disk is
	//		intended to be most convenient for developers and translators, but
	//		in memory it is more logical and efficient to store in a different
	//		order.  Locales cannot use dashes, since the resulting path will
	//		not evaluate as valid JS, so we translate them to underscores.
	
	//Find the best-match locale to load if we have available flat locales.
	var bestLocale = "";
	if(availableFlatLocales){
		var flatLocales = availableFlatLocales.split(",");
		for(var i = 0; i < flatLocales.length; i++){
			//Locale must match from start of string.
			//Using ["indexOf"] so customBase builds do not see
			//this as a dojo._base.array dependency.
			if(targetLocale["indexOf"](flatLocales[i]) == 0){
				if(flatLocales[i].length > bestLocale.length){
					bestLocale = flatLocales[i];
				}
			}
		}
		if(!bestLocale){
			bestLocale = "ROOT";
		}		
	}

	//See if the desired locale is already loaded.
	var tempLocale = availableFlatLocales ? bestLocale : targetLocale;
	var bundle = dojo._loadedModules[bundlePackage];
	var localizedBundle = null;
	if(bundle){
		if(dojo.config.localizationComplete && bundle._built){return;}
		var jsLoc = tempLocale.replace(/-/g, '_');
		var translationPackage = bundlePackage+"."+jsLoc;
		localizedBundle = dojo._loadedModules[translationPackage];
	}

	if(!localizedBundle){
		bundle = dojo["provide"](bundlePackage);
		var syms = dojo._getModuleSymbols(moduleName);
		var modpath = syms.concat("nls").join("/");
		var parent;

		dojo.i18n._searchLocalePath(tempLocale, availableFlatLocales, function(loc){
			var jsLoc = loc.replace(/-/g, '_');
			var translationPackage = bundlePackage + "." + jsLoc;
			var loaded = false;
			if(!dojo._loadedModules[translationPackage]){
				// Mark loaded whether it's found or not, so that further load attempts will not be made
				dojo["provide"](translationPackage);
				var module = [modpath];
				if(loc != "ROOT"){module.push(loc);}
				module.push(bundleName);
				var filespec = module.join("/") + '.js';
				loaded = dojo._loadPath(filespec, null, function(hash){
					// Use singleton with prototype to point to parent bundle, then mix-in result from loadPath
					var clazz = function(){};
					clazz.prototype = parent;
					bundle[jsLoc] = new clazz();
					for(var j in hash){ bundle[jsLoc][j] = hash[j]; }
				});
			}else{
				loaded = true;
			}
			if(loaded && bundle[jsLoc]){
				parent = bundle[jsLoc];
			}else{
				bundle[jsLoc] = parent;
			}
			
			if(availableFlatLocales){
				//Stop the locale path searching if we know the availableFlatLocales, since
				//the first call to this function will load the only bundle that is needed.
				return true;
			}
		});
	}

	//Save the best locale bundle as the target locale bundle when we know the
	//the available bundles.
	if(availableFlatLocales && targetLocale != bestLocale){
		bundle[targetLocale.replace(/-/g, '_')] = bundle[bestLocale.replace(/-/g, '_')];
	}
};

(function(){
	// If other locales are used, dojo.requireLocalization should load them as
	// well, by default. 
	// 
	// Override dojo.requireLocalization to do load the default bundle, then
	// iterate through the extraLocale list and load those translations as
	// well, unless a particular locale was requested.

	var extra = dojo.config.extraLocale;
	if(extra){
		if(!extra instanceof Array){
			extra = [extra];
		}

		var req = dojo.i18n._requireLocalization;
		dojo.i18n._requireLocalization = function(m, b, locale, availableFlatLocales){
			req(m,b,locale, availableFlatLocales);
			if(locale){return;}
			for(var i=0; i<extra.length; i++){
				req(m,b,extra[i], availableFlatLocales);
			}
		};
	}
})();

dojo.i18n._searchLocalePath = function(/*String*/locale, /*Boolean*/down, /*Function*/searchFunc){
	//	summary:
	//		A helper method to assist in searching for locale-based resources.
	//		Will iterate through the variants of a particular locale, either up
	//		or down, executing a callback function.  For example, "en-us" and
	//		true will try "en-us" followed by "en" and finally "ROOT".

	locale = dojo.i18n.normalizeLocale(locale);

	var elements = locale.split('-');
	var searchlist = [];
	for(var i = elements.length; i > 0; i--){
		searchlist.push(elements.slice(0, i).join('-'));
	}
	searchlist.push(false);
	if(down){searchlist.reverse();}

	for(var j = searchlist.length - 1; j >= 0; j--){
		var loc = searchlist[j] || "ROOT";
		var stop = searchFunc(loc);
		if(stop){ break; }
	}
};

dojo.i18n._preloadLocalizations = function(/*String*/bundlePrefix, /*Array*/localesGenerated){
	//	summary:
	//		Load built, flattened resource bundles, if available for all
	//		locales used in the page. Only called by built layer files.

	function preload(locale){
		locale = dojo.i18n.normalizeLocale(locale);
		dojo.i18n._searchLocalePath(locale, true, function(loc){
			for(var i=0; i<localesGenerated.length;i++){
				if(localesGenerated[i] == loc){
					dojo["require"](bundlePrefix+"_"+loc);
					return true; // Boolean
				}
			}
			return false; // Boolean
		});
	}
	preload();
	var extra = dojo.config.extraLocale||[];
	for(var i=0; i<extra.length; i++){
		preload(extra[i]);
	}
};

}

if(!dojo._hasResource["itme.ChassisMap._Layout"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap._Layout"] = true;
dojo.provide("itme.ChassisMap._Layout");





dojo.declare("itme.ChassisMap._Layout", null, {
  // public members FINAL
  ITE_Name: "ITE",
  FAN_Name: "FAN",
  FAN2_Name: "FAN2_",
  FANCTRL_Name: "FANCTRL_",
  PWR_Name: "PWR",
  MGT_Name: "MGT",
  IOM_Name: "IOM",
  DLM_Name: "DLink",
  SPC1_Name: "SPC1_",
  SPC2_Name: "SPC2_",
  MRI: null,
  IMAGES : new itme.ChassisMap.ImageResources(),
  
  // private members
  _args: null,                      // presently unused
  _chassisSlotInformation1: null,   // default view (defined below)
  _chassisSlotInformation2: null,   // alternate view
  _chassisSlotInformation3: null,   // default view - no spacing betwixt components
  _chassisSlotInformation4: null,
  _chassisSlotInformation5: null,
  _chassisSlotInformation6: null,
  _chassisLayoutInformation: null,  // used view, set in constructor
  
  _chassisMapFaceElements: [],      // stores the chassis face dom elements.  Indexed BOTH by name & numerically     
  _numFaceElements: 0,              // for numerical indexing of the face elements
  _chassisNameElement: null,        // the chassis header dome element
  _chassisOverlayElement: null,
  
  _chassisMapParentDOMNodeID: null,
  
  // 821w matches the given images sizes from karen.
  // 418 , 230 I think was the original
  // 600 , 300
  // 750 , 396
  _viewWidth: 892,          /*896 822*/ // default width , gets changed to match the given node, if given
  _viewHeight: 422,         /*440 422 457 442 */ // default width , gets changed to match the given node, if given
  _filterBarHeight: 32,     // filter bar height
  _containingDomNode: null, // if set, dimensions for the view are set from this node.
  
  _resizingTimeoutDelay: false,  // false when not set, otherwise used to track the window drag timeout so we only resize once per drag not on every event
  _resizeEnabled: false,
 
  constructor: function(kwArgs, chassisMapParentDOMNodeID)
  {
    
    this.MRI = dojo.i18n.getLocalization("itme", "Resources");
    
    //this._args = {};
    //dojo.mixin(this._args, kwArgs);

    // indexes to facilitate defining layouts locally
    var ITEIndex = 0;
    var FanIndex = 0;
    var PwrIndex = 0;
    var MgtIndex = 0;
    var IOmIndex = 0;
    var DLmIndex = 0;
    
    this._chassisMapParentDOMNodeID = chassisMapParentDOMNodeID;
    
    // DEFAULT 
    /*
    this._chassisSlotInformation1 = 
                  {"numberOfMapFaces":2, "mapFaces":[
                    {"mapName":"front","mapWidth":188,"mapHeight":162},
                    {"mapName":"back","mapWidth":188,"mapHeight":162} ],
                   "numberOfSlotTypes":7,"slots":[
                    {"slotType":this.ITE_Name,"slotCount":14,"slotWidth":91,"slotHeight":20,"slotPositioning":[
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":0},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":0},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":23},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":23},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":46},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":46},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":69},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":69},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":92},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":92},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":115},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":115},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":138},
                      {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":138} ] },
                    {"slotType":this.FAN_Name,"slotCount":10,"slotWidth":30,"slotHeight":29,"slotPositioning":[
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":0},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":32},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":64},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":96},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":128},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":0},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":32},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":64},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":96},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":128} ] },
                    {"slotType":this.FAN_Name,"slotCount":2,"slotWidth":54,"slotHeight":25,"slotPositioning":[
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":64,"Y":0},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":64,"Y":135} ] },
                    {"slotType":this.PWR_Name,"slotCount":6,"slotWidth":16,"slotHeight":51,"slotPositioning":[
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":12,"Y":0},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":12,"Y":54},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":12,"Y":108},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":155,"Y":0},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":155,"Y":54},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":155,"Y":108} ] },
                    {"slotType":this.MGT_Name,"slotCount":2,"slotWidth":9,"slotHeight":78,"slotPositioning":[
                      {"slotID":this.MGT_Name+(MgtIndex++),"mapFace":"back","X":0,"Y":0},
                      {"slotID":this.MGT_Name+(MgtIndex++),"mapFace":"back","X":174,"Y":81} ] },
                    {"slotType":this.IOM_Name,"slotCount":4,"slotWidth":11,"slotHeight":104,"slotPositioning":[
                      {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":64,"Y":28},
                      {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":78,"Y":28},
                      {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":92,"Y":28},
                      {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":106,"Y":28} ] },
                    {"slotType":this.DLM_Name,"slotCount":2,"slotWidth":9,"slotHeight":78,"slotPositioning":[
                      {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":174,"Y":0},
                      {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":0,"Y":81} ] }
                    ]
                };

    ITEIndex = FanIndex = PwrIndex = MgtIndex = IOmIndex = DLmIndex = 0;

    // ALTERNATE LAYOUT
    this._chassisSlotInformation2 =
                {"numberOfMapFaces":2, "mapFaces":[
                    {"mapName":"front","mapWidth":190,"mapHeight":165},
                    {"mapName":"back","mapWidth":190,"mapHeight":165} ],
                 "numberOfSlotTypes":6,"slots":[
                  {"slotType":this.ITE_Name,"slotCount":10,"slotWidth":185,"slotHeight":13,"slotPositioning":[
                    {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":0},
                    {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":16},
                    {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":32},
                    {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":48},
                    {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":64},
                    {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":80},
                    {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":96},
                    {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":112},
                    {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":128},
                    {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":144} ] },
                  {"slotType":this.FAN_Name,"slotCount":10,"slotWidth":34,"slotHeight":30,"slotPositioning":[
                    {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":0,"Y":18},
                    {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":37,"Y":18},
                    {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":74,"Y":18},
                    {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":111,"Y":18},
                    {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":148,"Y":18},
                    {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":0,"Y":113},
                    {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":37,"Y":113},
                    {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":74,"Y":113},
                    {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":111,"Y":113},
                    {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":148,"Y":113} ] },
                  {"slotType":this.PWR_Name,"slotCount":6,"slotWidth":20,"slotHeight":60,"slotPositioning":[
                    {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":15,"Y":50},
                    {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":38,"Y":50},
                    {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":61,"Y":50},
                    {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":84,"Y":50},
                    {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":107,"Y":50},
                    {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":130,"Y":50} ] },
                  {"slotType":this.MGT_Name,"slotCount":2,"slotWidth":12,"slotHeight":60,"slotPositioning":[
                    {"slotID":this.MGT_Name+(MgtIndex++),"mapFace":"back","X":0,"Y":50},
                    {"slotID":this.MGT_Name+(MgtIndex++),"mapFace":"back","X":175,"Y":50} ] },
                  {"slotType":this.IOM_Name,"slotCount":4,"slotWidth":91,"slotHeight":15,"slotPositioning":[
                    {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":0,"Y":0},
                    {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":94,"Y":0},
                    {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":0,"Y":144},
                    {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":94,"Y":144} ] },
                  {"slotType":this.DLM_Name,"slotCount":2,"slotWidth":8,"slotHeight":60,"slotPositioning":[
                    {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":153,"Y":50},
                    {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":164,"Y":50} ] }
                  ]
              };
              
    ITEIndex = FanIndex = PwrIndex = MgtIndex = IOmIndex = DLmIndex = 0;

    // DEFAULT + NO SPACING BETWEEN COMPONENTS
    this._chassisSlotInformation3 = 
              {"numberOfMapFaces":2, "mapFaces":[
                {"mapName":"front","mapWidth":190,"mapHeight":165},
                {"mapName":"back","mapWidth":190,"mapHeight":165} ],
               "numberOfSlotTypes":7,"slots":[
                {"slotType":this.ITE_Name,"slotCount":14,"slotWidth":94,"slotHeight":23,"slotPositioning":[
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":0},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":0},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":23},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":23},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":46},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":46},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":69},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":69},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":92},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":92},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":115},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":115},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":0,"Y":138},
                  {"slotID":this.ITE_Name+(ITEIndex++),"mapFace":"front","X":94,"Y":138} ] },
                {"slotType":this.FAN_Name,"slotCount":10,"slotWidth":33,"slotHeight":33,"slotPositioning":[
                  {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":0},
                  {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":33},
                  {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":66},
                  {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":99},
                  {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":132},
                  {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":0},
                  {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":33},
                  {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":66},
                  {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":99},
                  {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":132} ] },
                {"slotType":this.FAN_Name,"slotCount":2,"slotWidth":55,"slotHeight":28,"slotPositioning":[
                  {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":66,"Y":0},
                  {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":66,"Y":135} ] },
                {"slotType":this.PWR_Name,"slotCount":6,"slotWidth":19,"slotHeight":54,"slotPositioning":[
                  {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":12,"Y":0},
                  {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":12,"Y":54},
                  {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":12,"Y":108},
                  {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":155,"Y":0},
                  {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":155,"Y":54},
                  {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":155,"Y":108} ] },
                {"slotType":this.MGT_Name,"slotCount":2,"slotWidth":12,"slotHeight":81,"slotPositioning":[
                  {"slotID":this.MGT_Name+(MgtIndex++),"mapFace":"back","X":0,"Y":0},
                  {"slotID":this.MGT_Name+(MgtIndex++),"mapFace":"back","X":174,"Y":82} ] },
                {"slotType":this.IOM_Name,"slotCount":4,"slotWidth":12,"slotHeight":106,"slotPositioning":[
                  {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":66,"Y":29},
                  {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":80,"Y":29},
                  {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":93,"Y":29},
                  {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":108,"Y":29} ] },
                {"slotType":this.DLM_Name,"slotCount":2,"slotWidth":12,"slotHeight":81,"slotPositioning":[
                  {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":174,"Y":0},
                  {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":0,"Y":82} ] }
                ]
            };

    ITEIndex = FanIndex = PwrIndex = MgtIndex = IOmIndex = DLmIndex = 0;
    
    // OLD 
    this._chassisSlotInformation4 = 
                  {"numberOfMapFaces":2, "mapFaces":[
                    {"mapName":"front","mapWidth":188,"mapHeight":162},
                    {"mapName":"back","mapWidth":188,"mapHeight":162} ],
                   "numberOfSlotTypes":8,"slots":[
                    {"slotType":this.ITE_Name,"slotCount":10,"slotWidth":91,"slotHeight":20,"slotPositioning":[
                      {"slotID":this.ITE_Name+"0","mapFace":"front","X":0,"Y":0},
                      {"slotID":this.ITE_Name+"1","mapFace":"front","X":94,"Y":0},
                      {"slotID":this.ITE_Name+"3","mapFace":"front","X":0,"Y":46},
                      {"slotID":this.ITE_Name+"4","mapFace":"front","X":94,"Y":46},
                      {"slotID":this.ITE_Name+"5","mapFace":"front","X":0,"Y":69},
                      {"slotID":this.ITE_Name+"6","mapFace":"front","X":94,"Y":69},
                      {"slotID":this.ITE_Name+"8","mapFace":"front","X":0,"Y":115},
                      {"slotID":this.ITE_Name+"9","mapFace":"front","X":94,"Y":115},
                      {"slotID":this.ITE_Name+"10","mapFace":"front","X":0,"Y":138},
                      {"slotID":this.ITE_Name+"11","mapFace":"front","X":94,"Y":138} ] },
                    {"slotType":this.ITE_Name,"slotCount":2,"slotWidth":185,"slotHeight":20,"slotPositioning":[
                      {"slotID":this.ITE_Name+"2","mapFace":"front","X":0,"Y":23},
                      {"slotID":this.ITE_Name+"7","mapFace":"front","X":0,"Y":92} ] },
                    {"slotType":this.FAN_Name,"slotCount":10,"slotWidth":30,"slotHeight":29,"slotPositioning":[
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":0},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":32},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":64},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":96},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":31,"Y":128},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":0},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":32},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":64},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":96},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":122,"Y":128} ] },
                    {"slotType":this.FAN_Name,"slotCount":2,"slotWidth":54,"slotHeight":25,"slotPositioning":[
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":64,"Y":0},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":64,"Y":135} ] },
                    {"slotType":this.PWR_Name,"slotCount":6,"slotWidth":16,"slotHeight":51,"slotPositioning":[
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":12,"Y":0},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":12,"Y":54},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":12,"Y":108},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":155,"Y":0},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":155,"Y":54},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":155,"Y":108} ] },
                    {"slotType":this.MGT_Name,"slotCount":2,"slotWidth":9,"slotHeight":78,"slotPositioning":[
                      {"slotID":this.MGT_Name+(MgtIndex++),"mapFace":"back","X":0,"Y":0},
                      {"slotID":this.MGT_Name+(MgtIndex++),"mapFace":"back","X":174,"Y":81} ] },
                    {"slotType":this.IOM_Name,"slotCount":4,"slotWidth":11,"slotHeight":104,"slotPositioning":[
                      {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":64,"Y":28},
                      {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":78,"Y":28},
                      {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":92,"Y":28},
                      {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":106,"Y":28} ] },
                    {"slotType":this.DLM_Name,"slotCount":2,"slotWidth":9,"slotHeight":78,"slotPositioning":[
                      {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":174,"Y":0},
                      {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":0,"Y":81} ] }
                    ]
                };
    */
    ITEIndex = FanIndex = PwrIndex = MgtIndex = IOmIndex = DLmIndex = 0;
    
    // OLD 
    /*this._chassisSlotInformation5 = 
                  {"numberOfMapFaces":2, "mapFaces":[
                    {"mapName":"front","mapWidth":401,"mapHeight":343},
                    {"mapName":"back","mapWidth":401,"mapHeight":343} ],
                   "numberOfSlotTypes":7,"slots":[
                    {"slotType":this.ITE_Name,"slotCount":14,"slotWidth":194,"slotHeight":43,"slotPositioning":[
                      {"slotID":this.ITE_Name+"1","mapFace":"front","X":0,"Y":0},
                      {"slotID":this.ITE_Name+"2","mapFace":"front","X":199,"Y":0},
                      {"slotID":this.ITE_Name+"3","mapFace":"front","X":0,"Y":48},
                      {"slotID":this.ITE_Name+"4","mapFace":"front","X":199,"Y":48},
                      {"slotID":this.ITE_Name+"5","mapFace":"front","X":0,"Y":97},
                      {"slotID":this.ITE_Name+"6","mapFace":"front","X":199,"Y":97},
                      {"slotID":this.ITE_Name+"7","mapFace":"front","X":0,"Y":146},
                      {"slotID":this.ITE_Name+"8","mapFace":"front","X":199,"Y":146},
                      {"slotID":this.ITE_Name+"9","mapFace":"front","X":0,"Y":195},   
                      {"slotID":this.ITE_Name+"10","mapFace":"front","X":199,"Y":195}, 
                      {"slotID":this.ITE_Name+"11","mapFace":"front","X":0,"Y":244},
                      {"slotID":this.ITE_Name+"12","mapFace":"front","X":199,"Y":244},
                      {"slotID":this.ITE_Name+"13","mapFace":"front","X":0,"Y":293},
                      {"slotID":this.ITE_Name+"14","mapFace":"front","X":199,"Y":293} ] },
                    {"slotType":this.FAN_Name,"slotCount":10,"slotWidth":75,"slotHeight":63,"slotPositioning":[
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":71,"Y":0},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":71,"Y":68},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":71,"Y":136},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":71,"Y":204},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":71,"Y":272},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":246,"Y":0},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":246,"Y":68},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":246,"Y":136},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":246,"Y":204},
                      {"slotID":this.FAN_Name+(FanIndex++),"mapFace":"back","X":246,"Y":272} ] },
                    {"slotType":this.FAN2_Name,"slotCount":2,"slotWidth":88,"slotHeight":55,"slotPositioning":[
                      {"slotID":this.FAN2_Name+"0","mapFace":"back","X":152,"Y":0},
                      {"slotID":this.FAN2_Name+"1","mapFace":"back","X":152,"Y":281} ] },
                    {"slotType":this.PWR_Name,"slotCount":6,"slotWidth":40,"slotHeight":108,"slotPositioning":[
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":26,"Y":0},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":26,"Y":114},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":26,"Y":229},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":327,"Y":0},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":327,"Y":114},
                      {"slotID":this.PWR_Name+(PwrIndex++),"mapFace":"back","X":327,"Y":229} ] },
                    {"slotType":this.MGT_Name,"slotCount":2,"slotWidth":21,"slotHeight":166,"slotPositioning":[
                      {"slotID":this.MGT_Name+(MgtIndex++),"mapFace":"back","X":0,"Y":0},
                      {"slotID":this.MGT_Name+(MgtIndex++),"mapFace":"back","X":373,"Y":171} ] },
                    {"slotType":this.IOM_Name,"slotCount":4,"slotWidth":16,"slotHeight":215,"slotPositioning":[
                      {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":152,"Y":60},
                      {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":175,"Y":60},
                      {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":199,"Y":60},
                      {"slotID":this.IOM_Name+(IOmIndex++),"mapFace":"back","X":223,"Y":60} ] },
                    {"slotType":this.DLM_Name,"slotCount":2,"slotWidth":21,"slotHeight":166,"slotPositioning":[
                      {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":373,"Y":0},
                      {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":0,"Y":171} ] }
                    ]
                };
              itmeUtils.CMTrace("LAYOUT", "vhCheck: " + this._viewHeight);
    } , */
    
    this._chassisSlotInformation6 = 
                  {"numberOfMapFaces":2, "mapFaces":[
                    {"mapName":"front","mapWidth":401,"mapHeight":343},
                    {"mapName":"back","mapWidth":401,"mapHeight":343} ],
                   "numberOfSlotTypes":8,"slots":[
                    {"slotType":this.ITE_Name,"slotCount":14,"slotWidth":194,"slotHeight":43,"slotPositioning":[
                      {"slotID":this.ITE_Name+"13","mapFace":"front","X":0,"Y":0},
                      {"slotID":this.ITE_Name+"14","mapFace":"front","X":199,"Y":0},
                      {"slotID":this.ITE_Name+"11","mapFace":"front","X":0,"Y":48},
                      {"slotID":this.ITE_Name+"12","mapFace":"front","X":199,"Y":48},
                      {"slotID":this.ITE_Name+"9","mapFace":"front","X":0,"Y":97},
                      {"slotID":this.ITE_Name+"10","mapFace":"front","X":199,"Y":97},
                      {"slotID":this.ITE_Name+"7","mapFace":"front","X":0,"Y":146},
                      {"slotID":this.ITE_Name+"8","mapFace":"front","X":199,"Y":146},
                      {"slotID":this.ITE_Name+"5","mapFace":"front","X":0,"Y":195},  
                      {"slotID":this.ITE_Name+"6","mapFace":"front","X":199,"Y":195},
                      {"slotID":this.ITE_Name+"3","mapFace":"front","X":0,"Y":244},
                      {"slotID":this.ITE_Name+"4","mapFace":"front","X":199,"Y":244},
                      {"slotID":this.ITE_Name+"1","mapFace":"front","X":0,"Y":293},
                      {"slotID":this.ITE_Name+"2","mapFace":"front","X":199,"Y":293} ] },
                    {"slotType":this.FAN_Name,"slotCount":8,"slotWidth":75,"slotHeight":69,"slotPositioning":[
                      {"slotID":this.FAN_Name+"9","mapFace":"back","X":72,"Y":39},    // y = 36 + (1x3) = 39
                      {"slotID":this.FAN_Name+"8","mapFace":"back","X":72,"Y":111},  // y = 39 + 69 + (1x3) = 111
                      {"slotID":this.FAN_Name+"7","mapFace":"back","X":72,"Y":202},  // y = 183 + 16 + (1x3) = 202
                      {"slotID":this.FAN_Name+"6","mapFace":"back","X":72,"Y":274},  // y = 202 + 69 + (1x3) = 274
                      {"slotID":this.FAN_Name+"4","mapFace":"back","X":272,"Y":39},
                      {"slotID":this.FAN_Name+"3","mapFace":"back","X":272,"Y":111},
                      {"slotID":this.FAN_Name+"2","mapFace":"back","X":272,"Y":202},
                      {"slotID":this.FAN_Name+"1","mapFace":"back","X":272,"Y":274} ] },
                    {"slotType":this.FAN2_Name,"slotCount":2,"slotWidth":75,"slotHeight":36,"slotPositioning":[
                      {"slotID":this.FAN_Name+"10","mapFace":"back","X":72,"Y":0},       //x = 43 + 23 + (2x3) = 72
                      {"slotID":this.FAN_Name+"5","mapFace":"back","X":272,"Y":0} ] },  //x = 72 + 75 + 26 + 18 + 43 + 23 (5x3) = 272
                    {"slotType":this.FANCTRL_Name,"slotCount":2,"slotWidth":75,"slotHeight":16,"slotPositioning":[
                      {"slotID":this.FANCTRL_Name+"2","mapFace":"back","X":72,"Y":183},       //y = 111 + 69 + (1x3) = 183
                      {"slotID":this.FANCTRL_Name+"1","mapFace":"back","X":272,"Y":183} ] },  
                    {"slotType":this.PWR_Name,"slotCount":6,"slotWidth":43,"slotHeight":112,"slotPositioning":[
                      {"slotID":this.PWR_Name+"6","mapFace":"back","X":0,"Y":0},       // y = 0
                      {"slotID":this.PWR_Name+"5","mapFace":"back","X":0,"Y":115},     // y = 114 + (1x3) = 117
                      {"slotID":this.PWR_Name+"4","mapFace":"back","X":0,"Y":230},     // y = 117 + 114 + (1x3) = 234
                      {"slotID":this.PWR_Name+"3","mapFace":"back","X":200,"Y":0},     //x = 72 + 75 + 26 + 18 + (3x3) = 200
                      {"slotID":this.PWR_Name+"2","mapFace":"back","X":200,"Y":115},
                      {"slotID":this.PWR_Name+"1","mapFace":"back","X":200,"Y":230} ] },
                    {"slotType":this.SPC1_Name,"slotCount":1,"slotWidth":20,"slotHeight":343,"slotPositioning":[
                      {"slotID":this.SPC1_Name+"0","mapFace":"back","X":179,"Y":0} ] }, //x = 150 + 26 + (1x3) = 179  
                    {"slotType":this.MGT_Name,"slotCount":2,"slotWidth":18,"slotHeight":165,"slotPositioning":[
                      {"slotID":this.MGT_Name+"2","mapFace":"back","X":379,"Y":0},    //x = 350 + 26 + (1x3) = 379
                      {"slotID":this.MGT_Name+"1","mapFace":"back","X":379,"Y":179} ] },   //y = 176 + (1x3) = 179
                    {"slotType":this.IOM_Name,"slotCount":4,"slotWidth":23,"slotHeight":343,"slotPositioning":[
                      {"slotID":this.IOM_Name+1,"mapFace":"back","X":46,"Y":0},      // x = 43 + (1x3) = 46
                      {"slotID":this.IOM_Name+3,"mapFace":"back","X":150,"Y":0},    // x = 72 + 75 + (1x3) = 150
                      {"slotID":this.IOM_Name+2,"mapFace":"back","X":246,"Y":0},      // x = 200 + 43 + (1x3) = 246
                      {"slotID":this.IOM_Name+4,"mapFace":"back","X":350,"Y":0} ] }, // x = 272 + 75 + (1x3) = 350
                    /*{"slotType":this.DLM_Name,"slotCount":2,"slotWidth":26,"slotHeight":343,"slotPositioning":[
                      {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":150,"Y":0},   // x = 72 + 75 + (1x3) = 150
                      {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":350,"Y":0} ] },   // x = 272 + 75 + (1x3) = 350
                    /*{"slotType":this.DLM_Name,"slotCount":2,"slotWidth":21,"slotHeight":166,"slotPositioning":[
                      {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":373,"Y":0},
                      {"slotID":this.DLM_Name+(DLmIndex++),"mapFace":"back","X":0,"Y":171} ] }*/
                    ]
                };
    itmeUtils.CMTrace("LAYOUT", "vhCheck: " + this._viewHeight);
    } ,

    // PUBLIC
    setDimensionsBasedOnNode: function(domNode)
    {
      itmeUtils.CMTrace("LAYOUT", "setDimensionsBasedOnNode BEGIN");
      this._containingDomNode = domNode;
      // - Forced OVERRIDES - 
      domNode.style.width = this._viewWidth + "px"; /*896 822*/                     // test only
      domNode.style.height = this._viewHeight + "px"; /*422 457 442 */           // test only
      // end forced overrides
      
      var contentBox = dojo.contentBox(domNode);
      //TODO: do we need this? it's causing gray space problems, commented it out (Alex)
      //this._viewWidth = contentBox.w;
      //this._viewHeight = contentBox.h;

      itmeUtils.CMTrace("LAYOUT", "  vwCheck: " + this._viewWidth);
      itmeUtils.CMTrace("LAYOUT", "  vhCheck: " + this._viewHeight);
      itmeUtils.CMTrace("LAYOUT", "setDimensionsBasedOnNode END");
    },

    _toggleWaitingPanel: function(isShown, waitingString) 
    {
      if ( ! isShown )
      {
        if ( dojo.byId("hardwareViewWaitingPane"))
          dojo.destroy("hardwareViewWaitingPane");
      }
      else
      {
        if ( ! dojo.byId("hardwareViewWaitingPane"))
        { itmeUtils.createElement("hardwareViewWaitingPane",
                                  this._chassisMapFaceElements[0],
                                  { styleClass: "refreshPendingDiv", 
                                    style: { "position":"absolute", "top":"20px", "left":"20px"},
                                    innerHTML: this.IMAGES.buildIconImageTag("Processing", "", 22) + " " + waitingString } );
        }
      }
    },
    
    // PROTECTED
    _buildChassisMapHTML: function()
    {
      itmeUtils.CMTrace("LAYOUT", "buildChassisMapHTML BEGIN");
      itmeUtils.CMTrace("LAYOUT", "  view Width: " + this._viewWidth);
      itmeUtils.CMTrace("LAYOUT", "  view Height: " + this._viewHeight);
      
      // set the layout to use
      this._chassisLayoutInformation = this._chassisSlotInformation6;
      
      // get the parent element to which we will append the nameplate and chassis face elements
      var chassisParentElement = dojo.byId(this._chassisMapParentDOMNodeID);
      var contentBox = dojo.contentBox(chassisParentElement);
      itmeUtils.CMTrace("LAYOUT", "  parent Width: " + contentBox.w);
      itmeUtils.CMTrace("LAYOUT", "  parent Height: " + contentBox.h);
      dojo.contentBox(chassisParentElement, {w: this._viewWidth, h: this._viewHeight});
      contentBox = dojo.contentBox(chassisParentElement);
      itmeUtils.CMTrace("LAYOUT", "  parent Width2: " + contentBox.w);
      itmeUtils.CMTrace("LAYOUT", "  parent Height2: " + contentBox.h);      

      dojo.attr(chassisParentElement, "class", "ChassisFrame" + ((dojo.isIE) ? " IECSS" : ""));
      
      // append child elements
      //this._appendChassisNameDivElement(chassisParentElement);
      for ( var i = 0; i < this._chassisLayoutInformation.numberOfMapFaces; i++)
      {
        this._appendChassisFaceDivElement(chassisParentElement, 
                                          this._chassisLayoutInformation.mapFaces[i].mapName,
                                          this._chassisLayoutInformation.mapFaces[i].mapWidth,
                                          this._chassisLayoutInformation.mapFaces[i].mapHeight ); 
      }
      this._appendChassisObjectsDivElement(this._chassisMapFaceElements["front"]);
      this._appendChassisOverlayDivElement(chassisParentElement);
    
      // scale the elements to fit the view frame
      var scale = this._getFacesToFrameRatio(this._chassisLayoutInformation);
      this._reSizeChassisMapFacesToScale(scale);
      
      // build the chassis component layout from the layout data
      this._buildChassisMapFromData(chassisParentElement, scale);
      
      if ( this._resizeEnabled === true )
        dojo.connect(window, "onresize", this, this._onFrameResize);
      
      itmeUtils.CMTrace("LAYOUT", "buildChassisMapHTML END");
    },

    /*// public
    setVisible: function(visibility)
    {
      itmeUtils.CMTrace("LAYOUT", "setVisible ("+visibility+")");
      
      var vizLabel = "block";
      if ( visibility !== true)
        vizLabel = "none";
      
      this._chassisNameElement.style.display = vizLabel;
      for (var i = 0; i < this._numFaceElements; i++)
        this._chassisMapFaceElements[i].style.display = "block";
    },*/

    // private
    _appendChassisOverlayDivElement: function(chassisParentElement)
    { 
      this._chassisOverlayElement = itmeUtils.createElement("chassisOverlay",
                                      chassisParentElement,
                                      { styleClass: "ChassisOverlayFrame soria",
                                        style: {  "position": "relative", // styleHash
                                                  "left": "0px",
                                                  //"top": "0px",
                                                  "float": "left",
                                                  "height": this._filterBarHeight + "px", //75
                                                  "width": this._viewWidth,
                                                  "display": "block"} } );
    },
    
    // private
    _appendChassisNameDivElement: function(chassisParentElement)
    {
      this._chassisNameElement = itmeUtils.createElement("chassisName",
                                    chassisParentElement,
                                    { styleClass: "ChassisHeader",
                                      style: {  "position": "relative",
                                                "left": "0px",
                                                "float": "left",
                                                "height": "20px",
                                                "display": "block"} });

      this._chassisNameElement.style.width = this._viewWidth - this._getNodeFluffWidth(this._chassisNameElement) ;//- 82;
      //itmeUtils.CMTrace("LAYOUT", "sanityCheck: " + this._chassisNameElement.style.width + " , fluff:" + this._getNodeFluffWidth(this._chassisNameElement));
    },

    // private
    _appendChassisFaceDivElement: function (chassisParentElement, faceName, width, height)
    {
      itmeUtils.CMTrace("LAYOUT", "building face ("+faceName+") : width will be: " + width + " +37");
      //var imgPath = this.IMAGES.getImage("ChassisFront");
      //if ( faceName == "back" )
      //  imgPath = this.IMAGES.getImage("ChassisBack");
      this._chassisMapFaceElements[this._numFaceElements++] = itmeUtils.createElement(
              "chassis" + faceName, 
              chassisParentElement,
              { styleClass: "ChassisFace ChassisFace"+faceName,
                style: {  "width":(width+37) +"px", 
                          "height":(height+38)+"px" 
                          /*"backgroundImage": "url('"+imgPath+"')"*/} } );

      // also index the dom element for the faces by name
      this._chassisMapFaceElements[faceName] = dojo.byId("chassis" + faceName);
      itmeUtils.CMTrace("LAYOUT", "resulting width is ... " + this._chassisMapFaceElements[faceName].style.width);
    },
    
    // private : 
    _appendChassisObjectsDivElement: function(chassisFaceElement)
    {
      itmeUtils.CMTrace("LAYOUT", "building chassis object div " + chassisFaceElement.style.width);
      this._chassisNameElement = itmeUtils.createElement(
              "chassisObjectDiv",
              chassisFaceElement,
              { styleClass: "ChassisComponent", 
                style: {  "position":"absolute", 
                          "left":"0px", 
                          "top":"0px", 
                          "width":chassisFaceElement.style.width, 
                          "height":chassisFaceElement.style.height, 
                          "display":"block"} } );

      // also index the dom element for the faces by name
      this._chassisNameElement = dojo.byId("chassisObjectDiv");
      this._chassisNameElement.constraint = 16;
      itmeUtils.CMTrace("LAYOUT", "resulting width is ... " + this._chassisNameElement.style.width);


      itmeUtils.createElement(
              "chassisInternalBackground_", 
              chassisFaceElement,
              { styleClass: "ChassisFaceInternalBackground", 
                style: {  "position":"absolute", 
                          "top":"19px",
                          "left":"19px",
                          "width":parseInt(chassisFaceElement.style.width)-38 +"px", 
                          "height":parseInt(chassisFaceElement.style.height)-38+"px", 
                          "display":"block"} } );
    },

    // private : fluff is defined as the total pixels of the margin & border for the dimension
    _getNodeFluffWidth: function(node)
    {
      if (node.fluffW) 
        return node.fluffW;
      
      this._calculateAndSetNodeFluff(node);
      return node.fluffW;
    },
    
    // private : fluff is defined as the total pixels of the margin & border for the dimension
    _getNodeFluffHeight: function(node)
    {
      if (node.fluffH) 
        return node.fluffH;
      
      this._calculateAndSetNodeFluff(node);
      return node.fluffH;
    },
    
    // private
    _calculateAndSetNodeFluff: function(node)
    {
      var contentBox = dojo.contentBox(node); // matches defined sizes on first load
      var marginBox = dojo.marginBox(node);
      node.fluffW = (marginBox.w - contentBox.w);
      node.fluffH = (marginBox.h - contentBox.h);
    },
    
    // private
    _buildChassisMapFromData: function (chassisParentElement, scaleFactor)
    {
      for (var i = 0; i < this._chassisLayoutInformation.numberOfSlotTypes; i++) {
        if (!this._chassisLayoutInformation.slots[i]) 
          alert("Chassis Slot " + (i + 1) + " information is missing");
        else {
          for (var j = 0; j < this._chassisLayoutInformation.slots[i].slotCount; j++) {
            if (!this._chassisLayoutInformation.slots[i].slotPositioning[j]) 
              alert("Chassis Slot Position " + (j + 1) + " of slot type " + this._chassisLayoutInformation.slots[i].slotType + " is missing");
            else {
              var newElement = itmeUtils.createElement(
                        this._chassisLayoutInformation.slots[i].slotPositioning[j].slotID, 
                        this._chassisMapFaceElements[this._chassisLayoutInformation.slots[i].slotPositioning[j].mapFace],
                        { styleClass: "Component EmptySlot",
                          style: {  "position": "absolute",
                                    "width": parseInt(this._chassisLayoutInformation.slots[i].slotWidth*scaleFactor) + "px",
                                    "height": parseInt(this._chassisLayoutInformation.slots[i].slotHeight*scaleFactor) + "px",
                                    "left": (parseInt(this._chassisLayoutInformation.slots[i].slotPositioning[j].X * scaleFactor) +18)+ "px",
                                    "top": (parseInt(this._chassisLayoutInformation.slots[i].slotPositioning[j].Y * scaleFactor) +18)+ "px",
                                    "overflow": "hidden",
                                    "text-overflow": "ellipsis" } } );
                        
              newElement.constraint = this._determineElementConstraint(newElement);          
              // CHEAT:
              var imageStr = null;
              switch (this._chassisLayoutInformation.slots[i].slotType) {
                case this.PWR_Name:
                  imageStr = this.IMAGES.getImage("PowerUnitEmpty");
                  break;
                case this.FAN_Name:
                  imageStr = this.IMAGES.getImage("FanFillerLarge");
                  break;
                case this.FAN2_Name:
                  imageStr = this.IMAGES.getImage("FanFillerSmall");
                  break;
                case this.FANCTRL_Name:
                  imageStr = this.IMAGES.getImage("FanControlFiller");
                  break;
                case this.DLM_Name:
                  imageStr = this.IMAGES.getImage("SwitchFiller");
                  break;
                case this.MGT_Name:
                  imageStr = this.IMAGES.getImage("CMMFiller");
                  break;
                case this.IOM_Name:
                  imageStr = this.IMAGES.getImage("SwitchFiller");
                  break;
                case this.ITE_Name:
                  imageStr = this.IMAGES.getImage("EmptyBladeSlot");
                  break;
                case this.SPC1_Name: 
                  imageStr = this.IMAGES.getImage("TallSpacer");
                  break;
              }
              if (imageStr)
                newElement.innerHTML = "<img id='" + newElement.id + "_IMG' src='" + imageStr + "' width='" + newElement.style.width + "' height='" + newElement.style.height + "'>";
              // end CHEAT
              
            }
          }
        }
      }
    },

    // TYPE: RENDER
    _determineElementConstraint: function(element) {
      var cBox = dojo.contentBox(element);
      if ( cBox.w < cBox.h )
        return cBox.w - 2;
      return cBox.h - 2;
    },

    // private
    _getFacesToFrameRatio: function()
    {
      itmeUtils.CMTrace("LAYOUT", "_getFacesToFrameRatio");
      itmeUtils.initializeTimer("FaceToFrameRatio"); // for performance tracing
      
      var faceSizeInfo = this._calculateFacesInfo();
      
      itmeUtils.traceTimeDelta("LAYOUT", "Time for Faces Calculation", true);
      
      // HEIGHT - get name element information to subtract from usable vertical space 
      //var nameElementMarginBox = dojo.marginBox(this._chassisNameElement);
      // vertical size for the map faces = ChassisFrame - nameElement.h - fluffHeight (=margin&border of faces);
      var actualViewHeightForChassisFaces = this._viewHeight - /*nameElementMarginBox.h -*/ faceSizeInfo.fluffH;
      itmeUtils.CMTrace("LAYOUT", "Total frame height: "+ this._viewHeight + " - nameElement height: "+ /*nameElementMarginBox.h +*/ " - fluffHeight: " + faceSizeInfo.fluffH);
      itmeUtils.CMTrace("LAYOUT", "  = Actual View Height: " + actualViewHeightForChassisFaces);

      itmeUtils.traceTimeDelta("LAYOUT", "Time for Height Calculation", true);

      // WIDTH -       
      var actualViewWidthForChassisFaces = this._viewWidth - faceSizeInfo.fluffW;
      itmeUtils.CMTrace("LAYOUT", "view width: " + this._viewWidth + " , fluffW: " + faceSizeInfo.fluffW);
      if ( dojo.isIE)  // HACK. For some reason IE math doesn't add up the same... +12 seems to fix it.. not sure why yet.
        actualViewWidthForChassisFaces -= 12;
      itmeUtils.CMTrace("LAYOUT", "Total frame width: "+ this._viewWidth + " - fluffWidth: " + faceSizeInfo.fluffW);
      itmeUtils.CMTrace("LAYOUT", "  = Actual View width: "+ actualViewWidthForChassisFaces);

      itmeUtils.traceTimeDelta("LAYOUT", "Time for Width Calculation", true);
      
      var widthRatio = (actualViewWidthForChassisFaces) / faceSizeInfo.maxW; // had = (actualViewWidthForChassisFaces - 82) / faceSizeInfo.maxW;
      var heightRatio = (actualViewHeightForChassisFaces - this._filterBarHeight) / faceSizeInfo.maxH; 
      itmeUtils.CMTrace("LAYOUT", "target view size: w:"+this._viewWidth + " ,h:"+this._viewHeight);
      itmeUtils.CMTrace("LAYOUT", "target view size less fluff: w:"+actualViewWidthForChassisFaces + " ,h:"+actualViewHeightForChassisFaces);
      itmeUtils.CMTrace("LAYOUT", "ratio to target view size: w:"+widthRatio + " ,h:"+heightRatio);
      // take the smaller of the 2
      var usingRatio = Math.min(widthRatio, heightRatio);
      itmeUtils.CMTrace("LAYOUT", "using ratio: "+usingRatio);
      
      itmeUtils.traceTimeDelta("LAYOUT", "Final Processing", true);

      return 1;
      //return usingRatio;
    },

    // private
    // Iterates through the map faces defined and calculates the following
    // total size .totalW  .totalH  //
    // max size .maxW   .maxH
    _faceSizeInformation: null, // private var to cache results of the below
    _calculateFacesInfo: function()
    {
      if (this._faceSizeInformation)
        return this._faceSizeInformation;
        
      var sizes = [];
      sizes.maxW = sizes.maxH = sizes.fluffW = sizes.fluffH = sizes.totalW = sizes.totalH = 0;
      for ( var i = 0; i < this._chassisLayoutInformation.numberOfMapFaces; i++)
      {
        var contentBox = dojo.contentBox(this._chassisMapFaceElements[i]); // matches defined sizes on first load
        var marginBox = dojo.marginBox(this._chassisMapFaceElements[i]);

        sizes.maxW += this._chassisLayoutInformation.mapFaces[i].mapWidth; 
        sizes.fluffW += (marginBox.w - contentBox.w); // including border
        sizes.totalW += this._chassisLayoutInformation.mapFaces[i].mapWidth;
        
        if (this._chassisLayoutInformation.mapFaces[i].mapHeight > sizes.maxH) {
          sizes.maxH = this._chassisLayoutInformation.mapFaces[i].mapHeight;
          sizes.fluffH += (marginBox.h - contentBox.h); // including border
        }
        sizes.totalH += this._chassisLayoutInformation.mapFaces[i].mapHeight;
        
        itmeUtils.CMTrace("LAYOUT", "Face[" + i + "]: --------------------------");
        itmeUtils.CMTrace("LAYOUT", "   max: w:" + sizes.maxW + " ,h:" + sizes.maxH);
        itmeUtils.CMTrace("LAYOUT", "   face fluff: w:" + (marginBox.w - contentBox.w) + " , h:" + (marginBox.h - contentBox.h));
        itmeUtils.CMTrace("LAYOUT", "   face marginBox:  mb.w:" + marginBox.w + " , mb.h:" + marginBox.h);
        itmeUtils.CMTrace("LAYOUT", "   face contentBox: cb.w:" + contentBox.w + " , cb.h:" + contentBox.h);
      } 
      itmeUtils.CMTrace("LAYOUT", "Sum          : --------------------------");
      itmeUtils.CMTrace("LAYOUT", "   sizes: .maxW:" + sizes.maxW + " , maxH:" + sizes.maxH);
      itmeUtils.CMTrace("LAYOUT", "   sizes: .fluffW:" + sizes.fluffW + " , fluffH:" + sizes.fluffH);
      itmeUtils.CMTrace("LAYOUT", "   sizes: .totalW:" + sizes.totalW + " , totalH:" + sizes.totalH);
      itmeUtils.CMTrace("LAYOUT", "             : --------------------------");
      this._faceSizeInformation = sizes;
      return this._faceSizeInformation;
    },
    
    // private
    _reSizeChassisMapFacesToScale: function(scaleFactor)
    {
      for (var i = 0; i < this._chassisLayoutInformation.numberOfMapFaces; i++) {
        var computedStyle = dojo.style(this._chassisMapFaceElements[i]);
        var newWidth = parseInt(computedStyle.width) * scaleFactor;
        var newHeight = parseInt(computedStyle.height) * scaleFactor;
        itmeUtils.CMTrace("LAYOUT", "width delta: " + computedStyle.width + " -> " + newWidth);
        itmeUtils.CMTrace("LAYOUT", "height delta: " + computedStyle.height + " -> " + newHeight);
        this._chassisMapFaceElements[i].style.width = parseInt(newWidth);
        this._chassisMapFaceElements[i].style.height = parseInt(newHeight);
      }
    },
    
    // private
    _onFrameResize: function()
    {
      itmeUtils.CMTrace("LAYOUT", "RESIZE EVENT CAPTURED");
      if(this._resizingTimeoutDelay !== false)
        clearTimeout(this._resizingTimeoutDelay);
      this._resizingTimeoutDelay = setTimeout(dojo.hitch(this, this._resize), 500); //200 is time in miliseconds
    },
    
    // private
    _resize: function()
    {
      this._updateViewDimensions();
      var scaleFactor = this._getFacesToFrameRatio();
      itmeUtils.CMTrace("LAYOUT", " - Faces to Frame Ratio now: " + scaleFactor);
      
      // trace initial object sizs
      this.__traceObjectSizesAndPositioning();
      
      // RESIZE BEGIN
      this._resizingTimeoutDelay = false;
      var chassisLayoutInformation = this._chassisLayoutInformation;
      
      // Name element width = view width - name element fluff
      //this._chassisNameElement.style.width = this._viewWidth - this._getNodeFluffWidth(this._chassisNameElement);
      
      // Face elements size = defined size * scale factor
      for (var i = 0; i < this._numFaceElements; i++) {
        this._chassisMapFaceElements[i].style.width = parseInt( this._chassisLayoutInformation.mapFaces[i].mapWidth*scaleFactor);
        this._chassisMapFaceElements[i].style.height = parseInt( this._chassisLayoutInformation.mapFaces[i].mapHeight*scaleFactor);
      }
      
      // Element sizing = defined size * scale factor.  Additionally offsets are multiplied by the scale factor
      for (i = 0; i < chassisLayoutInformation.numberOfSlotTypes; i++) {
        if (!chassisLayoutInformation.slots[i]) 
          alert("Chassis Slot " + (i + 1) + " information is missing");
        else {
          
          for (var j = 0; j < chassisLayoutInformation.slots[i].slotCount; j++) {
            if (!chassisLayoutInformation.slots[i].slotPositioning[j]) 
              alert("Chassis Slot Position " + (j + 1) + " of slot type " + chassisLayoutInformation.slots[i].slotType + " is missing");
            else {
              
              // resize the element
              element = dojo.byId(chassisLayoutInformation.slots[i].slotPositioning[j].slotID);
              dojo.attr(element, "style", { "position": "absolute",
                                            "width": parseInt(chassisLayoutInformation.slots[i].slotWidth*scaleFactor) + "px",
                                            "height": parseInt(chassisLayoutInformation.slots[i].slotHeight*scaleFactor) + "px",
                                            "left": parseInt(chassisLayoutInformation.slots[i].slotPositioning[j].X * scaleFactor) + "px",
                                            "top": parseInt(chassisLayoutInformation.slots[i].slotPositioning[j].Y * scaleFactor) + "px",
                                            "overflow": "hidden",
                                            "text-overflow": "ellipsis" });
              
              // update the element constraint value
              element.constraint = this._determineElementConstraint(element);
              
              for (var sIndex in element.childNodes) {
                if (element.childNodes[sIndex].id) { // if this is a named node
                  element.childNodes[sIndex].style.width = parseInt(chassisLayoutInformation.slots[i].slotWidth*scaleFactor) + "px";
                  element.childNodes[sIndex].style.height = parseInt(chassisLayoutInformation.slots[i].slotHeight*scaleFactor) + "px";
                }
              }
              // if there is an image associated with the slot, resize that as well
              //if (dojo.byId(chassisLayoutInformation.slots[i].slotPositioning[j].slotID + "_IMG")) {
              //  element = dojo.byId(chassisLayoutInformation.slots[i].slotPositioning[j].slotID + "_IMG");
              //  element.style.width = parseInt(chassisLayoutInformation.slots[i].slotWidth*scaleFactor) + "px";
              //  element.style.height = parseInt(chassisLayoutInformation.slots[i].slotHeight*scaleFactor) + "px";
              //}
              
            }
          }
        }
      }
      
      // trace completed resizing object sizes
      this.__traceObjectSizesAndPositioning();
      
    },
    
    __traceObjectSizesAndPositioning: function() {
      itmeUtils.CMTrace("LAYOUT", "");
    },
    
    // private
    // gets the present container node size and sets our internal view dimensions we use for processing from it.
    _updateViewDimensions: function(){
      if (this._containingDomNode) { // can only get new dimensions if we were given a DOM node to base sizing off of.
        itmeUtils.CMTrace("LAYOUT", " - previous view dimensions: w: " + this._viewWidth + " , h: " + this._viewHeight);
        var contentBox = dojo.contentBox(this._containingDomNode);
        this._viewWidth = contentBox.w;
        this._viewHeight = contentBox.h;
        itmeUtils.CMTrace("LAYOUT", " - parent node id: " + this._containingDomNode.id);
        itmeUtils.CMTrace("LAYOUT", " - getting new view dimensions: w: " + this._viewWidth + " , h: " + this._viewHeight);
      }
    },
    
    // protected
    // for each SLOT of SLOTTYPE, merge those div elements (remove and rebuild the div tags)
    mergeSlots: function(slotId, slotType, slotsOccupied) {
      var elementArray = [];
      var topsMatch = true;
      var leftsMatch = true;
      var topMost = 0; var leftMost = 0; var rightMost = 0; var bottomMost = 0;
      var parentNode = null;
      for (var i = 0; i < slotsOccupied.length; i++)
      {
        elementArray[i] = dojo.byId(slotType+slotsOccupied[i]);
        
        if ( !parentNode )
          parentNode = elementArray[i].parentNode;  

        var right = parseInt(elementArray[i].style.left)+parseInt(elementArray[i].style.width);
        var bottom = parseInt(elementArray[i].style.top)+parseInt(elementArray[i].style.height);
        if ( topsMatch && ( i > 0 ) )
        {
          if ( elementArray[i].style.top != elementArray[i-1].style.top )
            topsMatch = false;
        }
        if ( leftsMatch && ( i > 0 ) )
        {
          if ( elementArray[i].style.left != elementArray[i-1].style.left )
            leftsMatch = false;
        }
        if ( i == 0 ) {
          topMost = parseInt(elementArray[i].style.top);
          leftMost = parseInt(elementArray[i].style.left);
          rightMost = right;
          bottomMost = bottom;
        }
        else {
          if ( parseInt(elementArray[i].style.top) < topMost )
            topMost = parseInt(elementArray[i].style.top);
          if ( parseInt(elementArray[i].style.left) < leftMost )
            leftMost = parseInt(elementArray[i].style.left);
          if ( right > rightMost )
            rightMost = right;
          if ( bottom > bottomMost )
            bottomMost = bottom;
        }
        
        //elementArray[i].style.backgroundImage = "none";
        parentNode.removeChild(elementArray[i]);
      }
      
      var newElement = itmeUtils.createElement(
                        slotType+slotId,
                        parentNode,
                        { styleClass: "Component EmptySlot", 
                          style: {  "position": "absolute",
                                    "width": (rightMost - leftMost) + "px",
                                    "height": (bottomMost - topMost) + "px",
                                    "left": leftMost+ "px",
                                    "top": topMost+ "px",
                                    "overflow": "hidden",
                                    "text-overflow": "ellipsis" } } );
                        
      newElement.constraint = this._determineElementConstraint(newElement);        
      return newElement;
    }


});

}

if(!dojo._hasResource["itme.ChassisMap._Selection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap._Selection"] = true;
dojo.provide("itme.ChassisMap._Selection");

dojo.declare("itme.ChassisMap._Selection", null, {

  _currentSelectionsByItemId: [],
  _refreshDelay: false,
  
  constructor: function(kwArgs){
    this._currentSelectionsByItemId = [];
  },

  // PUBLIC
  // A String or Comma delimited string containing 1 or more item ids of selected elements
  setSelection: function(selectedElementsByItemId) {
    if (selectedElementsByItemId) {
      itmeUtils.CMTrace("SELECTION", "Incoming Selection (ignored): " + selectedElementsByItemId);
    }

    var selectionList = dojo.cookie("chasisMap.currentSelection");
    itmeUtils.CMTrace("SELECTION", "selectionList = " + selectionList);

    if (selectionList) {
      var itsSplit = selectionList.split(",");
      itmeUtils.CMTrace("SELECTION", "TMP Split: " + itsSplit);
      for (var i = 0; i < itsSplit.length; i++)   
      {
        itmeUtils.CMTrace("SELECTION", "Selection["+i+"]: "+itsSplit[i]);
        this._currentSelectionsByItemId[itsSplit[i]+"_item"] = itsSplit[i];
      }
    }
    
    dojo.cookie("chasisMap.currentSelection", null, { expires: -1}); // remove the cookie if it exists
  },
  
  // protected - builds the selection list and kicks off a timer to fire the change
  _handleMultiSelectEvent: function(item) {
    // if this item is already selected, remove it
    if (this._currentSelectionsByItemId[item.getId() + "_item"]) {
      delete this._currentSelectionsByItemId[item.getId() + "_item"];
      dojo.removeClass(item.mapElement, "ComponentSelected");
    }
    else { // else add it 
      this._currentSelectionsByItemId[item.getId() + "_item"] = item.getId();
      dojo.addClass(item.mapElement, "ComponentSelected");
    }
    
    //TODO: I commented out all the ticker/refresh code since i don't know if it is needed (Alex)
    /*
    itmeUtils.CMTrace("SELECTION", "multi-select: current selections: " + itmeUtils.myJoin(this._currentSelectionsByItemId));
    itmeUtils.CMTrace("SELECTION", "Assoc array length: " + itmeUtils.associativeArrayLength(this._currentSelectionsByItemId));
    if (itmeUtils.associativeArrayLength(this._currentSelectionsByItemId) > 0) {
      itmeUtils.CMTrace("SELECTION", "refreshing ticker");
      this._refreshTicker();
    }
    else { // if the countdown for multi select is shown, hide it.
      itmeUtils.CMTrace("SELECTION", "hiding timer");
      var refreshDivElement = dojo.byId("refreshIncoming");
      if (refreshDivElement) {
        itmeUtils.CMTrace("SELECTION", "hiding timer");
        refreshDivElement.style.display = "none";
      }
    }*/
  },
  
  // protected - sets the selection list and kicks off a timer to fire the change
  _handleSingleSelectEvent: function(item) {
    // empty the selections list
    this._currentSelectionsByItemId = [];
    dojo.query(".ComponentSelected").forEach(function(node){
      dojo.removeClass(node, "ComponentSelected");
    });

    // if the countdown for multi select is shown, hide it.
    var refreshDivElement = dojo.byId("refreshIncoming");
    if (refreshDivElement)
      refreshDivElement.style.display = "none";
    
    // set selection
    this._currentSelectionsByItemId[item.getId()+"_item"] = item.getId();
    dojo.addClass(item.mapElement, "ComponentSelected");
  },
  
  // TYPE: ACTION
  _refreshTicker: function()
  {
    var refreshDivElement = dojo.byId("refreshIncoming");
    if(this._refreshDelay === false) {
      this._refreshTime = 5; // num seconds to pop

      itmeUtils.CMTrace("SELECTION", "refreshDivElement");
      if ( refreshDivElement ){
        refreshDivElement.style.display = "block";
      } else {
        refreshDivElement = itmeUtils.createElement(
            "refreshIncoming", 
            this._chassisMapFaceElements["front"],
            { styleClass: "refreshTimer", 
              style: {  "position": "absolute",
                        "right": "0px",
                        "top": "0px",
                        "overflow": "hidden",
                        "text-overflow": "ellipsis" },
              innerHTML: this.MRI.Refresh + this._refreshTime + this.MRI.Seconds } );
      }
      this._refreshDelay = setTimeout(dojo.hitch(this, this._refreshTicker), 1000);
      
    } 
    else {
      this._refreshTime -= 1; // num seconds to pop
      refreshDivElement.innerHTML = this.MRI.Refresh + this._refreshTime + this.MRI.Seconds;
      if (this._refreshTime > 0)
        this._refreshDelay = setTimeout(dojo.hitch(this, this._refreshTicker), 1000); // recurse in 1 second 
      else {
        // Process Page Refresh
        chassisMapRefresh(); //jsUtils
        this.cleanUp();
        processRefresh( dojo.cookie("chasisMap.currentSelection") );
        itmeUtils.CMTrace("SELECTION", "Refreshing page now: " + dojo.cookie("chasisMap.currentSelection") );

      }
    }
  }
});

}

if(!dojo._hasResource["itme.ChassisMap.OverlayBase"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.OverlayBase"] = true;
dojo.provide("itme.ChassisMap.OverlayBase");





/* Overlay interface rules 
 * 
 * Required Public Functions: 
 *   constructor(kwArgs , items (opt) ); 
 *   getId();     // return unique string for the overlay
 *   getTitle();  // return translatable string for the overlay
 *   getDescription(); // return translatable string describing the overlays purpose
 *   requiresProperties(); // return the dataset properties required for this overlay
 *   setItems(items); // chassis items list.  Note items.mapElement (if it exists) is the dom element associated with the item.
 *   renderOverlay(); // render or show the overlay
 *   unRenderOverlay(); // remove or hide the overlay rendering
 *   itemInformationModified(); NOT IMPLEMENTED YET // notification that info for an item has changed (re-render that element)
 *   
 *   // Guidelines for behavior
 *   // - for resizing to work, all created elements should become children of the items associated element (item.mapElement)
 *   // - unRender should hide created elements, so re-rendering only needs to show them instead of re-creating them.
 *   // - title and description should be translatable
 */

dojo.declare("itme.ChassisMap.OverlayBase", null, {
  _demoMode: false,
  _args: null,
  _items: null,
  _overlayId: null,
  _description: null,
  _title: null,
  _iconId: null,
  _requiredProperties: null,
  _availableFilters: null,
  _isEnabled: false,
  _flagNoTooltipRebuild: false,
  _hasBeenRendered: false,
  _isActive: null,
  MRI: null,
  IMAGES : new itme.ChassisMap.ImageResources(),
   
  constructor: function(kwArgs , items /* opt */){
    this._args = {    };
    
    this.MRI = dojo.i18n.getLocalization("itme", "Resources");
    //this._elements = elements;
    this._items = items;
    dojo.mixin(this._args, kwArgs);
  },
  
  setEnabled: function(isEnabled) {
    this._isEnabled = isEnabled;
  },
  
  isEnabled: function() {
    return this._isEnabled;
  },
  
  /* dom usable id name */
  getId: function()
  {
    return this._overlayId;
  },
  
  getTitle: function() {
    return this._title;
  },
  
  getDescription: function() {
    return this._description;
  },
  
  getIconId: function() {
    return this._iconId;
  },
  
  getAvailableFilters: function() {
    return this._availableFilters;
  },
  
  // utils should be able to do a comparison between available and the return
  requiresProperties: function() {
    return this._requiredProperties;
  },
  
  setItems: function(items) {
    itmeUtils.CMTrace("OVERLAY", "setItems reached");
    this._items = items;
  },
  
  renderOverlay: function(includeTooltip){
    if ( typeof(includeTooltip) == 'undefined' ) {
      if ( !this._hasBeenRendered )
        includeTooltip = true;
      else
        includeTooltip = !this._flagNoTooltipRebuild; // default buildTooltip to true if not passed in
    }
    this._flagNoTooltipRebuild = false;
    this._isEnabled = true;
    
    // iterate through items... which have the associated dom elements set as item.mapElement
    itmeUtils.CMTrace("OVERLAY", "rendering "+this._overlayId+" overlay - demoMode? "+ this._demoMode + " - +tooltip? " + includeTooltip);
    this._preProcessing(this._items);
    for (var i = 0; i < this._items.length; i++) {
      if (this._items[i].mapElement) {
        this._renderItem(this._items[i], includeTooltip);
        this._items[i].mapElement.CMTooltip.toggleFocusEnabled(this._overlayId, true);
        this._items[i].mapElement.CMTooltip.buildTooltip();
      }
    }
    this._postProcessing(this._items);
    this._hasBeenRendered = true;
  },
  
  
  _renderItem: function(item) {
    // OVERRIDE REQUIRED
  },
  
  _preProcessing: function(items) {
    // OVERRIDE IF NEEDED
  },
  
  _postProcessing: function(items) {
    // OVERRIDE IF NEEDED
  },
  
  unRenderOverlay: function(){
    this._isEnabled = false;
    itmeUtils.CMTrace("OVERLAY", "unrendering (BASE) "+this._overlayId+" overlay");
    for (var i = 0; i < this._items.length; i++)   {
      if (this._items[i].mapElement) {
        this._unsetOverlayForItem(this._items[i]);
        this._items[i].mapElement.CMTooltip.toggleFocusEnabled(this._overlayId, false);
        this._items[i].mapElement.CMTooltip.buildTooltip();
      }
    }
  },
  
  deleteOverlay: function() {
    // iterates through each element created by this overlay and deletes them, to be re-rendered
    this.unRenderOverlay();
    this._flagNoTooltipRebuild = true; // this is because when deleting the overlay, we don't delete the tooltips
    
    for (var i = 0; i < this._items.length; i++)   {
      if (this._items[i].mapElement) {
        this._deleteOverlayForItem(this._items[i]);
        this._items[i].mapElement.CMTooltip.toggleFocusEnabled(this._overlayId, false);
        //this._items[i].mapElement.CMTooltip.buildTooltip();
      }
    }
    
  },

  //override this method if 
  // 1. you did not use "_"+_overlayId to append element divs  or...
  // 2. you need to unrender more than 1 created div
  // 3. you do not want to unrender "_"+_overlayId default that this one does
  _unsetOverlayForItem: function(item)
  {
    itmeUtils.CMTrace("OVERLAY", "_unsetOverlayForItem (BASE): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId);
    if ( itemsDivElement) 
      itemsDivElement.style.display = "none";
  },
  
  _deleteOverlayForItem: function(item) {
    //itmeUtils.CMTrace("OVERLAY", "_deleteOverlayForItem (BASE): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId);
    if ( itemsDivElement) 
      itemsDivElement.parentNode.removeChild(itemsDivElement);
  },
  
  itemInformationModified: function(item){
    /* override me */
  },
  
  // PROTECTED - for use as needed:
  _setDimensionsAndTextDirection: function(item, itemsDivElement){
      
      var HeightToWidthFactorForTextRotation = 1.5;
      
      var cBox = dojo.contentBox(item.mapElement);
      
      
      if ((cBox.h / cBox.w) < HeightToWidthFactorForTextRotation) 
        // no text rotation
        dojo.contentBox(itemsDivElement, {w: cBox.w, h: cBox.h});
      else {
        // rotate text 
        dojo.addClass(itemsDivElement, "verticalizeText");
        
        if (dojo.isFF || itmeUtils.isChrome()) { // MOZILLA & CHROME 
          // mozilla based browsers leave the box in a strange position... 
          // ... this is a hack to fix that when the css rotates it 
          //itmeUtils.CMTrace("OVERLAY", "FF & MOzilla based rendering chkpoint");
          dojo.contentBox(itemsDivElement, { w: cBox.h, h: cBox.w });
          
          var magicNumber = (cBox.h - cBox.w) / 2; // I don't quite understand why this works, but it does.
          itemsDivElement.style.left = "-" + magicNumber + "px";
          itemsDivElement.style.top = magicNumber + "px";
        }
        else { // IE
          //itmeUtils.CMTrace("OVERLAY", "IE based rendering chkpoint");
          dojo.contentBox(itemsDivElement, { w: cBox.w, h: cBox.h });
        }
      }
  },
  
  _addPropertyToTooltipIfAvailable: function(item, propId) {
    prop = item.getProperty(propId);
    if ( prop ) {
      itmeUtils.CMTrace("OVERLAY", item.mapElement.id + " : Prop [ "+prop.getLabel()+"("+propId+") ] = " + prop.getDisplayValue());
      item.mapElement.CMTooltip.addFocusProperty(this._overlayId, prop.getLabel(), prop.getDisplayValue());
    }
  },
  
  setOverlayActiveState: function(activeState) {
    this._isActive = activeState;
  },
  
  isOverlayActive: function() {
    return this._isActive
  }

  

});

}

if(!dojo._hasResource["itme.ChassisMap.OverlayFilterItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.OverlayFilterItem"] = true;
dojo.provide("itme.ChassisMap.OverlayFilterItem");

dojo.declare("itme.ChassisMap.OverlayFilterItem", null, {
  //_args: null,
  _filterId: null,        // unique ID
  _icon: null,            // icon image
  _description: null,     // shown text
  _isCheckEnabled: false, // is this filter enabled
  _allowModification: true,
  //_parentElement: null,   // parent div element to render in
  //_filterPriority: null,  // needed to estabolish priority or ordering when filters overlap ( 0 - oo)
  
  constructor: function(kwArgs, filterId, iconId, description, enabled, allowModification) {
    this._args = {    };
    this._filterId = filterId;
    this._iconId = iconId;
    this._description = description;
    //this._priority = priority;
    this._isCheckEnabled = enabled;
    this._allowModification = allowModification;
    //dojo.mixin(this._args, kwArgs);
  },

  getId: function() {
    return this._filterId;
  },
  
  getIconId: function() {
    return this._iconId;
  },
  
  getDescription: function() {
    return this._description;
  },
  
  getPriority: function() {
    return this._priority;
  },
  
  setEnabled: function(isEnabled) {
    this._isCheckEnabled = isEnabled;
  },
  
  isEnabled: function() {
    return this._isCheckEnabled;
  },
  
  allowModification: function() {
    return this._allowModification;
  },
  
  renderItem: function(item) {
    // render this item   
    //   - in its own Div?
  }
  

});

}

if(!dojo._hasResource["itme.ChassisMap.StatusOverlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.StatusOverlay"] = true;
dojo.provide("itme.ChassisMap.StatusOverlay");




dojo.declare("itme.ChassisMap.StatusOverlay", [itme.ChassisMap.OverlayBase], {

  _diffForBorder: null,
  _maxShownActiveStatusEntriesForTooltip: 6,
    
  constructor: function(kwArgs){
    // parent constructor is always auto-called first
    this._overlayId = "status";
    this._description = this.MRI.StatusOverlayDetail;
    this._title = this.MRI.StatusTitle;
    this._requiredProperties = [ "STATUS" ];
    this._iconId = "StatusFilterIcon";
    
    this._availableFilters = [];
    this._criticalFilter = new itme.ChassisMap.OverlayFilterItem({}, "criticalFilter", "StatusCritical", this.MRI.Critical, true, false );
    this._availableFilters[0] = this._criticalFilter;
    this._warningFilter = new itme.ChassisMap.OverlayFilterItem({}, "warningFilter", "StatusWarningMinor", this.MRI.Warnings, true, true);
    this._availableFilters[1] = this._warningFilter;
    this._informationalFilter = new itme.ChassisMap.OverlayFilterItem({}, "informationalFilter", "StatusInformational", this.MRI.Informational, true, true); 
    this._availableFilters[2] = this._informationalFilter; 
    this._normalFilter = new itme.ChassisMap.OverlayFilterItem({}, "normalFilter", "StatusNormal", this.MRI.Normal, false, true); 
    this._availableFilters[3] = this._normalFilter; 
  },
  
  _renderItem: function(item, buildTooltip){
    var iconSize = 22, problems, statusStringForCSS, CommStateProp, isOK = true;
    var mapElementCBox = dojo.contentBox(item.mapElement);
    var itemsStatusDivElement = dojo.byId(item.mapElement.id + "_status");
    
    // if we've already created the element before just show it, otherwise create it
    if (itemsStatusDivElement) {
      itemsStatusDivElement.style.display = "block";
      if ( dojo.byId(item.mapElement.id + "_status2") )
        dojo.byId(item.mapElement.id + "_status2").style.display = "block";
    }
    else {
      // create element
      itemsStatusDivElement = itmeUtils.createElement(
          item.mapElement.id + "_status", 
          item.mapElement,
          { styleClass: "defaultStatus",
            baseDimensionsOnNode: item.mapElement} );
     
      // add element name
      if ( item.getType() == "com.ibm.sysmgt.platform.PhysicalPlatformFactory" ||
           item.getType() == "SystemChassis" ) 
        itmeUtils.insertEllipsedTextSpanIntoElement(itemsStatusDivElement, item.getLabel(), {styleClass: "secondaryElementName"}); // el, class, text
      
      if ( item.mapElement.constraint < iconSize )
        iconSize = item.mapElement.constraint;//imgStyleString = " style='height:"+item.mapElement.constraint+"px' ";

      problems = dojo.fromJson( itmeUtils.getProperty(item, "Problems") );
      if ( problems ) {
        //var problemTooltipHTML = "";  
        statusStringForCSS = null;

        CommStateProp = item.getProperty("CommunicationState");
        if ( CommStateProp && ( CommStateProp.getValue() == 3) ) // lost communication (aka OFFLINE) 
        {
          statusStringForCSS = "Offline";
          itemsStatusDivElement.innerHTML = this.IMAGES.buildIconImageTag("AccessOfflineOK", "", iconSize) + itemsStatusDivElement.innerHTML;
          isOK = false;
        }
        
        if (problems.PBLM_I && (problems.PBLM_I > 0)) {
          isOK = false;
          //problemTooltipHTML = problems.PBLM_I + this.IMAGES.buildIconImageTag("StatusInformational", "", 16);
          if ( this._informationalFilter.isEnabled() ) 
            itemsStatusDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusInformational", "", iconSize) + itemsStatusDivElement.innerHTML;
        } 
        if (problems.PBLM_W && (problems.PBLM_W > 0)) {
          isOK = false;
          //problemTooltipHTML = problems.PBLM_W + this.IMAGES.buildIconImageTag("StatusWarningMinor", "", 16) + " " + problemTooltipHTML;
          if ( this._warningFilter.isEnabled() ) {
            statusStringForCSS = "Attention";
            itemsStatusDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusWarningMinor", "", iconSize) + itemsStatusDivElement.innerHTML;
          }
        }
        if (problems.PBLM_E && (problems.PBLM_E > 0)) {
          isOK = false;
          //problemTooltipHTML = problems.PBLM_E + this.IMAGES.buildIconImageTag("StatusCritical", "", 16) + " " + problemTooltipHTML;
          if ( this._criticalFilter.isEnabled() ) {
            statusStringForCSS = "Critical";
            itemsStatusDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusCritical", "", iconSize) + itemsStatusDivElement.innerHTML;
          }
        }
        
        if ( (isOK == true) && (this._normalFilter.isEnabled()) ) {
          itemsStatusDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusNormal", "", iconSize) + itemsStatusDivElement.innerHTML;
        }
        
      }
      
      if ( buildTooltip ) {
        /*var statusArray = item.getStatuses();
        for (var i = 0; i < statusArray.length; i++) {
          itmeUtils.CMTrace("OVERLAY", "ITEM Status["+i+"]: " + statusArray[i].getId() + " - " + statusArray[i].getLabel());
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.Status, "<img src='" + statusArray[i].getImage() + "'> "+statusArray[i].getLabel() );
        }
        */
        //this._addPropertyToTooltipIfAvailable(item, "OperatingState");

        if ( CommStateProp && ( CommStateProp.getValue() == 3) ) // lost communication (aka OFFLINE), add offline icon to 'offline' string. 
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, CommStateProp.getLabel(), this.IMAGES.buildIconImageTag("AccessOfflineOK", "", 16) + " " + CommStateProp.getDisplayValue());
        else
          this._addPropertyToTooltipIfAvailable(item, "CommunicationState");
        
        //this._addPropertyToTooltipIfAvailable(item, "PowerState");

        this._addActiveStatusEntries(item);
      }
    
      // if we have a non-normal status we handle:
      if ( statusStringForCSS ) 
      {
        // for the initial div, append the status class
        dojo.addClass(itemsStatusDivElement, "ComponentStatus"+statusStringForCSS);
        
        // due to the border for this overlay, we shrink the element based our css border size x2
        this._calculateBorderDiff(mapElementCBox, itemsStatusDivElement);
        dojo.contentBox(itemsStatusDivElement, {w: mapElementCBox.w - this._diffForBorder,h: mapElementCBox.h - this._diffForBorder});
        
        // add yet another div for critical status, the css class should give it a red or yellow hue            
        var itemsStatusHighlightDivElement = itmeUtils.createElement(
            item.mapElement.id + "_status2", 
            item.mapElement,
            { styleClass: "ComponentStatus"+statusStringForCSS+"Shadow" } );

        dojo.contentBox(itemsStatusHighlightDivElement, {w: mapElementCBox.w, h: mapElementCBox.h }); 
      }
    }
  },

  _addActiveStatusEntries: function(item) {
    var activeStatusEntries = dojo.fromJson( itmeUtils.getProperty(item, "ACTIVESTATUSINFO") );
    if ( activeStatusEntries.STATUS_ENTRIES && (activeStatusEntries.STATUS_ENTRIES.length > 0) ) {

      if ( item.getType() == "Fan" || item.getType() == "PowerSupply")
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, "", this.MRI.FindActiveStatusOnChassis);

    	item.mapElement.CMTooltip.addFocusProperty(this._overlayId, "<hr>", "<hr>");
    	  
    	itmeUtils.CMTrace("OVERLAY", "Status entries on item ("+item.mapElement.id+") = " + activeStatusEntries);

      var sortedStatusEntryList = this._sortStatusEntryList(activeStatusEntries.STATUS_ENTRIES);
        
      for (var i = 0; i < sortedStatusEntryList.length; i++) {

        if ( i == this._maxShownActiveStatusEntriesForTooltip )
      	  item.mapElement.CMTooltip.addFocusProperty(this._overlayId, "", dojo.replace(this.MRI.MoreStatusEntries, [ sortedStatusEntryList.length - this._maxShownActiveStatusEntriesForTooltip ] ) );
        else if ( i > this._maxShownActiveStatusEntriesForTooltip )
      	  break;
        else {
          var imageTagString = "";
          if ( this._isStatusCritical(sortedStatusEntryList[i].SEVERITY))
        	  imageTagString = this.IMAGES.buildIconImageTag("StatusCritical", "", 16);
          if ( this._isStatusWarning(sortedStatusEntryList[i].SEVERITY))
        	  imageTagString = this.IMAGES.buildIconImageTag("StatusWarningMinor", "", 16);
          if ( this._isStatusInfo(sortedStatusEntryList[i].SEVERITY))
        	  imageTagString = this.IMAGES.buildIconImageTag("StatusInformational", "", 16);
          
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, imageTagString, sortedStatusEntryList[i].PROBLEMNAME);
        }
      }
    }
  },
  
  _sortStatusEntryList: function(statusEntryList) {
	  var newStatusEntryList = [];
	  var newListCounter = 0;
	  for ( var i = 0; i < statusEntryList.length; i++ ) {
		  if ( this._isStatusCritical(statusEntryList[i].SEVERITY))
			  newStatusEntryList[newListCounter++] = statusEntryList[i];
	  }
	  for ( i = 0; i < statusEntryList.length; i++ ) {
		  if ( this._isStatusWarning(statusEntryList[i].SEVERITY))
			  newStatusEntryList[newListCounter++] = statusEntryList[i];
	  }
	  for ( i = 0; i < statusEntryList.length; i++ ) {
		  if ( this._isStatusInfo(statusEntryList[i].SEVERITY))
			  newStatusEntryList[newListCounter++] = statusEntryList[i];
	  }
	  
	  return newStatusEntryList;
  },
  
  _isStatusCritical: function(statusId) {
	  if ( (statusId == 0)  || (statusId == 1))
		  return true;
	  return false;
  },
  
  _isStatusWarning: function(statusId) {
	  if ( (statusId == 2) || (statusId == 3) ) // MINOR or WARNING 
		return true;
	  return false;
  },
  
  _isStatusInfo: function(statusId) {
	  if ( statusId == 4 )
		  return true;
	  return false;
  },
  
  _isStatusUnknown: function(statusId) {
	  if ( statusId == 5 )
		  return true;
	  return false;
  },
  
  _isStatusNormal: function(statusId) {
	  if ( statusId == -1 )
		  return true;
	  return false;
  },
  
  _calculateBorderDiff: function(cBox, element) {
    this._diffForBorder = 4; // HACK:  IE failed to calculate border correctly.  CSS Margin = 2.  2x2 = 4.
    if (this._diffForBorder) {
      //itmeUtils.CMTrace("OVERLAY", "calc border diff returning : " + this._diffForBorder);
      return this._diffForBorder;
    }
      
    var mBox = dojo.marginBox(element);
    this._diffForBorder = mBox.w - cBox.w;
    return this._diffForBorder;
  },
  
  // renderOverlay: function(){
  // Default preprocessing for rendering.  Calls into the REQUIRED OVERRIDE method _renderItem
  
  // unRenderOverlay: function()
  // Unrender the overlay.  Parent function should suffice for all cases
  
  _unsetOverlayForItem: function(item)
  // Hide what was rendered by renderOverlay.  Override this if:
    // 1. you did not use "_"+_overlayId to append element divs  or...
    // 2. you need to unrender more than 1 created div
    // 3. you do not want to unrender "_"+_overlayId default that this base class does
  {
    this.inherited(arguments);
    //itmeUtils.CMTrace("OVERLAY", "_unsetOverlayForItem (STATUS): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "2");
    if ( itemsDivElement) 
      itemsDivElement.style.display = "none";
  },
  
  _deleteOverlayForItem: function(item) {
    this.inherited(arguments);
    //itmeUtils.CMTrace("OVERLAY", "_deleteOverlayForItem (BASE): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+"2");
    if ( itemsDivElement) 
      itemsDivElement.parentNode.removeChild(itemsDivElement);
  }
  
  // itemInformationModified: function(item)
  // Unused
  
  // getId: function()
  // Return the overlay id.  This is set in the constructor and should not need overriding.

  // getTitle: function() {
  // Return the title for the overlay.  This is set in the constructor and should not need overriding.

  // getDescription: function() {
  // Return the description for the overlay.  This is set in the constructor and should not need overriding.
  
  // requiresProperties: function() {
  // Unused
  // Gets the required properties for this overlay.  This is set in the constructor and should not need overriding.
  
  // setItems: function(items) {
  // Sets the items for the overlay. Parent function should suffice unless special processing is needed 
  
  
});

}

if(!dojo._hasResource["itme.ChassisMap.KnownHardwareInfo"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.KnownHardwareInfo"] = true;
dojo.provide("itme.ChassisMap.KnownHardwareInfo");

dojo.declare("itme.ChassisMap.KnownHardwareInfo", null, {

  _Nodes: [
      // FIREBIRDS ==========================
      {
        // 4-way PPC Blade - Firebird 4S (Full-wide) - Flex System p460 Compute Node
        Model: "42X",
        MachineType: "7895",
        FRU: "00E0728",
        ImageResource: "ITE_C5440P7",
        Slots: 2
      },
      {
        // 2-way PPC Blade - Firebird 2S(Half-wide) - Flex System p260 Compute Node
        Model: "22X",
        MachineType: "7895",
        FRU: "00E0728",
        ImageResource: "ITE_C5240P7",
        Slots: 1
      },
      // FSMS ==========================
      {
        // 1 way x86 blade - IBM Flex System Manager
        Model: "",//A1Y
        MachineType: "8731",
        FRU: "90Y3193",
        ImageResource: "FSM",
        isFSM: true
      },
      {
        // 1-way x86 Blade - IBM Flex System C3240 M5 Compute Node - IBM Flex System Manager
        Model: "",//01M
        MachineType: "7955",
        ImageResource: "FSM",
        FRU: "90Y3193",
        isFSM: true
      },
      // BLACKTIPS ==========================
      {
        // 2 way x86 blade - IBM Flex System C3240 M5 Compute Node - Flex System x240 Compute Node
        Model: "",//AC1
        MachineType: "8737",
        ImageResource: "ITE_C42x0M4_PEN",
        FRU: "81Y5128",
        Slots: 2
      },
      {
        // 2 way x86 blade - IBM Flex System C3240 M5 Compute Node - Flex System x240 Compute Node
        Model: "",//AC1
        MachineType: "8737",
        ImageResource: "ITE_C42x0M4",
        FRU: "81Y5128",
        Slots: 1
      },
      { // (KESTREL)
        //  - Flex System x220 Compute Node
        Model: "",
        MachineType: "7906",
        ImageResource: "ITE_C42x0M4_PEN",
        FRU: "",
        Slots: 2
      },
      { // (KESTREL)
        //  - Flex System x220 Compute Node
        Model: "",
        MachineType: "7906",
        ImageResource: "ITE_C42x0M4",
        FRU: "",
        Slots: 1
      },
      {
        //  - Flex System x240 Compute Node
        Model: "",
        MachineType: "7863",
        ImageResource: "ITE_C42x0M4",
        FRU: ""
      },
      { // (CONGO)
        //  - Flex System x440 Compute Node
        Model: "",
        MachineType: "7917",
        ImageResource: "ITE_C4440M4",
        FRU: ""
      },
      // STORAGE ==========================
      {
        //  - Flex System V7000 Storage Node
        Model: "A49",
        MachineType: "4939",
        ImageResource: "STRW_493949", // ?? need to determine between : SRC Controller model 4939-49 & SDE Expansion model 4939-29
        FRU: ""
      },
      {
        //  - Flex System V7000 Storage Node
        Model: "A29",
        MachineType: "4939",
        ImageResource: "STRW_493929", // ?? need to determine between : SRC Controller model 4939-49 & SDE Expansion model 4939-29
        FRU: ""
      },
      // OTHER ==========================
      { // (CMM)
        // - IBM Flex System Chassis Management Module
        Model: "",//68Y
        MachineType: "7030",
        ImageResource: "CMM",
        FRU: ""
      },
      { // (PME)
        // - IBM Flex System PCI Expansion Unit
        Model: "",//81Y
        MachineType: "8983",
        ImageResource: "PME",
        FRU: ""
      },
      // SWITCHES ==========================
      {
        // COMPASS
        // IBM Flex System EN4093 10Gb Virtual Fabric Scalable Switch - 10Gb BNT Switch Module for Accipiter chassis
        Model: "49Y",
        MachineType: "4270",
        ImageResource: "SwitchEN4093",
        FRU: "",
        ProductId: 322
      },
      { // - IBM Flex System EN4093 10Gb Virtual Fabric Scalable Switch-Upgrade 1
        Model: "49Y",
        MachineType: "4798",
        ImageResource: "SwitchEN4093",
        FRU: "",
        ProductId: 322
      },
      { // - IBM Flex System EN4093 10Gb Virtual Fabric Scalable Switch-Upgrade 2
        Model: "88Y",
        MachineType: "6037",
        ImageResource: "SwitchEN4093",
        FRU: "",
        ProductId: 322
      },
      { // POLLUX
        // IBM Flex System EN2092 1Gb Ethernet Scalable Switch - BL5221 BNT Ethernet Scalable Switch
        Model: "49Y",
        MachineType: "4294",
        ImageResource: "SwitchEN2092",
        FRU: "",
        ProductId: 347
      },
      { //  - IBM Flex System EN2092 1Gb Ethernet Scalable Switch-Upgrade 1
        Model: "90Y",
        MachineType: "3562",
        ImageResource: "SwitchEN2092",
        FRU: "",
        ProductId: 347
      },
      { //  - IBM Flex System EN2092 1Gb Ethernet Scalable Switch-10Gb Uplinks
        Model: "49Y",
        MachineType: "4298",
        ImageResource: "SwitchEN2092",
        FRU: "",
        ProductId: 347
      },
      { // PROCYON
        // IBM Flex System EN4091 10Gb Ethernet Pass-thru - Procyon switch for Accipiter

        Model: "88Y",
        MachineType: "6043",
        ImageResource: "SwitchEN4091",
        FRU: "",
        ProductId: 369
      },
      { // SAMURI FABRIC
        // IBM Flex System FC3171 8Gb SAN Switch - Switch Qlogic 8Gb 20-port ScSE – Samurai

        Model: "69Y",
        MachineType: "1930",
        ImageResource: "SwitchFC3171",
        FRU: "",
        ProductId: 327
      },
      { //  - IBM Flex System FC3171 8Gb SAN Pass-thru - IPM Qlogic 8Gb 20-port ScSE – Samurai
        Model: "69Y",
        MachineType: "1934",
        ImageResource: "SwitchFC3171",
        FRU: "",
        ProductId: 348
      },
      { // PHAROS
        // IBM Flex System FC5022 24-port 16Gb SAN Scalable Switch - IBM Flex System FC5022 16Gb SAN Scalable Switch - Brocade 16Gb ScSE (Pharos)
        Model: "88Y",
        MachineType: "6374",
        ImageResource: "SwitchFC5022",
        FRU: "",
        ProductId: 329
      },
      { // IBM Flex System FC5022 24-port 16Gb ESB SAN Scalable Switch - IBM Flex System FC5022 24-port 16Gb ESB SAN Scalable Switch
        Model: "90Y",
        MachineType: "9356",
        ImageResource: "SwitchFC5022",
        FRU: "",
        ProductId: 329
      },
      { // SUBMARINE
        // IBM Flex System IB6131 Infiniband Switch - QDR/FDR IB switch. Code name: Submarine 36 port Mellanox Infiniband Switch
        Model: "90Y",
        MachineType: "3450",
        ImageResource: "SwitchIB6131",
        FRU: "",
        ProductId: 365
      },
      { // IBM Flex System IB6131 Infiniband Switch - IBM Flex System IB6131 Infiniband Switch-FDR Upgrade
        Model: "90Y",
        MachineType: "3462",
        ImageResource: "SwitchIB6131",
        FRU: "",
        ProductId: 365
      },
      // WILDCAT - IBM Flex System EN4054 4-port 10Gb Ethernet Adapter - ?
      //         - IBM Flex System CN4054 10Gb Virtual Fabric Adapter - 90Y3554
      //         - IBM Flex System CN4054 Virtual Fabric Adapter-SW Upgrade - 90Y3558
      // MUSTANG - IBM Flex System EN2024 4-port 1Gb Ethernet Adapter - 49Y7900
      // NINJA - IBM Flex System FC3172 2-port 8Gb FC Adapter - 69Y1938
      { // CHINOOK
        //  - IBM Flex System FC5022 2-port 16Gb FC Adapter
        Model: "88Y",
        MachineType: "6370",
        ImageResource: "SwitchFC5022",
        FRU: "",
        ProductId: 9
      }
      // MALAYA - IBM Flex System IB6132 2-port QDR Infiniband Adapter - ?
      //        - IBM Flex System IB6132 2-port FDR Infiniband Adapter - 90Y3454
      //        - IBM Flex System EN4132 2-port 10Gb Ethernet Adapter - 90Y3466
      // RHEA - IBM Flex System FC3052 2-port 8Gb FC Adapter - 95Y2375

    ],

  constructor: function(kwArgs){},

  getImageResourceForModelAndType: function(model, machineType, numSlots) {
    for (var i = 0; i < this._Nodes.length; i++)
    {
      if ( machineType == this._Nodes[i].MachineType ) {
        if ( this._Nodes[i].Slots && this._Nodes[i].Slots != numSlots)
        //if ( this._Nodes[i].Model != "" && this._Nodes[i].Model != model )
          continue;

        return this._Nodes[i].ImageResource;
      }
    }
  },

  getImageResourceForProductId: function(productId) {
    if (!productId)
      return;

    for (var i = 0; i < this._Nodes.length; i++)
    {
      if ( this._Nodes[i].ProductId && (productId == this._Nodes[i].ProductId) )
        return this._Nodes[i].ImageResource;
    }
  },

  isFSM: function(machineType) {
    for (var i = 0; i < this._Nodes.length; i++)
    {
      if ( machineType == this._Nodes[i].MachineType ) {
        if ( this._Nodes[i].isFSM && this._Nodes[i].isFSM === true)
          return true;
      }
    }
    return false;
  }

});

}

if(!dojo._hasResource["itme.ChassisMap.PropertiesOverlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.PropertiesOverlay"] = true;
dojo.provide("itme.ChassisMap.PropertiesOverlay");





dojo.declare("itme.ChassisMap.PropertiesOverlay", [itme.ChassisMap.OverlayBase], {
   
  constructor: function(kwArgs){
    // parent constructor is always auto-called first
    this._overlayId = "name";
    this._description = this.MRI.NameOverlayDetail;
    this._title = this.MRI.NameOverlayTitle;
    this._iconId = "LabelIcon_22";
    
    this._availableFilters = [];
    this._nameFilter = new itme.ChassisMap.OverlayFilterItem({}, "nameFilter", "", this.MRI.Name, true, true); 
    this._availableFilters[0] = this._nameFilter; 
    this._ipFilter = new itme.ChassisMap.OverlayFilterItem({}, "ipFilter", "", "IP Address(s)", false, true );
    this._availableFilters[1] = this._ipFilter;
    this._fruFilter = new itme.ChassisMap.OverlayFilterItem({}, "fruFilter", "", "FRU", false, true);
    this._availableFilters[2] = this._fruFilter;
    this._serialNumberFilter = new itme.ChassisMap.OverlayFilterItem({}, "serialNumber", "", this.MRI.SerialNumber, false, true); 
    this._availableFilters[3] = this._serialNumberFilter; 

  },
  
  _renderItem: function(item, buildTooltip){
    var text = "", prop, enabledCount = 0, fontClass;
    
    // if we've already created the element before just show it, otherwise create it
    var overlayDivElement = dojo.byId(item.mapElement.id + "_name");
    if ( overlayDivElement)  {
      overlayDivElement.style.display = "block";
    }
    else { 
      // create it
      overlayDivElement = itmeUtils.createElement(
          item.mapElement.id + "_name", 
          item.mapElement,
          { elementType: "span",
            styleClass: "elementName" } );

      overlayDivElement.innerHTML = "";
      // set contents
      
      for ( var i = 0; i < this._availableFilters.length; i++ )
        if ( this._availableFilters[i].isEnabled() )
          enabledCount++;
          
      fontClass = "propElement";
      if ( enabledCount > 1) // 2 or 3
        fontClass = "propElement_smallFont";
      if ( enabledCount > 3) // 4 (max supported)
        fontClass = "propElement_smallestFont";

      var propStrings = [];
      var propIndex = 0;
      
      if ( this._nameFilter.isEnabled() ) 
        propStrings[propIndex++] = item.getLabel();

      if ( this._ipFilter.isEnabled() ) {
        prop = item.getProperty("IPv4Address");
        if ( prop && (prop.getDisplayValue() != "")) {
          /*var ellipsedString = itmeUtils.insertEllipsedTextSpanIntoElement(item.mapElement, 
                                                                           "IP: " + prop.getDisplayValue(), 
                                                                           { returnOnly: true, 
                                                                             insertAfter: "IP: ",
                                                                             styleClass: fontClass + " elementName"});*/
          propStrings[propIndex++] = "IP: " + prop.getDisplayValue();//ellipsedString;
        }
      }
      
      if ( this._fruFilter.isEnabled() ) {
        prop = item.getProperty("EnclosureFRU");
        if ( prop && (prop.getDisplayValue() != "")) 
          propStrings[propIndex++] = "FRU: " + prop.getDisplayValue();
      }
      
      if ( this._serialNumberFilter.isEnabled() ) {
        prop = item.getProperty("SerialNumber");
        if ( prop && (prop.getDisplayValue() != "") ) 
          propStrings[propIndex++] = "SN: " + prop.getDisplayValue();
      }
      
      itmeUtils.CMTrace("OVERLAY", "numProps: " + propIndex);

      for (i = 0; i < propStrings.length; i++) {
        itmeUtils.CMTrace("OVERLAY", "prop["+i+"]: " + propStrings[i]);
        if ( ( i == 1) && ( item.getType() == "SystemChassis" ) ) 
        {
          itmeUtils.createElement(
              item.mapElement.id + "_nameChassisPt2", 
              overlayDivElement,
              { elementType: "span",
                styleClass: "chassisBottomText " + fontClass,
                style: {"zIndex": "3"},
                innerHTML: propStrings[i]} ) ; 
        } else if ( ( i > 1) && ( item.getType() == "SystemChassis" ) )
          ;
        else {
          overlayDivElement.innerHTML = overlayDivElement.innerHTML + "<span class='"+fontClass+"'>"+propStrings[i]+"</span>";
        }
      }
      
      
      // verticalize text      
      this._setDimensionsAndTextDirection(item, overlayDivElement);  

      //this._addPropertyToTooltipIfAvailable(item, "DisplayName");
      if ( buildTooltip ) {
        this._addPropertyToTooltipIfAvailable(item, "ChassisSlot");
        this._addPropertyToTooltipIfAvailable(item, "OccupiedSlots");
        this._addPropertyToTooltipIfAvailable(item, "DisplayOperationalStatusTime");
        this._addOSProperty(item);
        this._addPropertyToTooltipIfAvailable(item, "HostName");
        this._addPropertyToTooltipIfAvailable(item, "IPv4Address");
        this._addPropertyToTooltipIfAvailable(item, "MACAddress");
        
        this._addPropertyToTooltipIfAvailable(item, "SystemBoardUUID");
        
        this._addPropertyToTooltipIfAvailable(item, "Architecture");
        this._addPropertyToTooltipIfAvailable(item, "Manufacturer");
        //this._addPropertyToTooltipIfAvailable(item, "SerialNumber");
        this._addPropertyToTooltipIfAvailable(item, "EnclosureFRU");
      }
    }

    overlayDivElement.innerHtml = "<span id='itemsNameOverlay'>"+item.getLabel() + "</span>";
  
  },
  
  _addOSProperty: function(item) {
    var frmProps = dojo.fromJson( itmeUtils.getProperty(item, "OTHERHWVIEWPROPS") );
    if ( this.isFSM(item) ) {// REDUNDANT FSM check, Do NOT show OS for FSM hardware.
      itmeUtils.CMTrace("OVERLAY", "node is FSM, NOT showing OS info for: " + item.getLabel());
      return;
    }
    if ( frmProps.OS_DATA ) {
      itmeUtils.CMTrace("OVERLAY", "node is NOT FSM, showing OS info if available for: " + item.getLabel());
      for ( var i = 0; i < frmProps.OS_DATA.length ; i++ ) {
        itmeUtils.CMTrace("OVERLAY", item.mapElement.id + " : Prop [ OS ["+i+"] ]"+frmProps.OS_DATA[i].OSLABEL+" / " + frmProps.OS_DATA[i].OSTYPE);
        if ( frmProps.OS_DATA[i].OSTYPE )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.OperatingSystem, frmProps.OS_DATA[i].OSTYPE);
        //item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.OSLABEL, frmProps.OS_DATA[i].OSLABEL+" / " + frmProps.OS_DATA[i].OSTYPE);
      }
    }
  },
  
  isFSM: function(item) {
    var prop = item.getProperty("MachineType");
    if ( !prop )
      return false;
    var machineType = prop.getDisplayValue();
    var hwInfo = new itme.ChassisMap.KnownHardwareInfo();
    itmeUtils.CMTrace("OVERLAY", "is FSM for : " + machineType);
    return hwInfo.isFSM(machineType);
  }
  
  /*_testIfStringNeedsEllipse: function(parentElement, propString) {
    itmeUtils.CMTrace("OVERLAY", "chkpt1");
    var parentBox = dojo.contentBox(parentElement);
    itmeUtils.CMTrace("OVERLAY", "chkpt2");
    var childNode = dojo.create("span", {innerHTML: "IP: " + propString});
    itmeUtils.CMTrace("OVERLAY", "chkpt3");
    var childBox = dojo.contentBox(childNode);
    itmeUtils.CMTrace("OVERLAY", "chkpt4");
    itmeUtils.CMTrace("OVERLAY", "parentW = " + parentBox.w + " , childW = "+ childBox.w);
    if ( childBox.w > parentBox.w )
      return true;
    return false;
  }*/
  
  // renderOverlay: function(){
  // Default preprocessing for rendering.  Calls into the REQUIRED OVERRIDE method _renderItem
  
  // unRenderOverlay: function()
  // Unrender the overlay.  Parent function should suffice for all cases
  
  // _unsetOverlayForItem: function(item)
  // Hide what was rendered by renderOverlay.  Override this if:
    // 1. you did not use "_"+_overlayId to append element divs  or...
    // 2. you need to unrender more than 1 created div
    // 3. you do not want to unrender "_"+_overlayId default that this base class does
  
  // itemInformationModified: function(item)
  // Unused
  
  // getId: function()
  // Return the overlay id.  This is set in the constructor and should not need overriding.

  // getTitle: function() {
  // Return the title for the overlay.  This is set in the constructor and should not need overriding.

  // getDescription: function() {
  // Return the description for the overlay.  This is set in the constructor and should not need overriding.
  
  // requiresProperties: function() {
  // Unused
  // Gets the required properties for this overlay.  This is set in the constructor and should not need overriding.
  
  // setItems: function(items) {
  // Sets the items for the overlay. Parent function should suffice unless special processing is needed 
  
});

}

if(!dojo._hasResource["itme.ChassisMap.LEDOverlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.LEDOverlay"] = true;
dojo.provide("itme.ChassisMap.LEDOverlay");




/*
 * Inherit from base overlay class itme.ChassisMapOverlay.
 * Declare the following variables in the constructor:
 *   _overlayId
 *   _description
 *   _title
 *   _requiredProperties (unimplemented)
 */
dojo.declare("itme.ChassisMap.LEDOverlay", [itme.ChassisMap.OverlayBase], {
  myIndex: 0,

  constructor: function(kwArgs){
    // parent constructor is always auto-called first
    this._overlayId = "led";
    this._description = "Highlight front panel LEDs";
    this._title = this.MRI.LEDTitle;
    this._iconId = "LEDFilterIcon";
    this._requiredProperties = ["LEDSERVICEINFO"];

    this._availableFilters = [];
    this._powerFilter = new itme.ChassisMap.OverlayFilterItem({}, "powerFilter", "LEDPowerOn", this.MRI.PowerLED, true, true );
    this._availableFilters[0] = this._powerFilter;
    this._errorFilter = new itme.ChassisMap.OverlayFilterItem({}, "errorFilter", "LEDErrorLogOn", "Event Log LED", true, true);
    this._availableFilters[1] = this._errorFilter;
    this._locationFilter = new itme.ChassisMap.OverlayFilterItem({}, "locationFilter", "LEDLocationOn", "Location LED", false, true);
    this._availableFilters[2] = this._locationFilter;
    this._faultFilter = new itme.ChassisMap.OverlayFilterItem({}, "faultFilter", "LEDFaultOn", "Fault LED", true, true);
    this._availableFilters[3] = this._faultFilter;
    // DHM: Other LED filter
    this._otherFilter = new itme.ChassisMap.OverlayFilterItem({}, "trayOpenFilter", "LEDTrayOpenOn", "Tray Open LED", true, true);
    this._availableFilters[3] = this._faultFilter;
    // **

    this._onlyOnFilter = new itme.ChassisMap.OverlayFilterItem({}, "onlyOnFilter", null, "Only Show ON", true, true);
    this._availableFilters[4] = this._onlyOnFilter;
  },

  _renderItem: function(item, buildTooltip){

    var overlayDivElement = dojo.byId(item.mapElement.id + "_led");

    // if we've already created the element before just show it, otherwise create it
    if ( overlayDivElement)  {
      overlayDivElement.style.display = "block";
      if ( dojo.byId(item.mapElement.id + "_led2") )
        dojo.byId(item.mapElement.id + "_led2").style.display = "block";
    }
    else {
      // create it
      overlayDivElement = itmeUtils.createElement(
            item.mapElement.id + "_led",
            item.mapElement,
            { styleClass: "LEDOverlayElement" } );

      this._addNameElement(item);
      //Set all LED's to unknown first
      var powerLedStatus = 3;
      var powerLedDescription = '';
      var eventLogLedStatus = 3;
      var eventLogLedDescription = '';
      var locationLedStatus = 3;
      var locationLedDescription = '';
      var faultLedStatus = 3;
      var faultLedDescription = '';
      var LEDStatus = dojo.fromJson(itmeUtils.getProperty(item, "LEDSERVICEINFO"));
      if ( LEDStatus && LEDStatus.LEDS ) {
        itmeUtils.CMTrace("OVERLAY", "LEDSERVICEINFO on item ("+item.mapElement.id+") = " + LEDStatus.LEDS);
        itmeUtils.CMTrace("OVERLAY", "LEDSERVICEINFO on item ("+item.mapElement.id+") = .length" + LEDStatus.LEDS.length);
        for (i = 0; i <  LEDStatus.LEDS.length; i++) {
          itmeUtils.CMTrace("OVERLAY", "LED: " + i);
          var currentLedDesc = this._getDisplayableLed(LEDStatus.LEDS[i].DESCRIPTION, item.getType());
          if(currentLedDesc == 'Power Led') {
              powerLedStatus = this._getLedStatus(LEDStatus.LEDS[i].STATE);
              itmeUtils.CMTrace("OVERLAY", "powerLedStatus: " + powerLedStatus);
          } else if(currentLedDesc == 'Event Log Led') {
              eventLogLedStatus = this._getLedStatus(LEDStatus.LEDS[i].STATE);
              itmeUtils.CMTrace("OVERLAY", "eventLogLedStatus: " + eventLogLedStatus);
          } else if(currentLedDesc == 'Location Led') {
              locationLedStatus = this._getLedStatus(LEDStatus.LEDS[i].STATE);
              itmeUtils.CMTrace("OVERLAY", "locationLedStatus: " + locationLedStatus);
          } else if(currentLedDesc == 'Fault Led') {
              faultLedStatus = this._getLedStatus(LEDStatus.LEDS[i].STATE);
              itmeUtils.CMTrace("OVERLAY", "faultLedStatus: " + faultLedStatus);
          }
        }
      }

      // set contents



      overlayDivElement.innerHTML = "";

      var iconSize = 22;
      if ( item.mapElement.constraint < 22 )
          iconSize = item.mapElement.constraint;

      if ( this._powerFilter.isEnabled() )
      { if ( powerLedStatus == 1 )
          overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDPowerOn", "", iconSize);
        else if ( !this._onlyOnFilter.isEnabled() )
          if(powerLedStatus == 0) {
              overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDPowerOff", "", iconSize);
          } else if ( eventLogLedStatus == 2 ) { //TODO: handle binking status, for now pretend it's same as ON
              overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDPowerOn", "", iconSize);
          } else { //TODO: handle unknown status

          }
      }
      if ( this._errorFilter.isEnabled() )
      { if ( eventLogLedStatus == 1 )
          overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDErrorLogOn", "", iconSize);
        else if ( !this._onlyOnFilter.isEnabled() )
            if(eventLogLedStatus == 0) {
              overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDErrorLogOff", "", iconSize);
            } else if ( eventLogLedStatus == 2 ) { //TODO: handle binking status, for now pretend it's same as ON
                overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDErrorLogOn", "", iconSize);
            } else { //TODO: handle unknown status

            }
      }
      if ( this._locationFilter.isEnabled() ) {
        if ( locationLedStatus == 1 || locationLedStatus == 2) { // Treat blink same as on
          overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDLocationOn", "", iconSize);
        } else if ( !this._onlyOnFilter.isEnabled() ) { // if we should show all
          if ( locationLedStatus == 0 ) {
            overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDLocationOff", "", iconSize);
          }
          else {
            // ignore unknown status
          }
        }
      }

      if ( this._faultFilter.isEnabled() )
      { if ( faultLedStatus == 1 )
          overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDFaultOn", "", iconSize);
        else if ( !this._onlyOnFilter.isEnabled() )
            if(faultLedStatus == 0) {
                 overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDFaultOff", "", iconSize);
      } else if ( eventLogLedStatus == 2 ) { //TODO: handle binking status, for now pretend it's same as ON
          overlayDivElement.innerHTML += this.IMAGES.buildIconImageTag("LEDFaultOn", "", iconSize);
      } else { //TODO: handle unknown status

      }
      }


      if ( buildTooltip )
      {
        if ( powerLedStatus == 1 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDPowerOn", "", 22) + " Power LED", "On" );
        else if ( powerLedStatus == 0 )
            item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDPowerOff", "", 22) + " Power LED", "Off" );
        else if ( powerLedStatus == 2 )
            item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDPowerOn", "", 22) + " Power LED", "Blinking" );

        if ( eventLogLedStatus == 1 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDErrorLogOn", "", 22) + " Event Log LED", "On" );
        else if ( eventLogLedStatus == 0 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDErrorLogOff", "", 22) + " Event Log LED", "Off" );
        else if ( powerLedStatus == 2 )
            item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDErrorLogOn", "", 22) + " Event Log LED", "Blinking" );

        if ( locationLedStatus == 1 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDLocationOn", "", 22) + " Location LED", "On" );
        else if ( locationLedStatus == 0 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDLocationOff", "", 22) + " Location LED", "Off" );
        else if ( locationLedStatus == 2 )
            item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDLocationOn", "", 22) + " Location LED", "Blinking" );

        if ( faultLedStatus == 1 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDFaultOn", "", 22) + " Fault LED", "On" );
        else if ( faultLedStatus == 0 )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDFaultOff", "", 22) + " Fault LED", "Off" );
        else if ( powerLedStatus == 2 )
            item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("LEDFaultOn", "", 22) + " Fault LED", "Blinking" );


      }


      //itmeUtils.CMTrace("OVERLAY", "Affecting CMTooltip : " + item.mapElement.CMTooltip);
      this.myIndex++;
    }
  },
  _getLedStatus: function(ledState) {
      if(ledState == 'Off') {
          return 0;
      } else if(ledState == 'On') {
          return 1;
      } else if(ledState == 'Blinking') {
          return 2;
      } else if(ledState == 'Unknown') {
          return 3;
      }
  },
  _getDisplayableLed: function(ledDescription, resourceType) {
      itmeUtils.CMTrace("OVERLAY", "description: " + ledDescription + ", type: " + resourceType);


      if (ledDescription == null || ledDescription == "") {
          return null;
      }
      if (resourceType === "Chassis" || resourceType === "SystemChassis") {
          return this._getDisplayableChassisLed(ledDescription);
      } else if (resourceType === "com.ibm.sysmgt.platform.PhysicalPlatformFactory") {
          return this._getDisplayableBladeLed(ledDescription);
      } else if (resourceType === "PowerSupply") {
          return this._getDisplayablePowerSupplyLed(ledDescription);
      } else if (resourceType === "Fan") {
          return this._getDisplayableFanLed(ledDescription);
      } else if (resourceType === "Iomodule" || resourceType === "Switch") {
          return this._getDisplayableIOModuleLed(ledDescription);
      }
      return null;
  },

  _getDisplayableChassisLed : function (ledDescription) {
      itmeUtils.CMTrace("OVERLAY", "chassis led description: " + ledDescription);
      //Accipiter chassis leds:
      if (ledDescription === "Cmm 1 Active" || ledDescription === "cmm 1 Active") {
          return "Power Led";
      } else if (ledDescription === "Chassis Information") {
          return "Event Log Led";
      } else if (ledDescription === "Chassis Location") {
          return "Location Led";
      } else if (ledDescription === "Chassis FAULT") {
          return "Fault Led";
      } //Legacy chassis leds:
      else if (ledDescription === "Information") {
          return "Event Log Led";
      } else if (ledDescription === "Location") {
          return "Location Led";
      } else if (ledDescription === "Fault") {
          return "Fault Led";
      }
      return null;
  },

  _getDisplayableBladeLed : function (ledDescription) {
      if (ledDescription === "Power") {
          return "Power Led";
      } else if (ledDescription === "Check Log") {
          return "Event Log Led";
      } else if (ledDescription === "Identification") {
          return "Location Led";
      } else if (ledDescription === "Fault") {
          return "Fault Led";
      }
      return null;
  },
  _getDisplayablePowerSupplyLed : function (ledDescription) {
      if (ledDescription.indexOf('FAULT') != -1) {
          return "Fault Led";
      }
      return null;
  },
  _getDisplayableFanLed : function (ledDescription) {
      if (ledDescription.indexOf('FAULT') != -1) {
          return "Fault Led";
      }
      return null;
  },
  _getDisplayableIOModuleLed : function (ledDescription) {
      if (ledDescription.indexOf('Enclosure Identify') != -1) {
          return "Location Led";
      } else if (ledDescription.indexOf('Power') != -1) {
          return "Power Led";
      } else if (ledDescription.indexOf('Fault') != -1) {
          return "Fault Led";
      }
      return null;
  },
  _addNameElement: function(item) {
    if ( item.getType() == "com.ibm.sysmgt.platform.PhysicalPlatformFactory") {
      var ledOverlayTextSpanElement = itmeUtils.createElement(
                  item.mapElement.id + "_led2",
                  item.mapElement,
                  { elementType: "span",
                    baseDimensionsOnNode: item.mapElement,
                    innerHTML: "<span class='secondaryElementName'>"+item.getLabel()+"</span>"});

      // verticalize text
      this._setDimensionsAndTextDirection(item, ledOverlayTextSpanElement);
    }
  },

  // unRenderOverlay: function()
  // Unrender the overlay.  Parent function should suffice for all cases

  _unsetOverlayForItem: function(item)
  // Hide what was rendered by renderOverlay.  Override this if:
    // 1. you did not use "_"+_overlayId to append element divs  or...
    // 2. you need to unrender more than 1 created div
    // 3. you do not want to unrender "_"+_overlayId default that this base class does
  {
    this.inherited(arguments);
    itmeUtils.CMTrace("OVERLAY", "_unsetOverlayForItem (LED): " + item.getLabel());
    var itemsLedDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "2");
    if ( itemsLedDivElement)
      itemsLedDivElement.style.display = "none";
    this.myIndex = 0;
  },

  _deleteOverlayForItem: function(item) {
    this.inherited(arguments);
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+"2");
    if ( itemsDivElement)
      itemsDivElement.parentNode.removeChild(itemsDivElement);
  }
  // itemInformationModified: function(item)
  // Unused

  // getId: function()
  // Return the overlay id.  This is set in the constructor and should not need overriding.

  // getTitle: function() {
  // Return the title for the overlay.  This is set in the constructor and should not need overriding.

  // getDescription: function() {
  // Return the description for the overlay.  This is set in the constructor and should not need overriding.

  // requiresProperties: function() {
  // Unused
  // Gets the required properties for this overlay.  This is set in the constructor and should not need overriding.

  // setItems: function(items) {
  // Sets the items for the overlay. Parent function should suffice unless special processing is needed

});

}

if(!dojo._hasResource["itme.ChassisMap.EnvironmentalsOverlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.EnvironmentalsOverlay"] = true;
dojo.provide("itme.ChassisMap.EnvironmentalsOverlay");




/*
 * Inherit from base overlay class itme.ChassisMap.OverlayBase
 * Declare the following variables in the constructor:
 *   _overlayId
 *   _description
 *   _title
 *   _requiredProperties (unimplemented)
 */
dojo.declare("itme.ChassisMap.EnvironmentalsOverlay", [itme.ChassisMap.OverlayBase], {
  myIndex: 0,
  _MAX_MIDPOINT: 40,
  _midPointTemp: 40,
  constructor: function(kwArgs){
    // parent constructor is always auto-called first
    this._overlayId = "heat";
    this._description = this.MRI.EnvButtonTooltip;
    this._title = this.MRI.ThermalTitle;
    this._requiredProperties = [ "AEM_DATA" ];
    this._iconId = "ThermalIcon_22";
    
    this._availableFilters = [];
    //this._coolingSubsys = new itme.ChassisMap.OverlayFilterItem({}, "coolingGauge", "ThermalIcon_22", this.MRI.SubsystemDetails, true, true );
    //this._availableFilters[0] = this._coolingSubsys;
 
    
    this._midPointTemp = 40;
  },
  
  _buildTooltip: true,
  
  /* Before we hit the main rendering path, we need to do some calculations based on the midpoint temperature for all blades + chassis
   * 
  */
  _preProcessing: function(items) {
    var min = 1000, max = -1000;
    for (var i = 0; i < items.length; i++) {
      if (this._items[i].mapElement) {
        var aemData = dojo.fromJson( itmeUtils.getProperty(this._items[i], "AEM_DATA") );
        if ( aemData.ENV_AMBIENTTEMP && (aemData.ENV_AMBIENTTEMP > 0) ) {
          itmeUtils.CMTrace("OVERLAY", "new temp: " + aemData.ENV_AMBIENTTEMP + " , curmin: " + min + " , curmax: " + max);
          if (aemData.ENV_AMBIENTTEMP > max)
            max = aemData.ENV_AMBIENTTEMP;
          if (aemData.ENV_AMBIENTTEMP < min)
            min = aemData.ENV_AMBIENTTEMP;
        }
      }
    }
    this._midPointTemp = (min + max) / 2;
    if ( this._midPointTemp > this._MAX_MIDPOINT )
      this._midPointTemp = this._MAX_MIDPOINT;
    itmeUtils.CMTrace("OVERLAY", "min: " + min + " , max: " + max + " , MIDPOINT: " + this._midPointTemp);
  },
  
  _renderItem: function(item, buildTooltip){
    var colorClass;
    itmeUtils.CMTrace("OVERLAY", "renderItem ("+item.getLabel()+")");

    this._buildTooltip = buildTooltip;
    
    var overlayDivElement = dojo.byId(item.mapElement.id + "_heat");

    // if we've already created the element before just show it, otherwise create it
    if ( overlayDivElement)  {
      itmeUtils.CMTrace("OVERLAY", "renderItem ("+item.getLabel()+") - re-Enabling overlay element");
      overlayDivElement.style.display = "block";
      dojo.byId(item.mapElement.id + "_heat2").style.display = "block";
      if ( item.getType() == "SystemChassis" ) {
         if ( dojo.byId(item.mapElement.id + "_heatBackExhaust")) 
            dojo.byId(item.mapElement.id + "_heatBackExhaust").style.display = "block";
      }
    } else { 
      itmeUtils.CMTrace("OVERLAY", "renderItem ("+item.getLabel()+") - building overlay element");
      
      // create it
      overlayDivElement = itmeUtils.createElement(
                  item.mapElement.id + "_heat",
                  item.mapElement,
                  { styleClass: "HEATOverlayElement",
                    baseDimensionsOnNode: item.mapElement} );  
        
      var DataArray = dojo.fromJson( itmeUtils.getProperty(item, "AEM_DATA") );

      // verticalize text      
      this._setDimensionsAndTextDirection(item, overlayDivElement);  

      
      overlayDivElement.innerHTML = "";
      // if No Data....
      colorClass = "elementName"; // default
      itmeUtils.CMTrace("OVERLAY", "testing no data");
      if ( this._isNoRenderableData(DataArray) ) {
        dojo.addClass(overlayDivElement, "NoData");
        colorClass = "elementName";
        overlayDivElement.innerHTML = this.MRI.NoData;
        if ( buildTooltip)
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, "Environmetals", this.MRI.NoData );
      }
      else {
        colorClass = this._mapAmbientTempToColorScale(DataArray);

        
        itmeUtils.CMTrace("OVERLAY", "adding props");
        if ( item.getType() == "SystemChassis" ) {
          this._addAmbientTemp(DataArray, item, overlayDivElement, true);
          this._addExhaustTempAndAirFlow(DataArray, item, overlayDivElement, true);
          this._addInputPower(DataArray, item, overlayDivElement, true);
          this._addOutputPower(DataArray, item, overlayDivElement, true);
        }
        else {
          this._addAmbientTemp(DataArray, item, overlayDivElement);
          //this._addExhaustTemp(DataArray, item, overlayDivElement, false);
          this._addInputPower(DataArray, item, overlayDivElement, false);
          this._addOutputPower(DataArray, item, overlayDivElement, false);
        }

        this._addPowerCap(DataArray, item, overlayDivElement);       

        // tooltip only property
        if ( DataArray.TIME_LAST_POLLED && buildTooltip)      
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.TimeLastPolled, DataArray.TIME_LAST_POLLED );

      }
      
            // add element name
      if ( item.getType() == "com.ibm.sysmgt.platform.PhysicalPlatformFactory" ||
           item.getType() == "SystemChassis" ) 
        itmeUtils.insertEllipsedTextSpanIntoElement(overlayDivElement, item.getLabel(), {styleClass: "secondaryElementName"}); // el, class, text

      // add yet another div for critical status, the css class should give it a red or yellow hue            
      itmeUtils.createElement(
                item.mapElement.id + "_heat2", 
                item.mapElement,
                { styleClass: "HEATOverlayElementShadow "+colorClass,
                  baseDimensionsOnNode: item.mapElement} ) ; 
          
      this.myIndex++;
    }
    
  },

  _isNoRenderableData: function(dataArray) {
    if ( !dataArray )
      return true;
    if ( ( !dataArray.ENV_AMBIENTTEMP || (dataArray.ENV_AMBIENTTEMP < 0) ) &&  
         ( !dataArray.ENV_EXHAUSTTEMP || (dataArray.ENV_EXHAUSTTEMP < 0) ) &&
         ( !dataArray.ENV_AIRFLOW || (dataArray.ENV_AIRFLOW < 0) ) &&
         ( !dataArray.PWR_ACAVGINPUT || (dataArray.PWR_ACAVGINPUT < 0) ) &&
         ( !dataArray.PWR_DCAVGINPUT || (dataArray.PWR_DCAVGINPUT < 0) ) &&
         ( !dataArray.PWR_DCAVGOUTPUT || (dataArray.PWR_DCAVGOUTPUT < 0) ) &&
         ( !dataArray.PWR_CAP || (dataArray.PWR_CAP < 0) ) )
         return true;
     return false;
  }, 
  
  _mapAmbientTempToColorScale: function(dataArray) {
  	var colorClass = "GrayGradient";

    itmeUtils.CMTrace("OVERLAY", "tempRange: " + dataArray.ENV_AMBTEMP_RANGE);
      switch (dataArray.ENV_AMBTEMP_RANGE) {
        case "10":  colorClass = "BlueGradient";    break;
        case "20":  colorClass = "GreenGradient";   break;
        case "30":  colorClass = "Yellow1Gradient"; break;
        case "40":  colorClass = "Orange1Gradient"; break;
        case "50":  colorClass = "RedGradient";     break;
      }
      return colorClass;
  },

  _addAmbientTemp: function(dataArray, item, divElement, showLabel) {
    var label = "";
    if ( dataArray.ENV_AMBIENTTEMP && (dataArray.ENV_AMBIENTTEMP > 0) ) {
      if ( showLabel )
        label = this.MRI.AmbientTemp;
      divElement.innerHTML = divElement.innerHTML + label + " " + Math.round(dataArray.ENV_AMBIENTTEMP) + " " + dataArray.TEMP_UNIT+"<br>";
      if ( this._buildTooltip )
      item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.AmbientTemp, dataArray.ENV_AMBIENTTEMP + " " + dataArray.TEMP_UNIT);
    }
  },

  _addExhaustTempAndAirFlow: function(dataArray, item, divElement, isChassis) {
    var overlayDivElementChassisBack, chassisBackFaceNode = dojo.byId("chassisback");
    if ( dataArray.ENV_EXHAUSTTEMP && (dataArray.ENV_EXHAUSTTEMP > 0) ) {
      if (this._buildTooltip)
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ExhaustTemp, dataArray.ENV_EXHAUSTTEMP + " " + dataArray.TEMP_UNIT);
      
      if ( ! isChassis ) 
        divElement.innerHTML = divElement.innerHTML + "" + Math.round(dataArray.ENV_EXHAUSTTEMP) + " " + dataArray.TEMP_UNIT+"<br>";
      else { // Chassis is a special case, we build a special element for the back chassis frame and add this text to it
        overlayDivElementChassisBack = dojo.byId(item.mapElement.id + "_heatBackExhaust");
        if (!overlayDivElementChassisBack)
	        overlayDivElementChassisBack = itmeUtils.createElement(
	                 item.mapElement.id + "_heatBackExhaust", 
	                 dojo.byId("chassisback"),
	                 { styleClass: "HEATOverlayElement",
	                   baseDimensionsOnNode:chassisBackFaceNode } );

        overlayDivElementChassisBack.innerHTML = overlayDivElementChassisBack.innerHTML + this.MRI.ExhaustTemp + " " + Math.floor(dataArray.ENV_EXHAUSTTEMP) + " " + dataArray.TEMP_UNIT + "<br>";
      }
    }
    
    if ( dataArray.ENV_AIRFLOW && (dataArray.ENV_AIRFLOW > 0) ) {
        if (this._buildTooltip)
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.Airflow, dataArray.ENV_AIRFLOW + " " + this.MRI.CFM);
        
        if ( ! isChassis ) 
          divElement.innerHTML = divElement.innerHTML + "" + Math.round(dataArray.ENV_AIRFLOW) + " " + this.MRI.CFM+"<br>";
        else { // Chassis is a special case, we build a special element for the back chassis frame and add this text to it
            overlayDivElementChassisBack = dojo.byId(item.mapElement.id + "_heatBackExhaust");
            if (!overlayDivElementChassisBack)
              overlayDivElementChassisBack = itmeUtils.createElement(
                         item.mapElement.id + "_heatBackExhaust", 
                         dojo.byId("chassisback"),
                         { styleClass: "HEATOverlayElement",
                           baseDimensionsOnNode:chassisBackFaceNode } );

          overlayDivElementChassisBack.innerHTML = divElement.innerHTML + "" + Math.round(dataArray.ENV_AIRFLOW) + " " + this.MRI.CFM+"<br>";
        }
      }
    
  },

  _addInputPower: function(dataArray, item, divElement, isChassis) {
    
    if ( dataArray.PWR_DCAVGINPUT && (dataArray.PWR_DCAVGINPUT > 0) ) {
      if (this._buildTooltip)
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.PowerIn, dataArray.PWR_DCAVGINPUT + " " + dataArray.POWER_UNIT );
      divElement.innerHTML = divElement.innerHTML + "<span id='"+item.mapElement.id+"_inputPower'>" + "" + Math.round(dataArray.PWR_DCAVGINPUT) + " " + dataArray.POWER_UNIT+"</span>";
    }
    else if ( dataArray.PWR_ACAVGINPUT && (dataArray.PWR_ACAVGINPUT > 0) ) {
      if (this._buildTooltip)
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.PowerIn, dataArray.PWR_ACAVGINPUT + " " + dataArray.POWER_UNIT );
      if ( !isChassis )
        divElement.innerHTML = divElement.innerHTML + "" + Math.round(dataArray.PWR_ACAVGINPUT) + " " + dataArray.POWER_UNIT;
      else 
        divElement.innerHTML = divElement.innerHTML + "<span id='"+item.mapElement.id+"_inputPower' class='chassisBottomText'>"+ "" + Math.round(dataArray.PWR_ACAVGINPUT) + " " + dataArray.POWER_UNIT +"</span>";        
      
    }
    // TODO: Handle Chassis Specific rendering location.
    
    // LEFT OFF HERE
    //  - AC goes into the chassis, DC out.  All other components in a chassis is DC in
    //  - need to clean up remaining properties
    //  - merge RTC code to get Alex's fix.
    //  - Work on power capping, slot thermal readings... ... see AEMOverlayNotes.txt
    
        
        
  },

  _addOutputPower: function(dataArray, item, divElement, isChassis) {
    if ( (dataArray.PWR_DCAVGOUTPUT) && (dataArray.PWR_DCAVGOUTPUT != -1) ) {
      if (this._buildTooltip)
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.PowerOut, dataArray.PWR_DCAVGOUTPUT + " " + dataArray.POWER_UNIT );
      if ( !isChassis )
        divElement.innerHTML = divElement.innerHTML + "" + Math.round(dataArray.PWR_DCAVGOUTPUT) + " " + dataArray.POWER_UNIT + "<br>";
      //else {
        // divElement.innerHTML = divElement.innerHTML + "<span class='chassisBottomText'>"+ "" + Math.round(dataArray.PWR_DCAVGOUTPUT) + " " + dataArray.POWER_UNIT +"</span>";
      //}
    }
    
  },

  _addPowerCap: function(dataArray, item, divElement) {
    var elly, inputPower;
    if ( (dataArray.PWR_CAP) && (dataArray.PWR_CAP != -1) ) {
      
      elly = dojo.byId(item.mapElement.id+"_inputPower");
      if ( !elly )
        elly = divElement;
      elly.innerHTML = elly.innerHTML + " / <span class='powerCapText'>" + dataArray.PWR_CAP + " " + dataArray.POWER_UNIT + "</span>";
      if (this._buildTooltip)
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.PowerCap, dataArray.PWR_CAP + " " + dataArray.POWER_UNIT );
      
      inputPower = 0;
      // get input power - compare to cap - if within 10%, colorize text
      if ( dataArray.PWR_DCAVGINPUT && (dataArray.PWR_DCAVGINPUT > 0) ) 
        inputPower = Math.round(dataArray.PWR_DCAVGINPUT); 
      else if ( dataArray.PWR_ACAVGINPUT && (dataArray.PWR_ACAVGINPUT > 0) ) 
        inputPower = Math.round(dataArray.PWR_ACAVGINPUT);
      // TEST:
      //inputPower = 700;
      itmeUtils.CMTrace("OVERLAY", "inputPower / Cap -   " + inputPower + " / " + dataArray.PWR_CAP + " = "+ ( inputPower / dataArray.PWR_CAP));
      if ( (inputPower / dataArray.PWR_CAP) > 0.9) 
        dojo.addClass(elly, "powerCapWarning");
      //else if ((inputPower / dataArray.PWR_CAP) > 0.99) 
      //  dojo.addClass(elly, "powerCapReached");
    }
  },

  // Unrender the overlay.  Parent function should suffice unless there is general, non-item specific things to deal with
  unRenderOverlay: function() {
	  this.inherited(arguments);
	  itmeUtils.CMTrace("OVERLAY", "unRenderOverlay (HEAT) ");
	  
	  var pieChartDivElement = dojo.byId("subsysPieChart");
	  if ( pieChartDivElement) 
		  pieChartDivElement.style.display = "none";

  },
  
  _unsetOverlayForItem: function(item)
  // Hide what was rendered by renderOverlay.  Override this if:
    // 1. you did not use "_"+_overlayId to append element divs  or...
    // 2. you need to unrender more than 1 created div
    // 3. you do not want to unrender "_"+_overlayId default that this base class does
  {
    this.inherited(arguments);
    itmeUtils.CMTrace("OVERLAY", "_unsetOverlayForItem (HEAT): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "2");
    if ( itemsDivElement) 
        itemsDivElement.style.display = "none";
    if ( item.getType() == "SystemChassis" ) {
        itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "BackExhaust");
        if ( itemsDivElement) 
            itemsDivElement.style.display = "none";
    }
  },
  
  // Delete the overlay.  Parent function should suffice unless there is general, non-item specific things to deal with
  deleteOverlay: function() {
    this.inherited(arguments);
    itmeUtils.CMTrace("OVERLAY", "deleting overlay (HEAT) ");

    if ( dijit.byId('subsysPieChart') )
      dijit.byId('subsysPieChart').destroyRecursive();
  },
  
  _deleteOverlayForItem: function(item) {
    this.inherited(arguments);
    itmeUtils.CMTrace("OVERLAY", "_deleteOverlayForItem (HEAT): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "2");
    if ( itemsDivElement)
      itemsDivElement.parentNode.removeChild(itemsDivElement);
    if ( item.getType() == "SystemChassis" ) {
      itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "BackExhaust");
      if ( itemsDivElement)
        itemsDivElement.parentNode.removeChild(itemsDivElement);
    }
  },
  
  _convertFahrenheitToCelsius: function(f) {
    // F = (9/5 * C) + 32
    // C = F - 32 * 5/9;
    return f - 32 * 5 / 9;
  }

  
  // itemInformationModified: function(item)
  // Unused
  
  // getId: function()
  // Return the overlay id.  This is set in the constructor and should not need overriding.

  // getTitle: function() {
  // Return the title for the overlay.  This is set in the constructor and should not need overriding.

  // getDescription: function() {
  // Return the description for the overlay.  This is set in the constructor and should not need overriding.
  
  // requiresProperties: function() {
  // Unused
  // Gets the required properties for this overlay.  This is set in the constructor and should not need overriding.
  
  // setItems: function(items) {
  // Sets the items for the overlay. Parent function should suffice unless special processing is needed 
  
});

}

if(!dojo._hasResource["itme.ChassisMap.AccessOverlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.AccessOverlay"] = true;
dojo.provide("itme.ChassisMap.AccessOverlay");




dojo.declare("itme.ChassisMap.AccessOverlay", [itme.ChassisMap.OverlayBase], {

  _diffForBorder: null,

  constructor: function(kwArgs){
    // parent constructor is always auto-called first
    this._overlayId = "access";
    this._description = this.MRI.AccessOverlayDetail;
    this._title = this.MRI.AccessTitle;
    this._requiredProperties = [ "AccessState" ];
    this._iconId = "AccessOnlineNone";
    
    this._availableFilters = [];
    this._okAccessOnlineFilter = new itme.ChassisMap.OverlayFilterItem({}, "okAccessOnline", "AccessOnlineOK", this.MRI.OKAccessOnline, true, true); 
    this._availableFilters[0] = this._okAccessOnlineFilter; 
    this._okAccessOfflinefilter = new itme.ChassisMap.OverlayFilterItem({}, "okAccessOffline", "AccessOfflineOK", this.MRI.OKAccessOffline, true, true); 
    this._availableFilters[1] = this._okAccessOfflinefilter; 
    this._partialAccessOnlineFilter = new itme.ChassisMap.OverlayFilterItem({}, "partialAccessOnline", "AccessOnlinePartial", this.MRI.PartialAccessOnline, true, true); 
    this._availableFilters[2] = this._partialAccessOnlineFilter; 
    this._partialAccessOfflinefilter = new itme.ChassisMap.OverlayFilterItem({}, "partialAccessOffline", "AccessOfflinePartial", this.MRI.PartialAccessOffline, true, true); 
    this._availableFilters[3] = this._partialAccessOfflinefilter; 
    this._noAccessOnlineFilter = new itme.ChassisMap.OverlayFilterItem({}, "noAccessOnline", "AccessOnlineNone", this.MRI.NoAccessOnline, true, true);
    this._availableFilters[4] = this._noAccessOnlineFilter;
    this._noAccessOfflineFilter = new itme.ChassisMap.OverlayFilterItem({}, "noAccessOffline", "AccessOfflineNone", this.MRI.NoAccessOffline, true, true );
    this._availableFilters[5] = this._noAccessOfflineFilter;
    this._unknownAccessFilter = new itme.ChassisMap.OverlayFilterItem({}, "unknownAccess", "AccessUnknown", this.MRI.UnknownAccess, true, true); 
    this._availableFilters[6] = this._unknownAccessFilter; 
    this._NAAccessFilter = new itme.ChassisMap.OverlayFilterItem({}, "NAAccess", "AccessNotApplicable", this.MRI.NAAccess, false, true); 
    this._availableFilters[7] = this._NAAccessFilter; 
    this._NotTrustedFilter = new itme.ChassisMap.OverlayFilterItem({}, "notTrusted", "AccessNotTrusted", this.MRI.NotTrustedAccess, true, true); 
    this._availableFilters[8] = this._NotTrustedFilter; 

  },
  
  _test: 100,
  _renderItem: function(item, buildTooltip){
    
    var itemsAccessDivElement = dojo.byId(item.mapElement.id + "_access");
    
    // if we've already created the element before just show it, otherwise create it
    if (itemsAccessDivElement) {
      itemsAccessDivElement.style.display = "block";
    }
    else {
      // create element
      itemsAccessDivElement = itmeUtils.createElement(item.mapElement.id + "_access",
                                                      item.mapElement,
                                                      { styleClass: "defaultStatus",
                                                        baseDimensionsOnNode: item.mapElement} ); 
      
      // add element name
      if ( item.getType() == "com.ibm.sysmgt.platform.PhysicalPlatformFactory" ||
           item.getType() == "SystemChassis" ) { 
        itmeUtils.insertEllipsedTextSpanIntoElement(itemsAccessDivElement, item.getLabel(), {styleClass: "secondaryElementName"}); // el, class, text
      }

      
      var accessState = itmeUtils.getProperty(item, "AccessState");
      if ( !accessState )
        itmeUtils.CMTrace("OVERLAY", "No Access property on item.  Nothing to render");
      else {
        
        itmeUtils.CMTrace("OVERLAY", "ITEM Access State ID: " + accessState.getId() );
        itmeUtils.CMTrace("OVERLAY", "ITEM Access State Value: " + accessState.getValue() );
        itmeUtils.CMTrace("OVERLAY", "ITEM Access State Img: " + accessState.getImage() );
        itmeUtils.CMTrace("OVERLAY", "ITEM Access State Scalar Img: " + accessState.getScalarImage() );
      
        //itmeUtils.CMTrace("OVERLAY", "Affecting CMTooltip : " + item.mapElement.CMTooltip);
  
        var imgIdString = "";
        var showMe = false;
        var switchValue = accessState.getValue();
        if ( this._demoMode )
          switchValue = this._test++;
        switch (switchValue) {//(accessState.getValue()) {
          case 100: 
            imgIdString = "AccessOnlineNone";
            showMe = this._noAccessOnlineFilter.isEnabled(); 
            break;
          case 101: 
            imgIdString = "AccessOfflineNone"; 
            showMe = this._noAccessOfflineFilter.isEnabled();
            break;
          case 102: 
            imgIdString = "AccessOnlinePartial"; 
            showMe = this._partialAccessOnlineFilter.isEnabled();
            break;
          case 103: 
            imgIdString = "AccessOfflinePartial"; 
            showMe = this._partialAccessOfflinefilter.isEnabled();
            break;
          case 104: 
            imgIdString = "AccessOnlineOK"; 
            showMe = this._okAccessOnlineFilter.isEnabled();
            break;
          case 105: 
            imgIdString = "AccessOfflineOK"; 
            showMe = this._okAccessOfflinefilter.isEnabled();
            break;
          case 106: 
            imgIdString = "AccessUnknown"; 
            showMe = this._unknownAccessFilter.isEnabled();
            break;
          case 107: 
            imgIdString = "AccessNotApplicable"; 
            showMe = this._NAAccessFilter.isEnabled();
            break;
          case 108: 
            //imgIdString = "AccessNotApplicable"; 
            //showMe = this._NAAccessFilter.isEnabled();
            break;
          case 109: 
            imgIdString = "AccessNotTrusted"; 
            showMe = this._NotTrustedFilter.isEnabled();
            break;
          default: break;
        }

        if ( showMe ) {  
          var iconSize = 26;
          if ( item.mapElement.constraint < 26 )
            iconSize = item.mapElement.constraint;
          itemsAccessDivElement.innerHTML = this.IMAGES.buildIconImageTag(imgIdString, 'accessImage', iconSize) + itemsAccessDivElement.innerHTML;
        }
        
        if ( buildTooltip )
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.AccessState, this.IMAGES.buildIconImageTag(imgIdString, 'accessImage', "16") + " " + accessState.getLabel() );
      }
    }

  },

  _calculateBorderDiff: function(cBox, element) {
    this._diffForBorder = 4; // HACK:  IE failed to calculate border correctly.  CSS Margin = 2.  2x2 = 4.
    if (this._diffForBorder) {
      itmeUtils.CMTrace("OVERLAY", "calc border diff returning : " + this._diffForBorder);
      return this._diffForBorder;
    }
      
    var mBox = dojo.marginBox(element);
    this._diffForBorder = mBox.w - cBox.w;
    return this._diffForBorder;
  },
  
  // renderOverlay: function(){
  // Default preprocessing for rendering.  Calls into the REQUIRED OVERRIDE method _renderItem
  
  // unRenderOverlay: function()
  // Unrender the overlay.  Parent function should suffice for all cases
  
  _unsetOverlayForItem: function(item)
  // Hide what was rendered by renderOverlay.  Override this if:
    // 1. you did not use "_"+_overlayId to append element divs  or...
    // 2. you need to unrender more than 1 created div
    // 3. you do not want to unrender "_"+_overlayId default that this base class does
  {
    this.inherited(arguments);
    //itmeUtils.CMTrace("OVERLAY", "_unsetOverlayForItem (STATUS): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+ "2");
    if ( itemsDivElement) 
      itemsDivElement.style.display = "none";
    this._test = 100;
  },
  
  _deleteOverlayForItem: function(item) {
    this.inherited(arguments);
    //itmeUtils.CMTrace("OVERLAY", "_deleteOverlayForItem (BASE): " + item.getLabel());
    var itemsDivElement = dojo.byId(item.mapElement.id + "_" + this._overlayId+"2");
    if ( itemsDivElement) 
      itemsDivElement.parentNode.removeChild(itemsDivElement);
  }
  
  // itemInformationModified: function(item)
  // Unused
  
  // getId: function()
  // Return the overlay id.  This is set in the constructor and should not need overriding.

  // getTitle: function() {
  // Return the title for the overlay.  This is set in the constructor and should not need overriding.

  // getDescription: function() {
  // Return the description for the overlay.  This is set in the constructor and should not need overriding.
  
  // requiresProperties: function() {
  // Unused
  // Gets the required properties for this overlay.  This is set in the constructor and should not need overriding.
  
  // setItems: function(items) {
  // Sets the items for the overlay. Parent function should suffice unless special processing is needed 
  
  
});

}

if(!dojo._hasResource["itme.ChassisMap.ComplianceOverlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.ComplianceOverlay"] = true;
dojo.provide("itme.ChassisMap.ComplianceOverlay");




dojo.declare("itme.ChassisMap.ComplianceOverlay", [itme.ChassisMap.OverlayBase], {
   
  constructor: function(kwArgs){
    // parent constructor is always auto-called first
    this._overlayId = "updates";
    this._description = this.MRI.ComplianceOverlayTitle;
    this._title = this.MRI.ComplianceOverlayTitle;
    this._iconId = "ComplianceIcon_22";
    //this._requiredProperties = [ "COMPLIANCEINFO" ];
    
    this._availableFilters = [];
    
    this._demoTestCounter = 0;
  },
  
  _renderItem: function(item){

    var frmProps = dojo.fromJson( itmeUtils.getProperty(item, "OTHERHWVIEWPROPS") );
    
    if ( item.getType() == "SystemChassis")
      return; // no-op on System Chassis node
      
    var overlayDivElement = dojo.byId(item.mapElement.id + "_updates");
    
    // if we've already created the element before just show it, otherwise create it
    if ( overlayDivElement)  
      overlayDivElement.style.display = "block";
    else { 
      // create it
      overlayDivElement = itmeUtils.createElement(
            item.mapElement.id + "_updates", 
            item.mapElement,
            { styleClass: "UpdatesOverlayElement" } );

      iconSize = 22;
      if ( item.mapElement.constraint < 22 )
        iconSize = item.mapElement.constraint;

      if ( frmProps.FIRMWARE ) {
        for ( var i = 0; i < frmProps.FIRMWARE.length; i++) {
          //item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.FirmwareLevel,  frmProps.FWLEVEL);
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, "Firmware: " + frmProps.FIRMWARE[i].FWTYPE, frmProps.FIRMWARE[i].FWLEVEL + " &nbsp&nbsp&nbsp&nbsp " + frmProps.FIRMWARE[i].FWDATE);
        }
      }
      
      var complianceData = dojo.fromJson( itmeUtils.getProperty(item, "COMPLIANCEINFO") ); 
      var frmProps = dojo.fromJson( itmeUtils.getProperty(item, "OTHERHWVIEWPROPS") );
      
      if ( complianceData.COMPLIANCELEVEL == "HEALTHY" )
      {
        overlayDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusNormal", "", iconSize) + "<span class='textBackgroundShadow'> " + this.MRI.Healthy + "</span>";
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ComplianceStatus, this.IMAGES.buildIconImageTag("StatusNormal", "", 32) + " " + this.MRI.Healthy );        
      }  
      else if ( complianceData.COMPLIANCELEVEL == "INFO" )
      {
        overlayDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusInformational", "", iconSize) + "<span class='textBackgroundShadow'> " + this.MRI.Informational + "</span>";
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ComplianceStatus, this.IMAGES.buildIconImageTag("StatusInformational", "", 32) + " " + this.MRI.Informational );        
      }
      else if ( complianceData.COMPLIANCELEVEL == "WARNING" )
      {
        overlayDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusWarningMinor", "", iconSize) + "<span class='textBackgroundShadow'> " + this.MRI.Warnings + "</span>";
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ComplianceStatus, this.IMAGES.buildIconImageTag("StatusWarningMinor", "", 32) + " " + this.MRI.Warnings );        
      }
      else if ( complianceData.COMPLIANCELEVEL == "ERROR" )
      {
        overlayDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusCritical", "", iconSize) + "<span class='textBackgroundShadow'> " + this.MRI.Errors + "</span>";
        item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ComplianceStatus, this.IMAGES.buildIconImageTag("StatusCritical", "", 32) + " " + this.MRI.Errors );        
      }
      else if ( complianceData.COMPLIANCELEVEL == "NOPOLICY" )
      {      
        // use the nodata style for NO POLICY      
        dojo.addClass(overlayDivElement, "NoData");
        dojo.addClass(overlayDivElement, "elementName");
        if ( item.getType() != "Fan" && 
             item.getType() != "PowerSupply" &&
             item.getType() != "PhysicalPackage" ) {
          overlayDivElement.innerHTML = this.MRI.NoPolicy;
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ComplianceStatus, this.MRI.NoCompliancePolicy );
        } else
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.MRI.ComplianceStatus, this.MRI.NotApplicable );
      }
      
      if ( item.getType() == "com.ibm.sysmgt.platform.PhysicalPlatformFactory") {
        if ( complianceData.BLADEWITHOS == false ) {
          overlayDivElement.innerHTML = overlayDivElement.innerHTML + "<br>" + this.IMAGES.buildIconImageTag("StatusInformational", "", iconSize) + "<span class='textBackgroundShadow'> " + this.MRI.NoOSDiscovered + "</span>";
          item.mapElement.CMTooltip.addFocusProperty(this._overlayId, this.IMAGES.buildIconImageTag("StatusInformational", "", 22),  this.MRI.NoOSDiscoveredUpdates);
        }
      }

      // verticalize text      
      this._setDimensionsAndTextDirection(item, overlayDivElement);  
    }

  }
  
  // renderOverlay: function(){
  // Default preprocessing for rendering.  Calls into the REQUIRED OVERRIDE method _renderItem
  
  // unRenderOverlay: function()
  // Unrender the overlay.  Parent function should suffice for all cases
  
  // _unsetOverlayForItem: function(item)
  // Hide what was rendered by renderOverlay.  Override this if:
    // 1. you did not use "_"+_overlayId to append element divs  or...
    // 2. you need to unrender more than 1 created div
    // 3. you do not want to unrender "_"+_overlayId default that this base class does
  
  // itemInformationModified: function(item)
  // Unused
  
  // getId: function()
  // Return the overlay id.  This is set in the constructor and should not need overriding.

  // getTitle: function() {
  // Return the title for the overlay.  This is set in the constructor and should not need overriding.

  // getDescription: function() {
  // Return the description for the overlay.  This is set in the constructor and should not need overriding.
  
  // requiresProperties: function() {
  // Unused
  // Gets the required properties for this overlay.  This is set in the constructor and should not need overriding.
  
  // setItems: function(items) {
  // Sets the items for the overlay. Parent function should suffice unless special processing is needed 
  
});

}

if(!dojo._hasResource["dojo.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.window"] = true;
dojo.provide("dojo.window");

dojo.window.getBox = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	var scrollRoot = (dojo.doc.compatMode == 'BackCompat') ? dojo.body() : dojo.doc.documentElement;

	// get scroll position
	var scroll = dojo._docScroll(); // scrollRoot.scrollTop/Left should work
	return { w: scrollRoot.clientWidth, h: scrollRoot.clientHeight, l: scroll.x, t: scroll.y };
};

dojo.window.get = function(doc){
	// summary:
	// 		Get window object associated with document doc

	// In some IE versions (at least 6.0), document.parentWindow does not return a
	// reference to the real window object (maybe a copy), so we must fix it as well
	// We use IE specific execScript to attach the real window reference to
	// document._parentWindow for later use
	if(dojo.isIE && window !== document.parentWindow){
		/*
		In IE 6, only the variable "window" can be used to connect events (others
		may be only copies).
		*/
		doc.parentWindow.execScript("document._parentWindow = window;", "Javascript");
		//to prevent memory leak, unset it after use
		//another possibility is to add an onUnload handler which seems overkill to me (liucougar)
		var win = doc._parentWindow;
		doc._parentWindow = null;
		return win;	//	Window
	}

	return doc.parentWindow || doc.defaultView;	//	Window
};

dojo.window.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view, if it is not already.
	
	// don't rely on node.scrollIntoView working just because the function is there

	try{ // catch unexpected/unrecreatable errors (#7808) since we can recover using a semi-acceptable native method
		node = dojo.byId(node);
		var doc = node.ownerDocument || dojo.doc,
			body = doc.body || dojo.body(),
			html = doc.documentElement || body.parentNode,
			isIE = dojo.isIE, isWK = dojo.isWebKit;
		// if an untested browser, then use the native method
		if((!(dojo.isMoz || isIE || isWK || dojo.isOpera) || node == body || node == html) && (typeof node.scrollIntoView != "undefined")){
			node.scrollIntoView(false); // short-circuit to native if possible
			return;
		}
		var backCompat = doc.compatMode == 'BackCompat',
			clientAreaRoot = backCompat? body : html,
			scrollRoot = isWK ? body : clientAreaRoot,
			rootWidth = clientAreaRoot.clientWidth,
			rootHeight = clientAreaRoot.clientHeight,
			rtl = !dojo._isBodyLtr(),
			nodePos = pos || dojo.position(node),
			el = node.parentNode,
			isFixed = function(el){
				return ((isIE <= 6 || (isIE && backCompat))? false : (dojo.style(el, 'position').toLowerCase() == "fixed"));
			};
		if(isFixed(node)){ return; } // nothing to do

		while(el){
			if(el == body){ el = scrollRoot; }
			var elPos = dojo.position(el),
				fixedPos = isFixed(el);
	
			if(el == scrollRoot){
				elPos.w = rootWidth; elPos.h = rootHeight;
				if(scrollRoot == html && isIE && rtl){ elPos.x += scrollRoot.offsetWidth-elPos.w; } // IE workaround where scrollbar causes negative x
				if(elPos.x < 0 || !isIE){ elPos.x = 0; } // IE can have values > 0
				if(elPos.y < 0 || !isIE){ elPos.y = 0; }
			}else{
				var pb = dojo._getPadBorderExtents(el);
				elPos.w -= pb.w; elPos.h -= pb.h; elPos.x += pb.l; elPos.y += pb.t;
			}
	
			if(el != scrollRoot){ // body, html sizes already have the scrollbar removed
				var clientSize = el.clientWidth,
					scrollBarSize = elPos.w - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					elPos.w = clientSize;
					if(isIE && rtl){ elPos.x += scrollBarSize; }
				}
				clientSize = el.clientHeight;
				scrollBarSize = elPos.h - clientSize;
				if(clientSize > 0 && scrollBarSize > 0){
					elPos.h = clientSize;
				}
			}
			if(fixedPos){ // bounded by viewport, not parents
				if(elPos.y < 0){
					elPos.h += elPos.y; elPos.y = 0;
				}
				if(elPos.x < 0){
					elPos.w += elPos.x; elPos.x = 0;
				}
				if(elPos.y + elPos.h > rootHeight){
					elPos.h = rootHeight - elPos.y;
				}
				if(elPos.x + elPos.w > rootWidth){
					elPos.w = rootWidth - elPos.x;
				}
			}
			// calculate overflow in all 4 directions
			var l = nodePos.x - elPos.x, // beyond left: < 0
				t = nodePos.y - Math.max(elPos.y, 0), // beyond top: < 0
				r = l + nodePos.w - elPos.w, // beyond right: > 0
				bot = t + nodePos.h - elPos.h; // beyond bottom: > 0
			if(r * l > 0){
				var s = Math[l < 0? "max" : "min"](l, r);
				nodePos.x += el.scrollLeft;
				el.scrollLeft += (isIE >= 8 && !backCompat && rtl)? -s : s;
				nodePos.x -= el.scrollLeft;
			}
			if(bot * t > 0){
				nodePos.y += el.scrollTop;
				el.scrollTop += Math[t < 0? "max" : "min"](t, bot);
				nodePos.y -= el.scrollTop;
			}
			el = (el != scrollRoot) && !fixedPos && el.parentNode;
		}	
	}catch(error){
		console.error('scrollIntoView: ' + error);
		node.scrollIntoView(false);
	}
};

}

if(!dojo._hasResource["dijit._base.manager"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.manager"] = true;
dojo.provide("dijit._base.manager");

dojo.declare("dijit.WidgetSet", null, {
	// summary:
	//		A set of widgets indexed by id. A default instance of this class is
	//		available as `dijit.registry`
	//
	// example:
	//		Create a small list of widgets:
	//		|	var ws = new dijit.WidgetSet();
	//		|	ws.add(dijit.byId("one"));
	//		| 	ws.add(dijit.byId("two"));
	//		|	// destroy both:
	//		|	ws.forEach(function(w){ w.destroy(); });
	//
	// example:
	//		Using dijit.registry:
	//		|	dijit.registry.forEach(function(w){ /* do something */ });

	constructor: function(){
		this._hash = {};
		this.length = 0;
	},

	add: function(/*dijit._Widget*/ widget){
		// summary:
		//		Add a widget to this list. If a duplicate ID is detected, a error is thrown.
		//
		// widget: dijit._Widget
		//		Any dijit._Widget subclass.
		if(this._hash[widget.id]){
			throw new Error("Tried to register widget with id==" + widget.id + " but that id is already registered");
		}
		this._hash[widget.id] = widget;
		this.length++;
	},

	remove: function(/*String*/ id){
		// summary:
		//		Remove a widget from this WidgetSet. Does not destroy the widget; simply
		//		removes the reference.
		if(this._hash[id]){
			delete this._hash[id];
			this.length--;
		}
	},

	forEach: function(/*Function*/ func, /* Object? */thisObj){
		// summary:
		//		Call specified function for each widget in this set.
		//
		// func:
		//		A callback function to run for each item. Is passed the widget, the index
		//		in the iteration, and the full hash, similar to `dojo.forEach`.
		//
		// thisObj:
		//		An optional scope parameter
		//
		// example:
		//		Using the default `dijit.registry` instance:
		//		|	dijit.registry.forEach(function(widget){
		//		|		console.log(widget.declaredClass);
		//		|	});
		//
		// returns:
		//		Returns self, in order to allow for further chaining.

		thisObj = thisObj || dojo.global;
		var i = 0, id;
		for(id in this._hash){
			func.call(thisObj, this._hash[id], i++, this._hash);
		}
		return this;	// dijit.WidgetSet
	},

	filter: function(/*Function*/ filter, /* Object? */thisObj){
		// summary:
		//		Filter down this WidgetSet to a smaller new WidgetSet
		//		Works the same as `dojo.filter` and `dojo.NodeList.filter`
		//
		// filter:
		//		Callback function to test truthiness. Is passed the widget
		//		reference and the pseudo-index in the object.
		//
		// thisObj: Object?
		//		Option scope to use for the filter function.
		//
		// example:
		//		Arbitrary: select the odd widgets in this list
		//		|	dijit.registry.filter(function(w, i){
		//		|		return i % 2 == 0;
		//		|	}).forEach(function(w){ /* odd ones */ });

		thisObj = thisObj || dojo.global;
		var res = new dijit.WidgetSet(), i = 0, id;
		for(id in this._hash){
			var w = this._hash[id];
			if(filter.call(thisObj, w, i++, this._hash)){
				res.add(w);
			}
		}
		return res; // dijit.WidgetSet
	},

	byId: function(/*String*/ id){
		// summary:
		//		Find a widget in this list by it's id.
		// example:
		//		Test if an id is in a particular WidgetSet
		//		| var ws = new dijit.WidgetSet();
		//		| ws.add(dijit.byId("bar"));
		//		| var t = ws.byId("bar") // returns a widget
		//		| var x = ws.byId("foo"); // returns undefined

		return this._hash[id];	// dijit._Widget
	},

	byClass: function(/*String*/ cls){
		// summary:
		//		Reduce this widgetset to a new WidgetSet of a particular `declaredClass`
		//
		// cls: String
		//		The Class to scan for. Full dot-notated string.
		//
		// example:
		//		Find all `dijit.TitlePane`s in a page:
		//		|	dijit.registry.byClass("dijit.TitlePane").forEach(function(tp){ tp.close(); });

		var res = new dijit.WidgetSet(), id, widget;
		for(id in this._hash){
			widget = this._hash[id];
			if(widget.declaredClass == cls){
				res.add(widget);
			}
		 }
		 return res; // dijit.WidgetSet
},

	toArray: function(){
		// summary:
		//		Convert this WidgetSet into a true Array
		//
		// example:
		//		Work with the widget .domNodes in a real Array
		//		|	dojo.map(dijit.registry.toArray(), function(w){ return w.domNode; });

		var ar = [];
		for(var id in this._hash){
			ar.push(this._hash[id]);
		}
		return ar;	// dijit._Widget[]
},

	map: function(/* Function */func, /* Object? */thisObj){
		// summary:
		//		Create a new Array from this WidgetSet, following the same rules as `dojo.map`
		// example:
		//		|	var nodes = dijit.registry.map(function(w){ return w.domNode; });
		//
		// returns:
		//		A new array of the returned values.
		return dojo.map(this.toArray(), func, thisObj); // Array
	},

	every: function(func, thisObj){
		// summary:
		// 		A synthetic clone of `dojo.every` acting explicitly on this WidgetSet
		//
		// func: Function
		//		A callback function run for every widget in this list. Exits loop
		//		when the first false return is encountered.
		//
		// thisObj: Object?
		//		Optional scope parameter to use for the callback

		thisObj = thisObj || dojo.global;
		var x = 0, i;
		for(i in this._hash){
			if(!func.call(thisObj, this._hash[i], x++, this._hash)){
				return false; // Boolean
			}
		}
		return true; // Boolean
	},

	some: function(func, thisObj){
		// summary:
		// 		A synthetic clone of `dojo.some` acting explictly on this WidgetSet
		//
		// func: Function
		//		A callback function run for every widget in this list. Exits loop
		//		when the first true return is encountered.
		//
		// thisObj: Object?
		//		Optional scope parameter to use for the callback

		thisObj = thisObj || dojo.global;
		var x = 0, i;
		for(i in this._hash){
			if(func.call(thisObj, this._hash[i], x++, this._hash)){
				return true; // Boolean
			}
		}
		return false; // Boolean
	}

});

(function(){

	/*=====
	dijit.registry = {
		// summary:
		//		A list of widgets on a page.
		// description:
		//		Is an instance of `dijit.WidgetSet`
	};
	=====*/
	dijit.registry = new dijit.WidgetSet();

	var hash = dijit.registry._hash,
		attr = dojo.attr,
		hasAttr = dojo.hasAttr,
		style = dojo.style;

	dijit.byId = function(/*String|dijit._Widget*/ id){
		// summary:
		//		Returns a widget by it's id, or if passed a widget, no-op (like dojo.byId())
		return typeof id == "string" ? hash[id] : id; // dijit._Widget
	};

	var _widgetTypeCtr = {};
	dijit.getUniqueId = function(/*String*/widgetType){
		// summary:
		//		Generates a unique id for a given widgetType
	
		var id;
		do{
			id = widgetType + "_" +
				(widgetType in _widgetTypeCtr ?
					++_widgetTypeCtr[widgetType] : _widgetTypeCtr[widgetType] = 0);
		}while(hash[id]);
		return dijit._scopeName == "dijit" ? id : dijit._scopeName + "_" + id; // String
	};
	
	dijit.findWidgets = function(/*DomNode*/ root){
		// summary:
		//		Search subtree under root returning widgets found.
		//		Doesn't search for nested widgets (ie, widgets inside other widgets).
	
		var outAry = [];
	
		function getChildrenHelper(root){
			for(var node = root.firstChild; node; node = node.nextSibling){
				if(node.nodeType == 1){
					var widgetId = node.getAttribute("widgetId");
					if(widgetId){
						outAry.push(hash[widgetId]);
					}else{
						getChildrenHelper(node);
					}
				}
			}
		}
	
		getChildrenHelper(root);
		return outAry;
	};
	
	dijit._destroyAll = function(){
		// summary:
		//		Code to destroy all widgets and do other cleanup on page unload
	
		// Clean up focus manager lingering references to widgets and nodes
		dijit._curFocus = null;
		dijit._prevFocus = null;
		dijit._activeStack = [];
	
		// Destroy all the widgets, top down
		dojo.forEach(dijit.findWidgets(dojo.body()), function(widget){
			// Avoid double destroy of widgets like Menu that are attached to <body>
			// even though they are logically children of other widgets.
			if(!widget._destroyed){
				if(widget.destroyRecursive){
					widget.destroyRecursive();
				}else if(widget.destroy){
					widget.destroy();
				}
			}
		});
	};
	
	if(dojo.isIE){
		// Only run _destroyAll() for IE because we think it's only necessary in that case,
		// and because it causes problems on FF.  See bug #3531 for details.
		dojo.addOnWindowUnload(function(){
			dijit._destroyAll();
		});
	}
	
	dijit.byNode = function(/*DOMNode*/ node){
		// summary:
		//		Returns the widget corresponding to the given DOMNode
		return hash[node.getAttribute("widgetId")]; // dijit._Widget
	};
	
	dijit.getEnclosingWidget = function(/*DOMNode*/ node){
		// summary:
		//		Returns the widget whose DOM tree contains the specified DOMNode, or null if
		//		the node is not contained within the DOM tree of any widget
		while(node){
			var id = node.getAttribute && node.getAttribute("widgetId");
			if(id){
				return hash[id];
			}
			node = node.parentNode;
		}
		return null;
	};

	var shown = (dijit._isElementShown = function(/*Element*/ elem){
		var s = style(elem);
		return (s.visibility != "hidden")
			&& (s.visibility != "collapsed")
			&& (s.display != "none")
			&& (attr(elem, "type") != "hidden");
	});
	
	dijit.hasDefaultTabStop = function(/*Element*/ elem){
		// summary:
		//		Tests if element is tab-navigable even without an explicit tabIndex setting
	
		// No explicit tabIndex setting, need to investigate node type
		switch(elem.nodeName.toLowerCase()){
			case "a":
				// An <a> w/out a tabindex is only navigable if it has an href
				return hasAttr(elem, "href");
			case "area":
			case "button":
			case "input":
			case "object":
			case "select":
			case "textarea":
				// These are navigable by default
				return true;
			case "iframe":
				// If it's an editor <iframe> then it's tab navigable.
				//TODO: feature detect "designMode" in elem.contentDocument?
				if(dojo.isMoz){
					try{
						return elem.contentDocument.designMode == "on";
					}catch(err){
						return false;
					}
				}else if(dojo.isWebKit){
					var doc = elem.contentDocument,
						body = doc && doc.body;
					return body && body.contentEditable == 'true';
				}else{
					// contentWindow.document isn't accessible within IE7/8
					// if the iframe.src points to a foreign url and this
					// page contains an element, that could get focus
					try{
						doc = elem.contentWindow.document;
						body = doc && doc.body;
						return body && body.firstChild && body.firstChild.contentEditable == 'true';
					}catch(e){
						return false;
					}
				}
			default:
				return elem.contentEditable == 'true';
		}
	};
	
	var isTabNavigable = (dijit.isTabNavigable = function(/*Element*/ elem){
		// summary:
		//		Tests if an element is tab-navigable
	
		// TODO: convert (and rename method) to return effective tabIndex; will save time in _getTabNavigable()
		if(attr(elem, "disabled")){
			return false;
		}else if(hasAttr(elem, "tabIndex")){
			// Explicit tab index setting
			return attr(elem, "tabIndex") >= 0; // boolean
		}else{
			// No explicit tabIndex setting, so depends on node type
			return dijit.hasDefaultTabStop(elem);
		}
	});

	dijit._getTabNavigable = function(/*DOMNode*/ root){
		// summary:
		//		Finds descendants of the specified root node.
		//
		// description:
		//		Finds the following descendants of the specified root node:
		//		* the first tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the last tab-navigable element in document order
		//		  without a tabIndex or with tabIndex="0"
		//		* the first element in document order with the lowest
		//		  positive tabIndex value
		//		* the last element in document order with the highest
		//		  positive tabIndex value
		var first, last, lowest, lowestTabindex, highest, highestTabindex;
		var walkTree = function(/*DOMNode*/parent){
			dojo.query("> *", parent).forEach(function(child){
				// Skip hidden elements, and also non-HTML elements (those in custom namespaces) in IE,
				// since show() invokes getAttribute("type"), which crash on VML nodes in IE.
				if((dojo.isIE && child.scopeName!=="HTML") || !shown(child)){
					return;
				}

				if(isTabNavigable(child)){
					var tabindex = attr(child, "tabIndex");
					if(!hasAttr(child, "tabIndex") || tabindex == 0){
						if(!first){ first = child; }
						last = child;
					}else if(tabindex > 0){
						if(!lowest || tabindex < lowestTabindex){
							lowestTabindex = tabindex;
							lowest = child;
						}
						if(!highest || tabindex >= highestTabindex){
							highestTabindex = tabindex;
							highest = child;
						}
					}
				}
				if(child.nodeName.toUpperCase() != 'SELECT'){
					walkTree(child);
				}
			});
		};
		if(shown(root)){ walkTree(root) }
		return { first: first, last: last, lowest: lowest, highest: highest };
	}
	dijit.getFirstInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is first in the tabbing order
		var elems = dijit._getTabNavigable(dojo.byId(root));
		return elems.lowest ? elems.lowest : elems.first; // DomNode
	};
	
	dijit.getLastInTabbingOrder = function(/*String|DOMNode*/ root){
		// summary:
		//		Finds the descendant of the specified root node
		//		that is last in the tabbing order
		var elems = dijit._getTabNavigable(dojo.byId(root));
		return elems.last ? elems.last : elems.highest; // DomNode
	};
	
	/*=====
	dojo.mixin(dijit, {
		// defaultDuration: Integer
		//		The default animation speed (in ms) to use for all Dijit
		//		transitional animations, unless otherwise specified
		//		on a per-instance basis. Defaults to 200, overrided by
		//		`djConfig.defaultDuration`
		defaultDuration: 200
	});
	=====*/
	
	dijit.defaultDuration = dojo.config["defaultDuration"] || 200;

})();

}

if(!dojo._hasResource["dijit._base.focus"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.focus"] = true;
dojo.provide("dijit._base.focus");


	// for dijit.isTabNavigable()

// summary:
//		These functions are used to query or set the focus and selection.
//
//		Also, they trace when widgets become activated/deactivated,
//		so that the widget can fire _onFocus/_onBlur events.
//		"Active" here means something similar to "focused", but
//		"focus" isn't quite the right word because we keep track of
//		a whole stack of "active" widgets.  Example: ComboButton --> Menu -->
//		MenuItem.  The onBlur event for ComboButton doesn't fire due to focusing
//		on the Menu or a MenuItem, since they are considered part of the
//		ComboButton widget.  It only happens when focus is shifted
//		somewhere completely different.

dojo.mixin(dijit, {
	// _curFocus: DomNode
	//		Currently focused item on screen
	_curFocus: null,

	// _prevFocus: DomNode
	//		Previously focused item on screen
	_prevFocus: null,

	isCollapsed: function(){
		// summary:
		//		Returns true if there is no text selected
		return dijit.getBookmark().isCollapsed;
	},

	getBookmark: function(){
		// summary:
		//		Retrieves a bookmark that can be used with moveToBookmark to return to the same range
		var bm, rg, tg, sel = dojo.doc.selection, cf = dijit._curFocus;

		if(dojo.global.getSelection){
			//W3C Range API for selections.
			sel = dojo.global.getSelection();
			if(sel){
				if(sel.isCollapsed){
					tg = cf? cf.tagName : "";
					if(tg){
						//Create a fake rangelike item to restore selections.
						tg = tg.toLowerCase();
						if(tg == "textarea" ||
								(tg == "input" && (!cf.type || cf.type.toLowerCase() == "text"))){
							sel = {
								start: cf.selectionStart,
								end: cf.selectionEnd,
								node: cf,
								pRange: true
							};
							return {isCollapsed: (sel.end <= sel.start), mark: sel}; //Object.
						}
					}
					bm = {isCollapsed:true};
				}else{
					rg = sel.getRangeAt(0);
					bm = {isCollapsed: false, mark: rg.cloneRange()};
				}
			}
		}else if(sel){
			// If the current focus was a input of some sort and no selection, don't bother saving
			// a native bookmark.  This is because it causes issues with dialog/page selection restore.
			// So, we need to create psuedo bookmarks to work with.
			tg = cf ? cf.tagName : "";
			tg = tg.toLowerCase();
			if(cf && tg && (tg == "button" || tg == "textarea" || tg == "input")){
				if(sel.type && sel.type.toLowerCase() == "none"){
					return {
						isCollapsed: true,
						mark: null
					}
				}else{
					rg = sel.createRange();
					return {
						isCollapsed: rg.text && rg.text.length?false:true,
						mark: {
							range: rg,
							pRange: true
						}
					};
				}
			}
			bm = {};

			//'IE' way for selections.
			try{
				// createRange() throws exception when dojo in iframe
				//and nothing selected, see #9632
				rg = sel.createRange();
				bm.isCollapsed = !(sel.type == 'Text' ? rg.htmlText.length : rg.length);
			}catch(e){
				bm.isCollapsed = true;
				return bm;
			}
			if(sel.type.toUpperCase() == 'CONTROL'){
				if(rg.length){
					bm.mark=[];
					var i=0,len=rg.length;
					while(i<len){
						bm.mark.push(rg.item(i++));
					}
				}else{
					bm.isCollapsed = true;
					bm.mark = null;
				}
			}else{
				bm.mark = rg.getBookmark();
			}
		}else{
			console.warn("No idea how to store the current selection for this browser!");
		}
		return bm; // Object
	},

	moveToBookmark: function(/*Object*/bookmark){
		// summary:
		//		Moves current selection to a bookmark
		// bookmark:
		//		This should be a returned object from dijit.getBookmark()

		var _doc = dojo.doc,
			mark = bookmark.mark;
		if(mark){
			if(dojo.global.getSelection){
				//W3C Rangi API (FF, WebKit, Opera, etc)
				var sel = dojo.global.getSelection();
				if(sel && sel.removeAllRanges){
					if(mark.pRange){
						var r = mark;
						var n = r.node;
						n.selectionStart = r.start;
						n.selectionEnd = r.end;
					}else{
						sel.removeAllRanges();
						sel.addRange(mark);
					}
				}else{
					console.warn("No idea how to restore selection for this browser!");
				}
			}else if(_doc.selection && mark){
				//'IE' way.
				var rg;
				if(mark.pRange){
					rg = mark.range;
				}else if(dojo.isArray(mark)){
					rg = _doc.body.createControlRange();
					//rg.addElement does not have call/apply method, so can not call it directly
					//rg is not available in "range.addElement(item)", so can't use that either
					dojo.forEach(mark, function(n){
						rg.addElement(n);
					});
				}else{
					rg = _doc.body.createTextRange();
					rg.moveToBookmark(mark);
				}
				rg.select();
			}
		}
	},

	getFocus: function(/*Widget?*/ menu, /*Window?*/ openedForWindow){
		// summary:
		//		Called as getFocus(), this returns an Object showing the current focus
		//		and selected text.
		//
		//		Called as getFocus(widget), where widget is a (widget representing) a button
		//		that was just pressed, it returns where focus was before that button
		//		was pressed.   (Pressing the button may have either shifted focus to the button,
		//		or removed focus altogether.)   In this case the selected text is not returned,
		//		since it can't be accurately determined.
		//
		// menu: dijit._Widget or {domNode: DomNode} structure
		//		The button that was just pressed.  If focus has disappeared or moved
		//		to this button, returns the previous focus.  In this case the bookmark
		//		information is already lost, and null is returned.
		//
		// openedForWindow:
		//		iframe in which menu was opened
		//
		// returns:
		//		A handle to restore focus/selection, to be passed to `dijit.focus`
		var node = !dijit._curFocus || (menu && dojo.isDescendant(dijit._curFocus, menu.domNode)) ? dijit._prevFocus : dijit._curFocus;
		return {
			node: node,
			bookmark: (node == dijit._curFocus) && dojo.withGlobal(openedForWindow || dojo.global, dijit.getBookmark),
			openedForWindow: openedForWindow
		}; // Object
	},

	focus: function(/*Object || DomNode */ handle){
		// summary:
		//		Sets the focused node and the selection according to argument.
		//		To set focus to an iframe's content, pass in the iframe itself.
		// handle:
		//		object returned by get(), or a DomNode

		if(!handle){ return; }

		var node = "node" in handle ? handle.node : handle,		// because handle is either DomNode or a composite object
			bookmark = handle.bookmark,
			openedForWindow = handle.openedForWindow,
			collapsed = bookmark ? bookmark.isCollapsed : false;

		// Set the focus
		// Note that for iframe's we need to use the <iframe> to follow the parentNode chain,
		// but we need to set focus to iframe.contentWindow
		if(node){
			var focusNode = (node.tagName.toLowerCase() == "iframe") ? node.contentWindow : node;
			if(focusNode && focusNode.focus){
				try{
					// Gecko throws sometimes if setting focus is impossible,
					// node not displayed or something like that
					focusNode.focus();
				}catch(e){/*quiet*/}
			}
			dijit._onFocusNode(node);
		}

		// set the selection
		// do not need to restore if current selection is not empty
		// (use keyboard to select a menu item) or if previous selection was collapsed
		// as it may cause focus shift (Esp in IE).
		if(bookmark && dojo.withGlobal(openedForWindow || dojo.global, dijit.isCollapsed) && !collapsed){
			if(openedForWindow){
				openedForWindow.focus();
			}
			try{
				dojo.withGlobal(openedForWindow || dojo.global, dijit.moveToBookmark, null, [bookmark]);
			}catch(e2){
				/*squelch IE internal error, see http://trac.dojotoolkit.org/ticket/1984 */
			}
		}
	},

	// _activeStack: dijit._Widget[]
	//		List of currently active widgets (focused widget and it's ancestors)
	_activeStack: [],

	registerIframe: function(/*DomNode*/ iframe){
		// summary:
		//		Registers listeners on the specified iframe so that any click
		//		or focus event on that iframe (or anything in it) is reported
		//		as a focus/click event on the <iframe> itself.
		// description:
		//		Currently only used by editor.
		// returns:
		//		Handle to pass to unregisterIframe()
		return dijit.registerWin(iframe.contentWindow, iframe);
	},

	unregisterIframe: function(/*Object*/ handle){
		// summary:
		//		Unregisters listeners on the specified iframe created by registerIframe.
		//		After calling be sure to delete or null out the handle itself.
		// handle:
		//		Handle returned by registerIframe()

		dijit.unregisterWin(handle);
	},

	registerWin: function(/*Window?*/targetWindow, /*DomNode?*/ effectiveNode){
		// summary:
		//		Registers listeners on the specified window (either the main
		//		window or an iframe's window) to detect when the user has clicked somewhere
		//		or focused somewhere.
		// description:
		//		Users should call registerIframe() instead of this method.
		// targetWindow:
		//		If specified this is the window associated with the iframe,
		//		i.e. iframe.contentWindow.
		// effectiveNode:
		//		If specified, report any focus events inside targetWindow as
		//		an event on effectiveNode, rather than on evt.target.
		// returns:
		//		Handle to pass to unregisterWin()

		// TODO: make this function private in 2.0; Editor/users should call registerIframe(),

		var mousedownListener = function(evt){
			dijit._justMouseDowned = true;
			setTimeout(function(){ dijit._justMouseDowned = false; }, 0);
			
			// workaround weird IE bug where the click is on an orphaned node
			// (first time clicking a Select/DropDownButton inside a TooltipDialog)
			if(dojo.isIE && evt && evt.srcElement && evt.srcElement.parentNode == null){
				return;
			}

			dijit._onTouchNode(effectiveNode || evt.target || evt.srcElement, "mouse");
		};
		//dojo.connect(targetWindow, "onscroll", ???);

		// Listen for blur and focus events on targetWindow's document.
		// IIRC, I'm using attachEvent() rather than dojo.connect() because focus/blur events don't bubble
		// through dojo.connect(), and also maybe to catch the focus events early, before onfocus handlers
		// fire.
		// Connect to <html> (rather than document) on IE to avoid memory leaks, but document on other browsers because
		// (at least for FF) the focus event doesn't fire on <html> or <body>.
		var doc = dojo.isIE ? targetWindow.document.documentElement : targetWindow.document;
		if(doc){
			if(dojo.isIE){
				doc.attachEvent('onmousedown', mousedownListener);
				var activateListener = function(evt){
					// IE reports that nodes like <body> have gotten focus, even though they have tabIndex=-1,
					// Should consider those more like a mouse-click than a focus....
					if(evt.srcElement.tagName.toLowerCase() != "#document" &&
						dijit.isTabNavigable(evt.srcElement)){
						dijit._onFocusNode(effectiveNode || evt.srcElement);
					}else{
						dijit._onTouchNode(effectiveNode || evt.srcElement);
					}
				};
				doc.attachEvent('onactivate', activateListener);
				var deactivateListener =  function(evt){
					dijit._onBlurNode(effectiveNode || evt.srcElement);
				};
				doc.attachEvent('ondeactivate', deactivateListener);

				return function(){
					doc.detachEvent('onmousedown', mousedownListener);
					doc.detachEvent('onactivate', activateListener);
					doc.detachEvent('ondeactivate', deactivateListener);
					doc = null;	// prevent memory leak (apparent circular reference via closure)
				};
			}else{
				doc.addEventListener('mousedown', mousedownListener, true);
				var focusListener = function(evt){
					dijit._onFocusNode(effectiveNode || evt.target);
				};
				doc.addEventListener('focus', focusListener, true);
				var blurListener = function(evt){
					dijit._onBlurNode(effectiveNode || evt.target);
				};
				doc.addEventListener('blur', blurListener, true);

				return function(){
					doc.removeEventListener('mousedown', mousedownListener, true);
					doc.removeEventListener('focus', focusListener, true);
					doc.removeEventListener('blur', blurListener, true);
					doc = null;	// prevent memory leak (apparent circular reference via closure)
				};
			}
		}
	},

	unregisterWin: function(/*Handle*/ handle){
		// summary:
		//		Unregisters listeners on the specified window (either the main
		//		window or an iframe's window) according to handle returned from registerWin().
		//		After calling be sure to delete or null out the handle itself.

		// Currently our handle is actually a function
		handle && handle();
	},

	_onBlurNode: function(/*DomNode*/ node){
		// summary:
		// 		Called when focus leaves a node.
		//		Usually ignored, _unless_ it *isn't* follwed by touching another node,
		//		which indicates that we tabbed off the last field on the page,
		//		in which case every widget is marked inactive
		dijit._prevFocus = dijit._curFocus;
		dijit._curFocus = null;

		if(dijit._justMouseDowned){
			// the mouse down caused a new widget to be marked as active; this blur event
			// is coming late, so ignore it.
			return;
		}

		// if the blur event isn't followed by a focus event then mark all widgets as inactive.
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
		}
		dijit._clearActiveWidgetsTimer = setTimeout(function(){
			delete dijit._clearActiveWidgetsTimer;
			dijit._setStack([]);
			dijit._prevFocus = null;
		}, 100);
	},

	_onTouchNode: function(/*DomNode*/ node, /*String*/ by){
		// summary:
		//		Callback when node is focused or mouse-downed
		// node:
		//		The node that was touched.
		// by:
		//		"mouse" if the focus/touch was caused by a mouse down event

		// ignore the recent blurNode event
		if(dijit._clearActiveWidgetsTimer){
			clearTimeout(dijit._clearActiveWidgetsTimer);
			delete dijit._clearActiveWidgetsTimer;
		}

		// compute stack of active widgets (ex: ComboButton --> Menu --> MenuItem)
		var newStack=[];
		try{
			while(node){
				var popupParent = dojo.attr(node, "dijitPopupParent");
				if(popupParent){
					node=dijit.byId(popupParent).domNode;
				}else if(node.tagName && node.tagName.toLowerCase() == "body"){
					// is this the root of the document or just the root of an iframe?
					if(node === dojo.body()){
						// node is the root of the main document
						break;
					}
					// otherwise, find the iframe this node refers to (can't access it via parentNode,
					// need to do this trick instead). window.frameElement is supported in IE/FF/Webkit
					node=dojo.window.get(node.ownerDocument).frameElement;
				}else{
					// if this node is the root node of a widget, then add widget id to stack,
					// except ignore clicks on disabled widgets (actually focusing a disabled widget still works,
					// to support MenuItem)
					var id = node.getAttribute && node.getAttribute("widgetId"),
						widget = id && dijit.byId(id);
					if(widget && !(by == "mouse" && widget.get("disabled"))){
						newStack.unshift(id);
					}
					node=node.parentNode;
				}
			}
		}catch(e){ /* squelch */ }

		dijit._setStack(newStack, by);
	},

	_onFocusNode: function(/*DomNode*/ node){
		// summary:
		//		Callback when node is focused

		if(!node){
			return;
		}

		if(node.nodeType == 9){
			// Ignore focus events on the document itself.  This is here so that
			// (for example) clicking the up/down arrows of a spinner
			// (which don't get focus) won't cause that widget to blur. (FF issue)
			return;
		}

		dijit._onTouchNode(node);

		if(node == dijit._curFocus){ return; }
		if(dijit._curFocus){
			dijit._prevFocus = dijit._curFocus;
		}
		dijit._curFocus = node;
		dojo.publish("focusNode", [node]);
	},

	_setStack: function(/*String[]*/ newStack, /*String*/ by){
		// summary:
		//		The stack of active widgets has changed.  Send out appropriate events and records new stack.
		// newStack:
		//		array of widget id's, starting from the top (outermost) widget
		// by:
		//		"mouse" if the focus/touch was caused by a mouse down event

		var oldStack = dijit._activeStack;
		dijit._activeStack = newStack;

		// compare old stack to new stack to see how many elements they have in common
		for(var nCommon=0; nCommon<Math.min(oldStack.length, newStack.length); nCommon++){
			if(oldStack[nCommon] != newStack[nCommon]){
				break;
			}
		}

		var widget;
		// for all elements that have gone out of focus, send blur event
		for(var i=oldStack.length-1; i>=nCommon; i--){
			widget = dijit.byId(oldStack[i]);
			if(widget){
				widget._focused = false;
				widget._hasBeenBlurred = true;
				if(widget._onBlur){
					widget._onBlur(by);
				}
				dojo.publish("widgetBlur", [widget, by]);
			}
		}

		// for all element that have come into focus, send focus event
		for(i=nCommon; i<newStack.length; i++){
			widget = dijit.byId(newStack[i]);
			if(widget){
				widget._focused = true;
				if(widget._onFocus){
					widget._onFocus(by);
				}
				dojo.publish("widgetFocus", [widget, by]);
			}
		}
	}
});

// register top window and all the iframes it contains
dojo.addOnLoad(function(){
	var handle = dijit.registerWin(window);
	if(dojo.isIE){
		dojo.addOnWindowUnload(function(){
			dijit.unregisterWin(handle);
			handle = null;
		})
	}
});

}

if(!dojo._hasResource["dojo.AdapterRegistry"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.AdapterRegistry"] = true;
dojo.provide("dojo.AdapterRegistry");

dojo.AdapterRegistry = function(/*Boolean?*/ returnWrappers){
	//	summary:
	//		A registry to make contextual calling/searching easier.
	//	description:
	//		Objects of this class keep list of arrays in the form [name, check,
	//		wrap, directReturn] that are used to determine what the contextual
	//		result of a set of checked arguments is. All check/wrap functions
	//		in this registry should be of the same arity.
	//	example:
	//	|	// create a new registry
	//	|	var reg = new dojo.AdapterRegistry();
	//	|	reg.register("handleString",
	//	|		dojo.isString,
	//	|		function(str){
	//	|			// do something with the string here
	//	|		}
	//	|	);
	//	|	reg.register("handleArr",
	//	|		dojo.isArray,
	//	|		function(arr){
	//	|			// do something with the array here
	//	|		}
	//	|	);
	//	|
	//	|	// now we can pass reg.match() *either* an array or a string and
	//	|	// the value we pass will get handled by the right function
	//	|	reg.match("someValue"); // will call the first function
	//	|	reg.match(["someValue"]); // will call the second

	this.pairs = [];
	this.returnWrappers = returnWrappers || false; // Boolean
}

dojo.extend(dojo.AdapterRegistry, {
	register: function(/*String*/ name, /*Function*/ check, /*Function*/ wrap, /*Boolean?*/ directReturn, /*Boolean?*/ override){
		//	summary: 
		//		register a check function to determine if the wrap function or
		//		object gets selected
		//	name:
		//		a way to identify this matcher.
		//	check:
		//		a function that arguments are passed to from the adapter's
		//		match() function.  The check function should return true if the
		//		given arguments are appropriate for the wrap function.
		//	directReturn:
		//		If directReturn is true, the value passed in for wrap will be
		//		returned instead of being called. Alternately, the
		//		AdapterRegistry can be set globally to "return not call" using
		//		the returnWrappers property. Either way, this behavior allows
		//		the registry to act as a "search" function instead of a
		//		function interception library.
		//	override:
		//		If override is given and true, the check function will be given
		//		highest priority. Otherwise, it will be the lowest priority
		//		adapter.
		this.pairs[((override) ? "unshift" : "push")]([name, check, wrap, directReturn]);
	},

	match: function(/* ... */){
		// summary:
		//		Find an adapter for the given arguments. If no suitable adapter
		//		is found, throws an exception. match() accepts any number of
		//		arguments, all of which are passed to all matching functions
		//		from the registered pairs.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[1].apply(this, arguments)){
				if((pair[3])||(this.returnWrappers)){
					return pair[2];
				}else{
					return pair[2].apply(this, arguments);
				}
			}
		}
		throw new Error("No match found");
	},

	unregister: function(name){
		// summary: Remove a named adapter from the registry

		// FIXME: this is kind of a dumb way to handle this. On a large
		// registry this will be slow-ish and we can use the name as a lookup
		// should we choose to trade memory for speed.
		for(var i = 0; i < this.pairs.length; i++){
			var pair = this.pairs[i];
			if(pair[0] == name){
				this.pairs.splice(i, 1);
				return true;
			}
		}
		return false;
	}
});

}

if(!dojo._hasResource["dijit._base.place"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.place"] = true;
dojo.provide("dijit._base.place");





dijit.getViewport = function(){
	// summary:
	//		Returns the dimensions and scroll position of the viewable area of a browser window

	return dojo.window.getBox();
};

/*=====
dijit.__Position = function(){
	// x: Integer
	//		horizontal coordinate in pixels, relative to document body
	// y: Integer
	//		vertical coordinate in pixels, relative to document body

	thix.x = x;
	this.y = y;
}
=====*/


dijit.placeOnScreen = function(
	/* DomNode */			node,
	/* dijit.__Position */	pos,
	/* String[] */			corners,
	/* dijit.__Position? */	padding){
	// summary:
	//		Positions one of the node's corners at specified position
	//		such that node is fully visible in viewport.
	// description:
	//		NOTE: node is assumed to be absolutely or relatively positioned.
	//	pos:
	//		Object like {x: 10, y: 20}
	//	corners:
	//		Array of Strings representing order to try corners in, like ["TR", "BL"].
	//		Possible values are:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//	padding:
	//		set padding to put some buffer around the element you want to position.
	// example:
	//		Try to place node's top right corner at (10,20).
	//		If that makes node go (partially) off screen, then try placing
	//		bottom left corner at (10,20).
	//	|	placeOnScreen(node, {x: 10, y: 20}, ["TR", "BL"])

	var choices = dojo.map(corners, function(corner){
		var c = { corner: corner, pos: {x:pos.x,y:pos.y} };
		if(padding){
			c.pos.x += corner.charAt(1) == 'L' ? padding.x : -padding.x;
			c.pos.y += corner.charAt(0) == 'T' ? padding.y : -padding.y;
		}
		return c;
	});

	return dijit._place(node, choices);
}

dijit._place = function(/*DomNode*/ node, /* Array */ choices, /* Function */ layoutNode){
	// summary:
	//		Given a list of spots to put node, put it at the first spot where it fits,
	//		of if it doesn't fit anywhere then the place with the least overflow
	// choices: Array
	//		Array of elements like: {corner: 'TL', pos: {x: 10, y: 20} }
	//		Above example says to put the top-left corner of the node at (10,20)
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
	//		for things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.

	// get {x: 10, y: 10, w: 100, h:100} type obj representing position of
	// viewport over document
	var view = dojo.window.getBox();

	// This won't work if the node is inside a <div style="position: relative">,
	// so reattach it to dojo.doc.body.   (Otherwise, the positioning will be wrong
	// and also it might get cutoff)
	if(!node.parentNode || String(node.parentNode.tagName).toLowerCase() != "body"){
		dojo.body().appendChild(node);
	}

	var best = null;
	dojo.some(choices, function(choice){
		var corner = choice.corner;
		var pos = choice.pos;

		// configure node to be displayed in given position relative to button
		// (need to do this in order to get an accurate size for the node, because
		// a tooltips size changes based on position, due to triangle)
		if(layoutNode){
			layoutNode(node, choice.aroundCorner, corner);
		}

		// get node's size
		var style = node.style;
		var oldDisplay = style.display;
		var oldVis = style.visibility;
		style.visibility = "hidden";
		style.display = "";
		var mb = dojo.marginBox(node);
		style.display = oldDisplay;
		style.visibility = oldVis;

		// coordinates and size of node with specified corner placed at pos,
		// and clipped by viewport
		var startX = Math.max(view.l, corner.charAt(1) == 'L' ? pos.x : (pos.x - mb.w)),
			startY = Math.max(view.t, corner.charAt(0) == 'T' ? pos.y : (pos.y - mb.h)),
			endX = Math.min(view.l + view.w, corner.charAt(1) == 'L' ? (startX + mb.w) : pos.x),
			endY = Math.min(view.t + view.h, corner.charAt(0) == 'T' ? (startY + mb.h) : pos.y),
			width = endX - startX,
			height = endY - startY,
			overflow = (mb.w - width) + (mb.h - height);

		if(best == null || overflow < best.overflow){
			best = {
				corner: corner,
				aroundCorner: choice.aroundCorner,
				x: startX,
				y: startY,
				w: width,
				h: height,
				overflow: overflow
			};
		}
		return !overflow;
	});

	node.style.left = best.x + "px";
	node.style.top = best.y + "px";
	if(best.overflow && layoutNode){
		layoutNode(node, best.aroundCorner, best.corner);
	}
	return best;
}

dijit.placeOnScreenAroundNode = function(
	/* DomNode */		node,
	/* DomNode */		aroundNode,
	/* Object */		aroundCorners,
	/* Function? */		layoutNode){

	// summary:
	//		Position node adjacent or kitty-corner to aroundNode
	//		such that it's fully visible in viewport.
	//
	// description:
	//		Place node such that corner of node touches a corner of
	//		aroundNode, and that node is fully visible.
	//
	// aroundCorners:
	//		Ordered list of pairs of corners to try matching up.
	//		Each pair of corners is represented as a key/value in the hash,
	//		where the key corresponds to the aroundNode's corner, and
	//		the value corresponds to the node's corner:
	//
	//	|	{ aroundNodeCorner1: nodeCorner1, aroundNodeCorner2: nodeCorner2, ...}
	//
	//		The following strings are used to represent the four corners:
	//			* "BL" - bottom left
	//			* "BR" - bottom right
	//			* "TL" - top left
	//			* "TR" - top right
	//
	// layoutNode: Function(node, aroundNodeCorner, nodeCorner)
	//		For things like tooltip, they are displayed differently (and have different dimensions)
	//		based on their orientation relative to the parent.   This adjusts the popup based on orientation.
	//
	// example:
	//	|	dijit.placeOnScreenAroundNode(node, aroundNode, {'BL':'TL', 'TR':'BR'});
	//		This will try to position node such that node's top-left corner is at the same position
	//		as the bottom left corner of the aroundNode (ie, put node below
	//		aroundNode, with left edges aligned).  If that fails it will try to put
	// 		the bottom-right corner of node where the top right corner of aroundNode is
	//		(ie, put node above aroundNode, with right edges aligned)
	//

	// get coordinates of aroundNode
	aroundNode = dojo.byId(aroundNode);
	var oldDisplay = aroundNode.style.display;
	aroundNode.style.display="";
	// #3172: use the slightly tighter border box instead of marginBox
	var aroundNodePos = dojo.position(aroundNode, true);
	aroundNode.style.display=oldDisplay;

	// place the node around the calculated rectangle
	return dijit._placeOnScreenAroundRect(node,
		aroundNodePos.x, aroundNodePos.y, aroundNodePos.w, aroundNodePos.h,	// rectangle
		aroundCorners, layoutNode);
};

/*=====
dijit.__Rectangle = function(){
	// x: Integer
	//		horizontal offset in pixels, relative to document body
	// y: Integer
	//		vertical offset in pixels, relative to document body
	// width: Integer
	//		width in pixels
	// height: Integer
	//		height in pixels

	this.x = x;
	this.y = y;
	this.width = width;
	this.height = height;
}
=====*/


dijit.placeOnScreenAroundRectangle = function(
	/* DomNode */			node,
	/* dijit.__Rectangle */	aroundRect,
	/* Object */			aroundCorners,
	/* Function */			layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except that the "around"
	//		parameter is an arbitrary rectangle on the screen (x, y, width, height)
	//		instead of a dom node.

	return dijit._placeOnScreenAroundRect(node,
		aroundRect.x, aroundRect.y, aroundRect.width, aroundRect.height,	// rectangle
		aroundCorners, layoutNode);
};

dijit._placeOnScreenAroundRect = function(
	/* DomNode */		node,
	/* Number */		x,
	/* Number */		y,
	/* Number */		width,
	/* Number */		height,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts coordinates
	//		of a rectangle to place node adjacent to.

	// TODO: combine with placeOnScreenAroundRectangle()

	// Generate list of possible positions for node
	var choices = [];
	for(var nodeCorner in aroundCorners){
		choices.push( {
			aroundCorner: nodeCorner,
			corner: aroundCorners[nodeCorner],
			pos: {
				x: x + (nodeCorner.charAt(1) == 'L' ? 0 : width),
				y: y + (nodeCorner.charAt(0) == 'T' ? 0 : height)
			}
		});
	}

	return dijit._place(node, choices, layoutNode);
};

dijit.placementRegistry= new dojo.AdapterRegistry();
dijit.placementRegistry.register("node",
	function(n, x){
		return typeof x == "object" &&
			typeof x.offsetWidth != "undefined" && typeof x.offsetHeight != "undefined";
	},
	dijit.placeOnScreenAroundNode);
dijit.placementRegistry.register("rect",
	function(n, x){
		return typeof x == "object" &&
			"x" in x && "y" in x && "width" in x && "height" in x;
	},
	dijit.placeOnScreenAroundRectangle);

dijit.placeOnScreenAroundElement = function(
	/* DomNode */		node,
	/* Object */		aroundElement,
	/* Object */		aroundCorners,
	/* Function */		layoutNode){

	// summary:
	//		Like dijit.placeOnScreenAroundNode(), except it accepts an arbitrary object
	//		for the "around" argument and finds a proper processor to place a node.

	return dijit.placementRegistry.match.apply(dijit.placementRegistry, arguments);
};

dijit.getPopupAroundAlignment = function(/*Array*/ position, /*Boolean*/ leftToRight){
	// summary:
	//		Transforms the passed array of preferred positions into a format suitable for passing as the aroundCorners argument to dijit.placeOnScreenAroundElement.
	//
	// position: String[]
	//		This variable controls the position of the drop down.
	//		It's an array of strings with the following values:
	//
	//			* before: places drop down to the left of the target node/widget, or to the right in
	//			  the case of RTL scripts like Hebrew and Arabic
	//			* after: places drop down to the right of the target node/widget, or to the left in
	//			  the case of RTL scripts like Hebrew and Arabic
	//			* above: drop down goes above target node
	//			* below: drop down goes below target node
	//
	//		The list is positions is tried, in order, until a position is found where the drop down fits
	//		within the viewport.
	//
	// leftToRight: Boolean
	//		Whether the popup will be displaying in leftToRight mode.
	//
	var align = {};
	dojo.forEach(position, function(pos){
		switch(pos){
			case "after":
				align[leftToRight ? "BR" : "BL"] = leftToRight ? "BL" : "BR";
				break;
			case "before":
				align[leftToRight ? "BL" : "BR"] = leftToRight ? "BR" : "BL";
				break;
			case "below":
				// first try to align left borders, next try to align right borders (or reverse for RTL mode)
				align[leftToRight ? "BL" : "BR"] = leftToRight ? "TL" : "TR";
				align[leftToRight ? "BR" : "BL"] = leftToRight ? "TR" : "TL";
				break;
			case "above":
			default:
				// first try to align left borders, next try to align right borders (or reverse for RTL mode)
				align[leftToRight ? "TL" : "TR"] = leftToRight ? "BL" : "BR";
				align[leftToRight ? "TR" : "TL"] = leftToRight ? "BR" : "BL";
				break;
		}
	});
	return align;
};

}

if(!dojo._hasResource["dijit._base.window"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.window"] = true;
dojo.provide("dijit._base.window");



dijit.getDocumentWindow = function(doc){
	return dojo.window.get(doc);
};

}

if(!dojo._hasResource["dijit._base.popup"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.popup"] = true;
dojo.provide("dijit._base.popup");





/*=====
dijit.popup.__OpenArgs = function(){
	// popup: Widget
	//		widget to display
	// parent: Widget
	//		the button etc. that is displaying this popup
	// around: DomNode
	//		DOM node (typically a button); place popup relative to this node.  (Specify this *or* "x" and "y" parameters.)
	// x: Integer
	//		Absolute horizontal position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// y: Integer
	//		Absolute vertical position (in pixels) to place node at.  (Specify this *or* "around" parameter.)
	// orient: Object|String
	//		When the around parameter is specified, orient should be an
	//		ordered list of tuples of the form (around-node-corner, popup-node-corner).
	//		dijit.popup.open() tries to position the popup according to each tuple in the list, in order,
	//		until the popup appears fully within the viewport.
	//
	//		The default value is {BL:'TL', TL:'BL'}, which represents a list of two tuples:
	//			1. (BL, TL)
	//			2. (TL, BL)
	//		where BL means "bottom left" and "TL" means "top left".
	//		So by default, it first tries putting the popup below the around node, left-aligning them,
	//		and then tries to put it above the around node, still left-aligning them.   Note that the
	//		default is horizontally reversed when in RTL mode.
	//
	//		When an (x,y) position is specified rather than an around node, orient is either
	//		"R" or "L".  R (for right) means that it tries to put the popup to the right of the mouse,
	//		specifically positioning the popup's top-right corner at the mouse position, and if that doesn't
	//		fit in the viewport, then it tries, in order, the bottom-right corner, the top left corner,
	//		and the top-right corner.
	// onCancel: Function
	//		callback when user has canceled the popup by
	//			1. hitting ESC or
	//			2. by using the popup widget's proprietary cancel mechanism (like a cancel button in a dialog);
	//			   i.e. whenever popupWidget.onCancel() is called, args.onCancel is called
	// onClose: Function
	//		callback whenever this popup is closed
	// onExecute: Function
	//		callback when user "executed" on the popup/sub-popup by selecting a menu choice, etc. (top menu only)
	// padding: dijit.__Position
	//		adding a buffer around the opening position. This is only useful when around is not set.
	this.popup = popup;
	this.parent = parent;
	this.around = around;
	this.x = x;
	this.y = y;
	this.orient = orient;
	this.onCancel = onCancel;
	this.onClose = onClose;
	this.onExecute = onExecute;
	this.padding = padding;
}
=====*/

dijit.popup = {
	// summary:
	//		This singleton is used to show/hide widgets as popups.

	// _stack: dijit._Widget[]
	//		Stack of currently popped up widgets.
	//		(someone opened _stack[0], and then it opened _stack[1], etc.)
	_stack: [],
	
	// _beginZIndex: Number
	//		Z-index of the first popup.   (If first popup opens other
	//		popups they get a higher z-index.)
	_beginZIndex: 1000,

	_idGen: 1,

	// TODO: in 2.0 change signature to always take widget
	moveOffScreen: function(/*Widget || DomNode*/ widget){
		// summary:
		//		Initialization for widgets that will be used as popups
		//
		// description:
		//		Puts widget inside a wrapper DIV (if not already in one), and
		//		positions wrapper DIV off screen, but not display:none, so that
		//		the widget doesn't appear in the page flow and/or cause a blank
		//		area at the bottom of the viewport (making scrollbar longer), but
		//		initialization of contained widgets works correctly

		var wrapper = widget.declaredClass ? widget._popupWrapper : (dojo.hasClass(widget.parentNode, "dijitPopup") && widget.parentNode),
			node = widget.domNode || widget;

		if(wrapper){
			// Widget already is encased in popup wrapper, just move wrapper off screen
			dojo.style(wrapper, {
				visibility: "hidden",
				// prevent transient scrollbar causing misalign (#5776), and initial flash in upper left (#10111)
				top: "-9999px"
			});
		}else{
			// Create an offscreen wrapper <div> for when this widget (in the future) will be used as a popup.
			// This is done early because of IE bugs where creating/moving DOM nodes causes focus
			// to go wonky, see tests/robot/Toolbar.html to reproduce
			wrapper = dojo.create("div",{
				"class":"dijitPopup",
				style:{
					visibility:"hidden",
					// prevent transient scrollbar causing misalign (#5776), and initial flash in upper left (#10111)
					top: "-9999px"
				},
				role: "presentation"
			}, dojo.body());
			wrapper.appendChild(node);

			var s = node.style;
			s.display = "";
			s.visibility = "";
			s.position = "";
			s.top = "0px";

			if(widget.declaredClass){		// TODO: in 2.0 change signature to always take widget, then remove if()
				widget._popupWrapper = wrapper;
				dojo.connect(widget, "destroy", function(){
					dojo.destroy(wrapper);
					delete widget._popupWrapper;
				});
			}
		}
	},

	getTopPopup: function(){
		// summary:
		//		Compute the closest ancestor popup that's *not* a child of another popup.
		//		Ex: For a TooltipDialog with a button that spawns a tree of menus, find the popup of the button.
		var stack = this._stack;
		for(var pi=stack.length-1; pi > 0 && stack[pi].parent === stack[pi-1].widget; pi--){
			/* do nothing, just trying to get right value for pi */
		}
		return stack[pi];
	},

	open: function(/*dijit.popup.__OpenArgs*/ args){
		// summary:
		//		Popup the widget at the specified position
		//
		// example:
		//		opening at the mouse position
		//		|		dijit.popup.open({popup: menuWidget, x: evt.pageX, y: evt.pageY});
		//
		// example:
		//		opening the widget as a dropdown
		//		|		dijit.popup.open({parent: this, popup: menuWidget, around: this.domNode, onClose: function(){...}});
		//
		//		Note that whatever widget called dijit.popup.open() should also listen to its own _onBlur callback
		//		(fired from _base/focus.js) to know that focus has moved somewhere else and thus the popup should be closed.

		var stack = this._stack,
			widget = args.popup,
			orient = args.orient || (
				(args.parent ? args.parent.isLeftToRight() : dojo._isBodyLtr()) ?
				{'BL':'TL', 'BR':'TR', 'TL':'BL', 'TR':'BR'} :
				{'BR':'TR', 'BL':'TL', 'TR':'BR', 'TL':'BL'}
			),
			around = args.around,
			id = (args.around && args.around.id) ? (args.around.id+"_dropdown") : ("popup_"+this._idGen++);


		// The wrapper may have already been created, but in case it wasn't, create here
		if(!widget._popupWrapper){
			this.moveOffScreen(widget);
		}
		var wrapper = widget._popupWrapper;

		dojo.attr(wrapper, {
			id: id,
			style: {
				zIndex: this._beginZIndex + stack.length
			},
			"class": "dijitPopup " + (widget.baseClass || widget["class"] || "").split(" ")[0] +"Popup",
			dijitPopupParent: args.parent ? args.parent.id : ""
		});

		if(dojo.isIE || dojo.isMoz){
			if(!widget.bgIframe){
				// setting widget.bgIframe triggers cleanup in _Widget.destroy()
				widget.bgIframe = new dijit.BackgroundIframe(wrapper);
			}
		}

		// position the wrapper node and make it visible
		var best = around ?
			dijit.placeOnScreenAroundElement(wrapper, around, orient, widget.orient ? dojo.hitch(widget, "orient") : null) :
			dijit.placeOnScreen(wrapper, args, orient == 'R' ? ['TR','BR','TL','BL'] : ['TL','BL','TR','BR'], args.padding);

		wrapper.style.visibility = "visible";
		widget.domNode.style.visibility = "visible";	// counteract effects from _HasDropDown

		var handlers = [];

		// provide default escape and tab key handling
		// (this will work for any widget, not just menu)
		handlers.push(dojo.connect(wrapper, "onkeypress", this, function(evt){
			if(evt.charOrCode == dojo.keys.ESCAPE && args.onCancel){
				dojo.stopEvent(evt);
				args.onCancel();
			}else if(evt.charOrCode === dojo.keys.TAB){
				dojo.stopEvent(evt);
				var topPopup = this.getTopPopup();
				if(topPopup && topPopup.onCancel){
					topPopup.onCancel();
				}
			}
		}));

		// watch for cancel/execute events on the popup and notify the caller
		// (for a menu, "execute" means clicking an item)
		if(widget.onCancel){
			handlers.push(dojo.connect(widget, "onCancel", args.onCancel));
		}

		handlers.push(dojo.connect(widget, widget.onExecute ? "onExecute" : "onChange", this, function(){
			var topPopup = this.getTopPopup();
			if(topPopup && topPopup.onExecute){
				topPopup.onExecute();
			}
		}));

		stack.push({
			widget: widget,
			parent: args.parent,
			onExecute: args.onExecute,
			onCancel: args.onCancel,
 			onClose: args.onClose,
			handlers: handlers
		});

		if(widget.onOpen){
			// TODO: in 2.0 standardize onShow() (used by StackContainer) and onOpen() (used here)
			widget.onOpen(best);
		}

		return best;
	},

	close: function(/*dijit._Widget*/ popup){
		// summary:
		//		Close specified popup and any popups that it parented

		var stack = this._stack;

		// Basically work backwards from the top of the stack closing popups
		// until we hit the specified popup, but IIRC there was some issue where closing
		// a popup would cause others to close too.  Thus if we are trying to close B in [A,B,C]
		// closing C might close B indirectly and then the while() condition will run where stack==[A]...
		// so the while condition is constructed defensively.
		while(dojo.some(stack, function(elem){return elem.widget == popup;})){
			var top = stack.pop(),
				widget = top.widget,
				onClose = top.onClose;

			if(widget.onClose){
				// TODO: in 2.0 standardize onHide() (used by StackContainer) and onClose() (used here)
				widget.onClose();
			}
			dojo.forEach(top.handlers, dojo.disconnect);

			// Move the widget plus it's wrapper off screen, unless it has already been destroyed in above onClose() etc.
			if(widget && widget.domNode){
				this.moveOffScreen(widget);
			}
                        
			if(onClose){
				onClose();
			}
		}
	}
};

// TODO: remove dijit._frames, it isn't being used much, since popups never release their
// iframes (see [22236])
dijit._frames = new function(){
	// summary:
	//		cache of iframes

	var queue = [];

	this.pop = function(){
		var iframe;
		if(queue.length){
			iframe = queue.pop();
			iframe.style.display="";
		}else{
			if(dojo.isIE < 9){
				var burl = dojo.config["dojoBlankHtmlUrl"] || (dojo.moduleUrl("dojo", "resources/blank.html")+"") || "javascript:\"\"";
				var html="<iframe src='" + burl + "'"
					+ " style='position: absolute; left: 0px; top: 0px;"
					+ "z-index: -1; filter:Alpha(Opacity=\"0\");'>";
				iframe = dojo.doc.createElement(html);
			}else{
			 	iframe = dojo.create("iframe");
				iframe.src = 'javascript:""';
				iframe.className = "dijitBackgroundIframe";
				dojo.style(iframe, "opacity", 0.1);
			}
			iframe.tabIndex = -1; // Magic to prevent iframe from getting focus on tab keypress - as style didn't work.
			dijit.setWaiRole(iframe,"presentation");
		}
		return iframe;
	};

	this.push = function(iframe){
		iframe.style.display="none";
		queue.push(iframe);
	}
}();


dijit.BackgroundIframe = function(/* DomNode */node){
	// summary:
	//		For IE/FF z-index schenanigans. id attribute is required.
	//
	// description:
	//		new dijit.BackgroundIframe(node)
	//			Makes a background iframe as a child of node, that fills
	//			area (and position) of node

	if(!node.id){ throw new Error("no id"); }
	if(dojo.isIE || dojo.isMoz){
		var iframe = dijit._frames.pop();
		node.appendChild(iframe);
		if(dojo.isIE<7){
			this.resize(node);
			this._conn = dojo.connect(node, 'onresize', this, function(){
				this.resize(node);
			});
		}else{
			dojo.style(iframe, {
				width: '100%',
				height: '100%'
			});
		}
		this.iframe = iframe;
	}
};

dojo.extend(dijit.BackgroundIframe, {
	resize: function(node){
		// summary:
		// 		resize the iframe so its the same size as node
		// description:
		//		this function is a no-op in all browsers except
		//		IE6, which does not support 100% width/height 
		//		of absolute positioned iframes
		if(this.iframe && dojo.isIE<7){
			dojo.style(this.iframe, {
				width: node.offsetWidth + 'px',
				height: node.offsetHeight + 'px'
			});
		}
	},
	destroy: function(){
		// summary:
		//		destroy the iframe
		if(this._conn){
			dojo.disconnect(this._conn);
			this._conn = null;
		}
		if(this.iframe){
			dijit._frames.push(this.iframe);
			delete this.iframe;
		}
	}
});

}

if(!dojo._hasResource["dijit._base.scroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.scroll"] = true;
dojo.provide("dijit._base.scroll");



dijit.scrollIntoView = function(/*DomNode*/ node, /*Object?*/ pos){
	// summary:
	//		Scroll the passed node into view, if it is not already.
	//		Deprecated, use `dojo.window.scrollIntoView` instead.
	
	dojo.window.scrollIntoView(node, pos);
};

}

if(!dojo._hasResource["dojo.uacss"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.uacss"] = true;
dojo.provide("dojo.uacss");

(function(){
	// summary:
	//		Applies pre-set CSS classes to the top-level HTML node, based on:
	// 			- browser (ex: dj_ie)
	//			- browser version (ex: dj_ie6)
	//			- box model (ex: dj_contentBox)
	//			- text direction (ex: dijitRtl)
	//
	//		In addition, browser, browser version, and box model are
	//		combined with an RTL flag when browser text is RTL.  ex: dj_ie-rtl.

	var d = dojo,
		html = d.doc.documentElement,
		ie = d.isIE,
		opera = d.isOpera,
		maj = Math.floor,
		ff = d.isFF,
		boxModel = d.boxModel.replace(/-/,''),

		classes = {
			dj_ie: ie,
			dj_ie6: maj(ie) == 6,
			dj_ie7: maj(ie) == 7,
			dj_ie8: maj(ie) == 8,
			dj_quirks: d.isQuirks,
			dj_iequirks: ie && d.isQuirks,

			// NOTE: Opera not supported by dijit
			dj_opera: opera,

			dj_khtml: d.isKhtml,

			dj_webkit: d.isWebKit,
			dj_safari: d.isSafari,
			dj_chrome: d.isChrome,

			dj_gecko: d.isMozilla,
			dj_ff3: maj(ff) == 3
		}; // no dojo unsupported browsers

	classes["dj_" + boxModel] = true;

	// apply browser, browser version, and box model class names
	var classStr = "";
	for(var clz in classes){
		if(classes[clz]){
			classStr += clz + " ";
		}
	}
	html.className = d.trim(html.className + " " + classStr);

	// If RTL mode, then add dj_rtl flag plus repeat existing classes with -rtl extension.
	// We can't run the code below until the <body> tag has loaded (so we can check for dir=rtl).  
	// Unshift() is to run sniff code before the parser.
	dojo._loaders.unshift(function(){
		if(!dojo._isBodyLtr()){
			var rtlClassStr = "dj_rtl dijitRtl " + classStr.replace(/ /g, "-rtl ")
			html.className = d.trim(html.className + " " + rtlClassStr);
		}
	});
})();

}

if(!dojo._hasResource["dijit._base.sniff"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.sniff"] = true;
// summary:
//		Applies pre-set CSS classes to the top-level HTML node, see
//		`dojo.uacss` for details.
//
//		Simply doing a require on this module will
//		establish this CSS.  Modified version of Morris' CSS hack.

dojo.provide("dijit._base.sniff");



}

if(!dojo._hasResource["dijit._base.typematic"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.typematic"] = true;
dojo.provide("dijit._base.typematic");

dijit.typematic = {
	// summary:
	//		These functions are used to repetitively call a user specified callback
	//		method when a specific key or mouse click over a specific DOM node is
	//		held down for a specific amount of time.
	//		Only 1 such event is allowed to occur on the browser page at 1 time.

	_fireEventAndReload: function(){
		this._timer = null;
		this._callback(++this._count, this._node, this._evt);
		
		// Schedule next event, timer is at most minDelay (default 10ms) to avoid
		// browser overload (particularly avoiding starving DOH robot so it never gets to send a mouseup)
		this._currentTimeout = Math.max(
			this._currentTimeout < 0 ? this._initialDelay :
				(this._subsequentDelay > 1 ? this._subsequentDelay : Math.round(this._currentTimeout * this._subsequentDelay)),
			this._minDelay);
		this._timer = setTimeout(dojo.hitch(this, "_fireEventAndReload"), this._currentTimeout);
	},

	trigger: function(/*Event*/ evt, /*Object*/ _this, /*DOMNode*/ node, /*Function*/ callback, /*Object*/ obj, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start a timed, repeating callback sequence.
		//		If already started, the function call is ignored.
		//		This method is not normally called by the user but can be
		//		when the normal listener code is insufficient.
		// evt:
		//		key or mouse event object to pass to the user callback
		// _this:
		//		pointer to the user's widget space.
		// node:
		//		the DOM node object to pass the the callback function
		// callback:
		//		function to call until the sequence is stopped called with 3 parameters:
		// count:
		//		integer representing number of repeated calls (0..n) with -1 indicating the iteration has stopped
		// node:
		//		the DOM node object passed in
		// evt:
		//		key or mouse event object
		// obj:
		//		user space object used to uniquely identify each typematic sequence
		// subsequentDelay (optional):
		//		if > 1, the number of milliseconds until the 3->n events occur
		//		or else the fractional time multiplier for the next event's delay, default=0.9
		// initialDelay (optional):
		//		the number of milliseconds until the 2nd event occurs, default=500ms
		// minDelay (optional):
		//		the maximum delay in milliseconds for event to fire, default=10ms
		if(obj != this._obj){
			this.stop();
			this._initialDelay = initialDelay || 500;
			this._subsequentDelay = subsequentDelay || 0.90;
			this._minDelay = minDelay || 10;
			this._obj = obj;
			this._evt = evt;
			this._node = node;
			this._currentTimeout = -1;
			this._count = -1;
			this._callback = dojo.hitch(_this, callback);
			this._fireEventAndReload();
			this._evt = dojo.mixin({faux: true}, evt);
		}
	},

	stop: function(){
		// summary:
		//		Stop an ongoing timed, repeating callback sequence.
		if(this._timer){
			clearTimeout(this._timer);
			this._timer = null;
		}
		if(this._obj){
			this._callback(-1, this._node, this._evt);
			this._obj = null;
		}
	},

	addKeyListener: function(/*DOMNode*/ node, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key.
		//		See also the trigger method for other parameters.
		// keyObject:
		//		an object defining the key to listen for:
		// 		charOrCode:
		//			the printable character (string) or keyCode (number) to listen for.
		// 		keyCode:
		//			(deprecated - use charOrCode) the keyCode (number) to listen for (implies charCode = 0).
		// 		charCode:
		//			(deprecated - use charOrCode) the charCode (number) to listen for.
		// 		ctrlKey:
		//			desired ctrl key state to initiate the callback sequence:
		//			- pressed (true)
		//			- released (false)
		//			- either (unspecified)
		// 		altKey:
		//			same as ctrlKey but for the alt key
		// 		shiftKey:
		//			same as ctrlKey but for the shift key
		// returns:
		//		an array of dojo.connect handles
		if(keyObject.keyCode){
			keyObject.charOrCode = keyObject.keyCode;
			dojo.deprecated("keyCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}else if(keyObject.charCode){
			keyObject.charOrCode = String.fromCharCode(keyObject.charCode);
			dojo.deprecated("charCode attribute parameter for dijit.typematic.addKeyListener is deprecated. Use charOrCode instead.", "", "2.0");
		}
		return [
			dojo.connect(node, "onkeypress", this, function(evt){
				if(evt.charOrCode == keyObject.charOrCode &&
				(keyObject.ctrlKey === undefined || keyObject.ctrlKey == evt.ctrlKey) &&
				(keyObject.altKey === undefined || keyObject.altKey == evt.altKey) &&
				(keyObject.metaKey === undefined || keyObject.metaKey == (evt.metaKey || false)) && // IE doesn't even set metaKey
				(keyObject.shiftKey === undefined || keyObject.shiftKey == evt.shiftKey)){
					dojo.stopEvent(evt);
					dijit.typematic.trigger(evt, _this, node, callback, keyObject, subsequentDelay, initialDelay, minDelay);
				}else if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			}),
			dojo.connect(node, "onkeyup", this, function(evt){
				if(dijit.typematic._obj == keyObject){
					dijit.typematic.stop();
				}
			})
		];
	},

	addMouseListener: function(/*DOMNode*/ node, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a typematic mouse click.
		//		See the trigger method for other parameters.
		// returns:
		//		an array of dojo.connect handles
		var dc = dojo.connect;
		return [
			dc(node, "mousedown", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
			}),
			dc(node, "mouseup", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mouseout", this, function(evt){
				dojo.stopEvent(evt);
				dijit.typematic.stop();
			}),
			dc(node, "mousemove", this, function(evt){
				evt.preventDefault();
			}),
			dc(node, "dblclick", this, function(evt){
				dojo.stopEvent(evt);
				if(dojo.isIE){
					dijit.typematic.trigger(evt, _this, node, callback, node, subsequentDelay, initialDelay, minDelay);
					setTimeout(dojo.hitch(this, dijit.typematic.stop), 50);
				}
			})
		];
	},

	addListener: function(/*Node*/ mouseNode, /*Node*/ keyNode, /*Object*/ keyObject, /*Object*/ _this, /*Function*/ callback, /*Number*/ subsequentDelay, /*Number*/ initialDelay, /*Number?*/ minDelay){
		// summary:
		//		Start listening for a specific typematic key and mouseclick.
		//		This is a thin wrapper to addKeyListener and addMouseListener.
		//		See the addMouseListener and addKeyListener methods for other parameters.
		// mouseNode:
		//		the DOM node object to listen on for mouse events.
		// keyNode:
		//		the DOM node object to listen on for key events.
		// returns:
		//		an array of dojo.connect handles
		return this.addKeyListener(keyNode, keyObject, _this, callback, subsequentDelay, initialDelay, minDelay).concat(
			this.addMouseListener(mouseNode, _this, callback, subsequentDelay, initialDelay, minDelay));
	}
};

}

if(!dojo._hasResource["dijit._base.wai"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base.wai"] = true;
dojo.provide("dijit._base.wai");

dijit.wai = {
	onload: function(){
		// summary:
		//		Detects if we are in high-contrast mode or not

		// This must be a named function and not an anonymous
		// function, so that the widget parsing code can make sure it
		// registers its onload function after this function.
		// DO NOT USE "this" within this function.

		// create div for testing if high contrast mode is on or images are turned off
		var div = dojo.create("div",{
			id: "a11yTestNode",
			style:{
				cssText:'border: 1px solid;'
					+ 'border-color:red green;'
					+ 'position: absolute;'
					+ 'height: 5px;'
					+ 'top: -999px;'
					+ 'background-image: url("' + (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")) + '");'
			}
		}, dojo.body());

		// test it
		var cs = dojo.getComputedStyle(div);
		if(cs){
			var bkImg = cs.backgroundImage;
			var needsA11y = (cs.borderTopColor == cs.borderRightColor) || (bkImg != null && (bkImg == "none" || bkImg == "url(invalid-url:)" ));
			dojo[needsA11y ? "addClass" : "removeClass"](dojo.body(), "dijit_a11y");
			if(dojo.isIE){
				div.outerHTML = "";		// prevent mixed-content warning, see http://support.microsoft.com/kb/925014
			}else{
				dojo.body().removeChild(div);
			}
		}
	}
};

// Test if computer is in high contrast mode.
// Make sure the a11y test runs first, before widgets are instantiated.
if(dojo.isIE || dojo.isMoz){	// NOTE: checking in Safari messes things up
	dojo._loaders.unshift(dijit.wai.onload);
}

dojo.mixin(dijit, {
	_XhtmlRoles: /banner|contentinfo|definition|main|navigation|search|note|secondary|seealso/,

	hasWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Determines if an element has a particular non-XHTML role.
		// returns:
		//		True if elem has the specific non-XHTML role attribute and false if not.
		// 		For backwards compatibility if role parameter not provided,
		// 		returns true if has non XHTML role
		var waiRole = this.getWaiRole(elem);
		return role ? (waiRole.indexOf(role) > -1) : (waiRole.length > 0);
	},

	getWaiRole: function(/*Element*/ elem){
		// summary:
		//		Gets the non-XHTML role for an element (which should be a wai role).
		// returns:
		//		The non-XHTML role of elem or an empty string if elem
		//		does not have a role.
		 return dojo.trim((dojo.attr(elem, "role") || "").replace(this._XhtmlRoles,"").replace("wairole:",""));
	},

	setWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Sets the role on an element.
		// description:
		//		Replace existing role attribute with new role.
		//		If elem already has an XHTML role, append this role to XHTML role
		//		and remove other ARIA roles.

		var curRole = dojo.attr(elem, "role") || "";
		if(!this._XhtmlRoles.test(curRole)){
			dojo.attr(elem, "role", role);
		}else{
			if((" "+ curRole +" ").indexOf(" " + role + " ") < 0){
				var clearXhtml = dojo.trim(curRole.replace(this._XhtmlRoles, ""));
				var cleanRole = dojo.trim(curRole.replace(clearXhtml, ""));
				dojo.attr(elem, "role", cleanRole + (cleanRole ? ' ' : '') + role);
			}
		}
	},

	removeWaiRole: function(/*Element*/ elem, /*String*/ role){
		// summary:
		//		Removes the specified non-XHTML role from an element.
		// 		Removes role attribute if no specific role provided (for backwards compat.)

		var roleValue = dojo.attr(elem, "role");
		if(!roleValue){ return; }
		if(role){
			var t = dojo.trim((" " + roleValue + " ").replace(" " + role + " ", " "));
			dojo.attr(elem, "role", t);
		}else{
			elem.removeAttribute("role");
		}
	},

	hasWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Determines if an element has a given state.
		// description:
		//		Checks for an attribute called "aria-"+state.
		// returns:
		//		true if elem has a value for the given state and
		//		false if it does not.

		return elem.hasAttribute ? elem.hasAttribute("aria-"+state) : !!elem.getAttribute("aria-"+state);
	},

	getWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Gets the value of a state on an element.
		// description:
		//		Checks for an attribute called "aria-"+state.
		// returns:
		//		The value of the requested state on elem
		//		or an empty string if elem has no value for state.

		return elem.getAttribute("aria-"+state) || "";
	},

	setWaiState: function(/*Element*/ elem, /*String*/ state, /*String*/ value){
		// summary:
		//		Sets a state on an element.
		// description:
		//		Sets an attribute called "aria-"+state.

		elem.setAttribute("aria-"+state, value);
	},

	removeWaiState: function(/*Element*/ elem, /*String*/ state){
		// summary:
		//		Removes a state from an element.
		// description:
		//		Sets an attribute called "aria-"+state.

		elem.removeAttribute("aria-"+state);
	}
});

}

if(!dojo._hasResource["dijit._base"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._base"] = true;
dojo.provide("dijit._base");











}

if(!dojo._hasResource["dijit._Widget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Widget"] = true;
dojo.provide("dijit._Widget");

dojo.require( "dijit._base" );


// This code is to assist deferring dojo.connect() calls in widgets (connecting to events on the widgets'
// DOM nodes) until someone actually needs to monitor that event.
dojo.connect(dojo, "_connect",
	function(/*dijit._Widget*/ widget, /*String*/ event){
		if(widget && dojo.isFunction(widget._onConnect)){
			widget._onConnect(event);
		}
	});

dijit._connectOnUseEventHandler = function(/*Event*/ event){};

// Keep track of where the last keydown event was, to help avoid generating
// spurious ondijitclick events when:
// 1. focus is on a <button> or <a>
// 2. user presses then releases the ENTER key
// 3. onclick handler fires and shifts focus to another node, with an ondijitclick handler
// 4. onkeyup event fires, causing the ondijitclick handler to fire
dijit._lastKeyDownNode = null;
if(dojo.isIE){
	(function(){
		var keydownCallback = function(evt){
			dijit._lastKeyDownNode = evt.srcElement;
		};
		dojo.doc.attachEvent('onkeydown', keydownCallback);
		dojo.addOnWindowUnload(function(){
			dojo.doc.detachEvent('onkeydown', keydownCallback);
		});
	})();
}else{
	dojo.doc.addEventListener('keydown', function(evt){
		dijit._lastKeyDownNode = evt.target;
	}, true);
}

(function(){

var _attrReg = {},	// cached results from getSetterAttributes
	getSetterAttributes = function(widget){
		// summary:
		//		Returns list of attributes with custom setters for specified widget
		var dc = widget.declaredClass;
		if(!_attrReg[dc]){
			var r = [],
				attrs,
				proto = widget.constructor.prototype;
			for(var fxName in proto){
				if(dojo.isFunction(proto[fxName]) && (attrs = fxName.match(/^_set([a-zA-Z]*)Attr$/)) && attrs[1]){
					r.push(attrs[1].charAt(0).toLowerCase() + attrs[1].substr(1));
				}
			}
			_attrReg[dc] = r;
		}
		return _attrReg[dc] || [];	// String[]
	};

dojo.declare("dijit._Widget", null, {
	// summary:
	//		Base class for all Dijit widgets.

	// id: [const] String
	//		A unique, opaque ID string that can be assigned by users or by the
	//		system. If the developer passes an ID which is known not to be
	//		unique, the specified ID is ignored and the system-generated ID is
	//		used instead.
	id: "",

	// lang: [const] String
	//		Rarely used.  Overrides the default Dojo locale used to render this widget,
	//		as defined by the [HTML LANG](http://www.w3.org/TR/html401/struct/dirlang.html#adef-lang) attribute.
	//		Value must be among the list of locales specified during by the Dojo bootstrap,
	//		formatted according to [RFC 3066](http://www.ietf.org/rfc/rfc3066.txt) (like en-us).
	lang: "",

	// dir: [const] String
	//		Bi-directional support, as defined by the [HTML DIR](http://www.w3.org/TR/html401/struct/dirlang.html#adef-dir)
	//		attribute. Either left-to-right "ltr" or right-to-left "rtl".  If undefined, widgets renders in page's
	//		default direction.
	dir: "",

	// class: String
	//		HTML class attribute
	"class": "",

	// style: String||Object
	//		HTML style attributes as cssText string or name/value hash
	style: "",

	// title: String
	//		HTML title attribute.
	//
	//		For form widgets this specifies a tooltip to display when hovering over
	//		the widget (just like the native HTML title attribute).
	//
	//		For TitlePane or for when this widget is a child of a TabContainer, AccordionContainer,
	//		etc., it's used to specify the tab label, accordion pane title, etc.
	title: "",

	// tooltip: String
	//		When this widget's title attribute is used to for a tab label, accordion pane title, etc.,
	//		this specifies the tooltip to appear when the mouse is hovered over that text.
	tooltip: "",

	// baseClass: [protected] String
	//		Root CSS class of the widget (ex: dijitTextBox), used to construct CSS classes to indicate
	//		widget state.
	baseClass: "",

	// srcNodeRef: [readonly] DomNode
	//		pointer to original DOM node
	srcNodeRef: null,

	// domNode: [readonly] DomNode
	//		This is our visible representation of the widget! Other DOM
	//		Nodes may by assigned to other properties, usually through the
	//		template system's dojoAttachPoint syntax, but the domNode
	//		property is the canonical "top level" node in widget UI.
	domNode: null,

	// containerNode: [readonly] DomNode
	//		Designates where children of the source DOM node will be placed.
	//		"Children" in this case refers to both DOM nodes and widgets.
	//		For example, for myWidget:
	//
	//		|	<div dojoType=myWidget>
	//		|		<b> here's a plain DOM node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//		|	</div>
	//
	//		containerNode would point to:
	//
	//		|		<b> here's a plain DOM node
	//		|		<span dojoType=subWidget>and a widget</span>
	//		|		<i> and another plain DOM node </i>
	//
	//		In templated widgets, "containerNode" is set via a
	//		dojoAttachPoint assignment.
	//
	//		containerNode must be defined for any widget that accepts innerHTML
	//		(like ContentPane or BorderContainer or even Button), and conversely
	//		is null for widgets that don't, like TextBox.
	containerNode: null,

/*=====
	// _started: Boolean
	//		startup() has completed.
	_started: false,
=====*/

	// attributeMap: [protected] Object
	//		attributeMap sets up a "binding" between attributes (aka properties)
	//		of the widget and the widget's DOM.
	//		Changes to widget attributes listed in attributeMap will be
	//		reflected into the DOM.
	//
	//		For example, calling attr('title', 'hello')
	//		on a TitlePane will automatically cause the TitlePane's DOM to update
	//		with the new title.
	//
	//		attributeMap is a hash where the key is an attribute of the widget,
	//		and the value reflects a binding to a:
	//
	//		- DOM node attribute
	// |		focus: {node: "focusNode", type: "attribute"}
	// 		Maps this.focus to this.focusNode.focus
	//
	//		- DOM node innerHTML
	//	|		title: { node: "titleNode", type: "innerHTML" }
	//		Maps this.title to this.titleNode.innerHTML
	//
	//		- DOM node innerText
	//	|		title: { node: "titleNode", type: "innerText" }
	//		Maps this.title to this.titleNode.innerText
	//
	//		- DOM node CSS class
	// |		myClass: { node: "domNode", type: "class" }
	//		Maps this.myClass to this.domNode.className
	//
	//		If the value is an array, then each element in the array matches one of the
	//		formats of the above list.
	//
	//		There are also some shorthands for backwards compatibility:
	//		- string --> { node: string, type: "attribute" }, for example:
	//	|	"focusNode" ---> { node: "focusNode", type: "attribute" }
	//		- "" --> { node: "domNode", type: "attribute" }
	attributeMap: {id:"", dir:"", lang:"", "class":"", style:"", title:""},

	// _deferredConnects: [protected] Object
	//		attributeMap addendum for event handlers that should be connected only on first use
	_deferredConnects: {
		onClick: "",
		onDblClick: "",
		onKeyDown: "",
		onKeyPress: "",
		onKeyUp: "",
		onMouseMove: "",
		onMouseDown: "",
		onMouseOut: "",
		onMouseOver: "",
		onMouseLeave: "",
		onMouseEnter: "",
		onMouseUp: ""
	},

	onClick: dijit._connectOnUseEventHandler,
	/*=====
	onClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onDblClick: dijit._connectOnUseEventHandler,
	/*=====
	onDblClick: function(event){
		// summary:
		//		Connect to this function to receive notifications of mouse double click events.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onKeyDown: dijit._connectOnUseEventHandler,
	/*=====
	onKeyDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being pressed down.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyPress: dijit._connectOnUseEventHandler,
	/*=====
	onKeyPress: function(event){
		// summary:
		//		Connect to this function to receive notifications of printable keys being typed.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onKeyUp: dijit._connectOnUseEventHandler,
	/*=====
	onKeyUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of keys being released.
		// event:
		//		key Event
		// tags:
		//		callback
	},
	=====*/
	onMouseDown: dijit._connectOnUseEventHandler,
	/*=====
	onMouseDown: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is pressed down.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseMove: dijit._connectOnUseEventHandler,
	/*=====
	onMouseMove: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves over nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOut: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOut: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseOver: dijit._connectOnUseEventHandler,
	/*=====
	onMouseOver: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto nodes contained within this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseLeave: dijit._connectOnUseEventHandler,
	/*=====
	onMouseLeave: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves off of this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseEnter: dijit._connectOnUseEventHandler,
	/*=====
	onMouseEnter: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse moves onto this widget.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/
	onMouseUp: dijit._connectOnUseEventHandler,
	/*=====
	onMouseUp: function(event){
		// summary:
		//		Connect to this function to receive notifications of when the mouse button is released.
		// event:
		//		mouse Event
		// tags:
		//		callback
	},
	=====*/

	// Constants used in templates

	// _blankGif: [protected] String
	//		Path to a blank 1x1 image.
	//		Used by <img> nodes in templates that really get their image via CSS background-image.
	_blankGif: (dojo.config.blankGif || dojo.moduleUrl("dojo", "resources/blank.gif")).toString(),

	//////////// INITIALIZATION METHODS ///////////////////////////////////////

	postscript: function(/*Object?*/params, /*DomNode|String*/srcNodeRef){
		// summary:
		//		Kicks off widget instantiation.  See create() for details.
		// tags:
		//		private
		this.create(params, srcNodeRef);
	},

	create: function(/*Object?*/params, /*DomNode|String?*/srcNodeRef){
		// summary:
		//		Kick off the life-cycle of a widget
		// params:
		//		Hash of initialization parameters for widget, including
		//		scalar values (like title, duration etc.) and functions,
		//		typically callbacks like onClick.
		// srcNodeRef:
		//		If a srcNodeRef (DOM node) is specified:
		//			- use srcNodeRef.innerHTML as my contents
		//			- if this is a behavioral widget then apply behavior
		//			  to that srcNodeRef
		//			- otherwise, replace srcNodeRef with my generated DOM
		//			  tree
		// description:
		//		Create calls a number of widget methods (postMixInProperties, buildRendering, postCreate,
		//		etc.), some of which of you'll want to override. See http://docs.dojocampus.org/dijit/_Widget
		//		for a discussion of the widget creation lifecycle.
		//
		//		Of course, adventurous developers could override create entirely, but this should
		//		only be done as a last resort.
		// tags:
		//		private

		// store pointer to original DOM tree
		this.srcNodeRef = dojo.byId(srcNodeRef);

		// For garbage collection.  An array of handles returned by Widget.connect()
		// Each handle returned from Widget.connect() is an array of handles from dojo.connect()
		this._connects = [];

		// For garbage collection.  An array of handles returned by Widget.subscribe()
		// The handle returned from Widget.subscribe() is the handle returned from dojo.subscribe()
		this._subscribes = [];

		// To avoid double-connects, remove entries from _deferredConnects
		// that have been setup manually by a subclass (ex, by dojoAttachEvent).
		// If a subclass has redefined a callback (ex: onClick) then assume it's being
		// connected to manually.
		this._deferredConnects = dojo.clone(this._deferredConnects);
		for(var attr in this.attributeMap){
			delete this._deferredConnects[attr]; // can't be in both attributeMap and _deferredConnects
		}
		for(attr in this._deferredConnects){
			if(this[attr] !== dijit._connectOnUseEventHandler){
				delete this._deferredConnects[attr];	// redefined, probably dojoAttachEvent exists
			}
		}

		//mixin our passed parameters
		if(this.srcNodeRef && (typeof this.srcNodeRef.id == "string")){ this.id = this.srcNodeRef.id; }
		if(params){
			this.params = params;
			dojo.mixin(this,params);
		}
		this.postMixInProperties();

		// generate an id for the widget if one wasn't specified
		// (be sure to do this before buildRendering() because that function might
		// expect the id to be there.)
		if(!this.id){
			this.id = dijit.getUniqueId(this.declaredClass.replace(/\./g,"_"));
		}
		dijit.registry.add(this);

		this.buildRendering();

		if(this.domNode){
			// Copy attributes listed in attributeMap into the [newly created] DOM for the widget.
			this._applyAttributes();

			var source = this.srcNodeRef;
			if(source && source.parentNode){
				source.parentNode.replaceChild(this.domNode, source);
			}

			// If the developer has specified a handler as a widget parameter
			// (ex: new Button({onClick: ...})
			// then naturally need to connect from DOM node to that handler immediately,
			for(attr in this.params){
				this._onConnect(attr);
			}
		}

		if(this.domNode){
			this.domNode.setAttribute("widgetId", this.id);
		}
		this.postCreate();

		// If srcNodeRef has been processed and removed from the DOM (e.g. TemplatedWidget) then delete it to allow GC.
		if(this.srcNodeRef && !this.srcNodeRef.parentNode){
			delete this.srcNodeRef;
		}

		this._created = true;
	},

	_applyAttributes: function(){
		// summary:
		//		Step during widget creation to copy all widget attributes to the
		//		DOM as per attributeMap and _setXXXAttr functions.
		// description:
		//		Skips over blank/false attribute values, unless they were explicitly specified
		//		as parameters to the widget, since those are the default anyway,
		//		and setting tabIndex="" is different than not setting tabIndex at all.
		//
		//		It processes the attributes in the attribute map first, and then
		//		it goes through and processes the attributes for the _setXXXAttr
		//		functions that have been specified
		// tags:
		//		private
		var condAttrApply = function(attr, scope){
			if((scope.params && attr in scope.params) || scope[attr]){
				scope.set(attr, scope[attr]);
			}
		};

		// Do the attributes in attributeMap
		for(var attr in this.attributeMap){
			condAttrApply(attr, this);
		}

		// And also any attributes with custom setters
		dojo.forEach(getSetterAttributes(this), function(a){
			if(!(a in this.attributeMap)){
				condAttrApply(a, this);
			}
		}, this);
	},

	postMixInProperties: function(){
		// summary:
		//		Called after the parameters to the widget have been read-in,
		//		but before the widget template is instantiated. Especially
		//		useful to set properties that are referenced in the widget
		//		template.
		// tags:
		//		protected
	},

	buildRendering: function(){
		// summary:
		//		Construct the UI for this widget, setting this.domNode
		// description:
		//		Most widgets will mixin `dijit._Templated`, which implements this
		//		method.
		// tags:
		//		protected
		this.domNode = this.srcNodeRef || dojo.create('div');
	},

	postCreate: function(){
		// summary:
		//		Processing after the DOM fragment is created
		// description:
		//		Called after the DOM fragment has been created, but not necessarily
		//		added to the document.  Do not include any operations which rely on
		//		node dimensions or placement.
		// tags:
		//		protected

		// baseClass is a single class name or occasionally a space-separated list of names.
		// Add those classes to the DOMNod.  If RTL mode then also add with Rtl suffix.		
		if(this.baseClass){
			var classes = this.baseClass.split(" ");
			if(!this.isLeftToRight()){
				classes = classes.concat( dojo.map(classes, function(name){ return name+"Rtl"; }));
			}
			dojo.addClass(this.domNode, classes);
		}
	},

	startup: function(){
		// summary:
		//		Processing after the DOM fragment is added to the document
		// description:
		//		Called after a widget and its children have been created and added to the page,
		//		and all related widgets have finished their create() cycle, up through postCreate().
		//		This is useful for composite widgets that need to control or layout sub-widgets.
		//		Many layout widgets can use this as a wiring phase.
		this._started = true;
	},

	//////////// DESTROY FUNCTIONS ////////////////////////////////

	destroyRecursive: function(/*Boolean?*/ preserveDom){
		// summary:
		// 		Destroy this widget and its descendants
		// description:
		//		This is the generic "destructor" function that all widget users
		// 		should call to cleanly discard with a widget. Once a widget is
		// 		destroyed, it is removed from the manager object.
		// preserveDom:
		//		If true, this method will leave the original DOM structure
		//		alone of descendant Widgets. Note: This will NOT work with
		//		dijit._Templated widgets.

		this._beingDestroyed = true;
		this.destroyDescendants(preserveDom);
		this.destroy(preserveDom);
	},

	destroy: function(/*Boolean*/ preserveDom){
		// summary:
		// 		Destroy this widget, but not its descendants.
		//		This method will, however, destroy internal widgets such as those used within a template.
		// preserveDom: Boolean
		//		If true, this method will leave the original DOM structure alone.
		//		Note: This will not yet work with _Templated widgets

		this._beingDestroyed = true;
		this.uninitialize();
		var d = dojo,
			dfe = d.forEach,
			dun = d.unsubscribe;
		dfe(this._connects, function(array){
			dfe(array, d.disconnect);
		});
		dfe(this._subscribes, function(handle){
			dun(handle);
		});

		// destroy widgets created as part of template, etc.
		dfe(this._supportingWidgets || [], function(w){
			if(w.destroyRecursive){
				w.destroyRecursive();
			}else if(w.destroy){
				w.destroy();
			}
		});

		this.destroyRendering(preserveDom);
		dijit.registry.remove(this.id);
		this._destroyed = true;
	},

	destroyRendering: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Destroys the DOM nodes associated with this widget
		// preserveDom:
		//		If true, this method will leave the original DOM structure alone
		//		during tear-down. Note: this will not work with _Templated
		//		widgets yet.
		// tags:
		//		protected

		if(this.bgIframe){
			this.bgIframe.destroy(preserveDom);
			delete this.bgIframe;
		}

		if(this.domNode){
			if(preserveDom){
				dojo.removeAttr(this.domNode, "widgetId");
			}else{
				dojo.destroy(this.domNode);
			}
			delete this.domNode;
		}

		if(this.srcNodeRef){
			if(!preserveDom){
				dojo.destroy(this.srcNodeRef);
			}
			delete this.srcNodeRef;
		}
	},

	destroyDescendants: function(/*Boolean?*/ preserveDom){
		// summary:
		//		Recursively destroy the children of this widget and their
		//		descendants.
		// preserveDom:
		//		If true, the preserveDom attribute is passed to all descendant
		//		widget's .destroy() method. Not for use with _Templated
		//		widgets.

		// get all direct descendants and destroy them recursively
		dojo.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive(preserveDom);
			}
		});
	},


	uninitialize: function(){
		// summary:
		//		Stub function. Override to implement custom widget tear-down
		//		behavior.
		// tags:
		//		protected
		return false;
	},

	////////////////// MISCELLANEOUS METHODS ///////////////////

	onFocus: function(){
		// summary:
		//		Called when the widget becomes "active" because
		//		it or a widget inside of it either has focus, or has recently
		//		been clicked.
		// tags:
		//		callback
	},

	onBlur: function(){
		// summary:
		//		Called when the widget stops being "active" because
		//		focus moved to something outside of it, or the user
		//		clicked somewhere outside of it, or the widget was
		//		hidden.
		// tags:
		//		callback
	},

	_onFocus: function(e){
		// summary:
		//		This is where widgets do processing for when they are active,
		//		such as changing CSS classes.  See onFocus() for more details.
		// tags:
		//		protected
		this.onFocus();
	},

	_onBlur: function(){
		// summary:
		//		This is where widgets do processing for when they stop being active,
		//		such as changing CSS classes.  See onBlur() for more details.
		// tags:
		//		protected
		this.onBlur();
	},

	_onConnect: function(/*String*/ event){
		// summary:
		//		Called when someone connects to one of my handlers.
		//		"Turn on" that handler if it isn't active yet.
		//
		//		This is also called for every single initialization parameter
		//		so need to do nothing for parameters like "id".
		// tags:
		//		private
		if(event in this._deferredConnects){
			var mapNode = this[this._deferredConnects[event] || 'domNode'];
			this.connect(mapNode, event.toLowerCase(), event);
			delete this._deferredConnects[event];
		}
	},

	_setClassAttr: function(/*String*/ value){
		// summary:
		//		Custom setter for the CSS "class" attribute
		// tags:
		//		protected
		var mapNode = this[this.attributeMap["class"] || 'domNode'];
		dojo.removeClass(mapNode, this["class"])
		this["class"] = value;
		dojo.addClass(mapNode, value);
	},

	_setStyleAttr: function(/*String||Object*/ value){
		// summary:
		//		Sets the style attribut of the widget according to value,
		//		which is either a hash like {height: "5px", width: "3px"}
		//		or a plain string
		// description:
		//		Determines which node to set the style on based on style setting
		//		in attributeMap.
		// tags:
		//		protected

		var mapNode = this[this.attributeMap.style || 'domNode'];

		// Note: technically we should revert any style setting made in a previous call
		// to his method, but that's difficult to keep track of.

		if(dojo.isObject(value)){
			dojo.style(mapNode, value);
		}else{
			if(mapNode.style.cssText){
				mapNode.style.cssText += "; " + value;
			}else{
				mapNode.style.cssText = value;
			}
		}

		this.style = value;
	},

	setAttribute: function(/*String*/ attr, /*anything*/ value){
		// summary:
		//		Deprecated.  Use set() instead.
		// tags:
		//		deprecated
		dojo.deprecated(this.declaredClass+"::setAttribute(attr, value) is deprecated. Use set() instead.", "", "2.0");
		this.set(attr, value);
	},

	_attrToDom: function(/*String*/ attr, /*String*/ value){
		// summary:
		//		Reflect a widget attribute (title, tabIndex, duration etc.) to
		//		the widget DOM, as specified in attributeMap.
		//
		// description:
		//		Also sets this["attr"] to the new value.
		//		Note some attributes like "type"
		//		cannot be processed this way as they are not mutable.
		//
		// tags:
		//		private

		var commands = this.attributeMap[attr];
		dojo.forEach(dojo.isArray(commands) ? commands : [commands], function(command){

			// Get target node and what we are doing to that node
			var mapNode = this[command.node || command || "domNode"];	// DOM node
			var type = command.type || "attribute";	// class, innerHTML, innerText, or attribute

			switch(type){
				case "attribute":
					if(dojo.isFunction(value)){ // functions execute in the context of the widget
						value = dojo.hitch(this, value);
					}

					// Get the name of the DOM node attribute; usually it's the same
					// as the name of the attribute in the widget (attr), but can be overridden.
					// Also maps handler names to lowercase, like onSubmit --> onsubmit
					var attrName = command.attribute ? command.attribute :
						(/^on[A-Z][a-zA-Z]*$/.test(attr) ? attr.toLowerCase() : attr);

					dojo.attr(mapNode, attrName, value);
					break;
				case "innerText":
					mapNode.innerHTML = "";
					mapNode.appendChild(dojo.doc.createTextNode(value));
					break;
				case "innerHTML":
					mapNode.innerHTML = value;
					break;
				case "class":
					dojo.removeClass(mapNode, this[attr]);
					dojo.addClass(mapNode, value);
					break;
			}
		}, this);
		this[attr] = value;
	},

	attr: function(/*String|Object*/name, /*Object?*/value){
		// summary:
		//		Set or get properties on a widget instance.
		//	name:
		//		The property to get or set. If an object is passed here and not
		//		a string, its keys are used as names of attributes to be set
		//		and the value of the object as values to set in the widget.
		//	value:
		//		Optional. If provided, attr() operates as a setter. If omitted,
		//		the current value of the named property is returned.
		// description:
		//		This method is deprecated, use get() or set() directly.

		// Print deprecation warning but only once per calling function
		if(dojo.config.isDebug){
			var alreadyCalledHash = arguments.callee._ach || (arguments.callee._ach = {}),
				caller = (arguments.callee.caller || "unknown caller").toString();
			if(!alreadyCalledHash[caller]){
				dojo.deprecated(this.declaredClass + "::attr() is deprecated. Use get() or set() instead, called from " +
				caller, "", "2.0");
				alreadyCalledHash[caller] = true;
			}
		}

		var args = arguments.length;
		if(args >= 2 || typeof name === "object"){ // setter
			return this.set.apply(this, arguments);
		}else{ // getter
			return this.get(name);
		}
	},
	
	get: function(name){
		// summary:
		//		Get a property from a widget.
		//	name:
		//		The property to get.
		// description:
		//		Get a named property from a widget. The property may
		//		potentially be retrieved via a getter method. If no getter is defined, this
		// 		just retrieves the object's property.  
		// 		For example, if the widget has a properties "foo"
		//		and "bar" and a method named "_getFooAttr", calling:
		//	|	myWidget.get("foo");
		//		would be equivalent to writing:
		//	|	widget._getFooAttr();
		//		and:
		//	|	myWidget.get("bar");
		//		would be equivalent to writing:
		//	|	widget.bar;
		var names = this._getAttrNames(name);
		return this[names.g] ? this[names.g]() : this[name];
	},
	
	set: function(name, value){
		// summary:
		//		Set a property on a widget
		//	name:
		//		The property to set. 
		//	value:
		//		The value to set in the property.
		// description:
		//		Sets named properties on a widget which may potentially be handled by a 
		// 		setter in the widget.
		// 		For example, if the widget has a properties "foo"
		//		and "bar" and a method named "_setFooAttr", calling:
		//	|	myWidget.set("foo", "Howdy!");
		//		would be equivalent to writing:
		//	|	widget._setFooAttr("Howdy!");
		//		and:
		//	|	myWidget.set("bar", 3);
		//		would be equivalent to writing:
		//	|	widget.bar = 3;
		//
		//	set() may also be called with a hash of name/value pairs, ex:
		//	|	myWidget.set({
		//	|		foo: "Howdy",
		//	|		bar: 3
		//	|	})
		//	This is equivalent to calling set(foo, "Howdy") and set(bar, 3)

		if(typeof name === "object"){
			for(var x in name){
				this.set(x, name[x]); 
			}
			return this;
		}
		var names = this._getAttrNames(name);
		if(this[names.s]){
			// use the explicit setter
			var result = this[names.s].apply(this, Array.prototype.slice.call(arguments, 1));
		}else{
			// if param is specified as DOM node attribute, copy it
			if(name in this.attributeMap){
				this._attrToDom(name, value);
			}
			var oldValue = this[name];
			// FIXME: what about function assignments? Any way to connect() here?
			this[name] = value;
		}
		return result || this;
	},
	
	_attrPairNames: {},		// shared between all widgets
	_getAttrNames: function(name){
		// summary:
		//		Helper function for get() and set().
		//		Caches attribute name values so we don't do the string ops every time.
		// tags:
		//		private

		var apn = this._attrPairNames;
		if(apn[name]){ return apn[name]; }
		var uc = name.charAt(0).toUpperCase() + name.substr(1);
		return (apn[name] = {
			n: name+"Node",
			s: "_set"+uc+"Attr",
			g: "_get"+uc+"Attr"
		});
	},

	toString: function(){
		// summary:
		//		Returns a string that represents the widget
		// description:
		//		When a widget is cast to a string, this method will be used to generate the
		//		output. Currently, it does not implement any sort of reversible
		//		serialization.
		return '[Widget ' + this.declaredClass + ', ' + (this.id || 'NO ID') + ']'; // String
	},

	getDescendants: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		This method should generally be avoided as it returns widgets declared in templates, which are
		//		supposed to be internal/hidden, but it's left here for back-compat reasons.

		return this.containerNode ? dojo.query('[widgetId]', this.containerNode).map(dijit.byNode) : []; // dijit._Widget[]
	},

	getChildren: function(){
		// summary:
		//		Returns all the widgets contained by this, i.e., all widgets underneath this.containerNode.
		//		Does not return nested widgets, nor widgets that are part of this widget's template.
		return this.containerNode ? dijit.findWidgets(this.containerNode) : []; // dijit._Widget[]
	},

	// nodesWithKeyClick: [private] String[]
	//		List of nodes that correctly handle click events via native browser support,
	//		and don't need dijit's help
	nodesWithKeyClick: ["input", "button"],

	connect: function(
			/*Object|null*/ obj,
			/*String|Function*/ event,
			/*String|Function*/ method){
		// summary:
		//		Connects specified obj/event to specified method of this object
		//		and registers for disconnect() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.connect, except with the
		//		implicit use of this widget as the target object.
		//		This version of connect also provides a special "ondijitclick"
		//		event which triggers on a click or space or enter keyup
		// returns:
		//		A handle that can be passed to `disconnect` in order to disconnect before
		//		the widget is destroyed.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when foo.bar() is called, call the listener we're going to
		//	|	// provide in the scope of btn
		//	|	btn.connect(foo, "bar", function(){
		//	|		console.debug(this.toString());
		//	|	});
		// tags:
		//		protected

		var d = dojo,
			dc = d._connect,
			handles = [];
		if(event == "ondijitclick"){
			// add key based click activation for unsupported nodes.
			// do all processing onkey up to prevent spurious clicks
			// for details see comments at top of this file where _lastKeyDownNode is defined
			if(dojo.indexOf(this.nodesWithKeyClick, obj.nodeName.toLowerCase()) == -1){ // is NOT input or button
				var m = d.hitch(this, method);
				handles.push(
					dc(obj, "onkeydown", this, function(e){
						//console.log(this.id + ": onkeydown, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if((e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
							// needed on IE for when focus changes between keydown and keyup - otherwise dropdown menus do not work
							dijit._lastKeyDownNode = e.target;
							e.preventDefault();		// stop event to prevent scrolling on space key in IE
						}
			 		}),
					dc(obj, "onkeyup", this, function(e){
						//console.log(this.id + ": onkeyup, e.target = ", e.target, ", lastKeyDownNode was ", dijit._lastKeyDownNode, ", equality is ", (e.target === dijit._lastKeyDownNode));
						if( (e.keyCode == d.keys.ENTER || e.keyCode == d.keys.SPACE) &&
							e.target === dijit._lastKeyDownNode &&
							!e.ctrlKey && !e.shiftKey && !e.altKey && !e.metaKey){
								//need reset here or have problems in FF when focus returns to trigger element after closing popup/alert
								dijit._lastKeyDownNode = null;
								return m(e);
						}
					})
				);
			}
			event = "onclick";
		}
		handles.push(dc(obj, event, this, method));

		this._connects.push(handles);
		return handles;		// _Widget.Handle
	},

	disconnect: function(/* _Widget.Handle */ handles){
		// summary:
		//		Disconnects handle created by `connect`.
		//		Also removes handle from this widget's list of connects.
		// tags:
		//		protected
		for(var i=0; i<this._connects.length; i++){
			if(this._connects[i] == handles){
				dojo.forEach(handles, dojo.disconnect);
				this._connects.splice(i, 1);
				return;
			}
		}
	},

	subscribe: function(
			/*String*/ topic,
			/*String|Function*/ method){
		// summary:
		//		Subscribes to the specified topic and calls the specified method
		//		of this object and registers for unsubscribe() on widget destroy.
		// description:
		//		Provide widget-specific analog to dojo.subscribe, except with the
		//		implicit use of this widget as the target object.
		// example:
		//	|	var btn = new dijit.form.Button();
		//	|	// when /my/topic is published, this button changes its label to
		//	|   // be the parameter of the topic.
		//	|	btn.subscribe("/my/topic", function(v){
		//	|		this.set("label", v);
		//	|	});
		var d = dojo,
			handle = d.subscribe(topic, this, method);

		// return handles for Any widget that may need them
		this._subscribes.push(handle);
		return handle;
	},

	unsubscribe: function(/*Object*/ handle){
		// summary:
		//		Unsubscribes handle created by this.subscribe.
		//		Also removes handle from this widget's list of subscriptions
		for(var i=0; i<this._subscribes.length; i++){
			if(this._subscribes[i] == handle){
				dojo.unsubscribe(handle);
				this._subscribes.splice(i, 1);
				return;
			}
		}
	},

	isLeftToRight: function(){
		// summary:
		//		Return this widget's explicit or implicit orientation (true for LTR, false for RTL)
		// tags:
		//		protected
		return this.dir ? (this.dir == "ltr") : dojo._isBodyLtr(); //Boolean
	},

	isFocusable: function(){
		// summary:
		//		Return true if this widget can currently be focused
		//		and false if not
		return this.focus && (dojo.style(this.domNode, "display") != "none");
	},

	placeAt: function(/* String|DomNode|_Widget */reference, /* String?|Int? */position){
		// summary:
		//		Place this widget's domNode reference somewhere in the DOM based
		//		on standard dojo.place conventions, or passing a Widget reference that
		//		contains and addChild member.
		//
		// description:
		//		A convenience function provided in all _Widgets, providing a simple
		//		shorthand mechanism to put an existing (or newly created) Widget
		//		somewhere in the dom, and allow chaining.
		//
		// reference:
		//		The String id of a domNode, a domNode reference, or a reference to a Widget posessing
		//		an addChild method.
		//
		// position:
		//		If passed a string or domNode reference, the position argument
		//		accepts a string just as dojo.place does, one of: "first", "last",
		//		"before", or "after".
		//
		//		If passed a _Widget reference, and that widget reference has an ".addChild" method,
		//		it will be called passing this widget instance into that method, supplying the optional
		//		position index passed.
		//
		// returns:
		//		dijit._Widget
		//		Provides a useful return of the newly created dijit._Widget instance so you
		//		can "chain" this function by instantiating, placing, then saving the return value
		//		to a variable.
		//
		// example:
		// | 	// create a Button with no srcNodeRef, and place it in the body:
		// | 	var button = new dijit.form.Button({ label:"click" }).placeAt(dojo.body());
		// | 	// now, 'button' is still the widget reference to the newly created button
		// | 	dojo.connect(button, "onClick", function(e){ console.log('click'); });
		//
		// example:
		// |	// create a button out of a node with id="src" and append it to id="wrapper":
		// | 	var button = new dijit.form.Button({},"src").placeAt("wrapper");
		//
		// example:
		// |	// place a new button as the first element of some div
		// |	var button = new dijit.form.Button({ label:"click" }).placeAt("wrapper","first");
		//
		// example:
		// |	// create a contentpane and add it to a TabContainer
		// |	var tc = dijit.byId("myTabs");
		// |	new dijit.layout.ContentPane({ href:"foo.html", title:"Wow!" }).placeAt(tc)

		if(reference.declaredClass && reference.addChild){
			reference.addChild(this, position);
		}else{
			dojo.place(this.domNode, reference, position);
		}
		return this;
	},

	_onShow: function(){
		// summary:
		//		Internal method called when this widget is made visible.
		//		See `onShow` for details.
		this.onShow();
	},

	onShow: function(){
		// summary:
		//		Called when this widget becomes the selected pane in a
		//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
		//		`dijit.layout.AccordionContainer`, etc.
		//
		//		Also called to indicate display of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
		// tags:
		//		callback
	},

	onHide: function(){
		// summary:
			//		Called when another widget becomes the selected pane in a
			//		`dijit.layout.TabContainer`, `dijit.layout.StackContainer`,
			//		`dijit.layout.AccordionContainer`, etc.
			//
			//		Also called to indicate hide of a `dijit.Dialog`, `dijit.TooltipDialog`, or `dijit.TitlePane`.
			// tags:
			//		callback
	},

	onClose: function(){
		// summary:
		//		Called when this widget is being displayed as a popup (ex: a Calendar popped
		//		up from a DateTextBox), and it is hidden.
		//		This is called from the dijit.popup code, and should not be called directly.
		//
		//		Also used as a parameter for children of `dijit.layout.StackContainer` or subclasses.
		//		Callback if a user tries to close the child.   Child will be closed if this function returns true.
		// tags:
		//		extension

		return true;		// Boolean
	}
});

})();

}

if(!dojo._hasResource["dijit._Container"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Container"] = true;
dojo.provide("dijit._Container");

dojo.declare("dijit._Container",
	null,
	{
		// summary:
		//		Mixin for widgets that contain a set of widget children.
		// description:
		//		Use this mixin for widgets that needs to know about and
		//		keep track of their widget children. Suitable for widgets like BorderContainer
		//		and TabContainer which contain (only) a set of child widgets.
		//
		//		It's not suitable for widgets like ContentPane
		//		which contains mixed HTML (plain DOM nodes in addition to widgets),
		//		and where contained widgets are not necessarily directly below
		//		this.containerNode.   In that case calls like addChild(node, position)
		//		wouldn't make sense.

		// isContainer: [protected] Boolean
		//		Indicates that this widget acts as a "parent" to the descendant widgets.
		//		When the parent is started it will call startup() on the child widgets.
		//		See also `isLayoutContainer`.
		isContainer: true,

		buildRendering: function(){
			this.inherited(arguments);
			if(!this.containerNode){
				// all widgets with descendants must set containerNode
	 				this.containerNode = this.domNode;
			}
		},

		addChild: function(/*dijit._Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Makes the given widget a child of this widget.
			// description:
			//		Inserts specified child widget's dom node as a child of this widget's
			//		container node, and possibly does other processing (such as layout).

			var refNode = this.containerNode;
			if(insertIndex && typeof insertIndex == "number"){
				var children = this.getChildren();
				if(children && children.length >= insertIndex){
					refNode = children[insertIndex-1].domNode;
					insertIndex = "after";
				}
			}
			dojo.place(widget.domNode, refNode, insertIndex);

			// If I've been started but the child widget hasn't been started,
			// start it now.  Make sure to do this after widget has been
			// inserted into the DOM tree, so it can see that it's being controlled by me,
			// so it doesn't try to size itself.
			if(this._started && !widget._started){
				widget.startup();
			}
		},

		removeChild: function(/*Widget or int*/ widget){
			// summary:
			//		Removes the passed widget instance from this widget but does
			//		not destroy it.  You can also pass in an integer indicating
			//		the index within the container to remove

			if(typeof widget == "number" && widget > 0){
				widget = this.getChildren()[widget];
			}

			if(widget){
				var node = widget.domNode;
				if(node && node.parentNode){
					node.parentNode.removeChild(node); // detach but don't destroy
				}
			}
		},

		hasChildren: function(){
			// summary:
			//		Returns true if widget has children, i.e. if this.containerNode contains something.
			return this.getChildren().length > 0;	// Boolean
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			// summary:
			//      Destroys all the widgets inside this.containerNode,
			//      but not this widget itself
			dojo.forEach(this.getChildren(), function(child){ child.destroyRecursive(preserveDom); });
		},

		_getSiblingOfChild: function(/*dijit._Widget*/ child, /*int*/ dir){
			// summary:
			//		Get the next or previous widget sibling of child
			// dir:
			//		if 1, get the next sibling
			//		if -1, get the previous sibling
			// tags:
			//      private
			var node = child.domNode,
				which = (dir>0 ? "nextSibling" : "previousSibling");
			do{
				node = node[which];
			}while(node && (node.nodeType != 1 || !dijit.byNode(node)));
			return node && dijit.byNode(node);	// dijit._Widget
		},

		getIndexOfChild: function(/*dijit._Widget*/ child){
			// summary:
			//		Gets the index of the child in this container or -1 if not found
			return dojo.indexOf(this.getChildren(), child);	// int
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under dojo.doc.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Startup all children of this widget
			dojo.forEach(this.getChildren(), function(child){ child.startup(); });

			this.inherited(arguments);
		}
	}
);

}

if(!dojo._hasResource["dijit._KeyNavContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._KeyNavContainer"] = true;
dojo.provide("dijit._KeyNavContainer");


dojo.declare("dijit._KeyNavContainer",
	dijit._Container,
	{

		// summary:
		//		A _Container with keyboard navigation of its children.
		// description:
		//		To use this mixin, call connectKeyNavHandlers() in
		//		postCreate() and call startupKeyNavChildren() in startup().
		//		It provides normalized keyboard and focusing code for Container
		//		widgets.
/*=====
		// focusedChild: [protected] Widget
		//		The currently focused child widget, or null if there isn't one
		focusedChild: null,
=====*/

		// tabIndex: Integer
		//		Tab index of the container; same as HTML tabIndex attribute.
		//		Note then when user tabs into the container, focus is immediately
		//		moved to the first item in the container.
		tabIndex: "0",

		_keyNavCodes: {},

		connectKeyNavHandlers: function(/*dojo.keys[]*/ prevKeyCodes, /*dojo.keys[]*/ nextKeyCodes){
			// summary:
			//		Call in postCreate() to attach the keyboard handlers
			//		to the container.
			// preKeyCodes: dojo.keys[]
			//		Key codes for navigating to the previous child.
			// nextKeyCodes: dojo.keys[]
			//		Key codes for navigating to the next child.
			// tags:
			//		protected

			var keyCodes = (this._keyNavCodes = {});
			var prev = dojo.hitch(this, this.focusPrev);
			var next = dojo.hitch(this, this.focusNext);
			dojo.forEach(prevKeyCodes, function(code){ keyCodes[code] = prev; });
			dojo.forEach(nextKeyCodes, function(code){ keyCodes[code] = next; });
			this.connect(this.domNode, "onkeypress", "_onContainerKeypress");
			this.connect(this.domNode, "onfocus", "_onContainerFocus");
		},

		startupKeyNavChildren: function(){
			// summary:
			//		Call in startup() to set child tabindexes to -1
			// tags:
			//		protected
			dojo.forEach(this.getChildren(), dojo.hitch(this, "_startupChild"));
		},

		addChild: function(/*dijit._Widget*/ widget, /*int?*/ insertIndex){
			// summary:
			//		Add a child to our _Container
			dijit._KeyNavContainer.superclass.addChild.apply(this, arguments);
			this._startupChild(widget);
		},

		focus: function(){
			// summary:
			//		Default focus() implementation: focus the first child.
			this.focusFirstChild();
		},

		focusFirstChild: function(){
			// summary:
			//		Focus the first focusable child in the container.
			// tags:
			//		protected
			var child = this._getFirstFocusableChild();
			if(child){ // edge case: Menu could be empty or hidden
				this.focusChild(child);
			}
		},

		focusNext: function(){
			// summary:
			//		Focus the next widget
			// tags:
			//		protected
			var child = this._getNextFocusableChild(this.focusedChild, 1);
			this.focusChild(child);
		},

		focusPrev: function(){
			// summary:
			//		Focus the last focusable node in the previous widget
			//		(ex: go to the ComboButton icon section rather than button section)
			// tags:
			//		protected
			var child = this._getNextFocusableChild(this.focusedChild, -1);
			this.focusChild(child, true);
		},

		focusChild: function(/*dijit._Widget*/ widget, /*Boolean*/ last){
			// summary:
			//		Focus widget.
			// widget:
			//		Reference to container's child widget
			// last:
			//		If true and if widget has multiple focusable nodes, focus the
			//		last one instead of the first one
			// tags:
			//		protected
			
			if(this.focusedChild && widget !== this.focusedChild){
				this._onChildBlur(this.focusedChild);
			}
			widget.focus(last ? "end" : "start");
			this.focusedChild = widget;
		},

		_startupChild: function(/*dijit._Widget*/ widget){
			// summary:
			//		Setup for each child widget
			// description:
			//		Sets tabIndex=-1 on each child, so that the tab key will 
			//		leave the container rather than visiting each child.
			// tags:
			//		private
			
			widget.set("tabIndex", "-1");
			
			this.connect(widget, "_onFocus", function(){
				// Set valid tabIndex so tabbing away from widget goes to right place, see #10272
				widget.set("tabIndex", this.tabIndex);
			});
			this.connect(widget, "_onBlur", function(){
				widget.set("tabIndex", "-1");
			});
		},

		_onContainerFocus: function(evt){
			// summary:
			//		Handler for when the container gets focus
			// description:
			//		Initially the container itself has a tabIndex, but when it gets
			//		focus, switch focus to first child...
			// tags:
			//		private

			// Note that we can't use _onFocus() because switching focus from the
			// _onFocus() handler confuses the focus.js code
			// (because it causes _onFocusNode() to be called recursively)

			// focus bubbles on Firefox,
			// so just make sure that focus has really gone to the container
			if(evt.target !== this.domNode){ return; }

			this.focusFirstChild();

			// and then set the container's tabIndex to -1,
			// (don't remove as that breaks Safari 4)
			// so that tab or shift-tab will go to the fields after/before
			// the container, rather than the container itself
			dojo.attr(this.domNode, "tabIndex", "-1");
		},

		_onBlur: function(evt){
			// When focus is moved away the container, and its descendant (popup) widgets,
			// then restore the container's tabIndex so that user can tab to it again.
			// Note that using _onBlur() so that this doesn't happen when focus is shifted
			// to one of my child widgets (typically a popup)
			if(this.tabIndex){
				dojo.attr(this.domNode, "tabIndex", this.tabIndex);
			}
			this.inherited(arguments);
		},

		_onContainerKeypress: function(evt){
			// summary:
			//		When a key is pressed, if it's an arrow key etc. then
			//		it's handled here.
			// tags:
			//		private
			if(evt.ctrlKey || evt.altKey){ return; }
			var func = this._keyNavCodes[evt.charOrCode];
			if(func){
				func();
				dojo.stopEvent(evt);
			}
		},

		_onChildBlur: function(/*dijit._Widget*/ widget){
			// summary:
			//		Called when focus leaves a child widget to go
			//		to a sibling widget.
			// tags:
			//		protected
		},

		_getFirstFocusableChild: function(){
			// summary:
			//		Returns first child that can be focused
			return this._getNextFocusableChild(null, 1);	// dijit._Widget
		},

		_getNextFocusableChild: function(child, dir){
			// summary:
			//		Returns the next or previous focusable child, compared
			//		to "child"
			// child: Widget
			//		The current widget
			// dir: Integer
			//		* 1 = after
			//		* -1 = before
			if(child){
				child = this._getSiblingOfChild(child, dir);
			}
			var children = this.getChildren();
			for(var i=0; i < children.length; i++){
				if(!child){
					child = children[(dir>0) ? 0 : (children.length-1)];
				}
				if(child.isFocusable()){
					return child;	// dijit._Widget
				}
				child = this._getSiblingOfChild(child, dir);
			}
			// no focusable child found
			return null;	// dijit._Widget
		}
	}
);

}

if(!dojo._hasResource["dojo.string"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.string"] = true;
dojo.provide("dojo.string");

/*=====
dojo.string = { 
	// summary: String utilities for Dojo
};
=====*/

dojo.string.rep = function(/*String*/str, /*Integer*/num){
	//	summary:
	//		Efficiently replicate a string `n` times.
	//	str:
	//		the string to replicate
	//	num:
	//		number of times to replicate the string
	
	if(num <= 0 || !str){ return ""; }
	
	var buf = [];
	for(;;){
		if(num & 1){
			buf.push(str);
		}
		if(!(num >>= 1)){ break; }
		str += str;
	}
	return buf.join("");	// String
};

dojo.string.pad = function(/*String*/text, /*Integer*/size, /*String?*/ch, /*Boolean?*/end){
	//	summary:
	//		Pad a string to guarantee that it is at least `size` length by
	//		filling with the character `ch` at either the start or end of the
	//		string. Pads at the start, by default.
	//	text:
	//		the string to pad
	//	size:
	//		length to provide padding
	//	ch:
	//		character to pad, defaults to '0'
	//	end:
	//		adds padding at the end if true, otherwise pads at start
	//	example:
	//	|	// Fill the string to length 10 with "+" characters on the right.  Yields "Dojo++++++".
	//	|	dojo.string.pad("Dojo", 10, "+", true);

	if(!ch){
		ch = '0';
	}
	var out = String(text),
		pad = dojo.string.rep(ch, Math.ceil((size - out.length) / ch.length));
	return end ? out + pad : pad + out;	// String
};

dojo.string.substitute = function(	/*String*/		template, 
									/*Object|Array*/map, 
									/*Function?*/	transform, 
									/*Object?*/		thisObject){
	//	summary:
	//		Performs parameterized substitutions on a string. Throws an
	//		exception if any parameter is unmatched.
	//	template: 
	//		a string with expressions in the form `${key}` to be replaced or
	//		`${key:format}` which specifies a format function. keys are case-sensitive. 
	//	map:
	//		hash to search for substitutions
	//	transform: 
	//		a function to process all parameters before substitution takes
	//		place, e.g. mylib.encodeXML
	//	thisObject: 
	//		where to look for optional format function; default to the global
	//		namespace
	//	example:
	//		Substitutes two expressions in a string from an Array or Object
	//	|	// returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// by providing substitution data in an Array
	//	|	dojo.string.substitute(
	//	|		"File '${0}' is not found in directory '${1}'.",
	//	|		["foo.html","/temp"]
	//	|	);
	//	|
	//	|	// also returns "File 'foo.html' is not found in directory '/temp'."
	//	|	// but provides substitution data in an Object structure.  Dotted
	//	|	// notation may be used to traverse the structure.
	//	|	dojo.string.substitute(
	//	|		"File '${name}' is not found in directory '${info.dir}'.",
	//	|		{ name: "foo.html", info: { dir: "/temp" } }
	//	|	);
	//	example:
	//		Use a transform function to modify the values:
	//	|	// returns "file 'foo.html' is not found in directory '/temp'."
	//	|	dojo.string.substitute(
	//	|		"${0} is not found in ${1}.",
	//	|		["foo.html","/temp"],
	//	|		function(str){
	//	|			// try to figure out the type
	//	|			var prefix = (str.charAt(0) == "/") ? "directory": "file";
	//	|			return prefix + " '" + str + "'";
	//	|		}
	//	|	);
	//	example:
	//		Use a formatter
	//	|	// returns "thinger -- howdy"
	//	|	dojo.string.substitute(
	//	|		"${0:postfix}", ["thinger"], null, {
	//	|			postfix: function(value, key){
	//	|				return value + " -- howdy";
	//	|			}
	//	|		}
	//	|	);

	thisObject = thisObject || dojo.global;
	transform = transform ? 
		dojo.hitch(thisObject, transform) : function(v){ return v; };

	return template.replace(/\$\{([^\s\:\}]+)(?:\:([^\s\:\}]+))?\}/g,
		function(match, key, format){
			var value = dojo.getObject(key, false, map);
			if(format){
				value = dojo.getObject(format, false, thisObject).call(thisObject, value, key);
			}
			return transform(value, key).toString();
		}); // String
};

/*=====
dojo.string.trim = function(str){
	//	summary:
	//		Trims whitespace from both sides of the string
	//	str: String
	//		String to be trimmed
	//	returns: String
	//		Returns the trimmed string
	//	description:
	//		This version of trim() was taken from [Steven Levithan's blog](http://blog.stevenlevithan.com/archives/faster-trim-javascript).
	//		The short yet performant version of this function is dojo.trim(),
	//		which is part of Dojo base.  Uses String.prototype.trim instead, if available.
	return "";	// String
}
=====*/

dojo.string.trim = String.prototype.trim ?
	dojo.trim : // aliasing to the native function
	function(str){
		str = str.replace(/^\s+/, '');
		for(var i = str.length - 1; i >= 0; i--){
			if(/\S/.test(str.charAt(i))){
				str = str.substring(0, i + 1);
				break;
			}
		}
		return str;
	};

}

if(!dojo._hasResource["dojo.date.stamp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.date.stamp"] = true;
dojo.provide("dojo.date.stamp");

// Methods to convert dates to or from a wire (string) format using well-known conventions

dojo.date.stamp.fromISOString = function(/*String*/formattedString, /*Number?*/defaultTime){
	//	summary:
	//		Returns a Date object given a string formatted according to a subset of the ISO-8601 standard.
	//
	//	description:
	//		Accepts a string formatted according to a profile of ISO8601 as defined by
	//		[RFC3339](http://www.ietf.org/rfc/rfc3339.txt), except that partial input is allowed.
	//		Can also process dates as specified [by the W3C](http://www.w3.org/TR/NOTE-datetime)
	//		The following combinations are valid:
	//
	//			* dates only
	//			|	* yyyy
	//			|	* yyyy-MM
	//			|	* yyyy-MM-dd
	// 			* times only, with an optional time zone appended
	//			|	* THH:mm
	//			|	* THH:mm:ss
	//			|	* THH:mm:ss.SSS
	// 			* and "datetimes" which could be any combination of the above
	//
	//		timezones may be specified as Z (for UTC) or +/- followed by a time expression HH:mm
	//		Assumes the local time zone if not specified.  Does not validate.  Improperly formatted
	//		input may return null.  Arguments which are out of bounds will be handled
	// 		by the Date constructor (e.g. January 32nd typically gets resolved to February 1st)
	//		Only years between 100 and 9999 are supported.
	//
  	//	formattedString:
	//		A string such as 2005-06-30T08:05:00-07:00 or 2005-06-30 or T08:05:00
	//
	//	defaultTime:
	//		Used for defaults for fields omitted in the formattedString.
	//		Uses 1970-01-01T00:00:00.0Z by default.

	if(!dojo.date.stamp._isoRegExp){
		dojo.date.stamp._isoRegExp =
//TODO: could be more restrictive and check for 00-59, etc.
			/^(?:(\d{4})(?:-(\d{2})(?:-(\d{2}))?)?)?(?:T(\d{2}):(\d{2})(?::(\d{2})(.\d+)?)?((?:[+-](\d{2}):(\d{2}))|Z)?)?$/;
	}

	var match = dojo.date.stamp._isoRegExp.exec(formattedString),
		result = null;

	if(match){
		match.shift();
		if(match[1]){match[1]--;} // Javascript Date months are 0-based
		if(match[6]){match[6] *= 1000;} // Javascript Date expects fractional seconds as milliseconds

		if(defaultTime){
			// mix in defaultTime.  Relatively expensive, so use || operators for the fast path of defaultTime === 0
			defaultTime = new Date(defaultTime);
			dojo.forEach(dojo.map(["FullYear", "Month", "Date", "Hours", "Minutes", "Seconds", "Milliseconds"], function(prop){
				return defaultTime["get" + prop]();
			}), function(value, index){
				match[index] = match[index] || value;
			});
		}
		result = new Date(match[0]||1970, match[1]||0, match[2]||1, match[3]||0, match[4]||0, match[5]||0, match[6]||0); //TODO: UTC defaults
		if(match[0] < 100){
			result.setFullYear(match[0] || 1970);
		}

		var offset = 0,
			zoneSign = match[7] && match[7].charAt(0);
		if(zoneSign != 'Z'){
			offset = ((match[8] || 0) * 60) + (Number(match[9]) || 0);
			if(zoneSign != '-'){ offset *= -1; }
		}
		if(zoneSign){
			offset -= result.getTimezoneOffset();
		}
		if(offset){
			result.setTime(result.getTime() + offset * 60000);
		}
	}

	return result; // Date or null
}

/*=====
	dojo.date.stamp.__Options = function(){
		//	selector: String
		//		"date" or "time" for partial formatting of the Date object.
		//		Both date and time will be formatted by default.
		//	zulu: Boolean
		//		if true, UTC/GMT is used for a timezone
		//	milliseconds: Boolean
		//		if true, output milliseconds
		this.selector = selector;
		this.zulu = zulu;
		this.milliseconds = milliseconds;
	}
=====*/

dojo.date.stamp.toISOString = function(/*Date*/dateObject, /*dojo.date.stamp.__Options?*/options){
	//	summary:
	//		Format a Date object as a string according a subset of the ISO-8601 standard
	//
	//	description:
	//		When options.selector is omitted, output follows [RFC3339](http://www.ietf.org/rfc/rfc3339.txt)
	//		The local time zone is included as an offset from GMT, except when selector=='time' (time without a date)
	//		Does not check bounds.  Only years between 100 and 9999 are supported.
	//
	//	dateObject:
	//		A Date object

	var _ = function(n){ return (n < 10) ? "0" + n : n; };
	options = options || {};
	var formattedDate = [],
		getter = options.zulu ? "getUTC" : "get",
		date = "";
	if(options.selector != "time"){
		var year = dateObject[getter+"FullYear"]();
		date = ["0000".substr((year+"").length)+year, _(dateObject[getter+"Month"]()+1), _(dateObject[getter+"Date"]())].join('-');
	}
	formattedDate.push(date);
	if(options.selector != "date"){
		var time = [_(dateObject[getter+"Hours"]()), _(dateObject[getter+"Minutes"]()), _(dateObject[getter+"Seconds"]())].join(':');
		var millis = dateObject[getter+"Milliseconds"]();
		if(options.milliseconds){
			time += "."+ (millis < 100 ? "0" : "") + _(millis);
		}
		if(options.zulu){
			time += "Z";
		}else if(options.selector != "time"){
			var timezoneOffset = dateObject.getTimezoneOffset();
			var absOffset = Math.abs(timezoneOffset);
			time += (timezoneOffset > 0 ? "-" : "+") + 
				_(Math.floor(absOffset/60)) + ":" + _(absOffset%60);
		}
		formattedDate.push(time);
	}
	return formattedDate.join('T'); // String
}

}

if(!dojo._hasResource["dojo.parser"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.parser"] = true;
dojo.provide("dojo.parser");


new Date("X"); // workaround for #11279, new Date("") == NaN

dojo.parser = new function(){
	// summary: The Dom/Widget parsing package

	var d = dojo;
	this._attrName = d._scopeName + "Type";
	this._query = "[" + this._attrName + "]";

	function val2type(/*Object*/ value){
		// summary:
		//		Returns name of type of given value.

		if(d.isString(value)){ return "string"; }
		if(typeof value == "number"){ return "number"; }
		if(typeof value == "boolean"){ return "boolean"; }
		if(d.isFunction(value)){ return "function"; }
		if(d.isArray(value)){ return "array"; } // typeof [] == "object"
		if(value instanceof Date) { return "date"; } // assume timestamp
		if(value instanceof d._Url){ return "url"; }
		return "object";
	}

	function str2obj(/*String*/ value, /*String*/ type){
		// summary:
		//		Convert given string value to given type
		switch(type){
			case "string":
				return value;
			case "number":
				return value.length ? Number(value) : NaN;
			case "boolean":
				// for checked/disabled value might be "" or "checked".  interpret as true.
				return typeof value == "boolean" ? value : !(value.toLowerCase()=="false");
			case "function":
				if(d.isFunction(value)){
					// IE gives us a function, even when we say something like onClick="foo"
					// (in which case it gives us an invalid function "function(){ foo }"). 
					//  Therefore, convert to string
					value=value.toString();
					value=d.trim(value.substring(value.indexOf('{')+1, value.length-1));
				}
				try{
					if(value === "" || value.search(/[^\w\.]+/i) != -1){
						// The user has specified some text for a function like "return x+5"
						return new Function(value);
					}else{
						// The user has specified the name of a function like "myOnClick"
						// or a single word function "return"
						return d.getObject(value, false) || new Function(value);
					}
				}catch(e){ return new Function(); }
			case "array":
				return value ? value.split(/\s*,\s*/) : [];
			case "date":
				switch(value){
					case "": return new Date("");	// the NaN of dates
					case "now": return new Date();	// current date
					default: return d.date.stamp.fromISOString(value);
				}
			case "url":
				return d.baseUrl + value;
			default:
				return d.fromJson(value);
		}
	}

	var instanceClasses = {
		// map from fully qualified name (like "dijit.Button") to structure like
		// { cls: dijit.Button, params: {label: "string", disabled: "boolean"} }
	};

	// Widgets like BorderContainer add properties to _Widget via dojo.extend().
	// If BorderContainer is loaded after _Widget's parameter list has been cached,
	// we need to refresh that parameter list (for _Widget and all widgets that extend _Widget).
	dojo.connect(dojo, "extend", function(){
		instanceClasses = {};
	});

	function getClassInfo(/*String*/ className){
		// className:
		//		fully qualified name (like "dijit.form.Button")
		// returns:
		//		structure like
		//			{ 
		//				cls: dijit.Button, 
		//				params: { label: "string", disabled: "boolean"}
		//			}

		if(!instanceClasses[className]){
			// get pointer to widget class
			var cls = d.getObject(className);
			if(!cls){ return null; }		// class not defined [yet]

			var proto = cls.prototype;
	
			// get table of parameter names & types
			var params = {}, dummyClass = {};
			for(var name in proto){
				if(name.charAt(0)=="_"){ continue; } 	// skip internal properties
				if(name in dummyClass){ continue; }		// skip "constructor" and "toString"
				var defVal = proto[name];
				params[name]=val2type(defVal);
			}

			instanceClasses[className] = { cls: cls, params: params };
		}
		return instanceClasses[className];
	}

	this._functionFromScript = function(script){
		var preamble = "";
		var suffix = "";
		var argsStr = script.getAttribute("args");
		if(argsStr){
			d.forEach(argsStr.split(/\s*,\s*/), function(part, idx){
				preamble += "var "+part+" = arguments["+idx+"]; ";
			});
		}
		var withStr = script.getAttribute("with");
		if(withStr && withStr.length){
			d.forEach(withStr.split(/\s*,\s*/), function(part){
				preamble += "with("+part+"){";
				suffix += "}";
			});
		}
		return new Function(preamble+script.innerHTML+suffix);
	}

	this.instantiate = function(/* Array */nodes, /* Object? */mixin, /* Object? */args){
		// summary:
		//		Takes array of nodes, and turns them into class instances and
		//		potentially calls a startup method to allow them to connect with
		//		any children.
		// nodes: Array
		//		Array of nodes or objects like
		//	|		{
		//	|			type: "dijit.form.Button",
		//	|			node: DOMNode,
		//	|			scripts: [ ... ],	// array of <script type="dojo/..."> children of node
		//	|			inherited: { ... }	// settings inherited from ancestors like dir, theme, etc.
		//	|		}
		// mixin: Object?
		//		An object that will be mixed in with each node in the array.
		//		Values in the mixin will override values in the node, if they
		//		exist.
		// args: Object?
		//		An object used to hold kwArgs for instantiation.
		//		Supports 'noStart' and inherited.
		var thelist = [], dp = dojo.parser;
		mixin = mixin||{};
		args = args||{};
		
		d.forEach(nodes, function(obj){
			if(!obj){ return; }

			// Get pointers to DOMNode, dojoType string, and clsInfo (metadata about the dojoType), etc.s
			var node, type, clsInfo, clazz, scripts;
			if(obj.node){
				// new format of nodes[] array, object w/lots of properties pre-computed for me
				node = obj.node;
				type = obj.type;
				clsInfo = obj.clsInfo || (type && getClassInfo(type));
				clazz = clsInfo && clsInfo.cls;
				scripts = obj.scripts;
			}else{
				// old (backwards compatible) format of nodes[] array, simple array of DOMNodes
				node = obj;
				type = dp._attrName in mixin ? mixin[dp._attrName] : node.getAttribute(dp._attrName);
				clsInfo = type && getClassInfo(type);
				clazz = clsInfo && clsInfo.cls;
				scripts = (clazz && (clazz._noScript || clazz.prototype._noScript) ? [] : 
							d.query("> script[type^='dojo/']", node));
			}
			if(!clsInfo){
				throw new Error("Could not load class '" + type);
			}

			// Setup hash to hold parameter settings for this widget.   Start with the parameter
			// settings inherited from ancestors ("dir" and "lang").
			// Inherited setting may later be overridden by explicit settings on node itself.
			var params = {},
				attributes = node.attributes;
			if(args.defaults){
				// settings for the document itself (or whatever subtree is being parsed)
				dojo.mixin(params, args.defaults);
			}
			if(obj.inherited){
				// settings from dir=rtl or lang=... on a node above this node
				dojo.mixin(params, obj.inherited);
			}

			// read parameters (ie, attributes) specified on DOMNode
			// clsInfo.params lists expected params like {"checked": "boolean", "n": "number"}
			for(var name in clsInfo.params){
				var item = name in mixin?{value:mixin[name],specified:true}:attributes.getNamedItem(name);
				if(!item || (!item.specified && (!dojo.isIE || name.toLowerCase()!="value"))){ continue; }
				var value = item.value;
				// Deal with IE quirks for 'class' and 'style'
				switch(name){
				case "class":
					value = "className" in mixin?mixin.className:node.className;
					break;
				case "style":
					value = "style" in mixin?mixin.style:(node.style && node.style.cssText); // FIXME: Opera?
				}
				var _type = clsInfo.params[name];
				if(typeof value == "string"){
					params[name] = str2obj(value, _type);
				}else{
					params[name] = value;
				}
			}

			// Process <script type="dojo/*"> script tags
			// <script type="dojo/method" event="foo"> tags are added to params, and passed to
			// the widget on instantiation.
			// <script type="dojo/method"> tags (with no event) are executed after instantiation
			// <script type="dojo/connect" event="foo"> tags are dojo.connected after instantiation
			// note: dojo/* script tags cannot exist in self closing widgets, like <input />
			var connects = [],	// functions to connect after instantiation
				calls = [];		// functions to call after instantiation

			d.forEach(scripts, function(script){
				node.removeChild(script);
				var event = script.getAttribute("event"),
					type = script.getAttribute("type"),
					nf = d.parser._functionFromScript(script);
				if(event){
					if(type == "dojo/connect"){
						connects.push({event: event, func: nf});
					}else{
						params[event] = nf;
					}
				}else{
					calls.push(nf);
				}
			});

			var markupFactory = clazz.markupFactory || clazz.prototype && clazz.prototype.markupFactory;
			// create the instance
			var instance = markupFactory ? markupFactory(params, node, clazz) : new clazz(params, node);
			thelist.push(instance);

			// map it to the JS namespace if that makes sense
			var jsname = node.getAttribute("jsId");
			if(jsname){
				d.setObject(jsname, instance);
			}

			// process connections and startup functions
			d.forEach(connects, function(connect){
				d.connect(instance, connect.event, null, connect.func);
			});
			d.forEach(calls, function(func){
				func.call(instance);
			});
		});

		// Call startup on each top level instance if it makes sense (as for
		// widgets).  Parent widgets will recursively call startup on their
		// (non-top level) children
		if(!mixin._started){
			// TODO: for 2.0, when old instantiate() API is desupported, store parent-child
			// relationships in the nodes[] array so that no getParent() call is needed.
			// Note that will  require a parse() call from ContentPane setting a param that the
			// ContentPane is the parent widget (so that the parse doesn't call startup() on the
			// ContentPane's children)
			d.forEach(thelist, function(instance){
				if(	!args.noStart && instance  && 
					instance.startup &&
					!instance._started && 
					(!instance.getParent || !instance.getParent())
				){
					instance.startup();
				}
			});
		}
		return thelist;
	};

	this.parse = function(/*DomNode?*/ rootNode, /* Object? */ args){
		// summary:
		//		Scan the DOM for class instances, and instantiate them.
		//
		// description:
		//		Search specified node (or root node) recursively for class instances,
		//		and instantiate them Searches for
		//		dojoType="qualified.class.name"
		//
		// rootNode: DomNode?
		//		A default starting root node from which to start the parsing. Can be
		//		omitted, defaulting to the entire document. If omitted, the `args`
		//		object can be passed in this place. If the `args` object has a 
		//		`rootNode` member, that is used.
		//
		// args:
		//		a kwArgs object passed along to instantiate()
		//		
		//			* noStart: Boolean?
		//				when set will prevent the parser from calling .startup()
		//				when locating the nodes. 
		//			* rootNode: DomNode?
		//				identical to the function's `rootNode` argument, though
		//				allowed to be passed in via this `args object. 
		//			* inherited: Object
		//				Hash possibly containing dir and lang settings to be applied to
		//				parsed widgets, unless there's another setting on a sub-node that overrides
		//
		//
		// example:
		//		Parse all widgets on a page:
		//	|		dojo.parser.parse();
		//
		// example:
		//		Parse all classes within the node with id="foo"
		//	|		dojo.parser.parse(dojo.byId(foo));
		//
		// example:
		//		Parse all classes in a page, but do not call .startup() on any 
		//		child
		//	|		dojo.parser.parse({ noStart: true })
		//
		// example:
		//		Parse all classes in a node, but do not call .startup()
		//	|		dojo.parser.parse(someNode, { noStart:true });
		//	|		// or
		// 	|		dojo.parser.parse({ noStart:true, rootNode: someNode });

		// determine the root node based on the passed arguments.
		var root;
		if(!args && rootNode && rootNode.rootNode){
			args = rootNode;
			root = args.rootNode;
		}else{
			root = rootNode;
		}

		var attrName = this._attrName;
		function scan(parent, list){
			// summary:
			//		Parent is an Object representing a DOMNode, with or without a dojoType specified.
			//		Scan parent's children looking for nodes with dojoType specified, storing in list[].
			//		If parent has a dojoType, also collects <script type=dojo/*> children and stores in parent.scripts[].
			// parent: Object
			//		Object representing the parent node, like
			//	|	{
			//	|		node: DomNode, 			// scan children of this node
			//	|		inherited: {dir: "rtl"},	// dir/lang setting inherited from above node
			//	|
			//	|		// attributes only set if node has dojoType specified
			//	|		scripts: [],			// empty array, put <script type=dojo/*> in here
			//	|		clsInfo: { cls: dijit.form.Button, ...}
			//	|	}
			// list: DomNode[]
			//		Output array of objects (same format as parent) representing nodes to be turned into widgets

			// Effective dir and lang settings on parent node, either set directly or inherited from grandparent
			var inherited = dojo.clone(parent.inherited);
			dojo.forEach(["dir", "lang"], function(name){
				var val = parent.node.getAttribute(name);
				if(val){
					inherited[name] = val;
				}
			});

			// if parent is a widget, then search for <script type=dojo/*> tags and put them in scripts[].
			var scripts = parent.clsInfo && !parent.clsInfo.cls.prototype._noScript ? parent.scripts : null;

			// unless parent is a widget with the stopParser flag set, continue search for dojoType, recursively
			var recurse = !parent.clsInfo || !parent.clsInfo.cls.prototype.stopParser;

			// scan parent's children looking for dojoType and <script type=dojo/*>
			for(var child = parent.node.firstChild; child; child = child.nextSibling){
				if(child.nodeType == 1){
					var type = recurse && child.getAttribute(attrName);
					if(type){
						// if dojoType specified, add to output array of nodes to instantiate
						var params = {
							"type": type,
							clsInfo: getClassInfo(type),	// note: won't find classes declared via dojo.Declaration
							node: child,
							scripts: [], // <script> nodes that are parent's children
							inherited: inherited // dir & lang attributes inherited from parent
						};
						list.push(params);

						// Recurse, collecting <script type="dojo/..."> children, and also looking for
						// descendant nodes with dojoType specified (unless the widget has the stopParser flag),
						scan(params, list);
					}else if(scripts && child.nodeName.toLowerCase() == "script"){
						// if <script type="dojo/...">, save in scripts[]
						type = child.getAttribute("type");
						if (type && /^dojo\/\w/i.test(type)) {
							scripts.push(child);
						}
					}else if(recurse){
						// Recurse, looking for grandchild nodes with dojoType specified
						scan({
							node: child,
							inherited: inherited
						}, list);
					}
				}
			}
		}

		// Make list of all nodes on page w/dojoType specified
		var list = [];
		scan({
			node: root ? dojo.byId(root) : dojo.body(),
			inherited: (args && args.inherited) || {
				dir: dojo._isBodyLtr() ? "ltr" : "rtl"
			}
		}, list);

		// go build the object instances
		return this.instantiate(list, null, args); // Array
	};
}();

//Register the parser callback. It should be the first callback
//after the a11y test.

(function(){
	var parseRunner = function(){ 
		if(dojo.config.parseOnLoad){
			dojo.parser.parse(); 
		}
	};

	// FIXME: need to clobber cross-dependency!!
	if(dojo.exists("dijit.wai.onload") && (dijit.wai.onload === dojo._loaders[0])){
		dojo._loaders.splice(1, 0, parseRunner);
	}else{
		dojo._loaders.unshift(parseRunner);
	}
})();

}

if(!dojo._hasResource["dojo.cache"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cache"] = true;
dojo.provide("dojo.cache");

/*=====
dojo.cache = { 
	// summary:
	// 		A way to cache string content that is fetchable via `dojo.moduleUrl`.
};
=====*/

(function(){
	var cache = {};
	dojo.cache = function(/*String||Object*/module, /*String*/url, /*String||Object?*/value){
		// summary:
		// 		A getter and setter for storing the string content associated with the
		// 		module and url arguments.
		// description:
		// 		module and url are used to call `dojo.moduleUrl()` to generate a module URL.
		// 		If value is specified, the cache value for the moduleUrl will be set to
		// 		that value. Otherwise, dojo.cache will fetch the moduleUrl and store it
		// 		in its internal cache and return that cached value for the URL. To clear
		// 		a cache value pass null for value. Since XMLHttpRequest (XHR) is used to fetch the
		// 		the URL contents, only modules on the same domain of the page can use this capability.
		// 		The build system can inline the cache values though, to allow for xdomain hosting.
		// module: String||Object
		// 		If a String, the module name to use for the base part of the URL, similar to module argument
		// 		to `dojo.moduleUrl`. If an Object, something that has a .toString() method that
		// 		generates a valid path for the cache item. For example, a dojo._Url object.
		// url: String
		// 		The rest of the path to append to the path derived from the module argument. If
		// 		module is an object, then this second argument should be the "value" argument instead.
		// value: String||Object?
		// 		If a String, the value to use in the cache for the module/url combination.
		// 		If an Object, it can have two properties: value and sanitize. The value property
		// 		should be the value to use in the cache, and sanitize can be set to true or false,
		// 		to indicate if XML declarations should be removed from the value and if the HTML
		// 		inside a body tag in the value should be extracted as the real value. The value argument
		// 		or the value property on the value argument are usually only used by the build system
		// 		as it inlines cache content.
		//	example:
		//		To ask dojo.cache to fetch content and store it in the cache (the dojo["cache"] style
		// 		of call is used to avoid an issue with the build system erroneously trying to intern
		// 		this example. To get the build system to intern your dojo.cache calls, use the
		// 		"dojo.cache" style of call):
		// 		|	//If template.html contains "<h1>Hello</h1>" that will be
		// 		|	//the value for the text variable.
		//		|	var text = dojo["cache"]("my.module", "template.html");
		//	example:
		//		To ask dojo.cache to fetch content and store it in the cache, and sanitize the input
		// 		 (the dojo["cache"] style of call is used to avoid an issue with the build system 
		// 		erroneously trying to intern this example. To get the build system to intern your
		// 		dojo.cache calls, use the "dojo.cache" style of call):
		// 		|	//If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		// 		|	//text variable will contain just "<h1>Hello</h1>".
		//		|	var text = dojo["cache"]("my.module", "template.html", {sanitize: true});
		//	example:
		//		Same example as previous, but demostrates how an object can be passed in as
		//		the first argument, then the value argument can then be the second argument.
		// 		|	//If template.html contains "<html><body><h1>Hello</h1></body></html>", the
		// 		|	//text variable will contain just "<h1>Hello</h1>".
		//		|	var text = dojo["cache"](new dojo._Url("my/module/template.html"), {sanitize: true});

		//Module could be a string, or an object that has a toString() method
		//that will return a useful path. If it is an object, then the "url" argument
		//will actually be the value argument.
		if(typeof module == "string"){
			var pathObj = dojo.moduleUrl(module, url);
		}else{
			pathObj = module;
			value = url;
		}
		var key = pathObj.toString();

		var val = value;
		if(value != undefined && !dojo.isString(value)){
			val = ("value" in value ? value.value : undefined);
		}

		var sanitize = value && value.sanitize ? true : false;

		if(typeof val == "string"){
			//We have a string, set cache value
			val = cache[key] = sanitize ? dojo.cache._sanitize(val) : val;
		}else if(val === null){
			//Remove cached value
			delete cache[key];
		}else{
			//Allow cache values to be empty strings. If key property does
			//not exist, fetch it.
			if(!(key in cache)){
				val = dojo._getText(key);
				cache[key] = sanitize ? dojo.cache._sanitize(val) : val;
			}
			val = cache[key];
		}
		return val; //String
	};

	dojo.cache._sanitize = function(/*String*/val){
		// summary: 
		//		Strips <?xml ...?> declarations so that external SVG and XML
		// 		documents can be added to a document without worry. Also, if the string
		//		is an HTML document, only the part inside the body tag is returned.
		// description:
		// 		Copied from dijit._Templated._sanitizeTemplateString.
		if(val){
			val = val.replace(/^\s*<\?xml(\s)+version=[\'\"](\d)*.(\d)*[\'\"](\s)*\?>/im, "");
			var matches = val.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
			if(matches){
				val = matches[1];
			}
		}else{
			val = "";
		}
		return val; //String
	};
})();

}

if(!dojo._hasResource["dijit._Templated"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Templated"] = true;
dojo.provide("dijit._Templated");






dojo.declare("dijit._Templated",
	null,
	{
		// summary:
		//		Mixin for widgets that are instantiated from a template

		// templateString: [protected] String
		//		A string that represents the widget template. Pre-empts the
		//		templatePath. In builds that have their strings "interned", the
		//		templatePath is converted to an inline templateString, thereby
		//		preventing a synchronous network call.
		//
		//		Use in conjunction with dojo.cache() to load from a file.
		templateString: null,

		// templatePath: [protected deprecated] String
		//		Path to template (HTML file) for this widget relative to dojo.baseUrl.
		//		Deprecated: use templateString with dojo.cache() instead.
		templatePath: null,

		// widgetsInTemplate: [protected] Boolean
		//		Should we parse the template to find widgets that might be
		//		declared in markup inside it?  False by default.
		widgetsInTemplate: false,

		// skipNodeCache: [protected] Boolean
		//		If using a cached widget template node poses issues for a
		//		particular widget class, it can set this property to ensure
		//		that its template is always re-built from a string
		_skipNodeCache: false,

		// _earlyTemplatedStartup: Boolean
		//		A fallback to preserve the 1.0 - 1.3 behavior of children in
		//		templates having their startup called before the parent widget
		//		fires postCreate. Defaults to 'false', causing child widgets to
		//		have their .startup() called immediately before a parent widget
		//		.startup(), but always after the parent .postCreate(). Set to
		//		'true' to re-enable to previous, arguably broken, behavior.
		_earlyTemplatedStartup: false,

		// _attachPoints: [private] String[]
		//		List of widget attribute names associated with dojoAttachPoint=... in the
		//		template, ex: ["containerNode", "labelNode"]
/*=====
 		_attachPoints: [],
 =====*/

		constructor: function(){
			this._attachPoints = [];
		},

		_stringRepl: function(tmpl){
			// summary:
			//		Does substitution of ${foo} type properties in template string
			// tags:
			//		private
			var className = this.declaredClass, _this = this;
			// Cache contains a string because we need to do property replacement
			// do the property replacement
			return dojo.string.substitute(tmpl, this, function(value, key){
				if(key.charAt(0) == '!'){ value = dojo.getObject(key.substr(1), false, _this); }
				if(typeof value == "undefined"){ throw new Error(className+" template:"+key); } // a debugging aide
				if(value == null){ return ""; }

				// Substitution keys beginning with ! will skip the transform step,
				// in case a user wishes to insert unescaped markup, e.g. ${!foo}
				return key.charAt(0) == "!" ? value :
					// Safer substitution, see heading "Attribute values" in
					// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
					value.toString().replace(/"/g,"&quot;"); //TODO: add &amp? use encodeXML method?
			}, this);
		},

		// method over-ride
		buildRendering: function(){
			// summary:
			//		Construct the UI for this widget from a template, setting this.domNode.
			// tags:
			//		protected

			// Lookup cached version of template, and download to cache if it
			// isn't there already.  Returns either a DomNode or a string, depending on
			// whether or not the template contains ${foo} replacement parameters.
			var cached = dijit._Templated.getCachedTemplate(this.templatePath, this.templateString, this._skipNodeCache);

			var node;
			if(dojo.isString(cached)){
				node = dojo._toDom(this._stringRepl(cached));
				if(node.nodeType != 1){
					// Flag common problems such as templates with multiple top level nodes (nodeType == 11)
					throw new Error("Invalid template: " + cached);
				}
			}else{
				// if it's a node, all we have to do is clone it
				node = cached.cloneNode(true);
			}

			this.domNode = node;

			// recurse through the node, looking for, and attaching to, our
			// attachment points and events, which should be defined on the template node.
			this._attachTemplateNodes(node);

			if(this.widgetsInTemplate){
				// Make sure dojoType is used for parsing widgets in template.
				// The dojo.parser.query could be changed from multiversion support.
				var parser = dojo.parser, qry, attr;
				if(parser._query != "[dojoType]"){
					qry = parser._query;
					attr = parser._attrName;
					parser._query = "[dojoType]";
					parser._attrName = "dojoType";
				}

				// Store widgets that we need to start at a later point in time
				var cw = (this._startupWidgets = dojo.parser.parse(node, {
					noStart: !this._earlyTemplatedStartup,
					inherited: {dir: this.dir, lang: this.lang}
				}));

				// Restore the query.
				if(qry){
					parser._query = qry;
					parser._attrName = attr;
				}

				this._supportingWidgets = dijit.findWidgets(node);

				this._attachTemplateNodes(cw, function(n,p){
					return n[p];
				});
			}

			this._fillContent(this.srcNodeRef);
		},

		_fillContent: function(/*DomNode*/ source){
			// summary:
			//		Relocate source contents to templated container node.
			//		this.containerNode must be able to receive children, or exceptions will be thrown.
			// tags:
			//		protected
			var dest = this.containerNode;
			if(source && dest){
				while(source.hasChildNodes()){
					dest.appendChild(source.firstChild);
				}
			}
		},

		_attachTemplateNodes: function(rootNode, getAttrFunc){
			// summary:
			//		Iterate through the template and attach functions and nodes accordingly.
			// description:
			//		Map widget properties and functions to the handlers specified in
			//		the dom node and it's descendants. This function iterates over all
			//		nodes and looks for these properties:
			//			* dojoAttachPoint
			//			* dojoAttachEvent
			//			* waiRole
			//			* waiState
			// rootNode: DomNode|Array[Widgets]
			//		the node to search for properties. All children will be searched.
			// getAttrFunc: Function?
			//		a function which will be used to obtain property for a given
			//		DomNode/Widget
			// tags:
			//		private

			getAttrFunc = getAttrFunc || function(n,p){ return n.getAttribute(p); };

			var nodes = dojo.isArray(rootNode) ? rootNode : (rootNode.all || rootNode.getElementsByTagName("*"));
			var x = dojo.isArray(rootNode) ? 0 : -1;
			for(; x<nodes.length; x++){
				var baseNode = (x == -1) ? rootNode : nodes[x];
				if(this.widgetsInTemplate && getAttrFunc(baseNode, "dojoType")){
					continue;
				}
				// Process dojoAttachPoint
				var attachPoint = getAttrFunc(baseNode, "dojoAttachPoint");
				if(attachPoint){
					var point, points = attachPoint.split(/\s*,\s*/);
					while((point = points.shift())){
						if(dojo.isArray(this[point])){
							this[point].push(baseNode);
						}else{
							this[point]=baseNode;
						}
						this._attachPoints.push(point);
					}
				}

				// Process dojoAttachEvent
				var attachEvent = getAttrFunc(baseNode, "dojoAttachEvent");
				if(attachEvent){
					// NOTE: we want to support attributes that have the form
					// "domEvent: nativeEvent; ..."
					var event, events = attachEvent.split(/\s*,\s*/);
					var trim = dojo.trim;
					while((event = events.shift())){
						if(event){
							var thisFunc = null;
							if(event.indexOf(":") != -1){
								// oh, if only JS had tuple assignment
								var funcNameArr = event.split(":");
								event = trim(funcNameArr[0]);
								thisFunc = trim(funcNameArr[1]);
							}else{
								event = trim(event);
							}
							if(!thisFunc){
								thisFunc = event;
							}
							this.connect(baseNode, event, thisFunc);
						}
					}
				}

				// waiRole, waiState
				var role = getAttrFunc(baseNode, "waiRole");
				if(role){
					dijit.setWaiRole(baseNode, role);
				}
				var values = getAttrFunc(baseNode, "waiState");
				if(values){
					dojo.forEach(values.split(/\s*,\s*/), function(stateValue){
						if(stateValue.indexOf('-') != -1){
							var pair = stateValue.split('-');
							dijit.setWaiState(baseNode, pair[0], pair[1]);
						}
					});
				}
			}
		},

		startup: function(){
			dojo.forEach(this._startupWidgets, function(w){
				if(w && !w._started && w.startup){
					w.startup();
				}
			});
			this.inherited(arguments);
		},

		destroyRendering: function(){
			// Delete all attach points to prevent IE6 memory leaks.
			dojo.forEach(this._attachPoints, function(point){
				delete this[point];
			}, this);
			this._attachPoints = [];

			this.inherited(arguments);
		}
	}
);

// key is either templatePath or templateString; object is either string or DOM tree
dijit._Templated._templateCache = {};

dijit._Templated.getCachedTemplate = function(templatePath, templateString, alwaysUseString){
	// summary:
	//		Static method to get a template based on the templatePath or
	//		templateString key
	// templatePath: String||dojo.uri.Uri
	//		The URL to get the template from.
	// templateString: String?
	//		a string to use in lieu of fetching the template from a URL. Takes precedence
	//		over templatePath
	// returns: Mixed
	//		Either string (if there are ${} variables that need to be replaced) or just
	//		a DOM tree (if the node can be cloned directly)

	// is it already cached?
	var tmplts = dijit._Templated._templateCache;
	var key = templateString || templatePath;
	var cached = tmplts[key];
	if(cached){
		try{
			// if the cached value is an innerHTML string (no ownerDocument) or a DOM tree created within the current document, then use the current cached value
			if(!cached.ownerDocument || cached.ownerDocument == dojo.doc){
				// string or node of the same document
				return cached;
			}
		}catch(e){ /* squelch */ } // IE can throw an exception if cached.ownerDocument was reloaded
		dojo.destroy(cached);
	}

	// If necessary, load template string from template path
	if(!templateString){
		templateString = dojo.cache(templatePath, {sanitize: true});
	}
	templateString = dojo.string.trim(templateString);

	if(alwaysUseString || templateString.match(/\$\{([^\}]+)\}/g)){
		// there are variables in the template so all we can do is cache the string
		return (tmplts[key] = templateString); //String
	}else{
		// there are no variables in the template so we can cache the DOM tree
		var node = dojo._toDom(templateString);
		if(node.nodeType != 1){
			throw new Error("Invalid template: " + templateString);
		}
		return (tmplts[key] = node); //Node
	}
};

if(dojo.isIE){
	dojo.addOnWindowUnload(function(){
		var cache = dijit._Templated._templateCache;
		for(var key in cache){
			var value = cache[key];
			if(typeof value == "object"){ // value is either a string or a DOM node template
				dojo.destroy(value);
			}
			delete cache[key];
		}
	});
}

// These arguments can be specified for widgets which are used in templates.
// Since any widget can be specified as sub widgets in template, mix it
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget,{
	dojoAttachEvent: "",
	dojoAttachPoint: "",
	waiRole: "",
	waiState:""
});

}

if(!dojo._hasResource["dijit._Contained"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._Contained"] = true;
dojo.provide("dijit._Contained");

dojo.declare("dijit._Contained",
		null,
		{
			// summary:
			//		Mixin for widgets that are children of a container widget
			//
			// example:
			// | 	// make a basic custom widget that knows about it's parents
			// |	dojo.declare("my.customClass",[dijit._Widget,dijit._Contained],{});

			getParent: function(){
				// summary:
				//		Returns the parent widget of this widget, assuming the parent
				//		specifies isContainer
				var parent = dijit.getEnclosingWidget(this.domNode.parentNode);
				return parent && parent.isContainer ? parent : null;
			},

			_getSibling: function(/*String*/ which){
				// summary:
				//      Returns next or previous sibling
				// which:
				//      Either "next" or "previous"
				// tags:
				//      private
				var node = this.domNode;
				do{
					node = node[which+"Sibling"];
				}while(node && node.nodeType != 1);
				return node && dijit.byNode(node);	// dijit._Widget
			},

			getPreviousSibling: function(){
				// summary:
				//		Returns null if this is the first child of the parent,
				//		otherwise returns the next element sibling to the "left".

				return this._getSibling("previous"); // dijit._Widget
			},

			getNextSibling: function(){
				// summary:
				//		Returns null if this is the last child of the parent,
				//		otherwise returns the next element sibling to the "right".

				return this._getSibling("next"); // dijit._Widget
			},

			getIndexInParent: function(){
				// summary:
				//		Returns the index of this widget within its container parent.
				//		It returns -1 if the parent does not exist, or if the parent
				//		is not a dijit._Container

				var p = this.getParent();
				if(!p || !p.getIndexOfChild){
					return -1; // int
				}
				return p.getIndexOfChild(this); // int
			}
		}
	);


}

if(!dojo._hasResource["dijit._CssStateMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._CssStateMixin"] = true;
dojo.provide("dijit._CssStateMixin");


dojo.declare("dijit._CssStateMixin", [], {
	// summary:
	//		Mixin for widgets to set CSS classes on the widget DOM nodes depending on hover/mouse press/focus
	//		state changes, and also higher-level state changes such becoming disabled or selected.
	//
	// description:
	//		By mixing this class into your widget, and setting the this.baseClass attribute, it will automatically
	//		maintain CSS classes on the widget root node (this.domNode) depending on hover,
	//		active, focus, etc. state.   Ex: with a baseClass of dijitButton, it will apply the classes
	//		dijitButtonHovered and dijitButtonActive, as the user moves the mouse over the widget and clicks it.
	//
	//		It also sets CSS like dijitButtonDisabled based on widget semantic state.
	//
	//		By setting the cssStateNodes attribute, a widget can also track events on subnodes (like buttons
	//		within the widget).

	// cssStateNodes: [protected] Object
	//		List of sub-nodes within the widget that need CSS classes applied on mouse hover/press and focus
	//.
	//		Each entry in the hash is a an attachpoint names (like "upArrowButton") mapped to a CSS class names
	//		(like "dijitUpArrowButton"). Example:
	//	|		{
	//	|			"upArrowButton": "dijitUpArrowButton",
	//	|			"downArrowButton": "dijitDownArrowButton"
	//	|		}
	//		The above will set the CSS class dijitUpArrowButton to the this.upArrowButton DOMNode when it
	//		is hovered, etc.
	cssStateNodes: {},

	postCreate: function(){
		this.inherited(arguments);

		// Automatically monitor mouse events (essentially :hover and :active) on this.domNode
		dojo.forEach(["onmouseenter", "onmouseleave", "onmousedown"], function(e){
			this.connect(this.domNode, e, "_cssMouseEvent");
		}, this);
		
		// Monitoring changes to disabled, readonly, etc. state, and update CSS class of root node
		this.connect(this, "set", function(name, value){
			if(arguments.length >= 2 && {disabled: true, readOnly: true, checked:true, selected:true}[name]){
				this._setStateClass();
			}
		});

		// The widget coming in/out of the focus change affects it's state
		dojo.forEach(["_onFocus", "_onBlur"], function(ap){
			this.connect(this, ap, "_setStateClass");
		}, this);

		// Events on sub nodes within the widget
		for(var ap in this.cssStateNodes){
			this._trackMouseState(this[ap], this.cssStateNodes[ap]);
		}
		// Set state initially; there's probably no hover/active/focus state but widget might be
		// disabled/readonly so we want to set CSS classes for those conditions.
		this._setStateClass();
	},

	_cssMouseEvent: function(/*Event*/ event){
		// summary:
		//	Sets _hovering and _active properties depending on mouse state,
		//	then calls _setStateClass() to set appropriate CSS classes for this.domNode.

		if(!this.disabled){
			switch(event.type){
				case "mouseenter":
				case "mouseover":	// generated on non-IE browsers even though we connected to mouseenter
					this._hovering = true;
					this._active = this._mouseDown;
					break;

				case "mouseleave":
				case "mouseout":	// generated on non-IE browsers even though we connected to mouseleave
					this._hovering = false;
					this._active = false;
					break;

				case "mousedown" :
					this._active = true;
					this._mouseDown = true;
					// Set a global event to handle mouseup, so it fires properly
					// even if the cursor leaves this.domNode before the mouse up event.
					// Alternately could set active=false on mouseout.
					var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
						this._active = false;
						this._mouseDown = false;
						this._setStateClass();
						this.disconnect(mouseUpConnector);
					});
					break;
			}
			this._setStateClass();
		}
	},

	_setStateClass: function(){
		// summary:
		//		Update the visual state of the widget by setting the css classes on this.domNode
		//		(or this.stateNode if defined) by combining this.baseClass with
		//		various suffixes that represent the current widget state(s).
		//
		// description:
		//		In the case where a widget has multiple
		//		states, it sets the class based on all possible
		//	 	combinations.  For example, an invalid form widget that is being hovered
		//		will be "dijitInput dijitInputInvalid dijitInputHover dijitInputInvalidHover".
		//
		//		The widget may have one or more of the following states, determined
		//		by this.state, this.checked, this.valid, and this.selected:
		//			- Error - ValidationTextBox sets this.state to "Error" if the current input value is invalid
		//			- Checked - ex: a checkmark or a ToggleButton in a checked state, will have this.checked==true
		//			- Selected - ex: currently selected tab will have this.selected==true
		//
		//		In addition, it may have one or more of the following states,
		//		based on this.disabled and flags set in _onMouse (this._active, this._hovering, this._focused):
		//			- Disabled	- if the widget is disabled
		//			- Active		- if the mouse (or space/enter key?) is being pressed down
		//			- Focused		- if the widget has focus
		//			- Hover		- if the mouse is over the widget

		// Compute new set of classes
		var newStateClasses = this.baseClass.split(" ");

		function multiply(modifier){
			newStateClasses = newStateClasses.concat(dojo.map(newStateClasses, function(c){ return c+modifier; }), "dijit"+modifier);
		}

		if(!this.isLeftToRight()){
			// For RTL mode we need to set an addition class like dijitTextBoxRtl.
			multiply("Rtl");
		}

		if(this.checked){
			multiply("Checked");
		}
		if(this.state){
			multiply(this.state);
		}
		if(this.selected){
			multiply("Selected");
		}

		if(this.disabled){
			multiply("Disabled");
		}else if(this.readOnly){
			multiply("ReadOnly");
		}else{
			if(this._active){
				multiply("Active");
			}else if(this._hovering){
				multiply("Hover");
			}
		}

		if(this._focused){
			multiply("Focused");
		}

		// Remove old state classes and add new ones.
		// For performance concerns we only write into domNode.className once.
		var tn = this.stateNode || this.domNode,
			classHash = {};	// set of all classes (state and otherwise) for node

		dojo.forEach(tn.className.split(" "), function(c){ classHash[c] = true; });

		if("_stateClasses" in this){
			dojo.forEach(this._stateClasses, function(c){ delete classHash[c]; });
		}

		dojo.forEach(newStateClasses, function(c){ classHash[c] = true; });

		var newClasses = [];
		for(var c in classHash){
			newClasses.push(c);
		}
		tn.className = newClasses.join(" ");

		this._stateClasses = newStateClasses;
	},

	_trackMouseState: function(/*DomNode*/ node, /*String*/ clazz){
		// summary:
		//		Track mouse/focus events on specified node and set CSS class on that node to indicate
		//		current state.   Usually not called directly, but via cssStateNodes attribute.
		// description:
		//		Given class=foo, will set the following CSS class on the node
		//			- fooActive: if the user is currently pressing down the mouse button while over the node
		//			- fooHover: if the user is hovering the mouse over the node, but not pressing down a button
		//			- fooFocus: if the node is focused
		//
		//		Note that it won't set any classes if the widget is disabled.
		// node: DomNode
		//		Should be a sub-node of the widget, not the top node (this.domNode), since the top node
		//		is handled specially and automatically just by mixing in this class.
		// clazz: String
		//		CSS class name (ex: dijitSliderUpArrow).

		// Current state of node (initially false)
		// NB: setting specifically to false because dojo.toggleClass() needs true boolean as third arg
		var hovering=false, active=false, focused=false;

		var self = this,
			cn = dojo.hitch(this, "connect", node);

		function setClass(){
			var disabled = ("disabled" in self && self.disabled) || ("readonly" in self && self.readonly);
			dojo.toggleClass(node, clazz+"Hover", hovering && !active && !disabled);
			dojo.toggleClass(node, clazz+"Active", active && !disabled);
			dojo.toggleClass(node, clazz+"Focused", focused && !disabled);
		}

		// Mouse
		cn("onmouseenter", function(){
			hovering = true;
			setClass();
		});
		cn("onmouseleave", function(){
			hovering = false;
			active = false;
			setClass();
		});
		cn("onmousedown", function(){
			active = true;
			setClass();
		});
		cn("onmouseup", function(){
			active = false;
			setClass();
		});

		// Focus
		cn("onfocus", function(){
			focused = true;
			setClass();
		});
		cn("onblur", function(){
			focused = false;
			setClass();
		});

		// Just in case widget is enabled/disabled while it has focus/hover/active state.
		// Maybe this is overkill.
		this.connect(this, "set", function(name, value){
			if(name == "disabled" || name == "readOnly"){
				setClass();
			}
		});
	}
});

}

if(!dojo._hasResource["dijit.MenuItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.MenuItem"] = true;
dojo.provide("dijit.MenuItem");






dojo.declare("dijit.MenuItem",
		[dijit._Widget, dijit._Templated, dijit._Contained, dijit._CssStateMixin],
		{
		// summary:
		//		A line item in a Menu Widget

		// Make 3 columns
		// icon, label, and expand arrow (BiDi-dependent) indicating sub-menu
		templateString: dojo.cache("dijit", "templates/MenuItem.html", "<tr class=\"dijitReset dijitMenuItem\" dojoAttachPoint=\"focusNode\" waiRole=\"menuitem\" tabIndex=\"-1\"\n\t\tdojoAttachEvent=\"onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick\">\n\t<td class=\"dijitReset dijitMenuItemIconCell\" waiRole=\"presentation\">\n\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitIcon dijitMenuItemIcon\" dojoAttachPoint=\"iconNode\"/>\n\t</td>\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" dojoAttachPoint=\"containerNode\"></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" dojoAttachPoint=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" waiRole=\"presentation\">\n\t\t<div dojoAttachPoint=\"arrowWrapper\" style=\"visibility: hidden\">\n\t\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuExpand\"/>\n\t\t\t<span class=\"dijitMenuExpandA11y\">+</span>\n\t\t</div>\n\t</td>\n</tr>\n"),

		attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
			label: { node: "containerNode", type: "innerHTML" },
			iconClass: { node: "iconNode", type: "class" }
		}),

		baseClass: "dijitMenuItem",

		// label: String
		//		Menu text
		label: '',

		// iconClass: String
		//		Class to apply to DOMNode to make it display an icon.
		iconClass: "",

		// accelKey: String
		//		Text for the accelerator (shortcut) key combination.
		//		Note that although Menu can display accelerator keys there
		//		is no infrastructure to actually catch and execute these
		//		accelerators.
		accelKey: "",

		// disabled: Boolean
		//		If true, the menu item is disabled.
		//		If false, the menu item is enabled.
		disabled: false,

		_fillContent: function(/*DomNode*/ source){
			// If button label is specified as srcNodeRef.innerHTML rather than
			// this.params.label, handle it here.
			if(source && !("label" in this.params)){
				this.set('label', source.innerHTML);
			}
		},

		postCreate: function(){
			this.inherited(arguments);
			dojo.setSelectable(this.domNode, false);
			var label = this.id+"_text";
			dojo.attr(this.containerNode, "id", label);
			if(this.accelKeyNode){
				dojo.attr(this.accelKeyNode, "id", this.id + "_accel");
				label += " " + this.id + "_accel";
			}
			dijit.setWaiState(this.domNode, "labelledby", label);
		},

		_onHover: function(){
			// summary:
			//		Handler when mouse is moved onto menu item
			// tags:
			//		protected
			this.getParent().onItemHover(this);
		},

		_onUnhover: function(){
			// summary:
			//		Handler when mouse is moved off of menu item,
			//		possibly to a child menu, or maybe to a sibling
			//		menuitem or somewhere else entirely.
			// tags:
			//		protected

			// if we are unhovering the currently selected item
			// then unselect it
			this.getParent().onItemUnhover(this);

			// _onUnhover() is called when the menu is hidden (collapsed), due to clicking
			// a MenuItem and having it execut.  When that happens, FF and IE don't generate
			// an onmouseout event for the MenuItem, so give _CssStateMixin some help
			this._hovering = false;
			this._setStateClass();
		},

		_onClick: function(evt){
			// summary:
			//		Internal handler for click events on MenuItem.
			// tags:
			//		private
			this.getParent().onItemClick(this, evt);
			dojo.stopEvent(evt);
		},

		onClick: function(/*Event*/ evt){
			// summary:
			//		User defined function to handle clicks
			// tags:
			//		callback
		},

		focus: function(){
			// summary:
			//		Focus on this MenuItem
			try{
				if(dojo.isIE == 8){
					// needed for IE8 which won't scroll TR tags into view on focus yet calling scrollIntoView creates flicker (#10275)
					this.containerNode.focus();
				}
				dijit.focus(this.focusNode);
			}catch(e){
				// this throws on IE (at least) in some scenarios
			}
		},

		_onFocus: function(){
			// summary:
			//		This is called by the focus manager when focus
			//		goes to this MenuItem or a child menu.
			// tags:
			//		protected
			this._setSelected(true);
			this.getParent()._onItemFocus(this);

			this.inherited(arguments);
		},

		_setSelected: function(selected){
			// summary:
			//		Indicate that this node is the currently selected one
			// tags:
			//		private

			/***
			 * TODO: remove this method and calls to it, when _onBlur() is working for MenuItem.
			 * Currently _onBlur() gets called when focus is moved from the MenuItem to a child menu.
			 * That's not supposed to happen, but the problem is:
			 * In order to allow dijit.popup's getTopPopup() to work,a sub menu's popupParent
			 * points to the parent Menu, bypassing the parent MenuItem... thus the
			 * MenuItem is not in the chain of active widgets and gets a premature call to
			 * _onBlur()
			 */

			dojo.toggleClass(this.domNode, "dijitMenuItemSelected", selected);
		},

		setLabel: function(/*String*/ content){
			// summary:
			//		Deprecated.   Use set('label', ...) instead.
			// tags:
			//		deprecated
			dojo.deprecated("dijit.MenuItem.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
			this.set("label", content);
		},

		setDisabled: function(/*Boolean*/ disabled){
			// summary:
			//		Deprecated.   Use set('disabled', bool) instead.
			// tags:
			//		deprecated
			dojo.deprecated("dijit.Menu.setDisabled() is deprecated.  Use set('disabled', bool) instead.", "", "2.0");
			this.set('disabled', disabled);
		},
		_setDisabledAttr: function(/*Boolean*/ value){
			// summary:
			//		Hook for attr('disabled', ...) to work.
			//		Enable or disable this menu item.
			this.disabled = value;
			dijit.setWaiState(this.focusNode, 'disabled', value ? 'true' : 'false');
		},
		_setAccelKeyAttr: function(/*String*/ value){
			// summary:
			//		Hook for attr('accelKey', ...) to work.
			//		Set accelKey on this menu item.
			this.accelKey=value;

			this.accelKeyNode.style.display=value?"":"none";
			this.accelKeyNode.innerHTML=value;
			//have to use colSpan to make it work in IE
			dojo.attr(this.containerNode,'colSpan',value?"1":"2");
		}
	});

}

if(!dojo._hasResource["dijit.PopupMenuItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.PopupMenuItem"] = true;
dojo.provide("dijit.PopupMenuItem");



dojo.declare("dijit.PopupMenuItem",
		dijit.MenuItem,
		{
		_fillContent: function(){
			// summary:
			//		When Menu is declared in markup, this code gets the menu label and
			//		the popup widget from the srcNodeRef.
			// description:
			//		srcNodeRefinnerHTML contains both the menu item text and a popup widget
			//		The first part holds the menu item text and the second part is the popup
			// example:
			// |	<div dojoType="dijit.PopupMenuItem">
			// |		<span>pick me</span>
			// |		<popup> ... </popup>
			// |	</div>
			// tags:
			//		protected

			if(this.srcNodeRef){
				var nodes = dojo.query("*", this.srcNodeRef);
				dijit.PopupMenuItem.superclass._fillContent.call(this, nodes[0]);

				// save pointer to srcNode so we can grab the drop down widget after it's instantiated
				this.dropDownContainer = this.srcNodeRef;
			}
		},

		startup: function(){
			if(this._started){ return; }
			this.inherited(arguments);

			// we didn't copy the dropdown widget from the this.srcNodeRef, so it's in no-man's
			// land now.  move it to dojo.doc.body.
			if(!this.popup){
				var node = dojo.query("[widgetId]", this.dropDownContainer)[0];
				this.popup = dijit.byNode(node);
			}
			dojo.body().appendChild(this.popup.domNode);
			this.popup.startup();

			this.popup.domNode.style.display="none";
			if(this.arrowWrapper){
				dojo.style(this.arrowWrapper, "visibility", "");
			}
			dijit.setWaiState(this.focusNode, "haspopup", "true");
		},

		destroyDescendants: function(){
			if(this.popup){
				// Destroy the popup, unless it's already been destroyed.  This can happen because
				// the popup is a direct child of <body> even though it's logically my child.
				if(!this.popup._destroyed){
					this.popup.destroyRecursive();
				}
				delete this.popup;
			}
			this.inherited(arguments);
		}
	});


}

if(!dojo._hasResource["dijit.CheckedMenuItem"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.CheckedMenuItem"] = true;
dojo.provide("dijit.CheckedMenuItem");



dojo.declare("dijit.CheckedMenuItem",
		dijit.MenuItem,
		{
		// summary:
		//		A checkbox-like menu item for toggling on and off

		templateString: dojo.cache("dijit", "templates/CheckedMenuItem.html", "<tr class=\"dijitReset dijitMenuItem\" dojoAttachPoint=\"focusNode\" waiRole=\"menuitemcheckbox\" tabIndex=\"-1\"\n\t\tdojoAttachEvent=\"onmouseenter:_onHover,onmouseleave:_onUnhover,ondijitclick:_onClick\">\n\t<td class=\"dijitReset dijitMenuItemIconCell\" waiRole=\"presentation\">\n\t\t<img src=\"${_blankGif}\" alt=\"\" class=\"dijitMenuItemIcon dijitCheckedMenuItemIcon\" dojoAttachPoint=\"iconNode\"/>\n\t\t<span class=\"dijitCheckedMenuItemIconChar\">&#10003;</span>\n\t</td>\n\t<td class=\"dijitReset dijitMenuItemLabel\" colspan=\"2\" dojoAttachPoint=\"containerNode,labelNode\"></td>\n\t<td class=\"dijitReset dijitMenuItemAccelKey\" style=\"display: none\" dojoAttachPoint=\"accelKeyNode\"></td>\n\t<td class=\"dijitReset dijitMenuArrowCell\" waiRole=\"presentation\">&nbsp;</td>\n</tr>\n"),

		// checked: Boolean
		//		Our checked state
		checked: false,
		_setCheckedAttr: function(/*Boolean*/ checked){
			// summary:
			//		Hook so attr('checked', bool) works.
			//		Sets the class and state for the check box.
			dojo.toggleClass(this.domNode, "dijitCheckedMenuItemChecked", checked);
			dijit.setWaiState(this.domNode, "checked", checked);
			this.checked = checked;
		},

		onChange: function(/*Boolean*/ checked){
			// summary:
			//		User defined function to handle check/uncheck events
			// tags:
			//		callback
		},

		_onClick: function(/*Event*/ e){
			// summary:
			//		Clicking this item just toggles its state
			// tags:
			//		private
			if(!this.disabled){
				this.set("checked", !this.checked);
				this.onChange(this.checked);
			}
			this.inherited(arguments);
		}
	});

}

if(!dojo._hasResource["dijit.MenuSeparator"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.MenuSeparator"] = true;
dojo.provide("dijit.MenuSeparator");





dojo.declare("dijit.MenuSeparator",
		[dijit._Widget, dijit._Templated, dijit._Contained],
		{
		// summary:
		//		A line between two menu items

		templateString: dojo.cache("dijit", "templates/MenuSeparator.html", "<tr class=\"dijitMenuSeparator\">\n\t<td class=\"dijitMenuSeparatorIconCell\">\n\t\t<div class=\"dijitMenuSeparatorTop\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n\t<td colspan=\"3\" class=\"dijitMenuSeparatorLabelCell\">\n\t\t<div class=\"dijitMenuSeparatorTop dijitMenuSeparatorLabel\"></div>\n\t\t<div class=\"dijitMenuSeparatorBottom\"></div>\n\t</td>\n</tr>\n"),

		postCreate: function(){
			dojo.setSelectable(this.domNode, false);
		},

		isFocusable: function(){
			// summary:
			//		Override to always return false
			// tags:
			//		protected

			return false; // Boolean
		}
	});


}

if(!dojo._hasResource["dijit.Menu"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Menu"] = true;
dojo.provide("dijit.Menu");







dojo.declare("dijit._MenuBase",
	[dijit._Widget, dijit._Templated, dijit._KeyNavContainer],
{
	// summary:
	//		Base class for Menu and MenuBar

	// parentMenu: [readonly] Widget
	//		pointer to menu that displayed me
	parentMenu: null,

	// popupDelay: Integer
	//		number of milliseconds before hovering (without clicking) causes the popup to automatically open.
	popupDelay: 500,

	startup: function(){
		if(this._started){ return; }

		dojo.forEach(this.getChildren(), function(child){ child.startup(); });
		this.startupKeyNavChildren();

		this.inherited(arguments);
	},

	onExecute: function(){
		// summary:
		//		Attach point for notification about when a menu item has been executed.
		//		This is an internal mechanism used for Menus to signal to their parent to
		//		close them, because they are about to execute the onClick handler.   In
		//		general developers should not attach to or override this method.
		// tags:
		//		protected
	},

	onCancel: function(/*Boolean*/ closeAll){
		// summary:
		//		Attach point for notification about when the user cancels the current menu
		//		This is an internal mechanism used for Menus to signal to their parent to
		//		close them.  In general developers should not attach to or override this method.
		// tags:
		//		protected
	},

	_moveToPopup: function(/*Event*/ evt){
		// summary:
		//		This handles the right arrow key (left arrow key on RTL systems),
		//		which will either open a submenu, or move to the next item in the
		//		ancestor MenuBar
		// tags:
		//		private

		if(this.focusedChild && this.focusedChild.popup && !this.focusedChild.disabled){
			this.focusedChild._onClick(evt);
		}else{
			var topMenu = this._getTopMenu();
			if(topMenu && topMenu._isMenuBar){
				topMenu.focusNext();
			}
		}
	},

	_onPopupHover: function(/*Event*/ evt){
		// summary:
		//		This handler is called when the mouse moves over the popup.
		// tags:
		//		private

		// if the mouse hovers over a menu popup that is in pending-close state,
		// then stop the close operation.
		// This can't be done in onItemHover since some popup targets don't have MenuItems (e.g. ColorPicker)
		if(this.currentPopup && this.currentPopup._pendingClose_timer){
			var parentMenu = this.currentPopup.parentMenu;
			// highlight the parent menu item pointing to this popup
			if(parentMenu.focusedChild){
				parentMenu.focusedChild._setSelected(false);
			}
			parentMenu.focusedChild = this.currentPopup.from_item;
			parentMenu.focusedChild._setSelected(true);
			// cancel the pending close
			this._stopPendingCloseTimer(this.currentPopup);
		}
	},

	onItemHover: function(/*MenuItem*/ item){
		// summary:
		//		Called when cursor is over a MenuItem.
		// tags:
		//		protected

		// Don't do anything unless user has "activated" the menu by:
		//		1) clicking it
		//		2) opening it from a parent menu (which automatically focuses it)
		if(this.isActive){
			this.focusChild(item);
			if(this.focusedChild.popup && !this.focusedChild.disabled && !this.hover_timer){
				this.hover_timer = setTimeout(dojo.hitch(this, "_openPopup"), this.popupDelay);
			}
		}
		// if the user is mixing mouse and keyboard navigation,
		// then the menu may not be active but a menu item has focus,
		// but it's not the item that the mouse just hovered over.
		// To avoid both keyboard and mouse selections, use the latest.
		if(this.focusedChild){
			this.focusChild(item);
		}
		this._hoveredChild = item;
	},

	_onChildBlur: function(item){
		// summary:
		//		Called when a child MenuItem becomes inactive because focus
		//		has been removed from the MenuItem *and* it's descendant menus.
		// tags:
		//		private
		this._stopPopupTimer();
		item._setSelected(false);
		// Close all popups that are open and descendants of this menu
		var itemPopup = item.popup;
		if(itemPopup){
			this._stopPendingCloseTimer(itemPopup);
			itemPopup._pendingClose_timer = setTimeout(function(){
				itemPopup._pendingClose_timer = null;
				if(itemPopup.parentMenu){
					itemPopup.parentMenu.currentPopup = null;
				}
				dijit.popup.close(itemPopup); // this calls onClose
			}, this.popupDelay);
		}
	},

	onItemUnhover: function(/*MenuItem*/ item){
		// summary:
		//		Callback fires when mouse exits a MenuItem
		// tags:
		//		protected

		if(this.isActive){
			this._stopPopupTimer();
		}
		if(this._hoveredChild == item){ this._hoveredChild = null; }
	},

	_stopPopupTimer: function(){
		// summary:
		//		Cancels the popup timer because the user has stop hovering
		//		on the MenuItem, etc.
		// tags:
		//		private
		if(this.hover_timer){
			clearTimeout(this.hover_timer);
			this.hover_timer = null;
		}
	},

	_stopPendingCloseTimer: function(/*dijit._Widget*/ popup){
		// summary:
		//		Cancels the pending-close timer because the close has been preempted
		// tags:
		//		private
		if(popup._pendingClose_timer){
			clearTimeout(popup._pendingClose_timer);
			popup._pendingClose_timer = null;
		}
	},

	_stopFocusTimer: function(){
		// summary:
		//		Cancels the pending-focus timer because the menu was closed before focus occured
		// tags:
		//		private
		if(this._focus_timer){
			clearTimeout(this._focus_timer);
			this._focus_timer = null;
		}
	},

	_getTopMenu: function(){
		// summary:
		//		Returns the top menu in this chain of Menus
		// tags:
		//		private
		for(var top=this; top.parentMenu; top=top.parentMenu);
		return top;
	},

	onItemClick: function(/*dijit._Widget*/ item, /*Event*/ evt){
		// summary:
		//		Handle clicks on an item.
		// tags:
		//		private

		// this can't be done in _onFocus since the _onFocus events occurs asynchronously
		if(typeof this.isShowingNow == 'undefined'){ // non-popup menu
			this._markActive();
		}

		this.focusChild(item);

		if(item.disabled){ return false; }

		if(item.popup){
			this._openPopup();
		}else{
			// before calling user defined handler, close hierarchy of menus
			// and restore focus to place it was when menu was opened
			this.onExecute();

			// user defined handler for click
			item.onClick(evt);
		}
	},

	_openPopup: function(){
		// summary:
		//		Open the popup to the side of/underneath the current menu item
		// tags:
		//		protected

		this._stopPopupTimer();
		var from_item = this.focusedChild;
		if(!from_item){ return; } // the focused child lost focus since the timer was started
		var popup = from_item.popup;
		if(popup.isShowingNow){ return; }
		if(this.currentPopup){
			this._stopPendingCloseTimer(this.currentPopup);
			dijit.popup.close(this.currentPopup);
		}
		popup.parentMenu = this;
		popup.from_item = from_item; // helps finding the parent item that should be focused for this popup
		var self = this;
		dijit.popup.open({
			parent: this,
			popup: popup,
			around: from_item.domNode,
			orient: this._orient || (this.isLeftToRight() ?
									{'TR': 'TL', 'TL': 'TR', 'BR': 'BL', 'BL': 'BR'} :
									{'TL': 'TR', 'TR': 'TL', 'BL': 'BR', 'BR': 'BL'}),
			onCancel: function(){ // called when the child menu is canceled
				// set isActive=false (_closeChild vs _cleanUp) so that subsequent hovering will NOT open child menus
				// which seems aligned with the UX of most applications (e.g. notepad, wordpad, paint shop pro)
				self.focusChild(from_item);	// put focus back on my node
				self._cleanUp();			// close the submenu (be sure this is done _after_ focus is moved)
				from_item._setSelected(true); // oops, _cleanUp() deselected the item
				self.focusedChild = from_item;	// and unset focusedChild
			},
			onExecute: dojo.hitch(this, "_cleanUp")
		});

		this.currentPopup = popup;
		// detect mouseovers to handle lazy mouse movements that temporarily focus other menu items
		popup.connect(popup.domNode, "onmouseenter", dojo.hitch(self, "_onPopupHover")); // cleaned up when the popped-up widget is destroyed on close

		if(popup.focus){
			// If user is opening the popup via keyboard (right arrow, or down arrow for MenuBar),
			// if the cursor happens to collide with the popup, it will generate an onmouseover event
			// even though the mouse wasn't moved.   Use a setTimeout() to call popup.focus so that
			// our focus() call overrides the onmouseover event, rather than vice-versa.  (#8742)
			popup._focus_timer = setTimeout(dojo.hitch(popup, function(){
				this._focus_timer = null;
				this.focus();
			}), 0);
		}
	},

	_markActive: function(){
		// summary:
		//              Mark this menu's state as active.
		//		Called when this Menu gets focus from:
		//			1) clicking it (mouse or via space/arrow key)
		//			2) being opened by a parent menu.
		//		This is not called just from mouse hover.
		//		Focusing a menu via TAB does NOT automatically set isActive
		//		since TAB is a navigation operation and not a selection one.
		//		For Windows apps, pressing the ALT key focuses the menubar
		//		menus (similar to TAB navigation) but the menu is not active
		//		(ie no dropdown) until an item is clicked.
		this.isActive = true;
		dojo.addClass(this.domNode, "dijitMenuActive");
		dojo.removeClass(this.domNode, "dijitMenuPassive");
	},

	onOpen: function(/*Event*/ e){
		// summary:
		//		Callback when this menu is opened.
		//		This is called by the popup manager as notification that the menu
		//		was opened.
		// tags:
		//		private

		this.isShowingNow = true;
		this._markActive();
	},

	_markInactive: function(){
		// summary:
		//		Mark this menu's state as inactive.
		this.isActive = false; // don't do this in _onBlur since the state is pending-close until we get here
		dojo.removeClass(this.domNode, "dijitMenuActive");
		dojo.addClass(this.domNode, "dijitMenuPassive");
	},

	onClose: function(){
		// summary:
		//		Callback when this menu is closed.
		//		This is called by the popup manager as notification that the menu
		//		was closed.
		// tags:
		//		private

		this._stopFocusTimer();
		this._markInactive();
		this.isShowingNow = false;
		this.parentMenu = null;
	},

	_closeChild: function(){
		// summary:
		//		Called when submenu is clicked or focus is lost.  Close hierarchy of menus.
		// tags:
		//		private
		this._stopPopupTimer();
		if(this.focusedChild){ // unhighlight the focused item
			this.focusedChild._setSelected(false);
			this.focusedChild._onUnhover();
			this.focusedChild = null;
		}
		if(this.currentPopup){
			// Close all popups that are open and descendants of this menu
			dijit.popup.close(this.currentPopup);
			this.currentPopup = null;
		}
	},

	_onItemFocus: function(/*MenuItem*/ item){
		// summary:
		//		Called when child of this Menu gets focus from:
		//			1) clicking it
		//			2) tabbing into it
		//			3) being opened by a parent menu.
		//		This is not called just from mouse hover.
		if(this._hoveredChild && this._hoveredChild != item){
			this._hoveredChild._onUnhover(); // any previous mouse movement is trumped by focus selection
		}
	},

	_onBlur: function(){
		// summary:
		//		Called when focus is moved away from this Menu and it's submenus.
		// tags:
		//		protected
		this._cleanUp();
		this.inherited(arguments);
	},

	_cleanUp: function(){
		// summary:
		//		Called when the user is done with this menu.  Closes hierarchy of menus.
		// tags:
		//		private

		this._closeChild(); // don't call this.onClose since that's incorrect for MenuBar's that never close
		if(typeof this.isShowingNow == 'undefined'){ // non-popup menu doesn't call onClose
			this._markInactive();
		}
	}
});

dojo.declare("dijit.Menu",
	dijit._MenuBase,
	{
	// summary
	//		A context menu you can assign to multiple elements

	// TODO: most of the code in here is just for context menu (right-click menu)
	// support.  In retrospect that should have been a separate class (dijit.ContextMenu).
	// Split them for 2.0

	constructor: function(){
		this._bindings = [];
	},

	templateString: dojo.cache("dijit", "templates/Menu.html", "<table class=\"dijit dijitMenu dijitMenuPassive dijitReset dijitMenuTable\" waiRole=\"menu\" tabIndex=\"${tabIndex}\" dojoAttachEvent=\"onkeypress:_onKeyPress\" cellspacing=0>\n\t<tbody class=\"dijitReset\" dojoAttachPoint=\"containerNode\"></tbody>\n</table>\n"),

	baseClass: "dijitMenu",

	// targetNodeIds: [const] String[]
	//		Array of dom node ids of nodes to attach to.
	//		Fill this with nodeIds upon widget creation and it becomes context menu for those nodes.
	targetNodeIds: [],

	// contextMenuForWindow: [const] Boolean
	//		If true, right clicking anywhere on the window will cause this context menu to open.
	//		If false, must specify targetNodeIds.
	contextMenuForWindow: false,

	// leftClickToOpen: [const] Boolean
	//		If true, menu will open on left click instead of right click, similiar to a file menu.
	leftClickToOpen: false,

	// refocus: Boolean
	// 		When this menu closes, re-focus the element which had focus before it was opened.
	refocus: true,

	postCreate: function(){
		if(this.contextMenuForWindow){
			this.bindDomNode(dojo.body());
		}else{
			// TODO: should have _setTargetNodeIds() method to handle initialization and a possible
			// later attr('targetNodeIds', ...) call.   There's also a problem that targetNodeIds[]
			// gets stale after calls to bindDomNode()/unBindDomNode() as it still is just the original list (see #9610)
			dojo.forEach(this.targetNodeIds, this.bindDomNode, this);
		}
		var k = dojo.keys, l = this.isLeftToRight();
		this._openSubMenuKey = l ? k.RIGHT_ARROW : k.LEFT_ARROW;
		this._closeSubMenuKey = l ? k.LEFT_ARROW : k.RIGHT_ARROW;
		this.connectKeyNavHandlers([k.UP_ARROW], [k.DOWN_ARROW]);
	},

	_onKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handle keyboard based menu navigation.
		// tags:
		//		protected

		if(evt.ctrlKey || evt.altKey){ return; }

		switch(evt.charOrCode){
			case this._openSubMenuKey:
				this._moveToPopup(evt);
				dojo.stopEvent(evt);
				break;
			case this._closeSubMenuKey:
				if(this.parentMenu){
					if(this.parentMenu._isMenuBar){
						this.parentMenu.focusPrev();
					}else{
						this.onCancel(false);
					}
				}else{
					dojo.stopEvent(evt);
				}
				break;
		}
	},

	// thanks burstlib!
	_iframeContentWindow: function(/* HTMLIFrameElement */iframe_el){
		// summary:
		//		Returns the window reference of the passed iframe
		// tags:
		//		private
		var win = dojo.window.get(this._iframeContentDocument(iframe_el)) ||
			// Moz. TODO: is this available when defaultView isn't?
			this._iframeContentDocument(iframe_el)['__parent__'] ||
			(iframe_el.name && dojo.doc.frames[iframe_el.name]) || null;
		return win;	//	Window
	},

	_iframeContentDocument: function(/* HTMLIFrameElement */iframe_el){
		// summary:
		//		Returns a reference to the document object inside iframe_el
		// tags:
		//		protected
		var doc = iframe_el.contentDocument // W3
			|| (iframe_el.contentWindow && iframe_el.contentWindow.document) // IE
			|| (iframe_el.name && dojo.doc.frames[iframe_el.name] && dojo.doc.frames[iframe_el.name].document)
			|| null;
		return doc;	//	HTMLDocument
	},

	bindDomNode: function(/*String|DomNode*/ node){
		// summary:
		//		Attach menu to given node
		node = dojo.byId(node);

		var cn;	// Connect node

		// Support context menus on iframes.   Rather than binding to the iframe itself we need
		// to bind to the <body> node inside the iframe.
		if(node.tagName.toLowerCase() == "iframe"){
			var iframe = node,
				win = this._iframeContentWindow(iframe);
			cn = dojo.withGlobal(win, dojo.body);
		}else{
			
			// To capture these events at the top level, attach to <html>, not <body>.
			// Otherwise right-click context menu just doesn't work.
			cn = (node == dojo.body() ? dojo.doc.documentElement : node);
		}


		// "binding" is the object to track our connection to the node (ie, the parameter to bindDomNode())
		var binding = {
			node: node,
			iframe: iframe
		};

		// Save info about binding in _bindings[], and make node itself record index(+1) into
		// _bindings[] array.   Prefix w/_dijitMenu to avoid setting an attribute that may
		// start with a number, which fails on FF/safari.
		dojo.attr(node, "_dijitMenu" + this.id, this._bindings.push(binding));

		// Setup the connections to monitor click etc., unless we are connecting to an iframe which hasn't finished
		// loading yet, in which case we need to wait for the onload event first, and then connect
		// On linux Shift-F10 produces the oncontextmenu event, but on Windows it doesn't, so
		// we need to monitor keyboard events in addition to the oncontextmenu event.
		var doConnects = dojo.hitch(this, function(cn){
			return [
				// TODO: when leftClickToOpen is true then shouldn't space/enter key trigger the menu,
				// rather than shift-F10?
				dojo.connect(cn, this.leftClickToOpen ? "onclick" : "oncontextmenu", this, function(evt){
					// Schedule context menu to be opened unless it's already been scheduled from onkeydown handler
					dojo.stopEvent(evt);
					this._scheduleOpen(evt.target, iframe, {x: evt.pageX, y: evt.pageY});
				}),
				dojo.connect(cn, "onkeydown", this, function(evt){
					if(evt.shiftKey && evt.keyCode == dojo.keys.F10){
						dojo.stopEvent(evt);
						this._scheduleOpen(evt.target, iframe);	// no coords - open near target node
					}
				})
			];	
		});
		binding.connects = cn ? doConnects(cn) : [];

		if(iframe){
			// Setup handler to [re]bind to the iframe when the contents are initially loaded,
			// and every time the contents change.
			// Need to do this b/c we are actually binding to the iframe's <body> node.
			// Note: can't use dojo.connect(), see #9609.

			binding.onloadHandler = dojo.hitch(this, function(){
				// want to remove old connections, but IE throws exceptions when trying to
				// access the <body> node because it's already gone, or at least in a state of limbo

				var win = this._iframeContentWindow(iframe);
					cn = dojo.withGlobal(win, dojo.body);
				binding.connects = doConnects(cn);
			});
			if(iframe.addEventListener){
				iframe.addEventListener("load", binding.onloadHandler, false);
			}else{
				iframe.attachEvent("onload", binding.onloadHandler);
			}
		}
	},

	unBindDomNode: function(/*String|DomNode*/ nodeName){
		// summary:
		//		Detach menu from given node

		var node;
		try{
			node = dojo.byId(nodeName);
		}catch(e){
			// On IE the dojo.byId() call will get an exception if the attach point was
			// the <body> node of an <iframe> that has since been reloaded (and thus the
			// <body> node is in a limbo state of destruction.
			return;
		}

		// node["_dijitMenu" + this.id] contains index(+1) into my _bindings[] array
		var attrName = "_dijitMenu" + this.id;
		if(node && dojo.hasAttr(node, attrName)){
			var bid = dojo.attr(node, attrName)-1, b = this._bindings[bid];
			dojo.forEach(b.connects, dojo.disconnect);

			// Remove listener for iframe onload events
			var iframe = b.iframe;
			if(iframe){
				if(iframe.removeEventListener){
					iframe.removeEventListener("load", b.onloadHandler, false);
				}else{
					iframe.detachEvent("onload", b.onloadHandler);
				}
			}

			dojo.removeAttr(node, attrName);
			delete this._bindings[bid];
		}
	},

	_scheduleOpen: function(/*DomNode?*/ target, /*DomNode?*/ iframe, /*Object?*/ coords){
		// summary:
		//		Set timer to display myself.  Using a timer rather than displaying immediately solves
		//		two problems:
		//
		//		1. IE: without the delay, focus work in "open" causes the system
		//		context menu to appear in spite of stopEvent.
		//
		//		2. Avoid double-shows on linux, where shift-F10 generates an oncontextmenu event
		//		even after a dojo.stopEvent(e).  (Shift-F10 on windows doesn't generate the
		//		oncontextmenu event.)

		if(!this._openTimer){
			this._openTimer = setTimeout(dojo.hitch(this, function(){
				delete this._openTimer;
				this._openMyself({
					target: target,
					iframe: iframe,
					coords: coords
				});
			}), 1);
		}
	},

	_openMyself: function(args){
		// summary:
		//		Internal function for opening myself when the user does a right-click or something similar.
		// args:
		//		This is an Object containing:
		//		* target:
		//			The node that is being clicked
		//		* iframe:
		//			If an <iframe> is being clicked, iframe points to that iframe
		//		* coords:
		//			Put menu at specified x/y position in viewport, or if iframe is
		//			specified, then relative to iframe.
		//
		//		_openMyself() formerly took the event object, and since various code references
		//		evt.target (after connecting to _openMyself()), using an Object for parameters
		//		(so that old code still works).

		var target = args.target,
			iframe = args.iframe,
			coords = args.coords;

		// Get coordinates to open menu, either at specified (mouse) position or (if triggered via keyboard)
		// then near the node the menu is assigned to.
		if(coords){
			if(iframe){
				// Specified coordinates are on <body> node of an <iframe>, convert to match main document
				var od = target.ownerDocument,
					ifc = dojo.position(iframe, true),
					win = this._iframeContentWindow(iframe),
					scroll = dojo.withGlobal(win, "_docScroll", dojo);
	
				var cs = dojo.getComputedStyle(iframe),
					tp = dojo._toPixelValue,
					left = (dojo.isIE && dojo.isQuirks ? 0 : tp(iframe, cs.paddingLeft)) + (dojo.isIE && dojo.isQuirks ? tp(iframe, cs.borderLeftWidth) : 0),
					top = (dojo.isIE && dojo.isQuirks ? 0 : tp(iframe, cs.paddingTop)) + (dojo.isIE && dojo.isQuirks ? tp(iframe, cs.borderTopWidth) : 0);

				coords.x += ifc.x + left - scroll.x;
				coords.y += ifc.y + top - scroll.y;
			}
		}else{
			coords = dojo.position(target, true);
			coords.x += 10;
			coords.y += 10;
		}

		var self=this;
		var savedFocus = dijit.getFocus(this);
		function closeAndRestoreFocus(){
			// user has clicked on a menu or popup
			if(self.refocus){
				dijit.focus(savedFocus);
			}
			dijit.popup.close(self);
		}
		dijit.popup.open({
			popup: this,
			x: coords.x,
			y: coords.y,
			onExecute: closeAndRestoreFocus,
			onCancel: closeAndRestoreFocus,
			orient: this.isLeftToRight() ? 'L' : 'R'
		});
		this.focus();

		this._onBlur = function(){
			this.inherited('_onBlur', arguments);
			// Usually the parent closes the child widget but if this is a context
			// menu then there is no parent
			dijit.popup.close(this);
			// don't try to restore focus; user has clicked another part of the screen
			// and set focus there
		};
	},

	uninitialize: function(){
 		dojo.forEach(this._bindings, function(b){ if(b){ this.unBindDomNode(b.node); } }, this);
 		this.inherited(arguments);
	}
}
);

// Back-compat (TODO: remove in 2.0)






}

if(!dojo._hasResource["itme.DojoMenuOverride"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.DojoMenuOverride"] = true;
dojo.provide("itme.DojoMenuOverride");


// this class differs from the dojo menu widget in one way.  
//  IF the menu item that receives the click is a 'CheckedMenuItem' it will NOT close the menu
//  Deviation from dojo called out below with DEVIATION
//  Confirmed to work with dojo 1.4 & 1.5
dojo.declare("itme.DojoMenuOverride", dijit.Menu,
 {
    onItemClick: function(/*dijit._Widget*/ item, /*Event*/ evt){
    // summary:
    //    Handle clicks on an item.
    // tags:
    //    private
    

    // this can't be done in _onFocus since the _onFocus events occurs asynchronously
    if(typeof this.isShowingNow == 'undefined'){ // non-popup menu
      this._markActive();
    }

    this.focusChild(item);

    if(item.disabled){ return false; }

    if(item.popup){
      this._openPopup();
    }else{
      // before calling user defined handler, close hierarchy of menus
      // and restore focus to place it was when menu was opened
      // BEGIN DEVIATION : Only call onExecute (closes the menu) if not a checkedMenuItem  
      if ( item.declaredClass != "dijit.CheckedMenuItem" )
      // END DEVIATION
       this.onExecute();

      // user defined handler for click
      item.onClick(evt);
    }
  }

 });

}

if(!dojo._hasResource["dojox.json.ref"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.json.ref"] = true;
dojo.provide("dojox.json.ref");



dojox.json.ref = {
	// summary:
	// 		Adds advanced JSON {de}serialization capabilities to the base json library.
	// 		This enhances the capabilities of dojo.toJson and dojo.fromJson,
	// 		adding referencing support, date handling, and other extra format handling.
	// 		On parsing, references are resolved. When references are made to
	// 		ids/objects that have been loaded yet, the loader function will be set to
	// 		_loadObject to denote a lazy loading (not loaded yet) object. 


	resolveJson: function(/*Object*/ root,/*Object?*/ args){
		// summary:
		// 		Indexes and resolves references in the JSON object.
		// description:
		// 		A JSON Schema object that can be used to advise the handling of the JSON (defining ids, date properties, urls, etc)
		//
		// root:
		//		The root object of the object graph to be processed
		// args:
		//		Object with additional arguments:
		//
		// The *index* parameter.
		//		This is the index object (map) to use to store an index of all the objects. 
		// 		If you are using inter-message referencing, you must provide the same object for each call.
		// The *defaultId* parameter.
		//		This is the default id to use for the root object (if it doesn't define it's own id)
		//	The *idPrefix* parameter.
		//		This the prefix to use for the ids as they enter the index. This allows multiple tables 
		// 		to use ids (that might otherwise collide) that enter the same global index. 
		// 		idPrefix should be in the form "/Service/".  For example,
		//		if the idPrefix is "/Table/", and object is encountered {id:"4",...}, this would go in the
		//		index as "/Table/4".
		//	The *idAttribute* parameter.
		//		This indicates what property is the identity property. This defaults to "id"
		//	The *assignAbsoluteIds* parameter.
		//		This indicates that the resolveJson should assign absolute ids (__id) as the objects are being parsed.
		//  
		// The *schemas* parameter
		//		This provides a map of schemas, from which prototypes can be retrieved
		// The *loader* parameter
		//		This is a function that is called added to the reference objects that can't be resolved (lazy objects)
		// return:
		//		An object, the result of the processing
		args = args || {};
		var idAttribute = args.idAttribute || 'id';
		var refAttribute = this.refAttribute;
		var idAsRef = args.idAsRef;
		var prefix = args.idPrefix || ''; 
		var assignAbsoluteIds = args.assignAbsoluteIds;
		var index = args.index || {}; // create an index if one doesn't exist
		var timeStamps = args.timeStamps;
		var ref,reWalk=[];
		var pathResolveRegex = /^(.*\/)?(\w+:\/\/)|[^\/\.]+\/\.\.\/|^.*\/(\/)/;
		var addProp = this._addProp;
		var F = function(){};
		function walk(it, stop, defaultId, needsPrefix, schema, defaultObject){
			// this walks the new graph, resolving references and making other changes
		 	var i, update, val, id = idAttribute in it ? it[idAttribute] : defaultId;
		 	if(idAttribute in it || ((id !== undefined) && needsPrefix)){
		 		id = (prefix + id).replace(pathResolveRegex,'$2$3');
		 	}
		 	var target = defaultObject || it;
			if(id !== undefined){ // if there is an id available...
				if(assignAbsoluteIds){
					it.__id = id;
				}
				if(args.schemas && (!(it instanceof Array)) && // won't try on arrays to do prototypes, plus it messes with queries 
		 					(val = id.match(/^(.+\/)[^\.\[]*$/))){ // if it has a direct table id (no paths)
		 			schema = args.schemas[val[1]];
				} 
				// if the id already exists in the system, we should use the existing object, and just 
				// update it... as long as the object is compatible
				if(index[id] && ((it instanceof Array) == (index[id] instanceof Array))){ 
					target = index[id];
					delete target.$ref; // remove this artifact
					delete target._loadObject;
					update = true;
				}else{
				 	var proto = schema && schema.prototype; // and if has a prototype
					if(proto){
						// if the schema defines a prototype, that needs to be the prototype of the object
						F.prototype = proto;
						target = new F();
					}
				}
				index[id] = target; // add the prefix, set _id, and index it
				if(timeStamps){
					timeStamps[id] = args.time;
				}
			}
			while(schema){
				var properties = schema.properties;
				if(properties){
					for(i in it){
						var propertyDefinition = properties[i];
						if(propertyDefinition && propertyDefinition.format == 'date-time' && typeof it[i] == 'string'){
							it[i] = dojo.date.stamp.fromISOString(it[i]);
						}
					}
				}
				schema = schema["extends"];
			}
			var length = it.length;
			for(i in it){
				if(i==length){
					break;		
				}
				if(it.hasOwnProperty(i)){
					val=it[i];
					if((typeof val =='object') && val && !(val instanceof Date) && i != '__parent'){
						ref=val[refAttribute] || (idAsRef && val[idAttribute]);
						if(!ref || !val.__parent){
							if(it != reWalk){
								val.__parent = target;
							}
						}
						if(ref){ // a reference was found
							// make sure it is a safe reference
							delete it[i];// remove the property so it doesn't resolve to itself in the case of id.propertyName lazy values
							var path = ref.toString().replace(/(#)([^\.\[])/,'$1.$2').match(/(^([^\[]*\/)?[^#\.\[]*)#?([\.\[].*)?/); // divide along the path
							if((ref = (path[1]=='$' || path[1]=='this' || path[1]=='') ? root : index[(prefix + path[1]).replace(pathResolveRegex,'$2$3')])){  // a $ indicates to start with the root, otherwise start with an id
								// if there is a path, we will iterate through the path references
								if(path[3]){
									path[3].replace(/(\[([^\]]+)\])|(\.?([^\.\[]+))/g,function(t,a,b,c,d){
										ref = ref && ref[b ? b.replace(/[\"\'\\]/,'') : d];
									});
								}
							}
							if(ref){
								val = ref;
							}else{
								// otherwise, no starting point was found (id not found), if stop is set, it does not exist, we have
								// unloaded reference, if stop is not set, it may be in a part of the graph not walked yet,
								// we will wait for the second loop
								if(!stop){
									var rewalking;
									if(!rewalking){
										reWalk.push(target); // we need to rewalk it to resolve references
									}
									rewalking = true; // we only want to add it once
									val = walk(val, false, val[refAttribute], true, propertyDefinition);
									// create a lazy loaded object
									val._loadObject = args.loader;
								}
							}
						}else{
							if(!stop){ // if we are in stop, that means we are in the second loop, and we only need to check this current one,
								// further walking may lead down circular loops
								val = walk(
									val,
									reWalk==it,
									id === undefined ? undefined : addProp(id, i), // the default id to use
									false,
									propertyDefinition, 
									// if we have an existing object child, we want to 
									// maintain it's identity, so we pass it as the default object
									target != it && typeof target[i] == 'object' && target[i] 
								);
							}
						}
					}
					it[i] = val;
					if(target!=it && !target.__isDirty){// do updates if we are updating an existing object and it's not dirty				
						var old = target[i];
						target[i] = val; // only update if it changed
						if(update && val !== old && // see if it is different 
								!target._loadObject && // no updates if we are just lazy loading
								!(i.charAt(0) == '_' && i.charAt(1) == '_') && i != "$ref" &&  
								!(val instanceof Date && old instanceof Date && val.getTime() == old.getTime()) && // make sure it isn't an identical date
								!(typeof val == 'function' && typeof old == 'function' && val.toString() == old.toString()) && // make sure it isn't an indentical function
								index.onUpdate){
							index.onUpdate(target,i,old,val); // call the listener for each update
						}
					}
				}
			}
	
			if(update && (idAttribute in it || target instanceof Array)){
				// this means we are updating with a full representation of the object, we need to remove deleted
				for(i in target){
					if(!target.__isDirty && target.hasOwnProperty(i) && !it.hasOwnProperty(i) && !(i.charAt(0) == '_' && i.charAt(1) == '_') && !(target instanceof Array && isNaN(i))){
						if(index.onUpdate && i != "_loadObject" && i != "_idAttr"){
							index.onUpdate(target,i,target[i],undefined); // call the listener for each update
						}
						delete target[i];
						while(target instanceof Array && target.length && target[target.length-1] === undefined){
							// shorten the target if necessary
							target.length--;
						}
					}
				}
			}else{
				if(index.onLoad){
					index.onLoad(target);
				}
			}
			return target;
		}
		if(root && typeof root == 'object'){
			root = walk(root,false,args.defaultId, true); // do the main walk through
			walk(reWalk,false); // re walk any parts that were not able to resolve references on the first round
		}
		return root;
	},


	fromJson: function(/*String*/ str,/*Object?*/ args){
	// summary:
	// 		evaluates the passed string-form of a JSON object.
	//
	// str:
	//		a string literal of a JSON item, for instance:
	//			'{ "foo": [ "bar", 1, { "baz": "thud" } ] }'
	// args: See resolveJson
	//
	// return:
	//		An object, the result of the evaluation
		function ref(target){ // support call styles references as well
			var refObject = {};
			refObject[this.refAttribute] = target;
			return refObject;
		}
		try{
			var root = eval('(' + str + ')'); // do the eval
		}catch(e){
			throw new SyntaxError("Invalid JSON string: " + e.message + " parsing: "+ str);
		}		
		if(root){
			return this.resolveJson(root, args);
		}
		return root;
	},
	
	toJson: function(/*Object*/ it, /*Boolean?*/ prettyPrint, /*Object?*/ idPrefix, /*Object?*/ indexSubObjects){
		// summary:
		//		Create a JSON serialization of an object.
		//		This has support for referencing, including circular references, duplicate references, and out-of-message references
		// 		id and path-based referencing is supported as well and is based on http://www.json.com/2007/10/19/json-referencing-proposal-and-library/.
		//
		// it:
		//		an object to be serialized.
		//
		// prettyPrint:
		//		if true, we indent objects and arrays to make the output prettier.
		//		The variable dojo.toJsonIndentStr is used as the indent string
		//		-- to use something other than the default (tab),
		//		change that variable before calling dojo.toJson().
		//
		// idPrefix: The prefix that has been used for the absolute ids
		//
		// return:
		//		a String representing the serialized version of the passed object.
		var useRefs = this._useRefs;
		var addProp = this._addProp;
		var refAttribute = this.refAttribute;
		idPrefix = idPrefix || ''; // the id prefix for this context
		var paths={};
		var generated = {};
		function serialize(it,path,_indentStr){
			if(typeof it == 'object' && it){
				var value;
				if(it instanceof Date){ // properly serialize dates
					return '"' + dojo.date.stamp.toISOString(it,{zulu:true}) + '"';
				}
				var id = it.__id;
				if(id){ // we found an identifiable object, we will just serialize a reference to it... unless it is the root
					if(path != '#' && ((useRefs && !id.match(/#/)) || paths[id])){
						var ref = id;	
						if(id.charAt(0)!='#'){
							if(it.__clientId == id){
								ref = "cid:" + id;
							}else if(id.substring(0, idPrefix.length) == idPrefix){ // see if the reference is in the current context
								// a reference with a prefix matching the current context, the prefix should be removed
								ref = id.substring(idPrefix.length);
							}else{
								// a reference to a different context, assume relative url based referencing
								ref = id;
							}
						}
						var refObject = {};
						refObject[refAttribute] = ref;
						return serialize(refObject,'#');
					}
					path = id;
				}else{
					it.__id = path; // we will create path ids for other objects in case they are circular
					generated[path] = it;
				}
				paths[path] = it;// save it here so they can be deleted at the end
				_indentStr = _indentStr || "";
				var nextIndent = prettyPrint ? _indentStr + dojo.toJsonIndentStr : "";
				var newLine = prettyPrint ? "\n" : "";
				var sep = prettyPrint ? " " : "";
	
				if(it instanceof Array){
					var res = dojo.map(it, function(obj,i){
						var val = serialize(obj, addProp(path, i), nextIndent);
						if(typeof val != "string"){
							val = "undefined";
						}
						return newLine + nextIndent + val;
					});
					return "[" + res.join("," + sep) + newLine + _indentStr + "]";
				}
	
				var output = [];
				for(var i in it){
					if(it.hasOwnProperty(i)){
						var keyStr;
						if(typeof i == "number"){
							keyStr = '"' + i + '"';
						}else if(typeof i == "string" && (i.charAt(0) != '_' || i.charAt(1) != '_')){
							// we don't serialize our internal properties __id and __clientId
							keyStr = dojo._escapeString(i);
						}else{
							// skip non-string or number keys
							continue;
						}
						var val = serialize(it[i],addProp(path, i),nextIndent);
						if(typeof val != "string"){
							// skip non-serializable values
							continue;
						}
						output.push(newLine + nextIndent + keyStr + ":" + sep + val);
					}
				}
				return "{" + output.join("," + sep) + newLine + _indentStr + "}";
			}else if(typeof it == "function" && dojox.json.ref.serializeFunctions){
				return it.toString();
			}
	
			return dojo.toJson(it); // use the default serializer for primitives
		}
		var json = serialize(it,'#','');
		if(!indexSubObjects){
			for(var i in generated)  {// cleanup the temporary path-generated ids
				delete generated[i].__id;
			}
		}
		return json;
	},
	_addProp: function(id, prop){
		return id + (id.match(/#/) ? id.length == 1 ? '' : '.' : '#') + prop;
	},
	//	refAttribute: String
	//		This indicates what property is the reference property. This acts like the idAttribute
	// 		except that this is used to indicate the current object is a reference or only partially 
	// 		loaded. This defaults to "$ref". 
	refAttribute: "$ref",
	_useRefs: false,
	serializeFunctions: false
}

}

if(!dojo._hasResource["dijit.form._FormWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormWidget"] = true;
dojo.provide("dijit.form._FormWidget");







dojo.declare("dijit.form._FormWidget", [dijit._Widget, dijit._Templated, dijit._CssStateMixin],
	{
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as <checkbox> or <button>,
	//		which can be children of a <form> node or a `dijit.form.Form` widget.
	//
	// description:
	//		Represents a single HTML element.
	//		All these widgets should have these attributes just like native HTML input elements.
	//		You can set them during widget construction or afterwards, via `dijit._Widget.attr`.
	//
	//		They also share some common methods.

	// name: String
	//		Name used when submitting form; same as "name" attribute or plain HTML elements
	name: "",

	// alt: String
	//		Corresponds to the native HTML <input> element's attribute.
	alt: "",

	// value: String
	//		Corresponds to the native HTML <input> element's attribute.
	value: "",

	// type: String
	//		Corresponds to the native HTML <input> element's attribute.
	type: "text",

	// tabIndex: Integer
	//		Order fields are traversed when user hits the tab key
	tabIndex: "0",

	// disabled: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "disabled='disabled'", or just "disabled".
	disabled: false,

	// intermediateChanges: Boolean
	//		Fires onChange for each value change or only on demand
	intermediateChanges: false,

	// scrollOnFocus: Boolean
	//		On focus, should this widget scroll into view?
	scrollOnFocus: true,

	// These mixins assume that the focus node is an INPUT, as many but not all _FormWidgets are.
	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
		value: "focusNode",
		id: "focusNode",
		tabIndex: "focusNode",
		alt: "focusNode",
		title: "focusNode"
	}),

	postMixInProperties: function(){
		// Setup name=foo string to be referenced from the template (but only if a name has been specified)
		// Unfortunately we can't use attributeMap to set the name due to IE limitations, see #8660
		// Regarding escaping, see heading "Attribute values" in
		// http://www.w3.org/TR/REC-html40/appendix/notes.html#h-B.3.2
		this.nameAttrSetting = this.name ? ('name="' + this.name.replace(/'/g, "&quot;") + '"') : '';
		this.inherited(arguments);
	},

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.domNode, "onmousedown", "_onMouseDown");
	},

	_setDisabledAttr: function(/*Boolean*/ value){
		this.disabled = value;
		dojo.attr(this.focusNode, 'disabled', value);
		if(this.valueNode){
			dojo.attr(this.valueNode, 'disabled', value);
		}
		dijit.setWaiState(this.focusNode, "disabled", value);

		if(value){
			// reset these, because after the domNode is disabled, we can no longer receive
			// mouse related events, see #4200
			this._hovering = false;
			this._active = false;

			// clear tab stop(s) on this widget's focusable node(s)  (ComboBox has two focusable nodes)
			var attachPointNames = "tabIndex" in this.attributeMap ? this.attributeMap.tabIndex : "focusNode";
			dojo.forEach(dojo.isArray(attachPointNames) ? attachPointNames : [attachPointNames], function(attachPointName){
				var node = this[attachPointName];
				// complex code because tabIndex=-1 on a <div> doesn't work on FF
				if(dojo.isWebKit || dijit.hasDefaultTabStop(node)){	// see #11064 about webkit bug
					node.setAttribute('tabIndex', "-1");
				}else{
					node.removeAttribute('tabIndex');				
				}
			}, this);
		}else{
			this.focusNode.setAttribute('tabIndex', this.tabIndex);
		}
	},

	setDisabled: function(/*Boolean*/ disabled){
		// summary:
		//		Deprecated.   Use set('disabled', ...) instead.
		dojo.deprecated("setDisabled("+disabled+") is deprecated. Use set('disabled',"+disabled+") instead.", "", "2.0");
		this.set('disabled', disabled);
	},

	_onFocus: function(e){
		if(this.scrollOnFocus){
			dojo.window.scrollIntoView(this.domNode);
		}
		this.inherited(arguments);
	},

	isFocusable: function(){
		// summary:
		//		Tells if this widget is focusable or not.   Used internally by dijit.
		// tags:
		//		protected
		return !this.disabled && this.focusNode && (dojo.style(this.domNode, "display") != "none");
	},

	focus: function(){
		// summary:
		//		Put focus on this widget
		if(!this.disabled){
			dijit.focus(this.focusNode);
		}
	},

	compare: function(/*anything*/val1, /*anything*/val2){
		// summary:
		//		Compare 2 values (as returned by attr('value') for this widget).
		// tags:
		//		protected
		if(typeof val1 == "number" && typeof val2 == "number"){
			return (isNaN(val1) && isNaN(val2)) ? 0 : val1 - val2;
		}else if(val1 > val2){
			return 1;
		}else if(val1 < val2){
			return -1;
		}else{
			return 0;
		}
	},

	onChange: function(newValue){
		// summary:
		//		Callback when this widget's value is changed.
		// tags:
		//		callback
	},

	// _onChangeActive: [private] Boolean
	//		Indicates that changes to the value should call onChange() callback.
	//		This is false during widget initialization, to avoid calling onChange()
	//		when the initial value is set.
	_onChangeActive: false,

	_handleOnChange: function(/*anything*/ newValue, /* Boolean? */ priorityChange){
		// summary:
		//		Called when the value of the widget is set.  Calls onChange() if appropriate
		// newValue:
		//		the new value
		// priorityChange:
		//		For a slider, for example, dragging the slider is priorityChange==false,
		//		but on mouse up, it's priorityChange==true.  If intermediateChanges==true,
		//		onChange is only called form priorityChange=true events.
		// tags:
		//		private
		this._lastValue = newValue;
		if(this._lastValueReported == undefined && (priorityChange === null || !this._onChangeActive)){
			// this block executes not for a change, but during initialization,
			// and is used to store away the original value (or for ToggleButton, the original checked state)
			this._resetValue = this._lastValueReported = newValue;
		}
		if((this.intermediateChanges || priorityChange || priorityChange === undefined) &&
			((typeof newValue != typeof this._lastValueReported) ||
				this.compare(newValue, this._lastValueReported) != 0)){
			this._lastValueReported = newValue;
			if(this._onChangeActive){
				if(this._onChangeHandle){
					clearTimeout(this._onChangeHandle);
				}
				// setTimout allows hidden value processing to run and
				// also the onChange handler can safely adjust focus, etc
				this._onChangeHandle = setTimeout(dojo.hitch(this,
					function(){
						this._onChangeHandle = null;
						this.onChange(newValue);
					}), 0); // try to collapse multiple onChange's fired faster than can be processed
			}
		}
	},

	create: function(){
		// Overrides _Widget.create()
		this.inherited(arguments);
		this._onChangeActive = true;
	},

	destroy: function(){
		if(this._onChangeHandle){ // destroy called before last onChange has fired
			clearTimeout(this._onChangeHandle);
			this.onChange(this._lastValueReported);
		}
		this.inherited(arguments);
	},

	setValue: function(/*String*/ value){
		// summary:
		//		Deprecated.   Use set('value', ...) instead.
		dojo.deprecated("dijit.form._FormWidget:setValue("+value+") is deprecated.  Use set('value',"+value+") instead.", "", "2.0");
		this.set('value', value);
	},

	getValue: function(){
		// summary:
		//		Deprecated.   Use get('value') instead.
		dojo.deprecated(this.declaredClass+"::getValue() is deprecated. Use get('value') instead.", "", "2.0");
		return this.get('value');
	},
	
	_onMouseDown: function(e){
		// If user clicks on the button, even if the mouse is released outside of it,
		// this button should get focus (to mimics native browser buttons).
		// This is also needed on chrome because otherwise buttons won't get focus at all,
		// which leads to bizarre focus restore on Dialog close etc.
		if(!e.ctrlKey && this.isFocusable()){ // !e.ctrlKey to ignore right-click on mac
			// Set a global event to handle mouseup, so it fires properly
			// even if the cursor leaves this.domNode before the mouse up event.
			var mouseUpConnector = this.connect(dojo.body(), "onmouseup", function(){
				if (this.isFocusable()) {
					this.focus();
				}
				this.disconnect(mouseUpConnector);
			});
		}
	}
});

dojo.declare("dijit.form._FormValueWidget", dijit.form._FormWidget,
{
	// summary:
	//		Base class for widgets corresponding to native HTML elements such as <input> or <select> that have user changeable values.
	// description:
	//		Each _FormValueWidget represents a single input value, and has a (possibly hidden) <input> element,
	//		to which it serializes it's input value, so that form submission (either normal submission or via FormBind?)
	//		works as expected.

	// Don't attempt to mixin the 'type', 'name' attributes here programatically -- they must be declared
	// directly in the template as read by the parser in order to function. IE is known to specifically
	// require the 'name' attribute at element creation time.   See #8484, #8660.
	// TODO: unclear what that {value: ""} is for; FormWidget.attributeMap copies value to focusNode,
	// so maybe {value: ""} is so the value *doesn't* get copied to focusNode?
	// Seems like we really want value removed from attributeMap altogether
	// (although there's no easy way to do that now)

	// readOnly: Boolean
	//		Should this widget respond to user input?
	//		In markup, this is specified as "readOnly".
	//		Similar to disabled except readOnly form values are submitted.
	readOnly: false,

	attributeMap: dojo.delegate(dijit.form._FormWidget.prototype.attributeMap, {
		value: "",
		readOnly: "focusNode"
	}),

	_setReadOnlyAttr: function(/*Boolean*/ value){
		this.readOnly = value;
		dojo.attr(this.focusNode, 'readOnly', value);
		dijit.setWaiState(this.focusNode, "readonly", value);
	},

	postCreate: function(){
		this.inherited(arguments);

		if(dojo.isIE){ // IE won't stop the event with keypress
			this.connect(this.focusNode || this.domNode, "onkeydown", this._onKeyDown);
		}
		// Update our reset value if it hasn't yet been set (because this.set()
		// is only called when there *is* a value)
		if(this._resetValue === undefined){
			this._resetValue = this.value;
		}
	},

	_setValueAttr: function(/*anything*/ newValue, /*Boolean, optional*/ priorityChange){
		// summary:
		//		Hook so attr('value', value) works.
		// description:
		//		Sets the value of the widget.
		//		If the value has changed, then fire onChange event, unless priorityChange
		//		is specified as null (or false?)
		this.value = newValue;
		this._handleOnChange(newValue, priorityChange);
	},

	_getValueAttr: function(){
		// summary:
		//		Hook so attr('value') works.
		return this._lastValue;
	},

	undo: function(){
		// summary:
		//		Restore the value to the last value passed to onChange
		this._setValueAttr(this._lastValueReported, false);
	},

	reset: function(){
		// summary:
		//		Reset the widget's value to what it was at initialization time
		this._hasBeenBlurred = false;
		this._setValueAttr(this._resetValue, true);
	},

	_onKeyDown: function(e){
		if(e.keyCode == dojo.keys.ESCAPE && !(e.ctrlKey || e.altKey || e.metaKey)){
			var te;
			if(dojo.isIE){
				e.preventDefault(); // default behavior needs to be stopped here since keypress is too late
				te = document.createEventObject();
				te.keyCode = dojo.keys.ESCAPE;
				te.shiftKey = e.shiftKey;
				e.srcElement.fireEvent('onkeypress', te);
			}
		}
	},

	_layoutHackIE7: function(){
		// summary:
		//		Work around table sizing bugs on IE7 by forcing redraw

		if(dojo.isIE == 7){ // fix IE7 layout bug when the widget is scrolled out of sight
			var domNode = this.domNode;
			var parent = domNode.parentNode;
			var pingNode = domNode.firstChild || domNode; // target node most unlikely to have a custom filter
			var origFilter = pingNode.style.filter; // save custom filter, most likely nothing
			var _this = this;
			while(parent && parent.clientHeight == 0){ // search for parents that haven't rendered yet
				(function ping(){
					var disconnectHandle = _this.connect(parent, "onscroll",
						function(e){
							_this.disconnect(disconnectHandle); // only call once
							pingNode.style.filter = (new Date()).getMilliseconds(); // set to anything that's unique
							setTimeout(function(){ pingNode.style.filter = origFilter }, 0); // restore custom filter, if any
						}
					);
				})();
				parent = parent.parentNode;
			}
		}
	}
});

}

if(!dojo._hasResource["dijit._HasDropDown"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._HasDropDown"] = true;
dojo.provide("dijit._HasDropDown");




dojo.declare("dijit._HasDropDown",
	null,
	{
		// summary:
		//		Mixin for widgets that need drop down ability.

		// _buttonNode: [protected] DomNode
		//		The button/icon/node to click to display the drop down.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then either focusNode or domNode (if focusNode is also missing) will be used.
		_buttonNode: null,

		// _arrowWrapperNode: [protected] DomNode
		//		Will set CSS class dijitUpArrow, dijitDownArrow, dijitRightArrow etc. on this node depending
		//		on where the drop down is set to be positioned.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then _buttonNode will be used.
		_arrowWrapperNode: null,

		// _popupStateNode: [protected] DomNode
		//		The node to set the popupActive class on.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then focusNode or _buttonNode (if focusNode is missing) will be used.
		_popupStateNode: null,

		// _aroundNode: [protected] DomNode
		//		The node to display the popup around.
		//		Can be set via a dojoAttachPoint assignment.
		//		If missing, then domNode will be used.
		_aroundNode: null,

		// dropDown: [protected] Widget
		//		The widget to display as a popup.  This widget *must* be
		//		defined before the startup function is called.
		dropDown: null,

		// autoWidth: [protected] Boolean
		//		Set to true to make the drop down at least as wide as this
		//		widget.  Set to false if the drop down should just be its
		//		default width
		autoWidth: true,

		// forceWidth: [protected] Boolean
		//		Set to true to make the drop down exactly as wide as this
		//		widget.  Overrides autoWidth.
		forceWidth: false,

		// maxHeight: [protected] Integer
		//		The max height for our dropdown.  Set to 0 for no max height.
		//		any dropdown taller than this will have scrollbars
		maxHeight: 0,

		// dropDownPosition: [const] String[]
		//		This variable controls the position of the drop down.
		//		It's an array of strings with the following values:
		//
		//			* before: places drop down to the left of the target node/widget, or to the right in
		//			  the case of RTL scripts like Hebrew and Arabic
		//			* after: places drop down to the right of the target node/widget, or to the left in
		//			  the case of RTL scripts like Hebrew and Arabic
		//			* above: drop down goes above target node
		//			* below: drop down goes below target node
		//
		//		The list is positions is tried, in order, until a position is found where the drop down fits
		//		within the viewport.
		//
		dropDownPosition: ["below","above"],

		// _stopClickEvents: Boolean
		//		When set to false, the click events will not be stopped, in
		//		case you want to use them in your subwidget
		_stopClickEvents: true,

		_onDropDownMouseDown: function(/*Event*/ e){
			// summary:
			//		Callback when the user mousedown's on the arrow icon

			if(this.disabled || this.readOnly){ return; }

			this._docHandler = this.connect(dojo.doc, "onmouseup", "_onDropDownMouseUp");

			this.toggleDropDown();
		},

		_onDropDownMouseUp: function(/*Event?*/ e){
			// summary:
			//		Callback when the user lifts their mouse after mouse down on the arrow icon.
			//		If the drop is a simple menu and the mouse is over the menu, we execute it, otherwise, we focus our
			//		dropDown node.  If the event is missing, then we are not
			//		a mouseup event.
			//
			//		This is useful for the common mouse movement pattern
			//		with native browser <select> nodes:
			//			1. mouse down on the select node (probably on the arrow)
			//			2. move mouse to a menu item while holding down the mouse button
			//			3. mouse up.  this selects the menu item as though the user had clicked it.
			if(e && this._docHandler){
				this.disconnect(this._docHandler);
			}
			var dropDown = this.dropDown, overMenu = false;

			if(e && this._opened){
				// This code deals with the corner-case when the drop down covers the original widget,
				// because it's so large.  In that case mouse-up shouldn't select a value from the menu.
				// Find out if our target is somewhere in our dropdown widget,
				// but not over our _buttonNode (the clickable node)
				var c = dojo.position(this._buttonNode, true);
				if(!(e.pageX >= c.x && e.pageX <= c.x + c.w) ||
					!(e.pageY >= c.y && e.pageY <= c.y + c.h)){
					var t = e.target;
					while(t && !overMenu){
						if(dojo.hasClass(t, "dijitPopup")){
							overMenu = true;
						}else{
							t = t.parentNode;
						}
					}
					if(overMenu){
						t = e.target;
						if(dropDown.onItemClick){
							var menuItem;
							while(t && !(menuItem = dijit.byNode(t))){
								t = t.parentNode;
							}
							if(menuItem && menuItem.onClick && menuItem.getParent){
								menuItem.getParent().onItemClick(menuItem, e);
							}
						}
						return;
					}
				}
			}
			if(this._opened && dropDown.focus && dropDown.autoFocus !== false){
				// Focus the dropdown widget - do it on a delay so that we
				// don't steal our own focus.
				window.setTimeout(dojo.hitch(dropDown, "focus"), 1);
			}
		},

		_onDropDownClick: function(/*Event*/ e){
			// the drop down was already opened on mousedown/keydown; just need to call stopEvent()
			if(this._stopClickEvents){
				dojo.stopEvent(e);
			}			
		},

		_setupDropdown: function(){
			// summary:
			//		set up nodes and connect our mouse and keypress events
			this._buttonNode = this._buttonNode || this.focusNode || this.domNode;
			this._popupStateNode = this._popupStateNode || this.focusNode || this._buttonNode;
			this._aroundNode = this._aroundNode || this.domNode;
			this.connect(this._buttonNode, "onmousedown", "_onDropDownMouseDown");
			this.connect(this._buttonNode, "onclick", "_onDropDownClick");
			this.connect(this._buttonNode, "onkeydown", "_onDropDownKeydown");
			this.connect(this._buttonNode, "onkeyup", "_onKey");

			// If we have a _setStateClass function (which happens when
			// we are a form widget), then we need to connect our open/close
			// functions to it
			if(this._setStateClass){
				this.connect(this, "openDropDown", "_setStateClass");
				this.connect(this, "closeDropDown", "_setStateClass");
			}

			// Add a class to the "dijitDownArrowButton" type class to _buttonNode so theme can set direction of arrow
			// based on where drop down will normally appear
			var defaultPos = {
					"after" : this.isLeftToRight() ? "Right" : "Left",
					"before" : this.isLeftToRight() ? "Left" : "Right",
					"above" : "Up",
					"below" : "Down",
					"left" : "Left",
					"right" : "Right"
			}[this.dropDownPosition[0]] || this.dropDownPosition[0] || "Down";
			dojo.addClass(this._arrowWrapperNode || this._buttonNode, "dijit" + defaultPos + "ArrowButton");
		},

		postCreate: function(){
			this._setupDropdown();
			this.inherited(arguments);
		},

		destroyDescendants: function(){
			if(this.dropDown){
				// Destroy the drop down, unless it's already been destroyed.  This can happen because
				// the drop down is a direct child of <body> even though it's logically my child.
				if(!this.dropDown._destroyed){
					this.dropDown.destroyRecursive();
				}
				delete this.dropDown;
			}
			this.inherited(arguments);
		},

		_onDropDownKeydown: function(/*Event*/ e){
			if(e.keyCode == dojo.keys.DOWN_ARROW || e.keyCode == dojo.keys.ENTER || e.keyCode == dojo.keys.SPACE){
				e.preventDefault();	// stop IE screen jump
			}
		},

		_onKey: function(/*Event*/ e){
			// summary:
			//		Callback when the user presses a key while focused on the button node

			if(this.disabled || this.readOnly){ return; }
			var d = this.dropDown;
			if(d && this._opened && d.handleKey){
				if(d.handleKey(e) === false){ return; }
			}
			if(d && this._opened && e.keyCode == dojo.keys.ESCAPE){
				this.toggleDropDown();
			}else if(d && !this._opened && 
					(e.keyCode == dojo.keys.DOWN_ARROW || e.keyCode == dojo.keys.ENTER || e.keyCode == dojo.keys.SPACE)){
				this.toggleDropDown();
				if(d.focus){
					setTimeout(dojo.hitch(d, "focus"), 1);
				}
			}
		},

		_onBlur: function(){
			// summary:
			//		Called magically when focus has shifted away from this widget and it's dropdown

			this.closeDropDown();
			// don't focus on button.  the user has explicitly focused on something else.
			this.inherited(arguments);
		},

		isLoaded: function(){
			// summary:
			//		Returns whether or not the dropdown is loaded.  This can
			//		be overridden in order to force a call to loadDropDown().
			// tags:
			//		protected

			return true;
		},

		loadDropDown: function(/* Function */ loadCallback){
			// summary:
			//		Loads the data for the dropdown, and at some point, calls
			//		the given callback
			// tags:
			//		protected

			loadCallback();
		},

		toggleDropDown: function(){
			// summary:
			//		Toggle the drop-down widget; if it is up, close it, if not, open it
			// tags:
			//		protected

			if(this.disabled || this.readOnly){ return; }
			this.focus();
			var dropDown = this.dropDown;
			if(!dropDown){ return; }
			if(!this._opened){
				// If we aren't loaded, load it first so there isn't a flicker
				if(!this.isLoaded()){
					this.loadDropDown(dojo.hitch(this, "openDropDown"));
					return;
				}else{
					this.openDropDown();
				}
			}else{
				this.closeDropDown();
			}
		},

		openDropDown: function(){
			// summary:
			//		Opens the dropdown for this widget - it returns the
			//		return value of dijit.popup.open
			// tags:
			//		protected

			var dropDown = this.dropDown;
			var ddNode = dropDown.domNode;
			var self = this;

			// Prepare our popup's height and honor maxHeight if it exists.

			// TODO: isn't maxHeight dependent on the return value from dijit.popup.open(),
			// ie, dependent on how much space is available (BK)

			if(!this._preparedNode){
				dijit.popup.moveOffScreen(dropDown);
				this._preparedNode = true;			
				// Check if we have explicitly set width and height on the dropdown widget dom node
				if(ddNode.style.width){
					this._explicitDDWidth = true;
				}
				if(ddNode.style.height){
					this._explicitDDHeight = true;
				}
			}

			// Code for resizing dropdown (height limitation, or increasing width to match my width)
			if(this.maxHeight || this.forceWidth || this.autoWidth){
				var myStyle = {
					display: "",
					visibility: "hidden"
				};
				if(!this._explicitDDWidth){
					myStyle.width = "";
				}
				if(!this._explicitDDHeight){
					myStyle.height = "";
				}
				dojo.style(ddNode, myStyle);
				
				// Get size of drop down, and determine if vertical scroll bar needed
				var mb = dojo.marginBox(ddNode);
				var overHeight = (this.maxHeight && mb.h > this.maxHeight);
				dojo.style(ddNode, {
					overflowX: "hidden",
					overflowY: overHeight ? "auto" : "hidden"
				});
				if(overHeight){
					mb.h = this.maxHeight;
					if("w" in mb){
						mb.w += 16;	// room for vertical scrollbar
					}
				}else{
					delete mb.h;
				}
				delete mb.t;
				delete mb.l;

				// Adjust dropdown width to match or be larger than my width
				if(this.forceWidth){
					mb.w = this.domNode.offsetWidth;
				}else if(this.autoWidth){
					mb.w = Math.max(mb.w, this.domNode.offsetWidth);
				}else{
					delete mb.w;
				}
				
				// And finally, resize the dropdown to calculated height and width
				if(dojo.isFunction(dropDown.resize)){
					dropDown.resize(mb);
				}else{
					dojo.marginBox(ddNode, mb);
				}
			}

			var retVal = dijit.popup.open({
				parent: this,
				popup: dropDown,
				around: this._aroundNode,
				orient: dijit.getPopupAroundAlignment((this.dropDownPosition && this.dropDownPosition.length) ? this.dropDownPosition : ["below"],this.isLeftToRight()),
				onExecute: function(){
					self.closeDropDown(true);
				},
				onCancel: function(){
					self.closeDropDown(true);
				},
				onClose: function(){
					dojo.attr(self._popupStateNode, "popupActive", false);
					dojo.removeClass(self._popupStateNode, "dijitHasDropDownOpen");
					self._opened = false;
					self.state = "";
				}
			});
			dojo.attr(this._popupStateNode, "popupActive", "true");
			dojo.addClass(self._popupStateNode, "dijitHasDropDownOpen");
			this._opened=true;
			this.state="Opened";
			// TODO: set this.checked and call setStateClass(), to affect button look while drop down is shown
			return retVal;
		},

		closeDropDown: function(/*Boolean*/ focus){
			// summary:
			//		Closes the drop down on this widget
			// tags:
			//		protected

			if(this._opened){
				if(focus){ this.focus(); }
				dijit.popup.close(this.dropDown);
				this._opened = false;
				this.state = "";
			}
		}

	}
);

}

if(!dojo._hasResource["dijit.form.Button"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.Button"] = true;
dojo.provide("dijit.form.Button");





dojo.declare("dijit.form.Button",
	dijit.form._FormWidget,
	{
	// summary:
	//		Basically the same thing as a normal HTML button, but with special styling.
	// description:
	//		Buttons can display a label, an icon, or both.
	//		A label should always be specified (through innerHTML) or the label
	//		attribute.  It can be hidden via showLabel=false.
	// example:
	// |	<button dojoType="dijit.form.Button" onClick="...">Hello world</button>
	//
	// example:
	// |	var button1 = new dijit.form.Button({label: "hello world", onClick: foo});
	// |	dojo.body().appendChild(button1.domNode);

	// label: HTML String
	//		Text to display in button.
	//		If the label is hidden (showLabel=false) then and no title has
	//		been specified, then label is also set as title attribute of icon.
	label: "",

	// showLabel: Boolean
	//		Set this to true to hide the label text and display only the icon.
	//		(If showLabel=false then iconClass must be specified.)
	//		Especially useful for toolbars.
	//		If showLabel=true, the label will become the title (a.k.a. tooltip/hint) of the icon.
	//
	//		The exception case is for computers in high-contrast mode, where the label
	//		will still be displayed, since the icon doesn't appear.
	showLabel: true,

	// iconClass: String
	//		Class to apply to DOMNode in button to make it display an icon
	iconClass: "",

	// type: String
	//		Defines the type of button.  "button", "submit", or "reset".
	type: "button",

	baseClass: "dijitButton",

	templateString: dojo.cache("dijit.form", "templates/Button.html", "<span class=\"dijit dijitReset dijitInline\"\n\t><span class=\"dijitReset dijitInline dijitButtonNode\"\n\t\tdojoAttachEvent=\"ondijitclick:_onButtonClick\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdojoAttachPoint=\"titleNode,focusNode\"\n\t\t\twaiRole=\"button\" waiState=\"labelledby-${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\" dojoAttachPoint=\"iconNode\"></span\n\t\t\t><span class=\"dijitReset dijitToggleButtonIconChar\">&#x25CF;</span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t\tdojoAttachPoint=\"containerNode\"\n\t\t\t></span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\"\n\t\tdojoAttachPoint=\"valueNode\"\n/></span>\n"),

	attributeMap: dojo.delegate(dijit.form._FormWidget.prototype.attributeMap, {
		value: "valueNode",
		iconClass: { node: "iconNode", type: "class" }
	}),


	_onClick: function(/*Event*/ e){
		// summary:
		//		Internal function to handle click actions
		if(this.disabled){
			return false;
		}
		this._clicked(); // widget click actions
		return this.onClick(e); // user click actions
	},

	_onButtonClick: function(/*Event*/ e){
		// summary:
		//		Handler when the user activates the button portion.
		if(this._onClick(e) === false){ // returning nothing is same as true
			e.preventDefault(); // needed for checkbox
		}else if(this.type == "submit" && !(this.valueNode||this.focusNode).form){ // see if a nonform widget needs to be signalled
			for(var node=this.domNode; node.parentNode/*#5935*/; node=node.parentNode){
				var widget=dijit.byNode(node);
				if(widget && typeof widget._onSubmit == "function"){
					widget._onSubmit(e);
					break;
				}
			}
		}else if(this.valueNode){
			this.valueNode.click();
			e.preventDefault(); // cancel BUTTON click and continue with hidden INPUT click
		}
	},

	_fillContent: function(/*DomNode*/ source){
		// Overrides _Templated._fillContent().
		// If button label is specified as srcNodeRef.innerHTML rather than
		// this.params.label, handle it here.
		if(source && (!this.params || !("label" in this.params))){
			this.set('label', source.innerHTML);
		}
	},

	postCreate: function(){
		dojo.setSelectable(this.focusNode, false);
		this.inherited(arguments);
	},

	_setShowLabelAttr: function(val){
		if(this.containerNode){
			dojo.toggleClass(this.containerNode, "dijitDisplayNone", !val);
		}
		this.showLabel = val;
	},

	onClick: function(/*Event*/ e){
		// summary:
		//		Callback for when button is clicked.
		//		If type="submit", return true to perform submit, or false to cancel it.
		// type:
		//		callback
		return true;		// Boolean
	},

	_clicked: function(/*Event*/ e){
		// summary:
		//		Internal overridable function for when the button is clicked
	},

	setLabel: function(/*String*/ content){
		// summary:
		//		Deprecated.  Use set('label', ...) instead.
		dojo.deprecated("dijit.form.Button.setLabel() is deprecated.  Use set('label', ...) instead.", "", "2.0");
		this.set("label", content);
	},

	_setLabelAttr: function(/*String*/ content){
		// summary:
		//		Hook for attr('label', ...) to work.
		// description:
		//		Set the label (text) of the button; takes an HTML string.
		this.containerNode.innerHTML = this.label = content;
		if(this.showLabel == false && !this.params.title){
			this.titleNode.title = dojo.trim(this.containerNode.innerText || this.containerNode.textContent || '');
		}
	}
});


dojo.declare("dijit.form.DropDownButton", [dijit.form.Button, dijit._Container, dijit._HasDropDown], {
	// summary:
	//		A button with a drop down
	//
	// example:
	// |	<button dojoType="dijit.form.DropDownButton" label="Hello world">
	// |		<div dojotype="dijit.Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new dijit.form.DropDownButton({ label: "hi", dropDown: new dijit.Menu(...) });
	// |	dojo.body().appendChild(button1);
	//

	baseClass : "dijitDropDownButton",

	templateString: dojo.cache("dijit.form", "templates/DropDownButton.html", "<span class=\"dijit dijitReset dijitInline\"\n\t><span class='dijitReset dijitInline dijitButtonNode'\n\t\tdojoAttachEvent=\"ondijitclick:_onButtonClick\" dojoAttachPoint=\"_buttonNode\"\n\t\t><span class=\"dijitReset dijitStretch dijitButtonContents\"\n\t\t\tdojoAttachPoint=\"focusNode,titleNode,_arrowWrapperNode\"\n\t\t\twaiRole=\"button\" waiState=\"haspopup-true,labelledby-${id}_label\"\n\t\t\t><span class=\"dijitReset dijitInline dijitIcon\"\n\t\t\t\tdojoAttachPoint=\"iconNode\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitButtonText\"\n\t\t\t\tdojoAttachPoint=\"containerNode,_popupStateNode\"\n\t\t\t\tid=\"${id}_label\"\n\t\t\t></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonInner\"></span\n\t\t\t><span class=\"dijitReset dijitInline dijitArrowButtonChar\">&#9660;</span\n\t\t></span\n\t></span\n\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" class=\"dijitOffScreen\"\n\t\tdojoAttachPoint=\"valueNode\"\n/></span>\n"),

	_fillContent: function(){
		// Overrides Button._fillContent().
		//
		// My inner HTML contains both the button contents and a drop down widget, like
		// <DropDownButton>  <span>push me</span>  <Menu> ... </Menu> </DropDownButton>
		// The first node is assumed to be the button content. The widget is the popup.

		if(this.srcNodeRef){ // programatically created buttons might not define srcNodeRef
			//FIXME: figure out how to filter out the widget and use all remaining nodes as button
			//	content, not just nodes[0]
			var nodes = dojo.query("*", this.srcNodeRef);
			dijit.form.DropDownButton.superclass._fillContent.call(this, nodes[0]);

			// save pointer to srcNode so we can grab the drop down widget after it's instantiated
			this.dropDownContainer = this.srcNodeRef;
		}
	},

	startup: function(){
		if(this._started){ return; }

		// the child widget from srcNodeRef is the dropdown widget.  Insert it in the page DOM,
		// make it invisible, and store a reference to pass to the popup code.
		if(!this.dropDown){
			var dropDownNode = dojo.query("[widgetId]", this.dropDownContainer)[0];
			this.dropDown = dijit.byNode(dropDownNode);
			delete this.dropDownContainer;
		}
		dijit.popup.moveOffScreen(this.dropDown);

		this.inherited(arguments);
	},

	isLoaded: function(){
		// Returns whether or not we are loaded - if our dropdown has an href,
		// then we want to check that.
		var dropDown = this.dropDown;
		return (!dropDown.href || dropDown.isLoaded);
	},

	loadDropDown: function(){
		// Loads our dropdown
		var dropDown = this.dropDown;
		if(!dropDown){ return; }
		if(!this.isLoaded()){
			var handler = dojo.connect(dropDown, "onLoad", this, function(){
				dojo.disconnect(handler);
				this.openDropDown();
			});
			dropDown.refresh();
		}else{
			this.openDropDown();
		}
	},

	isFocusable: function(){
		// Overridden so that focus is handled by the _HasDropDown mixin, not by
		// the _FormWidget mixin.
		return this.inherited(arguments) && !this._mouseDown;
	}
});

dojo.declare("dijit.form.ComboButton", dijit.form.DropDownButton, {
	// summary:
	//		A combination button and drop-down button.
	//		Users can click one side to "press" the button, or click an arrow
	//		icon to display the drop down.
	//
	// example:
	// |	<button dojoType="dijit.form.ComboButton" onClick="...">
	// |		<span>Hello world</span>
	// |		<div dojoType="dijit.Menu">...</div>
	// |	</button>
	//
	// example:
	// |	var button1 = new dijit.form.ComboButton({label: "hello world", onClick: foo, dropDown: "myMenu"});
	// |	dojo.body().appendChild(button1.domNode);
	//

	templateString: dojo.cache("dijit.form", "templates/ComboButton.html", "<table class=\"dijit dijitReset dijitInline dijitLeft\"\n\tcellspacing='0' cellpadding='0' waiRole=\"presentation\"\n\t><tbody waiRole=\"presentation\"><tr waiRole=\"presentation\"\n\t\t><td class=\"dijitReset dijitStretch dijitButtonNode\" dojoAttachPoint=\"buttonNode\" dojoAttachEvent=\"ondijitclick:_onButtonClick,onkeypress:_onButtonKeyPress\"\n\t\t><div id=\"${id}_button\" class=\"dijitReset dijitButtonContents\"\n\t\t\tdojoAttachPoint=\"titleNode\"\n\t\t\twaiRole=\"button\" waiState=\"labelledby-${id}_label\"\n\t\t\t><div class=\"dijitReset dijitInline dijitIcon\" dojoAttachPoint=\"iconNode\" waiRole=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitInline dijitButtonText\" id=\"${id}_label\" dojoAttachPoint=\"containerNode\" waiRole=\"presentation\"></div\n\t\t></div\n\t\t></td\n\t\t><td id=\"${id}_arrow\" class='dijitReset dijitRight dijitButtonNode dijitArrowButton'\n\t\t\tdojoAttachPoint=\"_popupStateNode,focusNode,_buttonNode\"\n\t\t\tdojoAttachEvent=\"onkeypress:_onArrowKeyPress\"\n\t\t\ttitle=\"${optionsTitle}\"\n\t\t\twaiRole=\"button\" waiState=\"haspopup-true\"\n\t\t\t><div class=\"dijitReset dijitArrowButtonInner\" waiRole=\"presentation\"></div\n\t\t\t><div class=\"dijitReset dijitArrowButtonChar\" waiRole=\"presentation\">&#9660;</div\n\t\t></td\n\t\t><td style=\"display:none !important;\"\n\t\t\t><input ${!nameAttrSetting} type=\"${type}\" value=\"${value}\" dojoAttachPoint=\"valueNode\"\n\t\t/></td></tr></tbody\n></table>\n"),

	attributeMap: dojo.mixin(dojo.clone(dijit.form.Button.prototype.attributeMap), {
		id: "",
		tabIndex: ["focusNode", "titleNode"],
		title: "titleNode"
	}),

	// optionsTitle: String
	//		Text that describes the options menu (accessibility)
	optionsTitle: "",

	baseClass: "dijitComboButton",

	// Set classes like dijitButtonContentsHover or dijitArrowButtonActive depending on
	// mouse action over specified node
	cssStateNodes: {
		"buttonNode": "dijitButtonNode",
		"titleNode": "dijitButtonContents",
		"_popupStateNode": "dijitDownArrowButton"
	},

	_focusedNode: null,

	_onButtonKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handler for right arrow key when focus is on left part of button
		if(evt.charOrCode == dojo.keys[this.isLeftToRight() ? "RIGHT_ARROW" : "LEFT_ARROW"]){
			dijit.focus(this._popupStateNode);
			dojo.stopEvent(evt);
		}
	},

	_onArrowKeyPress: function(/*Event*/ evt){
		// summary:
		//		Handler for left arrow key when focus is on right part of button
		if(evt.charOrCode == dojo.keys[this.isLeftToRight() ? "LEFT_ARROW" : "RIGHT_ARROW"]){
			dijit.focus(this.titleNode);
			dojo.stopEvent(evt);
		}
	},
	
	focus: function(/*String*/ position){
		// summary:
		//		Focuses this widget to according to position, if specified,
		//		otherwise on arrow node
		// position:
		//		"start" or "end"
		if(!this.disabled){
			dijit.focus(position == "start" ? this.titleNode : this._popupStateNode);
		}
	}
});

dojo.declare("dijit.form.ToggleButton", dijit.form.Button, {
	// summary:
	//		A button that can be in two states (checked or not).
	//		Can be base class for things like tabs or checkbox or radio buttons

	baseClass: "dijitToggleButton",

	// checked: Boolean
	//		Corresponds to the native HTML <input> element's attribute.
	//		In markup, specified as "checked='checked'" or just "checked".
	//		True if the button is depressed, or the checkbox is checked,
	//		or the radio button is selected, etc.
	checked: false,

	attributeMap: dojo.mixin(dojo.clone(dijit.form.Button.prototype.attributeMap), {
		checked:"focusNode"
	}),

	_clicked: function(/*Event*/ evt){
		this.set('checked', !this.checked);
	},

	_setCheckedAttr: function(/*Boolean*/ value, /* Boolean? */ priorityChange){
		this.checked = value;
		dojo.attr(this.focusNode || this.domNode, "checked", value);
		dijit.setWaiState(this.focusNode || this.domNode, "pressed", value);
		this._handleOnChange(value, priorityChange);
	},

	setChecked: function(/*Boolean*/ checked){
		// summary:
		//		Deprecated.   Use set('checked', true/false) instead.
		dojo.deprecated("setChecked("+checked+") is deprecated. Use set('checked',"+checked+") instead.", "", "2.0");
		this.set('checked', checked);
	},

	reset: function(){
		// summary:
		//		Reset the widget's value to what it was at initialization time

		this._hasBeenBlurred = false;

		// set checked state to original setting
		this.set('checked', this.params.checked || false);
	}
});

}

if(!dojo._hasResource["dijit.form.ComboButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ComboButton"] = true;
dojo.provide("dijit.form.ComboButton");


}

if(!dojo._hasResource["dijit.Tooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Tooltip"] = true;
dojo.provide("dijit.Tooltip");




dojo.declare(
	"dijit._MasterTooltip",
	[dijit._Widget, dijit._Templated],
	{
		// summary:
		//		Internal widget that holds the actual tooltip markup,
		//		which occurs once per page.
		//		Called by Tooltip widgets which are just containers to hold
		//		the markup
		// tags:
		//		protected

		// duration: Integer
		//		Milliseconds to fade in/fade out
		duration: dijit.defaultDuration,

		templateString: dojo.cache("dijit", "templates/Tooltip.html", "<div class=\"dijitTooltip dijitTooltipLeft\" id=\"dojoTooltip\">\n\t<div class=\"dijitTooltipContainer dijitTooltipContents\" dojoAttachPoint=\"containerNode\" waiRole='alert'></div>\n\t<div class=\"dijitTooltipConnector\"></div>\n</div>\n"),

		postCreate: function(){
			dojo.body().appendChild(this.domNode);

			this.bgIframe = new dijit.BackgroundIframe(this.domNode);

			// Setup fade-in and fade-out functions.
			this.fadeIn = dojo.fadeIn({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onShow") });
			this.fadeOut = dojo.fadeOut({ node: this.domNode, duration: this.duration, onEnd: dojo.hitch(this, "_onHide") });

		},

		show: function(/*String*/ innerHTML, /*DomNode*/ aroundNode, /*String[]?*/ position, /*Boolean*/ rtl){
			// summary:
			//		Display tooltip w/specified contents to right of specified node
			//		(To left if there's no space on the right, or if rtl == true)

			if(this.aroundNode && this.aroundNode === aroundNode){
				return;
			}

			if(this.fadeOut.status() == "playing"){
				// previous tooltip is being hidden; wait until the hide completes then show new one
				this._onDeck=arguments;
				return;
			}
			this.containerNode.innerHTML=innerHTML;

			var pos = dijit.placeOnScreenAroundElement(this.domNode, aroundNode, dijit.getPopupAroundAlignment((position && position.length) ? position : dijit.Tooltip.defaultPosition, !rtl), dojo.hitch(this, "orient"));

			// show it
			dojo.style(this.domNode, "opacity", 0);
			this.fadeIn.play();
			this.isShowingNow = true;
			this.aroundNode = aroundNode;
		},

		orient: function(/* DomNode */ node, /* String */ aroundCorner, /* String */ tooltipCorner){
			// summary:
			//		Private function to set CSS for tooltip node based on which position it's in.
			//		This is called by the dijit popup code.
			// tags:
			//		protected

			node.className = "dijitTooltip " +
				{
					"BL-TL": "dijitTooltipBelow dijitTooltipABLeft",
					"TL-BL": "dijitTooltipAbove dijitTooltipABLeft",
					"BR-TR": "dijitTooltipBelow dijitTooltipABRight",
					"TR-BR": "dijitTooltipAbove dijitTooltipABRight",
					"BR-BL": "dijitTooltipRight",
					"BL-BR": "dijitTooltipLeft"
				}[aroundCorner + "-" + tooltipCorner];
		},

		_onShow: function(){
			// summary:
			//		Called at end of fade-in operation
			// tags:
			//		protected
			if(dojo.isIE){
				// the arrow won't show up on a node w/an opacity filter
				this.domNode.style.filter="";
			}
		},

		hide: function(aroundNode){
			// summary:
			//		Hide the tooltip
			if(this._onDeck && this._onDeck[1] == aroundNode){
				// this hide request is for a show() that hasn't even started yet;
				// just cancel the pending show()
				this._onDeck=null;
			}else if(this.aroundNode === aroundNode){
				// this hide request is for the currently displayed tooltip
				this.fadeIn.stop();
				this.isShowingNow = false;
				this.aroundNode = null;
				this.fadeOut.play();
			}else{
				// just ignore the call, it's for a tooltip that has already been erased
			}
		},

		_onHide: function(){
			// summary:
			//		Called at end of fade-out operation
			// tags:
			//		protected

			this.domNode.style.cssText="";	// to position offscreen again
			this.containerNode.innerHTML="";
			if(this._onDeck){
				// a show request has been queued up; do it now
				this.show.apply(this, this._onDeck);
				this._onDeck=null;
			}
		}

	}
);

dijit.showTooltip = function(/*String*/ innerHTML, /*DomNode*/ aroundNode, /*String[]?*/ position, /*Boolean*/ rtl){
	// summary:
	//		Display tooltip w/specified contents in specified position.
	//		See description of dijit.Tooltip.defaultPosition for details on position parameter.
	//		If position is not specified then dijit.Tooltip.defaultPosition is used.
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.show(innerHTML, aroundNode, position, rtl);
};

dijit.hideTooltip = function(aroundNode){
	// summary:
	//		Hide the tooltip
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
	return dijit._masterTT.hide(aroundNode);
};

dojo.declare(
	"dijit.Tooltip",
	dijit._Widget,
	{
		// summary:
		//		Pops up a tooltip (a help message) when you hover over a node.

		// label: String
		//		Text to display in the tooltip.
		//		Specified as innerHTML when creating the widget from markup.
		label: "",

		// showDelay: Integer
		//		Number of milliseconds to wait after hovering over/focusing on the object, before
		//		the tooltip is displayed.
		showDelay: 400,

		// connectId: [const] String[]
		//		Id's of domNodes to attach the tooltip to.
		//		When user hovers over any of the specified dom nodes, the tooltip will appear.
		//
		//		Note: Currently connectId can only be specified on initialization, it cannot
		//		be changed via attr('connectId', ...)
		//
		//		Note: in 2.0 this will be renamed to connectIds for less confusion.
		connectId: [],

		// position: String[]
		//		See description of `dijit.Tooltip.defaultPosition` for details on position parameter.
		position: [],

		constructor: function(){
			// Map id's of nodes I'm connected to to a list of the this.connect() handles
			this._nodeConnectionsById = {};
		},

		_setConnectIdAttr: function(newIds){
			for(var oldId in this._nodeConnectionsById){
				this.removeTarget(oldId);
			}
			dojo.forEach(dojo.isArrayLike(newIds) ? newIds : [newIds], this.addTarget, this);
		},

		_getConnectIdAttr: function(){
			var ary = [];
			for(var id in this._nodeConnectionsById){
				ary.push(id);
			}
			return ary;
		},

		addTarget: function(/*DOMNODE || String*/ id){
			// summary:
			//		Attach tooltip to specified node, if it's not already connected
			var node = dojo.byId(id);
			if(!node){ return; }
			if(node.id in this._nodeConnectionsById){ return; }//Already connected

			this._nodeConnectionsById[node.id] = [
				this.connect(node, "onmouseenter", "_onTargetMouseEnter"),
				this.connect(node, "onmouseleave", "_onTargetMouseLeave"),
				this.connect(node, "onfocus", "_onTargetFocus"),
				this.connect(node, "onblur", "_onTargetBlur")
			];
		},

		removeTarget: function(/*DOMNODE || String*/ node){
			// summary:
			//		Detach tooltip from specified node

			// map from DOMNode back to plain id string
			var id = node.id || node;

			if(id in this._nodeConnectionsById){
				dojo.forEach(this._nodeConnectionsById[id], this.disconnect, this);
				delete this._nodeConnectionsById[id];
			}
		},

		postCreate: function(){
			dojo.addClass(this.domNode,"dijitTooltipData");
		},

		startup: function(){
			this.inherited(arguments);

			// If this tooltip was created in a template, or for some other reason the specified connectId[s]
			// didn't exist during the widget's initialization, then connect now.
			var ids = this.connectId;
			dojo.forEach(dojo.isArrayLike(ids) ? ids : [ids], this.addTarget, this);
		},

		_onTargetMouseEnter: function(/*Event*/ e){
			// summary:
			//		Handler for mouseenter event on the target node
			// tags:
			//		private
			this._onHover(e);
		},

		_onTargetMouseLeave: function(/*Event*/ e){
			// summary:
			//		Handler for mouseleave event on the target node
			// tags:
			//		private
			this._onUnHover(e);
		},

		_onTargetFocus: function(/*Event*/ e){
			// summary:
			//		Handler for focus event on the target node
			// tags:
			//		private

			this._focus = true;
			this._onHover(e);
		},

		_onTargetBlur: function(/*Event*/ e){
			// summary:
			//		Handler for blur event on the target node
			// tags:
			//		private

			this._focus = false;
			this._onUnHover(e);
		},

		_onHover: function(/*Event*/ e){
			// summary:
			//		Despite the name of this method, it actually handles both hover and focus
			//		events on the target node, setting a timer to show the tooltip.
			// tags:
			//		private
			if(!this._showTimer){
				var target = e.target;
				this._showTimer = setTimeout(dojo.hitch(this, function(){this.open(target)}), this.showDelay);
			}
		},

		_onUnHover: function(/*Event*/ e){
			// summary:
			//		Despite the name of this method, it actually handles both mouseleave and blur
			//		events on the target node, hiding the tooltip.
			// tags:
			//		private

			// keep a tooltip open if the associated element still has focus (even though the
			// mouse moved away)
			if(this._focus){ return; }

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			this.close();
		},

		open: function(/*DomNode*/ target){
 			// summary:
			//		Display the tooltip; usually not called directly.
			// tags:
			//		private

			if(this._showTimer){
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
			dijit.showTooltip(this.label || this.domNode.innerHTML, target, this.position, !this.isLeftToRight());

			this._connectNode = target;
			this.onShow(target, this.position);
		},

		close: function(){
			// summary:
			//		Hide the tooltip or cancel timer for show of tooltip
			// tags:
			//		private

			if(this._connectNode){
				// if tooltip is currently shown
				dijit.hideTooltip(this._connectNode);
				delete this._connectNode;
				this.onHide();
			}
			if(this._showTimer){
				// if tooltip is scheduled to be shown (after a brief delay)
				clearTimeout(this._showTimer);
				delete this._showTimer;
			}
		},

		onShow: function(target, position){
			// summary:
			//		Called when the tooltip is shown
			// tags:
			//		callback
		},

		onHide: function(){
			// summary:
			//		Called when the tooltip is hidden
			// tags:
			//		callback
		},

		uninitialize: function(){
			this.close();
			this.inherited(arguments);
		}
	}
);

// dijit.Tooltip.defaultPosition: String[]
//		This variable controls the position of tooltips, if the position is not specified to
//		the Tooltip widget or *TextBox widget itself.  It's an array of strings with the following values:
//
//			* before: places tooltip to the left of the target node/widget, or to the right in
//			  the case of RTL scripts like Hebrew and Arabic
//			* after: places tooltip to the right of the target node/widget, or to the left in
//			  the case of RTL scripts like Hebrew and Arabic
//			* above: tooltip goes above target node
//			* below: tooltip goes below target node
//
//		The list is positions is tried, in order, until a position is found where the tooltip fits
//		within the viewport.
//
//		Be careful setting this parameter.  A value of "above" may work fine until the user scrolls
//		the screen so that there's no room above the target node.   Nodes with drop downs, like
//		DropDownButton or FilteringSelect, are especially problematic, in that you need to be sure
//		that the drop down and tooltip don't overlap, even when the viewport is scrolled so that there
//		is only room below (or above) the target node, but not both.
dijit.Tooltip.defaultPosition = ["after", "before"];

}

if(!dojo._hasResource["itme.ChassisMap._Overlays"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap._Overlays"] = true;
dojo.provide("itme.ChassisMap._Overlays");
















var menuTimer = null;

dojo.declare("itme.ChassisMap._Overlays", null, {

  _currentOverlay: null,
  _overlays: [], 
  IMAGES : new itme.ChassisMap.ImageResources(),

  constructor: function(kwArgs){
    var StatOverlay = new itme.ChassisMap.StatusOverlay({}, null);
    this._overlays[0] = StatOverlay;
    var UpdatesOverlay = new itme.ChassisMap.ComplianceOverlay({}, null);
    this._overlays[1] = UpdatesOverlay;
    var AccessOverlay = new itme.ChassisMap.AccessOverlay({}, null);
    this._overlays[2] = AccessOverlay;
    var LEDOverlay = new itme.ChassisMap.LEDOverlay({}, null);
    this._overlays[3] = LEDOverlay;
    var NameOverlay = new itme.ChassisMap.PropertiesOverlay({}, null);
    this._overlays[4] = NameOverlay;
    var HeatOverlay = new itme.ChassisMap.EnvironmentalsOverlay({}, null);
    this._overlays[5] = HeatOverlay;
  },
  
  // protected
  _setItemsForOverlays: function(itemsList) {
    itmeUtils.CMTrace("OVERLAY", "overlay.setItems");
    for (var i = 0; i < this._overlays.length; i++)   
    {
      this._overlays[i].setItems(itemsList);
    }
  },
  
  // protected
  _renderDefaultOverlay: function() {
    var curOverlayId = dojo.cookie("chasisMap.currentOverlay");
    dojo.cookie("chasisMap.currentOverlay", null, { expires: -1}); // remove the cookie if it exists
    itmeUtils.CMTrace("OVERLAY", "COOKIE: removing cookie for overlay.  (was " + curOverlayId + ")");

    this._switchOverlays("status"); // always pre-render status for the colorization 
    if (curOverlayId) {
      this._switchOverlays(curOverlayId);
      this._currentOverlay = this._overlays[this._getOverlayIndex(curOverlayId)];
    }
  },
  
  // protected
  _switchOverlays: function(overlayId) {
    var overlayIndex = this._getOverlayIndex(overlayId);
    itmeUtils.CMTrace("OVERLAY", "switching overlay to: "+ overlayId);
    if (this._currentOverlay) {
      if (this._currentOverlay.getId() == overlayId) {
        itmeUtils.CMTrace("OVERLAY", "same overlay selected - not switching");
        return;
      }
      
      // unrender current overlay
      this._currentOverlay.unRenderOverlay();
      this._setButtonStyleUnpressed(this._currentOverlay.getId());
    }
    itmeUtils.CMTrace("OVERLAY", "switching overlays to: " + overlayId);
    
    // render selected overlay
    this._overlays[overlayIndex].renderOverlay();
    this._currentOverlay = this._overlays[overlayIndex];
    this._setButtonStylePressed(overlayId);
  },
  
  // protected
  _getOverlayIndex: function(overlayId)
  {
    for (var i = 0; i < this._overlays.length; i++)   
    {
      if ( this._overlays[i].getId() == overlayId )
        return i;
    }
  },
  

  // protected
  _insertOverlays: function(overlayFrameElement, itemsList){
    var i, buttonContainer, overlaysFilters, menu, labelString, button;
    // Add icons (as styles) for each overlay class
    for (i = 0; i < this._overlays.length; i++) 
      this._appendFilterButtonIconStyleToDocument(this._overlays[i].getId()+"_icon", this._overlays[i].getIconId());
    
    // build the menu button for each overlay class  
    for (i = 0; i < this._overlays.length; i++) {
      
      if ( this._isOverlayActive(this._overlays[i], itemsList) ) {

        buttonContainer = itmeUtils.createElement(
            this._overlays[i].getId() + "_buttonContainer", 
            overlayFrameElement, 
            { elementType: "span", 
              styleClass: "buttonContainer" } );
        
        overlaysFilters = this._overlays[i].getAvailableFilters();  
        
        button = dijit.byId(this._overlays[i].getId()+"_dojoButton");
        if ( button ) {
          button.destroyRecursive();
        }
        
        if ( overlaysFilters.length > 0 )
        { // build combobutton with menu flyout
          menu = new itme.DojoMenuOverride({
                style: "display: none;",
                onMouseLeave: function(e) { menuTimer = setTimeout(dojo.hitch(this, this.onCancel, true), 1000);},
                onMouseEnter: function(e) { clearTimeout(menuTimer); }
          });
          for ( var j = 0; j < overlaysFilters.length; j++ )
          {
            labelString = overlaysFilters[j].getDescription();
            if ( overlaysFilters[j].getIconId() )
              labelString = this.IMAGES.buildIconImageTag(overlaysFilters[j].getIconId(), "menuImage", 22) + overlaysFilters[j].getDescription();
            var menuItem1 = new dijit.CheckedMenuItem({
                label: labelString,
                disabled: (!overlaysFilters[j].allowModification()),
                checked: this._getFilterEnabledSettingFromCookieOrObject(this._overlays[i].getId(), overlaysFilters[j]),
                onClick: dojo.hitch(this, this._filterToggled, i, j)
            });
            menu.addChild(menuItem1);
            
            overlaysFilters[j].setEnabled( this._getFilterEnabledSettingFromCookieOrObject(this._overlays[i].getId(), overlaysFilters[j] ) );
          }
  
          itmeUtils.CMTrace("OVERLAY", "creating button: " + this._overlays[i].getId());
          button = new dijit.form.ComboButton({
              //label: this._overlays[i].getTitle(),
              dropDown: menu,
              optionsTitle: this._overlays[i].getDescription(),
              onClick: dojo.hitch(this, this._comboClicked, this._overlays[i].getId()),
              iconClass: this._overlays[i].getId()+"_icon",
              id: this._overlays[i].getId()+"_dojoButton",
              showLabel: false,
              type: "button",
              dropDownPosition: ["above"]
          });
          buttonContainer.appendChild(button.domNode);
          dojo.addClass(dojo.byId(this._overlays[i].getId()+"_dojoButton"), "overlayButton");
          this._setButtonStyleUnpressed(this._overlays[i].getId());
        }
        else
        { // build a standard dojo button
          itmeUtils.CMTrace("OVERLAY", "creating button: " + this._overlays[i].getId());
          button = new dijit.form.Button({
              optionsTitle: this._overlays[i].getDescription(),
              onClick: dojo.hitch(this, this._comboClicked, this._overlays[i].getId()),
              iconClass: this._overlays[i].getId()+"_icon",
              id: this._overlays[i].getId()+"_dojoButton",
              showLabel: false,
              type: "button"
          });
          buttonContainer.appendChild(button.domNode);
          dojo.addClass(dojo.byId(this._overlays[i].getId()+"_dojoButton"), "overlayButton");
          this._setButtonStyleUnpressed(this._overlays[i].getId());
        }     
          
        
        itmeUtils.createElement(
              "myButtonSeperator_"+i, 
              overlayFrameElement, 
              { styleClass: "buttonSeperator",
                innerHTML: this.IMAGES.buildIconImageTag("Seperator", "seperatorImg" + ((dojo.isIE) ? "" : " notIE"), 18) } );

        new dijit.Tooltip({
          connectId: [this._overlays[i].getId()+"_dojoButton"],
          label: this._overlays[i].getDescription(),
          position: "above",
          id: this._overlays[i].getId()+"_buttonTooltip"
        });
      }
    }
    itmeUtils.CMTrace("OVERLAY", "done inserting overlays");
  },
  
  
  _isOverlayActive: function(overlay, itemsList) {
    var requiredPropertiesArray, showOverlay, j, k, tempProp;
    
    if ( overlay.isOverlayActive() !== null ) 
      return overlay.isOverlayActive();
    else
    {
      itmeUtils.CMTrace("MAIN", "isOverlayActive: " + overlay.getId());
      showOverlay = true;
      requiredPropertiesArray = overlay.requiresProperties();
      if ( requiredPropertiesArray ) {
        for (j = 0; (j < requiredPropertiesArray.length) && (showOverlay==true) ; j++) {
          tempProp = itmeUtils.getProperty(itemsList[0], requiredPropertiesArray[j]);
          if ( (tempProp != null) && (tempProp == "DATA_COLLECTION_INACTIVE")) // no elements will have data
          {
            itmeUtils.CMTrace("MAIN", "overlay property is null or INACTIVE, NOT SHOWING");
            overlay.setOverlayActiveState(false);
            return false;
          }
          else if ((null == tempProp) || ("" == tempProp)) // maybe other elements in the list will have this property
          { 
            showOverlay = false;
            itmeUtils.CMTrace("MAIN", "prop for 1st element was empty, iterating over all items");
            for (k = 1; (k < itemsList.length) && (showOverlay == false); k++) { //  check other elements
              tempProp = itmeUtils.getProperty(itemsList[k], requiredPropertiesArray[j]);
              if ( (null != tempProp) && ("" != tempProp) )
                showOverlay = true;
            }
          }
        }
      }
      itmeUtils.CMTrace("MAIN", "isOverlayActive: " + overlay.getId() + " is returning - " + showOverlay);
      overlay.setOverlayActiveState(showOverlay);
      return showOverlay;
    }
  },
  
  _prefetchAsynchOverlayData: function(itemsList, alreadyFetchedProps) {
    var requiredPropertiesArray = [], i, j, overlayReqProps; 
    for (i = 0; i < this._overlays.length; i++) { 
      itmeUtils.CMTrace("OVERLAY", "getting props for overlay: " + this._overlays[i].getId());
      if ( this._isOverlayActive(this._overlays[i], itemsList) ) {
        overlayReqProps = this._overlays[i].requiresProperties();
        if ( overlayReqProps ) {
          // prefetch only those properties not in the default list 
          for (j = 0; j < overlayReqProps.length; j++) {
            if ( dojo.indexOf(alreadyFetchedProps, overlayReqProps[j]) < 0 )
              requiredPropertiesArray = requiredPropertiesArray.concat(overlayReqProps[j]);
          }
        }
      }
    }
      
    itmeUtils.CMTrace("MAIN", "prefetching overlay data for: " + requiredPropertiesArray);
    if ( requiredPropertiesArray ) {
      for (i = 0; (i < itemsList.length); i++) {
        itemsList[i].getProperties({properties: requiredPropertiesArray});
      }
    }
    itmeUtils.CMTrace("MAIN", "done prefetching overlay data for: " + requiredPropertiesArray);
  },
  
  
  _appendFilterButtonIconStyleToDocument: function(styleName, iconId) {
    
    var head = document.getElementsByTagName('head')[0];

    var style = document.createElement('style');
    var rules = document.createTextNode("."+styleName+" { background-image: url('"+this.IMAGES.iconPath(iconId, 22)+"'); background-repeat:no-repeat; width:23px; height:23px; margin:1px !important;}");
    style.type = 'text/css';
    if(style.styleSheet) {
      style.styleSheet.cssText = rules.nodeValue;
    } else { 
      style.appendChild(rules);
    }
    head.appendChild(style);
  },
  
  _comboClicked: function(overlayID) {
    try {
      this._switchOverlays(overlayID);
    } catch (err) { itmeUtils.CMTrace("OVERLAY", err); }
    return false;
  },
  
  _filterToggled: function(overlayIndex, filterIndex) {
    var wasEnabled = this._overlays[overlayIndex]._availableFilters[filterIndex].isEnabled();
    this._overlays[overlayIndex]._availableFilters[filterIndex].setEnabled(!wasEnabled);
    
    this._storeFilterStates();
    
    if ( this._overlays[overlayIndex].isEnabled() ) 
    {
      this._overlays[overlayIndex].deleteOverlay();
      this._overlays[overlayIndex].renderOverlay(false);
    }
    else
      this._overlays[overlayIndex].deleteOverlay();
    
  },
  
  _storeFilterStates: function() {
    var objOfOverlaysAndFilterStates = {};
    objOfOverlaysAndFilterStates.overlays = [];
    for (var i = 0; i < this._overlays.length; i++)   
    {
      var arrayOfFilters = [];
      var overlaysFilters = this._overlays[i].getAvailableFilters();  
      if ( overlaysFilters.length > 0 ) {
    	for ( var j = 0; j < overlaysFilters.length; j++ ) {
    	  arrayOfFilters[j] = {"id": overlaysFilters[j].getId(),
    						   "enabled": overlaysFilters[j].isEnabled()};
    	}
      }
      objOfOverlaysAndFilterStates.overlays[i] = {"id": this._overlays[i].getId(),
    		  									  "filters": arrayOfFilters};
    }
    myJsonString = dojox.json.ref.toJson(objOfOverlaysAndFilterStates);
    dojo.cookie("chasisMap.overlayFilterSettings", myJsonString, { expires: 365 }); 
    itmeUtils.CMTrace("OVERLAY", myJsonString);
  },
  
  _getFilterEnabledSettingFromCookieOrObject: function(overlayId, filterObject) {
    var currentFilterSettingsJSON = dojo.cookie("chasisMap.overlayFilterSettings");
    var currentFilterSettings = dojox.json.ref.fromJson(currentFilterSettingsJSON);
    if ( currentFilterSettings ) {
      for (i = 0; i < currentFilterSettings.overlays.length; i++) {
        if ( currentFilterSettings.overlays[i].id == overlayId) {
          for (j = 0; j < currentFilterSettings.overlays[i].filters.length; j++) {
            if (currentFilterSettings.overlays[i].filters[j].id == filterObject.getId())
              return currentFilterSettings.overlays[i].filters[j].enabled;
          }
        }
      }
    }
    return filterObject.isEnabled();
  },
  
  _setButtonStyleUnpressed: function(overlayID) {

    this._toggleButtonPressedStyleForElement(dojo.byId(overlayID+"_dojoButton_button"), false);
    this._toggleButtonPressedStyleForElement(dojo.byId(overlayID+"_dojoButton_arrow"), false);

    var simpleButtonElement = dojo.byId(overlayID+"_dojoButton");
    if ( simpleButtonElement )
      this._toggleButtonPressedStyleForElement(simpleButtonElement, false);
  },

  _setButtonStylePressed: function(overlayID) {
    this._toggleButtonPressedStyleForElement(dojo.byId(overlayID+"_dojoButton_button"), true);
    this._toggleButtonPressedStyleForElement(dojo.byId(overlayID+"_dojoButton_arrow"), true);

    var simpleButtonElement = dojo.byId(overlayID+"_dojoButton");
    if ( simpleButtonElement )
      this._toggleButtonPressedStyleForElement(simpleButtonElement, true);
  },
  
  _toggleButtonPressedStyleForElement: function(buttonElement, isPressed) 
  {
    if ( ! buttonElement )
      return;
      
    dojo.removeClass(buttonElement, (isPressed) ? "toggleButtonUnPressed" : "toggleButtonPressed");
    dojo.addClass(buttonElement, (isPressed) ? "toggleButtonPressed" : "toggleButtonUnPressed");
  }
  
});

}

if(!dojo._hasResource["itme.ChassisMap.ChassisMapTooltip"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.ChassisMapTooltip"] = true;
dojo.provide("itme.ChassisMap.ChassisMapTooltip");



 function CMAdditionalProperties(focusName)
  {
    this.name = focusName;
    this.isEnabled = false;
    this.valueNames = [];
    this.valueValues = [];
  }

  function CMPropIndexOf(array, valueToFind) {
    for ( var i = 0; i < array.length; i++) {
      if ( array[i].name == valueToFind)
        return i;
    }
    return -1;
  }

var myTooltipCounter = 0;

dojo.declare("itme.ChassisMap.ChassisMapTooltip", [/*dijit.Tooltip*/], {

  _parentElement: null,
  _numBaseProperties: 0,
  _baseProperties: [],
  _focusList: [],
  _title: null,
  _dijitTooltipObject: null,
  _isFocusEnabled: [],
  
  constructor: function(kwArgs, mapElement) {
    this._parentElement = mapElement; 
    this._parentElement.CMTooltip = this;
    this._focusList= [];
    this._baseProperties = [];
  },

 

  setTitle: function(title) {
    this._title = title;
  },
  
  addBaseProperty: function(name, value) {
    this._baseProperties[this._numBaseProperties++] = "<tr><td class='CMTooltipTitle'>"+name+"</td><td class='CMTooltipValue'>"+value+"</td></tr>";
  },
  
  addFocusProperty: function(focus, name, value) {
    
    var index = CMPropIndexOf(this._focusList, focus);
    if (index < 0) {
      index = this._focusList.length;
      this._focusList[index] = new CMAdditionalProperties(focus);
    }
    
    var len = this._focusList[index].valueNames.length;
    this._focusList[index].valueNames[len] = name;
    this._focusList[index].valueValues[len] = value;
  },
  
  buildTooltip: function() {
    var tooltipString = this.__addTooltipHeader() +
                        this.__addTooltipBaseProps() +
                        this.__addTooltipFocusProps() +
                        this.__addTooltipFooter();
    
    try {
      if (this._dijitTooltipObject) 
        this._dijitTooltipObject.attr("label", tooltipString);
      else { 
        this._dijitTooltipObject = new dijit.Tooltip({
          connectId: [this._parentElement],
          label: tooltipString,
          id: "myTooltip_" + myTooltipCounter
        });
        myTooltipCounter++;
      }
      this._dijitTooltipObject.defaultPosition= ["after", "before", "below"];
    } catch (err) {
      itmeUtils.CMTrace("MAIN", "dojo hitch error?: " + err);
    }
  },
  
  __addTooltipHeader: function() {
    
    var tooltipStr = "<div class='DeviceTooltip' style='max-width:450px'><h3 class='DeviceTooltipTitle'>" + this._title + "</h3>" +
                        "<div>" + "" + "</div>" +  
                        "<table>";
    return tooltipStr;
  },
  
  __addTooltipBaseProps: function() {
    
    var tooltipStr = "";
    for (var i = 0; i < this._baseProperties.length; i++)    
      tooltipStr += this._baseProperties[i];                    
    return tooltipStr;
  },

  __addTooltipFocusProps: function() {

    var tooltipStr = "";
    
    for ( var i = 0; i < this._focusList.length; i++) {
      if ( this._focusList[i].isEnabled ) {
        tooltipStr += "<tr><td colspan=2><hr class='tooltiphr'></td></tr>"; 
        for (var j = 0; j < this._focusList[i].valueNames.length; j++) {
          tooltipStr += "<tr><td class='CMTooltipTitle'>"+this._focusList[i].valueNames[j]+"</td><td class='CMTooltipValue'>"+this._focusList[i].valueValues[j]+"</td></tr>";;
        }     
      }
    }
    
    return tooltipStr;
  },

  __addTooltipFooter: function(tooltipStr) {
    
    var tooltipStr = "</table></div>";
    return tooltipStr;
  },
  
  toggleFocusEnabled: function(focus, bEnabled) {
    var index = CMPropIndexOf(this._focusList, focus);
    if ( index >= 0)
      this._focusList[index].isEnabled = bEnabled; 
  }
  
// Tooltip 

// constructor (mapElement);
// addBaseProperty(title, value);
// addFocusProperty(focus, title, value);
// switchTooltipFocus(focus)

// buildTooltip()
//  div
//    header
//    table
//      basePropertyTitle - basePropertyValue
//    /table

// if (activeFocus)

//    div id=focus
//      -------------------<hr>
//      table
// for (eachFocusProperty)
//        focusPropertyTitle - focusPropertyValue
//      /table
//    /div
//  /div


// 
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Status"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Status"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Status"); 
 
////////////////////////////////////
// Status
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Status", null, {
	// private members
	_version: null,
	_status: null,

	/*=====
	constructor_kwArgs: function(version, style) {
		// version: Float
		//		version used to communicate with rest service
		// style: Object
		//		json item style object
	},
	=====*/
	
	 constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Status object.
		this._version = kwArgs.version;
		this._status = kwArgs.status;		
		if (!this._status) {
			this._status = {};
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getAttribute: function(key) {
		return this._status[key];
	},
	
	setAttribute: function(key, value) {
		this._status[key] = value;
	},
	
	getId: function() {
		return this._status.id;
	},
	
	setId: function(id) {
		this._status.id = id;		
	},
	
	getValue: function() {
		return this._status.value;
	},
	
	setValue: function(value) {
		this._status.value = value;
	},
	
	getLabel: function() {
		if (this._status.label) {
			// string
			if (typeof this._status.label === "string") {
				return this._status.label;
			}
			// object with key/value info
			else {
				return this._status.label.value;
			}
		}
		
		return null;
	},
	
	setLabel: function(label) {
		this._status.label = label;
	},
	
	getImage: function() {
		return this._status.image;
	},
	
	setImage: function(image) {
		this._status.image = image;
	},
	
	getImageWidth: function() {
		return this._status.imageWidth;
	},
	
	setImageWidth: function(width) {
		this._status.imageWidth = width;
	},
	
	getImageHeight: function() {
		return this._status.imageHeight;
	},
	
	setImageHeight: function(height) {
		this._status.imageHeight = height;
	},
	
	isImageVisible: function() {
		return (this._status.imageVisible !== undefined ? this._status.imageVisible : true);
	},
	
	setImageVisible: function(visible) {
		this._status.imageVisible = visible;
	},
	
	getScalarImage: function() {
		return this._status.scalarImage;
	},
	
	setScalarImage: function(image) {
		this._status.scalarImage = image;
	},
	
	getScalarImageWidth: function() {
		return this._status.scalarImageWidth;
	},
	
	setScalarImageWidth: function(width) {
		this._status.scalarImageWidth = width;
	},
	
	getScalarImageHeight: function() {
		return this._status.scalarImageHeight;
	},
	
	setScalarImageHeight: function(height) {
		this._status.scalarImageHeight = height;
	},
	
	isScalarImageVisible: function() {
		return (this._status.scalarImageVisible !== undefined ? this._status.scalarImageVisible : true);
	},
	
	setScalarImageVisible: function(visible) {
		this._status.scalarImageVisible = visible;
	},
	
	getBackgroundScalarImage: function() {
		return this._status.backgroundScalarImage;
	},
	
	setBackgroundScalarImage: function(image) {
		this._status.backgroundScalarImage = image;
	},
	
	getBackgroundScalarImageWidth: function() {
		return this._status.backgroundScalarImageWidth;
	},
	
	setBackgroundScalarImageWidth: function(width) {
		this._status.backgroundScalarImageWidth = width;
	},
	
	getBackgroundScalarImageHeight: function() {
		return this._status.backgroundScalarImageHeight;
	},
	
	setBackgroundScalarImageHeight: function(height) {
		this._status.backgroundScalarImageHeight = height;
	},
	
	isBackgroundScalarImageVisible: function() {
		return (this._status.backgroundScalarImageVisible !== undefined ? this._status.backgroundScalarImageVisible : true);
	},
	
	setBackgroundScalarImageVisible: function(visible) {
		this._status.backgroundScalarImageVisible = visible;
	},
	
	getTooltip: function() {
		if (this._status.tooltip) {
			// string
			if (typeof this._status.tooltip === "string") {
				return this._status.tooltip;
			}
			// object with key/value info
			else {
				return this._status.tooltip.value;
			}
		}
		
		return null;
	},
	
	setTooltip: function(tooltip) {
		this._status.tooltip = tooltip;
	},
	
	getMessage: function() {
		return this._status.message;
	},
	
	setMessage: function(message) {
		this._status.message = message;
	},
	
	getColor: function() {
		return this._status.color;
	},
	
	setColor: function(color) {
		this._status.color = color;
	},
	
	clone: function(widgetDef, objectType) {
		var args = {
			status: {},
			version: this._version
		};		
		
		if (widgetDef) {
			// start with widgetDef default status
			var widgetDefaultStatus = null;
			switch(objectType) {
				case "item":
					widgetDefaultStatus = widgetDef.getDefaultItemStatus();
					break;
				case "link":
					widgetDefaultStatus = widgetDef.getDefaultLinkStatus();
					break;
				case "relationship":
					widgetDefaultStatus = widgetDef.getDefaultRelationshipStatus();					
			}			
			if (widgetDefaultStatus) {
				dojo.mixin(args.status, widgetDefaultStatus);
			}
		
			// mixin our status
			dojo.mixin(args.status, this._status);
			
			// mixin widgetDef status
			var widgetStatus = null;
			switch(objectType) {
				case "item":
					widgetStatus = widgetDef.getItemStatus(this.getId(), this.getValue());
					break;
				case "link":
					widgetStatus = widgetDef.getLinkStatus(this.getId(), this.getValue());
					break;
				case "relationship":
					widgetStatus = widgetDef.getRelationshipStatus(this.getId(), this.getValue());					
			}					
			if (widgetStatus) {
				dojo.mixin(args.status, widgetStatus);
			}
		}
		// just mixin our status
		else {
			dojo.mixin(args.status, this._status);
		}
			
		return new ibm.tivoli.navmodel.Status(args);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Property"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Property"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Property");
 


////////////////////////////////////
// Property
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Property", null, {
	// private members
	_prop: null,
	_dataset: null,
	_objectType: null,
	_value: null,
	_task: null,

	/*=====
	constructor_kwArgs: function(property, dataset) {
		// property: Object
		//		json property object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this item
		// objectType: String
		//		objectType of the owning object "item", "link", or "relationship"
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Property object.
		this._dataset = kwArgs.dataset;
		this._objectType = (kwArgs.objectType ? kwArgs.objectType : "item");
		this._prop = kwArgs.property;
		
		// optimization of json on the server can leave out:
		// "displayValue"
		if (!this._prop.displayValue) {
			this._prop.displayValue = this._prop.value;
		}		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	 
	clone: function() {		
		// summary:
		//		Clones the property object and returns a new one with the same values
		var args = {
			property: dojo.clone(this._prop),
			dataset: this._dataset,
			objectType: this._objectType
		};		
		
		return new ibm.tivoli.navmodel.Property(args);
	},

	mixin: function(property) {		
		// summary:
		//		Mixin contents of property with ours
		if (property !== this && property.getId() === this.getId()) {
			this._prop = property.getAttributes();
			
			// re-cache values on next request
			this._value = null;
			this._task = null;		
		}
	},

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._prop[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._prop[key] = value;
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the property
		return dojo.clone(this._prop);		
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the property
		dojo.mixin(this._prop, attr);
		
		// re-cache values on next request
		this._value = null;
		this._task = null;
	},

	getId: function() {		
		// summary:
		//		Returns the property id
		return this._prop.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the property id
		this._prop.id = id;
	},

	getLabel: function() {		
		// summary:
		//		Returns the property label
		return this._prop.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the property label
		this._prop.label = label;
	},
	
	getTooltip: function() {
		// summary:
		//		Returns the property tooltip
		return this._prop.tooltip;
	},
	
	setTooltip: function(tooltip) {
		// summary:
		//		Set the property tooltip
		this._prop.tooltip = tooltip;
	},

	getDisplayValue: function() {		
		// summary:
		//		Returns the globalized display value for the property
		return this._prop.displayValue;
	},

	setDisplayValue: function(displayValue) {		
		// summary:
		//		Set the globalized display value for the property
		this._prop.displayValue = displayValue;
	},

	getValue: function() {		
		// summary:
		//		Returns the actual value object for the property
		if (!this._value && this._prop.value !== null && this._prop.value !== undefined) {
			var valueObj = this._prop.value;
			
			switch (this.getValueType()) {
				case "date":
				case "datetime":
				case "time": 
					if (typeof valueObj === "number") {
						this._value = new Date(valueObj);
					}
					else if (typeof valueObj === "string") {
						this._value = dojo.date.stamp.fromISOString(valueObj);										
					}
					break;
				
				case "date[]":
				case "datetime[]":
				case "time[]":
					var dates = [];
					var length = valueObj.length;
					for (var index = 0;  index < length;  ++index) {
						if (typeof valueObj[index] === "number") {
							dates[index] = new Date(valueObj[index]);
						}
						else if (typeof valueObj[index] === "string") {
							dates[index] = dojo.date.stamp.fromISOString(valueObj[index]);						
						}										
					}
					this._value = dates;
					break;
				
				case "status":
					// full status (i.e. was not optimize)
					if (valueObj.label && valueObj.image) {
						this._value = new ibm.tivoli.navmodel.Status({status: valueObj, version: this._dataset.getVersion()});
					}
					// optimized status; look up with dataset
					else {
						var statusArgs = {
							type: this._objectType,
							statusId: valueObj.id,
							statusValue: valueObj.value				
						};
						this._value = this._dataset.getStatus(statusArgs);
					}
					break;
					
				default:
					this._value = valueObj;
			}
		}
		
		return this._value;
	},
	
	/*=====
	getStatusValue_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		WidgetDefinition to use for the merge		
	},
	=====*/
	
	getStatusValue: function(/*getStatusValue_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the "status" value object for the property but
		//		also merges in information from widget definition
		var args = {
			widgetDef: null,
			objectType: this._objectType
		};
		dojo.mixin(args, kwArgs);
		
		var status = null;
		if (this.getValueType() === "status") {
			status = this.getValue();
			if (status && args.widgetDef) {
				status = status.clone(args.widgetDef, args.objectType);
			}
		}
		
		return status;
	},

	setValue: function(value) {		
		// summary:
		//		Modifies the value object for the property
		this._value = value;
				
		switch (this.getValueType()) {
			case "date":
			case "datetime":
			case "time": 
				this._prop.value = value.getTime();
				break;
				
			case "date[]":
			case "datetime[]":
			case "time[]":
				this._prop.value = [];
				var length = value.length;
				for (var index = 0;  index < length;  ++index) {
					this._prop.value[index] = value[index].getTime();
				}
				break;
				
			default:
				this._prop.value = value;
		}
	},

	getValueType: function() {		
		// summary:
		//		Returns the type associated with the property value
		//		|	values:	"itemName", "itemType", "linkName", "linkType", "relationshipName", "relationshipType"
		//		|		"string", "string[]",
		//		|		"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|		"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",	
		//		|		"boolean", "boolean[]"	
		//		|		"enum", "map", "json"
		//		|		"status",
		//		|		"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|		"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|		"float", "double", "percentage", "float[]", "double[]"
		return this._prop.valueType;
	},

	setValueType: function(valueType) {		
		// summary:
		//		Set the type associated with the property value
		this._prop.valueType = valueType;
	},

	getValueState: function() {		
		// summary:
		//		Returns the state assoicated with the property value
		//		|	values:	"ok", "none", "pending", "error"
		return this._prop.valueState ? this._prop.valueState : "ok";
	},

	setValueState: function(valueState) {		
		// summary:
		//		Set the state associated with the property value
		this._prop.valueState = valueState;
	},

	hasMetadata: function() {		
		// summary:
		//		Returns true if the metadata is present in this property object
		return (this._prop.sortWeight !== undefined);
	},

	getValueUnits: function() {		
		// summary:
		//		Returns a units qualifier that can be assoicated with the property value
		//		example:  "seconds", "hours", "mph"
		return this._prop.units;
	},

	setValueUnits: function(units) {		
		// summary:
		//		Set units qualifier
		this._prop.units = units;
	},

	getValueMin: function() {		
		// summary:
		//		Returns minimum value;  for string properties this is minimum length
		return this._getMetadataObj(this._prop.valueMin);
	},

	setValueMin: function(valueMin) {		
		// summary:
		//		Set minimum value
		this._prop.valueMin = valueMin;
	},

	getValueMax: function() {		
		// summary:
		//		Returns maximum value;  for string properties this is maximum length
		return this._getMetadataObj(this._prop.valueMin);
	},

	setValueMax: function(valueMax) {		
		// summary:
		//		Set maximum value
		this._prop.valueMax = valueMax;
	},

	getValueDefault: function() {		
		// summary:
		//		Returns default value for property (i.e. if has no value and 
		//		a set defaults operation is chosen
		return this._getMetadataObj(this._prop.valueDefault);
	},

	setValueDefault: function(valueDefault) {		
		// summary:
		//		Set default value
		this._prop.valueDefault = valueDefault;
	},

	getSortWeight: function() {		
		// summary:
		//		Returns the sort weight for the property;  when sorting a diverse list of properties
		//		the sortWeight can be used to determine sort order.  Properties with equal sortWeight
		//		should be sorted alphabetically by the label.
		return this._prop.sortWeight;
	},

	setSortWeight: function(sortWeight) {		
		// summary:
		//		Set the sort weight
		this._prop.sortWeight = sortWeight;
	},

	isHidden: function() {		
		// summary:
		//		Returns true if the property should be hidden from the user
		return (this._prop.hidden !== undefined ? this._prop.hidden : false);
	},
	
	setHidden: function(hidden) {		
		this._prop.hidden = hidden;
	},

	isEditable: function() {		
		// summary:
		//		Returns true if the property is editable by the user
		return (this._prop.editable !== undefined ? this._prop.editable : true);
	},
	
	setEditable: function(editable) {
		this._prop.editable = editable;
	},

	isRequired: function() {		
		// summary:
		//		Returns true if the property is required (i.e. must have a value if edited)
		return (this._prop.required !== undefined ? this._prop.required : false);
	},
	
	setRequired: function(required) {
		this._prop.required = required;
	},

	isSensitive: function() {		
		// summary:
		//		Returns true if the property is sensitive and should not 
		//		be displayed in the clear
		return (this._prop.sensitive !== undefined ? this._prop.sensitive : false);
	},
	
	setSensitivie: function(sensitive) {
		this._prop.sensitive = sensitive;
	},
	
	getTask: function() {
		if (!this._task && this._prop.task) {
			this._task = new ibm.tivoli.navmodel.Task({task: this._prop.task, dataset: this._dataset});
		}
		
		return this._task;
	},
	
	setTask: function(task) {
		this._task = task;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_getMetadataObj: function(obj) {		
		// summary:
		//		Makes necessary conversion on objects for
		//		metadata functions like min,max,default
		if (obj) {
			switch (this.getValueType()) {
				case "date":
				case "datetime":
				case "time":
				case "date[]":
				case "datetime[]":
				case "time[]":
					if (typeof obj === "number") {
						return new Date(obj);
					}
					else if (typeof obj === "string") {
						return dojo.date.stamp.fromISOString(obj);						
					}									
			}		
		}
		
		return obj;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.LinkStyle"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.LinkStyle"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.LinkStyle"); 
 
////////////////////////////////////
// LinkStyle
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.LinkStyle", null, {
	// private members       
	_version: null,
	_style: null,
	
	/*=====
	constructor_kwArgs: function(version, style) {
		// version: Float
		//		version used to communicate with rest service
		// style: Object
		//		json item style object
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary
		//		Constructs LinkStyle object.
		this._version = kwArgs.version;
		this._style = kwArgs.style;		
		if (!this._style) {
			this._style = {};
		}	
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getAttribute: function(key) {
		return this._style[key];
	},
	
	setAttribute: function(key, value) {
		this._style[key] = value;
	},
	
	getId: function() {
		return this._style.id;
	},
	
	setId: function(id) {
		this._style = id;
	},
	
	getType: function() {
		return this._style.type;
	},
	
	setType: function(type) {
		this._style.type = type;
	},
	
	getLabel: function() {
		if (this._style.label) {
			// string
			if (typeof this._style.label === "string") {
				return this._style.label;
			}
			// object with key/value info
			else {
				return this._style.label.value;
			}
		}
		
		return null;
	},
	
	setLabel: function(label) {
		this._style.label = label;		
	},
	
	getLineStyle: function() {
		return this._style.lineStyle;
	},
	
	setLineStyle: function(lineStyle) {
		this._style.lineStyle = lineStyle;
	},
	
	getLineWidth: function() {
		return this._style.lineWidth;
	},
	
	setLineWidth: function(width) {
		this._style.lineWidth = width;
	},
	
	getLineColor: function() {
		return this._style.lineColor;
	},
	
	setLineColor: function(color) {
		this._style.lineColor = color;
	},
	
	getArrowColor: function() {
		return this._style.arrowColor;
	},

	setArrowColor: function(color) {
		this._style.arrowColor = color;
	},
	
	isLabelVisible: function() {
		return (this._style.labelVisible !== undefined ? this._style.labelVisible : false);
	},
	
	setLabelVisible: function(visible) {
		this._style.labelVisible = visible;
	},
	
	isLabelAutoRotate: function() {
		return (this._style.labelAutoRotate !== undefined ? this._style.labelAutoRotate : false);
	},
	
	setLabelAutoRotate: function(autoRotate) {
		this._style.labelAutoRotate = autoRotate;
	},

	getLabelParallelAlignment: function() {
		return (this._style.labelParallelAlignment !== undefined ? this._style.labelParallelAlignment : "middle");
	},
	
	setLabelParallelAlignment: function(parallelAlignment) {
		this._style.labelParallelAlignment = parallelAlignment;
	},
	
	getLabelOrthogonalAlignment: function() {
		return (this._style.labelOrthogonalAlignment !== undefined ? this._style.labelOrthogonalAlignment : "left");
	},
	
	setLabelOrthogonalAlignment: function(orthogonalAlignment) {
		this._style.labelOrthogonalAlignment = orthogonalAlignment;
	},

	getLabelDistance: function() {
		return (this._style.labelDistance !== undefined ? this._style.labelDistance : 5);
	},
	
	setLabelDistance: function(distance) {
		this._style.labelDistance = distance;
	},

	getLabelPositionType: function() {
		return (this._style.labelPositionType !== undefined ? this._style.labelPositionType : "relative");
	},
	
	setLabelPositionType: function(positionType) {
		this._style.labelPositionType = positionType;
	},

	getLabelPosition: function() {
		return (this._style.labelPosition !== undefined ? this._style.labelPosition : 0.5);
	},
	
	setLabelPosition: function(position) {
		this._style.labelPosition = position;
	},
	
	clone: function(widgetDef, objType) {
		var args = {
			style: {},
			version: this._version
		};		
		
		if (widgetDef) {
			// start with widgetDef default style
			var widgetDefaultStyle = null;
			switch(objType) {
				case "link":
					widgetDefaultStyle = widgetDef.getDefaultLinkStyle();
					break;
				case "relationship":
					widgetDefaultStyle = widgetDef.getDefaultRelationshipStyle();
			}
			if (widgetDefaultStyle) {
				dojo.mixin(args.style, widgetDefaultStyle);
			}
		
			// mixin our link style
			dojo.mixin(args.style, this._style);
			
			// mixin widgetDef style
			var widgetStyle = null;
			switch(objType) {
				case "link":
					widgetStyle = widgetDef.getLinkStyle(this.getType());
					break;
				case "relationship":
					widgetStyle = widgetDef.getRelationshipStyle(this.getType());
			}		
			if (widgetStyle) {
				dojo.mixin(args.style, widgetStyle);
			}
		}
		// just mixin our link style
		else {
			dojo.mixin(args.style, this._style);
		}
			
		return new ibm.tivoli.navmodel.LinkStyle(args);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Exception"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Exception"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Exception"); 
 
////////////////////////////////////
// Exception
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Exception", null, {
	// private members
	_version: null,
	_exception: null,
	
	/*=====
	constructor_kwArgs: function(version, exception) {
		// version: Float
		//		version used to communicate with rest service
		// exception: Object
		//		json exception object
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary
		//		Constructs Exception object.
		this._version = kwArgs.version;
		this._exception = kwArgs.exception;		
	},

	getMsgId: function() {
		return this._exception.msgId;
	},
	
	getMsgSeverity: function() {
		return this._exception.msgSeverity;
	},
	
	getMsgText: function() {
		return this._exception.msgText;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Utils"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Utils"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Utils"); 

////////////////////////////////////
// Utils
////////////////////////////////////

ibm.tivoli.navmodel.Utils.isTIP = function() {
	//	summary:
	//		Returns true if environment is running in Tivoli Integrated Portal  
  	var themes = ["TIPLight", "TIPDark", "ISCTheme"];
  	var themeCount = themes.length;
  	var body = dojo.body();
  	for (var index = 0;  index < themeCount;  ++index) {
  		if (dojo.hasClass(body, themes[index])) {
  			return true;
  		}
  	}
  	
  	return false;  
};

ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion = function() {
	return 1.0;
};

ibm.tivoli.navmodel.Utils.getContentType = function(version, format) {
	// summary:
	//		Return the HttpHeader "Content-Type" that REST service is expecting
	//		;format=xxxx   can be appended to indicate the json format object
	var contentType = "application/vnd.ibm.com.tivolidis.json;version=" + version;	
	if (format) {
		contentType += ";format=" + format;
	}
	
	return contentType;
};

ibm.tivoli.navmodel.Utils.getContent = function() {		
	// summary:
	//		Construct a "content" object for sending an xdr request;
	//		IE will sometimes return http requests from cache without unique parms
	var content = {};
		
	// add date and random
	var now = new Date();		
	content.d = now.getTime();
	content.r = Math.random();
		
	return content;
};

ibm.tivoli.navmodel.Utils.mixinParams = function(content, dataset) {		
	// summary:
	//		Mixin the "param_" qualified arguments;  all parameters are 
	//		prefixed with "param_" so the rest service can use them to identify
	//		dataset cache entry
	var parameters = dataset.getArg("parameters");
	for (var parm in parameters) {
		if (parameters.hasOwnProperty(parm)) {
			content["param_" + parm] = parameters[parm];
		}
	}
};

ibm.tivoli.navmodel.Utils.mixinProperties = function(content, key, properties) {		
	// summary:
	//		Mixin the "properties" into the content as a comma delimited list
	if (properties && properties.length > 0) {
		content[key] = properties.join(",");
	}
};

ibm.tivoli.navmodel.Utils.getParamUrl = function(url, dataset) {		
	// summary:
	//		Tack on dataset version and param_* values to the url
	var questionMark = false;
		
	// add "param_" parms
	var parameters = dataset.getArg("parameters");
	for (var parm in parameters) {
		if (parameters.hasOwnProperty(parm)) {
			if (!questionMark) {
				questionMark = true;
				url += "?";
			}
			else {
				url += "&";
			}
			url += ("param_" + parm + "=" + parameters[parm]);				
		}
	}
		
	// IE will sometimes return http requests from cache without unique parms
	var now = new Date();
	if (!questionMark) {
		questionMark = true;
		url += "?d=" + now.getTime() + "&r=" + Math.random();
	}
	else {
		url += "&d=" + now.getTime() + "&r=" + Math.random();
	}
	
	return url;	
};

ibm.tivoli.navmodel.Utils.getException = function(error, ioArgs, funcName, trace, version) {		
	// summary:
	//		Returns Exception object contained in the dojo.xhr ioArgs
	var exception = null;
	var responseText = ioArgs.xhr.responseText;
	if (responseText) {
		try {
			var jsonEx = dojo.fromJson(responseText);
			if (jsonEx.msgId) {
				trace.error(funcName + " generated exception=" + responseText, "getException");				
				exception = new ibm.tivoli.navmodel.Exception({exception: jsonEx, version: version});
			}
			else {
				trace.error(funcName + " generated exception (non rest) = " + error, "getException");				
			}
		}
		catch(e) {
			trace.error(funcName + " generated exception (non rest) = " + error, "getException");
			exception = new ibm.tivoli.navmodel.Exception({exception: {msgText: error}, version: version});
		}
			
		trace.error("exception url=" + ioArgs.url, "getException");							
	}
	
	return exception;
};

ibm.tivoli.navmodel.Utils.NorthernHemisphere = {
	"GMT-10:00": "US/Aleutian",
  	"GMT-9:00":  "US/Alaska",
  	"GMT-8:00":  "US/Pacific",
  	"GMT-7:00":  "US/Mountain",
	"GMT-6:00":  "US/Central",
	"GMT-5:00":  "US/Eastern",
	"GMT-4:00":  "Canada/Atlantic",
	"GMT-3:30":  "Canada/Newfoundland",
	"GMT-3:00":  "America/Godthab",
    "GMT-2:00":  "America/Noronha",
    "GMT-1:00":  "Atlantic/Azores",
    "GMT":       "Europe/London",
    "GMT+1:00":  "Europe/Amsterdam",
    "GMT+2:00":  "Europe/Athens",
    "GMT+3:00":  "Europe/Moscow",
    "GMT+3:30":  "Asia/Tehran",
    "GMT+4:00":  "Asia/Baku",
    "GMT+5:00":  "Asia/Yekaterinburg",
    "GMT+6:00":  "Asia/Novosibirsk",
    "GMT+7:00":  "Asia/Krasnoyarsk",
    "GMT+8:00":  "Asia/Irkutsk",
    "GMT+9:00":  "Asia/Yakutsk",
    "GMT+10:00": "Asia/Vladivostok"
};

ibm.tivoli.navmodel.Utils.SouthernHemisphere = {
	"GMT-4:00":  "America/Asuncion",
	"GMT-3:00":  "America/Montevideo",
	"GMT+2:00":  "Africa/Windhoek",
	"GMT+9:30":  "Australia/Adelaide",
	"GMT+10:00": "Australia/Sydney",
	"GMT+12:00": "Pacific/Fiji"
};

ibm.tivoli.navmodel.Utils.NoDST = {  
	"GMT-11:00": "US/Samoa",
	"GMT-10:00": "US/Hawaii",
	"GMT-7:00":	 "US/Arizona",
	"GMT-6:00":  "Canada/Saskatchewan",
	"GMT-5:00":  "US/East-Indiana",
	"GMT-4:30":  "America/Caracas",
	"GMT-4:00":  "America/Manaus",
	"GMT-3:00":  "America/Buenos_Aires",
	"GMT-2:00":  "Atlantic/South_Georgia",
	"GMT-1:00":  "Atlantic/Cape_Verde",
	"GMT":       "Africa/Monrovia",
	"GMT+1:00":  "Africa/Algiers",
	"GMT+2:00":  "Africa/Harare",
	"GMT+3:00":  "Asia/Baghdad",
	"GMT+4:00":  "Asia/Dubai",
	"GMT+4:30":  "Asia/Kabul",
	"GMT+5:00":  "Asia/Karachi",
	"GMT+5:30":  "Asia/Kolkata",
	"GMT+5:45":  "Asia/Katmandu",
	"GMT+6:00":  "Asia/Dhaka",
	"GMT+6:30":  "Asia/Rangoon",
	"GMT+7:00":  "Asia/Bangkok",
	"GMT+8:00":  "Asia/Hong_Kong",
	"GMT+9:00":  "Asia/Tokyo",
	"GMT+9:30":  "Australia/Darwin",
	"GMT+10:00": "Australia/Brisbane",
	"GMT+11:00": "Asia/Magadan",
	"GMT+12:00": "Pacific/Wake",
	"GMT+13:00": "Pacific/Tongatapu"
};

// OlsonTimeZone name
ibm.tivoli.navmodel.Utils.timeZoneName = null;

ibm.tivoli.navmodel.Utils.setTimeZone = function(timeZoneName) {
	// summary:
	//		Set the timeZoneName that will be returned by getTimeZone()	
	ibm.tivoli.navmodel.Utils.timeZoneName = timeZoneName;
};

ibm.tivoli.navmodel.Utils.getTimeZone = function() {
	// summary:
	//		Calculate the users Olson timezone name
	
	// if one has been set; just return it
	if (ibm.tivoli.navmodel.Utils.timeZoneName) {
		return ibm.tivoli.navmodel.Utils.timeZoneName;
	}
	
	var janUTC = ibm.tivoli.navmodel.Utils.getUTCOffset(1, 1);
	var janGMT = ibm.tivoli.navmodel.Utils.getGMTOffset(1, 1);
	var julyUTC = ibm.tivoli.navmodel.Utils.getUTCOffset(7, 1);
	var julyGMT = ibm.tivoli.navmodel.Utils.getGMTOffset(7, 1);	
	var dst = (janUTC !== julyUTC);	
	var tzName = null;
	
	// DST
	if (dst) {
		if (janUTC < julyUTC) {
			tzName = ibm.tivoli.navmodel.Utils.NorthernHemisphere[janGMT];
		}
		else {
			tzName = ibm.tivoli.navmodel.Utils.SouthernHemisphere[julyGMT];
		}
	}
	// No DST
	else {
		tzName = ibm.tivoli.navmodel.Utils.NoDST[janGMT];
	}
	
	// can't find it in tables; so return GMT offset
	if (!tzName) {
		tzName = ibm.tivoli.navmodel.Utils.getGMTOffset();
	}
	
	//alert('jan=' + janGMT + '  july=' + julyGMT + ' dst=' + dst + ' tz=' + tzName);
		
	return tzName;
};

ibm.tivoli.navmodel.Utils.getUTCOffset = function(month, day) {
	// summary:
	//		Calculate the users current GMT offset
	var date = new Date();
	if (month) {
		date.setMonth(month);
	}
	if (day) {
		date.setDate(day);
	}
	
	return -(date.getTimezoneOffset());	
};

ibm.tivoli.navmodel.Utils.getGMTOffset = function(month, day) {
	// summary:
	//		Calculate the users current GMT offset
	var date = new Date();
	if (month) {
		date.setMonth(month);
	}
	if (day) {
		date.setDate(day);
	}
	
	var timezone = "";
	var minutesFromGMT = date.getTimezoneOffset();
	var extraMinutes = minutesFromGMT%60;
	var gmtHours = null;
	var gmtMinutes = "00";
		
	if (extraMinutes === 0) {
		gmtHours = -(minutesFromGMT/60);
	}
	else {
		gmtHours = -((minutesFromGMT-extraMinutes)/60);
		gmtMinutes = (Math.abs(extraMinutes)).toString();		
	}	
		
	timezone = "GMT";
	if (gmtHours !== 0) {
		if (gmtHours > 0) {
			timezone += "+";				
		}
		timezone += gmtHours;
		timezone += ":";
		timezone += gmtMinutes;
	}
		
	return timezone;
};

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Trace"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Trace"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Trace");
 
dojo.declare("ibm.tivoli.navmodel.Trace", null, {
    // private members     
	traceName: "",
	level: 0,  // error
        
	constructor: function(name) {
		this.traceName = name;
				
		if (dojo.config.trace) {
			// "all" overrides individual class entries
			if (dojo.config.trace.all) {
				this.level = this._translateLevel("all", dojo.config.trace.all);
			}
			else {
				var lastDot = this.traceName.lastIndexOf(".");
				if (lastDot > 0) {
					var className = this.traceName.substring(lastDot+1);
					if (dojo.config.trace[className]) {
						this.level = this._translateLevel(className, dojo.config.trace[className]);
					}
				}
			}
		}					
 	},
        
	/////////////////////////////
	// public methods
	/////////////////////////////
	
	isError: function() {
		return (this.level >= 0);
	},
	
	isWarn: function() {
		return (this.level >= 1);
	},
	
	isInfo: function() {
		return (this.level >= 2);
	},
	
	isFine: function() {
		return (this.level >= 3);
	},
	
	isFiner: function() {
		return (this.level >= 4);
	},
	
	isFinest: function() {
		return (this.level >= 5);
	},
	
	error: function(dbgString, methodName) {
		if (this.isError()) {
			if (dbgString && dbgString instanceof Error) {
                console.error(this._output("EXCEPTION", methodName, dbgString.toString()));
	    	} 
	    	else if (dbgString && dbgString.isInstanceOf && dbgString.isInstanceOf(ibm.tivoli.navmodel.Exception)) {
	    		console.error(this._output("EXCEPTION", methodName, dbgString.getMsgText()));
	    	} else {
	    		console.error(this._output("ERROR", methodName, dbgString));
	    	}
	    }
	},
	
	warn: function(dbgString, methodName) {
    	if (this.isWarn()) {
    		console.warn(this._output("WARN", methodName, dbgString));
    	}
    },
    
	info: function(dbgString, methodName) {
    	if (this.isInfo()) {
    		console.info(this._output("INFO", methodName, dbgString));
    	}
    },
    
    fine: function(dbgString, methodName) {
    	if (this.isFine()) {
    		console.info(this._output("FINE", methodName, dbgString));
    	}
    },
    
    finer: function(dbgString, methodName) {
    	if (this.isFiner()) {
    		console.info(this._output("FINER", methodName, dbgString));
    	}
    },
    
	finest: function(dbgString, methodName) {
    	if (this.isFinest()) {
    		console.info(this._output("FINEST", methodName, dbgString));
    	}
    },
	
	entering: function(dbgString, methodName) {
    	if (this.isFinest()) {
    		console.info(this._output("ENTERING", methodName, dbgString));
    	}
    },
    
    exiting: function(dbgString, methodName) {
    	if (this.isFinest()) {
    		console.info(this._output("EXITING", methodName, dbgString));
    	}
    },
    
	object: function(dbgObject, methodName) {
    	if (this.isFine()) {
    		console.log(this._output("OBJECT", methodName, dojo.toJson(dbgObject)));
    	}
    },
    
	assert: function(predicate, message, methodName) {
		if (this.isFine()) {
			if (predicate === false ) {
				console.log(this._output("ASSERT", methodName, "assertion failed: " + message));
			}
		}		
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_translateLevel: function(className, value) {
		var level = 0;
		
		switch(value) {
			case "off":
				level = -1;
				break;
			case "error":
				level = 0;
				break;
			case "warn":
				level = 1;
				break;
			case "info":
				level = 2;
				break;
			case "fine":
				level = 3;
				break;
			case "finer":
				level = 4;
				break;
			case "finest":
				level = 5;
				break;
			default:
				console.log("className=" + className + " has invalid level=" + value);
		}
		
		return level;
	},
	
	//***** DEBUG::trace=ibm.tivoli.navmodel.Dataset, method=EXITING; event received, type = itemStatusChanged
	_output: function(level, methodName, message) {
		var outputMessage = "***** " + level;
		if (message) {
			outputMessage = outputMessage + ": " + message;
		}
		outputMessage = outputMessage + " ::trace=" + this.traceName;
		if (methodName) {
			outputMessage = outputMessage + ", method=" + methodName;
		}
		return outputMessage;
	}	
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Item"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Item"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Item"); 
 
// includes








ibm.tivoli.navmodel.Item._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Item");

////////////////////////////////////
// Item
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Item", null, {
	// private members       
	_item: null,
	_dataset: null,
	_props: null,
	_propsById: null,
	_nullPropIds: null,
	_allProps: false,
	_allMetadata: false,
	_status: null,
	_style: null,
	_defaultTask: null,
	_expiration: null,
	_trace: ibm.tivoli.navmodel.Item._trace,
	
	/*=====
	constructor_kwArgs: function(item, dataset, propIds, allProperties, allMetadata, expiration) {
		// item: Object
		//		son item object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this item
		// propIds: String[]
		//		optional array indicating the property Id's that were requested with 
		//		the item; so it can be determined if any properties are null
		// allProperties: Boolean
		//		optional: indicates if the item already has "all" properties
		// allMetadata: Boolean
		//		optional: indicates if item properties already have metadata
		// expiration: Long
		//		optional: expiration time;  will use (current time + interval) if not supplied
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Item object.
		this._item = kwArgs.item;
		if (!this._item) {
			this._item = {};
		}
		
		this._dataset = kwArgs.dataset;
		this._props = [];
		this._propsById = {};
		this._nullPropIds = {};			
		this._allProps = kwArgs.allProperties;
		this._allMetadata = kwArgs.allMetadata;
		
		if (this._item.properties) {
			var propsLength = this._item.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = new ibm.tivoli.navmodel.Property({property: this._item.properties[propIndex], dataset: this._dataset, objectType: "item"});
				this._props[propIndex] = prop;
				this._propsById[prop.getId()] = prop;
			}
			
			// see if we should mark any properties as null because they were not returned
			if (kwArgs.propIds) {
				propsLength = kwArgs.propIds.length;
				for (propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var propId = kwArgs.propIds[propIndex];				
					if (!this._propsById[propId]) {
						this._nullPropIds[propId] = true;
					}
				}
			}
		}
		
		// expiration was supplied
		if (kwArgs.expiration) {
			this._expiration = kwArgs.expiration;
		}
		// reset to current time + interval
		else {
			this.resetExpiration();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	mixin: function(item, resetLazyLoad) {
		// summary:
		//		Mixin properties and status from this Item object	
		if (item && item !== this) {
			this.removeAttribute("style");
			this._style = null;
			
			// reset lazy load indicators;  mixin will
			// provide the current values
			if (resetLazyLoad) {
				this.resetLazyLoad();
			}
			
			this.mixinAttributes(item.getAttributes());
			this.mixinProperties(item._props, item._nullPropIds);
			this._status = item.getStatuses({widgetDef: null});
			this._defaultTask = item.getDefaultTask();			
			this._allProps = (this._allProps || item._allProps);
			this._allMetadata = (item._allProps && item._allMetadata);				
		}	
	},

	getDataset: function() {
		// summary:
		//		Returns the dataset that owns the item
		return this._dataset;
	},

	getObjectType: function() {
		// summary:
		//		Indicate that this object is an "item"
		return "item";
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._item[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._item[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._item[key];
	},

	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the item
		var item = dojo.clone(this._item);
		
		// make sure "properties" are current as we may have received events
		if (this._props && this._props.length > 0) {
			item.properties = [];
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				item.properties[propIndex] = this._props[propIndex].getAttributes();
			}
		}
		
		return item;		
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the item
		dojo.mixin(this._item, attr);		
	},

	getUri: function() {
		if (!this._item.uri) {
			this._item.uri = this._dataset.getAttribute("uri") + "/items/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._item.uri;
	},
	
	getId: function() {		
		// summary:
		//		Returns item id
		return this._item.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the item id
		this._item.id = id;		
	},

	getType: function() {		
		// summary:
		//		Returns the type of item
		return this._item.type;
	},
	
	setType: function(type) {		
		// summary:
		//		Set the item type
		this._item.type = type;		
	},

	getTypeLabel: function() {		
		// summary:
		//		Returns globalized label for the item type
		var label = this._item.typeLabel;
		if (!label) {
			var style = this.getItemStyle({widgetDef: null});
			if (style) {
				label = style.getLabel();
			}
		}
		
		return label;
	},

	setTypeLabel: function(typeLabel) {		
		// summary:
		//		Set the globalized label for the item type
		this._item.typeLabel = typeLabel;		
	},

	getLabel: function() {		
		// summary:
		//		Returns globalized label for the item
		return this._item.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the globalized label for the item
		this._item.label = label;		
	},

	getDescription: function() {		
		// summary:
		//		Returns globalized description for the item
		return this._item.description;
	},

	setDescription: function(description) {		
		// summary:
		//		Set the globalized description for the item
		this._item.description = description;		
	},

	getTooltip: function() {		
		// summary:
		//		Returns globalized toolip for the item
		return this._item.tooltip;
	},

	setTooltip: function(tooltip) {		
		// summary:
		//		Set the globalized tooltip for the item
		this._item.tooltip = tooltip;
	},
	
	isLazyLoad: function() {		
		// summary:
		//		Returns true if the item is elligible for "lazy load"
		return (this._item.lazyLoad !== undefined ? this._item.lazyLoad : false);
	},

	setLazyLoad: function(lazyLoad) {		
		// summary:
		//		Set the lazy load attribute
		this._item.lazyLoad = lazyLoad;
	},

	getLazyLoadRelationships: function() {		
		// summary:
		//		Returns the relationships that should be explored for related items
		return this._item.relationshipsKey;
	},

	resetLazyLoad: function() {
		// summary:
		//		Resets current attributes determing whether a lazyLoad
		//		should be performed to find related items
		delete this._item.lazyLoad;
		delete this._item.relationshipsKey;				
	},
	
	setLazyLoadRelationships: function(relationshipsKey) {		
		// summary:
		//		Set the relationships that should be explored for related items
		this._item.relationshipsKey = relationshipsKey;
	},

	isSubgraph: function() {
		// summary:
		//		Returns true if the item represents a nested subgraph parent
		return (this._item.subgraph !== undefined ? this._item.subgraph : false);
	},
	
	setSubgraph: function(subgraph) {		
		// summary:
		//		Set the attribute indicating if the item is a nested subgraph parent
		this._item.subgraph = subgraph;
	},
	
	getSubgraphId: function() {
		// summary:
		//		Returns id of nested subgraph parent; no value if item is not in a subgraph
		return this._item.subgraphId;
	},
	
	setSubgraphId: function(subgraphId) {
		// summary:
		//		Set the id of the sub-graph parent
		this._item.subgraphId = subgraphId;
	},
	
	isSubgraphOrigin: function() {
		// summary:
		//		Returns true if the item is an origin within a nested subgraph
		return (this._item.subgraphOrigin !== undefined ? this._item.subgraphOrigin : false);
	},
	
	setSubgraphOrigin: function(subgraphOrigin) {
		// summary:
		//		Set the attribute indicating if the item is an origin within a nested subgraph
		this._item.subgraphOrigin = subgraphOrigin;
	},
	
	/*=====
	getStatuses_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/

	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns ibm.tivoli.navmodel.Status[] objects indicating the item status values
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if expired mixin from dataset cache
		this._validateExpiration();
				
		if (!this._status && this._item.status) {
			this._status = [];
			var statusLength = this._item.status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				if (this._item.status.label && this._item.status.image) {
					this._status.push(new ibm.tivoli.navmodel.Status({status: this._item.status[statusIndex], version: this._version()}));				
				}
				else {
					var statusArgs = {
						type: "item",
						statusId: this._item.status[statusIndex].id,
						statusValue: this._item.status[statusIndex].value				
					};
				
					var statusObj = this._dataset.getStatus(statusArgs);
					if (statusObj) {				
						this._status.push(statusObj);
					}
					else {
						this._trace.error("Could not locate status with id=" + statusArgs.statusId + " value=" + statusArgs.statusValue, "getStatuses");
					}
				}
			}			
		}
		
		// merge widget definition status
		var status = this._status;
		if (status && args.widgetDef) {
			status = this._status.slice(0);
			var statusLength2 = this._status.length;
			for (var statusIndex2 = 0;  statusIndex2 < statusLength2;  ++statusIndex2) {
				status[statusIndex2] = this._status[statusIndex2].clone(args.widgetDef, "item");
			}			
		}
		
		return status;		
	},

	setStatuses: function(statuses) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Status[] objects indicating the item status values
		this._status = statuses;
	},

	supportsProperties: function() {		
		// summary:
		//		Indicates if "Properties" is supported for the item
		return (this._item.supportsProperties !== undefined ? this._item.supportsProperties : true);
	},

	setSupportsProperties: function(supportsProperties) {		
		// summary:
		//		Set whether item  supports "Properties"
		this._item.supportsProperties = supportsProperties;
	},

	hasCachedMetadata: function() {		
		// summary:
		//		Returns true if item has all properties and associated metadata cached
		return this._allMetadata;
	},

	hasAllCachedProperties: function() {		
		// summary:
		//		Returns true if item has all properties cached
		return this._allProps;
	},

	hasCachedProperty: function(/*String*/ propId) {
		// summary:
		//		Returns true if we have the requested property in the cache
		// propId
		//		property id
		return (this._propsById[propId] ? true : false);
	},
	
	hasCachedProperties: function(/*String[]*/ properties) {		
		// summary:
		//		Returns true if we have all the requested properties in the cache
		// properties:
		//		array of property ids
		if (properties) {
			var propsLength = properties.length;
			
			// asking if we have "all" properties
			if (propsLength === 1 && properties[0] === "all") {
				return this._allProps;
			}
			
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (!this._propsById[properties[propIndex]]) {
					return false;
				}
			}
		}
		
		return true;
	},

	getCachedPropertyIds: function(expandAll) {		
		// summary:
		//		Returns array of property Ids currently cached in the item
		var propIds = [];
		
		// have all the properties
		if (!expandAll && this._allProps) {
			propIds.push("all");
		}
		// get individual property ids	
		else if (this._props) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				propIds.push(this._props[propIndex].getId());
			}
		}
		return propIds;
	},
	
	/*=====
	properties_onComplete: function(properties, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous
	},
	=====*/

	getProperties: function(/*getProperties_kwArgs*/ kwArgs) {	
		// summary:
		//		Asynchronously returns properties for the item
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: ["all"],
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		// if expired mixin from dataset cache
		this._validateExpiration(kwArgs);
				
		// fetch all properties
		if (args.properties.length === 1 && args.properties[0] === "all") {
			if (this._allProps && (!args.metadata || this._allMetadata)) {				
				if (args.sync) {
					args.onComplete(this._props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._props, args), 0);
				}		
			}
			else {
				this._fetchProperties(args);
			}
		}		
		// fetch selective properties
		else {
			// look up properties in cache
			var props = [];
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var propId = args.properties[propIndex];
				var prop = this._propsById[propId];				
				if (prop && (!args.metadata || prop.hasMetadata())) {
					props[propIndex] = prop;					
				}
				else {
					break;
				}
			}
			
			// if we found all properties in cache then return them
			if (props.length === propsLength) {
				if (args.sync) {
					args.onComplete(props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, props, args), 0);
				}
			}
			// otherwise we need to fetch
			else {
				this._fetchProperties(args);
			}
		}
	},

	setProperties: function(properties) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Property[] objects for the item
		//		
		//		Note: client side only operation;  no request to rest service
		this._propsById = {};
		this._nullPropIds = {};
		this._props = [];
		
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = properties[propIndex];
				this._propsById[prop.getId()] = prop;
			}
		}		
	},

	mixinProperties: function(properties, nullPropIds) {		
		// summary:
		//		Mixin the ibm.tivoli.navmodel.Property[] objects for the item
		//		
		//		Note: client side only operation;  no request to rest service
		
		// mixin property objects
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				this.mixinProperty(properties[propIndex]);
			}					
		}
		
		// mixin any propIds that were null
		if (nullPropIds) {
			for (var propId in nullPropIds) {
				if (nullPropIds.hasOwnProperty(propId)) {
					this._nullPropIds[propId] = true;
				}
			}
		}
	},

	getProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// if expired mixin from dataset cache
		this._validateExpiration();
						
		// check prop cache
		var prop = this._propsById[propId];
		
		if (!prop) {
			// we have asked for this property before and it wasn't found
			if (this._nullPropIds[propId]) {
				return null;
			}
			
			// check default properties on item	
			if (this._item.properties) {
				var propsLength = this._item.properties.length;
				for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var rawProp = this._item.properties[propIndex];
					if (rawProp.id === propId) {
						prop = new ibm.tivoli.navmodel.Property({property: rawProp, dataset: this._dataset, objectType: "item"});
						
						// cache property
						this._propsById[propId] = prop;
						break;
					}
				}				
			}
		}	
		
		// fetch it
		if (!prop && !this._allProps) {
			prop = this._fetchProperty(propId);			
		}		
		
		return prop;
	},

	setProperty: function(property) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Property for the item
		//		
		//		Note: client side only operation;  no request to rest service
		var found = this._propsById[property.getId()];		
		if (found) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (this._props[propIndex].getId() === property.getId()) {
					this._props[propIndex] = property;
					break;
				}
			}
		}
		else {
			this._props.push(property);
		}		
		
		this._propsById[property.getId()] = property;
	},

	mixinProperty: function(property) {		
		// summary:
		//		Mixin ibm.tivoli.navmodel.Property for the item
		//		
		//		Note: client side only operation;  no request to rest service
		var prop = this._propsById[property.getId()];
		if (prop) {
			prop.mixin(property);
		}
		else {
			this._props.push(property);
			this._propsById[property.getId()] = property;
		}	
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	editProperties_kwArgs: function(onComplete, onError, properties) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		array of property ids to set
	},
	=====*/

	editProperties: function(/*editProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously modify properties for the item
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.properties && args.properties.length > 0) {
			var item = dojo.clone(this._item);
			item.properties = [];
			
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				item.properties[propIndex] = args.properties[propIndex].getAttributes();
			}
			
			var json = dojo.toJson(item, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: false,			
				load: dojo.hitch(this, function(result, ioArgs) {
					args.onComplete(args);						
				}),
				error: dojo.hitch(function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperties", this._trace, this._version());				               
				})
			});	
		}
		// nothing to set
		else {
			setTimeout(dojo.hitch(this, args.onComplete, args), 0);
		}		
	},

	 editProperty: function(/*String*/ property) {		
		// summary:
		//		Synchronously modify property for the item
		// property:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		
		if (property) {
			var item = dojo.clone(this._item);
			item.properties = [property.getAttributes()];
			
			var json = dojo.toJson(item, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: true,			
				load: dojo.hitch(this, function(result, ioArgs) {						
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperty", this._trace, this._version());									              
				})
			});				
		}
		
		if (exception) {
			throw exception;
		}
	},

	hasOwnItemStyle: function() {		
		// summary:
		//		Returns true if the item has an ItemStyle that overrides the default
		//		item style for the item type  (i.e. different than the one retrieved
		//		from the dataset)
		return (this._item.style ? true: false);
	},
	
	/*=====
	getItemStyle_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/

	getItemStyle: function(/*getItemStyle_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.ItemStyle for this item
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (!this._style) {
			if (this._item.style) {
				this._style = new ibm.tivoli.navmodel.ItemStyle({style: this._item.style, version: this._version()});
			}
			else {
				this._style = this._dataset.getItemStyle({itemType: this.getType(), widgetDef: null});
			}
		}
		
		// merge widget definition styles 
		var style = this._style;
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (style) {
				style = style.clone(args.widgetDef);
			}
			// unknown style; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getItemStyle(this.getType());
				if (widgetStyle) {
					style = this._dataset._createItemStyle(args.widgetDef, widgetStyle);
				}
			}			
		}
		
		return style;
	},

	setItemStyle: function(style) {		
		// summary:
		//		Set item style for the item
		this._style = style;
	},

	getDefaultTask: function() {		
		// summary:
		//		Returns optional default task associated with the item
		if (!this._defaultTask && this._item.defaultTask) {
			this._defaultTask = new ibm.tivoli.navmodel.Task({task: this._item.defaultTask, dataset: this._dataset});
		}
		
		return this._defaultTask;
	},

	setDefaultTask: function(defaultTask) {		
		// summary:
		//		Set optional default task associated with the item
		this._defaultTask = defaultTask;
	},

	isOrigin: function() {		
		// summary:
		//		Returns true if the item is an origin of a topology or tree
		return (this._item.origin !== undefined ? this._item.origin : false);
	},

	setOrigin: function(isOrigin) {		
		// summary:
		//		Set the value indicating if the Item is an origin of a topology or tree
		this._item.origin = isOrigin;		
	},
	
	/*=====
	itemsAndLinks_onComplete: function(items, links, datasetItems, kwArgs){},
	=====*/
	
	/*=====
	getRelatedItems_kwArgs: function(onComplete, onError, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationships: String
		//		comma delimited list of relationships to determine related items
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		keywords: child, parent, all (if null; value from item will be used)
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns items that are related to the requested item
		var args = {                
			onComplete: function(items, links, datasetItems, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			itemId: this.getId(),
			relationships: this.getLazyLoadRelationships(),
			start: 0,
			count: null,
			properties: null,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		if (!args.relationships) {
			args.relationships = this.getLazyLoadRelationships();
		}
		
		if (!args.properties) {
			args.properties = this.getCachedPropertyIds();
		}
		
		// call the dataset to get the related items
		this._dataset.getRelatedItems(args);
	},

	isExpired: function() {		
		// summary:
		//		Returns true if the item is currently expired
		return (this._expiration ? false : true);
	},

	checkExpiration: function(nowTime) {		
		// summary:
		//		Check the expiration time on the item; return true if expired
		var expired = false;
		
		// already set to null from a previous check
		if (!this._expiration) {
			expired = true;
		}
		// check agaist the time to see if expired
		else if (nowTime >= this._expiration) {
			this._expiration = null;
			expired = true;
		}
		
		return expired;		
	},

	resetExpiration: function() {		
		// summary:
		//		Rest the expiration time for this item
		if (this._dataset.cacheExpires()) {
			this._expiration = this._dataset.getNewExpirationTime();
		}
		else {
			this._expiration = -1;
		}
	},
			
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},

	_validateExpiration: function(kwArgs) {		
		// summary:
		//		If item is expired; mixin with a current item from the dataset cache
		if (this._dataset.cacheExpires()) {
			// expired
			if (this.isExpired()) {
				var properties = null;
				var metadata = false;
				if (kwArgs) {
					properties = kwArgs.properties;
					metadata = kwArgs.metadata;
				}			
				if (!properties) {
					properties = this.getCachedPropertyIds();
				}
		
				var item = this._dataset.getItem({itemId: this.getId(), properties: properties, metadata: metadata});
				if (item) {
					this.mixin(item);
				}			
			}
			// NOT expired; reset expiration since touched
			else {
				this.resetExpiration();
			}
		}
	},
	
	/*=====
	fetchProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous	
	},
	=====*/
	
	_fetchProperties: function(/*fetchProperties_kwArgs*/ kwArgs) {	
		// summary:
		//		Asynchronously returns properties for the item
		this._props = [];
		this._allProps = false;
		this._allMetadata = false;
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		content.optimize = true;
		content.metadata = kwArgs.metadata;
		
		// are we fetching "all" properties
		var fetchAll = (kwArgs.properties.length === 1 && kwArgs.properties[0] === "all");
		
		return dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var props = [];
				var rawProps = [];
				if ( result && result.properties )
				  rawProps = result.properties;
				var rawPropsLength = rawProps.length;
				for (var propIndex = 0;  propIndex < rawPropsLength;  ++propIndex) {
					var prop = new ibm.tivoli.navmodel.Property({property: rawProps[propIndex], dataset: this._dataset, objectType: "item"});
					props[propIndex] = prop;
					
					if (fetchAll) {
						this._props[propIndex] = prop;
					}
					
					this._propsById[prop.getId()] = prop;
				}
				
				if (fetchAll) {
					this._allProps = true;
					this._allMetadata = kwArgs.metadata;
				}
				
				kwArgs.onComplete(props, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperties", this._trace, this._version()), ioArgs);			
			})
		});		
	},

	_fetchProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		var prop = null;
				
		var content = ibm.tivoli.navmodel.Utils.getContent();		
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		content.optimize = true;
		content.properties = propId;
		
		dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "itemView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				if (result.properties && result.properties.length > 0) {
					prop = new ibm.tivoli.navmodel.Property({property: result.properties[0], dataset: this._dataset, objectType: "item"});
				
					// cache property
					this._propsById[propId] = prop;
				}
				// property was not found; remember so we don't ask again
				else {
					this._nullPropIds[propId] = true;
				}		
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperty", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return prop;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.ItemStyle"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.ItemStyle"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.ItemStyle"); 
 
////////////////////////////////////
// ItemStyle
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.ItemStyle", null, {
	// private members       
	_version: null,
	_style: null,
	
	/*=====
	constructor_kwArgs: function(version, style) {
		// version: Float
		//		version used to communicate with rest service
		// style: Object
		//		json item style object
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary
		//		Constructs ItemStyle object.
		this._version = kwArgs.version;
		this._style = kwArgs.style;		
		if (!this._style) {
			this._style = {};
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getAttribute: function(key) {
		return this._style[key];
	},
	
	setAttribute: function(key, value) {
		this._style[key] = value;
	},
	
	getId: function() {
		return this._style.id;
	},
	
	setId: function(id) {
		this._style = id;
	},
	
	getType: function() {
		return this._style.type;
	},
	
	setType: function(type) {
		this._style.type = type;
	},
	
	getLabel: function() {
		if (this._style.label) {
			// string
			if (typeof this._style.label === "string") {
				return this._style.label;
			}
			// object with key/value info
			else {
				return this._style.label.value;
			}
		}
		
		return null;
	},
	
	setLabel: function(label) {
		this._style.label = label;		
	},
	
	getDescription: function() {
		if (this._style.description) {
			// string
			if (typeof this._style.description === "string") {
				return this._style.description;
			}
			// object with key/value info
			else {
				return this._style.description.value;
			}
		}
		
		return null;		
	},
	
	setDescription: function(description) {
		this._style.description = description;			
	},
	
	getSmallImage: function() {
		return this._style.smallImage;
	},
	
	setSmallImage: function(image) {
		this._style.smallImage = image;
	},
	
	getSmallImageWidth: function() {
		return this._style.smallImageWidth;
	},
	
	setSmallImageWidth: function(width) {
		this._style.smallImageWidth = width;
	},
	
	getSmallImageHeight: function() {
		return this._style.smallImageHeight;
	},
	
	setSmallImageHeight: function(height) {
		this._style.smallImageHeight = height;
	},
	
	getLargeImage: function() {
		return this._style.largeImage;
	},
	
	setLargeImage: function(image) {
		this._style.largeImage = image;
	},
	
	getLargeImageWidth: function() {
		return this._style.largeImageWidth;
	},
	
	setLargeImageWidth: function(width) {
		this._style.largeImageWidth = width;
	},
	
	getLargeImageHeight: function() {
		return this._style.largeImageHeight;
	},
	
	setLargeImageHeight: function(height) {
		this._style.largeImageHeight = height;
	},
	
	getScalarImage: function() {
		return this._style.scalarImage;
	},
	
	setScalarImage: function(image) {
		this._style.scalarImage = image;
	},
	
	getScalarImageWidth: function() {
		return this._style.scalarImageWidth;
	},
	
	setScalarImageWidth: function(width) {
		this._style.scalarImageWidth = width;
	},
	
	getScalarImageHeight: function() {
		return this._style.scalarImageHeight;
	},
	
	setScalarImageHeight: function(height) {
		this._style.scalarImageHeight = height;
	},
	
	isScalarImageVisible: function() {
		return (this._style.scalarImageVisible !== undefined ? this._style.scalarImageVisible : true);
	},
	
	setScalarImageVisible: function(visible) {
		this._style.scalarImageVisible = visible;
	},
	
	clone: function(widgetDef) {
		var args = {
			style: {},
			version: this._version
		};		
		
		if (widgetDef) {
			// start with widgetDef default style
			var widgetDefaultStyle = widgetDef.getDefaultItemStyle();
			if (widgetDefaultStyle) {
				dojo.mixin(args.style, widgetDefaultStyle);
			}
		
			// mixin our item style
			dojo.mixin(args.style, this._style);
			
			// mixin widgetDef style
			var widgetStyle = widgetDef.getItemStyle(this.getType());			
			if (widgetStyle) {
				dojo.mixin(args.style, widgetStyle);
			}
		}
		// just mixin our item style
		else {
			dojo.mixin(args.style, this._style);
		}
			
		return new ibm.tivoli.navmodel.ItemStyle(args);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Relationship"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Relationship"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Relationship"); 

// includes








ibm.tivoli.navmodel.Relationship._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Relationship");

////////////////////////////////////
// Relationship
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Relationship", null, {
	// private members       
	_relationship: null,
	_dataset: null,
	_props: null,
	_propsById: null,
	_nullPropIds: null,
	_allProps: false,
	_allMetadata: false,
	_status: null,
	_style: null,
	_defaultTask: null,
	_expiration: null,
	_trace: ibm.tivoli.navmodel.Relationship._trace,

	/*=====
	constructor_kwArgs: function(link, dataset, propIds, allProperties, allMetadata, expiration) {
		// link: Object
		//		json link object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this link
		// propIds: String[]
		//		optional array indicating the property Id's that were requested with 
		//		the item; so it can be determined if any properties are null
		// allProperties: Boolean
		//		optional: indicates if the link already has "all" properties
		// allMetadata: Boolean
		//		optional: indicates if link properties already have metadata
		// expiration: Long
		//		optional: expiration time;  will use (current time + interval) if not supplied
	},
	=====*/
	
	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Relationship object.
		this._relationship = kwArgs.relationship;
		if (this._relationship) {
			// optimization of json on the server can leave out the "id"
			// but we know the default format
			if (!this._relationship.id) {
				this._relationship.id = this._relationship.sourceItem + "_" + this._relationship.type + "_" + this._relationship.targetItem;
			}	
		}
		else {
			this._relationship = {};
		}
		
		this._dataset = kwArgs.dataset;		
		this._props = [];
		this._propsById = {};
		this._nullPropIds = {};
		this._allProps = kwArgs.allProperties;		
		this._allMetadata = kwArgs.allMetadata;
		
		if (this._relationship.properties) {
			var propsLength = this._relationship.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = new ibm.tivoli.navmodel.Property({property: this._relationship.properties[propIndex], dataset: this._dataset, objectType: "relationship"});
				this._props[propIndex] = prop;
				this._propsById[prop.getId()] = prop;
			}
			
			// see if we should mark any properties as null because they were not returned
			if (kwArgs.propIds) {
				propsLength = kwArgs.propIds.length;
				for (propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var propId = kwArgs.propIds[propIndex];				
					if (!this._propsById[propId]) {
						this._nullPropIds[propId] = true;
					}
				}
			}
		}
		
		// expiration was supplied
		if (kwArgs.expiration) {
			this._expiration = kwArgs.expiration;
		}
		// reset to current time + interval
		else {
			this.resetExpiration();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	mixin: function(relationship) {		
		// summary:
		//		Mixin properties and status from this Relationship object
		if (relationship && relationship !== this) {
			this.removeAttribute("style");
			this._style = null;
			
			this.mixinAttributes(relationship.getAttributes());
			this.mixinProperties(relationship._props, relationship._nullPropIds);
			this._status = relationship.getStatuses({widgetDef: null});
			this._defaultTask = relationship.getDefaultTask();
			this._allProps = (this._allProps || relationship._allProps);
			this._allMetadata = (relationship._allProps && relationship._allMetadata);			
		}
	},

	getDataset: function() {		
		// summary:
		//		Returns the dataset that owns the relationship
		return this._dataset;
	},	

	getObjectType: function() {		
		// summary:
		//		Indicate that this object is a "relationship"
		return "relationship";
	},

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._relationship[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._relationship[key] = value;
	},
	
	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._relationship[key];
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the relationship
		var relationship = dojo.clone(this._relationship);
		
		// make sure "properties" are current as we may have received events
		if (this._props && this._props.length > 0) {
			relationship.properties = [];
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				relationship.properties[propIndex] = this._props[propIndex].getAttributes();
			}
		}
		
		return relationship;
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the relationship
		dojo.mixin(this._relationship, attr);		
	},	

	getUri: function() {
		if (!this._relationship.uri) {
			this._relationship.uri = this._dataset.getAttribute("uri") + "/relationships/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._relationship.uri;
	},
	
	getId: function() {		
		// summary:
		//		Returns relationship id
		return this._relationship.id;
	},

	 setId: function(id) {		
		// summary:
		//		Set the relationship id
		this._relationship.id = id;
	},

	getType: function() {		
		// summary:
		//		Returns the type of relationship
		return this._relationship.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the relationship type
		this._relationship.type = type;
	},

	 getLabel: function() {		
		// summary:
		//		Returns globalized label for the relationship
		return this._relationship.label;
	},

	 setLabel: function(label) {		
		// summary:
		//		 Set the globalized label for the relationship
		this._relationship.label = label;
	},

	getTooltip: function() {		
		// summary:
		//		Returns globalized tooltip for the relationship
		return this._relationship.tooltip;
	},

	setTooltip: function(tooltip) {		
		// summary:
		//		Set the globalized tooltip for the relationship
		this._relationship.tooltip = tooltip;
	},

	getSourceItem: function() {
		// summary:
		//		Returns the sourceItem involved in the relationship	
		var item = this._dataset.getItem({itemId: this.getSourceItemId()});	
		return item;
	},	
	
	getSourceItemId: function() {		
		// summary:
		//		Returns source item id
		return this._relationship.sourceItem;
	},

	setSourceItemId: function(sourceItem) {		
		// summary:
		//		Set source item id
		this._relationship.sourceItem = sourceItem;
	},

	getTargetItem: function() {
		// summary:
		//		Returns the targetItem involved in the relationship	
		var item = this._dataset.getItem({itemId: this.getTargetItemId()});	
		return item;
	},	
	
	getTargetItemId: function() {		
		// summary:
		//		Returns target item id
		return this._relationship.targetItem;
	},

	setTargetItemId: function(targetItem) {		
		// summary:
		//		Set target item id
		this._relationship.targetItem = targetItem;
	},
	
	getLink: function() {
		return this._dataset.getLink(this.getLinkId());
	},
	
	getLinkId: function() {		
		// summary:
		//		Returns parent link id
		return (this._relationship.linkId !== undefined ? this._relationship.linkId : "");
	},

	setLinkId: function(linkId) {		
		// summary:
		//		Set parent link id
		this._relationship.linkId = linkId;
	},
	
	/*=====
	getStatuses_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/
	
	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns ibm.tivoli.navmodel.Status[] objects indicating the relationship status values
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		if (!this._status && this._relationship.status) {
			this._status = [];
			var statusLength = this._relationship.status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				if (this._relationship.status.label && this._relationship.status.image) {
					this._status.push(new ibm.tivoli.navmodel.Status({status: this._relationship.status[statusIndex], version: this._version()}));				
				}				
				else {
					var statusArgs = {
						type: "relationship",
						statusId: this._relationship.status[statusIndex].id,
						statusValue: this._relationship.status[statusIndex].value				
					};
				
					var statusObj = this._dataset.getStatus(statusArgs);
					if (statusObj) {				
						this._status.push(statusObj);
					}
					else {
						this._trace.error("Could not locate status with id=" + statusArgs.statusId + " value=" + statusArgs.statusValue, "getStatuses");
					}
				}
			}			
		}
		
		// merge widget definition status
		var status = this._status;
		if (status && args.widgetDef) {
			status = this._status.slice(0);
			var statusLength2 = this._status.length;
			for (var statusIndex2 = 0;  statusIndex2 < statusLength2;  ++statusIndex2) {
				status[statusIndex2] = this._status[statusIndex2].clone(args.widgetDef, "relationship");
			}					
		}
		
		return status;		
	},

	setStatuses: function(statuses) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Status[] objects indicating the relationship status values
		this._status = statuses;
	},

	supportsProperties: function() {		
		// summary:
		//		Indicates if "Properties" is supported for the relationship
		return (this._relationship.supportsProperties !== undefined ? this._relationship.supportsProperties : true);
	},

	setSupportsProperties: function(supportsProperties) {		
		// summary:
		//		Set whether relationship  supports "Properties"
		this._relationship.supportsProperties = supportsProperties;
	},

	hasCachedMetadata: function() {		
		// summary:
		//		Returns true if relationship has all properties and associated metadata cached
		return this._allMetadata;
	},

	hasAllCachedProperties: function() {		
		// summary:
		//		Returns true if relationship has all properties cached
		return this._allProps;
	},

	hasCachedProperty: function(/*String*/ propId) {
		// summary:
		//		Returns true if we have the requested property in the cache
		// propId
		//		property id
		return (this._propsById[propId] ? true : false);
	},
	
	hasCachedProperties: function(/*String[]*/ properties) {		
		// summary:
		//		Returns true if we have all the requested properties in the cache
		// properties:
		//		array of property ids
		if (properties) {
			var propsLength = properties.length;
			
			// asking if we have "all" properties
			if (propsLength === 1 && properties[0] === "all") {
				return this._allProps;
			}
			
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (!this._propsById[properties[propIndex]]) {
					return false;
				}
			}
		}
		
		return true;
	},

	getCachedPropertyIds: function(expandAll) {		
		// summary:
		//		Returns array of property Ids currently cached in the item
		var propIds = [];
		
		// have all the properties
		if (!expandAll && this._allProps) {
			propIds.push("all");
		}
		// get individual property ids	
		else if (this._props) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				propIds.push(this._props[propIndex].getId());
			}
		}
		return propIds;
	},
	
	/*=====
	properties_onComplete: function(properties, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous
	},
	=====*/

	getProperties: function(/*getProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the relationship
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: ["all"],
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// fetch all properties
		if (args.properties.length === 1 && args.properties[0] === "all") {
			if (this._allProps && (!args.metadata || this._allMetadata)) {				
				if (args.sync) {
					args.onComplete(this._props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._props, args), 0);
				}		
			}
			else {
				this._fetchProperties(args);
			}
		}		
		// fetch selective properties
		else {
			// look up properties in cache
			var props = [];
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var propId = args.properties[propIndex];
				var prop = this._propsById[propId];				
				if (prop && (!args.metadata || prop.hasMetadata())) {
					props[propIndex] = prop;					
				}
				else {
					break;
				}
			}
			
			// if we found all properties in cache then return them
			if (props.length === propsLength) {
				if (args.sync) {
					args.onComplete(props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, props, args), 0);
				}
			}
			// otherwise we need to fetch
			else {
				this._fetchProperties(args);
			}
		}		
	},

	setProperties: function(properties) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Property[] objects for the relationship
		//		
		//		Note: client side only operation;  no request to rest service
		this._propsById = {};
		this._nullPropIds = {};
		this._props = [];
		
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = properties[propIndex];
				this._propsById[prop.getId()] = prop;
			}
		}		
	},

	mixinProperties: function(properties, nullPropIds) {		
		// summary:
		//		Mixin the ibm.tivoli.navmodel.Property[] objects for the relationship
		//		
		//		Note: client side only operation;  no request to rest service
		
		// mixin property objects
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				this.mixinProperty(properties[propIndex]);
			}
		}
		
		// mixin any propIds that were null
		if (nullPropIds) {
			for (var propId in nullPropIds) {
				if (nullPropIds.hasOwnProperty(propId)) {
					this._nullPropIds[propId] = true;
				}
			}
		}
	},

	 getProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// check prop cache
		var prop = this._propsById[propId];
			
		if (!prop) {
			// we have asked for this property before and it wasn't found
			if (this._nullPropIds[propId]) {
				return null;
			}
			
			// check default properties on relationship
			if (this._relationship.properties) {
				var propsLength = this._relationship.properties.length;
				for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var rawProp = this._relationship.properties[propIndex];
					if (rawProp.id === propId) {
						prop = new ibm.tivoli.navmodel.Property({property: rawProp, dataset: this._dataset, objectType: "relationship"});
						
						// cache property
						this._propsById[propId] = prop;
						break;
					}
				}				
			}
		}	
		
		// fetch it
		if (!prop && !this._allProps) {
			prop = this._fetchProperty(propId);
			
			// cache property
			if (prop) {
				this._propsById[propId] = prop;
			}
		}		
		
		return prop;
	},

	setProperty: function(property) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Property for the relationship
		//		
		//		Note: client side only operation;  no request to rest service
		var found = this._propsById[property.getId()];		
		if (found) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (this._props[propIndex].getId() === property.getId()) {
					this._props[propIndex] = property;
					break;
				}
			}
		}
		else {
			this._props.push(property);
		}		
		
		this._propsById[property.getId()] = property;
	},
	
	mixinProperty: function(property) {		
		// summary:
		//		Mixin ibm.tivoli.navmodel.Property for the relationship
		//		
		//		Note: client side only operation;  no request to rest service
		var prop = this._propsById[property.getId()];
		if (prop) {
			prop.mixin(property);
		}
		else {
			this._props.push(property);
			this._propsById[property.getId()] = property;
		}	
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	editProperties_kwArgs: function(onComplete, onError, properties) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		array of property ids to set
	},
	=====*/
	
	editProperties: function(/*editProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously modify properties for the relationship
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.properties && args.properties.length > 0) {
			var relationship = dojo.clone(this._relationship);
			relationship.properties = [];
			
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				relationship.properties[propIndex] = args.properties[propIndex].getAttributes();
			}
			
			var json = dojo.toJson(relationship, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "relationshipView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: false,			
				load: dojo.hitch(this, function(result, ioArgs) {
					args.onComplete(args);						
				}),
				error: dojo.hitch(function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperties", this._trace, this._version());				               
				})
			});	
		}
		// nothing to set
		else {
			setTimeout(dojo.hitch(this, args.onComplete, args), 0);
		}		
	},
	
	 editProperty: function(/*String*/ property) {		
		// summary:
		//		Synchronously modify property for the relationship
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		
		if (property) {
			var relationship = dojo.clone(this._relationship);
			relationship.properties = [property.getAttributes()];
			
			var json = dojo.toJson(relationship, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "relationshipView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: true,			
				load: dojo.hitch(this, function(result, ioArgs) {						
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperty", this._trace, this._version());									              
				})
			});				
		}
		
		if (exception) {
			throw exception;
		}
	},

	hasOwnRelationshipStyle: function() {		
		// summary:
		//		Returns true if the relationship has a LinkStyle that overrides the default
		//		relationship style for the relationship type  (i.e. different than the one
		//		retrieved from the dataset)
		return (this._relationship.style ? true: false);
	},

	/*=====
	getRelationshipStyle_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/
	
	getRelationshipStyle: function(/*getRelationshipStyle_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.LinkStyle for this relationship
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (!this._style) {
			if (this._relationship.linkStyle) {
				this._style = new ibm.tivoli.navmodel.LinkStyle({style: this._relationship.style, version: this._version()});
			}
			else {
				this._style = this._dataset.getRelationshipStyle({relationshipType: this.getType(), widgetDef: null});
			}
		}
		
		// merge widget definition styles 
		var style = this._style;
		if (style && args.widgetDef) {
			// clone and merge widget definition attributes				
			if (style) {
				style = style.clone(args.widgetDef, "relationship");
			}
			// unknown style; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getRelationshipStyle(this.getType());
				if (widgetStyle) {
					style = this._dataset._createLinkStyle(args.widgetDef, widgetStyle, "relationship");
				}
			}							
		}
		
		return style;
	},

	setRelationshipStyle: function(style) {		
		// summary:
		//		Set link style for the link
		this._style = style;
	},

	getDefaultTask: function() {		
		// summary:
		//		Returns optional default task associated with the relationship
		if (!this._defaultTask && this._relationship.defaultTask) {
			this._defaultTask = new ibm.tivoli.navmodel.Task({task: this._relationship.defaultTask, dataset: this._dataset});
		}
		
		return this._defaultTask;
	},

	setDefaultTask: function(defaultTask) {		
		// summary:
		//		Set optional default task associated with the link
		this._defaultTask = defaultTask;
	},

	isExpired: function() {		
		// summary:
		//		Returns true if the item is currently expired
		return (this._expiration ? false : true);
	},

	checkExpiration: function(nowTime) {		
		// summary:
		//		Check the expiration time on the item; return true if expired
		var expired = false;
		
		// already set to null from a previous check
		if (!this._expiration) {
			expired = true;
		}
		// check agaist the time to see if expired
		else if (nowTime >= this._expiration) {
			this._expiration = null;
			expired = true;
		}
		
		return expired;		
	},

	resetExpiration: function() {		
		// summary:
		//		Rest the expiration time for this item
		if (this._dataset.cacheExpires()) {
			this._expiration = this._dataset.getNewExpirationTime();
		}
		else {
			this._expiration = -1;
		}
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},

	_validateExpiration: function(kwArgs) {		
		// summary:
		//		If item is expired; mixin with a current item from the dataset cache
		if (this._dataset.cacheExpires()) {
			// expired
			if (this.isExpired()) {
				var properties = null;
				var metadata = false;
				if (kwArgs) {
					properties = kwArgs.properties;
					metadata = kwArgs.metadata;
				}			
				if (!properties) {
					properties = this.getCachedPropertyIds();
				}
		
				var relationship = this._dataset.getRelationship({relationshipId: this.getId(), properties: properties, metadata: metadata});
				if (relationship) {
					this.mixin(relationship);
				}			
			}
			// NOT expired; reset expiration since touched
			else {
				this.resetExpiration();
			}
		}
	},
	
	/*=====
	fetchProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous	
	},
	=====*/
	
	_fetchProperties: function(/*fetchProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the link
		this._props = [];
		this._allProps = false;
		this._allMetadata = false;
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		content.optimize = true;
		content.metadata = kwArgs.metadata;
		
		// are we fetching "all" properties
		var fetchAll = (kwArgs.properties.length === 1 && kwArgs.properties[0] === "all");
		
		return dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "relationshipView")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var props = [];
				var rawProps = result.properties;
				var rawPropsLength = rawProps.length;
				for (var propIndex = 0;  propIndex < rawPropsLength;  ++propIndex) {
					var prop = new ibm.tivoli.navmodel.Property({property: rawProps[propIndex], dataset: this._dataset, objectType: "relationship"});
					props[propIndex] = prop;
					
					if (fetchAll) {
						this._props[propIndex] = prop;
					}
					
					this._propsById[prop.getId()] = prop;
				}
				
				if (fetchAll) {
					this._allProps = true;
					this._allMetadata = kwArgs.metadata;
				}
				
				kwArgs.onComplete(props, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperties", this._trace, this._version()), ioArgs);			
			})
		});		
	},

	_fetchProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		var prop = null;
				
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		content.optimize = true;
		content.properties = propId;
		
		dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "relationshipView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				if (result.properties && result.properties.length > 0) {
					prop = new ibm.tivoli.navmodel.Property({property: result.properties[0], dataset: this._dataset, objectType: "relationship"});
				
					// cache property
					this._propsById[propId] = prop;
				}
				// property was not found; remember so we don't ask again
				else {
					this._nullPropIds[propId] = true;
				}					
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperty", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return prop;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Link"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Link"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Link"); 
 
// includes









ibm.tivoli.navmodel.Link._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Link");

////////////////////////////////////
// Link
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Link", null, {
	// private members       
	_link: null,
	_dataset: null,
	_relationships: null,
	_props: null,
	_propsById: null,
	_nullPropIds: null,
	_allProps: false,
	_allMetadata: false,
	_status: null,
	_style: null,
	_defaultTask: null,
	_expiration: null,
	_trace: ibm.tivoli.navmodel.Link._trace,
	
	/*=====
	constructor_kwArgs: function(link, dataset, propIds, allProperties, allMetadata, expiration) {
		// link: Object
		//		json link object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this link
		// propIds: String[]
		//		optional array indicating the property Id's that were requested with 
		//		the item; so it can be determined if any properties are null
		// allProperties: Boolean
		//		optional: indicates if the link already has "all" properties
		// allMetadata: Boolean
		//		optional: indicates if link properties already have metadata
		// expiration: Long
		//		optional: expiration time;  will use (current time + interval) if not supplied
	},
	=====*/

	 constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Link object.
		this._link = kwArgs.link;
		if (this._link) {
			// optimization of json on the server can leave out the "id"
			// but we know the default format
			if (!this._link.id) {
				if (this._link.item1 < this._link.item2) {
					this._link.id = this._link.item1 + "_" + this._link.item2;
				}
				else {
					this._link.id = this._link.item2 + "_" + this._link.item1;
				}
			}
		}
		else {
			this._link = {};
		}
		
		this._dataset = kwArgs.dataset;
		this._props = [];
		this._propsById = {};
		this._nullPropIds = {};
		this._allProps = kwArgs.allProperties;		
		this._allMetadata = kwArgs.allMetadata;
		
		if (this._link.properties) {
			var propsLength = this._link.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = new ibm.tivoli.navmodel.Property({property: this._link.properties[propIndex], dataset: this._dataset, objectType: "link"});
				this._props[propIndex] = prop;
				this._propsById[prop.getId()] = prop;
			}
			
			// see if we should mark any properties as null because they were not returned
			if (kwArgs.propIds) {
				propsLength = kwArgs.propIds.length;
				for (propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var propId = kwArgs.propIds[propIndex];				
					if (!this._propsById[propId]) {
						this._nullPropIds[propId] = true;
					}
				}
			}
		}
				
		// expiration was supplied
		if (kwArgs.expiration) {
			this._expiration = kwArgs.expiration;
		}
		// reset to current time + interval
		else {
			this.resetExpiration();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	mixin: function(link) {		
		// summary:
		//		Mixin properties and status from this Link object
		if (link && link !== this) {
			this.removeAttribute("style");
			this._style = null;
			
			this.mixinAttributes(link.getAttributes());
			this.mixinProperties(link._props, link._nullPropIds);
			this._status = link.getStatuses({widgetDef: null});
			this._defaultTask = link.getDefaultTask();
			this._relationships = link.getRelationships();
			this._allProps = (this._allProps || link._allProps);
			this._allMetadata = (link._allProps && link._allMetadata);			
		}		
	},
	
	getDataset: function() {		
		// summary:
		//		Returns the dataset that owns the link
		return this._dataset;
	},

	getObjectType: function() {		
		// summary:
		//		Indicate that this object is a "link"
		return "link";
	},

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._link[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._link[key] = value;
	},

	removeAttribute: function(key) {
		// summary:
		//		Remove general attribute for requested key
		delete this._link[key];
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the link
		var link = dojo.clone(this._link);
		
		// make sure "properties" are current as we may have received events
		if (this._props && this._props.length > 0) {
			link.properties = [];
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				link.properties[propIndex] = this._props[propIndex].getAttributes();
			}
		}
		
		return link;
	},

	mixinAttributes: function(attr) {		
		// summary:
		//		Mixin general attributes for the link
		dojo.mixin(this._link, attr);		
	},

	getUri: function() {
		if (!this._link.uri) {
			this._link.uri = this._dataset.getAttribute("uri") + "/links/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._link.uri;
	},
	
	getId: function() {		
		// summary:
		//		Returns link id
		return this._link.id;
	},

	setId: function(id) {		
		// summary:
		//		Set the link id
		this._link.id = id;		
	},

	getType: function() {		
		// summary:
		//		Returns the type of link
		return this._link.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the link type
		this._link.type = type;		
	},

	getLabel: function() {		
		// summary:
		//		Returns globalized label for the link
		return this._link.label;
	},

	setLabel: function(label) {		
		// summary:
		//		Set the globalized label for the link
		this._link.label = label;		
	},

	getTooltip: function() {		
		// summary:
		//		Returns globalized tooltip for the link
		return this._link.tooltip;
	},

	setTooltip: function(tooltip) {		
		// summary:
		//		Set the globalized tooltip for the link
		this._link.tooltip = tooltip;
	},
	
	isContainment: function() {		
		// summary:
		//		Returns true if this link represents containment instead of
		//		connection via normal lines
		return (this._link.containment ? this._link.containment : false);
	},
	
	setContainment: function(isContainment) {		
		// summary:
		//		Set the value indicating containment
		this._link.containment = isContainment;
	},

	getDirection: function() {		
		// summary:
		//		Returns the direction associated with the link
		//		
		//		|	values:	"1_2"   	means direction of arrow flows from item1 to item2
		//		|		"2_1"		means direction of arrow flows from item2 to item1
		//		|		"both"	means that arrow flows in both directions
		return (this._link.direction ? this._link.direction : "1_2");
	},

	setDirection: function(direction) {		
		// summary:
		//		Set the direction associated with the link
		//		
		//		|	values:	"1_2"   	means direction of arrow flows from item1 to item2
		//		|		"2_1"		means direction of arrow flows from item2 to item1
		//		|		"both"	means that arrow flows in both directions
		this._link.direction = direction;
	},
	
	/*=====
	getStatuses_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override attributes
	},
	=====*/

	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns ibm.tivoli.navmodel.Status[] objects indicating the link status values
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		if (!this._status && this._link.status) {
			this._status = [];
			var statusLength = this._link.status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				if (this._link.status.label && this._link.status.image) {
					this._status.push(new ibm.tivoli.navmodel.Status({status: this._link.status[statusIndex], version: this._version()}));				
				}				
				else {
					var statusArgs = {
						type: "link",
						statusId: this._link.status[statusIndex].id,
						statusValue: this._link.status[statusIndex].value				
					};
				
					var statusObj = this._dataset.getStatus(statusArgs);
					if (statusObj) {				
						this._status.push(statusObj);
					}
					else {
						this._trace.error("Could not locate status with id=" + statusArgs.statusId + " value=" + statusArgs.statusValue, "getStatuses");
					}
				}
			}			
		}
		
		// merge widget definition status
		var status = this._status;
		if (status && args.widgetDef) {
			status = this._status.slice(0);
			var statusLength2 = this._status.length;
			for (var statusIndex2 = 0;  statusIndex2 < statusLength2;  ++statusIndex2) {
				status[statusIndex2] = this._status[statusIndex2].clone(args.widgetDef, "link");
			}
		}					
		
		return status;		
	},

	setStatuses: function(statuses) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Status[] objects indicating the link status values
		this._status = statuses;
	},


	getItem1: function() {
		// summary:
		//		Returns the item1 involved in the link		
		var item = this._dataset.getItem({itemId: this.getItem1Id()});		
		return item;
	},
	
	getItem1Id: function() {		
		// summary:
		//		Returns the id of item1 involved in the link;  this value can be undefined
		//		if the link is between two different datasets and the connection point
		//		is to the dataset and not a link within it
		return this._link.item1;
	},

	setItem1Id: function(item1) {		
		// summary:
		//		Set the id of item1 involved in the link
		this._link.item1 = item1;
	},

	getItem2: function() {
		// summary:
		//		Returns the item2 involved in the link		
		var item = this._dataset.getItem({itemId: this.getItem2Id()});	
		return item;
	},
		
	getItem2Id: function() {		
		// summary:
		//		Returns the id of item2 involved in the link;  this value can be undefined
		//		if the link is between two different datasets and the connection point
		//		is to the dataset and not a link within it
		return this._link.item2;
	},

	setItem2Id: function(item2) {		
		// summary:
		//		Set the id of item1 involved in the link
		this._link.item2 = item2;
	},

	getRelationshipCount: function() {		
		// summary:
		//		Returns the number of relationships involved in the link
		return this.getRelationships().length;
	},

	getRelationship: function(relationshipId) {		
		// summary:
		//		Returns requested relationship if it exists
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		var relationships = this.getRelationships();
		if (relationships) {
			var relLength = relationships.length;
			for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
				if (relationships[relIndex].getId() === relationshipId) {
					return relationships[relIndex];
				}
			}
		}
		
		return null;
	},

	getRelationships: function() {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.Relationship[] objects that are involved in the link
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		if (!this._relationships) {
			var direction = this.getDirection();
			this._relationships = [];
			var relationshipsLength = this._link.relationships.length;
			for (var relIndex = 0;  relIndex < relationshipsLength;  ++relIndex) {			
				var relationshipObj = this._link.relationships[relIndex];
				
				if (!relationshipObj.linkId) {
					relationshipObj.linkId = this.getId();
				}
				
				if (!relationshipObj.label) {
					relationshipObj.label = this.getLabel();
				}
				
				if (!relationshipObj.type) {
					relationshipObj.type = this.getType();
				}
				
				if (!relationshipObj.sourceItem) {
					switch(direction) {
						case "1_2":
						case "none":
							if (this.getItem1Id()) {
								relationshipObj.sourceItem = this.getItem1Id();
							}
							break;
						case "2_1":
							if (this.getItem2Id()) {
								relationshipObj.sourceItem = this.getItem2Id();
							}
					}	
				}
				
				if (!relationshipObj.targetItem) {
					switch(direction) {
						case "1_2":
						case "none":
							if (this.getItem2Id()) {
								relationshipObj.targetItem = this.getItem2Id();
							}
							break;
						case "2_1":
							if (this.getItem1Id()) {
								relationshipObj.targetItem = this.getItem1Id();
							}
					}					
				}
				
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: relationshipObj, dataset: this._dataset});
				this._relationships[relIndex] = this._dataset._cacheRelationship(relationship);
			}
		}

		return this._relationships;
	},

	setRelationships: function(relationships) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Relationship[] objects that are involved in the link
		this._relationships = relationships;
	},

	supportsProperties: function() {		
		// summary:
		//		Indicates if "Properties" is supported for the link
		return (this._link.supportsProperties !== undefined ? this._link.supportsProperties : true);
	},

	setSupportsProperties: function(supportsProperties) {		
		// summary:
		//		Set whether link  supports "Properties"
		this._link.supportsProperties = supportsProperties;
	},

	hasCachedMetadata: function() {		
		// summary:
		//		Returns true if link has all properties and associated metadata cached
		return this._allMetadata;
	},

	hasAllCachedProperties: function() {		
		// summary:
		//		Returns true if link has all proprties cached
		return this._allProps;
	},

	hasCachedProperty: function(/*String*/ propId) {
		// summary:
		//		Returns true if we have the requested property in the cache
		// propId
		//		property id
		return (this._propsById[propId] ? true : false);
	},
	
	hasCachedProperties: function(/*String[]*/ properties) {		
		// summary:
		//		Returns true if we have all the requested properties in the cache
		// properties:
		//		array of property ids
		if (properties) {
			var propsLength = properties.length;
			
			// asking if we have "all" properties
			if (propsLength === 1 && properties[0] === "all") {
				return this._allProps;
			}
			
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (!this._propsById[properties[propIndex]]) {
					return false;
				}
			}
		}
		
		return true;
	},

	getCachedPropertyIds: function(expandAll) {		
		// summary:
		//		Returns array of property Ids currently cached in the item
		var propIds = [];
		
		// have all the properties
		if (!expandAll && this._allProps) {
			propIds.push("all");
		}
		// get individual property ids	
		else if (this._props) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				propIds.push(this._props[propIndex].getId());
			}
		}
		return propIds;
	},
	
	/*=====
	properties_onComplete: function(properties, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous
	},
	=====*/

	getProperties: function(/*getProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the link
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: ["all"],
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// fetch all properties
		if (args.properties.length === 1 && args.properties[0] === "all") {
			if (this._allProps && (!args.metadata || this._allMetadata)) {				
				if (args.sync) {
					args.onComplete(this._props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._props, args), 0);
				}		
			}
			else {
				this._fetchProperties(args);
			}
		}		
		// fetch selective properties
		else {
			// look up properties in cache
			var props = [];
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var propId = args.properties[propIndex];
				var prop = this._propsById[propId];				
				if (prop && (!args.metadata || prop.hasMetadata())) {
					props[propIndex] = prop;					
				}
				else {
					break;
				}
			}
			
			// if we found all properties in cache then return them
			if (props.length === propsLength) {
				if (args.sync) {
					args.onComplete(props, args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, props, args), 0);
				}
			}
			// otherwise we need to fetch
			else {
				this._fetchProperties(args);
			}
		}		
	},
	
	setProperties: function(properties) {		
		// summary:
		//		Set the ibm.tivoli.navmodel.Property[] objects for the link
		//		
		//		Note: client side only operation;  no request to rest service
		this._propsById = {};
		this._nullPropIds = {};
		this._props = [];
		
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				var prop = properties[propIndex];
				this._propsById[prop.getId()] = prop;
			}
		}		
	},

	mixinProperties: function(properties, nullPropIds) {		
		// summary:
		//		Mixin the ibm.tivoli.navmodel.Property[] objects for the link
		//		
		//		Note: client side only operation;  no request to rest service
		
		// mixin property objects
		if (properties) {
			var propsLength = properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				this.mixinProperty(properties[propIndex]);
			}
		}
		
		// mixin any propIds that were null
		if (nullPropIds) {
			for (var propId in nullPropIds) {
				if (nullPropIds.hasOwnProperty(propId)) {
					this._nullPropIds[propId] = true;
				}
			}
		}
	},

	 getProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// if expired mixin from dataset cache
		this._validateExpiration();
		
		// check prop cache
		var prop = this._propsById[propId];
			
		if (!prop) {
			// we have asked for this property before and it wasn't found
			if (this._nullPropIds[propId]) {
				return null;
			}
			
			// check default properties on link	
			if (this._link.properties) {
				var propsLength = this._link.properties.length;
				for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
					var rawProp = this._link.properties[propIndex];
					if (rawProp.id === propId) {
						prop = new ibm.tivoli.navmodel.Property({property: rawProp, dataset: this._dataset, objectType: "link"});
						
						// cache property
						this._propsById[propId] = prop;
						break;
					}
				}				
			}
		}	
		
		// fetch it
		if (!prop && !this._allProps) {
			prop = this._fetchProperty(propId);
			
			// cache property
			if (prop) {
				this._propsById[propId] = prop;
			}
		}		
		
		return prop;
	},

	setProperty: function(property) {		
		// summary:
		//		Set ibm.tivoli.navmodel.Property for the item
		//		
		//		Note: client side only operation;  no request to rest service
		var found = this._propsById[property.getId()];		
		if (found) {
			var propsLength = this._props.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				if (this._props[propIndex].getId() === property.getId()) {
					this._props[propIndex] = property;
					break;
				}
			}
		}
		else {
			this._props.push(property);
		}		
		
		this._propsById[property.getId()] = property;
	},

	mixinProperty: function(property) {		
		// summary:
		//		Mixin ibm.tivoli.navmodel.Property for the link
		//		
		//		Note: client side only operation;  no request to rest service
		var prop = this._propsById[property.getId()];
		if (prop) {
			prop.mixin(property);
		}
		else {
			this._props.push(property);
			this._propsById[property.getId()] = property;
		}	
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	editProperties_kwArgs: function(onComplete, onError, properties) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		array of property ids to set
	},
	=====*/

	editProperties: function(/*editProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously modify properties for the link
		var args = {                
			onComplete: function(properties, args){
			},
			onError: function(error, exception, ioArgs){				
			},                
			properties: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.properties && args.properties.length > 0) {
			var link = dojo.clone(this._link);
			link.properties = [];
			
			var propsLength = args.properties.length;
			for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
				link.properties[propIndex] = args.properties[propIndex].getAttributes();
			}
			
			var json = dojo.toJson(link, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: false,			
				load: dojo.hitch(this, function(result, ioArgs) {
					args.onComplete(args);						
				}),
				error: dojo.hitch(function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperties", this._trace, this._version());				               
				})
			});	
		}
		// nothing to set
		else {
			setTimeout(dojo.hitch(this, args.onComplete, args), 0);
		}		
	},

	 editProperty: function(/*String*/ property) {		
		// summary:
		//		Synchronously modify property for the link
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		
		if (property) {
			var link = dojo.clone(this._link);
			link.properties = [property.getAttributes()];
			
			var json = dojo.toJson(link, false);
			
			dojo.xhrPost ({
				url: ibm.tivoli.navmodel.Utils.getParamUrl(this.getUri(), this._dataset),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: true,			
				load: dojo.hitch(this, function(result, ioArgs) {						
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "setProperty", this._trace, this._version());									              
				})
			});				
		}
		
		if (exception) {
			throw exception;
		}
	},

	hasOwnLinkStyle: function() {		
		// summary:
		//		Returns true if the link has a LinkStyle that overrides the default
		//		link style for the link type  (i.e. different than the one retrieved
		//		from the dataset)
		return (this._link.style ? true: false);
	},
	
	/*=====
	getLinkStyle_kwArgs: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override
	},
	=====*/
	
	getLinkStyle: function(/*getLinkStyle_kwArgs*/ kwArgs) {		
		// summary:
		//		Returns the ibm.tivoli.navmodel.LinkStyle for this link
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (!this._style) {
			if (this._link.style) {
				this._style = new ibm.tivoli.navmodel.LinkStyle({style: this._link.style, version: this._version()});
			}
			else {
				this._style = this._dataset.getLinkStyle({linkType: this.getType(), widgetDef: null});
			}
		}
		
		// merge widget definition styles 
		var style = this._style;
		if (style && args.widgetDef) {
			// clone and merge widget definition attributes				
			if (style) {
				style = style.clone(args.widgetDef, "link");
			}
			// unknown style; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getLinkStyle(this.getType());
				if (widgetStyle) {
					style = this._dataset._createLinkStyle(args.widgetDef, widgetStyle, "link");
				}
			}							
		}
		
		return style;
	},

	setLinkStyle: function(style) {		
		// summary:
		//		Set link style for the link
		this._style = style;
	},

	getDefaultTask: function() {		
		// summary:
		//		Returns optional default task associated with the link
		if (!this._defaultTask && this._link.defaultTask) {
			this._defaultTask = new ibm.tivoli.navmodel.Task({task: this._link.defaultTask, dataset: this._dataset});
		}
		
		return this._defaultTask;
	},

	setDefaultTask: function(defaultTask) {		
		// summary:
		//		Set optional default task associated with the link
		this._defaultTask = defaultTask;
	},

	isExpired: function() {		
		// summary:
		//		Returns true if the item is currently expired
		return (this._expiration ? false : true);
	},

	checkExpiration: function(nowTime) {		
		// summary:
		//		Check the expiration time on the item; return true if expired
		var expired = false;
		
		// already set to null from a previous check
		if (!this._expiration) {
			expired = true;
		}
		// check agaist the time to see if expired
		else if (nowTime >= this._expiration) {
			this._expiration = null;
			expired = true;
		}
		
		return expired;		
	},

	resetExpiration: function() {		
		// summary:
		//		Rest the expiration time for this item
		if (this._dataset.cacheExpires()) {
			this._expiration = this._dataset.getNewExpirationTime();
		}
		else {
			this._expiration = -1;
		}
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},

	_validateExpiration: function(kwArgs) {		
		// summary:
		//		If item is expired; mixin with a current item from the dataset cache
		if (this._dataset.cacheExpires()) {
			// expired
			if (this.isExpired()) {
				var properties = null;
				var metadata = false;
				if (kwArgs) {
					properties = kwArgs.properties;
					metadata = kwArgs.metadata;
				}			
				if (!properties) {
					properties = this.getCachedPropertyIds();
				}
		
				var link = this._dataset.getLink({linkId: this.getId(), properties: properties, metadata: metadata});
				if (link) {
					this.mixin(link);
				}			
			}
			// NOT expired; reset expiration since touched
			else {
				this.resetExpiration();
			}
		}
	},	
        
	/*=====
	fetchProperties_kwArgs: function(onComplete, onError, properties, metadata, sync) {
		// onComplete: properties_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// properties: String[]
		//		property ids to fetch or "all"
		// metadata: Boolean
		//		indicates whether metadata is required
		// sync: Boolean
		//		indicates if call is synchronous or asynchronous	
	},
	=====*/
	
	_fetchProperties: function(/*fetchProperties_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously returns properties for the link
		this._props = [];
		this._allProps = false;
		this._allMetadata = false;
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		content.optimize = true;
		content.metadata = kwArgs.metadata;
		
		// are we fetching "all" properties
		var fetchAll = (kwArgs.properties.length === 1 && kwArgs.properties[0] === "all");
		
		return dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var props = [];
				var rawProps = result.properties;
				var rawPropsLength = rawProps.length;
				for (var propIndex = 0;  propIndex < rawPropsLength;  ++propIndex) {
					var prop = new ibm.tivoli.navmodel.Property({property: rawProps[propIndex], dataset: this._dataset, objectType: "link"});
					props[propIndex] = prop;
					
					if (fetchAll) {
						this._props[propIndex] = prop;
					}
					
					this._propsById[prop.getId()] = prop;
				}
				
				if (fetchAll) {
					this._allProps = true;
					this._allMetadata = kwArgs.metadata;
				}
				
				kwArgs.onComplete(props, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperties", this._trace, this._version()), ioArgs);			
			})
		});		
	},

	_fetchProperty: function(/*String*/ propId) {		
		// summary:
		//		Synchronously returns requested property
		// propId:
		//		property id to return
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var exception = null;
		var prop = null;
			
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
		content.optimize = true;
		content.properties = propId;
		
		dojo.xhrGet({
			url: this.getUri(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "linkView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				if (result.properties && result.properties.length > 0) {
					prop = new ibm.tivoli.navmodel.Property({property: result.properties[0], dataset: this._dataset, objectType: "link"});
				
					// cache property
					this._propsById[propId] = prop;
				}
				// property was not found; remember so we don't ask again
				else {
					this._nullPropIds[propId] = true;
				}
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchProperty", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return prop;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Selection"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Selection"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Selection"); 
 
////////////////////////////////////
// Selection
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Selection", null, {
	// private members
	_version: null,
	_selection: null,
	
	/*=====
	constructor_kwArgs: function(version) {
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Selection object.
		this._version = kwArgs.version;
		this.clear();
	},
	
	/////////////////////////////
	// public methods
	/////////////////////////////

	getAttributes: function() {		
		// summary:
		//		Returns selection value
		return this._selection;
	},

	clear: function() {		
		// summary:
		//		Clear out selection contents
		this._selection = {selected: false, itemIds: [], linkIds: [], relationshipIds: []};
	},

	isEmpty: function() {		
		// summary:
		//		Returns true if selection is empty (i.e. no itemIds or linkIds)
		return this._isEmpty(this._selection);
	},

	addItem: function(itemId) {		
		// summary:
		//		Add an item id to the selection
		var selection = this._getSelection();		
		if (selection) {
			this._addItem(selection, itemId);
		}
	},

	addItems: function(itemIds) {		
		// summary:
		//		Add multiple item ids to the selection
		var idsLength = itemIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._addItem(selection, itemIds[idIndex]);
			}
		}		
	},

	removeItem: function(itemId) {		
		// summary:
		//		Remove an item id from the selection
		var selection = this._getSelection();		
		if (selection) {
			this._removeItem(selection, itemId);						
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeItems: function(itemIds) {		
		// summary:
		//		Remove multiple item ids from the selection
		var idsLength = itemIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection(ids);		
			if (selection) {
				this._removeItem(selection, itemIds[idIndex]);						
			}			
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	addLink: function(linkId) {		
		// summary:
		//		Add a link id to the selection
		var selection = this._getSelection();		
		if (selection) {
			this._addLink(selection, linkId);
		}
	},

	addLinks: function(linkIds) {		
		// summary:
		//		Add multiple link ids to the selection
		var idsLength = linkIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._addLink(selection, linkIds[idIndex]);
			}
		}
	},

	removeLink: function(linkId) {		
		// summary:
		//		Remove a link id from the selection
		var selection = this._getSelection();		
		if (selection) {
			this._removeLink(selection, linkId);			
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeLinks: function(linkIds) {		
		// summary:
		//		Remove multiple link ids from the selection
		var idsLength = linkIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._removeLink(selection, linkIds[idIndex]);			
			}
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},
	
	addRelationship: function(relationshipId) {		
		// summary:
		//		Add a relationship id to the selectio
		var selection = this._getSelection();		
		if (selection) {
			this._addRelationship(selection, relationshipId);
		}
	},

	addRelationships: function(relationshipIds) {		
		// summary:
		//		Add multiple relationship ids to the selection
		var idsLength = relationshipIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._addRelationship(selection, relationshipIds[idIndex]);
			}
		}
	},

	removeRelationship: function(elationshipId) {		
		// summary:
		//		Remove a relationship id from the selection
		var selection = this._getSelection();		
		if (selection) {
			this._removeRelationship(selection, relationshipId);			
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},

	removeRelationships: function(relationshipIds) {		
		// summary:
		//		Remove multiple relationship ids from the selection
		var idsLength = relationshipIds.length;
		for (var idIndex = 0;  idIndex < idsLength;  ++idIndex) {
			var selection = this._getSelection();		
			if (selection) {
				this._removeRelatinship(selection, relationshipIds[idIndex]);			
			}
		}
		
		// delete any empty selections in the hierarchy
		this._clean(this._selection);
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_getSelection: function(ids) {		
		// summary:
		//		Returns the selection object associated
		
		var selection = this._selection;
		return selection;	
	},

	_clean: function(selection) {		
		return this._isEmpty(selection);
	},

	_isEmpty: function(selection) {		
		// summary:
		//		Returns true if the selection is empty
		if (selection.selected) {
			return false;
		}
		
		if (selection.itemIds.length > 0) {
			return false;
		}
		
		if (selection.linkIds.length > 0) {
			return false;
		}
		
		if (selection.relationshipIds.length > 0) {
			return false;
		}
		
		return true;
	},

	_addItem: function(selection, itemId) {		
		// summary:
		//		Add item id to the selection object
		selection.itemIds.push(itemId);
	},
	
	_removeItem: function(selection, itemId) {		
		// summary:
		//		Remove item id from the selection object
		var itemIdsLength = selection.itemIds.length;
		for (var itemIdIndex = 0;  itemIdIndex < itemIdsLength;  ++itemIdIndex) {
			if (itemId === selection.itemIds[itemIdIndex]) {
				selection.itemIds.splice(itemIdIndex, 1);
				break;
			}
		}
	},

	_addLink: function(selection, linkId) {		
		// summary:
		//		Add link id to the selection object
		selection.linkIds.push(linkId);
	},

	_removeLink: function(selection, linkId) {		
		// summary:
		//		Remove link id from the selection object
		var linkIdsLength = selection.linkIds.length;
		for (var linkIdIndex = 0;  linkIdIndex < linkIdsLength;  ++linkIdIndex) {
			if (linkId === selection.linkIds[linkIdIndex]) {
				selection.linkIds.splice(linkIdIndex, 1);
				break;
			}
		}
	},

	_addRelationship: function(selection, relationshipId) {		
		// summary:
		//		Add relationship id to the selection object
		selection.relationshipIds.push(relationshipId);
	},

	_removeRelationship: function(selection, relationshipId) {		
		// summary:
		//		Remove relationship id from the selection object
		var relIdsLength = selection.relationshipIds.length;
		for (var relIdIndex = 0;  relIdIndex < relIdsLength;  ++relIdIndex) {
			if (relationshipId === selection.relationshipIds[relIdIndex]) {
				selection.relationshipIds.splice(relIdIndex, 1);
				break;
			}
		}
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Task"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Task"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Task"); 
 
// includes




ibm.tivoli.navmodel.Task._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Task");

////////////////////////////////////
// Task
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Task", null, {
	// private members
	_task: null,
	_dataset: null,
	_childTasks: null,
	_trace: ibm.tivoli.navmodel.Task._trace,
	
	/*=====
	constructor_kwArgs: function(task, dataset) {
		// task: Object
		//		json task object
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that created this task
	},
	=====*/
	
	 constructor: function(/*constructor_kwArgs*/ kwArgs) {				
		// summary:
		//		Constructs Task object.
		this._dataset = kwArgs.dataset;
		this._task = kwArgs.task;
		if (!this._task) {
			this._task = {};
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	getAttribute: function(key) {		
		// summary:
		//		Returns general attribute for requested key
		return this._task[key];
	},

	setAttribute: function(key, value) {		
		// summary:
		//		Set general attribute for requested key
		this._task[key] = value;
	},

	getVersion: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	},
		
	getUri: function() {
		if (!this._task.uri) {
			this._task.uri = this._dataset.getAttribute("uri") + "/tasks/" + encodeURIComponent(this.getId());
		}
	
		return this._dataset.getArg("baseUrl") + this._task.uri;
	},
	
	getId: function() {
		return this._task.id;
	},
	
	setId: function(id) {
		this._task.id = id;
	},
	
	getLabel: function() {
		return this._task.label;
	},
	
	setLabel: function(label) {
		this._task.label = label;
	},
	
	getDescription: function() {
		return this._task.description;
	},
	
	setDescription: function(description) {
		this._task.description = description;
	},
	
	getImage: function() {
		return this._task.image;
	},
	
	setImage: function(image) {
		this._task.image = image;
	},

	getCategory: function() {		
		// summary:
		//		Returns the task category (i.e. where the task is visualized)
		//		
		//		values are: menu, menu:navigation, toolbar, toolbar:button, toolbar:menu,
		//					breadcrumb, breadcrumb:navigation, and property
		return this._task.category;
	},
	
	setCategory: function(category) {
		this._task.category = category;
	},

	isServer: function() {		
		// summary:
		//		Indicates that task should be sent back to the server for execution
		return (this._task.server !== undefined ? this._task.server : false);
	},
	
	setServer: function(isServer) {
		this._task.server = isServer;
	},
	 
	getType: function() {		
		// summary:
		//		Returns the task type
		//		
		//		values are: none, url, page, portlet, clientevent, javascript, native,
		//					dialog, noninteractive, cascade, and separator
		return this._task.type;
	},

	setType: function(type) {		
		// summary:
		//		Set the task type
		//		
		//		values are: none, url, page, portlet, clientevent, javascript, native,
		//					dialog, noninteractive, cascade, and separator
		this._task.type = type;
	},
	
	isNormal: function() {
		return (!this.isNavigation() && !this.isCascade() && !this.isSeparator());
	},

	isNavigation: function() {
		return (this._task.type === "navigation");
	},
	
	isPageLaunch: function() {
		return (this._task.type === "page");
	},
	
	isPortletLaunch: function() {
		return (this._task.type === "portlet");
	},
	
	isUrlLaunch: function() {
		return (this._task.type === "url");
	},
	
	isJavascript: function() {
		return (this._task.type === "javascript");
	},
	
	isClientEvent: function() {
		return (this._task.type === "clientevent");
	},
	
	isCascade: function() {
		return (this._task.type === "cascade" && this._task.subTasks);
	},
	
	isSeparator: function() {
		return (this._task.type === "separator");
	},
	
	isResolved: function() {
		return (this._task.resolved !== undefined ? this._task.resolved : true);
	},
	
	setResolved: function(isResolved) {
		this._task.resolved = isResolved;
	},
	
	isDefault: function() {
		return (this._task["default"] !== undefined ? this._task["default"] : false);
	},
	
	setDefault: function(isDefault) {
		this._task["default"] = isDefault;
	},
	
	isEnabled: function() {
		return (this._task.enabled !== undefined ? this._task.enabled : true);
	},
	
	setEnabled: function(isEnabled) {
		this._task.enabled = isEnabled;
	},
	
	isDisabled: function() {
		return (this._task.enabled !== undefined ? !this._task.enabled : false);
	},
	
	setDisabled: function(isDisabled) {
		this._task.enabled = !isDisabled;
	},
	
	/*=====
	resolve_kwArgs: function(selection, itemIds, linkIds, itemIndexes) {
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds: String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
	},
	=====*/

	resolve: function(/*resolve_kwArgs*/ kwArgs) {		
		// summary:
		//		Re-fetch the task to resolve all client side launch parameters
		var args = {
			selection: null,
			itemIds: null,
			linkIds: null,
			itemIndexes: null
		};
		dojo.mixin(args, kwArgs);
		
		// selection was provided
		var json = "";
		if (args.selection) {
			json = dojo.toJson(args.selection.getAttributes(), false);
		}
			
		return dojo.xhrPost ({
			url: this._getParamURL(args),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "task"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "taskContext"),
				'Content-Length': json.length,
				'X-Method-Override': 'GET'
			},
			handleAs: "json",
			postData: json,
			sync: true,			
			load: dojo.hitch(this, function(result, ioArgs) {
				this._task = result;
			}),
			error: dojo.hitch(function(error, ioArgs) {
				ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "resolveLaunchParms", this._trace, this._version());					              
			})
		});
	},
	
	getLaunchUrl: function() {
		return this._task.url;
	},
	
	setLaunchUrl: function(url) {
		this._task.url = url;
	},
	
	getLaunchBrowserWindowId: function() {
		return this._task.browserWindowId;
	},
	
	setLaunchBrowserWindowId: function(windowId) {
		this._task.browserWindowId = windowId;
	},
	
	getLaunchPageId: function() {
		return this._task.pageId;
	},
	
	setLaunchPageId: function(pageId) {
		this._task.pageId = pageId;
	},
	
	getLaunchPortletName: function() {
		return this._task.portletName;
	},
	
	setLaunchPortletName: function(portletName) {
		this._task.portletName = portletName;
	},
	
	getLaunchPortletReuse: function() {
		return this._task.portletReuse;
	},
	
	setLaunchPortletReuse: function(reuse) {
		this._task.portletReuse = reuse;
	},
	
	getLaunchPortletAppUID: function() {
		return this._task.portletAppUID;
	},
	
	setLaunchPortletAppUID: function(appUID) {
		this._task.portletAppUID = appUID;
	},
	
	getLaunchEvent: function() {
		return this._task.event;
	},
	
	setLaunchEvent: function(event) {
		this._task.event = event;
	},
	
	getLaunchJavascript: function() {
		return this._task.javascript;
	},
	
	setLaunchJavascript: function(javascript) {
		this._task.javascript = javascript;
	},
	
	getLaunchApplicationParms: function() {
		return this._task.applicationParms;
	},
	
	setLaunchApplicationParms: function(applicationParms) {
		this._task.applicationParms = applicationParms;
	},
	
	/*
	 * Returns any child tasks if this task is a cascade
	 */
	getChildTasks: function() {
		if (this.isCascade()) {
			if (!this._childTasks) {
				this._childTasks = [];
				var subTaskLength = this._task.subTasks.length;
				for (var index = 0;  index < subTaskLength;  ++index) {
					this._childTasks[index] = new ibm.tivoli.navmodel.Task({task: this._task.subTasks[index], dataset: this._dataset});
				}
			}
			
			return this._childTasks;
		}
		
		return null;
	},
	
	setChildTasks: function(childTasks) {
		if (this.isCascade()) {
			this._childTasks = childTasks;
		}
	},
	
	/*=====
	result_onComplete: function(result, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	execute_kwArgs: function(onComplete, onError, selection, sync) {
		// onComplete: result_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for successful fetch
		// selection: Object:
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// selectionParms: Object:
		//		extra parameters that can be included in the selection when PUT
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	execute: function(/*execute_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously execute remote task
		
		// this is only for executing regular tasks
		if (this.isNavigation()) {
			alert('execute is only for non-navigation tasks');
			return;
		}
		
		var args = {                
			onComplete: function(result, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},         
			selection: null,
			selectionParms: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// get selection and add additional parameters	
		var selection = args.selection.getAttributes();
		if (args.selectionParms) {
			selection.parameters = args.selectionParms;
		}
			
		var json = dojo.toJson(selection, false);
			
		return dojo.xhrPost ({
			url: this._getParamURL({}),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "taskContext"),
				'Content-Length': json.length				
			},
			handleAs: "json",
			postData: json,
			sync: args.sync,		
			load: dojo.hitch(this, function(result, ioArgs) {
				if (this._trace.isFine()) {
					this._trace.fine("Execute for taskId=" + this.getId() + " returned successfully", "execute");
				}
							
				args.onComplete(result, args);				
			}),
			error: dojo.hitch(function(error, ioArgs) {
				args.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "execute", this._trace, this._version()), ioArgs);	               
			})
		});		
	},
	
	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, selection, sync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for successful fetch
		// selection: Object:
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	navigate: function(/*navigate_kwArgs*/ kwArgs) {		
		// summary:
		//		Asynchronously execute navigation task (i.e. returns new Dataset)
		
		// this is only for executing regular tasks
		if (!this.isNavigation()) {
			alert('navigate is only for navigation tasks');
			return;
		}
		
		var args = {                
			onComplete: function(dataset, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			taskId: this.getId(),
			taskCategory: this.getCategory(),                
			selection: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// call the dataset to do the navigation
		this._dataset.navigate(args);
	},	
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_version: function() {
		return this._dataset.getVersion();
	},

	_getParamURL: function(kwArgs) {		
		// summary:
		//		Build url necessary to do reference task and also includes
		//		all of the "param_" and "version" arguments
		var url = this.getUri();
		var questionMark = false;
		
		// add "param_" parms
		var parameters = this._dataset.getArg("parameters");
		for (var parm in parameters) {
			if (parameters.hasOwnProperty(parm)) {
				if (!questionMark) {
					questionMark = true;
					url += "?";
				}
				else {
					url += "&";
				}
				url += ("param_" + parm + "=" + parameters[parm]);				
			}
		}
		
		// IE will sometimes return http requests from cache without unique parms
		var now = new Date();
		if (!questionMark) {
			questionMark = true;
			url += "?d=" + now.getTime() + "&r=" + Math.random();
		}
		else {
			url += "&d=" + now.getTime() + "&r=" + Math.random();
		}
		
		// add "category"
		if(!questionMark) {
			questionMark = true;
			url += "?category=" + this.getCategory();
		}
		else {
			url += "&category=" + this.getCategory();
		}
		
		// indicate selected items
		if (kwArgs.itemIds) {
			if (!questionMark) {
				questionMark = true;
				url += "?itemIds=" + kwArgs.itemIds;
			}
			else {
				url += "&itemIds=" + kwArgs.itemIds;
			}			
		}
		
		// indicate selected links
		if (kwArgs.linkIds) {
			if (!questionMark) {
				questionMark = true;
				url += "?linkIds=" + kwArgs.linkInds;
			}
			else {
				url += "&linkIds=" + kwArgs.linkIds;
			}			
		}
		
		// indicate seleted item ranges
		if (kwArgs.itemIndexes) {
			if (!questionMark) {
				questionMark = true;
				url += "?itemIndexes=" + kwArgs.itemIndexes;
			}
			else {
				url += "&itemIndexes=" + kwArgs.itemIndexes;
			}			
		}
		
		return url;	
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Column"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Column"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Column"); 

// includes


////////////////////////////////////
// Column
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Column", null, {
	// private members
	_col: null,
	_objectType: null,
	_dataset: null,
	_condition: null,
	
	/*=====
	constructor_kwArgs: function(column, objectType, dataset) {
		// column: Object
		//		json column object
		// objectType: String
		//		object type: "item", "link", or "relationship"
		// dataset: Object
		//		ibm.tivoli.navmodel.Dataset that owns this column
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs Column object.		
		this._col = kwArgs.column;
		if (!this._col) {
			this._col = {};
		}
		
		this._objectType = kwArgs.objectType;
		this._dataset = kwArgs.dataset;
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general attribute for requested key
		return this._col[key];
	},

	setAttribute: function(key, value) {
		// summary:
		//		Set general attribute for requested key
		this._col[key] = value;
	},
	
	getAttributes: function() {		
		// summary:
		//		Get all general attributes for the column
		return dojo.clone(this._col);			
	},

	getDataset: function() {
		// summary:
		//		Returns the dataset that owns the column
		return this._dataset;
	},
	
	getId: function() {
		// summary:
		//		Returns column id
		return this._col.id;
	},

	setId: function(id) {
		// summary:
		//		Set column id
		this._col.id = id;
	},

	getLabel: function() {
		// summary:
		//		Returns globalized column header label
		if (this._col.label) {
			// string
			if (typeof this._col.label === "string") {
				return this._col.label;
			}
			// object with key/value info
			else {
				return this._col.label.value;
			}
		}
		
		return null;		
	},

	setLabel: function(label) {
		// summary:
		//		Set globalized column header label
		this._col.label = label;		
	},	
	
	getDescription: function() {
		// summary:
		//		Returns globalized description
		if (this._col.description) {
			// string
			if (typeof this._col.description === "string") {
				return this._col.description;
			}
			// object with key/value info
			else {
				return this._col.description.value;
			}
		}
		
		return null;
	},

	setDescription: function(description) {
		// summary:
		//		Set globalized description
		this._col.description = description;		
	},	
	
	getValueType: function() {
		// summary:
		//		Returns the type associated with the column
		//		
		//		|	values:	"itemName", "itemType", "linkName", "linkType", "relationshipName", "relationshipType",
		//		|			string", "string[]",
		//		|			"byte", "short", "int", "long", "byte[]", "short[]", "int[]", "long[]",
		//		|			"ubyte, "ushort", "uint", "ulong", "ubyte[]", "ushort[]", "uint[]", "ulong[]",
		//		|			"boolean", "boolean[]"
		//		|			"enum",
		//		|			"status"
		//		|			"date", "datetime", "time", "date[]", "datetime[]", "time[]"
		//		|			"isodate", isodatetime", "isotime", "isodate[]", "isodatetime[], "isotime[]", "isointerval", "isoduration"
		//		|			"float", "double", "percentage", "float[]", "double[]"
		return this._col.valueType;
	},

	setValueType: function(valueType) {
		// summary:
		//		Set the type associated with the column	
		this._col.valueType = valueType;
	},
	
	getWidth: function() {
		// summary:
		//		Returns column width string as: pixels, percentage, or em value
		return this._col.width;
	},

	setWidth: function(width) {
		// summary:
		//		Set the column width string as: pixels, percentage, or em value
		// width: String
		//		Column width value
		this._col.width = width;
	},
	
	isResizeable: function() {
		// summary:
		//		Returns true if column is resizeable by the user
		return (this._col.resizeable !== undefined ? this._col.resizeable : true);
	},
	
	setResizeable: function(resizeable) {
		// summary:
		//		Set if the column is to be resizeable by the user
		this._col.resizeable = resizeable;
	},
	
	isLineWrap: function() {
		// summary:
		//		Returns true if column allows line wrap to multiple lines
		return (this._col.lineWrap !== undefined ? this._col.lineWrap : false);
	},
	
	setLineWrap: function(lineWrap) {
		// summary:
		//		Set if the column allows line wrap to multiple lines
		this._col.lineWrap = lineWrap;
	},
	
	isLocked: function() {
		// summary:
		//		Returns true if column locked in non-scrollable region
		return (this._col.locked !== undefined ? this._col.locked : false);
	},
	
	setLocked: function(Locked) {
		// summary:
		//		Set whether the column is locked in non-scrollable region
		this._col.locked = locked;
	},
		
	isHidden: function() {
		// summary:
		//		Returns true if the column is to be hidden from the user
		return (this._col.hidden !== undefined ? this._col.hidden : false);
	},
	
	setHidden: function(hidden) {
		// summary:
		//		Set if the column is to be hidden from the user
		this._col.hidden = hidden;
	},
	
	getImage: function() {
		// summary:
		//		Returns optional image for the column
		return this._col.image;
	},
	
	setImage: function(image) {
		// summary:
		//		Set optional image for the column
		this._col.image = image;
	},
	
	getSortOrder: function() {
		// summary:
		//		Returns sort order priority (value is 0 based and should be -1 if not sorted)
		return (this._col.sortOrder !== undefined ? this._col.sortOrder : -1);
	},

	setSortOrder: function(sortOrder) {
		// summary:
		//		Set sort order priority (value is 0 based and should be -1 if not sorted)
		this._col.sortOrder = sortOrder;
	},

	isSortable: function() {
		// summary:
		//		Returns true if the column is sortable
		return (this._col.sortable !== undefined ? this._col.sortable : true);
	},

	setSortable: function(sortable) {
		// summary:
		//		Set if the column is sortable
		this._col.sortable = sortable;
	},

	isSortAscending: function() {
		// summary:
		//		Returns true if the sort is ascending or descending
		return (this._col.sortAscending !== undefined ? this._col.sortAscending : true);
	},

	setSortAscending: function(ascending) {
		// summary:
		//		Set if the sort is ascending or descending
		this._col.sortAscending = ascending;
	},

	isSearchable: function() {
		// summary:
		//		Returns true if the column is searchable
		return (this._col.searchable !== undefined ? this._col.searchable : true);
	},

	setSearchable: function(searchable) {
		// summary:
		//		Set if the column is searchable
		this._col.searchable = searchable;
	},

	isFilterable: function() {
		// summary:
		//		Returns true if the column is filterable
		return (this._col.filterable !== undefined ? this._col.filterable : true);
	},
	
	setFilterable: function(filterable) {
		// summary:
		//		Set if the column is filterable
		this._col.filterable = filterable;
	},
	
	isFiltered: function() {
		// summary:
		//		Returns true if the column is currently filtered
		return (this._col.filtered !== undefined ? this._col.filtered : false);
	},

	setFiltered: function(filtered) {
		// summary:
		//		Set if the column is currently filtered
		this._col.filtered = filtered;
	},
	
	getFilter: function() {
		// summary:
		//		Returns filter object for the column		
		return this._col.filter;
	},
	
	getUnsupportedFilterOperators: function() {
		// summary:
		//		Returns String[] of operators that are not supported for this
		//		column or null if all operators are supported.
		return (this._col.filter ? this._col.filter.unsupportedOperators : null);		
	},
	
	hasPossibleValues: function() {
		// summary:
		//		Returns true if the column supports a list of possible values		
		var filter = this._col.filter;
		return ((filter && (filter.possibleValues || filter.possibleValuesUri)) ? true : false);
	},
	
	hasInlinedPossibleValues: function() {
		// summary:
		//		Returns true if the column supports possible values and
		//		all of those values have been inlined in the json
		var filter = this._col.filter;
		return (filter && filter.possibleValues ? true : false);
	},
	
	hasUrlAddressablePossibleValues: function() {
		// summary:
		//		Returns true if the column supports possible values and
		//		the values are addressable via a url
		var filter = this._col.filter;
		return (filter && filter.possibleValuesUri ? true : false);
	},
	
	getPossibleValues: function(kwArgs) {
		// summary:
		//		Retrieves a list of possible values for columns with a valueType of
		//		enum, map, itemType, linkType, or relationshipType
		var args = {
			onComplete: function(values, totalCount, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
	
		var filter = this._col.filter;
		
		// possible values were inlined; just return them
		if (filter && filter.possibleValues) {
			if (args.start === 0 && args.count === null) {
				args.onComplete(filter.possibleValues, filter.possibleValues.length, args);
			}
			else {
				var values = [];
				var length = filter.possibleValues.length;
				var count = args.count ? args.count : length;
				for (var index = args.start;  index < length && count > 0;  ++index, --count) {
					values.push(filter.possibleValues[index]);
				}
				args.onComplete(values, length, args);
			}
		}
		// possible values are available via uri
		else if (filter && filter.possibleValuesUri) {
			var content = ibm.tivoli.navmodel.Utils.getContent();
			ibm.tivoli.navmodel.Utils.mixinParams(content, this._dataset);
			if (args.start) {
				content.start = args.start;
			}
			if (args.count) {
				content.count = args.count;
			}
			if (this._objectType) {
				content.type = this._objectType;
			}
			
			dojo.xhrGet({
				url: this._dataset.getArg("baseUrl") + filter.possibleValuesUri,
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version(), "columnValueList")
				},
				handleAs: "json",
				sync: args.sync,
				content: content,		
				load: dojo.hitch(this, function(result, ioArgs){
					args.onComplete(result.items, result.totalRows, args);					
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					args.onError(error, ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "getPossibleValues", this._trace, this._version()), ioArgs);
				})
			});	
		}
		// doesn't have any
		else {
			args.onComplete(null, 0, args);
		}		
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_version: function() {		
		// summary:
		//		Returns rest version
		return this._dataset.getVersion();
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.EventRouter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.EventRouter"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.EventRouter"); 



ibm.tivoli.navmodel.EventRouter._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.EventRouter");

////////////////////////////////////
// EventRouter
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.EventRouter", null, {
	// private members
	_args: null,
	_initialized: false,
	_globalObj: null,
	_trace: ibm.tivoli.navmodel.EventRouter._trace,
	
	/*=====
	constructor_kwArgs: function(eventDestination, timeout, pollDelay, pollErrorDelay, contextRoot) {
		// eventDestination: String
		//		destination to listen to (i.e. provided by Dataset)
		// timeout: int?
		//		number of seconds to wait for events  (optional)
		// pollDelay: int?
		//		number of ms to wait in between poll (optional)
		// pollErrorDelay: int?
		//		number of ms to wait after an error before next poll  (optional)
		// contextRoot: String?
		//		context root for requests to event servlet  (optional)		
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs EventRouter object.
		this._args = kwArgs;
		
		// determine the document that we want to anchor the javascript under
		try {
			this._globalObj = ISOLATION_INIFRAME ? parent : window;
		}
		catch(e) {
			this._globalObj = parent;
		}		
		
		// load javascript if we haven't already				
		this._loadEventRouterJavascript();	
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	/*=====
	addListener_event: function(event){},
	=====*/

	addListener: function(/*String*/ eventId, /*addListener_event*/ eventCallback) {
		// summary:
		//		Adds a listener for the requested eventId
		// eventId:
		//		id of events (i.e. provided by Dataset)
		// eventCallback:
		//		callback function to invoke when events are received
		if (this._initialized) {
			if (this._trace.isFinest()) {
				this._trace.finest("adding listener for eventId=" + eventId, "addListener");
			}
			
			this._getGlobalEventRouter().addListener(this, eventId, eventCallback);
		}
		else {
			if (this._trace.isFinest()) {
				this._trace.finest("Waiting on event_router.js to finish loading", "addListener");
			}
					
			setTimeout(dojo.hitch(this, function() {this.addListener(eventId, eventCallback);}), 50);
		}
	},

	removeListener: function(/*String*/ eventId) {
		// summary:
		//		Removes a listener for the requested eventId
		// eventId:
		//		id of events (i.e. provided by Dataset)
		if (this._initialized) {
			return this._getGlobalEventRouter().removeListener(this, eventId);
		}
		
		return 0;
	},

	ping: function() {
		// summary:
		//		Check if the event router is still alive
		console.assert(true);
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////

	_init: function() {	
		// summary:
		//		Initializes the event router
		var globalEventRouter = this._getGlobalEventRouter();
		
		if (globalEventRouter) {
			if (!globalEventRouter.isInitialized()) {
				if (this._trace.isFinest()) {
					this._trace.finest("initializing global event router", "_init");
				}
			
				globalEventRouter.init(this._args);
			}
			
			this._initialized = true;
		}
		else {		
			if (this._trace.isFinest()) {
				this._trace.finest("Waiting on event_router.js to finish loading", "_init");
			}
					
			setTimeout(dojo.hitch(this, function() {this._init();}), 50);
		}	
	},

	_loadEventRouterJavascript: function() {
		// summary:
		//		Load event_router.js using script tag
		var id = "ibm_tivoli_navmodel_event_router";
		var script = this._globalObj.document.getElementById(id);
		if (!script && !this._getGlobalEventRouter()) {
			if (this._trace.isFinest()) {
				this._trace.finest("Inserting event_router.js", "_loadEventRouterJavascript");
			}
			
			// so we can use inside onreadystatechange
			var that = this;
			
			// create the script tag
			script = this._globalObj.document.createElement('script');
			script.id = id;
			script.type = "text/javascript";
			script.src = dojo.moduleUrl("ibm.tivoli.navmodel", "event_router.js").uri;
			script.onload = dojo.hitch(this, this._init);
			script.onreadystatechange = function () { 
				if (this.readyState == 'loaded' || this.readyState == 'complete') {
					that._init();				
				} 
			};
			
			// insert the script tag		
			this._globalObj.document.body.appendChild(script);			
		}
		// script has loaded or in the process of loading
		else {	
			if (this._trace.isFinest()) {
				this._trace.finest("event_router.js is already loaded", "_loadEventRouterJavascript");
			}
			this._init();				
		}		
	},

	_getGlobalEventRouter: function() {
		// summary:
		//		Returns instance of global event router; depending on how javascript was loaded
		var eventRouter = this._globalObj.ibm_tivoli_navmodel_EventRouter;		
		if (!eventRouter) {
			try { 
				eventRouter = ibm_tivoli_navmodel_EventRouter;
			}
			catch (e) {
				// event_router.js has not been loaded
			}
		}
		
		return eventRouter;
	}
});


ibm.tivoli.navmodel.EventRouter.getPageId = function() {	
	// determine the document that we want to anchor the javascript under
	var globalObj = null;
	try {
		globalObj = ISOLATION_INIFRAME ? parent : window;		
	}
	catch(e) {
		globalObj = parent;
	}
	
	if (!globalObj.ibm_tivoli_navmodel_EventRouter_pageId) {
		var now = new Date();
		globalObj.ibm_tivoli_navmodel_EventRouter_pageId = now.getTime().toString();
	}
	
	return globalObj.ibm_tivoli_navmodel_EventRouter_pageId;
};

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Event"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Event"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Event"); 

// includes


////////////////////////////////////
// Event
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Event", null, {
	// private members
	_dataset: null,
	_event: null,
	_items: null,
	_links: null,
	_relationships: null,
	_props: null,
	_eventRouterStatus: null,
	
	/*=====
	constructor_kwArgs: function(dataset, event) {
		// dataset: Object
		//		the dataset that the event is for
		// event: Object
		//		raw event object
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs Event object.
		this._dataset = kwArgs.dataset;		
		this._event = kwArgs.event;
		this._items = [];
		this._links = [];
		this._relationships = [];
		this._props = [];
		
		// "eventRouterStatus" events originate from the EventRouter;
		// fill in this info to be consistent with other events
		if (this._event.id == "eventRouter") {
			this._event.providerId = this._dataset.getProviderId();
			this._event.datasourceId = this._dataset.getDatasourceId();
			this._event.datasetId = this._dataset.getId();
			this._event.version = this._dataset.getVersion();
		}
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	getDataset: function() {
		return this._dataset;
	},
	
	getVersion: function() {
		return this._event.version;
	},
	
	getProviderId: function() {
		return this._event.providerId;
	},
	
	getDatasourceId: function() {
		return this._event.datasourceId;
	},
	
	getDatasetId: function() {
		return this._event.datasetId;
	},
	
	getId: function() {
		return this._event.id;
	},
	
	getType: function() {
		return this._event.type;
	},
	
	getItems: function() {
		return this._items;
	},
	
	getLinks: function() {
		return this._links;
	},
	
	getRelationships: function() {
		return this._relationships;
	},
	
	getProperties: function() {
		return this._props;
	},
	
	getLinkChanged: function() {
		return this._links[0];
	},
	
	getLinkChangedReason: function() {
		return this._event.reason;
	},
	
	getLinkChangedRelationships: function() {
		return this._relationships;
	},
	
	getNewLabel: function() {
		return this._event.label;
	},
	
	getException: function() {
		return new ibm.tivoli.navmodel.Exception({exception: this._event.exception, version: this._dataset.getVersion()});
	},
	
	getEventRouterStatus: function() {
		return this._event.eventRouterStatus;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_addItem: function(item, props) {
		this._items.push(item);
		this._props.push(props);
	},
	
	_addLink: function(link, props) {
		this._links.push(link);
		this._props.push(props);
	},
	
	_addRelationship: function(relationship, props) {
		this._relationships.push(relationship);
		this._props.push(props);
	},
	
	_getEvent: function() {
		return this._event;
	},
	
	_getEventItems: function() {
		return this._event.items;
	},
	
	_getEventLinks: function() {
		return this._event.links;
	},
	
	_getEventLink: function() {
		return this._event.link;
	},
	
	_getEventRelationships: function() {
		return this._event.relationships;
	},
	
	_getEventNewLabel: function() {
		return this._event.label;
	},
	
	_getEventStatuses: function() {
		return this._event.status;
	},
	
	_getEventDatasetItemCount: function() {
		return this._event.datasetItemCount;
	},
	
	_getEventDatasetLinkCount: function() {
		return this._event.datasetLinkCount;
	},
	
	_getEventDatasetRelationshipCount: function() {
		return this._event.datasetRelationshipCount;
	}
});


////////////////////////////////////////////////////////////////////////
// ibm.tivoli.navmodel.ClientEvent
//
// For creating client initiated ibm.tivoli.navmodel.Event objects
////////////////////////////////////////////////////////////////////////

ibm.tivoli.navmodel.ClientEvent = {
	getChangedEvent: function(dataset) {
		// summary:
		//		Creates client-side event for type "changed"
		var eventObj = this._eventObj(dataset, "changed");		
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemsAddedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemsAdded");
		eventObj.datasetItemCount = dataset.getItemCount() + items.length;		
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemsRemovedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemsRemoved");
		eventObj.datasetItemCount = dataset.getItemCount() - items.length;		
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemStatusChangedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemStatusChanged");
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getItemPropertiesChangedEvent: function(dataset, items) {
		var eventObj = this._eventObj(dataset, "itemPropertiesChanged");
		this._mixinItems(eventObj, items);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinksAddedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linksAdded");
		eventObj.datasetLinkCount = dataset.getLinkCount() + links.length;		
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinksRemovedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linksRemoved");
		eventObj.datasetLinkCount = dataset.getLinkCount() - links.length;		
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinkStatusChangedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linkStatusChanged");
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinkPropertiesChangedEvent: function(dataset, links) {
		var eventObj = this._eventObj(dataset, "linkPropertiesChanged");
		this._mixinLinks(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLinkChangedEvent: function(dataset, link, relationships, relationshipAdded) {
		var eventObj = this._eventObj(dataset, "linkPropertiesChanged");
		eventObj.reason = (relationshipAdded ? "relationshipAdded" : "relationshipRemoved");
		eventObj.link = link.getAttributes();
		this._mixinRelationships(eventObj, links);
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	getLabelChangedEvent: function(dataset, newLabel) {
		var eventObj = this._eventObj(dataset, "labelChanged");
		eventObj.label = label;
		return new ibm.tivoli.navmodel.Event({dataset: dataset, event: eventObj});
	},
	
	_eventObj: function(dataset, type) {
		return {
			"id": dataset.getEventId(),
			"type": type,
			"providerId": dataset.getProviderId(),
			"datasourceId": dataset.getDatasourceId(),
			"datasetId": dataset.getId(),
			"version": dataset.getVersion()
		};		
	},
	
	_mixinItems: function(eventObj, items) {
		var itemObjs = [];
		var itemLength = items.length;
		for (var itemIndex = 0;  itemIndex < itemLength;  ++itemIndex) {
			itemObjs[itemIndex] = items[itemIndex].getAttributes();
		}
		eventObj.items = itemObjs;
	},
	
	_mixinLinks: function(eventObj, links) {
		var linkObjs = [];
		var linkLength = links.length;
		for (var linkIndex = 0;  linkIndex < linkLength;  ++linkIndex) {
			linkObjs[linkIndex] = links[linkIndex].getAttributes();
		}
		eventObj.links = linkObjs;
	},
	
	_mixinRelationships: function(eventObj, relationships) {
		var relationshipObjs = [];
		var relationshipLength = relationships.length;
		for (var relationshipIndex = 0;  relationshipIndex < relationshipLength;  ++relationshipIndex) {
			relationshipObjs[relationshipIndex] = relationships[relationshipIndex].getAttributes();
		}
		eventObj.relationships = relationshipObjs;
	}
};

}

if(!dojo._hasResource["ibm.tivoli.navmodel.WidgetDelta"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.WidgetDelta"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.WidgetDelta"); 
 
// includes


ibm.tivoli.navmodel.WidgetDelta._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.WidgetDelta");

////////////////////////////////////
// WidgetDelta
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.WidgetDelta", null, {
	// private members       
	_widgetDef: null,
	_type: null,
	_delta: null,
	_pending: false,	
	_trace: ibm.tivoli.navmodel.WidgetDelta._trace,

	/*=====
	 constructor_kwArgs: function(widgetDef, type) {
		// widgetDef: ibm.tivoli.navmodel.WidgetDef (or descendent)
		//		widget definition object to manipulate delta
		// type: String
		//		the delta type:  "user" or "shared"		
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		var args = {
			widgetDef: null,
			type: null
		};
		dojo.mixin(args, kwArgs);
		
		// summary:
		//		Constructs WidgetDelta object.
		this._widgetDef = args.widgetDef;
		this._type = (args.type ? args.type : this._widgetDef.getArg("type"));
		
		switch(this._type) {
			case "user":
				this._delta = this._widgetDef.getUserDelta();
				break;
			case "shared":
				this._delta = this._widgetDef.getSharedDelta();
				break;
			default:
				this._trace.error("Unknown delta type=" + this._type, "constructor");
		}
		
		if (this._delta) {
			this._delta = dojo.clone(this._delta);
		}
		else {
			this._delta = {};
		}		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
		
	getType: function() {
		// summary:
		//		Return delta type: either "user" or "shared"
		return this._type;
	},
	
	hasPendingChanges: function() {
		// summary:
		//		Returns true if changes are pending
		return this._pending;
	},
	
	save: function() {
		// summary:
		//		Save current delta changes to the widget registry;
		//		delete the delta if it is empty.
		if (this.hasPendingChanges()) {
			switch(this._type) {
				case "user":
					if (this._hasKeys(this._delta)) {
						this._widgetDef.setUserDelta(this._delta);
					}
					else {
						this._widgetDef.deleteUserDelta();
					}
					break;
				case "shared":			
					if (this._hasKeys(this._delta)) {
						this._widgetDef.setSharedDelta(this._delta);
					}
					else {
						this._widgetDef.deleteSharedDeleta();
					}
			}
			this._pending = false;
		}
	},
	
	clear: function() {
		// summary:
		//		Clear the entire delta
		this._delta = {};
		this._pending = true;
	},
	
	/*=====
	 clearSection_kwArgs: function(sectionKey) {
	 	//	sectionKey: String
		//		The key that identifies the section		
	},
	=====*/
	
	clearSection: function(/*clearSection_kwArgs*/ kwArgs) {
		if (this._delta[kwArgs.sectionKey]) {
			delete this._delta[kwArgs.sectionKey];			
		}
		
		this._pending = true;
	},
	
	/*=====
	 sectionAttribute_kwArgs: function(sectionKey, attrKey) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	attrKey: String
		//		The attribute in the section
	},
	=====*/
	
	getSectionAttribute: function(/*sectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Get a key/value in a specified section
		//
		//	Example:
		//	{	...
		//		"topology: {
		//			"backgroundColor": "white",
		//			...
		//		},
		//		...
		//	}
		//	Get "backgroundColor" attribute in "topology" section 
		//	var mode = delta.getSectionAttribute({sectionKey: "topology", attrKey: "backgroundColor"});		
		var value = null;
		var section = this._delta[kwArgs.sectionKey];
		
		// section exists in delta
		if (section) {
			value = section[kwArgs.attrKey];						
		}
		
		return value;	
	},
	
	/*=====
	 setSectionAttribute_kwArgs: function(sectionKey, attrKey, attrValue) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	attrKey: String
		//		The attribute in the section
		//	attrValue: any
		//		The value to be set on the attribute
	},
	=====*/
		
	setSectionAttribute: function(/*setSectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Set a key/value in a specified section
		//
		//	Example:
		//	{	...
		//		"topology: {
		//			"backgroundColor": "white",
		//			...
		//		},
		//		...
		//	}
		//	Set "backgroundColor" attribute in "topology" section 
		//	delta.setSectionAttribute({sectionKey: "topology", attrKey: "backgroundColor", attrValue: "gray"});		
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// section exists in delta
		if (section) {
			section[kwArgs.attrKey] = kwArgs.attrValue;			
			rc = true;			
		}
		// section does not exist in delta
		else {
			section = {};
			section[kwArgs.attrKey] = kwArgs.attrValue;
			this._delta[kwArgs.sectionKey] = section;
			rc = true;
		}
		
		this._pending = true;
		return rc;		
	},
	
	removeSectionAttribute: function(/*sectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		 Remove a key from a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"topology: {
		//			"backgroundColor": "white",
		//			...
		//		},
		//		...
		//	}
		//	Remove "backgroundColor" attribute from "topology" section
		//	delta.removeSectionAttribute({sectionKey: "topology", attrKey: "backgroundColor"});				
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// section exists in delta
		if (section) {
			delete section[kwArgs.attrKey];
			rc = true;
			
			// if section is empty; we can remove from delta
			if (!this._hasKeys(section)) {
				delete this._delta[kwArgs.sectionKey];
			}					
		}
			
		this._pending = true;					
		return rc;		
	},
	
	/*=====
	 arraySectionAttribute_kwArgs: function(sectionKey, elementKey, elementValue, attrKey) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	elementKey: String
		//		The key used to identify the array element
		//	elementValue: String
		//		The value used to identify the array element (i.e. compare to elementKey)
		//	attrKey: String
		//		The attribute in the array element			
	},
	=====*/
		
	getArraySectionAttribute: function(/*arraySectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Get a key/value in a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"tables: [
		//			{	"id": "ItemsTable",
		//				"filter": {
		//					...
		//				},
		//				...
		//			},
		//			...
		//		],
		//		...
		//	}
		//	Get "filter" attribute in "tables" section (i.e. the element where "id" is "ItemsTable")		
		//	var filter = delta.getArraySectionAttribute({sectionKey: "tables", elementKey: "id", elementValue: "ItemsTable", attrKey: "filter"});
		
		var value = null;
		var section = this._delta[kwArgs.sectionKey];
		
		// array section exists in delta
		if (section && dojo.isArray(section)) {
			var count = section.length;
			for (var index = 0;  index < count;  ++index) {
				var element = section[index];
				if (element[kwArgs.elementKey] === kwArgs.elementValue) {
					value = element[kwArgs.attrKey];
					break;
				}
			}				
		}
		
		return value;	
	},
		
	/*=====
	 setArraySectionAttribute_kwArgs: function(sectionKey, elementKey, elementValue, attrKey, attrValue) {
	 	//	sectionKey: String
		//		The key that identifies the section
		//	elementKey: String
		//		The key used to identify the array element
		//	elementValue: String
		//		The value used to identify the array element (i.e. compare to elementKey)
		//	attrKey: String
		//		The attribute to be modified in the array element
		//	attrValue: any
		//		The value to be set on the array element attribute
	},
	=====*/
	
	setArraySectionAttribute: function(/*setArraySectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		Set a key/value in a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"tables: [
		//			{	"id": "ItemsTable",
		//				"filter": {
		//					...
		//				},
		//				...
		//			},
		//			...
		//		],
		//		...
		//	}
		//	Set "filter" attribute in "tables" section (i.e. the element that has "id" = "ItemsTable")		
		//	delta.setArraySectionAttribute({sectionKey: "tables", elementKey: "id", elementValue: "ItemsTable", attrKey: "filter", attrValue: value});
		
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// array section exists in delta
		if (section && dojo.isArray(section)) {
			var count = section.length;
			for (var index = 0;  index < count;  ++index) {
				var element = section[index];
				if (element[kwArgs.elementKey] === kwArgs.elementValue) {
					element[kwArgs.attrKey] = kwArgs.attrValue;
					rc = true;
					break;
				}
			}				
		}
		// array section does not exist in delta
		else {
			var newElement = {};
			newElement[kwArgs.elementKey] = kwArgs.elementValue;
			newElement[kwArgs.attrKey] = kwArgs.attrValue;
			this._delta[kwArgs.sectionKey + "Identifier"] = kwArgs.elementKey;			
			this._delta[kwArgs.sectionKey] = [newElement];
			rc = true;
		}
		
		this._pending = true;					
		return rc;		
	},
	
	removeArraySectionAttribute: function(/*arraySectionAttribute_kwArgs*/ kwArgs) {
		// summary:
		//		 Remove a key from a specified array element (where section is an array)
		//
		//	Example:
		//	{	...
		//		"tables: [
		//			{	"id": "ItemsTable",
		//				"filter": {
		//					...
		//				},
		//				...
		//			},
		//			...
		//		],
		//		...
		//	}
		//	Remove "filter" attribute from "tables" section (i.e. the element that has "id" = "ItemsTable")		
		//	delta.removeArraySectionAttribute({sectionKey: "tables", elementKey: "id", elementValue: "ItemsTable", attrKey: "filter"});
				
		var rc = false;
		var section = this._delta[kwArgs.sectionKey];
		
		// array section exists in delta
		if (section && dojo.isArray(section)) {
			var count = section.length;
			for (var index = 0;  index < count;  ++index) {
				var element = section[index];
				if (element[kwArgs.elementKey] === kwArgs.elementValue) {					
					delete element[kwArgs.attrKey];
					rc = true;
					
					// if the only key left in the array element is the
					// elementKey; then we can remove this element
					var keys = this._getKeys(element);
					if (keys.length === 1 && keys[0] === kwArgs.elementKey) {
						section.splice(index,1);
					}
					
					// if array is empty; we can remove from the delta
					if (section.length === 0) {
						delete this._delta[kwArgs.sectionKey];
						delete this._delta[kwArgs.sectionKey + "Identifier"];
					}					
					break;
				}
			}				
		}
				
		this._pending = true;						
		return rc;		
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	_hasKeys: function(obj) {
		for (var key in obj) {
			if (obj.hasOwnProperty(key)) {
				return true;
			}
		}
		
		return false;
	},
	
	_getKeys: function(obj) {
		var keys = [];		
		for (var key in obj) {
			if (obj.hasOwnProperty(key)) {
				keys.push(key);
			}
		}
		return keys;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.AbstractWidgetDef"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.AbstractWidgetDef"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.AbstractWidgetDef"); 
 
// includes





ibm.tivoli.navmodel.AbstractWidgetDef._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.AbstractWidgetDef");

////////////////////////////////////
// AbstractWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.AbstractWidgetDef", null, {
	// private members       
	_args: null,
	_parentWidgetDef: null,	
	_widgetDef: null,
	_userDelta: null,
	_sharedDelta: null,
	_destroyed: false,
	_trace: ibm.tivoli.navmodel.AbstractWidgetDef._trace,

	/*=====
	 constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, profileID, parentWidgetDef, registryLocation, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// profileID: String
		//		optional unique id to associate with reading and writing the
		//		"user" and "shared" deltas to the widget definition.  If not
		//		supplied; a uniqueId is created consisting of all the related
		//		information (i.e. providerType, datasourceType, widgetType, etc)
		// parentWidgetDef: ibm.tivoli.navmodel.AbstractWidgetDef  (or descendent)
		//		optional parent widget definition
		// registryLocation: String
		//		Indicates if "remote" provider definition is in local registry or is
		//		located at the remote rest service.  
		//		values are: "local" or "remote"  (Default for TIP is "local")
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/

	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs AbstractWidgetDef object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		this._args = {
			baseUrl: null,
			provider: null,
			datasource: null,
			widget: null,
			definition: null,
			type: "user",
			profileID: null,
			parentWidgetDef: null,
			// TIP policy is to have remote provider widget definitions in local database
			registryLocation: (ibm.tivoli.navmodel.Utils.isTIP() ? "local" : null),
			version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion()
		};
		dojo.mixin(this._args, kwArgs);
		
		// convert datasource of "" to null
		if (this._args.datasource && this._args.datasource.length === 0) {
			this._args.datasource = null;
		}
		
		this._parentWidgetDef = this._args.parentWidgetDef;
		this._fetchWidgetDef();
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	
	validateDefaults: function() {
		// summary:
		//		This method is called after the widget def has been read from the server.
		//		Sub-class can override this method and validate/set any defaults that were
		//		not present in the widget def contents
	},
	
	getParentWidgetDef: function() {
		// summary:
		//		Returns parent widget definition 
		return this._parentWidgetDef;
	},
	
	setParentWidgetDef: function(parent) {
		// summary:
		//		Set parent widget definition; will force a
		//		re-fetch of the widget definition from server if
		//		parent has a profileID set and this definition does not.
		this._parentWidgetDef = parent;
		
		// if we do NOT have a profileID; and the new parent does;
		// then we need to re-fetch with the parents profileID
		if (parent && !this._args.profileID && parent.getProfileID()) {
			this._fetchWidgetDef();
		} 
	},
	
	getProfileID: function() {
		// summary:
		//		Returns delta id value if specified;  will inherit value from
		//		parent widget definition chain if not explicitly set
		var profileID = null;
		
		// profileID explicitly declared		
		if (this._args.profileID) {
			profileID = this._args.profileID;
		}
		// inherit profileID from parent
		else if (this._parentWidgetDef) {
			profileID = this._parentWidgetDef.getProfileID();			
		}
		
		return profileID;
	},
	
	setProfileID: function(/*String*/ profileID) {
		// summary:
		//		Set new profileID for the widget definition;
		//		will force a re-fetch of the widget definition from server
		this._args.profileID = profileID;
		this._fetchWidgetDef();
	},	
			
	getArg: function(key) {
		// summary:
		//		Returns a specific initial argument passed to the widget def
		return this._args[key];
	},
	
	destroy: function() {
		// summary:
		//		Do any necessary clean up 
		this._destroyed = true;
	},

	isDestroyed: function() {
		// summary:
		//		Returns true if the widget def has already been destroyed
		return this._destroyed;
	},

	getDef: function() {
		// summary:
		//		Returns the entire widget definition
		if (this._widgetDef) {
			return this._widgetDef;
		}

		return null;
	},

	getSection: function(section) {
		// summary:
		//		Returns the requested section of the widget definition
		if (this._widgetDef) {
			return this._widgetDef[section];
		}

		return null;
	},
	
	/////////////////////////////
	// meta section data
	/////////////////////////////
	
	getMeta: function() {
		return this._widgetDef.meta;		
	},
	
	getWidgetId: function() {
		return this._widgetDef.meta.id;
	},
	
	getWidgetType: function() {
		return this._widgetDef.meta.type;
	},
	
	getWidgetVersion: function() {
		return this._widgetDef.meta.version;
	},
	
	getWidgetLabel: function() {
		return this._widgetDef.meta.label.value;
	},
	
	getWidgetDescription: function() {
		return this._widgetDef.meta.description.value;
	},
	
	setMeta: function(meta) {
		this._widgetDef.meta = meta;		
	},
	
	setWidgetId: function(id) {
		this._widgetDef.meta.id = id;
	},
	
	setWidgetType: function(type) {
		this._widgetDef.meta.type = type;
	},
	
	setWidgetVersion: function(version) {
		this._widgetDef.meta.version = version;
	},
	
	setWidgetLabel: function(value) {
		this._widgetDef.meta.label.value = value;
	},
	
	setWidgetDescription: function(value) {
		this._widgetDef.meta.description.value = value;
	},

	/////////////////////////////
	// provider section data
	/////////////////////////////
	
	getProvider: function() {
		return this._widgetDef.provider;		
	},

	getProviderType: function() {
		return this._widgetDef.provider.type;
	},
	
	getProviderLabel: function() {
		return this._widgetDef.provider.label.value;
	},
	
	getProviderDescription: function() {
		return this._widgetDef.provider.description.value;
	},
	
	setProvider: function(provider) {
		this._widgetDef.provider = provider;		
	},

	setProviderType: function(type) {
		this._widgetDef.provider.type = type;
	},
	
	setProviderLabel: function(value) {
		this._widgetDef.provider.label.value = value;
	},
	
	setProviderDescription: function(value) {
		this._widgetDef.provider.description.value = value;
	},
	
	/////////////////////////////
	// datasource section data
	/////////////////////////////
	
	getDatasource: function() {
		return this._widgetDef.datasource;		
	},
	
	getDatasourceType: function() {
		if (this._widgetDef.datasource) {
			return this._widgetDef.datasource.type;
		}
		
		return null;
	},
	
	getDatasourceLabel: function() {
		if (this._widgetDef.datasource) {
			return this._widgetDef.datasource.label.value;
		}
		
		return null;
	},
	
	getDatasourceDescription: function() {
		if (this._widgetDef.datasource) {
			return this._widgetDef.datasource.description.value;
		}
		
		return null;
	},
	
	setDatasource: function(datasource) {
		this._widgetDef.datasource = datasource;		
	},
	
	setDataSetDatasourceType: function(type) {
		this._widgetDef.datasets[0].datasourceType = type;
	}, 
	
	setDatasourceType: function(type) {
		this._widgetDef.datasource.type = type;
	},
	
	setDatasourceLabel: function(value) {
		this._widgetDef.datasource.label.value = value;
	},
	
	setDatasourceDescription: function(value) {
		this._widgetDef.datasource.description.value = value;
	},	

	/////////////////////////////
	// details section data
	/////////////////////////////
	
	getDetails: function() {
		return this._widgetDef.details;		
	},
	
	getDetailsAttribute: function(attr) {
		return this._widgetDef.details[attr];
	},
	
	setDetails: function(details) {
		this._widgetDef.details = details;
	},
	
	setDetailsAttribute: function(key, value) {
		if (!this._widgetDef.details) {
			this._widgetDef.details = {};
		}
		
		this._widgetDef.details[key] = value;
	},
		
	/////////////////////////////
	// WidgetDelta
	/////////////////////////////
	getWidgetDelta: function(/*String*/ type) {
		// summary:
		//		Get ibm.tivoli.navmodel.WidgetDelta object
		//	type: String
		//		The delta type: "user" or "shared";  defaults to the 
		//		type passed to widget def constructor if not supplied.		
		if (!type) {
			type = this._args.type;			
		}
		
		if (!type || type === "base") {
			type = "user";
		}
		
		var delta = null;
		switch(type) {
			case "user":
				if (!this._userDelta) {
					this._userDelta = new ibm.tivoli.navmodel.WidgetDelta({widgetDef: this, type: type});
				}
				delta = this._userDelta;
				break;
			case "shared":
				if (!this._sharedDelta) {
					this._sharedDelta = new ibm.tivoli.navmodel.WidgetDelta({widgetDef: this, type: type});
				}
				delta = this._sharedDelta;
		}
				
		return delta;
	},
	
	saveWidgetDelta: function() {
		// summary:
		//		Save pending changes on ibm.tivoli.navmodel.WidgetDelta object(s)
		if (this._userDelta && this._userDelta.hasPendingChanges()) {
			this._userDelta.save();
		}
		
		if (this._sharedDelta && this._sharedDelta.hasPendingChanges()) {
			this._sharedDelta.save();
		}
	},
	
	hasPendingChanges: function() {
		// summary:
		//		Returns true if ibm.tivoli.navmodel.WidgetDelta object(s) have pending changes
		return ((this._userDelta && this._userDelta.hasPendingChanges()) || (this._sharedDelta && this._sharedDelta.hasPendingChanges()));
	},
	
	/////////////////////////////
	// raw user delta
	/////////////////////////////
	hasUserDelta: function() {
		// summary:
		//		indicates if raw user delta json object currently exists
		if (this._widgetDef.user) {
			return true;
		}
		
		return false;
	},
	
	getUserDelta: function() {
		// summary:
		//		Get raw user delta json object		
		return this._widgetDef.user;
	},
	
	setUserDelta: function(userDelta) {
		// summary:
		//		PUT raw user delta json object to the server	
		this._putDelta(userDelta, "user");
		this._fetchWidgetDef();
	},
	
	deleteUserDelta: function() {
		// summary:
		//		DELETE raw user delta json object on server
		this._deleteDelta("user");
		this._fetchWidgetDef();
	},
	
	/////////////////////////////
	// raw shared delta
	/////////////////////////////
	hasSharedDelta: function() {
		// summary:
		//		indicates if raw shared delta json object currently exists
		if (this._widgetDef.shared) {
			return true;
		}
		
		return false;
	},
	
	getSharedDelta: function() {
		// summary:
		//		Get raw shared delta json object		
		return this._widgetDef.shared;
	},
	
	setSharedDelta: function(sharedDelta) {
		// summary:
		//		PUT raw shared delta json object to the server	
		this._putDelta(sharedDelta, "shared");
		this._fetchWidgetDef();
	},
	
	deleteSharedDelta: function() {
		// summary:
		//		DELETE raw shared delta json object on server
		this._deleteDelta("shared");
		this._fetchWidgetDef();
	},
	
	/////////////////////////////
	// abstract methods
	/////////////////////////////
	getSelectionMode: function() {
		return null;
	},
	
	getItemStyles: function() {
		return null;		
	},
	
	getItemStyle: function(type) {
		return null;
	},
	
	getDefaultItemStyle: function() {
		return null;
	},
	
	getLinkStyles: function() {
		return null;		
	},
	
	getLinkStyle: function(type) {
		return null;		
	},

	getDefaultLinkStyle: function() {
		return null;
	},
	
	getItemStatuses: function() {
		return null;
	},
	
	getItemStatus: function(id) {
		return null;
	},
	
	getDefaultItemStatus: function() {
		return null;
	},
	
	getLinkStatuses: function() {
		return null;	
	},
	
	getLinkStatus: function(id) {
		return null;
	},
	
	getDefaultLinkStatus: function() {
		return null;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_version: function() {
		return this._args.version;
	},
	
	_fetchWidgetDef: function() {
		var exception = null;
		var profileID = this.getProfileID();
		var content = this._content();
		content.type = this._args.type;
		if (profileID) {
			content.profileID = profileID;
		}
		
		dojo.xhrGet({
			url: this._getWidgetDefinitionURL(),
			headers: {
				'Accept': "application/json"
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs) {
					this._widgetDef = result;
					this._userDelta = null;
					this._sharedDelta = null;
					this.validateDefaults();		
			}),
			error: dojo.hitch (this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_fetchWidgetDef", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	
	_putDelta: function(/*Object*/ delta, /*String*/ deltaType) {
		// summary:
		//		Save delta information for the widget definition
		// delta:
		//		contains the delta to the definition
		// deltaType:
		//		indicates the delta type:  "user" or "shared"
		var exception = null;		
		var json = dojo.toJson(delta, false);
		
		dojo.xhrPut({
			url: this._getPutURL(deltaType),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version()),
				'Content-Type': 'application/json',
				'Content-Length': json.length
			},
			handleAs: "json",
			putData: json,
			sync: true,
			load: dojo.hitch(this, function(result, ioArgs) {									
			}),
			error: dojo.hitch (this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_putDelta", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	
	_deleteDelta: function(/*String*/ deltaType) {
		// summary:
		//		Delete delta information for the widget definition
		// deltaType:
		//		indicates the delta type:  "user" or "shared"
		var exception = null;
		var profileID = this.getProfileID();
		var content = this._content();
		content.type = deltaType;
		if (profileID) {
			content.profileID = profileID;
		}
		
		dojo.xhrDelete({
			url: this._getWidgetDefinitionURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this._version())
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: function(result, ioArgs) {									
			},
			error: dojo.hitch (this, function(error, ioArgs){
				exception = ibm.tivoli.navmodel.Utils.getException(error, ioArgs, "_deleteDelta", this._trace, this._version());				
			})
		});
		
		if (exception) {
			throw exception;
		}
	},
	
	_content: function() {
		// summary:
		//		Construct a "content" object for sending an xdr request;
		//		IE will sometimes return http requests from cache without unique parms
		var content = {};
		
		// add date and random
		var now = new Date();		
		content.d = now.getTime();
		content.r = Math.random();
		
		// registryLocation if specified
		if (this._args.registryLocation) {
			content.registryLocation = this._args.registryLocation;
		}
		
		return content;
	},

	_getDatasourceURL: function() {
		// summary:
		//		Returns url for fetching a datasource
		var url = null;
		
		if (this._widgetDef) {
			var providerId = this._args.provider;
			if (this.getProviderType() === "any") {
				providerId = "any";
			}
			
			var datasourceId = this._args.datasource;
			if (this.getDatasourceType() === "any") {
				datasourceId = "any";
			}
			
			url = this._args.baseUrl + "/providers/" + encodeURIComponent(providerId);
			url += (datasourceId ? ("/datasources/" + encodeURIComponent(datasourceId)) : "");
		}
		else {
			url = this._args.baseUrl + "/providers/" + encodeURIComponent(this._args.provider);
			url += (this._args.datasource ? ("/datasources/" + encodeURIComponent(this._args.datasource)) : "");
		}
		
		return url;	
	},

	_getWidgetDefinitionURL: function() {
		// summary:
		//		Returns url for fetching a widget definition
		var url = this._getDatasourceURL();
		
		if (this._widgetDef) {
			url+= "/widgets/"+ this.getWidgetType() + "/definitions/" + this.getWidgetId() + "-" + this.getWidgetVersion();
		}
		else {
			url+= "/widgets/"+ this._args.widget + "/definitions/" + this._args.definition;			
		}
		
		return url;
	},
	
	_getPutURL: function(deltaType) {
		// summary:
		//		Build url necessary to do a put or post operation;
		//		have to add on the "type" and "version" ourself
		//		since the dojo.xhr function doesn't include the content
		var url = this._getWidgetDefinitionURL();
		var questionMark = false;
		
		// delta type
		if (!questionMark) {
			questionMark = true;
			url += "?type=" + deltaType;
		}
		else {
			url += "&type=" + deltaType;
		}
		
		// profileID if specified
		var profileID = this.getProfileID();
		if (profileID) {
			if (!questionMark) {
				questionMark = true;
				url += "?profileID=" + profileID;
			}
			else {
				url += "&profileID=" + profileID;
			}
		}
		
		// registryLocation if specified
		if (this._args.registryLocation) {
			if (!questionMark) {
				questionMark = true;
				url += "?registryLocation=" + this._args.registryLocation;
			}
			else {
				url += "&registryLocation=" + this._args.registryLocation;
			}
		}
						
		// IE will sometimes return http requests from cache without unique parms
		var now = new Date();
		if (!questionMark) {
			questionMark = true;
			url += "?d=" + now.getTime() + "&r=" + Math.random();
		}
		else {
			url += "&d=" + now.getTime() + "&r=" + Math.random();
		}
		
		return url;
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.WidgetDef"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.WidgetDef"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.WidgetDef"); 
 
// includes



////////////////////////////////////
// WidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.WidgetDef", ibm.tivoli.navmodel.AbstractWidgetDef, {
	// private members
	_itemStyles: null,
	_itemStatuses: null,
	_linkStyles: null,
	_linkStatuses: null,
	_relationshipStyles: null,
	_relationshipStatuses: null,
	
	// default styles and status
	_defaultItemStyle: null,
	_defaultItemStatus: null,
	_defaultLinkStyle: null,
	_defaultLinkStatus: null,
	_defaultRelationshipStyle: null,
	_defaultRelationshipStatus: null,
	
	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs WidgetDef object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		this._itemStyles = {};
		this._itemStatuses = {};
		this._linkStyles = {};
		this._linkStatuses = {};
		this._relationshipStyles = {};
		this._relationshipStatuses = {};
		
		this._defaultItemStyle = {
			"scalarImage": "[widget]/resources/common_assets/common_resource_icons/re64_unknownserver1.swf",
           	"scalarImageWidth": 61.011,
           	"scalarImageHeight": 53.596,
           	"smallImage": "[widget]/resources/common_assets/common_resource_icons/re64_unknownserver1_8.png",
           	"smallImageHeight": 16,
         	"smallImageWidth": 16,
         	"largeImage": "[widget]/resources/common_assets/common_resource_icons/re64_unknownserver1_8.png",
           	"largeImageHeight": 32,
         	"largeImageWidth": 32          	
		};
			
		this._defaultItemStatus = {
			"backgroundScalarImage": "[widget]\/resources\/common_assets\/status_icons\/BK_SWF\/unknownBk141x117gradient.swf",
            "backgroundScalarImageHeight": 117,
            "backgroundScalarImageWidth": 141,
            "backgroundScalarImageVisible": true,            
            "scalarImageHeight": 25,
            "scalarImageWidth": 25,
            "scalarImageVisible": true,
            "imageHeight": 16,
            "imageWidth": 16,
            "imageVisible": true
		};
		
		this._defaultLinkStyle = {
			"lineStyle": "solid",
			"lineWidth": 5, //For best results with link selection and seeing link flyovers on roll over, this is the minimum recommended width for links
			"lineColor": "#000000",
			"labelVisible": false,
			"labelAutoRotate": false,
			"labelParallelAlignment": "middle",
			"labelOrthogonalAlignment": "left",
			"labelDistance": 5,
			"labelPositionType": "relative",
			"labelPosition": 0.5
		};
		
		this._defaultLinkStatus = {            
            "scalarImageHeight": 25,
            "scalarImageWidth": 25,
            "scalarImageVisible": true,
            "imageHeight": 16,
            "imageWidth": 16,
            "imageVisible": true,
            "color": "#000000"
		};
		
		this._defaultRelationshipStyle = this._defaultLinkStyle;
		
		this._defaultRelationshipStatus = this._defaultLinkStatus;
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	destroy: function() {		
		// summary:
		//		Do any necessary clean up
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},
	
	
	//////////////////
	// pluggableview
	//////////////////
	getPluggableView: function() {
		if (!this._widgetDef.pluggableview) {
			this._widgetDef.pluggableview = {};
		}
		
		return this._widgetDef.pluggableview;
	},
	
	//////////
	// views 
	//////////
	getViews: function(){
		var pluggableview = this.getPluggableView();
		return pluggableview.views;
	},
	
	////////////
	// toolbar 
	////////////
	getToolbar: function() {
		var pluggableview = this.getPluggableView();		
		var returnValue = pluggableview.toolbar;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getToolbar) {
				returnValue = parent.getToolbar();
			}
		}		
		return returnValue;	
	},
	
	getToolbarRows: function(){
		var toolbar = this.getToolbar();
		return (toolbar ? toolbar.rows : []);
	},
	
	isToolbarHidden: function() {
		var toolbar = this.getToolbar();
		var returnValue = (toolbar ? toolbar.hidden : undefined);
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}	
	},
	
	//////////////////
	// selectionMode 
	/////////////////
	getSelectionMode: function(){
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.selectionMode;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getSelectionMode){
			returnValue = this._parentWidgetDef.getSelectionMode();
		}		
		return returnValue;			
	},
	
	///////////////
	// labelStyle 
	//////////////
	getLabelStyle: function(){
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.labelStyle;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getLabelStyle){
			returnValue = this._parentWidgetDef.getLabelStyle();
		}		
		return returnValue;
	},
	
	///////////////
	// itemStyles 
	//////////////
	getItemStyles: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.itemStyles;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getItemStyles){
			returnValue = this._parentWidgetDef.getItemStyles();
		}		
		return returnValue;			
	},
	
	getItemStyle: function(type) {
		var style = this._itemStyles[type];
		
		if (!style) {
			var styles = this.getItemStyles();
			if (styles) {
				var styleLength = styles.length;
				for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					if (styles[styleIndex].type === type) {
						style = styles[styleIndex];
						this._itemStyles[type] = style;
						break;
					}
				}
			}
		}
		
		return style;		
	},
	
	////////////////
	// itemStatuses 
	////////////////
	getItemStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.itemStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getItemStatuses){
			returnValue = this._parentWidgetDef.getItemStatuses();
		}		
		return returnValue;	
	},
	
	getItemStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._itemStatuses[key];
		
		if (!status) {
			var statuses = this.getItemStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id &&  statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._itemStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},
	
	///////////////
	// linkStyles 
	///////////////
	getLinkStyles: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.linkStyles;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getLinkStyles){
			returnValue = this._parentWidgetDef.getLinkStyles();
		}		
		return returnValue;			
	},
	
	getLinkStyle: function(type) {
		var style = this._linkStyles[type];
		
		if (!style) {
			var styles = this.getLinkStyles();
			if (styles) {
				var styleLength = styles.length;
				for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					if (styles[styleIndex].type === type) {
						style = styles[styleIndex];
						this._linkStyles[type] = style;
						break;
					}
				}
			}
		}
		
		return style;		
	},
	
	/////////////////
	// linkStatuses 
	////////////////
	getLinkStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.linkStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getLinkStatuses){
			returnValue = this._parentWidgetDef.getLinkStatuses();
		}		
		return returnValue;			
	},
	
	getLinkStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._linkStatuses[key];
		
		if (!status) {
			var statuses = this.getLinkStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id && statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._linkStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},
		
	///////////////////////
	// relationshipStyles 
	///////////////////////
	getRelationshipStyles: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.relationshipStyles;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getRelationshipStyles){
			returnValue = this._parentWidgetDef.getRelationshipStyles();
		}		
		return returnValue;
	},
	
	getRelationshipStyle: function(type) {
		var style = this._relationshipStyles[type];
		
		if (!style) {
			var styles = this.getRelationshipStyles();
			if (styles) {
				var styleLength = styles.length;
				for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					if (styles[styleIndex].type === type) {
						style = styles[styleIndex];
						this._relationshipStyles[type] = style;
						break;
					}
				}
			}
		}
		
		return style;		
	},
	
	/////////////////////////
	// relationshipStatuses 
	/////////////////////////
	getRelationshipStatuses: function() {
		var pluggableview = this.getPluggableView();
		var returnValue = pluggableview.relationshipStatuses;
		if(!returnValue && this._parentWidgetDef && this._parentWidgetDef.getRelationshipStatuses){
			returnValue = this._parentWidgetDef.getRelationshipStatuses();
		}		
		return returnValue;
	},
	
	getRelationshipStatus: function(id, value) {
		var key = id + "_" + value;
		var status = this._relationshipStatuses[key];
		
		if (!status) {
			var statuses = this.getRelationshipStatuses();
			if (statuses) {
				var statusLength = statuses.length;
				for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					if (statuses[statusIndex].id === id && statuses[statusIndex].value === value) {
						status = statuses[statusIndex];
						this._relationshipStatuses[key] = status;
						break;
					}
				}
			}
		}
		
		return status;
	},

	getDefaultItemStyle: function() {
		return this._defaultItemStyle;
	},
	
	getDefaultLinkStyle: function() {
		return this._defaultLinkStyle;
	},
	
	getDefaultRelationshipStyle: function() {
		return this._defaultRelationshipStyle;
	},
		
	getDefaultItemStatus: function() {
		return this._defaultItemStatus;
	},
	
	getDefaultLinkStatus: function() {
		return this._defaultLinkStatus;
	},
	
	getDefaultRelationshipStatus: function() {
		return this._defaultRelationshipStatus;
	},
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_getTypeSection: function() {
		var type = this.getWidgetType();
		if (type.indexOf("-") > 0) {
			var typeInfo = type.split("-");
			type = typeInfo[0];    		        
        }
        
        return this.getSection(type);
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.NavWidgetDef"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.NavWidgetDef"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.NavWidgetDef"); 
 
// includes


////////////////////////////////////
// NavWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.NavWidgetDef", ibm.tivoli.navmodel.WidgetDef, {
	
	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////
	getBreadcrumbs: function(){
		var returnValue = this._widgetDef.navwidget.breadcrumbs;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
		 	if (parent && parent.getBreadcrumbs) {
				returnValue = parent.getBreadcrumbs();
			}
		}		
		return returnValue;			
	},
	
	getNotebook: function(){
		var returnValue = this._widgetDef.navwidget.notebook;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
		 	if (parent && parent.getNotebook) {
				returnValue = parent.getNotebook();
			}
		}		
		return returnValue;			
	},
	
	getViewsLayout: function(){
		var returnValue = this._widgetDef.navwidget.viewsLayout;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
		 	if (parent && parent.getViewsLayout) {
				returnValue = parent.getViewsLayout();
			}
		}		
		return returnValue;					
	},
	

	destroy: function() {		
		// summary:
		//		Do any necessary clean up 
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	}
	
	/////////////////////////////
	// private methods
	/////////////////////////////

});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.TopoWidgetDef"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.TopoWidgetDef"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.TopoWidgetDef"); 
 
// includes


////////////////////////////////////
// TopoWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.TopoWidgetDef", ibm.tivoli.navmodel.WidgetDef, {

	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs TopoWidgetDef object.
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	validateDefaults: function() {
		// summary:
		//		This method is called after the widget def has been read from the server.
		//		Validate defaults that might not be present in the widget def contents.
		
		var layouts = this.getLayouts();
		if (layouts) {
			var length = layouts.length;
			for (var index = 0;  index < length;  ++index) {
				var layout = layouts[index];
				switch (layout.algorithm) {
					case "tree":
						if (!layout.layoutMode) {
							layout.layoutMode = "alternating radial";
						}
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "straight";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						if (!layout.globalAlignment) {
							layout.globalAlignment = "center";
						}
						if (!layout.levelAlignment) {
							layout.levelAlignment = "center";
						}
						if (!layout.aspectRatio) {
							layout.aspectRatio = "1";
						}
						break;
					case "hierarchical":
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.levelJustification) {
							layout.levelJustification = "bottom";
						}
						if (!layout.levelingStrategy) {
							layout.levelingStrategy = "optimal";
						}
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "straight";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						break;
					case "forceDirected":
						if (!layout.layoutMode) {
							layout.layoutMode = "fast multilevel";
						}
						if (!layout.preferredLinksLength) {
							layout.preferredLinksLength = "25";
						}
						if (!layout.linkStyle) {
							layout.linkStyle = "straight";
						}
						if (!layout.maxAllowedMovePerIteration) {
							layout.maxAllowedMovePerIteration = "5";
						}
						if (!layout.allowedNumberOfIterations) {
							layout.allowedNumberOfIterations = "250";
						}
						if (!layout.convergenceThreshold) {
							layout.convergenceThreshold = "0.1";
						}						
						break;
					case "shortLink":
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "direct";
						}
						if (!layout.globalSelfLinkStyle) {
							layout.globalSelfLinkStyle = "two bends orthogonal";
						}
						if (!layout.globalConnectorStyle) {
							layout.globalConnectorStyle = "automatic";
						}
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.levelJustification) {
							layout.levelJustification = "bottom";
						}
						if (!layout.levelingStrategy) {
							layout.levelingStrategy = "optimal";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						if (!layout.linkOffset) {
							layout.linkOffset = "1";
						}
						if (!layout.minFinalSegmentLength) {
							layout.minFinalSegmentLength = "10";
						}
						break;
					case "longLink":
						if (!layout.globalLinkStyle) {
							layout.globalLinkStyle = "direct";
						}
						if (!layout.flowDirection) {
							layout.flowDirection = "bottom";
						}
						if (!layout.levelJustification) {
							layout.levelJustification = "bottom";
						}
						if (!layout.levelingStrategy) {
							layout.levelingStrategy = "optimal";
						}
						if (!layout.connectorStyle) {
							layout.connectorStyle = "automatic";
						}
						if (!layout.verticalMinOffset) {
							layout.verticalMinOffset = "3";
						}
						if (!layout.horizontalMinOffset) {
							layout.horizontalMinOffset = "3";
						}
						break;
					case "grid":
						if (!layout.layoutMode) {
							layout.layoutMode = "tile to rows";
						}
						if (!layout.globalHorizontalAlignment) {
							layout.globalHorizontalAlignment = "center";
						}
						if (!layout.globalVerticalAlignment) {
							layout.globalVerticalAlignment = "center";
						}
						if (!layout.maxNumberOfNodesPerRowOrColumn) {
							layout.maxNumberOfNodesPerRowOrColumn = "3";
						}
				}
			}
		}
	},
	
	destroy: function() {
		// summary:
		//		Do any necessary clean up 
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},

	getDepth: function() {
		var returnValue = this._widgetDef.topology.depth;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getDepth) {
				returnValue = parent.getDepth();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1;
		}
	},
	
	getInitialZoomLevel: function() {
		var returnValue = this._widgetDef.topology.initialZoomLevel;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getInitialZoomLevel) {
				returnValue = parent.getInitialZoomLevel();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1; //Default to -1 which would result in just fitting the contents
		}
	},	

	zoomOnObjectAdded: function() {
		return this._widgetDef.topology.zoomOnObjectAdded;
	},

	zoomLevelOnObjectAdded: function() {
		return this._widgetDef.topology.zoomLevelOnObjectAdded;
	},

	getMaxDepth: function() {
		var returnValue = this._widgetDef.topology.maxDepth;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getMaxDepth) {
				returnValue = parent.getMaxDepth();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1;
		}
	},

	useNodeLevelAndPosition: function() {
		return this._widgetDef.topology.useNodeLevelAndPosition;
	},

	getNodeOrderDirection: function() {
		return this._widgetDef.topology.nodeOrderDirection;
	},

	getNodeOrderProperty: function() {
		return this._widgetDef.topology.nodeOrderProperty;
	},

	getBackgroundColor: function() {
		var returnValue = this._widgetDef.topology.backgroundColor;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getBackgroundColor) {
				returnValue = parent.getBackgroundColor();
			}
		}
		if(returnValue){
			return returnValue;
		} else {
			return -1;
		}
	},
	
	/////////////////////////////
	// footer section data
	/////////////////////////////
	
	getIncludeFooter: function() {
		var returnValue = this._widgetDef.topology.includeFooter;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getIncludeFooter) {
				returnValue = parent.getIncludeFooter();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return true;
		}
	},
	
	setIncludeFooter: function(includeFooter) {
		this._widgetDef.topology.includeFooter = includeFooter;
	},
	
	getHideFooterDateTime: function(){
		var returnValue = this._widgetDef.topology.hideFooterDateTime;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getHideFooterDateTime) {
				returnValue = parent.getHideFooterDateTime();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}
	},

	setHideFooterDateTime: function(hideFooterDateTime) {
		this._widgetDef.topology.hideFooterDateTime = hideFooterDateTime;
	},
	
	getFilters: function(){
		var returnValue = this._widgetDef.topology.filters;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getFilters) {
				returnValue = parent.getFilters();
			}
		}		
		return returnValue;			
	},

	/////////////////////////////
	// search section data
	/////////////////////////////

	isAdvSearchEnabled: function() {
		var returnValue = this._widgetDef.topology.isAdvSearchEnabled;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.isAdvSearchEnabled) {
				returnValue = parent.isAdvSearchEnabled();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}
	},
	
	getAdvSearchProperties: function() {
		var returnValue = this._widgetDef.topology.advSearchProperties;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getAdvSearchProperties) {
				returnValue = parent.getAdvSearchProperties();
			}
		}		
		return returnValue;	
	},	

	/////////////////////////////
	// layout section data
	/////////////////////////////	
	getLayouts: function() {
		var returnValue = this._widgetDef.topology.layout;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getLayouts) {
				returnValue = parent.getLayouts();
			}
		}		
		return returnValue;	
	},
	
	updateLayouts: function(layouts) {
		var delta = this.getWidgetDelta("user");
		if (delta) {
			delta.setSectionAttribute({sectionKey: "topology", attrKey: "layout", attrValue: layouts}); 
		} 
	},
	
	clearLayouts: function() {
		var delta = this.getWidgetDelta("user");
		if (delta) {
			delta.removeSectionAttribute({sectionKey: "topology", attrKey: "layout"}); 
		}
	},
	
	getLayout: function(/*String*/ id) {
		var layouts = this.getLayouts();
		for(var i=0;i<layouts.length;i++){
			if(layouts[i].id === id){
				return layouts[i];
			}
		}
	},
	
	getDefaultLayout: function() {
		var layouts = this.getLayouts();
		for(var i=0;i<layouts.length;i++){
			if(layouts[i].isDefault){
				return layouts[i];
			}
		}
	},
	
	getDefaultLayoutId: function() {
		var layouts = this.getLayouts();
		for(var i=0;i<layouts.length;i++){
			if(layouts[i].isDefault){
				return layouts[i].id;
			}
		}
	},
		
	getRenderRelationshipsInsteadOfLinks: function(){
		var returnValue = this._widgetDef.topology.renderRelationshipsInsteadOfLinks;
		if(returnValue===undefined) {
			var parent = this.getParentWidgetDef();
			if (parent && parent.getRenderRelationshipsInsteadOfLinks) {
				returnValue = parent.getRenderRelationshipsInsteadOfLinks();
			}
		}
		if(returnValue!==undefined){
			return returnValue;
		} else {
			return false;
		}		
	}
	
	/////////////////////////////
	// private methods
	/////////////////////////////	

});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.TableWidgetDef"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.TableWidgetDef"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.TableWidgetDef"); 
 
// includes


////////////////////////////////////
// TableWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.TableWidgetDef", ibm.tivoli.navmodel.WidgetDef, {

	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/ 
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {
		// summary:
		//		Constructs TopoWidgetDef object.
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	destroy: function() {
		// summary:
		//		Do any necessary clean up 
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},
		
	/////////////////////////////
	// dataset section data
	/////////////////////////////	
	getDataSet: function() {
		if (!this._widgetDef.datasets) {
			this._widgetDef.datasets = [];
		}
		
		if (this._widgetDef.datasets.length === 0) {
			this._widgetDef.datasets[0] = {};
		}
		
		return this._widgetDef.datasets;		
	},

	getDataSetID: function() {
		var datasets = this.getDataSet();
		return datasets[0].id;
	},

	getDataSetLabelKey: function() {
		var datasets = this.getDataSet();
		return datasets[0].label.key;
	},
	
	getDataSetLabelValue: function() {
		var datasets = this.getDataSet();
		return datasets[0].label.value;
	},
	
	getDataSetQuery: function() {
		var datasets = this.getDataSet();
		return datasets[0].query;
	},
	
	getDataSetColumns: function() {
		var datasets = this.getDataSet();
		return datasets[0].columns;		
	},
	
	getDataSetThresholds: function(){
		var datasets = this.getDataSet();
		return datasets[0].thresholds;
	},

	getDataSetParameters: function() {
		var datasets = this.getDataSet();
		return datasets[0].parameters;
	},
	
	getDataSetResourceBundle: function() {
		var datasets = this.getDataSet();
		return datasets[0].label.resourceBundle;
	},
	
	
	getDataSetDataSource: function() {
		var datasets = this.getDataSet();
		return datasets[0].datasource;
	},
	
	getDataSetDatasourceType: function() {
		var datasets = this.getDataSet();
		datasets[0].datasourceType = type;
	},
	
	setDataSet: function(datasets) {
		this._widgetDef.datasets = datasets;		
	},

	setDataSetID: function(id) {
		var datasets = this.getDataSet();
		datasets[0].id = id;
	},

	setDataSetLabelKey: function(key) {
		var datasets = this.getDataSet();
		datasets[0].label.key = key;
	},
	
	setDataSetLabelValue: function(value) {
		var datasets = this.getDataSet();
		datasets[0].label.value = value;
	},
	
	setDataSetQuery: function(query) {
		var datasets = this.getDataSet();
		datasets[0].query = query;
	},
	
	setDataSetColumns: function(columns) {
		var datasets = this.getDataSet();
		datasets[0].columns = columns;
	},

	setDataSetParameters: function(parameters) {
		var datasets = this.getDataSet();
		datasets[0].parameters = parameters;
	},
	
	setDataSetResourceBundle: function(bundle) {
		var datasets = this.getDataSet();
		datasets[0].label.resourceBundle = bundle;
	},
	
	/////////////////////////////
	// table section data
	/////////////////////////////	
	getTable: function() {
		if (!this._widgetDef.table) {
			this._widgetDef.table = {};
		}
		
		return this._widgetDef.table;		
	},
	
	getTableID: function() {
		var table = this.getTable();
		return table.id;		
	},
	
	getTableView: function() {
		var table = this.getTable();
		if (!table.view) {
			table.view = [];
		}
		
		return table.view;		
	},
	
	getTableViewElement: function(id) {
		var entry = null;
		var view = this.getTableView();
		if (view) {
			var length = view.length;
			for (var index = 0;  index < length;  ++index) {
				if (view[index].id === id) {
					entry = view[index];
					break;
				}
			}
		}
		
		return entry;
	},
	
	getTableCommon: function() {
		var table = this.getTable();
		return table.common;		
	},

	getTableCommonAutoUpdate: function() {
		var table = this.getTable();
		return table.common.autoUpdate;		
	},
	
	getTableCommonShowLastUpdateTime: function() {
		var table = this.getTable();
		return table.common.showLastUpdateTime;		
	},
	
	getTableCommonPaging: function() {
		var table = this.getTable();
		return table.common.paging;		
	},
	
	getTableCommonPagingSize: function() {
		var table = this.getTable();
		return table.common.pagingSize;		
	},
	
	getTableShowAdvancedFilter: function() {
		var table = this.getTable();
		return table.showAdvancedFilter;
	},
	
	getTableShowQuickFilter: function() {
		var table = this.getTable();
		return table.showQuickFilter;
	},
	
	getTableShowClearFilterConfirmation: function() {
		var table = this.getTable();
		return table.showClearFilterConfirmation;
	},
		
	getTableFilter: function() {
		var table = this.getTable();
		return table.filter;
	},
	
	getTableLabelKey: function() {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		
		return table.label.key;		
	},
	
	getTableLabelValue: function() {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		
		return table.label.value;		
	},
	
	getTableDescriptionKey: function() {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		
		return table.description.key;		
	},
	
	getTableDescriptionValue: function() {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		
		return table.description.value;		
	},
	
	getTableColumnReordering: function() {
		var table = this.getTable();
		return table.columnReordering;		
	},

	setTable: function(table) {
		this._widgetDef.table = table;		
	},
	
	setTableID: function(id) {
		var table = this.getTable();
		table.id = id;		
	},
	
	setTableView: function(view) {
		var table = this.getTable();
		table.view = view;
	},
	
	setTableCommon: function(common) {
		var table = this.getTable();
		table.common = common;		
	},

	setTableCommonAutoUpdate: function(autoUpdate) {
		var table = this.getTable();
		table.common.autoUpdate = autoUpdate;		
	},
	
	setTableCommonShowLastUpdateTime: function(showLastUpdateTime) {
		var table = this.getTable();
		table.common.showLastUpdateTime = showLastUpdateTime;		
	},
	
	setTableCommonPaging: function(paging) {
		var table = this.getTable();
		table.common.paging = paging;		
	},
	
	setTableCommonPagingSize: function(pagingSize) {
		var table = this.getTable();
		table.common.pagingSize = pagingSize;		
	},
	
	setTableShowAdvancedFilter: function(showAdvancedFilter) {
		var table = this.getTable();
		table.showAdvancedFilter = showAdvancedFilter;
	},
	
	setTableFilter: function(filter) {
		var table = this.getTable();
		table.filter = filter;
	},
	
	setTableLabelKey: function(key) {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		table.label.key = key;		
	},
	
	setTableLabelValue: function(value) {
		var table = this.getTable();
		if (!table.label) {
			table.label = {};
		}
		table.label.value = value;		
	},
	
	setTableDescriptionKey: function(key) {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		table.description.key = key;		
	},
	
	setTableDescriptionValue: function(value) {
		var table = this.getTable();
		if (!table.description) {
			table.description = {};
		}
		table.description.value = value;		
	},
	
	setTableColumnReordering: function(columnReordering) {
		var table = this.getTable();
		table.columnReordering = columnReordering;		
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.NotebookWidgetDef"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.NotebookWidgetDef"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.NotebookWidgetDef"); 
 
// includes


////////////////////////////////////
// NavWidgetDef
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.NotebookWidgetDef", ibm.tivoli.navmodel.WidgetDef, {

	/*=====
	constructor_kwArgs: function(baseUrl, provider, datasource, widget, definition, type, version) {
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// widget: String
		//		widget type in form <type>-<version>
		// definition: String
		//		widget definition id in form <id>-<version>
		// type: String
		//		definition type:  "user", "shared", or "base"
		// version: Float
		//		version used to communicate with rest service
	},
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs) {		
		// summary:
		//		Constructs NotebookWidgetDef object.
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	destroy: function() {		
		// summary:
		//		Do any necessary clean up
		if (!this.isDestroyed()) {
			// call base class
			this.inherited(arguments);
		}
	},
	
	getTabOrientation: function() {
		var returnValue = this._widgetDef.notebook.tabOrientation;
		if(!returnValue) {
			var parent = this.getParentWidgetDef();
		 	if (parent && parent.getTabOrientation) {
				returnValue = parent.getTabOrientation();
			}
		}		
		return returnValue;			
	}
	
	/////////////////////////////
	// private methods
	/////////////////////////////	

});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.AbstractDataset"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.AbstractDataset"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.AbstractDataset"); 
 
// includes




















ibm.tivoli.navmodel.AbstractDataset._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.AbstractDataset");

////////////////////////////////////
// AbstractDataset
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.AbstractDataset", null, {
	// private members       
	_args: null,
	
	// dataset attributes
	_dataset: null,
	_version: null,
	
	// widget definition supplied by server
	_widgetDefs: null,
	
	// item, link, relationship, caches
	_itemsById: null,
	_linksById: null,
	_relationshipsById: null,
	_cacheIntervalId: null,
	
	// styles and status caches
	_itemStyles: null,
	_itemStylesById: null,
	_itemStatus: null,
	_itemStatusById: null,	
	_linkStyles: null,
	_linkStylesById: null,
	_linkStatus: null,
	_linkStatusById: null,
	_relStyles: null,
	_relStylesById: null,
	_relStatus: null,
	_relStatusById: null,
	_status: null,
		
	// miscellaneous
	_toolbar: null,
	_eventRouter: null,
	_listeners: null,
	_destroyed: false,
	_trace: ibm.tivoli.navmodel.AbstractDataset._trace, 
	
	/*=====
	constructor_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	constructor_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	 constructor_kwArgs: function(onComplete, onError, baseUrl, provider, datasource, dataset, 
	 					parameters, defaultProperties, defaultLinkProperties, defaultRelProperties, 
	 					cacheThreshold, cacheValidationInterval, cacheExpirationTime, version) {
	 	// onComplete: constructor_onComplete
	 	//		callback to be invoked when the Dataset is initialized and ready for methods
	 	//		to be invoked  (i.e. asynchronous initialization must be performed). 
	 	//		Note: A null value will result in synchronous initialization.
	 	// onError: constructor_onError
	 	//		callback to be invoked if there is a problem constructing the Dataset
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// dataset: String
		//		dataset id
		// parameters: Object
		//		optional: extra parameters sent to dataset provider
		// defaultProperties: String[]
		//		optional: item property ids to fetch by default (can specify "all")
		// defaultLinkProperties: String[]
		//		optional: link property ids to fetch by default (can specify "all")
		// defaultRelProperties: String[]
		//		optional: relationship property ids to fetch by default (can specify "all")
		// cacheThreshold: int
		//		optional: cache objects if number of dataset items >= threshold
		// cacheValidationInterval: int
		//		optional: number of minutes in between cache validations
		// cacheExpirationTime: int
		//		optional: number of minutes objects can stay in cache while inactive
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	 },
	=====*/
	
	constructor: function(/*constructor_kwArgs*/ kwArgs, /*Object?*/ dataset) {
		// summary:
		//		Constructs AbstractDataset object.
		// description:
		//		The following options are normally not included in constructor (internal use only)
		//			createEventRouter: boolean - True indicates that a new event router should be created (vs. shared)
		//			dataset: object - Optional dataset object; will fetch from server if not provided
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		this._args = {
			onComplete: null,
			onError: null,
			baseUrl: "/ibm/tivoli/rest",
			provider: "",
			datasource: "",
			dataset: "",
			parameters: {},
			defaultProperties: [],
			defaultLinkProperties: [],
			defaultRelProperties: [],
			cacheThreshold:	250,
			cacheValidationInterval: 5,
			cacheExpirationTime: 15,
			version: ibm.tivoli.navmodel.Utils.getDefaultContentTypeVersion(),
			
			// The following options are normally not included in constructor
			createEventRouter: true
		};
		dojo.mixin(this._args, kwArgs);
		
		// init empty caches
		this._itemsById = {};
		this._linksById = {};
		this._relationshipsById = {};
	
		// add param_pageId to help build unique event router queue: <user>_<sessionId>_<pageId>
		this._args.parameters.pageId = ibm.tivoli.navmodel.EventRouter.getPageId();
		
		// add param_refId to make sure server dataset is unique
		if(typeof(this._args.parameters.refId) == "undefined"){ 
			this._args.parameters.refId = Math.random();
		}
		// empty refId supplied; so don't include in parameters
		else if (this._args.parameters.refId.length === 0) {
			delete this._args.parameters.refId;
		}

		// add param_tz for the user timezone
		if (!this._args.parameters.tz) {
			this._args.parameters.tz = ibm.tivoli.navmodel.Utils.getTimeZone();
		}
		
		// subclass can initialize the dataset
		var initArgs = {
			onComplete: this._args.onComplete,
			onError: this._args.onError,
			optimize: true,
			dataset: dataset,
			createEventRouter: this._args.createEventRouter
		};
		this._initDataset(initArgs);		
	},

	/////////////////////////////
	// public methods
	/////////////////////////////

	refresh: function() {
		// summary:
		//		This method forces the dataset object to do a full refresh of its 
		//		data and caches
		
		// clear caches, refresh, and notify listeners
		this.clientChangedEvent();
	},

	destroy: function() {
		// summary:
		//		This method should be called to clean up the Dataset object
		
		if (!this._destroyed) {
			// cancel the cache validation timer
			if (this._cacheIntervalId) {
				clearInterval(this._cacheIntervalId);
			}
			
			// do any clean up like unregistering from event router, etc
			this._cleanupDataset();
			
			// destroy widget definition if we have one
			if (this._widgetDefs) {
				var length = this._widgetDefs.length;
				for (var index = 0;  index < length;  ++index) {
					if (this._widgetDefs[index]) {
						this._widgetDefs[index].destroy();
					}
				}								
			}
			
			this._destroyed = true;
		}
	},
		
	getArgs: function() {
		// summary:
		//		Returns the initial arguments passed to the dataset
		return dojo.clone(this._args);
	},

	getArg: function(key) {
		// summary:
		//		Returns a specific initial argument passed to the dataset
		return this._args[key];
	},

	getAttribute: function(key) {
		// summary:
		//		Returns general dataset attribute for requested key
		return this._dataset[key];
	},

	getAttributes: function(keyPrefix) {
		// summary:
		//		Returns attribute map for all keys that begin with keyPrefix;  if no
		//		keyPrefix is supplied; all attributes are returned
		var attr = {};		
		if (keyPrefix) {
			for (var key in this._dataset) {
				if (this._dataset.hasOwnProperty(key)) {
					if (key.indexOf(keyPrefix) === 0) {
						attr[key] = this._dataset[key];
					}			
				}
			}
		}
		else {
			dojo.mixin(attr, this._dataset);
		}
		
		return attr;
	},

	setAttribute: function(key, value) {
		// summary:
		//		Set general dataset attribute for requested key
		this._dataset[key] = value;		
	},

	mixinAttributes: function(attr) {
		// summary:
		//		Mixin general dataset attributes
		dojo.mixin(this._dataset, attr);		
	},
	
	isClearCacheOnRefresh: function() {
		// summary:
		//		Returns true if the dataset is requesting that
		//		?clearCache be placed on url when doing a Refresh;
		//		which will cause CURI cache to purge the dataset and reconstruct		
		return (!this._dataset.eventsEnabled || this._dataset.clearCacheOnRefresh);
	},
	
	isEventsEnabled: function() {
		// summary:
		//		Returns true if the dataset is capable of producing events
		return this._dataset.eventsEnabled;
	},

	getEventListenerCount: function() {
		// summary:
		//		Returns number of registered event listeners
		var listenerCount = 0;
		if (this._listeners) {
			listenerCount = this._listeners.length;
		}
		
		return listenerCount;
	},
	
	/*=====
	_listenerCallback: function(listenerObj, event) {
		// listenerObj
		// event
	},
	=====*/

	addEventListener: function(/*Object*/ listenerObj, /*_listenerCallback*/ listenerCallback) {
		// summary:
		//		Adds an event listener to the Dataset
		// listenerObj:
		//		unique object (i.e. same object must be passed to removeEventListener)
		// listenerCallback
		//		listener callback function for receiving the events
		
		// remove listener if it already exists
		this.removeEventListener(listenerObj);
			
		// add new listener
		if (!this._listeners) {
			this._listeners = [];
		}		
		this._listeners.push({listenerObj: listenerObj, listenerCallback: listenerCallback});
	},
	
	removeEventListener: function(/*Object*/ listenerObj) {
		// summary:
		//		Removes an event listener from the Dataset
		// listernObj:
		//		previously registered listener object
		if (this._listeners) {
			var listenerLength = this._listeners.length;
			for (var listenerIndex = 0;  listenerIndex < listenerLength;  ++listenerIndex) {
				if (this._listeners[listenerIndex].listenerObj === listenerObj) {
					// remove listener
					this._listeners.splice(listenerIndex, 1);
					
					// last listener so null out array					
					if (this._listeners.length === 0) {
						this._listeners = null;
					}
					
					break;
				}
			}	
		}
	},	

	getVersion: function() {
		// summary:
		//		Returns the version used to communicate with rest service
		//		(i.e. json version)
		return this._version;
	},
	
	_setVersion: function(version) {
		// summary:
		//		Sets the version used to communicate with the rest servce
		//		(i.e. json version)
		this._version = version;
	},

	getId: function() {
		// summary:
		//		Returns the dataset id
		return this._dataset.id;
	},

	setId: function(id) {
		// summary:
		//		Set the dataset id
		this._dataset.id = id;
	},

	getProviderId: function() {
		// summary:
		//		Returns the provider id
		return this._dataset.providerId;
	},

	setProviderId: function(providerId) {
		// summary:
		//		Set the provider id
		this._dataset.providerId = providerId;
	},

	getDatasourceId: function() {
		// summary:
		//		Returns the datasource id
		return this._dataset.datasourceId;
	},
	
	setDatasourceId: function(datasourceId) {
		// summary:
		//		Set the datasource id
		this._dataset.datasourceId = datasourceId;
	},

	getEventId: function() {
		// summary:
		//		Returns the event id
		return this._dataset.eventId;
	},

	setEventId: function(eventId) {
		// summary:
		//		Set the event id
		this._dataset.eventId = eventId;
	},

	getType: function() {
		// summary:
		//		Returns the dataset type
		return this._dataset.type;
	},

	setType: function(type) {
		// summary:
		//		Set the dataset type
		this._dataset.type = type;		
	},

	getViewType: function() {
		// summary:
		//		Returns the preferred view type for rendering
		//		
		//		values: "graph", "table", "tree"
		return this._dataset.viewType;
	},

	setViewType: function(viewType) {
		// summary:
		//		Set the preferred view type for rendering
		this._dataset.viewType = viewType;		
	},
	
	isGraph: function() {
		// summary:
		//		Returns true if the dataset represents a graph
		return this.getViewType() === "graph";
	},

	isTable: function() {
		// summary:
		//		Returns true if the dataset represents a table
		return this.getViewType() === "table";
	},

	isTree: function() {
		// summary:
		//		Returns true if the dataset represents a tree
		return this.getViewType() === "tree";
	},

	isLazyLoad: function() {
		// summary:
		//		Returns true if the dataset designates that it should be lazy loaded
		return (this._dataset.lazyLoad !== undefined ? this._dataset.lazyLoad : false);
	},
	
	setLazyLoad: function(lazyLoad) {
		// summary:
		//		Set lazy load attribute for the dataset
		this._dataset.lazyLoad = lazyLoad;
	},
	
	getLazyLoadRelationships: function() {		
		// summary:
		//		Returns the relationships that should be explored for related items
		return this._dataset.relationshipsKey;
	},

	setLazyLoadRelationships: function(relationshipsKey) {		
		// summary:
		//		Set the relationships that should be explored for related items
		this._dataset.relationshipsKey = relationshipsKey;
	},
	
	getLabel: function() {	
		// summary:
		//		Returns globalized label for the dataset
		return this._dataset.label;
	},

	setLabel: function(label) {
		// summary:
		//		Set globalized label for the dataset
		this._dataset.label = label;		
	},

	getLabelQualifier: function() {
		// summary:
		//		Returns globalized label qualifier that can be used in conjunction with the label
		//		i.e.    label (labelQualifier)
		return this._dataset.labelQualifier;
	},

	setLabelQualifier: function(labelQualifier) {
		// summary:
		//		Set globalized label qualifier that can be used in conjunction with the label
		this._dataset.labelQualifier = labelQualifier;		
	},
	
	/*=====
	depth_kwArgs1: function(widgetDef) {
		// widgetDef: Object
		//		widget definition can override the depth value
	},
	=====*/

	getDepth: function(/*depth_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the default depth for the dataset if it is a graph
		//		(otherwise -1)
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.widgetDef) {
			return args.widgetDef.getDepth();
		}
				
		return (this._dataset.depth ? this._dataset.depth : -1);		
	},

	setDepth: function(depth) {
		// summary:
		//		Set the depth for the dataset if it is a graph
		// depth:
		//		New topology depth
		
		// update depth parameter
		var found = false;
		for (var parm in this._args.parameters) {
			if (parm === "depth" && this._args.parameters.hasOwnProperty(parm)) {
				this._args.parameters[parm] = depth;
				found = true;
				break;
			}
		}
		if (!found) {
			this._args.parameters.depth = depth;
		}
		
		// clear caches, refresh, and notify listeners
		var event = ibm.tivoli.navmodel.ClientEvent.getChangedEvent(this);
		this._eventHandler(event);		
	},

	getMaxDepth: function(/*depth_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the maximum depth for the dataset if it is a graph
		var args = {
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		if (args.widgetDef) {
			return args.widgetDef.getMaxDepth();
		}
		
		return (this._dataset.maxDepth ? this._dataset.maxDepth : -1);		
	},

	getDefaultProperties: function() {
		// summary:
		//		Returns default properties that are fetched with each item
		return this._args.defaultProperties;
	},
	
	setDefaultProperties: function(itemProperties) {
		// summary:
		//		Sets default properties that are fetched with each item
		// itemProperties:
		//		Array of item property ids
		this._args.defaultProperties = itemProperties;
	},

	getDefaultLinkProperties: function() {
		// summary:
		//		Returns default properties that are fetched with each link
		return this._args.defaultLinkProperties;
	},
	
	setDefaultLinkProperties: function(linkProperties) {
		// summary:
		//		Sets default properties that are fetched with each link
		// linkProperties:
		//		Array of link properties
		this._args.defaultLinkProperties = linkProperties;
	},

	getDefaultRelProperties: function() {
		// summary:
		//		Returns default properties that are fetched with each relationship
		return this._args.defaultRelProperties;
	},
	
	setDefaultRelProperties: function(relProperties) {
		// summary:
		//		Sets default properties that are fetched with each relationship
		// relProperties:
		//		Array of relationship properties
		this._args.defaultRelProperties = relProperties;
	},
	
	cacheExpires: function() {
		// summary:
		//		Returns true if the items, links, and relationships cache expires for this dataset.  If
		//		there are only a small number of objects the dataset will not bother with expiration.
		return (this._cacheIntervalId ? true : false);
	},

	getNewExpirationTime: function() {
		// summary:
		//		Returns an expiration time for a new item, link, or relationship
		var now = new Date();
		return now.getTime() + (this._args.cacheExpirationTime*60000);
	},
	
	getItemCount: function() {
		// summary:
		//		Returns the number of items associated with the dataset
		return this._dataset.itemCount;
	},
	
	/*=====
	itemsAndLinks_onComplete: function(items, links, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	getItemsAndLinks_kwArgs: function(onComplete, onError, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItemsAndLinks: function(/*getItemsAndLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items and links associated with the dataset
		var args = {                
			onComplete: function(items, links, kwArgs) {},
			onError: function(error, exception, ioArgs) {},
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		// fetch items and links from rest service
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, links, toolbar, args) {
			// cache items and links
			items = this._cacheItems(items);
			links = this._cacheLinks(links);
				
			// cache toolbar tasks				
			this._toolbar = toolbar;
				
			// call orignal callback
			onComplete(items, links, args);
		});			
		this._fetchItemsAndLinks(args);		
	},
	
	/*=====
	items_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	getItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions 
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItems: function(/*getItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset
		var args = {                
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			properties: this._args.defaultProperties,
			metadata: false, 
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		///////////////////////////////////////////////////////////
		// make sure we always return asynchronously in case we are 
		// being invoked by a dojo datastore
		///////////////////////////////////////////////////////////
		
		// are NOT any items
		if (this.getItemCount() === 0) {		
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete(null, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
			}			
		}
		// fetch items from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(items, args) {
				// cache items
				items = this._cacheItems(items);
				
				// call orignal callback
				onComplete(items, args);
			});
			this._fetchItems(args);
		}		
	},
	
	hasCachedItems: function() {
		// summary:
		//		indicates if dataset currently contains cached items			
		if (this._itemsById) {
			for (var id in this._itemsById) { 
    			if (this._itemsById.hasOwnProperty(id)) {
    				return true;
    			}
    		}
		}
		
		return false;
	},
	
	getCachedItems: function() {
		// summary:
		//		returns array with currently cached items	
		var items = [];
		this._traverseCachedItems(items);
		return items;
	},
	
	_traverseCachedItems: function(/*ibm.tivoli.navmodel.Item[]*/ items) {
		if (this._itemsById) {
			for (var id in this._itemsById) { 
    			if (this._itemsById.hasOwnProperty(id)) {
    				items.push(this._itemsById[id]);
    			}
    		}
		}
	},
	
	/*=====
	getItem_kwArgs: function(itemId, properties, metadata) {
		// itemId: String
		//		item id to get
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/

	getItem: function(/*getItem_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the requested item
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var args = {
			itemId: null,
			properties: null,
			metadata: false
		};
		dojo.mixin(args, kwArgs);
		
		// check cache
		var item = this._getCachedItem(args.itemId);
		
		// force a fetch if we do not have all the requested properties		
		if (item && ((args.metadata && !item.hasCachedMetadata()) || (args.properties && !item.hasCachedProperties(args.properties)))) {
			item = null;			
		}
		
		// need to fetch individual item
		if (!item) {
			if (!args.properties || args.properties.length === 0) {
				args.properties = this._args.defaultProperties;
			}
			
			item = this._fetchItem(args);
			item = this._cacheItem(item);			
		}	

		return item;
	},
	
	/*=====
	getOriginItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metdata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getOriginItems: function(/*getOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
		var args = {                
			onComplete: function(items, links, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			properties: this._args.defaultProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, args) {
			// cache items
			items = this._cacheItems(items, true);
				
			// call orignal callback
			onComplete(items, args);
		});
		this._fetchOriginItems(args);
	},
	
	/*=====
	getRelatedItems_kwArgs: function(onComplete, onError, itemId, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of the base item
		// relationships: String
		//		comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested item
		var args = {                
			onComplete: function(items, links, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			itemId: null,
			relationships: null,
			start: 0,
			count: null,                
			properties: this._args.defaultProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, links, args) {
			// cache items and links
			items = this._cacheItems(items, true);
			links = this._cacheLinks(links);			
				
			// if no related items were returned; reset lazyLoad indicators
			if (!items || items.length === 0) {
				var item = this._getCachedItem(kwArgs.itemId);
				if (item) {
					item.resetLazyLoad();
				}
			}
			
			// call orignal callback
			onComplete(items, links, args);
		});		
		this._fetchRelatedItems(args);
	},

	/*=====
	queryItems_kwArgs: function(onComplete, onError, query, start, count, properties, metadata, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// query: Object
		//		custom query object to be sent as request body
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
		var args = {                
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			query: {},                
			start: 0,
			count: null,
			properties: this._args.defaultProperties,
			metadata: false,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         
		
		var onComplete = args.onComplete;		
		args.onComplete = dojo.hitch(this, function(items, args) {
			// cache items
			items = this._cacheItems(items);
						
			// call orignal callback
			onComplete(items, args);
		});		
		this._queryItems(args);	
	},

	getLinkCount: function() {
		// summary:
		//		Returns the number of links associated with the dataset
		return this._dataset.linkCount;
	},

	/*=====
	links_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	getLinks_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of links to fetch; null means all
		// properties: String[]
		//		link property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	getLinks: function(/*getLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns links associated with the dataset
		var args = {                
			onComplete: function(links, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			properties: this._args.defaultLinkProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         

		// are NOT any links
		if (this.getLinkCount() === 0) {
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete(null, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
			}
		}
		// fetch links from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(links, args) {
				// cache links
				links = this._cacheLinks(links);
				
				// call orignal callback
				onComplete(links, args);
			});
			this._fetchLinks(args);
		}
	},
	
	hasCachedLinks: function() {
		// summary:
		//		indicates if dataset currently contains cached links			
		if (this._linksById) {
			for (var id in this._linksById) { 
    			if (this._linksById.hasOwnProperty(id)) {
    				return true;
    			}
    		}
		}
		
		return false;
	},
	
	getCachedLinks: function() {
		// summary:
		//		returns array with currently cached links		
		var links = [];
		this._traverseCachedLinks(links);		
		return links;
	},
	
	_traverseCachedLinks: function(/*ibm.tivoli.navmodel.Link[]*/ links) {
		if (this._linksById) {
			for (var id in this._linksById) { 
    			if (this._linksById.hasOwnProperty(id)) {
    				links.push(this._linksById[id]);
    			}
    		}
		}
	},

	/*=====
	getLink_kwArgs: function(itemId, properties, metadata) {
		// linkId: String
		//		link id to get
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	getLink: function(/*getLink_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the requested link
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var args = {
			linkId: null,
			properties: null,
			metadata: false
		};
		dojo.mixin(args, kwArgs);
		
		// check cache
		var link = this._getCachedLink(args.linkId);
		
		// force a fetch if we do not have all the requested properties		
		if (link && ((args.metadata && !link.hasCachedMetadata()) || (args.properties && !link.hasCachedProperties(args.properties)))) {
			link = null;			
		}
			
		// need to fetch individual link
		if (!link) {
			if (!args.properties || args.properties.length === 0) {
				args.properties = this._args.defaultLinkProperties;
			}
			
			link = this._fetchLink(args);
			link = this._cacheLink(link);			
		}	
		
		return link;
	},

	getRelationshipCount: function() {
		// summary:
		//		Returns the number of items associated with the dataset
		return this._dataset.relationshipCount;
	},
	
	/*=====
	relationships_onComplete: function(relationships, kwArgs){},
	=====*/
	
	/*=====
	getRelationships_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationships to fetch; null means all
		// properties: String[]
		//		relationship property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelationships: function(/*getRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Returns relationships associated with the dataset
		var args = {                
			onComplete: function(relationships, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			start: 0,
			count: null,
			properties: this._args.defaultRelProperties,
			metadata: false,
			sort: null,
			condition: null,
			sync: false
		};   
		dojo.mixin(args, kwArgs);         

		///////////////////////////////////////////////////////////
		// make sure we always return asynchronously in case we are 
		// being invoked by a dojo datastore
		///////////////////////////////////////////////////////////
		
		// are NOT any links/relationships
		if (this.getLinkCount() === 0) {
			args.numRows = 0;
			args.filteredRows = 0;
			args.totalRows = 0;				
			if (args.sync) {
				args.onComplete(null, args);
			}
			else {		
				setTimeout(dojo.hitch(this, args.onComplete, null, args), 0);
			}
		}
		// fetch relationships from rest service
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(relationships, args) {
				// cache relationships
				relationships = this._cacheRelationships(relationships);
				
				// call orignal callback
				onComplete(relationships, args);
			});
			this._fetchRelationships(args);
		}
	},
	
	hasCachedRelationships: function() {
		// summary:
		//		indicates if dataset currently contains cached relationships
		if (this._relationshipsById) {
			for (var id in this._relationshipsById) { 
    			if (this._relationshipsById.hasOwnProperty(id)) {
    				return true;
    			}
    		}
		}
		
		return false;
	},
	
	getCachedRelationships: function() {
		// summary:
		//		return array with currently cached relationships	
		var relationships = [];
		this._traverseCachedRelationships(relationships);
		return relationships;
	},
	
	_traverseCachedRelationships: function(/*ibm.tivoli.navmodel.Relationship[]*/ relationships) {
		if (this._relationshipsById) {
			for (var id in this._relationshipsById) { 
    			if (this._relationshipsById.hasOwnProperty(id)) {
    				relationships.push(this._relationshipsById[id]);
    			}
    		}
		}
	},
	
	/*=====
	getRelationship_kwArgs: function(relationshipId, properties, metadata) {
		// relationshipId: String
		//		relationship id to fetch
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	getRelationship: function(/*getRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Returns the requested relationship
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var args = {
			relationshipId: null,
			properties: null,
			metadata: false
		};
		dojo.mixin(args, kwArgs);
		
		// check cache
		var relationship = this._getCachedRelationship(args.relationshipId);

		// force a fetch if we do not have all the requested properties		
		if (relationship && ((args.metadata && !relationship.hasCachedMetadata()) || (args.properties && !relationship.hasCachedProperties(args.properties)))) {
			relationship = null;			
		}
						
		// need to fetch individual relationship
		if (!relationship) {
			if (!args.properties || args.properties.length === 0) {
				args.properties = this._args.defaultRelProperties;
			}
			
			relationship = this._fetchRelationship(args);
			relationship = this._cacheRelationship(relationship);
		}	
		
		return relationship;
	},
	
	/*=====
	tasks_onComplete: function(tasks, kwArgs){},
	=====*/
	
	/*=====
	getTasks_kwArgs: function(onComplete, onError, taskCategory, selection, itemIds, linkIds, itemIndexes, sync) {
		// onComplete: tasks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskCategory: String
		//		comma delimited list indicating type of tasks to fetch
		//		legal values are: menu, toolbar, and breadcrumb
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds, String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getTasks: function(/*getTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
		var args = {
			onComplete: function(tasks, kwArgs){
			},
			onError: function(error, exception, ioArgs){				
			},
			taskCategory: "menu",
			selection: null,
			itemIds: null,
			linkIds: null,
			itemIndexes: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// asking for "toolbar" without any selection and we previously
		// fetched the toolbar tasks with the items and links
		if (args.taskCategory === "toolbar" && this._toolbar &&
			(!args.selection || args.selection.isEmpty()) && !args.itemIds && !args.linkIds && !args.itemIndexes) {
			if (args.sync) {
				args.onComplete(this._toolbar, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._toolbar, args), 0);
			}		
		}		
		// fetch tasks from rest service
		else {
			this._fetchTasks(args);
		}
	},

	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, taskId, taskCategory, selection, sync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskId: String
		//		navigation task id
		// taskCategory: String
		//		navigation task category
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
		var args = {                
			onComplete: function(dataset, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},                
			taskId: null,
			taskCategory: "menu:navigation",
			selection: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		this._navigate(args);     
	},

	/*=====
	itemStyles_onComplete: function(itemStyles, kwArgs){},
	=====*/
	
	/*=====
	getItemStyles_kwArgs: function(onComplete, onError, start, count, widgetDef, sync) {
		// onComplete: itemStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of item styles to fetch;  null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getItemStyles: function(/*getItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
		var args = {
			onComplete: function(itemStyles, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// already have the item styles
		if (this._itemStyles) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeItemStyles(this._itemStyles, args.widgetDef), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeItemStyles(this._itemStyles, args.widgetDef), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(this._itemStyles, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._itemStyles, args), 0);
			}			
		}
		// need to fetch the item styles
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(itemStyles, args) {
				// cache styles
				if (args.start === 0 && args.count === null) {
					this._cacheItemStyles(itemStyles);					
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeItemStyles(itemStyles, args.widgetDef), args);
				}
				else {
					onComplete(itemStyles, args);
				}				
			});			
			this._fetchItemStyles(args);
		}
	},
	
	/*=====
	getItemStyle_kwArgs: function(itemType, widgetDef) {
		// itemType: String
		//		type of itemStyle
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/

	getItemStyle: function(/*getItemStyle_kwArgs*/ kwArgs) {
		// summary:
		//		Returns item style for requested itemType
		var itemStyle = null;
		var args = {
			itemType: "",
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if we don't have item styles fetch them
		if (!this._itemStyles) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(itemStyles, args){
					this._cacheItemStyles(itemStyles);					
				}),
				onError: function(error, exception, ioArgs){
				},
				start: 0,
				count: null,
				sync: true
			};
			this._fetchItemStyles(fetchArgs);
		}
		
		// locate requested item style in cache
		if (this._itemStylesById) {
			itemStyle = this._itemStylesById[args.itemType];
		}
		
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (itemStyle) {
				itemStyle = itemStyle.clone(args.widgetDef);					
			}
			// unknown style to dataset; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getItemStyle(args.itemType);
				if (widgetStyle) {
					itemStyle = this._createItemStyle(args.widgetDef, widgetStyle);
				}					
			}
		}
		
		return itemStyle;
	},
	
	/*=====
	linkStyles_onComplete: function(linkStyles, kwArgs){},
	=====*/
	
	/*=====
	getLinkStyles_kwArgs: function(onComplete, onError, start, count, widgetDef, sync) {
		// onComplete: linkStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of link styles to fetch; null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getLinkStyles: function(/*getLinkStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current link styles for the Dataset
		var args = {
			onComplete: function(linkStyles, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// already have the link styles
		if (this._linkStyles) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeLinkStyles(this._linkStyles, args.widgetDef, "link"), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeLinkStyles(this._linkStyles, args.widgetDef, "link"), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(this._linkStyles, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._linkStyles, args), 0);
			}
		}
		// need to fetch the link styles
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(linkStyles, args) {
				// cache styles
				if (args.start === 0 && args.count === null) {
					this._cacheLinkStyles(linkStyles);					
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeLinkStyles(linkStyles, args.widgetDef, "link"), args);
				}
				else {
					onComplete(linkStyles, args);
				}				
			});
			this._fetchLinkStyles(args);
		}
	},
	
	/*=====
	getLinkStyle_kwArgs: function(linkType, widgetDef) {
		// linkType: String
		//		type of linkStyle
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/
	
	getLinkStyle: function(/*getLinkStyle_kwArgs*/ kwArgs) {
		// summary:
		//		Returns link style for requested linkType
		var linkStyle = null;
		var args = {
			linkType: "",
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if we don't have link styles fetch them
		if (!this._linkStyles) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(linkStyles, args){
					this._cacheLinkStyles(linkStyles);
				}),
				onError: function(error, exception, ioArgs){
				},
				sync: true
			};
			this._fetchLinkStyles(fetchArgs);
		}
		
		// locate requested link style in cache
		if (this._linkStylesById) {
			linkStyle = this._linkStylesById[args.linkType];
		}
		
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (linkStyle) {
				linkStyle = linkStyle.clone(args.widgetDef, "link");					
			}
			// unknown style to dataset; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getLinkStyle(args.linkType);
				if (widgetStyle) {
					linkStyle = this._createLinkStyle(args.widgetDef, widgetStyle, "link");
				}
			}
		}
				
		return linkStyle;
	},
	
	/*=====
	relStyles_onComplete: function(relStyles, kwArgs){},
	=====*/
	
	/*=====
	getRelationshipStyles_kwArgs: function(onComplete, onError, start, count, widgetDef, sync) {
		// onComplete: relStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationship styles to fetch; null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getRelationshipStyles: function(/*getRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
		var args = {
			onComplete: function(relStyles, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		// already have the relationship styles
		if (this._relStyles) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeLinkStyles(this._relStyles, args.widgetDef, "relationship"), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeLinkStyles(this._relStyles, args.widgetDef, "relationship"), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(this._relStyles, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, this._relStyles, args), 0);
			}
		}
		// need to fetch the relationship styles
		else {
			var onComplete = args.onComplete;		
			args.onComplete = dojo.hitch(this, function(relStyles, args) {
				// cache styles
				if (args.start === 0 && args.count === null) {
					this._cacheRelationshipStyles(relStyles);					
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeLinkStyles(relStyles, args.widgetDef, "relationship"), args);
				}
				else {
					onComplete(relStyles, args);
				}				
			});
			this._fetchRelationshipStyles(args);
		}
	},
	
	/*=====
	getRelationshipStyle_kwArgs: function(relationshipType, widgetDef) {
		// relationshipType: String
		//		type of relationshipStyle
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/

	getRelationshipStyle: function(/*getRelationshipStyle_kwArgs*/ kwArgs) {
		// summary:
		//		Returns relationship style for requested relationshipType
		var relStyle = null;
		var args = {
			relationshipType: "",
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		// if we don't have relationship styles fetch them
		if (!this._relStyles) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(relStyles, args){
					this._cacheRelationshipStyles(relStyles);
				}),
				onError: function(error, exception, ioArgs){
				},
				sync: true
			};
			this._fetchRelationshipStyles(fetchArgs);
		}
		
		// locate requested relationship style in cache
		if (this._relStylesById) {
			relStyle = this._relStylesById[args.relationshipType];
		}
		
		if (args.widgetDef) {
			// clone and merge widget definition attributes				
			if (relStyle) {
				relStyle = relStyle.clone(args.widgetDef, "relationship");					
			}
			// unknown style to dataset; see if it exists in widget definition
			else {
				var widgetStyle = args.widgetDef.getRelationshipStyle(args.relationshipType);
				if (widgetStyle) {
					relStyle = this._createLinkStyle(args.widgetDef, widgetStyle, "relationship");
				}
			}
		}	
		
		return relStyle;
	},
		
	/*=====
	statuses_onComplete: function(statuses, kwArgs){},
	=====*/
	
	/*=====
	getStatuses_kwArgs: function(onComplete, onError, type, start, count, widgetDef, sync) {
		// onComplete: statuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// widgetDef: Object
		//		widget definition that can override attributes
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	getStatuses: function(/*getStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible status values for the object type
		var args = {
			onComplete: function(statuses, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			type: "item",
			start: 0,
			count: null,
			widgetDef: null,
			sync: false
		};
		dojo.mixin(args, kwArgs);
		
		var statuses = null;
		var statusesById = null;
		switch(args.type) {
			case "item":
				statuses = this._itemStatus;
				statusesById = this._itemStatusById;
				break;
			case "link":
				statuses = this._linkStatus;
				statusesById = this._linkStatusById;
				break;
			case "relationship":
				statuses = this._relStatus;
				statusesById = this._relStatusById;			
		}
		
		// already have the status values
		if (statuses) {
			if (args.widgetDef) {
				if (args.sync) {
					args.onComplete(this._mergeStatuses(statuses, args.widgetDef, args.type), args);
				}
				else {
					setTimeout(dojo.hitch(this, args.onComplete, this._mergeStatuses(statuses, args.widgetDef, args.type), args), 0);
				}
			}
			else if (args.sync) {
				args.onComplete(statuses, args);
			}
			else {
				setTimeout(dojo.hitch(this, args.onComplete, statuses, args), 0);
			}			
		}
		// need to fetch the status values
		else {
			var onComplete = args.onComplete;
			args.onComplete = dojo.hitch(this, function(fetchedStatuses, args) {
				// cache statuses
				if (args.start === 0 && args.count === null) {
					switch(args.type) {
						case "item":
							this._cacheItemStatuses(fetchedStatuses);
							break;
					case "link":
							this._cacheLinkStatuses(fetchedStatuses);
							break;
					case "relationship":
							this._cacheRelationshipStatuses(fetchedStatuses);
					}
				}
				
				// call orignal callback
				if (args.widgetDef) {
					onComplete(this._mergeStatuses(fetchedStatuses, args.widgetDef, args.type), args);
				}
				else {
					onComplete(fetchedStatuses, args);
				}
			});
			this._fetchStatuses(args);
		}
	},
	
	/*=====
	getStatus_kwArgs: function(type, statusId, statusValue, widgetDef) {
		// type: String
		//		object type: "item", "link", "relationship" 
		// statusId: String
		//		id of the status
		// statusValue: int
		//		value of the status
		// widgetDef: Object
		//		widget definition that can override attributes
	},
	=====*/

	getStatus: function(/*getStatus_kwArgs*/ kwArgs) {
		// summary:
		//		Returns status for requested status id and value
		var status = null;
		var args = {
			type: "item",
			statusId: "",
			statusValue: 0,
			widgetDef: null
		};
		dojo.mixin(args, kwArgs);
		
		var statuses = null;
		switch(args.type) {
			case "item":
				statuses = this._itemStatus;
				break;
			case "link":
				statuses = this._linkStatus;
				break;
			case "relationship":
				statuses = this._relStatus;
		}
		
		// if we don't have statuses fetch them
		if (!statuses) {
			var fetchArgs = {
				onComplete: dojo.hitch(this, function(fetchedStatuses, args){
					switch(args.type) {
						case "item":
							this._cacheItemStatuses(fetchedStatuses);
							break;
						case "link":
							this._cacheLinkStatuses(fetchedStatuses);
							break;
						case "relationship":
							this._cacheRelationshipStatuses(fetchedStatuses);
					}					
				}),
				onError: function(error, exception, ioArgs){
				},
				type: args.type,
				start: 0,
				count: null,
				sync: true
			};
			this._fetchStatuses(fetchArgs);
		}
		
		var statusesById = null;
		switch(args.type) {
			case "item":
				statusesById = this._itemStatusById;
				break;
			case "link":
				statusesById = this._linkStatusById;
				break;
			case "relationship":
				statusesById = this._relStatusById;			
		}
		
		// locate requested item status in cache
		if (statusesById) {
			var id = args.statusId + "_" + args.statusValue;
			status = statusesById[id];
		}
		
		if (status && args.widgetDef) {
			status = status.clone(widgetDef, args.type);			
		}
		
		return status;
	},
	
	/*=====
	cols_onComplete: function(cols, kwArgs){},
	=====*/
	
	/*=====
	getColumns_kwArgs: function(onComplete, onError, type, properties, start, count, metadata, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of column ids; or keyword "current", "default", or "all"
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// metadata: Boolean
		//		indicates if filter metadata should be returned with columns
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	getColumns: function(/*getColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current columns for the Dataset
		//		(i.e. the ones that should currently be dispayed)
		var args = {
			onComplete: function(cols, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			source: "dataset",
			type: "item",
			properties: ["current"],
			start: 0,
			count: null,
			metadata: true,
			sync: false
		};
		dojo.mixin(args, kwArgs);
				
		this._fetchColumns(args);	
	},
	
	/*=====
	setColumns_kwArgs: function(onComplete, onError, columns, type, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// columns: ibm.tivoli.navmodel.Column[]
		//		array of columns to be saved
		// type: String
		//		object type: "item", "link", "relationship" 
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Set list of current columns for the Dataset
		//		(with updated attributes for width, sort, etc)
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			columns: null,
			type: "item",
			sync: false
		};
		dojo.mixin(args, kwArgs);
				
		this._setColumns(args);	
	},
	
	hasWidgetDefs: function() {
		// summary:
		//		Returns true if the dataset defines widget definitions	
		return (this._dataset.widgetDefinitions && dojo.isArray(this._dataset.widgetDefinitions) && this._dataset.widgetDefinitions.length > 0);
	},
	
	/*=====
	getWidgetDef_kwArgs: function() {
		// widgetType: String
		//		the type of widget def (i.e. "navwidget", "topology", "table", etc)
	},
	=====*/
	
	getWidgetDef: function(/*getWidgetDef_kwArgs*/ kwArgs) {
		// summary:
		//		Returns widget definition for the requested widget type
		var args = {
			widgetType: null
		};
		dojo.mixin(args, kwArgs);
		
		return this._getWidgetDef(this._getWidgetDefIndex(args.widgetType));		
	},
	
	getWidgetDefs: function() {
		// summary:
		//		Returns all widget definitions defined by the dataset
		var widgetDefs = null;		
		if (this.hasWidgetDefs()) {
			widgetDefs = [];			
			var length = this._dataset.widgetDefinitions.length;
			for (var index = 0;  index < length;  ++index) {
				var widgetDef = this._getWidgetDef(index);
				if (widgetDef) {
					widgetDefs.push(widgetDef);
				}
			}
		}
		
		return widgetDefs;
	},
	
	/*=====
	item_onComplete: function(item, kwArgs){},
	=====*/
	
	/*=====
	createItem_kwArgs: function(onComplete, onError, item) {
		// onComplete: item_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object
		//		ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/

	createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new item in the dataset
		var args = {
			onComplete: function(item, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			item: null
		};
		dojo.mixin(args, kwArgs);
		
		this._createItem(args);
	},
	
	/*=====
	createItems_kwArgs: function(onComplete, onError, item) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object[]
		//		array of ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/

	createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
		var args = {
			onComplete: function(items, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			items: []
		};
		dojo.mixin(args, kwArgs);
		
		this._createItems(args);
	},
	
	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	deleteItem_kwArgs: function(onComplete, onError, itemId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of item to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of item
	},
	=====*/

	deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			itemId: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteItem(args);
	},
	
	/*=====
	deleteItems_kwArgs: function(onComplete, onError, itemIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemIds: String[]
		//		array of item ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of items
	},
	=====*/

	deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			itemIds: [],
			condition: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteItems(args);
	},
	
	/*=====
	link_onComplete: function(link, kwArgs){},
	=====*/
	
	/*=====
	createLink_kwArgs: function(onComplete, onError, link) {
		// onComplete: link_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// link: Object
		//		ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/

	createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
		var args = {
			onComplete: function(link, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			link: null
		};
		dojo.mixin(args, kwArgs);
		
		this._createLink(args);	
	},
	
	/*=====
	createLinks_kwArgs: function(onComplete, onError, links) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// links: Object[]
		//		array of ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/

	createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
		var args = {
			onComplete: function(links, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			links: []
		};
		dojo.mixin(args, kwArgs);
		
		this._createLinks(args);	
	},
	
	/*=====
	deleteLink_kwArgs: function(onComplete, onError, linkId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkId: String
		//		id of link to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of link
	},
	=====*/

	deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			linkId: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteLink(args);
	},
	
	/*=====
	deleteLinks_kwArgs: function(onComplete, onError, linkIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkIds: String[]
		//		array of link ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of links
	},
	=====*/

	deleteLinks: function(/*deleteLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			linkIds: [],
			condition: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteLinks(args);
	},
	
	/*=====
	relationship_onComplete: function(relationship, kwArgs){},
	=====*/
	
	/*=====
	createRelationship_kwArgs: function(onComplete, onError, relationship) {
		// onComplete: relationship_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object
		//		ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
		var args = {
			onComplete: function(relationship, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relationship: null
		};
		dojo.mixin(args, kwArgs);
		
		this._createRelationship(args);	
	},
	
	/*=====
	createRelationships_kwArgs: function(onComplete, onError, relationships) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object[]
		//		array of ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	createRelationships: function(/*createRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
		var args = {
			onComplete: function(relationships, kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relationships: []
		};
		dojo.mixin(args, kwArgs);
		
		this._createRelationships(args);	
	},
	
	/*=====
	deleteRelationship_kwArgs: function(onComplete, onError, relationshipId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipId: String
		//		id of relationship to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationship
	},
	=====*/

	deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relatonshipId: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteRelationship(args);
	},
	
	/*=====
	deleteRelationships_kwArgs: function(onComplete, onError, relationshipIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipIds: String[]
		//		array of relationship ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) and (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationships
	},
	=====*/

	deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
		var args = {
			onComplete: function(kwArgs){
			},
			onError: function(error, exception, ioArgs){
			},
			relatonshipIds: [],
			condition: null,
			remove: false
		};
		dojo.mixin(args, kwArgs);
		
		this._deleteRelationships(args);
	},

	clientChangedEvent: function() {
		// summary:
		//		Fire client side "changed" event
		var event = ibm.tivoli.navmodel.ClientEvent.getChangedEvent(this);
		this._eventHandler(event);	
	},

	clientItemsAddedEvent: function(items) {
		// summary:
		//		Fire client side "itemsAdded" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemsAddedEvent(this, items);
		this._eventHandler(event);
	},

	clientItemsRemovedEvent: function(items) {
		// summary:
		//		Fire client side "itemsRemoved" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemsRemovedEvent(this, items);
		this._eventHandler(event);
	},

	clientItemStatusChangedEvent: function(items) {
		// summary:
		//		Fire client side "itemStatusChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemStatusChangedEvent(this, items);
		this._eventHandler(event);	
	},

	clientItemPropertiesChangedEvent: function(items) {
		// summary:
		//		Fire client side "itemPropertiesChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getItemPropertiesChangedEvent(this, items);
		this._eventHandler(event);	
	},

	clientLinksAddedEvent: function(links) {
		// summary:
		//		Fire client side "linksAdded" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinksAddedEvent(this, items);
		this._eventHandler(event);
	},

	clientLinksRemovedEvent: function(links) {
		// summary:
		//		Fire client side "linksRemoved" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinksRemovedEvent(this, links);
		this._eventHandler(event);
	},

	clientLinkStatusChangedEvent: function(links) {
		// summary:
		//		Fire client side "linkStatusChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinkStatusChangedEvent(this, links);
		this._eventHandler(event);
	},

	clientLinkPropertiesChangedEvent: function(links) {
		// summary:
		//		Fire client side "linkPropertiesChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinkPropertiesChangedEvent(this, links);
		this._eventHandler(event);
	},

	clientLabelChangedEvent: function(newLabel) {
		// summary:
		//		Fire client side "labelChanged" event
		var event = ibm.tivoli.navmodel.ClientEvent.getLinkPropertiesChangedEvent(this, newLabel);
		this._eventHandler(event);	
	},
		
	/////////////////////////////
	// protected methods
	/////////////////////////////	
	_initEmptyDataset: function() {
		// summary:
		//		Reset internal dataset to an empty state
		this._version = this._args.version;		
		this._dataset = {};
		this._dataset.providerId = this._args.provider;
		this._dataset.datasourceId = this._args.datasource;
		this._dataset.id = this._args.dataset;
		this._dataset.eventId = this._dataset.id;
		this._dataset.eventsEnabled = false;
		this._dataset.viewType = "graph";		
		this._dataset.itemCount = 0;
		this._dataset.linkCount = 0;
		this._dataset.relationshipCount = 0;
		
		// establish our base dataset uri
		this._dataset.uri  = "/providers/" + encodeURIComponent(this._dataset.providerId);
		this._dataset.uri += (this._dataset.datasourceId !== "" ? ("/datasources/" + encodeURIComponent(this._dataset.datasourceId)) : "");
		this._dataset.uri += "/datasets/" + encodeURIComponent(this._dataset.id);		
	},

	_cleanupDataset: function() {
		// summary:
		//		Clean up by removing listener and attempting a deleteDataset if no other listeners
		
		// unregister from event router
		if (this._eventRouter) {
			var remainingListeners = this._eventRouter.removeListener(this.getEventId());
				
			// ok to delete dataset from REST service cache
			if (remainingListeners === 0) {
				this._deleteDataset();
			}
			
			this._eventRouter = null;
		}
		// no event router;  just do the delete
		else {
			this._deleteDataset();
		}
		
		// clear all item and link caches
		this._clearAllCaches();		
		this._initEmptyDataset();
	},

	_getWidgetDefIndex: function(/*String*/ widgetType) {
		// summary:
		//		Returns the index into widgetDefinitions array for the requested widgetType		
		if (widgetType && this.hasWidgetDefs()) {
			var length = this._dataset.widgetDefinitions.length;
			for (var index = 0;  index < length;  ++index) {
				var entry = this._dataset.widgetDefinitions[index];
				if (entry) {
					// entry should be in the form of  <widget type>:<widget id>:<widget class>
					var parts = entry.split(":");
					if (parts && parts.length == 3 && parts[0] === widgetType) {
						return index;
					}
				}
			}
		}
		
		return -1;
	},
	
	_getWidgetDef: function(/*int*/ index) {
		// summary:
		//		Return WidgetDef for the requested index
		if (!this._widgetDefs && this.hasWidgetDefs()) {
			this._widgetDefs = [];			
			var widgetDefsLength = this._dataset.widgetDefinitions.length;
			for (var widgetDefIndex = 0;  widgetDefIndex < widgetDefsLength;  ++widgetDefIndex) {
				this._widgetDefs[widgetDefIndex] = null;
			}
		}
		
		if (!this._widgetDefs || index < 0 || index >= this._widgetDefs.length) {
			return null;
		}
		
		if (!this._widgetDefs[index]) {
			var parts = this._dataset.widgetDefinitions[index].split(":");
			var widgetType = parts[0];
			var widgetId = parts[1];
			var widgetClass = parts[2];
				
			var objectClass = dojo.getObject(widgetClass);
			if (objectClass) {
				var args = {
					baseUrl: this._args.baseUrl,
					provider: this.getProviderId(),
					datasource: this.getDatasourceId(),
					widget: widgetType,
					definition: widgetId,
					type: "user",
					version: this.getVersion()
				};					
					
				try {
					this._widgetDefs[index] = new objectClass(args);
				}
				catch(ex) {
					this._trace.error("Exception constructing widget definition: ex=" + ex, "_getWidgetDef");					
				}
			}
			else {
				this._trace.error("Cannot instantiate widget def class=" + widgetClass, "_initWidgetDef");				
			}
		}
			
		return this._widgetDefs[index];		
	},

	_setEventRouter: function(eventRouter) {
		// summary:
		//		Associate our dataset with an existing EventRouter object
		if (eventRouter && this.isEventsEnabled()) {
			this._eventRouter = eventRouter;
			this._eventRouter.addListener(this.getEventId(), dojo.hitch(this, this._jsonEventHandler));
		}
	},

	_clearEventRouter: function() {
		// summary:
		//		Clear EventRouter for our dataset
		if (this._eventRouter) {
			this._eventRouter.removeListener(this.getEventId());
			this._eventRouter = null;
		}
	},

	_getEventRouter: function() {
		// summary:
		//		Returns current event router
		return this._eventRouter;
	},
	
	_setItemCount: function(itemCount) {
		// summary:
		//		Set internal dataset item count
		this._dataset.itemCount = itemCount;
	},

	_setLinkCount: function(linkCount) {
		// summary:
		//		Set internal dataset link count
		this._dataset.linkCount = linkCount;
	},
	
	_setRelationshipCount: function(relationshipCount) {
		// summary:
		//		Set internal dataset relationship count
		this._dataset.relationshipCount = relationshipCount;
	},

	_clearAllCaches: function() {
		// summary:
		//		Clear all item and link related caches
		this._clearItemCache();
		this._clearLinkCache();
		this._clearRelationshipCache();
		this._clearToolbarCache();
		this._clearItemStyleCache();
		this._clearItemStatusCache();
		this._clearLinkStyleCache();
		this._clearLinkStatusCache();
		this._clearRelationshipStyleCache();
		this._clearRelationshipStatusCache();
	},

	_clearItemCache: function() {
		// summary:
		//		Clear the item cache
		this._itemsById = {};		
	},
	
	_clearLinkCache: function() {
		// summary:
		//		Clear the link cache
		this._linksById = {};
	},

	_clearRelationshipCache: function() {
		// summary:
		//		Clear the relationship cache
		this._relationshipsById = {};
	},

	_clearToolbarCache: function() {
		// summary:
		//		Clear the "toolbar" cache
		this._toolbar = null;
	},

	_clearItemStyleCache: function() {
		// summary:
		//		Clear item style cache
		this._itemStyles = null;
		this._itemStylesById = null;
	},

	_clearItemStatusCache: function() {
		// summary:
		//		Clear item status cache
		this._itemStatus = null;
		this._itemStatusById = null;
	},

	_clearLinkStyleCache: function() {
		// summary:
		//		Clear link style cache
		this._linkStyles = null;
		this._linkStylesById = null;
	},

	_clearLinkStatusCache: function() {
		// summary:
		//		Clear link status cache
		
		//		protected
		this._linkStatus = null;
		this._linkStatusById = null;
	},

	_clearRelationshipStyleCache: function() {
		// summary:
		//		Clear relationship style cache
		this._relStyles = null;
		this._relStylesById = null;
	},

	_clearRelationshipStatusCache: function() {
		// summary:
		//		Clear relationship status cache
		this._relStatus = null;
		this._relStatusById = null;
	},

	_cacheItems: function(items, resetLazyLoad) {
		// summary:
		//		Cache items; merge if already cached
		if (items) {
			var itemsLength = items.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				items[itemIndex] = this._cacheItem(items[itemIndex], resetLazyLoad);
			}
		}
		
		return items;
	},

	_cacheItem: function(item, resetLazyLoad) {
		// summary:
		//		Cache item; merge if already cached
		var cachedItem = null;
		
		if (item) {
			// check item cache		
			cachedItem = this._getCachedItem(item.getId());
			
			// item is already in cache; merge property values
			if (cachedItem) {
				cachedItem.mixin(item, resetLazyLoad);
				
				// reset expiration to (now + interval) since item was just referenced
				if (this.cacheExpires()) {
					cachedItem.resetExpiration();
				}		
			}
			// cache new item
			else {
				this._itemsById[item.getId()] = item;
				cachedItem = item;				
			}
		}
		
		return cachedItem;
	},

	_getCachedItem: function(itemId) {
		// summary:
		//		look up item in cache
		return this._itemsById[itemId];
	},

	_cacheLinks: function(links) {
		// summary:
		//		Cache links; merge if already cached
		if (links) {
			var linksLength = links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				links[linkIndex] = this._cacheLink(links[linkIndex]);
			}
		}
		
		return links;
	},

	_cacheLink: function(link) {
		// summary:
		//		Cache link; merge if already cached
		var cachedLink = null;
		
		if (link) {
			// check link cache		
			cachedLink = this._getCachedLink(link.getId());
			
			// link is already in cache; merge property values
			if (cachedLink) {
				cachedLink.mixin(link);
				
				// reset expiration to (now + interval) since link was just referenced
				if (this.cacheExpires()) {
					cachedLink.resetExpiration();
				}		
			}
			// cache new link
			else {
				this._linksById[link.getId()] = link;
				cachedLink = link;				
			}
		}
		
		return cachedLink;
	},

	_getCachedLink: function(linkId) {
		// summary:
		//		look up link in cache
		return this._linksById[linkId];
	},

	_cacheRelationships: function(relationships) {
		// summary:
		//		Cache relationships; merge if already cached
		if (relationships) {
			var relsLength = relationships.length;
			for (var relIndex = 0;  relIndex < relsLength;  ++relIndex) {
				relationships[relIndex] = this._cacheRelationship(relationships[relIndex]);				
			}
		}
		
		return relationships;
	},

	_cacheRelationship: function(relationship) {
		// summary:
		//		Cache relationship; merge if already cached
		var cachedRelationship = null;
		
		if (relationship) {
			// check relationship cache		
			cachedRelationship = this._getCachedRelationship(relationship.getId());
			
			// relationship is already in cache; merge property values
			if (cachedRelationship) {
				cachedRelationship.mixin(relationship);
				
				// reset expiration to (now + interval) since relationship was just referenced
				if (this.cacheExpires()) {
					cachedRelationship.resetExpiration();
				}			
			}
			// cache new relationship
			else {
				this._relationshipsById[relationship.getId()] = relationship;
				cachedRelationship = relationship;
			}
		}
		
		return cachedRelationship;
	},

	_getCachedRelationship: function(relationshipId) {
		// summary:
		//		look up relationship in cache
		return this._relationshipsById[relationshipId];
	},

	_cacheItemStyles: function(styles) {
		// summary:
		//		Set item style cache
		var stylesById = {};
		if (styles) {
			var stylesLength = styles.length;
			for (var styleIndex = 0;  styleIndex < stylesLength;  ++styleIndex) {
				var style = styles[styleIndex];
				stylesById[style.getType()] = style;
			}
		}
		
		this._itemStylesById = stylesById;
		this._itemStyles = styles;
	},

	_cacheItemStatuses: function(statuses) {
		// summary:
		//		Set item status cache
		var statusesById = {};
		if (statuses) {
			var statusLength = statuses.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				var id = statuses[statusIndex].getId() + "_" + statuses[statusIndex].getValue();
				statusesById[id] = statuses[statusIndex];
			}
		}
		
		this._itemStatusById = statusesById;
		this._itemStatus = statuses;	
	},

	_cacheLinkStyles: function(styles) {
		// summary:
		//		Set link style cache
		var stylesById = {};
		if (styles) {
			var stylesLength = styles.length;
			for (var styleIndex = 0;  styleIndex < stylesLength;  ++styleIndex) {
				var style = styles[styleIndex];
				stylesById[style.getType()] = style;
			}
		}
		
		this._linkStylesById = stylesById;
		this._linkStyles = styles;
	},

	_cacheLinkStatuses: function(statuses) {
		// summary:
		//		Set link status cache
		var statusesById = {};
		if (statuses) {
			var statusLength = statuses.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				var id = statuses[statusIndex].getId() + "_" + statuses[statusIndex].getValue();
				statusesById[id] = statuses[statusIndex];
			}
		}
		
		this._linkStatusById = statusesById;
		this._linkStatus = statuses;
	},

	_cacheRelationshipStyles: function(styles) {
		// summary:
		//		Set relationship style cache
		var stylesById = {};
		if (styles) {
			var stylesLength = styles.length;
			for (var styleIndex = 0;  styleIndex < stylesLength;  ++styleIndex) {
				var style = styles[styleIndex];
				stylesById[style.getType()] = style;
			}
		}
		
		this._relStylesById = stylesById;
		this._relStyles = styles;
	},
	
	_cacheRelationshipStatuses: function(statuses) {
		// summary:
		//		Set relationship status cache
		var statusesById = {};
		if (statuses) {
			var statusLength = statuses.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				var id = statuses[statusIndex].getId() + "_" + statuses[statusIndex].getValue();
				statusesById[id] = statuses[statusIndex];
			}
		}
		
		this._relStatusById = statusesById;
		this._relStatus = statuses;
	},

	_mergeItemStyles: function(itemStyles, widgetDef) {
		// summary:
		//		Merge item styles with any pertinent information from widget definition
		var mergedStyles = itemStyles;
				
		// widget definition was provided
		if (widgetDef) {
			mergedStyles = itemStyles.slice(0);
			
			var styleMap = {};
			var styleLength = itemStyles.length;
			for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
				styleMap[itemStyles[styleIndex].getType()] = itemStyles[styleIndex];
				mergedStyles[styleIndex] = itemStyles[styleIndex].clone(widgetDef);					
			}
			
			// see if any styles are in widget definition that are not known to dataset
			var widgetStyles = widgetDef.getItemStyles();
			if (widgetStyles) {
				styleLength = widgetStyles.length;
				for (styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					var widgetStyle = widgetStyles[styleIndex];
					if (!styleMap[widgetStyle.type]) {
						mergedStyles.push(this._createItemStyle(widgetDef, widgetStyle));
					}
				}
			}			
		}
		
		return mergedStyles;
	},

	_createItemStyle: function(widgetDef, widgetStyle) {
		// summary:
		//		Create an ItemStyle from information in the widget definition
		var styleAttributes = {};
		var widgetDefaultStyle = widgetDef.getDefaultItemStyle();
		if (widgetDefaultStyle) {
			dojo.mixin(styleAttributes, widgetDefaultStyle);
		}
							
		dojo.mixin(styleAttributes, widgetStyle);
					
		return new ibm.tivoli.navmodel.ItemStyle({style: styleAttributes, version: this.getVersion()});
	},

	_mergeLinkStyles: function(linkStyles, widgetDef, objType) {
		// summary:
		//		Merge link styles with any pertinent information from widget definition
		var mergedStyles = linkStyles;
		
		// widget definition was provided
		if (widgetDef) {
			mergedStyles = linkStyles.slice(0);
			
			var styleMap = {};
			var styleLength = linkStyles.length;
			for (var styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
				styleMap[linkStyles[styleIndex].getType()] = linkStyles[styleIndex];
				mergedStyles[styleIndex] = linkStyles[styleIndex].clone(widgetDef, objType);				
			}
			
			// see if any styles are in widget definition that are not known to dataset
			var widgetStyles = widgetDef.getLinkStyles();
			if (widgetStyles) {
				styleLength = widgetStyles.length;
				for (styleIndex = 0;  styleIndex < styleLength;  ++styleIndex) {
					var widgetStyle = widgetStyles[styleIndex];
					if (!styleMap[widgetStyle.type]) {
						mergedStyles.push(this._createLinkStyle(widgetDef, widgetStyle, objType));
					}
				}
			}			
		}
		
		return mergedStyles;
	},

	_createLinkStyle: function(widgetDef, widgetStyle, objType) {
		// summary:
		//		Create a LinkStyle from information in the widget definition
		var styleAttributes = {};
		var widgetDefaultStyle = null;
		switch(objType) {
			case "link":
				widgetDefaultStyle = widgetDef.getDefaultLinkStyle();
				break;
			case "relationship":
				widgetDefaultStyle = widgetDef.getDefaultRelationshipStyle();
		}
		
		if (widgetDefaultStyle) {
			dojo.mixin(styleAttributes, widgetDefaultStyle);
		}
							
		dojo.mixin(styleAttributes, widgetStyle);
					
		return new ibm.tivoli.navmodel.LinkStyle({style: styleAttributes, version: this.getVersion()});
	},

	_mergeStatuses: function(status, widgetDef, objType) {
		// summary:
		//		Merge status with any pertinent information from widget definition
		var mergedStatus = status;
		
		// widget definition was provided
		if (widgetDef) {
			mergedStatus = status.slice(0);
			
			var statusMap = {};
			var statusLength = status.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				statusMap[status[statusIndex].getId()] = status[statusIndex];
				mergedStatus[statusIndex] = status[statusIndex].clone(widgetDef, objType);
			}
			
			// see if any status values are in widget definition that are not known to dataset
			var widgetStatuses = null;
			switch(objType) {
				case "item":
					widgetStatuses = widgetDef.getItemStatuses();
					break;
				case "link":
					widgetStatuses = widgetDef.getLinkStatuses();
					break;
				case "relationship":
					widgetStatuses = widgetDef.getRelationshipStatuses();
			}
			
			if (widgetStatuses) {
				statusLength = widgetStatuses.length;
				for (statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
					var widgetStatus = widgetStatuses[statusIndex];
					if (!statusMap[widgetStatus.id]) {
						mergedStatus.push(this._createStatus(widgetDef, widgetStatus, objType));
					}
				}
			}
		}
		
		return mergedStatus;
	},

	_createStatus: function(widgetDef, widgetStatus, objType) {
		// summary:
		//		Create an Status from information in the widget definition
		var statusAttributes = {};
		var widgetDefaultStatus = null;
		switch(objType) {
			case "item":
				widgetDefaultStatus = widgetDef.getDefaultItemStatus();
				break;
			case "link":
				widgetDefaultStatus = widgetDef.getDefaultLinkStatus();
				break;
			case "relationship":
				widgetDefaultStatus = widgetDef.getDefaultRelationshipStatus();
		}
		
		if (widgetDefaultStatus) {
			dojo.mixin(statusAttributes, widgetDefaultStatus);
		}
							
		dojo.mixin(statusAttributes, widgetStatus);
					
		return new ibm.tivoli.navmodel.Status({status: statusAttributes, version: this.getVersion()});
	},
		
	_notifyEventListeners: function(event) {
		// summary:
		//		Pass ibm.tivoli.navmodel.Event to all registered listeners
		if (this._listeners) {
			var listenerLength = this._listeners.length;
			for (var listenerIndex = 0;  listenerIndex < listenerLength;  ++listenerIndex) {
				try {
					var listenerObj = this._listeners[listenerIndex].listenerObj;
					this._listeners[listenerIndex].listenerCallback(listenerObj, event);
				}
				catch (ex) {
					this._trace.error("Exception forwarding eventId=" + event.getId() + " type=" + event.getType() + " ex=" + ex, "_sendEvent");
				}
			}				
		}		
	},
	
	_jsonEventHandler: function(jsonEventObj) {
		// summary:
		//		This is the routine that we register with the EventRouter to handle
		//		raw json events that come from the server
		var event = new ibm.tivoli.navmodel.Event({dataset: this, event: jsonEventObj});
		this._eventHandler(event);
	},

	_eventHandler: function(event) {
		// summary:
		//		Handles ibm.tivoli.navmodel.Event objects
		if (event.getId() === this.getEventId()) {
			if (this._trace.isFine()) {
				this._trace.fine("datasetId=" + this.getId() + " event=" + event.getType() + " eventId=" + event.getId(), "_eventHandler");
			}
			
			switch(event.getType()) {
				case "changed":
					this._handleChangedEvent(event);
					break;
				case "labelChanged":
					this._handleLabelChangedEvent(event);
					break;
				case "statusChanged":
					this._handleStatusChangedEvent(event);
					break;					
				case "itemsAdded":
					this._handleItemsAddedEvent(event);
					break;
				case "itemsRemoved":
					this._handleItemsRemovedEvent(event);
					break;
				case "itemStatusChanged":
					this._handleItemStatusChangedEvent(event);
					break;
				case "itemPropertiesChanged":
					this._handleItemPropertiesChangedEvent(event);
					break;
				case "linksAdded":
					this._handleLinksAddedEvent(event);
					break;
				case "linksRemoved":
					this._handleLinksRemovedEvent(event);
					break;
				case "linkStatusChanged":
					this._handleLinkStatusChangedEvent(event);
					break;
				case "linkPropertiesChanged":
					this._handleLinkPropertiesChangedEvent(event);
					break;
				case "linkChanged":
					this._handleLinkChangedEvent(event);
					break;
				case "relationshipStatusChanged":
					this._handleRelationshipStatusChangedEvent(event);
					break;
				case "relationshipPropertiesChanged":
					this._handleRelationshipPropertiesChangedEvent(event);
					break;
				case "exception":
					if (this._trace.isFine()) {
						this._trace.fine("	exception msgText=" + event.getException().getMsgText(), "_eventHandler");
					}
					break;
				case "eventRouter":
					if (this._trace.isFine()) {
						this._trace.fine("	eventRouterStatus=" + event.getEventRouterStatus(), "_eventHandler");
					}
			}
			
			this._notifyEventListeners(event);
		}		
	},

	_handleChangedEvent: function(event) {
		// summary:
		//		Handle "changed" event type
		this._clearAllCaches();
		
		var initArgs = {
			optimize: true,
			createEventRouter: event.createEventRouter			
		};
		this._initDataset(initArgs);
	},
	
	_handleLabelChangedEvent: function(event) {
		// summary:
		//		Handle "labelChanged" event type
		this._dataset.label = event._getEventNewLabel();
		
		if (this._trace.isFinest()) {
			this._trace.finest("datasetId=" + this.getId() + " label=" + this._dataset.label, "_handleLabelChangedEvent");
		}
	},

	_handleStatusChangedEvent: function(event) {
		// summary:
		//		Handle "statusChanged" event type
		var eventStatus = event._getEventStatuses();
		if (eventStatus) {		
			var status = [];
			var statusLength = eventStatus.length;
			for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
				status[statusIndex] = new ibm.tivoli.navmodel.Status({status: eventStatus[statusIndex], version: this.getVersion()});
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " status=" + status[statusIndex].getLabel(), "_handleStatusChangedEvent");
				}
			}
		}
	},

	_handleItemsAddedEvent: function(event) {
		// summary:
		//		Handle "itemsAdded" event type
		
		// get new item count for dataset
		var itemCount = event._getEventDatasetItemCount();
		if (itemCount !== undefined) {
			this._setItemCount(itemCount);
		}
		
		// might have new list of styles
		this._clearItemStyleCache();
			
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItems[itemIndex].id, "_handleItemsAddedEvent");
				}
				
				var eventItem = eventItems[itemIndex];
				var newItem = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});
				newItem = this._cacheItem(newItem);													
				event._addItem(newItem);				
			}					
		}
	},

	_handleItemsRemovedEvent: function(event) {
		// summary:
		//		Handle "itemsRemoved" event type
		
		// get new item count for dataset
		var itemCount = event._getEventDatasetItemCount();
		if (itemCount !== undefined) {
			this._setItemCount(itemCount);
	    	if (itemCount === 0) {
	    		this._clearItemCache();
			}
		}
		
		// might have new list of styles
		this._clearItemStyleCache();
			
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = itemsLength - 1;  itemIndex >= 0;  --itemIndex) {
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItems[itemIndex].id, "_handleItemsRemovedEvent");
				}
				
				var eventItem = eventItems[itemIndex];
				var deletedItem = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});		
				var item = this._getCachedItem(eventItem.id);
				if (item) {
					delete this._itemsById[item.getId()];
					event._addItem(item);
				}
				else {	
					event._addItem(deletedItem);
				}			
			}	
		}
	},

	_handleItemStatusChangedEvent: function(event) {
		// summary:
		//		Handle "itemStatusChanged" event type
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				var eventItem = eventItems[itemIndex];	
				var item = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});		
				item = this._cacheItem(item);					
				event._addItem(item);
				
				if (this._trace.isFinest()) {
					var status = item.getStatuses({widgetDef: null});
					if (status) {
						var statusLength = status.length;
						for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
							this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItem.id + " status=" + status[statusIndex].getLabel(), "_handleItemStatusChangedEvent");
						}					
					}
				}			
			}
		}
	},

	_handleItemPropertiesChangedEvent: function(event) {
		// summary:
		//		Handle "itemPropertiesChanged" event type
		var eventItems = event._getEventItems();
		if (eventItems) {
			var itemsLength = eventItems.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				var eventItem = eventItems[itemIndex];							
				var props = [];
				var properties = eventItem.properties;					
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var property = new ibm.tivoli.navmodel.Property({property: properties[propIndex], dataset: this, objectType: "item"});
						props[propIndex] = property;						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.getId() + " itemId=" + eventItem.id + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleItemPropertiesChangedEvent");
						}																	
					}
				}
								
				var item = new ibm.tivoli.navmodel.Item({item: eventItem, dataset: this});				
				item = this._cacheItem(item);					
				event._addItem(item, props);				
			}
		}
	},

	_handleLinksAddedEvent: function(event) {
		// summary:
		//		Handle "linksAdded" event type
		
		// get new link and relationship counts for dataset
		var linkCount = event._getEventDatasetLinkCount();
		var relCount = event._getEventDatasetRelationshipCount();
		if (linkCount !== undefined) {
			this._setLinkCount(linkCount);
		}
		if (relCount !== undefined) {
			this._setRelationshipCount(relCount);
		}
		
		// might have new list of styles
		this._clearLinkStyleCache();
		this._clearRelationshipStyleCache(); 
			
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
		    var eventLinksLength = eventLinks.length;
			for (var linkIndex = 0;  linkIndex < eventLinksLength;  ++linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var newLink = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				newLink = this._cacheLink(newLink);				
				event._addLink(newLink);
				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " linkId=" + newLink.getId(), "_handleLinksAddedEvent");
				}				
			}
		}
	},

	_handleLinksRemovedEvent: function(event) {
		// summary:
		//		Handle "linksRemoved" event type
		
		// get new link count for dataset
		var linkCount = event._getEventDatasetLinkCount();
		var relCount = event._getEventDatasetRelationshipCount();
		if (linkCount !== undefined) {
			this._setLinkCount(linkCount);
			if (linkCount === 0) {
	    		this._clearLinkCache();	    		
			}
		}
		if (relCount !== undefined) {
		 	this._setRelationshipCount(relCount);			
		}
		
		// might have new list of styles
		this._clearLinkStyleCache();
		this._clearRelationshipStyleCache();
		    
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
		    var eventLinksLength = eventLinks.length;
			for (var linkIndex = eventLinksLength - 1;  linkIndex >= 0;  --linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var deletedLink = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				var link = this._getCachedLink(eventLink.id);
				if (link) {
					delete this._linksById[link.getId()];
					event._addLink(link);
				}
				else {	
					event._addLink(deletedLink);
				}
				
				if (this._trace.isFinest()) {
					this._trace.finest("datasetId=" + this.getId() + " linkId=" + deletedLink.getId(), "_handleLinksRemovedEvent");
				}	
			}
		}
	},
	
	_handleLinkStatusChangedEvent: function(event) {
		// summary:
		//		Handle "linkStatusChanged" event type
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
			var eventLinksLength = eventLinks.length;
			for (var linkIndex = 0;  linkIndex < eventLinksLength;  ++linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var link = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				link = this._cacheLink(link);					
				event._addLink(link);
				
				if (this._trace.isFinest()) {
					var status = link.getStatuses({widgetDef: null});
					if (status) {
						var statusLength = status.length;
						for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
							this._trace.finest("datasetId=" + this.getId() + " linkId=" + link.getId() + " status=" + status[statusIndex].getLabel(), "_handleLinkStatusChangedEvent");
						}					
					}
				}		
			}
		}
	},

	_handleLinkPropertiesChangedEvent: function(event) {
		// summary:
		//		Handle "linkPropertiesChanged" event type
		var eventLinks = event._getEventLinks();
		if (eventLinks) {
			var eventLinksLength = eventLinks.length;
			for (var linkIndex = 0;  linkIndex < eventLinksLength;  ++linkIndex) {
				var eventLink = eventLinks[linkIndex];
				var link = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
				var props = [];
				var properties = eventLink.properties;					
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var property = new ibm.tivoli.navmodel.Property({property: properties[propIndex], dataset: this, objectType: "link"});
						props[propIndex] = property;						
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.getId() + " linkId=" + link.getId() + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleLinkPropertiesChangedEvent");
						}
					}
				}
				
				link = this._cacheLink(link);					
				event._addLink(link, props);
			}
		}		
	},

	_handleLinkChangedEvent: function(event) {
		// summary:
		//		Handle "linkChanged" event type
		
		// might have new list of styles
		this._clearLinkStyleCache();
		this._clearRelationshipStyleCache();
		    
		var eventLink = event._getEventLink();
		var link = new ibm.tivoli.navmodel.Link({link: eventLink, dataset: this});
		link = this._cacheLink(link);	
		event._addLink(link);
				
		if (this._trace.isFinest()) {
			this._trace.finest("datasetId=" + this.getId() + " linkId=" + link.getId() + " reason=" + event.getLinkChangedReason(), "_handleLinkChangedEvent");
		}						
		
		// affected relationships (reason is "relationshipAdded" or "relationshipRemoved")
		var eventRelationships = event._getEventRelationships();
		if (eventRelationships) {
			var eventRelLength = eventRelationships.length;
			for (var eventRelIndex = 0;  eventRelIndex < eventRelLength;  ++eventRelIndex) {
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: eventRelationships[eventRelIndex], dataset: this});
				event._addRelationship(relationship);
			}
		}
	},

	_handleRelationshipStatusChangedEvent: function(event) {
		// summary:
		//		Handle "relationshipStatusChanged" event type
		var eventRels = event._getEventRelationships();
		if (eventRels) {
			var relLength = eventRels.length;
			for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
				var eventRel = eventRels[relIndex];
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: eventRel, dataset: this});
				relationship = this._cacheRelationship(relationship);
				event._addRelationship(relationship);
				
				if (this._trace.isFinest()) {
					var status = relationship.getStatuses({widgetDef: null});
					if (status) {
						var statusLength = status.length;
						for (var statusIndex = 0;  statusIndex < statusLength;  ++statusIndex) {
							this._trace.finest("datasetId=" + this.getId() + " relationshipId=" + eventRel.id + " status=" + status[statusIndex].getLabel(), "_handleRelationshipStatusChangedEvent");
						}					
					}
				}
			}
		}		
	},

	_handleRelationshipPropertiesChangedEvent: function(event) {
		// summary:
		//		Handle "relationshipPropertiesChanged" event type
		var eventRels = event._getEventRelationships();
		if (eventRels) {
			var relsLength = eventRels.length;
			for (var relIndex = 0;  relIndex < relsLength;  ++relIndex) {
				var eventRel = eventRels[relIndex];
				var props = [];
				var properties = eventRel.properties;					
				if (properties) {
					var propsLength = properties.length;
					for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
						var property = new ibm.tivoli.navmodel.Property({property: properties[propIndex], dataset: this, objectType: "relationship"});
						props[propIndex] = property;							
						if (this._trace.isFinest()) {
							this._trace.finest("datasetId=" + this.getId() + " relationshipId=" + eventRel.id + " prop=" + property.getId() + " value=" + property.getDisplayValue(), "_handleRelationshipPropertiesChangedEvent");
						}
					}					
				}
					
				var relationship = new ibm.tivoli.navmodel.Relationship({relationship: eventRel, dataset: this});
				relationship = this._cacheRelationship(relationship);
				event._addRelationship(relationship, props);				
			}
		}
	},
		
	_validateCache: function() {
		// summary:
		//		Check cache for items, links, and relationships that haven't been used for awhile;
		//		remove from cache
		this._trace.entering("starting validate cache", "_validateCache");
		
		this._validate(this._itemsById);
		this._validate(this._linksById);
		this._validate(this._relationshipsById);
		
		this._trace.exiting("ending validate cache", "_validateCache");				
	},
	
	_validate: function(cache) {
		// summary:
		//		Check expiration for supplied cache; items, links, or relationships
		var nowDate = new Date();
		var now = nowDate.getTime();
		
		for (var id in cache) { 
    		if (cache.hasOwnProperty(id)) {
    			var obj = cache[id];
    			if (obj && obj.checkExpiration(now)) {
    				delete cache[id];
    			}
    		}        
		} 		
	},
	
	/////////////////////////////
	// abstract methods
	/////////////////////////////
	
	/*=====
	initDataset_kwArgs: function(createEventRouter) {
		// createEventRouter: Boolean
		//		boolean indicating whether new event router should be created
	},
	=====*/
	
	_initDataset: function(/*initDataset_kwArgs*/ kwArgs) {
		// summary:
		//		Initialize internal dataset object
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// determine if we need to start up the cache interval to purge
		// resources that have not been touched in a while
		if (!this._cacheIntervalId && this.getItemCount() >= this._args.cacheThreshold && this._args.cacheValidationInterval) {
			this._cacheIntervalId = setInterval(dojo.hitch(this, this._validateCache), (this._args.cacheValidationInterval*60000));
		}
	},

	_deleteDataset: function() {
		// summary:
		//		Dataset is no longer in use	
	},
	
	/*=====
	fetchItemsAndLinks_onComplete: function(items, links, toolbar, kwArgs){},
	=====*/
	
	/*=====
	fetchItemsAndLinks_kwArgs: function(onComplete, onError, sync) {
		// onComplete: fetchItemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	_fetchItemsAndLinks: function(/*fetchItemsAndLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items and links associated with the dataset
	},
	
	_fetchItems: function(/*getItems_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch items from rest service
	},

	_fetchItem: function(/*getItem_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual item from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return null;
	},
	
	_fetchOriginItems: function(/*getOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
	},
	
	_fetchRelatedItems: function(/*getRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested item
	},

	_queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
	},

	_fetchLinks: function(/*getLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch links from rest service
	},

	_fetchLink: function(/*getLink_kwArgs*/ linkId) {
		// summary:
		//		Synchronously fetch an individual link from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return null;
	},
		
	_fetchRelationships: function(/*getRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch relationships from rest service
	},

	_fetchRelationship: function(/*getRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual relationship from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		return null;
	},

	_fetchItemStyles: function(/*getItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
	},
	
	_fetchLinkStyles: function(/*getLinkStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current link styles for the Dataset
	},
	
	_fetchRelationshipStyles: function(/*getRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
	},
	
	/*=====
	fetchStatuses_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: relStatuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type for columns: "item", "link", "relationship"
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchStatuses: function(/*fetchStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible relationship status values for the Dataset
	},
	
	_fetchTasks: function(/*getTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
	},
	
	_navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
	},
	
	/*=====
	fetchColumns_kwArgs: function(onComplete, onError, type, properties, start, count, metadaa, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of ids in which to fetch columns;
		//		if not specified the "current" columns will be fetched
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// metadata: Boolean
		//		indicates if filter metadata should be returned with columns
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/

	_fetchColumns: function(/*fetchColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch columns from rest service
	},
	
	_setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Save columns to rest service
	},

	_createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new item in the dataset
	},
	
	_createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
	},
	
	_deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
	},
	
	_deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
	},
	
	_createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
	},
	
	_createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
	},
	
	_deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
	},
	
	_deleteLinks: function(/*deleteLinks_kwArgs*/  kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
	},

	_createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
	},

	_createRelationships: function(/*createRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
	},
	
	_deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
	},
	
	_deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
	}
});

}

if(!dojo._hasResource["ibm.tivoli.navmodel.Dataset"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["ibm.tivoli.navmodel.Dataset"] = true;
/* 
 * Licensed Materials - Property of IBM
 * 
 * TIVOTCT00
 * 
 * �Restricted Materials of IBM�
 * 
 * Copyright IBM Corp. 2011, 2012 All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or 
 * disclosure restricted by GSA ADP Schedule Contract with 
 * IBM Corp. 
 */

dojo.provide("ibm.tivoli.navmodel.Dataset"); 
 
// includes


ibm.tivoli.navmodel.Dataset._trace = new ibm.tivoli.navmodel.Trace("ibm.tivoli.navmodel.Dataset");

////////////////////////////////////
// Dataset
//
// NOTE: do not call any abstract methods directly;
// call the public version of the methods defined in
// AbstractDataset instead and it will in turn call
// the abstract overrides.
////////////////////////////////////  
dojo.declare("ibm.tivoli.navmodel.Dataset", ibm.tivoli.navmodel.AbstractDataset, {
	// private members
	_optimize: true,
	_trace: ibm.tivoli.navmodel.Dataset._trace, 
	
	/*=====
	constructor_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	constructor_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	 constructor_kwArgs: function(onComplete, onError, baseUrl, provider, datasource, dataset, 
	 					parameters, defaultProperties, defaultLinkProperties, defaultRelProperties, 
	 					cacheThreshold, cacheValidationInterval, cacheExpirationTime, version) {
		// onComplete: constructor_onComplete
	 	//		callback to be invoked when the Dataset is initialized and ready for methods
	 	//		to be invoked  (i.e. asynchronous initialization must be performed). 
	 	//		Note: A null value will result in synchronous initialization.
	 	// onError: constructor_onError
	 	//		callback to be invoked if there is a problem constructing the Dataset
		// baseUrl: String
		//		base url for identifying rest service
		// provider: String
		//		provider id
		// datasource: String
		//		datasource id
		// dataset: String
		//		dataset id
		// parameters: Object
		//		optional: extra parameters sent to dataset provider
		// defaultProperties: String[]
		//		optional: item property ids to fetch by default (can specify "all")
		// defaultLinkProperties: String[]
		//		optional: link property ids to fetch by default (can specify "all")
		// defaultRelProperties: String[]
		//		optional: relationship property ids to fetch by default (can specify "all")
		// cacheThreshold: int[]
		//		optional: cache objects if number of dataset items >= threshold
		// cacheValidationInterval: int[]
		//		optional: number of minutes in between cache validations
		// cacheExpirationTime: int[]
		//		optional: number of minutes objects can stay in cache while inactive
		// version: Float
		//		optional: version used to communicate with rest service (i.e. json version)
	},
	=====*/

	constructor: function(/*constructor_kwArgs_kwArgs*/ kwArgs, dataset) {	
		// summary: 
		//		Constructs Dataset object.
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception		
	},

	//////////////////////////////////////////////////////
	// abstract method overrides
	//
	// NOTE: do not call any abstract methods directly;
	// call the public version of the methods defined in
	// AbstractDataset instead and it will in turn call
	// the abstract overrides.
	//////////////////////////////////////////////////////

	/*=====
	initDataset_kwArgs: function(createEventRouter, optimize) {
		// createEventRouter: Boolean
		//		boolean indicating whether new event router should be created
		// optimize: Boolean
		//		optimize json that is returned; remove items that can be calculated
		//		or can be retrieved via additional urls
	},
	=====*/
	
	_initDataset: function(/*initDataset_kwArgs*/ kwArgs) {
		// summary:
		//		Initialize internal dataset object
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		
		// start out with empty dataset
		this._initEmptyDataset();		
		this._optimize = kwArgs.optimize;
		
		// make synchronous call if no onComplete callback
		var synchronous = (kwArgs.onComplete ? false : true);
		var args = arguments;
		var exception = null;
		
		// dataset was NOT provided; fetch it
		if (!kwArgs.dataset) {			
			var content = ibm.tivoli.navmodel.Utils.getContent();
			this._mixinStandard(content, kwArgs);			
			ibm.tivoli.navmodel.Utils.mixinParams(content, this);
			
			dojo.xhrGet({
				url: this._getDatasetURL(),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset")
				},
				handleAs: "json",
				sync: synchronous,
				content: content,
				load: dojo.hitch(this, function(result, ioArgs) {
					// pull version from the Content-Type response header
					this._setVersion(this._extractContentTypeVersion(ioArgs));
					
					// mixin all dataset attributes
					this.mixinAttributes(result);
					
					// listen for asynchronous events
					if (kwArgs.createEventRouter && this.isEventsEnabled()) {
						this._createEventRouter();
					}
		
					// call base class
					this.inherited(args);
					
					// call onComplete callback
					if (kwArgs.onComplete) {
						kwArgs.onComplete(this, kwArgs);
					}							
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					exception = this._getException(error, ioArgs, "_initDataset");					
					if (kwArgs.onError) {
						kwArgs.onError(error, exception, ioArgs);
					}						
				})
			});	
			
			if (synchronous && exception) {
				throw exception;
			}						
		}
		
		// dataset was provided
		else {
			this.mixinAttributes(kwArgs.dataset);
			
			// listen for asynchronous events
			if (kwArgs.createEventRouter && this.isEventsEnabled()) {
				this._createEventRouter();
			}
		
			// call base class
			this.inherited(args);			
		}
	},
	
	_createEventRouter: function() {
		// summary:
		//		create event router object and set on the base class
		var baseUrl = this.getArg("baseUrl");
		var contextRoot = baseUrl.substring(0, baseUrl.lastIndexOf("/"));
		var eventRouterArgs = { 
			eventDestination: this.getAttribute("eventDestination"),
			contextRoot: contextRoot
		};		
		var eventRouter = new ibm.tivoli.navmodel.EventRouter(eventRouterArgs);
			
		this._setEventRouter(eventRouter);		
	},

	_deleteDataset: function() {
		// summary:
		//		Dataset is no longer in use
		if (this._trace.isFinest()) {
			this._trace.finest("sending DELETE for dataset=" + this.getId(), "_deleteDataset");
		}
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrDelete({
			url: this._getDatasetURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){			
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				this._getException(error, ioArgs, "_deleteDataset");
			})
		});
	},
	
	

	/*=====
	fetchItemsAndLinks_onComplete: function(items, links, datasetItems, toolbar, kwArgs){},
	=====*/
	
	/*=====
	_onError: function(error, exception, ioArgs){},
	=====*/
	
	/*=====
	fetchItemsAndLinks_kwArgs: function(onComplete, onError, sync) {
		// onComplete: fetchItemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchItemsAndLinks: function(/*fetchItemsAndLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns both the items and links associated with the dataset
		var defaultProperties = this.getArg("defaultProperties");
		var defaultLinkProperties = this.getArg("defaultLinkProperties");
		var defaultRelProperties = this.getArg("defaultRelProperties");
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "itemProperties", defaultProperties);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "linkProperties", defaultLinkProperties);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "relationshipProperties", defaultRelProperties);		
		content.optimize = this._optimize;
		content.items = true;
		content.links = true;
		content.toolbarTasks = true;
		
		return dojo.xhrGet({
			url: this._getDatasetURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];				
				var links = [];
				var toolbar = [];
				var styles = null;
				var status = null;
				var length;
				var index;
				var version = this.getVersion();
				
				this._setItemCount(result.itemCount);
				this._setLinkCount(result.linkCount);
				this._setRelationshipCount(result.relationshipCount);
										
				if (result.itemStyles) {
					styles = [];
					length = result.itemStyles.length;
					for (index = 0;  index < length;  ++index) {
						styles[index] = new ibm.tivoli.navmodel.ItemStyle({style: result.itemStyles[index], version: version});
					}
					this._cacheItemStyles(styles);
				}
				
				if (result.itemStatus) {
					status = [];
					length = result.itemStatus.length;
					for (index = 0;  index < length;  ++index) {
						status[index] = new ibm.tivoli.navmodel.Status({status: result.itemStatus[index], version: version});
					}
					this._cacheItemStatuses(status);
				}
				
				if (result.items) {
					var allProps = this._isAllProps(defaultProperties);					
					var propIds = this._getPropIds(defaultProperties, allProps);
					length = result.items.length;
					for (index = 0;  index < length;  ++index) {
						items[index] = new ibm.tivoli.navmodel.Item({item: result.items[index], dataset: this, propIds: propIds, allProperties: allProps, expiration: expiration});
					}
				}
				
				if (result.linkStyles) {
					styles = [];
					length = result.linkStyles.length;
					for (index = 0;  index < length;  ++index) {
						styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.linkStyles[index], version: version});
					}
					this._cacheLinkStyles(styles);
				}
				
				if (result.relationshipStyles) {
					styles = [];
					length = result.relationshipStyles.length;
					for (index = 0;  index < length;  ++index) {
						styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.relationshipStyles[index], version: version});
					}
					this._cacheRelationshipStyles(styles);
				}
				
				if (result.linkStatus) {
					status = [];
					length = result.linkStatus.length;
					for (index = 0;  index < length;  ++index) {
						status[index] = new ibm.tivoli.navmodel.Status({status: result.linkStatus[index], version: version});
					}
					this._cacheLinkStatuses(status);
				}
				
				if (result.relationshipStatus) {
					status = [];
					length = result.relationshipStatus.length;
					for (index = 0;  index < length;  ++index) {
						status[index] = new ibm.tivoli.navmodel.Status({status: result.relationshipStatus[index], version: version});
					}
					this._cacheRelationshipStatuses(status);
				}
				
				if (result.links) {
					var allLinkProps = this._isAllProps(defaultLinkProperties);
					var linkPropIds = this._getPropIds(defaultLinkProperties, allLinkProps);
					length = result.links.length;
					for (index = 0;  index < length;  ++index) {
						links[index] = new ibm.tivoli.navmodel.Link({link: result.links[index], dataset: this, propIds: linkPropIds, allProperties: allLinkProps, expiration: expiration});
					}
				}
				
				if (result.toolbarTasks) {
					length = result.toolbarTasks.length;
					for (index = 0;  index < length;  ++index) {
						toolbar[index] = new ibm.tivoli.navmodel.Task({task: result.toolbarTasks[index], dataset: this});
					}
				}
				
				kwArgs.onComplete(items, links, toolbar, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItemsAndLinks"), ioArgs);
			})
		});
	},

	/*=====
	items_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	fetchItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchItems: function(/*fetchItems_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch items from rest service
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getItemsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var items = [];				
				if (result.items) {
					var expiration = this.getNewExpirationTime();
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						var item = result.items[itemIndex];
						if (item.datasetId && item.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(item.datasetId);
							items[itemIndex] = new ibm.tivoli.navmodel.Item({item: item, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});							
						}
						else {
							items[itemIndex] = new ibm.tivoli.navmodel.Item({item: item, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
					}
				}
				
				// make sure our count is accurate
				this._setItemCount(result.totalRows);
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				kwArgs.onComplete(items, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItems"), ioArgs);
			})
		});
	},
	
	/*=====
	fetchItem_kwArgs: function(itemId, properties, metadata) {
		// itemId: String
		//		item id to get
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchItem: function(/*fetchItem_kwArg*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual item from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var item = null;
		var exception = null;
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		dojo.xhrGet({
			url: this._getItemURL(kwArgs.itemId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var allProps = this._isAllProps(kwArgs.properties);
				var propIds = this._getPropIds(kwArgs.properties, allProps);
				item = new ibm.tivoli.navmodel.Item({item: result, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata});											
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_fetchItem");
			})
		});
		
		if (exception) {
			throw exception;
		}
				
		return item;
	},

	/*=====
	fetchOriginItems_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metdata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchOriginItems: function(/*fetchOriginItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns orign items of the dataset
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getOriginItemsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "originItemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();				
				var items = [];
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
					}
				}
				
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				
				kwArgs.onComplete(items, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchOriginItems"), ioArgs);
			})
		});
	},

	/*=====
	itemsAndLinks_onComplete: function(items, links, datasetItems, kwArgs){},
	=====*/
	
	/*=====
	fetchRelatedItems_kwArgs: function(onComplete, onError, itemId, relationships, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: itemsAndLinks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of the base item
		// relationships: String
		//		comma delimited list of relationships to determine related items; 
		//		can also be keywords: child, parent, all
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelatedItems: function(/*fetchRelatedItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items that are related to the requested item
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		// relationships to explore
		if (kwArgs.relationships) {
			content.relationships = kwArgs.relationships;
		}
		
		return dojo.xhrGet({
			url: this._getRelatedItemsURL(kwArgs.itemId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relatedItemViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];
				if (result.items) {
					if (result.datasetItemCount !== undefined) {
						this._setItemCount(result.datasetItemCount);
					}
					
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});							
					}
				}
				
				var links = [];
				if (result.links) {
					if (result.datasetLinkCount !== undefined) {
						this._setLinkCount(result.datasetLinkCount);
					}
					
					if (result.datasetRelationshipCount !== undefined) {
						this._setRelationshipCount(result.datasetRelationshipCount);
					}					
					
					var linksLength = result.links.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						links[linkIndex] = new ibm.tivoli.navmodel.Link({link: result.links[linkIndex], dataset: this, expiration: expiration});
					}
				}
				
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;
				
				kwArgs.onComplete(items, links, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelatedItems"), ioArgs);
			})
		});
	},
	
	/*=====
	queryItems_kwArgs: function(onComplete, onError, query, start, count, properties, metadata, sync) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// query: Object
		//		custom query object to be sent as request body
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// properties: String[]
		//		item property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	 _queryItems: function(/*queryItems_kwArgs*/ kwArgs) {
		// summary:
		//		Returns items associated with the dataset that match
		//		the provided custom query
		var json = dojo.toJson(kwArgs.query, false);
				
		return dojo.xhrPost({
			url: this._getItemsQueryURL(kwArgs),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "customQuery"),
				'Content-Length': json.length,
				'X-Method-Override': 'Custom-GET'
			},
			handleAs: "json",
			postData: json,
			sync: kwArgs.sync,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var items = [];				
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);				
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						items[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
					}
				}
				
				kwArgs.onComplete(items, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_queryItems"), ioArgs);
			})
		});
	},
	
	/*=====
	links_onComplete: function(items, kwArgs){},
	=====*/

	/*=====
	fetchLinks_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of links to fetch; null means all
		// properties: String[]
		//		link property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchLinks: function(/*fetchLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch links from rest service
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getLinksURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var links = [];							
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var linksLength = result.items.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						var link = result.items[linkIndex];
						if (link.datasetId && link.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(link.datasetId);
							links[linkIndex] = new ibm.tivoli.navmodel.Link({link: link, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
						else {									
							links[linkIndex] = new ibm.tivoli.navmodel.Link({link: link, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
					}
				}
				
				// make sure our count is accurate
				this._setLinkCount(result.totalRows);
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;				
				kwArgs.onComplete(links, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchLinks"), ioArgs);
			})
		});
	},
	
	/*=====
	fetchLink_kwArgs: function(itemId, properties, metadata) {
		// linkId: String
		//		link id to get
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchLink: function(/*fetchLink_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual link from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var link = null;
		var exception = null;
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		dojo.xhrGet({
			url: this._getLinkURL(kwArgs.linkId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var allProps = this._isAllProps(kwArgs.properties);
				var propIds = this._getPropIds(kwArgs.properties, allProps);
				link = new ibm.tivoli.navmodel.Link({link: result, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata});			
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_fetchLink");
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return link;
	},

	/*=====
	relationships_onComplete: function(relationships, kwArgs){},
	=====*/
	
	/*=====
	fetchRelationships_kwArgs: function(onComplete, onError, start, count, properties, metadata, sort, condition, sync) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error conditions
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationships to fetch; null means all
		// properties: String[]
		//		relationship property ids to fetch by default (can specify "all")
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
		// sort: String
		//		comma deliminated propId string (preceded with optional +/- indicating ascending/descending)
		//		example:  prop1,-prop2,prop3
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelationships: function(/*fetchRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch relationships from rest service
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		return dojo.xhrGet({
			url: this._getRelationshipsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipViewList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var expiration = this.getNewExpirationTime();
				var relationships = [];				
				if (result.items) {
					var allProps = this._isAllProps(kwArgs.properties);
					var propIds = this._getPropIds(kwArgs.properties, allProps);
					var relLength = result.items.length;
					for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
						var relationship = result.items[relIndex];
						if (relationship.datasetId && relationship.datasetId !== this.getId()) {
							var dataset = this.getDescendentDataset(relationship.datasetId);
							relationships[relIndex] = new ibm.tivoli.navmodel.Relationship({relationship: relationship, dataset: dataset, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}
						else {									
							relationships[relIndex] = new ibm.tivoli.navmodel.Relationship({relationship: relationship, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata, expiration: expiration});
						}						
					}
				}
								
				// make sure our count is accurate
				this._setRelationshipCount(result.totalRows);
					
				// place counts in the arguments map
				kwArgs.numRows = result.numRows;
				kwArgs.filteredRows = result.filteredRows;
				kwArgs.totalRows = result.totalRows;				
				kwArgs.onComplete(relationships, kwArgs);	
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelationships"), ioArgs);
			})
		});
	},

	/*=====
	fetchRelationship_kwArgs: function(relationshipId, properties, metadata) {
		// relationshipId: String
		//		relationship id to fetch
		// properties: String[]
		//		array of property ids to pre-fetch
		// metadata: Boolean
		//		indicates if metadata should be fetched with the properties
	},
	=====*/
	
	_fetchRelationship: function(/*fetchRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Synchronously fetch an individual relationship from the rest service
		// exceptions:
		//		throws ibm.tivoli.navmodel.Exception
		var relationship = null;
		var exception = null;
		
		// need original "param_" values to locate cached dataset	
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		ibm.tivoli.navmodel.Utils.mixinProperties(content, "properties", kwArgs.properties);
		
		dojo.xhrGet({
			url: this._getRelationshipURL(kwArgs.relationshipId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipView")
			},
			handleAs: "json",
			sync: true,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var allProps = this._isAllProps(kwArgs.properties);
				var propIds = this._getPropIds(kwArgs.properties, allProps);
				relationship = new ibm.tivoli.navmodel.Relationship({relationship: result, dataset: this, propIds: propIds, allProperties: allProps, allMetadata: kwArgs.metadata});			
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				exception = this._getException(error, ioArgs, "_fetchRelationship");
			})
		});
		
		if (exception) {
			throw exception;
		}
		
		return relationship;
	},
	
	/*=====
	itemStyles_onComplete: function(itemStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchItemStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: itemStyle_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of item styles to fetch;  null means all
		// sync: Boolean
		//		indicates if fetch is synchronous		
	},
	=====*/
	
	_fetchItemStyles: function(/*fetchItemStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current item styles for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrGet({
			url: this._getItemStylesURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemStyleList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var styles = [];
				var stylesLength = result.items.length;
				for (var index = 0;  index < stylesLength;  ++index) {
					styles[index] = new ibm.tivoli.navmodel.ItemStyle({style: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(styles, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchItemStyles"), ioArgs);
			})
		});
	},

	/*=====
	linkStyles_onComplete: function(linkStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchLinkStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: linkStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of link styles to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchLinkStyles: function(/*fetchLinkStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current link styles for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrGet({
			url: this._getLinkStylesURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkStyleList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var styles = [];
				var stylesLength = result.items.length;
				for (var index = 0;  index < stylesLength;  ++index) {
					styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(styles, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchLinkStyles"), ioArgs);
			})
		});		
	},

	/*=====
	relStyles_onComplete: function(relStyles, kwArgs){},
	=====*/
	
	/*=====
	fetchRelationshipStyles_kwArgs: function(onComplete, onError, start, count, sync) {
		// onComplete: relStyles_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// start: int
		//		start index to fetch
		// count: int
		//		number of relationship styles to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchRelationshipStyles: function(/*fetchRelationshipStyles_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of current relationship styles for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		
		return dojo.xhrGet({
			url: this._getRelationshipStylesURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkStyleList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var styles = [];
				var stylesLength = result.items.length;
				for (var index = 0;  index < stylesLength;  ++index) {
					styles[index] = new ibm.tivoli.navmodel.LinkStyle({style: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(styles, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchRelationshipStyles"), ioArgs);
			})
		});		
	},

	/*=====
	itemStatuses_onComplete: function(itemStatuses, kwArgs){},
	=====*/
	
	/*=====
	fetchStatuses_kwArgs: function(onComplete, onError, type, start, count, sync) {
		// onComplete: itemStatuses_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type for columns: "item", "link", "relationship"
		// start: int
		//		start index to fetch
		// count: int
		//		number of status values to fetch; null means all
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	_fetchStatuses: function(/*fetchStatuses_kwArgs*/ kwArgs) {
		// summary:
		//		Returns list of possible item status values for the Dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);		
		if (kwArgs.type) {
			content.type = kwArgs.type;
		}
		
		return dojo.xhrGet({
			url: this._getStatusURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "statusList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var status = [];
				var statusLength = result.items.length;
				for (var index = 0;  index < statusLength;  ++index) {
					status[index] = new ibm.tivoli.navmodel.Status({status: result.items[index], version: this.getVersion()});
				}
				
				kwArgs.onComplete(status, kwArgs);				
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchStatuses"), ioArgs);
			})
		});
	},

	/*=====
	tasks_onComplete: function(tasks, kwArgs){},
	=====*/
	
	/*=====
	fetchTasks_kwArgs: function(onComplete, onError, taskCategory, selection, itemIds, linkIds, itemIndexes, sync) {
		// onComplete: tasks_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskCategory: String
		//		comma delimited list indicating type of tasks to fetch
		//		legal values are: menu, toolbar, and breadcrumb
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// itemIds: String
		//		comma delimited list of item ids that are selected
		// linkIds, String
		//		comma delimited list of link ids that are selected
		// itemIndexes: String
		//		comma delimited list of item ids or ranges that are selected
		//		ex: "0,1,10,20" or "0-10,30,40-50"
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchTasks: function(/*fetchTasks_kwArgs*/ kwArgs) {
		// summary:
		//		Returns tasks associated with the dataset
		
		// use "selection"
		if (kwArgs.selection) {
			var json = dojo.toJson(kwArgs.selection.getAttributes(), false);
			
			return dojo.xhrPost ({
				url: this._getTasksParamURL(kwArgs),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskList"),
					'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskContext"),
					'Content-Length': json.length
				},
				handleAs: "json",
				postData: json,
				sync: kwArgs.sync,			
				load: dojo.hitch(this, function(result, ioArgs) {
					var tasks = [];
					var rawTasks = result.items;
					var rawTasksLength = rawTasks.length;
					for (var index = 0;  index < rawTasksLength;  ++index) {
						tasks[index] = new ibm.tivoli.navmodel.Task({task: rawTasks[index], dataset: this});
					}
					kwArgs.onComplete(tasks, kwArgs);
				}),
				error: dojo.hitch(this, function(error, ioArgs) {
					kwArgs.onError(error, this._getException(error, ioArgs, "getTasks"), ioArgs);	               
				})
			});
		}
		
		// use "itemIds", "linkIds", and "itemIndexes"
		else {
			// need original "param_" values to locate cached dataset
			var content = ibm.tivoli.navmodel.Utils.getContent();
			this._mixinStandard(content, kwArgs);
			ibm.tivoli.navmodel.Utils.mixinParams(content, this);
			
			// comma delimitted categories (i.e. "menu,toolbar,breadcrumb")
			content.category = kwArgs.taskCategory;

			// indicate what is selected
			if (kwArgs.itemIds) {
				content.itemIds = kwArgs.itemIds;
			}
			if (kwArgs.linkIds) {
				content.linkIds = kwArgs.linkIds;
			}
			if (kwArgs.itemIndexes) {
				content.itemIndexes = kwArgs.itemIndexes;
			}

			return dojo.xhrGet({
				url: this._getTasksURL(),
				headers: {
					'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskList")
				},
				handleAs: "json",
				sync: kwArgs.sync,
				content: content,
				load: dojo.hitch(this, function(result, ioArgs){
					var tasks = [];
					var rawTasks = result.items;
					var rawTasksLength = rawTasks.length;
					for (var index = 0;  index < rawTasksLength;  ++index) {
						tasks[index] = new ibm.tivoli.navmodel.Task({task: rawTasks[index], dataset: this});
					}
					kwArgs.onComplete(tasks, kwArgs);
				}),
				error: dojo.hitch(this, function(error, ioArgs){
					kwArgs.onError(error, this._getException(error, ioArgs, "getTasks()"), ioArgs);
				})
			});
		}
	},

	/*=====
	dataset_onComplete: function(dataset, kwArgs){},
	=====*/
	
	/*=====
	navigate_kwArgs: function(onComplete, onError, taskId, taskCategory, selection, sync) {
		// onComplete: dataset_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// taskId: String
		//		navigation task id
		// taskCategory: String
		//		navigation task category
		// selection: Object
		//		ibm.tivoli.navmodel.Selection object
		//		if selection is specified: itemIds, linkIds, itemIndexes is ignored
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_navigate: function(/*navigate_kwArgs*/ kwArgs) {
		// summary:
		//		Navigate to another Dataset from a navigation task
				
		// use "selection"
		var json = dojo.toJson(kwArgs.selection.getAttributes(), false);
			
		return dojo.xhrPost ({
			url: this._getNavigateParamURL(kwArgs),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "dataset"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "taskContext"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: kwArgs.sync,			
			load: dojo.hitch(this, function(result, ioArgs) {
				var navigateArgs = this.getArgs();
				navigateArgs.dataset = result.id;
				navigateArgs.parameters = {refId: ""};  // CURI compatability: include empty refId so we don't create one
				navigateArgs.defaultProperties = [];
				navigateArgs.defaultLinkProperties = [];
				navigateArgs.defaultRelProperties = [];
				navigateArgs.createEventRouter = false; // will share EventRouter
				navigateArgs.onComplete = null;
				navigateArgs.onError = null;
						
		  		// construct navigation dataset (share EventRouter)
				var dataset = new ibm.tivoli.navmodel.Dataset(navigateArgs, result);
				dataset._setEventRouter(this._getEventRouter());
			
				// pass new dataset back to caller
				kwArgs.onComplete(dataset, kwArgs);		
			}),
			error: dojo.hitch(this, function(error, ioArgs) {
				kwArgs.onError(error, this._getException(error, ioArgs, "navigate"), ioArgs);	               
			})
		});			      
	},

	/*=====
	cols_onComplete: function(cols, kwArgs){},
	=====*/
	
	/*=====
	fetchColumns_kwArgs: function(onComplete, onError, type, properties, start, count, metadata, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// type: String
		//		object type: "item", "link", "relationship" 
		// properties: String[]
		//		array of column ids; or keyword "current", "default", or "all"
		// start: int
		//		start index to fetch
		// count: int
		//		number of items to fetch; null means all
		// metadata: Boolean
		//		indicates if filter metadata should be returned with columns		
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_fetchColumns: function(/*fetchColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Fetch columns from rest service
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		this._mixinStandard(content, kwArgs);		
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.source) {
			content.source = kwArgs.source;
		}
		if (kwArgs.type) {
			content.type = kwArgs.type;
		}
		if (kwArgs.properties) {
			content.properties = kwArgs.properties.join(",");
		}		

		return dojo.xhrGet({
			url: this._getColumnsURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "columnList")
			},
			handleAs: "json",
			sync: kwArgs.sync,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				var cols = [];
				var colsLength = result.items.length;
				for (var index = 0;  index < colsLength;  ++index) {
					cols[index] = new ibm.tivoli.navmodel.Column({column: result.items[index], objectType: kwArgs.type, dataset: this});
				}
				
				kwArgs.onComplete(cols, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_fetchColumns"), ioArgs);
			})
		});
	},

	/*=====
	setColumns_kwArgs: function(onComplete, onError, columns, type, sync) {
		// onComplete: cols_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// columns: ibm.tivoli.navmodel.Column[]
		//		array of columns to be saved
		// type: String
		//		object type: "item", "link", "relationship" 
		// sync: Boolean
		//		indicates if fetch is synchronous
	},
	=====*/
	
	_setColumns: function(/*setColumns_kwArgs*/ kwArgs) {
		// summary:
		//		Save columns to rest service
		
		// pull out raw javascript column data
		var columns = [];
		if (kwArgs.columns) {
			var count = kwArgs.columns.length;
			for (var index = 0;  index < count;  ++index) {
				var col = kwArgs.columns[index].getAttributes();
				delete col.filter;
				columns.push(col);
			}
		}		
		var json = dojo.toJson({columns: columns}, false);
		
		// need original "param_" values to locate cached dataset
		return dojo.xhrPost({
			url: this._getColumnsParamURL(kwArgs),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "columnList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: kwArgs.sync,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_setColumns"), ioArgs);
			})
		});
	},

	/*=====
	item_onComplete: function(item, kwArgs){},
	=====*/
	
	/*=====
	createItem_kwArgs: function(onComplete, onError, item) {
		// onComplete: item_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object
		//		ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/
	
	_createItem: function(/*createItem_kwArgs*/ kwArgs) {
		// summary:
		//			Asynchronously create a new item in the dataset	
		var item = kwArgs.item;
		var json = dojo.toJson(item.getAttributes(), false);
		
		return dojo.xhrPost({
			url: this._getItemParamURL(item.getId()),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemView"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemView"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newItem = new ibm.tivoli.navmodel.Item({item: result, dataset: this, allProperties: true, allMetadata: false});
				kwArgs.onComplete(newItem, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createItem"), ioArgs);
			})
		});
	},

	/*=====
	createItems_kwArgs: function(onComplete, onError, item) {
		// onComplete: items_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// item: Object[]
		//		array of ibm.tivoli.navmodel.Item to be created on server
	},
	=====*/
	
	_createItems: function(/*createItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new items in the dataset
		var itemsObj = {"items": []};
		if (kwArgs.items) {
			var itemsLength = kwArgs.items.length;
			for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
				itemsObj.items[itemIndex] = kwArgs.items[itemIndex].getAttributes();
			}
		}
	
		var json = dojo.toJson(itemsObj, false);
		
		return dojo.xhrPost({
			url: this._getItemsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "itemViewList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newItems = [];				
				if (result.items) {
					var itemsLength = result.items.length;
					for (var itemIndex = 0;  itemIndex < itemsLength;  ++itemIndex) {
						newItems[itemIndex] = new ibm.tivoli.navmodel.Item({item: result.items[itemIndex], dataset: this, allProperties: true, allMetadata: false});
					}
				}
				kwArgs.onComplete(newItems, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createItems"), ioArgs);
			})
		});
	},

	/*=====
	_onComplete: function(kwArgs){},
	=====*/
	
	/*=====
	deleteItem_kwArgs: function(onComplete, onError, itemId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemId: String
		//		id of item to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of item
	},
	=====*/
	
	_deleteItem: function(/*deleteItem_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a item from the dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.remove) {
			content.remove = kwArgs.remove;
		}
		
		return dojo.xhrDelete({
			url: this._getItemURL(kwArgs.itemId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: false,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteItem"), ioArgs);
			})
		});
	},
	
	/*=====
	deleteItems_kwArgs: function(onComplete, onError, itemIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// itemIds: String[]
		//		array of item ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of items
	},
	=====*/
	
	_deleteItems: function(/*deleteItems_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete items from the dataset
		
		// build up the delete filter
		var deleteFilter = {};
		if (kwArgs.itemIds && kwArgs.itemIds.length > 0) {
			deleteFilter.identifiers = kwArgs.itemIds;
		}
		if (kwArgs.condition) {
			deleteFilter.condition = kwArgs.condition;
		}
		if (kwArgs.remove) {
			deleteFilter.remove = kwArgs.remove;
		}
		
		var json = dojo.toJson(deleteFilter, false);
				
		return dojo.xhrPost({
			url: this._getItemsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "deleteFilter"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-DELETE'
			},
			handleAs: "json",
			postData: json,
			sync: false,			
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteItems"), ioArgs);
			})
		});
	},

	/*=====
	link_onComplete: function(link, kwArgs){},
	=====*/
	
	/*=====
	createLink_kwArgs: function(onComplete, onError, link) {
		// onComplete: link_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// link: Object
		//		ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/
	
	_createLink: function(/*createLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new link in the dataset
		var link = kwArgs.link;
		var json = dojo.toJson(link.getAttributes(), false);
		
		return dojo.xhrPost({
			url: this._getLinkParamURL(link.getId()),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkView"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkView"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newLink = new ibm.tivoli.navmodel.Link({link: result, dataset: this, allProperties: true, allMetadata: false});
				kwArgs.onComplete(newLink, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createLink"), ioArgs);
			})
		});
	},
	
	/*=====
	createLinks_kwArgs: function(onComplete, onError, links) {
		// onComplete: links_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// links: Object[]
		//		array of ibm.tivoli.navmodel.Link to be created on server
	},
	=====*/
	
	_createLinks: function(/*createLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new links in the dataset
		var linksObj = {"links": []};
		if (kwArgs.links) {
			var linksLength = kwArgs.links.length;
			for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
				linksObj.links[linkIndex] = kwArgs.links[linkIndex].getAttributes();
			}
		}
	
		var json = dojo.toJson(linksObj, false);
		
		return dojo.xhrPost({
			url: this._getLinksParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "linkViewList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newLinks = [];				
				if (result.items) {
					var linksLength = result.items.length;
					for (var linkIndex = 0;  linkIndex < linksLength;  ++linkIndex) {
						newLinks[linkIndex] = new ibm.tivoli.navmodel.Link({link: result.items[linkIndex], dataset: this, allProperties: true, allMetadata: false});
					}
				}
				kwArgs.onComplete(newLinks, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createLinks"), ioArgs);
			})
		});
	},

	/*=====
	deleteLink_kwArgs: function(onComplete, onError, linkId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkId: String
		//		id of link to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of link
	},
	=====*/
	
	_deleteLink: function(/*deleteLink_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a link from the dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.remove) {
			content.remove = kwArgs.remove;
		}
		
		return dojo.xhrDelete({
			url: this._getLinkURL(kwArgs.linkId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: false,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteLink"), ioArgs);
			})
		});
	},

	/*=====
	deleteLinks_kwArgs: function(onComplete, onError, linkIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// linkIds: String[]
		//		array of link ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of links
	},
	=====*/
	
	_deleteLinks: function(/*deleteLinks_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete links from the dataset
		
		// build up the delete filter
		var deleteFilter = {};
		if (kwArgs.linkIds && kwArgs.linkIds.length > 0) {
			deleteFilter.identifiers = kwArgs.linkIds;
		}
		if (kwArgs.condition) {
			deleteFilter.condition = kwArgs.condition;
		}
		if (kwArgs.remove) {
			deleteFilter.remove = kwArgs.remove;
		}
		
		var json = dojo.toJson(deleteFilter, false);
				
		return dojo.xhrPost({
			url: this._getLinksParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "deleteFilter"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-DELETE'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteLinks"), ioArgs);
			})
		});
	},
	
	/*=====
	relationship_onComplete: function(relationship, kwArgs){},
	=====*/
	
	/*=====
	createRelationship_kwArgs: function(onComplete, onError, relationship) {
		// onComplete: relationship_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object
		//		ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	_createRelationship: function(/*createRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create a new relationship in the dataset
		var relationship = kwArgs.relationship;
		var json = dojo.toJson(relationship.getAttributes(), false);
		
		return dojo.xhrPost({
			url: this._getRelationshipParamURL(relationship.getId()),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipView"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipView"),
				'Content-Length': json.length
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newRelationship = new ibm.tivoli.navmodel.Relationship({relationship: result, dataset: this, allProperties: true, allMetadata: false});				
				kwArgs.onComplete(newRelationship, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createRelationship"), ioArgs);
			})
		});
	},
	
	/*=====
	createRelationships_kwArgs: function(onComplete, onError, relationships) {
		// onComplete: relationships_onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationship: Object[]
		//		array of ibm.tivoli.navmodel.Relationship to be created on server
	},
	=====*/
	
	_createRelationships: function(/*createRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously create new relationships in the dataset
		var relationshipsObj = {"relationships": []};
		if (kwArgs.relationships) {
			var relLength = kwArgs.relationships.length;
			for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
				relationshipsObj.relationships[relIndex] = kwArgs.relationships[relIndex].getAttributes();
			}
		}
	
		var json = dojo.toJson(relationshipsObj, false);
		
		return dojo.xhrPost({
			url: this._getRelationshipsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipViewList"),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "relationshipViewList"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-PUT'
			},
			handleAs: "json",
			postData: json,
			sync: false,
			load: dojo.hitch(this, function(result, ioArgs){
				var newRelationships = [];				
				if (result.items) {
					var relLength = result.items.length;
					for (var relIndex = 0;  relIndex < relLength;  ++relIndex) {
						newRelationships[relIndex] = new ibm.tivoli.navmodel.Relationship({relationship: result.items[relIndex], dataset: this, allProperties: true, allMetadata: false});
					}
				}
				kwArgs.onComplete(newRelationships, kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_createRelationships"), ioArgs);
			})
		});
	},

	/*=====
	deleteRelationship_kwArgs: function(onComplete, onError, relationshipId, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipId: String
		//		id of relationship to be deleted
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationship
	},
	=====*/
	
	_deleteRelationship: function(/*deleteRelationship_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete a relationship from the dataset
		
		// need original "param_" values to locate cached dataset
		var content = ibm.tivoli.navmodel.Utils.getContent();
		ibm.tivoli.navmodel.Utils.mixinParams(content, this);
		if (kwArgs.remove) {
			content.remove = kwArgs.remove;
		}
				
		return dojo.xhrDelete({
			url: this._getRelationshipURL(kwArgs.relationshipId),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion())
			},
			handleAs: "json",
			sync: false,
			content: content,
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteRelationship"), ioArgs);
			})
		});
	},

	/*=====
	deleteRelationships_kwArgs: function(onComplete, onError, relationshipIds, condition, remove) {
		// onComplete: _onComplete
		//		callback for successful fetch
		// onError: _onError
		//		callback for error condition
		// relationshipIds: String[]
		//		array of relationship ids to be deleted
		// condition: String
		//		condition expression string 
		//		example: (prop1 = 123) or (prop2 contains 'abc')
		// remove: Boolean
		//		true indicates simple remove from dataset; false indicates permanent delete of relationships
	},
	=====*/
	
	_deleteRelationships: function(/*deleteRelationships_kwArgs*/ kwArgs) {
		// summary:
		//		Asynchronously delete relationships from the dataset
		
		// build up the delete filter
		var deleteFilter = {};
		if (kwArgs.relationshipIds && kwArgs.relationshipIds.length > 0) {
			deleteFilter.identifiers = kwArgs.relationshipIds;
		}
		if (kwArgs.condition) {
			deleteFilter.condition = kwArgs.condition;
		}
		if (kwArgs.remove) {
			deleteFilter.remove = kwArgs.remove;
		}
		
		var json = dojo.toJson(deleteFilter, false);
			
		return dojo.xhrPost({
			url: this._getRelationshipsParamURL(),
			headers: {
				'Accept': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion()),
				'Content-Type': ibm.tivoli.navmodel.Utils.getContentType(this.getVersion(), "deleteFilter"),
				'Content-Length': json.length,
				'X-Method-Override': 'Multi-DELETE'
			},
			handleAs: "json",
			postData: json,
			sync: false,			
			load: dojo.hitch(this, function(result, ioArgs){
				kwArgs.onComplete(kwArgs);
			}),
			error: dojo.hitch(this, function(error, ioArgs){
				kwArgs.onError(error, this._getException(error, ioArgs, "_deleteRelationships"), ioArgs);
			})
		});
	},	
	
	/////////////////////////////
	// private methods
	/////////////////////////////
	
	_extractContentTypeVersion: function(ioArgs) {
		var version = this.getVersion();
		var contentType = ioArgs.xhr.getResponseHeader("Content-Type");
		if (contentType) {
			var parms = contentType.split(";");
			var length = parms.length;
			for (var index = 0;  index < length;  ++index) {
				if (parms[index].indexOf("version=") === 0) {								
					version = parms[index].substring(8)*1;
				}
			}
		}
		
		return version;		
	},
									
	_mixinStandard: function(content, kwArgs) {
		// summary:
		//		Add common arguments if provided in kwArgs
		if (kwArgs.clearCache) {
			content.clearCache = kwArgs.clearCache;
		}
		
		if (this._optimize) {
			content.optimize = this._optimize;
		}
		
		if (kwArgs.start && kwArgs.start > 0) {
			content.start = kwArgs.start;
		}
		
		if (kwArgs.count && kwArgs.count > 0) {
			content.count = kwArgs.count;
		}
		
		if (kwArgs.metadata) {
			content.metadata = true;
		}
		
		if (kwArgs.sort) {
			content.sort = kwArgs.sort; 
		}
		
		if (kwArgs.condition) {
			content.condition = kwArgs.condition;
		}
	},
	
	_getDatasetURL: function() {
		// summary:
		//		Returns url for for fetching dataset
		
		return this.getArg("baseUrl") + this.getAttribute("uri");
	},
	
	_getOriginItemsURL: function() {
		// summary:
		//		Returns url for fetching origin items
		return  this._getDatasetURL() + "/originItems";
	},

	_getItemsURL: function() {
		// summary:
		//		Returns url for fetching items for a dataset 
		return this._getDatasetURL() + "/items";		
	},

	_getItemsParamURL: function() {
		// summary:
		//		Returns url for fetching items and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getItemsURL());	
	},

	_getItemsQueryURL: function(kwArgs) {
		// summary:
		//		Returns url for custom query of items
		var url = this._getItemsParamURL();
		var questionMark = (url.indexOf('?') > 0);
		
		if (this._optimize) {
			if (!questionMark) {
				questionMark = true;
				url += "?optimize=" + this._optimize;
			}
			else {
				url += "&optimize=" + this._optimize;
			}			
		}
		
		if (kwArgs.start && kwArgs.start > 0) {
			if (!questionMark) {
				questionMark = true;
				url += "?start=" + kwArgs.start;
			}
			else {
				url += "&start=" + kwArgs.start;
			}
		}
		
		if (kwArgs.count && kwArgs.count > 0) {
			if (!questionMark) {
				questionMark = true;
				url += "?count=" + kwArgs.count;
			}
			else {
				url += "&count=" + kwArgs.count;
			}
		}
		
		if (kwArgs.properties && kwArgs.properties.length > 0) {
			if (!questionMark) {
				questionMark = true;
				url += "?properties=" + kwArgs.properties.join(",");
			}
			else {
				url += "&properties=" + kwArgs.properties.join(",");
			}
			
			if (kwArgs.metadata) {
				url += "&metadata=true";
			}
		}
				
		return url;
	},

	_getItemURL: function(itemId) {
		// summary:
		//		Returns url for fetching a specific item
		return this._getItemsURL() + "/" + encodeURIComponent(itemId);		
	},
	
	_getItemParamURL: function(itemId) {
		// summary:
		//		Returns url for item and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getItemURL(itemId));				
	},

	_getRelatedItemsURL: function(itemId) {
		// summary:
		//		Returns url for fetching related items
		return this._getItemURL(itemId) + "/relatedItems";	
	},

	_getLinksURL: function() {
		// summary:
		//		Returns url for fetching links for a dataset
		return this._getDatasetURL() + "/links";
	},

	_getLinksParamURL: function() {
		// summary:
		//		Returns url for fetching links and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getLinksURL()); 		
	},

	_getLinkURL: function(linkId) {
		// summary:
		//		Returns url for fetching a specific link
		return this._getLinksURL() + "/" + encodeURIComponent(linkId);		
	},

	_getLinkParamURL: function(linkId) {
		// summary:
		//		Returns url for link and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getLinkURL(linkId));				
	},

	_getRelationshipsURL: function() {
		// summary:
		//		Returns url for fetching relationships for a dataset
		return this._getDatasetURL() + "/relationships";		
	},

	_getRelationshipsParamURL: function() {
		// summary:
		//		Returns url for fetching relationships and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getRelationshipsURL()); 		
	},

	_getRelationshipURL: function(relationshipId) {
		// summary:
		//		Returns url for fetching a specific relationship
		return this._getRelationshipsURL() + "/" + encodeURIComponent(relationshipId);		
	},
	
	_getRelationshipParamURL: function(relationshipId) {
		// summary:
		//		Returns url for relationship and also includes
		//		all of the "param_" and "version" arguments
		return this._appendParams(this._getRelationshipURL(relationshipId));				
	},
	
	_getTasksURL: function() {
		// summary:
		//		Returns url for fetching tasks for a dataset
		return this._getDatasetURL() + "/tasks";		
	},

	_getTasksParamURL: function(kwArgs) {
		// summary:
		//		Returns url for fetching tasks for a dataset and also includes
		//		all of the "param_" and "version" arguments
		var url = this._appendParams(this._getTasksURL());		
		var questionMark = (url.indexOf('?') > 0);
		
		// optimize
		if (this._optimize) {
			if (!questionMark) {
				questionMark = true;
				url += "?optimize=" + this._optimize;
			}
			else {
				url += "&optimize=" + this._optimize;
			}			
		}
		
		// task category
		if (kwArgs.taskCategory) {
			if (!questionMark) {
				questionMark = true;
				url += "?category=" + kwArgs.taskCategory;
			}
			else {
				url += "&category=" + kwArgs.taskCategory;
			}			
		}
		
		return url;		
	},

	_getTaskURL: function(taskId) {
		// summary:
		//		Returns url for fetching a specific task
		return this._getTasksURL() + "/" + encodeURIComponent(taskId);		
	},

	_getNavigateParamURL: function(kwArgs) {
		// summary:
		//		Returns url for navigating a specific task and also includes
		//		all of the "param_" and "version" arguments
		var url = this._appendParams(this._getTaskURL(kwArgs.taskId));
		var questionMark = (url.indexOf('?') > 0);
		
		// optimize
		if (this._optimize) {
			if (!questionMark) {
				questionMark = true;
				url += "?optimize=" + this._optimize;
			}
			else {
				url += "&optimize=" + this._optimize;
			}			
		}
		
		// task category; should indicate navigation
		if (kwArgs.taskCategory) {
			if (!questionMark) {
				questionMark = true;
				url += "?category=" + kwArgs.taskCategory;
			}
			else {
				url += "&category=" + kwArgs.taskCategory;
			}			
		}
			
		return url;
	},
	
	_getItemStylesURL: function() {
		// summary:
		//		Returns url for fetching current item styles for a dataset
		return this._getDatasetURL() + "/itemStyles";	
	},

	_getLinkStylesURL: function() {
		// summary:
		//		Returns url for fetching current link styles for a dataset
		return this._getDatasetURL() + "/linkStyles";
	},

	_getRelationshipStylesURL: function() {
		// summary:
		//		Returns url for fetching current relationship styles for a dataset
		return this._getDatasetURL() + "/relationshipStyles";
	},

	_getStatusURL: function() {
		// summary:
		//		Returns url for fetching possible status values
		return this._getDatasetURL() + "/status";
	},

	_getColumnsURL: function() {
		// summary:
		//		Returns url for fetching current columns for a dataset
		return this._getDatasetURL() + "/columns";
	},
	
	_getColumnsParamURL: function(kwArgs) {
		// summary:
		//		Returns url for saving current tasks and also includes
		//		all of the "param_" and "version" arguments
		var url = this._appendParams(this._getColumnsURL());
		var questionMark = (url.indexOf('?') > 0);
		
		// type
		if (kwArgs.type) {
			if (!questionMark) {
				questionMark = true;
				url += "?type=" + kwArgs.type;
			}
			else {
				url += "&type=" + kwArgs.type;
			}			
		}
			
		return url;
	},

	_appendParams: function(url) {
		// summary:
		//		Append "param_" and "version" request parameters to the url
		//		so that dataset can be obtained from the REST service cache
		var questionMark = false;
		
		// add "param_" parms
		var parameters = this.getArg("parameters");
		for (var parm in parameters) {
			if (parameters.hasOwnProperty(parm)) {
				if (!questionMark) {
					questionMark = true;
					url += "?";
				}
				else {
					url += "&";
				}
				url += ("param_" + parm + "=" + parameters[parm]);				
			}
		}
		
		// IE will sometimes return http requests from cache without unique parms
		var now = new Date();
		if (!questionMark) {
			questionMark = true;
			url += "?d=" + now.getTime() + "&r=" + Math.random();
		}
		else {
			url += "&d=" + now.getTime() + "&r=" + Math.random();
		}	
		
		return url;
	},
	
	_getException: function(error, ioArgs, functionName) {
		return ibm.tivoli.navmodel.Utils.getException(error, ioArgs, functionName, this._trace, this.getVersion());
	},
	
	_isAllProps: function(properties) {
		return (properties && properties.length === 1 && properties[0] === "all");
	},
	
	_getPropIds: function(properties, allProps) {
		if (properties && properties.length > 0 && !allProps) {
			return properties;
		}
		
		return null;
	}
});

}

if(!dojo._hasResource["dojo.dnd.common"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.common"] = true;
dojo.provide("dojo.dnd.common");

dojo.dnd.getCopyKeyState = dojo.isCopyKey;

dojo.dnd._uniqueId = 0;
dojo.dnd.getUniqueId = function(){
	// summary:
	//		returns a unique string for use with any DOM element
	var id;
	do{
		id = dojo._scopeName + "Unique" + (++dojo.dnd._uniqueId);
	}while(dojo.byId(id));
	return id;
};

dojo.dnd._empty = {};

dojo.dnd.isFormElement = function(/*Event*/ e){
	// summary:
	//		returns true if user clicked on a form element
	var t = e.target;
	if(t.nodeType == 3 /*TEXT_NODE*/){
		t = t.parentNode;
	}
	return " button textarea input select option ".indexOf(" " + t.tagName.toLowerCase() + " ") >= 0;	// Boolean
};

}

if(!dojo._hasResource["dojo.dnd.autoscroll"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.autoscroll"] = true;
dojo.provide("dojo.dnd.autoscroll");

dojo.dnd.getViewport = function(){
	// summary:
	//		Returns a viewport size (visible part of the window)

	// TODO: remove this when getViewport() moved to dojo core, see #7028

	// FIXME: need more docs!!
	var d = dojo.doc, dd = d.documentElement, w = window, b = dojo.body();
	if(dojo.isMozilla){
		return {w: dd.clientWidth, h: w.innerHeight};	// Object
	}else if(!dojo.isOpera && w.innerWidth){
		return {w: w.innerWidth, h: w.innerHeight};		// Object
	}else if (!dojo.isOpera && dd && dd.clientWidth){
		return {w: dd.clientWidth, h: dd.clientHeight};	// Object
	}else if (b.clientWidth){
		return {w: b.clientWidth, h: b.clientHeight};	// Object
	}
	return null;	// Object
};

dojo.dnd.V_TRIGGER_AUTOSCROLL = 32;
dojo.dnd.H_TRIGGER_AUTOSCROLL = 32;

dojo.dnd.V_AUTOSCROLL_VALUE = 16;
dojo.dnd.H_AUTOSCROLL_VALUE = 16;

dojo.dnd.autoScroll = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the window, if
	//		necesary
	// e: Event
	//		onmousemove event

	// FIXME: needs more docs!
	var v = dojo.dnd.getViewport(), dx = 0, dy = 0;
	if(e.clientX < dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = -dojo.dnd.H_AUTOSCROLL_VALUE;
	}else if(e.clientX > v.w - dojo.dnd.H_TRIGGER_AUTOSCROLL){
		dx = dojo.dnd.H_AUTOSCROLL_VALUE;
	}
	if(e.clientY < dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = -dojo.dnd.V_AUTOSCROLL_VALUE;
	}else if(e.clientY > v.h - dojo.dnd.V_TRIGGER_AUTOSCROLL){
		dy = dojo.dnd.V_AUTOSCROLL_VALUE;
	}
	window.scrollBy(dx, dy);
};

dojo.dnd._validNodes = {"div": 1, "p": 1, "td": 1};
dojo.dnd._validOverflow = {"auto": 1, "scroll": 1};

dojo.dnd.autoScrollNodes = function(e){
	// summary:
	//		a handler for onmousemove event, which scrolls the first avaialble
	//		Dom element, it falls back to dojo.dnd.autoScroll()
	// e: Event
	//		onmousemove event

	// FIXME: needs more docs!
	for(var n = e.target; n;){
		if(n.nodeType == 1 && (n.tagName.toLowerCase() in dojo.dnd._validNodes)){
			var s = dojo.getComputedStyle(n);
			if(s.overflow.toLowerCase() in dojo.dnd._validOverflow){
				var b = dojo._getContentBox(n, s), t = dojo.position(n, true);
				//console.log(b.l, b.t, t.x, t.y, n.scrollLeft, n.scrollTop);
				var w = Math.min(dojo.dnd.H_TRIGGER_AUTOSCROLL, b.w / 2), 
					h = Math.min(dojo.dnd.V_TRIGGER_AUTOSCROLL, b.h / 2),
					rx = e.pageX - t.x, ry = e.pageY - t.y, dx = 0, dy = 0;
				if(dojo.isWebKit || dojo.isOpera){
					// FIXME: this code should not be here, it should be taken into account 
					// either by the event fixing code, or the dojo.position()
					// FIXME: this code doesn't work on Opera 9.5 Beta
					rx += dojo.body().scrollLeft, ry += dojo.body().scrollTop;
				}
				if(rx > 0 && rx < b.w){
					if(rx < w){
						dx = -w;
					}else if(rx > b.w - w){
						dx = w;
					}
				}
				//console.log("ry =", ry, "b.h =", b.h, "h =", h);
				if(ry > 0 && ry < b.h){
					if(ry < h){
						dy = -h;
					}else if(ry > b.h - h){
						dy = h;
					}
				}
				var oldLeft = n.scrollLeft, oldTop = n.scrollTop;
				n.scrollLeft = n.scrollLeft + dx;
				n.scrollTop  = n.scrollTop  + dy;
				if(oldLeft != n.scrollLeft || oldTop != n.scrollTop){ return; }
			}
		}
		try{
			n = n.parentNode;
		}catch(x){
			n = null;
		}
	}
	dojo.dnd.autoScroll(e);
};

}

if(!dojo._hasResource["dojo.dnd.Mover"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Mover"] = true;
dojo.provide("dojo.dnd.Mover");




dojo.declare("dojo.dnd.Mover", null, {
	constructor: function(node, e, host){
		// summary:
		//		an object, which makes a node follow the mouse. 
		//		Used as a default mover, and as a base class for custom movers.
		// node: Node
		//		a node (or node's id) to be moved
		// e: Event
		//		a mouse event, which started the move;
		//		only pageX and pageY properties are used
		// host: Object?
		//		object which implements the functionality of the move,
		//	 	and defines proper events (onMoveStart and onMoveStop)
		this.node = dojo.byId(node);
		this.marginBox = {l: e.pageX, t: e.pageY};
		this.mouseButton = e.button;
		var h = this.host = host, d = node.ownerDocument, 
			firstEvent = dojo.connect(d, "onmousemove", this, "onFirstMove");
		this.events = [
			dojo.connect(d, "onmousemove", this, "onMouseMove"),
			dojo.connect(d, "onmouseup",   this, "onMouseUp"),
			// cancel text selection and text dragging
			dojo.connect(d, "ondragstart",   dojo.stopEvent),
			dojo.connect(d.body, "onselectstart", dojo.stopEvent),
			firstEvent
		];
		// notify that the move has started
		if(h && h.onMoveStart){
			h.onMoveStart(this);
		}
	},
	// mouse event processors
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove
		// e: Event
		//		mouse event
		dojo.dnd.autoScroll(e);
		var m = this.marginBox;
		this.host.onMove(this, {l: m.l + e.pageX, t: m.t + e.pageY}, e);
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		if(dojo.isWebKit && dojo.isMac && this.mouseButton == 2 ? 
				e.button == 0 : this.mouseButton == e.button){
			this.destroy();
		}
		dojo.stopEvent(e);
	},
	// utilities
	onFirstMove: function(e){
		// summary:
		//		makes the node absolute; it is meant to be called only once. 
		// 		relative and absolutely positioned nodes are assumed to use pixel units
		var s = this.node.style, l, t, h = this.host;
		switch(s.position){
			case "relative":
			case "absolute":
				// assume that left and top values are in pixels already
				l = Math.round(parseFloat(s.left)) || 0;
				t = Math.round(parseFloat(s.top)) || 0;
				break;
			default:
				s.position = "absolute";	// enforcing the absolute mode
				var m = dojo.marginBox(this.node);
				// event.pageX/pageY (which we used to generate the initial
				// margin box) includes padding and margin set on the body.
				// However, setting the node's position to absolute and then
				// doing dojo.marginBox on it *doesn't* take that additional
				// space into account - so we need to subtract the combined
				// padding and margin.  We use getComputedStyle and
				// _getMarginBox/_getContentBox to avoid the extra lookup of
				// the computed style. 
				var b = dojo.doc.body;
				var bs = dojo.getComputedStyle(b);
				var bm = dojo._getMarginBox(b, bs);
				var bc = dojo._getContentBox(b, bs);
				l = m.l - (bc.l - bm.l);
				t = m.t - (bc.t - bm.t);
				break;
		}
		this.marginBox.l = l - this.marginBox.l;
		this.marginBox.t = t - this.marginBox.t;
		if(h && h.onFirstMove){
			h.onFirstMove(this, e);
		}
		dojo.disconnect(this.events.pop());
	},
	destroy: function(){
		// summary:
		//		stops the move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		// undo global settings
		var h = this.host;
		if(h && h.onMoveStop){
			h.onMoveStop(this);
		}
		// destroy objects
		this.events = this.node = this.host = null;
	}
});

}

if(!dojo._hasResource["dojo.dnd.Moveable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.Moveable"] = true;
dojo.provide("dojo.dnd.Moveable");



/*=====
dojo.declare("dojo.dnd.__MoveableArgs", [], {
	// handle: Node||String
	//		A node (or node's id), which is used as a mouse handle.
	//		If omitted, the node itself is used as a handle.
	handle: null,

	// delay: Number
	//		delay move by this number of pixels
	delay: 0,

	// skip: Boolean
	//		skip move of form elements
	skip: false,

	// mover: Object
	//		a constructor of custom Mover
	mover: dojo.dnd.Mover
});
=====*/

dojo.declare("dojo.dnd.Moveable", null, {
	// object attributes (for markup)
	handle: "",
	delay: 0,
	skip: false,
	
	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.__MoveableArgs?
		//		optional parameters
		this.node = dojo.byId(node);
		if(!params){ params = {}; }
		this.handle = params.handle ? dojo.byId(params.handle) : null;
		if(!this.handle){ this.handle = this.node; }
		this.delay = params.delay > 0 ? params.delay : 0;
		this.skip  = params.skip;
		this.mover = params.mover ? params.mover : dojo.dnd.Mover;
		this.events = [
			dojo.connect(this.handle, "onmousedown", this, "onMouseDown"),
			// cancel text selection and text dragging
			dojo.connect(this.handle, "ondragstart",   this, "onSelectStart"),
			dojo.connect(this.handle, "onselectstart", this, "onSelectStart")
		];
	},

	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.Moveable(node, params);
	},

	// methods
	destroy: function(){
		// summary:
		//		stops watching for possible move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		this.events = this.node = this.handle = null;
	},
	
	// mouse event processors
	onMouseDown: function(e){
		// summary:
		//		event processor for onmousedown, creates a Mover for the node
		// e: Event
		//		mouse event
		if(this.skip && dojo.dnd.isFormElement(e)){ return; }
		if(this.delay){
			this.events.push(
				dojo.connect(this.handle, "onmousemove", this, "onMouseMove"),
				dojo.connect(this.handle, "onmouseup", this, "onMouseUp")
			);
			this._lastX = e.pageX;
			this._lastY = e.pageY;
		}else{
			this.onDragDetected(e);
		}
		dojo.stopEvent(e);
	},
	onMouseMove: function(e){
		// summary:
		//		event processor for onmousemove, used only for delayed drags
		// e: Event
		//		mouse event
		if(Math.abs(e.pageX - this._lastX) > this.delay || Math.abs(e.pageY - this._lastY) > this.delay){
			this.onMouseUp(e);
			this.onDragDetected(e);
		}
		dojo.stopEvent(e);
	},
	onMouseUp: function(e){
		// summary:
		//		event processor for onmouseup, used only for delayed drags
		// e: Event
		//		mouse event
		for(var i = 0; i < 2; ++i){
			dojo.disconnect(this.events.pop());
		}
		dojo.stopEvent(e);
	},
	onSelectStart: function(e){
		// summary:
		//		event processor for onselectevent and ondragevent
		// e: Event
		//		mouse event
		if(!this.skip || !dojo.dnd.isFormElement(e)){
			dojo.stopEvent(e);
		}
	},
	
	// local events
	onDragDetected: function(/* Event */ e){
		// summary:
		//		called when the drag is detected;
		//		responsible for creation of the mover
		new this.mover(this.node, e, this);
	},
	onMoveStart: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called before every move operation
		dojo.publish("/dnd/move/start", [mover]);
		dojo.addClass(dojo.body(), "dojoMove"); 
		dojo.addClass(this.node, "dojoMoveItem"); 
	},
	onMoveStop: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called after every move operation
		dojo.publish("/dnd/move/stop", [mover]);
		dojo.removeClass(dojo.body(), "dojoMove");
		dojo.removeClass(this.node, "dojoMoveItem");
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover, /* Event */ e){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		
		// default implementation does nothing
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop, /* Event */ e){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		this.onMoving(mover, leftTop);
		var s = mover.node.style;
		s.left = leftTop.l + "px";
		s.top  = leftTop.t + "px";
		this.onMoved(mover, leftTop);
	},
	onMoving: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called before every incremental move; can be overwritten.
		
		// default implementation does nothing
	},
	onMoved: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called after every incremental move; can be overwritten.
		
		// default implementation does nothing
	}
});

}

if(!dojo._hasResource["dojo.dnd.move"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.move"] = true;
dojo.provide("dojo.dnd.move");




/*=====
dojo.declare("dojo.dnd.move.__constrainedMoveableArgs", [dojo.dnd.__MoveableArgs], {
	// constraints: Function
	//		Calculates a constraint box.
	//		It is called in a context of the moveable object.
	constraints: function(){},

	// within: Boolean
	//		restrict move within boundaries.
	within: false
});
=====*/

dojo.declare("dojo.dnd.move.constrainedMoveable", dojo.dnd.Moveable, {
	// object attributes (for markup)
	constraints: function(){},
	within: false,
	
	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.constrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary:
		//		an object that makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__constrainedMoveableArgs?
		//		an optional object with additional parameters;
		//		the rest is passed to the base class
		if(!params){ params = {}; }
		this.constraints = params.constraints;
		this.within = params.within;
	},
	onFirstMove: function(/* dojo.dnd.Mover */ mover){
		// summary:
		//		called during the very first move notification;
		//		can be used to initialize coordinates, can be overwritten.
		var c = this.constraintBox = this.constraints.call(this, mover);
		c.r = c.l + c.w;
		c.b = c.t + c.h;
		if(this.within){
			var mb = dojo.marginBox(mover.node);
			c.r -= mb.w;
			c.b -= mb.h;
		}
	},
	onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
		// summary:
		//		called during every move notification;
		//		should actually move the node; can be overwritten.
		var c = this.constraintBox, s = mover.node.style;
		s.left = (leftTop.l < c.l ? c.l : c.r < leftTop.l ? c.r : leftTop.l) + "px";
		s.top  = (leftTop.t < c.t ? c.t : c.b < leftTop.t ? c.b : leftTop.t) + "px";
	}
});

/*=====
dojo.declare("dojo.dnd.move.__boxConstrainedMoveableArgs", [dojo.dnd.move.__constrainedMoveableArgs], {
	// box: Object
	//		a constraint box
	box: {}
});
=====*/

dojo.declare("dojo.dnd.move.boxConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// box:
	//		object attributes (for markup)
	box: {},
	
	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.boxConstrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__boxConstrainedMoveableArgs?
		//		an optional object with parameters
		var box = params && params.box;
		this.constraints = function(){ return box; };
	}
});

/*=====
dojo.declare("dojo.dnd.move.__parentConstrainedMoveableArgs", [dojo.dnd.move.__constrainedMoveableArgs], {
	// area: String
	//		A parent's area to restrict the move.
	//		Can be "margin", "border", "padding", or "content".
	area: ""
});
=====*/

dojo.declare("dojo.dnd.move.parentConstrainedMoveable", dojo.dnd.move.constrainedMoveable, {
	// area:
	//		object attributes (for markup)
	area: "content",

	// markup methods
	markupFactory: function(params, node){
		return new dojo.dnd.move.parentConstrainedMoveable(node, params);
	},

	constructor: function(node, params){
		// summary:
		//		an object, which makes a node moveable
		// node: Node
		//		a node (or node's id) to be moved
		// params: dojo.dnd.move.__parentConstrainedMoveableArgs?
		//		an optional object with parameters
		var area = params && params.area;
		this.constraints = function(){
			var n = this.node.parentNode, 
				s = dojo.getComputedStyle(n), 
				mb = dojo._getMarginBox(n, s);
			if(area == "margin"){
				return mb;	// Object
			}
			var t = dojo._getMarginExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "border"){
				return mb;	// Object
			}
			t = dojo._getBorderExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			if(area == "padding"){
				return mb;	// Object
			}
			t = dojo._getPadExtents(n, s);
			mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
			return mb;	// Object
		};
	}
});

// WARNING: below are obsolete objects, instead of custom movers use custom moveables (above)

dojo.dnd.move.constrainedMover = function(fun, within){
	// summary:
	//		returns a constrained version of dojo.dnd.Mover
	// description:
	//		this function produces n object, which will put a constraint on 
	//		the margin box of dragged object in absolute coordinates
	// fun: Function
	//		called on drag, and returns a constraint box
	// within: Boolean
	//		if true, constraints the whole dragged object withtin the rectangle, 
	//		otherwise the constraint is applied to the left-top corner

	dojo.deprecated("dojo.dnd.move.constrainedMover, use dojo.dnd.move.constrainedMoveable instead");
	var mover = function(node, e, notifier){
		dojo.dnd.Mover.call(this, node, e, notifier);
	};
	dojo.extend(mover, dojo.dnd.Mover.prototype);
	dojo.extend(mover, {
		onMouseMove: function(e){
			// summary: event processor for onmousemove
			// e: Event: mouse event
			dojo.dnd.autoScroll(e);
			var m = this.marginBox, c = this.constraintBox,
				l = m.l + e.pageX, t = m.t + e.pageY;
			l = l < c.l ? c.l : c.r < l ? c.r : l;
			t = t < c.t ? c.t : c.b < t ? c.b : t;
			this.host.onMove(this, {l: l, t: t});
		},
		onFirstMove: function(){
			// summary: called once to initialize things; it is meant to be called only once
			dojo.dnd.Mover.prototype.onFirstMove.call(this);
			var c = this.constraintBox = fun.call(this);
			c.r = c.l + c.w;
			c.b = c.t + c.h;
			if(within){
				var mb = dojo.marginBox(this.node);
				c.r -= mb.w;
				c.b -= mb.h;
			}
		}
	});
	return mover;	// Object
};

dojo.dnd.move.boxConstrainedMover = function(box, within){
	// summary:
	//		a specialization of dojo.dnd.constrainedMover, which constrains to the specified box
	// box: Object
	//		a constraint box (l, t, w, h)
	// within: Boolean
	//		if true, constraints the whole dragged object withtin the rectangle, 
	//		otherwise the constraint is applied to the left-top corner

	dojo.deprecated("dojo.dnd.move.boxConstrainedMover, use dojo.dnd.move.boxConstrainedMoveable instead");
	return dojo.dnd.move.constrainedMover(function(){ return box; }, within);	// Object
};

dojo.dnd.move.parentConstrainedMover = function(area, within){
	// summary:
	//		a specialization of dojo.dnd.constrainedMover, which constrains to the parent node
	// area: String
	//		"margin" to constrain within the parent's margin box, "border" for the border box,
	//		"padding" for the padding box, and "content" for the content box; "content" is the default value.
	// within: Boolean
	//		if true, constraints the whole dragged object within the rectangle, 
	//		otherwise the constraint is applied to the left-top corner

	dojo.deprecated("dojo.dnd.move.parentConstrainedMover, use dojo.dnd.move.parentConstrainedMoveable instead");
	var fun = function(){
		var n = this.node.parentNode, 
			s = dojo.getComputedStyle(n), 
			mb = dojo._getMarginBox(n, s);
		if(area == "margin"){
			return mb;	// Object
		}
		var t = dojo._getMarginExtents(n, s);
		mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
		if(area == "border"){
			return mb;	// Object
		}
		t = dojo._getBorderExtents(n, s);
		mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
		if(area == "padding"){
			return mb;	// Object
		}
		t = dojo._getPadExtents(n, s);
		mb.l += t.l, mb.t += t.t, mb.w -= t.w, mb.h -= t.h;
		return mb;	// Object
	};
	return dojo.dnd.move.constrainedMover(fun, within);	// Object
};

// patching functions one level up for compatibility

dojo.dnd.constrainedMover = dojo.dnd.move.constrainedMover;
dojo.dnd.boxConstrainedMover = dojo.dnd.move.boxConstrainedMover;
dojo.dnd.parentConstrainedMover = dojo.dnd.move.parentConstrainedMover;

}

if(!dojo._hasResource["dojo.dnd.TimedMoveable"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.dnd.TimedMoveable"] = true;
dojo.provide("dojo.dnd.TimedMoveable");



/*=====
dojo.declare("dojo.dnd.__TimedMoveableArgs", [dojo.dnd.__MoveableArgs], {
	// timeout: Number
	//		delay move by this number of ms,
	//		accumulating position changes during the timeout
	timeout: 0
});
=====*/

(function(){
	// precalculate long expressions
	var oldOnMove = dojo.dnd.Moveable.prototype.onMove;
		
	dojo.declare("dojo.dnd.TimedMoveable", dojo.dnd.Moveable, {
		// summary:
		//		A specialized version of Moveable to support an FPS throttling.
		//		This class puts an upper restriction on FPS, which may reduce 
		//		the CPU load. The additional parameter "timeout" regulates
		//		the delay before actually moving the moveable object.
		
		// object attributes (for markup)
		timeout: 40,	// in ms, 40ms corresponds to 25 fps
	
		constructor: function(node, params){
			// summary:
			//		an object that makes a node moveable with a timer
			// node: Node||String
			//		a node (or node's id) to be moved
			// params: dojo.dnd.__TimedMoveableArgs
			//		object with additional parameters.
			
			// sanitize parameters
			if(!params){ params = {}; }
			if(params.timeout && typeof params.timeout == "number" && params.timeout >= 0){
				this.timeout = params.timeout;
			}
		},
	
		// markup methods
		markupFactory: function(params, node){
			return new dojo.dnd.TimedMoveable(node, params);
		},
	
		onMoveStop: function(/* dojo.dnd.Mover */ mover){
			if(mover._timer){
				// stop timer
				clearTimeout(mover._timer)
				// reflect the last received position
				oldOnMove.call(this, mover, mover._leftTop)
			}
			dojo.dnd.Moveable.prototype.onMoveStop.apply(this, arguments);
		},
		onMove: function(/* dojo.dnd.Mover */ mover, /* Object */ leftTop){
			mover._leftTop = leftTop;
			if(!mover._timer){
				var _t = this;	// to avoid using dojo.hitch()
				mover._timer = setTimeout(function(){
					// we don't have any pending requests
					mover._timer = null;
					// reflect the last received position
					oldOnMove.call(_t, mover, mover._leftTop);
				}, this.timeout);
			}
		}
	});
})();

}

if(!dojo._hasResource["dojo.fx.Toggler"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx.Toggler"] = true;
dojo.provide("dojo.fx.Toggler");

dojo.declare("dojo.fx.Toggler", null, {
	// summary:
	//		A simple `dojo.Animation` toggler API.
	//
	// description:
	//		class constructor for an animation toggler. It accepts a packed
	//		set of arguments about what type of animation to use in each
	//		direction, duration, etc. All available members are mixed into 
	//		these animations from the constructor (for example, `node`, 
	//		`showDuration`, `hideDuration`). 
	//
	// example:
	//	|	var t = new dojo.fx.Toggler({
	//	|		node: "nodeId",
	//	|		showDuration: 500,
	//	|		// hideDuration will default to "200"
	//	|		showFunc: dojo.fx.wipeIn, 
	//	|		// hideFunc will default to "fadeOut"
	//	|	});
	//	|	t.show(100); // delay showing for 100ms
	//	|	// ...time passes...
	//	|	t.hide();

	// node: DomNode
	//		the node to target for the showing and hiding animations
	node: null,

	// showFunc: Function
	//		The function that returns the `dojo.Animation` to show the node
	showFunc: dojo.fadeIn,

	// hideFunc: Function	
	//		The function that returns the `dojo.Animation` to hide the node
	hideFunc: dojo.fadeOut,

	// showDuration:
	//		Time in milliseconds to run the show Animation
	showDuration: 200,

	// hideDuration:
	//		Time in milliseconds to run the hide Animation
	hideDuration: 200,

	// FIXME: need a policy for where the toggler should "be" the next
	// time show/hide are called if we're stopped somewhere in the
	// middle.
	// FIXME: also would be nice to specify individual showArgs/hideArgs mixed into
	// each animation individually. 
	// FIXME: also would be nice to have events from the animations exposed/bridged

	/*=====
	_showArgs: null,
	_showAnim: null,

	_hideArgs: null,
	_hideAnim: null,

	_isShowing: false,
	_isHiding: false,
	=====*/

	constructor: function(args){
		var _t = this;

		dojo.mixin(_t, args);
		_t.node = args.node;
		_t._showArgs = dojo.mixin({}, args);
		_t._showArgs.node = _t.node;
		_t._showArgs.duration = _t.showDuration;
		_t.showAnim = _t.showFunc(_t._showArgs);

		_t._hideArgs = dojo.mixin({}, args);
		_t._hideArgs.node = _t.node;
		_t._hideArgs.duration = _t.hideDuration;
		_t.hideAnim = _t.hideFunc(_t._hideArgs);

		dojo.connect(_t.showAnim, "beforeBegin", dojo.hitch(_t.hideAnim, "stop", true));
		dojo.connect(_t.hideAnim, "beforeBegin", dojo.hitch(_t.showAnim, "stop", true));
	},

	show: function(delay){
		// summary: Toggle the node to showing
		// delay: Integer?
		//		Ammount of time to stall playing the show animation
		return this.showAnim.play(delay || 0);
	},

	hide: function(delay){
		// summary: Toggle the node to hidden
		// delay: Integer?
		//		Ammount of time to stall playing the hide animation
		return this.hideAnim.play(delay || 0);
	}
});

}

if(!dojo._hasResource["dojo.fx"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.fx"] = true;
dojo.provide("dojo.fx");
 // FIXME: remove this back-compat require in 2.0 
/*=====
dojo.fx = {
	// summary: Effects library on top of Base animations
};
=====*/
(function(){
	
	var d = dojo, 
		_baseObj = {
			_fire: function(evt, args){
				if(this[evt]){
					this[evt].apply(this, args||[]);
				}
				return this;
			}
		};

	var _chain = function(animations){
		this._index = -1;
		this._animations = animations||[];
		this._current = this._onAnimateCtx = this._onEndCtx = null;

		this.duration = 0;
		d.forEach(this._animations, function(a){
			this.duration += a.duration;
			if(a.delay){ this.duration += a.delay; }
		}, this);
	};
	d.extend(_chain, {
		_onAnimate: function(){
			this._fire("onAnimate", arguments);
		},
		_onEnd: function(){
			d.disconnect(this._onAnimateCtx);
			d.disconnect(this._onEndCtx);
			this._onAnimateCtx = this._onEndCtx = null;
			if(this._index + 1 == this._animations.length){
				this._fire("onEnd");
			}else{
				// switch animations
				this._current = this._animations[++this._index];
				this._onAnimateCtx = d.connect(this._current, "onAnimate", this, "_onAnimate");
				this._onEndCtx = d.connect(this._current, "onEnd", this, "_onEnd");
				this._current.play(0, true);
			}
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			if(!this._current){ this._current = this._animations[this._index = 0]; }
			if(!gotoStart && this._current.status() == "playing"){ return this; }
			var beforeBegin = d.connect(this._current, "beforeBegin", this, function(){
					this._fire("beforeBegin");
				}),
				onBegin = d.connect(this._current, "onBegin", this, function(arg){
					this._fire("onBegin", arguments);
				}),
				onPlay = d.connect(this._current, "onPlay", this, function(arg){
					this._fire("onPlay", arguments);
					d.disconnect(beforeBegin);
					d.disconnect(onBegin);
					d.disconnect(onPlay);
				});
			if(this._onAnimateCtx){
				d.disconnect(this._onAnimateCtx);
			}
			this._onAnimateCtx = d.connect(this._current, "onAnimate", this, "_onAnimate");
			if(this._onEndCtx){
				d.disconnect(this._onEndCtx);
			}
			this._onEndCtx = d.connect(this._current, "onEnd", this, "_onEnd");
			this._current.play.apply(this._current, arguments);
			return this;
		},
		pause: function(){
			if(this._current){
				var e = d.connect(this._current, "onPause", this, function(arg){
						this._fire("onPause", arguments);
						d.disconnect(e);
					});
				this._current.pause();
			}
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			this.pause();
			var offset = this.duration * percent;
			this._current = null;
			d.some(this._animations, function(a){
				if(a.duration <= offset){
					this._current = a;
					return true;
				}
				offset -= a.duration;
				return false;
			});
			if(this._current){
				this._current.gotoPercent(offset / this._current.duration, andPlay);
			}
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			if(this._current){
				if(gotoEnd){
					for(; this._index + 1 < this._animations.length; ++this._index){
						this._animations[this._index].stop(true);
					}
					this._current = this._animations[this._index];
				}
				var e = d.connect(this._current, "onStop", this, function(arg){
						this._fire("onStop", arguments);
						d.disconnect(e);
					});
				this._current.stop();
			}
			return this;
		},
		status: function(){
			return this._current ? this._current.status() : "stopped";
		},
		destroy: function(){
			if(this._onAnimateCtx){ d.disconnect(this._onAnimateCtx); }
			if(this._onEndCtx){ d.disconnect(this._onEndCtx); }
		}
	});
	d.extend(_chain, _baseObj);

	dojo.fx.chain = function(/*dojo.Animation[]*/ animations){
		// summary: 
		//		Chain a list of `dojo.Animation`s to run in sequence
		//
		// description:
		//		Return a `dojo.Animation` which will play all passed
		//		`dojo.Animation` instances in sequence, firing its own
		//		synthesized events simulating a single animation. (eg:
		//		onEnd of this animation means the end of the chain, 
		//		not the individual animations within)
		//
		// example:
		//	Once `node` is faded out, fade in `otherNode`
		//	|	dojo.fx.chain([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		return new _chain(animations) // dojo.Animation
	};

	var _combine = function(animations){
		this._animations = animations||[];
		this._connects = [];
		this._finished = 0;

		this.duration = 0;
		d.forEach(animations, function(a){
			var duration = a.duration;
			if(a.delay){ duration += a.delay; }
			if(this.duration < duration){ this.duration = duration; }
			this._connects.push(d.connect(a, "onEnd", this, "_onEnd"));
		}, this);
		
		this._pseudoAnimation = new d.Animation({curve: [0, 1], duration: this.duration});
		var self = this;
		d.forEach(["beforeBegin", "onBegin", "onPlay", "onAnimate", "onPause", "onStop", "onEnd"], 
			function(evt){
				self._connects.push(d.connect(self._pseudoAnimation, evt,
					function(){ self._fire(evt, arguments); }
				));
			}
		);
	};
	d.extend(_combine, {
		_doAction: function(action, args){
			d.forEach(this._animations, function(a){
				a[action].apply(a, args);
			});
			return this;
		},
		_onEnd: function(){
			if(++this._finished > this._animations.length){
				this._fire("onEnd");
			}
		},
		_call: function(action, args){
			var t = this._pseudoAnimation;
			t[action].apply(t, args);
		},
		play: function(/*int?*/ delay, /*Boolean?*/ gotoStart){
			this._finished = 0;
			this._doAction("play", arguments);
			this._call("play", arguments);
			return this;
		},
		pause: function(){
			this._doAction("pause", arguments);
			this._call("pause", arguments);
			return this;
		},
		gotoPercent: function(/*Decimal*/percent, /*Boolean?*/ andPlay){
			var ms = this.duration * percent;
			d.forEach(this._animations, function(a){
				a.gotoPercent(a.duration < ms ? 1 : (ms / a.duration), andPlay);
			});
			this._call("gotoPercent", arguments);
			return this;
		},
		stop: function(/*boolean?*/ gotoEnd){
			this._doAction("stop", arguments);
			this._call("stop", arguments);
			return this;
		},
		status: function(){
			return this._pseudoAnimation.status();
		},
		destroy: function(){
			d.forEach(this._connects, dojo.disconnect);
		}
	});
	d.extend(_combine, _baseObj);

	dojo.fx.combine = function(/*dojo.Animation[]*/ animations){
		// summary: 
		//		Combine a list of `dojo.Animation`s to run in parallel
		//
		// description:
		//		Combine an array of `dojo.Animation`s to run in parallel, 
		//		providing a new `dojo.Animation` instance encompasing each
		//		animation, firing standard animation events.
		//
		// example:
		//	Fade out `node` while fading in `otherNode` simultaneously
		//	|	dojo.fx.combine([
		//	|		dojo.fadeIn({ node:node }),
		//	|		dojo.fadeOut({ node:otherNode })
		//	|	]).play();
		//
		// example:
		//	When the longest animation ends, execute a function:
		//	|	var anim = dojo.fx.combine([
		//	|		dojo.fadeIn({ node: n, duration:700 }),
		//	|		dojo.fadeOut({ node: otherNode, duration: 300 })
		//	|	]);
		//	|	dojo.connect(anim, "onEnd", function(){
		//	|		// overall animation is done.
		//	|	});
		//	|	anim.play(); // play the animation
		//
		return new _combine(animations); // dojo.Animation
	};

	dojo.fx.wipeIn = function(/*Object*/ args){
		// summary:
		//		Expand a node to it's natural height.
		//
		// description:
		//		Returns an animation that will expand the
		//		node defined in 'args' object from it's current height to
		//		it's natural height (with no scrollbar).
		//		Node must have no margin/border/padding.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		//
		// example:
		//	|	dojo.fx.wipeIn({
		//	|		node:"someId"
		//	|	}).play()
		var node = args.node = d.byId(args.node), s = node.style, o;

		var anim = d.animateProperty(d.mixin({
			properties: {
				height: {
					// wrapped in functions so we wait till the last second to query (in case value has changed)
					start: function(){
						// start at current [computed] height, but use 1px rather than 0
						// because 0 causes IE to display the whole panel
						o = s.overflow;
						s.overflow = "hidden";
						if(s.visibility == "hidden" || s.display == "none"){
							s.height = "1px";
							s.display = "";
							s.visibility = "";
							return 1;
						}else{
							var height = d.style(node, "height");
							return Math.max(height, 1);
						}
					},
					end: function(){
						return node.scrollHeight;
					}
				}
			}
		}, args));

		d.connect(anim, "onEnd", function(){ 
			s.height = "auto";
			s.overflow = o;
		});

		return anim; // dojo.Animation
	}

	dojo.fx.wipeOut = function(/*Object*/ args){
		// summary:
		//		Shrink a node to nothing and hide it. 
		//
		// description:
		//		Returns an animation that will shrink node defined in "args"
		//		from it's current height to 1px, and then hide it.
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on)
		// 
		// example:
		//	|	dojo.fx.wipeOut({ node:"someId" }).play()
		
		var node = args.node = d.byId(args.node), s = node.style, o;
		
		var anim = d.animateProperty(d.mixin({
			properties: {
				height: {
					end: 1 // 0 causes IE to display the whole panel
				}
			}
		}, args));

		d.connect(anim, "beforeBegin", function(){
			o = s.overflow;
			s.overflow = "hidden";
			s.display = "";
		});
		d.connect(anim, "onEnd", function(){
			s.overflow = o;
			s.height = "auto";
			s.display = "none";
		});

		return anim; // dojo.Animation
	}

	dojo.fx.slideTo = function(/*Object*/ args){
		// summary:
		//		Slide a node to a new top/left position
		//
		// description:
		//		Returns an animation that will slide "node" 
		//		defined in args Object from its current position to
		//		the position defined by (args.left, args.top).
		//
		// args: Object
		//		A hash-map of standard `dojo.Animation` constructor properties
		//		(such as easing: node: duration: and so on). Special args members
		//		are `top` and `left`, which indicate the new position to slide to.
		//
		// example:
		//	|	dojo.fx.slideTo({ node: node, left:"40", top:"50", units:"px" }).play()

		var node = args.node = d.byId(args.node), 
			top = null, left = null;

		var init = (function(n){
			return function(){
				var cs = d.getComputedStyle(n);
				var pos = cs.position;
				top = (pos == 'absolute' ? n.offsetTop : parseInt(cs.top) || 0);
				left = (pos == 'absolute' ? n.offsetLeft : parseInt(cs.left) || 0);
				if(pos != 'absolute' && pos != 'relative'){
					var ret = d.position(n, true);
					top = ret.y;
					left = ret.x;
					n.style.position="absolute";
					n.style.top=top+"px";
					n.style.left=left+"px";
				}
			};
		})(node);
		init();

		var anim = d.animateProperty(d.mixin({
			properties: {
				top: args.top || 0,
				left: args.left || 0
			}
		}, args));
		d.connect(anim, "beforeBegin", anim, init);

		return anim; // dojo.Animation
	}

})();

}

if(!dojo._hasResource["dijit.form._FormMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form._FormMixin"] = true;
dojo.provide("dijit.form._FormMixin");



dojo.declare("dijit.form._FormMixin", null,
	{
	// summary:
	//		Mixin for containers of form widgets (i.e. widgets that represent a single value
	//		and can be children of a <form> node or dijit.form.Form widget)
	// description:
	//		Can extract all the form widgets
	//		values and combine them into a single javascript object, or alternately
	//		take such an object and set the values for all the contained
	//		form widgets

/*=====
    // value: Object
	//		Name/value hash for each child widget with a name and value.
	//		Child widgets without names are not part of the hash.
	// 
	//		If there are multiple child widgets w/the same name, value is an array,
	//		unless they are radio buttons in which case value is a scalar (since only
	//		one radio button can be checked at a time).
	//
	//		If a child widget's name is a dot separated list (like a.b.c.d), it's a nested structure.
	//
	//		Example:
	//	|	{ name: "John Smith", interests: ["sports", "movies"] }
=====*/

	//	TODO:
	//	* Repeater
	//	* better handling for arrays.  Often form elements have names with [] like
	//	* people[3].sex (for a list of people [{name: Bill, sex: M}, ...])
	//
	//

		reset: function(){
			dojo.forEach(this.getDescendants(), function(widget){
				if(widget.reset){
					widget.reset();
				}
			});
		},

		validate: function(){
			// summary:
			//		returns if the form is valid - same as isValid - but
			//			provides a few additional (ui-specific) features.
			//			1 - it will highlight any sub-widgets that are not
			//				valid
			//			2 - it will call focus() on the first invalid
			//				sub-widget
			var didFocus = false;
			return dojo.every(dojo.map(this.getDescendants(), function(widget){
				// Need to set this so that "required" widgets get their
				// state set.
				widget._hasBeenBlurred = true;
				var valid = widget.disabled || !widget.validate || widget.validate();
				if(!valid && !didFocus){
					// Set focus of the first non-valid widget
					dojo.window.scrollIntoView(widget.containerNode || widget.domNode);
					widget.focus();
					didFocus = true;
				}
	 			return valid;
	 		}), function(item){ return item; });
		},

		setValues: function(val){
			dojo.deprecated(this.declaredClass+"::setValues() is deprecated. Use set('value', val) instead.", "", "2.0");
			return this.set('value', val);
		},
		_setValueAttr: function(/*object*/obj){
			// summary:
			//		Fill in form values from according to an Object (in the format returned by attr('value'))

			// generate map from name --> [list of widgets with that name]
			var map = { };
			dojo.forEach(this.getDescendants(), function(widget){
				if(!widget.name){ return; }
				var entry = map[widget.name] || (map[widget.name] = [] );
				entry.push(widget);
			});

			for(var name in map){
				if(!map.hasOwnProperty(name)){
					continue;
				}
				var widgets = map[name],						// array of widgets w/this name
					values = dojo.getObject(name, false, obj);	// list of values for those widgets

				if(values === undefined){
					continue;
				}
				if(!dojo.isArray(values)){
					values = [ values ];
				}
				if(typeof widgets[0].checked == 'boolean'){
					// for checkbox/radio, values is a list of which widgets should be checked
					dojo.forEach(widgets, function(w, i){
						w.set('value', dojo.indexOf(values, w.value) != -1);
					});
				}else if(widgets[0].multiple){
					// it takes an array (e.g. multi-select)
					widgets[0].set('value', values);
				}else{
					// otherwise, values is a list of values to be assigned sequentially to each widget
					dojo.forEach(widgets, function(w, i){
						w.set('value', values[i]);
					});
				}
			}

			/***
			 * 	TODO: code for plain input boxes (this shouldn't run for inputs that are part of widgets)

			dojo.forEach(this.containerNode.elements, function(element){
				if(element.name == ''){return};	// like "continue"
				var namePath = element.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var p=namePath[j - 1];
					// repeater support block
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if

						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
						myObj=myObj[nameA[0]][nameIndex];
						continue;
					} // repeater support ends

					if(typeof(myObj[p]) == "undefined"){
						myObj=undefined;
						break;
					};
					myObj=myObj[p];
				}

				if(typeof(myObj) == "undefined"){
					return;		// like "continue"
				}
				if(typeof(myObj[name]) == "undefined" && this.ignoreNullValues){
					return;		// like "continue"
				}

				// TODO: widget values (just call attr('value', ...) on the widget)

				// TODO: maybe should call dojo.getNodeProp() instead
				switch(element.type){
					case "checkbox":
						element.checked = (name in myObj) &&
							dojo.some(myObj[name], function(val){ return val == element.value; });
						break;
					case "radio":
						element.checked = (name in myObj) && myObj[name] == element.value;
						break;
					case "select-multiple":
						element.selectedIndex=-1;
						dojo.forEach(element.options, function(option){
							option.selected = dojo.some(myObj[name], function(val){ return option.value == val; });
						});
						break;
					case "select-one":
						element.selectedIndex="0";
						dojo.forEach(element.options, function(option){
							option.selected = option.value == myObj[name];
						});
						break;
					case "hidden":
					case "text":
					case "textarea":
					case "password":
						element.value = myObj[name] || "";
						break;
				}
	  		});
	  		*/
		},

		getValues: function(){
			dojo.deprecated(this.declaredClass+"::getValues() is deprecated. Use get('value') instead.", "", "2.0");
			return this.get('value');
		},
		_getValueAttr: function(){
			// summary:
			// 		Returns Object representing form values.
			// description:
			//		Returns name/value hash for each form element.
			//		If there are multiple elements w/the same name, value is an array,
			//		unless they are radio buttons in which case value is a scalar since only
			//		one can be checked at a time.
			//
			//		If the name is a dot separated list (like a.b.c.d), creates a nested structure.
			//		Only works on widget form elements.
			// example:
			//		| { name: "John Smith", interests: ["sports", "movies"] }

			// get widget values
			var obj = { };
			dojo.forEach(this.getDescendants(), function(widget){
				var name = widget.name;
				if(!name || widget.disabled){ return; }

				// Single value widget (checkbox, radio, or plain <input> type widget
				var value = widget.get('value');

				// Store widget's value(s) as a scalar, except for checkboxes which are automatically arrays
				if(typeof widget.checked == 'boolean'){
					if(/Radio/.test(widget.declaredClass)){
						// radio button
						if(value !== false){
							dojo.setObject(name, value, obj);
						}else{
							// give radio widgets a default of null
							value = dojo.getObject(name, false, obj);
							if(value === undefined){
								dojo.setObject(name, null, obj);
							}
						}
					}else{
						// checkbox/toggle button
						var ary=dojo.getObject(name, false, obj);
						if(!ary){
							ary=[];
							dojo.setObject(name, ary, obj);
						}
						if(value !== false){
							ary.push(value);
						}
					}
				}else{
					var prev=dojo.getObject(name, false, obj);
					if(typeof prev != "undefined"){
						if(dojo.isArray(prev)){
							prev.push(value);
						}else{
							dojo.setObject(name, [prev, value], obj);
						}
					}else{
						// unique name
						dojo.setObject(name, value, obj);
					}
				}
			});

			/***
			 * code for plain input boxes (see also dojo.formToObject, can we use that instead of this code?
			 * but it doesn't understand [] notation, presumably)
			var obj = { };
			dojo.forEach(this.containerNode.elements, function(elm){
				if(!elm.name)	{
					return;		// like "continue"
				}
				var namePath = elm.name.split(".");
				var myObj=obj;
				var name=namePath[namePath.length-1];
				for(var j=1,len2=namePath.length;j<len2;++j){
					var nameIndex = null;
					var p=namePath[j - 1];
					var nameA=p.split("[");
					if(nameA.length > 1){
						if(typeof(myObj[nameA[0]]) == "undefined"){
							myObj[nameA[0]]=[ ];
						} // if
						nameIndex=parseInt(nameA[1]);
						if(typeof(myObj[nameA[0]][nameIndex]) == "undefined"){
							myObj[nameA[0]][nameIndex] = { };
						}
					} else if(typeof(myObj[nameA[0]]) == "undefined"){
						myObj[nameA[0]] = { }
					} // if

					if(nameA.length == 1){
						myObj=myObj[nameA[0]];
					} else{
						myObj=myObj[nameA[0]][nameIndex];
					} // if
				} // for

				if((elm.type != "select-multiple" && elm.type != "checkbox" && elm.type != "radio") || (elm.type == "radio" && elm.checked)){
					if(name == name.split("[")[0]){
						myObj[name]=elm.value;
					} else{
						// can not set value when there is no name
					}
				} else if(elm.type == "checkbox" && elm.checked){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					myObj[name].push(elm.value);
				} else if(elm.type == "select-multiple"){
					if(typeof(myObj[name]) == 'undefined'){
						myObj[name]=[ ];
					}
					for(var jdx=0,len3=elm.options.length; jdx<len3; ++jdx){
						if(elm.options[jdx].selected){
							myObj[name].push(elm.options[jdx].value);
						}
					}
				} // if
				name=undefined;
			}); // forEach
			***/
			return obj;
		},

		// TODO: ComboBox might need time to process a recently input value.  This should be async?
	 	isValid: function(){
	 		// summary:
	 		//		Returns true if all of the widgets are valid

	 		// This also populate this._invalidWidgets[] array with list of invalid widgets...
	 		// TODO: put that into separate function?   It's confusing to have that as a side effect
	 		// of a method named isValid().

			this._invalidWidgets = dojo.filter(this.getDescendants(), function(widget){
				return !widget.disabled && widget.isValid && !widget.isValid();
	 		});
			return !this._invalidWidgets.length;
		},


		onValidStateChange: function(isValid){
			// summary:
			//		Stub function to connect to if you want to do something
			//		(like disable/enable a submit button) when the valid
			//		state changes on the form as a whole.
		},

		_widgetChange: function(widget){
			// summary:
			//		Connected to a widget's onChange function - update our
			//		valid state, if needed.
			var isValid = this._lastValidState;
			if(!widget || this._lastValidState === undefined){
				// We have passed a null widget, or we haven't been validated
				// yet - let's re-check all our children
				// This happens when we connect (or reconnect) our children
				isValid = this.isValid();
				if(this._lastValidState === undefined){
					// Set this so that we don't fire an onValidStateChange
					// the first time
					this._lastValidState = isValid;
				}
			}else if(widget.isValid){
				this._invalidWidgets = dojo.filter(this._invalidWidgets || [], function(w){
					return (w != widget);
				}, this);
				if(!widget.isValid() && !widget.get("disabled")){
					this._invalidWidgets.push(widget);
				}
				isValid = (this._invalidWidgets.length === 0);
			}
			if(isValid !== this._lastValidState){
				this._lastValidState = isValid;
				this.onValidStateChange(isValid);
			}
		},

		connectChildren: function(){
			// summary:
			//		Connects to the onChange function of all children to
			//		track valid state changes.  You can call this function
			//		directly, ex. in the event that you programmatically
			//		add a widget to the form *after* the form has been
			//		initialized.
			dojo.forEach(this._changeConnections, dojo.hitch(this, "disconnect"));
			var _this = this;

			// we connect to validate - so that it better reflects the states
			// of the widgets - also, we only connect if it has a validate
			// function (to avoid too many unneeded connections)
			var conns = (this._changeConnections = []);
			dojo.forEach(dojo.filter(this.getDescendants(),
				function(item){ return item.validate; }
			),
			function(widget){
				// We are interested in whenever the widget is validated - or
				// whenever the disabled attribute on that widget is changed
				conns.push(_this.connect(widget, "validate",
									dojo.hitch(_this, "_widgetChange", widget)));
				conns.push(_this.connect(widget, "_setDisabledAttr",
									dojo.hitch(_this, "_widgetChange", widget)));
			});

			// Call the widget change function to update the valid state, in
			// case something is different now.
			this._widgetChange(null);
		},

		startup: function(){
			this.inherited(arguments);
			// Initialize our valid state tracking.  Needs to be done in startup
			// because it's not guaranteed that our children are initialized
			// yet.
			this._changeConnections = [];
			this.connectChildren();
		}
	});

}

if(!dojo._hasResource["dijit._DialogMixin"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit._DialogMixin"] = true;
dojo.provide("dijit._DialogMixin");



dojo.declare("dijit._DialogMixin", null,
	{
		// summary:
		//		This provides functions useful to Dialog and TooltipDialog

		attributeMap: dijit._Widget.prototype.attributeMap,

		execute: function(/*Object*/ formContents){
			// summary:
			//		Callback when the user hits the submit button.
			//		Override this method to handle Dialog execution.
			// description:
			//		After the user has pressed the submit button, the Dialog
			//		first calls onExecute() to notify the container to hide the
			//		dialog and restore focus to wherever it used to be.
			//
			//		*Then* this method is called.
			// type:
			//		callback
		},

		onCancel: function(){
			// summary:
			//	    Called when user has pressed the Dialog's cancel button, to notify container.
			// description:
			//	    Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit.form.DropDownButton`)
			// type:
			//		protected
		},

		onExecute: function(){
			// summary:
			//	    Called when user has pressed the dialog's OK button, to notify container.
			// description:
			//	    Developer shouldn't override or connect to this method;
			//		it's a private communication device between the TooltipDialog
			//		and the thing that opened it (ex: `dijit.form.DropDownButton`)
			// type:
			//		protected
		},

		_onSubmit: function(){
			// summary:
			//		Callback when user hits submit button
			// type:
			//		protected
			this.onExecute();	// notify container that we are about to execute
			this.execute(this.get('value'));
		},

		_getFocusItems: function(/*Node*/ dialogNode){
			// summary:
			//		Find focusable Items each time a dialog is opened,
			//		setting _firstFocusItem and _lastFocusItem
			// tags:
			//		protected

			var elems = dijit._getTabNavigable(dojo.byId(dialogNode));
			this._firstFocusItem = elems.lowest || elems.first || dialogNode;
			this._lastFocusItem = elems.last || elems.highest || this._firstFocusItem;
			if(dojo.isMoz && this._firstFocusItem.tagName.toLowerCase() == "input" &&
					dojo.getNodeProp(this._firstFocusItem, "type").toLowerCase() == "file"){
				// FF doesn't behave well when first element is input type=file, set first focusable to dialog container
				dojo.attr(dialogNode, "tabIndex", "0");
				this._firstFocusItem = dialogNode;
			}
		}
	}
);

}

if(!dojo._hasResource["dijit.DialogUnderlay"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.DialogUnderlay"] = true;
dojo.provide("dijit.DialogUnderlay");






dojo.declare(
	"dijit.DialogUnderlay",
	[dijit._Widget, dijit._Templated],
	{
		// summary:
		//		The component that blocks the screen behind a `dijit.Dialog`
		//
		// description:
		// 		A component used to block input behind a `dijit.Dialog`. Only a single
		//		instance of this widget is created by `dijit.Dialog`, and saved as
		//		a reference to be shared between all Dialogs as `dijit._underlay`
		//
		//		The underlay itself can be styled based on and id:
		//	|	#myDialog_underlay { background-color:red; }
		//
		//		In the case of `dijit.Dialog`, this id is based on the id of the Dialog,
		//		suffixed with _underlay.

		// Template has two divs; outer div is used for fade-in/fade-out, and also to hold background iframe.
		// Inner div has opacity specified in CSS file.
		templateString: "<div class='dijitDialogUnderlayWrapper'><div class='dijitDialogUnderlay' dojoAttachPoint='node'></div></div>",

		// Parameters on creation or updatable later

		// dialogId: String
		//		Id of the dialog.... DialogUnderlay's id is based on this id
		dialogId: "",

		// class: String
		//		This class name is used on the DialogUnderlay node, in addition to dijitDialogUnderlay
		"class": "",

		attributeMap: { id: "domNode" },

		_setDialogIdAttr: function(id){
			dojo.attr(this.node, "id", id + "_underlay");
		},

		_setClassAttr: function(clazz){
			this.node.className = "dijitDialogUnderlay " + clazz;
		},

		postCreate: function(){
			// summary:
			//		Append the underlay to the body
			dojo.body().appendChild(this.domNode);
		},

		layout: function(){
			// summary:
			//		Sets the background to the size of the viewport
			//
			// description:
			//		Sets the background to the size of the viewport (rather than the size
			//		of the document) since we need to cover the whole browser window, even
			//		if the document is only a few lines long.
			// tags:
			//		private

			var is = this.node.style,
				os = this.domNode.style;

			// hide the background temporarily, so that the background itself isn't
			// causing scrollbars to appear (might happen when user shrinks browser
			// window and then we are called to resize)
			os.display = "none";

			// then resize and show
			var viewport = dojo.window.getBox();
			os.top = viewport.t + "px";
			os.left = viewport.l + "px";
			is.width = viewport.w + "px";
			is.height = viewport.h + "px";
			os.display = "block";
		},

		show: function(){
			// summary:
			//		Show the dialog underlay
			this.domNode.style.display = "block";
			this.layout();
			this.bgIframe = new dijit.BackgroundIframe(this.domNode);
		},

		hide: function(){
			// summary:
			//		Hides the dialog underlay
			this.bgIframe.destroy();
			delete this.bgIframe;
			this.domNode.style.display = "none";
		}
	}
);

}

if(!dojo._hasResource["dijit.layout._LayoutWidget"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._LayoutWidget"] = true;
dojo.provide("dijit.layout._LayoutWidget");





dojo.declare("dijit.layout._LayoutWidget",
	[dijit._Widget, dijit._Container, dijit._Contained],
	{
		// summary:
		//		Base class for a _Container widget which is responsible for laying out its children.
		//		Widgets which mixin this code must define layout() to manage placement and sizing of the children.

		// baseClass: [protected extension] String
		//		This class name is applied to the widget's domNode
		//		and also may be used to generate names for sub nodes,
		//		for example dijitTabContainer-content.
		baseClass: "dijitLayoutContainer",

		// isLayoutContainer: [protected] Boolean
		//		Indicates that this widget is going to call resize() on its
		//		children widgets, setting their size, when they become visible.
		isLayoutContainer: true,

		postCreate: function(){
			dojo.addClass(this.domNode, "dijitContainer");

			this.inherited(arguments);
		},

		startup: function(){
			// summary:
			//		Called after all the widgets have been instantiated and their
			//		dom nodes have been inserted somewhere under dojo.doc.body.
			//
			//		Widgets should override this method to do any initialization
			//		dependent on other widgets existing, and then call
			//		this superclass method to finish things off.
			//
			//		startup() in subclasses shouldn't do anything
			//		size related because the size of the widget hasn't been set yet.

			if(this._started){ return; }

			// Need to call inherited first - so that child widgets get started
			// up correctly
			this.inherited(arguments);

			// If I am a not being controlled by a parent layout widget...
			var parent = this.getParent && this.getParent()
			if(!(parent && parent.isLayoutContainer)){
				// Do recursive sizing and layout of all my descendants
				// (passing in no argument to resize means that it has to glean the size itself)
				this.resize();

				// Since my parent isn't a layout container, and my style *may be* width=height=100%
				// or something similar (either set directly or via a CSS class),
				// monitor when my size changes so that I can re-layout.
				// For browsers where I can't directly monitor when my size changes,
				// monitor when the viewport changes size, which *may* indicate a size change for me.
				this.connect(dojo.isIE ? this.domNode : dojo.global, 'onresize', function(){
					// Using function(){} closure to ensure no arguments to resize.
					this.resize();
				});
			}
		},

		resize: function(changeSize, resultSize){
			// summary:
			//		Call this to resize a widget, or after its size has changed.
			// description:
			//		Change size mode:
			//			When changeSize is specified, changes the marginBox of this widget
			//			and forces it to relayout its contents accordingly.
			//			changeSize may specify height, width, or both.
			//
			//			If resultSize is specified it indicates the size the widget will
			//			become after changeSize has been applied.
			//
			//		Notification mode:
			//			When changeSize is null, indicates that the caller has already changed
			//			the size of the widget, or perhaps it changed because the browser
			//			window was resized.  Tells widget to relayout its contents accordingly.
			//
			//			If resultSize is also specified it indicates the size the widget has
			//			become.
			//
			//		In either mode, this method also:
			//			1. Sets this._borderBox and this._contentBox to the new size of
			//				the widget.  Queries the current domNode size if necessary.
			//			2. Calls layout() to resize contents (and maybe adjust child widgets).
			//
			// changeSize: Object?
			//		Sets the widget to this margin-box size and position.
			//		May include any/all of the following properties:
			//	|	{w: int, h: int, l: int, t: int}
			//
			// resultSize: Object?
			//		The margin-box size of this widget after applying changeSize (if
			//		changeSize is specified).  If caller knows this size and
			//		passes it in, we don't need to query the browser to get the size.
			//	|	{w: int, h: int}

			var node = this.domNode;

			// set margin box size, unless it wasn't specified, in which case use current size
			if(changeSize){
				dojo.marginBox(node, changeSize);

				// set offset of the node
				if(changeSize.t){ node.style.top = changeSize.t + "px"; }
				if(changeSize.l){ node.style.left = changeSize.l + "px"; }
			}

			// If either height or width wasn't specified by the user, then query node for it.
			// But note that setting the margin box and then immediately querying dimensions may return
			// inaccurate results, so try not to depend on it.
			var mb = resultSize || {};
			dojo.mixin(mb, changeSize || {});	// changeSize overrides resultSize
			if( !("h" in mb) || !("w" in mb) ){
				mb = dojo.mixin(dojo.marginBox(node), mb);	// just use dojo.marginBox() to fill in missing values
			}

			// Compute and save the size of my border box and content box
			// (w/out calling dojo.contentBox() since that may fail if size was recently set)
			var cs = dojo.getComputedStyle(node);
			var me = dojo._getMarginExtents(node, cs);
			var be = dojo._getBorderExtents(node, cs);
			var bb = (this._borderBox = {
				w: mb.w - (me.w + be.w),
				h: mb.h - (me.h + be.h)
			});
			var pe = dojo._getPadExtents(node, cs);
			this._contentBox = {
				l: dojo._toPixelValue(node, cs.paddingLeft),
				t: dojo._toPixelValue(node, cs.paddingTop),
				w: bb.w - pe.w,
				h: bb.h - pe.h
			};

			// Callback for widget to adjust size of its children
			this.layout();
		},

		layout: function(){
			// summary:
			//		Widgets override this method to size and position their contents/children.
			//		When this is called this._contentBox is guaranteed to be set (see resize()).
			//
			//		This is called after startup(), and also when the widget's size has been
			//		changed.
			// tags:
			//		protected extension
		},

		_setupChild: function(/*dijit._Widget*/child){
			// summary:
			//		Common setup for initial children and children which are added after startup
			// tags:
			//		protected extension

			dojo.addClass(child.domNode, this.baseClass+"-child");
			if(child.baseClass){
				dojo.addClass(child.domNode, this.baseClass+"-"+child.baseClass);
			}
		},

		addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
			// Overrides _Container.addChild() to call _setupChild()
			this.inherited(arguments);
			if(this._started){
				this._setupChild(child);
			}
		},

		removeChild: function(/*dijit._Widget*/ child){
			// Overrides _Container.removeChild() to remove class added by _setupChild()
			dojo.removeClass(child.domNode, this.baseClass+"-child");
			if(child.baseClass){
				dojo.removeClass(child.domNode, this.baseClass+"-"+child.baseClass);
			}
			this.inherited(arguments);
		}
	}
);

dijit.layout.marginBox2contentBox = function(/*DomNode*/ node, /*Object*/ mb){
	// summary:
	//		Given the margin-box size of a node, return its content box size.
	//		Functions like dojo.contentBox() but is more reliable since it doesn't have
	//		to wait for the browser to compute sizes.
	var cs = dojo.getComputedStyle(node);
	var me = dojo._getMarginExtents(node, cs);
	var pb = dojo._getPadBorderExtents(node, cs);
	return {
		l: dojo._toPixelValue(node, cs.paddingLeft),
		t: dojo._toPixelValue(node, cs.paddingTop),
		w: mb.w - (me.w + pb.w),
		h: mb.h - (me.h + pb.h)
	};
};

(function(){
	var capitalize = function(word){
		return word.substring(0,1).toUpperCase() + word.substring(1);
	};

	var size = function(widget, dim){
		// size the child
		widget.resize ? widget.resize(dim) : dojo.marginBox(widget.domNode, dim);

		// record child's size, but favor our own numbers when we have them.
		// the browser lies sometimes
		dojo.mixin(widget, dojo.marginBox(widget.domNode));
		dojo.mixin(widget, dim);
	};

	dijit.layout.layoutChildren = function(/*DomNode*/ container, /*Object*/ dim, /*Object[]*/ children){
		// summary
		//		Layout a bunch of child dom nodes within a parent dom node
		// container:
		//		parent node
		// dim:
		//		{l, t, w, h} object specifying dimensions of container into which to place children
		// children:
		//		an array like [ {domNode: foo, layoutAlign: "bottom" }, {domNode: bar, layoutAlign: "client"} ]

		// copy dim because we are going to modify it
		dim = dojo.mixin({}, dim);

		dojo.addClass(container, "dijitLayoutContainer");

		// Move "client" elements to the end of the array for layout.  a11y dictates that the author
		// needs to be able to put them in the document in tab-order, but this algorithm requires that
		// client be last.
		children = dojo.filter(children, function(item){ return item.layoutAlign != "client"; })
			.concat(dojo.filter(children, function(item){ return item.layoutAlign == "client"; }));

		// set positions/sizes
		dojo.forEach(children, function(child){
			var elm = child.domNode,
				pos = child.layoutAlign;

			// set elem to upper left corner of unused space; may move it later
			var elmStyle = elm.style;
			elmStyle.left = dim.l+"px";
			elmStyle.top = dim.t+"px";
			elmStyle.bottom = elmStyle.right = "auto";

			dojo.addClass(elm, "dijitAlign" + capitalize(pos));

			// set size && adjust record of remaining space.
			// note that setting the width of a <div> may affect its height.
			if(pos == "top" || pos == "bottom"){
				size(child, { w: dim.w });
				dim.h -= child.h;
				if(pos == "top"){
					dim.t += child.h;
				}else{
					elmStyle.top = dim.t + dim.h + "px";
				}
			}else if(pos == "left" || pos == "right"){
				size(child, { h: dim.h });
				dim.w -= child.w;
				if(pos == "left"){
					dim.l += child.w;
				}else{
					elmStyle.left = dim.l + dim.w + "px";
				}
			}else if(pos == "client"){
				size(child, dim);
			}
		});
	};

})();

}

if(!dojo._hasResource["dojo.html"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.html"] = true;
dojo.provide("dojo.html");

// the parser might be needed..
 

(function(){ // private scope, sort of a namespace

	// idCounter is incremented with each instantiation to allow asignment of a unique id for tracking, logging purposes
	var idCounter = 0, 
		d = dojo;
	
	dojo.html._secureForInnerHtml = function(/*String*/ cont){
		// summary:
		//		removes !DOCTYPE and title elements from the html string.
		// 
		//		khtml is picky about dom faults, you can't attach a style or <title> node as child of body
		//		must go into head, so we need to cut out those tags
		//	cont:
		//		An html string for insertion into the dom
		//	
		return cont.replace(/(?:\s*<!DOCTYPE\s[^>]+>|<title[^>]*>[\s\S]*?<\/title>)/ig, ""); // String
	};

/*====
	dojo.html._emptyNode = function(node){
		// summary:
		//		removes all child nodes from the given node
		//	node: DOMNode
		//		the parent element
	};
=====*/
	dojo.html._emptyNode = dojo.empty;

	dojo.html._setNodeContent = function(/* DomNode */ node, /* String|DomNode|NodeList */ cont){
		// summary:
		//		inserts the given content into the given node
		//	node:
		//		the parent element
		//	content:
		//		the content to be set on the parent element. 
		//		This can be an html string, a node reference or a NodeList, dojo.NodeList, Array or other enumerable list of nodes
		
		// always empty
		d.empty(node);

		if(cont) {
			if(typeof cont == "string") {
				cont = d._toDom(cont, node.ownerDocument);
			}
			if(!cont.nodeType && d.isArrayLike(cont)) {
				// handle as enumerable, but it may shrink as we enumerate it
				for(var startlen=cont.length, i=0; i<cont.length; i=startlen==cont.length ? i+1 : 0) {
					d.place( cont[i], node, "last");
				}
			} else {
				// pass nodes, documentFragments and unknowns through to dojo.place
				d.place(cont, node, "last");
			}
		}

		// return DomNode
		return node;
	};

	// we wrap up the content-setting operation in a object
	dojo.declare("dojo.html._ContentSetter", null, 
		{
			// node: DomNode|String
			//		An node which will be the parent element that we set content into
			node: "",

			// content: String|DomNode|DomNode[]
			//		The content to be placed in the node. Can be an HTML string, a node reference, or a enumerable list of nodes
			content: "",
			
			// id: String?
			//		Usually only used internally, and auto-generated with each instance 
			id: "",

			// cleanContent: Boolean
			//		Should the content be treated as a full html document, 
			//		and the real content stripped of <html>, <body> wrapper before injection
			cleanContent: false,
			
			// extractContent: Boolean
			//		Should the content be treated as a full html document, and the real content stripped of <html>, <body> wrapper before injection
			extractContent: false,

			// parseContent: Boolean
			//		Should the node by passed to the parser after the new content is set
			parseContent: false,
			
			// lifecyle methods
			constructor: function(/* Object */params, /* String|DomNode */node){
				//	summary:
				//		Provides a configurable, extensible object to wrap the setting on content on a node
				//		call the set() method to actually set the content..
 
				// the original params are mixed directly into the instance "this"
				dojo.mixin(this, params || {});

				// give precedence to params.node vs. the node argument
				// and ensure its a node, not an id string
				node = this.node = dojo.byId( this.node || node );
	
				if(!this.id){
					this.id = [
						"Setter",
						(node) ? node.id || node.tagName : "", 
						idCounter++
					].join("_");
				}
			},
			set: function(/* String|DomNode|NodeList? */ cont, /* Object? */ params){
				// summary:
				//		front-end to the set-content sequence 
				//	cont:
				//		An html string, node or enumerable list of nodes for insertion into the dom
				//		If not provided, the object's content property will be used
				if(undefined !== cont){
					this.content = cont;
				}
				// in the re-use scenario, set needs to be able to mixin new configuration
				if(params){
					this._mixin(params);
				}

				this.onBegin();
				this.setContent();
				this.onEnd();

				return this.node;
			},
			setContent: function(){
				// summary:
				//		sets the content on the node 

				var node = this.node; 
				if(!node) {
				    // can't proceed
					throw new Error(this.declaredClass + ": setContent given no node");
				}
				try{
					node = dojo.html._setNodeContent(node, this.content);
				}catch(e){
					// check if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV
	
					// FIXME: need to allow the user to provide a content error message string
					var errMess = this.onContentError(e); 
					try{
						node.innerHTML = errMess;
					}catch(e){
						console.error('Fatal ' + this.declaredClass + '.setContent could not change content due to '+e.message, e);
					}
				}
				// always put back the node for the next method
				this.node = node; // DomNode
			},
			
			empty: function() {
				// summary
				//	cleanly empty out existing content

				// destroy any widgets from a previous run
				// NOTE: if you dont want this you'll need to empty 
				// the parseResults array property yourself to avoid bad things happenning
				if(this.parseResults && this.parseResults.length) {
					dojo.forEach(this.parseResults, function(w) {
						if(w.destroy){
							w.destroy();
						}
					});
					delete this.parseResults;
				}
				// this is fast, but if you know its already empty or safe, you could 
				// override empty to skip this step
				dojo.html._emptyNode(this.node);
			},
	
			onBegin: function(){
				// summary
				//		Called after instantiation, but before set(); 
				//		It allows modification of any of the object properties 
				//		- including the node and content provided - before the set operation actually takes place
				//		This default implementation checks for cleanContent and extractContent flags to 
				//		optionally pre-process html string content
				var cont = this.content;
	
				if(dojo.isString(cont)){
					if(this.cleanContent){
						cont = dojo.html._secureForInnerHtml(cont);
					}
  
					if(this.extractContent){
						var match = cont.match(/<body[^>]*>\s*([\s\S]+)\s*<\/body>/im);
						if(match){ cont = match[1]; }
					}
				}

				// clean out the node and any cruft associated with it - like widgets
				this.empty();
				
				this.content = cont;
				return this.node; /* DomNode */
			},
	
			onEnd: function(){
				// summary
				//		Called after set(), when the new content has been pushed into the node
				//		It provides an opportunity for post-processing before handing back the node to the caller
				//		This default implementation checks a parseContent flag to optionally run the dojo parser over the new content
				if(this.parseContent){
					// populates this.parseResults if you need those..
					this._parse();
				}
				return this.node; /* DomNode */
			},
	
			tearDown: function(){
				// summary
				//		manually reset the Setter instance if its being re-used for example for another set()
				// description
				//		tearDown() is not called automatically. 
				//		In normal use, the Setter instance properties are simply allowed to fall out of scope
				//		but the tearDown method can be called to explicitly reset this instance.
				delete this.parseResults; 
				delete this.node; 
				delete this.content; 
			},
  
			onContentError: function(err){
				return "Error occured setting content: " + err; 
			},
			
			_mixin: function(params){
				// mix properties/methods into the instance
				// TODO: the intention with tearDown is to put the Setter's state 
				// back to that of the original constructor (vs. deleting/resetting everything regardless of ctor params)
				// so we could do something here to move the original properties aside for later restoration
				var empty = {}, key;
				for(key in params){
					if(key in empty){ continue; }
					// TODO: here's our opportunity to mask the properties we dont consider configurable/overridable
					// .. but history shows we'll almost always guess wrong
					this[key] = params[key]; 
				}
			},
			_parse: function(){
				// summary: 
				//		runs the dojo parser over the node contents, storing any results in this.parseResults
				//		Any errors resulting from parsing are passed to _onError for handling

				var rootNode = this.node;
				try{
					// store the results (widgets, whatever) for potential retrieval
					this.parseResults = dojo.parser.parse({
						rootNode: rootNode,
						dir: this.dir,
						lang: this.lang
					});
				}catch(e){
					this._onError('Content', e, "Error parsing in _ContentSetter#"+this.id);
				}
			},
  
			_onError: function(type, err, consoleText){
				// summary:
				//		shows user the string that is returned by on[type]Error
				//		overide/implement on[type]Error and return your own string to customize
				var errText = this['on' + type + 'Error'].call(this, err);
				if(consoleText){
					console.error(consoleText, err);
				}else if(errText){ // a empty string won't change current content
					dojo.html._setNodeContent(this.node, errText, true);
				}
			}
	}); // end dojo.declare()

	dojo.html.set = function(/* DomNode */ node, /* String|DomNode|NodeList */ cont, /* Object? */ params){
			// summary:
			//		inserts (replaces) the given content into the given node. dojo.place(cont, node, "only")
			//		may be a better choice for simple HTML insertion.
			// description:
			//		Unless you need to use the params capabilities of this method, you should use
			//		dojo.place(cont, node, "only"). dojo.place() has more robust support for injecting
			//		an HTML string into the DOM, but it only handles inserting an HTML string as DOM
			//		elements, or inserting a DOM node. dojo.place does not handle NodeList insertions
			//		or the other capabilities as defined by the params object for this method.
			//	node:
			//		the parent element that will receive the content
			//	cont:
			//		the content to be set on the parent element. 
			//		This can be an html string, a node reference or a NodeList, dojo.NodeList, Array or other enumerable list of nodes
			//	params: 
			//		Optional flags/properties to configure the content-setting. See dojo.html._ContentSetter
			//	example:
			//		A safe string/node/nodelist content replacement/injection with hooks for extension
			//		Example Usage: 
			//		dojo.html.set(node, "some string"); 
			//		dojo.html.set(node, contentNode, {options}); 
			//		dojo.html.set(node, myNode.childNodes, {options}); 
		if(undefined == cont){
			console.warn("dojo.html.set: no cont argument provided, using empty string");
			cont = "";
		}	
		if(!params){
			// simple and fast
			return dojo.html._setNodeContent(node, cont, true);
		}else{ 
			// more options but slower
			// note the arguments are reversed in order, to match the convention for instantiation via the parser
			var op = new dojo.html._ContentSetter(dojo.mixin( 
					params, 
					{ content: cont, node: node } 
			));
			return op.set();
		}
	};
})();

}

if(!dojo._hasResource["dijit.layout.ContentPane"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.ContentPane"] = true;
dojo.provide("dijit.layout.ContentPane");



	// for dijit.layout.marginBox2contentBox()






dojo.declare(
	"dijit.layout.ContentPane", dijit._Widget,
{
	// summary:
	//		A widget that acts as a container for mixed HTML and widgets, and includes an Ajax interface
	// description:
	//		A widget that can be used as a stand alone widget
	//		or as a base class for other widgets.
	//
	//		Handles replacement of document fragment using either external uri or javascript
	//		generated markup or DOM content, instantiating widgets within that content.
	//		Don't confuse it with an iframe, it only needs/wants document fragments.
	//		It's useful as a child of LayoutContainer, SplitContainer, or TabContainer.
	//		But note that those classes can contain any widget as a child.
	// example:
	//		Some quick samples:
	//		To change the innerHTML use .set('content', '<b>new content</b>')
	//
	//		Or you can send it a NodeList, .set('content', dojo.query('div [class=selected]', userSelection))
	//		please note that the nodes in NodeList will copied, not moved
	//
	//		To do a ajax update use .set('href', url)

	// href: String
	//		The href of the content that displays now.
	//		Set this at construction if you want to load data externally when the
	//		pane is shown.  (Set preload=true to load it immediately.)
	//		Changing href after creation doesn't have any effect; Use set('href', ...);
	href: "",

/*=====
	// content: String || DomNode || NodeList || dijit._Widget
	//		The innerHTML of the ContentPane.
	//		Note that the initialization parameter / argument to attr("content", ...)
	//		can be a String, DomNode, Nodelist, or _Widget.
	content: "",
=====*/

	// extractContent: Boolean
	//		Extract visible content from inside of <body> .... </body>.
	//		I.e., strip <html> and <head> (and it's contents) from the href
	extractContent: false,

	// parseOnLoad: Boolean
	//		Parse content and create the widgets, if any.
	parseOnLoad: true,

	// preventCache: Boolean
	//		Prevent caching of data from href's by appending a timestamp to the href.
	preventCache: false,

	// preload: Boolean
	//		Force load of data on initialization even if pane is hidden.
	preload: false,

	// refreshOnShow: Boolean
	//		Refresh (re-download) content when pane goes from hidden to shown
	refreshOnShow: false,

	// loadingMessage: String
	//		Message that shows while downloading
	loadingMessage: "<span class='dijitContentPaneLoading'>${loadingState}</span>",

	// errorMessage: String
	//		Message that shows if an error occurs
	errorMessage: "<span class='dijitContentPaneError'>${errorState}</span>",

	// isLoaded: [readonly] Boolean
	//		True if the ContentPane has data in it, either specified
	//		during initialization (via href or inline content), or set
	//		via attr('content', ...) / attr('href', ...)
	//
	//		False if it doesn't have any content, or if ContentPane is
	//		still in the process of downloading href.
	isLoaded: false,

	baseClass: "dijitContentPane",

	// doLayout: Boolean
	//		- false - don't adjust size of children
	//		- true - if there is a single visible child widget, set it's size to
	//				however big the ContentPane is
	doLayout: true,

	// ioArgs: Object
	//		Parameters to pass to xhrGet() request, for example:
	// |	<div dojoType="dijit.layout.ContentPane" href="./bar" ioArgs="{timeout: 500}">
	ioArgs: {},

	// isContainer: [protected] Boolean
	//		Indicates that this widget acts as a "parent" to the descendant widgets.
	//		When the parent is started it will call startup() on the child widgets.
	//		See also `isLayoutContainer`.
	isContainer: true,

	// isLayoutContainer: [protected] Boolean
	//		Indicates that this widget will call resize() on it's child widgets
	//		when they become visible.
	isLayoutContainer: true,

	// onLoadDeferred: [readonly] dojo.Deferred
	//		This is the `dojo.Deferred` returned by attr('href', ...) and refresh().
	//		Calling onLoadDeferred.addCallback() or addErrback() registers your
	//		callback to be called only once, when the prior attr('href', ...) call or
	//		the initial href parameter to the constructor finishes loading.
	//
	//		This is different than an onLoad() handler which gets called any time any href is loaded.
	onLoadDeferred: null,

	// Override _Widget's attributeMap because we don't want the title attribute (used to specify
	// tab labels) to be copied to ContentPane.domNode... otherwise a tooltip shows up over the
	// entire pane.
	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
		title: []
	}),

	postMixInProperties: function(){
		this.inherited(arguments);
		var messages = dojo.i18n.getLocalization("dijit", "loading", this.lang);
		this.loadingMessage = dojo.string.substitute(this.loadingMessage, messages);
		this.errorMessage = dojo.string.substitute(this.errorMessage, messages);

		// Detect if we were initialized with data
		if(!this.href && this.srcNodeRef && this.srcNodeRef.innerHTML){
			this.isLoaded = true;
		}
	},

	buildRendering: function(){
		// Overrides Widget.buildRendering().
		// Since we have no template we need to set this.containerNode ourselves.
		// For subclasses of ContentPane do have a template, does nothing.
		this.inherited(arguments);
		if(!this.containerNode){
			// make getDescendants() work
			this.containerNode = this.domNode;
		}
	},

	postCreate: function(){
		// remove the title attribute so it doesn't show up when hovering
		// over a node
		this.domNode.title = "";

		if(!dojo.attr(this.domNode,"role")){
			dijit.setWaiRole(this.domNode, "group");
		}

		dojo.addClass(this.domNode, this.baseClass);
	},

	startup: function(){
		// summary:
		//		See `dijit.layout._LayoutWidget.startup` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.
		if(this._started){ return; }

		var parent = dijit._Contained.prototype.getParent.call(this);
		this._childOfLayoutWidget = parent && parent.isLayoutContainer;

		// I need to call resize() on my child/children (when I become visible), unless
		// I'm the child of a layout widget in which case my parent will call resize() on me and I'll do it then.
		this._needLayout = !this._childOfLayoutWidget;

		if(this.isLoaded){
			dojo.forEach(this.getChildren(), function(child){
				child.startup();
			});
		}

		if(this._isShown() || this.preload){
			this._onShow();
		}

		this.inherited(arguments);
	},

	_checkIfSingleChild: function(){
		// summary:
		//		Test if we have exactly one visible widget as a child,
		//		and if so assume that we are a container for that widget,
		//		and should propogate startup() and resize() calls to it.
		//		Skips over things like data stores since they aren't visible.

		var childNodes = dojo.query("> *", this.containerNode).filter(function(node){
				return node.tagName !== "SCRIPT"; // or a regexp for hidden elements like script|area|map|etc..
			}),
			childWidgetNodes = childNodes.filter(function(node){
				return dojo.hasAttr(node, "dojoType") || dojo.hasAttr(node, "widgetId");
			}),
			candidateWidgets = dojo.filter(childWidgetNodes.map(dijit.byNode), function(widget){
				return widget && widget.domNode && widget.resize;
			});

		if(
			// all child nodes are widgets
			childNodes.length == childWidgetNodes.length &&

			// all but one are invisible (like dojo.data)
			candidateWidgets.length == 1
		){
			this._singleChild = candidateWidgets[0];
		}else{
			delete this._singleChild;
		}

		// So we can set overflow: hidden to avoid a safari bug w/scrollbars showing up (#9449)
		dojo.toggleClass(this.containerNode, this.baseClass + "SingleChild", !!this._singleChild);
	},

	setHref: function(/*String|Uri*/ href){
		// summary:
		//		Deprecated.   Use set('href', ...) instead.
		dojo.deprecated("dijit.layout.ContentPane.setHref() is deprecated. Use set('href', ...) instead.", "", "2.0");
		return this.set("href", href);
	},
	_setHrefAttr: function(/*String|Uri*/ href){
		// summary:
		//		Hook so attr("href", ...) works.
		// description:
		//		Reset the (external defined) content of this pane and replace with new url
		//		Note: It delays the download until widget is shown if preload is false.
		//	href:
		//		url to the page you want to get, must be within the same domain as your mainpage

		// Cancel any in-flight requests (an attr('href') will cancel any in-flight attr('href', ...))
		this.cancel();

		this.onLoadDeferred = new dojo.Deferred(dojo.hitch(this, "cancel"));

		this.href = href;

		// _setHrefAttr() is called during creation and by the user, after creation.
		// only in the second case do we actually load the URL; otherwise it's done in startup()
		if(this._created && (this.preload || this._isShown())){
			this._load();
		}else{
			// Set flag to indicate that href needs to be loaded the next time the
			// ContentPane is made visible
			this._hrefChanged = true;
		}

		return this.onLoadDeferred;		// dojo.Deferred
	},

	setContent: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Deprecated.   Use set('content', ...) instead.
		dojo.deprecated("dijit.layout.ContentPane.setContent() is deprecated.  Use set('content', ...) instead.", "", "2.0");
		this.set("content", data);
	},
	_setContentAttr: function(/*String|DomNode|Nodelist*/data){
		// summary:
		//		Hook to make attr("content", ...) work.
		//		Replaces old content with data content, include style classes from old content
		//	data:
		//		the new Content may be String, DomNode or NodeList
		//
		//		if data is a NodeList (or an array of nodes) nodes are copied
		//		so you can import nodes from another document implicitly

		// clear href so we can't run refresh and clear content
		// refresh should only work if we downloaded the content
		this.href = "";

		// Cancel any in-flight requests (an attr('content') will cancel any in-flight attr('href', ...))
		this.cancel();

		// Even though user is just setting content directly, still need to define an onLoadDeferred
		// because the _onLoadHandler() handler is still getting called from setContent()
		this.onLoadDeferred = new dojo.Deferred(dojo.hitch(this, "cancel"));

		this._setContent(data || "");

		this._isDownloaded = false; // mark that content is from a attr('content') not an attr('href')

		return this.onLoadDeferred; 	// dojo.Deferred
	},
	_getContentAttr: function(){
		// summary:
		//		Hook to make attr("content") work
		return this.containerNode.innerHTML;
	},

	cancel: function(){
		// summary:
		//		Cancels an in-flight download of content
		if(this._xhrDfd && (this._xhrDfd.fired == -1)){
			this._xhrDfd.cancel();
		}
		delete this._xhrDfd; // garbage collect

		this.onLoadDeferred = null;
	},

	uninitialize: function(){
		if(this._beingDestroyed){
			this.cancel();
		}
		this.inherited(arguments);
	},

	destroyRecursive: function(/*Boolean*/ preserveDom){
		// summary:
		//		Destroy the ContentPane and its contents

		// if we have multiple controllers destroying us, bail after the first
		if(this._beingDestroyed){
			return;
		}
		this.inherited(arguments);
	},

	resize: function(changeSize, resultSize){
		// summary:
		//		See `dijit.layout._LayoutWidget.resize` for description.
		//		Although ContentPane doesn't extend _LayoutWidget, it does implement
		//		the same API.

		// For the TabContainer --> BorderContainer --> ContentPane case, _onShow() is
		// never called, so resize() is our trigger to do the initial href download.
		if(!this._wasShown){
			this._onShow();
		}

		this._resizeCalled = true;

		// Set margin box size, unless it wasn't specified, in which case use current size.
		if(changeSize){
			dojo.marginBox(this.domNode, changeSize);
		}

		// Compute content box size of containerNode in case we [later] need to size our single child.
		var cn = this.containerNode;
		if(cn === this.domNode){
			// If changeSize or resultSize was passed to this method and this.containerNode ==
			// this.domNode then we can compute the content-box size without querying the node,
			// which is more reliable (similar to LayoutWidget.resize) (see for example #9449).
			var mb = resultSize || {};
			dojo.mixin(mb, changeSize || {}); // changeSize overrides resultSize
			if(!("h" in mb) || !("w" in mb)){
				mb = dojo.mixin(dojo.marginBox(cn), mb); // just use dojo.marginBox() to fill in missing values
			}
			this._contentBox = dijit.layout.marginBox2contentBox(cn, mb);
		}else{
			this._contentBox = dojo.contentBox(cn);
		}

		// Make my children layout, or size my single child widget
		this._layoutChildren();
	},

	_isShown: function(){
		// summary:
		//		Returns true if the content is currently shown.
		// description:
		//		If I am a child of a layout widget then it actually returns true if I've ever been visible,
		//		not whether I'm currently visible, since that's much faster than tracing up the DOM/widget
		//		tree every call, and at least solves the performance problem on page load by deferring loading
		//		hidden ContentPanes until they are first shown

		if(this._childOfLayoutWidget){
			// If we are TitlePane, etc - we return that only *IF* we've been resized
			if(this._resizeCalled && "open" in this){
				return this.open;
			}
			return this._resizeCalled;
		}else if("open" in this){
			return this.open;		// for TitlePane, etc.
		}else{
			// TODO: with _childOfLayoutWidget check maybe this branch no longer necessary?
			var node = this.domNode;
			return (node.style.display != 'none') && (node.style.visibility != 'hidden') && !dojo.hasClass(node, "dijitHidden");
		}
	},

	_onShow: function(){
		// summary:
		//		Called when the ContentPane is made visible
		// description:
		//		For a plain ContentPane, this is called on initialization, from startup().
		//		If the ContentPane is a hidden pane of a TabContainer etc., then it's
		//		called whenever the pane is made visible.
		//
		//		Does necessary processing, including href download and layout/resize of
		//		child widget(s)

		if(this.href){
			if(!this._xhrDfd && // if there's an href that isn't already being loaded
				(!this.isLoaded || this._hrefChanged || this.refreshOnShow)
			){
				this.refresh();
			}
		}else{
			// If we are the child of a layout widget then the layout widget will call resize() on
			// us, and then we will size our child/children.   Otherwise, we need to do it now.
			if(!this._childOfLayoutWidget && this._needLayout){
				// If a layout has been scheduled for when we become visible, do it now
				this._layoutChildren();
			}
		}

		this.inherited(arguments);

		// Need to keep track of whether ContentPane has been shown (which is different than
		// whether or not it's currently visible).
		this._wasShown = true;
	},

	refresh: function(){
		// summary:
		//		[Re]download contents of href and display
		// description:
		//		1. cancels any currently in-flight requests
		//		2. posts "loading..." message
		//		3. sends XHR to download new data

		// Cancel possible prior in-flight request
		this.cancel();

		this.onLoadDeferred = new dojo.Deferred(dojo.hitch(this, "cancel"));
		this._load();
		return this.onLoadDeferred;
	},

	_load: function(){
		// summary:
		//		Load/reload the href specified in this.href

		// display loading message
		this._setContent(this.onDownloadStart(), true);

		var self = this;
		var getArgs = {
			preventCache: (this.preventCache || this.refreshOnShow),
			url: this.href,
			handleAs: "text"
		};
		if(dojo.isObject(this.ioArgs)){
			dojo.mixin(getArgs, this.ioArgs);
		}

		var hand = (this._xhrDfd = (this.ioMethod || dojo.xhrGet)(getArgs));

		hand.addCallback(function(html){
			try{
				self._isDownloaded = true;
				self._setContent(html, false);
				self.onDownloadEnd();
			}catch(err){
				self._onError('Content', err); // onContentError
			}
			delete self._xhrDfd;
			return html;
		});

		hand.addErrback(function(err){
			if(!hand.canceled){
				// show error message in the pane
				self._onError('Download', err); // onDownloadError
			}
			delete self._xhrDfd;
			return err;
		});

		// Remove flag saying that a load is needed
		delete this._hrefChanged;
	},

	_onLoadHandler: function(data){
		// summary:
		//		This is called whenever new content is being loaded
		this.isLoaded = true;
		try{
			this.onLoadDeferred.callback(data);
			this.onLoad(data);
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onLoad code: ' + e.message);
		}
	},

	_onUnloadHandler: function(){
		// summary:
		//		This is called whenever the content is being unloaded
		this.isLoaded = false;
		try{
			this.onUnload();
		}catch(e){
			console.error('Error '+this.widgetId+' running custom onUnload code: ' + e.message);
		}
	},

	destroyDescendants: function(){
		// summary:
		//		Destroy all the widgets inside the ContentPane and empty containerNode

		// Make sure we call onUnload (but only when the ContentPane has real content)
		if(this.isLoaded){
			this._onUnloadHandler();
		}

		// Even if this.isLoaded == false there might still be a "Loading..." message
		// to erase, so continue...

		// For historical reasons we need to delete all widgets under this.containerNode,
		// even ones that the user has created manually.
		var setter = this._contentSetter;
		dojo.forEach(this.getChildren(), function(widget){
			if(widget.destroyRecursive){
				widget.destroyRecursive();
			}
		});
		if(setter){
			// Most of the widgets in setter.parseResults have already been destroyed, but
			// things like Menu that have been moved to <body> haven't yet
			dojo.forEach(setter.parseResults, function(widget){
				if(widget.destroyRecursive && widget.domNode && widget.domNode.parentNode == dojo.body()){
					widget.destroyRecursive();
				}
			});
			delete setter.parseResults;
		}

		// And then clear away all the DOM nodes
		dojo.html._emptyNode(this.containerNode);

		// Delete any state information we have about current contents
		delete this._singleChild;
	},

	_setContent: function(cont, isFakeContent){
		// summary:
		//		Insert the content into the container node

		// first get rid of child widgets
		this.destroyDescendants();

		// dojo.html.set will take care of the rest of the details
		// we provide an override for the error handling to ensure the widget gets the errors
		// configure the setter instance with only the relevant widget instance properties
		// NOTE: unless we hook into attr, or provide property setters for each property,
		// we need to re-configure the ContentSetter with each use
		var setter = this._contentSetter;
		if(! (setter && setter instanceof dojo.html._ContentSetter)){
			setter = this._contentSetter = new dojo.html._ContentSetter({
				node: this.containerNode,
				_onError: dojo.hitch(this, this._onError),
				onContentError: dojo.hitch(this, function(e){
					// fires if a domfault occurs when we are appending this.errorMessage
					// like for instance if domNode is a UL and we try append a DIV
					var errMess = this.onContentError(e);
					try{
						this.containerNode.innerHTML = errMess;
					}catch(e){
						console.error('Fatal '+this.id+' could not change content due to '+e.message, e);
					}
				})/*,
				_onError */
			});
		};

		var setterParams = dojo.mixin({
			cleanContent: this.cleanContent,
			extractContent: this.extractContent,
			parseContent: this.parseOnLoad,
			dir: this.dir,
			lang: this.lang
		}, this._contentSetterParams || {});

		dojo.mixin(setter, setterParams);

		setter.set( (dojo.isObject(cont) && cont.domNode) ? cont.domNode : cont );

		// setter params must be pulled afresh from the ContentPane each time
		delete this._contentSetterParams;

		if(!isFakeContent){
			// Startup each top level child widget (and they will start their children, recursively)
			dojo.forEach(this.getChildren(), function(child){
				// The parser has already called startup on all widgets *without* a getParent() method
				if(!this.parseOnLoad || child.getParent){
					child.startup();
				}
			}, this);

			// Call resize() on each of my child layout widgets,
			// or resize() on my single child layout widget...
			// either now (if I'm currently visible)
			// or when I become visible
			this._scheduleLayout();

			this._onLoadHandler(cont);
		}
	},

	_onError: function(type, err, consoleText){
		this.onLoadDeferred.errback(err);

		// shows user the string that is returned by on[type]Error
		// overide on[type]Error and return your own string to customize
		var errText = this['on' + type + 'Error'].call(this, err);
		if(consoleText){
			console.error(consoleText, err);
		}else if(errText){// a empty string won't change current content
			this._setContent(errText, true);
		}
	},

	_scheduleLayout: function(){
		// summary:
		//		Call resize() on each of my child layout widgets, either now
		//		(if I'm currently visible) or when I become visible
		if(this._isShown()){
			this._layoutChildren();
		}else{
			this._needLayout = true;
		}
	},

	_layoutChildren: function(){
		// summary:
		//		Since I am a Container widget, each of my children expects me to
		//		call resize() or layout() on them.
		// description:
		//		Should be called on initialization and also whenever we get new content
		//		(from an href, or from attr('content', ...))... but deferred until
		//		the ContentPane is visible

		if(this.doLayout){
			this._checkIfSingleChild();
		}

		if(this._singleChild && this._singleChild.resize){
			var cb = this._contentBox || dojo.contentBox(this.containerNode);

			// note: if widget has padding this._contentBox will have l and t set,
			// but don't pass them to resize() or it will doubly-offset the child
			this._singleChild.resize({w: cb.w, h: cb.h});
		}else{
			// All my child widgets are independently sized (rather than matching my size),
			// but I still need to call resize() on each child to make it layout.
			dojo.forEach(this.getChildren(), function(widget){
				if(widget.resize){
					widget.resize();
				}
			});
		}
		delete this._needLayout;
	},

	// EVENT's, should be overide-able
	onLoad: function(data){
		// summary:
		//		Event hook, is called after everything is loaded and widgetified
		// tags:
		//		callback
	},

	onUnload: function(){
		// summary:
		//		Event hook, is called before old content is cleared
		// tags:
		//		callback
	},

	onDownloadStart: function(){
		// summary:
		//		Called before download starts.
		// description:
		//		The string returned by this function will be the html
		//		that tells the user we are loading something.
		//		Override with your own function if you want to change text.
		// tags:
		//		extension
		return this.loadingMessage;
	},

	onContentError: function(/*Error*/ error){
		// summary:
		//		Called on DOM faults, require faults etc. in content.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		By default (if this method is not overriden), it returns
		//		nothing, so the error message is just printed to the console.
		// tags:
		//		extension
	},

	onDownloadError: function(/*Error*/ error){
		// summary:
		//		Called when download error occurs.
		//
		//		In order to display an error message in the pane, return
		//		the error message from this method, as an HTML string.
		//
		//		Default behavior (if this method is not overriden) is to display
		//		the error message inside the pane.
		// tags:
		//		extension
		return this.errorMessage;
	},

	onDownloadEnd: function(){
		// summary:
		//		Called when download is finished.
		// tags:
		//		callback
	}
});

}

if(!dojo._hasResource["dijit.TooltipDialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.TooltipDialog"] = true;
dojo.provide("dijit.TooltipDialog");






dojo.declare(
		"dijit.TooltipDialog",
		[dijit.layout.ContentPane, dijit._Templated, dijit.form._FormMixin, dijit._DialogMixin],
		{
			// summary:
			//		Pops up a dialog that appears like a Tooltip

			// title: String
			// 		Description of tooltip dialog (required for a11y)
			title: "",

			// doLayout: [protected] Boolean
			//		Don't change this parameter from the default value.
			//		This ContentPane parameter doesn't make sense for TooltipDialog, since TooltipDialog
			//		is never a child of a layout container, nor can you specify the size of
			//		TooltipDialog in order to control the size of an inner widget.
			doLayout: false,

			// autofocus: Boolean
			// 		A Toggle to modify the default focus behavior of a Dialog, which
			// 		is to focus on the first dialog element after opening the dialog.
			//		False will disable autofocusing. Default: true
			autofocus: true,

			// baseClass: [protected] String
			//		The root className to use for the various states of this widget
			baseClass: "dijitTooltipDialog",

			// _firstFocusItem: [private] [readonly] DomNode
			//		The pointer to the first focusable node in the dialog.
			//		Set by `dijit._DialogMixin._getFocusItems`.
			_firstFocusItem: null,

			// _lastFocusItem: [private] [readonly] DomNode
			//		The pointer to which node has focus prior to our dialog.
			//		Set by `dijit._DialogMixin._getFocusItems`.
			_lastFocusItem: null,

			templateString: dojo.cache("dijit", "templates/TooltipDialog.html", "<div waiRole=\"presentation\">\n\t<div class=\"dijitTooltipContainer\" waiRole=\"presentation\">\n\t\t<div class =\"dijitTooltipContents dijitTooltipFocusNode\" dojoAttachPoint=\"containerNode\" tabindex=\"-1\" waiRole=\"dialog\"></div>\n\t</div>\n\t<div class=\"dijitTooltipConnector\" waiRole=\"presentation\"></div>\n</div>\n"),

			postCreate: function(){
				this.inherited(arguments);
				this.connect(this.containerNode, "onkeypress", "_onKey");
				this.containerNode.title = this.title;
			},

			orient: function(/*DomNode*/ node, /*String*/ aroundCorner, /*String*/ corner){
				// summary:
				//		Configure widget to be displayed in given position relative to the button.
				//		This is called from the dijit.popup code, and should not be called
				//		directly.
				// tags:
				//		protected
				var c = this._currentOrientClass;
				if(c){
					dojo.removeClass(this.domNode, c);
				}
				c = "dijitTooltipAB"+(corner.charAt(1) == 'L'?"Left":"Right")+" dijitTooltip"+(corner.charAt(0) == 'T' ? "Below" : "Above");
				dojo.addClass(this.domNode, c);
				this._currentOrientClass = c;
			},

			focus: function(){
				// summary:
				//		Focus on first field
				this._getFocusItems(this.containerNode);
				dijit.focus(this._firstFocusItem);
			},

			onOpen: function(/*Object*/ pos){
				// summary:
				//		Called when dialog is displayed.
				//		This is called from the dijit.popup code, and should not be called directly.
				// tags:
				//		protected

				this.orient(this.domNode,pos.aroundCorner, pos.corner);
				this._onShow(); // lazy load trigger
			},

			onClose: function(){
				// summary:
				//		Called when dialog is hidden.
				//		This is called from the dijit.popup code, and should not be called directly.
				// tags:
				//		protected
				this.onHide();
			},

			_onKey: function(/*Event*/ evt){
				// summary:
				//		Handler for keyboard events
				// description:
				//		Keep keyboard focus in dialog; close dialog on escape key
				// tags:
				//		private

				var node = evt.target;
				var dk = dojo.keys;
				if(evt.charOrCode === dk.TAB){
					this._getFocusItems(this.containerNode);
				}
				var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
				if(evt.charOrCode == dk.ESCAPE){
					// Use setTimeout to avoid crash on IE, see #10396.
					setTimeout(dojo.hitch(this, "onCancel"), 0);
					dojo.stopEvent(evt);
				}else if(node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === dk.TAB){
					if(!singleFocusItem){
						dijit.focus(this._lastFocusItem); // send focus to last item in dialog
					}
					dojo.stopEvent(evt);
				}else if(node == this._lastFocusItem && evt.charOrCode === dk.TAB && !evt.shiftKey){
					if(!singleFocusItem){
						dijit.focus(this._firstFocusItem); // send focus to first item in dialog
					}
					dojo.stopEvent(evt);
				}else if(evt.charOrCode === dk.TAB){
					// we want the browser's default tab handling to move focus
					// but we don't want the tab to propagate upwards
					evt.stopPropagation();
				}
			}
		}
	);

}

if(!dojo._hasResource["dijit.Dialog"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.Dialog"] = true;
dojo.provide("dijit.Dialog");















/*=====
dijit._underlay = function(kwArgs){
	// summary:
	//		A shared instance of a `dijit.DialogUnderlay`
	//
	// description:
	//		A shared instance of a `dijit.DialogUnderlay` created and
	//		used by `dijit.Dialog`, though never created until some Dialog
	//		or subclass thereof is shown.
};
=====*/

dojo.declare(
	"dijit._DialogBase",
	[dijit._Templated, dijit.form._FormMixin, dijit._DialogMixin, dijit._CssStateMixin],
	{
		// summary:
		//		A modal dialog Widget
		//
		// description:
		//		Pops up a modal dialog window, blocking access to the screen
		//		and also graying out the screen Dialog is extended from
		//		ContentPane so it supports all the same parameters (href, etc.)
		//
		// example:
		// |	<div dojoType="dijit.Dialog" href="test.html"></div>
		//
		// example:
		// |	var foo = new dijit.Dialog({ title: "test dialog", content: "test content" };
		// |	dojo.body().appendChild(foo.domNode);
		// |	foo.startup();

		templateString: dojo.cache("dijit", "templates/Dialog.html", "<div class=\"dijitDialog\" tabindex=\"-1\" waiRole=\"dialog\" waiState=\"labelledby-${id}_title\">\n\t<div dojoAttachPoint=\"titleBar\" class=\"dijitDialogTitleBar\">\n\t<span dojoAttachPoint=\"titleNode\" class=\"dijitDialogTitle\" id=\"${id}_title\"></span>\n\t<span dojoAttachPoint=\"closeButtonNode\" class=\"dijitDialogCloseIcon\" dojoAttachEvent=\"onclick: onCancel\" title=\"${buttonCancel}\">\n\t\t<span dojoAttachPoint=\"closeText\" class=\"closeText\" title=\"${buttonCancel}\">x</span>\n\t</span>\n\t</div>\n\t\t<div dojoAttachPoint=\"containerNode\" class=\"dijitDialogPaneContent\"></div>\n</div>\n"),
		
		baseClass: "dijitDialog",
		
		cssStateNodes: {
			closeButtonNode: "dijitDialogCloseIcon"
		},

		attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
			title: [
				{ node: "titleNode", type: "innerHTML" },
				{ node: "titleBar", type: "attribute" }
			],
			"aria-describedby":""
		}),

		// open: Boolean
		//		True if Dialog is currently displayed on screen.
		open: false,

		// duration: Integer
		//		The time in milliseconds it takes the dialog to fade in and out
		duration: dijit.defaultDuration,

		// refocus: Boolean
		// 		A Toggle to modify the default focus behavior of a Dialog, which
		// 		is to re-focus the element which had focus before being opened.
		//		False will disable refocusing. Default: true
		refocus: true,

		// autofocus: Boolean
		// 		A Toggle to modify the default focus behavior of a Dialog, which
		// 		is to focus on the first dialog element after opening the dialog.
		//		False will disable autofocusing. Default: true
		autofocus: true,

		// _firstFocusItem: [private] [readonly] DomNode
		//		The pointer to the first focusable node in the dialog.
		//		Set by `dijit._DialogMixin._getFocusItems`.
		_firstFocusItem: null,

		// _lastFocusItem: [private] [readonly] DomNode
		//		The pointer to which node has focus prior to our dialog.
		//		Set by `dijit._DialogMixin._getFocusItems`.
		_lastFocusItem: null,

		// doLayout: [protected] Boolean
		//		Don't change this parameter from the default value.
		//		This ContentPane parameter doesn't make sense for Dialog, since Dialog
		//		is never a child of a layout container, nor can you specify the size of
		//		Dialog in order to control the size of an inner widget.
		doLayout: false,

		// draggable: Boolean
		//		Toggles the moveable aspect of the Dialog. If true, Dialog
		//		can be dragged by it's title. If false it will remain centered
		//		in the viewport.
		draggable: true,

		//aria-describedby: String
		//		Allows the user to add an aria-describedby attribute onto the dialog.   The value should
		//		be the id of the container element of text that describes the dialog purpose (usually
		//		the first text in the dialog).
		//		<div dojoType="dijit.Dialog" aria-describedby="intro" .....>
		//			<div id="intro">Introductory text</div>
		//			<div>rest of dialog contents</div>
		//		</div>
		"aria-describedby":"",

		postMixInProperties: function(){
			var _nlsResources = dojo.i18n.getLocalization("dijit", "common");
			dojo.mixin(this, _nlsResources);
			this.inherited(arguments);
		},

		postCreate: function(){
			dojo.style(this.domNode, {
				display: "none",
				position:"absolute"
			});
			dojo.body().appendChild(this.domNode);

			this.inherited(arguments);

			this.connect(this, "onExecute", "hide");
			this.connect(this, "onCancel", "hide");
			this._modalconnects = [];
		},

		onLoad: function(){
			// summary:
			//		Called when data has been loaded from an href.
			//		Unlike most other callbacks, this function can be connected to (via `dojo.connect`)
			//		but should *not* be overriden.
			// tags:
			//		callback

			// when href is specified we need to reposition the dialog after the data is loaded
			// and find the focusable elements
			this._position();
			if(this.autofocus){
				this._getFocusItems(this.domNode);
				dijit.focus(this._firstFocusItem);
			}
			this.inherited(arguments);
		},

		_endDrag: function(e){
			// summary:
			//		Called after dragging the Dialog. Saves the position of the dialog in the viewport.
			// tags:
			//		private
			if(e && e.node && e.node === this.domNode){
				this._relativePosition = dojo.position(e.node);
			}
		},

		_setup: function(){
			// summary:
			//		Stuff we need to do before showing the Dialog for the first
			//		time (but we defer it until right beforehand, for
			//		performance reasons).
			// tags:
			//		private

			var node = this.domNode;

			if(this.titleBar && this.draggable){
				this._moveable = (dojo.isIE == 6) ?
					new dojo.dnd.TimedMoveable(node, { handle: this.titleBar }) :	// prevent overload, see #5285
					new dojo.dnd.Moveable(node, { handle: this.titleBar, timeout: 0 });
				this._dndListener = dojo.subscribe("/dnd/move/stop",this,"_endDrag");
			}else{
				dojo.addClass(node,"dijitDialogFixed");
			}

			this.underlayAttrs = {
				dialogId: this.id,
				"class": dojo.map(this["class"].split(/\s/), function(s){ return s+"_underlay"; }).join(" ")
			};

			this._fadeIn = dojo.fadeIn({
				node: node,
				duration: this.duration,
				beforeBegin: dojo.hitch(this, function(){
					var underlay = dijit._underlay;
					if(!underlay){
						underlay = dijit._underlay = new dijit.DialogUnderlay(this.underlayAttrs);
					}else{
						underlay.set(this.underlayAttrs);
					}

					var ds = dijit._dialogStack,
						zIndex = 948 + ds.length*2;
					if(ds.length == 1){	// first dialog
						underlay.show();
					}
					dojo.style(dijit._underlay.domNode, 'zIndex', zIndex);
					dojo.style(this.domNode, 'zIndex', zIndex + 1);
				}),
				onEnd: dojo.hitch(this, function(){
					if(this.autofocus){
						// find focusable Items each time dialog is shown since if dialog contains a widget the
						// first focusable items can change
						this._getFocusItems(this.domNode);
						dijit.focus(this._firstFocusItem);
					}
				})
			 });

			this._fadeOut = dojo.fadeOut({
				node: node,
				duration: this.duration,
				onEnd: dojo.hitch(this, function(){
					node.style.display = "none";

					// Restore the previous dialog in the stack, or if this is the only dialog
					// then restore to original page
					var ds = dijit._dialogStack;
					if(ds.length == 0){
						dijit._underlay.hide();
					}else{
						dojo.style(dijit._underlay.domNode, 'zIndex', 948 + ds.length*2);
						dijit._underlay.set(ds[ds.length-1].underlayAttrs);
					}

					// Restore focus to wherever it was before this dialog was displayed
					if(this.refocus){
						var focus = this._savedFocus;

						// If we are returning control to a previous dialog but for some reason
						// that dialog didn't have a focused field, set focus to first focusable item.
						// This situation could happen if two dialogs appeared at nearly the same time,
						// since a dialog doesn't set it's focus until the fade-in is finished.
						if(ds.length > 0){
							var pd = ds[ds.length-1];
							if(!dojo.isDescendant(focus.node, pd.domNode)){
								pd._getFocusItems(pd.domNode);
								focus = pd._firstFocusItem;
							}
						}

						dijit.focus(focus);
					}
				})
			 });
		},

		uninitialize: function(){
			var wasPlaying = false;
			if(this._fadeIn && this._fadeIn.status() == "playing"){
				wasPlaying = true;
				this._fadeIn.stop();
			}
			if(this._fadeOut && this._fadeOut.status() == "playing"){
				wasPlaying = true;
				this._fadeOut.stop();
			}
			
			// Hide the underlay, unless the underlay widget has already been destroyed
			// because we are being called during page unload (when all widgets are destroyed)
			if((this.open || wasPlaying) && !dijit._underlay._destroyed){
				dijit._underlay.hide();
			}

			if(this._moveable){
				this._moveable.destroy();
			}
			this.inherited(arguments);
		},

		_size: function(){
			// summary:
			// 		If necessary, shrink dialog contents so dialog fits in viewport
			// tags:
			//		private

			this._checkIfSingleChild();

			// If we resized the dialog contents earlier, reset them back to original size, so
			// that if the user later increases the viewport size, the dialog can display w/out a scrollbar.
			// Need to do this before the dojo.marginBox(this.domNode) call below.
			if(this._singleChild){
				if(this._singleChildOriginalStyle){
					this._singleChild.domNode.style.cssText = this._singleChildOriginalStyle;
				}
				delete this._singleChildOriginalStyle;
			}else{
				dojo.style(this.containerNode, {
					width:"auto",
					height:"auto"
				});
			}

			var mb = dojo.marginBox(this.domNode);
			var viewport = dojo.window.getBox();
			if(mb.w >= viewport.w || mb.h >= viewport.h){
				// Reduce size of dialog contents so that dialog fits in viewport

				var w = Math.min(mb.w, Math.floor(viewport.w * 0.75)),
					h = Math.min(mb.h, Math.floor(viewport.h * 0.75));

				if(this._singleChild && this._singleChild.resize){
					this._singleChildOriginalStyle = this._singleChild.domNode.style.cssText;
					this._singleChild.resize({w: w, h: h});
				}else{
					dojo.style(this.containerNode, {
						width: w + "px",
						height: h + "px",
						overflow: "auto",
						position: "relative"	// workaround IE bug moving scrollbar or dragging dialog
					});
				}
			}else{
				if(this._singleChild && this._singleChild.resize){
					this._singleChild.resize();
				}
			}
		},

		_position: function(){
			// summary:
			//		Position modal dialog in the viewport. If no relative offset
			//		in the viewport has been determined (by dragging, for instance),
			//		center the node. Otherwise, use the Dialog's stored relative offset,
			//		and position the node to top: left: values based on the viewport.
			// tags:
			//		private
			if(!dojo.hasClass(dojo.body(),"dojoMove")){
				var node = this.domNode,
					viewport = dojo.window.getBox(),
					p = this._relativePosition,
					bb = p ? null : dojo._getBorderBox(node),
					l = Math.floor(viewport.l + (p ? p.x : (viewport.w - bb.w) / 2)),
					t = Math.floor(viewport.t + (p ? p.y : (viewport.h - bb.h) / 2))
				;
				dojo.style(node,{
					left: l + "px",
					top: t + "px"
				});
			}
		},

		_onKey: function(/*Event*/ evt){
			// summary:
			//		Handles the keyboard events for accessibility reasons
			// tags:
			//		private

			var ds = dijit._dialogStack;
			if(ds[ds.length-1] != this){
				// console.debug(this.id + ': skipping because', this, 'is not the active dialog');
				return;
			}

			if(evt.charOrCode){
				var dk = dojo.keys;
				var node = evt.target;
				if(evt.charOrCode === dk.TAB){
					this._getFocusItems(this.domNode);
				}
				var singleFocusItem = (this._firstFocusItem == this._lastFocusItem);
				// see if we are shift-tabbing from first focusable item on dialog
				if(node == this._firstFocusItem && evt.shiftKey && evt.charOrCode === dk.TAB){
					if(!singleFocusItem){
						dijit.focus(this._lastFocusItem); // send focus to last item in dialog
					}
					dojo.stopEvent(evt);
				}else if(node == this._lastFocusItem && evt.charOrCode === dk.TAB && !evt.shiftKey){
					if(!singleFocusItem){
						dijit.focus(this._firstFocusItem); // send focus to first item in dialog
					}
					dojo.stopEvent(evt);
				}else{
					// see if the key is for the dialog
					while(node){
						if(node == this.domNode || dojo.hasClass(node, "dijitPopup")){
							if(evt.charOrCode == dk.ESCAPE){
								this.onCancel();
							}else{
								return; // just let it go
							}
						}
						node = node.parentNode;
					}
					// this key is for the disabled document window
					if(evt.charOrCode !== dk.TAB){ // allow tabbing into the dialog for a11y
						dojo.stopEvent(evt);
					// opera won't tab to a div
					}else if(!dojo.isOpera){
						try{
							this._firstFocusItem.focus();
						}catch(e){ /*squelch*/ }
					}
				}
			}
		},

		show: function(){
			// summary:
			//		Display the dialog
			if(this.open){ return; }

			// first time we show the dialog, there's some initialization stuff to do
			if(!this._alreadyInitialized){
				this._setup();
				this._alreadyInitialized=true;
			}

			if(this._fadeOut.status() == "playing"){
				this._fadeOut.stop();
			}

			this._modalconnects.push(dojo.connect(window, "onscroll", this, "layout"));
			this._modalconnects.push(dojo.connect(window, "onresize", this, function(){
				// IE gives spurious resize events and can actually get stuck
				// in an infinite loop if we don't ignore them
				var viewport = dojo.window.getBox();
				if(!this._oldViewport ||
						viewport.h != this._oldViewport.h ||
						viewport.w != this._oldViewport.w){
					this.layout();
					this._oldViewport = viewport;
				}
			}));
			this._modalconnects.push(dojo.connect(dojo.doc.documentElement, "onkeypress", this, "_onKey"));

			dojo.style(this.domNode, {
				opacity:0,
				display:""
			});

			this.open = true;
			this._onShow(); // lazy load trigger

			this._size();
			this._position();
			dijit._dialogStack.push(this);
			this._fadeIn.play();

			this._savedFocus = dijit.getFocus(this);
		},

		hide: function(){
			// summary:
			//		Hide the dialog

			// if we haven't been initialized yet then we aren't showing and we can just return
			// or if we aren't the active dialog, don't allow us to close yet
			var ds = dijit._dialogStack;
			if(!this._alreadyInitialized || this != ds[ds.length-1]){
				return;
			}

			if(this._fadeIn.status() == "playing"){
				this._fadeIn.stop();
			}

			// throw away current active dialog from stack -- making the previous dialog or the node on the original page active
			ds.pop();

			this._fadeOut.play();

			if(this._scrollConnected){
				this._scrollConnected = false;
			}
			dojo.forEach(this._modalconnects, dojo.disconnect);
			this._modalconnects = [];

			if(this._relativePosition){
				delete this._relativePosition;
			}
			this.open = false;

			this.onHide();
		},

		layout: function(){
			// summary:
			//		Position the Dialog and the underlay
			// tags:
			//		private
			if(this.domNode.style.display != "none"){
				if(dijit._underlay){	// avoid race condition during show()
					dijit._underlay.layout();
				}
				this._position();
			}
		},

		destroy: function(){
			dojo.forEach(this._modalconnects, dojo.disconnect);
			if(this.refocus && this.open){
				setTimeout(dojo.hitch(dijit,"focus",this._savedFocus), 25);
			}
			if(this._dndListener){
				dojo.unsubscribe(this._dndListener);
			}
			this.inherited(arguments);
		}
	}
);

dojo.declare(
	"dijit.Dialog",
	[dijit.layout.ContentPane, dijit._DialogBase],
	{}
);

// Stack of currenctly displayed dialogs, layered on top of each other
dijit._dialogStack = [];

// For back-compat.  TODO: remove in 2.0


}

if(!dojo._hasResource["itme.ChassisMap.ChassisMap"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.ChassisMap.ChassisMap"] = true;
dojo.provide("itme.ChassisMap.ChassisMap");

















dojo.declare("itme.ChassisMap.ChassisMap", [itme.ChassisMap._Utils,
                                 itme.ChassisMap._Layout,
                                 itme.ChassisMap._Selection,
                                 itme.ChassisMap._Overlays], {
  // private members
  _dataSet: null,
  _demoMode: false,
  _args: null,
  _itemsList: null,
  _itemIndex: 0,
  _datasetTarget: "CHASSIS_MAP_COLLECTION",
  _datasetParms: null,
  _refreshTime: null,
  _isRendered: false,
  _isRefreshPath: false,
  _noDataEncouteredCount: 0,
  _defaultProperties: null,
  //_TEST_NODATA: false,
  _issues: {
    warnings: []
  },
  MRI: null,
  IMAGES : new itme.ChassisMap.ImageResources(),

  constructor: function(kwArgs, chassisMapParentDOMNodeID, groupTarget, chassisId){
    this._args = {    };

    
    this.MRI = dojo.i18n.getLocalization("itme", "Resources");

    dojo.addOnUnload(dojo.hitch(this, this.cleanUp));

    itmeUtils.CMTrace("MAIN", "Constructor");

    this._defaultProperties = ["AccessState", "OTHERHWVIEWPROPS", "ChassisSlot", "OccupiedSlots", "Model", "STATUS",
                               "MachineType", "SerialNumber", "Bay", "ObjectType", "COMPLIANCEINFO", "IPv4Address",
                               "EnclosureFRU", "Status", "DisplayName", "ACTIVESTATUSINFO", "CommunicationState",
                               "Architecture", "Manufacturer", "PowerState", "SystemBoardUUID", "MACAddress",
                               "DisplayOperationalStatusTime", "Problems", "OperatingState", "HostName"];

    this._unSetBadPrototypes();

    dojo.mixin(this._args, kwArgs);

    if ( groupTarget )
      this._datasetTarget = groupTarget;
    if ( chassisId )
      this._datasetParms = chassisId;
    //this._chassisMapLayout = new itme.ChassisMapLayout({});

    //setTimeout(dojo.hitch(this, this._NoDataTestM01), 30000);

    this.cleanUp(); // ensures all previous data and dojo widgets are cleaned up
  },

  // TYPE: RENDER
  // public : set initial dimensions from the given node.
  setInitialSizingDimensionsFromNode: function(nodeToGetInitialSizingFrom)
  {
    this.setDimensionsBasedOnNode(nodeToGetInitialSizingFrom);
  },

  // public : given the parameters in the constructor, build and render the chassis map
  // TYPE: MAIN
  renderMap: function(dataset)
  {
    if ( dataset )
      this._acquireData(dataset);
    else
      this._acquireData();
  },

  // TYPE: MAIN
  // private
  _acquireData: function(dataset)
  {

    itmeUtils.CMTrace("MAIN", "ChassisMap.acquireData() method ENTERED");
    var onChassisRestQueryReturnFunctions = { // execution will resume here after makeRestQuery
      onComplete: dojo.hitch(this, this._onChassisDataRetrieved),
      onError: dojo.hitch(this, this._onChassisDataRetrievalFailed)
    };

    if ( ! this._isRefreshPath )
    {
      this.cleanUp();
      this._buildChassisMapHTML();
      this._toggleWaitingPanel(true, this.MRI.Loading );
    }

    if ( dataset )
      this._dataset = dataset;
    else {
      itmeUtils.CMTrace("MAIN", "ChassisMap.acquireData() - building dataset for target: " + this._datasetTarget + " , datasetParms: "+ this._datasetParms);
      try {
        this._dataset = new ibm.tivoli.navmodel.Dataset({
            baseUrl: "/ibm/tivoli/rest",
            provider: "USMI",
            datasource: "",
            dataset: this._datasetTarget,
            parameters: {
                  homeParms: this._datasetParms
            },
            defaultProperties: this._defaultProperties
        });
      } catch (err) {
        this._onChassisDataRetrievalFailed("dataset constructor failed");
        return;
      }
    }
    itmeUtils.CMTrace("MAIN", "fetching dataset items...");
    itmeUtils.initializeTimer("ChassiMapTimer");

    this._dataset.addEventListener(this, dojo.hitch(this, this._eventHandler));
    this._dataset.getItemsAndLinks(onChassisRestQueryReturnFunctions);
    itmeUtils.CMTrace("MAIN", "sent request for dataset items...");
  },

  _eventTimeout: null,
  _initialEventTime: null,
  // when an event is fired on a data item we're listening to, refresh the hardware view.
  // Note: many events may come through really quickly, so after 3 seconds of no more events, the refresh will fire.
  _eventHandler: function(listenerObject, event) {
    itmeUtils.CMTrace("MAIN", "EVENT RECIEVED");
    //itmeUtils.CMTrace("MAIN", "event datasetId=" + event.getDataset().getId() + " event=" + event.getType() +  " eventId=" + event.getId());

    clearTimeout(this._eventTimeout); // reset the timeout window.
    if ( this._initialEventTime == null ) {
      this._initialEventTime = new Date();
      this._eventTimeout = setTimeout(dojo.hitch(this, this._refreshView, "evented"), 5000);
    // FORCE a refresh if blocked for 30 seconds or more by constant incoming events.
    } else if ( ((new Date()).getTime() - this._initialEventTime.getTime()) > 30000 ){
      itmeUtils.CMTrace("MAIN", "Persistent events coming in.  30 seconds since first event, forcing a refresh.");
      clearTimeout(this._eventTimeout);
      this._refreshView("evented");
    }
    else {
      this._eventTimeout = setTimeout(dojo.hitch(this, this._refreshView, "evented"), 5000);
    }
  },

  // TYPE: MAIN
  // private: execution resumes here when dataset query completes
  _onChassisDataRetrieved: function(items, links, childDatasetItems)
  {
    itmeUtils.CMTrace("MAIN", "chassis items retrieval RETURNED count: " + items.length);
    itmeUtils.traceTimeDelta("ChassiMapTimer", "HWView - Dataset Retrieval", true);

    this._fetchItemsProperties(items);

    if (this._itemsList.length === 0)  { // DO NOT FAIL, wait x seconds and try again.
      this._noDataEncouteredCount++;
      //itmeUtils.CMTrace("MAIN", "DEBUG: No Data (or test no data).  Count = " + this._noDataEncouteredCount);
      if (this._noDataEncouteredCount > 3) {
        this._noDataEncouteredCount = 0;
        //itmeUtils.CMTrace("MAIN", "DEBUG: No data exceeded 3 attempts, swapping to table view");
        this._displayErrorAndChangeToTableView(this.MRI.ERROR_HWVIEW_NoData);
      } else {
        //itmeUtils.CMTrace("MAIN", "DEBUG: Refresh in 10 seconds...");
        clearTimeout(this._eventTimeout);
        this._eventTimeout = setTimeout(dojo.hitch(this, this._refreshView, "evented"), 10000);
        this._toggleWaitingPanel(false);
      }
      return;
    } else
      this._noDataEncouteredCount = 0;


    itmeUtils.traceTimeDelta("ChassiMapTimer", "HWView - Propterties Retrieval", true);

    //Process the list of links
    //if (links) {
    //  for (var index = 0;  index < links.length;  ++index) {
    //    var link = links[index];
    //
    //    itmeUtils.CMTrace("MAIN", "dataset link["+index+"] : " + link);
    //  }
    //}

    // buildChassisMap
    this._toggleWaitingPanel(false);
    this._renderMap();
  },

  /*
  _NoDataTestM01: function() {
    itmeUtils.CMTrace("MAIN", "DEBUG: _NoDataTestM01 ENTERED");
    this._TEST_NODATA = true;
    this._eventHandler("","");
    setTimeout(dojo.hitch(this, this._NoDataTestM02), 45000);
  },

  _NoDataTestM02: function() {
    itmeUtils.CMTrace("MAIN", "DEBUG: _NoDataTestM02 ENTERED");
    this._TEST_NODATA = false;
    //this._eventHandler("","");
    //setTimeout(dojo.hitch(this, this._NoDataTestM02), 45000);
  },
  */

  _fetchItemsProperties: function(items) {
    var index;
    this._itemsList = items; // store the item list of other member functions to access

    for (index = 0;  index < this._itemsList.length;  ++index) {
      this._itemIndex = index;
      var item = this._itemsList[index];
      itmeUtils.CMTrace("MAIN", (index+1)+". id: " + item.getId() + " , type: " + item.getType() + " , label: " + item.getLabel() );

      var itemPropArgs = {
          onComplete: dojo.hitch(this, this._onPropertiesDataRetrieved, index),
          onError:    dojo.hitch(this, this._onPropertiesDataRetrievedFailed)
      };

      //this._itemsList[index].getProperties(itemPropArgs);
    }
  },

  // TYPE: ERROR HANDLING
  _onChassisDataRetrievalFailed: function(error, request){
    itmeUtils.CMError("MAIN", "The items request to the datastore failed.  " + error); // go straight to the error log
    itmeUtils.CMError("MAIN", " - request: " + request);

    this._displayErrorAndChangeToTableView(this.MRI.ERROR_HWView_Unknown);
  },

  // TYPE: ERROR HANDLING
  _onPropertiesDataRetrievedFailed: function(error, request){
    itmeUtils.CMError("MAIN", "The properties request to the datastore failed.  " + error); // go straight to the error log
    itmeUtils.CMError("MAIN", " - request: " + request);

    this._displayErrorAndChangeToTableView(this.MRI.ERROR_HWView_PropertyDataRequestFailed);
    //if ( request )
    //  errorMessage += "<br><br>Request:<pre>"+request+"</pre>";
    //errorMessage += "<pre>"+error +"</pre>"
    //              + "<button dojoType='dijit.form.Button' type='submit' onClick='processRefresh(\"tableView\");'>"+ this.MRI.SwitchToTableView + "</button>";
    //var foo = new dijit.Dialog({ title: this.MRI.ERROR_HWView_Title, content: errorMessage });
    //foo.show();
  },

  // TYPE: ERROR HANDLING
  _displayErrorAndChangeToTableView: function(errorMessage) {

    var errorDialogString = this.MRI.ERROR_HWView_DataRequestFailed;
    //if ( request )
    //  errorDialogString += "<br><br>Request:<pre>"+request+"</pre>";
    errorDialogString += "<br><pre>"+errorMessage +"</pre>";

    if (errorMessage == this.MRI.ERROR_HWVIEW_NoData)
      processRefresh("nodata");
    else {
      //alert(FSMFlightRecorder.toString("\n"));
      processRefresh("tableView");
      setTimeout('processRefresh("tableView_cmerror'+ errorDialogString+'")', 3000);
    }
  },

  // TYPE: MAIN
  _onPropertiesDataRetrieved: function(itemIndex, properties, args){
    var i;

    if ( false  ) {
      itmeUtils.CMTrace("MAIN", (itemIndex+1) +". " + this._itemsList[itemIndex].getLabel());

      for (i = 0; i < properties.length; i++) {
        itmeUtils.CMTrace("MAIN", "  [ " + properties[i].getId() + " ] - " + properties[i].getDisplayValue());
      }
    }
  },

  // TYPE: RENDER
  _renderMap: function()  {
    var index, i;
    itmeUtils.CMTrace("MAIN", " _renderMap reached");

    if ( this._isRefreshPath ) {
      this.cleanUp({renderedOnly:true}); // true indicates only cleaning rendered objects
      this._buildChassisMapHTML();
      this._isRefreshPath = false;
    }
    this._isRendered = true;
    this._insertRefreshButton();

    itmeUtils.traceTimeDelta("ChassiMapTimer", "HWView - Chassis Map HTML Building", true);

    // iterate through items.
    for (index = 0; index < this._itemsList.length; index++) {
      var item = this._itemsList[index];

      var mapElement = this._associateAndModifyDomElementForItem(item);
      if (mapElement) { // if the element is not compliant, its skipped (mock objects)
        // add selection handler
        itmeUtils.makeEvent1Param(mapElement, dojo.hitch(this, this._onElementClicked), item, "click");
        this._buildTooltip(item, mapElement);
      }
      else
        itmeUtils.CMTrace("MAIN", "bad element skipped: " + item.getId() + " - " + item.getLabel());
    }

    itmeUtils.CMTrace("MAIN", "Warnings during render: " + this._issues.warnings.length);
    /*if ( this._issues.warnings.length > 0 ) {
      this._insertNotificationArea();
      var tooltipStr = "";
      for ( i = 0; i < this._issues.warnings.length; i++)
        tooltipStr = this._issues.warnings[i] + "<br>";

      new dijit.Tooltip({
        connectId: ["info_buttonContainer"],
        label: tooltipStr,
        position: "above",
        id: "warnings_tooltip"
      });
    }*/

    itmeUtils.traceTimeDelta("ChassiMapTimer", "HWView - ChassisMap Item Processing", true);

    this._insertOverlays(this._chassisOverlayElement, this._itemsList);
    this._setItemsForOverlays(this._itemsList);
    this._renderDefaultOverlay();
    this._prefetchAsynchOverlayData(this._itemsList, this._defaultProperties);
  },

  _insertNotificationArea: function() {

    this._appendFilterButtonIconStyleToDocument("notification_icon", "StatusWarningMinor");

    var buttonContainer = itmeUtils.createElement("info_buttonContainer",
                                                  this._chassisOverlayElement,
                                                  { elementType: "span",
                                                    styleClass: "hwViewWarnings",
                                                    style: { "position":"absolute", "right":"30px"} });

    buttonContainer.innerHTML = this.IMAGES.buildIconImageTag("StatusInformational", "", 22);
    /*var button = new dijit.form.Button({
            optionsTitle: "Refresh the hardware view",
            onClick: dojo.hitch(this, this._refreshView),
            iconClass: "refresh_icon",
            id: "refresh_dojoButton",
            showLabel: false,
            type: "button"
        });
    buttonContainer.appendChild(button.domNode);
     */

  },


  _insertRefreshButton: function() {

    this._appendFilterButtonIconStyleToDocument("refresh_icon", "RefreshIcon");

    var buttonContainer = itmeUtils.createElement("refresh_buttonContainer",
                                                  this._chassisOverlayElement,
                                                  { elementType: "span",
                                                    styleClass: "buttonContainer",
                                                    style: { "position":"absolute", "right":"0px"} });

    var button = new dijit.form.Button({
            optionsTitle: "Refresh the hardware view",
            onClick: dojo.hitch(this, this._refreshView),
            iconClass: "refresh_icon",
            id: "refresh_dojoButton",
            showLabel: false,
            type: "button"
        });
    buttonContainer.appendChild(button.domNode);


    new dijit.Tooltip({
      connectId: ["refresh_dojoButton"],
      label: "Refresh the hardware view",
      position: "above",
      id: "refresh_buttonTooltip"
    });
  },

  _refreshView: function(source) {
    //FSMFlightRecorder.test();
    itmeUtils.CMTrace("MAIN", "refreshing...");
    itmeUtils.CMTrace("MAIN", "DEBUG: refreshView - isRendered: " + this._isRendered);
    if ( !this._isRendered )
      return;
    if ( this._currentOverlay !== null )
      dojo.cookie("chasisMap.currentOverlay", this._currentOverlay.getId(), { expires: 1});

    this._toggleWaitingPanel(true, this.MRI.Refreshing);
    this._isRefreshPath = true;

    if ( source && (source == "evented") )
      ;
    else {
      processRefresh("refresh");
    }

    this.cleanUp({ dataOnly: true });
    this.renderMap();
  },

  // TYPE: RENDER
  // private.  Given a chassis item, connect it to its associated dom node and modify that node
  _associateAndModifyDomElementForItem: function(item) { // return mapElement
    var mapElement = null;
    var isChassis = false;
    var occupiedSlots, chassisSlot = null;

    var frmProps = dojo.fromJson( itmeUtils.getProperty(item, "OTHERHWVIEWPROPS") );

    // CHEAT.  For now, handle specific types of nodes in a specific way.
    switch (item.getType()) {
      case "PowerSupply":
        if ( frmProps.SLOT && frmProps.SLOT[0] ) {
          mapElement = dojo.byId(this.PWR_Name + frmProps.SLOT[0]);
          item.myImage = this.IMAGES.getImage("PowerUnit");
        }
        else {
          itmeUtils.CMError("MAIN", "DM SLOT positioning is unavailable for: " + item.getId() + ", " + item.getLabel());
          this._issues.warnings.push("The hardware view does not have slot positioning information for item: " + item.getId() + ", " + item.getLabel());
        }
        break;
      case "Fan":
        if ( frmProps.SLOT && frmProps.SLOT[0] ) {
          if ( frmProps.SLOT[0] == 5 || frmProps.SLOT[0] == 10 )
            item.myImage = this.IMAGES.getImage("FanUnit2");
          else
            item.myImage = this.IMAGES.getImage("FanUnit1");
          mapElement = dojo.byId(this.FAN_Name + frmProps.SLOT[0]);
        }
        else {
          itmeUtils.CMError("MAIN", "DM SLOT positioning is unavailable for: " + item.getId() + ", " + item.getLabel());
          this._issues.warnings.push("The hardware view does not have slot positioning information for item: " + item.getId() + ", " + item.getLabel());
        }
        break;
      case "PhysicalPackage":
        if ( frmProps.SLOT && frmProps.SLOT[0] ) {
          item.myImage = this.IMAGES.getImage("FanControlUnit");
          mapElement = dojo.byId(this.FANCTRL_Name + frmProps.SLOT[0]);
        }
        else {
          itmeUtils.CMError("MAIN", "DM SLOT positioning is unavailable for: " + item.getId() + ", " + item.getLabel());
          this._issues.warnings.push("The hardware view does not have slot positioning information for item: " + item.getId() + ", " + item.getLabel());
        }
        break;
      case "com.ibm.sysmgt.platform.PhysicalPlatformFactory":
        if ( frmProps.SLOT && frmProps.SLOT[0] ) {
          if ( frmProps.SLOT.length > 1 )
            this.mergeSlots(frmProps.SLOT[0], this.ITE_Name, frmProps.SLOT);

          mapElement = dojo.byId(this.ITE_Name + (frmProps.SLOT[0]));

          if (frmProps.EXPANSIONTYPE == "SEN") {
            item.myImage = this.IMAGES.getImage("ITE_C42x0M4_SEN"); // this._getBladeImageByItem(item, frmProps.SLOT.length);
          } else if (frmProps.EXPANSIONTYPE == "PEN") {
            item.myImage = this.IMAGES.getImage("ITE_C42x0M4_PEN"); ;
          } else {
            item.myImage = this._getBladeImageByItem(item, frmProps.SLOT.length);
          }

        }
        else {
          itmeUtils.CMWarn("MAIN", "DM SLOT positioning is unavailable for: " + item.getId() + ", " + item.getLabel());
          chassisSlot = itmeUtils.getProperty(item, "ChassisSlot");
          if (chassisSlot == null) {// no SLOT or ChassisSlot... not enough info to place in map
            itmeUtils.CMError("The hardware view does not have any slot positioning information for item: " + item.getId() + ", " + item.getLabel());
            this._issues.warnings.push("The hardware view does not have slot positioning information for item: " + item.getId() + ", " + item.getLabel());
            break;
          }

          occupiedSlots = itmeUtils.getProperty(item, "OccupiedSlots");
          if ( occupiedSlots && occupiedSlots.length > 1 ) {
            numSlots = occupiedSlots.length;
            this.mergeSlots(chassisSlot, this.ITE_Name, occupiedSlots);
            item.myImage = this._getBladeImageByItem(item, occupiedSlots.length);
          }
          else
            item.myImage = this._getBladeImageByItem(item, 1);

          mapElement = dojo.byId(this.ITE_Name + chassisSlot);
        }
        break;
      case "Switch":
      case "PassThroughModule":
        //occupiedSlots = itmeUtils.getProperty(item, "OccupiedSlots");
        if ( frmProps.SLOT && frmProps.SLOT[0] )
          mapElement = dojo.byId(this.IOM_Name + frmProps.SLOT[0]);
        else {
          itmeUtils.CMError("MAIN", "DM SLOT positioning is unavailable for: " + item.getId() + ", " + item.getLabel());
          this._issues.warnings.push("The hardware view does not have slot positioning information for item: " + item.getId() + ", " + item.getLabel());
        }

        item.myImage = this._getIOModuleImage(item.getType(), frmProps.PRODUCTID);
        break;
      case "BladeAdminServer":
        if ( frmProps.SLOT && frmProps.SLOT[0] )
          mapElement = dojo.byId(this.MGT_Name + frmProps.SLOT[0]);
        else {
          itmeUtils.CMError("MAIN", "DM SLOT positioning is unavailable for: " + item.getId() + ", " + item.getLabel());
          this._issues.warnings.push("The hardware view does not have slot positioning information for item: " + item.getId() + ", " + item.getLabel());
        }
        item.myImage = this.IMAGES.getImage("CMM");
        break;
      case "SystemChassis": // special case
        mapElement = this._chassisNameElement;
        isChassis = true;
        setTimeout(dojo.hitch(this, this._DisplayUndiscoveredElements, frmProps.UNDISCOVERED), 750);
        break;
      case "StoragePlatform":
        occupiedSlots = itmeUtils.getProperty(item, "OccupiedSlots");
        chassisSlot = itmeUtils.getProperty(item, "ChassisSlot");
          if ( occupiedSlots && occupiedSlots.length > 1 ) {
            numSlots = occupiedSlots.length;
            this.mergeSlots(frmProps.SLOT[0], this.ITE_Name, occupiedSlots);
            item.myImage = this._getBladeImageByItem(item, occupiedSlots.length);
          }
          else {
             item.myImage = this._getBladeImageByItem(item, 1);
          }
        mapElement = dojo.byId(this.ITE_Name + frmProps.SLOT[0]);
        break;
    }

    if (mapElement) {
      dojo.addClass(mapElement, "ComponentSlotOccupied");
      dojo.removeClass(mapElement, "EmptySlot");
      item.mapElement = mapElement;

      if (isChassis === false) { // chassis logic was already handled
        imageString = "<img id='" + mapElement.id + "_IMG' src='" + item.myImage + "' width='" + mapElement.style.width + "' height='" + mapElement.style.height + "' alt='" + item.getLabel() + "'>";
        mapElement.innerHTML = imageString; // + "<span style='position:absolute;top:0;left:0'><font size='-2' color='white'>" + item.getLabel() + "</font></span>";
      }

      if (this._currentSelectionsByItemId[item.getId()+"_item"]) {
        itmeUtils.CMTrace("SELECTION", "selectedItem: " + item.getId());
        dojo.addClass(mapElement, "ComponentSelected");
      }
    }

    return mapElement;
  },

  _DisplayUndiscoveredElements: function(undiscoveredElements) {
    var i, mapElement, image, unDiscoveredItemDivElement, typeID, typeString, statusString;
    itmeUtils.CMTrace("MAIN", "DEBUG:  undiscoveredElements.length = " + undiscoveredElements.length);
    for (i = 0; i < undiscoveredElements.length; i++)
    {
      if ( ! (undiscoveredElements[i].SLOT) )
        continue;

      switch(undiscoveredElements[i].TYPE)
      { case "ITE":
          typeID = this.ITE_Name;
          typeString = this.MRI.Node;
          break;
        case "Switch":
          typeID = this.IOM_Name;
          image = this._getIOModuleImage("Switch", undiscoveredElements[i].PRODUCTID);
          typeString = this.MRI.Switch;
          break;
        case "Fan":
          typeID = this.FAN_Name;
          if ( undiscoveredElements[i].SLOT[0] == 5 || undiscoveredElements[i].SLOT[0] == 10 )
            image = this.IMAGES.getImage("FanUnit2");
          else
            image = this.IMAGES.getImage("FanUnit1");
          typeString = this.MRI.Fan;
          break;
        case "PowerSupply":
          typeID = this.PWR_Name;
          image = this.IMAGES.getImage("PowerUnit");
          typeString = this.MRI.PowerSupply;
          break;
        case "CMM":
          typeID = this.MGT_Name;
          image = this.IMAGES.getImage("CMM");
          typeString = this.MRI.BladeMgmtModule;
          break;
        case "Passthrough":
          typeID = this.IOM_Name;
          image = this._getIOModuleImage("PassThroughModule", undiscoveredElements[i].PRODUCTID);
          typeString = this.MRI.PassthroughModule;
          break;
        case "FanLogic":
          typeID = this.FANCTRL_Name;
          image = this.IMAGES.getImage("FanControlUnit");
          typeString = this.MRI.FanLogicModule;
          break;
        case "SCU":
          typeID = this.ITE_Name;
          image = this._getStorageBladeImage(undiscoveredElements[i]);
          typeString = this._getStorageBladeDescription(undiscoveredElements[i]);
          break;
        default:
          // Without continue here, an unmatched undiscovered type will have the previous
          // typeID set and may apply the previous image to a slot of another type.
          continue;
       }


      // whoopsies
      mapElement = dojo.byId(typeID + undiscoveredElements[i].SLOT[0]);
      if ( dojo.hasClass(mapElement, "ComponentSlotOccupied") ) {
        itmeUtils.CMTrace("MAIN", "UnDiscovered "+undiscoveredElements[i].TYPE+" trying to fill occupied slot: " + undiscoveredElements[i].SLOT[0]);
        continue;
      }

      if ( undiscoveredElements[i].SLOT.length > 1 )
        mapElement = this.mergeSlots(undiscoveredElements[i].SLOT[0], this.ITE_Name, undiscoveredElements[i].SLOT);

      dojo.removeClass(mapElement, "EmptySlot");
      dojo.addClass(mapElement, "ComponentNotDiscovered");

      unDiscoveredItemDivElement = itmeUtils.createElement(
        mapElement.id + "_unDiscEl",
        mapElement,
        { styleClass: "ComponentNotDiscoveredOverlay",
          baseDimensionsOnNode: mapElement} );

      var tooltip = new itme.ChassisMap.ChassisMapTooltip({}, mapElement);
      tooltip.setTitle(dojo.replace("Detected {0}", [typeString]));

      if ( undiscoveredElements[i].TYPE == "SCU" ) {
           unDiscoveredItemDivElement.innerHTML = this.MRI.Pending;
      }

      if ( undiscoveredElements[i].TYPE == "ITE" )
      {
        if (undiscoveredElements[i].EXPANSIONTYPE) {
          image = this._getBladeImageWithExpansion(undiscoveredElements[i].EXPANSIONTYPE);
        } else if ( undiscoveredElements[i].MTYPE )
          image = this._getBladeImage("", undiscoveredElements[i].MTYPE, undiscoveredElements[i].SLOT.length);
        else
          image = this._getBladeImage("", "", undiscoveredElements[i].SLOT.length);

        switch (undiscoveredElements[i].STATE)
        { case 0:
            if ( undiscoveredElements[i].STATUS == 15 )
              statusString = this.MRI.Flashing;
            else
              statusString = this.MRI.Discovering;
            tooltip.addBaseProperty( this.MRI.Status , statusString);
            unDiscoveredItemDivElement.innerHTML = this.MRI.Pending;
            break;
          case 5:
            tooltip.addBaseProperty( this.MRI.Status , this.IMAGES.buildIconImageTag("StatusNormal", "", 16) + " " + this.MRI.OK);
            unDiscoveredItemDivElement.innerHTML = this.MRI.Pending;
            break;
          case 10:
            tooltip.addBaseProperty( this.MRI.Status , this.IMAGES.buildIconImageTag("StatusWarningMinor", "", 16) + " " + this.MRI.Warning);
            unDiscoveredItemDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusWarningMinor", "", 16) + " " + this.MRI.Pending;
            break;
          case 20:
            tooltip.addBaseProperty( this.MRI.Status , this.IMAGES.buildIconImageTag("StatusCritical", "", 16) + " " + this.MRI.Critical);
            unDiscoveredItemDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusCritical", "", 16) + " " + this.MRI.Pending;
            break;
          case 30:
            if ( undiscoveredElements[i].STATUS == 12 )
              statusString = this.MRI.CommError;
            else
              statusString = this.MRI.Error;
            tooltip.addBaseProperty( this.MRI.Status , this.IMAGES.buildIconImageTag("StatusCritical", "", 16) + " " + statusString);
            tooltip.addBaseProperty( "" , this.MRI.HWNodeWarning);
            unDiscoveredItemDivElement.innerHTML = this.IMAGES.buildIconImageTag("StatusCritical", "", 16) + " " + this.MRI.Error;
            break;
        }
      }

      if ( undiscoveredElements[i].TYPE == "SCU" || undiscoveredElements[i].TYPE == "ITE" ) {
        if ( undiscoveredElements[i].SERIAL )
          tooltip.addBaseProperty( this.MRI.SerialNumber, undiscoveredElements[i].SERIAL );
        if ( undiscoveredElements[i].ARCH )
          tooltip.addBaseProperty( this.MRI.Architecture, undiscoveredElements[i].ARCH );
        if ( undiscoveredElements[i].MTYPE )
          tooltip.addBaseProperty( this.MRI.MachineType, undiscoveredElements[i].MTYPE );
        if ( undiscoveredElements[i].MNUMBER )
          tooltip.addBaseProperty( this.MRI.Model, undiscoveredElements[i].MNUMBER );
      }

      tooltip.buildTooltip();
      imageString = "<img id='und_"+undiscoveredElements[i].TYPE+"_" + undiscoveredElements[i].SLOT[0] + "_IMG' src='" + image + "' width='" + mapElement.style.width + "' height='" + mapElement.style.height + "' >";
      mapElement.innerHTML = imageString + mapElement.innerHTML;
    }
  },

  _getBladeImageByItem: function (item, numSlots) {
    var model = this._getProperty(item, "Model");
    var machineType = this._getProperty(item, "MachineType");
    return this._getBladeImage(model, machineType, numSlots);
  },

  _getBladeImage: function(model, machineType, numSlots) {
    var hwInfo = new itme.ChassisMap.KnownHardwareInfo();
    var imgName = hwInfo.getImageResourceForModelAndType(model, machineType, numSlots);

    if ( imgName )
      return this.IMAGES.getImage(imgName);
    else {
      if ( numSlots > 1)
        return this.IMAGES.getImage("DefaultITE_2x");
      else
        return this.IMAGES.getImage("DefaultITE_1x");
    }
  },

  _getBladeImageWithExpansion: function (expansionType) {
    if (expansionType == "PEN") {
      return this.IMAGES.getImage("ITE_C42x0M4_PEN");
    } else {
      return  this.IMAGES.getImage("ITE_C42x0M4_SEN");
    }
  },

  _getStorageBladeImage: function (item) {
    if (item.PRODUCTID == "384") { return this.IMAGES.getImage("STRW_493949"); } // 0x0180 = SRC
    if (item.PRODUCTID == "385") { return this.IMAGES.getImage("STRW_493929"); } // 0x0181 = SDE
    if (item.model == "A49") { return this.IMAGES.getImage("STRW_493949"); } // 4939A49 = SRC
    if (item.model == "A29") { return this.IMAGES.getImage("STRW_493929"); } // 4939A29 = SDE
    return this.IMAGES.getImage("STRW_493949"); // SRC
  },

  _getStorageBladeDescription: function (item) {
    if (item.PRODUCTID == "384") { return this.MRI.StorageRaidController; } // 0x0180 = SRC
    if (item.PRODUCTID == "385") { return this.MRI.StorageDiskExpansion; }  // 0x0181 = SDE
    if (item.model == "A49") { return this.MRI.StorageRaidController; } // 4939A49 = SRC
    if (item.model == "A29") { return this.MRI.StorageDiskExpansion; }  // 4939A29 = SDE
    return this.MRI.StorageRaidController; // SRC
  },

  _getIOModuleImage: function(type, productId) {
    var hwInfo = new itme.ChassisMap.KnownHardwareInfo();
    var imgName = hwInfo.getImageResourceForProductId(productId);

    if ( imgName )
      return this.IMAGES.getImage(imgName);
    else {
      if (type == "PassThroughModule")
        return this.IMAGES.getImage("SwitchEN4091");
      else
        return this.IMAGES.getImage("SwitchEN2092");
    }
  },

  _endsWith: function(string, suffix) {
    return string.indexOf(suffix, string.length - suffix.length) !== -1;
  },

    // TYPE: RENDER
  _buildTooltip: function(item, mapElement){

    // get properties we may display
    var commState = item.getProperty("CommunicationState");
    var model = this._getProperty(item, "Model");
    //var status = this._getProperty(item, "STATUS");
    var machineType = this._getProperty(item, "MachineType");
    var serialNumber = this._getProperty(item, "SerialNumber");
    var bay = this._getProperty(item, "Bay");
    var accessState = this._getProperty(item, "AccessState");
    itmeUtils.CMTrace("OVERLAY", "building new tooltip for " + mapElement.id);
    var tooltip = new itme.ChassisMap.ChassisMapTooltip({}, mapElement);
    tooltip.setTitle( item.getLabel() ); //itemStyle.getLabel() );
    var objProp = item.getProperty("ObjectType");
    var problems = dojo.fromJson( this._getProperty(item, "Problems") );

    if (model && machineType && machineType==="4939") {
      // Replace Storage Platform with more specific type
      if (this._endsWith(model,"49")) {
        tooltip.addBaseProperty( this.MRI.Type, this.MRI.StorageRaidController ); //item.getLabel());
      } else {
        tooltip.addBaseProperty( this.MRI.Type, this.MRI.StorageDiskExpansion ); //item.getLabel());
      }
    } else {
      tooltip.addBaseProperty( this.MRI.Type, objProp.getDisplayValue() ); //item.getLabel());
    }


    // FRM - HA properties
    var frmProps = dojo.fromJson( itmeUtils.getProperty(item, "OTHERHWVIEWPROPS") );

    if ( frmProps.ISLOGICALFRM == true) {
      tooltip.addBaseProperty(this.MRI.ProductHardware, this.MRI.SWProductName );
      if ( frmProps.HA )
        tooltip.addBaseProperty(this.MRI.HARoleSubProp, frmProps.HA );
    }

    if (bay)
      tooltip.addBaseProperty(this.MRI.Bay , bay );
    //if (status)
    //  tooltip.addBaseProperty(this.MRI.Status , "<img src='" + status.getImage() + "'> "+status.getLabel() );
    if (problems)
    {
      var probString = "";
      if (problems.PBLM_E && (problems.PBLM_E > 0))
        probString = problems.PBLM_E + this.IMAGES.buildIconImageTag("StatusCritical", "", 16) + " ";
      if (problems.PBLM_W && (problems.PBLM_W > 0))
        probString = probString + problems.PBLM_W + this.IMAGES.buildIconImageTag("StatusWarningMinor", "", 16) + " ";
      if (problems.PBLM_I && (problems.PBLM_I > 0))
        probString = probString + problems.PBLM_I + this.IMAGES.buildIconImageTag("StatusInformational", "", 16) + " ";

      if (commState && (commState.getValue() == 3) )
        probString = probString + this.IMAGES.buildIconImageTag("AccessOfflineOK", "", 16) + " " + this.MRI.Offline + " ";

      if (probString == "") // no other 'problems', so show 'ok'
        probString = this.IMAGES.buildIconImageTag("StatusNormal", "", 16) + " " + this.MRI.OK;

      tooltip.addBaseProperty(this.MRI.Problems , probString );
    }
    if (model && machineType)
      tooltip.addBaseProperty(this.MRI.ModelMachineType , model + " ("+this.MRI.Type + " " + machineType + ")");
    else if (model)
      tooltip.addBaseProperty(this.MRI.Model , model);
    if (serialNumber)
      tooltip.addBaseProperty(this.MRI.SerialNumber , serialNumber);

    var clusterName = frmProps.CLUSTERNAME;
    var enclosureId = frmProps.ENCLOSUREID;
    if (clusterName) {
      tooltip.addBaseProperty(this.MRI.ClusterName, clusterName);
    }
    if (enclosureId) {
      tooltip.addBaseProperty(this.MRI.EnclosureId , enclosureId);
    }

    tooltip.buildTooltip();
  },

  // TYPE: ACTION, STATE
  _onElementClicked: function(item, event)
  {
    //If a refresh is in progress, don't allow selection, simply return
    if ( dojo.byId("hardwareViewWaitingPane")){
        if(dojo.byId("hardwareViewWaitingPane").style.display == 'block') {
            return;
        }
    }
    //itmeUtils.CMTrace("SELECTION", "PING PING ON CLICK PING");
    //itmeUtils.CMTrace("SELECTION", "Event??? " + event);
    var multiSelect = itmeUtils.isMultiSelectKeyUsed(event);
    if (item) {

      clearTimeout(this._refreshDelay);
      this._refreshDelay = false;

      if (multiSelect === true)
        this._handleMultiSelectEvent(item);
      else  // single selection
        this._handleSingleSelectEvent(item);

      //this._clearStorageElementSelection(multiSelect);

      dojo.cookie("chasisMap.currentOverlay", this._currentOverlay.getId(), { expires: 1});
      dojo.cookie("chasisMap.currentSelection", itmeUtils.myJoin(this._currentSelectionsByItemId), { expires: 1});
      itmeUtils.CMTrace("SELECTION", "COOKIE: adding cookie to set current overlay to: " + this._currentOverlay.getId());
      itmeUtils.CMTrace("SELECTION", "COOKIE: adding cookie to set selection to: " + itmeUtils.myJoin(this._currentSelectionsByItemId));

      // page reload now if single select case
      //TODO: reevaluate if we need any extra processing for multi select, treating it the same as single select (Alex)
    //  if (multiSelect === false)
    //  {
        //chassisMapRefresh(); //jsUtils
        //this.cleanUp();
        //Use cookie instead of item, to handle both single/multi selection with this call
        processRefresh( dojo.cookie("chasisMap.currentSelection") );
        this._toggleWaitingPanel(true, this.MRI.Refreshing);
        this._isRefreshPath = true;
        this._hideRefreshPanelWhenAjaxIsDone();
    }
  },

   _selectedStorageElement: null, // Used by jsUtils to build menu
  _selectedStorageElementGraphic: null,

  // DHM: Commented out for interim GA2 (or GA3 depending on who's counting)
  // _clearStorageElementSelection: function(multiSelect) {
  //   this._selectedStorageElement = null;
  //   if (multiSelect === true) {
  //     // DHM: need to ensure the SCU selection frame is cleared, single select clears the selection
  //     dojo.removeClass(this._selectedStorageElementGraphic, "ComponentSelected");
  //   }
  //   this._selectedStorageElementGraphic = null;
  // },

  // DHM: Commented out for interim GA2 (or GA3 depending on who's counting)
  // _onStorageElementClicked: function (item, event) {
  //     //If a refresh is in progress, don't allow selection, simply return
  //     if ( dojo.byId("hardwareViewWaitingPane")){
  //         if(dojo.byId("hardwareViewWaitingPane").style.display == 'block') {
  //             return;
  //         }
  //     }
  //     if (item) {
  //       clearTimeout(this._refreshDelay);
  //       this._refreshDelay = false;
  //       this._currentSelectionsByItemId = [];
  //         dojo.query(".ComponentSelected").forEach(function(node) {
  //           dojo.removeClass(node, "ComponentSelected");
  //         });
  //       dojo.addClass(item, "ComponentSelected");
  //       this._selectedStorageElement = item.SCU;
  //       this._selectedStorageElementGraphic = item;
  //     }

  //      processRefresh("9999"); // Seems to require a value
  //      // processRefresh(item.SCU.OID);

  //     this._toggleWaitingPanel(true, this.MRI.Refreshing);
  //     this._isRefreshPath = true;
  //     this._hideRefreshPanelWhenAjaxIsDone();
  //     // dojo.html.set(dojo.byId("prediscoveredActions"), "<p>loaded!</p>");
  // },

  /*_showRefreshPanel: function() {
    if ( dojo.byId("refreshPending"))
	    dojo.byId("refreshPending").style.display = "block";
    else {
        itmeUtils.createElement("refreshPending",
                                this._chassisMapFaceElements[0],
                                {styleClass: "refreshPendingDiv",
                                 style: { "position":"absolute", "top":"20px", "left":"20px"},
                                 innerHTML: this.IMAGES.buildIconImageTag("Processing", "", 22) + " " + this.MRI.Refreshing});
     }
  },*/

  _hideRefreshPanelWhenAjaxIsDone: function() {
		if(ajaxInProgress) {
			setTimeout(dojo.hitch(this, this._hideRefreshPanelWhenAjaxIsDone), 120);
		} else {
			dojo.byId("hardwareViewWaitingPane").style.display = "none";
		}


   },

  // public: unrender or otherwise delete resources on close or refresh
  //  parms: renderedOnly , do not clear out backend data hooks
  //  parms: dataOnly , clear out back end data hooks only, not the visuals
   // default: clear everything
  cleanUp: function(parms)
  {
    var button, counter, aTooltip, i;
    if (!parms) parms = {};
    renderedOnly = (typeof(renderedOnly) != 'undefined') ? renderedOnly : false;
    itmeUtils.CMTrace("MAIN", "HWView cleanUp entry");

    // if refreshTimer (caused by incoming event notifications) is active for any reason....
    if (!parms.renderedOnly) { // clear data unless rendered only
      itmeUtils.CMTrace("MAIN", "HWView cleanUp clearing data and event listener (if applicable)");
      clearTimeout(this._eventTimeout);
      this._initialEventTime = null;
      if ( this._dataset ) {
        this._dataset.destroy();
        this._dataset = null;
      }
      this._issues.warnings = [];
    }

    if ( !parms.dataOnly ) { // clear rendered unless data only
      this._isRendered = false;
      if ( dojo.byId(this._chassisMapParentDOMNodeID))
        dojo.empty(this._chassisMapParentDOMNodeID);
      this.refreshPrep();

      for ( i = 0; i < this._overlays.length; i++) {
        //itmeUtils.CMTrace("MAIN", "DESTROYING : " + this._overlays[i].getId()+"_dojoButton");
        button = dijit.byId(this._overlays[i].getId()+"_dojoButton");
        if ( button ) {
          button.destroyRecursive();
        }
      }
      button = dijit.byId("refresh_dojoButton");
      if ( button ) {
        button.destroyRecursive();
      }

      counter = 0;
      // cleanUp dojo tooltips.
      aTooltip = dijit.byId("myTooltip_"+counter);
      while (aTooltip) {
        aTooltip.close();
        aTooltip.destroyRecursive();
        counter++;
        aTooltip = dijit.byId("myTooltip_"+counter);
      }
      myTooltipCounter = 0;

      for (i = 0; i < this._overlays.length; i++)
      {
        buttonTooltip = dijit.byId(this._overlays[i].getId()+"_buttonTooltip");
        if ( buttonTooltip )
        {
          buttonTooltip.close();
          buttonTooltip.destroyRecursive();
        }
      }

      buttonTooltip = dijit.byId("refresh_buttonTooltip");
      if ( buttonTooltip )
      {
        buttonTooltip.close();
        buttonTooltip.destroyRecursive();
      }
      warningsTooltip = dijit.byId("warnings_tooltip");
      if ( warningsTooltip )
      {
        warningsTooltip.close();
        warningsTooltip.destroyRecursive();
      }
    }
  },

  refreshPrep: function() {
    this._currentOverlay = null; // reset this so we can re-render the overlays
    this._numFaceElements = 0;
  }


});

}

if(!dojo._hasResource["itme.finder.TaskCategory"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.finder.TaskCategory"] = true;
dojo.provide("itme.finder.TaskCategory");
dojo.declare("itme.finder.TaskCategory", null, {
	// private members       
  	_category: null,   
  	_ids: null,
  	_names: null,
  	_selectedIndexes: null,
  	_currentArrayIndex: 0,
  	_currentSelectedIndex: 0,
  	  
  	constructor: function(category){    
    	this._category = category;
    	this._ids = new Array(500);
    	this._names= new Array(500);
    	this._selectedIndexes= new Array(500);
  	},
  	add: function(id, name) {
  		this._ids[this._currentArrayIndex] = id;
  		this._names[this._currentArrayIndex] = name;
  		this._currentArrayIndex = this._currentArrayIndex + 1;  	
  		return this._currentArrayIndex - 1;	
  	},
  	getCategory: function() {
  		return this._category;
  	},
  	clearSelection: function() {
  		this._selectedIndexes = null;
  		this._currentSelectedIndex = 0;
  		this._selectedIndexes = new Array(500);
  	},
  	addSelected: function(index) {
  		this._selectedIndexes[this._currentSelectedIndex] = index;
  		this._currentSelectedIndex = this._currentSelectedIndex + 1; 
  	},
  	getSelectedCount: function() {
  		return this._currentSelectedIndex;
  	},
  	getSelectedName: function(index) {
  		return this._names[this._selectedIndexes[index]];
  	},
  	getSelectedId: function(index) {
  		return this._ids[this._selectedIndexes[index]];
  	},
  	getIds: function() {
  		return this._ids;
  	}
  	
});

}

if(!dojo._hasResource["dojo.data.util.filter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.util.filter"] = true;
dojo.provide("dojo.data.util.filter");

dojo.data.util.filter.patternToRegExp = function(/*String*/pattern, /*boolean?*/ ignoreCase){
	//	summary:  
	//		Helper function to convert a simple pattern to a regular expression for matching.
	//	description:
	//		Returns a regular expression object that conforms to the defined conversion rules.
	//		For example:  
	//			ca*   -> /^ca.*$/
	//			*ca*  -> /^.*ca.*$/
	//			*c\*a*  -> /^.*c\*a.*$/
	//			*c\*a?*  -> /^.*c\*a..*$/
	//			and so on.
	//
	//	pattern: string
	//		A simple matching pattern to convert that follows basic rules:
	//			* Means match anything, so ca* means match anything starting with ca
	//			? Means match single character.  So, b?b will match to bob and bab, and so on.
	//      	\ is an escape character.  So for example, \* means do not treat * as a match, but literal character *.
	//				To use a \ as a character in the string, it must be escaped.  So in the pattern it should be 
	//				represented by \\ to be treated as an ordinary \ character instead of an escape.
	//
	//	ignoreCase:
	//		An optional flag to indicate if the pattern matching should be treated as case-sensitive or not when comparing
	//		By default, it is assumed case sensitive.

	var rxp = "^";
	var c = null;
	for(var i = 0; i < pattern.length; i++){
		c = pattern.charAt(i);
		switch(c){
			case '\\':
				rxp += c;
				i++;
				rxp += pattern.charAt(i);
				break;
			case '*':
				rxp += ".*"; break;
			case '?':
				rxp += "."; break;
			case '$':
			case '^':
			case '/':
			case '+':
			case '.':
			case '|':
			case '(':
			case ')':
			case '{':
			case '}':
			case '[':
			case ']':
				rxp += "\\"; //fallthrough
			default:
				rxp += c;
		}
	}
	rxp += "$";
	if(ignoreCase){
		return new RegExp(rxp,"mi"); //RegExp
	}else{
		return new RegExp(rxp,"m"); //RegExp
	}
	
};

}

if(!dojo._hasResource["dojo.data.util.sorter"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.util.sorter"] = true;
dojo.provide("dojo.data.util.sorter");

dojo.data.util.sorter.basicComparator = function(	/*anything*/ a, 
													/*anything*/ b){
	//	summary:  
	//		Basic comparision function that compares if an item is greater or less than another item
	//	description:  
	//		returns 1 if a > b, -1 if a < b, 0 if equal.
	//		'null' values (null, undefined) are treated as larger values so that they're pushed to the end of the list.
	//		And compared to each other, null is equivalent to undefined.
	
	//null is a problematic compare, so if null, we set to undefined.
	//Makes the check logic simple, compact, and consistent
	//And (null == undefined) === true, so the check later against null
	//works for undefined and is less bytes.
	var r = -1;
	if(a === null){
		a = undefined;
	}
	if(b === null){
		b = undefined;
	}
	if(a == b){
		r = 0; 
	}else if(a > b || a == null){
		r = 1; 
	}
	return r; //int {-1,0,1}
};

dojo.data.util.sorter.createSortFunction = function(	/* attributes array */sortSpec,
														/*dojo.data.core.Read*/ store){
	//	summary:  
	//		Helper function to generate the sorting function based off the list of sort attributes.
	//	description:  
	//		The sort function creation will look for a property on the store called 'comparatorMap'.  If it exists
	//		it will look in the mapping for comparisons function for the attributes.  If one is found, it will
	//		use it instead of the basic comparator, which is typically used for strings, ints, booleans, and dates.
	//		Returns the sorting function for this particular list of attributes and sorting directions.
	//
	//	sortSpec: array
	//		A JS object that array that defines out what attribute names to sort on and whether it should be descenting or asending.
	//		The objects should be formatted as follows:
	//		{
	//			attribute: "attributeName-string" || attribute,
	//			descending: true|false;   // Default is false.
	//		}
	//	store: object
	//		The datastore object to look up item values from.
	//
	var sortFunctions=[];

	function createSortFunction(attr, dir, comp, s){
		//Passing in comp and s (comparator and store), makes this
		//function much faster.
		return function(itemA, itemB){
			var a = s.getValue(itemA, attr);
			var b = s.getValue(itemB, attr);
			return dir * comp(a,b); //int
		};
	}
	var sortAttribute;
	var map = store.comparatorMap;
	var bc = dojo.data.util.sorter.basicComparator;
	for(var i = 0; i < sortSpec.length; i++){
		sortAttribute = sortSpec[i];
		var attr = sortAttribute.attribute;
		if(attr){
			var dir = (sortAttribute.descending) ? -1 : 1;
			var comp = bc;
			if(map){
				if(typeof attr !== "string" && ("toString" in attr)){
					 attr = attr.toString();
				}
				comp = map[attr] || bc;
			}
			sortFunctions.push(createSortFunction(attr, 
				dir, comp, store));
		}
	}
	return function(rowA, rowB){
		var i=0;
		while(i < sortFunctions.length){
			var ret = sortFunctions[i++](rowA, rowB);
			if(ret !== 0){
				return ret;//int
			}
		}
		return 0; //int  
	}; // Function
};

}

if(!dojo._hasResource["dojo.data.util.simpleFetch"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.data.util.simpleFetch"] = true;
dojo.provide("dojo.data.util.simpleFetch");


dojo.data.util.simpleFetch.fetch = function(/* Object? */ request){
	//	summary:
	//		The simpleFetch mixin is designed to serve as a set of function(s) that can
	//		be mixed into other datastore implementations to accelerate their development.  
	//		The simpleFetch mixin should work well for any datastore that can respond to a _fetchItems() 
	//		call by returning an array of all the found items that matched the query.  The simpleFetch mixin
	//		is not designed to work for datastores that respond to a fetch() call by incrementally
	//		loading items, or sequentially loading partial batches of the result
	//		set.  For datastores that mixin simpleFetch, simpleFetch 
	//		implements a fetch method that automatically handles eight of the fetch()
	//		arguments -- onBegin, onItem, onComplete, onError, start, count, sort and scope
	//		The class mixing in simpleFetch should not implement fetch(),
	//		but should instead implement a _fetchItems() method.  The _fetchItems() 
	//		method takes three arguments, the keywordArgs object that was passed 
	//		to fetch(), a callback function to be called when the result array is
	//		available, and an error callback to be called if something goes wrong.
	//		The _fetchItems() method should ignore any keywordArgs parameters for
	//		start, count, onBegin, onItem, onComplete, onError, sort, and scope.  
	//		The _fetchItems() method needs to correctly handle any other keywordArgs
	//		parameters, including the query parameter and any optional parameters 
	//		(such as includeChildren).  The _fetchItems() method should create an array of 
	//		result items and pass it to the fetchHandler along with the original request object 
	//		-- or, the _fetchItems() method may, if it wants to, create an new request object 
	//		with other specifics about the request that are specific to the datastore and pass 
	//		that as the request object to the handler.
	//
	//		For more information on this specific function, see dojo.data.api.Read.fetch()
	request = request || {};
	if(!request.store){
		request.store = this;
	}
	var self = this;

	var _errorHandler = function(errorData, requestObject){
		if(requestObject.onError){
			var scope = requestObject.scope || dojo.global;
			requestObject.onError.call(scope, errorData, requestObject);
		}
	};

	var _fetchHandler = function(items, requestObject){
		var oldAbortFunction = requestObject.abort || null;
		var aborted = false;

		var startIndex = requestObject.start?requestObject.start:0;
		var endIndex = (requestObject.count && (requestObject.count !== Infinity))?(startIndex + requestObject.count):items.length;

		requestObject.abort = function(){
			aborted = true;
			if(oldAbortFunction){
				oldAbortFunction.call(requestObject);
			}
		};

		var scope = requestObject.scope || dojo.global;
		if(!requestObject.store){
			requestObject.store = self;
		}
		if(requestObject.onBegin){
			requestObject.onBegin.call(scope, items.length, requestObject);
		}
		if(requestObject.sort){
			items.sort(dojo.data.util.sorter.createSortFunction(requestObject.sort, self));
		}
		if(requestObject.onItem){
			for(var i = startIndex; (i < items.length) && (i < endIndex); ++i){
				var item = items[i];
				if(!aborted){
					requestObject.onItem.call(scope, item, requestObject);
				}
			}
		}
		if(requestObject.onComplete && !aborted){
			var subset = null;
			if(!requestObject.onItem){
				subset = items.slice(startIndex, endIndex);
			}
			requestObject.onComplete.call(scope, subset, requestObject);
		}
	};
	this._fetchItems(request, _fetchHandler, _errorHandler);
	return request;	// Object
};

}

if(!dojo._hasResource["dojox.data.AndOrReadStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojox.data.AndOrReadStore"] = true;
dojo.provide("dojox.data.AndOrReadStore");





dojo.declare("dojox.data.AndOrReadStore", null,{
	//	summary:
	//		AndOrReadStore uses ItemFileReadStore as a base, modifying only the query (_fetchItems) section.
	//		Supports queries of the form: query:"id:1* OR dept:'Sales Department' || (id:2* && NOT dept:S*)"
	//		Includes legacy/widget support via:  
	//			query:{complexQuery:"id:1* OR dept:'Sales Department' || (id:2* && NOT dept:S*)"}
	//		The ItemFileReadStore implements the dojo.data.api.Read API and reads
	//		data from JSON files that have contents in this format --
	//		{ items: [
	//			{ name:'Kermit', color:'green', age:12, friends:['Gonzo', {_reference:{name:'Fozzie Bear'}}]},
	//			{ name:'Fozzie Bear', wears:['hat', 'tie']},
	//			{ name:'Miss Piggy', pets:'Foo-Foo'}
	//		]}
	//		Note that it can also contain an 'identifer' property that specified which attribute on the items 
	//		in the array of items that acts as the unique identifier for that item.
	//
	constructor: function(/* Object */ keywordParameters){
		//	summary: constructor
		//	keywordParameters: {url: String}
		//	keywordParameters: {data: jsonObject}
		//	keywordParameters: {typeMap: object)
		//		The structure of the typeMap object is as follows:
		//		{
		//			type0: function || object,
		//			type1: function || object,
		//			...
		//			typeN: function || object
		//		}
		//		Where if it is a function, it is assumed to be an object constructor that takes the 
		//		value of _value as the initialization parameters.  If it is an object, then it is assumed
		//		to be an object of general form:
		//		{
		//			type: function, //constructor.
		//			deserialize:	function(value) //The function that parses the value and constructs the object defined by type appropriately.
		//		}
	
		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = [];
		this._loadFinished = false;
		this._jsonFileUrl = keywordParameters.url;
		this._ccUrl = keywordParameters.url;
		this.url = keywordParameters.url;
		this._jsonData = keywordParameters.data;
		this.data = null;
		this._datatypeMap = keywordParameters.typeMap || {};
		if(!this._datatypeMap['Date']){
			//If no default mapping for dates, then set this as default.
			//We use the dojo.date.stamp here because the ISO format is the 'dojo way'
			//of generically representing dates.
			this._datatypeMap['Date'] = {
											type: Date,
											deserialize: function(value){
												return dojo.date.stamp.fromISOString(value);
											}
										};
		}
		this._features = {'dojo.data.api.Read':true, 'dojo.data.api.Identity':true};
		this._itemsByIdentity = null;
		this._storeRefPropName = "_S"; // Default name for the store reference to attach to every item.
		this._itemNumPropName = "_0"; // Default Item Id for isItem to attach to every item.
		this._rootItemPropName = "_RI"; // Default Item Id for isItem to attach to every item.
		this._reverseRefMap = "_RRM"; // Default attribute for constructing a reverse reference map for use with reference integrity
		this._loadInProgress = false; //Got to track the initial load to prevent duelling loads of the dataset.
		this._queuedFetches = [];

		if(keywordParameters.urlPreventCache !== undefined){
			this.urlPreventCache = keywordParameters.urlPreventCache?true:false;
		}
		if(keywordParameters.hierarchical !== undefined){
			this.hierarchical = keywordParameters.hierarchical?true:false;
		}
		if(keywordParameters.clearOnClose){
			this.clearOnClose = true;
		}
	},
	
	url: "", // use "" rather than undefined for the benefit of the parser (#3539)

	//Internal var, crossCheckUrl.  Used so that setting either url or _jsonFileUrl, can still trigger a reload
	//when clearOnClose and close is used.
	_ccUrl: "",

	data: null, //Make this parser settable.

	typeMap: null, //Make this parser settable.

	//Parameter to allow users to specify if a close call should force a reload or not.
	//By default, it retains the old behavior of not clearing if close is called.  But
	//if set true, the store will be reset to default state.  Note that by doing this,
	//all item handles will become invalid and a new fetch must be issued.
	clearOnClose: false,

	//Parameter to allow specifying if preventCache should be passed to the xhrGet call or not when loading data from a url.  
	//Note this does not mean the store calls the server on each fetch, only that the data load has preventCache set as an option.
	//Added for tracker: #6072
	urlPreventCache: false,

	//Parameter to indicate to process data from the url as hierarchical 
	//(data items can contain other data items in js form).  Default is true 
	//for backwards compatibility.  False means only root items are processed 
	//as items, all child objects outside of type-mapped objects and those in 
	//specific reference format, are left straight JS data objects.
	hierarchical: true,

	_assertIsItem: function(/* item */ item){
		//	summary:
		//		This function tests whether the item passed in is indeed an item in the store.
		//	item: 
		//		The item to test for being contained by the store.
		if(!this.isItem(item)){ 
			throw new Error("dojox.data.AndOrReadStore: Invalid item argument.");
		}
	},

	_assertIsAttribute: function(/* attribute-name-string */ attribute){
		//	summary:
		//		This function tests whether the item passed in is indeed a valid 'attribute' like type for the store.
		//	attribute: 
		//		The attribute to test for being contained by the store.
		if(typeof attribute !== "string"){ 
			throw new Error("dojox.data.AndOrReadStore: Invalid attribute argument.");
		}
	},

	getValue: function(	/* item */ item, 
						/* attribute-name-string */ attribute, 
						/* value? */ defaultValue){
		//	summary: 
		//		See dojo.data.api.Read.getValue()
		var values = this.getValues(item, attribute);
		return (values.length > 0)?values[0]:defaultValue; // mixed
	},

	getValues: function(/* item */ item, 
						/* attribute-name-string */ attribute){
		//	summary: 
		//		See dojo.data.api.Read.getValues()

		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		var arr = item[attribute] || [];
		// Clone it before returning.  refs: #10474
		return arr.slice(0, arr.length); // Array
	},

	getAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getAttributes()
		this._assertIsItem(item);
		var attributes = [];
		for(var key in item){
			// Save off only the real item attributes, not the special id marks for O(1) isItem.
			if((key !== this._storeRefPropName) && (key !== this._itemNumPropName) && (key !== this._rootItemPropName) && (key !== this._reverseRefMap)){
				attributes.push(key);
			}
		}
		return attributes; // Array
	},

	hasAttribute: function(	/* item */ item,
							/* attribute-name-string */ attribute){
		//	summary: 
		//		See dojo.data.api.Read.hasAttribute()
		this._assertIsItem(item);
		this._assertIsAttribute(attribute);
		return (attribute in item);
	},

	containsValue: function(/* item */ item, 
							/* attribute-name-string */ attribute, 
							/* anything */ value){
		//	summary: 
		//		See dojo.data.api.Read.containsValue()
		var regexp = undefined;
		if(typeof value === "string"){
			regexp = dojo.data.util.filter.patternToRegExp(value, false);
		}
		return this._containsValue(item, attribute, value, regexp); //boolean.
	},

	_containsValue: function(	/* item */ item, 
								/* attribute-name-string */ attribute, 
								/* anything */ value,
								/* RegExp?*/ regexp){
		//	summary: 
		//		Internal function for looking at the values contained by the item.
		//	description: 
		//		Internal function for looking at the values contained by the item.  This 
		//		function allows for denoting if the comparison should be case sensitive for
		//		strings or not (for handling filtering cases where string case should not matter)
		//	
		//	item:
		//		The data item to examine for attribute values.
		//	attribute:
		//		The attribute to inspect.
		//	value:	
		//		The value to match.
		//	regexp:
		//		Optional regular expression generated off value if value was of string type to handle wildcarding.
		//		If present and attribute values are string, then it can be used for comparison instead of 'value'
		return dojo.some(this.getValues(item, attribute), function(possibleValue){
			if(possibleValue !== null && !dojo.isObject(possibleValue) && regexp){
				if(possibleValue.toString().match(regexp)){
					return true; // Boolean
				}
			}else if(value === possibleValue){
				return true; // Boolean
			}
		});
	},

	isItem: function(/* anything */ something){
		//	summary: 
		//		See dojo.data.api.Read.isItem()
		if(something && something[this._storeRefPropName] === this){
			if(this._arrayOfAllItems[something[this._itemNumPropName]] === something){
				return true;
			}
		}
		return false; // Boolean
	},

	isItemLoaded: function(/* anything */ something){
		//	summary: 
		//		See dojo.data.api.Read.isItemLoaded()
		return this.isItem(something); //boolean
	},

	loadItem: function(/* object */ keywordArgs){
		//	summary: 
		//		See dojo.data.api.Read.loadItem()
		this._assertIsItem(keywordArgs.item);
	},

	getFeatures: function(){
		//	summary: 
		//		See dojo.data.api.Read.getFeatures()
		return this._features; //Object
	},

	getLabel: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getLabel()
		if(this._labelAttr && this.isItem(item)){
			return this.getValue(item,this._labelAttr); //String
		}
		return undefined; //undefined
	},

	getLabelAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Read.getLabelAttributes()
		if(this._labelAttr){
			return [this._labelAttr]; //array
		}
		return null; //null
	},

	_fetchItems: function(	/* Object */ keywordArgs, 
							/* Function */ findCallback, 
							/* Function */ errorCallback){
		//	summary: 
		//		See dojo.data.util.simpleFetch.fetch()
		//		filter modified to permit complex queries where
		//			logical operators are case insensitive:  
		//			, NOT AND OR ( ) ! && ||
		//			Note:  "," included for quoted/string legacy queries. 
		var self = this;
		var filter = function(requestArgs, arrayOfItems){
			var items = [];
			if(requestArgs.query){
				//Complete copy, we may have to mess with it.
				//Safer than clone, which does a shallow copy, I believe.
				var query = dojo.fromJson(dojo.toJson(requestArgs.query));
				//Okay, object form query, we have to check to see if someone mixed query methods (such as using FilteringSelect
				//with a complexQuery).  In that case, the params need to be anded to the complex query statement.
				//See defect #7980
				if(typeof query == "object" ){
					var count = 0;
					var p;
					for(p in query){
						count++;
					}
					if(count > 1 && query.complexQuery){
						var cq = query.complexQuery;
						var wrapped = false;
						for(p in query){
							if(p !== "complexQuery"){ 
								//We should wrap this in () as it should and with the entire complex query
								//Not just part of it.
								if(!wrapped){
									cq = "( " + cq + " )";
									wrapped = true;
								}
								//Make sure strings are quoted when going into complexQuery merge.
								var v = requestArgs.query[p];
								if(dojo.isString(v)){
									v = "'" + v + "'";
								}
								cq += " AND " + p + ":" + v;
								delete query[p];
								
							}
						}
						query.complexQuery = cq;
					}
				}

				var ignoreCase = requestArgs.queryOptions ? requestArgs.queryOptions.ignoreCase : false; 
				//for complex queries only:  pattern = query[:|=]"NOT id:23* AND (type:'test*' OR dept:'bob') && !filed:true"
				//logical operators are case insensitive:  , NOT AND OR ( ) ! && ||  // "," included for quoted/string legacy queries.
				if(typeof query != "string"){
					query = dojo.toJson(query);	
					query = query.replace(/\\\\/g,"\\"); //counter toJson expansion of backslashes, e.g., foo\\*bar test.
				}
				query = query.replace(/\\"/g,"\"");   //ditto, for embedded \" in lieu of " availability.
				var complexQuery = dojo.trim(query.replace(/{|}/g,"")); //we can handle these, too.
				var pos2, i;
				if(complexQuery.match(/"? *complexQuery *"?:/)){ //case where widget required a json object, so use complexQuery:'the real query'
					complexQuery = dojo.trim(complexQuery.replace(/"?\s*complexQuery\s*"?:/,""));
					var quotes = ["'",'"'];
					var pos1,colon;
					var flag = false;
					for(i = 0; i<quotes.length; i++){
						pos1 = complexQuery.indexOf(quotes[i]);
						pos2 = complexQuery.indexOf(quotes[i],1);
						colon = complexQuery.indexOf(":",1);
						if(pos1 === 0 && pos2 != -1 && colon < pos2){
							flag = true;
							break;
						} //first two sets of quotes don't occur before the first colon.	
					}	
					if(flag){	//dojo.toJson, and maybe user, adds surrounding quotes, which we need to remove.
						complexQuery = complexQuery.replace(/^\"|^\'|\"$|\'$/g,"");
					}
				} //end query="{complexQuery:'id:1* || dept:Sales'}" parsing (for when widget required json object query).
				var complexQuerySave = complexQuery;
				//valid logical operators.
				var begRegExp = /^,|^NOT |^AND |^OR |^\(|^\)|^!|^&&|^\|\|/i; //trailing space on some tokens on purpose.
				var sQuery = ""; //will be eval'ed for each i-th candidateItem, based on query components.
				var op = "";
				var val = "";
				var pos = -1;
				var err = false;
				var key = "";
				var value = "";
				var tok = "";
				pos2 = -1;
				for(i = 0; i < arrayOfItems.length; ++i){
					var match = true;
					var candidateItem = arrayOfItems[i];
					if(candidateItem === null){
						match = false;
					}else{
						//process entire string for this i-th candidateItem.
						complexQuery = complexQuerySave; //restore query for next candidateItem.
						sQuery = "";
						//work left to right, finding either key:value pair or logical operator at the beginning of the complexQuery string.
						//when found, concatenate to sQuery and remove from complexQuery and loop back.
						while(complexQuery.length > 0 && !err){
							op = complexQuery.match(begRegExp);
							
							//get/process/append one or two leading logical operators.
							while(op && !err){ //look for leading logical operators.
								complexQuery = dojo.trim(complexQuery.replace(op[0],""));
								op = dojo.trim(op[0]).toUpperCase();
								//convert some logical operators to their javascript equivalents for later eval.
								op = op == "NOT" ? "!" : op == "AND" || op == "," ? "&&" : op == "OR" ? "||" : op; 
								op = " " + op + " ";
								sQuery += op;
								op = complexQuery.match(begRegExp);
							}//end op && !err
							
							//now get/process/append one key:value pair.
							if(complexQuery.length > 0){
								pos = complexQuery.indexOf(":");
								if(pos == -1){
									err = true;
									break;
								}else{
									key = dojo.trim(complexQuery.substring(0,pos).replace(/\"|\'/g,""));
									complexQuery = dojo.trim(complexQuery.substring(pos + 1));
									tok = complexQuery.match(/^\'|^\"/);	//quoted?
									if(tok){
										tok = tok[0];
										pos = complexQuery.indexOf(tok);
										pos2 = complexQuery.indexOf(tok,pos + 1);
										if(pos2 == -1){
											err = true;
											break;
										}	
										value = complexQuery.substring(pos + 1,pos2);
										if(pos2 == complexQuery.length - 1){ //quote is last character
											complexQuery = "";
										}else{
											complexQuery = dojo.trim(complexQuery.substring(pos2 + 1));
										}
										sQuery += self._containsValue(candidateItem, key, value, dojo.data.util.filter.patternToRegExp(value, ignoreCase));
									}
									else{ //not quoted, so a space, comma, or closing parens (or the end) will be the break.
										tok = complexQuery.match(/\s|\)|,/);
										if(tok){
											var pos3 = new Array(tok.length);
											for(var j = 0;j<tok.length;j++){ 
												pos3[j] = complexQuery.indexOf(tok[j]);
											}
											pos = pos3[0];
											if(pos3.length > 1){
												for(var j=1;j<pos3.length;j++){
													pos = Math.min(pos,pos3[j]);													
												}
											}
											value = dojo.trim(complexQuery.substring(0,pos));
											complexQuery = dojo.trim(complexQuery.substring(pos));
										}else{ //not a space, so must be at the end of the complexQuery.
											value = dojo.trim(complexQuery);
											complexQuery = "";
										} //end  inner if(tok) else
										sQuery += self._containsValue(candidateItem, key, value, dojo.data.util.filter.patternToRegExp(value, ignoreCase));
									} //end outer if(tok) else
								} //end found ":"
							} //end if(complexQuery.length > 0)
						} //end while complexQuery.length > 0 && !err, so finished the i-th item.
						match = eval(sQuery);
					} //end else is non-null candidateItem.
					if(match){
						items.push(candidateItem);
					}
				} //end for/next of all items.
				if(err){
					//soft fail.
					items = [];
					console.log("The store's _fetchItems failed, probably due to a syntax error in query.");
				}
				findCallback(items, requestArgs);
			}else{
				// No query...
				// We want a copy to pass back in case the parent wishes to sort the array. 
				// We shouldn't allow resort of the internal list, so that multiple callers 
				// can get lists and sort without affecting each other.  We also need to
				// filter out any null values that have been left as a result of deleteItem()
				// calls in ItemFileWriteStore.
				for(var i = 0; i < arrayOfItems.length; ++i){
					var item = arrayOfItems[i];
					if(item !== null){
						items.push(item);
					}
				}
				findCallback(items, requestArgs);
			} //end if there is a query.
		}; //end filter function

		if(this._loadFinished){
			filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
		}else{
			if(this._jsonFileUrl !== this._ccUrl){
				dojo.deprecated("dojox.data.AndOrReadStore: ", 
								"To change the url, set the url property of the store," +
								" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
				this._ccUrl = this._jsonFileUrl;
				this.url = this._jsonFileUrl;
			}else if(this.url !== this._ccUrl){
				this._jsonFileUrl = this.url;
				this._ccUrl = this.url;
			}
			//See if there was any forced reset of data.
			if(this.data != null && this._jsonData == null){
				this._jsonData = this.data;
				this.data = null;
			}
			if(this._jsonFileUrl){
				//If fetches come in before the loading has finished, but while
				//a load is in progress, we have to defer the fetching to be 
				//invoked in the callback.
				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs, filter: filter});
				}else{
					this._loadInProgress = true;
					var getArgs = {
							url: self._jsonFileUrl, 
							handleAs: "json-comment-optional",
							preventCache: this.urlPreventCache
						};
					var getHandler = dojo.xhrGet(getArgs);
					getHandler.addCallback(function(data){
						try{
							self._getItemsFromLoadedData(data);
							self._loadFinished = true;
							self._loadInProgress = false;
							
							filter(keywordArgs, self._getItemsArray(keywordArgs.queryOptions));
							self._handleQueuedFetches();
						}catch(e){
							self._loadFinished = true;
							self._loadInProgress = false;
							errorCallback(e, keywordArgs);
						}
					});
					getHandler.addErrback(function(error){
						self._loadInProgress = false;
						errorCallback(error, keywordArgs);
					});
					
					//Wire up the cancel to abort of the request
					//This call cancel on the deferred if it hasn't been called
					//yet and then will chain to the simple abort of the
					//simpleFetch keywordArgs
					var oldAbort = null;
					if(keywordArgs.abort){
						oldAbort = keywordArgs.abort;
					}
					keywordArgs.abort = function(){
						var df = getHandler;
						if(df && df.fired === -1){
							df.cancel();
							df = null;
						}
						if(oldAbort){
							oldAbort.call(keywordArgs);
						}
					};
				}
			}else if(this._jsonData){
				try{
					this._loadFinished = true;
					this._getItemsFromLoadedData(this._jsonData);
					this._jsonData = null;
					filter(keywordArgs, this._getItemsArray(keywordArgs.queryOptions));
				}catch(e){
					errorCallback(e, keywordArgs);
				}
			}else{
				errorCallback(new Error("dojox.data.AndOrReadStore: No JSON source data was provided as either URL or a nested Javascript object."), keywordArgs);
			}
		} //end deferred fetching.
	}, //end _fetchItems

	_handleQueuedFetches: function(){
		//	summary: 
		//		Internal function to execute delayed request in the store.
		//Execute any deferred fetches now.
		if(this._queuedFetches.length > 0){
			for(var i = 0; i < this._queuedFetches.length; i++){
				var fData = this._queuedFetches[i];
				var delayedQuery = fData.args;
				var delayedFilter = fData.filter;
				if(delayedFilter){
					delayedFilter(delayedQuery, this._getItemsArray(delayedQuery.queryOptions)); 
				}else{
					this.fetchItemByIdentity(delayedQuery);
				}
			}
			this._queuedFetches = [];
		}
	},

	_getItemsArray: function(/*object?*/queryOptions){
		//	summary: 
		//		Internal function to determine which list of items to search over.
		//	queryOptions: The query options parameter, if any.
		if(queryOptions && queryOptions.deep){
			return this._arrayOfAllItems; 
		}
		return this._arrayOfTopLevelItems;
	},

	close: function(/*dojo.data.api.Request || keywordArgs || null */ request){
		//	summary: 
		//		See dojo.data.api.Read.close()
		if(this.clearOnClose &&
			this._loadFinished &&
			!this._loadInProgress){
			 //Reset all internalsback to default state.  This will force a reload
			 //on next fetch.  This also checks that the data or url param was set 
			 //so that the store knows it can get data.  Without one of those being set,
			 //the next fetch will trigger an error.

			 if(((this._jsonFileUrl == "" || this._jsonFileUrl == null) && 
				 (this.url == "" || this.url == null)
				) && this.data == null){
				 console.debug("dojox.data.AndOrReadStore: WARNING!  Data reload " +
					" information has not been provided." + 
					"  Please set 'url' or 'data' to the appropriate value before" +
					" the next fetch");
			 }
			 this._arrayOfAllItems = [];
			 this._arrayOfTopLevelItems = [];
			 this._loadFinished = false;
			 this._itemsByIdentity = null;
			 this._loadInProgress = false;
			 this._queuedFetches = [];
		 }
	},

	_getItemsFromLoadedData: function(/* Object */ dataObject){
		//	summary:
		//		Function to parse the loaded data into item format and build the internal items array.
		//	description:
		//		Function to parse the loaded data into item format and build the internal items array.
		//
		//	dataObject:
		//		The JS data object containing the raw data to convery into item format.
		//
		// 	returns: array
		//		Array of items in store item format.
		
		// First, we define a couple little utility functions...
		
		var self = this;
		function valueIsAnItem(/* anything */ aValue){
			// summary:
			//		Given any sort of value that could be in the raw json data,
			//		return true if we should interpret the value as being an
			//		item itself, rather than a literal value or a reference.
			// example:
			// 	|	false == valueIsAnItem("Kermit");
			// 	|	false == valueIsAnItem(42);
			// 	|	false == valueIsAnItem(new Date());
			// 	|	false == valueIsAnItem({_type:'Date', _value:'May 14, 1802'});
			// 	|	false == valueIsAnItem({_reference:'Kermit'});
			// 	|	true == valueIsAnItem({name:'Kermit', color:'green'});
			// 	|	true == valueIsAnItem({iggy:'pop'});
			// 	|	true == valueIsAnItem({foo:42});
			var isItem = (
				(aValue !== null) &&
				(typeof aValue === "object") &&
				(!dojo.isArray(aValue)) &&
				(!dojo.isFunction(aValue)) &&
				(aValue.constructor == Object) &&
				(typeof aValue._reference === "undefined") && 
				(typeof aValue._type === "undefined") && 
				(typeof aValue._value === "undefined") &&
				self.hierarchical
			);
			return isItem;
		}
		
		function addItemAndSubItemsToArrayOfAllItems(/* Item */ anItem){
			self._arrayOfAllItems.push(anItem);
			for(var attribute in anItem){
				var valueForAttribute = anItem[attribute];
				if(valueForAttribute){
					if(dojo.isArray(valueForAttribute)){
						var valueArray = valueForAttribute;
						for(var k = 0; k < valueArray.length; ++k){
							var singleValue = valueArray[k];
							if(valueIsAnItem(singleValue)){
								addItemAndSubItemsToArrayOfAllItems(singleValue);
							}
						}
					}else{
						if(valueIsAnItem(valueForAttribute)){
							addItemAndSubItemsToArrayOfAllItems(valueForAttribute);
						}
					}
				}
			}
		}

		this._labelAttr = dataObject.label;

		// We need to do some transformations to convert the data structure
		// that we read from the file into a format that will be convenient
		// to work with in memory.

		// Step 1: Walk through the object hierarchy and build a list of all items
		var i;
		var item;
		this._arrayOfAllItems = [];
		this._arrayOfTopLevelItems = dataObject.items;

		for(i = 0; i < this._arrayOfTopLevelItems.length; ++i){
			item = this._arrayOfTopLevelItems[i];
			addItemAndSubItemsToArrayOfAllItems(item);
			item[this._rootItemPropName]=true;
		}

		// Step 2: Walk through all the attribute values of all the items, 
		// and replace single values with arrays.  For example, we change this:
		//		{ name:'Miss Piggy', pets:'Foo-Foo'}
		// into this:
		//		{ name:['Miss Piggy'], pets:['Foo-Foo']}
		// 
		// We also store the attribute names so we can validate our store  
		// reference and item id special properties for the O(1) isItem
		var allAttributeNames = {};
		var key;

		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			for(key in item){
				if(key !== this._rootItemPropName){
					var value = item[key];
					if(value !== null){
						if(!dojo.isArray(value)){
							item[key] = [value];
						}
					}else{
						item[key] = [null];
					}
				}
				allAttributeNames[key]=key;
			}
		}

		// Step 3: Build unique property names to use for the _storeRefPropName and _itemNumPropName
		// This should go really fast, it will generally never even run the loop.
		while(allAttributeNames[this._storeRefPropName]){
			this._storeRefPropName += "_";
		}
		while(allAttributeNames[this._itemNumPropName]){
			this._itemNumPropName += "_";
		}
		while(allAttributeNames[this._reverseRefMap]){
			this._reverseRefMap += "_";
		}

		// Step 4: Some data files specify an optional 'identifier', which is 
		// the name of an attribute that holds the identity of each item. 
		// If this data file specified an identifier attribute, then build a 
		// hash table of items keyed by the identity of the items.
		var arrayOfValues;

		var identifier = dataObject.identifier;
		if(identifier){
			this._itemsByIdentity = {};
			this._features['dojo.data.api.Identity'] = identifier;
			for(i = 0; i < this._arrayOfAllItems.length; ++i){
				item = this._arrayOfAllItems[i];
				arrayOfValues = item[identifier];
				var identity = arrayOfValues[0];
				if(!this._itemsByIdentity[identity]){
					this._itemsByIdentity[identity] = item;
				}else{
					if(this._jsonFileUrl){
						throw new Error("dojox.data.AndOrReadStore:  The json data as specified by: [" + this._jsonFileUrl + "] is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}else if(this._jsonData){
						throw new Error("dojox.data.AndOrReadStore:  The json data provided by the creation arguments is malformed.  Items within the list have identifier: [" + identifier + "].  Value collided: [" + identity + "]");
					}
				}
			}
		}else{
			this._features['dojo.data.api.Identity'] = Number;
		}

		// Step 5: Walk through all the items, and set each item's properties 
		// for _storeRefPropName and _itemNumPropName, so that store.isItem() will return true.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i];
			item[this._storeRefPropName] = this;
			item[this._itemNumPropName] = i;
		}

		// Step 6: We walk through all the attribute values of all the items,
		// looking for type/value literals and item-references.
		//
		// We replace item-references with pointers to items.  For example, we change:
		//		{ name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
		// into this:
		//		{ name:['Kermit'], friends:[miss_piggy] } 
		// (where miss_piggy is the object representing the 'Miss Piggy' item).
		//
		// We replace type/value pairs with typed-literals.  For example, we change:
		//		{ name:['Nelson Mandela'], born:[{_type:'Date', _value:'July 18, 1918'}] }
		// into this:
		//		{ name:['Kermit'], born:(new Date('July 18, 1918')) } 
		//
		// We also generate the associate map for all items for the O(1) isItem function.
		for(i = 0; i < this._arrayOfAllItems.length; ++i){
			item = this._arrayOfAllItems[i]; // example: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
			for(key in item){
				arrayOfValues = item[key]; // example: [{_reference:{name:'Miss Piggy'}}]
				for(var j = 0; j < arrayOfValues.length; ++j){
					value = arrayOfValues[j]; // example: {_reference:{name:'Miss Piggy'}}
					if(value !== null && typeof value == "object"){
						if(("_type" in value) && ("_value" in value)){
							var type = value._type; // examples: 'Date', 'Color', or 'ComplexNumber'
							var mappingObj = this._datatypeMap[type]; // examples: Date, dojo.Color, foo.math.ComplexNumber, {type: dojo.Color, deserialize(value){ return new dojo.Color(value)}}
							if(!mappingObj){ 
								throw new Error("dojox.data.AndOrReadStore: in the typeMap constructor arg, no object class was specified for the datatype '" + type + "'");
							}else if(dojo.isFunction(mappingObj)){
								arrayOfValues[j] = new mappingObj(value._value);
							}else if(dojo.isFunction(mappingObj.deserialize)){
								arrayOfValues[j] = mappingObj.deserialize(value._value);
							}else{
								throw new Error("dojox.data.AndOrReadStore: Value provided in typeMap was neither a constructor, nor a an object with a deserialize function");
							}
						}
						if(value._reference){
							var referenceDescription = value._reference; // example: {name:'Miss Piggy'}
							if(!dojo.isObject(referenceDescription)){
								// example: 'Miss Piggy'
								// from an item like: { name:['Kermit'], friends:[{_reference:'Miss Piggy'}]}
								arrayOfValues[j] = this._getItemByIdentity(referenceDescription);
							}else{
								// example: {name:'Miss Piggy'}
								// from an item like: { name:['Kermit'], friends:[{_reference:{name:'Miss Piggy'}}] }
								for(var k = 0; k < this._arrayOfAllItems.length; ++k){
									var candidateItem = this._arrayOfAllItems[k];
									var found = true;
									for(var refKey in referenceDescription){
										if(candidateItem[refKey] != referenceDescription[refKey]){ 
											found = false; 
										}
									}
									if(found){ 
										arrayOfValues[j] = candidateItem; 
									}
								}
							}
							if(this.referenceIntegrity){
								var refItem = arrayOfValues[j];
								if(this.isItem(refItem)){
									this._addReferenceToMap(refItem, item, key);
								}
							}
						}else if(this.isItem(value)){
							//It's a child item (not one referenced through _reference).  
							//We need to treat this as a referenced item, so it can be cleaned up
							//in a write store easily.
							if(this.referenceIntegrity){
								this._addReferenceToMap(value, item, key);
							}
						}
					}
				}
			}
		}
	},

	_addReferenceToMap: function(/*item*/ refItem, /*item*/ parentItem, /*string*/ attribute){
		 //	summary:
		 //		Method to add an reference map entry for an item and attribute.
		 //	description:
		 //		Method to add an reference map entry for an item and attribute. 		 //
		 //	refItem:
		 //		The item that is referenced.
		 //	parentItem:
		 //		The item that holds the new reference to refItem.
		 //	attribute:
		 //		The attribute on parentItem that contains the new reference.
		 
		 //Stub function, does nothing.  Real processing is in ItemFileWriteStore.
	},

	getIdentity: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Identity.getIdentity()
		var identifier = this._features['dojo.data.api.Identity'];
		if(identifier === Number){
			return item[this._itemNumPropName]; // Number
		}else{
			var arrayOfValues = item[identifier];
			if(arrayOfValues){
				return arrayOfValues[0]; // Object || String
			}
		}
		return null; // null
	},

	fetchItemByIdentity: function(/* Object */ keywordArgs){
		//	summary: 
		//		See dojo.data.api.Identity.fetchItemByIdentity()

		// Hasn't loaded yet, we have to trigger the load.
		if(!this._loadFinished){
			var self = this;
			if(this._jsonFileUrl !== this._ccUrl){
				dojo.deprecated("dojox.data.AndOrReadStore: ", 
								"To change the url, set the url property of the store," +
								" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
				this._ccUrl = this._jsonFileUrl;
				this.url = this._jsonFileUrl;
			}else if(this.url !== this._ccUrl){
				this._jsonFileUrl = this.url;
				this._ccUrl = this.url;
			}
			//See if there was any forced reset of data.
			if(this.data != null && this._jsonData == null){
				this._jsonData = this.data;
				this.data = null;
			}
			if(this._jsonFileUrl){

				if(this._loadInProgress){
					this._queuedFetches.push({args: keywordArgs});
				}else{
					this._loadInProgress = true;
					var getArgs = {
							url: self._jsonFileUrl, 
							handleAs: "json-comment-optional",
							preventCache: this.urlPreventCache
					};
					var getHandler = dojo.xhrGet(getArgs);
					getHandler.addCallback(function(data){
						var scope = keywordArgs.scope?keywordArgs.scope:dojo.global;
						try{
							self._getItemsFromLoadedData(data);
							self._loadFinished = true;
							self._loadInProgress = false;
							var item = self._getItemByIdentity(keywordArgs.identity);
							if(keywordArgs.onItem){
								keywordArgs.onItem.call(scope, item);
							}
							self._handleQueuedFetches();
						}catch(error){
							self._loadInProgress = false;
							if(keywordArgs.onError){
								keywordArgs.onError.call(scope, error);
							}
						}
					});
					getHandler.addErrback(function(error){
						self._loadInProgress = false;
						if(keywordArgs.onError){
							var scope = keywordArgs.scope?keywordArgs.scope:dojo.global;
							keywordArgs.onError.call(scope, error);
						}
					});
				}

			}else if(this._jsonData){
				// Passed in data, no need to xhr.
				self._getItemsFromLoadedData(self._jsonData);
				self._jsonData = null;
				self._loadFinished = true;
				var item = self._getItemByIdentity(keywordArgs.identity);
				if(keywordArgs.onItem){
					var scope = keywordArgs.scope?keywordArgs.scope:dojo.global;
					keywordArgs.onItem.call(scope, item);
				}
			} 
		}else{
			// Already loaded.  We can just look it up and call back.
			var item = this._getItemByIdentity(keywordArgs.identity);
			if(keywordArgs.onItem){
				var scope = keywordArgs.scope?keywordArgs.scope:dojo.global;
				keywordArgs.onItem.call(scope, item);
			}
		}
	},

	_getItemByIdentity: function(/* Object */ identity){
		//	summary:
		//		Internal function to look an item up by its identity map.
		var item = null;
		if(this._itemsByIdentity){
			item = this._itemsByIdentity[identity];
		}else{
			item = this._arrayOfAllItems[identity];
		}
		if(item === undefined){
			item = null;
		}
		return item; // Object
	},

	getIdentityAttributes: function(/* item */ item){
		//	summary: 
		//		See dojo.data.api.Identity.getIdentifierAttributes()
		 
		var identifier = this._features['dojo.data.api.Identity'];
		if(identifier === Number){
			// If (identifier === Number) it means getIdentity() just returns
			// an integer item-number for each item.  The dojo.data.api.Identity
			// spec says we need to return null if the identity is not composed 
			// of attributes 
			return null; // null
		}else{
			return [identifier]; // Array
		}
	},
	
	_forceLoad: function(){
		//	summary: 
		//		Internal function to force a load of the store if it hasn't occurred yet.  This is required
		//		for specific functions to work properly.  
		var self = this;
		if(this._jsonFileUrl !== this._ccUrl){
			dojo.deprecated("dojox.data.AndOrReadStore: ", 
							"To change the url, set the url property of the store," +
							" not _jsonFileUrl.  _jsonFileUrl support will be removed in 2.0");
			this._ccUrl = this._jsonFileUrl;
			this.url = this._jsonFileUrl;
		}else if(this.url !== this._ccUrl){
			this._jsonFileUrl = this.url;
			this._ccUrl = this.url;
		}
		//See if there was any forced reset of data.
		if(this.data != null && this._jsonData == null){
			this._jsonData = this.data;
			this.data = null;
		}
		if(this._jsonFileUrl){
				var getArgs = {
					url: self._jsonFileUrl, 
					handleAs: "json-comment-optional",
					preventCache: this.urlPreventCache,
					sync: true
				};
			var getHandler = dojo.xhrGet(getArgs);
			getHandler.addCallback(function(data){
				try{
					//Check to be sure there wasn't another load going on concurrently 
					//So we don't clobber data that comes in on it.  If there is a load going on
					//then do not save this data.  It will potentially clobber current data.
					//We mainly wanted to sync/wait here.
					//TODO:  Revisit the loading scheme of this store to improve multi-initial
					//request handling.
					if(self._loadInProgress !== true && !self._loadFinished){
						self._getItemsFromLoadedData(data);
						self._loadFinished = true;
					}else if(self._loadInProgress){
						//Okay, we hit an error state we can't recover from.  A forced load occurred
						//while an async load was occurring.  Since we cannot block at this point, the best
						//that can be managed is to throw an error.
						throw new Error("dojox.data.AndOrReadStore:  Unable to perform a synchronous load, an async load is in progress."); 
					}
				}catch(e){
					console.log(e);
					throw e;
				}
			});
			getHandler.addErrback(function(error){
				throw error;
			});
		}else if(this._jsonData){
			self._getItemsFromLoadedData(self._jsonData);
			self._jsonData = null;
			self._loadFinished = true;
		} 
	}
});
//Mix in the simple fetch implementation to this class.
dojo.extend(dojox.data.AndOrReadStore,dojo.data.util.simpleFetch);



}

if(!dojo._hasResource["itme.finder.FinderBaseDataStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.finder.FinderBaseDataStore"] = true;
dojo.provide("itme.finder.FinderBaseDataStore");


 

dojo.declare("itme.finder.FinderBaseDataStore", null, {
    _dataStore: null,
  	_type: null,  
  	_categories: null,    
    constructor: function(type, domId){  
    	this._type = type;
    }, 
    getCategories : function() {
    	return this._categories;
    },
    populateCategoriesWithSelectedItems: function(myitems) {		
 		var i = 0;
 		for(i = 0; i < myitems.length; i++) { 			
 			var item = myitems[i];
 			this._categories[item.categoryIdx].addSelected(item.objectIdx);
 		} 	
 		phxTrace('populatedCategories for ' + i + ' items');
	},
	populateSearchResultItem: function(resultItemDomElement, finderObj, extraClass, notClickable) {
		var classname = 'SelectableCommonAction';
		if(notClickable) {
			classname = 'notSelectableCommonAction';
		}
		if(extraClass) classname = extraClass + " " + classname;		
		dojo.addClass(resultItemDomElement, classname);
		if(notClickable != true) {
 			dojo.connect(dojo.byId(resultItemDomElement), "onmouseover", function() { this.className=classname + "Hover"; });        
        	dojo.connect(dojo.byId(resultItemDomElement), "onmouseout", function() { this.className=classname; });
        	dojo.connect(dojo.byId(resultItemDomElement), "onclick", dojo.hitch(finderObj, finderObj.doClick));		
        }
	},
	createCategoryHTMLElements: function(numberOfResults, finderObj, moreId, moreText, notClickable) {
		var maxNumberOfResults = 32;
		var numberOfItemsCurrentlyShowing = 0;
		var lastCategory = false;
		var i = 1;
		for(i = 1; i < this._categories.length; i++) {
			if(this._categories[i]) {				
 				var count = this._categories[i].getSelectedCount();
 				if(count == 0) {
 					if(lastCategory == true) {
 						break;
 					}
 					continue;
 				} 					
 				if(maxNumberOfResults <= 0) {
 					var moreActionsDiv = dojo.create("div", {id: moreId, 
 					innerHTML:'<BR/><span> ... ' + ' ' + moreText + '</span>'}, dojo.byId(finderObj._finderIdPrefix + this._type + "_containerDiv"));      
 					this.populateSearchResultItem(moreActionsDiv, finderObj, 'finderMoreTasks', notClickable); 
 										
 					break;
 				}
 				var j;
 				//create category section 				
 				//phxTrace('creating category: ' + this._categories[i].getCategory());
 				var sectionDomNode = dojo.create("div", {id: "categorySection"+ this._type + "#_"+ i},  dojo.byId(finderObj._finderIdPrefix + this._type+"_containerDiv"));
              	var sectionTitleDomNode = objHighlightSearchStr(sectionDomNode, dijit.byId(finderObj._finderIdPrefix + "searchBox").attr("value"), this._categories[i].getCategory(), 'true'); 
                if(this._type == 'resources') {
                //  Figure out what to do for selecting a whole group of resources of same type
                //  Problem when we have more than 1 chassis, what chassis map to launch?
                //	this.populateSearchResultItem(sectionDomNode, finderObj, sectionDomNode.className);
                //	sectionTitleDomNode.title = 'Select all resources of this type';
                
                /*var launchAllSpan = dojo.create("span", {
                	innerHTML: ' - Select all for:'}, 
                	sectionTitleDomNode);
				dojo.addClass(launchAllSpan, 'SelectableCommonAction');
 				dojo.connect(launchAllSpan, "onmouseover", function() { this.className="SelectableCommonActionHover"; });        
        		dojo.connect(launchAllSpan, "onmouseout", function() { this.className="SelectableCommonAction"; });
       
    		    var dropDown = dojo.create("select", {
                	name:  this._categories[i].getCategory()}, 
                	sectionTitleDomNode);
                var option1 = dojo.create("option", {
                	value: 'chassis1', selected:'selected', innerHTML: 'MyChassis1'}, 
                	dropDown);
                var option2 = dojo.create("option", {
                	value: 'chassis2', innerHTML: 'MyChassis2'}, 
                	dropDown);
    		   
    		    var tooltipDiv = dojo.create('div', {});
    			dojo.create('div', {innerHTML: 'Select all resources of this type for: '}, tooltipDiv);
   		 		dojo.create('div', {innerHTML: 'chassis 1'}, tooltipDiv);
   		 		dojo.create('div', {innerHTML: 'chassis 2'}, tooltipDiv);
       			var dlg = new dijit.Tooltip({showDelay:0, connectId: [sectionTitleDomNode], label: tooltipDiv.innerHTML});
    			*/
   		   					
   		   		}      	
                //dojo.addClass(sectionTitle, "categorySectionTitle");
 				for(j = 0; j < count; j++) {
 					if(maxNumberOfResults == 0) break; 					
 					var itemId = this._type + "#_" + this._categories[i].getSelectedId(j);
 					var itemDiv = dojo.create("div", {id: itemId}, "categorySection" +this._type+"#_"+ i);      
          			objHighlightSearchStr(itemDiv, dijit.byId(finderObj._finderIdPrefix + "searchBox").attr("value"), this._categories[i].getSelectedName(j)); 
                    this.populateSearchResultItem(itemId, finderObj);
                    maxNumberOfResults--;
                    numberOfItemsCurrentlyShowing++;
 				} 	
 				    	
 				maxNumberOfResults = maxNumberOfResults - 2;
 				if(lastCategory == true) {
 					break;
 				}		
 			} else {
 				i = -1;
 				lastCategory = true; 
 			}
 		}
	},
	clearSelection: function() {
		var i;
		for(i = 0; i < this._categories.length; i++) {
			if(this._categories[i]) {
				this._categories[i].clearSelection();
			} else {
				break;
			}
		}
	},
    populateItems : function(list) {
    	phxTrace('inpopulate');
    	this._categories = new Array(200);
    	if(list == null || list == "") {
    		return;
    	}
    	var itemsFromList = list.split('#_');
    	var size = itemsFromList.length;
    	var i = 0;
    	var items = new Array(size);
    	this._categories[0] = new itme.finder.TaskCategory("Uncategorized");
    	for(i = 0; i < itemsFromList.length; i++) {    	
    		var object = itemsFromList[i];
    		var attributes = object.split(';');
   			var catIdx = 0;
   			var objectIdx;
   			for(catIdx = 0; catIdx < this._categories.length; catIdx++) {
   				if(this._categories[catIdx]) {
   					if(this._categories[catIdx].getCategory() == attributes[2]) {
   						var tempcategory = this._categories[catIdx]
   						objectIdx = tempcategory.add(attributes[0], attributes[1]);  
    					break;  					
    				} 
    			} else {
    				this._categories[catIdx] = new itme.finder.TaskCategory(attributes[2]);
    				objectIdx = this._categories[catIdx].add(attributes[0], attributes[1]); 
    				break;    					
    			}
    		}    			
    		items[i] = {'id': attributes[0], 'name':attributes[1], 'category':attributes[2], 'categoryIdx':catIdx, 'objectIdx':objectIdx};
    	}    
    	phxTrace('parsed items, total: ' + i);	
    	var dataStore = { identifier: 'id', label: 'name', items: items };     
    	this._dataStore = new  dojox.data.AndOrReadStore({data:dataStore});       	
    	phxTrace(this._dataStore);	
    	
    }
});

}

if(!dojo._hasResource["itme.finder.TaskDataStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.finder.TaskDataStore"] = true;
dojo.provide("itme.finder.TaskDataStore");



dojo.declare("itme.finder.TaskDataStore", [itme.finder.FinderBaseDataStore], {
	
    constructor: function(type, domId){  	
    	this.populateItems(dojo.byId(domId + ':taskFinderData').innerHTML);
    },
    populateItems: function() {
    	this.inherited(arguments);    
    },    
   
	
    
	initiateTaskSearch: function(searchString, finderObj) {
		query = "name:'*" + searchString + "*' OR category:'*" + searchString + "*'";			
		itmeUtils.traceTimeDelta("finderTimer", "before initiating search", true); 
		var request = this._dataStore.fetch({
			query:{
				complexQuery: query
			}, queryOptions: {
				ignoreCase: true
			}, 
 			onError: dojo.hitch(finderObj, finderObj.fetchFailed), 
 			onComplete: dojo.hitch(finderObj, finderObj.doTaskSearch)
 		});		
 		
	
	}
});
	

}

if(!dojo._hasResource["itme.finder.ResourceDataStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.finder.ResourceDataStore"] = true;
dojo.provide("itme.finder.ResourceDataStore");



dojo.declare("itme.finder.ResourceDataStore", [itme.finder.FinderBaseDataStore], {
   constructor: function(type, domId){  	
    	this.populateItems(dojo.byId(domId + ':resourceFinderData').innerHTML);
    },
    populateItems: function() {
    	this.inherited(arguments);    
    },      
	
    
	initiateTaskSearch: function(searchString, finderObj) {	
		query = "name:'*" + searchString + "*' OR category:'*" + searchString + "*'";			
		itmeUtils.traceTimeDelta("finderTimer", "before initiating search", true); 
  		if(this._dataStore) {
  			var request = this._dataStore.fetch({
				query:{
					complexQuery: query
				}, queryOptions: {
					ignoreCase: true
				}, 
 				onError: dojo.hitch(finderObj, finderObj.fetchFailed), 
 				onComplete: dojo.hitch(finderObj, finderObj.doResourceSearch)
 			});	
  		} else {
  			finderObj.doResourceSearch(new Array(0), null);
  		}
			
	}
});
	

}

if(!dojo._hasResource["itme.finder.GroupDataStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.finder.GroupDataStore"] = true;
dojo.provide("itme.finder.GroupDataStore");



dojo.declare("itme.finder.GroupDataStore", [itme.finder.FinderBaseDataStore], {
   constructor: function(type, domId){  	
    	this.populateItems(dojo.byId(domId + ':groupFinderData').innerHTML);
    },
    populateItems: function() {
    	this.inherited(arguments);    
    },      
	
    
	initiateTaskSearch: function(searchString, finderObj) {	
		query = "name:'*" + searchString + "*' OR category:'*" + searchString + "*'";			
		itmeUtils.traceTimeDelta("finderTimer", "before initiating search", true); 
  		if(this._dataStore) {
  			var request = this._dataStore.fetch({
				query:{
					complexQuery: query
				}, queryOptions: {
					ignoreCase: true
				}, 
 				onError: dojo.hitch(finderObj, finderObj.fetchFailed), 
 				onComplete: dojo.hitch(finderObj, finderObj.doGroupSearch)
 			});	
  		} else {
  			finderObj.doGroupSearch(new Array(0), null);
  		}
			
	}
});
	

}

if(!dojo._hasResource["itme.finder.ContextTaskDataStore"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.finder.ContextTaskDataStore"] = true;
dojo.provide("itme.finder.ContextTaskDataStore");
dojo.declare("itme.finder.ContextTaskDataStore", [itme.finder.FinderBaseDataStore], {
	
    constructor: function(type, list){  	
    	this.populateItems(list);
    },
    populateItems: function() {
    	this.inherited(arguments);    
    },    
   
	
    
	initiateTaskSearch: function(searchString, finderObj) {
		query = "name:'*" + searchString + "*' OR category:'*" + searchString + "*'";			
		itmeUtils.traceTimeDelta("finderTimer", "before initiating search", true); 
		if(this._dataStore) {
  			var request = this._dataStore.fetch({
				query:{
					complexQuery: query
				}, queryOptions: {
					ignoreCase: true
				}, 
 				onError: dojo.hitch(finderObj, finderObj.fetchFailed), 
 				onComplete: dojo.hitch(finderObj, finderObj.doContextTaskSearch)
 			});		
 		} else {
  			finderObj.doContextTaskSearch(new Array(0), null);
  		}
 		
	
	}
});
	

}

if(!dojo._hasResource["dijit.form.TextBox"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.TextBox"] = true;
dojo.provide("dijit.form.TextBox");



dojo.declare(
	"dijit.form.TextBox",
	dijit.form._FormValueWidget,
	{
		// summary:
		//		A base class for textbox form inputs

		// trim: Boolean
		//		Removes leading and trailing whitespace if true.  Default is false.
		trim: false,

		// uppercase: Boolean
		//		Converts all characters to uppercase if true.  Default is false.
		uppercase: false,

		// lowercase: Boolean
		//		Converts all characters to lowercase if true.  Default is false.
		lowercase: false,

		// propercase: Boolean
		//		Converts the first character of each word to uppercase if true.
		propercase: false,

		//	maxLength: String
		//		HTML INPUT tag maxLength declaration.
		maxLength: "",

		//	selectOnClick: [const] Boolean
		//		If true, all text will be selected when focused with mouse
		selectOnClick: false,

		//	placeHolder: String
		//		Defines a hint to help users fill out the input field (as defined in HTML 5).
		//		This should only contain plain text (no html markup).
		placeHolder: "",
		
		templateString: dojo.cache("dijit.form", "templates/TextBox.html", "<div class=\"dijit dijitReset dijitInline dijitLeft\" id=\"widget_${id}\" waiRole=\"presentation\"\n\t><div class=\"dijitReset dijitInputField dijitInputContainer\"\n\t\t><input class=\"dijitReset dijitInputInner\" dojoAttachPoint='textbox,focusNode' autocomplete=\"off\"\n\t\t\t${!nameAttrSetting} type='${type}'\n\t/></div\n></div>\n"),
		_singleNodeTemplate: '<input class="dijit dijitReset dijitLeft dijitInputField" dojoAttachPoint="textbox,focusNode" autocomplete="off" type="${type}" ${!nameAttrSetting} />',

		_buttonInputDisabled: dojo.isIE ? "disabled" : "", // allows IE to disallow focus, but Firefox cannot be disabled for mousedown events

		baseClass: "dijitTextBox",

		attributeMap: dojo.delegate(dijit.form._FormValueWidget.prototype.attributeMap, {
			maxLength: "focusNode"
		}),
		
		postMixInProperties: function(){
			var type = this.type.toLowerCase();
			if(this.templateString && this.templateString.toLowerCase() == "input" || ((type == "hidden" || type == "file") && this.templateString == dijit.form.TextBox.prototype.templateString)){
				this.templateString = this._singleNodeTemplate;
			}
			this.inherited(arguments);
		},

		_setPlaceHolderAttr: function(v){
			this.placeHolder = v;
			if(!this._phspan){
				this._attachPoints.push('_phspan');
				/* dijitInputField class gives placeHolder same padding as the input field
				 * parent node already has dijitInputField class but it doesn't affect this <span>
				 * since it's position: absolute.
				 */
				this._phspan = dojo.create('span',{className:'dijitPlaceHolder dijitInputField'},this.textbox,'after');
			}
			this._phspan.innerHTML="";
			this._phspan.appendChild(document.createTextNode(v));
			
			this._updatePlaceHolder();
		},
		
		_updatePlaceHolder: function(){
			if(this._phspan){
				this._phspan.style.display=(this.placeHolder&&!this._focused&&!this.textbox.value)?"":"none";
			}
		},

		_getValueAttr: function(){
			// summary:
			//		Hook so attr('value') works as we like.
			// description:
			//		For `dijit.form.TextBox` this basically returns the value of the <input>.
			//
			//		For `dijit.form.MappedTextBox` subclasses, which have both
			//		a "displayed value" and a separate "submit value",
			//		This treats the "displayed value" as the master value, computing the
			//		submit value from it via this.parse().
			return this.parse(this.get('displayedValue'), this.constraints);
		},

		_setValueAttr: function(value, /*Boolean?*/ priorityChange, /*String?*/ formattedValue){
			// summary:
			//		Hook so attr('value', ...) works.
			//
			// description:
			//		Sets the value of the widget to "value" which can be of
			//		any type as determined by the widget.
			//
			// value:
			//		The visual element value is also set to a corresponding,
			//		but not necessarily the same, value.
			//
			// formattedValue:
			//		If specified, used to set the visual element value,
			//		otherwise a computed visual value is used.
			//
			// priorityChange:
			//		If true, an onChange event is fired immediately instead of
			//		waiting for the next blur event.

			var filteredValue;
			if(value !== undefined){
				// TODO: this is calling filter() on both the display value and the actual value.
				// I added a comment to the filter() definition about this, but it should be changed.
				filteredValue = this.filter(value);
				if(typeof formattedValue != "string"){
					if(filteredValue !== null && ((typeof filteredValue != "number") || !isNaN(filteredValue))){
						formattedValue = this.filter(this.format(filteredValue, this.constraints));
					}else{ formattedValue = ''; }
				}
			}
			if(formattedValue != null && formattedValue != undefined && ((typeof formattedValue) != "number" || !isNaN(formattedValue)) && this.textbox.value != formattedValue){
				this.textbox.value = formattedValue;
			}

			this._updatePlaceHolder();

			this.inherited(arguments, [filteredValue, priorityChange]);
		},

		// displayedValue: String
		//		For subclasses like ComboBox where the displayed value
		//		(ex: Kentucky) and the serialized value (ex: KY) are different,
		//		this represents the displayed value.
		//
		//		Setting 'displayedValue' through attr('displayedValue', ...)
		//		updates 'value', and vice-versa.  Otherwise 'value' is updated
		//		from 'displayedValue' periodically, like onBlur etc.
		//
		//		TODO: move declaration to MappedTextBox?
		//		Problem is that ComboBox references displayedValue,
		//		for benefit of FilteringSelect.
		displayedValue: "",

		getDisplayedValue: function(){
			// summary:
			//		Deprecated.   Use set('displayedValue') instead.
			// tags:
			//		deprecated
			dojo.deprecated(this.declaredClass+"::getDisplayedValue() is deprecated. Use set('displayedValue') instead.", "", "2.0");
			return this.get('displayedValue');
		},

		_getDisplayedValueAttr: function(){
			// summary:
			//		Hook so attr('displayedValue') works.
			// description:
			//		Returns the displayed value (what the user sees on the screen),
			// 		after filtering (ie, trimming spaces etc.).
			//
			//		For some subclasses of TextBox (like ComboBox), the displayed value
			//		is different from the serialized value that's actually
			//		sent to the server (see dijit.form.ValidationTextBox.serialize)

			return this.filter(this.textbox.value);
		},

		setDisplayedValue: function(/*String*/value){
			// summary:
			//		Deprecated.   Use set('displayedValue', ...) instead.
			// tags:
			//		deprecated
			dojo.deprecated(this.declaredClass+"::setDisplayedValue() is deprecated. Use set('displayedValue', ...) instead.", "", "2.0");
			this.set('displayedValue', value);
		},

		_setDisplayedValueAttr: function(/*String*/value){
			// summary:
			//		Hook so attr('displayedValue', ...) works.
			// description:
			//		Sets the value of the visual element to the string "value".
			//		The widget value is also set to a corresponding,
			//		but not necessarily the same, value.

			if(value === null || value === undefined){ value = '' }
			else if(typeof value != "string"){ value = String(value) }
			this.textbox.value = value;
			this._setValueAttr(this.get('value'), undefined, value);
		},

		format: function(/* String */ value, /* Object */ constraints){
			// summary:
			//		Replacable function to convert a value to a properly formatted string.
			// tags:
			//		protected extension
			return ((value == null || value == undefined) ? "" : (value.toString ? value.toString() : value));
		},

		parse: function(/* String */ value, /* Object */ constraints){
			// summary:
			//		Replacable function to convert a formatted string to a value
			// tags:
			//		protected extension

			return value;	// String
		},

		_refreshState: function(){
			// summary:
			//		After the user types some characters, etc., this method is
			//		called to check the field for validity etc.  The base method
			//		in `dijit.form.TextBox` does nothing, but subclasses override.
			// tags:
			//		protected
		},

		_onInput: function(e){
			if(e && e.type && /key/i.test(e.type) && e.keyCode){
				switch(e.keyCode){
					case dojo.keys.SHIFT:
					case dojo.keys.ALT:
					case dojo.keys.CTRL:
					case dojo.keys.TAB:
						return;
				}
			}
			if(this.intermediateChanges){
				var _this = this;
				// the setTimeout allows the key to post to the widget input box
				setTimeout(function(){ _this._handleOnChange(_this.get('value'), false); }, 0);
			}
			this._refreshState();
		},

		postCreate: function(){
			// setting the value here is needed since value="" in the template causes "undefined"
			// and setting in the DOM (instead of the JS object) helps with form reset actions
			if(dojo.isIE){ // IE INPUT tag fontFamily has to be set directly using STYLE
				// the setTimeout gives IE a chance to render the TextBox and to deal with font inheritance
				setTimeout(dojo.hitch(this, function(){
					var s = dojo.getComputedStyle(this.domNode);
					if(s){
						var ff = s.fontFamily;
						if(ff){
							var inputs = this.domNode.getElementsByTagName("INPUT");
							if(inputs){
								for(var i=0; i < inputs.length; i++){
									inputs[i].style.fontFamily = ff;
								}
							}
						}
					}
				}), 0);
			}
			this.textbox.setAttribute("value", this.textbox.value); // DOM and JS values shuld be the same
			this.inherited(arguments);
			if(dojo.isMoz || dojo.isOpera){
				this.connect(this.textbox, "oninput", this._onInput);
			}else{
				this.connect(this.textbox, "onkeydown", this._onInput);
				this.connect(this.textbox, "onkeyup", this._onInput);
				this.connect(this.textbox, "onpaste", this._onInput);
				this.connect(this.textbox, "oncut", this._onInput);
			}
		},

		_blankValue: '', // if the textbox is blank, what value should be reported
		filter: function(val){
			// summary:
			//		Auto-corrections (such as trimming) that are applied to textbox
			//		value on blur or form submit.
			// description:
			//		For MappedTextBox subclasses, this is called twice
			// 			- once with the display value
			//			- once the value as set/returned by attr('value', ...)
			//		and attr('value'), ex: a Number for NumberTextBox.
			//
			//		In the latter case it does corrections like converting null to NaN.  In
			//		the former case the NumberTextBox.filter() method calls this.inherited()
			//		to execute standard trimming code in TextBox.filter().
			//
			//		TODO: break this into two methods in 2.0
			//
			// tags:
			//		protected extension
			if(val === null){ return this._blankValue; }
			if(typeof val != "string"){ return val; }
			if(this.trim){
				val = dojo.trim(val);
			}
			if(this.uppercase){
				val = val.toUpperCase();
			}
			if(this.lowercase){
				val = val.toLowerCase();
			}
			if(this.propercase){
				val = val.replace(/[^\s]+/g, function(word){
					return word.substring(0,1).toUpperCase() + word.substring(1);
				});
			}
			return val;
		},

		_setBlurValue: function(){
			this._setValueAttr(this.get('value'), true);
		},

		_onBlur: function(e){
			if(this.disabled){ return; }
			this._setBlurValue();
			this.inherited(arguments);

			if(this._selectOnClickHandle){
				this.disconnect(this._selectOnClickHandle);
			}
			if(this.selectOnClick && dojo.isMoz){
				this.textbox.selectionStart = this.textbox.selectionEnd = undefined; // clear selection so that the next mouse click doesn't reselect
			}
			
			this._updatePlaceHolder();
		},

		_onFocus: function(/*String*/ by){
			if(this.disabled || this.readOnly){ return; }

			// Select all text on focus via click if nothing already selected.
			// Since mouse-up will clear the selection need to defer selection until after mouse-up.
			// Don't do anything on focus by tabbing into the widget since there's no associated mouse-up event.
			if(this.selectOnClick && by == "mouse"){
				this._selectOnClickHandle = this.connect(this.domNode, "onmouseup", function(){
					// Only select all text on first click; otherwise users would have no way to clear
					// the selection.
					this.disconnect(this._selectOnClickHandle);

					// Check if the user selected some text manually (mouse-down, mouse-move, mouse-up)
					// and if not, then select all the text
					var textIsNotSelected;
					if(dojo.isIE){
						var range = dojo.doc.selection.createRange();
						var parent = range.parentElement();
						textIsNotSelected = parent == this.textbox && range.text.length == 0;
					}else{
						textIsNotSelected = this.textbox.selectionStart == this.textbox.selectionEnd;
					}
					if(textIsNotSelected){
						dijit.selectInputText(this.textbox);
					}
				});
			}

			this._updatePlaceHolder();
			
			this._refreshState();
			this.inherited(arguments);
		},

		reset: function(){
			// Overrides dijit._FormWidget.reset().
			// Additionally resets the displayed textbox value to ''
			this.textbox.value = '';
			this.inherited(arguments);
		}
	}
);

dijit.selectInputText = function(/*DomNode*/element, /*Number?*/ start, /*Number?*/ stop){
	// summary:
	//		Select text in the input element argument, from start (default 0), to stop (default end).

	// TODO: use functions in _editor/selection.js?
	var _window = dojo.global;
	var _document = dojo.doc;
	element = dojo.byId(element);
	if(isNaN(start)){ start = 0; }
	if(isNaN(stop)){ stop = element.value ? element.value.length : 0; }
	dijit.focus(element);
	if(_document["selection"] && dojo.body()["createTextRange"]){ // IE
		if(element.createTextRange){
			var range = element.createTextRange();
			with(range){
				collapse(true);
				moveStart("character", -99999); // move to 0
				moveStart("character", start); // delta from 0 is the correct position
				moveEnd("character", stop-start);
				select();
			}
		}
	}else if(_window["getSelection"]){
		if(element.setSelectionRange){
			element.setSelectionRange(start, stop);
		}
	}
};

}

if(!dojo._hasResource["dojo.regexp"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.regexp"] = true;
dojo.provide("dojo.regexp");

/*=====
dojo.regexp = {
	// summary: Regular expressions and Builder resources
};
=====*/

dojo.regexp.escapeString = function(/*String*/str, /*String?*/except){
	//	summary:
	//		Adds escape sequences for special characters in regular expressions
	// except:
	//		a String with special characters to be left unescaped

	return str.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, function(ch){
		if(except && except.indexOf(ch) != -1){
			return ch;
		}
		return "\\" + ch;
	}); // String
}

dojo.regexp.buildGroupRE = function(/*Object|Array*/arr, /*Function*/re, /*Boolean?*/nonCapture){
	//	summary:
	//		Builds a regular expression that groups subexpressions
	//	description:
	//		A utility function used by some of the RE generators. The
	//		subexpressions are constructed by the function, re, in the second
	//		parameter.  re builds one subexpression for each elem in the array
	//		a, in the first parameter. Returns a string for a regular
	//		expression that groups all the subexpressions.
	// arr:
	//		A single value or an array of values.
	// re:
	//		A function. Takes one parameter and converts it to a regular
	//		expression. 
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. Defaults to false

	// case 1: a is a single value.
	if(!(arr instanceof Array)){
		return re(arr); // String
	}

	// case 2: a is an array
	var b = [];
	for(var i = 0; i < arr.length; i++){
		// convert each elem to a RE
		b.push(re(arr[i]));
	}

	 // join the REs as alternatives in a RE group.
	return dojo.regexp.group(b.join("|"), nonCapture); // String
}

dojo.regexp.group = function(/*String*/expression, /*Boolean?*/nonCapture){
	// summary:
	//		adds group match to expression
	// nonCapture:
	//		If true, uses non-capturing match, otherwise matches are retained
	//		by regular expression. 
	return "(" + (nonCapture ? "?:":"") + expression + ")"; // String
}

}

if(!dojo._hasResource["dojo.cookie"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dojo.cookie"] = true;
dojo.provide("dojo.cookie");



/*=====
dojo.__cookieProps = function(){
	//	expires: Date|String|Number?
	//		If a number, the number of days from today at which the cookie
	//		will expire. If a date, the date past which the cookie will expire.
	//		If expires is in the past, the cookie will be deleted.
	//		If expires is omitted or is 0, the cookie will expire when the browser closes. << FIXME: 0 seems to disappear right away? FF3.
	//	path: String?
	//		The path to use for the cookie.
	//	domain: String?
	//		The domain to use for the cookie.
	//	secure: Boolean?
	//		Whether to only send the cookie on secure connections
	this.expires = expires;
	this.path = path;
	this.domain = domain;
	this.secure = secure;
}
=====*/


dojo.cookie = function(/*String*/name, /*String?*/value, /*dojo.__cookieProps?*/props){
	//	summary: 
	//		Get or set a cookie.
	//	description:
	// 		If one argument is passed, returns the value of the cookie
	// 		For two or more arguments, acts as a setter.
	//	name:
	//		Name of the cookie
	//	value:
	//		Value for the cookie
	//	props: 
	//		Properties for the cookie
	//	example:
	//		set a cookie with the JSON-serialized contents of an object which
	//		will expire 5 days from now:
	//	|	dojo.cookie("configObj", dojo.toJson(config), { expires: 5 });
	//	
	//	example:
	//		de-serialize a cookie back into a JavaScript object:
	//	|	var config = dojo.fromJson(dojo.cookie("configObj"));
	//	
	//	example:
	//		delete a cookie:
	//	|	dojo.cookie("configObj", null, {expires: -1});
	var c = document.cookie;
	if(arguments.length == 1){
		var matches = c.match(new RegExp("(?:^|; )" + dojo.regexp.escapeString(name) + "=([^;]*)"));
		return matches ? decodeURIComponent(matches[1]) : undefined; // String or undefined
	}else{
		props = props || {};
// FIXME: expires=0 seems to disappear right away, not on close? (FF3)  Change docs?
		var exp = props.expires;
		if(typeof exp == "number"){ 
			var d = new Date();
			d.setTime(d.getTime() + exp*24*60*60*1000);
			exp = props.expires = d;
		}
		if(exp && exp.toUTCString){ props.expires = exp.toUTCString(); }

		value = encodeURIComponent(value);
		var updatedCookie = name + "=" + value, propName;
		for(propName in props){
			updatedCookie += "; " + propName;
			var propValue = props[propName];
			if(propValue !== true){ updatedCookie += "=" + propValue; }
		}
		document.cookie = updatedCookie;
	}
};

dojo.cookie.isSupported = function(){
	//	summary:
	//		Use to determine if the current browser supports cookies or not.
	//		
	//		Returns true if user allows cookies.
	//		Returns false if user doesn't allow cookies.

	if(!("cookieEnabled" in navigator)){
		this("__djCookieTest__", "CookiesAllowed");
		navigator.cookieEnabled = this("__djCookieTest__") == "CookiesAllowed";
		if(navigator.cookieEnabled){
			this("__djCookieTest__", "", {expires: -1});
		}
	}
	return navigator.cookieEnabled;
};

}

if(!dojo._hasResource["dijit.form.ToggleButton"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.form.ToggleButton"] = true;
dojo.provide("dijit.form.ToggleButton");


}

if(!dojo._hasResource["dijit.layout.StackController"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.StackController"] = true;
dojo.provide("dijit.layout.StackController");







dojo.declare(
		"dijit.layout.StackController",
		[dijit._Widget, dijit._Templated, dijit._Container],
		{
			// summary:
			//		Set of buttons to select a page in a page list.
			// description:
			//		Monitors the specified StackContainer, and whenever a page is
			//		added, deleted, or selected, updates itself accordingly.

			templateString: "<span wairole='tablist' dojoAttachEvent='onkeypress' class='dijitStackController'></span>",

			// containerId: [const] String
			//		The id of the page container that I point to
			containerId: "",

			// buttonWidget: [const] String
			//		The name of the button widget to create to correspond to each page
			buttonWidget: "dijit.layout._StackButton",

			postCreate: function(){
				dijit.setWaiRole(this.domNode, "tablist");

				this.pane2button = {};		// mapping from pane id to buttons
				this.pane2handles = {};		// mapping from pane id to this.connect() handles

				// Listen to notifications from StackContainer
				this.subscribe(this.containerId+"-startup", "onStartup");
				this.subscribe(this.containerId+"-addChild", "onAddChild");
				this.subscribe(this.containerId+"-removeChild", "onRemoveChild");
				this.subscribe(this.containerId+"-selectChild", "onSelectChild");
				this.subscribe(this.containerId+"-containerKeyPress", "onContainerKeyPress");
			},

			onStartup: function(/*Object*/ info){
				// summary:
				//		Called after StackContainer has finished initializing
				// tags:
				//		private
				dojo.forEach(info.children, this.onAddChild, this);
				if(info.selected){
					// Show button corresponding to selected pane (unless selected
					// is null because there are no panes)
					this.onSelectChild(info.selected);
				}
			},

			destroy: function(){
				for(var pane in this.pane2button){
					this.onRemoveChild(dijit.byId(pane));
				}
				this.inherited(arguments);
			},

			onAddChild: function(/*dijit._Widget*/ page, /*Integer?*/ insertIndex){
				// summary:
				//		Called whenever a page is added to the container.
				//		Create button corresponding to the page.
				// tags:
				//		private

				// create an instance of the button widget
				var cls = dojo.getObject(this.buttonWidget);
				var button = new cls({
					id: this.id + "_" + page.id,
					label: page.title,
					dir: page.dir,
					lang: page.lang,
					showLabel: page.showTitle,
					iconClass: page.iconClass,
					closeButton: page.closable,
					title: page.tooltip
				});
				dijit.setWaiState(button.focusNode,"selected", "false");
				this.pane2handles[page.id] = [
					this.connect(page, 'set', function(name, value){
						var buttonAttr = {
							title: 'label',
							showTitle: 'showLabel',
							iconClass: 'iconClass',
							closable: 'closeButton',
							tooltip: 'title'
						}[name];
						if(buttonAttr){
							button.set(buttonAttr, value);
						}
					}),
					this.connect(button, 'onClick', dojo.hitch(this,"onButtonClick", page)),
					this.connect(button, 'onClickCloseButton', dojo.hitch(this,"onCloseButtonClick", page))
				];
				this.addChild(button, insertIndex);
				this.pane2button[page.id] = button;
				page.controlButton = button;	// this value might be overwritten if two tabs point to same container
				if(!this._currentChild){ // put the first child into the tab order
					button.focusNode.setAttribute("tabIndex", "0");
					dijit.setWaiState(button.focusNode, "selected", "true");
					this._currentChild = page;
				}
				// make sure all tabs have the same length
				if(!this.isLeftToRight() && dojo.isIE && this._rectifyRtlTabList){
					this._rectifyRtlTabList();
				}
			},

			onRemoveChild: function(/*dijit._Widget*/ page){
				// summary:
				//		Called whenever a page is removed from the container.
				//		Remove the button corresponding to the page.
				// tags:
				//		private

				if(this._currentChild === page){ this._currentChild = null; }
				dojo.forEach(this.pane2handles[page.id], this.disconnect, this);
				delete this.pane2handles[page.id];
				var button = this.pane2button[page.id];
				if(button){
					this.removeChild(button);
					delete this.pane2button[page.id];
					button.destroy();
				}
				delete page.controlButton;
			},

			onSelectChild: function(/*dijit._Widget*/ page){
				// summary:
				//		Called when a page has been selected in the StackContainer, either by me or by another StackController
				// tags:
				//		private

				if(!page){ return; }

				if(this._currentChild){
					var oldButton=this.pane2button[this._currentChild.id];
					oldButton.set('checked', false);
					dijit.setWaiState(oldButton.focusNode, "selected", "false");
					oldButton.focusNode.setAttribute("tabIndex", "-1");
				}

				var newButton=this.pane2button[page.id];
				newButton.set('checked', true);
				dijit.setWaiState(newButton.focusNode, "selected", "true");
				this._currentChild = page;
				newButton.focusNode.setAttribute("tabIndex", "0");
				var container = dijit.byId(this.containerId);
				dijit.setWaiState(container.containerNode, "labelledby", newButton.id);
			},

			onButtonClick: function(/*dijit._Widget*/ page){
				// summary:
				//		Called whenever one of my child buttons is pressed in an attempt to select a page
				// tags:
				//		private

				var container = dijit.byId(this.containerId);
				container.selectChild(page);
			},

			onCloseButtonClick: function(/*dijit._Widget*/ page){
				// summary:
				//		Called whenever one of my child buttons [X] is pressed in an attempt to close a page
				// tags:
				//		private

				var container = dijit.byId(this.containerId);
				container.closeChild(page);
				if(this._currentChild){
					var b = this.pane2button[this._currentChild.id];
					if(b){
						dijit.focus(b.focusNode || b.domNode);
					}
				}
			},

			// TODO: this is a bit redundant with forward, back api in StackContainer
			adjacent: function(/*Boolean*/ forward){
				// summary:
				//		Helper for onkeypress to find next/previous button
				// tags:
				//		private

				if(!this.isLeftToRight() && (!this.tabPosition || /top|bottom/.test(this.tabPosition))){ forward = !forward; }
				// find currently focused button in children array
				var children = this.getChildren();
				var current = dojo.indexOf(children, this.pane2button[this._currentChild.id]);
				// pick next button to focus on
				var offset = forward ? 1 : children.length - 1;
				return children[ (current + offset) % children.length ]; // dijit._Widget
			},

			onkeypress: function(/*Event*/ e){
				// summary:
				//		Handle keystrokes on the page list, for advancing to next/previous button
				//		and closing the current page if the page is closable.
				// tags:
				//		private

				if(this.disabled || e.altKey ){ return; }
				var forward = null;
				if(e.ctrlKey || !e._djpage){
					var k = dojo.keys;
					switch(e.charOrCode){
						case k.LEFT_ARROW:
						case k.UP_ARROW:
							if(!e._djpage){ forward = false; }
							break;
						case k.PAGE_UP:
							if(e.ctrlKey){ forward = false; }
							break;
						case k.RIGHT_ARROW:
						case k.DOWN_ARROW:
							if(!e._djpage){ forward = true; }
							break;
						case k.PAGE_DOWN:
							if(e.ctrlKey){ forward = true; }
							break;
						case k.DELETE:
							if(this._currentChild.closable){
								this.onCloseButtonClick(this._currentChild);
							}
							dojo.stopEvent(e);
							break;
						default:
							if(e.ctrlKey){
								if(e.charOrCode === k.TAB){
									this.adjacent(!e.shiftKey).onClick();
									dojo.stopEvent(e);
								}else if(e.charOrCode == "w"){
									if(this._currentChild.closable){
										this.onCloseButtonClick(this._currentChild);
									}
									dojo.stopEvent(e); // avoid browser tab closing.
								}
							}
					}
					// handle page navigation
					if(forward !== null){
						this.adjacent(forward).onClick();
						dojo.stopEvent(e);
					}
				}
			},

			onContainerKeyPress: function(/*Object*/ info){
				// summary:
				//		Called when there was a keypress on the container
				// tags:
				//		private
				info.e._djpage = info.page;
				this.onkeypress(info.e);
			}
	});


dojo.declare("dijit.layout._StackButton",
		dijit.form.ToggleButton,
		{
		// summary:
		//		Internal widget used by StackContainer.
		// description:
		//		The button-like or tab-like object you click to select or delete a page
		// tags:
		//		private

		// Override _FormWidget.tabIndex.
		// StackContainer buttons are not in the tab order by default.
		// Probably we should be calling this.startupKeyNavChildren() instead.
		tabIndex: "-1",

		postCreate: function(/*Event*/ evt){
			dijit.setWaiRole((this.focusNode || this.domNode), "tab");
			this.inherited(arguments);
		},

		onClick: function(/*Event*/ evt){
			// summary:
			//		This is for TabContainer where the tabs are <span> rather than button,
			//		so need to set focus explicitly (on some browsers)
			//		Note that you shouldn't override this method, but you can connect to it.
			dijit.focus(this.focusNode);

			// ... now let StackController catch the event and tell me what to do
		},

		onClickCloseButton: function(/*Event*/ evt){
			// summary:
			//		StackContainer connects to this function; if your widget contains a close button
			//		then clicking it should call this function.
			//		Note that you shouldn't override this method, but you can connect to it.
			evt.stopPropagation();
		}
	});


}

if(!dojo._hasResource["dijit.layout.StackContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.StackContainer"] = true;
dojo.provide("dijit.layout.StackContainer");






dojo.declare(
	"dijit.layout.StackContainer",
	dijit.layout._LayoutWidget,
	{
	// summary:
	//		A container that has multiple children, but shows only
	//		one child at a time
	//
	// description:
	//		A container for widgets (ContentPanes, for example) That displays
	//		only one Widget at a time.
	//
	//		Publishes topics [widgetId]-addChild, [widgetId]-removeChild, and [widgetId]-selectChild
	//
	//		Can be base class for container, Wizard, Show, etc.

	// doLayout: Boolean
	//		If true, change the size of my currently displayed child to match my size
	doLayout: true,

	// persist: Boolean
	//		Remembers the selected child across sessions
	persist: false,

	baseClass: "dijitStackContainer",

/*=====
	// selectedChildWidget: [readonly] dijit._Widget
	//		References the currently selected child widget, if any.
	//		Adjust selected child with selectChild() method.
	selectedChildWidget: null,
=====*/

	postCreate: function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "dijitLayoutContainer");
		dijit.setWaiRole(this.containerNode, "tabpanel");
		this.connect(this.domNode, "onkeypress", this._onKeyPress);
	},

	startup: function(){
		if(this._started){ return; }

		var children = this.getChildren();

		// Setup each page panel to be initially hidden
		dojo.forEach(children, this._setupChild, this);

		// Figure out which child to initially display, defaulting to first one
		if(this.persist){
			this.selectedChildWidget = dijit.byId(dojo.cookie(this.id + "_selectedChild"));
		}else{
			dojo.some(children, function(child){
				if(child.selected){
					this.selectedChildWidget = child;
				}
				return child.selected;
			}, this);
		}
		var selected = this.selectedChildWidget;
		if(!selected && children[0]){
			selected = this.selectedChildWidget = children[0];
			selected.selected = true;
		}

		// Publish information about myself so any StackControllers can initialize.
		// This needs to happen before this.inherited(arguments) so that for
		// TabContainer, this._contentBox doesn't include the space for the tab labels.
		dojo.publish(this.id+"-startup", [{children: children, selected: selected}]);

		// Startup each child widget, and do initial layout like setting this._contentBox,
		// then calls this.resize() which does the initial sizing on the selected child.
		this.inherited(arguments);
	},

	resize: function(){
		// Resize is called when we are first made visible (it's called from startup()
		// if we are initially visible).   If this is the first time we've been made
		// visible then show our first child.
		var selected = this.selectedChildWidget;
		if(selected && !this._hasBeenShown){
			this._hasBeenShown = true;
			this._showChild(selected);
		}
		this.inherited(arguments);
	},

	_setupChild: function(/*dijit._Widget*/ child){
		// Overrides _LayoutWidget._setupChild()

		this.inherited(arguments);

		dojo.removeClass(child.domNode, "dijitVisible");
		dojo.addClass(child.domNode, "dijitHidden");

		// remove the title attribute so it doesn't show up when i hover
		// over a node
		child.domNode.title = "";
	},

	addChild: function(/*dijit._Widget*/ child, /*Integer?*/ insertIndex){
		// Overrides _Container.addChild() to do layout and publish events

		this.inherited(arguments);

		if(this._started){
			dojo.publish(this.id+"-addChild", [child, insertIndex]);

			// in case the tab titles have overflowed from one line to two lines
			// (or, if this if first child, from zero lines to one line)
			// TODO: w/ScrollingTabController this is no longer necessary, although
			// ScrollTabController.resize() does need to get called to show/hide
			// the navigation buttons as appropriate, but that's handled in ScrollingTabController.onAddChild()
			this.layout();

			// if this is the first child, then select it
			if(!this.selectedChildWidget){
				this.selectChild(child);
			}
		}
	},

	removeChild: function(/*dijit._Widget*/ page){
		// Overrides _Container.removeChild() to do layout and publish events

		this.inherited(arguments);

		if(this._started){
			// this will notify any tablists to remove a button; do this first because it may affect sizing
			dojo.publish(this.id + "-removeChild", [page]);
		}

		// If we are being destroyed than don't run the code below (to select another page), because we are deleting
		// every page one by one
		if(this._beingDestroyed){ return; }

		// Select new page to display, also updating TabController to show the respective tab.
		// Do this before layout call because it can affect the height of the TabController.
		if(this.selectedChildWidget === page){
			this.selectedChildWidget = undefined;
			if(this._started){
				var children = this.getChildren();
				if(children.length){
					this.selectChild(children[0]);
				}
			}
		}

		if(this._started){
			// In case the tab titles now take up one line instead of two lines
			// (note though that ScrollingTabController never overflows to multiple lines),
			// or the height has changed slightly because of addition/removal of tab which close icon
			this.layout();
		}
	},

	selectChild: function(/*dijit._Widget|String*/ page, /*Boolean*/ animate){
		// summary:
		//		Show the given widget (which must be one of my children)
		// page:
		//		Reference to child widget or id of child widget

		page = dijit.byId(page);

		if(this.selectedChildWidget != page){
			// Deselect old page and select new one
			this._transition(page, this.selectedChildWidget, animate);
			this.selectedChildWidget = page;
			dojo.publish(this.id+"-selectChild", [page]);

			if(this.persist){
				dojo.cookie(this.id + "_selectedChild", this.selectedChildWidget.id);
			}
		}
	},

	_transition: function(/*dijit._Widget*/newWidget, /*dijit._Widget*/oldWidget){
		// summary:
		//		Hide the old widget and display the new widget.
		//		Subclasses should override this.
		// tags:
		//		protected extension
		if(oldWidget){
			this._hideChild(oldWidget);
		}
		this._showChild(newWidget);

		// Size the new widget, in case this is the first time it's being shown,
		// or I have been resized since the last time it was shown.
		// Note that page must be visible for resizing to work.
		if(newWidget.resize){
			if(this.doLayout){
				newWidget.resize(this._containerContentBox || this._contentBox);
			}else{
				// the child should pick it's own size but we still need to call resize()
				// (with no arguments) to let the widget lay itself out
				newWidget.resize();
			}
		}
	},

	_adjacent: function(/*Boolean*/ forward){
		// summary:
		//		Gets the next/previous child widget in this container from the current selection.
		var children = this.getChildren();
		var index = dojo.indexOf(children, this.selectedChildWidget);
		index += forward ? 1 : children.length - 1;
		return children[ index % children.length ]; // dijit._Widget
	},

	forward: function(){
		// summary:
		//		Advance to next page.
		this.selectChild(this._adjacent(true), true);
	},

	back: function(){
		// summary:
		//		Go back to previous page.
		this.selectChild(this._adjacent(false), true);
	},

	_onKeyPress: function(e){
		dojo.publish(this.id+"-containerKeyPress", [{ e: e, page: this}]);
	},

	layout: function(){
		// Implement _LayoutWidget.layout() virtual method.
		if(this.doLayout && this.selectedChildWidget && this.selectedChildWidget.resize){
			this.selectedChildWidget.resize(this._containerContentBox || this._contentBox);
		}
	},

	_showChild: function(/*dijit._Widget*/ page){
		// summary:
		//		Show the specified child by changing it's CSS, and call _onShow()/onShow() so
		//		it can do any updates it needs regarding loading href's etc.
		var children = this.getChildren();
		page.isFirstChild = (page == children[0]);
		page.isLastChild = (page == children[children.length-1]);
		page.selected = true;

		dojo.removeClass(page.domNode, "dijitHidden");
		dojo.addClass(page.domNode, "dijitVisible");

		page._onShow();
	},

	_hideChild: function(/*dijit._Widget*/ page){
		// summary:
		//		Hide the specified child by changing it's CSS, and call _onHide() so
		//		it's notified.
		page.selected=false;
		dojo.removeClass(page.domNode, "dijitVisible");
		dojo.addClass(page.domNode, "dijitHidden");

		page.onHide();
	},

	closeChild: function(/*dijit._Widget*/ page){
		// summary:
		//		Callback when user clicks the [X] to remove a page.
		//		If onClose() returns true then remove and destroy the child.
		// tags:
		//		private
		var remove = page.onClose(this, page);
		if(remove){
			this.removeChild(page);
			// makes sure we can clean up executeScripts in ContentPane onUnLoad
			page.destroyRecursive();
		}
	},

	destroyDescendants: function(/*Boolean*/preserveDom){
		dojo.forEach(this.getChildren(), function(child){
			this.removeChild(child);
			child.destroyRecursive(preserveDom);
		}, this);
	}
});

// For back-compat, remove for 2.0



// These arguments can be specified for the children of a StackContainer.
// Since any widget can be specified as a StackContainer child, mix them
// into the base widget class.  (This is a hack, but it's effective.)
dojo.extend(dijit._Widget, {
	// selected: Boolean
	//		Parameter for children of `dijit.layout.StackContainer` or subclasses.
	//		Specifies that this widget should be the initially displayed pane.
	//		Note: to change the selected child use `dijit.layout.StackContainer.selectChild`
	selected: false,

	// closable: Boolean
	//		Parameter for children of `dijit.layout.StackContainer` or subclasses.
	//		True if user can close (destroy) this child, such as (for example) clicking the X on the tab.
	closable: false,

	// iconClass: String
	//		Parameter for children of `dijit.layout.StackContainer` or subclasses.
	//		CSS Class specifying icon to use in label associated with this pane.
	iconClass: "",

	// showTitle: Boolean
	//		Parameter for children of `dijit.layout.StackContainer` or subclasses.
	//		When true, display title of this widget as tab label etc., rather than just using
	//		icon specified in iconClass
	showTitle: true
});

}

if(!dojo._hasResource["dijit.layout._TabContainerBase"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout._TabContainerBase"] = true;
dojo.provide("dijit.layout._TabContainerBase");




dojo.declare("dijit.layout._TabContainerBase",
	[dijit.layout.StackContainer, dijit._Templated],
	{
	// summary:
	//		Abstract base class for TabContainer.   Must define _makeController() to instantiate
	//		and return the widget that displays the tab labels
	// description:
	//		A TabContainer is a container that has multiple panes, but shows only
	//		one pane at a time.  There are a set of tabs corresponding to each pane,
	//		where each tab has the name (aka title) of the pane, and optionally a close button.

	// tabPosition: String
	//		Defines where tabs go relative to tab content.
	//		"top", "bottom", "left-h", "right-h"
	tabPosition: "top",

	baseClass: "dijitTabContainer",

	// tabStrip: Boolean
	//		Defines whether the tablist gets an extra class for layouting, putting a border/shading
	//		around the set of tabs.
	tabStrip: false,

	// nested: Boolean
	//		If true, use styling for a TabContainer nested inside another TabContainer.
	//		For tundra etc., makes tabs look like links, and hides the outer
	//		border since the outer TabContainer already has a border.
	nested: false,

	templateString: dojo.cache("dijit.layout", "templates/TabContainer.html", "<div class=\"dijitTabContainer\">\n\t<div class=\"dijitTabListWrapper\" dojoAttachPoint=\"tablistNode\"></div>\n\t<div dojoAttachPoint=\"tablistSpacer\" class=\"dijitTabSpacer ${baseClass}-spacer\"></div>\n\t<div class=\"dijitTabPaneWrapper ${baseClass}-container\" dojoAttachPoint=\"containerNode\"></div>\n</div>\n"),

	postMixInProperties: function(){
		// set class name according to tab position, ex: dijitTabContainerTop
		this.baseClass += this.tabPosition.charAt(0).toUpperCase() + this.tabPosition.substr(1).replace(/-.*/, "");

		this.srcNodeRef && dojo.style(this.srcNodeRef, "visibility", "hidden");

		this.inherited(arguments);
	},

	postCreate: function(){
		this.inherited(arguments);

		// Create the tab list that will have a tab (a.k.a. tab button) for each tab panel
		this.tablist = this._makeController(this.tablistNode);

		if(!this.doLayout){ dojo.addClass(this.domNode, "dijitTabContainerNoLayout"); }

		if(this.nested){
			/* workaround IE's lack of support for "a > b" selectors by
			 * tagging each node in the template.
			 */
			dojo.addClass(this.domNode, "dijitTabContainerNested");
			dojo.addClass(this.tablist.containerNode, "dijitTabContainerTabListNested");
			dojo.addClass(this.tablistSpacer, "dijitTabContainerSpacerNested");
			dojo.addClass(this.containerNode, "dijitTabPaneWrapperNested");
		}else{
			dojo.addClass(this.domNode, "tabStrip-" + (this.tabStrip ? "enabled" : "disabled"));
		}
	},

	_setupChild: function(/*dijit._Widget*/ tab){
		// Overrides StackContainer._setupChild().
		dojo.addClass(tab.domNode, "dijitTabPane");
		this.inherited(arguments);
	},

	startup: function(){
		if(this._started){ return; }

		// wire up the tablist and its tabs
		this.tablist.startup();

		this.inherited(arguments);
	},

	layout: function(){
		// Overrides StackContainer.layout().
		// Configure the content pane to take up all the space except for where the tabs are

		if(!this._contentBox || typeof(this._contentBox.l) == "undefined"){return;}

		var sc = this.selectedChildWidget;

		if(this.doLayout){
			// position and size the titles and the container node
			var titleAlign = this.tabPosition.replace(/-h/, "");
			this.tablist.layoutAlign = titleAlign;
			var children = [this.tablist, {
				domNode: this.tablistSpacer,
				layoutAlign: titleAlign
			}, {
				domNode: this.containerNode,
				layoutAlign: "client"
			}];
			dijit.layout.layoutChildren(this.domNode, this._contentBox, children);

			// Compute size to make each of my children.
			// children[2] is the margin-box size of this.containerNode, set by layoutChildren() call above
			this._containerContentBox = dijit.layout.marginBox2contentBox(this.containerNode, children[2]);

			if(sc && sc.resize){
				sc.resize(this._containerContentBox);
			}
		}else{
			// just layout the tab controller, so it can position left/right buttons etc.
			if(this.tablist.resize){
				this.tablist.resize({w: dojo.contentBox(this.domNode).w});
			}

			// and call resize() on the selected pane just to tell it that it's been made visible
			if(sc && sc.resize){
				sc.resize();
			}
		}
	},

	destroy: function(){
		if(this.tablist){
			this.tablist.destroy();
		}
		this.inherited(arguments);
	}
});


}

if(!dojo._hasResource["dijit.layout.TabController"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.TabController"] = true;
dojo.provide("dijit.layout.TabController");



// Menu is used for an accessible close button, would be nice to have a lighter-weight solution





dojo.declare("dijit.layout.TabController",
	dijit.layout.StackController,
{
	// summary:
	// 		Set of tabs (the things with titles and a close button, that you click to show a tab panel).
	//		Used internally by `dijit.layout.TabContainer`.
	// description:
	//		Lets the user select the currently shown pane in a TabContainer or StackContainer.
	//		TabController also monitors the TabContainer, and whenever a pane is
	//		added or deleted updates itself accordingly.
	// tags:
	//		private

	templateString: "<div wairole='tablist' dojoAttachEvent='onkeypress:onkeypress'></div>",

	// tabPosition: String
	//		Defines where tabs go relative to the content.
	//		"top", "bottom", "left-h", "right-h"
	tabPosition: "top",

	// buttonWidget: String
	//		The name of the tab widget to create to correspond to each page
	buttonWidget: "dijit.layout._TabButton",

	_rectifyRtlTabList: function(){
		// summary:
		//		For left/right TabContainer when page is RTL mode, rectify the width of all tabs to be equal, otherwise the tab widths are different in IE

		if(0 >= this.tabPosition.indexOf('-h')){ return; }
		if(!this.pane2button){ return; }

		var maxWidth = 0;
		for(var pane in this.pane2button){
			var ow = this.pane2button[pane].innerDiv.scrollWidth;
			maxWidth = Math.max(maxWidth, ow);
		}
		//unify the length of all the tabs
		for(pane in this.pane2button){
			this.pane2button[pane].innerDiv.style.width = maxWidth + 'px';
		}
	}
});

dojo.declare("dijit.layout._TabButton",
	dijit.layout._StackButton,
	{
	// summary:
	//		A tab (the thing you click to select a pane).
	// description:
	//		Contains the title of the pane, and optionally a close-button to destroy the pane.
	//		This is an internal widget and should not be instantiated directly.
	// tags:
	//		private

	// baseClass: String
	//		The CSS class applied to the domNode.
	baseClass: "dijitTab",

	// Apply dijitTabCloseButtonHover when close button is hovered
	cssStateNodes: {
		closeNode: "dijitTabCloseButton"
	},

	templateString: dojo.cache("dijit.layout", "templates/_TabButton.html", "<div waiRole=\"presentation\" dojoAttachPoint=\"titleNode\" dojoAttachEvent='onclick:onClick'>\n    <div waiRole=\"presentation\" class='dijitTabInnerDiv' dojoAttachPoint='innerDiv'>\n        <div waiRole=\"presentation\" class='dijitTabContent' dojoAttachPoint='tabContent'>\n        \t<div waiRole=\"presentation\" dojoAttachPoint='focusNode'>\n\t\t        <img src=\"${_blankGif}\" alt=\"\" class=\"dijitIcon\" dojoAttachPoint='iconNode' />\n\t\t        <span dojoAttachPoint='containerNode' class='tabLabel'></span>\n\t\t        <span class=\"dijitInline dijitTabCloseButton dijitTabCloseIcon\" dojoAttachPoint='closeNode'\n\t\t        \t\tdojoAttachEvent='onclick: onClickCloseButton' waiRole=\"presentation\">\n\t\t            <span dojoAttachPoint='closeText' class='dijitTabCloseText'>x</span\n\t\t        ></span>\n\t\t\t</div>\n        </div>\n    </div>\n</div>\n"),

	// Override _FormWidget.scrollOnFocus.
	// Don't scroll the whole tab container into view when the button is focused.
	scrollOnFocus: false,

	postMixInProperties: function(){
		// Override blank iconClass from Button to do tab height adjustment on IE6,
		// to make sure that tabs with and w/out close icons are same height
		if(!this.iconClass){
			this.iconClass = "dijitTabButtonIcon";
		}
	},

	postCreate: function(){
		this.inherited(arguments);
		dojo.setSelectable(this.containerNode, false);

		// If a custom icon class has not been set for the
		// tab icon, set its width to one pixel. This ensures
		// that the height styling of the tab is maintained,
		// as it is based on the height of the icon.
		// TODO: I still think we can just set dijitTabButtonIcon to 1px in CSS <Bill>
		if(this.iconNode.className == "dijitTabButtonIcon"){
			dojo.style(this.iconNode, "width", "1px");
		}
	},

	startup: function(){
		this.inherited(arguments);
		var n = this.domNode;

		// Required to give IE6 a kick, as it initially hides the
		// tabs until they are focused on.
		setTimeout(function(){
			n.className = n.className;
		}, 1);
	},

	_setCloseButtonAttr: function(disp){
		this.closeButton = disp;
		dojo.toggleClass(this.innerDiv, "dijitClosable", disp);
		this.closeNode.style.display = disp ? "" : "none";
		if(disp){
			var _nlsResources = dojo.i18n.getLocalization("dijit", "common");
			if(this.closeNode){
				dojo.attr(this.closeNode,"title", _nlsResources.itemClose);
			}
			// add context menu onto title button
			var _nlsResources = dojo.i18n.getLocalization("dijit", "common");
			this._closeMenu = new dijit.Menu({
				id: this.id+"_Menu",
				dir: this.dir,
				lang: this.lang,
				targetNodeIds: [this.domNode]
			});

			this._closeMenu.addChild(new dijit.MenuItem({
				label: _nlsResources.itemClose,
				dir: this.dir,
				lang: this.lang,
				onClick: dojo.hitch(this, "onClickCloseButton")
			}));
		}else{
			if(this._closeMenu){
				this._closeMenu.destroyRecursive();
				delete this._closeMenu;
			}
		}
	},
	_setLabelAttr: function(/*String*/ content){
		// summary:
		//		Hook for attr('label', ...) to work.
		// description:
		//		takes an HTML string.
		//		Inherited ToggleButton implementation will Set the label (text) of the button; 
		//		Need to set the alt attribute of icon on tab buttons if no label displayed
			this.inherited(arguments);
			if(this.showLabel == false && !this.params.title){
				this.iconNode.alt = dojo.trim(this.containerNode.innerText || this.containerNode.textContent || '');
			}
		},

	destroy: function(){
		if(this._closeMenu){
			this._closeMenu.destroyRecursive();
			delete this._closeMenu;
		}
		this.inherited(arguments);
	}
});

}

if(!dojo._hasResource["dijit.layout.ScrollingTabController"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.ScrollingTabController"] = true;
dojo.provide("dijit.layout.ScrollingTabController");




dojo.declare("dijit.layout.ScrollingTabController",
	dijit.layout.TabController,
	{
	// summary:
	//		Set of tabs with left/right arrow keys and a menu to switch between tabs not
	//		all fitting on a single row.
	//		Works only for horizontal tabs (either above or below the content, not to the left
	//		or right).
	// tags:
	//		private

	templateString: dojo.cache("dijit.layout", "templates/ScrollingTabController.html", "<div class=\"dijitTabListContainer-${tabPosition}\" style=\"visibility:hidden\">\n\t<div dojoType=\"dijit.layout._ScrollingTabControllerButton\"\n\t\t\tclass=\"tabStripButton-${tabPosition}\"\n\t\t\tid=\"${id}_menuBtn\" iconClass=\"dijitTabStripMenuIcon\"\n\t\t\tdojoAttachPoint=\"_menuBtn\" showLabel=false>&#9660;</div>\n\t<div dojoType=\"dijit.layout._ScrollingTabControllerButton\"\n\t\t\tclass=\"tabStripButton-${tabPosition}\"\n\t\t\tid=\"${id}_leftBtn\" iconClass=\"dijitTabStripSlideLeftIcon\"\n\t\t\tdojoAttachPoint=\"_leftBtn\" dojoAttachEvent=\"onClick: doSlideLeft\" showLabel=false>&#9664;</div>\n\t<div dojoType=\"dijit.layout._ScrollingTabControllerButton\"\n\t\t\tclass=\"tabStripButton-${tabPosition}\"\n\t\t\tid=\"${id}_rightBtn\" iconClass=\"dijitTabStripSlideRightIcon\"\n\t\t\tdojoAttachPoint=\"_rightBtn\" dojoAttachEvent=\"onClick: doSlideRight\" showLabel=false>&#9654;</div>\n\t<div class='dijitTabListWrapper' dojoAttachPoint='tablistWrapper'>\n\t\t<div wairole='tablist' dojoAttachEvent='onkeypress:onkeypress'\n\t\t\t\tdojoAttachPoint='containerNode' class='nowrapTabStrip'></div>\n\t</div>\n</div>\n"),

	// useMenu:[const] Boolean
	//		True if a menu should be used to select tabs when they are too
	//		wide to fit the TabContainer, false otherwise.
	useMenu: true,

	// useSlider: [const] Boolean
	//		True if a slider should be used to select tabs when they are too
	//		wide to fit the TabContainer, false otherwise.
	useSlider: true,

	// tabStripClass: String
	//		The css class to apply to the tab strip, if it is visible.
	tabStripClass: "",

	widgetsInTemplate: true,

	// _minScroll: Number
	//		The distance in pixels from the edge of the tab strip which,
	//		if a scroll animation is less than, forces the scroll to
	//		go all the way to the left/right.
	_minScroll: 5,

	attributeMap: dojo.delegate(dijit._Widget.prototype.attributeMap, {
		"class": "containerNode"
	}),

	postCreate: function(){
		this.inherited(arguments);
		var n = this.domNode;

		this.scrollNode = this.tablistWrapper;
		this._initButtons();

		if(!this.tabStripClass){
			this.tabStripClass = "dijitTabContainer" +
				this.tabPosition.charAt(0).toUpperCase() +
				this.tabPosition.substr(1).replace(/-.*/, "") +
				"None";
			dojo.addClass(n, "tabStrip-disabled")
		}

		dojo.addClass(this.tablistWrapper, this.tabStripClass);
	},

	onStartup: function(){
		this.inherited(arguments);

		// Do not show the TabController until the related
		// StackController has added it's children.  This gives
		// a less visually jumpy instantiation.
		dojo.style(this.domNode, "visibility", "visible");
		this._postStartup = true;
	},

	onAddChild: function(page, insertIndex){
		this.inherited(arguments);
		var menuItem;
		if(this.useMenu){
			var containerId = this.containerId;
			menuItem = new dijit.MenuItem({
				id: page.id + "_stcMi",
				label: page.title,
				dir: page.dir,
				lang: page.lang,
				onClick: dojo.hitch(this, function(){
					var container = dijit.byId(containerId);
					container.selectChild(page);
				})
			});
			this._menuChildren[page.id] = menuItem;
			this._menu.addChild(menuItem, insertIndex);
		}

		// update the menuItem label when the button label is updated
		this.pane2handles[page.id].push(
			this.connect(this.pane2button[page.id], "set", function(name, value){
				if(this._postStartup){
					if(name == "label"){
						if(menuItem){
							menuItem.set(name, value);
						}
	
						// The changed label will have changed the width of the
						// buttons, so do a resize
						if(this._dim){
							this.resize(this._dim);
						}
					}
				}
			})
		);

		// Increment the width of the wrapper when a tab is added
		// This makes sure that the buttons never wrap.
		// The value 200 is chosen as it should be bigger than most
		// Tab button widths.
		dojo.style(this.containerNode, "width",
			(dojo.style(this.containerNode, "width") + 200) + "px");
	},

	onRemoveChild: function(page, insertIndex){
		// null out _selectedTab because we are about to delete that dom node
		var button = this.pane2button[page.id];
		if(this._selectedTab === button.domNode){
			this._selectedTab = null;
		}

		// delete menu entry corresponding to pane that was removed from TabContainer
		if(this.useMenu && page && page.id && this._menuChildren[page.id]){
			this._menu.removeChild(this._menuChildren[page.id]);
			this._menuChildren[page.id].destroy();
			delete this._menuChildren[page.id];
		}

		this.inherited(arguments);
	},

	_initButtons: function(){
		// summary:
		//		Creates the buttons used to scroll to view tabs that
		//		may not be visible if the TabContainer is too narrow.
		this._menuChildren = {};

		// Make a list of the buttons to display when the tab labels become
		// wider than the TabContainer, and hide the other buttons.
		// Also gets the total width of the displayed buttons.
		this._btnWidth = 0;
		this._buttons = dojo.query("> .tabStripButton", this.domNode).filter(function(btn){
			if((this.useMenu && btn == this._menuBtn.domNode) ||
				(this.useSlider && (btn == this._rightBtn.domNode || btn == this._leftBtn.domNode))){
				this._btnWidth += dojo.marginBox(btn).w;
				return true;
			}else{
				dojo.style(btn, "display", "none");
				return false;
			}
		}, this);

		if(this.useMenu){
			// Create the menu that is used to select tabs.
			this._menu = new dijit.Menu({
				id: this.id + "_menu",
				dir: this.dir,
				lang: this.lang,
				targetNodeIds: [this._menuBtn.domNode],
				leftClickToOpen: true,
				refocus: false	// selecting a menu item sets focus to a TabButton
			});
			this._supportingWidgets.push(this._menu);
		}
	},

	_getTabsWidth: function(){
		var children = this.getChildren();
		if(children.length){
			var leftTab = children[this.isLeftToRight() ? 0 : children.length - 1].domNode,
				rightTab = children[this.isLeftToRight() ? children.length - 1 : 0].domNode;
			return rightTab.offsetLeft + dojo.style(rightTab, "width") - leftTab.offsetLeft;
		}else{
			return 0;
		}
	},

	_enableBtn: function(width){
		// summary:
		//		Determines if the tabs are wider than the width of the TabContainer, and
		//		thus that we need to display left/right/menu navigation buttons.
		var tabsWidth = this._getTabsWidth();
		width = width || dojo.style(this.scrollNode, "width");
		return tabsWidth > 0 && width < tabsWidth;
	},

	resize: function(dim){
		// summary:
		//		Hides or displays the buttons used to scroll the tab list and launch the menu
		//		that selects tabs.

		if(this.domNode.offsetWidth == 0){
			return;
		}

		// Save the dimensions to be used when a child is renamed.
		this._dim = dim;

		// Set my height to be my natural height (tall enough for one row of tab labels),
		// and my content-box width based on margin-box width specified in dim parameter.
		// But first reset scrollNode.height in case it was set by layoutChildren() call
		// in a previous run of this method.
		this.scrollNode.style.height = "auto";
		this._contentBox = dijit.layout.marginBox2contentBox(this.domNode, {h: 0, w: dim.w});
		this._contentBox.h = this.scrollNode.offsetHeight;
		dojo.contentBox(this.domNode, this._contentBox);

		// Show/hide the left/right/menu navigation buttons depending on whether or not they
		// are needed.
		var enable = this._enableBtn(this._contentBox.w);
		this._buttons.style("display", enable ? "" : "none");

		// Position and size the navigation buttons and the tablist
		this._leftBtn.layoutAlign = "left";
		this._rightBtn.layoutAlign = "right";
		this._menuBtn.layoutAlign = this.isLeftToRight() ? "right" : "left";
		dijit.layout.layoutChildren(this.domNode, this._contentBox,
			[this._menuBtn, this._leftBtn, this._rightBtn, {domNode: this.scrollNode, layoutAlign: "client"}]);

		// set proper scroll so that selected tab is visible
		if(this._selectedTab){
			if(this._anim && this._anim.status() == "playing"){
				this._anim.stop();
			}
			var w = this.scrollNode,
				sl = this._convertToScrollLeft(this._getScrollForSelectedTab());
			w.scrollLeft = sl;
		}

		// Enable/disabled left right buttons depending on whether or not user can scroll to left or right
		this._setButtonClass(this._getScroll());
		
		this._postResize = true;
	},

	_getScroll: function(){
		// summary:
		//		Returns the current scroll of the tabs where 0 means
		//		"scrolled all the way to the left" and some positive number, based on #
		//		of pixels of possible scroll (ex: 1000) means "scrolled all the way to the right"
		var sl = (this.isLeftToRight() || dojo.isIE < 8 || (dojo.isIE && dojo.isQuirks) || dojo.isWebKit) ? this.scrollNode.scrollLeft :
				dojo.style(this.containerNode, "width") - dojo.style(this.scrollNode, "width")
					 + (dojo.isIE == 8 ? -1 : 1) * this.scrollNode.scrollLeft;
		return sl;
	},

	_convertToScrollLeft: function(val){
		// summary:
		//		Given a scroll value where 0 means "scrolled all the way to the left"
		//		and some positive number, based on # of pixels of possible scroll (ex: 1000)
		//		means "scrolled all the way to the right", return value to set this.scrollNode.scrollLeft
		//		to achieve that scroll.
		//
		//		This method is to adjust for RTL funniness in various browsers and versions.
		if(this.isLeftToRight() || dojo.isIE < 8 || (dojo.isIE && dojo.isQuirks) || dojo.isWebKit){
			return val;
		}else{
			var maxScroll = dojo.style(this.containerNode, "width") - dojo.style(this.scrollNode, "width");
			return (dojo.isIE == 8 ? -1 : 1) * (val - maxScroll);
		}
	},

	onSelectChild: function(/*dijit._Widget*/ page){
		// summary:
		//		Smoothly scrolls to a tab when it is selected.

		var tab = this.pane2button[page.id];
		if(!tab || !page){return;}

		// Scroll to the selected tab, except on startup, when scrolling is handled in resize()
		var node = tab.domNode;
		if(this._postResize && node != this._selectedTab){
			this._selectedTab = node;

			var sl = this._getScroll();

			if(sl > node.offsetLeft ||
					sl + dojo.style(this.scrollNode, "width") <
					node.offsetLeft + dojo.style(node, "width")){
				this.createSmoothScroll().play();
			}
		}

		this.inherited(arguments);
	},

	_getScrollBounds: function(){
		// summary:
		//		Returns the minimum and maximum scroll setting to show the leftmost and rightmost
		//		tabs (respectively)
		var children = this.getChildren(),
			scrollNodeWidth = dojo.style(this.scrollNode, "width"),		// about 500px
			containerWidth = dojo.style(this.containerNode, "width"),	// 50,000px
			maxPossibleScroll = containerWidth - scrollNodeWidth,	// scrolling until right edge of containerNode visible
			tabsWidth = this._getTabsWidth();

		if(children.length && tabsWidth > scrollNodeWidth){
			// Scrolling should happen
			return {
				min: this.isLeftToRight() ? 0 : children[children.length-1].domNode.offsetLeft,
				max: this.isLeftToRight() ?
					(children[children.length-1].domNode.offsetLeft + dojo.style(children[children.length-1].domNode, "width")) - scrollNodeWidth :
					maxPossibleScroll
			};
		}else{
			// No scrolling needed, all tabs visible, we stay either scrolled to far left or far right (depending on dir)
			var onlyScrollPosition = this.isLeftToRight() ? 0 : maxPossibleScroll;
			return {
				min: onlyScrollPosition,
				max: onlyScrollPosition
			};
		}
	},

	_getScrollForSelectedTab: function(){
		// summary:
		//		Returns the scroll value setting so that the selected tab
		//		will appear in the center
		var w = this.scrollNode,
			n = this._selectedTab,
			scrollNodeWidth = dojo.style(this.scrollNode, "width"),
			scrollBounds = this._getScrollBounds();

		// TODO: scroll minimal amount (to either right or left) so that
		// selected tab is fully visible, and just return if it's already visible?
		var pos = (n.offsetLeft + dojo.style(n, "width")/2) - scrollNodeWidth/2;
		pos = Math.min(Math.max(pos, scrollBounds.min), scrollBounds.max);

		// TODO:
		// If scrolling close to the left side or right side, scroll
		// all the way to the left or right.  See this._minScroll.
		// (But need to make sure that doesn't scroll the tab out of view...)
		return pos;
	},

	createSmoothScroll : function(x){
		// summary:
		//		Creates a dojo._Animation object that smoothly scrolls the tab list
		//		either to a fixed horizontal pixel value, or to the selected tab.
		// description:
		//		If an number argument is passed to the function, that horizontal
		//		pixel position is scrolled to.  Otherwise the currently selected
		//		tab is scrolled to.
		// x: Integer?
		//		An optional pixel value to scroll to, indicating distance from left.

		// Calculate position to scroll to
		if(arguments.length > 0){
			// position specified by caller, just make sure it's within bounds
			var scrollBounds = this._getScrollBounds();
			x = Math.min(Math.max(x, scrollBounds.min), scrollBounds.max);
		}else{
			// scroll to center the current tab
			x = this._getScrollForSelectedTab();
		}

		if(this._anim && this._anim.status() == "playing"){
			this._anim.stop();
		}

		var self = this,
			w = this.scrollNode,
			anim = new dojo._Animation({
				beforeBegin: function(){
					if(this.curve){ delete this.curve; }
					var oldS = w.scrollLeft,
						newS = self._convertToScrollLeft(x);
					anim.curve = new dojo._Line(oldS, newS);
				},
				onAnimate: function(val){
					w.scrollLeft = val;
				}
			});
		this._anim = anim;

		// Disable/enable left/right buttons according to new scroll position
		this._setButtonClass(x);

		return anim; // dojo._Animation
	},

	_getBtnNode: function(e){
		// summary:
		//		Gets a button DOM node from a mouse click event.
		// e:
		//		The mouse click event.
		var n = e.target;
		while(n && !dojo.hasClass(n, "tabStripButton")){
			n = n.parentNode;
		}
		return n;
	},

	doSlideRight: function(e){
		// summary:
		//		Scrolls the menu to the right.
		// e:
		//		The mouse click event.
		this.doSlide(1, this._getBtnNode(e));
	},

	doSlideLeft: function(e){
		// summary:
		//		Scrolls the menu to the left.
		// e:
		//		The mouse click event.
		this.doSlide(-1,this._getBtnNode(e));
	},

	doSlide: function(direction, node){
		// summary:
		//		Scrolls the tab list to the left or right by 75% of the widget width.
		// direction:
		//		If the direction is 1, the widget scrolls to the right, if it is
		//		-1, it scrolls to the left.

		if(node && dojo.hasClass(node, "dijitTabDisabled")){return;}

		var sWidth = dojo.style(this.scrollNode, "width");
		var d = (sWidth * 0.75) * direction;

		var to = this._getScroll() + d;

		this._setButtonClass(to);

		this.createSmoothScroll(to).play();
	},

	_setButtonClass: function(scroll){
		// summary:
		//		Disables the left scroll button if the tabs are scrolled all the way to the left,
		//		or the right scroll button in the opposite case.
		// scroll: Integer
		//		amount of horizontal scroll

		var scrollBounds = this._getScrollBounds();
		this._leftBtn.set("disabled", scroll <= scrollBounds.min);
		this._rightBtn.set("disabled", scroll >= scrollBounds.max);
	}
});

dojo.declare("dijit.layout._ScrollingTabControllerButton",
	dijit.form.Button,
	{
		baseClass: "dijitTab tabStripButton",

		templateString: dojo.cache("dijit.layout", "templates/_ScrollingTabControllerButton.html", "<div dojoAttachEvent=\"onclick:_onButtonClick\">\n\t<div waiRole=\"presentation\" class=\"dijitTabInnerDiv\" dojoattachpoint=\"innerDiv,focusNode\">\n\t\t<div waiRole=\"presentation\" class=\"dijitTabContent dijitButtonContents\" dojoattachpoint=\"tabContent\">\n\t\t\t<img waiRole=\"presentation\" alt=\"\" src=\"${_blankGif}\" class=\"dijitTabStripIcon\" dojoAttachPoint=\"iconNode\"/>\n\t\t\t<span dojoAttachPoint=\"containerNode,titleNode\" class=\"dijitButtonText\"></span>\n\t\t</div>\n\t</div>\n</div>\n"),

		// Override inherited tabIndex: 0 from dijit.form.Button, because user shouldn't be
		// able to tab to the left/right/menu buttons
		tabIndex: "-1"
	}
);

}

if(!dojo._hasResource["dijit.layout.TabContainer"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["dijit.layout.TabContainer"] = true;
dojo.provide("dijit.layout.TabContainer");





dojo.declare("dijit.layout.TabContainer",
	dijit.layout._TabContainerBase,
	{
		// summary:
		//		A Container with tabs to select each child (only one of which is displayed at a time).
		// description:
		//		A TabContainer is a container that has multiple panes, but shows only
		//		one pane at a time.  There are a set of tabs corresponding to each pane,
		//		where each tab has the name (aka title) of the pane, and optionally a close button.

		// useMenu: [const] Boolean
		//		True if a menu should be used to select tabs when they are too
		//		wide to fit the TabContainer, false otherwise.
		useMenu: true,

		// useSlider: [const] Boolean
		//		True if a slider should be used to select tabs when they are too
		//		wide to fit the TabContainer, false otherwise.
		useSlider: true,

		// controllerWidget: String
		//		An optional parameter to override the widget used to display the tab labels
		controllerWidget: "",

		_makeController: function(/*DomNode*/ srcNode){
			// summary:
			//		Instantiate tablist controller widget and return reference to it.
			//		Callback from _TabContainerBase.postCreate().
			// tags:
			//		protected extension

			var cls = this.baseClass + "-tabs" + (this.doLayout ? "" : " dijitTabNoLayout"),
				TabController = dojo.getObject(this.controllerWidget);

			return new TabController({
				id: this.id + "_tablist",
				dir: this.dir,
				lang: this.lang,
				tabPosition: this.tabPosition,
				doLayout: this.doLayout,
				containerId: this.id,
				"class": cls,
				nested: this.nested,
				useMenu: this.useMenu,
				useSlider: this.useSlider,
				tabStripClass: this.tabStrip ? this.baseClass + (this.tabStrip ? "":"No") + "Strip": null
			}, srcNode);
		},

		postMixInProperties: function(){
			this.inherited(arguments);

			// Scrolling controller only works for horizontal non-nested tabs
			if(!this.controllerWidget){
				this.controllerWidget = (this.tabPosition == "top" || this.tabPosition == "bottom") && !this.nested ?
							"dijit.layout.ScrollingTabController" : "dijit.layout.TabController";
			}
		}
});


}

if(!dojo._hasResource["itme.finder.Finder"]){ //_hasResource checks added by build. Do not use _hasResource directly in your code.
dojo._hasResource["itme.finder.Finder"] = true;
dojo.provide("itme.finder.Finder");













dojo.declare("itme.finder.Finder", null, {
  MRI: null,
	_searchTextHintString: null, 
    _finderIdPrefix: null,
    _contextFinderMenuText: null,
    _tabbedContainer: null,
	// jsp input fields references, for communication between finder and bean    
  	_selectedTaskInput: null,
    _selectedTaskButton: null,  
    _selectedChassisListInput: null,   
    _selectedChassisMapListInput: null, 
    _selectedChassisTextInput: null,  
    _homeParamsForDataSet: null,     
    // references to finder elements
    _finderDomObj: null, 				//finder main dom container
  	_searchDetailDlg: null,  			//finder results tooltip dialog
  	
  	// finder state variables
  	_searchDlgIsOpen: false,
  	_searchDlgInitialized: false,
    _latestSearchEntry: null,
    _currentView: '0', 
    _refreshImage: null,
    
  	//Data stores containing cached lists
  	_taskDataStore: null,
    _resourceDataStore: null,  
    _groupDataStore: null,  
    _contextTaskDataStore: null,   
    
    
    constructor: function(domId, finderIdPrefix){  	
      
      this.MRI = dojo.i18n.getLocalization("itme", "Resources");
      
   		itmeUtils.initializeTimer("finderTimer");   		
    	this._finderIdPrefix = finderIdPrefix;
    	this.initializeFieldReferences(domId);
    	if(this._finderDomObj) {} else {return;}
    	if(finderIdPrefix == 'finder') {
    		this._searchTextHintString = this.MRI.FinderInitialText;  
    	} else {
    		var prefixString = this.MRI.ContextFinderInitialText;
    		var menuString = this.MRI.ContextFinderTaskMenuFor;
    		var endString = '';
    		var selected;
    		if(isChassisMapRendered()) {
    			selected = this._selectedChassisMapListInput.innerHTML.split(','); 
    		} else {   	
    			selected = this._selectedChassisListInput.innerHTML.split(',');	
    		}
    		if(selected.length > 0) {
    			if(selected.length == 1) {
    				endString = this._selectedChassisTextInput.innerHTML;
    				menuString = menuString + this._selectedChassisTextInput.innerHTML;
    			} else {
    				var resourceType = this.MRI.Chassis;
    				if(isChassisMapRendered()) {
    					resourceType = this.MRI.Resources;
    				}
    				endString = selected.length + " " + resourceType;
    				menuString = menuString + selected.length + " " + resourceType;
    			}
    			this._searchTextHintString = prefixString + endString;      
    			this._contextFinderMenuText = menuString;
    		}	
    		
    	}
    	this.initializeFinderHtml();
    	if(finderIdPrefix == 'finder') {
   			this.initializeDataStores(domId); 	
    	} 
    	setTimeout(dojo.hitch(this, 'incrementRefreshMinute'), 60000);
  	},
  	incrementRefreshMinute: function() {
  	    this._selectedChassisMapListInput.title = parseInt(this._selectedChassisMapListInput.title) + 1;
  		if(this._refreshImage != null) {
  			this.changeRefreshCount();
  		}  		
    	setTimeout(dojo.hitch(this, 'incrementRefreshMinute'), 60000);
  	},
  	changeRefreshCount: function() {
  		var count = parseInt(this._selectedChassisMapListInput.title);
  		if (count < 60) {
	  		console.info(count + "," + this.MRI.RefreshCache + "," + this.MRI.Minutes);
  			this._refreshImage.title = dojo.replace(this.MRI.RefreshCache, [count, this.MRI.Minutes]);  			
	  		console.info(count);
  		} else if (count < 1440) {
  			var hours = Math.floor(count/60);
  			this._refreshImage.title = dojo.replace(this.MRI.RefreshCache, [hours, this.MRI.Hours]);   			
  		} else {
  		    var days = Math.floor(count/1440);  			
  			this._refreshImage.title = dojo.replace(this.MRI.RefreshCache, [days, this.MRI.Days]);    		
  		}
  	},
  	initializeDataStores: function(domId) {  	
    	this._taskDataStore = new itme.finder.TaskDataStore('tasks', domId);   
    	this._resourceDataStore = new itme.finder.ResourceDataStore('resources', domId); 
    	this._groupDataStore = new itme.finder.GroupDataStore('groups', domId); 
  	},
  	initializeFieldReferences: function(domId) {  	
    	this._selectedTaskInput = dojo.byId(domId + ':finderSelected');
        this._selectedTaskButton = dojo.byId(domId + ':launchButton');  
        this._selectedChassisListInput = dojo.byId(domId + ':selectedChassisList');  
        this._selectedChassisTextInput = dojo.byId(domId + ':selectedChassisText');  
        this._selectedChassisMapListInput = dojo.byId(domId + ':selectedChassisMapOidsCSVReadOnly');  
  		this._finderDomObj = dojo.byId(this._finderIdPrefix); 
  		var homeParmsValue = dojo.byId(domId + ':chassisMapHomeParms2').innerHTML;		
  		this._homeParamsForDataSet =  Math.floor(Math.random()*50000) + '#' + homeParmsValue;
  		
  	},
  	initializeFinderHtml: function() {  
  		dojo.empty(this._finderDomObj);	 
  		if(dijit.byId(this._finderIdPrefix + "searchBox")) {
  			dijit.byId(this._finderIdPrefix + "searchBox").destroy();
  		}
  		if(dijit.byId(this._finderIdPrefix + "dialog")) {
  			dijit.byId(this._finderIdPrefix + "dialog").destroy();
  		}
  		var tooltipHelp = this.MRI.FinderTooltipHelp;
        if(this._finderIdPrefix == 'contextFinder') {
            tooltipHelp = this.MRI.ContextFinderTooltipHelp;
        }   
    	var innerDiv = dojo.create("div", {title: tooltipHelp}, this._finderDomObj);
    	var finderInputFieldDomObj = dojo.create("input", {}, innerDiv);  
       	this._searchDetailDlg = new dijit.TooltipDialog({id:this._finderIdPrefix + "dialog", style:"width:450px;height:150px;"});
    	this._searchDetailDlg.startup(); 
    	
  		     
  		var urlString = contextPath + '/images/ac16_search.png';
        phxTrace(urlString);
  		var textBoxxerroo = new dijit.form.TextBox({
    		value:this._searchTextHintString, 
    		style:"padding-top:3px; width:160px; padding-left:20px;", 
    		id:this._finderIdPrefix + "searchBox", 
    		title: tooltipHelp,
    		
    		onKeyUp:dojo.hitch(this, this.finderKeyPressed), 
    		onBlur:dojo.hitch(this, this.doSearchBoxBlur), 
    		onFocus:dojo.hitch(this, this.doSearchBoxFocus)}, finderInputFieldDomObj);  
    	  	
  		textBoxxerroo.attr("class", "finderSearchBox");
  	},
    fetchFailed: function(error, request) {
                phxTrace("lookup failed.");
                phxTrace(error);
    }, 
	doSearchBoxBlur: function() {
		if (dijit.byId(this._finderIdPrefix + "searchBox").attr("value") == "") {
			dijit.byId(this._finderIdPrefix + "searchBox").attr("value", this._searchTextHintString);
  		} 
	},
	doSearchBoxFocus: function() {
		if (dijit.byId(this._finderIdPrefix + "searchBox").attr("value") == this._searchTextHintString) {
			dijit.byId(this._finderIdPrefix + "searchBox").attr("value", "");
  		} 
  		dijit.byId(this._finderIdPrefix + "searchBox").focus();
	},	
	refreshCache: function() {
   		this._selectedTaskInput.value = "REFRESH_CACHE";
		this._selectedTaskButton.click();	
	
	},
	closeSearchDetailDlg: function() {	    
   		if (this._searchDetailDlg) { dijit.popup.close(this._searchDetailDlg); }
// 		dijit.byId(this._finderIdPrefix + "searchBox").attr("value", this._searchTextHintString)
    	this._searchDlgIsOpen = false;
    	this._latestSearchEntry = '';
//    	var obj = dijit.byId(this._finderIdPrefix + "searchBox").domNode;
//    	obj.blur();
	},	
	doTaskSearch: function(myitems, myrequest) {		
    	this.doSearch(myitems, myrequest, this._taskDataStore, 'special#_com.ibm.usmi.console.taskmap', this.MRI.MoreTasks, 'tasks', false);
 	},
 	doContextTaskSearch: function(myitems, myrequest) {		
    	this.doSearch(myitems, myrequest, this._contextTaskDataStore, 'special#_com.ibm.usmi.console.taskmap', this.MRI.MoreTasks, 'contextTasks', false);
 	},
	doResourceSearch: function(myitems, myrequest) {		
    	this.doSearch(myitems, myrequest, this._resourceDataStore, 'special#_com.ibm.usmi.console.tasks.QuickFindTask', this.MRI.MoreResources, 'resources', false);
	},	
	doGroupSearch: function(myitems, myrequest) {		
    	this.doSearch(myitems, myrequest, this._groupDataStore, 'special#_none', this.MRI.MoreGroups, 'groups', true);
	},		
	doSearch: function(myitems, myrequest, dataStore, moreItemsTask, moreItemsText, typeOfSearch, moreItemsNotClickable) {		
    	phxTrace('searchCompleted: ' + myitems.length + ' results');      	
  		var searchString = dijit.byId(this._finderIdPrefix + "searchBox").attr("value");
 		if (searchString == "" || searchString == this._searchTextHintString) return; 
 		if(myitems) {
 			if(myitems.length > 0) {
 				dataStore.clearSelection(); 				
 				itmeUtils.traceTimeDelta("finderTimer", "processing results", true); 
 				dataStore.populateCategoriesWithSelectedItems(myitems);
 				itmeUtils.traceTimeDelta("finderTimer", "results populated", true); 
  				this.initializeDialogConstruct(typeOfSearch, myitems.length);    	      	   	
  				itmeUtils.traceTimeDelta("finderTimer", "initialized dialog", true); 	
 				dataStore.createCategoryHTMLElements(myitems.length, this, moreItemsTask, moreItemsText, moreItemsNotClickable);
 				itmeUtils.traceTimeDelta("finderTimer", "results UI created", true);  
 			} else {
 				this.initializeDialogConstruct(typeOfSearch);  			
				dojo.create("div", {innerHTML:this.MRI.NoMatchesFound}, dojo.byId(this._finderIdPrefix + typeOfSearch + "_containerDiv")); 
 			}	 	
  			//dojo.byId(this._finderIdPrefix + typeOfSearch + "Menu_count").innerHTML = " (" +myitems.length + ")";		
 		}
 		if(typeOfSearch == 'tasks') { 	//Last Tab kicks off the render logic	
        	this._tabbedContainer.startup();	
 			var selectedTab = this._tabbedContainer.selectedChildWidget
 			dojo.removeClass(selectedTab.domNode, "dijitHidden");
			dojo.addClass(selectedTab.domNode, "dijitVisible");
 		}
 		
	},		
		
	doClick: function(event) {		
		var selectedTaskId = event.currentTarget.id;				
		this.closeSearchDetailDlg();
		phxTrace('doclick' + selectedTaskId);
		var type = selectedTaskId.split('#_')[0];
		var itemId = selectedTaskId.split('#_')[1];
		if(type.indexOf('categorySection') != -1) {
			var category = this._resourceDataStore.getCategories()[itemId];
			var ids = category.getIds();
			selectedTaskId = 'resourceList#_';
			var i;
			for (i = 0; i < ids.length; i++) {
				if(i != 0) {
					selectedTaskId = selectedTaskId + ",";
				}
				selectedTaskId = selectedTaskId + ids[i];
			}
		} else if(type == 'resources') {
			if(dojo.byId('ChassisMap')) {
				
			}
			dojo.cookie("chasisMap.currentSelection", itemId, { expires: 1});
		} 
		
		this._selectedTaskInput.value = selectedTaskId + "#_" + dijit.byId(this._finderIdPrefix + "searchBox").attr("value");
		this._selectedTaskButton.click();	
	},
  	finderKeyPressed: function(event){
  		phxTrace('keyPressed');
    	if (event.keyCode == 27) {
     		dijit.byId(this._finderIdPrefix + "searchBox").attr("value", "");
  		}  		
  		var searchStringEntered = dijit.byId(this._finderIdPrefix + "searchBox").attr("value");
  		if(this._latestSearchEntry == searchStringEntered) {
  			return;
  		} else {
  			this._latestSearchEntry = searchStringEntered;
  		}
  		if ((searchStringEntered == "") || (searchStringEntered == this._searchTextHintString)) {   //|| searchStringEntered.length == 1
       		this.closeSearchDetailDlg();
    		return;
  		}   	
  		if (!this._searchDlgIsOpen) {
       		dijit.popup.open({    popup: this._searchDetailDlg,
	                      around: dijit.byId(this._finderIdPrefix + "searchBox").domNode, 
                          onCancel: dojo.hitch(this, this.closeSearchDetailDlg),
                          parent: dijit.byId(this._finderIdPrefix + "searchBox")}); 
        }  
		this._searchDlgIsOpen = true;
        dijit.byId(this._finderIdPrefix + "searchBox").focus();
		var searchTerm = searchStringEntered;  
  		phxTrace('query: ' + searchTerm); 
  		this._searchDlgInitialized = false;
  		if(this._finderIdPrefix == 'contextFinder') {
  			
  		    dojo.destroy('searchingText');
  			this.initiateContextSearch(searchTerm, false, 0);
  		} else {
  			this._resourceDataStore.initiateTaskSearch(searchTerm, this); 	
  			this._groupDataStore.initiateTaskSearch(searchTerm, this);  	
  			this._taskDataStore.initiateTaskSearch(searchTerm, this);   		
  		}
  		
  		
 	},
 	initiateContextSearch: function(searchTerm, isTextCreated, state) {
 		var stateNumber = state % 3;
 		if( this._contextTaskDataStore) { 
 			if(isTextCreated) {
  				dojo.destroy('searchingText'); 
  			}	
  			this._contextTaskDataStore.initiateTaskSearch(searchTerm, this); 
  		} else {
  			if(!isTextCreated) {
  				dojo.create("div", {id:'searchingText', innerHTML:this.MRI['Searching' + stateNumber]}, this._searchDetailDlg.containerNode); 
  			} else {
  				dojo.byId('searchingText').innerHTML= this.MRI['Searching' + stateNumber];
  			}
  			setTimeout(dojo.hitch(this, 'initiateContextSearch', searchTerm, true, state + 1), 200);
  		} 
 	},
 	initializeDialogConstruct: function(typeOfSearch, items) {
 		
 		if(this._searchDlgInitialized == false) {
 			this._searchDlgInitialized = true;
 			dojo.empty(this._searchDetailDlg.containerNode);   
  			dojo.create("img",{onclick:dojo.hitch(this, this.closeSearchDetailDlg),
   		   					src:contextPath + "/images/title_close.gif", 
   		   					title:this.MRI.Close, 
   		   					className:"closeImg"}, this._searchDetailDlg.containerNode); 
   		   	this._refreshImage = dojo.create("img",{onclick:dojo.hitch(this, this.refreshCache),
   		   					src:contextPath + "/images/hardwareView/ac16_refresh.png",    		   					
   		   					className:"refreshImg"}, this._searchDetailDlg.containerNode); 
   		   	this.changeRefreshCount();
 			var header = dojo.create("span", {id: this._finderIdPrefix + "finderMenu", 
 									          style:"display:inline;"},  this._searchDetailDlg.containerNode);        	
        	if(typeOfSearch == 'contextTasks') {
        		var span1 = dojo.create("span", {id: this._finderIdPrefix + 'contextTasks' + "Menu", 
        										 className:'finderMenuItemSelected', 
        										 innerHTML:this._contextFinderMenuText},  header);
       	 		var div = dojo.create("div", {id: this._finderIdPrefix + 'contextTasks' + "_containerDiv"},  this._searchDetailDlg.containerNode);
        	} else {
        		var mainDivTabContainer = dojo.create("div", {id:'mainDivTabContainer'}, this._searchDetailDlg.containerNode);
 				if(dijit.byId(this._finderIdPrefix + "tab")) {
  					dijit.byId(this._finderIdPrefix + "tab").destroy();
  				}    		
  				this._tabbedContainer = new dijit.layout.TabContainer({id:this._finderIdPrefix + "tab", style:"width:100%;", doLayout:true, useMenu:false, useSlider:false}, mainDivTabContainer);
 				var lastSelectedTab = dojo.cookie("finder.currentView");     
 				if(lastSelectedTab != null && lastSelectedTab.length > 0) {
 					this._currentView = lastSelectedTab;
 				}   		
 				this._tabbedContainer.addChild(this.initializeMenu('resources', this.MRI.FinderResourceMenu, header, this._tabbedContainer, (this._currentView == 'resources')));
        		this._tabbedContainer.addChild(this.initializeMenu('groups', this.MRI.FinderGroupMenu, header, this._tabbedContainer, (this._currentView == 'groups')));
        		this._tabbedContainer.addChild(this.initializeMenu('tasks', this.MRI.FinderTaskMenu , header, this._tabbedContainer, (this._currentView == 'tasks')));
        		dojo.connect(this._tabbedContainer,"selectChild", dojo.hitch(this, this.tabSelected));        		
	    		
        	}
 		}     
 		//add the count in parenthesis to the tabs
 		var numItems;
 		if(items) numItems = items;
 		else numItems = 0;
 		if(this._tabbedContainer) {
        	var children = this._tabbedContainer.getChildren();
        	var idx = 0;
        	if(typeOfSearch == 'groups') {
        		idx = 1;
        	} else if(typeOfSearch == 'tasks') {
        		idx = 2;
        	}       		
        	children[idx].title = children[idx].title + "(" + numItems + ")"; 
        }	
 	},
 	tabSelected: function(child) { 	
 		if(child.content.id.indexOf('tasks') != -1) {
 			this._currentView = 'tasks';
 		} else if(child.content.id.indexOf('resources') != -1){
 			this._currentView = 'resources'; 			
 		} else if(child.content.id.indexOf('groups') != -1){
 			this._currentView = 'groups'; 			
 		} 		
	    dojo.cookie("finder.currentView", this._currentView, { expires: 30});
 		phxTrace(child);
 		phxTrace('Current selection is ' + this._currentView);
 	},
 	initializeMenu: function(type, headerText, header, tabbedContainer, isSelected) {
 			var div = dojo.create("div", {id: this._finderIdPrefix + type + "_containerDiv"});
       	 	var cp1 = new dijit.layout.ContentPane({title: headerText,content: div, refreshOnShow:true, selected: isSelected});
       	 	return cp1;        	       
 	},
 	initializeTaskCache: function() {
 	phxTrace('selectedChassisList: ' + this._selectedChassisListInput.innerHTML);
    
    var collectionId = "ALL_CHASSIS";
    var selection = this._selectedChassisListInput.innerHTML;
    if(isChassisMapRendered()) {
    	selection = this._selectedChassisMapListInput.innerHTML;
    	collectionId = "CHASSIS_MAP_COLLECTION";
    }
    if(selection == "") {
    	return;
    }
 	var dataset = new ibm.tivoli.navmodel.Dataset({
        baseUrl: "/ibm/tivoli/rest",
        provider: "USMI",                 
        datasource: "",
        dataset: collectionId,    
        parameters: {
              homeParms: this._homeParamsForDataSet
        },   
        defaultProperties: ["all"],
        useDatastore: false
    });  
    phxTrace("fetching dataset items... for finder");
    dataset._fetchTasks({onComplete: dojo.hitch(this, this.datasetTasksRetrieved),  
    				  onError: dojo.hitch(this, this.datasetRetrievedError),
    				  itemIds: selection});    
    phxTrace("done fetching dataset items for finder");
    phxTrace(dataset);
 	}, 
 	
 	datasetItemsRetrieved: function(items) {    
    	phxTrace(items);    	
    	for(var i = 0; i < items.length; i++) {
    		phxTrace('getting tasks for ' + items[i].getId());
    		items[i].getDataset()._fetchTasks({onComplete: dojo.hitch(this, this.datasetTasksRetrieved),  
    				  						  onError: dojo.hitch(this, this.datasetRetrievedError),
    				  						  itemIds: items[i].getId()});
    	}
    	
  	},
  	datasetTasksRetrieved: function(tasks, args) {
    	var taskList = this.createRecursiveContextTaskList(tasks, '', true, 'General');   
    	this._contextTaskDataStore = new itme.finder.ContextTaskDataStore('contextTasks', taskList);  
  	},
  	createRecursiveContextTaskList: function(tasks, taskList, first, category) {
  		for(var i = 0; i < tasks.length; i++) {
    		
    		var task = tasks[i];
    		if(task.getType() == 'separator') {
    			continue;
    		} else if(task.getType() == 'cascade') {
    			taskList = this.createRecursiveContextTaskList(task.getChildTasks(), taskList, first, task.getLabel());
    			continue;
    		} else if(first) { first = false;} else {
    			taskList = taskList + '#_';
    		}
    		var taskId = task.getId();
    		if(taskId == null || taskId.length == 0) {
    			if(i == 0) first = true;    
    			continue;			
    		} 
    		taskList = taskList + taskId + ';';
    		taskList = taskList + task.getLabel() + ';';
    		taskList = taskList + category;    		
    	}
    	return taskList;
  	},
 	datasetRetrievedError: function(error, request){
    	phxTrace("The items request to the datastore failed.  " + error);
    	phxTrace("- request: " + request);
  	}
 	
 	
 	
});

}

var finder;
var contextFinder;
function tmpLoadFinderRequires() {   
	itmeUtils.initializeTimer("finderAllTimer");       
  itmeUtils.traceTimeDelta("finderAllTimer", "finder init", true); 	
    
    
}

function delayRequires() { 	
	  
	contextFinder.initializeTaskCache();	
}

function createFinder(FinderParentDOMNodeID, finderDomElement) {
    // the tundra class is necessary for the tooltip pointer to function.  There may be other ways to accomplish this if necessary.
    	
    dojo.addClass(document.body, "tundra");   // tundra class hack    
  	delete Object.prototype.toJSONString;
    if(finder) {
    	finder.initializeFieldReferences(FinderParentDOMNodeID, finderDomElement);
    	itmeUtils.traceTimeDelta("finderAllTimer", "finder init cache", true); 	
    	finder.initializeDataStores(FinderParentDOMNodeID);
    	itmeUtils.traceTimeDelta("finderAllTimer", "finder end cache", true); 	
    	return;
    }
    finder = new itme.finder.Finder(FinderParentDOMNodeID, finderDomElement); 
    itmeUtils.traceTimeDelta("finderAllTimer", "finder finish", true); 	
}
function createContextFinder(FinderParentDOMNodeID, finderDomElement) {
    phxTrace("contextFinder - createFinder, finderDomElement: " + finderDomElement);
    // the tundra class is necessary for the tooltip pointer to function.  There may be other ways to accomplish this if necessary.
    dojo.addClass(document.body, "tundra");   // tundra class hack    
  	//delete Object.prototype.toJSONString;
    
    //if(contextFinder) {
   // 	contextFinder.initializeFieldReferences(FinderParentDOMNodeID, finderDomElement);
    //	return;
   // }
    contextFinder = new itme.finder.Finder(FinderParentDOMNodeID, finderDomElement); 
    setTimeout('delayRequires()', 20);

}

function isChassisMapRendered() {
	if(dojo.byId('TheChassisMapIsRendered')) {
		return true;
	}
	return false;
}
function objHighlightSearchStr(targEl, searchStr, targetStr, isCategory) {
	targEl.innerHTML = "";
  var type = "span"; var textClass = "";
  if(isCategory) {
  	type = "h1";
  	textClass = "categorySectionTitle";
  }
  var resultEl = dojo.create(type, {className: textClass}, targEl);  

 				
  var re = new RegExp(searchStr, "i");  
  var startPos = targetStr.search(re); 
   
  //if no match just return the string as-is 
  if (startPos <= -1){
    dojo.create("span", {innerHTML:targetStr}, resultEl);
    return resultEl;    
  } 

  var s0 = dojo.create("span", {innerHTML:targetStr.substring(0, startPos)}, resultEl);
  var s1 = dojo.create("span", {innerHTML:targetStr.substring(startPos, startPos + searchStr.length)}, resultEl);
  dojo.addClass(s1, "SearchSubstringHighlight");  
  dojo.create("span", {innerHTML:targetStr.substring(startPos + searchStr.length, targetStr.length)}, resultEl);
  return resultEl;
  
}

// Right now this part is not explicitly used, but may be used to assist with dojo builds, do try to keep this current.
























/**
 *
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// AMQ Ajax handler
// This class provides the main API for using the Ajax features of AMQ. It
// allows JMS messages to be sent and received from javascript when used
// with the org.apache.activemq.web.MessageListenerServlet.
//
// This version of the file provides an adapter interface for the jquery library
// and a namespace for the Javascript file, private/public variables and
// methods, and other scripting niceties. -- jim cook 2007/08/28

var org = org || {};
org.activemq = org.activemq || {};

org.activemq.AmqAdapter = {
	isTIP: false,
	
/**
 * Initialize the adapter
 */
	init: function(options) {
		// determine if we are running in TIP
		try {
			if (tipdojo) {
				this.isTIP = true;
			}
		}
		catch (e) {
			this.isTIP = false;	
		}
	},

/**
 *  Implement this method to make an AJAX call to the AjaxServlet. An
 *  options object will accompany this class and will contain the properties
 *  that describe the details of the AJAX call. The options object will
 *  have the following properties:
 *
 *  - method:  'get' or 'post'
 *  - data:    query data to accompany the post or get.
 *  - success: A callback function that is invoked upon successful
 *             completion of the AJAX call. The parameter is:
 *             - data: The result of the AJAX call. In the case of XML
 *                     data should resolve to a Document element.
 *  - error:   A callback when some type of error occurs. The callback
 *             function's parameters should be:
 *             - xhr:    The XmlHttpRequest object.
 *             - status: A text string of the status.
 *             - ex:     The exception that caused the error.
 */
	ajax: function(uri, options) {
		// post
		if (options.method == 'post') {
			if (options.data)
			{	uri += "?";
				uri += options.data;
			}
			
			if (this.isTIP) {
				tipdojo.xhr("post", {
					url: uri,
					sync: false,
					handleAs: "xml",										
					load : options.success ? options.success : function() {},
					error: options.error ? function(ex, ioargs) {
							options.error(ioargs.xhr,ioargs.xhr.status, ex);
						} : function() {}				
				});
			}
			else {
				dojo.xhr("post", {
					url: uri,
					sync: false,
					handleAs: "xml",										
					load : options.success ? options.success : function() {},
					error: options.error ? function(ex, ioargs) {
							options.error(ioargs.xhr,ioargs.xhr.status, ex);
						} : function() {}				
				});
			}
		}
		// get
		else {
			if (options.data)
			{	uri += "?";
				uri += options.data;
			}
			
			if (this.isTIP) {
				tipdojo.xhrGet({
					url: uri,
					sync: false,
					handleAs: "xml",				
					load : options.success ? options.success : function() {},
					error: options.error ? function(ex, ioargs) {
							options.error(ioargs.xhr,ioargs.xhr.status, ex);
						} : function() {}
				});
			}
			else {
				dojo.xhrGet({
					url: uri,
					sync: false,
					handleAs: "xml",				
					load : options.success ? options.success : function() {},
					error: options.error ? function(ex, ioargs) {
							options.error(ioargs.xhr,ioargs.xhr.status, ex);
						} : function() {}
				});
			}
		}		
	},

	log: function(message, exception) {
		if (typeof console != 'undefined' && console.log) {
			console.log(message);
		}
	}
};



org.activemq.Amq = function() {
	var connectStatusHandler = null;
	
	// Just a shortcut to eliminate some redundant typing.
	var adapter = org.activemq.AmqAdapter;

	if (typeof adapter == 'undefined') {
		throw 'An org.activemq.AmqAdapter must be declared before the amq.js script file.';
	}

	// Indicates whether init() has been invoked
	var initialized = false;
	
	// Indicates whether terminate() has been invoked
	var terminated = false;

	// Inidicates whether logging is active or not. Not by default.
	var logging = false;
	
	// The URI of the AjaxServlet.
	var uri;

	// The number of seconds that the long-polling socket will stay connected.
	// Best to keep this to a value less than one minute.
	var timeout = 25;
	
	// Poll delay. if set to positive integer, this is the time to wait in ms
	// before sending the next poll after the last completes.
	var pollDelay = 0;

	// 5 second delay if an error occurs during poll. This could be due to
	// server capacity problems or a timeout condition.
	var pollErrorDelay = 5000;

	// Map of handlers that will respond to message receipts. The id used during
	// addListener(id, destination, handler) is used to key the callback
	// handler.  
	var messageHandlers = {};
	
	// Indicates whether an AJAX post call is in progress.
	var batchInProgress = false;

	// A collection of pending messages that accumulate when an AJAX call is in
	// progress. These messages will be delivered as soon as the current call
	// completes. The array contains objects in the format { destination,
	// message, messageType }.
	var messageQueue = [];

	/**
	 * Iterate over the returned XML and for each message in the response, 
	 * invoke the handler with the matching id.
	 */
	var messageHandler = function(data) {
		var response = data.getElementsByTagName("ajax-response");
		if (response !== null && response.length == 1) {
			if (connectStatusHandler) {
				connectStatusHandler(true);
			}
			var responses = response[0].childNodes;    // <response>
			for (var i = 0; i < responses.length; i++) {
				var responseElement = responses[i];

				// only process nodes of type element.....
				if (responseElement.nodeType != 1) {
					continue;
				}

				var id = responseElement.getAttribute('id');				
				var handler = messageHandlers[id];				
				if (handler) {
					// Loop thru and handle each <message>
					for (var j = 0; j < responseElement.childNodes.length; j++) {
						var msg = responseElement.childNodes[j];
						handler(msg);					
					}
				}
				else if (logging) {
					adapter.log('No handler found to match message with id = ' + id);
				}
			}
		}
	};

	var errorHandler = function(xhr, status, ex) {		
		if (connectStatusHandler) {
			connectStatusHandler(false);
		}
		if (logging) {
			adapter.log('Error occurred in ajax call. HTTP result: ' + xhr.status + ', status: ' + status);
		}
	};

	var pollErrorHandler = function(xhr, status, ex) {	
		if (connectStatusHandler) {
			connectStatusHandler(false);
		}
		
		if (status === 'error' && xhr.status === 0) {
			if (logging) {
				adapter.log('Server connection dropped.');
			}			
		}
		else if (logging) {
				adapter.log('Error occurred in poll. HTTP result: ' + xhr.status + ', status: ' + status);
		}
		
		setTimeout(function() { sendPoll(); }, pollErrorDelay);
	};

	var pollHandler = function(data) {
		try {		
			messageHandler(data);
		} 
		catch(e) {
			if (logging) {
				adapter.log('Exception in the poll handler: ' + data, e);
			}
			//throw(e);
		} 
		finally {
			setTimeout(function() { sendPoll(); }, pollDelay);
		}		
	};

	var sendPoll = function() {
		if (!terminated) {
			// Workaround IE6 bug where it caches the response
			// Generate a unique query string with date and random
			var now = new Date();		
			var data = 'timeout=' + (timeout * 1000) + '&d=' + now.getTime() + '&r=' + Math.random();
				 		
			var options = { method: 'get',
				data: data,
				success: pollHandler,
				error: pollErrorHandler};
			adapter.ajax(uri, options);	
		}
		else { 
			terminated = false;
		}
	};

	startBatch = function() {
		batchInProgress = true;
	};

	endBatch = function() {
		if (messageQueue.length > 0) {
			var body = buildParams(messageQueue);
			messageQueue.length = 0;
			startBatch();
			adapter.ajax(uri, {
				method: 'post',
				data: body,
				success: endBatch, 
				error: errorHandler});
		} else {
			batchInProgress = false;
		}
	};
		
	var sendJmsMessage = function(destination, message, type) {
		// Add message to outbound queue
		if (batchInProgress) {
			messageQueue[messageQueue.length] = {
				destination: destination,
				message: message,
				messageType: type
			};
		} else {
			startBatch();
			adapter.ajax(uri, { method: 'post',
				data: 'destination=' + destination + '&message=' + message + '&type=' + type,
				error: function(xhr, status, ex) {
					batchInProgress = false;
					errorHandler(xhr, status, ex);								
				},
				success: endBatch});
		}
	};

	var buildParams = function(msgs) {
		var s = [];
		for (var i = 0, c = msgs.length; i < c; i++) {
			if (i !== 0) {
				s[s.length] = '&';
			}
			
			s[s.length] = ((i === 0) ? 'destination' : 'd' + i);
			s[s.length] = '=';
			s[s.length] = msgs[i].destination;
			s[s.length] = ((i === 0) ? '&message' : '&m' + i);
			s[s.length] = '=';
			s[s.length] = msgs[i].message;
			s[s.length] = ((i === 0) ? '&type' : '&t' + i);
			s[s.length] = '=';
			s[s.length] = msgs[i].messageType;
		}
		return s.join('');
	};

	return {
		isInitialized: function() {
			return initialized;
		},

		init: function(options) {
			initialized = true;
			
			// set up init options
			connectStatusHandler = options.connectStatusHandler;
			uri = options.uri || '/amq';
			timeout = typeof options.timeout == 'number' ? options.timeout : 25;
			pollDelay = typeof options.pollDelay == 'number' ? options.pollDelay : 0;
			pollErrorDelay = typeof options.pollErrorDelay == 'number' ? options.pollErrorDelay : 5000;
			logging = options.logging;
			
			// initialize ajax adapter
			adapter.init(options);
			
			// initiate polling
			sendPoll();					
		},

		terminate: function() {
			terminated = true;
			initialized = false;			
		},
		
		setConnectStatusHandler: function(handler) {
			connectStatusHandler = handler;
		},

		// Send a JMS message to a destination (eg topic://MY.TOPIC).  Message
		// should be xml or encoded xml content.
		sendMessage : function(destination, message) {
			sendJmsMessage(destination, message, 'send');
		},

		// Listen on a channel or topic.
		// handler must be a function taking a message argument
		addListener : function(id, destination, handler) {
			messageHandlers[id] = handler;
			sendJmsMessage(destination, id, 'listen');		
		},

		// Remove Listener from channel or topic.
		removeListener : function(id, destination) {
			delete messageHandlers[id];
			sendJmsMessage(destination, id, 'unlisten');
		}
	};
}();














function ibm_tivoli_navmodel_EventRouter() {}

// data members
ibm_tivoli_navmodel_EventRouter.initialized = false;
ibm_tivoli_navmodel_EventRouter.isTIP = false;
ibm_tivoli_navmodel_EventRouter.eventDestination = null;
ibm_tivoli_navmodel_EventRouter.timeout = 15;
ibm_tivoli_navmodel_EventRouter.pollDelay = 0;
ibm_tivoli_navmodel_EventRouter.pollErrorDelay = 5000;
ibm_tivoli_navmodel_EventRouter.contextRoot = "/ibm/tivoli/amq";
ibm_tivoli_navmodel_EventRouter.registry = {};


/*
 * Returns true if already initialized
 */
ibm_tivoli_navmodel_EventRouter.isInitialized = function() {
	return this.initialized;
};


/*
 * Initializes connection with event feed
 */
ibm_tivoli_navmodel_EventRouter.init = function(args) {
	if (!this.initialized) {
		this.initialized = true;
		
		this.eventDestination = args.eventDestination;
		
		if (args.timeout !== undefined) {
			this.timeout = args.timeout;
		}
		
		if (args.pollDelay !== undefined) {
			this.pollDelay = args.pollDelay;
		}
		
		if (args.pollErrorDelay !== undefined) {
			this.pollErrorDelay = args.pollErrorDelay;
		}
		
		if (args.contextRoot !== undefined) {
			this.contextRoot = args.contextRoot + "/amq";
		}
		
		// determine if we are running in TIP
		try {
			if (tipdojo) {
				this.isTIP = true;			
			}
		}
		catch (e) {
			this.isTIP = false;		
		}
	}
	
	// initialize amq
	if (!org.activemq.Amq.isInitialized()) {							
		var initArgs = {
			uri: this.contextRoot, 
			timeout: this.timeout,
			pollDelay: this.pollDelay,
			pollErrorDelay: this.pollErrorDelay,
			connectStatusHandler: ibm_tivoli_navmodel_EventRouter.connectionStatus,
			logging: true
		};
			
		this.info("***** event_router - initializing event feed");
		org.activemq.Amq.init(initArgs);	
		org.activemq.Amq.addListener("ibm_tivoli_navmodel_EventRouter", this.eventDestination, ibm_tivoli_navmodel_EventRouter.routeEvent);	
	}
};


/*
 * Terminates connection with event feed
 */
ibm_tivoli_navmodel_EventRouter.terminate = function() {
	if (org.activemq.Amq.isInitialized()) {							
		this.info("***** event_router - terminating event feed");
		org.activemq.Amq.removeListener("ibm_tivoli_navmodel_EventRouter", this.eventDestination);
		org.activemq.Amq.terminate();

	}
	if (this.initialized) {							
		this.info("***** event_router - terminating");
		this.terminated = true;
		this.initialized = false;
	}
};


/*
 * Adds listener for the requested eventId
 */
ibm_tivoli_navmodel_EventRouter.addListener = function(listener, eventId, eventCallback) {
	// add entry if it doesn't already exist
	if (!this.registry[eventId]) {		
		this.registry[eventId] = {listeners: []};
	}
		
	// append listener to the entry
	var listenersLength = this.registry[eventId].listeners.length;
	this.info("***** event_router - adding listener for eventId=" + eventId + " at index=" + listenersLength);		
	this.registry[eventId].listeners[listenersLength] = {listener: listener, eventCallback: eventCallback};	
	
	// validate listeners
	this.validate();
};


/*
 * Removes listener for the requested eventId
 */
ibm_tivoli_navmodel_EventRouter.removeListener = function(listener, eventId) {
	var listenersLength = 0;
	
	if (this.registry[eventId]) {
		listenersLength = this.registry[eventId].listeners.length;
		for (var listenerIndex = 0;  listenerIndex < listenersLength;  ++listenerIndex) {
			// remove listener from entry
			if (this.registry[eventId].listeners[listenerIndex].listener === listener) {
				this.info("***** event_router - remove listener for eventId=" + eventId + " at index=" + listenerIndex);
				this.registry[eventId].listeners.splice(listenerIndex, 1);
					
				// remove entry if no more listeners
				listenersLength = this.registry[eventId].listeners.length;
				if (listenersLength === 0) {
					delete this.registry[eventId];
					
					// no more listeners so shut down event feed
					if (!this.haveListeners()) {
						this.terminate();
					}					
				}					
				break;
			}
		}
	}	
		
	return listenersLength;		
};


/*
 * Returns registered listener count
 */
ibm_tivoli_navmodel_EventRouter.getListenerCount = function() {
	var listeners = 0;
	
	for (var eventId in this.registry) {
		if (this.registry.hasOwnProperty(eventId)) {
			var listenerLength = this.registry[eventId].listeners.length;				
			listeners += listenerLength;		
		}
	}
	
	return listeners;
};


/*
 * Returns true if we have at least one listener
 */
ibm_tivoli_navmodel_EventRouter.haveListeners = function() {
	for (var eventId in this.registry) {
		if (this.registry.hasOwnProperty(eventId)) {
			var listenerLength = this.registry[eventId].listeners.length;				
			if (listenerLength > 0) {
				return true;
			}		
		}
	}
	
	return false;
};


/*
 * Validate registered liseners; remove any that can no longer be pinged
 */
ibm_tivoli_navmodel_EventRouter.validate = function() {
	for (var eventId in this.registry) {
		if (this.registry.hasOwnProperty(eventId)) {
			var listenersLength = this.registry[eventId].listeners.length;				
			for (var listenerIndex = listenersLength - 1;  listenerIndex >= 0;  --listenerIndex) {
				var listener = this.registry[eventId].listeners[listenerIndex].listener;
				try {
					listener.ping();								
				}
				catch (e) {	
					this.warn("***** event_router - validate is removing invalid listener ");
					this.removeListener(listener, eventId);
					//invalid[invalid.length] = listener;
				}				
			}
		}
	}
};


/*
 * Routes events from event feed to the listeners;
 * it is being invoked without a "this" so make sure
 * to use the global references to variables
 */
ibm_tivoli_navmodel_EventRouter.routeEvent = function(jmsMsg) {
	if (jmsMsg.data.length > 0) {			
		var event = null;
		if (ibm_tivoli_navmodel_EventRouter.isTIP) {
			event = tipdojo.fromJson(jmsMsg.data);
		}
		else {
			event = dojo.fromJson(jmsMsg.data);
		}
			
		var eventId = event.id;
		var registry = ibm_tivoli_navmodel_EventRouter.registry;	
			
		if (registry[eventId]) {
			var listenersLength = registry[eventId].listeners.length;						
			for (var listenerIndex = listenersLength - 1;  listenerIndex >= 0;  --listenerIndex) {
				try {
					var eventCallback = registry[eventId].listeners[listenerIndex].eventCallback;				
					eventCallback(event);									
				}
				catch (e) {	
					// see if listener is still pingable				
					var listener = registry[eventId].listeners[listenerIndex].listener;
					try {
						listener.ping();
						ibm_tivoli_navmodel_EventRouter.error("***** event_router - Exception when routing eventId=" + eventId + " exception=" + e);					
					}
					catch (e2) {
						// listener is dead we must remove it
						ibm_tivoli_navmodel_EventRouter.warn("***** event_router - routeEvent is removing invalid listener");
						ibm_tivoli_navmodel_EventRouter.removeListener(listener, eventId);
					}
				}				
			}
		}
	}
};


/*
 * Connection status handler;  called from event feed;
 * it is being invoked without a "this" so make sure to
 * use the global references to variables
 */
ibm_tivoli_navmodel_EventRouter.connectionStatus = function(status) {
	//ibm_tivoli_navmodel_EventRouter.info("***** event_router - connection status=" + status);
};


/*
 * Log info message
 */
ibm_tivoli_navmodel_EventRouter.info = function(msg) {
	console.info(msg);
	//alert(msg);
};


/*
 * Log warning message
 */		
ibm_tivoli_navmodel_EventRouter.warn = function(msg) {
	console.warn(msg);
	//alert(msg);
};


/*
 * Log error message
 */	
ibm_tivoli_navmodel_EventRouter.error = function(msg) {
	console.error(msg);
	//alert(msg);
};
	

	
dojo.i18n._preloadLocalizations("dojo.nls.dojo", ["ROOT","ar","ca","cs","da","de","de-de","el","en","en-gb","en-us","es","es-es","fi","fi-fi","fr","fr-fr","he","he-il","hu","it","it-it","ja","ja-jp","ko","ko-kr","nb","nl","nl-nl","pl","pt","pt-br","pt-pt","ru","sk","sl","sv","th","tr","xx","zh","zh-cn","zh-tw"]);


	//Check if document already complete, and if so, just trigger page load
	//listeners. NOTE: does not work with Firefox before 3.6. To support
	//those browsers, set djConfig.afterOnLoad = true when you know Dojo is added
	//after page load. Using a timeout so the rest of this
	//script gets evaluated properly. This work needs to happen after the
	//dojo.config.require work done in dojo._base.
	if(dojo.isBrowser && (document.readyState === "complete" || dojo.config.afterOnLoad)){
		window.setTimeout(dojo._loadInit, 100);
	}
})();

