#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
# This script is used by High Avaliability for 
# invocation as part of HA configuration or termination.
# The script is called to start, end ldap server or return ldap
# server status on FSM.
# This script should be put in /opt/ibm/director/ha/plugins/tsa/


PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)
#### Use a special path for TSA monitoring scripts
PATH=/bin:/usr/bin:/sbin:/usr/sbin:/usr/sbin/rsct/bin
export PATH

EXPLOITER_COMMON_CODE=/opt/ibm/director/ha/bin/include/smha_exploiter_common.sh
SLAPD_BIN="/usr/lib/openldap/slapd"

if [ -f $EXPLOITER_COMMON_CODE ]; then
    source $EXPLOITER_COMMON_CODE
else
    echo "$PROGNAME:Exploiter common code not found."
    exit 1
fi
#############################################################

RC=$TSA_UNKNOWN

Action=${1}

case ${Action} in
start)
    smha_record_tsa_action $Action LDAP "-"
    #normally, you would call your startup process here
    #e.g. /etc/init.d/mydaemon start
    smha_debug_tsa "fsm ldap server started"
    /etc/init.d/ldap start

    ;;

stop)
    smha_record_tsa_action $Action LDAP "-"
    #normally, you would call your stop process here
    #e.g. /etc/init.d/mydaemon stop
    # you may want to perform some sanity checking here to make sure
    # the process really died.  Especially in the case if the process
    # contains files in shared storage which needs to be unmounted
    # in the case of a manual failover.
    
    smha_debug_tsa "fsm ldap server stopped"
    /etc/init.d/ldap stop

    ;;

status)    
    #normally, you would call your status process here
    #e.g. /etc/init.d/mydaemon status
    #statusrc=$?
    /etc/init.d/ldap status

    if [ $? == 0 ]; then
        RC=${TSA_ONLINE}
    else
        RC=${TSA_OFFLINE}
    fi
    echo $RC
    ;;
esac
#debug_tsa "$PROGNAME $Action: rc=$RC"
smha_record_tsa_action $Action LDAP $RC
exit $RC